with Ada.Containers;                  use Ada.Containers;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Vectors;
with Ada.Directories;
with Ada.Exceptions;
with Ada.Finalization;
with Ada.Strings.Wide_Wide_Unbounded; use Ada.Strings.Wide_Wide_Unbounded;
pragma Warnings (Off, "internal");
with Ada.Strings.Wide_Wide_Unbounded.Aux;
use Ada.Strings.Wide_Wide_Unbounded.Aux;
pragma Warnings (On, "internal");
with Ada.Text_IO;                     use Ada.Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Unchecked_Deallocation;
with System;
with GNATCOLL.Traces;
with GNAT.Traceback.Symbolic;
with Langkit_Support.Hashes;  use Langkit_Support.Hashes;
with Langkit_Support.Images;  use Langkit_Support.Images;
with Langkit_Support.Relative_Get;
pragma Warnings (Off, "referenced");
with Langkit_Support.Adalog.Abstract_Relation;
use Langkit_Support.Adalog.Abstract_Relation;
with Langkit_Support.Adalog.Debug;
use Langkit_Support.Adalog.Debug;
with Langkit_Support.Adalog.Operations;
use Langkit_Support.Adalog.Operations;
with Langkit_Support.Adalog.Predicates;
use Langkit_Support.Adalog.Predicates;
with Langkit_Support.Adalog.Pure_Relations;
use Langkit_Support.Adalog.Pure_Relations;
pragma Warnings (On, "referenced");
with Libadalang.Private_Converters;
use Libadalang.Private_Converters;
with Libadalang.Introspection_Implementation;
pragma Warnings (Off, "referenced");
          with Ada.Containers.Hashed_Sets;
          with Libadalang.Env_Hooks;
            use Libadalang.Env_Hooks;
          with Libadalang.Implementation.Extensions;
            use Libadalang.Implementation.Extensions;
          with Libadalang.Internal_Default_Provider;
          with Libadalang.Sources;
pragma Warnings (On, "referenced");
package body Libadalang.Implementation is
   use Libadalang.Common.Precomputed_Symbols;
   package Context_Vectors is new Ada.Containers.Vectors
     (Index_Type   => Positive,
      Element_Type => Internal_Context);
   type Contexts_Destructor is limited
      new Ada.Finalization.Limited_Controlled with null record;
   overriding procedure Finalize (CD : in out Contexts_Destructor);
   --  Helper to destroy all contexts when terminating the process
   protected Context_Pool is
      procedure Acquire (Context : out Internal_Context)
         with Post => Context /= null;
      --  If a context is free for reuse, increment its serial number and
      --  return it. Otherwise, allocate a new one. In any case, this does not
      --  initialize it, except for the Serial_Number field.
      procedure Release (Context : in out Internal_Context)
         with Pre  => Context /= null,
              Post => Context = null;
      --  Tag Context as free for reuse and set it to null
      procedure Free;
      --  Free all contexts in this pool. Intended to be called only when the
      --  process is terminating, to avoid reported memory leaks.
   private
      Available : Context_Vectors.Vector;
      --  List of allocated contexts that can be re-used right now
      CD : Contexts_Destructor with Unreferenced;
      --  Singleton whose only purpose is to free all contexts in Available
      --  when finalized.
   end Context_Pool;
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure);
   --  Common underlying implementation for Register_Destroyable_Gen
   pragma Warnings (Off, "referenced");
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access;
   pragma Warnings (On, "referenced");
   procedure Destroy (Env : in out Lexical_Env_Access);
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean;
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean;
   --  Those maps are used to give unique ids to lexical envs while pretty
   --  printing them.
   package Address_To_Id_Maps is new Ada.Containers.Hashed_Maps
     (Lexical_Env, Integer, Hash, "=");
   type Dump_Lexical_Env_State is record
      Env_Ids : Address_To_Id_Maps.Map;
      --  Mapping: Lexical_Env -> Integer, used to remember which unique Ids we
      --  assigned to the lexical environments we found.
      Next_Id : Positive := 1;
      --  Id to assign to the next unknown lexical environment
      Root_Env : Lexical_Env;
      --  Lexical environment we consider a root (this is the Root_Scope from
      --  the current analysis context), or null if unknown.
   end record;
   --  Holder for the state of lexical environment dumpers
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String;
   --  If E is known, return its unique Id from State. Otherwise, assign it a
   --  new unique Id and return it.
   ----------------------------
   -- Construct_Entity_Array --
   ----------------------------
   function Construct_Entity_Array
     (V : AST_Envs.Entity_Vectors.Vector) return Internal_Entity_Array_Access
   is
      Ret : Internal_Entity_Array_Access :=
        Create_Internal_Entity_Array (V.Length);
   begin
      for J in V.First_Index .. V.Last_Index loop
         Ret.Items (J) := V.Get (J);
      end loop;
      declare
         Tmp : AST_Envs.Entity_Vectors.Vector := V;
      begin
         Tmp.Destroy;
      end;
      return Ret;
   end Construct_Entity_Array;
   ----------------
   -- Enter_Call --
   ----------------
   procedure Enter_Call
     (Context : Internal_Context; Call_Depth : access Natural)
   is
      Max             : Natural renames Context.Max_Call_Depth;
      Current         : Natural renames Context.Current_Call_Depth;
      High_Water_Mark : Natural renames Context.Call_Depth_High_Water_Mark;
   begin
      Current := Current + 1;
      High_Water_Mark := Natural'Max (High_Water_Mark, Current);
      Call_Depth.all := Current;
      if Current > Max then
         raise Property_Error with "stack overflow";
      end if;
   end Enter_Call;
   ---------------
   -- Exit_Call --
   ---------------
   procedure Exit_Call (Context : Internal_Context; Call_Depth : Natural) is
      Current : Natural renames Context.Current_Call_Depth;
   begin
      if Call_Depth /= Current then
         raise Unexpected_Call_Depth with
            "Langkit code generation bug for call depth handling detected";
      end if;
      Current := Current - 1;
   end Exit_Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Symbol_Type) return Character_Type_Array_Access is
      T      : constant Text_Type := Image (Self);
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (T'Length);
   begin
      Result.Items := T;
      return Result;
   end Image;
   ------------------
   -- Context_Pool --
   ------------------
   protected body Context_Pool is
      -------------
      -- Acquire --
      -------------
      procedure Acquire (Context : out Internal_Context) is
      begin
         if Available.Is_Empty then
            Context := new Analysis_Context_Type;
            Context.Serial_Number := 1;
         else
            Context := Available.Last_Element;
            Available.Delete_Last;
         end if;
      end Acquire;
      -------------
      -- Release --
      -------------
      procedure Release (Context : in out Internal_Context) is
      begin
         Available.Append (Context);
         Context.Serial_Number := Context.Serial_Number + 1;
         Context := null;
      end Release;
      ----------
      -- Free --
      ----------
      procedure Free is
      begin
         for C of Available loop
            Free (C);
         end loop;
      end Free;
   end Context_Pool;
   --------------
   -- Finalize --
   --------------
   overriding procedure Finalize (CD : in out Contexts_Destructor) is
      pragma Unreferenced (CD);
   begin
      Context_Pool.Free;
   end Finalize;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (File_Reader : in out Internal_File_Reader_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_File_Reader'Class, Internal_File_Reader_Access);
   begin
      if File_Reader /= null and then File_Reader.all.Dec_Ref then
         Destroy (File_Reader);
      end if;
   end Dec_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Provider : in out Internal_Unit_Provider_Access) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Internal_Unit_Provider'Class, Internal_Unit_Provider_Access);
   begin
      if Provider /= null and then Provider.all.Dec_Ref then
         Destroy (Provider);
      end if;
   end Dec_Ref;
   ----------------
   -- Get_Env_Id --
   ----------------
   function Get_Env_Id
     (E : Lexical_Env; State : in out Dump_Lexical_Env_State) return String
   is
      C        : Address_To_Id_Maps.Cursor;
      Inserted : Boolean;
   begin
      if E = Null_Lexical_Env then
         return "$null";
      elsif E = State.Root_Env then
         --  Insert root env with a special Id so that we only print it once
         State.Env_Ids.Insert (E, -1, C, Inserted);
         return "$root";
      end if;
      State.Env_Ids.Insert (E, State.Next_Id, C, Inserted);
      if Inserted then
         State.Next_Id := State.Next_Id + 1;
      end if;
      return '@' & Stripped_Image (Address_To_Id_Maps.Element (C));
   end Get_Env_Id;
   pragma Warnings (Off, "referenced");
   function To_Lookup_Kind_Type (K : Lookup_Kind) return Lookup_Kind_Type
   is
     (Lookup_Kind_Type'Val (Lookup_Kind'Pos (K)));
   pragma Warnings (On, "referenced");
   --------------------
   -- Create_Context --
   --------------------
   function Create_Context
     (Charset        : String;
      File_Reader    : Internal_File_Reader_Access;
      Unit_Provider  : Internal_Unit_Provider_Access;
      With_Trivia    : Boolean;
      Tab_Stop       : Positive;
      Max_Call_Depth : Natural := 1000)
      return Internal_Context
   is
      Actual_Charset : constant String :=
        (if Charset = "" then Default_Charset else Charset);
      Symbols        : constant Precomputed_Symbol_Table
        := Create_Symbol_Table;
      Context        : Internal_Context;
   begin
      Context_Pool.Acquire (Context);
      Context.Ref_Count := 1;
      Context.Symbols := Symbol_Table (Symbols);
      Context.Charset := To_Unbounded_String (Actual_Charset);
      Context.Tab_Stop := Tab_Stop;
      Context.With_Trivia := With_Trivia;
      Context.Root_Scope := Create_Static_Lexical_Env
        (Parent => AST_Envs.No_Env_Getter,
         Node   => null);
      --  Create a new ownership share for File_Reader so that it lives at
      --  least as long as this analysis context.
      Context.File_Reader := File_Reader;
      if Context.File_Reader /= null then
         Context.File_Reader.Inc_Ref;
      end if;
      --  Create a new ownership share for Unit_Provider so that it lives at
      --  least as long as this analysis context.
      Context.Unit_Provider := Unit_Provider;
      if Context.Unit_Provider /= null then
         Context.Unit_Provider.Inc_Ref;
      end if;
         if Context.Unit_Provider = null then
            Context.Unit_Provider := Libadalang.Internal_Default_Provider.Create;
         end if;
      Initialize (Context.Parser);
      Context.Discard_Errors_In_Populate_Lexical_Env := True;
      Context.Logic_Resolution_Timeout :=
        Langkit_Support.Adalog.Default_Timeout_Ticks_Number;
      Context.In_Populate_Lexical_Env := False;
      Context.Cache_Version := 0;
      Context.Reparse_Cache_Version := 0;
      Context.Rewriting_Handle := No_Rewriting_Handle_Pointer;
      Context.Templates_Unit := No_Analysis_Unit;
      Context.Max_Call_Depth := Max_Call_Depth;
      Context.Available_Rebindings := Env_Rebindings_Vectors.Empty_Vector;
Fetch_Standard (Context);
      return Context;
   end Create_Context;
   -----------------
   -- Create_Unit --
   -----------------
   function Create_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Unit : Internal_Unit;
   begin
      Unit := Create_Special_Unit
        (Context, Normalized_Filename, Charset, Rule);
      Context.Units.Insert (Normalized_Filename, Unit);
      return Unit;
   end Create_Unit;
   --------------
   -- Get_Unit --
   --------------
   function Get_Unit
     (Context           : Internal_Context;
      Filename, Charset : String;
      Reparse           : Boolean;
      Input             : Internal_Lexer_Input;
      Rule              : Grammar_Rule;
      Is_Internal       : Boolean := False) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant GNATCOLL.VFS.Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur     : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
      Created : constant Boolean := Cur = No_Element;
      Unit    : Internal_Unit;
      Actual_Charset : Unbounded_String;
      Refined_Input  : Internal_Lexer_Input := Input;
   begin
      --  Determine which encoding to use. Use the Charset parameter (if
      --  provided), otherwise use the context-wide default.
      Actual_Charset := (if Charset'Length /= 0
                         then To_Unbounded_String (Charset)
                         else Context.Charset);
      if Refined_Input.Kind = File then
         Refined_Input.Filename := Normalized_Filename;
      end if;
      if Refined_Input.Kind in File | Bytes_Buffer then
         Refined_Input.Charset := Actual_Charset;
         --  Unless the caller requested a specific charset for this unit,
         --  allow the lexer to automatically discover the source file encoding
         --  before defaulting to the context-specific one. We do this trying
         --  to match a byte order mark.
         Refined_Input.Read_BOM := Charset'Length = 0;
      end if;
      --  Create the Internal_Unit if needed
      Unit :=
        (if Created
         then Create_Unit (Context, Normalized_Filename,
                           To_String (Actual_Charset), Rule)
         else Element (Cur));
      --  If an internal unit is requested, set the corresponding flag.
      --  Otherwise, make sure that the unit we return isn't internal.
      if Is_Internal then
         Unit.Is_Internal := True;
      end if;
      --  (Re)parse it if needed
      if Created or else Reparse then
         --  It is illegal to reparse an internal unit for public API users.
         --  Since public APIs do not allow to pass True to Is_Internal, we can
         --  check here that only the implementation can ask to reparse an
         --  internal unit.
         if Unit.Is_Internal and then not Is_Internal then
            raise Precondition_Failure with "cannot reparse an internal unit";
         end if;
         declare
            Reparsed : Reparsed_Unit;
         begin
            Do_Parsing (Unit, Refined_Input, Reparsed);
            Update_After_Reparse (Unit, Reparsed);
         end;
         --  Now that we have removed reparsed the unit, update its current
         --  charset.
         Unit.Charset := Actual_Charset;
      end if;
      return Unit;
   end Get_Unit;
   --------------
   -- Has_Unit --
   --------------
   function Has_Unit
     (Context : Internal_Context; Unit_Filename : String) return Boolean is
   begin
      return Context.Units.Contains
        (Normalized_Unit_Filename (Context, Unit_Filename));
   end Has_Unit;
   -------------------
   -- Get_From_File --
   -------------------
   function Get_From_File
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Reparse  : Boolean;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind     => File,
         Charset  => <>,
         Read_BOM => False,
         Filename => <>);
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      return Get_Unit (Context, Filename, Charset, Reparse, Input, Rule);
   end Get_From_File;
   ---------------------
   -- Get_From_Buffer --
   ---------------------
   function Get_From_Buffer
     (Context  : Internal_Context;
      Filename : String;
      Charset  : String;
      Buffer   : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      Input : constant Internal_Lexer_Input :=
        (Kind        => Bytes_Buffer,
         Charset     => <>,
         Read_BOM    => False,
         Bytes       => Buffer'Address,
         Bytes_Count => Buffer'Length);
   begin
      if Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot parse from buffer during tree rewriting";
      elsif Context.File_Reader /= null then
         raise Precondition_Failure with
            "cannot parse from buffer with a file reader";
      end if;
      return Get_Unit (Context, Filename, Charset, True, Input, Rule);
   end Get_From_Buffer;
   --------------------
   -- Get_With_Error --
   --------------------
   function Get_With_Error
     (Context  : Internal_Context;
      Filename : String;
      Error    : Text_Type;
      Charset  : String;
      Rule     : Grammar_Rule) return Internal_Unit
   is
      use Units_Maps;
      Normalized_Filename : constant Virtual_File :=
         Normalized_Unit_Filename (Context, Filename);
      Cur                 : constant Cursor :=
         Context.Units.Find (Normalized_Filename);
   begin
      if Cur = No_Element then
         declare
            Unit : constant Internal_Unit := Create_Unit
              (Context, Normalized_Filename, Charset, Rule);
         begin
            Append (Unit.Diagnostics, No_Source_Location_Range, Error);
            return Unit;
         end;
      else
         return Element (Cur);
      end if;
   end Get_With_Error;
   -----------------------
   -- Get_From_Provider --
   -----------------------
   function Get_From_Provider
     (Context : Internal_Context;
      Name    : Text_Type;
      Kind    : Analysis_Unit_Kind;
      Charset : String;
      Reparse : Boolean) return Internal_Unit is
   begin
      if Reparse and then Has_Rewriting_Handle (Context) then
         raise Precondition_Failure with
            "cannot reparse during tree rewriting";
      end if;
      return Context.Unit_Provider.Get_Unit
        (Context, Name, Kind, Charset, Reparse);
   exception
      when Property_Error =>
         raise Invalid_Unit_Name_Error with
            "Invalid unit name: " & Image (Name, With_Quotes => True)
            & " (" & Analysis_Unit_Kind'Image (Kind) & ")";
   end Get_From_Provider;
   -------------------
   -- Unit_Provider --
   -------------------
   function Unit_Provider
     (Context : Internal_Context) return Internal_Unit_Provider_Access
   is (Context.Unit_Provider);
   ----------
   -- Hash --
   ----------
   function Hash (Context : Internal_Context) return Hash_Type is
      function H is new Hash_Access (Analysis_Context_Type, Internal_Context);
   begin
      return H (Context);
   end Hash;
   ---------------------
   -- Has_With_Trivia --
   ---------------------
   function Has_With_Trivia (Context : Internal_Context) return Boolean is
   begin
      return Context.With_Trivia;
   end Has_With_Trivia;
   --------------------------------------------
   -- Discard_Errors_In_Populate_Lexical_Env --
   --------------------------------------------
   procedure Discard_Errors_In_Populate_Lexical_Env
     (Context : Internal_Context; Discard : Boolean) is
   begin
      Context.Discard_Errors_In_Populate_Lexical_Env := Discard;
   end Discard_Errors_In_Populate_Lexical_Env;
   ----------------------------------
   -- Set_Logic_Resolution_Timeout --
   ----------------------------------
   procedure Set_Logic_Resolution_Timeout
     (Context : Internal_Context; Timeout : Natural) is
   begin
      Context.Logic_Resolution_Timeout := Timeout;
   end Set_Logic_Resolution_Timeout;
   --------------------------
   -- Has_Rewriting_Handle --
   --------------------------
   function Has_Rewriting_Handle (Context : Internal_Context) return Boolean is
   begin
      return Context.Rewriting_Handle /= No_Rewriting_Handle_Pointer;
   end Has_Rewriting_Handle;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Context : Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Context : in out Internal_Context) is
   begin
      if Context /= null then
         Context.Ref_Count := Context.Ref_Count - 1;
         if Context.Ref_Count = 0 then
            Destroy (Context);
         end if;
      end if;
   end Dec_Ref;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Context : in out Internal_Context) is
   begin
      --  Destroy all named environment data structures
      for Desc of Context.Named_Envs loop
         for V of Desc.Foreign_Nodes loop
            V.Destroy;
         end loop;
         Destroy (Desc);
      end loop;
      Context.Named_Envs.Clear;
      --  If we are asked to free this context, it means that no one else have
      --  references to its analysis units, so it's safe to destroy these.
      for Unit of Context.Units loop
         Destroy (Unit);
      end loop;
      Context.Units := Units_Maps.Empty_Map;
      Context.Filenames := Virtual_File_Maps.Empty_Map;
      declare
         procedure Destroy is new Ada.Unchecked_Deallocation
           (Env_Rebindings_Type, Env_Rebindings);
         AR : Env_Rebindings_Vectors.Vector renames
            Context.Available_Rebindings;
         R  : Env_Rebindings;
      begin
         for I in AR.First_Index .. AR.Last_Index loop
            R := AR.Get (I);
            Destroy (R);
         end loop;
         AR.Destroy;
      end;
      Destroy (Context.Templates_Unit);
      AST_Envs.Destroy (Context.Root_Scope);
      Destroy (Context.Symbols);
      Destroy (Context.Parser);
      Dec_Ref (Context.File_Reader);
      Dec_Ref (Context.Unit_Provider);
      Context_Pool.Release (Context);
   end Destroy;
   -------------
   -- Context --
   -------------
   function Context (Unit : Internal_Unit) return Internal_Context is
   begin
      return Unit.Context;
   end Context;
   ----------
   -- Hash --
   ----------
   function Hash (Unit : Internal_Unit) return Hash_Type is
      function H is new Hash_Access (Analysis_Unit_Type, Internal_Unit);
   begin
      return H (Unit);
   end Hash;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String) is
      Dummy : constant Internal_Unit := Get_From_File
        (Unit.Context, +Unit.Filename.Full_Name, Charset,
         Reparse => True,
         Rule    => Unit.Rule);
   begin
      null;
   end Reparse;
   -------------
   -- Reparse --
   -------------
   procedure Reparse (Unit : Internal_Unit; Charset : String; Buffer : String)
   is
      Dummy : constant Internal_Unit := Get_From_Buffer
        (Unit.Context, +Unit.Filename.Full_Name, Charset, Buffer, Unit.Rule);
   begin
      null;
   end Reparse;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   procedure Populate_Lexical_Env (Unit : Internal_Unit) is
      Context : constant Internal_Context := Unit.Context;
      Has_Errors : Boolean := False;
      --  Whether at least one Property_Error occurred during this PLE pass
      Saved_In_Populate_Lexical_Env : constant Boolean :=
         Unit.Context.In_Populate_Lexical_Env;
      procedure Reset_Envs_Caches (Unit : Internal_Unit) is
         procedure Internal (Node : Bare_Ada_Node) is
         begin
            if Node = null then
               return;
            end if;
            Reset_Caches (Node.Self_Env);
            for I in 1 .. Children_Count (Node) loop
               Internal (Child (Node, I));
            end loop;
         end Internal;
      begin
         Internal (Unit.AST_Root);
      end Reset_Envs_Caches;
   begin
      --  TODO??? Handle env invalidation when reparsing a unit and when a
      --  previous call raised a Property_Error.
      if Unit.Is_Env_Populated then
         return;
      end if;
      Unit.Is_Env_Populated := True;
      if Unit.AST_Root = null then
         return;
      end if;
      GNATCOLL.Traces.Trace (Main_Trace, "Populating lexical envs for unit: "
                                         & Basename (Unit));
      GNATCOLL.Traces.Increase_Indent (Main_Trace);
      Context.In_Populate_Lexical_Env := True;
         Has_Errors := Populate_Lexical_Env (Unit.AST_Root);
      Context.In_Populate_Lexical_Env :=
         Saved_In_Populate_Lexical_Env;
      GNATCOLL.Traces.Decrease_Indent (Main_Trace);
      GNATCOLL.Traces.Trace
        (Main_Trace,
         "Finished populating lexical envs for unit: " & Basename (Unit));
      Reset_Envs_Caches (Unit);
      if Has_Errors and then not Context.Discard_Errors_In_Populate_Lexical_Env
      then
         raise Property_Error with
            "errors occurred in Populate_Lexical_Env";
      end if;
   end Populate_Lexical_Env;
   ------------------
   -- Get_Filename --
   ------------------
   function Get_Filename (Unit : Internal_Unit) return String is
     (+Unit.Filename.Full_Name);
   -----------------
   -- Get_Charset --
   -----------------
   function Get_Charset (Unit : Internal_Unit) return String is
   begin
      return To_String (Unit.Charset);
   end Get_Charset;
   ---------------------
   -- Has_Diagnostics --
   ---------------------
   function Has_Diagnostics (Unit : Internal_Unit) return Boolean is
   begin
      return not Unit.Diagnostics.Is_Empty;
   end Has_Diagnostics;
   -----------------
   -- Diagnostics --
   -----------------
   function Diagnostics (Unit : Internal_Unit) return Diagnostics_Array is
      Result : Diagnostics_Array (1 .. Natural (Unit.Diagnostics.Length));
      I      : Natural := 1;
   begin
      for D of Unit.Diagnostics loop
         Result (I) := D;
         I := I + 1;
      end loop;
      return Result;
   end Diagnostics;
   ---------------------------
   -- Format_GNU_Diagnostic --
   ---------------------------
   function Format_GNU_Diagnostic
     (Unit : Internal_Unit; D : Diagnostic) return String
   is
      Filename : constant String := Basename (Unit);
      Sloc     : constant Source_Location := Start_Sloc (D.Sloc_Range);
      Msg      : constant String :=
         Image
           (Ada.Strings.Wide_Wide_Unbounded.To_Wide_Wide_String (D.Message));
   begin
      return (Filename
              & (if Sloc = No_Source_Location then "" else ":" & Image (Sloc))
              & ": " & Msg);
   end Format_GNU_Diagnostic;
   ----------
   -- Root --
   ----------
   function Root (Unit : Internal_Unit) return Bare_Ada_Node is
     (Unit.AST_Root);
   -----------------
   -- First_Token --
   -----------------
   function First_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.TDH'Access,
                            First_Token_Or_Trivia (Unit.TDH)));
   ----------------
   -- Last_Token --
   ----------------
   function Last_Token (Unit : Internal_Unit) return Token_Reference is
     (Wrap_Token_Reference (Unit.TDH'Access, Last_Token_Or_Trivia (Unit.TDH)));
   -----------------
   -- Token_Count --
   -----------------
   function Token_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Tokens.Length);
   ------------------
   -- Trivia_Count --
   ------------------
   function Trivia_Count (Unit : Internal_Unit) return Natural is
     (Unit.TDH.Trivias.Length);
   ----------
   -- Text --
   ----------
   function Text (Unit : Internal_Unit) return Text_Type is
   begin
      return Text (First_Token (Unit), Last_Token (Unit));
   end Text;
   ------------------
   -- Lookup_Token --
   ------------------
   function Lookup_Token
     (Unit : Internal_Unit; Sloc : Source_Location) return Token_Reference
   is
      Result : constant Token_Or_Trivia_Index := Lookup_Token (Unit.TDH, Sloc);
   begin
      return Wrap_Token_Reference (Unit.TDH'Access, Result);
   end Lookup_Token;
   ----------------------
   -- Dump_Lexical_Env --
   ----------------------
   procedure Dump_Lexical_Env (Unit : Internal_Unit) is
      Node     : constant Bare_Ada_Node := Unit.AST_Root;
      Root_Env : constant Lexical_Env := Unit.Context.Root_Scope;
      State    : Dump_Lexical_Env_State := (Root_Env => Root_Env, others => <>);
      ----------------
      -- Get_Parent --
      ----------------
      function Get_Parent (Env : Lexical_Env) return Lexical_Env is
         E : constant Lexical_Env_Access := Unwrap (Env);
      begin
         return Get_Env (E.Parent, No_Entity_Info);
      end Get_Parent;
      --------------------------
      -- Explore_Parent_Chain --
      --------------------------
      procedure Explore_Parent_Chain (Env : Lexical_Env) is
         P : Lexical_Env;
      begin
         if Env /= Null_Lexical_Env then
            P := Get_Parent (Env);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (P, State));
            Explore_Parent_Chain (P);
         end if;
      end Explore_Parent_Chain;
      --------------
      -- Internal --
      --------------
      procedure Internal (Current : Bare_Ada_Node) is
         Explore_Parent : Boolean := False;
         Env, Parent    : Lexical_Env;
      begin
         if Current = null then
            return;
         end if;
         --  We only dump environments that we haven't dumped before. This way
         --  we'll only dump environments at the site of their creation, and
         --  not in any subsequent link. We use the Env_Ids map to check which
         --  envs we have already seen or not.
         if not State.Env_Ids.Contains (Current.Self_Env) then
            Env := Current.Self_Env;
            Parent := Get_Parent (Env);
            Explore_Parent := not State.Env_Ids.Contains (Parent);
            Dump_One_Lexical_Env
              (Env, Get_Env_Id (Env, State), Get_Env_Id (Parent, State));
            if Explore_Parent then
               Explore_Parent_Chain (Parent);
            end if;
         end if;
         for Child of Internal_Bare_Ada_Node_Array'(Children (Current))
         loop
            Internal (Child);
         end loop;
      end Internal;
      --  This procedure implements the main recursive logic of dumping the
      --  environments.
   begin
      Internal (Bare_Ada_Node (Node));
   end Dump_Lexical_Env;
   --------------
   -- Get_Line --
   --------------
   function Get_Line
     (Unit : Internal_Unit; Line_Number : Positive) return Text_Type
   is
   begin
      return Get_Line (Unit.TDH, Line_Number);
   end Get_Line;
   -----------
   -- Print --
   -----------
   procedure Print (Unit : Internal_Unit; Show_Slocs : Boolean) is
   begin
      if Unit.AST_Root = null then
         Put_Line ("<empty analysis unit>");
      else
         Print (Unit.AST_Root, Show_Slocs);
      end if;
   end Print;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia (Unit : Internal_Unit) is
      procedure Process (Trivia : Token_Index) is
         Data : constant Stored_Token_Data :=
            Unit.TDH.Trivias.Get (Natural (Trivia)).T;
      begin
         Put_Line (Image (Text (Unit.TDH, Data)));
      end Process;
      Last_Token : constant Token_Index :=
         Token_Index (Token_Vectors.Last_Index (Unit.TDH.Tokens) - 1);
      --  Index for the last token in Unit excluding the Termination token
      --  (hence the -1).
   begin
      for Tok of Get_Leading_Trivias (Unit.TDH) loop
         Process (Tok);
      end loop;
      PP_Trivia (Unit.AST_Root);
      for Tok of Get_Trivias (Unit.TDH, Last_Token) loop
         Process (Tok);
      end loop;
   end PP_Trivia;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Unit : in out Internal_Unit) is
   begin
      if Unit = No_Analysis_Unit then
         return;
      end if;
      Unit.Exiled_Entries.Destroy;
      Unit.Foreign_Nodes.Destroy;
      Unit.Exiled_Entries_In_NED.Destroy;
      Unit.Exiled_Envs.Destroy;
      Unit.Named_Envs.Destroy;
      Analysis_Unit_Sets.Destroy (Unit.Referenced_Units);
         Destroy (Unit.Memoization_Map);
      Destroy_Rebindings (Unit.Rebindings'Access);
      Unit.Rebindings.Destroy;
      if Unit.AST_Root /= null then
         Destroy (Unit.AST_Root);
      end if;
      Free (Unit.TDH);
      Free (Unit.AST_Mem_Pool);
      Destroy_Unit_Destroyables (Unit);
      Destroyable_Vectors.Destroy (Unit.Destroyables);
      Free (Unit);
   end Destroy;
   -------------------
   -- Is_Token_Node --
   -------------------
   function Is_Token_Node (Node : Bare_Ada_Node) return Boolean is
   begin
      return Is_Token_Node (Node.Kind);
   end Is_Token_Node;
   ------------------
   -- Is_Synthetic --
   ------------------
   function Is_Synthetic (Node : Bare_Ada_Node) return Boolean is
   begin
      return Node.Kind in Synthetic_Nodes;
   end Is_Synthetic;
   ------------------------------
   -- Register_Destroyable_Gen --
   ------------------------------
   procedure Register_Destroyable_Gen
     (Unit : Internal_Unit; Object : T_Access)
   is
      function Convert is new Ada.Unchecked_Conversion
        (System.Address, Destroy_Procedure);
      procedure Destroy_Procedure (Object : in out T_Access) renames Destroy;
   begin
      Register_Destroyable_Helper
        (Unit,
         Object.all'Address,
         Convert (Destroy_Procedure'Address));
   end Register_Destroyable_Gen;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Ada_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Absolute_Get
        (T : Bare_Ada_Node_Array_Access; Index : Integer)
         return Bare_Ada_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Ada_Node_Array_Access) return Bare_Ada_Node_Array_Access is
      Ret : Bare_Ada_Node_Array_Access := Create_Bare_Ada_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Ada_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Ada_Node_Array (Items_Count : Natural) return Bare_Ada_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Ada_Node_Array_Type
       else new Bare_Ada_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Ada_Node_Array
     (Items : Internal_Bare_Ada_Node_Array) return Bare_Ada_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Ada_Node_Array_Type;
      end if;
      return new Bare_Ada_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Ada_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Anonymous_Expr_Decl_Vectors is new Langkit_Support.Vectors (Bare_Anonymous_Expr_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Anonymous_Expr_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Anonymous_Expr_Decl
   is
      function Absolute_Get
        (T : Bare_Anonymous_Expr_Decl_Array_Access; Index : Integer)
         return Bare_Anonymous_Expr_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Anonymous_Expr_Decl,
         Sequence_Type => Bare_Anonymous_Expr_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Anonymous_Expr_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Bare_Anonymous_Expr_Decl_Array_Access is
      Ret : Bare_Anonymous_Expr_Decl_Array_Access := Create_Bare_Anonymous_Expr_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Anonymous_Expr_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Anonymous_Expr_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Anonymous_Expr_Decl_Array (Items_Count : Natural) return Bare_Anonymous_Expr_Decl_Array_Access
   is (if Items_Count = 0
       then No_Bare_Anonymous_Expr_Decl_Array_Type
       else new Bare_Anonymous_Expr_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Anonymous_Expr_Decl_Array
     (Items : Internal_Bare_Anonymous_Expr_Decl_Array) return Bare_Anonymous_Expr_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Anonymous_Expr_Decl_Array_Type;
      end if;
      return new Bare_Anonymous_Expr_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Anonymous_Expr_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Anonymous_Expr_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Bare_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Compilation_Unit
   is
      function Absolute_Get
        (T : Bare_Compilation_Unit_Array_Access; Index : Integer)
         return Bare_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Compilation_Unit,
         Sequence_Type => Bare_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Compilation_Unit_Array_Access) return Bare_Compilation_Unit_Array_Access is
      Ret : Bare_Compilation_Unit_Array_Access := Create_Bare_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Compilation_Unit_Array (Items_Count : Natural) return Bare_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Bare_Compilation_Unit_Array_Type
       else new Bare_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Compilation_Unit_Array
     (Items : Internal_Bare_Compilation_Unit_Array) return Bare_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Compilation_Unit_Array_Type;
      end if;
      return new Bare_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Compilation_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Name_Vectors is new Langkit_Support.Vectors (Bare_Name);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Name
   is
      function Absolute_Get
        (T : Bare_Name_Array_Access; Index : Integer)
         return Bare_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Name,
         Sequence_Type => Bare_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Name_Array_Access) return Bare_Name_Array_Access is
      Ret : Bare_Name_Array_Access := Create_Bare_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Name_Array (Items_Count : Natural) return Bare_Name_Array_Access
   is (if Items_Count = 0
       then No_Bare_Name_Array_Type
       else new Bare_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Name_Array
     (Items : Internal_Bare_Name_Array) return Bare_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Name_Array_Type;
      end if;
      return new Bare_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Name_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Single_Tok_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Single_Tok_Node
   is
      function Absolute_Get
        (T : Bare_Single_Tok_Node_Array_Access; Index : Integer)
         return Bare_Single_Tok_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Single_Tok_Node,
         Sequence_Type => Bare_Single_Tok_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Single_Tok_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Single_Tok_Node_Array_Access) return Bare_Single_Tok_Node_Array_Access is
      Ret : Bare_Single_Tok_Node_Array_Access := Create_Bare_Single_Tok_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Single_Tok_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Single_Tok_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Single_Tok_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Single_Tok_Node_Array (Items_Count : Natural) return Bare_Single_Tok_Node_Array_Access
   is (if Items_Count = 0
       then No_Bare_Single_Tok_Node_Array_Type
       else new Bare_Single_Tok_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Single_Tok_Node_Array
     (Items : Internal_Bare_Single_Tok_Node_Array) return Bare_Single_Tok_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Single_Tok_Node_Array_Type;
      end if;
      return new Bare_Single_Tok_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Single_Tok_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Single_Tok_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Bare_Use_Clause_Vectors is new Langkit_Support.Vectors (Bare_Use_Clause);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Bare_Use_Clause_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Use_Clause
   is
      function Absolute_Get
        (T : Bare_Use_Clause_Array_Access; Index : Integer)
         return Bare_Use_Clause
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Use_Clause,
         Sequence_Type => Bare_Use_Clause_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Use_Clause;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Bare_Ada_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Bare_Use_Clause_Array_Access) return Bare_Use_Clause_Array_Access is
      Ret : Bare_Use_Clause_Array_Access := Create_Bare_Use_Clause_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Use_Clause_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Bare_Use_Clause_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Use_Clause_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Bare_Use_Clause_Array (Items_Count : Natural) return Bare_Use_Clause_Array_Access
   is (if Items_Count = 0
       then No_Bare_Use_Clause_Array_Type
       else new Bare_Use_Clause_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Bare_Use_Clause_Array
     (Items : Internal_Bare_Use_Clause_Array) return Bare_Use_Clause_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Bare_Use_Clause_Array_Type;
      end if;
      return new Bare_Use_Clause_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Bare_Use_Clause_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Use_Clause_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Character_Type_Vectors is new Langkit_Support.Vectors (Character_Type);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Character_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Character_Type
   is
      function Absolute_Get
        (T : Character_Type_Array_Access; Index : Integer)
         return Character_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Character_Type,
         Sequence_Type => Character_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Character_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return Chars.NUL;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Character_Type_Array_Access) return Character_Type_Array_Access is
      Ret : Character_Type_Array_Access := Create_Character_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Character_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Character_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Character_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Character_Type_Array (Items_Count : Natural) return Character_Type_Array_Access
   is (if Items_Count = 0
       then No_Character_Type_Array_Type
       else new Character_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Character_Type_Array
     (Items : Text_Type) return Character_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Character_Type_Array_Type;
      end if;
      return new Character_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
      function Create_Character_Type_Array
        (Items : Unbounded_Text_Type) return Character_Type_Array_Access
      is
         Result : constant Character_Type_Array_Access :=
            Create_Character_Type_Array (Length (Items));
         S      : Big_Wide_Wide_String_Access;
         L      : Natural;
      begin
         Get_Wide_Wide_String (Items, S, L);
         Result.Items (1 .. L) := S.all (1 .. L);
         return Result;
      end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Character_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Character_Type_Array_Access) return String is
            begin
               return Image (A.Items);
      end Trace_Image;
      package Character_Type_Array_Access_Vectors is new Langkit_Support.Vectors (Character_Type_Array_Access);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Character_Type_Array_Access_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Character_Type_Array_Access
   is
      function Absolute_Get
        (T : Character_Type_Array_Access_Array_Access; Index : Integer)
         return Character_Type_Array_Access
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Character_Type_Array_Access,
         Sequence_Type => Character_Type_Array_Access_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Character_Type_Array_Access;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Character_Type_Array_Type;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Character_Type_Array_Access_Array_Access) return Character_Type_Array_Access_Array_Access is
      Ret : Character_Type_Array_Access_Array_Access := Create_Character_Type_Array_Access_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Character_Type_Array_Access_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Character_Type_Array_Access_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Character_Type_Array_Access_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Character_Type_Array_Access_Array (Items_Count : Natural) return Character_Type_Array_Access_Array_Access
   is (if Items_Count = 0
       then No_Character_Type_Array_Access_Array_Type
       else new Character_Type_Array_Access_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Character_Type_Array_Access_Array
     (Items : Internal_Character_Type_Array_Access_Array) return Character_Type_Array_Access_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Character_Type_Array_Access_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Character_Type_Array_Access_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Character_Type_Array_Access_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Character_Type_Array_Access_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Aspect_Vectors is new Langkit_Support.Vectors (Internal_Aspect);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Aspect_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Aspect
   is
      function Absolute_Get
        (T : Internal_Aspect_Array_Access; Index : Integer)
         return Internal_Aspect
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Aspect,
         Sequence_Type => Internal_Aspect_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Aspect;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Aspect;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Aspect_Array_Access) return Internal_Aspect_Array_Access is
      Ret : Internal_Aspect_Array_Access := Create_Internal_Aspect_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Aspect_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Aspect_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Aspect_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Aspect_Array (Items_Count : Natural) return Internal_Aspect_Array_Access
   is (if Items_Count = 0
       then No_Internal_Aspect_Array_Type
       else new Internal_Aspect_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Aspect_Array
     (Items : Internal_Internal_Aspect_Array) return Internal_Aspect_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Aspect_Array_Type;
      end if;
      return new Internal_Aspect_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Aspect_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Aspect_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Completion_Item_Vectors is new Langkit_Support.Vectors (Internal_Completion_Item);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Completion_Item_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Completion_Item
   is
      function Absolute_Get
        (T : Internal_Completion_Item_Array_Access; Index : Integer)
         return Internal_Completion_Item
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Completion_Item,
         Sequence_Type => Internal_Completion_Item_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Completion_Item;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Completion_Item;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Completion_Item_Array_Access) return Internal_Completion_Item_Array_Access is
      Ret : Internal_Completion_Item_Array_Access := Create_Internal_Completion_Item_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Completion_Item_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Completion_Item_Array (Items_Count : Natural) return Internal_Completion_Item_Array_Access
   is (if Items_Count = 0
       then No_Internal_Completion_Item_Array_Type
       else new Internal_Completion_Item_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Completion_Item_Array
     (Items : Internal_Internal_Completion_Item_Array) return Internal_Completion_Item_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Completion_Item_Array_Type;
      end if;
      return new Internal_Completion_Item_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Completion_Item_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- To_Iterator --
      -----------------
      function To_Iterator
        (Self    : Internal_Completion_Item_Array_Access;
         Context : Internal_Context) return Internal_Completion_Item_Iterator_Access
      is
      begin
         Inc_Ref (Self);
         return new Internal_Internal_Completion_Item_Iterator'
           (Ref_Count  => 1,
            Safety_Net => Create_Safety_Net (Context),
            Elements   => Self,
            Index      => 1);
      end To_Iterator;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Discriminant_Values_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Discriminant_Values
   is
      function Absolute_Get
        (T : Internal_Discriminant_Values_Array_Access; Index : Integer)
         return Internal_Discriminant_Values
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Discriminant_Values,
         Sequence_Type => Internal_Discriminant_Values_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Discriminant_Values;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Discriminant_Values;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Discriminant_Values_Array_Access) return Internal_Discriminant_Values_Array_Access is
      Ret : Internal_Discriminant_Values_Array_Access := Create_Internal_Discriminant_Values_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Discriminant_Values_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Discriminant_Values_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Discriminant_Values_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Discriminant_Values_Array (Items_Count : Natural) return Internal_Discriminant_Values_Array_Access
   is (if Items_Count = 0
       then No_Internal_Discriminant_Values_Array_Type
       else new Internal_Discriminant_Values_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Discriminant_Values_Array
     (Items : Internal_Internal_Discriminant_Values_Array) return Internal_Discriminant_Values_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Discriminant_Values_Array_Type;
      end if;
      return new Internal_Discriminant_Values_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Discriminant_Values_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Discriminant_Values_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Doc_Annotation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Doc_Annotation
   is
      function Absolute_Get
        (T : Internal_Doc_Annotation_Array_Access; Index : Integer)
         return Internal_Doc_Annotation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Doc_Annotation,
         Sequence_Type => Internal_Doc_Annotation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Doc_Annotation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Doc_Annotation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Doc_Annotation_Array_Access) return Internal_Doc_Annotation_Array_Access is
      Ret : Internal_Doc_Annotation_Array_Access := Create_Internal_Doc_Annotation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Doc_Annotation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Doc_Annotation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Doc_Annotation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Doc_Annotation_Array (Items_Count : Natural) return Internal_Doc_Annotation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Doc_Annotation_Array_Type
       else new Internal_Doc_Annotation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Doc_Annotation_Array
     (Items : Internal_Internal_Doc_Annotation_Array) return Internal_Doc_Annotation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Doc_Annotation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Doc_Annotation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Doc_Annotation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Doc_Annotation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Vectors is new Langkit_Support.Vectors (Internal_Entity);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity
   is
      function Absolute_Get
        (T : Internal_Entity_Array_Access; Index : Integer)
         return Internal_Entity
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity,
         Sequence_Type => Internal_Entity_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Array_Access) return Internal_Entity_Array_Access is
      Ret : Internal_Entity_Array_Access := Create_Internal_Entity_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Array (Items_Count : Natural) return Internal_Entity_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Array
     (Items : AST_Envs.Entity_Array) return Internal_Entity_Array_Access
   is (if Items'Length = 0
       then No_Internal_Entity_Array_Type
       else new Internal_Entity_Array_Record'
         (N         => Items'Length,
          Items     => Implementation.Internal_Internal_Entity_Array (Items),
          Ref_Count => 1));
   function Create_Internal_Entity_Array
     (Items : Internal_Internal_Entity_Array) return Internal_Entity_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Array_Type;
      end if;
      return new Internal_Entity_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Aspect_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Aspect_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Aspect_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Aspect_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Aspect_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Aspect_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Aspect_Assoc,
         Sequence_Type => Internal_Entity_Aspect_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Aspect_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Aspect_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Aspect_Assoc_Array_Access) return Internal_Entity_Aspect_Assoc_Array_Access is
      Ret : Internal_Entity_Aspect_Assoc_Array_Access := Create_Internal_Entity_Aspect_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Aspect_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Aspect_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Aspect_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Aspect_Assoc_Array (Items_Count : Natural) return Internal_Entity_Aspect_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Aspect_Assoc_Array_Type
       else new Internal_Entity_Aspect_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Aspect_Assoc_Array
     (Items : Internal_Internal_Entity_Aspect_Assoc_Array) return Internal_Entity_Aspect_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Aspect_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Aspect_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Aspect_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Aspect_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Formal_Param_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Formal_Param_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Formal_Param_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Formal_Param_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Formal_Param_Decl,
         Sequence_Type => Internal_Entity_Base_Formal_Param_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Formal_Param_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Formal_Param_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
      Ret : Internal_Entity_Base_Formal_Param_Decl_Array_Access := Create_Internal_Entity_Base_Formal_Param_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Formal_Param_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Formal_Param_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Formal_Param_Decl_Array_Type
       else new Internal_Entity_Base_Formal_Param_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Formal_Param_Decl_Array
     (Items : Internal_Internal_Entity_Base_Formal_Param_Decl_Array) return Internal_Entity_Base_Formal_Param_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Formal_Param_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Formal_Param_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Base_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Base_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Base_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Base_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Base_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Base_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Base_Type_Decl,
         Sequence_Type => Internal_Entity_Base_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Base_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Base_Type_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
      Ret : Internal_Entity_Base_Type_Decl_Array_Access := Create_Internal_Entity_Base_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Base_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Base_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Base_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Base_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Base_Type_Decl_Array_Type
       else new Internal_Entity_Base_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Base_Type_Decl_Array
     (Items : Internal_Internal_Entity_Base_Type_Decl_Array) return Internal_Entity_Base_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Base_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Base_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Base_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Base_Type_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Base_Type_Decl_Array_Access) return Internal_Entity_Base_Type_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Base_Type_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Base_Type_Decl_Array_Access :=
               Create_Internal_Entity_Base_Type_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Base_Type_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Basic_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Basic_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Assoc
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Assoc_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Assoc,
         Sequence_Type => Internal_Entity_Basic_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Assoc_Array_Access) return Internal_Entity_Basic_Assoc_Array_Access is
      Ret : Internal_Entity_Basic_Assoc_Array_Access := Create_Internal_Entity_Basic_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Assoc_Array (Items_Count : Natural) return Internal_Entity_Basic_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Assoc_Array_Type
       else new Internal_Entity_Basic_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Assoc_Array
     (Items : Internal_Internal_Entity_Basic_Assoc_Array) return Internal_Entity_Basic_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Assoc_Array_Type;
      end if;
      return new Internal_Entity_Basic_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Basic_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Basic_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Basic_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Basic_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Basic_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Basic_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Basic_Decl,
         Sequence_Type => Internal_Entity_Basic_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Basic_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Basic_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
      Ret : Internal_Entity_Basic_Decl_Array_Access := Create_Internal_Entity_Basic_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Basic_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Basic_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Basic_Decl_Array (Items_Count : Natural) return Internal_Entity_Basic_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Basic_Decl_Array_Type
       else new Internal_Entity_Basic_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Basic_Decl_Array
     (Items : Internal_Internal_Entity_Basic_Decl_Array) return Internal_Entity_Basic_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Basic_Decl_Array_Type;
      end if;
      return new Internal_Entity_Basic_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Basic_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Basic_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Basic_Decl_Array_Access) return Internal_Entity_Basic_Decl_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Basic_Decl,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Basic_Decl_Array_Access :=
               Create_Internal_Entity_Basic_Decl_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Basic_Decl renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Internal_Entity_Compilation_Unit_Vectors is new Langkit_Support.Vectors (Internal_Entity_Compilation_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Compilation_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Compilation_Unit
   is
      function Absolute_Get
        (T : Internal_Entity_Compilation_Unit_Array_Access; Index : Integer)
         return Internal_Entity_Compilation_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Compilation_Unit,
         Sequence_Type => Internal_Entity_Compilation_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Compilation_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Compilation_Unit;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
      Ret : Internal_Entity_Compilation_Unit_Array_Access := Create_Internal_Entity_Compilation_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Compilation_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Compilation_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Compilation_Unit_Array (Items_Count : Natural) return Internal_Entity_Compilation_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Compilation_Unit_Array_Type
       else new Internal_Entity_Compilation_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Compilation_Unit_Array
     (Items : Internal_Internal_Entity_Compilation_Unit_Array) return Internal_Entity_Compilation_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Compilation_Unit_Array_Type;
      end if;
      return new Internal_Entity_Compilation_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Compilation_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Compilation_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Entity_Compilation_Unit_Array_Access) return Internal_Entity_Compilation_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Entity_Compilation_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Entity_Compilation_Unit_Array_Access :=
               Create_Internal_Entity_Compilation_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Entity_Compilation_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Declarative_Part_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Declarative_Part
   is
      function Absolute_Get
        (T : Internal_Entity_Declarative_Part_Array_Access; Index : Integer)
         return Internal_Entity_Declarative_Part
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Declarative_Part,
         Sequence_Type => Internal_Entity_Declarative_Part_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Declarative_Part;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Declarative_Part;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Declarative_Part_Array_Access) return Internal_Entity_Declarative_Part_Array_Access is
      Ret : Internal_Entity_Declarative_Part_Array_Access := Create_Internal_Entity_Declarative_Part_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Declarative_Part_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Declarative_Part_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Declarative_Part_Array (Items_Count : Natural) return Internal_Entity_Declarative_Part_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Declarative_Part_Array_Type
       else new Internal_Entity_Declarative_Part_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Declarative_Part_Array
     (Items : Internal_Internal_Entity_Declarative_Part_Array) return Internal_Entity_Declarative_Part_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Declarative_Part_Array_Type;
      end if;
      return new Internal_Entity_Declarative_Part_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Declarative_Part_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Declarative_Part_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Defining_Name_Vectors is new Langkit_Support.Vectors (Internal_Entity_Defining_Name);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Defining_Name_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Defining_Name
   is
      function Absolute_Get
        (T : Internal_Entity_Defining_Name_Array_Access; Index : Integer)
         return Internal_Entity_Defining_Name
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Defining_Name,
         Sequence_Type => Internal_Entity_Defining_Name_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Defining_Name;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Defining_Name;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Defining_Name_Array_Access) return Internal_Entity_Defining_Name_Array_Access is
      Ret : Internal_Entity_Defining_Name_Array_Access := Create_Internal_Entity_Defining_Name_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Defining_Name_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Defining_Name_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Defining_Name_Array (Items_Count : Natural) return Internal_Entity_Defining_Name_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Defining_Name_Array_Type
       else new Internal_Entity_Defining_Name_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Defining_Name_Array
     (Items : Internal_Internal_Entity_Defining_Name_Array) return Internal_Entity_Defining_Name_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Defining_Name_Array_Type;
      end if;
      return new Internal_Entity_Defining_Name_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Defining_Name_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Defining_Name_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Expr,
         Sequence_Type => Internal_Entity_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Expr_Array_Access) return Internal_Entity_Expr_Array_Access is
      Ret : Internal_Entity_Expr_Array_Access := Create_Internal_Entity_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Expr_Array (Items_Count : Natural) return Internal_Entity_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Expr_Array_Type
       else new Internal_Entity_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Expr_Array
     (Items : Internal_Internal_Entity_Expr_Array) return Internal_Entity_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Expr_Array_Type;
      end if;
      return new Internal_Entity_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Generic_Instantiation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Generic_Instantiation
   is
      function Absolute_Get
        (T : Internal_Entity_Generic_Instantiation_Array_Access; Index : Integer)
         return Internal_Entity_Generic_Instantiation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Generic_Instantiation,
         Sequence_Type => Internal_Entity_Generic_Instantiation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Generic_Instantiation;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Generic_Instantiation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Internal_Entity_Generic_Instantiation_Array_Access is
      Ret : Internal_Entity_Generic_Instantiation_Array_Access := Create_Internal_Entity_Generic_Instantiation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Generic_Instantiation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Generic_Instantiation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Generic_Instantiation_Array (Items_Count : Natural) return Internal_Entity_Generic_Instantiation_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Generic_Instantiation_Array_Type
       else new Internal_Entity_Generic_Instantiation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Generic_Instantiation_Array
     (Items : Internal_Internal_Entity_Generic_Instantiation_Array) return Internal_Entity_Generic_Instantiation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Generic_Instantiation_Array_Type;
      end if;
      return new Internal_Entity_Generic_Instantiation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Generic_Instantiation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Generic_Instantiation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Param_Spec_Vectors is new Langkit_Support.Vectors (Internal_Entity_Param_Spec);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Param_Spec_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Param_Spec
   is
      function Absolute_Get
        (T : Internal_Entity_Param_Spec_Array_Access; Index : Integer)
         return Internal_Entity_Param_Spec
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Param_Spec,
         Sequence_Type => Internal_Entity_Param_Spec_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Param_Spec;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Param_Spec;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Param_Spec_Array_Access) return Internal_Entity_Param_Spec_Array_Access is
      Ret : Internal_Entity_Param_Spec_Array_Access := Create_Internal_Entity_Param_Spec_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Param_Spec_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Param_Spec_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Param_Spec_Array (Items_Count : Natural) return Internal_Entity_Param_Spec_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Param_Spec_Array_Type
       else new Internal_Entity_Param_Spec_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Param_Spec_Array
     (Items : Internal_Internal_Entity_Param_Spec_Array) return Internal_Entity_Param_Spec_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Param_Spec_Array_Type;
      end if;
      return new Internal_Entity_Param_Spec_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Param_Spec_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Param_Spec_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Pragma_Node_Vectors is new Langkit_Support.Vectors (Internal_Entity_Pragma_Node);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Pragma_Node_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Pragma_Node
   is
      function Absolute_Get
        (T : Internal_Entity_Pragma_Node_Array_Access; Index : Integer)
         return Internal_Entity_Pragma_Node
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Pragma_Node,
         Sequence_Type => Internal_Entity_Pragma_Node_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Pragma_Node;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Pragma_Node;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Pragma_Node_Array_Access) return Internal_Entity_Pragma_Node_Array_Access is
      Ret : Internal_Entity_Pragma_Node_Array_Access := Create_Internal_Entity_Pragma_Node_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Pragma_Node_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Pragma_Node_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Pragma_Node_Array (Items_Count : Natural) return Internal_Entity_Pragma_Node_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Pragma_Node_Array_Type
       else new Internal_Entity_Pragma_Node_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Pragma_Node_Array
     (Items : Internal_Internal_Entity_Pragma_Node_Array) return Internal_Entity_Pragma_Node_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Pragma_Node_Array_Type;
      end if;
      return new Internal_Entity_Pragma_Node_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Pragma_Node_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Pragma_Node_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Decl_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Decl);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Type_Decl_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Decl
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Decl_Array_Access; Index : Integer)
         return Internal_Entity_Type_Decl
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Decl,
         Sequence_Type => Internal_Entity_Type_Decl_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Decl;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Decl;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Decl_Array_Access) return Internal_Entity_Type_Decl_Array_Access is
      Ret : Internal_Entity_Type_Decl_Array_Access := Create_Internal_Entity_Type_Decl_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Decl_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Decl_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Decl_Array (Items_Count : Natural) return Internal_Entity_Type_Decl_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Decl_Array_Type
       else new Internal_Entity_Type_Decl_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Decl_Array
     (Items : Internal_Internal_Entity_Type_Decl_Array) return Internal_Entity_Type_Decl_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Decl_Array_Type;
      end if;
      return new Internal_Entity_Type_Decl_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Decl_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Decl_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Type_Expr_Vectors is new Langkit_Support.Vectors (Internal_Entity_Type_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Type_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Type_Expr
   is
      function Absolute_Get
        (T : Internal_Entity_Type_Expr_Array_Access; Index : Integer)
         return Internal_Entity_Type_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Type_Expr,
         Sequence_Type => Internal_Entity_Type_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Type_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Type_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Type_Expr_Array_Access) return Internal_Entity_Type_Expr_Array_Access is
      Ret : Internal_Entity_Type_Expr_Array_Access := Create_Internal_Entity_Type_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Type_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Type_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Type_Expr_Array (Items_Count : Natural) return Internal_Entity_Type_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Type_Expr_Array_Type
       else new Internal_Entity_Type_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Type_Expr_Array
     (Items : Internal_Internal_Entity_Type_Expr_Array) return Internal_Entity_Type_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Type_Expr_Array_Type;
      end if;
      return new Internal_Entity_Type_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Type_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Type_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Entity_Variant_Vectors is new Langkit_Support.Vectors (Internal_Entity_Variant);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Entity_Variant_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Entity_Variant
   is
      function Absolute_Get
        (T : Internal_Entity_Variant_Array_Access; Index : Integer)
         return Internal_Entity_Variant
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Entity_Variant,
         Sequence_Type => Internal_Entity_Variant_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Entity_Variant;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Entity_Variant;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Entity_Variant_Array_Access) return Internal_Entity_Variant_Array_Access is
      Ret : Internal_Entity_Variant_Array_Access := Create_Internal_Entity_Variant_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Variant_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Entity_Variant_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Variant_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Entity_Variant_Array (Items_Count : Natural) return Internal_Entity_Variant_Array_Access
   is (if Items_Count = 0
       then No_Internal_Entity_Variant_Array_Type
       else new Internal_Entity_Variant_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Entity_Variant_Array
     (Items : Internal_Internal_Entity_Variant_Array) return Internal_Entity_Variant_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Entity_Variant_Array_Type;
      end if;
      return new Internal_Entity_Variant_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Entity_Variant_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Variant_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Env_Assoc,
         Sequence_Type => Internal_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Env_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Env_Assoc_Array_Access) return Internal_Env_Assoc_Array_Access is
      Ret : Internal_Env_Assoc_Array_Access := Create_Internal_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Env_Assoc_Array (Items_Count : Natural) return Internal_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Env_Assoc_Array_Type
       else new Internal_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Env_Assoc_Array
     (Items : Internal_Internal_Env_Assoc_Array) return Internal_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Env_Assoc_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Env_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Expected_Type_For_Expr_Vectors is new Langkit_Support.Vectors (Internal_Expected_Type_For_Expr);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Expected_Type_For_Expr_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Expected_Type_For_Expr
   is
      function Absolute_Get
        (T : Internal_Expected_Type_For_Expr_Array_Access; Index : Integer)
         return Internal_Expected_Type_For_Expr
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Expected_Type_For_Expr,
         Sequence_Type => Internal_Expected_Type_For_Expr_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Expected_Type_For_Expr;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Expected_Type_For_Expr;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Internal_Expected_Type_For_Expr_Array_Access is
      Ret : Internal_Expected_Type_For_Expr_Array_Access := Create_Internal_Expected_Type_For_Expr_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Expected_Type_For_Expr_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Expected_Type_For_Expr_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Expected_Type_For_Expr_Array (Items_Count : Natural) return Internal_Expected_Type_For_Expr_Array_Access
   is (if Items_Count = 0
       then No_Internal_Expected_Type_For_Expr_Array_Type
       else new Internal_Expected_Type_For_Expr_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Expected_Type_For_Expr_Array
     (Items : Internal_Internal_Expected_Type_For_Expr_Array) return Internal_Expected_Type_For_Expr_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Expected_Type_For_Expr_Array_Type;
      end if;
      return new Internal_Expected_Type_For_Expr_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Expected_Type_For_Expr_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Expected_Type_For_Expr_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Inner_Env_Assoc_Vectors is new Langkit_Support.Vectors (Internal_Inner_Env_Assoc);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Inner_Env_Assoc_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Inner_Env_Assoc
   is
      function Absolute_Get
        (T : Internal_Inner_Env_Assoc_Array_Access; Index : Integer)
         return Internal_Inner_Env_Assoc
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Inner_Env_Assoc,
         Sequence_Type => Internal_Inner_Env_Assoc_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Inner_Env_Assoc;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Inner_Env_Assoc;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Inner_Env_Assoc_Array_Access) return Internal_Inner_Env_Assoc_Array_Access is
      Ret : Internal_Inner_Env_Assoc_Array_Access := Create_Internal_Inner_Env_Assoc_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Inner_Env_Assoc_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Inner_Env_Assoc_Array (Items_Count : Natural) return Internal_Inner_Env_Assoc_Array_Access
   is (if Items_Count = 0
       then No_Internal_Inner_Env_Assoc_Array_Type
       else new Internal_Inner_Env_Assoc_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Inner_Env_Assoc_Array
     (Items : Internal_Internal_Inner_Env_Assoc_Array) return Internal_Inner_Env_Assoc_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Inner_Env_Assoc_Array_Type;
      end if;
      return new Internal_Inner_Env_Assoc_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Inner_Env_Assoc_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Param_Actual_Vectors is new Langkit_Support.Vectors (Internal_Param_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Param_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Actual
   is
      function Absolute_Get
        (T : Internal_Param_Actual_Array_Access; Index : Integer)
         return Internal_Param_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Actual,
         Sequence_Type => Internal_Param_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Actual;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Actual_Array_Access) return Internal_Param_Actual_Array_Access is
      Ret : Internal_Param_Actual_Array_Access := Create_Internal_Param_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Actual_Array (Items_Count : Natural) return Internal_Param_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Actual_Array_Type
       else new Internal_Param_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Actual_Array
     (Items : Internal_Internal_Param_Actual_Array) return Internal_Param_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Actual_Array_Type;
      end if;
      return new Internal_Param_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Actual_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Param_Match_Vectors is new Langkit_Support.Vectors (Internal_Param_Match);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Param_Match_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Param_Match
   is
      function Absolute_Get
        (T : Internal_Param_Match_Array_Access; Index : Integer)
         return Internal_Param_Match
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Param_Match,
         Sequence_Type => Internal_Param_Match_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Param_Match;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Param_Match;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Param_Match_Array_Access) return Internal_Param_Match_Array_Access is
      Ret : Internal_Param_Match_Array_Access := Create_Internal_Param_Match_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Param_Match_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Param_Match_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Param_Match_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Param_Match_Array (Items_Count : Natural) return Internal_Param_Match_Array_Access
   is (if Items_Count = 0
       then No_Internal_Param_Match_Array_Type
       else new Internal_Param_Match_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Param_Match_Array
     (Items : Internal_Internal_Param_Match_Array) return Internal_Param_Match_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Param_Match_Array_Type;
      end if;
      return new Internal_Param_Match_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Param_Match_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Param_Match_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Ref_Result_Vectors is new Langkit_Support.Vectors (Internal_Ref_Result);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Ref_Result_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Ref_Result
   is
      function Absolute_Get
        (T : Internal_Ref_Result_Array_Access; Index : Integer)
         return Internal_Ref_Result
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Ref_Result,
         Sequence_Type => Internal_Ref_Result_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Ref_Result;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Ref_Result;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Ref_Result_Array_Access) return Internal_Ref_Result_Array_Access is
      Ret : Internal_Ref_Result_Array_Access := Create_Internal_Ref_Result_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Ref_Result_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Ref_Result_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Ref_Result_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Ref_Result_Array (Items_Count : Natural) return Internal_Ref_Result_Array_Access
   is (if Items_Count = 0
       then No_Internal_Ref_Result_Array_Type
       else new Internal_Ref_Result_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Ref_Result_Array
     (Items : Internal_Internal_Ref_Result_Array) return Internal_Ref_Result_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Ref_Result_Array_Type;
      end if;
      return new Internal_Ref_Result_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Ref_Result_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Ref_Result_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Shape_Vectors is new Langkit_Support.Vectors (Internal_Shape);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Shape_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Shape
   is
      function Absolute_Get
        (T : Internal_Shape_Array_Access; Index : Integer)
         return Internal_Shape
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Shape,
         Sequence_Type => Internal_Shape_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Shape;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Shape;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Shape_Array_Access) return Internal_Shape_Array_Access is
      Ret : Internal_Shape_Array_Access := Create_Internal_Shape_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Shape_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Shape_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Shape_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Shape_Array (Items_Count : Natural) return Internal_Shape_Array_Access
   is (if Items_Count = 0
       then No_Internal_Shape_Array_Type
       else new Internal_Shape_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Shape_Array
     (Items : Internal_Internal_Shape_Array) return Internal_Shape_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Shape_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Shape_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Shape_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Shape_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Single_Actual_Vectors is new Langkit_Support.Vectors (Internal_Single_Actual);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Single_Actual_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Actual
   is
      function Absolute_Get
        (T : Internal_Single_Actual_Array_Access; Index : Integer)
         return Internal_Single_Actual
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Actual,
         Sequence_Type => Internal_Single_Actual_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Actual;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Actual;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Actual_Array_Access) return Internal_Single_Actual_Array_Access is
      Ret : Internal_Single_Actual_Array_Access := Create_Internal_Single_Actual_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Actual_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Actual_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Actual_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Actual_Array (Items_Count : Natural) return Internal_Single_Actual_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Actual_Array_Type
       else new Internal_Single_Actual_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Actual_Array
     (Items : Internal_Internal_Single_Actual_Array) return Internal_Single_Actual_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Actual_Array_Type;
      end if;
      return new Internal_Single_Actual_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Actual_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Actual_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Single_Formal_Vectors is new Langkit_Support.Vectors (Internal_Single_Formal);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Single_Formal_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Single_Formal
   is
      function Absolute_Get
        (T : Internal_Single_Formal_Array_Access; Index : Integer)
         return Internal_Single_Formal
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Single_Formal,
         Sequence_Type => Internal_Single_Formal_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Single_Formal;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return No_Single_Formal;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Single_Formal_Array_Access) return Internal_Single_Formal_Array_Access is
      Ret : Internal_Single_Formal_Array_Access := Create_Internal_Single_Formal_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Single_Formal_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Single_Formal_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Single_Formal_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Single_Formal_Array (Items_Count : Natural) return Internal_Single_Formal_Array_Access
   is (if Items_Count = 0
       then No_Internal_Single_Formal_Array_Type
       else new Internal_Single_Formal_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Single_Formal_Array
     (Items : Internal_Internal_Single_Formal_Array) return Internal_Single_Formal_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Single_Formal_Array_Type;
      end if;
      return new Internal_Single_Formal_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Single_Formal_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Single_Formal_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Substitution_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Substitution
   is
      function Absolute_Get
        (T : Internal_Substitution_Array_Access; Index : Integer)
         return Internal_Substitution
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Substitution,
         Sequence_Type => Internal_Substitution_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Substitution;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return No_Substitution;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Substitution_Array_Access) return Internal_Substitution_Array_Access is
      Ret : Internal_Substitution_Array_Access := Create_Internal_Substitution_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Substitution_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Substitution_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Substitution_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Substitution_Array (Items_Count : Natural) return Internal_Substitution_Array_Access
   is (if Items_Count = 0
       then No_Internal_Substitution_Array_Type
       else new Internal_Substitution_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Substitution_Array
     (Items : Internal_Internal_Substitution_Array) return Internal_Substitution_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Substitution_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Internal_Substitution_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Substitution_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Substitution_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Internal_Unit_Vectors is new Langkit_Support.Vectors (Internal_Unit);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Internal_Unit_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Internal_Unit
   is
      function Absolute_Get
        (T : Internal_Unit_Array_Access; Index : Integer)
         return Internal_Unit
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Internal_Unit,
         Sequence_Type => Internal_Unit_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Internal_Unit;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
      Ret : Internal_Unit_Array_Access := Create_Internal_Unit_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Unit_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Internal_Unit_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Unit_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Internal_Unit_Array (Items_Count : Natural) return Internal_Unit_Array_Access
   is (if Items_Count = 0
       then No_Internal_Unit_Array_Type
       else new Internal_Unit_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Internal_Unit_Array
     (Items : Internal_Internal_Unit_Array) return Internal_Unit_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Internal_Unit_Array_Type;
      end if;
      return new Internal_Unit_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Internal_Unit_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Unit_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      function Make_Unique (A : Internal_Unit_Array_Access) return Internal_Unit_Array_Access is
         --  To keep the order of the result deterministic, this function works
         --  in two steps. First, determine for each item in A whether it
         --  duplicate a previous item. Only then, go through all items in A
         --  and add only non-duplicated ones to the result.
         Duplicates : array (A.Items'Range) of Boolean := (others => False);
         --  For each item in A, indicate whether the value is a duplicate of a
         --  previous one in the array.
         Unique_Elements : Natural;
         --  Number of unique elements in A
      begin
         --  Step 1: compute Duplicates
         declare
            package Sets is new Ada.Containers.Hashed_Sets
              (Element_Type        => Internal_Unit,
               "="                 => "=",
               Equivalent_Elements => "=",
               Hash                => Hash);
            Set : Sets.Set;
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if Set.Contains (Item) then
                     Duplicates (I) := True;
                  else
                     Set.Insert (Item);
                  end if;
               end;
            end loop;
            Unique_Elements := Natural (Set.Length);
         end;
         --  Step 2: create the result
         declare
            Next   : Positive := 1;
            Result : constant Internal_Unit_Array_Access :=
               Create_Internal_Unit_Array (Unique_Elements);
         begin
            for I in A.Items'Range loop
               declare
                  Item : Internal_Unit renames A.Items (I);
               begin
                  if not Duplicates (I) then
                     Result.Items (Next) := Item;
                     Next := Next + 1;
                  end if;
               end;
            end loop;
            pragma Assert (Next = Result.N + 1);
            return Result;
         end;
      end Make_Unique;
      package Lexical_Env_Vectors is new Langkit_Support.Vectors (Lexical_Env);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Lexical_Env_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Lexical_Env
   is
      function Absolute_Get
        (T : Lexical_Env_Array_Access; Index : Integer)
         return Lexical_Env
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Lexical_Env,
         Sequence_Type => Lexical_Env_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Lexical_Env;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Empty_Env;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Lexical_Env_Array_Access) return Lexical_Env_Array_Access is
      Ret : Lexical_Env_Array_Access := Create_Lexical_Env_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Lexical_Env_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Lexical_Env_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Lexical_Env_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Lexical_Env_Array (Items_Count : Natural) return Lexical_Env_Array_Access
   is (if Items_Count = 0
       then No_Lexical_Env_Array_Type
       else new Lexical_Env_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Lexical_Env_Array
     (Items : Internal_Lexical_Env_Array) return Lexical_Env_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Lexical_Env_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Lexical_Env_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Lexical_Env_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               not Equivalent (L.Items (I), R.Items (I))
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Lexical_Env_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Logic_Equation_Vectors is new Langkit_Support.Vectors (Logic_Equation);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Logic_Equation_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Logic_Equation
   is
      function Absolute_Get
        (T : Logic_Equation_Array_Access; Index : Integer)
         return Logic_Equation
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Logic_Equation,
         Sequence_Type => Logic_Equation_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Logic_Equation;
   begin
      if Relative_Get (T, Index, Result) then
            Inc_Ref (Result);
         return Result;
      elsif Or_Null then
         return Null_Logic_Equation;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Logic_Equation_Array_Access) return Logic_Equation_Array_Access is
      Ret : Logic_Equation_Array_Access := Create_Logic_Equation_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
         for Item of Ret.Items loop
            Inc_Ref (Item);
         end loop;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Logic_Equation_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Logic_Equation_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Logic_Equation_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
            for Item of T.Items loop
               Dec_Ref (Item);
            end loop;
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Logic_Equation_Array (Items_Count : Natural) return Logic_Equation_Array_Access
   is (if Items_Count = 0
       then No_Logic_Equation_Array_Type
       else new Logic_Equation_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Logic_Equation_Array
     (Items : Internal_Logic_Equation_Array) return Logic_Equation_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Logic_Equation_Array_Type;
      end if;
         for El of Items loop
            Inc_Ref (El);
         end loop;
      return new Logic_Equation_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Logic_Equation_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Logic_Equation_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
      package Symbol_Type_Vectors is new Langkit_Support.Vectors (Symbol_Type);
   ---------
   -- Get --
   ---------
   function Get
     (T       : Symbol_Type_Array_Access;
      Index   : Integer;
      Or_Null : Boolean := False) return Symbol_Type
   is
      function Absolute_Get
        (T : Symbol_Type_Array_Access; Index : Integer)
         return Symbol_Type
      is
        (T.Items (Index + 1)); --  T.Items is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Symbol_Type,
         Sequence_Type => Symbol_Type_Array_Access,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Symbol_Type;
   begin
      if Relative_Get (T, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   ------------
   -- Concat --
   ------------
   function Concat (L, R : Symbol_Type_Array_Access) return Symbol_Type_Array_Access is
      Ret : Symbol_Type_Array_Access := Create_Symbol_Type_Array (Length (L) + Length (R));
   begin
      Ret.Items := L.Items & R.Items;
      return Ret;
   end Concat;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Symbol_Type_Array_Access) is
   begin
      if T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   ------------
   -- Length --
   ------------
   function Length (T : Symbol_Type_Array_Access) return Natural is (T.N);
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Symbol_Type_Array_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
   function Create_Symbol_Type_Array (Items_Count : Natural) return Symbol_Type_Array_Access
   is (if Items_Count = 0
       then No_Symbol_Type_Array_Type
       else new Symbol_Type_Array_Record'(N => Items_Count, Ref_Count => 1, Items => <>));
   function Create_Symbol_Type_Array
     (Items : Internal_Symbol_Type_Array) return Symbol_Type_Array_Access is
   begin
      if Items'Length = 0 then
         return No_Symbol_Type_Array_Type;
      end if;
      return new Symbol_Type_Array_Record'
        (N => Items'Length, Ref_Count => 1, Items => Items);
   end;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (L, R : Symbol_Type_Array_Access) return Boolean is
   begin
      if L.N /= R.N then
         return False;
      end if;
      for I in L.Items'Range loop
         if
               L.Items (I) /= R.Items (I)
         then
            return False;
         end if;
      end loop;
      return True;
   end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Symbol_Type_Array_Access) return String is
               Result : Unbounded_String;
            begin
               Append (Result, "[");
               for I in A.Items'Range loop
                  if I > A.Items'First then
                     Append (Result, ", ");
                  end if;
                  Append (Result, Trace_Image (A.Items (I)));
               end loop;
               Append (Result, "]");
               return To_String (Result);
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Bare_Ada_Node_Iterator_Access;
      Element : out Bare_Ada_Node) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Bare_Ada_Node_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Bare_Ada_Node_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Bare_Ada_Node_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Completion_Item_Iterator_Access;
      Element : out Internal_Completion_Item) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Completion_Item_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Completion_Item_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Completion_Item_Iterator_Access) return String is
      begin
         return "<Iterator of CompletionItem, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Entity_Iterator_Access;
      Element : out Internal_Entity) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Entity_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Entity_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Entity_Iterator_Access) return String is
      begin
         return "<Iterator of AdaNode.entity, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ----------
   -- Next --
   ----------
   function Next
     (T       : Internal_Inner_Env_Assoc_Iterator_Access;
      Element : out Internal_Inner_Env_Assoc) return Boolean is
   begin
      if T = null then
         raise Property_Error with "null access dereference";
      end if;
      Check_Safety_Net (T.Safety_Net);
      if T.Index > T.Elements.Items'Last then
         return False;
      else
         Element := T.Elements.Items (T.Index);
         T.Index := T.Index + 1;
         return True;
      end if;
   end Next;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (T : Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T /= null and then T.Ref_Count >= 0 then
         T.Ref_Count := T.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (T : in out Internal_Inner_Env_Assoc_Iterator_Access) is
   begin
      if T = null or else T.Ref_Count < 0 then
         return;
      end if;
      if T.Ref_Count = 1 then
         Dec_Ref (T.Elements);
         Free (T);
      else
         T.Ref_Count := T.Ref_Count - 1;
         T := null;
      end if;
   end Dec_Ref;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (A : Internal_Inner_Env_Assoc_Iterator_Access) return String is
      begin
         return "<Iterator of InnerEnvAssoc, index="
                & A.Index'Image & ">";
      end Trace_Image;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Internal_Unit) return Boolean is
   begin
      return Left.Filename < Right.Filename;
   end "<";
function Hash (Key : Mmz_Key_Item) return Hash_Type;
function Equivalent (L, R : Mmz_Key_Item) return Boolean;
procedure Destroy (Key : in out Mmz_Key_Array_Access);
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key_Item) return Boolean is
begin
   if L.Kind /= R.Kind then
      return False;
   end if;
   case L.Kind is
         when Mmz_Bare_Ada_Node =>
               return L.As_Bare_Ada_Node = R.As_Bare_Ada_Node;
         when Mmz_Bare_Anonymous_Expr_Decl =>
               return L.As_Bare_Anonymous_Expr_Decl = R.As_Bare_Anonymous_Expr_Decl;
         when Mmz_Bare_Base_Aggregate =>
               return L.As_Bare_Base_Aggregate = R.As_Bare_Base_Aggregate;
         when Mmz_Bare_Base_Id =>
               return L.As_Bare_Base_Id = R.As_Bare_Base_Id;
         when Mmz_Bare_Base_Subp_Spec =>
               return L.As_Bare_Base_Subp_Spec = R.As_Bare_Base_Subp_Spec;
         when Mmz_Bare_Base_Type_Decl =>
               return L.As_Bare_Base_Type_Decl = R.As_Bare_Base_Type_Decl;
         when Mmz_Bare_Basic_Decl =>
               return L.As_Bare_Basic_Decl = R.As_Bare_Basic_Decl;
         when Mmz_Bare_Body_Node =>
               return L.As_Bare_Body_Node = R.As_Bare_Body_Node;
         when Mmz_Bare_Classwide_Type_Decl =>
               return L.As_Bare_Classwide_Type_Decl = R.As_Bare_Classwide_Type_Decl;
         when Mmz_Bare_Compilation_Unit =>
               return L.As_Bare_Compilation_Unit = R.As_Bare_Compilation_Unit;
         when Mmz_Bare_Declarative_Part =>
               return L.As_Bare_Declarative_Part = R.As_Bare_Declarative_Part;
         when Mmz_Bare_Defining_Name =>
               return L.As_Bare_Defining_Name = R.As_Bare_Defining_Name;
         when Mmz_Bare_End_Name =>
               return L.As_Bare_End_Name = R.As_Bare_End_Name;
         when Mmz_Bare_Enum_Literal_Decl =>
               return L.As_Bare_Enum_Literal_Decl = R.As_Bare_Enum_Literal_Decl;
         when Mmz_Bare_For_Loop_Spec =>
               return L.As_Bare_For_Loop_Spec = R.As_Bare_For_Loop_Spec;
         when Mmz_Bare_For_Loop_Var_Decl =>
               return L.As_Bare_For_Loop_Var_Decl = R.As_Bare_For_Loop_Var_Decl;
         when Mmz_Bare_Generic_Instantiation =>
               return L.As_Bare_Generic_Instantiation = R.As_Bare_Generic_Instantiation;
         when Mmz_Bare_Name =>
               return L.As_Bare_Name = R.As_Bare_Name;
         when Mmz_Bare_Type_Decl =>
               return L.As_Bare_Type_Decl = R.As_Bare_Type_Decl;
         when Mmz_Boolean =>
               return L.As_Boolean = R.As_Boolean;
         when Mmz_Env_Rebindings =>
               return L.As_Env_Rebindings = R.As_Env_Rebindings;
         when Mmz_Integer =>
               return L.As_Integer = R.As_Integer;
         when Mmz_Internal_Entity_Info =>
               return L.As_Internal_Entity_Info = R.As_Internal_Entity_Info;
         when Mmz_Internal_Metadata =>
               return L.As_Internal_Metadata = R.As_Internal_Metadata;
         when Mmz_Lexical_Env =>
               return Equivalent (L.As_Lexical_Env, R.As_Lexical_Env);
         when Mmz_Symbol_Type =>
               return L.As_Symbol_Type = R.As_Symbol_Type;
   end case;
end Equivalent;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key_Item) return Hash_Type is
begin
   case Key.Kind is
         when Mmz_Bare_Ada_Node =>
            return Hash (Key.As_Bare_Ada_Node);
         when Mmz_Bare_Anonymous_Expr_Decl =>
            return Hash (Key.As_Bare_Anonymous_Expr_Decl);
         when Mmz_Bare_Base_Aggregate =>
            return Hash (Key.As_Bare_Base_Aggregate);
         when Mmz_Bare_Base_Id =>
            return Hash (Key.As_Bare_Base_Id);
         when Mmz_Bare_Base_Subp_Spec =>
            return Hash (Key.As_Bare_Base_Subp_Spec);
         when Mmz_Bare_Base_Type_Decl =>
            return Hash (Key.As_Bare_Base_Type_Decl);
         when Mmz_Bare_Basic_Decl =>
            return Hash (Key.As_Bare_Basic_Decl);
         when Mmz_Bare_Body_Node =>
            return Hash (Key.As_Bare_Body_Node);
         when Mmz_Bare_Classwide_Type_Decl =>
            return Hash (Key.As_Bare_Classwide_Type_Decl);
         when Mmz_Bare_Compilation_Unit =>
            return Hash (Key.As_Bare_Compilation_Unit);
         when Mmz_Bare_Declarative_Part =>
            return Hash (Key.As_Bare_Declarative_Part);
         when Mmz_Bare_Defining_Name =>
            return Hash (Key.As_Bare_Defining_Name);
         when Mmz_Bare_End_Name =>
            return Hash (Key.As_Bare_End_Name);
         when Mmz_Bare_Enum_Literal_Decl =>
            return Hash (Key.As_Bare_Enum_Literal_Decl);
         when Mmz_Bare_For_Loop_Spec =>
            return Hash (Key.As_Bare_For_Loop_Spec);
         when Mmz_Bare_For_Loop_Var_Decl =>
            return Hash (Key.As_Bare_For_Loop_Var_Decl);
         when Mmz_Bare_Generic_Instantiation =>
            return Hash (Key.As_Bare_Generic_Instantiation);
         when Mmz_Bare_Name =>
            return Hash (Key.As_Bare_Name);
         when Mmz_Bare_Type_Decl =>
            return Hash (Key.As_Bare_Type_Decl);
         when Mmz_Boolean =>
            return Hash (Key.As_Boolean);
         when Mmz_Env_Rebindings =>
            return Hash (Key.As_Env_Rebindings);
         when Mmz_Integer =>
            return Hash (Key.As_Integer);
         when Mmz_Internal_Entity_Info =>
            return Hash (Key.As_Internal_Entity_Info);
         when Mmz_Internal_Metadata =>
            return Hash (Key.As_Internal_Metadata);
         when Mmz_Lexical_Env =>
            return Hash (Key.As_Lexical_Env);
         when Mmz_Symbol_Type =>
            return Hash (Key.As_Symbol_Type);
   end case;
end Hash;
----------
-- Hash --
----------
function Hash (Key : Mmz_Key) return Hash_Type is
   Result : Hash_Type := Mmz_Property'Pos (Key.Property);
begin
   for K of Key.Items.all loop
      Result := Combine (Result, Hash (K));
   end loop;
   return Result;
end Hash;
----------------
-- Equivalent --
----------------
function Equivalent (L, R : Mmz_Key) return Boolean is
   L_Items : Mmz_Key_Array renames L.Items.all;
   R_Items : Mmz_Key_Array renames R.Items.all;
begin
   if L.Property /= R.Property or else L_Items'Length /= R_Items'Length then
      return False;
   end if;
   for I in L_Items'Range loop
      if not Equivalent (L_Items (I), R_Items (I)) then
         return False;
      end if;
   end loop;
   return True;
end Equivalent;
-------------
-- Destroy --
-------------
procedure Destroy (Map : in out Memoization_Maps.Map) is
   use Memoization_Maps;
   --  We need keys and values to be valid when clearing the memoization map,
   --  but on the other hand we need to free keys and values as well. To
   --  achieve both goals, we first copy key and values into arrays, then we
   --  clear the map, and then we free keys/values in the arrays. Allocate both
   --  arrays on the heap to avoid stack overflow, as they can be quite big.
   Length : constant Natural := Natural (Map.Length);
   type Key_Array is array (1 .. Length) of Mmz_Key_Array_Access;
   type Key_Array_Access is access Key_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Key_Array, Key_Array_Access);
   type Value_Array is array (1 .. Length) of Mmz_Value;
   type Value_Array_Access is access Value_Array;
   procedure Free is new Ada.Unchecked_Deallocation
     (Value_Array, Value_Array_Access);
   Keys   : Key_Array_Access := new Key_Array;
   Values : Value_Array_Access := new Value_Array;
   I      : Positive := 1;
begin
   for Cur in Map.Iterate loop
      Keys (I) := Key (Cur).Items;
      Values (I) := Element (Cur);
      I := I + 1;
   end loop;
   Map.Clear;
   for K_Array of Keys.all loop
      Destroy (K_Array);
   end loop;
      for V of Values.all loop
         case V.Kind is
               when Mmz_Internal_Entity_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Array_Access);
               when Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access);
               when Mmz_Internal_Entity_Base_Type_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Base_Type_Decl_Array_Access);
               when Mmz_Internal_Entity_Basic_Decl_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Basic_Decl_Array_Access);
               when Mmz_Internal_Entity_Compilation_Unit_Array_Access =>
                  Dec_Ref (V.As_Internal_Entity_Compilation_Unit_Array_Access);
               when Mmz_Internal_Inner_Env_Assoc_Array_Access =>
                  Dec_Ref (V.As_Internal_Inner_Env_Assoc_Array_Access);
               when Mmz_Internal_Param_Match_Array_Access =>
                  Dec_Ref (V.As_Internal_Param_Match_Array_Access);
               when Mmz_Lexical_Env =>
                  Dec_Ref (V.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Keys);
   Free (Values);
end Destroy;
-------------
-- Destroy --
-------------
procedure Destroy (Key : in out Mmz_Key_Array_Access) is
   procedure Free is new Ada.Unchecked_Deallocation
     (Mmz_Key_Array, Mmz_Key_Array_Access);
begin
      for K of Key.all loop
         case K.Kind is
               when Mmz_Lexical_Env =>
                  Dec_Ref (K.As_Lexical_Env);
            when others => null;
         end case;
      end loop;
   Free (Key);
end Destroy;
-------------------------
-- Find_Memoized_Value --
-------------------------
function Find_Memoized_Value
  (Unit       : Internal_Unit;
   Handle     : out Memoization_Handle;
   Value      : out Mmz_Value;
   Create_Key : access function return Mmz_Key) return Boolean
is
   Inserted : Boolean;
begin
   --  Make sure that we don't lookup stale caches
   Reset_Caches (Unit);
   --  Initialize handle: create the key and create a cursor pointing to an
   --  existing entry.
   Handle.Key := Create_Key.all;
   Handle.Cache_Version := Unit.Cache_Version;
   Value := (Kind => Mmz_Evaluating);
   Unit.Memoization_Map.Insert (Handle.Key, Value, Handle.Cur, Inserted);
   --  No existing entry yet? The above just created one. Otherwise, destroy
   --  our key and reuse the existing entry's.
   if not Inserted then
      Destroy (Handle.Key.Items);
      Handle.Key := Memoization_Maps.Key (Handle.Cur);
      Value := Memoization_Maps.Element (Handle.Cur);
   end if;
   return not Inserted;
end Find_Memoized_Value;
------------------------
-- Add_Memoized_Value --
------------------------
procedure Add_Memoized_Value
  (Unit   : Internal_Unit;
   Handle : in out Memoization_Handle;
   Value  : Mmz_Value;
   Stored : out Boolean) is
begin
   --  If Handle was created using a memoization map that has been since then
   --  reset, do nothing: the result can be partly stale due to the event that
   --  triggered the memoization tables reset.
   Stored := Unit.Cache_Version <= Handle.Cache_Version;
   if Stored then
      Unit.Memoization_Map.Replace_Element (Handle.Cur, Value);
   end if;
end Add_Memoized_Value;
   -------------------
   -- Solve_Wrapper --
   -------------------
   function Solve_Wrapper
     (R            : Relation;
      Context_Node : Bare_Ada_Node) return Boolean is
   begin
      if Context_Node /= null and then Langkit_Support.Adalog.Debug.Debug then
         Assign_Names_To_Logic_Vars (Context_Node);
      end if;
      begin
         return Solve (R, Context_Node.Unit.Context.Logic_Resolution_Timeout);
      exception
         when Langkit_Support.Adalog.Early_Binding_Error =>
            raise Property_Error with "invalid equation for logic resolution";
         when Langkit_Support.Adalog.Timeout_Error =>
            raise Property_Error with "logic resolution timed out";
      end;
   end Solve_Wrapper;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Env : in out Lexical_Env_Access) is
      Mutable_Env : Lexical_Env :=
        (Wrap (Env), 0, Env.Kind, No_Generic_Unit, 0);
   begin
      Destroy (Mutable_Env);
      Env := null;
   end Destroy;
   ----------------
   -- Initialize --
   ----------------
   procedure Initialize
     (Self              : Bare_Ada_Node;
      Kind              : Ada_Node_Kind_Type;
      Unit              : Internal_Unit;
      Token_Start_Index : Token_Index;
      Token_End_Index   : Token_Index;
      Parent            : Bare_Ada_Node := null;
      Self_Env          : Lexical_Env := AST_Envs.Empty_Env) is
   begin
      pragma Unreferenced (Kind);
      Self.Parent := Parent;
      Self.Unit := Unit;
      Self.Token_Start_Index := Token_Start_Index;
      Self.Token_End_Index := Token_End_Index;
      Self.Self_Env := Self_Env;
      Self.Last_Attempted_Child := -1;
   end Initialize;
   --------------------
   -- Use_Direct_Env --
   --------------------
   procedure Use_Direct_Env (State : in out PLE_Node_State; Env : Lexical_Env)
   is
   begin
      State.Current_Env := Env;
      State.Current_NED := null;
   end Use_Direct_Env;
   -------------------
   -- Use_Named_Env --
   -------------------
   procedure Use_Named_Env
     (State   : in out PLE_Node_State;
      Context : Internal_Context;
      Name    : Symbol_Type) is
   begin
      State.Current_NED := Get_Named_Env_Descriptor (Context, Name);
      State.Current_Env := State.Current_NED.Env_With_Precedence;
   end Use_Named_Env;
   ---------------------
   -- Set_Initial_Env --
   ---------------------
   procedure Set_Initial_Env
     (Self     : Bare_Ada_Node;
      State    : in out PLE_Node_State;
      Name     : Symbol_Type;
      Resolver : Lexical_Env_Resolver) is
   begin
      --  An empty name is the way for the expression to say to fallback on the
      --  direct initial environment computation.
      if Name /= null then
         Use_Named_Env (State, Self.Unit.Context, Name);
      else
         Use_Direct_Env
           (State,
            Resolver ((Node => Self, Info => No_Entity_Info)));
      end if;
   end Set_Initial_Env;
   ----------------
   -- Add_To_Env --
   ----------------
   procedure Add_To_Env
     (Self              : Bare_Ada_Node;
      State             : PLE_Node_State;
      Key               : Symbol_Type;
      Value             : Bare_Ada_Node;
      MD                : Internal_Metadata;
      Resolver          : Entity_Resolver;
      Dest_Env_Name     : Symbol_Type;
      Dest_Env_Fallback : Lexical_Env;
      DSL_Location      : String)
   is
      Context    : constant Internal_Context := Self.Unit.Context;
      Root_Scope : Lexical_Env renames Context.Root_Scope;
      --  Shortcuts
      Dest_Env : Lexical_Env;
      Dest_NED : Named_Env_Descriptor_Access;
      --  Description for the destination environment
   begin
      --  Sanitize the content to add to the destination environment: Key,
      --  Value and MD (Resolver is always correct).
      if Key = null or else Value = null then
         return;
      end if;
      if Value.Unit /= Self.Unit then
         raise Property_Error with "Cannot add_to_env an AST node that comes"
                                   & " from another analysis unit";
      end if;
         --  Make sure metadata does not contain any foreign node
         if (MD.Primitive /= null and then MD.Primitive.Unit /= Self.Unit) or else (MD.Primitive_Real_Type /= null and then MD.Primitive_Real_Type.Unit /= Self.Unit)
         then
            raise Property_Error
               with "Cannot add metadata that contains foreign nodes";
         end if;
      --  Then determine the destination environment
      if Dest_Env_Name /= null then
         --  There is an environment name: just lookup the corresponding
         --  NED/env.
         Dest_NED := Get_Named_Env_Descriptor (Context, Dest_Env_Name);
         Dest_Env := Dest_NED.Env_With_Precedence;
      elsif Dest_Env_Fallback /= Empty_Env then
         --  There is an explicit destination environment
         Dest_NED := null;
         Dest_Env := Dest_Env_Fallback;
      else
         --  Just use the current environment
         Dest_NED := State.Current_NED;
         Dest_Env := State.Current_Env;
      end if;
      --  Sanitize it
      if Dest_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot add elements to a lexical env that is not static-primary";
      elsif
         --  Since lexical envs need to sort the foreign nodes they contain,
         --  and that the total order on nodes is not defined for synthetic
         --  nodes, it is not possible to add a synthetic node to a foreign
         --  lexical environment.
         --
         --  This reasoning applies to environments that belong to foreign
         --  units, but also to the root environment.
         Is_Foreign (Dest_Env, Self) and then Is_Synthetic (Value)
      then
         raise Property_Error with
            "Cannot add a synthetic node to a lexical env from another"
            & " analysis unit";
      elsif
         --  If requested, reject foreign destination environments.
         --
         --  Note that this checks only explicit destination environments
         --  (Dest_Env_Fallback): Set_Initial_Env already sanitized initial
         --  environments (State.Current_Env). Also note that Dest_Env_Fallback
         --  is Empty_Env (i.e. the fallback env expression is not evaluated)
         --  if we had a non-null env name (no need to fallback if we use a
         --  named environment).
         --
         --  This is an attempt at identifying uses of the unsound relocation
         --  mechanism (as opposed to named environments), so this applies to
         --  all foreign environments (root scope included).
         DSL_Location'Length > 0
         and then Is_Foreign_Strict (Dest_Env_Fallback, Self)
      then
         raise Property_Error with
            "unsound foreign environment in AddToEnv (" & DSL_Location & ")";
      end if;
      --  Now that everything is sanitized, we can proceed with the actual
      --  key/value pair addition. Note that this does nothing if Dest_Env
      --  ended up empty.
      Add (Dest_Env, Key, Value, MD, Resolver);
      --  If we're adding the element to an environment by env name, we must
      --  register this association in two places: in the target named env
      --  entry, and in Value's unit.
      if Dest_NED /= null then
         declare
            use NED_Assoc_Maps;
            FN    : Map renames Dest_NED.Foreign_Nodes;
            Dummy : Boolean;
            Cur   : Cursor;
         begin
            FN.Insert (Key      => Key,
                       New_Item => Internal_Map_Node_Vectors.Empty_Vector,
                       Position => Cur,
                       Inserted => Dummy);
            declare
               V : Internal_Map_Node_Vectors.Vector renames
                  FN.Reference (Cur);
            begin
               V.Append ((Value, MD, Resolver));
            end;
         end;
         Value.Unit.Exiled_Entries_In_NED.Append ((Dest_NED, Key, Value));
      --  Otherwise, if we're adding the element to an environment that belongs
      --  to a different unit, or to the root scope, then:
      elsif Dest_Env = Root_Scope or else Is_Foreign_Strict (Dest_Env, Self)
      then
         --  Add the environment, the key, and the value to the list of entries
         --  contained in other units, so we can remove them when reparsing
         --  Val's unit.
         Value.Unit.Exiled_Entries.Append ((Dest_Env, Key, Value));
         if Dest_Env /= Root_Scope then
            --  Add Val to the list of foreign nodes that Dest_Env's unit
            --  contains, so that when that unit is reparsed, we can call
            --  Add_To_Env again on those nodes.
            Convert_Unit (Dest_Env.Owner).Foreign_Nodes.Append
              ((Value, Self.Unit));
         end if;
      end if;
   end Add_To_Env;
   -------------
   -- Ref_Env --
   -------------
   procedure Ref_Env
     (Self                : Bare_Ada_Node;
      Dest_Env            : Lexical_Env;
      Ref_Env_Nodes       : in out Bare_Ada_Node_Array_Access;
      Resolver            : Lexical_Env_Resolver;
      Kind                : Ref_Kind;
      Cats                : Ref_Categories;
      Shed_Rebindings     : Boolean) is
   begin
      for N of Ref_Env_Nodes.Items loop
         if N /= null then
            if N.Unit /= Self.Unit then
               raise Property_Error with
                  "attempt to add a referenced environment to a foreign unit";
            end if;
            Reference (Dest_Env, N, Resolver, Kind, Cats, Shed_Rebindings);
         end if;
      end loop;
      Dec_Ref (Ref_Env_Nodes);
   end Ref_Env;
   -------------
   -- Add_Env --
   -------------
   procedure Add_Env
     (Self              : Bare_Ada_Node;
      State             : in out PLE_Node_State;
      No_Parent         : Boolean;
      Transitive_Parent : Boolean;
      Resolver          : Lexical_Env_Resolver;
      Names             : in out Symbol_Type_Array_Access)
   is
      Parent_From_Name : constant Boolean := State.Current_NED /= null;
      --  Does the parent environment comes from a named environment lookup?
      Parent_Foreign : constant Boolean :=
         Is_Foreign_Strict (State.Current_Env, Self);
      --  Determine how to get the parent of this new environment:
      --
      --  (1) no parent if requested;
      --  (2) the current environment as the static parent if it comes from a
      --      named env lookup or if it is not foreign (or is the empty/root
      --      environment);
      --  (3) a dynamic parent in all other cases (the current environment is
      --      foreign and not fetched as a named environment.
      Parent_Getter : constant Env_Getter :=
        (if No_Parent
         then AST_Envs.No_Env_Getter
         elsif Parent_From_Name or else not Parent_Foreign
         then AST_Envs.Simple_Env_Getter (State.Current_Env)
         else AST_Envs.Dyn_Env_Getter (Resolver, Self));
   begin
      --  Create the environment itself
      Self.Self_Env := Create_Static_Lexical_Env
        (Parent            => Parent_Getter,
         Node              => Self,
         Transitive_Parent => Transitive_Parent);
      --  If the parent of this new environment comes from a named environment
      --  lookup, register this new environment so that its parent is updated
      --  when the precence for this named environment changes.
      if Parent_From_Name then
         declare
            NED : constant Named_Env_Descriptor_Access := State.Current_NED;
         begin
            Self.Unit.Exiled_Envs.Append ((NED, Self.Self_Env));
            NED.Foreign_Envs.Insert (Self, Self.Self_Env);
         end;
      end if;
      --  From now on, the current environment is Self.Self_Env, with a direct
      --  access to it. It does not go through the env naming scheme, since
      --  only this node and its children (i.e. non-foreign nodes) will access
      --  it as a "current" environment during PLE.
      Use_Direct_Env (State, Self.Self_Env);
      --  Register the environment we just created on all the requested names
      if Names /= null then
         declare
            Context   : constant Internal_Context := Self.Unit.Context;
            Env       : constant Lexical_Env := Self.Self_Env;
            NENU      : NED_Maps.Map renames
               State.Unit_State.Named_Envs_Needing_Update;
         begin
            for N of Names.Items loop
               Register_Named_Env (Context, N, Env, NENU);
            end loop;
            Dec_Ref (Names);
         end;
      end if;
   end Add_Env;
   ---------------------
   -- Pre_Env_Actions --
   ---------------------
   procedure Pre_Env_Actions
     (Self            : Bare_Ada_Node;
      State           : in out PLE_Node_State;
      Add_To_Env_Only : Boolean := False) is
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            With_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Type_Clause =>
            Use_Type_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Use_Package_Clause =>
            Use_Package_Clause_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Access_To_Subp_Def =>
            Access_To_Subp_Def_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Named_Stmt =>
            Named_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Extended_Return_Stmt =>
            Extended_Return_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Begin_Block .. Ada_Decl_Block =>
            Block_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Stmt =>
            For_Loop_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            Accept_Stmt_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Pragma_Node =>
            Pragma_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Decl_Expr =>
            Decl_Expr_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Private_Part =>
            Private_Part_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Decl =>
            Single_Task_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Protected_Decl =>
            Single_Protected_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Renaming_Decl =>
            Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
            Object_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Number_Decl =>
            Number_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Label_Decl =>
            Label_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Renaming_Decl =>
            Generic_Subp_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Renaming_Decl =>
            Generic_Package_Renaming_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Instantiation =>
            Generic_Subp_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Instantiation =>
            Generic_Package_Instantiation_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Decl =>
            Generic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Decl =>
            Generic_Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_For_Loop_Var_Decl =>
            For_Loop_Var_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Handler =>
            Exception_Handler_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Exception_Decl =>
            Exception_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Index_Spec =>
            Entry_Index_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body =>
            Task_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body =>
            Protected_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body =>
            Package_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Body =>
            Entry_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Body_Stub =>
            Task_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Subp_Body_Stub =>
            Subp_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Body_Stub =>
            Protected_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Body_Stub =>
            Package_Body_Stub_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            Base_Subp_Body_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Subp_Internal =>
            Generic_Subp_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Enum_Literal_Decl =>
            Enum_Literal_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Entry_Decl =>
            Entry_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
            Basic_Subp_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Type_Decl =>
            Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Single_Task_Type_Decl =>
            Single_Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Task_Type_Decl =>
            Task_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Protected_Type_Decl =>
            Protected_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            Incomplete_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            Base_Type_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Package_Decl =>
            Package_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Generic_Package_Internal =>
            Generic_Package_Internal_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Param_Spec =>
            Param_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Discriminant_Spec =>
            Discriminant_Spec_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Component_Decl =>
            Component_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
            when Ada_Abstract_State_Decl =>
            Abstract_State_Decl_Pre_Env_Actions (Self, State, Add_To_Env_Only);
      when others =>  null; 
   end case;
   end Pre_Env_Actions;
   ----------------------
   -- Post_Env_Actions --
   ----------------------
   pragma Warnings (Off, "referenced");
   procedure Post_Env_Actions
     (Self : Bare_Ada_Node; State : in out PLE_Node_State)
   is
      pragma Warnings (On, "referenced");
   begin
   case Self.Kind is
            when Ada_With_Clause =>
            null;
            when Ada_Use_Type_Clause =>
            null;
            when Ada_Use_Package_Clause =>
            null;
            when Ada_Access_To_Subp_Def =>
            null;
            when Ada_Named_Stmt =>
            null;
            when Ada_Extended_Return_Stmt =>
            null;
            when Ada_Begin_Block .. Ada_Decl_Block =>
            null;
            when Ada_For_Loop_Stmt =>
            null;
            when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
            null;
            when Ada_Pragma_Node =>
            null;
            when Ada_Decl_Expr =>
            null;
            when Ada_Private_Part =>
            null;
            when Ada_Single_Task_Decl =>
            null;
            when Ada_Single_Protected_Decl =>
            null;
            when Ada_Package_Renaming_Decl =>
            null;
            when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
            null;
            when Ada_Number_Decl =>
            null;
            when Ada_Label_Decl =>
            null;
            when Ada_Generic_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Package_Renaming_Decl =>
            null;
            when Ada_Generic_Subp_Instantiation =>
            null;
            when Ada_Generic_Package_Instantiation =>
            null;
            when Ada_Generic_Subp_Decl =>
            null;
            when Ada_Generic_Package_Decl =>
            null;
            when Ada_For_Loop_Var_Decl =>
            null;
            when Ada_Exception_Handler =>
            null;
            when Ada_Exception_Decl =>
            null;
            when Ada_Entry_Index_Spec =>
            null;
            when Ada_Task_Body =>
            null;
            when Ada_Protected_Body =>
            null;
            when Ada_Package_Body =>
            null;
            when Ada_Entry_Body =>
            null;
            when Ada_Task_Body_Stub =>
            null;
            when Ada_Subp_Body_Stub =>
            null;
            when Ada_Protected_Body_Stub =>
            null;
            when Ada_Package_Body_Stub =>
            null;
            when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
            null;
            when Ada_Generic_Subp_Internal =>
            null;
            when Ada_Enum_Literal_Decl =>
            null;
            when Ada_Entry_Decl =>
            null;
            when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
            null;
            when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
            null;
            when Ada_Type_Decl =>
            Type_Decl_Post_Env_Actions (Self, State);
            when Ada_Single_Task_Type_Decl =>
            null;
            when Ada_Task_Type_Decl =>
            null;
            when Ada_Protected_Type_Decl =>
            null;
            when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
            null;
            when Ada_Classwide_Type_Decl =>
            null;
            when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
            null;
            when Ada_Package_Decl =>
            null;
            when Ada_Generic_Package_Internal =>
            null;
            when Ada_Param_Spec =>
            null;
            when Ada_Discriminant_Spec =>
            null;
            when Ada_Component_Decl =>
            null;
            when Ada_Abstract_State_Decl =>
            null;
      when others =>  null; 
   end case;
   end Post_Env_Actions;
   ----------------
   -- Get_Symbol --
   ----------------
   function Get_Symbol
     (Node : Bare_Ada_Node) return Symbol_Type is
   begin
      if Node = null then
         raise Property_Error with "cannot get the symbol of a null node";
      end if;
      return Get_Symbol (Token (Node, Node.Token_Start_Index));
   end Get_Symbol;
   ----------
   -- Text --
   ----------
   function Text
     (Node : Bare_Ada_Node) return Text_Type
   is
   begin
      if Node = null then
         raise Property_Error with "cannot get the text of a null node";
      end if;
      declare
         Start_T : constant Token_Reference :=
            Token (Node, Node.Token_Start_Index);
         End_T   : constant Token_Reference :=
            Token (Node, Node.Token_End_Index);
      begin
         --  No text is associated to synthetic and ghost nodes
         if Is_Synthetic (Node) then
            return "";
         end if;
         if Is_Ghost (Node) then
            return "";
         end if;
         return Text (Start_T, End_T);
      end;
   end Text;
   ---------------------
   -- Is_Visible_From --
   ---------------------
   function Is_Visible_From
     (Referenced_Env, Base_Env : Lexical_Env) return Boolean
   is
      Referenced_Unit : constant Internal_Unit :=
         Convert_Unit (Referenced_Env.Owner);
      Base_Unit       : constant Internal_Unit :=
         Convert_Unit (Base_Env.Owner);
   begin
      if Referenced_Unit = null then
         raise Property_Error with
            "referenced environment does not belong to any analysis unit";
      elsif Base_Unit = null then
         raise Property_Error with
            "base environment does not belong to any analysis unit";
      end if;
      return Is_Referenced_From (Referenced_Unit, Base_Unit);
   end Is_Visible_From;
   ----------
   -- Unit --
   ----------
   function Unit (Node : Bare_Ada_Node) return Internal_Unit is
   begin
      return Node.Unit;
   end Unit;
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node;
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node);
   --  Implementation helpers for the looking up process
   -----------------
   -- Set_Parents --
   -----------------
   procedure Set_Parents
     (Node, Parent : Bare_Ada_Node)
   is
   begin
      if Node = null then
         return;
      end if;
      Node.Parent := Bare_Ada_Node (Parent);
      for I in 1 .. Children_Count (Node) loop
         Set_Parents (Child (Node, I), Node);
      end loop;
   end Set_Parents;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Node : Bare_Ada_Node) is
   begin
      if Node = null then
         return;
      end if;
      Free_User_Fields (Node);
      for I in 1 .. Children_Count (Node) loop
         Destroy (Child (Node, I));
      end loop;
   end Destroy;
   -----------
   -- Child --
   -----------
   function Child (Node  : Bare_Ada_Node;
                   Index : Positive) return Bare_Ada_Node
   is
      Result          : Bare_Ada_Node;
      Index_In_Bounds : Boolean;
   begin
      Get_Child (Node, Index, Index_In_Bounds, Result);
      return Result;
   end Child;
   --------------
   -- Traverse --
   --------------
   function Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
              return Visit_Status)
     return Visit_Status
   is
      Status : Visit_Status := Into;
   begin
      if Node /= null then
         Status := Visit (Node);
         --  Skip processing the child nodes if the returned status is Over
         --  or Stop. In the former case the previous call to Visit has taken
         --  care of processing the needed childs, and in the latter case we
         --  must immediately stop processing the tree.
         if Status = Into then
            for I in 1 .. Children_Count (Node) loop
               declare
                  Cur_Child : constant Bare_Ada_Node :=
                     Child (Node, I);
               begin
                  if Cur_Child /= null then
                     Status := Traverse (Cur_Child, Visit);
                     exit when Status /= Into;
                  end if;
               end;
            end loop;
         end if;
      end if;
      if Status = Stop then
         return Stop;
      --  At this stage the Over status has no sense and we just continue
      --  processing the tree.
      else
         return Into;
      end if;
   end Traverse;
   --------------
   -- Traverse --
   --------------
   procedure Traverse
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node)
                               return Visit_Status)
   is
      Result_Status : Visit_Status;
      pragma Unreferenced (Result_Status);
   begin
      Result_Status := Traverse (Node, Visit);
   end Traverse;
   ------------------------
   -- Traverse_With_Data --
   ------------------------
   function Traverse_With_Data
     (Node  : Bare_Ada_Node;
      Visit : access function (Node : Bare_Ada_Node;
                               Data : in out Data_Type)
                               return Visit_Status;
      Data  : in out Data_Type)
      return Visit_Status
   is
      function Helper (Node : Bare_Ada_Node) return Visit_Status;
      ------------
      -- Helper --
      ------------
      function Helper (Node : Bare_Ada_Node) return Visit_Status is
      begin
         return Visit (Node, Data);
      end Helper;
      Saved_Data : Data_Type;
      Result     : Visit_Status;
   begin
      if Reset_After_Traversal then
         Saved_Data := Data;
      end if;
      Result := Traverse (Node, Helper'Access);
      if Reset_After_Traversal then
         Data := Saved_Data;
      end if;
      return Result;
   end Traverse_With_Data;
   ----------------
   -- Sloc_Range --
   ----------------
   function Sloc_Range
     (Node : Bare_Ada_Node) return Source_Location_Range
   is
      type Token_Anchor is (T_Start, T_End);
      type Token_Pos is record
         Pos    : Token_Index;
         Anchor : Token_Anchor;
      end record;
      TDH                    : Token_Data_Handler renames Node.Unit.TDH;
      Token_Start, Token_End : Token_Pos;
      function Get (Index : Token_Index) return Stored_Token_Data is
        (Get_Token (TDH, Index));
      function Sloc (T : Token_Pos) return Source_Location is
        (if T.Anchor = T_Start
         then Sloc_Start (TDH, Get (T.Pos))
         else Sloc_End (TDH, Get (T.Pos)));
   begin
      if Is_Synthetic (Node) then
         return Sloc_Range (Node.Parent);
      end if;
      if Is_Ghost (Node) then
         Token_Start := (if Node.Token_Start_Index = 1
                         then (1, T_Start)
                         else (Node.Token_Start_Index - 1, T_End));
         Token_End := Token_Start;
      else
         Token_Start := (Node.Token_Start_Index, T_Start);
         Token_End := (Node.Token_End_Index, T_End);
      end if;
      if Snaps_At_Start (Node)
         and then not Is_Ghost (Node)
         and then Token_Start.Pos /= 1
      then
         Token_Start := (Token_Start.Pos - 1, T_End);
      end if;
      if Snaps_At_End (Node) and then Token_End.Pos /= Last_Token (TDH) then
         Token_End := (Token_End.Pos + 1, T_Start);
      end if;
      return Make_Range (Sloc (Token_Start), Sloc (Token_End));
   end Sloc_Range;
   ------------
   -- Lookup --
   ------------
   function Lookup
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      Position : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      if Sloc = No_Source_Location then
         return null;
      end if;
      Lookup_Relative
        (Bare_Ada_Node (Node), Sloc, Position, Result);
      return Result;
   end Lookup;
   ---------------------
   -- Lookup_Internal --
   ---------------------
   function Lookup_Internal
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Bare_Ada_Node
   is
      --  For this implementation helper (i.e. internal primitive), we can
      --  assume that all lookups fall into this node's sloc range.
      pragma Assert (Compare (Sloc_Range (Node), Sloc) = Inside);
      Children : constant Internal_Bare_Ada_Node_Array :=
         Implementation.Children (Node);
      Pos      : Relative_Position;
      Result   : Bare_Ada_Node;
   begin
      --  Look for a child node that contains Sloc (i.e. return the most
      --  precise result).
      for Child of Children loop
         --  Note that we assume here that child nodes are ordered so that the
         --  first one has a sloc range that is before the sloc range of the
         --  second child node, etc.
         if Child /= null then
            Lookup_Relative (Child, Sloc, Pos, Result);
            case Pos is
               when Before =>
                   --  If this is the first node, Sloc is before it, so we can
                   --  stop here.  Otherwise, Sloc is between the previous
                   --  child node and the next one...  so we can stop here,
                   --  too.
                   return Node;
               when Inside =>
                   return Result;
               when After =>
                   --  Sloc is after the current child node, so see with the
                   --  next one.
                   null;
            end case;
         end if;
      end loop;
      --  If we reach this point, we found no children that covers Sloc, but
      --  Node still covers it (see the assertion).
      return Node;
   end Lookup_Internal;
   -------------
   -- Compare --
   -------------
   function Compare
     (Node : Bare_Ada_Node;
      Sloc : Source_Location) return Relative_Position is
   begin
      return Compare (Sloc_Range (Node), Sloc);
   end Compare;
   ---------------------
   -- Lookup_Relative --
   ---------------------
   procedure Lookup_Relative
     (Node       : Bare_Ada_Node;
      Sloc       : Source_Location;
      Position   : out Relative_Position;
      Node_Found : out Bare_Ada_Node)
   is
      Result : constant Relative_Position :=
        Compare (Node, Sloc);
   begin
      Position := Result;
      Node_Found := (if Result = Inside
                     then Lookup_Internal (Node, Sloc)
                     else null);
   end Lookup_Relative;
   -------------
   -- Compare --
   -------------
   function Compare
     (Left, Right : Bare_Ada_Node;
      Relation    : Comparison_Relation) return Boolean
   is
      LS, RS : Source_Location;
   begin
      if Left = null or else Right = null or else Left.Unit /= Right.Unit then
         raise Property_Error with "invalid node comparison";
      end if;
      LS := Start_Sloc (Sloc_Range (Left));
      RS := Start_Sloc (Sloc_Range (Right));
      return (case Relation is
              when Langkit_Support.Types.Less_Than        => LS < RS,
              when Langkit_Support.Types.Less_Or_Equal    => LS <= RS,
              when Langkit_Support.Types.Greater_Than     => LS > RS,
              when Langkit_Support.Types.Greater_Or_Equal => LS >= RS);
   end Compare;
   --------------
   -- Children --
   --------------
   function Children
     (Node : Bare_Ada_Node) return Internal_Bare_Ada_Node_Array
   is
      First : constant Integer := Bare_Ada_Node_Vectors.Index_Type'First;
      Last  : constant Integer := First + Children_Count (Node) - 1;
   begin
      return A : Internal_Bare_Ada_Node_Array (First .. Last)
      do
         for I in First .. Last loop
            A (I) := Child (Node, I);
         end loop;
      end return;
   end Children;
   function Children
     (Node : Bare_Ada_Node) return Bare_Ada_Node_Array_Access
   is
      C : Internal_Bare_Ada_Node_Array := Children (Node);
   begin
      return Ret : Bare_Ada_Node_Array_Access :=
         Create_Bare_Ada_Node_Array (C'Length)
      do
         Ret.Items := C;
      end return;
   end Children;
   ---------
   -- Get --
   ---------
   function Get
     (Node    : Bare_Ada_List;
      Index   : Integer;
      Or_Null : Boolean := False) return Bare_Ada_Node
   is
      function Length (Node : Bare_Ada_List) return Natural
      is (Node.Count);
      --  Wrapper around the Length primitive to get the compiler happy for the
      --  the package instantiation below.
      function Absolute_Get
        (L     : Bare_Ada_List;
         Index : Integer) return Bare_Ada_Node
      is (L.Nodes.all (Index + 1));
      --  L.Nodes is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Bare_Ada_Node,
         Sequence_Type => Bare_Ada_List,
         Length        => Length,
         Get           => Absolute_Get);
      Result : Bare_Ada_Node;
   begin
      if Node = null and then Or_Null then
         return null;
      elsif Relative_Get (Node, Index, Result) then
         return Result;
      elsif Or_Null then
         return null;
      else
         raise Property_Error with "out-of-bounds AST list access";
      end if;
   end Get;
   ---------------
   -- PP_Trivia --
   ---------------
   procedure PP_Trivia
     (Node        : Bare_Ada_Node;
      Line_Prefix : String := "")
   is
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      Put_Line (Line_Prefix & Kind_Name (Node));
      for C of Children_And_Trivia (Node) loop
         case C.Kind is
            when Trivia =>
               Put_Line (Children_Prefix & Image (Text (C.Trivia)));
            when Child =>
               PP_Trivia (C.Node, Children_Prefix);
         end case;
      end loop;
   end PP_Trivia;
   --------------------------
   -- Populate_Lexical_Env --
   --------------------------
   function Populate_Lexical_Env (Node : Bare_Ada_Node) return Boolean is
      Context    : constant Internal_Context := Node.Unit.Context;
      Unit_State : aliased PLE_Unit_State := (Named_Envs_Needing_Update => <>);
      Root_State : constant PLE_Node_State :=
        (Unit_State  => Unit_State'Unchecked_Access,
         Current_Env => Context.Root_Scope,
         Current_NED => null);
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean;
      --  Do the lexical env population on Node and recurse on its children
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State);
      --  Given a node and its PLE state, register Node.Self_Env as being
      --  initialized through the named environment mechanism, if that's indeed
      --  the case. Do nothing otherwise.
      -----------------------
      -- Populate_Internal --
      -----------------------
      function Populate_Internal
        (Node         : Bare_Ada_Node;
         Parent_State : PLE_Node_State) return Boolean
      is
         Result : Boolean := False;
         State  : PLE_Node_State := Parent_State;
      begin
         if Node = null then
            return Result;
         end if;
         --  By default (i.e. unless env actions add a new env), the
         --  environment we store in Node is the current one.
         Node.Self_Env := State.Current_Env;
         --  Run pre/post actions, and run PLE on children in between. Make
         --  sure we register the potential foreign Node.Self_Env environment
         --  at the end, even when an exception interrupts PLE to keep the
         --  state consistent.
         begin
            Pre_Env_Actions (Node, State);
            if State.Current_Env /= Null_Lexical_Env then
               Node.Self_Env := State.Current_Env;
               Register_Foreign_Env (Node, State);
            end if;
            --  Call recursively on children
            for I in First_Child_Index (Node) .. Last_Child_Index (Node) loop
               Result := Populate_Internal
                 (Child (Node, I), State) or else Result;
            end loop;
            Post_Env_Actions (Node, State);
         exception
            when Exc : Property_Error =>
               if PLE_Errors_Trace.Is_Active then
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      "Exception raised during PLE "
                      & Ada.Exceptions.Exception_Name (Exc) & " : "
                      & Ada.Exceptions.Exception_Message (Exc));
                   GNATCOLL.Traces.Trace
                     (PLE_Errors_Trace,
                      GNAT.Traceback.Symbolic.Symbolic_Traceback (Exc));
               end if;
               Register_Foreign_Env (Node, State);
               return True;
         end;
         return Result;
      end Populate_Internal;
      --------------------------
      -- Register_Foreign_Env --
      --------------------------
      procedure Register_Foreign_Env
        (Node : Bare_Ada_Node; State : PLE_Node_State) is
      begin
         if State.Current_NED /= null then
            State.Current_NED.Nodes_With_Foreign_Env.Insert (Node);
            Node.Unit.Nodes_With_Foreign_Env.Insert (Node, State.Current_NED);
         end if;
      end Register_Foreign_Env;
   begin
      --  This function is meant to be called during an existing PLE pass. If
      --  if is called outside of this context, run the PLE pass on Node's
      --  analysis unit. Likewise, if PLE has not run on the unit that owns
      --  this PLE unit yet, do a full run, which will in the end trigger the
      --  PLE on this PLE unit.
      --
      --  We do this so that as soon as PLE is required on a PLE unit: the
      --  whole unit end up with its lexical environments populated.
      if not Context.In_Populate_Lexical_Env then
         begin
            Populate_Lexical_Env (Node.Unit);
            return False;
         exception
            when Property_Error =>
               return True;
         end;
      end if;
         --  This is intended to be called on the root node only
         if Node.Parent /= null then
            raise Program_Error;
         end if;
      return Result : constant Boolean :=
         Populate_Internal (Node, Root_State)
      do
         Update_Named_Envs (Unit_State.Named_Envs_Needing_Update);
      end return;
   end Populate_Lexical_Env;
   ------------------------------
   -- AST_Envs_Node_Text_Image --
   ------------------------------
   function AST_Envs_Node_Text_Image
     (Node  : Bare_Ada_Node;
      Short : Boolean := True) return Text_Type is
   begin
      if Short then
         if Node = null then
            return "null";
         end if;
         return To_Text (Basename (Node.Unit))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node))));
      else
         return Short_Text_Image (Node);
      end if;
   end AST_Envs_Node_Text_Image;
   -------------------
   -- Is_Rebindable --
   -------------------
   function Is_Rebindable (Node : Bare_Ada_Node) return Boolean is
   begin
         return Node.Kind in Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl;
   end Is_Rebindable;
   -----------------------
   -- Acquire_Rebinding --
   -----------------------
   function Acquire_Rebinding
     (Node             : Bare_Ada_Node;
      Parent           : Env_Rebindings;
      Old_Env, New_Env : Lexical_Env) return Env_Rebindings
   is
      Result    : Env_Rebindings;
      Available : Env_Rebindings_Vectors.Vector renames
         Node.Unit.Context.Available_Rebindings;
   begin
      --  Use an existing and available Env_Rebindings_Type record for Node's
      --  Context, otherwise allocate a new rebinding.
      Result := (if Available.Is_Empty
                 then new Env_Rebindings_Type'(Version => 0, others => <>)
                 else Available.Pop);
      Result.Parent := Parent;
      Result.Old_Env := Old_Env;
      Result.New_Env := New_Env;
      Result.Children := Env_Rebindings_Vectors.Empty_Vector;
      return Result;
   end Acquire_Rebinding;
   -----------------------
   -- Release_Rebinding --
   -----------------------
   procedure Release_Rebinding (Self : in out Env_Rebindings) is
      Available : Env_Rebindings_Vectors.Vector renames
         Unwrap (Self.Old_Env).Node.Unit.Context.Available_Rebindings;
   begin
      --  Bumping the version number, to invalidate existing references to
      --  Self.
      Self.Version := Self.Version + 1;
      Self.Children.Destroy;
      Available.Append (Self);
      Self := null;
   end Release_Rebinding;
   ------------------------
   -- Register_Rebinding --
   ------------------------
   procedure Register_Rebinding
     (Node : Bare_Ada_Node; Rebinding : Env_Rebindings) is
   begin
      Node.Unit.Rebindings.Append (Rebinding);
   end Register_Rebinding;
   --------------------
   -- Element_Parent --
   --------------------
   function Element_Parent
     (Node : Bare_Ada_Node) return Bare_Ada_Node
   is (Node.Parent);
   ---------------
   -- Node_Unit --
   ---------------
   function Node_Unit (Node : Bare_Ada_Node) return Generic_Unit_Ptr is
   begin
      return Convert_Unit (Node.Unit);
   end Node_Unit;
   ----------
   -- Hash --
   ----------
   function Hash (Node : Bare_Ada_Node) return Hash_Type
   is
      function H is new Hash_Access
        (Root_Node_Record, Bare_Ada_Node);
   begin
      return H (Node);
   end Hash;
      function Hash (B : Boolean) return Hash_Type is (Boolean'Pos (B));
      function Hash (I : Integer) return Hash_Type is (Hash_Type'Mod (I));
   ------------------------
   -- Named environments --
   ------------------------
   ---------
   -- Add --
   ---------
   procedure Add
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : AST_Envs.Internal_Map_Node)
   is
      use NED_Assoc_Maps;
      Pos   : Cursor;
      Dummy : Boolean;
   begin
      --  Make sure there is a vector entry for Key
      Self.Insert (Key, Internal_Map_Node_Vectors.Empty_Vector, Pos, Dummy);
      --  Append Node to that vector
      declare
         V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Pos);
      begin
         V.Append (Node);
      end;
   end Add;
   ------------
   -- Remove --
   ------------
   procedure Remove
     (Self : in out NED_Assoc_Maps.Map;
      Key  : Symbol_Type;
      Node : Bare_Ada_Node)
   is
      use NED_Assoc_Maps;
      V : Internal_Map_Node_Vectors.Vector renames Self.Reference (Key);
   begin
      --  Remove the (assumed unique) entry in V whose node is Node. The order
      --  of items in V is not significant, so we can use Pop for efficient
      --  removal. Do the traversal in reverse order for correctness.
      for I in reverse 1 .. V.Length loop
         if V.Get_Access (I).Node = Node then
            V.Pop (I);
            exit;
         end if;
      end loop;
   end Remove;
   ------------------------------
   -- Get_Named_Env_Descriptor --
   ------------------------------
   function Get_Named_Env_Descriptor
     (Context : Internal_Context;
      Name    : Symbol_Type) return Named_Env_Descriptor_Access
   is
      use NED_Maps;
      --  Look for an existing entry for Name
      Pos : constant Cursor := Context.Named_Envs.Find (Name);
   begin
      if Has_Element (Pos) then
         return Element (Pos);
      end if;
      --  There is no such entry: create one
      return Result : constant Named_Env_Descriptor_Access :=
         new Named_Env_Descriptor'
           (Name                   => Name,
            Envs                   => <>,
            Env_With_Precedence    => Empty_Env,
            Foreign_Nodes          => <>,
            Foreign_Envs           => <>,
            Nodes_With_Foreign_Env => <>)
      do
         Context.Named_Envs.Insert (Name, Result);
      end return;
   end Get_Named_Env_Descriptor;
   ------------------------
   -- Register_Named_Env --
   ------------------------
   procedure Register_Named_Env
     (Context                   : Internal_Context;
      Name                      : Symbol_Type;
      Env                       : Lexical_Env;
      Named_Envs_Needing_Update : in out NED_Maps.Map)
   is
      NED_Access : constant Named_Env_Descriptor_Access :=
         Get_Named_Env_Descriptor (Context, Name);
      NED        : Named_Env_Descriptor renames NED_Access.all;
      Node       : constant Bare_Ada_Node := Env_Node (Env);
   begin
      NED.Envs.Insert (Node, Env);
      Node.Unit.Named_Envs.Append ((Name, Env));
      --  If that insertion must change the env that has precedence, signal
      --  that NED requires an update.
      if NED.Envs.First_Element /= NED.Env_With_Precedence then
         Named_Envs_Needing_Update.Include (Name, NED_Access);
      end if;
   end Register_Named_Env;
   ----------------------
   -- Update_Named_Env --
   ----------------------
   procedure Update_Named_Envs (Named_Envs : NED_Maps.Map) is
   begin
      for Cur in Named_Envs.Iterate loop
         declare
            NE      : Named_Env_Descriptor renames NED_Maps.Element (Cur).all;
            New_Env : constant Lexical_Env :=
              (if NE.Envs.Is_Empty
               then Empty_Env
               else NE.Envs.First_Element);
         begin
            --  If there was an environment with precedence, remove its foreign
            --  nodes.
            if NE.Env_With_Precedence /= Empty_Env then
               for Cur in NE.Foreign_Nodes.Iterate loop
                  declare
                     Key   : constant Symbol_Type :=
                        NED_Assoc_Maps.Key (Cur);
                     Nodes : Internal_Map_Node_Vectors.Vector renames
                        NE.Foreign_Nodes.Reference (Cur);
                  begin
                     for N of Nodes loop
                        Remove (NE.Env_With_Precedence, Key, N.Node);
                     end loop;
                  end;
               end loop;
            end if;
            --  Now, set the new environment that has precedence
            NE.Env_With_Precedence := New_Env;
            --  Add the foreign nodes to the new environment with precedence,
            --  if any.
            for Cur in NE.Foreign_Nodes.Iterate loop
               declare
                  Key   : constant Symbol_Type :=
                     NED_Assoc_Maps.Key (Cur);
                  Nodes : Internal_Map_Node_Vectors.Vector renames
                     NE.Foreign_Nodes.Reference (Cur);
               begin
                  for N of Nodes loop
                     Add (New_Env, Key, N.Node, N.MD, N.Resolver);
                  end loop;
               end;
            end loop;
            --  Set the parent environment of all foreign environments
            for Cur in NE.Foreign_Envs.Iterate loop
               declare
                  Env : Lexical_Env_Record renames
                     Unwrap (Sorted_Env_Maps.Element (Cur)).all;
               begin
                  Env.Parent := Simple_Env_Getter (New_Env);
               end;
            end loop;
            --  Update nodes whose environment was the old env with precedence
            for N of NE.Nodes_With_Foreign_Env loop
               N.Self_Env := New_Env;
            end loop;
         end;
      end loop;
   end Update_Named_Envs;
   --------------------------
   -- Big integers wrapper --
   --------------------------
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Image : String; Base : Integer := 10) return Big_Integer_Type
   is
      use GNATCOLL.GMP;
      use GNATCOLL.GMP.Integers;
   begin
      return new Big_Integer_Record'(Value     => Make (Image, Int (Base)),
                                     Ref_Count => 1);
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer
     (Big_Int : GNATCOLL.GMP.Integers.Big_Integer) return Big_Integer_Type
   is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (Big_Int);
      return Result;
   end Create_Big_Integer;
   ------------------------
   -- Create_Big_Integer --
   ------------------------
   function Create_Big_Integer (Int : Integer) return Big_Integer_Type is
      Result : constant Big_Integer_Type :=
         new Big_Integer_Record'(Value     => <>,
                                 Ref_Count => 1);
   begin
      Result.Value.Set (GNATCOLL.GMP.Long (Int));
      return Result;
   end Create_Big_Integer;
   -------------------------------
   -- Create_Public_Big_Integer --
   -------------------------------
   function Create_Public_Big_Integer
     (Big_Int : Big_Integer_Type) return GNATCOLL.GMP.Integers.Big_Integer is
   begin
      return Result : GNATCOLL.GMP.Integers.Big_Integer do
         Result.Set (Big_Int.Value);
      end return;
   end Create_Public_Big_Integer;
   -----------------
   -- Trace_Image --
   -----------------
   function Trace_Image (I : Big_Integer_Type) return String is
   begin
      return GNATCOLL.GMP.Integers.Image (I.Value);
   end Trace_Image;
   ----------------
   -- To_Integer --
   ----------------
   function To_Integer (Big_Int : Big_Integer_Type) return Integer is
      Image : constant String := Big_Int.Value.Image;
   begin
      return Integer'Value (Image);
   exception
      when Constraint_Error =>
         raise Property_Error with "out of range big integer";
   end To_Integer;
   -------------
   -- Inc_Ref --
   -------------
   procedure Inc_Ref (Big_Int : Big_Integer_Type) is
   begin
      if Big_Int.Ref_Count /= -1 then
         Big_Int.Ref_Count := Big_Int.Ref_Count + 1;
      end if;
   end Inc_Ref;
   -------------
   -- Dec_Ref --
   -------------
   procedure Dec_Ref (Big_Int : in out Big_Integer_Type) is
      procedure Destroy is new Ada.Unchecked_Deallocation
        (Big_Integer_Record, Big_Integer_Type);
   begin
      if Big_Int = null or else Big_Int.Ref_Count = -1 then
         return;
      end if;
      Big_Int.Ref_Count := Big_Int.Ref_Count - 1;
      if Big_Int.Ref_Count = 0 then
         Destroy (Big_Int);
      end if;
   end Dec_Ref;
   ----------------
   -- Equivalent --
   ----------------
   function Equivalent (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value = Right.Value;
   end Equivalent;
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value < Right.Value;
   end "<";
   ----------
   -- "<=" --
   ----------
   function "<=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value <= Right.Value;
   end "<=";
   ---------
   -- ">" --
   ---------
   function ">" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value > Right.Value;
   end ">";
   ----------
   -- ">=" --
   ----------
   function ">=" (Left, Right : Big_Integer_Type) return Boolean is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Left.Value >= Right.Value;
   end ">=";
   ---------
   -- "+" --
   ---------
   function "+" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value + Right.Value);
   end "+";
   ---------
   -- "-" --
   ---------
   function "-" (Left, Right : Big_Integer_Type) return Big_Integer_Type is
      use type GNATCOLL.GMP.Integers.Big_Integer;
   begin
      return Create_Big_Integer (Left.Value - Right.Value);
   end "-";
   ------------------
   -- Unit_Version --
   ------------------
   function Unit_Version (Unit : Generic_Unit_Ptr) return Version_Number is
   begin
      return Convert_Unit (Unit).Unit_Version;
   end Unit_Version;
   ----------------------
   -- Short_Text_Image --
   ----------------------
   function Short_Text_Image (Self : Bare_Ada_Node) return Text_Type
   is
   begin
      if Self = null then
         return "None";
      end if;
   case Self.Kind is
            when Ada_Char_Literal .. Ada_String_Literal =>
         return Base_Id_Short_Image (Self);
            when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
         return Basic_Decl_Short_Image (Self);
      when others => 
         return "<" & To_Text (Kind_Name (Self))
                & " "
                & To_Text
                  (Ada.Directories.Simple_Name
                     (Get_Filename (Unit (Self))))
                & ":" & To_Text (Image (Sloc_Range (Self))) & ">";
   end case;
   end Short_Text_Image;
   --------------------
   -- Snaps_At_Start --
   --------------------
   function Snaps_At_Start (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return False;
   end case;
   end Snaps_At_Start;
   ------------------
   -- Snaps_At_End --
   ------------------
   function Snaps_At_End (Self : Bare_Ada_Node) return Boolean is
   begin
   case Self.Kind is
            when Ada_Handled_Stmts =>
         return True;
            when Ada_Declarative_Part .. Ada_Public_Part =>
         return True;
      when others => 
         return Is_Incomplete (Self);
   end case;
   end Snaps_At_End;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True)
      return Bare_Ada_Node_Array_Access
   is
      Count : Natural := 0;
      Start : Bare_Ada_Node :=
        (if With_Self then Node else Node.Parent);
      Cur   : Bare_Ada_Node := Start;
   begin
      while Cur /= null loop
         Count := Count + 1;
         Cur := Cur.Parent;
      end loop;
      declare
         Result : constant Bare_Ada_Node_Array_Access :=
            Create_Bare_Ada_Node_Array (Count);
      begin
         Cur := Start;
         for I in Result.Items'Range loop
            Result.Items (I) := Cur;
            Cur := Cur.Parent;
         end loop;
         return Result;
      end;
   end Parents;
   -----------------------
   -- First_Child_Index --
   -----------------------
   function First_Child_Index (Node : Bare_Ada_Node) return Natural
   is (1);
   ----------------------
   -- Last_Child_Index --
   ----------------------
   function Last_Child_Index (Node : Bare_Ada_Node) return Natural
   is (Children_Count (Node));
   ---------------
   -- Get_Child --
   ---------------
   procedure Get_Child
     (Node            : Bare_Ada_Node;
      Index           : Positive;
      Index_In_Bounds : out Boolean;
      Result          : out Bare_Ada_Node)
   is
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      Index_In_Bounds := True;
      Result := null;
      case Ada_Ada_Node (K) is
when Ada_Ada_List =>
declare
N_Bare_Ada_List : constant Bare_Ada_List := Node;
begin
                    if Index > N_Bare_Ada_List.Count then
                        Index_In_Bounds := False;
                    else
                        Result := N_Bare_Ada_List.Nodes (Index);
                    end if;
                    return;
end;
when Ada_Constrained_Array_Indices_Range =>
declare
N_Bare_Constrained_Array_Indices : constant Bare_Constrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Constrained_Array_Indices.Constrained_Array_Indices_F_List;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Indices_Range =>
declare
N_Bare_Unconstrained_Array_Indices : constant Bare_Unconstrained_Array_Indices := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Indices.Unconstrained_Array_Indices_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Assoc_Range =>
declare
N_Bare_Aspect_Assoc : constant Bare_Aspect_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Aspect_Assoc.Aspect_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_At_Clause_Range =>
declare
N_Bare_At_Clause : constant Bare_At_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_At_Clause.At_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Def_Clause_Range =>
declare
N_Bare_Attribute_Def_Clause : constant Bare_Attribute_Def_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Attribute_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Def_Clause.Attribute_Def_Clause_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Rep_Clause_Range =>
declare
N_Bare_Enum_Rep_Clause : constant Bare_Enum_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Type_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Enum_Rep_Clause.Enum_Rep_Clause_F_Aggregate;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Rep_Clause_Range =>
declare
N_Bare_Record_Rep_Clause : constant Bare_Record_Rep_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_At_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Rep_Clause.Record_Rep_Clause_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aspect_Spec_Range =>
declare
N_Bare_Aspect_Spec : constant Bare_Aspect_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aspect_Spec.Aspect_Spec_F_Aspect_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Case_Assoc_Range =>
declare
N_Bare_Contract_Case_Assoc : constant Bare_Contract_Case_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Guard;
                            return;
                        when 2 =>
                            Result := N_Bare_Contract_Case_Assoc.Contract_Case_Assoc_F_Consequence;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Argument_Assoc_Range =>
declare
N_Bare_Pragma_Argument_Assoc : constant Bare_Pragma_Argument_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Argument_Assoc.Pragma_Argument_Assoc_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Spec_Range =>
declare
N_Bare_Entry_Spec : constant Bare_Entry_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Family_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Spec.Entry_Spec_F_Entry_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Spec_Range =>
declare
N_Bare_Subp_Spec : constant Bare_Subp_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Spec.Subp_Spec_F_Subp_Returns;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_List_Range =>
declare
N_Bare_Component_List : constant Bare_Component_List := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_List.Component_List_F_Components;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_List.Component_List_F_Variant_Part;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Known_Discriminant_Part_Range =>
declare
N_Bare_Known_Discriminant_Part : constant Bare_Known_Discriminant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Known_Discriminant_Part.Known_Discriminant_Part_F_Discr_Specs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Completion_Formal_Params_Range =>
declare
N_Bare_Entry_Completion_Formal_Params : constant Bare_Entry_Completion_Formal_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Completion_Formal_Params.Entry_Completion_Formal_Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal_Part_Range =>
declare
N_Bare_Generic_Formal_Part : constant Bare_Generic_Formal_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal_Part.Generic_Formal_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Record_Def =>
declare
N_Bare_Base_Record_Def : constant Bare_Base_Record_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Record_Def.Base_Record_Def_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Aggregate_Assoc_Range =>
declare
N_Bare_Aggregate_Assoc : constant Bare_Aggregate_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_Designators;
                            return;
                        when 2 =>
                            Result := N_Bare_Aggregate_Assoc.Aggregate_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Assoc_Range =>
declare
N_Bare_Discriminant_Assoc : constant Bare_Discriminant_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Assoc.Discriminant_Assoc_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Assoc.Discriminant_Assoc_F_Discr_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Iterated_Assoc_Range =>
declare
N_Bare_Iterated_Assoc : constant Bare_Iterated_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Iterated_Assoc.Iterated_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Assoc_Range =>
declare
N_Bare_Param_Assoc : constant Bare_Param_Assoc := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_Designator;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Assoc.Param_Assoc_F_R_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Range =>
declare
N_Bare_Abstract_State_Decl : constant Bare_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Abstract_State_Decl.Abstract_State_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Expr_Decl_Range =>
declare
N_Bare_Anonymous_Expr_Decl : constant Bare_Anonymous_Expr_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Expr_Decl.Anonymous_Expr_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Decl_Range =>
declare
N_Bare_Component_Decl : constant Bare_Component_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Component_Def;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Component_Decl.Component_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Spec_Range =>
declare
N_Bare_Discriminant_Spec : constant Bare_Discriminant_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Type_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Discriminant_Spec.Discriminant_Spec_F_Default_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Formal =>
declare
N_Bare_Generic_Formal : constant Bare_Generic_Formal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Formal.Generic_Formal_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Param_Spec_Range =>
declare
N_Bare_Param_Spec : constant Bare_Param_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Mode;
                            return;
                        when 4 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Type_Expr;
                            return;
                        when 5 =>
                            Result := N_Bare_Param_Spec.Param_Spec_F_Default_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Package_Decl =>
declare
N_Bare_Base_Package_Decl : constant Bare_Base_Package_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Public_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_Private_Part;
                            return;
                        when 5 =>
                            Result := N_Bare_Base_Package_Decl.Base_Package_Decl_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Type_Decl.Base_Type_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Type_Decl (K) is
when Ada_Subtype_Decl_Range =>
declare
N_Bare_Subtype_Decl : constant Bare_Subtype_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Subtype;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Decl.Subtype_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Incomplete_Type_Decl_Range =>
declare
N_Bare_Incomplete_Type_Decl : constant Bare_Incomplete_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Incomplete_Type_Decl.Incomplete_Type_Decl_F_Discriminants;
                            return;
                        when others => null;
                    end case;
case Ada_Incomplete_Type_Decl_Range (K) is
when Ada_Incomplete_Tagged_Type_Decl_Range =>
declare
N_Bare_Incomplete_Tagged_Type_Decl : constant Bare_Incomplete_Tagged_Type_Decl := N_Bare_Incomplete_Type_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Incomplete_Tagged_Type_Decl.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Protected_Type_Decl_Range =>
declare
N_Bare_Protected_Type_Decl : constant Bare_Protected_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Interfaces;
                            return;
                        when 5 =>
                            Result := N_Bare_Protected_Type_Decl.Protected_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Type_Decl_Range =>
declare
N_Bare_Task_Type_Decl : constant Bare_Task_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Type_Decl.Task_Type_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Decl_Range =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Discriminants;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Type_Def;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Decl.Type_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Classic_Subp_Decl =>
declare
N_Bare_Classic_Subp_Decl : constant Bare_Classic_Subp_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Classic_Subp_Decl.Classic_Subp_Decl_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Classic_Subp_Decl (K) is
when Ada_Abstract_Subp_Decl_Range =>
declare
N_Bare_Abstract_Subp_Decl : constant Bare_Abstract_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Abstract_Subp_Decl.Abstract_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Formal_Subp_Decl =>
declare
N_Bare_Formal_Subp_Decl : constant Bare_Formal_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Default_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Formal_Subp_Decl.Formal_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Decl_Range =>
declare
N_Bare_Subp_Decl : constant Bare_Subp_Decl := N_Bare_Classic_Subp_Decl;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Decl.Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Entry_Decl_Range =>
declare
N_Bare_Entry_Decl : constant Bare_Entry_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Decl.Entry_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Literal_Decl_Range =>
declare
N_Bare_Enum_Literal_Decl : constant Bare_Enum_Literal_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Literal_Decl.Enum_Literal_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Internal_Range =>
declare
N_Bare_Generic_Subp_Internal : constant Bare_Generic_Subp_Internal := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Subp_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Internal.Generic_Subp_Internal_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Subp_Body =>
declare
N_Bare_Base_Subp_Body : constant Bare_Base_Subp_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Subp_Body.Base_Subp_Body_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
case Ada_Base_Subp_Body (K) is
when Ada_Expr_Function_Range =>
declare
N_Bare_Expr_Function : constant Bare_Expr_Function := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Expr;
                            return;
                        when 4 =>
                            Result := N_Bare_Expr_Function.Expr_Function_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Null_Subp_Decl_Range =>
declare
N_Bare_Null_Subp_Decl : constant Bare_Null_Subp_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Null_Subp_Decl.Null_Subp_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Range =>
declare
N_Bare_Subp_Body : constant Bare_Subp_Body := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Aspects;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Decls;
                            return;
                        when 5 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_Stmts;
                            return;
                        when 6 =>
                            Result := N_Bare_Subp_Body.Subp_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Renaming_Decl_Range =>
declare
N_Bare_Subp_Renaming_Decl : constant Bare_Subp_Renaming_Decl := N_Bare_Base_Subp_Body;
begin
case Index is
                        when 3 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Subp_Renaming_Decl.Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Package_Body_Stub_Range =>
declare
N_Bare_Package_Body_Stub : constant Bare_Package_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body_Stub.Package_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Stub_Range =>
declare
N_Bare_Protected_Body_Stub : constant Bare_Protected_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body_Stub.Protected_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subp_Body_Stub_Range =>
declare
N_Bare_Subp_Body_Stub : constant Bare_Subp_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Subp_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Subp_Body_Stub.Subp_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Stub_Range =>
declare
N_Bare_Task_Body_Stub : constant Bare_Task_Body_Stub := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body_Stub.Task_Body_Stub_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Body_Range =>
declare
N_Bare_Entry_Body : constant Bare_Entry_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Entry_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Index_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Barrier;
                            return;
                        when 5 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Decls;
                            return;
                        when 6 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_Stmts;
                            return;
                        when 7 =>
                            Result := N_Bare_Entry_Body.Entry_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Body_Range =>
declare
N_Bare_Package_Body : constant Bare_Package_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Package_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Package_Body.Package_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Package_Body.Package_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Body_Range =>
declare
N_Bare_Protected_Body : constant Bare_Protected_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Protected_Body.Protected_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Body_Range =>
declare
N_Bare_Task_Body : constant Bare_Task_Body := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Decls;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Body.Task_Body_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Task_Body.Task_Body_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Entry_Index_Spec_Range =>
declare
N_Bare_Entry_Index_Spec : constant Bare_Entry_Index_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Entry_Index_Spec.Entry_Index_Spec_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Decl_Range =>
declare
N_Bare_Exception_Decl : constant Bare_Exception_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Decl.Exception_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exception_Handler_Range =>
declare
N_Bare_Exception_Handler : constant Bare_Exception_Handler := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Handled_Exceptions;
                            return;
                        when 3 =>
                            Result := N_Bare_Exception_Handler.Exception_Handler_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Var_Decl_Range =>
declare
N_Bare_For_Loop_Var_Decl : constant Bare_For_Loop_Var_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Var_Decl.For_Loop_Var_Decl_F_Id_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Decl =>
declare
N_Bare_Generic_Decl : constant Bare_Generic_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Decl.Generic_Decl_F_Formal_Part;
                            return;
                        when others => null;
                    end case;
case Ada_Generic_Decl (K) is
when Ada_Generic_Package_Decl_Range =>
declare
N_Bare_Generic_Package_Decl : constant Bare_Generic_Package_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Package_Decl.Generic_Package_Decl_F_Package_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Decl_Range =>
declare
N_Bare_Generic_Subp_Decl : constant Bare_Generic_Subp_Decl := N_Bare_Generic_Decl;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Decl.Generic_Subp_Decl_F_Subp_Decl;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Generic_Package_Instantiation_Range =>
declare
N_Bare_Generic_Package_Instantiation : constant Bare_Generic_Package_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Generic_Pkg_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Params;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Package_Instantiation.Generic_Package_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Instantiation_Range =>
declare
N_Bare_Generic_Subp_Instantiation : constant Bare_Generic_Subp_Instantiation := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Overriding;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Kind;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Subp_Name;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Generic_Subp_Name;
                            return;
                        when 5 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Params;
                            return;
                        when 6 =>
                            Result := N_Bare_Generic_Subp_Instantiation.Generic_Subp_Instantiation_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Package_Renaming_Decl_Range =>
declare
N_Bare_Generic_Package_Renaming_Decl : constant Bare_Generic_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Package_Renaming_Decl.Generic_Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Generic_Subp_Renaming_Decl_Range =>
declare
N_Bare_Generic_Subp_Renaming_Decl : constant Bare_Generic_Subp_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Renames;
                            return;
                        when 4 =>
                            Result := N_Bare_Generic_Subp_Renaming_Decl.Generic_Subp_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Decl_Range =>
declare
N_Bare_Label_Decl : constant Bare_Label_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label_Decl.Label_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Decl_Range =>
declare
N_Bare_Named_Stmt_Decl : constant Bare_Named_Stmt_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt_Decl.Named_Stmt_Decl_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Number_Decl_Range =>
declare
N_Bare_Number_Decl : constant Bare_Number_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Number_Decl.Number_Decl_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Object_Decl_Range =>
declare
N_Bare_Object_Decl : constant Bare_Object_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Ids;
                            return;
                        when 2 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Aliased;
                            return;
                        when 3 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Mode;
                            return;
                        when 5 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Type_Expr;
                            return;
                        when 6 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Default_Expr;
                            return;
                        when 7 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Renaming_Clause;
                            return;
                        when 8 =>
                            Result := N_Bare_Object_Decl.Object_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Package_Renaming_Decl_Range =>
declare
N_Bare_Package_Renaming_Decl : constant Bare_Package_Renaming_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Renames;
                            return;
                        when 3 =>
                            Result := N_Bare_Package_Renaming_Decl.Package_Renaming_Decl_F_Aspects;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Protected_Decl_Range =>
declare
N_Bare_Single_Protected_Decl : constant Bare_Single_Protected_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Aspects;
                            return;
                        when 3 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Interfaces;
                            return;
                        when 4 =>
                            Result := N_Bare_Single_Protected_Decl.Single_Protected_Decl_F_Definition;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Single_Task_Decl_Range =>
declare
N_Bare_Single_Task_Decl : constant Bare_Single_Task_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Single_Task_Decl.Single_Task_Decl_F_Task_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Alternative_Range =>
declare
N_Bare_Case_Stmt_Alternative : constant Bare_Case_Stmt_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt_Alternative.Case_Stmt_Alternative_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Prelude;
                            return;
                        when 2 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Body;
                            return;
                        when 3 =>
                            Result := N_Bare_Compilation_Unit.Compilation_Unit_F_Pragmas;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Clause_Range =>
declare
N_Bare_Component_Clause : constant Bare_Component_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Position;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Clause.Component_Clause_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Component_Def_Range =>
declare
N_Bare_Component_Def : constant Bare_Component_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Aliased;
                            return;
                        when 2 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Has_Constant;
                            return;
                        when 3 =>
                            Result := N_Bare_Component_Def.Component_Def_F_Type_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delta_Constraint_Range =>
declare
N_Bare_Delta_Constraint : constant Bare_Delta_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Delta_Constraint.Delta_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Digits_Constraint_Range =>
declare
N_Bare_Digits_Constraint : constant Bare_Digits_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Digits_Constraint.Digits_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discriminant_Constraint_Range =>
declare
N_Bare_Discriminant_Constraint : constant Bare_Discriminant_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discriminant_Constraint.Discriminant_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Index_Constraint_Range =>
declare
N_Bare_Index_Constraint : constant Bare_Index_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Index_Constraint.Index_Constraint_F_Constraints;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Constraint_Range =>
declare
N_Bare_Range_Constraint : constant Bare_Range_Constraint := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Constraint.Range_Constraint_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Declarative_Part_Range =>
declare
N_Bare_Declarative_Part : constant Bare_Declarative_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Declarative_Part.Declarative_Part_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Expr_Part_Range =>
declare
N_Bare_Elsif_Expr_Part : constant Bare_Elsif_Expr_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Expr_Part.Elsif_Expr_Part_F_Then_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Elsif_Stmt_Part_Range =>
declare
N_Bare_Elsif_Stmt_Part : constant Bare_Elsif_Stmt_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Elsif_Stmt_Part.Elsif_Stmt_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abstract_State_Decl_Expr_Range =>
declare
N_Bare_Abstract_State_Decl_Expr : constant Bare_Abstract_State_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abstract_State_Decl_Expr.Abstract_State_Decl_Expr_F_State_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Allocator_Range =>
declare
N_Bare_Allocator : constant Bare_Allocator := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Allocator.Allocator_F_Subpool;
                            return;
                        when 2 =>
                            Result := N_Bare_Allocator.Allocator_F_Type_Or_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Base_Aggregate =>
declare
N_Bare_Base_Aggregate : constant Bare_Base_Aggregate := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Ancestor_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Aggregate.Base_Aggregate_F_Assocs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Bin_Op_Range =>
declare
N_Bare_Bin_Op : constant Bare_Bin_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Left;
                            return;
                        when 2 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Bin_Op.Bin_Op_F_Right;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Alternative_Range =>
declare
N_Bare_Case_Expr_Alternative : constant Bare_Case_Expr_Alternative := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr_Alternative.Case_Expr_Alternative_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Expr_Range =>
declare
N_Bare_Case_Expr : constant Bare_Case_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Expr.Case_Expr_F_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Expr_Range =>
declare
N_Bare_If_Expr : constant Bare_If_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Then_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Expr.If_Expr_F_Else_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Contract_Cases_Range =>
declare
N_Bare_Contract_Cases : constant Bare_Contract_Cases := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Contract_Cases.Contract_Cases_F_Contract_Cases;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Expr_Range =>
declare
N_Bare_Decl_Expr : constant Bare_Decl_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Expr.Decl_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Membership_Expr_Range =>
declare
N_Bare_Membership_Expr : constant Bare_Membership_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Op;
                            return;
                        when 3 =>
                            Result := N_Bare_Membership_Expr.Membership_Expr_F_Membership_Exprs;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Attribute;
                            return;
                        when 3 =>
                            Result := N_Bare_Attribute_Ref.Attribute_Ref_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Call_Expr.Call_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Defining_Name_Range =>
declare
N_Bare_Defining_Name : constant Bare_Defining_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Defining_Name.Defining_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Discrete_Subtype_Name_Range =>
declare
N_Bare_Discrete_Subtype_Name : constant Bare_Discrete_Subtype_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Discrete_Subtype_Name.Discrete_Subtype_Name_F_Subtype;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Dotted_Name_Range =>
declare
N_Bare_Dotted_Name : constant Bare_Dotted_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Dotted_Name.Dotted_Name_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_End_Name_Range =>
declare
N_Bare_End_Name : constant Bare_End_Name := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_End_Name.End_Name_F_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Explicit_Deref.Explicit_Deref_F_Prefix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Qual_Expr_Range =>
declare
N_Bare_Qual_Expr : constant Bare_Qual_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Prefix;
                            return;
                        when 2 =>
                            Result := N_Bare_Qual_Expr.Qual_Expr_F_Suffix;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Expr_Range =>
declare
N_Bare_Paren_Expr : constant Bare_Paren_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Expr.Paren_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Quantified_Expr_Range =>
declare
N_Bare_Quantified_Expr : constant Bare_Quantified_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Quantifier;
                            return;
                        when 2 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Loop_Spec;
                            return;
                        when 3 =>
                            Result := N_Bare_Quantified_Expr.Quantified_Expr_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Expr_Range =>
declare
N_Bare_Raise_Expr : constant Bare_Raise_Expr := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Expr.Raise_Expr_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Un_Op_Range =>
declare
N_Bare_Un_Op : constant Bare_Un_Op := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Op;
                            return;
                        when 2 =>
                            Result := N_Bare_Un_Op.Un_Op_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Handled_Stmts_Range =>
declare
N_Bare_Handled_Stmts : constant Bare_Handled_Stmts := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Handled_Stmts.Handled_Stmts_F_Exceptions;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Library_Item_Range =>
declare
N_Bare_Library_Item : constant Bare_Library_Item := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Has_Private;
                            return;
                        when 2 =>
                            Result := N_Bare_Library_Item.Library_Item_F_Item;
                            return;
                        when others => null;
                    end case;
end;
when Ada_For_Loop_Spec_Range =>
declare
N_Bare_For_Loop_Spec : constant Bare_For_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Var_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Loop_Type;
                            return;
                        when 3 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Has_Reverse;
                            return;
                        when 4 =>
                            Result := N_Bare_For_Loop_Spec.For_Loop_Spec_F_Iter_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_While_Loop_Spec_Range =>
declare
N_Bare_While_Loop_Spec : constant Bare_While_Loop_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_While_Loop_Spec.While_Loop_Spec_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Multi_Abstract_State_Decl_Range =>
declare
N_Bare_Multi_Abstract_State_Decl : constant Bare_Multi_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Multi_Abstract_State_Decl.Multi_Abstract_State_Decl_F_Decls;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Params_Range =>
declare
N_Bare_Params : constant Bare_Params := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Params.Params_F_Params;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Paren_Abstract_State_Decl_Range =>
declare
N_Bare_Paren_Abstract_State_Decl : constant Bare_Paren_Abstract_State_Decl := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Paren_Abstract_State_Decl.Paren_Abstract_State_Decl_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Pragma_Node_Range =>
declare
N_Bare_Pragma_Node : constant Bare_Pragma_Node := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Id;
                            return;
                        when 2 =>
                            Result := N_Bare_Pragma_Node.Pragma_Node_F_Args;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Protected_Def_Range =>
declare
N_Bare_Protected_Def : constant Bare_Protected_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Public_Part;
                            return;
                        when 2 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_Private_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Protected_Def.Protected_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Range_Spec_Range =>
declare
N_Bare_Range_Spec : constant Bare_Range_Spec := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Range_Spec.Range_Spec_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Renaming_Clause_Range =>
declare
N_Bare_Renaming_Clause : constant Bare_Renaming_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Renaming_Clause.Renaming_Clause_F_Renamed_Object;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_When_Part_Range =>
declare
N_Bare_Select_When_Part : constant Bare_Select_When_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_When_Part.Select_When_Part_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Accept_Stmt_Range =>
declare
N_Bare_Accept_Stmt : constant Bare_Accept_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Entry_Index_Expr;
                            return;
                        when 3 =>
                            Result := N_Bare_Accept_Stmt.Accept_Stmt_F_Params;
                            return;
                        when others => null;
                    end case;
case Ada_Accept_Stmt_Range (K) is
when Ada_Accept_Stmt_With_Stmts_Range =>
declare
N_Bare_Accept_Stmt_With_Stmts : constant Bare_Accept_Stmt_With_Stmts := N_Bare_Accept_Stmt;
begin
case Index is
                        when 4 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_Stmts;
                            return;
                        when 5 =>
                            Result := N_Bare_Accept_Stmt_With_Stmts.Accept_Stmt_With_Stmts_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Base_Loop_Stmt =>
declare
N_Bare_Base_Loop_Stmt : constant Bare_Base_Loop_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Spec;
                            return;
                        when 2 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Base_Loop_Stmt.Base_Loop_Stmt_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Begin_Block_Range =>
declare
N_Bare_Begin_Block : constant Bare_Begin_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_Stmts;
                            return;
                        when 2 =>
                            Result := N_Bare_Begin_Block.Begin_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decl_Block_Range =>
declare
N_Bare_Decl_Block : constant Bare_Decl_Block := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Decls;
                            return;
                        when 2 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Decl_Block.Decl_Block_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Case_Stmt_Range =>
declare
N_Bare_Case_Stmt : constant Bare_Case_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_Case_Stmt.Case_Stmt_F_Alternatives;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Extended_Return_Stmt_Range =>
declare
N_Bare_Extended_Return_Stmt : constant Bare_Extended_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Extended_Return_Stmt.Extended_Return_Stmt_F_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_If_Stmt_Range =>
declare
N_Bare_If_Stmt : constant Bare_If_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Cond_Expr;
                            return;
                        when 2 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Then_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Alternatives;
                            return;
                        when 4 =>
                            Result := N_Bare_If_Stmt.If_Stmt_F_Else_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Named_Stmt_Range =>
declare
N_Bare_Named_Stmt : constant Bare_Named_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Decl;
                            return;
                        when 2 =>
                            Result := N_Bare_Named_Stmt.Named_Stmt_F_Stmt;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Select_Stmt_Range =>
declare
N_Bare_Select_Stmt : constant Bare_Select_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Guards;
                            return;
                        when 2 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Else_Stmts;
                            return;
                        when 3 =>
                            Result := N_Bare_Select_Stmt.Select_Stmt_F_Abort_Stmts;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Abort_Stmt_Range =>
declare
N_Bare_Abort_Stmt : constant Bare_Abort_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Abort_Stmt.Abort_Stmt_F_Names;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Assign_Stmt_Range =>
declare
N_Bare_Assign_Stmt : constant Bare_Assign_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Dest;
                            return;
                        when 2 =>
                            Result := N_Bare_Assign_Stmt.Assign_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Call_Stmt_Range =>
declare
N_Bare_Call_Stmt : constant Bare_Call_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Call_Stmt.Call_Stmt_F_Call;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Delay_Stmt_Range =>
declare
N_Bare_Delay_Stmt : constant Bare_Delay_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Has_Until;
                            return;
                        when 2 =>
                            Result := N_Bare_Delay_Stmt.Delay_Stmt_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Exit_Stmt_Range =>
declare
N_Bare_Exit_Stmt : constant Bare_Exit_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Loop_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Exit_Stmt.Exit_Stmt_F_Cond_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Goto_Stmt_Range =>
declare
N_Bare_Goto_Stmt : constant Bare_Goto_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Goto_Stmt.Goto_Stmt_F_Label_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Label_Range =>
declare
N_Bare_Label : constant Bare_Label := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Label.Label_F_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Raise_Stmt_Range =>
declare
N_Bare_Raise_Stmt : constant Bare_Raise_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Exception_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Raise_Stmt.Raise_Stmt_F_Error_Message;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Requeue_Stmt_Range =>
declare
N_Bare_Requeue_Stmt : constant Bare_Requeue_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Call_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Requeue_Stmt.Requeue_Stmt_F_Has_Abort;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Return_Stmt_Range =>
declare
N_Bare_Return_Stmt : constant Bare_Return_Stmt := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Return_Stmt.Return_Stmt_F_Return_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subunit_Range =>
declare
N_Bare_Subunit : constant Bare_Subunit := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subunit.Subunit_F_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Subunit.Subunit_F_Body;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Task_Def_Range =>
declare
N_Bare_Task_Def : constant Bare_Task_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Interfaces;
                            return;
                        when 2 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Public_Part;
                            return;
                        when 3 =>
                            Result := N_Bare_Task_Def.Task_Def_F_Private_Part;
                            return;
                        when 4 =>
                            Result := N_Bare_Task_Def.Task_Def_F_End_Name;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Access_Def =>
declare
N_Bare_Access_Def : constant Bare_Access_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Access_Def.Access_Def_F_Has_Not_Null;
                            return;
                        when others => null;
                    end case;
case Ada_Access_Def (K) is
when Ada_Access_To_Subp_Def_Range =>
declare
N_Bare_Access_To_Subp_Def : constant Bare_Access_To_Subp_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Has_Protected;
                            return;
                        when 3 =>
                            Result := N_Bare_Access_To_Subp_Def.Access_To_Subp_Def_F_Subp_Spec;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Access_Def_Range =>
declare
N_Bare_Anonymous_Type_Access_Def : constant Bare_Anonymous_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Anonymous_Type_Access_Def.Anonymous_Type_Access_Def_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Type_Access_Def_Range =>
declare
N_Bare_Type_Access_Def : constant Bare_Type_Access_Def := N_Bare_Access_Def;
begin
case Index is
                        when 2 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_All;
                            return;
                        when 3 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Has_Constant;
                            return;
                        when 4 =>
                            Result := N_Bare_Type_Access_Def.Type_Access_Def_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
end;
when Ada_Array_Type_Def_Range =>
declare
N_Bare_Array_Type_Def : constant Bare_Array_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Indices;
                            return;
                        when 2 =>
                            Result := N_Bare_Array_Type_Def.Array_Type_Def_F_Component_Type;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Derived_Type_Def_Range =>
declare
N_Bare_Derived_Type_Def : constant Bare_Derived_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Limited;
                            return;
                        when 3 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_Synchronized;
                            return;
                        when 4 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Subtype_Indication;
                            return;
                        when 5 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Interfaces;
                            return;
                        when 6 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Record_Extension;
                            return;
                        when 7 =>
                            Result := N_Bare_Derived_Type_Def.Derived_Type_Def_F_Has_With_Private;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Enum_Type_Def_Range =>
declare
N_Bare_Enum_Type_Def : constant Bare_Enum_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Enum_Type_Def.Enum_Type_Def_F_Enum_Literals;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Interface_Type_Def_Range =>
declare
N_Bare_Interface_Type_Def : constant Bare_Interface_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interface_Kind;
                            return;
                        when 2 =>
                            Result := N_Bare_Interface_Type_Def.Interface_Type_Def_F_Interfaces;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Mod_Int_Type_Def_Range =>
declare
N_Bare_Mod_Int_Type_Def : constant Bare_Mod_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Mod_Int_Type_Def.Mod_Int_Type_Def_F_Expr;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Private_Type_Def_Range =>
declare
N_Bare_Private_Type_Def : constant Bare_Private_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Private_Type_Def.Private_Type_Def_F_Has_Limited;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Decimal_Fixed_Point_Def_Range =>
declare
N_Bare_Decimal_Fixed_Point_Def : constant Bare_Decimal_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Digits;
                            return;
                        when 3 =>
                            Result := N_Bare_Decimal_Fixed_Point_Def.Decimal_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Floating_Point_Def_Range =>
declare
N_Bare_Floating_Point_Def : constant Bare_Floating_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Num_Digits;
                            return;
                        when 2 =>
                            Result := N_Bare_Floating_Point_Def.Floating_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Ordinary_Fixed_Point_Def_Range =>
declare
N_Bare_Ordinary_Fixed_Point_Def : constant Bare_Ordinary_Fixed_Point_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Delta;
                            return;
                        when 2 =>
                            Result := N_Bare_Ordinary_Fixed_Point_Def.Ordinary_Fixed_Point_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Record_Type_Def_Range =>
declare
N_Bare_Record_Type_Def : constant Bare_Record_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Abstract;
                            return;
                        when 2 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Tagged;
                            return;
                        when 3 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Has_Limited;
                            return;
                        when 4 =>
                            Result := N_Bare_Record_Type_Def.Record_Type_Def_F_Record_Def;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Signed_Int_Type_Def_Range =>
declare
N_Bare_Signed_Int_Type_Def : constant Bare_Signed_Int_Type_Def := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Signed_Int_Type_Def.Signed_Int_Type_Def_F_Range;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Anonymous_Type_Range =>
declare
N_Bare_Anonymous_Type : constant Bare_Anonymous_Type := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Anonymous_Type.Anonymous_Type_F_Type_Decl;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Subtype_Indication_Range =>
declare
N_Bare_Subtype_Indication : constant Bare_Subtype_Indication := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Has_Not_Null;
                            return;
                        when 2 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Name;
                            return;
                        when 3 =>
                            Result := N_Bare_Subtype_Indication.Subtype_Indication_F_Constraint;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Unconstrained_Array_Index_Range =>
declare
N_Bare_Unconstrained_Array_Index : constant Bare_Unconstrained_Array_Index := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Unconstrained_Array_Index.Unconstrained_Array_Index_F_Subtype_Indication;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Package_Clause_Range =>
declare
N_Bare_Use_Package_Clause : constant Bare_Use_Package_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Package_Clause.Use_Package_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Use_Type_Clause_Range =>
declare
N_Bare_Use_Type_Clause : constant Bare_Use_Type_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Has_All;
                            return;
                        when 2 =>
                            Result := N_Bare_Use_Type_Clause.Use_Type_Clause_F_Types;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Range =>
declare
N_Bare_Variant : constant Bare_Variant := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant.Variant_F_Choices;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant.Variant_F_Components;
                            return;
                        when others => null;
                    end case;
end;
when Ada_Variant_Part_Range =>
declare
N_Bare_Variant_Part : constant Bare_Variant_Part := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Discr_Name;
                            return;
                        when 2 =>
                            Result := N_Bare_Variant_Part.Variant_Part_F_Variant;
                            return;
                        when others => null;
                    end case;
end;
when Ada_With_Clause_Range =>
declare
N_Bare_With_Clause : constant Bare_With_Clause := Node;
begin
case Index is
                        when 1 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Limited;
                            return;
                        when 2 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Has_Private;
                            return;
                        when 3 =>
                            Result := N_Bare_With_Clause.With_Clause_F_Packages;
                            return;
                        when others => null;
                    end case;
end;
when others => null;
end case;
      --  Execution should reach this point iff nothing matched this index, so
      --  we must be out of bounds.
      Index_In_Bounds := False;
   end Get_Child;
   -----------
   -- Print --
   -----------
   procedure Print
     (Node        : Bare_Ada_Node;
      Show_Slocs  : Boolean;
      Line_Prefix : String := "")
   is
      K               : constant Ada_Node_Kind_Type := Node.Kind;
      Attr_Prefix     : constant String := Line_Prefix & "|";
      Children_Prefix : constant String := Line_Prefix & "|  ";
   begin
      Put (Line_Prefix & Kind_Name (Node));
      if Show_Slocs then
         Put ("[" & Image (Sloc_Range (Node)) & "]");
      end if;
      if Is_Incomplete (Node) then
         Put (" <<INCOMPLETE>>");
      end if;
      if Is_Token_Node (Node.Kind) then
         Put_Line (": " & Image (Text (Node)));
      elsif Node.Kind not in Ada_Ada_List then
         New_Line;
      end if;
         --  List nodes are displayed in a special way (they have no field)
         if K in Ada_Ada_List then
            if Node.Count = 0 then
               Put_Line (": <empty list>");
               return;
            end if;
            New_Line;
            for Child of Node.Nodes (1 .. Node.Count) loop
               if Child /= null then
                  Print (Child, Show_Slocs, Line_Prefix & "|  ");
               end if;
            end loop;
            return;
         end if;
         --  This is for regular nodes: display each field
         declare
            use Libadalang.Introspection_Implementation;
            Field_List : constant Syntax_Field_Reference_Array :=
               Syntax_Fields (K);
         begin
            for I in Field_List'Range loop
               declare
                  Child : constant Bare_Ada_Node :=
                     Implementation.Child (Node, I);
               begin
                  Put
                    (Attr_Prefix
                     & Image (Syntax_Field_Name (Field_List (I)))
                     & ":");
                  if Child /= null then
                     New_Line;
                     Print (Child, Show_Slocs, Children_Prefix);
                  else
                     Put_Line (" <null>");
                  end if;
               end;
            end loop;
         end;
   end Print;
   ------------
   -- Parent --
   ------------
   function Parent (Node : Bare_Ada_Node) return Bare_Ada_Node is
   begin
      return Node.Parent;
   end Parent;
   ------------------
   -- Stored_Token --
   ------------------
   function Stored_Token
     (Node  : Bare_Ada_Node;
      Token : Token_Reference) return Token_Index
   is
      Index : constant Token_Or_Trivia_Index := Get_Token_Index (Token);
   begin
      if Node.Unit.TDH'Access /= Get_Token_TDH (Token) then
         raise Property_Error with
           ("Cannot associate a token and a node from different analysis"
            & " units");
      elsif Index.Trivia /= No_Token_Index then
         raise Property_Error with
           ("A node cannot hold trivia");
      end if;
      return Index.Token;
   end Stored_Token;
   -------------------------
   -- Children_And_Trivia --
   -------------------------
   function Children_And_Trivia
     (Node : Bare_Ada_Node) return Bare_Children_Array
   is
      package Children_Vectors is new Ada.Containers.Vectors
        (Positive, Bare_Child_Record);
      use Children_Vectors;
      Ret_Vec : Vector;
      TDH     : Token_Data_Handler renames Node.Unit.TDH;
      procedure Append_Trivias (First, Last : Token_Index);
      --  Append all the trivias of tokens between indices First and Last to
      --  the returned vector.
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array;
      --  Return an array for all children in Parent that are not null
      --------------------
      -- Append_Trivias --
      --------------------
      procedure Append_Trivias (First, Last : Token_Index) is
      begin
         for I in First .. Last loop
            for D of Get_Trivias (TDH, I) loop
               Ret_Vec.Append
                 (Bare_Child_Record'
                    (Kind   => Trivia,
                     Trivia => Wrap_Token_Reference (TDH'Access, (I, D))));
            end loop;
         end loop;
      end Append_Trivias;
      ---------------------
      -- Filter_Children --
      ---------------------
      function Filter_Children
        (Parent : Bare_Ada_Node)
         return Internal_Bare_Ada_Node_Array
      is
         Children : constant Internal_Bare_Ada_Node_Array :=
            Implementation.Children (Parent);
         Result   : Internal_Bare_Ada_Node_Array (Children'Range);
         Next     : Integer := Result'First;
      begin
         for I in Children'Range loop
            if Children (I) /= null then
               Result (Next) := Children (I);
               Next := Next + 1;
            end if;
         end loop;
         return Result (Result'First .. Next - 1);
      end Filter_Children;
      First_Child : constant Positive := 1;
      N_Children  : constant Internal_Bare_Ada_Node_Array :=
         Filter_Children (Node);
   begin
      if N_Children'Length > 0
        and then (Node.Token_Start_Index
                    /= N_Children (First_Child).Token_Start_Index)
      then
         Append_Trivias (Node.Token_Start_Index,
                         N_Children (First_Child).Token_Start_Index - 1);
      end if;
      --  Append each node to Ret_Vec, and append trivia that follow after each
      --  non-ghost nodes.
      for I in N_Children'Range loop
         Ret_Vec.Append (Bare_Child_Record'(Child, N_Children (I)));
         if not Is_Ghost (N_Children (I)) then
            Append_Trivias (N_Children (I).Token_End_Index,
                            (if I = N_Children'Last
                             then Node.Token_End_Index - 1
                             else N_Children (I + 1).Token_Start_Index - 1));
         end if;
      end loop;
      declare
         A : Bare_Children_Array (1 .. Natural (Ret_Vec.Length));
      begin
         for I in A'Range loop
            A (I) := Ret_Vec.Element (I);
         end loop;
         return A;
      end;
   end Children_And_Trivia;
   --------------
   -- Is_Ghost --
   --------------
   function Is_Ghost (Node : Bare_Ada_Node) return Boolean
   is (Node.Token_End_Index = No_Token_Index);
   -------------------
   -- Is_Incomplete --
   -------------------
   function Is_Incomplete (Node : Bare_Ada_Node) return Boolean
   is
      LGC : Bare_Ada_Node;
   begin
     if Is_List_Node (Node.Kind) then
        LGC := (if Last_Child_Index (Node) /= 0
                then Child (Node, Last_Child_Index (Node))
                else null);
        return LGC /= null and then Is_Incomplete (LGC);
      else
         return Node.Last_Attempted_Child > -1;
      end if;
   end;
   -----------------
   -- Token_Start --
   -----------------
   function Token_Start (Node : Bare_Ada_Node) return Token_Reference
   is (Token (Node, Node.Token_Start_Index));
   ---------------
   -- Token_End --
   ---------------
   function Token_End (Node : Bare_Ada_Node) return Token_Reference
   is
     (if Node.Token_End_Index = No_Token_Index
      then Token_Start (Node)
      else Token (Node, Node.Token_End_Index));
   -----------
   -- Token --
   -----------
   function Token
     (Node  : Bare_Ada_Node;
      Index : Token_Index) return Token_Reference
   is
     (Wrap_Token_Reference (Token_Data (Node.Unit), (Index, No_Token_Index)));
   ---------
   -- "<" --
   ---------
   function "<" (Left, Right : Bare_Ada_Node) return Boolean is
   begin
      --  Reject invalid inputs
      if Left /= null and Is_Synthetic (Left) then
         raise Property_Error with "left node is synthetic";
      elsif Right /= null and Is_Synthetic (Right) then
         raise Property_Error with "right node is synthetic";
      end if;
      --  Null nodes come first
      if Left = null then
         return Right /= null;
      elsif Right = null then
         return False;
      end if;
      --  So we have two non-null nodes. Sort by unit filename
      if Left.Unit < Right.Unit then
         return True;
      elsif Left.Unit /= Right.Unit then
         return False;
      end if;
      --  Both nodes come from the same unit: compare their token indexes
      if Left.Token_Start_Index < Right.Token_Start_Index then
         return True;
      elsif Left.Token_Start_Index > Right.Token_Start_Index then
         return False;
      else
         return Left.Token_End_Index < Right.Token_End_Index;
      end if;
   end "<";
   -------------
   -- Is_Null --
   -------------
   function Is_Null (Node : Bare_Ada_Node) return Boolean
   is (Node = null);
   ----------
   -- Kind --
   ----------
   function Kind (Node : Bare_Ada_Node) return Ada_Node_Kind_Type
   is (Node.Kind);
   -----------------
   -- Child_Index --
   -----------------
   function Child_Index (Node : Bare_Ada_Node) return Integer
   is
      N : Bare_Ada_Node := null;
   begin
      if Node.Parent = null then
         raise Property_Error with
            "Trying to get the child index of a root node";
      end if;
      for I in First_Child_Index (Node.Parent)
            .. Last_Child_Index (Node.Parent)
      loop
         N := Child (Node.Parent, I);
         if N = Node then
            return I - 1;
         end if;
      end loop;
      --  If we reach this point, then Node isn't a Child of Node.Parent. This
      --  is not supposed to happen.
      raise Program_Error;
   end Child_Index;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      Offset : Integer) return Bare_Ada_Node is
   begin
      --  Root nodes have no sibling: handle them now to avoid invalid requests
      --  in the code below.
      if Node.Parent = null then
         return null;
      end if;
      declare
         Node_Index : constant Positive := Child_Index (Node) + 1;
         --  Child_Index is 0-based, but the Child primitive expects a 1-based
         --  index.
         Sibling_Index : constant Integer := Node_Index + Offset;
      begin
         --  Child returns null for out-of-bound indexes
         return (if Sibling_Index >= 1
                 then Child (Node.Parent, Sibling_Index)
                 else null);
      end;
   end Fetch_Sibling;
   -------------------
   -- Fetch_Sibling --
   -------------------
   function Fetch_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info;
      Offset : Integer) return Internal_Entity
   is
      Sibling : constant Bare_Ada_Node := Fetch_Sibling (Node, Offset);
   begin
      --  Don't forget to clear entity info if the result is nul
      return (if Sibling = null
              then No_Entity
              else (Sibling, E_Info));
   end Fetch_Sibling;
   ----------------------
   -- Previous_Sibling --
   ----------------------
   function Previous_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, -1);
   end Previous_Sibling;
   ------------------
   -- Next_Sibling --
   ------------------
   function Next_Sibling
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      return Fetch_Sibling (Node, E_Info, 1);
   end Next_Sibling;
   -------------
   -- Combine --
   -------------
   function Combine
     (L, R : Internal_Metadata) return Internal_Metadata
   is
      Ret : Internal_Metadata := No_Metadata;
   begin
         Ret.Dottable_Subp := L.Dottable_Subp or R.Dottable_Subp;
         Ret.Access_Entity := L.Access_Entity or R.Access_Entity;
         if L.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := R.Primitive;
         elsif R.Primitive = No_Bare_Ada_Node then
            Ret.Primitive := L.Primitive;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
         if L.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := R.Primitive_Real_Type;
         elsif R.Primitive_Real_Type = No_Bare_Ada_Node then
            Ret.Primitive_Real_Type := L.Primitive_Real_Type;
         else
            raise Property_Error with "Wrong combine for metadata field";
         end if;
      return Ret;
   end Combine;
   -------------------------------
   -- Create_Static_Lexical_Env --
   -------------------------------
   function Create_Static_Lexical_Env
     (Parent            : Env_Getter;
      Node              : Bare_Ada_Node;
      Transitive_Parent : Boolean := False) return Lexical_Env
   is
      Unit : constant Internal_Unit :=
        (if Node = null then null else Node.Unit);
   begin
      return Result : Lexical_Env := Create_Lexical_Env
        (Parent, Node, Transitive_Parent, Convert_Unit (Unit))
      do
         if Unit /= null then
            Register_Destroyable (Unit, Unwrap (Result.Env));
         end if;
      end return;
   end Create_Static_Lexical_Env;
   ---------
   -- Get --
   ---------
   function Get
     (A     : AST_Envs.Entity_Array;
      Index : Integer) return Internal_Entity
   is
      function Length (A : AST_Envs.Entity_Array) return Natural
      is (A'Length);
      function Get
        (A     : AST_Envs.Entity_Array;
         Index : Integer) return Internal_Entity
      is (A (Index + 1)); --  A is 1-based but Index is 0-based
      function Relative_Get is new Langkit_Support.Relative_Get
        (Item_Type     => Entity,
         Sequence_Type => AST_Envs.Entity_Array,
         Length        => Length,
         Get           => Get);
      Result : Internal_Entity;
   begin
      if Relative_Get (A, Index, Result) then
         return Result;
      else
         raise Property_Error with "out-of-bounds array access";
      end if;
   end Get;
   -----------
   -- Group --
   -----------
   function Group
     (Envs   : Lexical_Env_Array_Access;
      Env_Md : Internal_Metadata := No_Metadata) return Lexical_Env
   is (Group (Lexical_Env_Array (Envs.Items), Env_Md));
      --
      --  Untyped wrappers for Bare_Ada_Node
      --
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Base_Type_Decl
      --
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
      --
      --  Untyped wrappers for Bare_Type_Decl
      --
function Type_Decl_P_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
function Type_Decl_P_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Body_Node
      --
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Package_Body
      --
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Task_Body
      --
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env;
      --
      --  Untyped wrappers for Bare_Generic_Instantiation
      --
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access;
      --
      --  Untyped wrappers for Bare_Generic_Subp_Instantiation
      --
function Generic_Subp_Instantiation_P_Designated_Subp
  (E : Entity
  ) return Internal_Entity;
      --
      --  Untyped wrappers for Bare_Name
      --
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env;
   ------------------
   -- Children_Env --
   ------------------
   function Children_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is (Rebind_Env (Node.Self_Env, E_Info));
   --------------
   -- Node_Env --
   --------------
   function Node_Env
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Lexical_Env
   is
      function Get_Base_Env return Lexical_Env;
      --  Return the environment that we need to rebind before returning
      ------------------
      -- Get_Base_Env --
      ------------------
      function Get_Base_Env return Lexical_Env is
         pragma Warnings (Off, "referenced");
         function Get_Parent_Env return Lexical_Env;
         pragma Warnings (On, "referenced");
         --------------------
         -- Get_Parent_Env --
         --------------------
         function Get_Parent_Env return Lexical_Env is
            Parent : constant Lexical_Env := AST_Envs.Parent (Node.Self_Env);
         begin
            --  If Node is the root scope or the empty environment, Parent can
            --  be a wrapper around the null node. Turn this into the
            --  Empty_Env, as null envs are erroneous values in properties.
            return (if Unwrap (Parent) = null
                    then Empty_Env
                    else Parent);
         end Get_Parent_Env;
      begin
         return
           (if Node.Kind in Ada_Generic_Package_Internal | Ada_Package_Decl | Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl | Ada_Protected_Type_Decl | Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl | Ada_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl | Ada_Subp_Decl | Ada_Entry_Decl | Ada_Enum_Literal_Decl | Ada_Generic_Subp_Internal | Ada_Expr_Function | Ada_Null_Subp_Decl | Ada_Subp_Body | Ada_Subp_Renaming_Decl | Ada_Package_Body_Stub | Ada_Protected_Body_Stub | Ada_Subp_Body_Stub | Ada_Task_Body_Stub | Ada_Entry_Body | Ada_Package_Body | Ada_Protected_Body | Ada_Task_Body | Ada_Exception_Handler | Ada_Generic_Package_Decl | Ada_Generic_Subp_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Subp_Instantiation | Ada_Generic_Package_Renaming_Decl | Ada_Generic_Subp_Renaming_Decl | Ada_Package_Renaming_Decl | Ada_Single_Protected_Decl | Ada_Single_Task_Decl | Ada_Private_Part | Ada_Decl_Expr | Ada_Accept_Stmt | Ada_Accept_Stmt_With_Stmts | Ada_For_Loop_Stmt | Ada_Begin_Block | Ada_Decl_Block | Ada_Extended_Return_Stmt | Ada_Named_Stmt | Ada_Access_To_Subp_Def
            then Get_Parent_Env
            else Node.Self_Env);
      end Get_Base_Env;
      Base_Env : Lexical_Env := Get_Base_Env;
      Result   : constant Lexical_Env := Rebind_Env (Base_Env, E_Info);
   begin
      Dec_Ref (Base_Env);
      return Result;
   end Node_Env;
   ------------
   -- Parent --
   ------------
   function Parent
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity is
   begin
      --  TODO: shed entity information as appropriate
      return (Node.Parent, E_Info);
   end Parent;
   -------------
   -- Parents --
   -------------
   function Parents
     (Node      : Bare_Ada_Node;
      With_Self : Boolean := True;
      E_Info    : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Parents : Bare_Ada_Node_Array_Access := Parents (Node, With_Self);
      Result       : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Parents.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Parents.Items'Range loop
         Result.Items (I) := (Bare_Parents.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Parents);
      return Result;
   end Parents;
   --------------
   -- Children --
   --------------
   function Children
     (Node   : Bare_Ada_Node;
      E_Info : Internal_Entity_Info := No_Entity_Info)
      return Internal_Entity_Array_Access
   is
      Bare_Children : Bare_Ada_Node_Array_Access := Children (Node);
      Result        : Internal_Entity_Array_Access :=
         Create_Internal_Entity_Array (Bare_Children.N);
   begin
      --  TODO: shed entity information as appropriate
      for I in Bare_Children.Items'Range loop
         Result.Items (I) := (Bare_Children.Items (I), E_Info);
      end loop;
      Dec_Ref (Bare_Children);
      return Result;
   end Children;
   --------------------------------
   -- Assign_Names_To_Logic_Vars --
   --------------------------------
   procedure Assign_Names_To_Logic_Vars (Node : Bare_Ada_Node) is
      pragma Warnings (Off, "referenced");
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String);
      --  Assign a name to the LV logic variable. Node must be the node that
      --  owns LV, and Field must be the name of the field in Node that holds
      --  LV.
      ------------
      -- Assign --
      ------------
      procedure Assign
        (Node  : Bare_Ada_Node;
         LV    : in out Logic_Var_Record;
         Field : String) is
      begin
         LV.Dbg_Name := new String'
           (Image (Short_Text_Image (Node)) & "." & Field);
      end Assign;
      K : constant Ada_Node_Kind_Type := Node.Kind;
      pragma Warnings (On, "referenced");
   begin
      case Ada_Ada_Node (K) is
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Assign (N_Bare_Attribute_Ref, N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var, "Attribute_Ref_R_Ref_Var");
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Assign (N_Bare_Call_Expr, N_Bare_Call_Expr.Call_Expr_R_Called_Spec, "Call_Expr_R_Called_Spec");
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Assign (N_Bare_Explicit_Deref, N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec, "Explicit_Deref_R_Called_Spec");
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Assign (N_Bare_Target_Name, N_Bare_Target_Name.Target_Name_R_Ref_Var, "Target_Name_R_Ref_Var");
end;
when others => null;
end case;
      for Child of Internal_Bare_Ada_Node_Array'(Children (Node)) loop
         if Child /= null then
            Assign_Names_To_Logic_Vars (Child);
         end if;
      end loop;
   end Assign_Names_To_Logic_Vars;
   ----------------
   -- Text_Image --
   ----------------
   function Text_Image (Ent : Internal_Entity) return Text_Type is
   begin
      if Ent.Node /= null then
         declare
            Node_Image : constant Text_Type := Short_Text_Image (Ent.Node);
         begin
            return
            (if Ent.Info.Rebindings /= null
             then "<| "
             & Node_Image (Node_Image'First + 1 .. Node_Image'Last - 1) & " "
             & AST_Envs.Text_Image (Ent.Info.Rebindings) & " |>"
             else Node_Image);
         end;
      else
         return "None";
      end if;
   end Text_Image;
   ---------------------
   -- Full_Sloc_Image --
   ---------------------
   function Full_Sloc_Image (Node : Bare_Ada_Node) return Character_Type_Array_Access
   is
      Res      : constant Text_Type :=
        To_Text
          (Ada.Directories.Simple_Name
             (Get_Filename (Unit (Node))))
           & ":" & To_Text (Image (Start_Sloc (Sloc_Range (Node)))) & ": ";
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (Res'Length);
   begin
      Result.Items := Res;
      return Result;
   end Full_Sloc_Image;
   -----------
   -- Image --
   -----------
   function Image (Ent : Internal_Entity) return String is
      Result : constant Text_Type := Text_Image (Ent);
   begin
      return Image (Result);
   end Image;
   ---------------
   -- Can_Reach --
   ---------------
   function Can_Reach (El, From : Bare_Ada_Node) return Boolean is
   begin
      --  Since this function is only used to implement sequential semantics in
      --  envs, we consider that elements coming from different units are
      --  always visible for each other, and let the user implement language
      --  specific visibility rules in the DSL.
      if El = null or else From = null or else El.Unit /= From.Unit then
         return True;
      end if;
      return Compare
        (Start_Sloc (Sloc_Range (El)),
         Start_Sloc (Sloc_Range (From))) = After;
   end Can_Reach;
   -----------------
   -- Hash_Entity --
   -----------------
   function Hash_Entity (Self : Internal_Entity) return Hash_Type is
   begin
      return Combine (Hash (Self.Node), Hash (Self.Info.Rebindings));
   end Hash_Entity;
   --------------------
   -- Compare_Entity --
   --------------------
   function Compare_Entity (Left, Right : Internal_Entity) return Boolean
   is
   begin
      return (Left.Node = Right.Node
              and then Left.Info.Rebindings = Right.Info.Rebindings);
   end Compare_Entity;
   --------------------------------
   -- Create_Dynamic_Lexical_Env --
   --------------------------------
   function Create_Dynamic_Lexical_Env
     (Self              : Bare_Ada_Node;
      Assocs_Getter     : Inner_Env_Assocs_Resolver;
      Assoc_Resolver    : Entity_Resolver;
      Transitive_Parent : Boolean) return Lexical_Env
   is
      Unit : constant Internal_Unit := Self.Unit;
   begin
      --  This restriction is necessary to avoid relocation issues when
      --  Self.Self_Env is terminated.
      if Is_Foreign_Strict (Self.Self_Env, Self) then
         raise Property_Error with
           ("cannot create a dynamic lexical env when Self.Self_Env is"
            & " foreign");
      end if;
      return Result : constant Lexical_Env := Create_Dynamic_Lexical_Env
        (Parent            => No_Env_Getter,
         Node              => Self,
         Transitive_Parent => Transitive_Parent,
         Owner             => Convert_Unit (Unit),
         Assocs_Getter     => Assocs_Getter,
         Assoc_Resolver    => Assoc_Resolver)
      do
         --  Since dynamic lexical environments can only be created in lazy
         --  field initializers, it is fine to tie Result's lifetime to the
         --  its owning unit's lifetime.
         Register_Destroyable (Unit, Unwrap (Result));
      end return;
   end Create_Dynamic_Lexical_Env;
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node);
   --  Helper for the Register_Destroyable above
   ------------
   -- Length --
   ------------
   function Length (Node : Bare_Ada_List) return Natural
   is (if Node = null then 0 else Children_Count (Node));
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (B : Boolean) return String is
      begin
         return (if B then "True" else "False");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (I : Integer) return String is
      begin
         return Integer'Image (I);
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (S : Symbol_Type) return String is
      begin
         return (if S = null
                 then "None"
                 else Image (S.all, With_Quotes => True));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Env : Lexical_Env) return String is
      begin
         case Env.Kind is
         when Static_Primary =>
            return "<LexicalEnv static-primary for "
                   & Trace_Image (Env_Node (Env)) & ">";
         when others =>
            return "<LexicalEnv synthetic>";
         end case;
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (R : Env_Rebindings) return String is
      begin
         return Image (Text_Image (R));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Unit : Internal_Unit) return String is
      begin
         return "Internal_Unit (""" & Basename (Unit) & """)";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Eq : Logic_Equation) return String is
         pragma Unreferenced (Eq);
      begin
         return "<LogicEquation>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (Var : Logic_Var) return String is
         pragma Unreferenced (Var);
      begin
         return "<LogicVariable>";
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image (K : Analysis_Unit_Kind) return String is
      begin
         return Analysis_Unit_Kind'Image (K);
      end Trace_Image;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Metadata) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine ((Hash (R.Dottable_Subp), Hash (R.Access_Entity), Hash (R.Primitive), Hash (R.Primitive_Real_Type)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Metadata) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Dottable_Subp => " & Trace_Image (R.Dottable_Subp)
                        & ", "
                     & "Access_Entity => " & Trace_Image (R.Access_Entity)
                        & ", "
                     & "Primitive => " & Trace_Image (R.Primitive)
                        & ", "
                     & "Primitive_Real_Type => " & Trace_Image (R.Primitive_Real_Type)
               & ")");
      end Trace_Image;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Info) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine ((Hash (R.MD), Hash (R.Rebindings), Hash (R.From_Rebound)));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "MD => " & Trace_Image (R.MD)
                        & ", "
                     & "Rebindings => " & Trace_Image (R.Rebindings)
                        & ", "
                     & "From_Rebound => " & Trace_Image (R.From_Rebound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity
        (Node : Bare_Ada_Node; Info : Internal_Entity_Info)
         return Internal_Entity is
      begin
         if Node = null then
            return No_Entity;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr
        (Node : Bare_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Expr is
      begin
         if Node = null then
            return No_Entity_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Aspect) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Exists => " & Trace_Image (R.Exists)
                        & ", "
                     & "Node => " & Trace_Image (R.Node)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl
        (Node : Bare_Basic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Basic_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Completion_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Is_Dot_Call => " & Trace_Image (R.Is_Dot_Call)
                        & ", "
                     & "Is_Visible => " & Trace_Image (R.Is_Visible)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Ada_List
        (Node : Bare_Ada_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_List is
      begin
         if Node = null then
            return No_Entity_Ada_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ada_Node_List
        (Node : Bare_Ada_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Ada_Node_List is
      begin
         if Node = null then
            return No_Entity_Ada_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ada_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Alternatives_List
        (Node : Bare_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name
        (Node : Bare_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Name is
      begin
         if Node = null then
            return No_Entity_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Tok_Node
        (Node : Bare_Single_Tok_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Tok_Node is
      begin
         if Node = null then
            return No_Entity_Single_Tok_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Tok_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Id
        (Node : Bare_Base_Id; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Id is
      begin
         if Node = null then
            return No_Entity_Base_Id;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Id) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier
        (Node : Bare_Identifier; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier is
      begin
         if Node = null then
            return No_Entity_Identifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Discriminant_Values) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Discriminant => " & Trace_Image (R.Discriminant)
                        & ", "
                     & "Values => " & Trace_Image (R.Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Doc_Annotation) is
      begin
               Inc_Ref (R.Key);
               Inc_Ref (R.Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Doc_Annotation) is
      begin
               Dec_Ref (R.Key);
               Dec_Ref (R.Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Doc_Annotation) return Boolean is
      begin
         return Equivalent (L.Key, R.Key) and then Equivalent (L.Value, R.Value);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Doc_Annotation) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      function Create_Internal_Entity_Abort_Node
        (Node : Bare_Abort_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Node is
      begin
         if Node = null then
            return No_Entity_Abort_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Absent
        (Node : Bare_Abort_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Absent is
      begin
         if Node = null then
            return No_Entity_Abort_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Present
        (Node : Bare_Abort_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Present is
      begin
         if Node = null then
            return No_Entity_Abort_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt
        (Node : Bare_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt is
      begin
         if Node = null then
            return No_Entity_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Simple_Stmt
        (Node : Bare_Simple_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Simple_Stmt is
      begin
         if Node = null then
            return No_Entity_Simple_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Simple_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abort_Stmt
        (Node : Bare_Abort_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Abort_Stmt is
      begin
         if Node = null then
            return No_Entity_Abort_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abort_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Node
        (Node : Bare_Abstract_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Node is
      begin
         if Node = null then
            return No_Entity_Abstract_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Absent
        (Node : Bare_Abstract_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Absent is
      begin
         if Node = null then
            return No_Entity_Abstract_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Subp_Decl
        (Node : Bare_Basic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Basic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classic_Subp_Decl
        (Node : Bare_Classic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Classic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Subp_Decl
        (Node : Bare_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Formal_Subp_Decl
        (Node : Bare_Abstract_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Present
        (Node : Bare_Abstract_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Present is
      begin
         if Node = null then
            return No_Entity_Abstract_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl
        (Node : Bare_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_Expr
        (Node : Bare_Abstract_State_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_State_Decl_List
        (Node : Bare_Abstract_State_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_State_Decl_List is
      begin
         if Node = null then
            return No_Entity_Abstract_State_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_State_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Abstract_Subp_Decl
        (Node : Bare_Abstract_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Abstract_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Abstract_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Abstract_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Composite_Stmt
        (Node : Bare_Composite_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Composite_Stmt is
      begin
         if Node = null then
            return No_Entity_Composite_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Composite_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt
        (Node : Bare_Accept_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node : Bare_Accept_Stmt_With_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Accept_Stmt_With_Stmts is
      begin
         if Node = null then
            return No_Entity_Accept_Stmt_With_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Accept_Stmt_With_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Def
        (Node : Bare_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Def is
      begin
         if Node = null then
            return No_Entity_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_Def
        (Node : Bare_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_Def is
      begin
         if Node = null then
            return No_Entity_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Access_To_Subp_Def
        (Node : Bare_Access_To_Subp_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Access_To_Subp_Def is
      begin
         if Node = null then
            return No_Entity_Access_To_Subp_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Access_To_Subp_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Aggregate
        (Node : Bare_Base_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Aggregate is
      begin
         if Node = null then
            return No_Entity_Base_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate
        (Node : Bare_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate is
      begin
         if Node = null then
            return No_Entity_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc
        (Node : Bare_Basic_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aggregate_Assoc
        (Node : Bare_Aggregate_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aggregate_Assoc is
      begin
         if Node = null then
            return No_Entity_Aggregate_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aggregate_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Node
        (Node : Bare_Aliased_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Node is
      begin
         if Node = null then
            return No_Entity_Aliased_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Absent
        (Node : Bare_Aliased_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Absent is
      begin
         if Node = null then
            return No_Entity_Aliased_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aliased_Present
        (Node : Bare_Aliased_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Aliased_Present is
      begin
         if Node = null then
            return No_Entity_Aliased_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aliased_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Node
        (Node : Bare_All_Node; Info : Internal_Entity_Info)
         return Internal_Entity_All_Node is
      begin
         if Node = null then
            return No_Entity_All_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Absent
        (Node : Bare_All_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_All_Absent is
      begin
         if Node = null then
            return No_Entity_All_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_All_Present
        (Node : Bare_All_Present; Info : Internal_Entity_Info)
         return Internal_Entity_All_Present is
      begin
         if Node = null then
            return No_Entity_All_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_All_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Allocator
        (Node : Bare_Allocator; Info : Internal_Entity_Info)
         return Internal_Entity_Allocator is
      begin
         if Node = null then
            return No_Entity_Allocator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Allocator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Expr_Decl
        (Node : Bare_Anonymous_Expr_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Expr_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Expr_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Expr_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Expr
        (Node : Bare_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type
        (Node : Bare_Anonymous_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Access_Def
        (Node : Bare_Base_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Base_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Access_Def
        (Node : Bare_Anonymous_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Type_Decl
        (Node : Bare_Base_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Base_Type_Decl) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Decl
        (Node : Bare_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Anonymous_Type_Decl
        (Node : Bare_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Indices
        (Node : Bare_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Array_Type_Def
        (Node : Bare_Array_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Array_Type_Def is
      begin
         if Node = null then
            return No_Entity_Array_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Array_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc
        (Node : Bare_Aspect_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Assoc_List
        (Node : Bare_Aspect_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Aspect_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Clause
        (Node : Bare_Aspect_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Clause is
      begin
         if Node = null then
            return No_Entity_Aspect_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Aspect_Spec
        (Node : Bare_Aspect_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Aspect_Spec is
      begin
         if Node = null then
            return No_Entity_Aspect_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Aspect_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assign_Stmt
        (Node : Bare_Assign_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Assign_Stmt is
      begin
         if Node = null then
            return No_Entity_Assign_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assign_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Assoc_List
        (Node : Bare_Basic_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Basic_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Assoc_List
        (Node : Bare_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_At_Clause
        (Node : Bare_At_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_At_Clause is
      begin
         if Node = null then
            return No_Entity_At_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_At_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Def_Clause
        (Node : Bare_Attribute_Def_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Def_Clause is
      begin
         if Node = null then
            return No_Entity_Attribute_Def_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Def_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Attribute_Ref
        (Node : Bare_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc
        (Node : Bare_Base_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc is
      begin
         if Node = null then
            return No_Entity_Base_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Assoc_List
        (Node : Bare_Base_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Base_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Decl
        (Node : Bare_Base_Formal_Param_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Formal_Param_Holder
        (Node : Bare_Base_Formal_Param_Holder; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Formal_Param_Holder is
      begin
         if Node = null then
            return No_Entity_Base_Formal_Param_Holder;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Formal_Param_Holder) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Loop_Stmt
        (Node : Bare_Base_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Base_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Package_Decl
        (Node : Bare_Base_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Record_Def
        (Node : Bare_Base_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Record_Def is
      begin
         if Node = null then
            return No_Entity_Base_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Node
        (Node : Bare_Body_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Node is
      begin
         if Node = null then
            return No_Entity_Body_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Body
        (Node : Bare_Base_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subp_Spec
        (Node : Bare_Base_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Base_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Base_Subtype_Decl
        (Node : Bare_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Basic_Decl_List
        (Node : Bare_Basic_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Basic_Decl_List is
      begin
         if Node = null then
            return No_Entity_Basic_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Basic_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Block_Stmt
        (Node : Bare_Block_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Block_Stmt is
      begin
         if Node = null then
            return No_Entity_Block_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Block_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Begin_Block
        (Node : Bare_Begin_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Begin_Block is
      begin
         if Node = null then
            return No_Entity_Begin_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Begin_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bin_Op
        (Node : Bare_Bin_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Bin_Op is
      begin
         if Node = null then
            return No_Entity_Bin_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bin_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Body_Stub
        (Node : Bare_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Box_Expr
        (Node : Bare_Box_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Box_Expr is
      begin
         if Node = null then
            return No_Entity_Box_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Box_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Aggregate
        (Node : Bare_Bracket_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Aggregate
        (Node : Bare_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Bracket_Delta_Aggregate
        (Node : Bare_Bracket_Delta_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Bracket_Delta_Aggregate is
      begin
         if Node = null then
            return No_Entity_Bracket_Delta_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Bracket_Delta_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Expr
        (Node : Bare_Call_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Expr is
      begin
         if Node = null then
            return No_Entity_Call_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Call_Stmt
        (Node : Bare_Call_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Call_Stmt is
      begin
         if Node = null then
            return No_Entity_Call_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Call_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Cond_Expr
        (Node : Bare_Cond_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Cond_Expr is
      begin
         if Node = null then
            return No_Entity_Cond_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Cond_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr
        (Node : Bare_Case_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr is
      begin
         if Node = null then
            return No_Entity_Case_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative
        (Node : Bare_Case_Expr_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Expr_Alternative_List
        (Node : Bare_Case_Expr_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Expr_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Expr_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Expr_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt
        (Node : Bare_Case_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt is
      begin
         if Node = null then
            return No_Entity_Case_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative
        (Node : Bare_Case_Stmt_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Case_Stmt_Alternative_List
        (Node : Bare_Case_Stmt_Alternative_List; Info : Internal_Entity_Info)
         return Internal_Entity_Case_Stmt_Alternative_List is
      begin
         if Node = null then
            return No_Entity_Case_Stmt_Alternative_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Case_Stmt_Alternative_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Char_Literal
        (Node : Bare_Char_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Char_Literal is
      begin
         if Node = null then
            return No_Entity_Char_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Char_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Classwide_Type_Decl
        (Node : Bare_Classwide_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Classwide_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Classwide_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Classwide_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit
        (Node : Bare_Compilation_Unit; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit;
         end if;
         return (Node => Node, Info => Info);
      end;
      ----------
      -- Hash --
      ----------
      pragma Warnings (Off, "referenced");
      function Hash (R : Internal_Entity_Compilation_Unit) return Hash_Type is
         pragma Warnings (On, "referenced");
      begin
            return Combine
              (Hash (R.Node), Hash (R.Info));
      end Hash;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Compilation_Unit_List
        (Node : Bare_Compilation_Unit_List; Info : Internal_Entity_Info)
         return Internal_Entity_Compilation_Unit_List is
      begin
         if Node = null then
            return No_Entity_Compilation_Unit_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Compilation_Unit_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Clause
        (Node : Bare_Component_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Clause is
      begin
         if Node = null then
            return No_Entity_Component_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Decl
        (Node : Bare_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_Def
        (Node : Bare_Component_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Component_Def is
      begin
         if Node = null then
            return No_Entity_Component_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Component_List
        (Node : Bare_Component_List; Info : Internal_Entity_Info)
         return Internal_Entity_Component_List is
      begin
         if Node = null then
            return No_Entity_Component_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Component_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Concrete_Formal_Subp_Decl
        (Node : Bare_Concrete_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Concrete_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Concrete_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Concrete_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Node
        (Node : Bare_Constant_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Node is
      begin
         if Node = null then
            return No_Entity_Constant_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Absent
        (Node : Bare_Constant_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Absent is
      begin
         if Node = null then
            return No_Entity_Constant_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constant_Present
        (Node : Bare_Constant_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Constant_Present is
      begin
         if Node = null then
            return No_Entity_Constant_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constant_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Array_Indices
        (Node : Bare_Constrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Constrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Indication
        (Node : Bare_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constrained_Subtype_Indication
        (Node : Bare_Constrained_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Constrained_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Constrained_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constrained_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint
        (Node : Bare_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint is
      begin
         if Node = null then
            return No_Entity_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Constraint_List
        (Node : Bare_Constraint_List; Info : Internal_Entity_Info)
         return Internal_Entity_Constraint_List is
      begin
         if Node = null then
            return No_Entity_Constraint_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Constraint_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc
        (Node : Bare_Contract_Case_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Case_Assoc_List
        (Node : Bare_Contract_Case_Assoc_List; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Case_Assoc_List is
      begin
         if Node = null then
            return No_Entity_Contract_Case_Assoc_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Case_Assoc_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Contract_Cases
        (Node : Bare_Contract_Cases; Info : Internal_Entity_Info)
         return Internal_Entity_Contract_Cases is
      begin
         if Node = null then
            return No_Entity_Contract_Cases;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Contract_Cases) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Type_Def
        (Node : Bare_Real_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Type_Def is
      begin
         if Node = null then
            return No_Entity_Real_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decimal_Fixed_Point_Def
        (Node : Bare_Decimal_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Decimal_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Decimal_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decimal_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Block
        (Node : Bare_Decl_Block; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Block is
      begin
         if Node = null then
            return No_Entity_Decl_Block;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Block) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_Expr
        (Node : Bare_Decl_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_Expr is
      begin
         if Node = null then
            return No_Entity_Decl_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Decl_List
        (Node : Bare_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Decl_List is
      begin
         if Node = null then
            return No_Entity_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Declarative_Part
        (Node : Bare_Declarative_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Declarative_Part is
      begin
         if Node = null then
            return No_Entity_Declarative_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Declarative_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name
        (Node : Bare_Defining_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name is
      begin
         if Node = null then
            return No_Entity_Defining_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Defining_Name_List
        (Node : Bare_Defining_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Defining_Name_List is
      begin
         if Node = null then
            return No_Entity_Defining_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Defining_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delay_Stmt
        (Node : Bare_Delay_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Delay_Stmt is
      begin
         if Node = null then
            return No_Entity_Delay_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delay_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Delta_Constraint
        (Node : Bare_Delta_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Delta_Constraint is
      begin
         if Node = null then
            return No_Entity_Delta_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Delta_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Derived_Type_Def
        (Node : Bare_Derived_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Derived_Type_Def is
      begin
         if Node = null then
            return No_Entity_Derived_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Derived_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Digits_Constraint
        (Node : Bare_Digits_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Digits_Constraint is
      begin
         if Node = null then
            return No_Entity_Digits_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Digits_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Base_Subtype_Decl
        (Node : Bare_Discrete_Base_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Base_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Discrete_Base_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Base_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Indication
        (Node : Bare_Discrete_Subtype_Indication; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Indication is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Indication;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Indication) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discrete_Subtype_Name
        (Node : Bare_Discrete_Subtype_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Discrete_Subtype_Name is
      begin
         if Node = null then
            return No_Entity_Discrete_Subtype_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discrete_Subtype_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Assoc
        (Node : Bare_Discriminant_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Assoc is
      begin
         if Node = null then
            return No_Entity_Discriminant_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Identifier_List
        (Node : Bare_Identifier_List; Info : Internal_Entity_Info)
         return Internal_Entity_Identifier_List is
      begin
         if Node = null then
            return No_Entity_Identifier_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Identifier_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Choice_List
        (Node : Bare_Discriminant_Choice_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Choice_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Choice_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Choice_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Constraint
        (Node : Bare_Discriminant_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Constraint is
      begin
         if Node = null then
            return No_Entity_Discriminant_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Part
        (Node : Bare_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec
        (Node : Bare_Discriminant_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Discriminant_Spec_List
        (Node : Bare_Discriminant_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Discriminant_Spec_List is
      begin
         if Node = null then
            return No_Entity_Discriminant_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Discriminant_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Dotted_Name
        (Node : Bare_Dotted_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Dotted_Name is
      begin
         if Node = null then
            return No_Entity_Dotted_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Dotted_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part
        (Node : Bare_Elsif_Expr_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Expr_Part_List
        (Node : Bare_Elsif_Expr_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Expr_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Expr_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Expr_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part
        (Node : Bare_Elsif_Stmt_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Elsif_Stmt_Part_List
        (Node : Bare_Elsif_Stmt_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Elsif_Stmt_Part_List is
      begin
         if Node = null then
            return No_Entity_Elsif_Stmt_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Elsif_Stmt_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_End_Name
        (Node : Bare_End_Name; Info : Internal_Entity_Info)
         return Internal_Entity_End_Name is
      begin
         if Node = null then
            return No_Entity_End_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_End_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Body
        (Node : Bare_Entry_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Body is
      begin
         if Node = null then
            return No_Entity_Entry_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Completion_Formal_Params
        (Node : Bare_Entry_Completion_Formal_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Completion_Formal_Params is
      begin
         if Node = null then
            return No_Entity_Entry_Completion_Formal_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Completion_Formal_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Decl
        (Node : Bare_Entry_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Decl is
      begin
         if Node = null then
            return No_Entity_Entry_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Index_Spec
        (Node : Bare_Entry_Index_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Index_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Index_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Index_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Entry_Spec
        (Node : Bare_Entry_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Entry_Spec is
      begin
         if Node = null then
            return No_Entity_Entry_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Entry_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Lit_Synth_Type_Expr
        (Node : Bare_Enum_Lit_Synth_Type_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Lit_Synth_Type_Expr is
      begin
         if Node = null then
            return No_Entity_Enum_Lit_Synth_Type_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Lit_Synth_Type_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl
        (Node : Bare_Enum_Literal_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Literal_Decl_List
        (Node : Bare_Enum_Literal_Decl_List; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Literal_Decl_List is
      begin
         if Node = null then
            return No_Entity_Enum_Literal_Decl_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Literal_Decl_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Rep_Clause
        (Node : Bare_Enum_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Enum_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Subp_Spec
        (Node : Bare_Enum_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Enum_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Enum_Type_Def
        (Node : Bare_Enum_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Enum_Type_Def is
      begin
         if Node = null then
            return No_Entity_Enum_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Enum_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Decl
        (Node : Bare_Error_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Decl is
      begin
         if Node = null then
            return No_Entity_Error_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Error_Stmt
        (Node : Bare_Error_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Error_Stmt is
      begin
         if Node = null then
            return No_Entity_Error_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Error_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Decl
        (Node : Bare_Exception_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Decl is
      begin
         if Node = null then
            return No_Entity_Exception_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exception_Handler
        (Node : Bare_Exception_Handler; Info : Internal_Entity_Info)
         return Internal_Entity_Exception_Handler is
      begin
         if Node = null then
            return No_Entity_Exception_Handler;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exception_Handler) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Exit_Stmt
        (Node : Bare_Exit_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Exit_Stmt is
      begin
         if Node = null then
            return No_Entity_Exit_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Exit_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Explicit_Deref
        (Node : Bare_Explicit_Deref; Info : Internal_Entity_Info)
         return Internal_Entity_Explicit_Deref is
      begin
         if Node = null then
            return No_Entity_Explicit_Deref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Explicit_Deref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_List
        (Node : Bare_Expr_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_List is
      begin
         if Node = null then
            return No_Entity_Expr_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Alternatives_List
        (Node : Bare_Expr_Alternatives_List; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Alternatives_List is
      begin
         if Node = null then
            return No_Entity_Expr_Alternatives_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Alternatives_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Expr_Function
        (Node : Bare_Expr_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Expr_Function is
      begin
         if Node = null then
            return No_Entity_Expr_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Expr_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt
        (Node : Bare_Extended_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Object_Decl
        (Node : Bare_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Extended_Return_Stmt_Object_Decl
        (Node : Bare_Extended_Return_Stmt_Object_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Extended_Return_Stmt_Object_Decl is
      begin
         if Node = null then
            return No_Entity_Extended_Return_Stmt_Object_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Extended_Return_Stmt_Object_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Floating_Point_Def
        (Node : Bare_Floating_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Floating_Point_Def is
      begin
         if Node = null then
            return No_Entity_Floating_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Floating_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Spec
        (Node : Bare_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Spec
        (Node : Bare_For_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_For_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Stmt
        (Node : Bare_For_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_For_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_For_Loop_Var_Decl
        (Node : Bare_For_Loop_Var_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_For_Loop_Var_Decl is
      begin
         if Node = null then
            return No_Entity_For_Loop_Var_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_For_Loop_Var_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Formal_Discrete_Type_Def
        (Node : Bare_Formal_Discrete_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Formal_Discrete_Type_Def is
      begin
         if Node = null then
            return No_Entity_Formal_Discrete_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Formal_Discrete_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Decl
        (Node : Bare_Generic_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal
        (Node : Bare_Generic_Formal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal is
      begin
         if Node = null then
            return No_Entity_Generic_Formal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Obj_Decl
        (Node : Bare_Generic_Formal_Obj_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Obj_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Obj_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Obj_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Package
        (Node : Bare_Generic_Formal_Package; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Package is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Package;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Package) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Part
        (Node : Bare_Generic_Formal_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Part is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node : Bare_Generic_Formal_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Formal_Type_Decl
        (Node : Bare_Generic_Formal_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Formal_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Formal_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Formal_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Instantiation
        (Node : Bare_Generic_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Decl
        (Node : Bare_Generic_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Instantiation
        (Node : Bare_Generic_Package_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Internal
        (Node : Bare_Generic_Package_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Renaming_Decl
        (Node : Bare_Generic_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Package_Renaming_Decl
        (Node : Bare_Generic_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Decl
        (Node : Bare_Generic_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Instantiation
        (Node : Bare_Generic_Subp_Instantiation; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Instantiation is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Instantiation;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Instantiation) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Internal
        (Node : Bare_Generic_Subp_Internal; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Internal is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Internal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Internal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Generic_Subp_Renaming_Decl
        (Node : Bare_Generic_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Generic_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Generic_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Generic_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Goto_Stmt
        (Node : Bare_Goto_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Goto_Stmt is
      begin
         if Node = null then
            return No_Entity_Goto_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Goto_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Handled_Stmts
        (Node : Bare_Handled_Stmts; Info : Internal_Entity_Info)
         return Internal_Entity_Handled_Stmts is
      begin
         if Node = null then
            return No_Entity_Handled_Stmts;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Handled_Stmts) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Expr
        (Node : Bare_If_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_If_Expr is
      begin
         if Node = null then
            return No_Entity_If_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_If_Stmt
        (Node : Bare_If_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_If_Stmt is
      begin
         if Node = null then
            return No_Entity_If_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_If_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Type_Decl
        (Node : Bare_Incomplete_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Incomplete_Tagged_Type_Decl
        (Node : Bare_Incomplete_Tagged_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Incomplete_Tagged_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Incomplete_Tagged_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Incomplete_Tagged_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Index_Constraint
        (Node : Bare_Index_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Index_Constraint is
      begin
         if Node = null then
            return No_Entity_Index_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Index_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Num_Literal
        (Node : Bare_Num_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Num_Literal is
      begin
         if Node = null then
            return No_Entity_Num_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Num_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Int_Literal
        (Node : Bare_Int_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Int_Literal is
      begin
         if Node = null then
            return No_Entity_Int_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Int_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind
        (Node : Bare_Interface_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind is
      begin
         if Node = null then
            return No_Entity_Interface_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Limited
        (Node : Bare_Interface_Kind_Limited; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Limited is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Limited;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Limited) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Protected
        (Node : Bare_Interface_Kind_Protected; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Protected is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Protected;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Protected) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Synchronized
        (Node : Bare_Interface_Kind_Synchronized; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Synchronized is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Synchronized;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Synchronized) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Kind_Task
        (Node : Bare_Interface_Kind_Task; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Kind_Task is
      begin
         if Node = null then
            return No_Entity_Interface_Kind_Task;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Kind_Task) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Interface_Type_Def
        (Node : Bare_Interface_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Interface_Type_Def is
      begin
         if Node = null then
            return No_Entity_Interface_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Interface_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type
        (Node : Bare_Iter_Type; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type is
      begin
         if Node = null then
            return No_Entity_Iter_Type;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_In
        (Node : Bare_Iter_Type_In; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_In is
      begin
         if Node = null then
            return No_Entity_Iter_Type_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iter_Type_Of
        (Node : Bare_Iter_Type_Of; Info : Internal_Entity_Info)
         return Internal_Entity_Iter_Type_Of is
      begin
         if Node = null then
            return No_Entity_Iter_Type_Of;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iter_Type_Of) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Iterated_Assoc
        (Node : Bare_Iterated_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Iterated_Assoc is
      begin
         if Node = null then
            return No_Entity_Iterated_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Iterated_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Known_Discriminant_Part
        (Node : Bare_Known_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Known_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Known_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Known_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label
        (Node : Bare_Label; Info : Internal_Entity_Info)
         return Internal_Entity_Label is
      begin
         if Node = null then
            return No_Entity_Label;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Label_Decl
        (Node : Bare_Label_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Label_Decl is
      begin
         if Node = null then
            return No_Entity_Label_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Label_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Library_Item
        (Node : Bare_Library_Item; Info : Internal_Entity_Info)
         return Internal_Entity_Library_Item is
      begin
         if Node = null then
            return No_Entity_Library_Item;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Library_Item) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Node
        (Node : Bare_Limited_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Node is
      begin
         if Node = null then
            return No_Entity_Limited_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Absent
        (Node : Bare_Limited_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Absent is
      begin
         if Node = null then
            return No_Entity_Limited_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Limited_Present
        (Node : Bare_Limited_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Limited_Present is
      begin
         if Node = null then
            return No_Entity_Limited_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Limited_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Loop_Stmt
        (Node : Bare_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Membership_Expr
        (Node : Bare_Membership_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Membership_Expr is
      begin
         if Node = null then
            return No_Entity_Membership_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Membership_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mod_Int_Type_Def
        (Node : Bare_Mod_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Mod_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Mod_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mod_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode
        (Node : Bare_Mode; Info : Internal_Entity_Info)
         return Internal_Entity_Mode is
      begin
         if Node = null then
            return No_Entity_Mode;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Default
        (Node : Bare_Mode_Default; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Default is
      begin
         if Node = null then
            return No_Entity_Mode_Default;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Default) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In
        (Node : Bare_Mode_In; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In is
      begin
         if Node = null then
            return No_Entity_Mode_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_In_Out
        (Node : Bare_Mode_In_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_In_Out is
      begin
         if Node = null then
            return No_Entity_Mode_In_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_In_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Mode_Out
        (Node : Bare_Mode_Out; Info : Internal_Entity_Info)
         return Internal_Entity_Mode_Out is
      begin
         if Node = null then
            return No_Entity_Mode_Out;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Mode_Out) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Abstract_State_Decl
        (Node : Bare_Multi_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Multi_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Multi_Dim_Array_Assoc
        (Node : Bare_Multi_Dim_Array_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Multi_Dim_Array_Assoc is
      begin
         if Node = null then
            return No_Entity_Multi_Dim_Array_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Multi_Dim_Array_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Name_List
        (Node : Bare_Name_List; Info : Internal_Entity_Info)
         return Internal_Entity_Name_List is
      begin
         if Node = null then
            return No_Entity_Name_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Name_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt
        (Node : Bare_Named_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt is
      begin
         if Node = null then
            return No_Entity_Named_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Named_Stmt_Decl
        (Node : Bare_Named_Stmt_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Named_Stmt_Decl is
      begin
         if Node = null then
            return No_Entity_Named_Stmt_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Named_Stmt_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null
        (Node : Bare_Not_Null; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null is
      begin
         if Node = null then
            return No_Entity_Not_Null;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Absent
        (Node : Bare_Not_Null_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Absent is
      begin
         if Node = null then
            return No_Entity_Not_Null_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Not_Null_Present
        (Node : Bare_Not_Null_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Not_Null_Present is
      begin
         if Node = null then
            return No_Entity_Not_Null_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Not_Null_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Component_Decl
        (Node : Bare_Null_Component_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Component_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Component_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Component_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Literal
        (Node : Bare_Null_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Literal is
      begin
         if Node = null then
            return No_Entity_Null_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Aggregate
        (Node : Bare_Null_Record_Aggregate; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Aggregate is
      begin
         if Node = null then
            return No_Entity_Null_Record_Aggregate;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Aggregate) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Record_Def
        (Node : Bare_Null_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Record_Def is
      begin
         if Node = null then
            return No_Entity_Null_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Stmt
        (Node : Bare_Null_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Stmt is
      begin
         if Node = null then
            return No_Entity_Null_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Null_Subp_Decl
        (Node : Bare_Null_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Null_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Null_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Null_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Number_Decl
        (Node : Bare_Number_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Number_Decl is
      begin
         if Node = null then
            return No_Entity_Number_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Number_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op
        (Node : Bare_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Op is
      begin
         if Node = null then
            return No_Entity_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Abs
        (Node : Bare_Op_Abs; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Abs is
      begin
         if Node = null then
            return No_Entity_Op_Abs;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Abs) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And
        (Node : Bare_Op_And; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And is
      begin
         if Node = null then
            return No_Entity_Op_And;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_And_Then
        (Node : Bare_Op_And_Then; Info : Internal_Entity_Info)
         return Internal_Entity_Op_And_Then is
      begin
         if Node = null then
            return No_Entity_Op_And_Then;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_And_Then) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Concat
        (Node : Bare_Op_Concat; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Concat is
      begin
         if Node = null then
            return No_Entity_Op_Concat;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Concat) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Div
        (Node : Bare_Op_Div; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Div is
      begin
         if Node = null then
            return No_Entity_Op_Div;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Div) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Double_Dot
        (Node : Bare_Op_Double_Dot; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Double_Dot is
      begin
         if Node = null then
            return No_Entity_Op_Double_Dot;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Double_Dot) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Eq
        (Node : Bare_Op_Eq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Eq is
      begin
         if Node = null then
            return No_Entity_Op_Eq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Eq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gt
        (Node : Bare_Op_Gt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gt is
      begin
         if Node = null then
            return No_Entity_Op_Gt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Gte
        (Node : Bare_Op_Gte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Gte is
      begin
         if Node = null then
            return No_Entity_Op_Gte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Gte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_In
        (Node : Bare_Op_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_In is
      begin
         if Node = null then
            return No_Entity_Op_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lt
        (Node : Bare_Op_Lt; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lt is
      begin
         if Node = null then
            return No_Entity_Op_Lt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Lte
        (Node : Bare_Op_Lte; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Lte is
      begin
         if Node = null then
            return No_Entity_Op_Lte;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Lte) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Minus
        (Node : Bare_Op_Minus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Minus is
      begin
         if Node = null then
            return No_Entity_Op_Minus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Minus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mod
        (Node : Bare_Op_Mod; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mod is
      begin
         if Node = null then
            return No_Entity_Op_Mod;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mod) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Mult
        (Node : Bare_Op_Mult; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Mult is
      begin
         if Node = null then
            return No_Entity_Op_Mult;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Mult) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Neq
        (Node : Bare_Op_Neq; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Neq is
      begin
         if Node = null then
            return No_Entity_Op_Neq;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Neq) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not
        (Node : Bare_Op_Not; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not is
      begin
         if Node = null then
            return No_Entity_Op_Not;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Not_In
        (Node : Bare_Op_Not_In; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Not_In is
      begin
         if Node = null then
            return No_Entity_Op_Not_In;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Not_In) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or
        (Node : Bare_Op_Or; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or is
      begin
         if Node = null then
            return No_Entity_Op_Or;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Or_Else
        (Node : Bare_Op_Or_Else; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Or_Else is
      begin
         if Node = null then
            return No_Entity_Op_Or_Else;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Or_Else) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Plus
        (Node : Bare_Op_Plus; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Plus is
      begin
         if Node = null then
            return No_Entity_Op_Plus;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Plus) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Pow
        (Node : Bare_Op_Pow; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Pow is
      begin
         if Node = null then
            return No_Entity_Op_Pow;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Pow) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Rem
        (Node : Bare_Op_Rem; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Rem is
      begin
         if Node = null then
            return No_Entity_Op_Rem;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Rem) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Op_Xor
        (Node : Bare_Op_Xor; Info : Internal_Entity_Info)
         return Internal_Entity_Op_Xor is
      begin
         if Node = null then
            return No_Entity_Op_Xor;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Op_Xor) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Ordinary_Fixed_Point_Def
        (Node : Bare_Ordinary_Fixed_Point_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Ordinary_Fixed_Point_Def is
      begin
         if Node = null then
            return No_Entity_Ordinary_Fixed_Point_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Ordinary_Fixed_Point_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Others_Designator
        (Node : Bare_Others_Designator; Info : Internal_Entity_Info)
         return Internal_Entity_Others_Designator is
      begin
         if Node = null then
            return No_Entity_Others_Designator;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Others_Designator) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Node
        (Node : Bare_Overriding_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Node is
      begin
         if Node = null then
            return No_Entity_Overriding_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Not_Overriding
        (Node : Bare_Overriding_Not_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Not_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Not_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Not_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Overriding
        (Node : Bare_Overriding_Overriding; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Overriding is
      begin
         if Node = null then
            return No_Entity_Overriding_Overriding;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Overriding) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Overriding_Unspecified
        (Node : Bare_Overriding_Unspecified; Info : Internal_Entity_Info)
         return Internal_Entity_Overriding_Unspecified is
      begin
         if Node = null then
            return No_Entity_Overriding_Unspecified;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Overriding_Unspecified) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body
        (Node : Bare_Package_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body is
      begin
         if Node = null then
            return No_Entity_Package_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Body_Stub
        (Node : Bare_Package_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Package_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Decl
        (Node : Bare_Package_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Package_Renaming_Decl
        (Node : Bare_Package_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Package_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Package_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Package_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Assoc
        (Node : Bare_Param_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Assoc is
      begin
         if Node = null then
            return No_Entity_Param_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec
        (Node : Bare_Param_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec is
      begin
         if Node = null then
            return No_Entity_Param_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Param_Spec_List
        (Node : Bare_Param_Spec_List; Info : Internal_Entity_Info)
         return Internal_Entity_Param_Spec_List is
      begin
         if Node = null then
            return No_Entity_Param_Spec_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Param_Spec_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Params
        (Node : Bare_Params; Info : Internal_Entity_Info)
         return Internal_Entity_Params is
      begin
         if Node = null then
            return No_Entity_Params;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Params) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Abstract_State_Decl
        (Node : Bare_Paren_Abstract_State_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Abstract_State_Decl is
      begin
         if Node = null then
            return No_Entity_Paren_Abstract_State_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Abstract_State_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Paren_Expr
        (Node : Bare_Paren_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Paren_Expr is
      begin
         if Node = null then
            return No_Entity_Paren_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Paren_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Parent_List
        (Node : Bare_Parent_List; Info : Internal_Entity_Info)
         return Internal_Entity_Parent_List is
      begin
         if Node = null then
            return No_Entity_Parent_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Parent_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Argument_Assoc
        (Node : Bare_Pragma_Argument_Assoc; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Argument_Assoc is
      begin
         if Node = null then
            return No_Entity_Pragma_Argument_Assoc;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Argument_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node
        (Node : Bare_Pragma_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node is
      begin
         if Node = null then
            return No_Entity_Pragma_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Pragma_Node_List
        (Node : Bare_Pragma_Node_List; Info : Internal_Entity_Info)
         return Internal_Entity_Pragma_Node_List is
      begin
         if Node = null then
            return No_Entity_Pragma_Node_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Pragma_Node_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Node
        (Node : Bare_Private_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Node is
      begin
         if Node = null then
            return No_Entity_Private_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Absent
        (Node : Bare_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Absent is
      begin
         if Node = null then
            return No_Entity_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Part
        (Node : Bare_Private_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Part is
      begin
         if Node = null then
            return No_Entity_Private_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Present
        (Node : Bare_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Present is
      begin
         if Node = null then
            return No_Entity_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Private_Type_Def
        (Node : Bare_Private_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Private_Type_Def is
      begin
         if Node = null then
            return No_Entity_Private_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Private_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Node
        (Node : Bare_Protected_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Node is
      begin
         if Node = null then
            return No_Entity_Protected_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Absent
        (Node : Bare_Protected_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Absent is
      begin
         if Node = null then
            return No_Entity_Protected_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body
        (Node : Bare_Protected_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body is
      begin
         if Node = null then
            return No_Entity_Protected_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Body_Stub
        (Node : Bare_Protected_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Protected_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Def
        (Node : Bare_Protected_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Def is
      begin
         if Node = null then
            return No_Entity_Protected_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Present
        (Node : Bare_Protected_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Present is
      begin
         if Node = null then
            return No_Entity_Protected_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Protected_Type_Decl
        (Node : Bare_Protected_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Protected_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Protected_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Protected_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Public_Part
        (Node : Bare_Public_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Public_Part is
      begin
         if Node = null then
            return No_Entity_Public_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Public_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Qual_Expr
        (Node : Bare_Qual_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Qual_Expr is
      begin
         if Node = null then
            return No_Entity_Qual_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Qual_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantified_Expr
        (Node : Bare_Quantified_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Quantified_Expr is
      begin
         if Node = null then
            return No_Entity_Quantified_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantified_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier
        (Node : Bare_Quantifier; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier is
      begin
         if Node = null then
            return No_Entity_Quantifier;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_All
        (Node : Bare_Quantifier_All; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_All is
      begin
         if Node = null then
            return No_Entity_Quantifier_All;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_All) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Quantifier_Some
        (Node : Bare_Quantifier_Some; Info : Internal_Entity_Info)
         return Internal_Entity_Quantifier_Some is
      begin
         if Node = null then
            return No_Entity_Quantifier_Some;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Quantifier_Some) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Expr
        (Node : Bare_Raise_Expr; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Expr is
      begin
         if Node = null then
            return No_Entity_Raise_Expr;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Raise_Stmt
        (Node : Bare_Raise_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Raise_Stmt is
      begin
         if Node = null then
            return No_Entity_Raise_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Raise_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Constraint
        (Node : Bare_Range_Constraint; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Constraint is
      begin
         if Node = null then
            return No_Entity_Range_Constraint;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Constraint) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Range_Spec
        (Node : Bare_Range_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Range_Spec is
      begin
         if Node = null then
            return No_Entity_Range_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Range_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Real_Literal
        (Node : Bare_Real_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_Real_Literal is
      begin
         if Node = null then
            return No_Entity_Real_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Real_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Def
        (Node : Bare_Record_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Def is
      begin
         if Node = null then
            return No_Entity_Record_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Rep_Clause
        (Node : Bare_Record_Rep_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Rep_Clause is
      begin
         if Node = null then
            return No_Entity_Record_Rep_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Rep_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Record_Type_Def
        (Node : Bare_Record_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Record_Type_Def is
      begin
         if Node = null then
            return No_Entity_Record_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Record_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Relation_Op
        (Node : Bare_Relation_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Relation_Op is
      begin
         if Node = null then
            return No_Entity_Relation_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Relation_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Renaming_Clause
        (Node : Bare_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Requeue_Stmt
        (Node : Bare_Requeue_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Requeue_Stmt is
      begin
         if Node = null then
            return No_Entity_Requeue_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Requeue_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Return_Stmt
        (Node : Bare_Return_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Return_Stmt is
      begin
         if Node = null then
            return No_Entity_Return_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Return_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Node
        (Node : Bare_Reverse_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Node is
      begin
         if Node = null then
            return No_Entity_Reverse_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Absent
        (Node : Bare_Reverse_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Absent is
      begin
         if Node = null then
            return No_Entity_Reverse_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Reverse_Present
        (Node : Bare_Reverse_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Reverse_Present is
      begin
         if Node = null then
            return No_Entity_Reverse_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Reverse_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_Stmt
        (Node : Bare_Select_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_Select_Stmt is
      begin
         if Node = null then
            return No_Entity_Select_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part
        (Node : Bare_Select_When_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part is
      begin
         if Node = null then
            return No_Entity_Select_When_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Select_When_Part_List
        (Node : Bare_Select_When_Part_List; Info : Internal_Entity_Info)
         return Internal_Entity_Select_When_Part_List is
      begin
         if Node = null then
            return No_Entity_Select_When_Part_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Select_When_Part_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Signed_Int_Type_Def
        (Node : Bare_Signed_Int_Type_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Signed_Int_Type_Def is
      begin
         if Node = null then
            return No_Entity_Signed_Int_Type_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Signed_Int_Type_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Protected_Decl
        (Node : Bare_Single_Protected_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Protected_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Protected_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Protected_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Decl
        (Node : Bare_Single_Task_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Type_Decl
        (Node : Bare_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Single_Task_Type_Decl
        (Node : Bare_Single_Task_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Single_Task_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Single_Task_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Single_Task_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Stmt_List
        (Node : Bare_Stmt_List; Info : Internal_Entity_Info)
         return Internal_Entity_Stmt_List is
      begin
         if Node = null then
            return No_Entity_Stmt_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Stmt_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_String_Literal
        (Node : Bare_String_Literal; Info : Internal_Entity_Info)
         return Internal_Entity_String_Literal is
      begin
         if Node = null then
            return No_Entity_String_Literal;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_String_Literal) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body
        (Node : Bare_Subp_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body is
      begin
         if Node = null then
            return No_Entity_Subp_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Body_Stub
        (Node : Bare_Subp_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Subp_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Decl
        (Node : Bare_Subp_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind
        (Node : Bare_Subp_Kind; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind is
      begin
         if Node = null then
            return No_Entity_Subp_Kind;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Function
        (Node : Bare_Subp_Kind_Function; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Function is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Function;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Function) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Kind_Procedure
        (Node : Bare_Subp_Kind_Procedure; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Kind_Procedure is
      begin
         if Node = null then
            return No_Entity_Subp_Kind_Procedure;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Kind_Procedure) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Renaming_Decl
        (Node : Bare_Subp_Renaming_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Renaming_Decl is
      begin
         if Node = null then
            return No_Entity_Subp_Renaming_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Renaming_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subp_Spec
        (Node : Bare_Subp_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_Subp_Spec is
      begin
         if Node = null then
            return No_Entity_Subp_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subp_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subtype_Decl
        (Node : Bare_Subtype_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Subtype_Decl is
      begin
         if Node = null then
            return No_Entity_Subtype_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subtype_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Subunit
        (Node : Bare_Subunit; Info : Internal_Entity_Info)
         return Internal_Entity_Subunit is
      begin
         if Node = null then
            return No_Entity_Subunit;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Subunit) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Node
        (Node : Bare_Synchronized_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Node is
      begin
         if Node = null then
            return No_Entity_Synchronized_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Absent
        (Node : Bare_Synchronized_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Absent is
      begin
         if Node = null then
            return No_Entity_Synchronized_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synchronized_Present
        (Node : Bare_Synchronized_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Synchronized_Present is
      begin
         if Node = null then
            return No_Entity_Synchronized_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synchronized_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synth_Anonymous_Type_Decl
        (Node : Bare_Synth_Anonymous_Type_Decl; Info : Internal_Entity_Info)
         return Internal_Entity_Synth_Anonymous_Type_Decl is
      begin
         if Node = null then
            return No_Entity_Synth_Anonymous_Type_Decl;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synth_Anonymous_Type_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Synthetic_Renaming_Clause
        (Node : Bare_Synthetic_Renaming_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Synthetic_Renaming_Clause is
      begin
         if Node = null then
            return No_Entity_Synthetic_Renaming_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Synthetic_Renaming_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Node
        (Node : Bare_Tagged_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Node is
      begin
         if Node = null then
            return No_Entity_Tagged_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Absent
        (Node : Bare_Tagged_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Absent is
      begin
         if Node = null then
            return No_Entity_Tagged_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Tagged_Present
        (Node : Bare_Tagged_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Tagged_Present is
      begin
         if Node = null then
            return No_Entity_Tagged_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Tagged_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Target_Name
        (Node : Bare_Target_Name; Info : Internal_Entity_Info)
         return Internal_Entity_Target_Name is
      begin
         if Node = null then
            return No_Entity_Target_Name;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Target_Name) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body
        (Node : Bare_Task_Body; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body is
      begin
         if Node = null then
            return No_Entity_Task_Body;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Body_Stub
        (Node : Bare_Task_Body_Stub; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Body_Stub is
      begin
         if Node = null then
            return No_Entity_Task_Body_Stub;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Body_Stub) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Task_Def
        (Node : Bare_Task_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Task_Def is
      begin
         if Node = null then
            return No_Entity_Task_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Task_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Terminate_Alternative
        (Node : Bare_Terminate_Alternative; Info : Internal_Entity_Info)
         return Internal_Entity_Terminate_Alternative is
      begin
         if Node = null then
            return No_Entity_Terminate_Alternative;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Terminate_Alternative) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Type_Access_Def
        (Node : Bare_Type_Access_Def; Info : Internal_Entity_Info)
         return Internal_Entity_Type_Access_Def is
      begin
         if Node = null then
            return No_Entity_Type_Access_Def;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Type_Access_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Un_Op
        (Node : Bare_Un_Op; Info : Internal_Entity_Info)
         return Internal_Entity_Un_Op is
      begin
         if Node = null then
            return No_Entity_Un_Op;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Un_Op) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index
        (Node : Bare_Unconstrained_Array_Index; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Index_List
        (Node : Bare_Unconstrained_Array_Index_List; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Index_List is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Index_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Index_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unconstrained_Array_Indices
        (Node : Bare_Unconstrained_Array_Indices; Info : Internal_Entity_Info)
         return Internal_Entity_Unconstrained_Array_Indices is
      begin
         if Node = null then
            return No_Entity_Unconstrained_Array_Indices;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unconstrained_Array_Indices) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Unknown_Discriminant_Part
        (Node : Bare_Unknown_Discriminant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Unknown_Discriminant_Part is
      begin
         if Node = null then
            return No_Entity_Unknown_Discriminant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Unknown_Discriminant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Node
        (Node : Bare_Until_Node; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Node is
      begin
         if Node = null then
            return No_Entity_Until_Node;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Node) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Absent
        (Node : Bare_Until_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Absent is
      begin
         if Node = null then
            return No_Entity_Until_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Until_Present
        (Node : Bare_Until_Present; Info : Internal_Entity_Info)
         return Internal_Entity_Until_Present is
      begin
         if Node = null then
            return No_Entity_Until_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Until_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Update_Attribute_Ref
        (Node : Bare_Update_Attribute_Ref; Info : Internal_Entity_Info)
         return Internal_Entity_Update_Attribute_Ref is
      begin
         if Node = null then
            return No_Entity_Update_Attribute_Ref;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Update_Attribute_Ref) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Clause
        (Node : Bare_Use_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Package_Clause
        (Node : Bare_Use_Package_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Package_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Package_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Package_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Use_Type_Clause
        (Node : Bare_Use_Type_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_Use_Type_Clause is
      begin
         if Node = null then
            return No_Entity_Use_Type_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Use_Type_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant
        (Node : Bare_Variant; Info : Internal_Entity_Info)
         return Internal_Entity_Variant is
      begin
         if Node = null then
            return No_Entity_Variant;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_List
        (Node : Bare_Variant_List; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_List is
      begin
         if Node = null then
            return No_Entity_Variant_List;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_List) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_Variant_Part
        (Node : Bare_Variant_Part; Info : Internal_Entity_Info)
         return Internal_Entity_Variant_Part is
      begin
         if Node = null then
            return No_Entity_Variant_Part;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_Variant_Part) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Spec
        (Node : Bare_While_Loop_Spec; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Spec is
      begin
         if Node = null then
            return No_Entity_While_Loop_Spec;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Spec) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_While_Loop_Stmt
        (Node : Bare_While_Loop_Stmt; Info : Internal_Entity_Info)
         return Internal_Entity_While_Loop_Stmt is
      begin
         if Node = null then
            return No_Entity_While_Loop_Stmt;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_While_Loop_Stmt) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Clause
        (Node : Bare_With_Clause; Info : Internal_Entity_Info)
         return Internal_Entity_With_Clause is
      begin
         if Node = null then
            return No_Entity_With_Clause;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Clause) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private
        (Node : Bare_With_Private; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private is
      begin
         if Node = null then
            return No_Entity_With_Private;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Absent
        (Node : Bare_With_Private_Absent; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Absent is
      begin
         if Node = null then
            return No_Entity_With_Private_Absent;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Absent) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      function Create_Internal_Entity_With_Private_Present
        (Node : Bare_With_Private_Present; Info : Internal_Entity_Info)
         return Internal_Entity_With_Private_Present is
      begin
         if Node = null then
            return No_Entity_With_Private_Present;
         end if;
         return (Node => Node, Info => Info);
      end;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Entity_With_Private_Present) return String is
         pragma Warnings (On, "referenced");
      begin
            return Image (Entity'(Node => R.Node, Info => R.Info));
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Env_Assoc) is
      begin
               Inc_Ref (R.Dest_Env);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Env_Assoc) is
      begin
               Dec_Ref (R.Dest_Env);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Env_Assoc) return Boolean is
      begin
         return L.Key = R.Key and then L.Val = R.Val and then Equivalent (L.Dest_Env, R.Dest_Env) and then L.Metadata = R.Metadata;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Val => " & Trace_Image (R.Val)
                        & ", "
                     & "Dest_Env => " & Trace_Image (R.Dest_Env)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Eval_Discrete_Range) is
      begin
               Inc_Ref (R.Low_Bound);
               Inc_Ref (R.High_Bound);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Eval_Discrete_Range) is
      begin
               Dec_Ref (R.Low_Bound);
               Dec_Ref (R.High_Bound);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Eval_Discrete_Range) return Boolean is
      begin
         return Equivalent (L.Low_Bound, R.Low_Bound) and then Equivalent (L.High_Bound, R.High_Bound);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Eval_Discrete_Range) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Low_Bound => " & Trace_Image (R.Low_Bound)
                        & ", "
                     & "High_Bound => " & Trace_Image (R.High_Bound)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Expected_Type_For_Expr) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Expected_Type => " & Trace_Image (R.Expected_Type)
                        & ", "
                     & "Expr => " & Trace_Image (R.Expr)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Inner_Env_Assoc) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Key => " & Trace_Image (R.Key)
                        & ", "
                     & "Val => " & Trace_Image (R.Val)
                        & ", "
                     & "Metadata => " & Trace_Image (R.Metadata)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Logic_Val_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Success => " & Trace_Image (R.Success)
                        & ", "
                     & "Value => " & Trace_Image (R.Value)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Multidim_Aggregate_Info) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Agg => " & Trace_Image (R.Agg)
                        & ", "
                     & "Typ => " & Trace_Image (R.Typ)
                        & ", "
                     & "Rank => " & Trace_Image (R.Rank)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Param => " & Trace_Image (R.Param)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Single_Actual) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Assoc => " & Trace_Image (R.Assoc)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Single_Formal) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Name => " & Trace_Image (R.Name)
                        & ", "
                     & "Spec => " & Trace_Image (R.Spec)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Param_Match) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Has_Matched => " & Trace_Image (R.Has_Matched)
                        & ", "
                     & "Actual => " & Trace_Image (R.Actual)
                        & ", "
                     & "Formal => " & Trace_Image (R.Formal)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Ref_Result) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Ref => " & Trace_Image (R.Ref)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Decl) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Decl => " & Trace_Image (R.Decl)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Refd_Def) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Def_Name => " & Trace_Image (R.Def_Name)
                        & ", "
                     & "Kind => " & Trace_Image (R.Kind)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Shape) is
      begin
               Inc_Ref (R.Components);
               Inc_Ref (R.Discriminants_Values);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Shape) is
      begin
               Dec_Ref (R.Components);
               Dec_Ref (R.Discriminants_Values);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Shape) return Boolean is
      begin
         return Equivalent (L.Components, R.Components) and then Equivalent (L.Discriminants_Values, R.Discriminants_Values);
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Shape) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "Components => " & Trace_Image (R.Components)
                        & ", "
                     & "Discriminants_Values => " & Trace_Image (R.Discriminants_Values)
               & ")");
      end Trace_Image;
      -------------
      -- Inc_Ref --
      -------------
      procedure Inc_Ref (R : Internal_Substitution) is
      begin
               Inc_Ref (R.To_Value);
      end Inc_Ref;
      -------------
      -- Dec_Ref --
      -------------
      procedure Dec_Ref (R : in out Internal_Substitution) is
      begin
               Dec_Ref (R.To_Value);
      end Dec_Ref;
      ----------------
      -- Equivalent --
      ----------------
      function Equivalent (L, R : Internal_Substitution) return Boolean is
      begin
         return L.From_Decl = R.From_Decl and then Equivalent (L.To_Value, R.To_Value) and then L.Value_Type = R.Value_Type;
      end Equivalent;
      -----------------
      -- Trace_Image --
      -----------------
      pragma Warnings (Off, "referenced");
      function Trace_Image (R : Internal_Substitution) return String is
         pragma Warnings (On, "referenced");
      begin
            return
              ("("
                     & "From_Decl => " & Trace_Image (R.From_Decl)
                        & ", "
                     & "To_Value => " & Trace_Image (R.To_Value)
                        & ", "
                     & "Value_Type => " & Trace_Image (R.Value_Type)
               & ")");
      end Trace_Image;
   pragma Warnings (Off, "referenced");
   type Logic_Converter_Default is null record;
   No_Logic_Converter_Default : constant Logic_Converter_Default :=
     (null record);
   function Convert
     (Self : Logic_Converter_Default;
      From : Internal_Entity) return Internal_Entity
   is
      pragma Unreferenced (Self);
   begin
      return From;
   end Convert;
   type Equals_Data_Default is null record;
   No_Equals_Data_Default : constant Equals_Data_Default := (null record);
   function Eq_Default
     (Data : Equals_Data_Default; L, R : Internal_Entity) return Boolean
   is (Equivalent (L, R))
      with Inline;
   pragma Warnings (On, "referenced");
   type Ada_Node_P_Is_Not_Null_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Ada_Node_P_Is_Not_Null_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Ada_Node_P_Is_Not_Null_0_Predicate_Caller is
   begin
      return Ada_Node_P_Is_Not_Null_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Ada_Node_P_Is_Not_Null_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Ada_Node_P_Is_Not_Null (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Ada_Node_P_Is_Not_Null_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Ada_Node_P_Is_Not_Null_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Ada_Node_P_Is_Not_Null_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Ada_Node_P_Is_Not_Null_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Basic_Decl;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (
         Field_0 : Internal_Entity_Basic_Decl;
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Basic_Decl_P_Subp_Decl_Match_Signature (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Is_Subprogram_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Is_Subprogram_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Is_Subprogram_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Is_Subprogram_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Is_Subprogram_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Basic_Decl_P_Is_Subprogram (Node_0.Node);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Is_Subprogram_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Is_Subprogram_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Is_Subprogram_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Is_Subprogram_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Boolean;
      Dbg_Img : String_Access := null
   ) return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
            Field_1 => Field_1,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Basic_Decl_P_Is_Stream_Subprogram_For_Type (Node_0.Node, Self.Field_0, Self.Field_1, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Array_Or_Rec (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Array_Or_Rec_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Task_Type (Node_0.Node);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Task_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Task_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Char_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Char_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Char_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Base_Type_Decl_P_Is_Universal_Type (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Universal_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Universal_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller is record
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller'(
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
         pragma Unreferenced (Self);
   begin
      return Base_Type_Decl_P_Is_Not_Universal_Type (Node_0.Node, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Access_Type_Predicate (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Access_Type_Predicate_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Iterator_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Iterator_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Iterator_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Discrete_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Discrete_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Str_Type_Or_Null (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ; Node_1 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Subp_Access_Of (Node_0.Node, (Node => Node_1.Node, Info => Node_1.Info), Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Subp_Access_Of_0_Pred is new Predicate_2
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller is record
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (
         Field_0 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller'(
            Field_0 => Field_0,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Is_Array_Def_With_Deref (Node_0.Node, Self.Field_0, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   type Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller is record
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
      Dbg_Img : String_Access := null;
   end record;
   function Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (
         Field_0 : Internal_Entity_Base_Type_Decl;
         Field_1 : Bare_Ada_Node;
      Dbg_Img : String_Access := null
   ) return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller'(
            Field_0 => Field_0,
            Field_1 => Field_1,
         Dbg_Img => Dbg_Img
      );
   end;
   ----------
   -- Call --
   ----------
   function Call
     (Self       : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller
     ; Node_0 : Internal_Entity
     ) return Boolean
   is
   begin
      return Base_Type_Decl_P_Matching_Allocator_Type (Node_0.Node, Self.Field_0, Self.Field_1, E_Info => Node_0.Info);
   end Call;
   -----------
   -- Image --
   -----------
   function Image (Self : Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller) return String
   is (if Self.Dbg_Img /= null then Self.Dbg_Img.all else "");
   ----------
   -- Free --
   ----------
   procedure Free (Self : in out Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller) is
      procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
   begin
      Free (Self.Dbg_Img);
   end Free;
   package Base_Type_Decl_P_Matching_Allocator_Type_0_Pred is new Predicate_1
     (El_Type        => Internal_Entity,
      Var            => Eq_Node.Refs.Raw_Logic_Var,
      Predicate_Type => Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate_Caller,
      Free           => Free,
      Image          => Image);
   package Bind_Default_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "",
      Equals_Image     => "",
      One_Side_Convert => False
      );
   type Logic_Converter_500 is
      null record;
   No_Logic_Converter_500 : constant Logic_Converter_500 := (
         null record
   );
   function Convert (Self : Logic_Converter_500; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_500; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_560 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_560 : constant Equals_Data_560 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_560
     (Data : Equals_Data_560; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_prefix_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_prefix_type's"
            & " ""container_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Prefix_Type
          (Node             => L.Node,
           Container_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_560;
   package Bind_500_560 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_500,
      No_Data          => No_Logic_Converter_500,
      Equals_Data      => Equals_Data_560,
      No_Equals_Data   => No_Equals_Data_560,
      Convert          => Convert,
      Equals           => Eq_560,
      Convert_Image    =>
        "BaseTypeDecl.anonymous_access_type_or_null",
      Equals_Image     => "BaseTypeDecl.matching_prefix_type",
      One_Side_Convert => True
      );
   type Logic_Converter_570 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_570 : constant Logic_Converter_570 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_570; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_570; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Base_Subtype_Or_Null
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_567 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_567 : constant Equals_Data_567 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_567
     (Data : Equals_Data_567; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_type's"
            & " ""expected_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Type
          (Node             => L.Node,
           Expected_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_567;
   package Bind_570_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_570,
      No_Data          => No_Logic_Converter_570,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "BaseTypeDecl.base_subtype_or_null",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_553 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_553 : constant Logic_Converter_553 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_553; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_553; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Comp_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_553_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_553,
      No_Data          => No_Logic_Converter_553,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "BaseTypeDecl.comp_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Equals_Data_566 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_566 : constant Equals_Data_566 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_566
     (Data : Equals_Data_566; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_assign_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_assign_type's"
            & " ""expected_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Assign_Type
          (Node             => L.Node,
           Expected_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_566;
   package Bind_Default_566 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_566,
      No_Equals_Data   => No_Equals_Data_566,
      Convert          => Convert,
      Equals           => Eq_566,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_assign_type",
      One_Side_Convert => False
      );
   type Equals_Data_562 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_562 : constant Equals_Data_562 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_562
     (Data : Equals_Data_562; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_prim_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_prim_type's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Prim_Type
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_562;
   package Bind_Default_562 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_562,
      No_Equals_Data   => No_Equals_Data_562,
      Convert          => Convert,
      Equals           => Eq_562,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_formal_prim_type",
      One_Side_Convert => False
      );
   type Equals_Data_564 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_564 : constant Equals_Data_564 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_564
     (Data : Equals_Data_564; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Type
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_564;
   package Bind_Default_564 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_564,
      No_Equals_Data   => No_Equals_Data_564,
      Convert          => Convert,
      Equals           => Eq_564,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_formal_type",
      One_Side_Convert => False
      );
   package Bind_Default_560 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_560,
      No_Equals_Data   => No_Equals_Data_560,
      Convert          => Convert,
      Equals           => Eq_560,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_prefix_type",
      One_Side_Convert => False
      );
   package Bind_Default_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_Default,
      No_Data          => No_Logic_Converter_Default,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => False
      );
   type Logic_Converter_502 is
      null record;
   No_Logic_Converter_502 : constant Logic_Converter_502 := (
         null record
   );
   function Convert (Self : Logic_Converter_502; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_502; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Base_Type_Decl_P_Model_Of_Type
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_502_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_502,
      No_Data          => No_Logic_Converter_502,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "BaseTypeDecl.model_of_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_170 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_170 : constant Logic_Converter_170 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_170; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_170; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Basic_Decl_P_Identity_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_170_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_170,
      No_Data          => No_Logic_Converter_170,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "BasicDecl.identity_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   type Logic_Converter_175 is
      null record;
   No_Logic_Converter_175 : constant Logic_Converter_175 := (
         null record
   );
   function Convert (Self : Logic_Converter_175; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_175; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Base_Subp_Spec;
   begin
      Ret := Basic_Decl_P_Subp_Spec_Or_Null
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_175_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_175,
      No_Data          => No_Logic_Converter_175,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "BasicDecl.subp_spec_or_null",
      Equals_Image     => "",
      One_Side_Convert => True
      );
   type Logic_Converter_539 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_539 : constant Logic_Converter_539 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_539; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_539; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Dispatcher_Base_Type_Decl_P_Accessed_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   type Equals_Data_563 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Equals_Data_563 : constant Equals_Data_563 := (
         Origin => No_Bare_Ada_Node
   );
   function Eq_563
     (Data : Equals_Data_563; L, R : Internal_Entity) return Boolean is
   begin
      --  If any node pointer is null, then use that for equality
      if L.Node = null or else R.Node = null then
         return L.Node = R.Node;
      end if;
      --  Check that both arguments have appropriate types for the property
      --  call.
      if L.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type_inverted's ""self"" argument";
      elsif R.Node.Kind not in Ada_Base_Type_Decl then
         raise Property_Error with
            "Wrong type for BaseTypeDecl.matching_formal_type_inverted's"
            & " ""formal_type"" argument";
      end if;
      --  All is good: do the call
      declare
         R_Entity : constant Internal_Entity_Base_Type_Decl := (R.Node, R.Info);
      begin
         return Base_Type_Decl_P_Matching_Formal_Type_Inverted
          (Node             => L.Node,
           Formal_Type => R_Entity,
              Origin => Data.Origin,
           E_Info          => L.Info);
       end;
   end Eq_563;
   package Bind_539_563 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_539,
      No_Data          => No_Logic_Converter_539,
      Equals_Data      => Equals_Data_563,
      No_Equals_Data   => No_Equals_Data_563,
      Convert          => Convert,
      Equals           => Eq_563,
      Convert_Image    =>
        "BaseTypeDecl.accessed_type",
      Equals_Image     => "BaseTypeDecl.matching_formal_type_inverted",
      One_Side_Convert => True
      );
   type Logic_Converter_527 is
      null record;
   No_Logic_Converter_527 : constant Logic_Converter_527 := (
         null record
   );
   function Convert (Self : Logic_Converter_527; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_527; From : Internal_Entity) return Internal_Entity is
         pragma Unreferenced (Self);
      Ret : Internal_Entity_Classwide_Type_Decl;
   begin
      Ret := Dispatcher_Base_Type_Decl_P_Classwide_Type
        (Node => From.Node,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_527_Default is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_527,
      No_Data          => No_Logic_Converter_527,
      Equals_Data      => Equals_Data_Default,
      No_Equals_Data   => No_Equals_Data_Default,
      Convert          => Convert,
      Equals           => Eq_Default,
      Convert_Image    =>
        "BaseTypeDecl.classwide_type",
      Equals_Image     => "",
      One_Side_Convert => True
      );
   type Logic_Converter_173 is
      record
            Origin : Bare_Ada_Node;
      end record;
   No_Logic_Converter_173 : constant Logic_Converter_173 := (
         Origin => No_Bare_Ada_Node
   );
   function Convert (Self : Logic_Converter_173; From : Internal_Entity) return Internal_Entity
      with Inline;
   -------------
   -- Convert --
   -------------
   function Convert (Self : Logic_Converter_173; From : Internal_Entity) return Internal_Entity is
      Ret : Internal_Entity_Base_Type_Decl;
   begin
      Ret := Dispatcher_Basic_Decl_P_Expr_Type
        (Node => From.Node,
            Origin => Self.Origin,
         E_Info => From.Info);
      return (Node => Ret.Node, Info => Ret.Info);
   end Convert;
   package Bind_173_567 is new Eq_Node.Raw_Custom_Bind
     (Converter        => Logic_Converter_173,
      No_Data          => No_Logic_Converter_173,
      Equals_Data      => Equals_Data_567,
      No_Equals_Data   => No_Equals_Data_567,
      Convert          => Convert,
      Equals           => Eq_567,
      Convert_Image    =>
        "BasicDecl.expr_type",
      Equals_Image     => "BaseTypeDecl.matching_type",
      One_Side_Convert => True
      );
   --
   --  Primitives for Bare_Ada_Node
   --
--# property-start AdaNode.declarative_scope /usr/src/packages/BUILD/ada/ast.py:93
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Declarative_Scope
  (Node : Bare_Ada_Node
  )
   return Bare_Declarative_Part
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_2;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
            procedure Finalizer_Scope_2 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5 '<Cast to DeclarativePart>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:94
--# expr-start 4 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:94
--# expr-start 3 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:94
--# expr-start 1 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:94
Node_Parents := Parents (Self, True); 
--# expr-done 1
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_0 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_0
               --# expr-start 2 '<IsA DeclarativePart>' Is_A /usr/src/packages/BUILD/ada/ast.py:95
Is_A := Item_0 /= null 
and then Item_0.Kind in Ada_Declarative_Part_Range; 
--# expr-done 2
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_0;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4
   if Get_Result = null
      or else Get_Result.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_2;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Declarative_Scope;
--# end
--# property-start '[dispatcher]AdaNode.custom_id_text' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_Signed_Int_Type_Def | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.custom_id_text
                  Property_Result := Ada_Node_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.custom_id_text
                  Property_Result := Basic_Decl_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.custom_id_text
                  Property_Result := Anonymous_Type_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.custom_id_text
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.custom_id_text
                  Property_Result := Subtype_Indication_P_Custom_Id_Text
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.is_contract_aspect /usr/src/packages/BUILD/ada/ast.py:112
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Contract_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
      Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 48 '<Block>' Let_Result None
--# scope-start
--# expr-start 7 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 6 '<Symbol Pre>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pre); 
--# expr-done 6
Is_Equal := Name = Sym; 
--# expr-done 7
if Is_Equal then
   If_Result := True;
else
   --# expr-start 9 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 8 '<Symbol Pre'"'"'Class>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pre_Class); 
--# expr-done 8
Is_Equal_1 := Name = Sym_1; 
--# expr-done 9
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 11 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 10 '<Symbol Post>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Post); 
--# expr-done 10
Is_Equal_2 := Name = Sym_2; 
--# expr-done 11
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 13 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 12 '<Symbol Post'"'"'Class>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Post_Class); 
--# expr-done 12
Is_Equal_3 := Name = Sym_3; 
--# expr-done 13
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 15 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 14 '<Symbol Refined_Post>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Post); 
--# expr-done 14
Is_Equal_4 := Name = Sym_4; 
--# expr-done 15
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 17 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 16 '<Symbol Invariant>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Invariant); 
--# expr-done 16
Is_Equal_5 := Name = Sym_5; 
--# expr-done 17
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 19 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 18 '<Symbol Invariant'"'"'Class>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Invariant_Class); 
--# expr-done 18
Is_Equal_6 := Name = Sym_6; 
--# expr-done 19
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 21 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 20 '<Symbol Type_Invariant>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Invariant); 
--# expr-done 20
Is_Equal_7 := Name = Sym_7; 
--# expr-done 21
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 23 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 22 '<Symbol Type_Invariant'"'"'Class>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Invariant_Class); 
--# expr-done 22
Is_Equal_8 := Name = Sym_8; 
--# expr-done 23
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 25 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 24 '<Symbol Predicate>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Predicate); 
--# expr-done 24
Is_Equal_9 := Name = Sym_9; 
--# expr-done 25
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 27 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 26 '<Symbol Static_Predicate>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Static_Predicate); 
--# expr-done 26
Is_Equal_10 := Name = Sym_10; 
--# expr-done 27
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 29 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 28 '<Symbol Dynamic_Predicate>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Dynamic_Predicate); 
--# expr-done 28
Is_Equal_11 := Name = Sym_11; 
--# expr-done 29
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 31 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 30 '<Symbol Default_Initial_Condition>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Default_Initial_Condition); 
--# expr-done 30
Is_Equal_12 := Name = Sym_12; 
--# expr-done 31
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 33 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 32 '<Symbol Contract_Cases>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Contract_Cases); 
--# expr-done 32
Is_Equal_13 := Name = Sym_13; 
--# expr-done 33
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 35 '<Eq>' Is_Equal_14 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 34 '<Symbol Test_Case>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 34
Is_Equal_14 := Name = Sym_14; 
--# expr-done 35
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 37 '<Eq>' Is_Equal_15 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 36 '<Symbol Global>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 36
Is_Equal_15 := Name = Sym_15; 
--# expr-done 37
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 39 '<Eq>' Is_Equal_16 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 38 '<Symbol Refined_Global>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Global); 
--# expr-done 38
Is_Equal_16 := Name = Sym_16; 
--# expr-done 39
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 41 '<Eq>' Is_Equal_17 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 40 '<Symbol Refined_State>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_State); 
--# expr-done 40
Is_Equal_17 := Name = Sym_17; 
--# expr-done 41
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 43 '<Eq>' Is_Equal_18 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 42 '<Symbol Depends>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 42
Is_Equal_18 := Name = Sym_18; 
--# expr-done 43
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 45 '<Eq>' Is_Equal_19 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 44 '<Symbol Refined_Depends>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Depends); 
--# expr-done 44
Is_Equal_19 := Name = Sym_19; 
--# expr-done 45
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 47 '<Eq>' Is_Equal_20 /usr/src/packages/BUILD/ada/ast.py:113
--# expr-start 46 '<Symbol Predicate_Failure>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Predicate_Failure); 
--# expr-done 46
Is_Equal_20 := Name = Sym_20; 
--# expr-done 47
   If_Result_19 := Is_Equal_20;
end if;
Let_Result := If_Result_19; 
--# end
--# expr-done 48
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Contract_Aspect;
--# end
--# property-start AdaNode.in_contract /usr/src/packages/BUILD/ada/ast.py:125
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Contract
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_6;
      Var_Expr : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null_1 : Boolean;
Not_Val : Boolean;
            procedure Finalizer_Scope_6 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_6;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 58 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:125
--# expr-start 57 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:125
--# expr-start 56 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:125
--# expr-start 55 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:125
--# expr-start 49 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:125
Node_Parents := Parents (Self, True); 
--# expr-done 49
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_1 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_1
--# expr-start 50 '<Cast to AspectAssoc>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:126
Cast_Expr := Item_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 50
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 54 '<FieldAccess .is_contract_aspect(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:127
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 53 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:127
--# expr-start 52 '<as_bare_entity(<FieldAccess .id>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:127
--# expr-start 51 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:127
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 51
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 52
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => If_Result.Node);
--# end
--# expr-done 53
--# property-call-start AdaNode.is_contract_aspect
Fld_2 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Fld_1);
--# end
--# expr-done 54
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_1;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 55
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 56
Is_Null_1 := Get_Result = null; 
--# expr-done 57
Not_Val := not (Is_Null_1); 
--# expr-done 58
      Property_Result := Not_Val;
   --# end
      Finalizer_Scope_6;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_6;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_In_Contract;
--# end
--# property-start AdaNode.in_aspect /usr/src/packages/BUILD/ada/ast.py:132
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_In_Aspect
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   Property_Result : Boolean;
            procedure Finalizer_Scope_7;
      Var_Expr : Bare_Aspect_Assoc;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Aspect_Assoc;
Fld : Bare_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_7 is
            begin
                     Dec_Ref (Node_Parents);
            end Finalizer_Scope_7;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 63 '<Block>' Let_Result None
--# scope-start
--# expr-start 59 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:136
Node_Parents := Parents (Self, True); 
--# expr-done 59
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
   begin
      for Item_2 of
            Collection.Items
      loop
   --# scope-start
            --# bind p Item_2
--# expr-start 60 '<Cast to AspectAssoc>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:137
Cast_Expr := Item_2; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Aspect_Assoc_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 60
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 62 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:137
--# expr-start 61 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:137
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr.Aspect_Assoc_F_Id;
--# expr-done 61
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Name);
--# end
--# expr-done 62
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
            if Result_Var then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 63
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_7;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_7;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_In_Aspect;
--# end
--# property-start AdaNode.empty_env /usr/src/packages/BUILD/ada/ast.py:140
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Empty_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_8;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_8 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld);
            end Finalizer_Scope_8;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 69 '<FieldAccess .get_empty_env>' Fld /usr/src/packages/BUILD/ada/ast.py:141
--# expr-start 68 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:141
--# expr-start 67 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:141
--# expr-start 66 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:141
--# expr-start 64 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:141
Node_Parents := Parents (Self, True); 
--# expr-done 64
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_3 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_3
               --# expr-start 65 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:141
Is_A := Item_3 /= null 
and then Item_3.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 65
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_3;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 66
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 67
   if Get_Result = null
      or else Get_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 68
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Compilation_Unit_P_Get_Empty_Env (Node => Cast_Result);
--# expr-done 69
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_8;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_8;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Empty_Env;
--# end
--# property-start AdaNode.is_not_null /usr/src/packages/BUILD/ada/ast.py:146
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Not_Null
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 72 '<Block>' Let_Result None
--# scope-start
--# expr-start 71 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:154
--# expr-start 70 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:154
Is_Null := Ent.Node = null; 
--# expr-done 70
Not_Val := not (Is_Null); 
--# expr-done 71
Let_Result := Not_Val; 
--# end
--# expr-done 72
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Not_Null;
--# end
--# property-start AdaNode.eval_discrete_range /usr/src/packages/BUILD/ada/ast.py:157
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Eval_Discrete_Range
  (Node : Bare_Ada_Node
      ; Dr : Internal_Discrete_Range
  )
   return Internal_Eval_Discrete_Range
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dr Dr
   Property_Result : Internal_Eval_Discrete_Range;
            procedure Finalizer_Scope_10;
            procedure Finalizer_Scope_1464;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Scope_Result : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Result_Var : Big_Integer_Type;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
New_Struct : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Is_Equal : Boolean;
If_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Let_Result : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
            procedure Finalizer_Scope_10 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Big_Int);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (New_Struct);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_10;
            procedure Finalizer_Scope_1464 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1464;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 81 '<Block>' Let_Result None
--# scope-start
--# expr-start 80 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:161
--# expr-start 73 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:162
Is_Equal := Dr = No_Discrete_Range; 
--# expr-done 73
if Is_Equal then
   If_Result := raise Property_Error with "Attempting to evaluate a null discrete range";
else
   --# expr-start 79 '<New InternalEvalDiscreteRange>' New_Struct /usr/src/packages/BUILD/ada/ast.py:167
--# expr-start 75 '<FieldAccess .eval_as_int>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:172
--# expr-start 74 '<FieldAccess .high_bound>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:172
Fld_2 := Dr.High_Bound;
--# expr-done 74
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 75
--# expr-start 76 '<FieldAccess .low_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:168
Fld := Dr.Low_Bound;
--# expr-done 76
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind lb Var_Expr
--# expr-start 77 '<FieldAccess .eval_as_int>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:169
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 77
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1464;
   Result_Var := Scope_Result;
else
   --# expr-start 78 '<BigInteger 0>' Big_Int /usr/src/packages/BUILD/ada/ast.py:170
Big_Int := Create_Big_Integer ("0"); 
--# expr-done 78
   Result_Var := Big_Int;
end if;
      Inc_Ref (Result_Var);
Inc_Ref (Fld_3);
Inc_Ref (Result_Var);
New_Struct := (High_Bound => Fld_3, Low_Bound => Result_Var); 
--# expr-done 79
   If_Result := New_Struct;
end if;
      Inc_Ref (If_Result);
--# expr-done 80
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 81
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_10;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_10;
               Finalizer_Scope_1464;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Eval_Discrete_Range;
--# end
--# property-start AdaNode.string_join /usr/src/packages/BUILD/ada/ast.py:177
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_String_Join
  (Node : Bare_Ada_Node
      ; Strns : Character_Type_Array_Access_Array_Access
      ; Sep : Character_Type_Array_Access
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind strns Strns
   --# bind sep Sep
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_12;
            procedure Finalizer_Scope_1465;
      Block_Var : Integer;
Len : Integer;
Concat_Result : Character_Type_Array_Access;
Arith_Result : Integer;
Is_Equal : Boolean;
If_Result : Character_Type_Array_Access;
I : Integer;
Map_Result : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_12 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_12;
            procedure Finalizer_Scope_1465 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1465;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 88 '<Block>' Let_Result None
--# scope-start
--# expr-start 82 '<length(<Var strns>)>' Len /usr/src/packages/BUILD/ada/ast.py:181
Len := Length (Strns); 
--# expr-done 82
Block_Var := Len; 
--# bind arr_len Block_Var
--# expr-start 87 '<mapcat>' Map_Result /usr/src/packages/BUILD/ada/ast.py:183
declare
   Map_Result_Vec : Character_Type_Vectors.Vector;
begin
         I := 0;
      declare
         Collection : constant Character_Type_Array_Access_Array_Access := Strns;
      begin
         for Item_4 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_4
               --# bind i I
      --# expr-start 86 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:184
--# expr-start 84 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:184
--# expr-start 83 '<Op ->' Arith_Result /usr/src/packages/BUILD/ada/ast.py:184
Arith_Result := (Block_Var - 1); 
--# expr-done 83
Is_Equal := I = Arith_Result; 
--# expr-done 84
if Is_Equal then
   If_Result := Item_4;
else
   --# expr-start 85 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:184
Concat_Result := Concat (Item_4, Sep); 
--# expr-done 85
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 86
         for Item_To_Append of
               If_Result.Items
         loop
            Character_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
               I := I + 1;
   --# end
      Finalizer_Scope_1465;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array
        (Items_Count => Natural (Character_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 87
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 88
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_12;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_12;
               Finalizer_Scope_1465;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_String_Join;
--# end
--# property-start AdaNode.sym_join /usr/src/packages/BUILD/ada/ast.py:188
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sym_Join
  (Node : Bare_Ada_Node
      ; Syms : Symbol_Type_Array_Access
      ; Sep : Character_Type_Array_Access
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind syms Syms
   --# bind sep Sep
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_13;
            procedure Finalizer_Scope_1466;
      Fld : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Fld_1 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_13 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_13;
            procedure Finalizer_Scope_1466 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1466;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 92 '<Block>' Let_Result None
--# scope-start
--# expr-start 91 '<FieldAccess .string_join(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:192
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 90 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:192
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Symbol_Type_Array_Access := Syms;
      begin
         for Item_5 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_5
      --# expr-start 89 '<FieldAccess .image>' Fld /usr/src/packages/BUILD/ada/ast.py:192
      if Item_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Image (Self => Item_5);
--# expr-done 89
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1466;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 90
--# property-call-start AdaNode.string_join
Fld_1 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Self, Strns => Map_Result, Sep => Sep);
--# end
--# expr-done 91
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 92
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_13;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_13;
               Finalizer_Scope_1466;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Sym_Join;
--# end
--# property-start AdaNode.enclosing_compilation_unit /usr/src/packages/BUILD/ada/ast.py:196
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Enclosing_Compilation_Unit
  (Node : Bare_Ada_Node
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_14;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_14 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_14;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 98 '<Block>' Let_Result None
--# scope-start
--# expr-start 97 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:204
--# expr-start 96 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:204
--# expr-start 95 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:204
--# expr-start 93 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:204
Node_Parents := Parents (Self, True); 
--# expr-done 93
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_6 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_6
               --# expr-start 94 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:205
Is_A := Item_6 /= null 
and then Item_6.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 94
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_6;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 95
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 96
   if Get_Result = null
      or else Get_Result.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 97
Let_Result := Cast_Result; 
--# end
--# expr-done 98
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_14;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_14;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Enclosing_Compilation_Unit;
--# end
--# property-start AdaNode.is_children_env /usr/src/packages/BUILD/ada/ast.py:209
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Children_Env
  (Node : Bare_Ada_Node
      ; Parent : Lexical_Env
      ; Current_Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind parent Parent
   --# bind current_env Current_Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_15;
      Is_Equal : Boolean;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_15 is
            begin
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_15;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 105 '<Block>' Let_Result None
--# scope-start
--# expr-start 104 '<Cond>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:214
--# expr-start 99 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:215
Is_Equal := Equivalent (Parent, AST_Envs.Empty_Env); 
--# expr-done 99
if Is_Equal then
   If_Result_2 := False;
else
--# expr-start 100 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:216
Is_Equal_1 := Equivalent (Current_Env, Parent); 
--# expr-done 100
if Is_Equal_1 then
   If_Result_1 := True;
else
--# expr-start 101 '<IsNull>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:217
Is_Equal_2 := Equivalent (Current_Env, Empty_Env); 
--# expr-done 101
if Is_Equal_2 then
   If_Result := False;
else
   --# expr-start 103 '<FieldAccess .is_children_env(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:218
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 102 '<env_parent(<Var current_env>)>' Env_Parent /usr/src/packages/BUILD/ada/ast.py:218
Env_Parent := AST_Envs.Parent (Current_Env); 
--# expr-done 102
--# property-call-start AdaNode.is_children_env
Fld := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Parent, Current_Env => Env_Parent);
--# end
--# expr-done 103
   If_Result := Fld;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 104
Let_Result := If_Result_2; 
--# end
--# expr-done 105
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_15;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_15;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Children_Env;
--# end
--# property-start AdaNode.trigger_access_entity /usr/src/packages/BUILD/ada/ast.py:222
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Trigger_Access_Entity
  (Node : Bare_Ada_Node
      ; Val : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind val Val
   Property_Result : Internal_Entity;
      Block_Var : Internal_Metadata;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Update_Result : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 116 '<Block>' Let_Result None
--# scope-start
--# expr-start 108 '<StructUpdate>' Update_Result /usr/src/packages/BUILD/ada/ast.py:227
--# expr-start 107 '<FieldAccess .md>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:227
--# expr-start 106 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:227
Fld := Ent.Info;
--# expr-done 106
Fld_1 := Fld.MD;
--# expr-done 107
   Update_Result.Dottable_Subp := Fld_1.Dottable_Subp;
   Update_Result.Primitive := Fld_1.Primitive;
   Update_Result.Primitive_Real_Type := Fld_1.Primitive_Real_Type;
   Update_Result.Access_Entity := Val;
--# expr-done 108
Block_Var := Update_Result; 
--# bind new_md Block_Var
--# expr-start 115 '<New InternalEntity>' New_Struct_1 None
--# expr-start 113 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:230
--# expr-start 110 '<FieldAccess .from_rebound>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:233
--# expr-start 109 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:233
Fld_5 := Ent.Info;
--# expr-done 109
Fld_6 := Fld_5.From_Rebound;
--# expr-done 110
--# expr-start 112 '<FieldAccess .rebindings>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:231
--# expr-start 111 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:231
Fld_3 := Ent.Info;
--# expr-done 111
Fld_4 := Fld_3.Rebindings;
--# expr-done 112
New_Struct := (From_Rebound => Fld_6, MD => Block_Var, Rebindings => Fld_4); 
--# expr-done 113
--# expr-start 114 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:230
Fld_2 := Ent.Node;
--# expr-done 114
New_Struct_1 := (Info => New_Struct, Node => Fld_2); 
--# expr-done 115
Let_Result := New_Struct_1; 
--# end
--# expr-done 116
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Trigger_Access_Entity;
--# end
--# property-start AdaNode.without_md /usr/src/packages/BUILD/ada/ast.py:238
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Without_Md
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 124 '<Block>' Let_Result None
--# scope-start
--# expr-start 123 '<New InternalEntity>' New_Struct_1 None
--# expr-start 121 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:243
--# expr-start 118 '<FieldAccess .from_rebound>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:246
--# expr-start 117 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:246
Fld_3 := Ent.Info;
--# expr-done 117
Fld_4 := Fld_3.From_Rebound;
--# expr-done 118
--# expr-start 120 '<FieldAccess .rebindings>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:244
--# expr-start 119 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:244
Fld_1 := Ent.Info;
--# expr-done 119
Fld_2 := Fld_1.Rebindings;
--# expr-done 120
New_Struct := (From_Rebound => Fld_4, MD => No_Metadata, Rebindings => Fld_2); 
--# expr-done 121
--# expr-start 122 '<FieldAccess .node>' Fld /usr/src/packages/BUILD/ada/ast.py:243
Fld := Ent.Node;
--# expr-done 122
New_Struct_1 := (Info => New_Struct, Node => Fld); 
--# expr-done 123
Let_Result := New_Struct_1; 
--# end
--# expr-done 124
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Without_Md;
--# end
--# property-start '[dispatcher]AdaNode.complete' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Iterator_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Completion_Item_Iterator_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.complete
                  Property_Result := Ada_Node_P_Complete
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.complete
                  Property_Result := Dotted_Name_P_Complete
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Complete;
--# end
--# property-start AdaNode.valid_keywords /usr/src/packages/BUILD/ada/ast.py:264
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Valid_Keywords
  (Node : Bare_Ada_Node
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_20;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Sym_68 : Symbol_Type;
Sym_69 : Symbol_Type;
Sym_70 : Symbol_Type;
Sym_71 : Symbol_Type;
Sym_72 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_20 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_20;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 199 '<Block>' Let_Result None
--# scope-start
--# expr-start 198 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:272
--# expr-start 125 '<Symbol abort>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abort); 
--# expr-done 125
--# expr-start 126 '<Symbol abs>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs_204); 
--# expr-done 126
--# expr-start 127 '<Symbol abstract>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abstract); 
--# expr-done 127
--# expr-start 128 '<Symbol accept>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Accept); 
--# expr-done 128
--# expr-start 129 '<Symbol access>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access_207); 
--# expr-done 129
--# expr-start 130 '<Symbol aliased>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aliased); 
--# expr-done 130
--# expr-start 131 '<Symbol all>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_All); 
--# expr-done 131
--# expr-start 132 '<Symbol and>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And_211); 
--# expr-done 132
--# expr-start 133 '<Symbol array>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Array); 
--# expr-done 133
--# expr-start 134 '<Symbol at>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_At); 
--# expr-done 134
--# expr-start 135 '<Symbol begin>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Begin); 
--# expr-done 135
--# expr-start 136 '<Symbol body>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Body); 
--# expr-done 136
--# expr-start 137 '<Symbol case>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Case); 
--# expr-done 137
--# expr-start 138 '<Symbol constant>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant); 
--# expr-done 138
--# expr-start 139 '<Symbol declare>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Declare); 
--# expr-done 139
--# expr-start 140 '<Symbol delay>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Delay); 
--# expr-done 140
--# expr-start 141 '<Symbol delta>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Delta_220); 
--# expr-done 141
--# expr-start 142 '<Symbol digits>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Digits_221); 
--# expr-done 142
--# expr-start 143 '<Symbol do>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Do); 
--# expr-done 143
--# expr-start 144 '<Symbol else>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Else); 
--# expr-done 144
--# expr-start 145 '<Symbol elsif>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elsif); 
--# expr-done 145
--# expr-start 146 '<Symbol end>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_End); 
--# expr-done 146
--# expr-start 147 '<Symbol entry>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Entry); 
--# expr-done 147
--# expr-start 148 '<Symbol exception>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception); 
--# expr-done 148
--# expr-start 149 '<Symbol exit>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exit); 
--# expr-done 149
--# expr-start 150 '<Symbol for>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_For); 
--# expr-done 150
--# expr-start 151 '<Symbol function>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Function); 
--# expr-done 151
--# expr-start 152 '<Symbol generic>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Generic); 
--# expr-done 152
--# expr-start 153 '<Symbol goto>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Goto); 
--# expr-done 153
--# expr-start 154 '<Symbol if>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_If); 
--# expr-done 154
--# expr-start 155 '<Symbol in>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_In); 
--# expr-done 155
--# expr-start 156 '<Symbol interface>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface_235); 
--# expr-done 156
--# expr-start 157 '<Symbol is>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Is); 
--# expr-done 157
--# expr-start 158 '<Symbol limited>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Limited); 
--# expr-done 158
--# expr-start 159 '<Symbol loop>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop); 
--# expr-done 159
--# expr-start 160 '<Symbol mod>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_239); 
--# expr-done 160
--# expr-start 161 '<Symbol new>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_New); 
--# expr-done 161
--# expr-start 162 '<Symbol not>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not_241); 
--# expr-done 162
--# expr-start 163 '<Symbol null>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Null); 
--# expr-done 163
--# expr-start 164 '<Symbol others>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Others); 
--# expr-done 164
--# expr-start 165 '<Symbol out>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Out); 
--# expr-done 165
--# expr-start 166 '<Symbol of>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Of); 
--# expr-done 166
--# expr-start 167 '<Symbol or>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or_244); 
--# expr-done 167
--# expr-start 168 '<Symbol overriding>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Overriding); 
--# expr-done 168
--# expr-start 169 '<Symbol package>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Package); 
--# expr-done 169
--# expr-start 170 '<Symbol pragma>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pragma); 
--# expr-done 170
--# expr-start 171 '<Symbol private>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Private); 
--# expr-done 171
--# expr-start 172 '<Symbol procedure>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Procedure); 
--# expr-done 172
--# expr-start 173 '<Symbol protected>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Protected); 
--# expr-done 173
--# expr-start 174 '<Symbol raise>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Raise); 
--# expr-done 174
--# expr-start 175 '<Symbol range>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range_254); 
--# expr-done 175
--# expr-start 176 '<Symbol record>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Record); 
--# expr-done 176
--# expr-start 177 '<Symbol rem>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem_256); 
--# expr-done 177
--# expr-start 178 '<Symbol renames>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Renames); 
--# expr-done 178
--# expr-start 179 '<Symbol requeue>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Requeue); 
--# expr-done 179
--# expr-start 180 '<Symbol return>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Return); 
--# expr-done 180
--# expr-start 181 '<Symbol reverse>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Reverse); 
--# expr-done 181
--# expr-start 182 '<Symbol select>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Select); 
--# expr-done 182
--# expr-start 183 '<Symbol separate>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Separate); 
--# expr-done 183
--# expr-start 184 '<Symbol some>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Some); 
--# expr-done 184
--# expr-start 185 '<Symbol subtype>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Subtype); 
--# expr-done 185
--# expr-start 186 '<Symbol synchronized>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Synchronized); 
--# expr-done 186
--# expr-start 187 '<Symbol tagged>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tagged); 
--# expr-done 187
--# expr-start 188 '<Symbol task>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task); 
--# expr-done 188
--# expr-start 189 '<Symbol terminate>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Terminate); 
--# expr-done 189
--# expr-start 190 '<Symbol then>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Then); 
--# expr-done 190
--# expr-start 191 '<Symbol type>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type); 
--# expr-done 191
--# expr-start 192 '<Symbol until>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Until); 
--# expr-done 192
--# expr-start 193 '<Symbol use>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Use); 
--# expr-done 193
--# expr-start 194 '<Symbol when>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_When); 
--# expr-done 194
--# expr-start 195 '<Symbol while>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_While); 
--# expr-done 195
--# expr-start 196 '<Symbol with>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_With); 
--# expr-done 196
--# expr-start 197 '<Symbol xor>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor_280); 
--# expr-done 197
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1, 3 => Sym_2, 4 => Sym_3, 5 => Sym_4, 6 => Sym_5, 7 => Sym_6, 8 => Sym_7, 9 => Sym_8, 10 => Sym_9, 11 => Sym_10, 12 => Sym_11, 13 => Sym_12, 14 => Sym_13, 15 => Sym_14, 16 => Sym_15, 17 => Sym_16, 18 => Sym_17, 19 => Sym_18, 20 => Sym_19, 21 => Sym_20, 22 => Sym_21, 23 => Sym_22, 24 => Sym_23, 25 => Sym_24, 26 => Sym_25, 27 => Sym_26, 28 => Sym_27, 29 => Sym_28, 30 => Sym_29, 31 => Sym_30, 32 => Sym_31, 33 => Sym_32, 34 => Sym_33, 35 => Sym_34, 36 => Sym_35, 37 => Sym_36, 38 => Sym_37, 39 => Sym_38, 40 => Sym_39, 41 => Sym_40, 42 => Sym_41, 43 => Sym_42, 44 => Sym_43, 45 => Sym_44, 46 => Sym_45, 47 => Sym_46, 48 => Sym_47, 49 => Sym_48, 50 => Sym_49, 51 => Sym_50, 52 => Sym_51, 53 => Sym_52, 54 => Sym_53, 55 => Sym_54, 56 => Sym_55, 57 => Sym_56, 58 => Sym_57, 59 => Sym_58, 60 => Sym_59, 61 => Sym_60, 62 => Sym_61, 63 => Sym_62, 64 => Sym_63, 65 => Sym_64, 66 => Sym_65, 67 => Sym_66, 68 => Sym_67, 69 => Sym_68, 70 => Sym_69, 71 => Sym_70, 72 => Sym_71, 73 => Sym_72)); 
--# expr-done 198
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 199
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_20;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_20;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Valid_Keywords;
--# end
--# property-start AdaNode.generic_instantiations /usr/src/packages/BUILD/ada/ast.py:287
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_21;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_21 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_21;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 203 '<Block>' Let_Result None
--# scope-start
--# expr-start 202 '<FieldAccess .generic_instantiations_internal(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:293
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 201 '<FieldAccess .rebindings>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:293
--# expr-start 200 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:293
Fld := Ent.Info;
--# expr-done 200
Fld_1 := Fld.Rebindings;
--# expr-done 201
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 202
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 203
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_21;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_21;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Generic_Instantiations;
--# end
--# property-start AdaNode.generic_instantiations_internal /usr/src/packages/BUILD/ada/ast.py:296
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Generic_Instantiations_Internal
  (Node : Bare_Ada_Node
      ; R : Env_Rebindings
  )
   return Internal_Entity_Generic_Instantiation_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind r R
   Property_Result : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_22;
      Head : Internal_Entity_Generic_Instantiation;
Tail : Internal_Entity_Generic_Instantiation_Array_Access;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Instantiation;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Generic_Instantiation_Array_Access;
Singleton : Internal_Entity_Generic_Instantiation_Array_Access;
Concat_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result : Internal_Entity_Generic_Instantiation_Array_Access;
Is_Equal : Boolean;
If_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
Let_Result_1 : Internal_Entity_Generic_Instantiation_Array_Access;
            procedure Finalizer_Scope_22 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Tail);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_22;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 214 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 213 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:297
--# expr-start 204 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:298
Is_Equal := R = null; 
--# expr-done 204
if Is_Equal then
   If_Result_1 := No_Internal_Entity_Generic_Instantiation_Array_Type;
else
   --# expr-start 212 '<Let head, tail>' Let_Result /usr/src/packages/BUILD/ada/ast.py:301
--# scope-start
--# expr-start 208 '<as_bare_entity(<Cast to GenericInstantiation>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:302
--# expr-start 207 '<Cast to GenericInstantiation>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:302
--# expr-start 206 '<env_node(<FieldAccess .new_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:302
--# expr-start 205 '<FieldAccess .new_env>' Fld /usr/src/packages/BUILD/ada/ast.py:302
      if R = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := R.New_Env;
Inc_Ref (Fld);
--# expr-done 205
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 206
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result := Env_Node;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 207
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 208
Head := If_Result; 
--# bind head Head
--# expr-start 210 '<FieldAccess .generic_instantiations_internal(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:304
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 209 '<FieldAccess .get_parent>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:304
      if R = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := R.Parent;
--# expr-done 209
--# property-call-start AdaNode.generic_instantiations_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations_Internal (Node => Self, R => Fld_1);
--# end
--# expr-done 210
Tail := Fld_2; Inc_Ref (Tail);
--# bind tail Tail
--# expr-start 211 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:305
                Singleton := Create_Internal_Entity_Generic_Instantiation_Array (Items_Count => 1);
                Singleton.Items (1) := Head;
Concat_Result := Concat (Singleton, Tail); 
--# expr-done 211
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 212
   If_Result_1 := Let_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 213
Let_Result_1 := If_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 214
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_22;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_22;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Generic_Instantiations_Internal;
--# end
--# property-start AdaNode.logic_val /usr/src/packages/BUILD/ada/ast.py:313
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Logic_Val
  (Node : Bare_Ada_Node
      ; From_Node : Internal_Entity
      ; Lvar : Logic_Var
  )
   return Internal_Logic_Val_Result
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind from_node From_Node
   --# bind lvar Lvar
   Property_Result : Internal_Logic_Val_Result;
      Block_Var : Boolean;
Fld : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
If_Result_1 : Internal_Entity;
New_Struct : Internal_Logic_Val_Result;
Let_Result : Internal_Logic_Val_Result;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 219 '<Block>' Let_Result None
--# scope-start
--# expr-start 215 '<FieldAccess .resolve_names_from_closest_entry_point>' Fld /usr/src/packages/BUILD/ada/ast.py:314
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => From_Node.Node, E_Info => From_Node.Info);
--# end
--# expr-done 215
Block_Var := Fld; 
--# bind success Block_Var
--# expr-start 218 '<New InternalLogicValResult>' New_Struct /usr/src/packages/BUILD/ada/ast.py:316
--# expr-start 217 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:316
if Block_Var then
   --# expr-start 216 '<get_value(<Var lvar>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:317
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Lvar); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Lvar); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 216
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity;
end if;
--# expr-done 217
New_Struct := (Success => Block_Var, Value => If_Result_1); 
--# expr-done 218
Let_Result := New_Struct; 
--# end
--# expr-done 219
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Logic_Val;
--# end
--# property-start AdaNode.semantic_parent_helper /usr/src/packages/BUILD/ada/ast.py:321
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent_Helper
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1470;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Internal_Entity;
Env_Node : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Env_Parent : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Result_Var : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1470 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Parent);
            end Finalizer_Scope_1470;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 224 '<Block>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 221 '<as_entity(<env_node(<Var env>)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:322
--# expr-start 220 '<env_node(<Var env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:322
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 220
Is_Null := Env_Node = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Env_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 221
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 223 '<FieldAccess .semantic_parent_helper(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:323
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 222 '<env_parent(<Var env>)>' Env_Parent /usr/src/packages/BUILD/ada/ast.py:323
Env_Parent := AST_Envs.Parent (Var_Expr); 
--# expr-done 222
--# property-call-start AdaNode.semantic_parent_helper
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Env_Parent, E_Info => Ent.Info);
--# end
--# expr-done 223
   Result_Var := Fld;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1470;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 224
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1470;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Semantic_Parent_Helper;
--# end
--# property-start AdaNode.semantic_parent /usr/src/packages/BUILD/ada/ast.py:327
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Semantic_Parent
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_25;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_25 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_25;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 227 '<Block>' Let_Result None
--# scope-start
--# expr-start 226 '<FieldAccess .semantic_parent_helper(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:336
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 225 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:336
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 225
--# property-call-start AdaNode.semantic_parent_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent_Helper (Node => Ent.Node, Env => Fld, E_Info => Ent.Info);
--# end
--# expr-done 226
Let_Result := Fld_1; 
--# end
--# expr-done 227
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_25;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_25;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Semantic_Parent;
--# end
--# property-start AdaNode.parent_basic_decl /usr/src/packages/BUILD/ada/ast.py:339
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Basic_Decl
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1472;
            procedure Finalizer_Scope_1473;
      Var_Expr : Internal_Entity;
Inst : Bare_Basic_Decl;
Var_Expr_1 : Env_Rebindings;
Designated : Internal_Entity;
Var_Expr_2 : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_3 : Internal_Entity_Generic_Subp_Instantiation;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Cast_Expr : Bare_Basic_Decl;
Cast_Result_1 : Bare_Generic_Package_Instantiation;
As_Entity : Internal_Entity_Generic_Package_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Package_Instantiation;
Fld_4 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_1 : Internal_Entity_Base_Package_Decl;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_2 : Bare_Generic_Subp_Instantiation;
As_Entity_1 : Internal_Entity_Generic_Subp_Instantiation;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Generic_Subp_Instantiation;
Fld_5 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_A : Boolean;
If_Result_2 : Internal_Entity;
Fld_6 : Internal_Entity_Info;
Fld_7 : Env_Rebindings;
Fld_8 : Env_Rebindings;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_10 : Bare_Ada_Node;
Fld_11 : Bare_Ada_Node;
Cast_Result_5 : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Fld_12 : Internal_Entity_Basic_Decl;
Result_Var_3 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
Is_A_2 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1472 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1472;
            procedure Finalizer_Scope_1473 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1473;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 260 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 228 '<FieldAccess .semantic_parent>' Fld /usr/src/packages/BUILD/ada/ast.py:348
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 228
Var_Expr := Fld;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind sp Var_Expr
--# expr-start 259 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:349
--# expr-start 229 '<IsA GenericPackageInternal>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:350
Is_A_1 := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Package_Internal_Range; 
--# expr-done 229
if Is_A_1 then
   If_Result_4 := True;
else
   --# expr-start 230 '<IsA GenericSubpInternal>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:350
Is_A_2 := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 230
   If_Result_4 := Is_A_2;
end if;
if If_Result_4 then
   --# expr-start 256 '<Let inst>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:351
--# scope-start
--# expr-start 235 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:353
--# expr-start 234 '<env_node(<Then <FieldAccess .rebindings>: <Var Var_Expr_1> <FieldAccess .new_env>>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:353
--# expr-start 232 '<FieldAccess .rebindings>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:353
--# expr-start 231 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:353
Fld_1 := Var_Expr.Info;
--# expr-done 231
Fld_2 := Fld_1.Rebindings;
--# expr-done 232
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 233 '<FieldAccess .new_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:353
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr_1.New_Env;
Inc_Ref (Fld_3);
--# expr-done 233
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1473;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Node := AST_Envs.Env_Node (Result_Var); 
--# expr-done 234
   if Env_Node = null
      or else Env_Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 235
Inst := Cast_Result; 
--# bind inst Inst
--# expr-start 255 '<Let designated>' Let_Result /usr/src/packages/BUILD/ada/ast.py:359
--# scope-start
--# expr-start 243 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:360
--# expr-start 236 '<IsA GenericPackageInternal>' Is_A /usr/src/packages/BUILD/ada/ast.py:361
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Generic_Package_Internal_Range; 
--# expr-done 236
if Is_A then
--# expr-start 238 '<as_bare_entity(<Cast to GenericPackageInstantiation>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:363
--# expr-start 237 '<Cast to GenericPackageInstantiation>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:363
Cast_Expr := Inst; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 237
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Package_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 238
Var_Expr_2 := If_Result;
if Var_Expr_2 /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 239 '<FieldAccess .designated_package>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:363
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld_4 := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 239
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Package_Decl;
end if;
      Cast_Result_3 := Create_Internal_Entity
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   If_Result_2 := Cast_Result_3;
else
--# expr-start 241 '<as_bare_entity(<Cast to GenericSubpInstantiation>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:366
--# expr-start 240 '<Cast to GenericSubpInstantiation>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:366
Cast_Expr_1 := Inst; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Generic_Subp_Instantiation_Range
   then
      Cast_Result_2 := Cast_Expr_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 240
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Generic_Subp_Instantiation;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 241
Var_Expr_3 := If_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Subp_Instantiation then
   --# scope-start
--# expr-start 242 '<FieldAccess .designated_subp>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:366
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld_5 := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 242
Scope_Result_2 := Fld_5;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
   If_Result_2 := Result_Var_2;
end if;
--# expr-done 243
Designated := If_Result_2; 
--# bind designated Designated
--# expr-start 254 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:368
--# expr-start 246 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:369
--# expr-start 244 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:369
Fld_10 := Designated.Node;
--# expr-done 244
      Cast_Result_5 := Fld_10;
--# expr-start 245 '<FieldAccess .node>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:369
Fld_11 := Var_Expr.Node;
--# expr-done 245
Is_Equal := Cast_Result_5 = Fld_11; 
--# expr-done 246
if Is_Equal then
   --# expr-start 251 '<New InternalEntityBasicDecl>' New_Struct_1 None
--# expr-start 250 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:372
--# expr-start 249 '<FieldAccess .get_parent>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:374
--# expr-start 248 '<FieldAccess .rebindings>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:374
--# expr-start 247 '<FieldAccess .info>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:374
Fld_6 := Var_Expr.Info;
--# expr-done 247
Fld_7 := Fld_6.Rebindings;
--# expr-done 248
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Fld_7.Parent;
--# expr-done 249
New_Struct := (From_Rebound => False, MD => No_Metadata, Rebindings => Fld_8); 
--# expr-done 250
New_Struct_1 := (Info => New_Struct, Node => Inst); 
--# expr-done 251
   If_Result_3 := New_Struct_1;
else
   --# expr-start 253 '<Cast to BasicDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:378
--# expr-start 252 '<parent(<Var sp>)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:378
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 252
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 253
   If_Result_3 := Cast_Result_4;
end if;
--# expr-done 254
Let_Result := If_Result_3; 
--# end
--# expr-done 255
Let_Result_1 := Let_Result; 
--# end
--# expr-done 256
   If_Result_5 := Let_Result_1;
else
--# expr-start 257 '<Cast to BasicDecl>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:382
Cast_Expr_2 := Var_Expr; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_6 := No_Entity_Basic_Decl;
   end if;
--# expr-done 257
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_3 := Var_Expr_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 258 '<FieldAccess .parent_basic_decl>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:382
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_12 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 258
   Result_Var_3 := Fld_12;
end if;
   If_Result_5 := Result_Var_3;
end if;
--# expr-done 259
Scope_Result_4 := If_Result_5;
   --# end
      Finalizer_Scope_1472;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var_4; 
--# end
--# expr-done 260
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1472;
               Finalizer_Scope_1473;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Basic_Decl;
--# end
--# property-start AdaNode.designated_compilation_unit /usr/src/packages/BUILD/ada/ast.py:406
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Designated_Compilation_Unit
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
  )
   return Bare_Compilation_Unit
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   Property_Result : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1493;
            procedure Finalizer_Scope_1494;
      Block_Var : Internal_Unit;
Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Compilation_Unit;
Match_1 : Bare_Compilation_Unit_List;
Match_2 : Bare_Ada_Node;
Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Item_8 : Bare_Compilation_Unit;
Fld_2 : Symbol_Type_Array_Access;
Is_Equal : Boolean;
Map_Result : Bare_Compilation_Unit_Array_Access;
Get_Result : Bare_Compilation_Unit;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Let_Result : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Cast_Result_1 : Bare_Compilation_Unit_List;
Let_Result_1 : Bare_Compilation_Unit;
Scope_Result_1 : Bare_Compilation_Unit;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Bare_Compilation_Unit;
Scope_Result_2 : Bare_Compilation_Unit;
Match_Result : Bare_Compilation_Unit;
Scope_Result_3 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
Let_Result_3 : Bare_Compilation_Unit;
            procedure Finalizer_Scope_1493 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1493;
            procedure Finalizer_Scope_1494 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1494;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 268 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 261 '<FieldAccess .get_unit(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:414
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed);
--# expr-done 261
Block_Var := Fld; 
--# bind designated_analysis_unit Block_Var
--# expr-start 262 '<FieldAccess .root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:417
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Block_Var.AST_Root;
--# expr-done 262
Var_Expr := Fld_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 267 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:417
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Compilation_Unit =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind single Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Compilation_Unit_List =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind multi Match_1
--# expr-start 266 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:424
--# expr-start 265 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:424
declare
   Map_Result_Vec : Bare_Compilation_Unit_Vectors.Vector;
begin
      if Match_1 = null then
         Map_Result := Create_Bare_Compilation_Unit_Array (0);
      else
      declare
         Collection : constant Bare_Compilation_Unit_List := Match_1;
      begin
         for Untyped_Item_8 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_8 := Untyped_Item_8; 
   --# scope-start
               --# bind c Item_8
               --# expr-start 264 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:425
--# expr-start 263 '<FieldAccess .syntactic_fully_qualified_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:425
      if Item_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Item_8);
--# end
--# expr-done 263
Is_Equal := Equivalent (Fld_2, Name); 
--# expr-done 264
               if Is_Equal then
         declare
            Item_To_Append : constant Bare_Compilation_Unit :=
               Item_8;
         begin
            Bare_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1494;
         end loop;
      end;
      Map_Result := Create_Bare_Compilation_Unit_Array
        (Items_Count => Natural (Bare_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 265
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 266
Let_Result_1 := Get_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
      Finalizer_Scope_1493;
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Case_Stmt_Alternative_List | Ada_Contract_Case_Assoc_List .. Ada_Case_Stmt_Alternative | Ada_Component_Clause .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected analysis unit root"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 267
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 268
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1493;
               Finalizer_Scope_1494;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Designated_Compilation_Unit;
--# end
--# property-start AdaNode.get_unit_root_decl /usr/src/packages/BUILD/ada/ast.py:434
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Get_Unit_Root_Decl
  (Node : Bare_Ada_Node
      ; Name : Symbol_Type_Array_Access
      ; Kind : Analysis_Unit_Kind
      ; Load_If_Needed : Boolean
         := True
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind name Name
   --# bind kind Kind
   --# bind load_if_needed Load_If_Needed
   Property_Result : Bare_Basic_Decl;
      Block_Var : Bare_Compilation_Unit;
Var_Expr : Bare_Compilation_Unit;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Result_Var : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 271 '<Block>' Let_Result None
--# scope-start
--# expr-start 269 '<FieldAccess .designated_compilation_unit(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:441
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.designated_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Name, Kind => Kind, Load_If_Needed => Load_If_Needed);
--# end
--# expr-done 269
Block_Var := Fld; 
--# bind cu Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 270 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:442
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Var_Expr);
--# end
--# expr-done 270
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 271
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Get_Unit_Root_Decl;
--# end
--# property-start '[dispatcher]AdaNode.xref_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_All_Present | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Record_Def | Ada_Discriminant_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Constant_Absent .. Ada_Constant_Present | Ada_Declarative_Part .. Ada_Elsif_Expr_Part | Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Discrete_Subtype_Name | Ada_Handled_Stmts .. Ada_Limited_Present | Ada_Mode_Default .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Paren_Abstract_State_Decl | Ada_Private_Absent .. Ada_Quantifier_Some | Ada_Renaming_Clause .. Ada_Reverse_Present | Ada_Error_Stmt | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Tagged_Present | Ada_Anonymous_Type_Access_Def | Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_equation
                  Property_Result := Ada_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.xref_equation
                  Property_Result := Constrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.xref_equation
                  Property_Result := Unconstrained_Array_Indices_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_equation
                  Property_Result := Aspect_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_At_Clause =>
                  --# property-call-start AtClause.xref_equation
                  Property_Result := At_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Def_Clause =>
                  --# property-call-start AttributeDefClause.xref_equation
                  Property_Result := Attribute_Def_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Rep_Clause =>
                  --# property-call-start EnumRepClause.xref_equation
                  Property_Result := Enum_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Rep_Clause =>
                  --# property-call-start RecordRepClause.xref_equation
                  Property_Result := Record_Rep_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.xref_equation
                  Property_Result := Base_Subp_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_equation
                  Property_Result := Aggregate_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_equation
                  Property_Result := Iterated_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_equation
                  Property_Result := Param_Assoc_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_equation
                  Property_Result := Component_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_equation
                  Property_Result := Discriminant_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_equation
                  Property_Result := Generic_Formal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_equation
                  Property_Result := Param_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_equation
                  Property_Result := Subtype_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_equation
                  Property_Result := Protected_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_equation
                  Property_Result := Task_Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.xref_equation
                  Property_Result := Type_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.xref_equation
                  Property_Result := Formal_Subp_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_equation
                  Property_Result := Expr_Function_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_equation
                  Property_Result := Subp_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_equation
                  Property_Result := Entry_Body_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_equation
                  Property_Result := Entry_Index_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_equation
                  Property_Result := Exception_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_equation
                  Property_Result := Exception_Handler_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_equation
                  Property_Result := Generic_Instantiation_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_equation
                  Property_Result := Generic_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_equation
                  Property_Result := Number_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_equation
                  Property_Result := Object_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_equation
                  Property_Result := Package_Renaming_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_equation
                  Property_Result := Single_Protected_Decl_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_equation
                  Property_Result := Case_Stmt_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_equation
                  Property_Result := Component_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Def =>
                  --# property-call-start ComponentDef.xref_equation
                  Property_Result := Component_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delta_Constraint =>
                  --# property-call-start DeltaConstraint.xref_equation
                  Property_Result := Delta_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Digits_Constraint =>
                  --# property-call-start DigitsConstraint.xref_equation
                  Property_Result := Digits_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Constraint =>
                  --# property-call-start DiscriminantConstraint.xref_equation
                  Property_Result := Discriminant_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Index_Constraint =>
                  --# property-call-start IndexConstraint.xref_equation
                  Property_Result := Index_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Constraint =>
                  --# property-call-start RangeConstraint.xref_equation
                  Property_Result := Range_Constraint_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_equation
                  Property_Result := Elsif_Stmt_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl_Expr =>
                  --# property-call-start AbstractStateDeclExpr.xref_equation
                  Property_Result := Abstract_State_Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Allocator =>
                  --# property-call-start Allocator.xref_equation
                  Property_Result := Allocator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
                  --# property-call-start BaseAggregate.xref_equation
                  Property_Result := Base_Aggregate_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.xref_equation
                  Property_Result := Bin_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Box_Expr =>
                  --# property-call-start BoxExpr.xref_equation
                  Property_Result := Box_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.xref_equation
                  Property_Result := Case_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.xref_equation
                  Property_Result := If_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decl_Expr =>
                  --# property-call-start DeclExpr.xref_equation
                  Property_Result := Decl_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Membership_Expr =>
                  --# property-call-start MembershipExpr.xref_equation
                  Property_Result := Membership_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref =>
                  --# property-call-start AttributeRef.xref_equation
                  Property_Result := Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Update_Attribute_Ref =>
                  --# property-call-start UpdateAttributeRef.xref_equation
                  Property_Result := Update_Attribute_Ref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.xref_equation
                  Property_Result := Call_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.xref_equation
                  Property_Result := Defining_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.xref_equation
                  Property_Result := Dotted_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_equation
                  Property_Result := End_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.xref_equation
                  Property_Result := Explicit_Deref_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.xref_equation
                  Property_Result := Qual_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Identifier =>
                  --# property-call-start BaseId.xref_equation
                  Property_Result := Base_Id_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal =>
                  --# property-call-start CharLiteral.xref_equation
                  Property_Result := Char_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.xref_equation
                  Property_Result := Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_String_Literal =>
                  --# property-call-start StringLiteral.xref_equation
                  Property_Result := String_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Literal =>
                  --# property-call-start NullLiteral.xref_equation
                  Property_Result := Null_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Int_Literal =>
                  --# property-call-start IntLiteral.xref_equation
                  Property_Result := Int_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Real_Literal =>
                  --# property-call-start RealLiteral.xref_equation
                  Property_Result := Real_Literal_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.xref_equation
                  Property_Result := Target_Name_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Paren_Expr =>
                  --# property-call-start ParenExpr.xref_equation
                  Property_Result := Paren_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Quantified_Expr =>
                  --# property-call-start QuantifiedExpr.xref_equation
                  Property_Result := Quantified_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Expr =>
                  --# property-call-start RaiseExpr.xref_equation
                  Property_Result := Raise_Expr_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.xref_equation
                  Property_Result := Un_Op_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_equation
                  Property_Result := For_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_While_Loop_Spec =>
                  --# property-call-start WhileLoopSpec.xref_equation
                  Property_Result := While_Loop_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Others_Designator =>
                  --# property-call-start OthersDesignator.xref_equation
                  Property_Result := Others_Designator_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_equation
                  Property_Result := Pragma_Node_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Range_Spec =>
                  --# property-call-start RangeSpec.xref_equation
                  Property_Result := Range_Spec_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_When_Part =>
                  --# property-call-start SelectWhenPart.xref_equation
                  Property_Result := Select_When_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Accept_Stmt_With_Stmts =>
                  --# property-call-start AcceptStmt.xref_equation
                  Property_Result := Accept_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Stmt .. Ada_While_Loop_Stmt =>
                  --# property-call-start BaseLoopStmt.xref_equation
                  Property_Result := Base_Loop_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Begin_Block .. Ada_Decl_Block =>
                  --# property-call-start BlockStmt.xref_equation
                  Property_Result := Block_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Case_Stmt =>
                  --# property-call-start CaseStmt.xref_equation
                  Property_Result := Case_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Extended_Return_Stmt =>
                  --# property-call-start ExtendedReturnStmt.xref_equation
                  Property_Result := Extended_Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_If_Stmt =>
                  --# property-call-start IfStmt.xref_equation
                  Property_Result := If_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt =>
                  --# property-call-start NamedStmt.xref_equation
                  Property_Result := Named_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Select_Stmt =>
                  --# property-call-start SelectStmt.xref_equation
                  Property_Result := Select_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abort_Stmt =>
                  --# property-call-start AbortStmt.xref_equation
                  Property_Result := Abort_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Assign_Stmt =>
                  --# property-call-start AssignStmt.xref_equation
                  Property_Result := Assign_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Stmt =>
                  --# property-call-start CallStmt.xref_equation
                  Property_Result := Call_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Delay_Stmt =>
                  --# property-call-start DelayStmt.xref_equation
                  Property_Result := Delay_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exit_Stmt =>
                  --# property-call-start ExitStmt.xref_equation
                  Property_Result := Exit_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Goto_Stmt =>
                  --# property-call-start GotoStmt.xref_equation
                  Property_Result := Goto_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Label =>
                  --# property-call-start Label.xref_equation
                  Property_Result := Label_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Null_Stmt =>
                  --# property-call-start NullStmt.xref_equation
                  Property_Result := Null_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Raise_Stmt =>
                  --# property-call-start RaiseStmt.xref_equation
                  Property_Result := Raise_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Requeue_Stmt =>
                  --# property-call-start RequeueStmt.xref_equation
                  Property_Result := Requeue_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Return_Stmt =>
                  --# property-call-start ReturnStmt.xref_equation
                  Property_Result := Return_Stmt_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Terminate_Alternative =>
                  --# property-call-start TerminateAlternative.xref_equation
                  Property_Result := Terminate_Alternative_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_equation
                  Property_Result := Subunit_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Def =>
                  --# property-call-start TaskDef.xref_equation
                  Property_Result := Task_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.xref_equation
                  Property_Result := Access_To_Subp_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.xref_equation
                  Property_Result := Type_Access_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_equation
                  Property_Result := Array_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.xref_equation
                  Property_Result := Derived_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.xref_equation
                  Property_Result := Enum_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.xref_equation
                  Property_Result := Formal_Discrete_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.xref_equation
                  Property_Result := Interface_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.xref_equation
                  Property_Result := Mod_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.xref_equation
                  Property_Result := Private_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.xref_equation
                  Property_Result := Real_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.xref_equation
                  Property_Result := Record_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.xref_equation
                  Property_Result := Signed_Int_Type_Def_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.xref_equation
                  Property_Result := Anonymous_Type_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.xref_equation
                  Property_Result := Subtype_Indication_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Package_Clause =>
                  --# property-call-start UsePackageClause.xref_equation
                  Property_Result := Use_Package_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Use_Type_Clause =>
                  --# property-call-start UseTypeClause.xref_equation
                  Property_Result := Use_Type_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_equation
                  Property_Result := Variant_Part_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_equation
                  Property_Result := With_Clause_P_Xref_Equation
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Equation;
--# end
--# property-start '[dispatcher]AdaNode.xref_stop_resolution' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Record_Def | Ada_Discriminant_Assoc | Ada_Param_Assoc .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_stop_resolution
                  Property_Result := Ada_Node_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.xref_stop_resolution
                  Property_Result := Aggregate_Assoc_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.xref_stop_resolution
                  Property_Result := Iterated_Assoc_P_Xref_Stop_Resolution
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.stop_resolution_equation /usr/src/packages/BUILD/ada/ast.py:472
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Stop_Resolution_Equation
  (Node : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_33;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_33 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_33;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:472") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_33;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_33;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Stop_Resolution_Equation;
--# end
--# property-start AdaNode.sub_equation /usr/src/packages/BUILD/ada/ast.py:475
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Sub_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_34;
      Fld : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_34 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_34;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 276 '<Block>' Let_Result None
--# scope-start
--# expr-start 275 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:483
--# expr-start 272 '<FieldAccess .xref_stop_resolution>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:483
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node);
--# end
--# expr-done 272
if Fld_2 then
   --# expr-start 273 '<FieldAccess .stop_resolution_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:484
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.stop_resolution_equation
Fld := Libadalang.Implementation.Ada_Node_P_Stop_Resolution_Equation (Node => Ent.Node);
--# end
--# expr-done 273
   If_Result := Fld;
else
   --# expr-start 274 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:485
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 274
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 275
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 276
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_34;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_34;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Sub_Equation;
--# end
--# property-start AdaNode.resolve_children_names /usr/src/packages/BUILD/ada/ast.py:498
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Children_Names
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_36;
      Var_Expr : Bare_Ada_Node;
Node_Children : Bare_Ada_Node_Array_Access;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Fld_2 : Boolean;
If_Result_3 : Boolean;
Fld_3 : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_36 is
            begin
                     Dec_Ref (Node_Children);
            end Finalizer_Scope_36;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 286 '<Block>' Let_Result None
--# scope-start
--# expr-start 277 '<children(<Var Self>)>' Node_Children /usr/src/packages/BUILD/ada/ast.py:503
Node_Children := Children (Self); 
--# expr-done 277
Quantifier_Result := True;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Node_Children;
   begin
      for Item_9 of
            Collection.Items
      loop
   --# scope-start
            --# bind c Item_9
Var_Expr := Item_9;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 285 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:506
--# expr-start 278 '<FieldAccess .xref_entry_point>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:507
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Var_Expr);
--# end
--# expr-done 278
if Fld_3 then
   If_Result_4 := True;
else
--# expr-start 282 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:509
--# expr-start 279 '<FieldAccess .xref_stop_resolution>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:509
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Var_Expr);
--# end
--# expr-done 279
if Fld_1 then
   --# expr-start 281 '<FieldAccess .resolve_own_names>' Fld /usr/src/packages/BUILD/ada/ast.py:509
--# expr-start 280 '<as_entity(<Var c>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:509
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 280
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# expr-done 281
   If_Result_1 := Fld;
else
   If_Result_1 := True;
end if;
--# expr-done 282
if If_Result_1 then
   --# expr-start 284 '<FieldAccess .resolve_children_names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:510
--# expr-start 283 '<as_entity(<Var c>)>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:510
Is_Null_1 := Var_Expr = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
As_Entity_1 := (Info => E_Info, Node => Var_Expr); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 283
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => If_Result_2.Node, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 284
   If_Result_3 := Fld_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 285
Scope_Result := If_Result_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
            if not (Result_Var) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 286
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_36;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_36;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Children_Names;
--# end
--# property-start AdaNode.resolve_names_internal /usr/src/packages/BUILD/ada/ast.py:516
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 289 '<Block>' Let_Result None
--# scope-start
--# expr-start 287 '<FieldAccess .resolve_own_names>' Fld /usr/src/packages/BUILD/ada/ast.py:520
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# expr-done 287
if Fld then
   --# expr-start 288 '<FieldAccess .resolve_children_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:520
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 288
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 289
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_Internal;
--# end
--# property-start AdaNode.resolve_names_internal_with_eq /usr/src/packages/BUILD/ada/ast.py:523
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_Internal_With_Eq
  (Node : Bare_Ada_Node
      ; Additional_Equation : Logic_Equation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind additional_equation Additional_Equation
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_38;
      Fld : Logic_Equation;
And_Pred : Logic_Equation;
Solve_Success : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_38 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_38;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 294 '<Block>' Let_Result None
--# scope-start
--# expr-start 292 '<solve(<And>)>' Solve_Success /usr/src/packages/BUILD/ada/ast.py:528
--# expr-start 291 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:528
--# expr-start 290 '<FieldAccess .xref_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:528
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 290
And_Pred := Logic_And (Fld, Additional_Equation, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:528") else null)); 
--# expr-done 291
Solve_Success := Solve_Wrapper (And_Pred, Self); 
--# expr-done 292
if Solve_Success then
   --# expr-start 293 '<FieldAccess .resolve_children_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:529
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_children_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Children_Names (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 293
   If_Result := Fld_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 294
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_38;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_38;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_Internal_With_Eq;
--# end
--# property-start '[dispatcher]AdaNode.xref_entry_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Ada_Node (Self.Kind) is
               when Ada_Abort_Absent .. Ada_Unconstrained_Array_Indices | Ada_Aspect_Spec .. Ada_Pragma_Argument_Assoc | Ada_Component_List .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Null_Subp_Decl .. Ada_Subp_Body | Ada_Package_Body_Stub .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Task_Body | Ada_Error_Decl | Ada_For_Loop_Var_Decl .. Ada_Generic_Subp_Decl | Ada_Label_Decl .. Ada_Named_Stmt_Decl | Ada_Single_Task_Decl | Ada_Compilation_Unit | Ada_Component_Def .. Ada_Elsif_Expr_Part | Ada_Abstract_State_Decl_Expr .. Ada_Call_Expr | Ada_Discrete_Subtype_Name .. Ada_Dotted_Name | Ada_Explicit_Deref .. Ada_Limited_Present | Ada_While_Loop_Spec .. Ada_Paren_Abstract_State_Decl | Ada_Private_Absent .. Ada_Select_When_Part | Ada_Subp_Kind_Function .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_Type_Access_Def | Ada_Derived_Type_Def .. Ada_Until_Present | Ada_Variant | Ada_With_Private_Absent .. Ada_With_Private_Present =>
                  --# property-call-start AdaNode.xref_entry_point
                  Property_Result := Ada_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Aspect_Assoc =>
                  --# property-call-start AspectAssoc.xref_entry_point
                  Property_Result := Aspect_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_At_Clause .. Ada_Record_Rep_Clause =>
                  --# property-call-start AspectClause.xref_entry_point
                  Property_Result := Aspect_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.xref_entry_point
                  Property_Result := Base_Subp_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.xref_entry_point
                  Property_Result := Param_Assoc_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.xref_entry_point
                  Property_Result := Component_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.xref_entry_point
                  Property_Result := Discriminant_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.xref_entry_point
                  Property_Result := Generic_Formal_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.xref_entry_point
                  Property_Result := Param_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.xref_entry_point
                  Property_Result := Subtype_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.xref_entry_point
                  Property_Result := Protected_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.xref_entry_point
                  Property_Result := Task_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Type_Decl =>
                  --# property-call-start TypeDecl.xref_entry_point
                  Property_Result := Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start AnonymousTypeDecl.xref_entry_point
                  Property_Result := Anonymous_Type_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Expr_Function =>
                  --# property-call-start ExprFunction.xref_entry_point
                  Property_Result := Expr_Function_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subp_Renaming_Decl =>
                  --# property-call-start SubpRenamingDecl.xref_entry_point
                  Property_Result := Subp_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.xref_entry_point
                  Property_Result := Entry_Body_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.xref_entry_point
                  Property_Result := Entry_Index_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.xref_entry_point
                  Property_Result := Exception_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.xref_entry_point
                  Property_Result := Exception_Handler_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericInstantiation.xref_entry_point
                  Property_Result := Generic_Instantiation_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericRenamingDecl.xref_entry_point
                  Property_Result := Generic_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.xref_entry_point
                  Property_Result := Number_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.xref_entry_point
                  Property_Result := Object_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.xref_entry_point
                  Property_Result := Package_Renaming_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.xref_entry_point
                  Property_Result := Single_Protected_Decl_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Case_Stmt_Alternative =>
                  --# property-call-start CaseStmtAlternative.xref_entry_point
                  Property_Result := Case_Stmt_Alternative_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Component_Clause =>
                  --# property-call-start ComponentClause.xref_entry_point
                  Property_Result := Component_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Elsif_Stmt_Part =>
                  --# property-call-start ElsifStmtPart.xref_entry_point
                  Property_Result := Elsif_Stmt_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.xref_entry_point
                  Property_Result := Defining_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.xref_entry_point
                  Property_Result := End_Name_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_For_Loop_Spec =>
                  --# property-call-start ForLoopSpec.xref_entry_point
                  Property_Result := For_Loop_Spec_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Pragma_Node =>
                  --# property-call-start Pragma.xref_entry_point
                  Property_Result := Pragma_Node_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Accept_Stmt .. Ada_Terminate_Alternative =>
                  --# property-call-start Stmt.xref_entry_point
                  Property_Result := Stmt_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Subunit =>
                  --# property-call-start Subunit.xref_entry_point
                  Property_Result := Subunit_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.xref_entry_point
                  Property_Result := Array_Type_Def_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Use_Package_Clause .. Ada_Use_Type_Clause =>
                  --# property-call-start UseClause.xref_entry_point
                  Property_Result := Use_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_Variant_Part =>
                  --# property-call-start VariantPart.xref_entry_point
                  Property_Result := Variant_Part_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
               when Ada_With_Clause =>
                  --# property-call-start WithClause.xref_entry_point
                  Property_Result := With_Clause_P_Xref_Entry_Point
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Ada_Node_P_Xref_Entry_Point;
--# end
--# property-start AdaNode.resolve_names /usr/src/packages/BUILD/ada/ast.py:548
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_40;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_40 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_40;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Resolve_Names,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 300 '<Block>' Let_Result None
--# scope-start
--# expr-start 299 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_names_internal>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:554
--# expr-start 295 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:555
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 295
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 298 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_names_internal>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:556
--# expr-start 296 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:556
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 296
Bound_Origin := Fld_1; 
--# expr-start 297 '<FieldAccess .resolve_names_internal>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:556
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 297
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 298
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 299
Let_Result := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 300
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_40;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_40;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names;
--# end
--# property-start AdaNode.resolve_names_from_closest_entry_point /usr/src/packages/BUILD/ada/ast.py:560
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Names_From_Closest_Entry_Point
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
            procedure Finalizer_Scope_41;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
Fld_3 : Internal_Entity;
Fld_4 : Boolean;
Fld_5 : Lexical_Env := Empty_Env;
Bound_Env_1 : Lexical_Env := Empty_Env;
Fld_6 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_7 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
Fld_8 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_9 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_41 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bound_Env_1);
            end Finalizer_Scope_41;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 317 '<Block>' Let_Result None
--# scope-start
--# expr-start 316 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:582
--# expr-start 301 '<FieldAccess .xref_entry_point>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:583
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_9 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Ent.Node);
--# end
--# expr-done 301
if Fld_9 then
   --# expr-start 306 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:584
--# expr-start 302 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 302
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 305 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:586
--# expr-start 303 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:586
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 303
Bound_Origin := Fld_1; 
--# expr-start 304 '<FieldAccess .resolve_own_names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:586
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# expr-done 304
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 305
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 306
   If_Result_2 := Dyn_Var_Bind_Result_1;
else
--# expr-start 308 '<FieldAccess .resolve_names_from_closest_entry_point>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:589
--# expr-start 307 '<parent(<Var Ent>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:589
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 307
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_from_closest_entry_point
Fld_4 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_From_Closest_Entry_Point (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 308
if Fld_4 then
   --# expr-start 315 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:589
--# expr-start 309 '<FieldAccess .xref_stop_resolution>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:590
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_stop_resolution'
Fld_8 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Stop_Resolution (Node => Ent.Node);
--# end
--# expr-done 309
if Fld_8 then
   --# expr-start 314 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>)>' Dyn_Var_Bind_Result_3 /usr/src/packages/BUILD/ada/ast.py:591
--# expr-start 310 '<FieldAccess .children_env>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:592
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 310
Bound_Env_1 := Fld_5; Inc_Ref (Bound_Env_1);
--# expr-start 313 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .resolve_own_names>)>' Dyn_Var_Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:593
--# expr-start 311 '<FieldAccess .origin_node>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:593
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 311
Bound_Origin_1 := Fld_6; 
--# expr-start 312 '<FieldAccess .resolve_own_names>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:594
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Ada_Node_P_Resolve_Own_Names (Node => Ent.Node, Env => Bound_Env_1, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# expr-done 312
Dyn_Var_Bind_Result_2 := Fld_7; 
--# expr-done 313
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 314
   If_Result := Dyn_Var_Bind_Result_3;
else
   If_Result := True;
end if;
--# expr-done 315
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 316
Let_Result := If_Result_2; 
--# end
--# expr-done 317
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_41;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_41;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Names_From_Closest_Entry_Point;
--# end
--# property-start AdaNode.parent_unit_env_helper /usr/src/packages/BUILD/ada/ast.py:601
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env_Helper
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind unit Unit
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_42;
            procedure Finalizer_Scope_1504;
            procedure Finalizer_Scope_1505;
      Var_Expr : Lexical_Env := Empty_Env;
Var_Expr_1 : Bare_Ada_Node;
Env_Parent : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
If_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_42 is
            begin
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_42;
            procedure Finalizer_Scope_1504 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1504;
            procedure Finalizer_Scope_1505 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1505;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 324 '<Block>' Let_Result None
--# scope-start
--# expr-start 318 '<env_parent(<Var env>)>' Env_Parent /usr/src/packages/BUILD/ada/ast.py:602
Env_Parent := AST_Envs.Parent (Env); 
--# expr-done 318
Var_Expr := Env_Parent;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind parent_env Var_Expr
--# expr-start 319 '<env_node(<Var parent_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:602
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 319
Var_Expr_1 := Env_Node;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind parent_node Var_Expr_1
--# expr-start 323 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:603
--# expr-start 321 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:604
--# expr-start 320 '<FieldAccess .unit>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:604
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Var_Expr_1);
--# expr-done 320
Is_Equal := Fld_1 = Unit; 
--# expr-done 321
if Is_Equal then
   --# expr-start 322 '<FieldAccess .parent_unit_env_helper(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_unit_env_helper
Fld := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Unit, Env => Var_Expr);
--# end
--# expr-done 322
   If_Result := Fld;
else
   If_Result := Var_Expr;
end if;
      Inc_Ref (If_Result);
--# expr-done 323
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1505;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1504;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 324
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_42;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_42;
               Finalizer_Scope_1504;
               Finalizer_Scope_1505;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Unit_Env_Helper;
--# end
--# property-start AdaNode.parent_unit_env /usr/src/packages/BUILD/ada/ast.py:611
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Parent_Unit_Env
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_43;
            procedure Finalizer_Scope_1506;
      Var_Expr : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld : Internal_Unit;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_43 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_43;
            procedure Finalizer_Scope_1506 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1506;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 328 '<Block>' Let_Result None
--# scope-start
Var_Expr := Env;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind env Var_Expr
--# expr-start 327 '<FieldAccess .parent_unit_env_helper(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:617
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 326 '<FieldAccess .unit>' Fld /usr/src/packages/BUILD/ada/ast.py:617
--# expr-start 325 '<env_node(<Var env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:617
Env_Node := AST_Envs.Env_Node (Var_Expr); 
--# expr-done 325
      if Env_Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Env_Node);
--# expr-done 326
--# property-call-start AdaNode.parent_unit_env_helper
Fld_1 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env_Helper (Node => Self, Unit => Fld, Env => Var_Expr);
--# end
--# expr-done 327
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1506;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 328
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_43;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_43;
               Finalizer_Scope_1506;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Parent_Unit_Env;
--# end
--# property-start AdaNode.std /usr/src/packages/BUILD/ada/ast.py:629
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 335 '<as_bare_entity(<FieldAccess .item>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 334 '<FieldAccess .item>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 333 '<Cast to LibraryItem>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 332 '<FieldAccess .body>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 331 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 330 '<FieldAccess .root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:630
--# expr-start 329 '<FieldAccess .standard_unit>' Fld /usr/src/packages/BUILD/ada/ast.py:630
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Ada_Node_P_Standard_Unit (Node => Self);
--# expr-done 329
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 330
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 331
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Body;
--# expr-done 332
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 333
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 334
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 335
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std;
--# end
--# property-start AdaNode.std_env /usr/src/packages/BUILD/ada/ast.py:638
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_46;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_46 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_46;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 337 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:639
--# expr-start 336 '<FieldAccess .std>' Fld /usr/src/packages/BUILD/ada/ast.py:639
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std
Fld := Libadalang.Implementation.Ada_Node_P_Std (Node => Self);
--# end
--# expr-done 336
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 337
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_46;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_46;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Env;
--# end
--# property-start AdaNode.std_entity /usr/src/packages/BUILD/ada/ast.py:643
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 341 '<Block>' Let_Result None
--# scope-start
--# expr-start 340 '<FieldAccess .std_entity_implem(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:644
--# expr-start 339 '<FieldAccess .root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:644
--# expr-start 338 '<FieldAccess .unit>' Fld /usr/src/packages/BUILD/ada/ast.py:644
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 338
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 339
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_entity_implem
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity_Implem (Node => Fld_1, Sym => Sym);
--# end
--# expr-done 340
Let_Result := Fld_2; 
--# end
--# expr-done 341
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Entity;
--# end
--# property-start AdaNode.std_entity_implem /usr/src/packages/BUILD/ada/ast.py:650
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Std_Entity_Implem
  (Node : Bare_Ada_Node
      ; Sym : Symbol_Type
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_48;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_48 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_48;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Ada_Node_Ada_Node_P_Std_Entity_Implem,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Ada_Node,
                                As_Bare_Ada_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Sym);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 344 '<Block>' Let_Result None
--# scope-start
--# expr-start 343 '<EnvGet(<FieldAccess .std_env>, <Var sym>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:651
--# expr-start 342 '<FieldAccess .std_env>' Fld /usr/src/packages/BUILD/ada/ast.py:651
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.std_env
Fld := Libadalang.Implementation.Ada_Node_P_Std_Env (Node => Self);
--# end
--# expr-done 342
Env_Get_Result := AST_Envs.Get_First (Self => Fld, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 343
Let_Result := Env_Get_Result; 
--# end
--# expr-done 344
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_48;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity,
                     As_Internal_Entity => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_48;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Std_Entity_Implem;
--# end
--# property-start AdaNode.bool_type /usr/src/packages/BUILD/ada/ast.py:655
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 346 '<FieldAccess .std_entity(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:656
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 345 '<Symbol Boolean>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Boolean); 
--# expr-done 345
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 346
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Bool_Type;
--# end
--# property-start AdaNode.int_type /usr/src/packages/BUILD/ada/ast.py:660
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 348 '<FieldAccess .std_entity(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:661
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 347 '<Symbol Integer>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Integer); 
--# expr-done 347
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 348
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Int_Type;
--# end
--# property-start AdaNode.universal_int_type /usr/src/packages/BUILD/ada/ast.py:665
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 350 '<FieldAccess .std_entity(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:666
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 349 '<Symbol Universal_Int_Type_>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Universal_Int_Type); 
--# expr-done 349
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 350
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Int_Type;
--# end
--# property-start AdaNode.universal_real_type /usr/src/packages/BUILD/ada/ast.py:670
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Sym : Symbol_Type;
Fld : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 352 '<FieldAccess .std_entity(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:671
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 351 '<Symbol Universal_Real_Type_>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Universal_Real_Type); 
--# expr-done 351
--# property-call-start AdaNode.std_entity
Fld := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 352
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Real_Type;
--# end
--# property-start AdaNode.exc_id_type /usr/src/packages/BUILD/ada/ast.py:676
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Exc_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_53;
            procedure Finalizer_Scope_1507;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_53 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_53;
            procedure Finalizer_Scope_1507 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1507;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 360 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:677
--# expr-start 359 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Exception_Id)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:677
--# expr-start 356 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:677
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 355 '<ArrayLiteral>' Array_Lit None
--# expr-start 353 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 353
--# expr-start 354 '<Symbol Exceptions>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exceptions); 
--# expr-done 354
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 355
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 356
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 357 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:677
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 357
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1507;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 358 '<Symbol Exception_Id>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception_Id); 
--# expr-done 358
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 359
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 360
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_53;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_53;
               Finalizer_Scope_1507;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Exc_Id_Type;
--# end
--# property-start AdaNode.task_id_type /usr/src/packages/BUILD/ada/ast.py:686
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Task_Id_Type
  (Node : Bare_Ada_Node
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_54;
            procedure Finalizer_Scope_1508;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_54 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_54;
            procedure Finalizer_Scope_1508 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1508;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 368 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:687
--# expr-start 367 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Task_Id)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:687
--# expr-start 364 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:687
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 363 '<ArrayLiteral>' Array_Lit None
--# expr-start 361 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 361
--# expr-start 362 '<Symbol Task_Identification>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task_Identification); 
--# expr-done 362
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 363
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 364
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 365 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:687
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 365
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1508;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 366 '<Symbol Task_Id>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Task_Id); 
--# expr-done 366
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 367
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 368
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_54;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_54;
               Finalizer_Scope_1508;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Task_Id_Type;
--# end
--# property-start AdaNode.has_with_visibility /usr/src/packages/BUILD/ada/ast.py:696
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_With_Visibility
  (Node : Bare_Ada_Node
      ; Refd_Unit : Internal_Unit
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind refd_unit Refd_Unit
   Property_Result : Boolean;
            procedure Finalizer_Scope_55;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Var_Expr_2 : Internal_Unit;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Bare_Compilation_Unit;
Fld : Internal_Unit;
Fld_1 : Boolean;
Fld_2 : Internal_Unit;
Fld_3 : Bare_Basic_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Unit;
Fld_8 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Unit;
Fld_11 : Bare_Ada_Node;
Fld_12 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_13 : Boolean;
If_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_3 : Boolean;
Fld_14 : Bare_Ada_Node;
Cast_Result_1 : Bare_Compilation_Unit;
Fld_15 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_55 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_55;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 390 '<Block>' Let_Result None
--# scope-start
--# expr-start 370 '<FieldAccess .is_referenced_from(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:704
      if Refd_Unit = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 369 '<FieldAccess .unit>' Fld /usr/src/packages/BUILD/ada/ast.py:704
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 369
Fld_1 := Libadalang.Implementation.Is_Referenced_From (Self => Refd_Unit, Unit => Fld);
--# expr-done 370
if Fld_1 then
   If_Result := True;
else
--# expr-start 375 '<env_node(<FieldAccess .parent_unit_env(...)>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:705
--# expr-start 374 '<FieldAccess .parent_unit_env(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:705
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 373 '<FieldAccess .children_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:710
--# expr-start 372 '<FieldAccess .top_level_decl(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:710
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 371 '<FieldAccess .unit>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:710
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 371
--# property-call-start AdaNode.top_level_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => Fld_2);
--# end
--# expr-done 372
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 373
--# property-call-start AdaNode.parent_unit_env
Fld_5 := Libadalang.Implementation.Ada_Node_P_Parent_Unit_Env (Node => Self, Env => Fld_4);
--# end
--# expr-done 374
Env_Node := AST_Envs.Env_Node (Fld_5); 
--# expr-done 375
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 376 '<FieldAccess .has_with_visibility(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:705
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr, Refd_Unit => Refd_Unit);
--# end
--# expr-done 376
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_3 := True;
else
--# expr-start 380 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:718
--# expr-start 379 '<as_bare_entity(<FieldAccess .top_level_decl(...)>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:718
--# expr-start 378 '<FieldAccess .top_level_decl(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 377 '<FieldAccess .unit>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:719
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 377
--# property-call-start AdaNode.top_level_decl
Fld_8 := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => Fld_7);
--# end
--# expr-done 378
Is_Null := Fld_8 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 379
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 380
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
   --# bind b Var_Expr_1
--# expr-start 386 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:721
--# expr-start 381 '<FieldAccess .is_library_item>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:722
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Var_Expr_1.Node);
--# end
--# expr-done 381
if Fld_13 then
--# expr-start 383 '<FieldAccess .referenced_unit(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:723
--# expr-start 382 '<FieldAccess .defining_name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:723
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 382
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Fld_9.Node, Kind => Unit_Specification);
--# end
--# expr-done 383
Var_Expr_2 := Fld_10;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind u Var_Expr_2
--# expr-start 384 '<FieldAccess .root>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:724
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Var_Expr_2.AST_Root;
--# expr-done 384
Var_Expr_3 := Fld_11;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 385 '<FieldAccess .has_with_visibility(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:724
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_with_visibility
Fld_12 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Var_Expr_3, Refd_Unit => Refd_Unit);
--# end
--# expr-done 385
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
--# expr-done 386
Scope_Result_3 := If_Result_2;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_3 := Result_Var_3;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
--# expr-start 388 '<Cast to CompilationUnit>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:737
--# expr-start 387 '<FieldAccess .root>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:737
      if Refd_Unit = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Refd_Unit.AST_Root;
--# expr-done 387
   if Fld_14 = null
      or else Fld_14.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Fld_14;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 388
Var_Expr_4 := Cast_Result_1;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 389 '<FieldAccess .is_text_io_child>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:737
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_text_io_child
Fld_15 := Libadalang.Implementation.Compilation_Unit_P_Is_Text_Io_Child (Node => Var_Expr_4);
--# end
--# expr-done 389
Scope_Result_4 := Fld_15;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
   If_Result_4 := Result_Var_4;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 390
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_55;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_55;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Has_With_Visibility;
--# end
--# property-start AdaNode.has_visibility /usr/src/packages/BUILD/ada/ast.py:741
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Has_Visibility
  (Node : Bare_Ada_Node
      ; Other_Entity : Internal_Entity
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind other_entity Other_Entity
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Generic_Package_Instantiation;
Var_Expr_1 : Internal_Entity_Package_Renaming_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Instantiation;
Fld : Internal_Entity_Info;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Package_Renaming_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Fld_6 : Internal_Unit;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 403 '<Block>' Let_Result None
--# scope-start
--# expr-start 391 '<Cast to GenericPackageInstantiation>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:745
Cast_Expr := Other_Entity; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 391
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Generic_Package_Instantiation then
   --# scope-start
--# expr-start 393 '<FieldAccess .from_rebound>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:745
--# expr-start 392 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:745
Fld := Var_Expr.Info;
--# expr-done 392
Fld_1 := Fld.From_Rebound;
--# expr-done 393
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   If_Result := True;
else
--# expr-start 394 '<Cast to PackageRenamingDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:747
Cast_Expr_1 := Other_Entity; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 394
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Package_Renaming_Decl then
   --# scope-start
--# expr-start 396 '<FieldAccess .from_rebound>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:747
--# expr-start 395 '<FieldAccess .info>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:747
Fld_2 := Var_Expr_1.Info;
--# expr-done 395
Fld_3 := Fld_2.From_Rebound;
--# expr-done 396
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 399 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:750
--# expr-start 398 '<FieldAccess .is_compilation_unit_root>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:750
--# expr-start 397 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:750
Cast_Expr_2 := Other_Entity; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 397
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Cast_Result_2.Node);
--# end
--# expr-done 398
Not_Val := not (Fld_4); 
--# expr-done 399
   If_Result_1 := Not_Val;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 402 '<FieldAccess .has_with_visibility(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:753
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 401 '<FieldAccess .unit>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:753
--# expr-start 400 '<FieldAccess .node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:753
Fld_5 := Other_Entity.Node;
--# expr-done 400
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Unit (Node => Fld_5);
--# expr-done 401
--# property-call-start AdaNode.has_with_visibility
Fld_7 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_6);
--# end
--# expr-done 402
   If_Result_2 := Fld_7;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 403
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Has_Visibility;
--# end
--# property-start AdaNode.resolve_generic_actual /usr/src/packages/BUILD/ada/ast.py:757
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Resolve_Generic_Actual
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1525;
      Match_0 : Internal_Entity_Anonymous_Expr_Decl;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Scope_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Dyn_Var_Bind_Result : Internal_Entity;
Result_Var : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Anonymous_Expr_Decl;
Cast_Expr : Internal_Entity_Anonymous_Expr_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_1 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Match_Result : Internal_Entity;
Let_Result_3 : Internal_Entity;
            procedure Finalizer_Scope_1525 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1525;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 411 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 410 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:761
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Expr_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Expr_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind aod Match_0
Cast_Expr := Match_0; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind n Match_1
--# expr-start 405 '<Cast to AdaNode.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:767
--# expr-start 404 '<FieldAccess .name_designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:767
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 404
      Cast_Result := Create_Internal_Entity
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 405
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 409 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <at(<FieldAccess .env_elements>, 0)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:769
--# expr-start 406 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:769
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 406
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 408 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:769
--# expr-start 407 '<FieldAccess .env_elements>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:769
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_2 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Match_1.Node, Env => Bound_Env, E_Info => Match_1.Info);
--# end
--# expr-done 407
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 408
Dyn_Var_Bind_Result := Get_Result; 
--# expr-done 409
   Result_Var := Dyn_Var_Bind_Result;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_1525;
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 410
Let_Result_3 := Match_Result; 
--# end
--# expr-done 411
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1525;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Resolve_Generic_Actual;
--# end
--# property-start AdaNode.top_level_use_package_clauses /usr/src/packages/BUILD/ada/ast.py:776
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_58;
            procedure Finalizer_Scope_1530;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_11 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Package_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_12 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_58 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_58;
            procedure Finalizer_Scope_1530 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1530;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 423 '<Block>' Let_Result None
--# scope-start
--# expr-start 422 '<mapcat>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:783
--# expr-start 417 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:783
--# expr-start 415 '<FieldAccess .prelude>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:783
--# expr-start 414 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:783
--# expr-start 413 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:783
--# expr-start 412 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:783
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 412
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 413
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 414
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 415
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_11 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_11 := Untyped_Item_11; 
   --# scope-start
               --# bind p Item_11
               --# expr-start 416 '<IsA UsePackageClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:785
Is_A := Item_11 /= null 
and then Item_11.Kind in Ada_Use_Package_Clause_Range; 
--# expr-done 416
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_11;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 417
declare
   Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_10 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_10
      --# expr-start 421 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:787
--# expr-start 419 '<FieldAccess .packages>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:787
--# expr-start 418 '<Cast to UsePackageClause>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:787
Cast_Expr := Item_10; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 418
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Use_Package_Clause_F_Packages;
--# expr-done 419
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_12 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_12 := Untyped_Item_12; 
   --# scope-start
               --# bind n Item_12
      --# expr-start 420 '<Cast to AdaNode>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:788
Cast_Expr_1 := Item_12; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 420
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 421
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1530;
         end loop;
      end;
      Map_Result_2 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_2_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 422
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 423
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_58;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_58;
               Finalizer_Scope_1530;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Use_Package_Clauses;
--# end
--# property-start AdaNode.top_level_use_type_clauses /usr/src/packages/BUILD/ada/ast.py:794
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Type_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_59;
            procedure Finalizer_Scope_1533;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Item_14 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Type_Clause;
Fld_3 : Bare_Name_List;
Cast_Expr_1 : Bare_Name;
Cast_Result_2 : Bare_Ada_Node;
Item_15 : Bare_Name;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Map_Result_2 : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_59 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_59;
            procedure Finalizer_Scope_1533 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1533;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 435 '<Block>' Let_Result None
--# scope-start
--# expr-start 434 '<mapcat>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:801
--# expr-start 429 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:801
--# expr-start 427 '<FieldAccess .prelude>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:801
--# expr-start 426 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:801
--# expr-start 425 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:801
--# expr-start 424 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:801
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 424
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 425
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 426
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Compilation_Unit_F_Prelude;
--# expr-done 427
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_14 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_14 := Untyped_Item_14; 
   --# scope-start
               --# bind p Item_14
               --# expr-start 428 '<IsA UseTypeClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:803
Is_A := Item_14 /= null 
and then Item_14.Kind in Ada_Use_Type_Clause_Range; 
--# expr-done 428
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_14;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 429
declare
   Map_Result_2_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Map_Result;
      begin
         for Item_13 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_13
      --# expr-start 433 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:805
--# expr-start 431 '<FieldAccess .types>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:805
--# expr-start 430 '<Cast to UseTypeClause>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:805
Cast_Expr := Item_13; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Type_Clause_Range
   then
      Cast_Result_1 := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 430
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Use_Type_Clause_F_Types;
--# expr-done 431
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_3;
      begin
         for Untyped_Item_15 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_15 := Untyped_Item_15; 
   --# scope-start
               --# bind n Item_15
      --# expr-start 432 '<Cast to AdaNode>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:806
Cast_Expr_1 := Item_15; 
      Cast_Result_2 := Cast_Expr_1;
--# expr-done 432
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result_2;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 433
         for Item_To_Append of
               Map_Result_1.Items
         loop
            Bare_Ada_Node_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1533;
         end loop;
      end;
      Map_Result_2 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_2_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 434
Let_Result := Map_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 435
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_59;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_59;
               Finalizer_Scope_1533;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Use_Type_Clauses;
--# end
--# property-start AdaNode.top_level_use_clauses /usr/src/packages/BUILD/ada/ast.py:812
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Use_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Use_Clause_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_60;
      Block_Var : Bare_Compilation_Unit;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Use_Clause;
Item_16 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Bare_Use_Clause_Array_Access;
Let_Result : Bare_Use_Clause_Array_Access;
            procedure Finalizer_Scope_60 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_60;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 443 '<Block>' Let_Result None
--# scope-start
--# expr-start 438 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:817
--# expr-start 437 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:817
--# expr-start 436 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:817
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 436
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 437
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 438
Block_Var := Cast_Result; 
--# bind cu Block_Var
--# expr-start 442 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:818
--# expr-start 439 '<FieldAccess .prelude>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:818
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Block_Var.Compilation_Unit_F_Prelude;
--# expr-done 439
declare
   Map_Result_Vec : Bare_Use_Clause_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Bare_Use_Clause_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_16 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_16 := Untyped_Item_16; 
   --# scope-start
               --# bind p Item_16
               --# expr-start 440 '<IsA UseClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:820
Is_A := Item_16 /= null 
and then Item_16.Kind in Ada_Use_Clause; 
--# expr-done 440
               if Is_A then
      --# expr-start 441 '<Cast to UseClause>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:819
Cast_Expr := Item_16; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Use_Clause
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 441
         declare
            Item_To_Append : constant Bare_Use_Clause :=
               Cast_Result_1;
         begin
            Bare_Use_Clause_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Use_Clause_Array
        (Items_Count => Natural (Bare_Use_Clause_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Use_Clause_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Use_Clause_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Use_Clause_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 442
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 443
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_60;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_60;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Use_Clauses;
--# end
--# property-start AdaNode.top_level_with_package_clauses /usr/src/packages/BUILD/ada/ast.py:824
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_With_Package_Clauses
  (Node : Bare_Ada_Node
  )
   return Bare_Name_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_61;
            procedure Finalizer_Scope_1536;
      Var_Expr : Bare_With_Clause;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Ada_Node_List;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_With_Clause;
Fld_2 : Bare_Name_List;
Scope_Result : Bare_Name_List;
Result_Var : Bare_Name_List;
Item_272 : Bare_Name;
Map_Result : Bare_Name_Array_Access;
Item_17 : Bare_Ada_Node;
Map_Result_1 : Bare_Name_Array_Access;
Let_Result : Bare_Name_Array_Access;
            procedure Finalizer_Scope_61 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_61;
            procedure Finalizer_Scope_1536 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1536;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 450 '<Block>' Let_Result None
--# scope-start
--# expr-start 449 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:830
--# expr-start 445 '<FieldAccess .prelude>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:830
--# expr-start 444 '<FieldAccess .enclosing_compilation_unit>' Fld /usr/src/packages/BUILD/ada/ast.py:830
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 444
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Compilation_Unit_F_Prelude;
--# expr-done 445
declare
   Map_Result_1_Vec : Bare_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result_1 := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_17 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_17 := Untyped_Item_17; 
   --# scope-start
               --# bind p Item_17
      --# expr-start 448 '<take_while>' Map_Result None
--# expr-start 446 '<Cast to WithClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:832
Cast_Expr := Item_17; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_With_Clause_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 446
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 447 '<FieldAccess .packages>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:832
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.With_Clause_F_Packages;
--# expr-done 447
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
declare
   Map_Result_Vec : Bare_Name_Vectors.Vector;
begin
      if Result_Var = null then
         Map_Result := Create_Bare_Name_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Result_Var;
      begin
         for Untyped_Item_272 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_272 := Untyped_Item_272; 
   --# scope-start
         declare
            Item_To_Append : constant Bare_Name :=
               Item_272;
         begin
            Bare_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Name_Array
        (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Name_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 448
         for Item_To_Append of
               Map_Result.Items
         loop
            Bare_Name_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1536;
         end loop;
      end;
      Map_Result_1 := Create_Bare_Name_Array
        (Items_Count => Natural (Bare_Name_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Name_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Name_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Name_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 449
Let_Result := Map_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 450
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_61;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_61;
               Finalizer_Scope_1536;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_With_Package_Clauses;
--# end
--# property-start AdaNode.use_clauses_in_spec_of_subp_body /usr/src/packages/BUILD/ada/ast.py:836
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_62;
            procedure Finalizer_Scope_1539;
            procedure Finalizer_Scope_1540;
      Block_Var : Internal_Entity_Base_Subp_Body;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Base_Subp_Body;
As_Entity : Internal_Entity_Base_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Body;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_1 : Bare_Use_Clause_Array_Access;
Scope_Result : Bare_Use_Clause_Array_Access;
Result_Var : Bare_Use_Clause_Array_Access;
As_Entity_1 : Internal_Entity_Use_Clause;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Use_Clause;
Fld_2 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_62 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_62;
            procedure Finalizer_Scope_1539 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1539;
            procedure Finalizer_Scope_1540 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1540;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 460 '<Block>' Let_Result None
--# scope-start
--# expr-start 452 '<as_bare_entity(<Cast to BaseSubpBody>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:841
--# expr-start 451 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:841
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 451
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Body;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 452
Block_Var := If_Result; 
--# bind body Block_Var
--# expr-start 454 '<bind(<DynamicVariable imprecise_fallback>, True, <FieldAccess .decl_part>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:842
Bound_Imprecise_Fallback := True; 
--# expr-start 453 '<FieldAccess .decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:842
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Block_Var.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 453
Dyn_Var_Bind_Result := Fld; 
--# expr-done 454
Block_Var_1 := Dyn_Var_Bind_Result; 
--# bind decl Block_Var_1
--# expr-start 459 '<env_group(<map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:843
--# expr-start 458 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:843
Var_Expr := Block_Var_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 455 '<FieldAccess .top_level_use_clauses>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:843
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_clauses
Fld_1 := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Clauses (Node => Var_Expr.Node);
--# end
--# expr-done 455
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1539;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Use_Clause_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Use_Clause_Array_Access := Result_Var;
      begin
         for Item_18 of
               Collection.Items
         loop
   --# scope-start
               --# bind clause Item_18
      --# expr-start 457 '<FieldAccess .used_envs>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:844
--# expr-start 456 '<as_bare_entity(<Var clause>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:844
Is_Null_1 := Item_18 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Use_Clause;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Item_18); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 456
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UseClause.used_envs
Fld_2 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 457
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1540;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 458
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 459
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 460
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_62;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_62;
               Finalizer_Scope_1539;
               Finalizer_Scope_1540;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body;
--# end
--# property-start AdaNode.use_clauses_in_generic_formal_part /usr/src/packages/BUILD/ada/ast.py:848
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_63;
            procedure Finalizer_Scope_1542;
      Block_Var : Internal_Entity_Generic_Decl;
Var_Expr : Internal_Entity_Body_Node;
Var_Expr_1 : Internal_Entity_Generic_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Internal_Entity_Generic_Formal_Part;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_63 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_63;
            procedure Finalizer_Scope_1542 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1542;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 467 '<Block>' Let_Result None
--# scope-start
--# expr-start 462 '<Cast to Body>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:856
--# expr-start 461 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:856
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 461
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 462
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 463 '<FieldAccess .safe_generic_decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:856
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 463
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Block_Var := Result_Var; 
--# bind gen_decl Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 465 '<FieldAccess .use_clauses_envs>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:858
--# expr-start 464 '<FieldAccess .formal_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:858
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_1.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_1.Info);
--# expr-done 464
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericFormalPart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Generic_Formal_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 465
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1542;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 466 '<FieldAccess .empty_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:859
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 466
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 467
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_63;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_63;
               Finalizer_Scope_1542;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Use_Clauses_In_Generic_Formal_Part;
--# end
--# property-start AdaNode.nested_generic_formal_part /usr/src/packages/BUILD/ada/ast.py:863
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Nested_Generic_Formal_Part
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_64;
            procedure Finalizer_Scope_1544;
      Block_Var : Internal_Entity_Generic_Decl;
Var_Expr : Internal_Entity_Body_Node;
Var_Expr_1 : Internal_Entity_Generic_Decl;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_64 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_64;
            procedure Finalizer_Scope_1544 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1544;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 474 '<Block>' Let_Result None
--# scope-start
--# expr-start 469 '<Cast to Body>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:877
--# expr-start 468 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:877
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 468
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 469
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 470 '<FieldAccess .safe_generic_decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:877
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.safe_generic_decl_part
Fld := Libadalang.Implementation.Body_Node_P_Safe_Generic_Decl_Part (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 470
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Block_Var := Result_Var; 
--# bind gen_decl Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind gd Var_Expr_1
--# expr-start 472 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:879
--# expr-start 471 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:879
Fld_1 := Var_Expr_1.Node;
--# expr-done 471
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 472
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1544;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 473 '<FieldAccess .empty_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:880
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.empty_env
Fld_3 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 473
   Result_Var_1 := Fld_3;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 474
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_64;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_64;
               Finalizer_Scope_1544;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Nested_Generic_Formal_Part;
--# end
--# property-start AdaNode.is_package /usr/src/packages/BUILD/ada/ast.py:884
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Is_Package
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 476 '<Block>' Let_Result None
--# scope-start
--# expr-start 475 '<IsA PackageDecl, PackageBody, GenericPackageInstantiation, PackageRenamingDecl, GenericPackageDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:888
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Package_Body_Range | Ada_Generic_Package_Instantiation_Range | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range; 
--# expr-done 475
Let_Result := Is_A; 
--# end
--# expr-done 476
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Is_Package;
--# end
--# property-start AdaNode.default_initial_env /usr/src/packages/BUILD/ada/ast.py:892
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Default_Initial_Env
  (Node : Bare_Ada_Node
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_66;
            procedure Finalizer_Scope_1545;
      Var_Expr : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_66 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_66;
            procedure Finalizer_Scope_1545 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1545;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 480 '<Block>' Let_Result None
--# scope-start
--# expr-start 477 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:897
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 477
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 478 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:897
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 478
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1545;
   Result_Var := Scope_Result;
else
   --# expr-start 479 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:898
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 479
   Result_Var := Fld_2;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 480
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_66;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_66;
               Finalizer_Scope_1545;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Default_Initial_Env;
--# end
--# property-start AdaNode.initial_env /usr/src/packages/BUILD/ada/ast.py:901
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Initial_Env
  (Node : Bare_Ada_Node
      ; Scope : Lexical_Env
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind scope Scope
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_67;
            procedure Finalizer_Scope_1546;
      Var_Expr : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_67 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_67;
            procedure Finalizer_Scope_1546 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1546;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 481 '<Block>' Let_Result None
--# scope-start
Var_Expr := Scope;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind s Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1546;
   Result_Var := Scope_Result;
else
   Result_Var := Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 481
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_67;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_67;
               Finalizer_Scope_1546;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Initial_Env;
--# end
--# property-start AdaNode.top_level_decl /usr/src/packages/BUILD/ada/ast.py:909
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Top_Level_Decl
  (Node : Bare_Ada_Node
      ; Unit : Internal_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind unit Unit
   Property_Result : Bare_Basic_Decl;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Internal_Unit;
Fld : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Let_Result : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 485 '<Block>' Let_Result None
--# scope-start
Var_Expr_1 := Unit;
if Var_Expr_1 /= null then
   --# scope-start
--# expr-start 482 '<FieldAccess .root>' Fld /usr/src/packages/BUILD/ada/ast.py:914
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Var_Expr_1.AST_Root;
--# expr-done 482
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 484 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:914
--# expr-start 483 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:914
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 483
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Cast_Result);
--# end
--# expr-done 484
Scope_Result_1 := Fld_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 485
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Top_Level_Decl;
--# end
--# property-start AdaNode.unpack_formals /usr/src/packages/BUILD/ada/ast.py:917
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_69;
      Fld : Internal_Unit;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Single_Formal_Array_Access;
Let_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_69 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_69;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 489 '<Block>' Let_Result None
--# scope-start
--# expr-start 488 '<FieldAccess .unpack_formals_impl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:921
--# expr-start 487 '<FieldAccess .root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:921
--# expr-start 486 '<FieldAccess .unit>' Fld /usr/src/packages/BUILD/ada/ast.py:921
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 486
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.AST_Root;
--# expr-done 487
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals_impl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals_Impl (Node => Fld_1, Formal_Params => Formal_Params);
--# end
--# expr-done 488
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 489
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_69;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_69;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Unpack_Formals;
--# end
--# property-start AdaNode.unpack_formals_impl /usr/src/packages/BUILD/ada/ast.py:924
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Unpack_Formals_Impl
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_70;
            procedure Finalizer_Scope_1547;
      Fld : Internal_Entity_Defining_Name_Array_Access;
New_Struct : Internal_Single_Formal;
Map_Result : Internal_Single_Formal_Array_Access;
Map_Result_1 : Internal_Single_Formal_Array_Access;
Let_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_70 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_70;
            procedure Finalizer_Scope_1547 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1547;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 494 '<Block>' Let_Result None
--# scope-start
--# expr-start 493 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:925
declare
   Map_Result_1_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Formal_Params;
      begin
         for Item_19 of
               Collection.Items
         loop
   --# scope-start
               --# bind spec Item_19
      --# expr-start 492 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:926
--# expr-start 490 '<FieldAccess .defining_names>' Fld /usr/src/packages/BUILD/ada/ast.py:926
      if Item_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Item_19.Node, E_Info => Item_19.Info);
--# end
--# expr-done 490
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld;
      begin
         for Item_20 of
               Collection.Items
         loop
   --# scope-start
               --# bind id Item_20
      --# expr-start 491 '<New InternalSingleFormal>' New_Struct /usr/src/packages/BUILD/ada/ast.py:926
New_Struct := (Name => Item_20, Spec => Item_19); 
--# expr-done 491
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               New_Struct;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 492
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Single_Formal_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1547;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 493
Let_Result := Map_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 494
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_70;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_70;
               Finalizer_Scope_1547;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Unpack_Formals_Impl;
--# end
--# property-start AdaNode.match_formals /usr/src/packages/BUILD/ada/ast.py:932
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Match_Formals
  (Node : Bare_Ada_Node
      ; Formal_Params : Internal_Entity_Base_Formal_Param_Decl_Array_Access
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind formal_params Formal_Params
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_71;
            procedure Finalizer_Scope_1550;
            procedure Finalizer_Scope_1556;
      Block_Var : Internal_Single_Formal_Array_Access;
Var_Expr : Internal_Entity_Assoc_List;
Idx : Integer;
Var_Expr_1 : Internal_Single_Formal;
Var_Expr_2 : Bare_Base_Id;
Var_Expr_3 : Internal_Single_Formal;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Single_Actual_Array_Access;
Arith_Result : Integer;
If_Result : Integer;
Get_Result : Internal_Single_Formal;
New_Struct : Internal_Param_Match;
Scope_Result : Internal_Param_Match;
Result_Var : Internal_Param_Match;
Let_Result : Internal_Param_Match;
Fld_2 : Bare_Base_Id;
Fld_3 : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Name;
Fld_4 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
New_Struct_1 : Internal_Param_Match;
Scope_Result_1 : Internal_Param_Match;
Result_Var_1 : Internal_Param_Match;
Scope_Result_2 : Internal_Param_Match;
Result_Var_2 : Internal_Param_Match;
Fld_5 : Bare_Base_Id;
Is_Null : Boolean;
If_Result_1 : Internal_Param_Match;
I : Integer;
Map_Result_1 : Internal_Param_Match_Array_Access;
Scope_Result_3 : Internal_Param_Match_Array_Access;
Result_Var_3 : Internal_Param_Match_Array_Access;
Let_Result_1 : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_71 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_71;
            procedure Finalizer_Scope_1550 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1550;
            procedure Finalizer_Scope_1556 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1556;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 512 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 495 '<FieldAccess .unpack_formals(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:940
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals
Fld := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Formal_Params);
--# end
--# expr-done 495
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind unpacked_formals Block_Var
Var_Expr := Params;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 511 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:942
--# expr-start 496 '<FieldAccess .unpacked_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:942
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 496
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
         I := 0;
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_1;
      begin
         for Item_21 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_21
               --# bind i I
      --# expr-start 510 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:942
--# expr-start 498 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:943
--# expr-start 497 '<FieldAccess .name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:943
Fld_5 := Item_21.Name;
--# expr-done 497
Is_Null := Fld_5 = null; 
--# expr-done 498
if Is_Null then
   --# expr-start 503 '<Let idx>' Let_Result /usr/src/packages/BUILD/ada/ast.py:945
--# scope-start
--# expr-start 500 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:946
if Is_Dottable_Subp then
   --# expr-start 499 '<Op +>' Arith_Result /usr/src/packages/BUILD/ada/ast.py:946
Arith_Result := (I + 1); 
--# expr-done 499
   If_Result := Arith_Result;
else
   If_Result := I;
end if;
--# expr-done 500
Idx := If_Result; 
--# bind idx Idx
--# expr-start 501 '<at(<Var unpacked_formals>, <Var idx>)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:949
Get_Result := Get (Block_Var, Idx, True); 
--# expr-done 501
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Single_Formal then
   --# scope-start
   --# bind sp Var_Expr_1
--# expr-start 502 '<New InternalParamMatch>' New_Struct /usr/src/packages/BUILD/ada/ast.py:950
New_Struct := (Actual => Item_21, Formal => Var_Expr_1, Has_Matched => True); 
--# expr-done 502
Scope_Result := New_Struct;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Param_Match;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 503
   If_Result_1 := Let_Result;
else
--# expr-start 504 '<FieldAccess .name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:960
Fld_2 := Item_21.Name;
--# expr-done 504
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind id Var_Expr_2
--# expr-start 508 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:961
--# expr-start 507 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:961
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
      begin
         for Item_22 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_22
               --# expr-start 506 '<FieldAccess .matches(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:961
--# expr-start 505 '<FieldAccess .name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:961
Fld_3 := Item_22.Name;
--# expr-done 505
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Var_Expr_2; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.matches
Fld_4 := Libadalang.Implementation.Name_P_Matches (Node => Fld_3.Node, N => Cast_Result);
--# end
--# expr-done 506
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_22;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 507
Get_Result_1 := Get (Map_Result, 0, True); 
--# expr-done 508
Var_Expr_3 := Get_Result_1;
if Var_Expr_3 /= No_Single_Formal then
   --# scope-start
   --# bind sp Var_Expr_3
--# expr-start 509 '<New InternalParamMatch>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:962
New_Struct_1 := (Actual => Item_21, Formal => Var_Expr_3, Has_Matched => True); 
--# expr-done 509
Scope_Result_1 := New_Struct_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Param_Match;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_1556;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Param_Match;
end if;
   If_Result_1 := Result_Var_2;
end if;
--# expr-done 510
         declare
            Item_To_Append : constant Internal_Param_Match :=
               If_Result_1;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 511
Scope_Result_3 := Map_Result_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1550;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 512
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_71;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_71;
               Finalizer_Scope_1550;
               Finalizer_Scope_1556;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Match_Formals;
--# end
--# property-start AdaNode.choice_match /usr/src/packages/BUILD/ada/ast.py:971
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Choice_Match
  (Node : Bare_Ada_Node
      ; Value : Big_Integer_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind value Value
   Property_Result : Boolean;
            procedure Finalizer_Scope_1559;
            procedure Finalizer_Scope_1560;
            procedure Finalizer_Scope_1562;
            procedure Finalizer_Scope_1565;
            procedure Finalizer_Scope_1566;
      Match_0 : Internal_Entity_Bin_Op;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Subtype_Indication;
Match_3 : Internal_Entity_Expr;
Match_4 : Internal_Entity_Others_Designator;
Match_5 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Discrete_Range;
Edr : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Var_Expr_2 : Internal_Discrete_Range;
Edr_1 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld : Internal_Entity_Expr;
Fld_1 : Big_Integer_Type;
Comp_Result : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Big_Integer_Type;
Comp_Result_1 : Boolean;
If_Result : Boolean;
Fld_4 : Big_Integer_Type;
Is_Equal : Boolean;
Fld_5 : Internal_Entity_Op;
Is_A : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Discrete_Range;
Fld_8 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_9 : Big_Integer_Type;
Comp_Result_2 : Boolean;
Fld_10 : Big_Integer_Type;
Comp_Result_3 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Bound_Origin : Bare_Ada_Node;
Bound_Imprecise_Fallback : Boolean;
Fld_11 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Dyn_Var_Bind_Result_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_1 : Boolean;
Fld_12 : Big_Integer_Type;
Is_Equal_1 : Boolean;
Result_Var_1 : Boolean;
Fld_13 : Internal_Discrete_Range;
Fld_14 : Internal_Eval_Discrete_Range := No_Eval_Discrete_Range;
Fld_15 : Big_Integer_Type;
Comp_Result_4 : Boolean;
Fld_16 : Big_Integer_Type;
Comp_Result_5 : Boolean;
If_Result_4 : Boolean;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Bound_Origin_1 : Bare_Ada_Node;
Bound_Imprecise_Fallback_1 : Boolean;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
Dyn_Var_Bind_Result_3 : Boolean;
If_Result_5 : Boolean;
Fld_19 : Big_Integer_Type;
Is_Equal_2 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Bin_Op;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_2 : Internal_Entity_Subtype_Indication;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_5 : Internal_Entity;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Match_Result : Boolean;
Let_Result_8 : Boolean;
            procedure Finalizer_Scope_1559 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1559;
            procedure Finalizer_Scope_1560 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1560;
            procedure Finalizer_Scope_1562 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Edr);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1562;
            procedure Finalizer_Scope_1565 is
            begin
                     Dec_Ref (Fld_14);
                     Dec_Ref (Edr_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_1565;
            procedure Finalizer_Scope_1566 is
            begin
                     Dec_Ref (Fld_19);
            end Finalizer_Scope_1566;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 545 '<Block>' Let_Result_8 None
--# scope-start
--# expr-start 544 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:982
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bo Match_0
--# expr-start 521 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:986
--# expr-start 514 '<IsA Op.DoubleDot>' Is_A /usr/src/packages/BUILD/ada/ast.py:988
--# expr-start 513 '<FieldAccess .op>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:988
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Op (Node => Match_0.Node.Bin_Op_F_Op, Info => Match_0.Info);
--# expr-done 513
Is_A := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Op_Double_Dot_Range; 
--# expr-done 514
if Is_A then
--# expr-start 516 '<FieldAccess .eval_as_int>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:990
--# expr-start 515 '<FieldAccess .left>' Fld /usr/src/packages/BUILD/ada/ast.py:990
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Left, Info => Match_0.Info);
--# expr-done 515
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_1 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 516
Comp_Result := Value >= Fld_1; 
if Comp_Result then
--# expr-start 518 '<FieldAccess .eval_as_int>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:991
--# expr-start 517 '<FieldAccess .right>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:991
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Match_0.Node.Bin_Op_F_Right, Info => Match_0.Info);
--# expr-done 517
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_3 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 518
Comp_Result_1 := Value <= Fld_3; 
   If_Result := Comp_Result_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
else
   --# expr-start 520 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:993
--# expr-start 519 '<FieldAccess .eval_as_int>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:993
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_4 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 519
Is_Equal := Equivalent (Value, Fld_4); 
--# expr-done 520
   If_Result_1 := Is_Equal;
end if;
--# expr-done 521
Let_Result_2 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
      Finalizer_Scope_1559;
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 522 '<FieldAccess .name_designated_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:998
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 522
Var_Expr := Fld_6;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 523 '<FieldAccess .discrete_range>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:999
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 523
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_1
--# expr-start 527 '<Let edr>' Let_Result /usr/src/packages/BUILD/ada/ast.py:1000
--# scope-start
--# expr-start 524 '<FieldAccess .eval_discrete_range(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_8 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_1);
--# end
--# expr-done 524
Edr := Fld_8; Inc_Ref (Edr);
--# bind edr Edr
--# expr-start 525 '<FieldAccess .low_bound>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1002
Fld_9 := Edr.Low_Bound;
Inc_Ref (Fld_9);
--# expr-done 525
Comp_Result_2 := Value >= Fld_9; 
if Comp_Result_2 then
--# expr-start 526 '<FieldAccess .high_bound>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:1002
Fld_10 := Edr.High_Bound;
Inc_Ref (Fld_10);
--# expr-done 526
Comp_Result_3 := Value <= Fld_10; 
   If_Result_2 := Comp_Result_3;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 527
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_1562;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
if Result_Var then
   --# expr-start 530 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:1006
Bound_Origin := Self; 
--# expr-start 529 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:1007
Bound_Imprecise_Fallback := False; 
--# expr-start 528 '<FieldAccess .satisfies_type_predicates(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:1008
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_11 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Var_Expr.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 528
Dyn_Var_Bind_Result := Fld_11; 
--# expr-done 529
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 530
   If_Result_3 := Dyn_Var_Bind_Result_1;
else
   If_Result_3 := False;
end if;
Scope_Result_1 := If_Result_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 532 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:1011
--# expr-start 531 '<FieldAccess .eval_as_int>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:1011
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_12 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 531
Is_Equal_1 := Equivalent (Value, Fld_12); 
--# expr-done 532
   Result_Var_1 := Is_Equal_1;
end if;
Let_Result_3 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
      Finalizer_Scope_1560;
            Match_Result := Scope_Result_4; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind st Match_2
--# expr-start 533 '<FieldAccess .discrete_range>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:1015
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_13 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 533
Var_Expr_2 := Fld_13;
if Var_Expr_2 /= No_Discrete_Range then
   --# scope-start
   --# bind dr Var_Expr_2
--# expr-start 537 '<Let edr>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:1016
--# scope-start
--# expr-start 534 '<FieldAccess .eval_discrete_range(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:1017
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.eval_discrete_range
Fld_14 := Libadalang.Implementation.Ada_Node_P_Eval_Discrete_Range (Node => Self, Dr => Var_Expr_2);
--# end
--# expr-done 534
Edr_1 := Fld_14; Inc_Ref (Edr_1);
--# bind edr Edr_1
--# expr-start 535 '<FieldAccess .low_bound>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:1018
Fld_15 := Edr_1.Low_Bound;
Inc_Ref (Fld_15);
--# expr-done 535
Comp_Result_4 := Value >= Fld_15; 
if Comp_Result_4 then
--# expr-start 536 '<FieldAccess .high_bound>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:1018
Fld_16 := Edr_1.High_Bound;
Inc_Ref (Fld_16);
--# expr-done 536
Comp_Result_5 := Value <= Fld_16; 
   If_Result_4 := Comp_Result_5;
else
   If_Result_4 := False;
end if;
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 537
Scope_Result_2 := Let_Result_1;
   --# end
      Finalizer_Scope_1565;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
if Result_Var_2 then
   --# expr-start 541 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>)>' Dyn_Var_Bind_Result_3 /usr/src/packages/BUILD/ada/ast.py:1022
Bound_Origin_1 := Self; 
--# expr-start 540 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .satisfies_type_predicates(...)>)>' Dyn_Var_Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:1023
Bound_Imprecise_Fallback_1 := False; 
--# expr-start 539 '<FieldAccess .satisfies_type_predicates(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:1024
--# expr-start 538 '<FieldAccess .designated_type>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:1024
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_17 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_2.Node, Origin => Bound_Origin_1, E_Info => Match_2.Info);
--# end
--# expr-done 538
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_18 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Fld_17.Node, Value => Value, Imprecise_Fallback => Bound_Imprecise_Fallback_1, Origin => Bound_Origin_1, E_Info => Fld_17.Info);
--# end
--# expr-done 539
Dyn_Var_Bind_Result_2 := Fld_18; 
--# expr-done 540
Dyn_Var_Bind_Result_3 := Dyn_Var_Bind_Result_2; 
--# expr-done 541
   If_Result_5 := Dyn_Var_Bind_Result_3;
else
   If_Result_5 := False;
end if;
Let_Result_4 := If_Result_5; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind e Match_3
--# expr-start 543 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:1029
--# expr-start 542 '<FieldAccess .eval_as_int>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:1029
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_19 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 542
Is_Equal_2 := Equivalent (Value, Fld_19); 
--# expr-done 543
Let_Result_5 := Is_Equal_2; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
      Finalizer_Scope_1566;
            Match_Result := Scope_Result_6; 
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_6 := True; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_7 := False; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 544
Let_Result_8 := Match_Result; 
--# end
--# expr-done 545
      Property_Result := Let_Result_8;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1559;
               Finalizer_Scope_1560;
               Finalizer_Scope_1562;
               Finalizer_Scope_1565;
               Finalizer_Scope_1566;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Choice_Match;
--# end
--# property-start AdaNode.gnat_xref /usr/src/packages/BUILD/ada/ast.py:1038
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Gnat_Xref
  (Node : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity :=
     Internal_Entity'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1582;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Boolean;
Var_Expr_8 : Internal_Entity_Defining_Name;
Var_Expr_9 : Internal_Entity_Base_Type_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Name;
Dbd : Internal_Entity_Basic_Decl;
Var_Expr_12 : Internal_Entity_Generic_Instantiation;
Var_Expr_13 : Internal_Entity_Defining_Name;
Var_Expr_14 : Internal_Entity_Basic_Decl;
Var_Expr_15 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Bound_Origin : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_2 : Internal_Entity;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_5 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_7 : Internal_Entity;
Is_A_4 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Defining_Name;
Is_A_5 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Abstract_Subp_Decl;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Internal_Entity_Defining_Name;
Is_A_6 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Basic_Subp_Decl;
Fld_13 : Internal_Entity_Base_Subp_Spec;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Scope_Result_7 : Internal_Entity_Defining_Name;
Fld_18 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Scope_Result_8 : Internal_Entity_Defining_Name;
Result_Var_8 : Internal_Entity_Defining_Name;
Scope_Result_9 : Internal_Entity_Defining_Name;
Result_Var_9 : Internal_Entity_Defining_Name;
Cast_Expr_3 : Internal_Entity;
Cast_Result_6 : Internal_Entity_Name;
Fld_19 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Fld_20 : Internal_Entity_Basic_Decl;
Is_A_7 : Boolean;
Cast_Expr_4 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_21 : Internal_Entity_Defining_Name;
Is_A_8 : Boolean;
Fld_22 : Internal_Entity_Generic_Instantiation_Array_Access;
Get_Result : Internal_Entity_Generic_Instantiation;
Cast_Expr_5 : Internal_Entity_Generic_Instantiation;
Cast_Result_8 : Internal_Entity_Basic_Decl;
Fld_23 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Is_A_9 : Boolean;
Cast_Expr_6 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Object_Decl;
Fld_24 : Internal_Entity_Basic_Decl;
Fld_25 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_12 : Internal_Entity_Defining_Name;
Scope_Result_13 : Internal_Entity_Defining_Name;
Result_Var_13 : Internal_Entity_Defining_Name;
Is_A_10 : Boolean;
Cast_Expr_7 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Base_Subp_Body;
Fld_26 : Internal_Entity_Basic_Decl;
Scope_Result_14 : Internal_Entity_Basic_Decl;
Result_Var_14 : Internal_Entity_Basic_Decl;
Fld_27 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
Scope_Result_15 : Internal_Entity_Defining_Name;
Result_Var_15 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
If_Result_7 : Internal_Entity_Defining_Name;
If_Result_8 : Internal_Entity_Defining_Name;
If_Result_9 : Internal_Entity_Defining_Name;
If_Result_10 : Internal_Entity_Defining_Name;
Dyn_Var_Bind_Result : Internal_Entity_Defining_Name;
Let_Result_1 : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1582 is
            begin
                     Dec_Ref (Fld_22);
            end Finalizer_Scope_1582;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 601 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 547 '<FieldAccess .enclosing_defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:1044
--# expr-start 546 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1044
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 546
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 547
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 548 '<FieldAccess .basic_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1045
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 548
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var := Result_Var; 
--# bind bd Block_Var
--# expr-start 600 '<bind(<DynamicVariable origin>, <Var Self>, <Cond>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:1047
Bound_Origin := Self; 
--# expr-start 599 '<Cond>' If_Result_10 /usr/src/packages/BUILD/ada/ast.py:1047
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 549 '<IsA ParamSpec>' Is_A /usr/src/packages/BUILD/ada/ast.py:1048
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 549
Scope_Result_1 := Is_A;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 551 '<IsA SubpDecl, ExprFunction, GenericSubpInternal, BaseTypeDecl>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:1049
--# expr-start 550 '<FieldAccess .semantic_parent>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1049
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 550
Is_A_1 := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Subp_Decl_Range | Ada_Expr_Function_Range | Ada_Generic_Subp_Internal_Range | Ada_Base_Type_Decl; 
--# expr-done 551
   If_Result := Is_A_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 554 '<FieldAccess .defining_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1052
--# expr-start 553 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1052
--# expr-start 552 '<FieldAccess .semantic_parent>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1052
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 552
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 553
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 554
   If_Result_10 := Fld_4;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 555 '<IsA DiscriminantSpec>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:1054
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Discriminant_Spec_Range; 
--# expr-done 555
Scope_Result_2 := Is_A_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 558 '<FieldAccess .defining_name>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1055
--# expr-start 557 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:1055
--# expr-start 556 '<FieldAccess .semantic_parent>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1055
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_5 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 556
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 557
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 558
   If_Result_9 := Fld_6;
else
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 559 '<IsA ParamSpec>' Is_A_3 /usr/src/packages/BUILD/ada/ast.py:1057
Is_A_3 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 559
Scope_Result_3 := Is_A_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
   --# expr-start 561 '<IsA AbstractSubpDecl, FormalSubpDecl, NullSubpDecl>' Is_A_4 /usr/src/packages/BUILD/ada/ast.py:1058
--# expr-start 560 '<FieldAccess .semantic_parent>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1058
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_7 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 560
Is_A_4 := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Abstract_Subp_Decl_Range | Ada_Formal_Subp_Decl | Ada_Null_Subp_Decl_Range; 
--# expr-done 561
   If_Result_1 := Is_A_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 564 '<FieldAccess .defining_name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1060
--# expr-start 563 '<Cast to BasicDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:1060
--# expr-start 562 '<FieldAccess .semantic_parent>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1060
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_8 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 562
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 563
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 564
   If_Result_8 := Fld_9;
else
Var_Expr_4 := Block_Var;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_4
--# expr-start 565 '<IsA AbstractSubpDecl>' Is_A_5 /usr/src/packages/BUILD/ada/ast.py:1062
Is_A_5 := Var_Expr_4.Node /= null 
and then Var_Expr_4.Node.Kind in Ada_Abstract_Subp_Decl_Range; 
--# expr-done 565
Scope_Result_4 := Is_A_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   --# expr-start 569 '<FieldAccess .defining_name>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:1063
--# expr-start 568 '<FieldAccess .get_primitive_subp_first_type>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:1063
--# expr-start 567 '<FieldAccess .subp_decl_spec>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:1063
--# expr-start 566 '<Cast to AbstractSubpDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:1063
Cast_Expr_1 := Block_Var; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Abstract_Subp_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Abstract_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_4 := No_Entity_Abstract_Subp_Decl;
   end if;
--# expr-done 566
      if Cast_Result_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_4.Node, E_Info => Cast_Result_4.Info);
--# end
--# expr-done 567
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 568
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 569
   If_Result_7 := Fld_12;
else
Var_Expr_5 := Block_Var;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 570 '<IsA BasicSubpDecl>' Is_A_6 /usr/src/packages/BUILD/ada/ast.py:1066
Is_A_6 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 570
Scope_Result_5 := Is_A_6;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 573 '<FieldAccess .get_primitive_subp_first_type>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:1067
--# expr-start 572 '<FieldAccess .subp_decl_spec>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:1067
--# expr-start 571 '<Cast to BasicSubpDecl>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:1067
Cast_Expr_2 := Block_Var; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 571
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_5.Node, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 572
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_14 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 573
Var_Expr_6 := Fld_14;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind prim_typ Var_Expr_6
--# expr-start 574 '<FieldAccess .is_tagged_type>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:1070
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Var_Expr_6.Node, Origin => Bound_Origin, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 574
Var_Expr_7 := Fld_15;
if Var_Expr_7 /= False then
   --# scope-start
   --# bind _ Var_Expr_7
--# expr-start 575 '<FieldAccess .private_completion>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:1071
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_16 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 575
Var_Expr_9 := Fld_16;
if Var_Expr_9 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pc Var_Expr_9
--# expr-start 576 '<FieldAccess .defining_name>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:1072
      if Var_Expr_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_9.Node, E_Info => Var_Expr_9.Info);
--# end
--# expr-done 576
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
Var_Expr_8 := Result_Var_6;
if Var_Expr_8 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_8
Scope_Result_7 := Var_Expr_8;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   --# expr-start 577 '<FieldAccess .defining_name>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:1073
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 577
   Result_Var_7 := Fld_18;
end if;
Scope_Result_8 := Result_Var_7;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := No_Entity_Defining_Name;
end if;
Scope_Result_9 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_9;
else
--# expr-start 578 '<Cast to Name>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:1077
Cast_Expr_3 := Ent; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_6 := No_Entity_Name;
   end if;
--# expr-done 578
Var_Expr_11 := Cast_Result_6;
if Var_Expr_11 /= No_Entity_Name then
   --# scope-start
--# expr-start 579 '<FieldAccess .gnat_xref_decl>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:1077
      if Var_Expr_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.gnat_xref_decl
Fld_19 := Libadalang.Implementation.Name_P_Gnat_Xref_Decl (Node => Var_Expr_11.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 579
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind ret Var_Expr_10
--# expr-start 598 '<Let dbd>' Let_Result /usr/src/packages/BUILD/ada/ast.py:1079
--# scope-start
--# expr-start 580 '<FieldAccess .basic_decl>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:1079
      if Var_Expr_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_20 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 580
Dbd := Fld_20; 
--# bind dbd Dbd
--# expr-start 597 '<Cond>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:1079
--# expr-start 581 '<IsA ParamSpec>' Is_A_7 /usr/src/packages/BUILD/ada/ast.py:1080
Is_A_7 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 581
if Is_A_7 then
   --# expr-start 583 '<FieldAccess .decl_param(...)>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:1081
--# expr-start 582 '<Cast to ParamSpec>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:1081
Cast_Expr_4 := Dbd; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 582
      if Cast_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_21 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result_7.Node, Param => Var_Expr_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 583
   If_Result_5 := Fld_21;
else
--# expr-start 584 '<IsA GenericSubpInternal, GenericPackageInternal>' Is_A_8 /usr/src/packages/BUILD/ada/ast.py:1083
Is_A_8 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Generic_Subp_Internal_Range | Ada_Generic_Package_Internal_Range; 
--# expr-done 584
if Is_A_8 then
--# expr-start 586 '<at(<FieldAccess .generic_instantiations>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1084
--# expr-start 585 '<FieldAccess .generic_instantiations>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:1084
      if Dbd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.generic_instantiations
Fld_22 := Libadalang.Implementation.Ada_Node_P_Generic_Instantiations (Node => Dbd.Node, E_Info => Dbd.Info);
--# end
--# expr-done 585
Get_Result := Get (Fld_22, 0, True); 
--# expr-done 586
Var_Expr_12 := Get_Result;
if Var_Expr_12 /= No_Entity_Generic_Instantiation then
   --# scope-start
   --# bind gi Var_Expr_12
--# expr-start 588 '<FieldAccess .defining_name>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:1085
--# expr-start 587 '<Cast to BasicDecl>' Cast_Result_8 /usr/src/packages/BUILD/ada/ast.py:1085
Cast_Expr_5 := Var_Expr_12; 
      Cast_Result_8 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# expr-done 587
      if Cast_Result_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_23 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_8.Node, E_Info => Cast_Result_8.Info);
--# end
--# expr-done 588
Scope_Result_11 := Fld_23;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   Result_Var_11 := Var_Expr_10;
end if;
   If_Result_4 := Result_Var_11;
else
--# expr-start 589 '<IsA ObjectDecl>' Is_A_9 /usr/src/packages/BUILD/ada/ast.py:1089
Is_A_9 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 589
if Is_A_9 then
--# expr-start 591 '<FieldAccess .public_part_decl>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:1090
--# expr-start 590 '<Cast to ObjectDecl>' Cast_Result_9 /usr/src/packages/BUILD/ada/ast.py:1090
Cast_Expr_6 := Dbd; 
   if Cast_Expr_6.Node = null
      or else Cast_Expr_6.Node.Kind in Ada_Object_Decl_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Object_Decl
        (Node => Cast_Expr_6.Node,
         Info => Cast_Expr_6.Info);
   else
         Cast_Result_9 := No_Entity_Object_Decl;
   end if;
--# expr-done 590
      if Cast_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ObjectDecl.public_part_decl
Fld_24 := Libadalang.Implementation.Object_Decl_P_Public_Part_Decl (Node => Cast_Result_9.Node, E_Info => Cast_Result_9.Info);
--# end
--# expr-done 591
Var_Expr_14 := Fld_24;
if Var_Expr_14 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ppd Var_Expr_14
--# expr-start 592 '<FieldAccess .defining_name>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:1091
      if Var_Expr_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_14.Node, E_Info => Var_Expr_14.Info);
--# end
--# expr-done 592
Scope_Result_12 := Fld_25;
   --# end
   Result_Var_12 := Scope_Result_12;
else
   Result_Var_12 := No_Entity_Defining_Name;
end if;
Var_Expr_13 := Result_Var_12;
if Var_Expr_13 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_13
Scope_Result_13 := Var_Expr_13;
   --# end
   Result_Var_13 := Scope_Result_13;
else
   Result_Var_13 := Var_Expr_10;
end if;
   If_Result_3 := Result_Var_13;
else
--# expr-start 593 '<IsA BaseSubpBody>' Is_A_10 /usr/src/packages/BUILD/ada/ast.py:1095
Is_A_10 := Dbd.Node /= null 
and then Dbd.Node.Kind in Ada_Base_Subp_Body; 
--# expr-done 593
if Is_A_10 then
   --# expr-start 596 '<FieldAccess .defining_name>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:1096
--# expr-start 595 '<FieldAccess .decl_part>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:1096
--# expr-start 594 '<Cast to BaseSubpBody>' Cast_Result_10 /usr/src/packages/BUILD/ada/ast.py:1096
Cast_Expr_7 := Dbd; 
   if Cast_Expr_7.Node = null
      or else Cast_Expr_7.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_10 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_7.Node,
         Info => Cast_Expr_7.Info);
   else
         Cast_Result_10 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 594
      if Cast_Result_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_26 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_10.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_10.Info);
--# end
--# expr-done 595
Var_Expr_15 := Fld_26;
if Var_Expr_15 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr_15
Scope_Result_14 := Var_Expr_15;
   --# end
   Result_Var_14 := Scope_Result_14;
else
   Result_Var_14 := Dbd;
end if;
      if Result_Var_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_27 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Result_Var_14.Node, E_Info => Result_Var_14.Info);
--# end
--# expr-done 596
   If_Result_2 := Fld_27;
else
   If_Result_2 := Var_Expr_10;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
--# expr-done 597
Let_Result := If_Result_5; 
--# end
--# expr-done 598
Scope_Result_15 := Let_Result;
   --# end
      Finalizer_Scope_1582;
   Result_Var_15 := Scope_Result_15;
else
   Result_Var_15 := No_Entity_Defining_Name;
end if;
   If_Result_6 := Result_Var_15;
end if;
   If_Result_7 := If_Result_6;
end if;
   If_Result_8 := If_Result_7;
end if;
   If_Result_9 := If_Result_8;
end if;
   If_Result_10 := If_Result_9;
end if;
--# expr-done 599
Dyn_Var_Bind_Result := If_Result_10; 
--# expr-done 600
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 601
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1582;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Gnat_Xref;
--# end
--# property-start AdaNode.env_get_real_from_node /usr/src/packages/BUILD/ada/ast.py:1105
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_Real_From_Node
  (Node : Bare_Ada_Node
      ; From_Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind from_node From_Node
   Property_Result : Bare_Ada_Node;
            procedure Finalizer_Scope_74;
      C : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
Is_Null_1 : Boolean;
If_Result_1 : Bare_Ada_Node;
Let_Result_1 : Bare_Ada_Node;
            procedure Finalizer_Scope_74 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_74;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 611 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 610 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1114
--# expr-start 602 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:1114
Is_Null_1 := From_Node = null; 
--# expr-done 602
if Is_Null_1 then
   If_Result_1 := From_Node;
else
   --# expr-start 609 '<Let c>' Let_Result /usr/src/packages/BUILD/ada/ast.py:1114
--# scope-start
--# expr-start 606 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1115
--# expr-start 605 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1115
--# expr-start 603 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:1115
Node_Parents := Parents (From_Node, True); 
--# expr-done 603
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_23 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_23
               --# expr-start 604 '<IsA BasicSubpDecl, GenericInstantiation>' Is_A /usr/src/packages/BUILD/ada/ast.py:1116
Is_A := Item_23 /= null 
and then Item_23.Kind in Ada_Basic_Subp_Decl | Ada_Generic_Instantiation; 
--# expr-done 604
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_23;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 605
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 606
C := Get_Result; 
--# bind c C
--# expr-start 608 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1117
--# expr-start 607 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:1117
Is_Null := C = null; 
--# expr-done 607
if Is_Null then
   If_Result := From_Node;
else
   If_Result := C;
end if;
--# expr-done 608
Let_Result := If_Result; 
--# end
--# expr-done 609
   If_Result_1 := Let_Result;
end if;
--# expr-done 610
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 611
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_74;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_74;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get_Real_From_Node;
--# end
--# property-start AdaNode.entity_no_md /usr/src/packages/BUILD/ada/ast.py:1121
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Entity_No_Md
  (Node : Bare_Ada_Node
      ; N : Bare_Ada_Node
      ; Rebindings : Env_Rebindings
      ; From_Rebound : Boolean
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   --# bind rebindings Rebindings
   --# bind from_rebound From_Rebound
   Property_Result : Internal_Entity;
      New_Struct : Internal_Entity_Info;
Is_Null : Boolean;
If_Result : Internal_Entity_Info;
New_Struct_1 : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 616 '<Block>' Let_Result None
--# scope-start
--# expr-start 615 '<New InternalEntity>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:1127
--# expr-start 614 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1129
--# expr-start 612 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:1129
Is_Null := N = null; 
--# expr-done 612
if Is_Null then
   If_Result := No_Entity_Info;
else
   --# expr-start 613 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:1129
New_Struct := (From_Rebound => From_Rebound, MD => No_Metadata, Rebindings => Rebindings); 
--# expr-done 613
   If_Result := New_Struct;
end if;
--# expr-done 614
New_Struct_1 := (Info => If_Result, Node => N); 
--# expr-done 615
Let_Result := New_Struct_1; 
--# end
--# expr-done 616
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Entity_No_Md;
--# end
--# property-start AdaNode.bool_bind /usr/src/packages/BUILD/ada/ast.py:1137
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Bool_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_76;
      Fld : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_76 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_76;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 619 '<Block>' Let_Result None
--# scope-start
--# expr-start 618 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:1142
Type_Var.Value := No_Entity;
Eq_Node.Refs.Reset (Type_Var.all);
Eq_Node.Refs.Destroy (Type_Var.all);
--# expr-start 617 '<FieldAccess .bool_type>' Fld /usr/src/packages/BUILD/ada/ast.py:1142
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 617
Bind_Result := Bind_Default_562.Create (Type_Var, Fld, No_Logic_Converter_Default, Equals_Data_562'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1142") else null)); 
--# expr-done 618
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 619
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_76;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_76;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Bool_Bind;
--# end
--# property-start AdaNode.env_mappings /usr/src/packages/BUILD/ada/ast.py:1146
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Mappings
  (Node : Bare_Ada_Node
      ; Defining_Names : Bare_Defining_Name_List
      ; Value : Bare_Ada_Node
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind defining_names Defining_Names
   --# bind value Value
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_77;
            procedure Finalizer_Scope_1589;
      Fld : Symbol_Type;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Item_24 : Bare_Defining_Name;
Map_Result : Internal_Env_Assoc_Array_Access;
Let_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_77 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_77;
            procedure Finalizer_Scope_1589 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1589;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 623 '<Block>' Let_Result None
--# scope-start
--# expr-start 622 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1151
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      if Defining_Names = null then
         Map_Result := Create_Internal_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Defining_Names;
      begin
         for Untyped_Item_24 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_24 := Untyped_Item_24; 
   --# scope-start
               --# bind n Item_24
      --# expr-start 621 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:1152
--# expr-start 620 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:1152
      if Item_24 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_24);
--# end
--# expr-done 620
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Value); 
--# expr-done 621
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1589;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 622
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 623
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_77;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_77;
               Finalizer_Scope_1589;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Mappings;
--# end
--# property-start AdaNode.type_bind_val /usr/src/packages/BUILD/ada/ast.py:1155
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Type_Bind_Val
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Internal_Entity
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_78;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_78 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_78;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 625 '<Block>' Let_Result None
--# scope-start
--# expr-start 624 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:1156
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Bind_Result := Bind_Default_567.Create (Left, Right, No_Logic_Converter_Default, Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1156") else null)); 
--# expr-done 624
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 625
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_78;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_78;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Type_Bind_Val;
--# end
--# property-start AdaNode.type_bind_var /usr/src/packages/BUILD/ada/ast.py:1159
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Type_Bind_Var
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_79;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_79 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_79;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 627 '<Block>' Let_Result None
--# scope-start
--# expr-start 626 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:1160
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Right.Value := No_Entity;
Eq_Node.Refs.Reset (Right.all);
Eq_Node.Refs.Destroy (Right.all);
Bind_Result := Bind_Default_567.Create (Left, Right, No_Logic_Converter_Default, Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1160") else null)); 
--# expr-done 626
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 627
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_79;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_79;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Type_Bind_Var;
--# end
--# property-start AdaNode.comp_bind /usr/src/packages/BUILD/ada/ast.py:1163
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Comp_Bind
  (Node : Bare_Ada_Node
      ; Left : Logic_Var
      ; Right : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind left Left
   --# bind right Right
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_80;
      Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_80 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_80;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 629 '<Block>' Let_Result None
--# scope-start
--# expr-start 628 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:1164
Left.Value := No_Entity;
Eq_Node.Refs.Reset (Left.all);
Eq_Node.Refs.Destroy (Left.all);
Right.Value := No_Entity;
Eq_Node.Refs.Reset (Right.all);
Eq_Node.Refs.Destroy (Right.all);
Bind_Result := Bind_553_567.Create (Left, Right, Logic_Converter_553'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:1164") else null)); 
--# expr-done 628
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 629
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_80;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_80;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Comp_Bind;
--# end
--# property-start AdaNode.universal_int_bind /usr/src/packages/BUILD/ada/ast.py:1169
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Int_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_81;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_81 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_81;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 632 '<Block>' Let_Result None
--# scope-start
--# expr-start 631 '<FieldAccess .type_bind_val(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1175
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 630 '<FieldAccess .universal_int_type>' Fld /usr/src/packages/BUILD/ada/ast.py:1175
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 630
--# property-call-start AdaNode.type_bind_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Type_Var, Right => Fld, Origin => Origin);
--# end
--# expr-done 631
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 632
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_81;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_81;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Int_Bind;
--# end
--# property-start AdaNode.universal_real_bind /usr/src/packages/BUILD/ada/ast.py:1178
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Universal_Real_Bind
  (Node : Bare_Ada_Node
      ; Type_Var : Logic_Var
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_var Type_Var
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_82;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_82 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_82;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 635 '<Block>' Let_Result None
--# scope-start
--# expr-start 634 '<FieldAccess .type_bind_val(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1183
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 633 '<FieldAccess .universal_real_type>' Fld /usr/src/packages/BUILD/ada/ast.py:1183
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 633
--# property-call-start AdaNode.type_bind_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Type_Var, Right => Fld, Origin => Origin);
--# end
--# expr-done 634
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 635
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_82;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_82;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Universal_Real_Bind;
--# end
--# property-start AdaNode.origin_node /usr/src/packages/BUILD/ada/ast.py:1186
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Origin_Node
  (Node : Bare_Ada_Node
  )
   return Bare_Ada_Node
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node;
      Fld : Boolean;
If_Result : Bare_Ada_Node;
Let_Result : Bare_Ada_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 638 '<Block>' Let_Result None
--# scope-start
--# expr-start 637 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1191
--# expr-start 636 '<FieldAccess .in_contract>' Fld /usr/src/packages/BUILD/ada/ast.py:1191
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.in_contract
Fld := Libadalang.Implementation.Ada_Node_P_In_Contract (Node => Self);
--# end
--# expr-done 636
if Fld then
   If_Result := No_Bare_Ada_Node;
else
   If_Result := Self;
end if;
--# expr-done 637
Let_Result := If_Result; 
--# end
--# expr-done 638
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Origin_Node;
--# end
--# property-start AdaNode.env_hook /usr/src/packages/BUILD/ada/ast.py:1194
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Hook
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Match_2_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Body_Node;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Fld_3 : Boolean;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_3 : Bare_Library_Item;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Bare_Subunit;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Bare_Ada_Node;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Match_Result_1 : Boolean;
Let_Result_6 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 645 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 644 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:1201
--# expr-start 639 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1201
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 639
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_0 := Cast_Result_3; 
--# bind _ Match_0
--# expr-start 642 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:1202
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind b Match_0_1
--# expr-start 640 '<FieldAccess .env_hook_body>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1203
      if Match_0_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.env_hook_body
Fld_1 := Libadalang.Implementation.Body_Node_P_Env_Hook_Body (Node => Match_0_1);
--# end
--# expr-done 640
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind bd Match_1_1
--# expr-start 641 '<FieldAccess .env_hook_basic_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1204
      if Match_1_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Match_1_1);
--# end
--# expr-done 641
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 642
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_1 := Cast_Result_4; 
--# bind su Match_1
--# expr-start 643 '<FieldAccess .env_hook_subunit>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1207
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.env_hook_subunit
Fld_3 := Libadalang.Implementation.Subunit_P_Env_Hook_Subunit (Node => Match_1);
--# end
--# expr-done 643
Let_Result_4 := Fld_3; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix_1;
Match_2 := Cast_Result_5; 
--# bind _ Match_2
Let_Result_5 := False; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
end case;
--# expr-done 644
Let_Result_6 := Match_Result_1; 
--# end
--# expr-done 645
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Hook;
--# end
--# property-start AdaNode.env_get /usr/src/packages/BUILD/ada/ast.py:1212
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_85;
      Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_85 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_85;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 648 '<Block>' Let_Result None
--# scope-start
--# expr-start 647 '<EnvGet(<Var env>, <Var symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:1224
--# expr-start 646 '<FieldAccess .env_get_real_from_node(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1224
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 646
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Symbol, Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Fld)); 
--# expr-done 647
Let_Result := Env_Get_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 648
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_85;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_85;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get;
--# end
--# property-start AdaNode.env_get_first /usr/src/packages/BUILD/ada/ast.py:1228
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Env_Get_First
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Symbol : Symbol_Type
      ; Lookup : Lookup_Kind
         := Recursive
      ; From_Node : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
  )
   return Internal_Entity
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind symbol Symbol
   --# bind lookup Lookup
   --# bind from_node From_Node
   --# bind categories Categories
   Property_Result : Internal_Entity;
      Fld : Bare_Ada_Node;
Env_Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 651 '<Block>' Let_Result None
--# scope-start
--# expr-start 650 '<EnvGet(<Var env>, <Var symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:1240
--# expr-start 649 '<FieldAccess .env_get_real_from_node(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1241
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_get_real_from_node
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get_Real_From_Node (Node => Self, From_Node => From_Node);
--# end
--# expr-done 649
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Symbol, Lookup_Kind => To_Lookup_Kind_Type (Lookup), Categories => Categories, From => Fld); 
--# expr-done 650
Let_Result := Env_Get_Result; 
--# end
--# expr-done 651
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Env_Get_First;
--# end
--# property-start AdaNode.custom_id_text /usr/src/packages/BUILD/ada/ast.py:103
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Custom_Id_Text
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_4;
      Array_Lit : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_4 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_4;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 653 '<Block>' Let_Result None
--# scope-start
--# expr-start 652 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:109
Array_Lit := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 652
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 653
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_4;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_4;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Custom_Id_Text;
--# end
--# property-start AdaNode.complete /usr/src/packages/BUILD/ada/ast.py:251
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Complete
  (Node : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Iterator_Access
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_18;
      Fld : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Internal_Unit;
Fld_5 : Boolean;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Iter : Internal_Completion_Item_Iterator_Access;
Let_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_18 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Iter);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_18;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 665 '<Block>' Let_Result None
--# scope-start
--# expr-start 664 '<FieldAccess .to_iterator>' Iter /usr/src/packages/BUILD/ada/ast.py:255
--# expr-start 663 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:255
--# expr-start 655 '<EnvGet(<FieldAccess .children_env>, <No SymbolType>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:255
--# expr-start 654 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:255
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 654
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 655
declare
   Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_7 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_7
      --# expr-start 662 '<New InternalCompletionItem>' New_Struct /usr/src/packages/BUILD/ada/ast.py:256
--# expr-start 656 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:257
Cast_Expr := Item_7; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 656
--# expr-start 659 '<FieldAccess .dottable_subp>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:258
--# expr-start 658 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:258
--# expr-start 657 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:258
Fld_1 := Item_7.Info;
--# expr-done 657
Fld_2 := Fld_1.MD;
--# expr-done 658
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 659
--# expr-start 661 '<FieldAccess .has_with_visibility(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:259
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 660 '<FieldAccess .unit>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:259
      if Item_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Unit (Node => Item_7.Node);
--# expr-done 660
--# property-call-start AdaNode.has_with_visibility
Fld_5 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_4);
--# end
--# expr-done 661
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_3, Is_Visible => Fld_5); 
--# expr-done 662
         declare
            Item_To_Append : constant Internal_Completion_Item :=
               New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Completion_Item_Array
        (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 663
Iter := To_Iterator (Map_Result, Self.Unit.Context); 
--# expr-done 664
Let_Result := Iter; Inc_Ref (Let_Result);
--# end
--# expr-done 665
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_18;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_18;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Complete;
--# end
 function Ada_Node_P_Xref_Equation
  (Node : Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is (raise Property_Error
    with "Property AdaNode.xref_equation not implemented on type "
    & Kind_Name (Node));
--# property-start AdaNode.xref_stop_resolution /usr/src/packages/BUILD/ada/ast.py:471
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Stop_Resolution
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Xref_Stop_Resolution;
--# end
--# property-start AdaNode.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:531
pragma Warnings (Off, "is not referenced");
 function Ada_Node_P_Xref_Entry_Point
  (Node : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Ada_Node := Bare_Ada_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ada_Node_P_Xref_Entry_Point;
--# end
function Ada_Node_P_Resolve_Generic_Actual
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Internal_Entity :=
      Ada_Node_P_Resolve_Generic_Actual (E.Node, E_Info);
begin
      return (Node => Result.Node, Info => Result.Info);
end;
function Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body
  (E : Entity
  ) return Lexical_Env
is
   Result : constant Lexical_Env :=
      Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body (E.Node);
begin
      return Result;
end;
function Ada_Node_P_Use_Clauses_In_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
   Result : constant Lexical_Env :=
      Ada_Node_P_Use_Clauses_In_Generic_Formal_Part (E.Node);
begin
      return Result;
end;
function Ada_Node_P_Nested_Generic_Formal_Part
  (E : Entity
  ) return Lexical_Env
is
   Result : constant Lexical_Env :=
      Ada_Node_P_Nested_Generic_Formal_Part (E.Node);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Abort_Node
   --
--# property-start '[dispatcher]Abort.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abort_Node_P_As_Bool
  (Node : Bare_Abort_Node
  )
   return Boolean
is
   Self : Bare_Abort_Node := Bare_Abort_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Abort_Node (Self.Kind) is
               when Ada_Abort_Absent =>
                  --# property-call-start Abort.Absent.as_bool
                  Property_Result := Abort_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abort_Present =>
                  --# property-call-start Abort.Present.as_bool
                  Property_Result := Abort_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Abort_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Absent
   --
--# property-start Abort.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3919
pragma Warnings (Off, "is not referenced");
 function Abort_Absent_P_As_Bool
  (Node : Bare_Abort_Absent
  )
   return Boolean
is
   Self : Bare_Abort_Absent := Bare_Abort_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abort_Present
   --
--# property-start Abort.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3919
pragma Warnings (Off, "is not referenced");
 function Abort_Present_P_As_Bool
  (Node : Bare_Abort_Present
  )
   return Boolean
is
   Self : Bare_Abort_Present := Bare_Abort_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Node
   --
--# property-start '[dispatcher]Abstract.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Abstract_Node_P_As_Bool
  (Node : Bare_Abstract_Node
  )
   return Boolean
is
   Self : Bare_Abstract_Node := Bare_Abstract_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Abstract_Node (Self.Kind) is
               when Ada_Abstract_Absent =>
                  --# property-call-start Abstract.Absent.as_bool
                  Property_Result := Abstract_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Abstract_Present =>
                  --# property-call-start Abstract.Present.as_bool
                  Property_Result := Abstract_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Abstract_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Absent
   --
--# property-start Abstract.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3863
pragma Warnings (Off, "is not referenced");
 function Abstract_Absent_P_As_Bool
  (Node : Bare_Abstract_Absent
  )
   return Boolean
is
   Self : Bare_Abstract_Absent := Bare_Abstract_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Abstract_Present
   --
--# property-start Abstract.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3863
pragma Warnings (Off, "is not referenced");
 function Abstract_Present_P_As_Bool
  (Node : Bare_Abstract_Present
  )
   return Boolean
is
   Self : Bare_Abstract_Present := Bare_Abstract_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Ada_List
   --
   --
   --  Primitives for Bare_Ada_Node_List
   --
   --
   --  Primitives for Bare_Abstract_State_Decl_List
   --
   --
   --  Primitives for Bare_Alternatives_List
   --
   --
   --  Primitives for Bare_Constraint_List
   --
   --
   --  Primitives for Bare_Decl_List
   --
   --
   --  Primitives for Bare_Stmt_List
   --
   --
   --  Primitives for Bare_Aspect_Assoc_List
   --
   --
   --  Primitives for Bare_Base_Assoc_List
   --
   --
   --  Primitives for Bare_Basic_Assoc_List
   --
   --
   --  Primitives for Bare_Assoc_List
   --
--# property-start AssocList.unpacked_params /usr/src/packages/BUILD/ada/ast.py:11300
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Unpacked_Params
  (Node : Bare_Assoc_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Actual_Array_Access
is
   Self : Bare_Assoc_List := Bare_Assoc_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assoc_List :=
     Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_888;
            procedure Finalizer_Scope_1551;
      Names : Bare_Ada_Node_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld : Bare_Assoc_List;
Fld_1 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Base_Id;
New_Struct : Internal_Single_Actual;
Singleton : Internal_Single_Actual_Array_Access;
Cast_Expr_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Id;
New_Struct_1 : Internal_Single_Actual;
Is_A : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Len : Integer;
Is_Equal : Boolean;
If_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result : Internal_Single_Actual_Array_Access;
Item_25 : Internal_Entity_Basic_Assoc;
Bare_Item_25 : Bare_Basic_Assoc;
Map_Result_1 : Internal_Single_Actual_Array_Access;
Let_Result_1 : Internal_Single_Actual_Array_Access;
            procedure Finalizer_Scope_888 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_888;
            procedure Finalizer_Scope_1551 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Names);
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1551;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 677 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 676 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11306
Saved := Ent; 
Fld := Saved.Node;
declare
   Map_Result_1_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      if Fld = null then
         Map_Result_1 := Create_Internal_Single_Actual_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld;
      begin
         for Untyped_Item_25 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_25 := Untyped_Item_25; 
Is_Null := Bare_Item_25 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_1 := Saved.Info;
As_Entity := (Info => Fld_1, Node => Bare_Item_25); 
   If_Result := As_Entity;
end if;
                  Item_25 := If_Result; 
   --# scope-start
               --# bind pa Item_25
      --# expr-start 675 '<Let names>' Let_Result /usr/src/packages/BUILD/ada/ast.py:11306
--# scope-start
--# expr-start 666 '<FieldAccess .names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11306
      if Item_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_25.Node);
--# end
--# expr-done 666
Names := Fld_2; Inc_Ref (Names);
--# bind names Names
--# expr-start 674 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11306
--# expr-start 668 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11307
--# expr-start 667 '<length(<Var names>)>' Len /usr/src/packages/BUILD/ada/ast.py:11307
Len := Length (Names); 
--# expr-done 667
Is_Equal := Len = 0; 
--# expr-done 668
if Is_Equal then
   --# expr-start 669 '<New InternalSingleActual>' New_Struct /usr/src/packages/BUILD/ada/ast.py:11308
Cast_Expr := No_Bare_Ada_Node; 
      Cast_Result := Cast_Expr;
New_Struct := (Assoc => Item_25, Name => Cast_Result); 
--# expr-done 669
                Singleton := Create_Internal_Single_Actual_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_1 := Singleton;
else
   --# expr-start 673 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11309
declare
   Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Names;
      begin
         for Item_26 of
               Collection.Items
         loop
   --# scope-start
               --# bind i Item_26
               --# expr-start 670 '<IsA BaseId>' Is_A /usr/src/packages/BUILD/ada/ast.py:11311
Is_A := Item_26 /= null 
and then Item_26.Kind in Ada_Base_Id; 
--# expr-done 670
               if Is_A then
      --# expr-start 672 '<New InternalSingleActual>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:11310
--# expr-start 671 '<Cast to BaseId>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11310
Cast_Expr_1 := Item_26; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 671
New_Struct_1 := (Assoc => Item_25, Name => Cast_Result_1); 
--# expr-done 672
         declare
            Item_To_Append : constant Internal_Single_Actual :=
               New_Struct_1;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 673
   If_Result_1 := Map_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 674
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 675
         for Item_To_Append of
               Let_Result.Items
         loop
            Internal_Single_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1551;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 676
Let_Result_1 := Map_Result_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 677
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_888;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_888;
               Finalizer_Scope_1551;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assoc_List_P_Unpacked_Params;
--# end
--# property-start AssocList.zip_with_params /usr/src/packages/BUILD/ada/ast.py:11317
pragma Warnings (Off, "is not referenced");
 function Assoc_List_P_Zip_With_Params
  (Node : Bare_Assoc_List
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Actual_Array_Access
is
   Self : Bare_Assoc_List := Bare_Assoc_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assoc_List :=
     Internal_Entity_Assoc_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_889;
            procedure Finalizer_Scope_1602;
            procedure Finalizer_Scope_1603;
            procedure Finalizer_Scope_1604;
            procedure Finalizer_Scope_1605;
            procedure Finalizer_Scope_1606;
            procedure Finalizer_Scope_1607;
            procedure Finalizer_Scope_1608;
            procedure Finalizer_Scope_1609;
            procedure Finalizer_Scope_1610;
            procedure Finalizer_Scope_1612;
            procedure Finalizer_Scope_1615;
            procedure Finalizer_Scope_1617;
            procedure Finalizer_Scope_1623;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Assoc;
Block_Var_3 : Internal_Param_Actual_Array_Access;
Block_Var_4 : Internal_Param_Actual_Array_Access;
Block_Var_5 : Internal_Param_Actual_Array_Access;
Block_Var_6 : Internal_Param_Actual_Array_Access;
Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Generic_Instantiation;
Match_2 : Internal_Entity_Discriminant_Constraint;
Match_3 : Internal_Entity_Base_Aggregate;
Match_4 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_3 : Internal_Entity_Generic_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Record_Def;
Var_Expr_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Decl : Internal_Entity_Generic_Formal_Subp_Decl;
Subp : Internal_Entity_Formal_Subp_Decl;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Var_Expr_9 : Internal_Entity_Generic_Formal_Subp_Decl;
Var_Expr_10 : Internal_Entity_Formal_Subp_Decl;
Var_Expr_11 : Internal_Entity_Basic_Assoc;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Name;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Expr : Bare_Assoc_List;
Cast_Result_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_11 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_12 : Internal_Entity_Base_Type_Decl;
Fld_13 : Internal_Entity_Base_Record_Def;
Fld_14 : Internal_Entity_Component_List;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_4 : Internal_Entity_Generic_Instantiation;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_5 : Internal_Entity_Discriminant_Constraint;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_6 : Internal_Entity_Base_Aggregate;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Cast_Result_7 : Internal_Entity;
Let_Result_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Match_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_10 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_5 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Saved : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Item_27 : Internal_Entity_Basic_Assoc;
Bare_Item_27 : Bare_Basic_Assoc;
Fld_18 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Map_Result : Internal_Entity_Basic_Assoc_Array_Access;
Get_Result : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Param_Match_Array_Access;
Fld_20 : Internal_Single_Formal;
Fld_21 : Internal_Entity_Defining_Name;
Fld_22 : Internal_Single_Actual;
Fld_23 : Internal_Entity_Basic_Assoc;
Fld_24 : Internal_Entity_Expr;
New_Struct : Internal_Param_Actual;
Map_Result_1 : Internal_Param_Actual_Array_Access;
Scope_Result_11 : Internal_Param_Actual_Array_Access;
Result_Var_6 : Internal_Param_Actual_Array_Access;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_8 : Internal_Entity_Generic_Formal_Subp_Decl;
Cast_Expr_2 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_9 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_25 : Internal_Entity_Basic_Decl;
Cast_Result_10 : Internal_Entity_Formal_Subp_Decl;
Fld_26 : Internal_Entity_Defining_Name;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Internal_Entity;
Cast_Result_11 : Internal_Entity_Generic_Instantiation;
Fld_29 : Internal_Entity_Basic_Decl;
Fld_30 : Internal_Entity_Defining_Name;
Scope_Result_12 : Internal_Entity_Defining_Name;
Result_Var_7 : Internal_Entity_Defining_Name;
Cast_Result_12 : Internal_Entity_Expr;
Fld_31 : Internal_Entity_Expr;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Expr;
New_Struct_1 : Internal_Param_Actual;
Let_Result_5 : Internal_Param_Actual;
Cast_Expr_3 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_13 : Internal_Entity_Generic_Formal_Subp_Decl;
Fld_32 : Internal_Entity_Basic_Decl;
Cast_Result_14 : Internal_Entity_Formal_Subp_Decl;
Fld_33 : Internal_Entity_Expr;
Is_A_2 : Boolean;
Fld_34 : Internal_Entity_Defining_Name;
Fld_35 : Internal_Entity_Defining_Name;
Cast_Result_15 : Internal_Entity;
Cast_Result_16 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Scope_Result_13 : Boolean;
Result_Var_8 : Boolean;
Scope_Result_14 : Boolean;
Result_Var_9 : Boolean;
Map_Result_2 : Internal_Param_Actual_Array_Access;
Scope_Result_15 : Internal_Param_Actual_Array_Access;
Result_Var_10 : Internal_Param_Actual_Array_Access;
Concat_Result : Internal_Param_Actual_Array_Access;
Fld_36 : Internal_Single_Formal_Array_Access;
Fld_37 : Internal_Entity_Defining_Name;
Fld_38 : Internal_Entity_Expr;
New_Struct_2 : Internal_Param_Actual;
Fld_39 : Internal_Entity_Defining_Name;
Fld_40 : Internal_Entity_Defining_Name;
Cast_Result_17 : Internal_Entity;
Cast_Result_18 : Internal_Entity;
Is_Equiv_1 : Boolean;
Quantifier_Result_2 : Boolean;
Not_Val_1 : Boolean;
Map_Result_3 : Internal_Param_Actual_Array_Access;
Scope_Result_16 : Internal_Param_Actual_Array_Access;
Result_Var_11 : Internal_Param_Actual_Array_Access;
Concat_Result_1 : Internal_Param_Actual_Array_Access;
Let_Result_6 : Internal_Param_Actual_Array_Access;
            procedure Finalizer_Scope_889 is
            begin
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Scope_Result_15);
                     Dec_Ref (Result_Var_10);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Scope_Result_16);
                     Dec_Ref (Result_Var_11);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_889;
            procedure Finalizer_Scope_1602 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_1602;
            procedure Finalizer_Scope_1603 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1603;
            procedure Finalizer_Scope_1604 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1604;
            procedure Finalizer_Scope_1605 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1605;
            procedure Finalizer_Scope_1606 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_1606;
            procedure Finalizer_Scope_1607 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_1607;
            procedure Finalizer_Scope_1608 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1608;
            procedure Finalizer_Scope_1609 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_2);
            end Finalizer_Scope_1609;
            procedure Finalizer_Scope_1610 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_1610;
            procedure Finalizer_Scope_1612 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_1612;
            procedure Finalizer_Scope_1615 is
            begin
                     Dec_Ref (Var_Expr_6);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1615;
            procedure Finalizer_Scope_1617 is
            begin
                     Dec_Ref (Var_Expr_7);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1617;
            procedure Finalizer_Scope_1623 is
            begin
                     Dec_Ref (Fld_36);
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_1623;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 748 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 681 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <Cast to Name>: <Var e> <FieldAccess .is_dot_call>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:11325
Bound_Imprecise_Fallback := False; 
--# expr-start 679 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11326
--# expr-start 678 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:11326
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 678
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 679
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 680 '<FieldAccess .is_dot_call>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11326
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dot_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Var_Expr.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 680
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 681
Block_Var := Dyn_Var_Bind_Result; 
--# bind is_dottable_subp Block_Var
--# expr-start 682 '<parent(<Var Ent>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11329
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 682
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 699 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:11329
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind e Match_0
--# expr-start 683 '<FieldAccess .called_subp_spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11330
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_3 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 683
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 684 '<FieldAccess .abstract_formal_params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11330
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 684
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1604;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_1603;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind i Match_1
--# expr-start 687 '<Cast to GenericDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11333
--# expr-start 686 '<FieldAccess .referenced_decl>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11333
--# expr-start 685 '<FieldAccess .generic_entity_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11333
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 685
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_6 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 686
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Decl;
   end if;
--# expr-done 687
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 689 '<FieldAccess .abstract_formal_params>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11333
--# expr-start 688 '<FieldAccess .formal_part>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11333
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_3.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_3.Info);
--# expr-done 688
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 689
Scope_Result_2 := Fld_8;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1606;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1605;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Discriminant_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Discriminant_Constraint
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind c Match_2
--# expr-start 690 '<FieldAccess .subtype>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11337
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld_9 := Libadalang.Implementation.Constraint_P_Subtype (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 690
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 691 '<FieldAccess .discriminants_list>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11337
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 691
Scope_Result_3 := Fld_10;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1608;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_2 := Result_Var_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_1607;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Aggregate .. Ada_Null_Record_Aggregate =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Aggregate
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind a Match_3
--# expr-start 698 '<bind(<DynamicVariable origin>, <Var Self>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <FieldAccess .record_def>: <Var Var_Expr_5> <FieldAccess .abstract_formal_params_for_assocs(...)>>)>)>' Dyn_Var_Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:11339
Cast_Expr := Self; 
      Cast_Result_2 := Cast_Expr;
Bound_Origin := Cast_Result_2; 
--# expr-start 697 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <FieldAccess .record_def>: <Var Var_Expr_5> <FieldAccess .abstract_formal_params_for_assocs(...)>>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:11339
--# expr-start 692 '<FieldAccess .node_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11340
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 692
Bound_Env := Fld_11; Inc_Ref (Bound_Env);
--# expr-start 694 '<FieldAccess .record_def>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11341
--# expr-start 693 '<FieldAccess .expression_type>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11341
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_12 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 693
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_13 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_12.Node, Origin => Bound_Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 694
Var_Expr_5 := Fld_13;
if Var_Expr_5 /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 696 '<FieldAccess .abstract_formal_params_for_assocs(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11341
--# expr-start 695 '<FieldAccess .components>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11341
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Component_List (Node => Var_Expr_5.Node.Base_Record_Def_F_Components, Info => Var_Expr_5.Info);
--# expr-done 695
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_15 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Fld_14.Node, Assocs => Ent, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 696
Scope_Result_4 := Fld_15;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1610;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Dyn_Var_Bind_Result_1 := Result_Var_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 697
Dyn_Var_Bind_Result_2 := Dyn_Var_Bind_Result_1; Inc_Ref (Dyn_Var_Bind_Result_2);
--# expr-done 698
Let_Result_3 := Dyn_Var_Bind_Result_2; Inc_Ref (Let_Result_3);
--# end
Scope_Result_8 := Let_Result_3;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_1609;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Generic_Subp_Decl | Ada_Generic_Package_Renaming_Decl .. Ada_Digits_Constraint | Ada_Index_Constraint .. Ada_Allocator | Ada_Bin_Op .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind _ Match_4
Let_Result_4 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result_4);
--# end
Scope_Result_9 := Let_Result_4;
   Inc_Ref (Scope_Result_9);
   --# end
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
end case;
--# expr-done 699
Scope_Result_10 := Match_Result;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_1602;
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_5);
Block_Var_1 := Result_Var_5; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 703 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11348
--# expr-start 702 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11348
Saved := Ent; 
Fld_16 := Saved.Node;
declare
   Map_Result_Vec : Internal_Entity_Basic_Assoc_Vectors.Vector;
begin
      if Fld_16 = null then
         Map_Result := Create_Internal_Entity_Basic_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_16;
      begin
         for Untyped_Item_27 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_27 := Untyped_Item_27; 
Is_Null := Bare_Item_27 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_17 := Saved.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_27); 
   If_Result := As_Entity;
end if;
                  Item_27 := If_Result; 
   --# scope-start
               --# bind assoc Item_27
--# expr-start 700 '<FieldAccess .names>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11349
      if Item_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Item_27.Node);
--# end
--# expr-done 700
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Fld_18;
   begin
      for Item_28 of
            Collection.Items
      loop
   --# scope-start
            --# bind n Item_28
         --# expr-start 701 '<IsA OthersDesignator>' Is_A /usr/src/packages/BUILD/ada/ast.py:11350
Is_A := Item_28 /= null 
and then Item_28.Kind in Ada_Others_Designator_Range; 
--# expr-done 701
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Assoc :=
               Item_27;
         begin
            Internal_Entity_Basic_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1612;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Basic_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Assoc_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 702
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 703
Block_Var_2 := Get_Result; 
--# bind others_assoc Block_Var_2
Var_Expr_6 := Block_Var_1;
      Inc_Ref (Var_Expr_6);
if Var_Expr_6 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_6
--# expr-start 711 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11355
--# expr-start 704 '<FieldAccess .match_formals(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11355
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.match_formals
Fld_19 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Block_Var_1, Params => Ent, Is_Dottable_Subp => Block_Var);
--# end
--# expr-done 704
declare
   Map_Result_1_Vec : Internal_Param_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_19;
      begin
         for Item_29 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_29
      --# expr-start 710 '<New InternalParamActual>' New_Struct /usr/src/packages/BUILD/ada/ast.py:11356
--# expr-start 707 '<FieldAccess .expr>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:11358
--# expr-start 706 '<FieldAccess .assoc>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:11358
--# expr-start 705 '<FieldAccess .actual>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:11358
Fld_22 := Item_29.Actual;
--# expr-done 705
Fld_23 := Fld_22.Assoc;
--# expr-done 706
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_23.Node, E_Info => Fld_23.Info);
--# end
--# expr-done 707
--# expr-start 709 '<FieldAccess .name>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:11357
--# expr-start 708 '<FieldAccess .formal>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11357
Fld_20 := Item_29.Formal;
--# expr-done 708
Fld_21 := Fld_20.Name;
--# expr-done 709
New_Struct := (Actual => Fld_24, Param => Fld_21); 
--# expr-done 710
         declare
            Item_To_Append : constant Internal_Param_Actual :=
               New_Struct;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Actual_Array
        (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Actual_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Actual_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 711
Scope_Result_11 := Map_Result_1;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_1615;
   Result_Var_6 := Scope_Result_11;
else
   Result_Var_6 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Block_Var_3 := Result_Var_6; Inc_Ref (Block_Var_3);
--# bind explicit_matches Block_Var_3
Var_Expr_7 := Block_Var_1;
      Inc_Ref (Var_Expr_7);
if Var_Expr_7 /= No_Internal_Entity_Base_Formal_Param_Decl_Array_Type then
   --# scope-start
   --# bind _ Var_Expr_7
--# expr-start 736 '<filter_map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:11363
declare
   Map_Result_2_Vec : Internal_Param_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Block_Var_1;
      begin
         for Item_30 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_30
--# expr-start 712 '<Cast to GenericFormalSubpDecl>' Cast_Result_13 /usr/src/packages/BUILD/ada/ast.py:11382
Cast_Expr_3 := Item_30; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_13 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_13 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 712
Var_Expr_9 := Cast_Result_13;
if Var_Expr_9 /= No_Entity_Generic_Formal_Subp_Decl then
   --# scope-start
   --# bind fd Var_Expr_9
--# expr-start 714 '<Cast to FormalSubpDecl>' Cast_Result_14 /usr/src/packages/BUILD/ada/ast.py:11383
--# expr-start 713 '<FieldAccess .decl>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:11383
      if Var_Expr_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Basic_Decl (Node => Var_Expr_9.Node.Generic_Formal_F_Decl, Info => Var_Expr_9.Info);
--# expr-done 713
   if Fld_32.Node = null
      or else Fld_32.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_14 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_32.Node,
         Info => Fld_32.Info);
   else
         Cast_Result_14 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 714
Var_Expr_10 := Cast_Result_14;
if Var_Expr_10 /= No_Entity_Formal_Subp_Decl then
   --# scope-start
   --# bind subp Var_Expr_10
--# expr-start 716 '<IsA BoxExpr, Name>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:11387
--# expr-start 715 '<FieldAccess .default_expr>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:11387
      if Var_Expr_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity_Expr (Node => Var_Expr_10.Node.Formal_Subp_Decl_F_Default_Expr, Info => Var_Expr_10.Info);
--# expr-done 715
Is_A_2 := Fld_33.Node /= null 
and then Fld_33.Node.Kind in Ada_Box_Expr_Range | Ada_Name; 
--# expr-done 716
if Is_A_2 then
   --# expr-start 720 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11390
Quantifier_Result_1 := False;
   declare
      Collection : constant Internal_Param_Actual_Array_Access := Block_Var_3;
   begin
      for Item_31 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_31
         --# expr-start 719 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:11391
--# expr-start 717 '<FieldAccess .param>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:11391
Fld_34 := Item_31.Param;
--# expr-done 717
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
--# expr-start 718 '<FieldAccess .defining_name>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:11391
      if Var_Expr_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_35 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_10.Node, E_Info => Var_Expr_10.Info);
--# end
--# expr-done 718
      Cast_Result_16 := Create_Internal_Entity
        (Node => Fld_35.Node,
         Info => Fld_35.Info);
Is_Equiv := Equivalent (Cast_Result_15, Cast_Result_16); 
--# expr-done 719
            if Is_Equiv then
               Quantifier_Result_1 := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result_1); 
--# expr-done 720
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
Scope_Result_13 := If_Result_2;
   --# end
   Result_Var_8 := Scope_Result_13;
else
   Result_Var_8 := False;
end if;
Scope_Result_14 := Result_Var_8;
   --# end
   Result_Var_9 := Scope_Result_14;
else
   Result_Var_9 := False;
end if;
               if Result_Var_9 then
      --# expr-start 735 '<Let decl, subp>' Let_Result_5 /usr/src/packages/BUILD/ada/ast.py:11366
--# scope-start
--# expr-start 721 '<Cast to GenericFormalSubpDecl>' Cast_Result_8 /usr/src/packages/BUILD/ada/ast.py:11368
Cast_Expr_1 := Item_30; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_8 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_8 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 721
Decl := Cast_Result_8; 
--# bind decl Decl
--# expr-start 724 '<Cast to FormalSubpDecl>' Cast_Result_10 /usr/src/packages/BUILD/ada/ast.py:11369
--# expr-start 723 '<FieldAccess .decl>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:11369
--# expr-start 722 '<Cast to GenericFormalSubpDecl>' Cast_Result_9 /usr/src/packages/BUILD/ada/ast.py:11369
Cast_Expr_2 := Item_30; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Formal_Subp_Decl_Range
   then
      Cast_Result_9 := Create_Internal_Entity_Generic_Formal_Subp_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_9 := No_Entity_Generic_Formal_Subp_Decl;
   end if;
--# expr-done 722
      if Cast_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_9.Node.Generic_Formal_F_Decl, Info => Cast_Result_9.Info);
--# expr-done 723
   if Fld_25.Node = null
      or else Fld_25.Node.Kind in Ada_Formal_Subp_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Fld_25.Node,
         Info => Fld_25.Info);
   else
         Cast_Result_10 := No_Entity_Formal_Subp_Decl;
   end if;
--# expr-done 724
Subp := Cast_Result_10; 
--# bind subp Subp
--# expr-start 734 '<New InternalParamActual>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:11371
--# expr-start 732 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11373
--# expr-start 726 '<IsA Name>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:11374
--# expr-start 725 '<FieldAccess .default_expr>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:11374
      if Subp.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 725
Is_A_1 := Fld_31.Node /= null 
and then Fld_31.Node.Kind in Ada_Name; 
--# expr-done 726
if Is_A_1 then
   --# expr-start 727 '<FieldAccess .default_expr>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:11375
      if Subp.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => Subp.Node.Formal_Subp_Decl_F_Default_Expr, Info => Subp.Info);
--# expr-done 727
   If_Result_1 := Fld_27;
else
--# expr-start 730 '<FieldAccess .designated_subprogram_from(...)>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:11376
      if Subp.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 729 '<Cast to GenericInstantiation>' Cast_Result_11 /usr/src/packages/BUILD/ada/ast.py:11377
--# expr-start 728 '<parent(<Var Ent>)>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:11377
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 728
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Generic_Instantiation
   then
      Cast_Result_11 := Create_Internal_Entity_Generic_Instantiation
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         Cast_Result_11 := No_Entity_Generic_Instantiation;
   end if;
--# expr-done 729
--# property-call-start FormalSubpDecl.designated_subprogram_from
Fld_29 := Libadalang.Implementation.Formal_Subp_Decl_P_Designated_Subprogram_From (Node => Subp.Node, Inst => Cast_Result_11, E_Info => Subp.Info);
--# end
--# expr-done 730
Var_Expr_8 := Fld_29;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 731 '<FieldAccess .defining_name>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:11376
      if Var_Expr_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_30 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_8.Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 731
Scope_Result_12 := Fld_30;
   --# end
   Result_Var_7 := Scope_Result_12;
else
   Result_Var_7 := No_Entity_Defining_Name;
end if;
      Cast_Result_12 := Create_Internal_Entity_Expr
        (Node => Result_Var_7.Node,
         Info => Result_Var_7.Info);
   If_Result_1 := Cast_Result_12;
end if;
--# expr-done 732
--# expr-start 733 '<FieldAccess .defining_name>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:11372
      if Decl.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Decl.Node, E_Info => Decl.Info);
--# end
--# expr-done 733
New_Struct_1 := (Actual => If_Result_1, Param => Fld_26); 
--# expr-done 734
Let_Result_5 := New_Struct_1; 
--# end
--# expr-done 735
         declare
            Item_To_Append : constant Internal_Param_Actual :=
               Let_Result_5;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Param_Actual_Array
        (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Param_Actual_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Param_Actual_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 736
Scope_Result_15 := Map_Result_2;
   Inc_Ref (Scope_Result_15);
   --# end
      Finalizer_Scope_1617;
   Result_Var_10 := Scope_Result_15;
else
   Result_Var_10 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_10);
Block_Var_4 := Result_Var_10; Inc_Ref (Block_Var_4);
--# bind default_subp_matches Block_Var_4
--# expr-start 737 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:11398
Concat_Result := Concat (Block_Var_3, Block_Var_4); 
--# expr-done 737
Block_Var_5 := Concat_Result; Inc_Ref (Block_Var_5);
--# bind given_matches Block_Var_5
Var_Expr_11 := Block_Var_2;
if Var_Expr_11 /= No_Entity_Basic_Assoc then
   --# scope-start
   --# bind oa Var_Expr_11
--# expr-start 746 '<filter_map>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:11401
--# expr-start 738 '<FieldAccess .unpack_formals(...)>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:11401
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.unpack_formals
Fld_36 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Block_Var_1);
--# end
--# expr-done 738
declare
   Map_Result_3_Vec : Internal_Param_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_36;
      begin
         for Item_32 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_32
               --# expr-start 742 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:11406
Quantifier_Result_2 := False;
   declare
      Collection : constant Internal_Param_Actual_Array_Access := Block_Var_5;
   begin
      for Item_33 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_33
         --# expr-start 741 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:11406
--# expr-start 739 '<FieldAccess .param>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:11406
Fld_39 := Item_33.Param;
--# expr-done 739
      Cast_Result_17 := Create_Internal_Entity
        (Node => Fld_39.Node,
         Info => Fld_39.Info);
--# expr-start 740 '<FieldAccess .name>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:11406
Fld_40 := Item_32.Name;
--# expr-done 740
      Cast_Result_18 := Create_Internal_Entity
        (Node => Fld_40.Node,
         Info => Fld_40.Info);
Is_Equiv_1 := Equivalent (Cast_Result_17, Cast_Result_18); 
--# expr-done 741
            if Is_Equiv_1 then
               Quantifier_Result_2 := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val_1 := not (Quantifier_Result_2); 
--# expr-done 742
               if Not_Val_1 then
      --# expr-start 745 '<New InternalParamActual>' New_Struct_2 /usr/src/packages/BUILD/ada/ast.py:11402
--# expr-start 743 '<FieldAccess .expr>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:11404
      if Var_Expr_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 743
--# expr-start 744 '<FieldAccess .name>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:11403
Fld_37 := Item_32.Name;
--# expr-done 744
New_Struct_2 := (Actual => Fld_38, Param => Fld_37); 
--# expr-done 745
         declare
            Item_To_Append : constant Internal_Param_Actual :=
               New_Struct_2;
         begin
            Internal_Param_Actual_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Param_Actual_Array
        (Items_Count => Natural (Internal_Param_Actual_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Param_Actual_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Param_Actual_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Param_Actual_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 746
Scope_Result_16 := Map_Result_3;
   Inc_Ref (Scope_Result_16);
   --# end
      Finalizer_Scope_1623;
   Result_Var_11 := Scope_Result_16;
else
   Result_Var_11 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_11);
Block_Var_6 := Result_Var_11; Inc_Ref (Block_Var_6);
--# bind others_matches Block_Var_6
--# expr-start 747 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:11410
Concat_Result_1 := Concat (Block_Var_5, Block_Var_6); 
--# expr-done 747
Let_Result_6 := Concat_Result_1; Inc_Ref (Let_Result_6);
--# end
--# expr-done 748
      Property_Result := Let_Result_6;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_889;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_889;
               Finalizer_Scope_1602;
               Finalizer_Scope_1603;
               Finalizer_Scope_1604;
               Finalizer_Scope_1605;
               Finalizer_Scope_1606;
               Finalizer_Scope_1607;
               Finalizer_Scope_1608;
               Finalizer_Scope_1609;
               Finalizer_Scope_1610;
               Finalizer_Scope_1612;
               Finalizer_Scope_1615;
               Finalizer_Scope_1617;
               Finalizer_Scope_1623;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assoc_List_P_Zip_With_Params;
--# end
   --
   --  Primitives for Bare_Basic_Decl_List
   --
   --
   --  Primitives for Bare_Case_Expr_Alternative_List
   --
   --
   --  Primitives for Bare_Case_Stmt_Alternative_List
   --
   --
   --  Primitives for Bare_Compilation_Unit_List
   --
   --
   --  Primitives for Bare_Contract_Case_Assoc_List
   --
   --
   --  Primitives for Bare_Defining_Name_List
   --
   --
   --  Primitives for Bare_Discriminant_Spec_List
   --
   --
   --  Primitives for Bare_Elsif_Expr_Part_List
   --
   --
   --  Primitives for Bare_Elsif_Stmt_Part_List
   --
   --
   --  Primitives for Bare_Enum_Literal_Decl_List
   --
   --
   --  Primitives for Bare_Expr_List
   --
   --
   --  Primitives for Bare_Expr_Alternatives_List
   --
   --
   --  Primitives for Bare_Identifier_List
   --
   --
   --  Primitives for Bare_Discriminant_Choice_List
   --
   --
   --  Primitives for Bare_Name_List
   --
   --
   --  Primitives for Bare_Parent_List
   --
   --
   --  Primitives for Bare_Param_Spec_List
   --
   --
   --  Primitives for Bare_Pragma_Node_List
   --
   --
   --  Primitives for Bare_Select_When_Part_List
   --
   --
   --  Primitives for Bare_Unconstrained_Array_Index_List
   --
   --
   --  Primitives for Bare_Variant_List
   --
   --
   --  Primitives for Bare_Aliased_Node
   --
--# property-start '[dispatcher]Aliased.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Aliased_Node_P_As_Bool
  (Node : Bare_Aliased_Node
  )
   return Boolean
is
   Self : Bare_Aliased_Node := Bare_Aliased_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Aliased_Node (Self.Kind) is
               when Ada_Aliased_Absent =>
                  --# property-call-start Aliased.Absent.as_bool
                  Property_Result := Aliased_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Aliased_Present =>
                  --# property-call-start Aliased.Present.as_bool
                  Property_Result := Aliased_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Aliased_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Absent
   --
--# property-start Aliased.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3887
pragma Warnings (Off, "is not referenced");
 function Aliased_Absent_P_As_Bool
  (Node : Bare_Aliased_Absent
  )
   return Boolean
is
   Self : Bare_Aliased_Absent := Bare_Aliased_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aliased_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Aliased_Present
   --
--# property-start Aliased.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3887
pragma Warnings (Off, "is not referenced");
 function Aliased_Present_P_As_Bool
  (Node : Bare_Aliased_Present
  )
   return Boolean
is
   Self : Bare_Aliased_Present := Bare_Aliased_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aliased_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Node
   --
--# property-start '[dispatcher]All.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_All_Node_P_As_Bool
  (Node : Bare_All_Node
  )
   return Boolean
is
   Self : Bare_All_Node := Bare_All_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_All_Node (Self.Kind) is
               when Ada_All_Absent =>
                  --# property-call-start All.Absent.as_bool
                  Property_Result := All_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_All_Present =>
                  --# property-call-start All.Present.as_bool
                  Property_Result := All_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_All_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Absent
   --
--# property-start All.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3911
pragma Warnings (Off, "is not referenced");
 function All_Absent_P_As_Bool
  (Node : Bare_All_Absent
  )
   return Boolean
is
   Self : Bare_All_Absent := Bare_All_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end All_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_All_Present
   --
--# property-start All.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3911
pragma Warnings (Off, "is not referenced");
 function All_Present_P_As_Bool
  (Node : Bare_All_Present
  )
   return Boolean
is
   Self : Bare_All_Present := Bare_All_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end All_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Array_Indices
   --
--# property-start '[dispatcher]ArrayIndices.ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Ndims
  (Node : Bare_Array_Indices
  )
   return Integer
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.ndims
                  Property_Result := Constrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.ndims
                  Property_Result := Unconstrained_Array_Indices_P_Ndims
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Ndims;
--# end
--# property-start '[dispatcher]ArrayIndices.constrain_index_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.constrain_index_expr
                  Property_Result := Constrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.constrain_index_expr
                  Property_Result := Unconstrained_Array_Indices_P_Constrain_Index_Expr
                    (Self
                        , Index_Expr
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start '[dispatcher]ArrayIndices.index_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Index_Type
  (Node : Bare_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.index_type
                  Property_Result := Constrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.index_type
                  Property_Result := Unconstrained_Array_Indices_P_Index_Type
                    (Self
                        , Dim
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Index_Type;
--# end
--# property-start '[dispatcher]ArrayIndices.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Array_Indices_P_Is_Static
  (Node : Bare_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Indices := Bare_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Array_Indices (Self.Kind) is
               when Ada_Constrained_Array_Indices =>
                  --# property-call-start ConstrainedArrayIndices.is_static
                  Property_Result := Constrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Unconstrained_Array_Indices =>
                  --# property-call-start UnconstrainedArrayIndices.is_static
                  Property_Result := Unconstrained_Array_Indices_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Constrained_Array_Indices
   --
      procedure Initialize_Fields_For_Constrained_Array_Indices
        (Self : Bare_Constrained_Array_Indices
         ; Constrained_Array_Indices_F_List : Bare_Constraint_List
        ) is
      begin
            Self.Constrained_Array_Indices_F_List := Constrained_Array_Indices_F_List;
      end Initialize_Fields_For_Constrained_Array_Indices;
   function Constrained_Array_Indices_F_List
     (Node : Bare_Constrained_Array_Indices) return Bare_Constraint_List
   is
   begin
         return Node.Constrained_Array_Indices_F_List;
   end;
--# property-start ConstrainedArrayIndices.ndims /usr/src/packages/BUILD/ada/ast.py:5999
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Ndims
  (Node : Bare_Constrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Constraint_List;
Len : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 750 '<length(<FieldAccess .list>)>' Len /usr/src/packages/BUILD/ada/ast.py:5999
--# expr-start 749 '<FieldAccess .list>' Fld /usr/src/packages/BUILD/ada/ast.py:5999
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Constrained_Array_Indices_F_List;
--# expr-done 749
Len := Length (Fld); 
--# expr-done 750
      Property_Result := Len;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Ndims;
--# end
--# property-start ConstrainedArrayIndices.constrain_index_expr /usr/src/packages/BUILD/ada/ast.py:6002
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Constrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_516;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_516 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_516;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 754 '<Block>' Let_Result None
--# scope-start
--# expr-start 753 '<FieldAccess .type_bind_val(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6003
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 751 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:6003
      if Index_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Index_Expr.Node);
--# expr-done 751
--# expr-start 752 '<FieldAccess .index_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6003
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 752
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 753
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 754
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_516;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_516;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start ConstrainedArrayIndices.xref_equation /usr/src/packages/BUILD/ada/ast.py:6006
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Constrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_517;
            procedure Finalizer_Scope_1627;
            procedure Finalizer_Scope_1628;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Item_34 : Internal_Entity;
Bare_Item_34 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_517 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_517;
            procedure Finalizer_Scope_1627 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1627;
            procedure Finalizer_Scope_1628 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Pred);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1628;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 767 '<Block>' Let_Result None
--# scope-start
--# expr-start 766 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6007
--# expr-start 765 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6007
--# expr-start 755 '<FieldAccess .list>' Fld /usr/src/packages/BUILD/ada/ast.py:6007
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 755
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Constraint_List := Fld_1;
      begin
         for Untyped_Item_34 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_34 := Untyped_Item_34; 
Is_Null := Bare_Item_34 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_34); 
   If_Result := As_Entity;
end if;
                  Item_34 := If_Result; 
   --# scope-start
               --# bind index Item_34
      --# expr-start 764 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:6009
--# expr-start 756 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6009
      if Item_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_34.Node, Env => Env, Origin => Origin, E_Info => Item_34.Info);
--# end
--# expr-done 756
--# expr-start 757 '<Cast to Expr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6010
Cast_Expr := Item_34; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 757
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 763 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:6012
--# expr-start 760 '<FieldAccess .type_bind_val(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6012
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 758 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6012
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 758
--# expr-start 759 '<FieldAccess .int_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6012
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.int_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 759
--# property-call-start AdaNode.type_bind_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Fld_5, Origin => Origin);
--# end
--# expr-done 760
--# expr-start 762 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred /usr/src/packages/BUILD/ada/ast.py:6013
--# expr-start 761 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:6013
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 761
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Pred := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_7, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 762
Or_Pred := Logic_Or (Fld_6, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6012") else null)); 
--# expr-done 763
Scope_Result := Or_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1628;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6014") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6009") else null)); 
--# expr-done 764
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1627;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 765
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6007") else null)); 
--# expr-done 766
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 767
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_517;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_517;
               Finalizer_Scope_1627;
               Finalizer_Scope_1628;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start ConstrainedArrayIndices.index_type /usr/src/packages/BUILD/ada/ast.py:6019
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Index_Type
  (Node : Bare_Constrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_518;
      Block_Var : Boolean;
Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Expr;
Match_2 : Internal_Entity;
Node_Parents : Bare_Ada_Node_Array_Access;
Fld : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Constraint_List;
Fld_3 : Bare_Constraint_List;
Fld_4 : Internal_Entity_Info;
Get_Result_1 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_518 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_518;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 779 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 773 '<FieldAccess .resolve_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6021
--# expr-start 772 '<as_entity(<at(<filter>, 0)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6021
--# expr-start 771 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:6021
--# expr-start 770 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6021
--# expr-start 768 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:6021
Node_Parents := Parents (Self, True); 
--# expr-done 768
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_35 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_35
               --# expr-start 769 '<FieldAccess .xref_entry_point>' Fld /usr/src/packages/BUILD/ada/ast.py:6022
      if Item_35 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Item_35);
--# end
--# expr-done 769
               if Fld then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_35;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 770
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 771
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 772
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 773
Block_Var := Fld_1; 
--# expr-start 774 '<FieldAccess .list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6025
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 774
Fld_3 := Fld_2.Node;
Get_Result_1 := Get (Fld_3, Dim, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_4 := Fld_2.Info;
As_Entity_1 := (Info => Fld_4, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 778 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:6025
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind st Match_0
--# expr-start 775 '<FieldAccess .designated_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6026
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 775
Let_Result := Fld_5; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 777 '<Cast to BaseTypeDecl.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6027
--# expr-start 776 '<FieldAccess .type_val>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6027
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_6 := Libadalang.Implementation.Expr_P_Type_Val (Node => Match_1.Node);
--# end
--# expr-done 776
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 777
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 778
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 779
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_518;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_518;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Index_Type;
--# end
--# property-start ConstrainedArrayIndices.is_static /usr/src/packages/BUILD/ada/ast.py:6031
pragma Warnings (Off, "is not referenced");
 function Constrained_Array_Indices_P_Is_Static
  (Node : Bare_Constrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constrained_Array_Indices := Bare_Constrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constrained_Array_Indices :=
     Internal_Entity_Constrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Constraint_List;
Fld_1 : Bare_Constraint_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_36 : Internal_Entity;
Bare_Item_36 : Bare_Ada_Node;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 780 '<FieldAccess .list>' Fld /usr/src/packages/BUILD/ada/ast.py:6031
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Constrained_Array_Indices_F_List, Info => Ent.Info);
--# expr-done 780
Fld_1 := Fld.Node;
Quantifier_Result := True;
   if Fld_1 /= null then
   declare
      Collection : constant Bare_Constraint_List := Fld_1;
   begin
      for Untyped_Item_36 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_36 := Untyped_Item_36; 
Is_Null := Bare_Item_36 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_36); 
   If_Result := As_Entity;
end if;
               Item_36 := If_Result; 
   --# scope-start
            --# bind t Item_36
         --# expr-start 786 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:6032
      if Item_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_36; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 781 '<FieldAccess .is_static_subtype>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6033
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_3 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 781
Let_Result := Fld_3; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind e Match_1
--# expr-start 783 '<FieldAccess .is_static_expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6034
--# expr-start 782 '<FieldAccess .left>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6034
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 782
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_5 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 783
if Fld_5 then
   --# expr-start 785 '<FieldAccess .is_static_expr>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:6034
--# expr-start 784 '<FieldAccess .right>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6034
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 784
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 785
   If_Result_1 := Fld_7;
else
   If_Result_1 := False;
end if;
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 786
            if not (Match_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Unconstrained_Array_Indices
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Indices
        (Self : Bare_Unconstrained_Array_Indices
         ; Unconstrained_Array_Indices_F_Types : Bare_Unconstrained_Array_Index_List
        ) is
      begin
            Self.Unconstrained_Array_Indices_F_Types := Unconstrained_Array_Indices_F_Types;
      end Initialize_Fields_For_Unconstrained_Array_Indices;
   function Unconstrained_Array_Indices_F_Types
     (Node : Bare_Unconstrained_Array_Indices) return Bare_Unconstrained_Array_Index_List
   is
   begin
         return Node.Unconstrained_Array_Indices_F_Types;
   end;
--# property-start UnconstrainedArrayIndices.ndims /usr/src/packages/BUILD/ada/ast.py:5972
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Ndims
  (Node : Bare_Unconstrained_Array_Indices
  )
   return Integer
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
      Fld : Bare_Unconstrained_Array_Index_List;
Len : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 788 '<length(<FieldAccess .types>)>' Len /usr/src/packages/BUILD/ada/ast.py:5972
--# expr-start 787 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:5972
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Unconstrained_Array_Indices_F_Types;
--# expr-done 787
Len := Length (Fld); 
--# expr-done 788
      Property_Result := Len;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Ndims;
--# end
--# property-start UnconstrainedArrayIndices.constrain_index_expr /usr/src/packages/BUILD/ada/ast.py:5975
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Constrain_Index_Expr
  (Node : Bare_Unconstrained_Array_Indices
      ; Index_Expr : Internal_Entity_Expr
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind index_expr Index_Expr
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_511;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_511 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_511;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 792 '<Block>' Let_Result None
--# scope-start
--# expr-start 791 '<FieldAccess .type_bind_val(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5976
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 789 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:5976
      if Index_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Index_Expr.Node);
--# expr-done 789
--# expr-start 790 '<FieldAccess .index_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5976
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Ent.Node, Dim => Dim, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 790
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 791
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 792
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_511;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_511;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Constrain_Index_Expr;
--# end
--# property-start UnconstrainedArrayIndices.index_type /usr/src/packages/BUILD/ada/ast.py:5979
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Index_Type
  (Node : Bare_Unconstrained_Array_Indices
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Unconstrained_Array_Index;
Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Unconstrained_Array_Index;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 795 '<Block>' Let_Result None
--# scope-start
--# expr-start 793 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:5980
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 793
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, Dim, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Unconstrained_Array_Index then
   --# scope-start
--# expr-start 794 '<FieldAccess .designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5980
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UnconstrainedArrayIndex.designated_type
Fld_3 := Libadalang.Implementation.Unconstrained_Array_Index_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 794
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 795
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Index_Type;
--# end
--# property-start UnconstrainedArrayIndices.xref_equation /usr/src/packages/BUILD/ada/ast.py:5983
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Xref_Equation
  (Node : Bare_Unconstrained_Array_Indices
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_513;
            procedure Finalizer_Scope_1639;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Logic_Equation;
Item_37 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_37 : Bare_Unconstrained_Array_Index;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_513 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_513;
            procedure Finalizer_Scope_1639 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1639;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 801 '<Block>' Let_Result None
--# scope-start
--# expr-start 800 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:5984
--# expr-start 799 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5984
--# expr-start 796 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:5984
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 796
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
      begin
         for Untyped_Item_37 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_37 := Untyped_Item_37; 
Is_Null := Bare_Item_37 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_37); 
   If_Result := As_Entity;
end if;
                  Item_37 := If_Result; 
   --# scope-start
               --# bind typ Item_37
      --# expr-start 798 '<FieldAccess .sub_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5985
--# expr-start 797 '<FieldAccess .subtype_indication>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5985
      if Item_37.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_37.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_37.Info);
--# expr-done 797
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 798
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1639;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 799
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5984") else null)); 
--# expr-done 800
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 801
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_513;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_513;
               Finalizer_Scope_1639;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Xref_Equation;
--# end
--# property-start UnconstrainedArrayIndices.is_static /usr/src/packages/BUILD/ada/ast.py:5988
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Indices_P_Is_Static
  (Node : Bare_Unconstrained_Array_Indices
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Unconstrained_Array_Indices := Bare_Unconstrained_Array_Indices (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Indices :=
     Internal_Entity_Unconstrained_Array_Indices'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Unconstrained_Array_Index_List;
Fld_1 : Bare_Unconstrained_Array_Index_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Unconstrained_Array_Index;
Is_Null : Boolean;
If_Result : Internal_Entity_Unconstrained_Array_Index;
Fld_3 : Internal_Entity_Subtype_Indication;
Fld_4 : Boolean;
Item_38 : Internal_Entity_Unconstrained_Array_Index;
Bare_Item_38 : Bare_Unconstrained_Array_Index;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 802 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:5988
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Unconstrained_Array_Index_List (Node => Ent.Node.Unconstrained_Array_Indices_F_Types, Info => Ent.Info);
--# expr-done 802
Fld_1 := Fld.Node;
Quantifier_Result := True;
   if Fld_1 /= null then
   declare
      Collection : constant Bare_Unconstrained_Array_Index_List := Fld_1;
   begin
      for Untyped_Item_38 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_38 := Untyped_Item_38; 
Is_Null := Bare_Item_38 = null; 
if Is_Null then
   If_Result := No_Entity_Unconstrained_Array_Index;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_38); 
   If_Result := As_Entity;
end if;
               Item_38 := If_Result; 
   --# scope-start
            --# bind t Item_38
         --# expr-start 804 '<FieldAccess .is_static_subtype>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5989
--# expr-start 803 '<FieldAccess .subtype_indication>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5989
      if Item_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subtype_Indication (Node => Item_38.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Item_38.Info);
--# expr-done 803
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_4 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 804
            if not (Fld_4) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Indices_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Aspect_Assoc
   --
      procedure Initialize_Fields_For_Aspect_Assoc
        (Self : Bare_Aspect_Assoc
         ; Aspect_Assoc_F_Id : Bare_Name
         ; Aspect_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Aspect_Assoc_F_Id := Aspect_Assoc_F_Id;
            Self.Aspect_Assoc_F_Expr := Aspect_Assoc_F_Expr;
      end Initialize_Fields_For_Aspect_Assoc;
   function Aspect_Assoc_F_Id
     (Node : Bare_Aspect_Assoc) return Bare_Name
   is
   begin
         return Node.Aspect_Assoc_F_Id;
   end;
   function Aspect_Assoc_F_Expr
     (Node : Bare_Aspect_Assoc) return Bare_Expr
   is
   begin
         return Node.Aspect_Assoc_F_Expr;
   end;
--# property-start AspectAssoc.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7285
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Entry_Point
  (Node : Bare_Aspect_Assoc
  )
   return Boolean
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Xref_Entry_Point;
--# end
--# property-start AspectAssoc.xref_equation /usr/src/packages/BUILD/ada/ast.py:7288
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Xref_Equation
  (Node : Bare_Aspect_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aspect_Assoc :=
     Internal_Entity_Aspect_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_658;
            procedure Finalizer_Scope_1641;
            procedure Finalizer_Scope_1643;
      Block_Var : Bare_Ada_Node;
Var_Expr : Internal_Entity_Expr;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Name;
Sym : Symbol_Type;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_10 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Is_A : Boolean;
Fld_11 : Internal_Entity_Name;
Fld_12 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Fld_13 : Internal_Entity_Name;
Fld_14 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_15 : Internal_Entity_Name;
Fld_16 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_17 : Internal_Entity_Name;
Fld_18 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
Fld_21 : Bare_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
And_Pred : Logic_Equation;
Is_A_1 : Boolean;
Fld_24 : Internal_Entity_Name;
Fld_25 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_4 : Boolean;
Fld_26 : Bare_Expr;
Cast_Result_2 : Bare_Name;
Fld_27 : Logic_Var;
Fld_28 : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Name;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Bind_Result : Logic_Equation;
Is_A_2 : Boolean;
Fld_30 : Internal_Entity_Name;
Fld_31 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Fld_32 : Bare_Expr;
Cast_Result_5 : Bare_Name;
Fld_33 : Logic_Var;
Fld_34 : Internal_Entity_Expr;
Cast_Result_6 : Internal_Entity_Name;
Fld_35 : Internal_Entity_Array_Access;
Is_A_3 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result_7 : Internal_Entity_Basic_Subp_Decl;
Fld_36 : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_6 : Internal_Entity;
Cast_Result_8 : Internal_Entity_Basic_Subp_Decl;
Fld_37 : Internal_Entity_Base_Subp_Spec;
Fld_38 : Boolean;
If_Result_7 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_39 : Internal_Entity_Name;
Sym_7 : Symbol_Type;
Fld_40 : Boolean;
Fld_41 : Internal_Entity_Name;
Sym_8 : Symbol_Type;
Fld_42 : Boolean;
If_Result_8 : Boolean;
Fld_43 : Internal_Entity_Expr;
Fld_44 : Logic_Equation;
Fld_45 : Internal_Entity_Name;
Sym_9 : Symbol_Type;
Fld_46 : Boolean;
Logic_True : Logic_Equation;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
Logic_True_2 : Logic_Equation;
Or_Pred : Logic_Equation;
If_Result_9 : Logic_Equation;
If_Result_10 : Logic_Equation;
If_Result_11 : Logic_Equation;
If_Result_12 : Logic_Equation;
If_Result_13 : Logic_Equation;
If_Result_14 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_658 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_35);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_44);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (If_Result_12);
                     Dec_Ref (If_Result_13);
                     Dec_Ref (If_Result_14);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_658;
            procedure Finalizer_Scope_1641 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_1641;
            procedure Finalizer_Scope_1643 is
            begin
                     Dec_Ref (Fld_48);
            end Finalizer_Scope_1643;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 892 '<Block>' Let_Result None
--# scope-start
--# expr-start 807 '<parent(<parent(<parent(<Var Self>)>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7289
--# expr-start 806 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7289
--# expr-start 805 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7289
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 805
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 806
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 807
Block_Var := Fld_2; 
--# bind target Block_Var
--# expr-start 891 '<Cond>' If_Result_14 /usr/src/packages/BUILD/ada/ast.py:7290
--# expr-start 810 '<FieldAccess .name_is(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7292
--# expr-start 808 '<FieldAccess .id>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7292
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 808
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 809 '<Symbol Iterable>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 809
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Sym);
--# end
--# expr-done 810
if Fld_4 then
   --# expr-start 820 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:7293
--# expr-start 819 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7293
--# expr-start 814 '<FieldAccess .unpacked_params>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7293
--# expr-start 813 '<FieldAccess .assocs>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7293
--# expr-start 812 '<Cast to Aggregate>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7293
--# expr-start 811 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7293
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 811
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result := No_Entity_Aggregate;
   end if;
--# expr-done 812
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result.Node.Base_Aggregate_F_Assocs, Info => Cast_Result.Info);
--# expr-done 813
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 814
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_39 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_39
      --# expr-start 818 '<FieldAccess .xref_no_overloading(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7295
--# expr-start 817 '<Cast to Name>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7295
--# expr-start 816 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7295
--# expr-start 815 '<FieldAccess .assoc>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7295
Fld_8 := Item_39.Assoc;
--# expr-done 815
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 816
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 817
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 818
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1641;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 819
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7293") else null)); 
--# expr-done 820
   If_Result_14 := Logic_Boolean_Op;
else
--# expr-start 821 '<IsA BasicSubpDecl, BaseSubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:7300
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 821
if Is_A then
--# expr-start 825 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 823 '<FieldAccess .name_symbol>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 822 '<FieldAccess .id>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 822
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_11.Node);
--# end
--# expr-done 823
--# expr-start 824 '<Symbol Pre>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pre); 
--# expr-done 824
Is_Equal := Fld_12 = Sym_1; 
--# expr-done 825
if Is_Equal then
   If_Result := True;
else
   --# expr-start 829 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 827 '<FieldAccess .name_symbol>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 826 '<FieldAccess .id>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 826
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_13.Node);
--# end
--# expr-done 827
--# expr-start 828 '<Symbol Post>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Post); 
--# expr-done 828
Is_Equal_1 := Fld_14 = Sym_2; 
--# expr-done 829
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 833 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 831 '<FieldAccess .name_symbol>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 830 '<FieldAccess .id>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:7301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 830
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_15.Node);
--# end
--# expr-done 831
--# expr-start 832 '<Symbol Model_Pre>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Pre); 
--# expr-done 832
Is_Equal_2 := Fld_16 = Sym_3; 
--# expr-done 833
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 837 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 835 '<FieldAccess .name_symbol>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:7301
--# expr-start 834 '<FieldAccess .id>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:7301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 834
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_17.Node);
--# end
--# expr-done 835
--# expr-start 836 '<Symbol Model_Post>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Post); 
--# expr-done 836
Is_Equal_3 := Fld_18 = Sym_4; 
--# expr-done 837
   If_Result_2 := Is_Equal_3;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 843 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7305
--# expr-start 839 '<FieldAccess .sub_equation>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:7305
--# expr-start 838 '<FieldAccess .expr>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:7305
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 838
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 839
--# expr-start 842 '<FieldAccess .bool_bind(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:7306
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 841 '<FieldAccess .type_var>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:7306
--# expr-start 840 '<FieldAccess .expr>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:7306
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Self.Aspect_Assoc_F_Expr;
--# expr-done 840
      if Fld_21 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_21);
--# expr-done 841
--# property-call-start AdaNode.bool_bind
Fld_23 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_22, Origin => Origin);
--# end
--# expr-done 842
And_Pred := Logic_And (Fld_20, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7305") else null)); 
--# expr-done 843
   If_Result_13 := And_Pred;
else
--# expr-start 844 '<IsA BaseTypeDecl>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:7309
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Base_Type_Decl; 
--# expr-done 844
if Is_A_1 then
   --# expr-start 848 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:7310
--# expr-start 846 '<FieldAccess .name_symbol>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:7310
--# expr-start 845 '<FieldAccess .id>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:7310
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 845
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 846
--# expr-start 847 '<Symbol Model_Of>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 847
Is_Equal_4 := Fld_25 = Sym_5; 
--# expr-done 848
   If_Result_4 := Is_Equal_4;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 855 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:7311
--# expr-start 851 '<FieldAccess .ref_var>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:7311
--# expr-start 850 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:7311
--# expr-start 849 '<FieldAccess .expr>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:7311
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Aspect_Assoc_F_Expr;
--# expr-done 849
   if Fld_26 = null
      or else Fld_26.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_26;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 850
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2);
--# end
--# expr-done 851
Fld_27.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_27.all);
Eq_Node.Refs.Destroy (Fld_27.all);
--# expr-start 854 '<FieldAccess .name_designated_type>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:7312
--# expr-start 853 '<Cast to Name>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:7312
--# expr-start 852 '<FieldAccess .expr>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:7312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 852
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Name
   then
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 853
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_29 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result_3.Node, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 854
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
Bind_Result := Bind_Default_Default.Create (Fld_27, Cast_Result_4, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7311") else null)); 
--# expr-done 855
   If_Result_12 := Bind_Result;
else
--# expr-start 856 '<IsA BasicSubpDecl>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:7315
Is_A_2 := Block_Var /= null 
and then Block_Var.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 856
if Is_A_2 then
   --# expr-start 860 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:7316
--# expr-start 858 '<FieldAccess .name_symbol>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:7316
--# expr-start 857 '<FieldAccess .id>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:7316
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 857
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 858
--# expr-start 859 '<Symbol Model_Of>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 859
Is_Equal_5 := Fld_31 = Sym_6; 
--# expr-done 860
   If_Result_5 := Is_Equal_5;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   --# expr-start 876 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:7317
--# expr-start 863 '<FieldAccess .ref_var>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:7318
--# expr-start 862 '<Cast to Name>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:7318
--# expr-start 861 '<FieldAccess .expr>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:7318
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Self.Aspect_Assoc_F_Expr;
--# expr-done 861
   if Fld_32 = null
      or else Fld_32.Kind in Ada_Name
   then
      Cast_Result_5 := Fld_32;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 862
      if Cast_Result_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_5);
--# end
--# expr-done 863
Fld_33.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_33.all);
Eq_Node.Refs.Destroy (Fld_33.all);
--# expr-start 875 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:7319
--# expr-start 874 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:7319
--# expr-start 866 '<FieldAccess .all_env_elements_internal>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:7319
--# expr-start 865 '<Cast to Name>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:7319
--# expr-start 864 '<FieldAccess .expr>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:7319
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 864
   if Fld_34.Node = null
      or else Fld_34.Node.Kind in Ada_Name
   then
      Cast_Result_6 := Create_Internal_Entity_Name
        (Node => Fld_34.Node,
         Info => Fld_34.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 865
      if Cast_Result_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_35 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result_6.Node, E_Info => Cast_Result_6.Info);
--# end
--# expr-done 866
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_35;
      begin
         for Item_40 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_40
--# expr-start 867 '<IsA BasicSubpDecl>' Is_A_3 /usr/src/packages/BUILD/ada/ast.py:7322
Is_A_3 := Item_40.Node /= null 
and then Item_40.Node.Kind in Ada_Basic_Subp_Decl; 
--# expr-done 867
if Is_A_3 then
   --# expr-start 873 '<FieldAccess .match_signature(...)>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:7323
--# expr-start 869 '<FieldAccess .subp_decl_spec>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:7323
--# expr-start 868 '<Cast to BasicSubpDecl>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:7323
Cast_Expr := Item_40; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Subp_Decl;
   end if;
--# expr-done 868
      if Cast_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_36 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_7.Node, E_Info => Cast_Result_7.Info);
--# end
--# expr-done 869
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 872 '<FieldAccess .subp_decl_spec>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:7325
--# expr-start 871 '<Cast to BasicSubpDecl>' Cast_Result_8 /usr/src/packages/BUILD/ada/ast.py:7325
--# expr-start 870 '<as_entity(<Var target>)>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:7325
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result_6 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Block_Var); 
   If_Result_6 := As_Entity;
end if;
--# expr-done 870
   if If_Result_6.Node = null
      or else If_Result_6.Node.Kind in Ada_Basic_Subp_Decl
   then
      Cast_Result_8 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => If_Result_6.Node,
         Info => If_Result_6.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 871
      if Cast_Result_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_37 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Cast_Result_8.Node, E_Info => Cast_Result_8.Info);
--# end
--# expr-done 872
--# property-call-start BaseSubpSpec.match_signature
Fld_38 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_36.Node, Other => Fld_37, Match_Name => False, E_Info => Fld_36.Info);
--# end
--# expr-done 873
   If_Result_7 := Fld_38;
else
   If_Result_7 := False;
end if;
               if If_Result_7 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_40;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 874
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 875
Bind_Result_1 := Bind_Default_Default.Create (Fld_33, Get_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7317") else null)); 
--# expr-done 876
   If_Result_11 := Bind_Result_1;
else
--# expr-start 879 '<FieldAccess .name_is(...)>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:7334
--# expr-start 877 '<FieldAccess .id>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:7334
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_39 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 877
      if Fld_39.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 878 '<Symbol Global>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 878
--# property-call-start Name.name_is
Fld_40 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_39.Node, Sym => Sym_7);
--# end
--# expr-done 879
if Fld_40 then
   If_Result_8 := True;
else
   --# expr-start 882 '<FieldAccess .name_is(...)>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:7334
--# expr-start 880 '<FieldAccess .id>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:7334
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 880
      if Fld_41.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 881 '<Symbol Refined_Global>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Global); 
--# expr-done 881
--# property-call-start Name.name_is
Fld_42 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_41.Node, Sym => Sym_8);
--# end
--# expr-done 882
   If_Result_8 := Fld_42;
end if;
if If_Result_8 then
   --# expr-start 884 '<FieldAccess .sub_equation>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:7335
--# expr-start 883 '<FieldAccess .expr>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:7335
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 883
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_44 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_43.Node, Env => Env, Origin => Origin, E_Info => Fld_43.Info);
--# end
--# expr-done 884
   If_Result_10 := Fld_44;
else
--# expr-start 887 '<FieldAccess .name_is(...)>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:7340
--# expr-start 885 '<FieldAccess .id>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:7340
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Create_Internal_Entity_Name (Node => Ent.Node.Aspect_Assoc_F_Id, Info => Ent.Info);
--# expr-done 885
      if Fld_45.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 886 '<Symbol Convention>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 886
--# property-call-start Name.name_is
Fld_46 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_45.Node, Sym => Sym_9);
--# end
--# expr-done 887
if Fld_46 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7340") else null)); 
   If_Result_9 := Logic_True;
else
   --# expr-start 890 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:7345
--# expr-start 888 '<FieldAccess .expr>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:7345
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Create_Internal_Entity_Expr (Node => Ent.Node.Aspect_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 888
Var_Expr := Fld_47;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 889 '<FieldAccess .sub_equation>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:7345
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_48 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 889
Scope_Result := Fld_48;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1643;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7345") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7346") else null)); 
Or_Pred := Logic_Or (Result_Var, Logic_True_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7345") else null)); 
--# expr-done 890
   If_Result_9 := Or_Pred;
end if;
      Inc_Ref (If_Result_9);
   If_Result_10 := If_Result_9;
end if;
      Inc_Ref (If_Result_10);
   If_Result_11 := If_Result_10;
end if;
      Inc_Ref (If_Result_11);
   If_Result_12 := If_Result_11;
end if;
      Inc_Ref (If_Result_12);
   If_Result_13 := If_Result_12;
end if;
      Inc_Ref (If_Result_13);
   If_Result_14 := If_Result_13;
end if;
      Inc_Ref (If_Result_14);
--# expr-done 891
Let_Result := If_Result_14; Inc_Ref (Let_Result);
--# end
--# expr-done 892
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_658;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_658;
               Finalizer_Scope_1641;
               Finalizer_Scope_1643;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Xref_Equation;
--# end
--# property-start AspectAssoc.aspect_name /usr/src/packages/BUILD/ada/ast.py:7350
pragma Warnings (Off, "is not referenced");
 function Aspect_Assoc_P_Aspect_Name
  (Node : Bare_Aspect_Assoc
      ; N : Internal_Entity_Name
  )
   return Character_Type_Array_Access
is
   Self : Bare_Aspect_Assoc := Bare_Aspect_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_659;
            procedure Finalizer_Scope_1644;
            procedure Finalizer_Scope_1645;
      Match_0 : Internal_Entity_Base_Id;
Match_1 : Internal_Entity_Attribute_Ref;
Match_2 : Internal_Entity_Name;
Fld : Symbol_Type;
Fld_1 : Character_Type_Array_Access;
Fld_2 : Internal_Entity_Name;
Fld_3 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Fld_6 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Base_Id;
Let_Result : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Let_Result_1 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_2 : Character_Type_Array_Access;
Scope_Result_2 : Character_Type_Array_Access;
Match_Result : Character_Type_Array_Access;
Let_Result_3 : Character_Type_Array_Access;
            procedure Finalizer_Scope_659 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_659;
            procedure Finalizer_Scope_1644 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1644;
            procedure Finalizer_Scope_1645 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1645;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 904 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 903 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:7357
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := N; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bid Match_0
--# expr-start 894 '<FieldAccess .image>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7358
--# expr-start 893 '<FieldAccess .sym>' Fld /usr/src/packages/BUILD/ada/ast.py:7358
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0.Node);
--# end
--# expr-done 893
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Fld);
--# expr-done 894
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1644;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ar Match_1
--# expr-start 902 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:7359
--# expr-start 898 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:7359
--# expr-start 896 '<FieldAccess .aspect_name(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7359
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 895 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7359
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Attribute_Ref_F_Prefix, Info => Match_1.Info);
--# expr-done 895
--# property-call-start AspectAssoc.aspect_name
Fld_3 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Self, N => Fld_2);
--# end
--# expr-done 896
--# expr-start 897 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:7360
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (39))); 
--# expr-done 897
Concat_Result := Concat (Fld_3, Array_Lit); 
--# expr-done 898
--# expr-start 901 '<FieldAccess .image>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7361
--# expr-start 900 '<FieldAccess .sym>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7361
--# expr-start 899 '<FieldAccess .attribute>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7361
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Match_1.Node.Attribute_Ref_F_Attribute, Info => Match_1.Info);
--# expr-done 899
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_5 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_4.Node);
--# end
--# expr-done 900
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Fld_5);
--# expr-done 901
Concat_Result_1 := Concat (Concat_Result, Fld_6); 
--# expr-done 902
Let_Result_1 := Concat_Result_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1645;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "aspect_name called on an invalid aspect name"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 903
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 904
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_659;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_659;
               Finalizer_Scope_1644;
               Finalizer_Scope_1645;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Assoc_P_Aspect_Name;
--# end
   --
   --  Primitives for Bare_Aspect_Clause
   --
--# property-start AspectClause.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7120
pragma Warnings (Off, "is not referenced");
 function Aspect_Clause_P_Xref_Entry_Point
  (Node : Bare_Aspect_Clause
  )
   return Boolean
is
   Self : Bare_Aspect_Clause := Bare_Aspect_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aspect_Clause_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_At_Clause
   --
      procedure Initialize_Fields_For_At_Clause
        (Self : Bare_At_Clause
         ; At_Clause_F_Name : Bare_Base_Id
         ; At_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.At_Clause_F_Name := At_Clause_F_Name;
            Self.At_Clause_F_Expr := At_Clause_F_Expr;
      end Initialize_Fields_For_At_Clause;
   function At_Clause_F_Name
     (Node : Bare_At_Clause) return Bare_Base_Id
   is
   begin
         return Node.At_Clause_F_Name;
   end;
   function At_Clause_F_Expr
     (Node : Bare_At_Clause) return Bare_Expr
   is
   begin
         return Node.At_Clause_F_Expr;
   end;
--# property-start AtClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:7219
pragma Warnings (Off, "is not referenced");
 function At_Clause_P_Xref_Equation
  (Node : Bare_At_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_At_Clause := Bare_At_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_At_Clause :=
     Internal_Entity_At_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_647;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_647 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_647;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 910 '<Block>' Let_Result None
--# scope-start
--# expr-start 909 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7220
--# expr-start 906 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7220
--# expr-start 905 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:7220
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.At_Clause_F_Name, Info => Ent.Info);
--# expr-done 905
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 906
--# expr-start 908 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7220
--# expr-start 907 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7220
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.At_Clause_F_Expr, Info => Ent.Info);
--# expr-done 907
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 908
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7220") else null)); 
--# expr-done 909
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 910
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_647;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_647;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end At_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Attribute_Def_Clause
   --
      procedure Initialize_Fields_For_Attribute_Def_Clause
        (Self : Bare_Attribute_Def_Clause
         ; Attribute_Def_Clause_F_Attribute_Expr : Bare_Name
         ; Attribute_Def_Clause_F_Expr : Bare_Expr
        ) is
      begin
            Self.Attribute_Def_Clause_F_Attribute_Expr := Attribute_Def_Clause_F_Attribute_Expr;
            Self.Attribute_Def_Clause_F_Expr := Attribute_Def_Clause_F_Expr;
      end Initialize_Fields_For_Attribute_Def_Clause;
   function Attribute_Def_Clause_F_Attribute_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Name
   is
   begin
         return Node.Attribute_Def_Clause_F_Attribute_Expr;
   end;
   function Attribute_Def_Clause_F_Expr
     (Node : Bare_Attribute_Def_Clause) return Bare_Expr
   is
   begin
         return Node.Attribute_Def_Clause_F_Expr;
   end;
--# property-start AttributeDefClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:7144
pragma Warnings (Off, "is not referenced");
 function Attribute_Def_Clause_P_Xref_Equation
  (Node : Bare_Attribute_Def_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Def_Clause := Bare_Attribute_Def_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Def_Clause :=
     Internal_Entity_Attribute_Def_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_643;
            procedure Finalizer_Scope_1647;
      Block_Var : Internal_Entity_Attribute_Ref;
Block_Var_1 : Symbol_Type;
Var_Expr : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Attribute_Ref;
Fld_1 : Internal_Entity_Identifier;
Fld_2 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_6 : Logic_Var;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
If_Result_3 : Logic_Equation;
Fld_11 : Internal_Entity_Name;
Fld_12 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_643 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_643;
            procedure Finalizer_Scope_1647 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1647;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 941 '<Block>' Let_Result None
--# scope-start
--# expr-start 912 '<Cast to AttributeRef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7145
--# expr-start 911 '<FieldAccess .attribute_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Ent.Info);
--# expr-done 911
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 912
Block_Var := Cast_Result; 
--# bind attr Block_Var
--# expr-start 914 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7146
--# expr-start 913 '<FieldAccess .attribute>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7146
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Identifier (Node => Block_Var.Node.Attribute_Ref_F_Attribute, Info => Block_Var.Info);
--# expr-done 913
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 914
Block_Var_1 := Fld_2; 
--# bind rel_name Block_Var_1
--# expr-start 940 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:7148
--# expr-start 937 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:7148
--# expr-start 916 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:7149
--# expr-start 915 '<Symbol Read>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 915
Is_Equal := Block_Var_1 = Sym; 
--# expr-done 916
if Is_Equal then
   If_Result := True;
else
   --# expr-start 918 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:7149
--# expr-start 917 '<Symbol Write>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 917
Is_Equal_1 := Block_Var_1 = Sym_1; 
--# expr-done 918
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 920 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:7149
--# expr-start 919 '<Symbol Input>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 919
Is_Equal_2 := Block_Var_1 = Sym_2; 
--# expr-done 920
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 922 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:7149
--# expr-start 921 '<Symbol Output>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 921
Is_Equal_3 := Block_Var_1 = Sym_3; 
--# expr-done 922
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 934 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7150
--# expr-start 925 '<FieldAccess .xref_no_overloading>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7150
--# expr-start 924 '<Cast to Name>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7150
--# expr-start 923 '<FieldAccess .expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7150
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 923
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 924
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_4 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 925
--# expr-start 933 '<Predicate on BasicDecl.is_stream_subprogram_for_type>' Pred /usr/src/packages/BUILD/ada/ast.py:7151
--# expr-start 928 '<FieldAccess .ref_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7153
--# expr-start 927 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:7153
--# expr-start 926 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 926
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 927
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Cast_Result_2.Node);
--# end
--# expr-done 928
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
--# expr-start 930 '<FieldAccess .name_designated_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7154
--# expr-start 929 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7154
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Block_Var.Node.Attribute_Ref_F_Prefix, Info => Block_Var.Info);
--# expr-done 929
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 930
--# expr-start 932 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:7155
--# expr-start 931 '<Symbol Input>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 931
Is_Equal_4 := Block_Var_1 = Sym_4; 
--# expr-done 932
Pred := Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Pred.Create (Fld_6, Create_Basic_Decl_P_Is_Stream_Subprogram_For_Type_0_Predicate (Fld_8, Is_Equal_4, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Is_Stream_Subprogram_For_Type")            else null))); 
--# expr-done 933
And_Pred := Logic_And (Fld_4, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7150") else null)); 
--# expr-done 934
   If_Result_3 := And_Pred;
else
   --# expr-start 936 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7158
--# expr-start 935 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7158
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Expr (Node => Ent.Node.Attribute_Def_Clause_F_Expr, Info => Ent.Info);
--# expr-done 935
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 936
   If_Result_3 := Fld_10;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 937
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Attribute_Ref then
   --# scope-start
   --# bind ar Var_Expr
--# expr-start 939 '<FieldAccess .sub_equation>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7160
--# expr-start 938 '<FieldAccess .prefix>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7160
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Attribute_Ref_F_Prefix, Info => Var_Expr.Info);
--# expr-done 938
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 939
Scope_Result := Fld_12;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1647;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7160") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (If_Result_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7148") else null)); 
--# expr-done 940
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 941
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_643;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_643;
               Finalizer_Scope_1647;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Def_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Enum_Rep_Clause
   --
      procedure Initialize_Fields_For_Enum_Rep_Clause
        (Self : Bare_Enum_Rep_Clause
         ; Enum_Rep_Clause_F_Type_Name : Bare_Name
         ; Enum_Rep_Clause_F_Aggregate : Bare_Base_Aggregate
        ) is
      begin
            Self.Enum_Rep_Clause_F_Type_Name := Enum_Rep_Clause_F_Type_Name;
            Self.Enum_Rep_Clause_F_Aggregate := Enum_Rep_Clause_F_Aggregate;
      end Initialize_Fields_For_Enum_Rep_Clause;
   function Enum_Rep_Clause_F_Type_Name
     (Node : Bare_Enum_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Enum_Rep_Clause_F_Type_Name;
   end;
   function Enum_Rep_Clause_F_Aggregate
     (Node : Bare_Enum_Rep_Clause) return Bare_Base_Aggregate
   is
   begin
         return Node.Enum_Rep_Clause_F_Aggregate;
   end;
--# property-start EnumRepClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:7131
pragma Warnings (Off, "is not referenced");
 function Enum_Rep_Clause_P_Xref_Equation
  (Node : Bare_Enum_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Rep_Clause := Bare_Enum_Rep_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Rep_Clause :=
     Internal_Entity_Enum_Rep_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_642;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_642 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_642;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 944 '<Block>' Let_Result None
--# scope-start
--# expr-start 943 '<FieldAccess .xref_no_overloading>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7133
--# expr-start 942 '<FieldAccess .type_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7133
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Enum_Rep_Clause_F_Type_Name, Info => Ent.Info);
--# expr-done 942
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 943
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 944
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_642;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_642;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Record_Rep_Clause
   --
      procedure Initialize_Fields_For_Record_Rep_Clause
        (Self : Bare_Record_Rep_Clause
         ; Record_Rep_Clause_F_Name : Bare_Name
         ; Record_Rep_Clause_F_At_Expr : Bare_Expr
         ; Record_Rep_Clause_F_Components : Bare_Ada_Node_List
        ) is
      begin
            Self.Record_Rep_Clause_F_Name := Record_Rep_Clause_F_Name;
            Self.Record_Rep_Clause_F_At_Expr := Record_Rep_Clause_F_At_Expr;
            Self.Record_Rep_Clause_F_Components := Record_Rep_Clause_F_Components;
      end Initialize_Fields_For_Record_Rep_Clause;
   function Record_Rep_Clause_F_Name
     (Node : Bare_Record_Rep_Clause) return Bare_Name
   is
   begin
         return Node.Record_Rep_Clause_F_Name;
   end;
   function Record_Rep_Clause_F_At_Expr
     (Node : Bare_Record_Rep_Clause) return Bare_Expr
   is
   begin
         return Node.Record_Rep_Clause_F_At_Expr;
   end;
   function Record_Rep_Clause_F_Components
     (Node : Bare_Record_Rep_Clause) return Bare_Ada_Node_List
   is
   begin
         return Node.Record_Rep_Clause_F_Components;
   end;
--# property-start RecordRepClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:7201
pragma Warnings (Off, "is not referenced");
 function Record_Rep_Clause_P_Xref_Equation
  (Node : Bare_Record_Rep_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Rep_Clause := Bare_Record_Rep_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Record_Rep_Clause :=
     Internal_Entity_Record_Rep_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_646;
            procedure Finalizer_Scope_1648;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_646 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_646;
            procedure Finalizer_Scope_1648 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1648;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 950 '<Block>' Let_Result None
--# scope-start
--# expr-start 949 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7202
--# expr-start 946 '<FieldAccess .xref_no_overloading>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7203
--# expr-start 945 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:7203
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Record_Rep_Clause_F_Name, Info => Ent.Info);
--# expr-done 945
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 946
--# expr-start 947 '<FieldAccess .at_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7204
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Record_Rep_Clause_F_At_Expr, Info => Ent.Info);
--# expr-done 947
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 948 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7205
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 948
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1648;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7206") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7202") else null)); 
--# expr-done 949
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 950
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_646;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_646;
               Finalizer_Scope_1648;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Rep_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Aspect_Spec
   --
      procedure Initialize_Fields_For_Aspect_Spec
        (Self : Bare_Aspect_Spec
         ; Aspect_Spec_F_Aspect_Assocs : Bare_Aspect_Assoc_List
        ) is
      begin
            Self.Aspect_Spec_F_Aspect_Assocs := Aspect_Spec_F_Aspect_Assocs;
      end Initialize_Fields_For_Aspect_Spec;
   function Aspect_Spec_F_Aspect_Assocs
     (Node : Bare_Aspect_Spec) return Bare_Aspect_Assoc_List
   is
   begin
         return Node.Aspect_Spec_F_Aspect_Assocs;
   end;
   --
   --  Primitives for Bare_Base_Assoc
   --
--# property-start '[dispatcher]BaseAssoc.assoc_expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Assoc_P_Assoc_Expr
  (Node : Bare_Base_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Assoc := Bare_Base_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Assoc (Self.Kind) is
               when Ada_Contract_Case_Assoc =>
                  --# property-call-start ContractCaseAssoc.assoc_expr
                  Property_Result := Contract_Case_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Pragma_Argument_Assoc =>
                  --# property-call-start PragmaArgumentAssoc.assoc_expr
                  Property_Result := Pragma_Argument_Assoc_P_Assoc_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Contract_Case_Assoc
   --
      procedure Initialize_Fields_For_Contract_Case_Assoc
        (Self : Bare_Contract_Case_Assoc
         ; Contract_Case_Assoc_F_Guard : Bare_Ada_Node
         ; Contract_Case_Assoc_F_Consequence : Bare_Expr
        ) is
      begin
            Self.Contract_Case_Assoc_F_Guard := Contract_Case_Assoc_F_Guard;
            Self.Contract_Case_Assoc_F_Consequence := Contract_Case_Assoc_F_Consequence;
      end Initialize_Fields_For_Contract_Case_Assoc;
   function Contract_Case_Assoc_F_Guard
     (Node : Bare_Contract_Case_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Contract_Case_Assoc_F_Guard;
   end;
   function Contract_Case_Assoc_F_Consequence
     (Node : Bare_Contract_Case_Assoc) return Bare_Expr
   is
   begin
         return Node.Contract_Case_Assoc_F_Consequence;
   end;
--# property-start ContractCaseAssoc.assoc_expr /usr/src/packages/BUILD/ada/ast.py:8988
pragma Warnings (Off, "is not referenced");
 function Contract_Case_Assoc_P_Assoc_Expr
  (Node : Bare_Contract_Case_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Contract_Case_Assoc := Bare_Contract_Case_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Contract_Case_Assoc :=
     Internal_Entity_Contract_Case_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 951 '<FieldAccess .consequence>' Fld /usr/src/packages/BUILD/ada/ast.py:8988
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Contract_Case_Assoc_F_Consequence, Info => Ent.Info);
--# expr-done 951
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Contract_Case_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Pragma_Argument_Assoc
   --
      procedure Initialize_Fields_For_Pragma_Argument_Assoc
        (Self : Bare_Pragma_Argument_Assoc
         ; Pragma_Argument_Assoc_F_Id : Bare_Identifier
         ; Pragma_Argument_Assoc_F_Expr : Bare_Expr
        ) is
      begin
            Self.Pragma_Argument_Assoc_F_Id := Pragma_Argument_Assoc_F_Id;
            Self.Pragma_Argument_Assoc_F_Expr := Pragma_Argument_Assoc_F_Expr;
      end Initialize_Fields_For_Pragma_Argument_Assoc;
   function Pragma_Argument_Assoc_F_Id
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Identifier
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Id;
   end;
   function Pragma_Argument_Assoc_F_Expr
     (Node : Bare_Pragma_Argument_Assoc) return Bare_Expr
   is
   begin
         return Node.Pragma_Argument_Assoc_F_Expr;
   end;
--# property-start PragmaArgumentAssoc.assoc_expr /usr/src/packages/BUILD/ada/ast.py:7112
pragma Warnings (Off, "is not referenced");
 function Pragma_Argument_Assoc_P_Assoc_Expr
  (Node : Bare_Pragma_Argument_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Pragma_Argument_Assoc := Bare_Pragma_Argument_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Argument_Assoc :=
     Internal_Entity_Pragma_Argument_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 952 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7112
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Pragma_Argument_Assoc_F_Expr, Info => Ent.Info);
--# expr-done 952
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Argument_Assoc_P_Assoc_Expr;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Holder
   --
--# property-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.abstract_formal_params
                  Property_Result := Base_Subp_Spec_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_List =>
                  --# property-call-start ComponentList.abstract_formal_params
                  Property_Result := Component_List_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Unknown_Discriminant_Part =>
                  --# property-call-start DiscriminantPart.abstract_formal_params
                  Property_Result := Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Known_Discriminant_Part =>
                  --# property-call-start KnownDiscriminantPart.abstract_formal_params
                  Property_Result := Known_Discriminant_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Completion_Formal_Params =>
                  --# property-call-start EntryCompletionFormalParams.abstract_formal_params
                  Property_Result := Entry_Completion_Formal_Params_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Part =>
                  --# property-call-start GenericFormalPart.abstract_formal_params
                  Property_Result := Generic_Formal_Part_P_Abstract_Formal_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.unpacked_formal_params /usr/src/packages/BUILD/ada/ast.py:3105
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Unpacked_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Formal_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_209;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
            procedure Finalizer_Scope_209 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_209;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 954 '<FieldAccess .unpack_formals(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3106
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 953 '<FieldAccess .abstract_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3106
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 953
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 954
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_209;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_209;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Unpacked_Formal_Params;
--# end
--# property-start BaseFormalParamHolder.match_param_list /usr/src/packages/BUILD/ada/ast.py:3113
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_210;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_210 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_210;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 957 '<Block>' Let_Result None
--# scope-start
--# expr-start 956 '<FieldAccess .match_formals(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3115
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 955 '<FieldAccess .abstract_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3116
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 955
--# property-call-start AdaNode.match_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp);
--# end
--# expr-done 956
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 957
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_210;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_210;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Param_List;
--# end
--# property-start BaseFormalParamHolder.nb_min_params /usr/src/packages/BUILD/ada/ast.py:3119
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Min_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_211;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_211 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_211;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 963 '<length(<filter>)>' Len /usr/src/packages/BUILD/ada/ast.py:3120
--# expr-start 962 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3120
--# expr-start 959 '<FieldAccess .unpacked_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3120
--# expr-start 958 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3120
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 958
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 959
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld;
      begin
         for Item_41 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_41
               --# expr-start 961 '<FieldAccess .is_mandatory>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3121
--# expr-start 960 '<FieldAccess .spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3121
Fld_1 := Item_41.Spec;
--# expr-done 960
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_1.Node);
--# end
--# expr-done 961
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_41;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 962
Len := Length (Map_Result); 
--# expr-done 963
      Property_Result := Len;
   --# end
      Finalizer_Scope_211;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_211;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Nb_Min_Params;
--# end
--# property-start BaseFormalParamHolder.nb_max_params /usr/src/packages/BUILD/ada/ast.py:3129
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Nb_Max_Params
  (Node : Bare_Base_Formal_Param_Holder
  )
   return Integer
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Integer;
            procedure Finalizer_Scope_212;
      As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Single_Formal_Array_Access;
Len : Integer;
            procedure Finalizer_Scope_212 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_212;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 966 '<length(<FieldAccess .unpacked_formal_params>)>' Len /usr/src/packages/BUILD/ada/ast.py:3130
--# expr-start 965 '<FieldAccess .unpacked_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3130
--# expr-start 964 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3130
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 964
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 965
Len := Length (Fld); 
--# expr-done 966
      Property_Result := Len;
   --# end
      Finalizer_Scope_212;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_212;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Nb_Max_Params;
--# end
--# property-start BaseFormalParamHolder.paramless /usr/src/packages/BUILD/ada/ast.py:3138
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Paramless
  (Node : Bare_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Can_Be : Boolean
         := True
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind dottable_subp Dottable_Subp
   --# bind can_be Can_Be
   Property_Result : Boolean;
      Block_Var : Integer;
Fld : Integer;
Fld_1 : Integer;
If_Result : Integer;
Is_Equal : Boolean;
If_Result_1 : Boolean;
Is_Equal_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 972 '<Block>' Let_Result None
--# scope-start
--# expr-start 969 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:3144
if Can_Be then
   --# expr-start 967 '<FieldAccess .nb_min_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Self);
--# end
--# expr-done 967
   If_Result := Fld;
else
   --# expr-start 968 '<FieldAccess .nb_max_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Self);
--# end
--# expr-done 968
   If_Result := Fld_1;
end if;
--# expr-done 969
Block_Var := If_Result; 
--# bind nb_params Block_Var
if Dottable_Subp then
   --# expr-start 970 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:3146
Is_Equal := Block_Var = 1; 
--# expr-done 970
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 971 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:3147
Is_Equal_1 := Block_Var = 0; 
--# expr-done 971
   If_Result_2 := Is_Equal_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 972
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Paramless;
--# end
--# property-start BaseFormalParamHolder.is_matching_param_list /usr/src/packages/BUILD/ada/ast.py:3151
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Is_Matching_Param_List
  (Node : Bare_Base_Formal_Param_Holder
      ; Params : Internal_Entity_Assoc_List
      ; Is_Dottable_Subp : Boolean
      ; Env : Lexical_Env
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind params Params
   --# bind is_dottable_subp Is_Dottable_Subp
   --# bind env Env
   Property_Result : Boolean;
            procedure Finalizer_Scope_214;
      Block_Var : Internal_Entity_Base_Formal_Param_Holder;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Integer;
Block_Var_3 : Integer;
As_Entity : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Fld : Internal_Param_Match_Array_Access;
Fld_1 : Integer;
Arith_Result : Integer;
Fld_2 : Integer;
If_Result_1 : Integer;
Fld_3 : Integer;
Arith_Result_1 : Integer;
Fld_4 : Integer;
If_Result_2 : Integer;
Fld_5 : Bare_Assoc_List;
Len : Integer;
Comp_Result : Boolean;
Fld_6 : Boolean;
Quantifier_Result : Boolean;
If_Result_3 : Boolean;
Fld_7 : Internal_Single_Formal;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl;
Fld_9 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Len_1 : Integer;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_214 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_214;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 991 '<Block>' Let_Result None
--# scope-start
--# expr-start 973 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3157
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Holder;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 973
Block_Var := If_Result; 
--# bind bare Block_Var
--# expr-start 974 '<FieldAccess .match_param_list(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:3158
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.match_param_list
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Block_Var.Node, Params => Params, Is_Dottable_Subp => Is_Dottable_Subp, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 974
Block_Var_1 := Fld; Inc_Ref (Block_Var_1);
--# bind match_list Block_Var_1
--# expr-start 978 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:3160
if Is_Dottable_Subp then
   --# expr-start 976 '<Op ->' Arith_Result /usr/src/packages/BUILD/ada/ast.py:3160
--# expr-start 975 '<FieldAccess .nb_max_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3160
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 975
Arith_Result := (Fld_1 - 1); 
--# expr-done 976
   If_Result_1 := Arith_Result;
else
   --# expr-start 977 '<FieldAccess .nb_max_params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3160
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Block_Var.Node);
--# end
--# expr-done 977
   If_Result_1 := Fld_2;
end if;
--# expr-done 978
Block_Var_2 := If_Result_1; 
--# bind nb_max_params Block_Var_2
--# expr-start 982 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:3163
if Is_Dottable_Subp then
   --# expr-start 980 '<Op ->' Arith_Result_1 /usr/src/packages/BUILD/ada/ast.py:3163
--# expr-start 979 '<FieldAccess .nb_min_params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3163
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 979
Arith_Result_1 := (Fld_3 - 1); 
--# expr-done 980
   If_Result_2 := Arith_Result_1;
else
   --# expr-start 981 '<FieldAccess .nb_min_params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3163
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_min_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Min_Params (Node => Block_Var.Node);
--# end
--# expr-done 981
   If_Result_2 := Fld_4;
end if;
--# expr-done 982
Block_Var_3 := If_Result_2; 
--# bind nb_min_params Block_Var_3
--# expr-start 983 '<length(<Var params>)>' Len /usr/src/packages/BUILD/ada/ast.py:3167
Fld_5 := Params.Node; 
Len := Length (Fld_5); 
--# expr-done 983
Comp_Result := Len <= Block_Var_2; 
if Comp_Result then
Quantifier_Result := True;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
   begin
      for Item_42 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_42
         --# expr-start 984 '<FieldAccess .has_matched>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3168
Fld_6 := Item_42.Has_Matched;
--# expr-done 984
            if not (Fld_6) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_3 := Quantifier_Result;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 990 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:3169
--# expr-start 989 '<length(<filter>)>' Len_1 /usr/src/packages/BUILD/ada/ast.py:3169
--# expr-start 988 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3169
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_1;
      begin
         for Item_43 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_43
               --# expr-start 987 '<FieldAccess .is_mandatory>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3170
--# expr-start 986 '<FieldAccess .spec>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3170
--# expr-start 985 '<FieldAccess .formal>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3170
Fld_7 := Item_43.Formal;
--# expr-done 985
Fld_8 := Fld_7.Spec;
--# expr-done 986
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamDecl.is_mandatory'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory (Node => Fld_8.Node);
--# end
--# expr-done 987
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_43;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 988
Len_1 := Length (Map_Result); 
--# expr-done 989
Is_Equal := Len_1 = Block_Var_3; 
--# expr-done 990
   If_Result_4 := Is_Equal;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 991
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_214;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_214;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Is_Matching_Param_List;
--# end
--# property-start BaseFormalParamHolder.param_types /usr/src/packages/BUILD/ada/ast.py:3176
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Param_Types
  (Node : Bare_Base_Formal_Param_Holder
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_215;
      Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_215 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_215;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 997 '<Block>' Let_Result None
--# scope-start
--# expr-start 996 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3180
--# expr-start 992 '<FieldAccess .unpacked_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3180
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 992
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld;
      begin
         for Item_44 of
               Collection.Items
         loop
   --# scope-start
               --# bind fp Item_44
      --# expr-start 995 '<FieldAccess .real_designated_type(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3181
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 994 '<FieldAccess .type_expression>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3181
--# expr-start 993 '<FieldAccess .spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3181
Fld_1 := Item_44.Spec;
--# expr-done 993
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 994
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld_2, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 995
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 996
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 997
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_215;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_215;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Param_Types;
--# end
--# property-start BaseFormalParamHolder.real_type /usr/src/packages/BUILD/ada/ast.py:3186
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity;
Block_Var_2 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Internal_Entity_Anonymous_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Internal_Entity_Info;
Fld_6 : Boolean;
Fld_7 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Fld_12 : Bare_Ada_Node;
Fld_13 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Fld_14 : Internal_Entity_Info;
Fld_15 : Internal_Metadata;
Fld_16 : Bare_Ada_Node;
Scope_Result_3 : Bare_Ada_Node;
Fld_17 : Bare_Base_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Result_Var_3 : Bare_Ada_Node;
Fld_18 : Internal_Entity_Info;
Fld_19 : Env_Rebindings;
Fld_20 : Internal_Entity_Info;
Fld_21 : Boolean;
Fld_22 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Anonymous_Type_Decl;
Fld_23 : Internal_Entity_Type_Def;
Is_A : Boolean;
Not_Val : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Internal_Entity_Base_Type_Decl;
Fld_26 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_6 : Internal_Entity_Base_Type_Decl;
Result_Var_6 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1034 '<Block>' Let_Result None
--# scope-start
--# expr-start 1006 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3193
--# expr-start 1005 '<FieldAccess .entity_no_md(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3193
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1000 '<FieldAccess .primitive>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3194
--# expr-start 999 '<FieldAccess .md>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3194
--# expr-start 998 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:3194
Fld := Ent.Info;
--# expr-done 998
Fld_1 := Fld.MD;
--# expr-done 999
Fld_2 := Fld_1.Primitive;
--# expr-done 1000
--# expr-start 1002 '<FieldAccess .rebindings>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3195
--# expr-start 1001 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3195
Fld_3 := Ent.Info;
--# expr-done 1001
Fld_4 := Fld_3.Rebindings;
--# expr-done 1002
--# expr-start 1004 '<FieldAccess .from_rebound>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3196
--# expr-start 1003 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3196
Fld_5 := Ent.Info;
--# expr-done 1003
Fld_6 := Fld_5.From_Rebound;
--# expr-done 1004
--# property-call-start AdaNode.entity_no_md
Fld_7 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Fld_2, Rebindings => Fld_4, From_Rebound => Fld_6);
--# end
--# expr-done 1005
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1006
Block_Var := Cast_Result; 
--# bind prim_type Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1007 '<FieldAccess .canonical_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3201
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1007
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1008 '<FieldAccess .without_md>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3201
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_9 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1008
Scope_Result_1 := Fld_9;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Block_Var_1 := Result_Var_1; 
--# bind canon_prim_type Block_Var_1
Var_Expr_2 := Typ;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_2
--# expr-start 1010 '<FieldAccess .without_md>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3203
--# expr-start 1009 '<FieldAccess .canonical_type>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3203
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1009
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_11 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 1010
Scope_Result_2 := Fld_11;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
Block_Var_2 := Result_Var_2; 
--# bind canon_typ Block_Var_2
--# expr-start 1033 '<Cond>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:3205
--# expr-start 1013 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:3206
--# expr-start 1011 '<FieldAccess .node>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3206
Fld_12 := Block_Var_1.Node;
--# expr-done 1011
      Cast_Result_1 := Fld_12;
--# expr-start 1012 '<FieldAccess .node>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:3206
Fld_13 := Block_Var_2.Node;
--# expr-done 1012
Is_Equal := Cast_Result_1 = Fld_13; 
--# expr-done 1013
if Is_Equal then
   --# expr-start 1023 '<Cast to BaseTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:3208
--# expr-start 1022 '<FieldAccess .entity_no_md(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:3208
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1016 '<FieldAccess .primitive_real_type>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:3209
--# expr-start 1015 '<FieldAccess .md>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:3209
--# expr-start 1014 '<FieldAccess .info>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:3209
Fld_14 := Ent.Info;
--# expr-done 1014
Fld_15 := Fld_14.MD;
--# expr-done 1015
Fld_16 := Fld_15.Primitive_Real_Type;
--# expr-done 1016
Var_Expr_3 := Fld_16;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_3 := Var_Expr_3;
   --# end
   Result_Var_3 := Scope_Result_3;
else
--# expr-start 1017 '<FieldAccess .node>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:3209
Fld_17 := Typ.Node;
--# expr-done 1017
      Cast_Result_2 := Fld_17;
   Result_Var_3 := Cast_Result_2;
end if;
--# expr-start 1019 '<FieldAccess .rebindings>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:3210
--# expr-start 1018 '<FieldAccess .info>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:3210
Fld_18 := Ent.Info;
--# expr-done 1018
Fld_19 := Fld_18.Rebindings;
--# expr-done 1019
--# expr-start 1021 '<FieldAccess .from_rebound>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:3211
--# expr-start 1020 '<FieldAccess .info>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:3211
Fld_20 := Ent.Info;
--# expr-done 1020
Fld_21 := Fld_20.From_Rebound;
--# expr-done 1021
--# property-call-start AdaNode.entity_no_md
Fld_22 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Result_Var_3, Rebindings => Fld_19, From_Rebound => Fld_21);
--# end
--# expr-done 1022
   if Fld_22.Node = null
      or else Fld_22.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_22.Node,
         Info => Fld_22.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1023
   If_Result_2 := Cast_Result_3;
else
--# expr-start 1024 '<Cast to AnonymousTypeDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:3217
Cast_Expr := Typ; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_4 := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 1024
Var_Expr_4 := Cast_Result_4;
if Var_Expr_4 /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
   --# bind td Var_Expr_4
--# expr-start 1027 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:3218
--# expr-start 1026 '<IsA AccessToSubpDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:3218
--# expr-start 1025 '<FieldAccess .type_def>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:3218
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Type_Def (Node => Var_Expr_4.Node.Type_Decl_F_Type_Def, Info => Var_Expr_4.Info);
--# expr-done 1025
Is_A := Fld_23.Node /= null 
and then Fld_23.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 1026
Not_Val := not (Is_A); 
--# expr-done 1027
Scope_Result_4 := Not_Val;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
--# expr-start 1028 '<FieldAccess .accessed_type>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:3220
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_24 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1028
Var_Expr_5 := Fld_24;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_5
--# expr-start 1029 '<FieldAccess .real_type(...)>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:3221
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_25 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr_5, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1029
Var_Expr_6 := Fld_25;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind rat Var_Expr_6
--# expr-start 1032 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:3222
--# expr-start 1030 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:3223
Cast_Expr_1 := Var_Expr_5; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr_6; 
      Cast_Result_6 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_5, Cast_Result_6); 
--# expr-done 1030
if Is_Equiv then
   If_Result := Typ;
else
   --# expr-start 1031 '<FieldAccess .anonymous_access_type>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:3225
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_26 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 1031
   If_Result := Fld_26;
end if;
--# expr-done 1032
Scope_Result_5 := If_Result;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_6 := Result_Var_5;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var_6;
else
   If_Result_1 := Typ;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 1033
Let_Result := If_Result_2; 
--# end
--# expr-done 1034
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Real_Type;
--# end
--# property-start BaseFormalParamHolder.real_designated_type /usr/src/packages/BUILD/ada/ast.py:3235
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Real_Designated_Type
  (Node : Bare_Base_Formal_Param_Holder
      ; Typ : Internal_Entity_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Env_Rebindings;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Rt : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Internal_Metadata;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Type_Decl;
Fld_6 : Env_Rebindings;
Scope_Result : Env_Rebindings;
Result_Var : Env_Rebindings;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Bare_Type_Expr;
Fld_10 : Internal_Entity_Info;
Fld_11 : Internal_Metadata;
Fld_12 : Internal_Entity_Info;
Fld_13 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_14 : Internal_Entity_Info;
Fld_15 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Type_Expr;
Fld_16 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Info;
Fld_18 : Internal_Metadata;
Fld_19 : Bare_Ada_Node;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1066 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 1039 '<as_bare_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3294
--# expr-start 1038 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3294
--# expr-start 1037 '<FieldAccess .primitive_real_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3294
--# expr-start 1036 '<FieldAccess .md>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3294
--# expr-start 1035 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:3294
Fld := Ent.Info;
--# expr-done 1035
Fld_1 := Fld.MD;
--# expr-done 1036
Fld_2 := Fld_1.Primitive_Real_Type;
--# expr-done 1037
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_2;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1038
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1039
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1044 '<FieldAccess .find_base_type_rebindings(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3294
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1043 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:3296
--# expr-start 1042 '<FieldAccess .primitive>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3296
--# expr-start 1041 '<FieldAccess .md>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3296
--# expr-start 1040 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3296
Fld_3 := Ent.Info;
--# expr-done 1040
Fld_4 := Fld_3.MD;
--# expr-done 1041
Fld_5 := Fld_4.Primitive;
--# expr-done 1042
   if Fld_5 = null
      or else Fld_5.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Fld_5;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1043
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Var_Expr.Node, Target => Cast_Result_1, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1044
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := null;
end if;
Block_Var := Result_Var; 
--# bind base_rebindings Block_Var
--# expr-start 1045 '<FieldAccess .designated_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3300
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 1045
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 1065 '<Let rt>' Let_Result /usr/src/packages/BUILD/ada/ast.py:3301
--# scope-start
--# expr-start 1046 '<FieldAccess .real_type(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3302
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.real_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Type (Node => Ent.Node, Typ => Var_Expr_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1046
Rt := Fld_8; 
--# bind rt Rt
--# expr-start 1064 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:3303
--# expr-start 1047 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:3304
Cast_Expr := Var_Expr_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Rt; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1047
if Is_Equiv then
   --# expr-start 1063 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:3305
--# expr-start 1051 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:3306
--# expr-start 1050 '<FieldAccess .primitive_real_type>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:3306
--# expr-start 1049 '<FieldAccess .md>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:3306
--# expr-start 1048 '<FieldAccess .info>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:3306
Fld_17 := Ent.Info;
--# expr-done 1048
Fld_18 := Fld_17.MD;
--# expr-done 1049
Fld_19 := Fld_18.Primitive_Real_Type;
--# expr-done 1050
Is_Null_1 := Fld_19 = null; 
--# expr-done 1051
if Is_Null_1 then
   If_Result_1 := Var_Expr_1;
else
   --# expr-start 1062 '<FieldAccess .designated_type>' Fld_16 None
--# expr-start 1061 '<New InternalEntityTypeExpr>' New_Struct_1 None
--# expr-start 1059 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:3310
--# expr-start 1053 '<FieldAccess .from_rebound>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:3314
--# expr-start 1052 '<FieldAccess .info>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:3314
Fld_14 := Typ.Info;
--# expr-done 1052
Fld_15 := Fld_14.From_Rebound;
--# expr-done 1053
--# expr-start 1055 '<FieldAccess .md>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3311
--# expr-start 1054 '<FieldAccess .info>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3311
Fld_10 := Typ.Info;
--# expr-done 1054
Fld_11 := Fld_10.MD;
--# expr-done 1055
--# expr-start 1058 '<concat_rebindings(<FieldAccess .rebindings>, <Var base_rebindings>)>' Rebinding /usr/src/packages/BUILD/ada/ast.py:3312
--# expr-start 1057 '<FieldAccess .rebindings>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:3312
--# expr-start 1056 '<FieldAccess .info>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3312
Fld_12 := Typ.Info;
--# expr-done 1056
Fld_13 := Fld_12.Rebindings;
--# expr-done 1057
Rebinding := AST_Envs.Combine (Fld_13, Block_Var); 
--# expr-done 1058
New_Struct := (From_Rebound => Fld_15, MD => Fld_11, Rebindings => Rebinding); 
--# expr-done 1059
--# expr-start 1060 '<FieldAccess .node>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3309
Fld_9 := Typ.Node;
--# expr-done 1060
New_Struct_1 := (Info => New_Struct, Node => Fld_9); 
--# expr-done 1061
      if New_Struct_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => New_Struct_1.Node, Origin => Origin, E_Info => New_Struct_1.Info);
--# end
--# expr-done 1062
   If_Result_1 := Fld_16;
end if;
--# expr-done 1063
   If_Result_2 := If_Result_1;
else
   If_Result_2 := Rt;
end if;
--# expr-done 1064
Let_Result := If_Result_2; 
--# end
--# expr-done 1065
Scope_Result_1 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result_1 := Result_Var_1; 
--# end
--# expr-done 1066
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Real_Designated_Type;
--# end
--# property-start BaseFormalParamHolder.call_argument_equation /usr/src/packages/BUILD/ada/ast.py:3324
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Call_Argument_Equation
  (Node : Bare_Base_Formal_Param_Holder
      ; Param : Internal_Entity_Base_Formal_Param_Decl
      ; Arg : Internal_Entity_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind param Param
   --# bind arg Arg
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_218;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_218 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_218;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1071 '<Block>' Let_Result None
--# scope-start
--# expr-start 1070 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:3332
--# expr-start 1067 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:3333
      if Arg.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Arg.Node);
--# expr-done 1067
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
--# expr-start 1069 '<FieldAccess .real_designated_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3334
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1068 '<FieldAccess .type_expression>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3334
      if Param.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Param.Node, E_Info => Param.Info);
--# end
--# expr-done 1068
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Fld_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1069
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_564.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_564'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3332") else null)); 
--# expr-done 1070
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1071
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_218;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_218;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Call_Argument_Equation;
--# end
--# property-start BaseFormalParamHolder.match_formal_params /usr/src/packages/BUILD/ada/ast.py:3339
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Formal_Params
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
            procedure Finalizer_Scope_219;
      Block_Var : Internal_Single_Formal_Array_Access;
Block_Var_1 : Internal_Single_Formal_Array_Access;
Block_Var_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Block_Var_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_4 : Bare_Base_Formal_Param_Holder;
Fld_5 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Len : Integer;
Len_1 : Integer;
Is_Equal : Boolean;
Fld_7 : Bare_Ada_Node;
Bound_Origin_2 : Bare_Ada_Node;
Not_Val : Boolean;
Fld_8 : Internal_Entity_Defining_Name;
Get_Result : Internal_Single_Formal;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_11 : Boolean;
If_Result : Boolean;
Get_Result_1 : Internal_Entity_Base_Type_Decl;
Get_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Boolean;
I : Integer;
Quantifier_Result : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_219 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Block_Var_3);
            end Finalizer_Scope_219;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1095 '<Block>' Let_Result None
--# scope-start
--# expr-start 1072 '<FieldAccess .unpacked_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:3346
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1072
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind self_params Block_Var
--# expr-start 1073 '<FieldAccess .unpacked_formal_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3347
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1073
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind other_params Block_Var_1
--# expr-start 1076 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .param_types>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:3349
--# expr-start 1074 '<FieldAccess .origin_node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3349
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1074
Bound_Origin := Fld_2; 
--# expr-start 1075 '<FieldAccess .param_types>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3349
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1075
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 1076
Block_Var_2 := Dyn_Var_Bind_Result; Inc_Ref (Block_Var_2);
--# bind self_types Block_Var_2
--# expr-start 1080 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .param_types>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:3350
--# expr-start 1078 '<FieldAccess .origin_node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3350
--# expr-start 1077 '<FieldAccess .node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3350
Fld_4 := Other.Node;
--# expr-done 1077
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_4);
--# end
--# expr-done 1078
Bound_Origin_1 := Fld_5; 
--# expr-start 1079 '<FieldAccess .param_types>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3351
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.param_types
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Param_Types (Node => Other.Node, Origin => Bound_Origin_1, E_Info => Other.Info);
--# end
--# expr-done 1079
Dyn_Var_Bind_Result_1 := Fld_6; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 1080
Block_Var_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Block_Var_3);
--# bind other_types Block_Var_3
--# expr-start 1083 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:3353
--# expr-start 1081 '<length(<Var self_params>)>' Len /usr/src/packages/BUILD/ada/ast.py:3353
Len := Length (Block_Var); 
--# expr-done 1081
--# expr-start 1082 '<length(<Var other_params>)>' Len_1 /usr/src/packages/BUILD/ada/ast.py:3353
Len_1 := Length (Block_Var_1); 
--# expr-done 1082
Is_Equal := Len = Len_1; 
--# expr-done 1083
if Is_Equal then
   --# expr-start 1094 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <AllQuantifier>)>' Dyn_Var_Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:3354
--# expr-start 1084 '<FieldAccess .origin_node>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3354
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_7 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1084
Bound_Origin_2 := Fld_7; 
Quantifier_Result := True;
      I := 0;
   declare
      Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
   begin
      for Item_45 of
            Collection.Items
      loop
   --# scope-start
            --# bind p Item_45
            --# bind i I
--# expr-start 1085 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:3356
Not_Val := not (Match_Names); 
--# expr-done 1085
if Not_Val then
   If_Result := True;
else
   --# expr-start 1090 '<FieldAccess .matches(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3357
--# expr-start 1086 '<FieldAccess .name>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3357
Fld_8 := Item_45.Name;
--# expr-done 1086
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1089 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3357
--# expr-start 1088 '<FieldAccess .name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3357
--# expr-start 1087 '<at(<Var other_params>, <Var i>)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:3357
Get_Result := Get (Block_Var_1, I, True); 
--# expr-done 1087
Fld_9 := Get_Result.Name;
--# expr-done 1088
Fld_10 := Fld_9.Node;
--# expr-done 1089
      Cast_Result := Fld_10;
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => Fld_8.Node, N => Cast_Result);
--# end
--# expr-done 1090
   If_Result := Fld_11;
end if;
if If_Result then
--# expr-start 1091 '<at(<Var self_types>, <Var i>)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:3358
Get_Result_1 := Get (Block_Var_2, I, True); 
--# expr-done 1091
Var_Expr := Get_Result_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1093 '<FieldAccess .matching_type(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3358
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1092 '<at(<Var other_types>, <Var i>)>' Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:3358
Get_Result_2 := Get (Block_Var_3, I, True); 
--# expr-done 1092
--# property-call-start BaseTypeDecl.matching_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr.Node, Expected_Type => Get_Result_2, Origin => Bound_Origin_2, E_Info => Var_Expr.Info);
--# end
--# expr-done 1093
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_1 := Result_Var;
else
   If_Result_1 := False;
end if;
            if not (If_Result_1) then
               Quantifier_Result := False;
               exit;
            end if;
            I := I + 1;
   --# end
      end loop;
   end;
Dyn_Var_Bind_Result_2 := Quantifier_Result; 
--# expr-done 1094
   If_Result_2 := Dyn_Var_Bind_Result_2;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1095
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_219;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_219;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Formal_Params;
--# end
--# property-start '[dispatcher]BaseFormalParamHolder.match_other' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Holder (Self.Kind) is
               when Ada_Component_List .. Ada_Generic_Formal_Part =>
                  --# property-call-start BaseFormalParamHolder.match_other
                  Property_Result := Base_Formal_Param_Holder_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Spec .. Ada_Subp_Spec =>
                  --# property-call-start BaseSubpSpec.match_other
                  Property_Result := Base_Subp_Spec_P_Match_Other
                    (Self
                        , Other
                        , Match_Names
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Holder_P_Match_Other;
--# end
--# property-start BaseFormalParamHolder.match_other /usr/src/packages/BUILD/ada/ast.py:3363
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Holder_P_Match_Other
  (Node : Bare_Base_Formal_Param_Holder
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Holder := Bare_Base_Formal_Param_Holder (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Holder :=
     Internal_Entity_Base_Formal_Param_Holder'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1097 '<Block>' Let_Result None
--# scope-start
--# expr-start 1096 '<FieldAccess .match_formal_params(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:3371
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Ent.Node, Other => Other, Match_Names => Match_Names, E_Info => Ent.Info);
--# end
--# expr-done 1096
Let_Result := Fld; 
--# end
--# expr-done 1097
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Holder_P_Match_Other;
--# end
   --
   --  Primitives for Bare_Base_Subp_Spec
   --
--# property-start '[dispatcher]BaseSubpSpec.name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Name
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.name
                  Property_Result := Entry_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.name
                  Property_Result := Enum_Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.name
                  Property_Result := Subp_Spec_P_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Name;
--# end
--# property-start '[dispatcher]BaseSubpSpec.returns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Returns
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.returns
                  Property_Result := Entry_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.returns
                  Property_Result := Enum_Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.returns
                  Property_Result := Subp_Spec_P_Returns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Returns;
--# end
--# property-start '[dispatcher]BaseSubpSpec.params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subp_Spec_P_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subp_Spec (Self.Kind) is
               when Ada_Entry_Spec =>
                  --# property-call-start EntrySpec.params
                  Property_Result := Entry_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Subp_Spec =>
                  --# property-call-start EnumSubpSpec.params
                  Property_Result := Enum_Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Spec =>
                  --# property-call-start SubpSpec.params
                  Property_Result := Subp_Spec_P_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subp_Spec_P_Params;
--# end
--# property-start BaseSubpSpec.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:12671
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Abstract_Formal_Params
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_975;
      Fld : Internal_Entity_Param_Spec_Array_Access;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_975 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_975;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1100 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12672
--# expr-start 1098 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:12672
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1098
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Param_Spec_Array_Access := Fld;
      begin
         for Item_46 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_46
      --# expr-start 1099 '<Cast to BaseFormalParamDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12672
Cast_Expr := Item_46; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1099
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1100
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_975;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_975;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Abstract_Formal_Params;
--# end
--# property-start BaseSubpSpec.match_return_type /usr/src/packages/BUILD/ada/ast.py:12676
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_4 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null_1 : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Type_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Fld_8 : Internal_Entity_Type_Expr;
Is_Null_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_9 : Bare_Ada_Node;
Bound_Origin_2 : Bare_Ada_Node;
Fld_10 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1121 '<Block>' Let_Result None
--# scope-start
--# expr-start 1103 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .return_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:12682
--# expr-start 1101 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:12682
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1101
Bound_Origin := Fld; 
--# expr-start 1102 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12682
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1102
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 1103
Block_Var := Dyn_Var_Bind_Result; 
--# bind self_ret Block_Var
--# expr-start 1107 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .return_type>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:12683
--# expr-start 1105 '<FieldAccess .origin_node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12683
--# expr-start 1104 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12683
Fld_2 := Other.Node;
--# expr-done 1104
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_2);
--# end
--# expr-done 1105
Bound_Origin_1 := Fld_3; 
--# expr-start 1106 '<FieldAccess .return_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12683
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Other.Node, Origin => Bound_Origin_1, E_Info => Other.Info);
--# end
--# expr-done 1106
Dyn_Var_Bind_Result_1 := Fld_4; 
--# expr-done 1107
Block_Var_1 := Dyn_Var_Bind_Result_1; 
--# bind other_ret Block_Var_1
--# expr-start 1109 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12685
--# expr-start 1108 '<FieldAccess .returns>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12685
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1108
Is_Null := Fld_5.Node = null; 
--# expr-done 1109
if Is_Null then
   --# expr-start 1111 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:12685
--# expr-start 1110 '<FieldAccess .returns>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12685
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1110
Is_Null_1 := Fld_6.Node = null; 
--# expr-done 1111
   If_Result := Is_Null_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_3 := True;
else
--# expr-start 1114 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12687
--# expr-start 1113 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:12687
--# expr-start 1112 '<FieldAccess .returns>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12687
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1112
Is_Null_2 := Fld_7.Node = null; 
--# expr-done 1113
Not_Val := not (Is_Null_2); 
--# expr-done 1114
if Not_Val then
   --# expr-start 1117 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:12687
--# expr-start 1116 '<IsNull>' Is_Null_3 /usr/src/packages/BUILD/ada/ast.py:12687
--# expr-start 1115 '<FieldAccess .returns>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12687
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1115
Is_Null_3 := Fld_8.Node = null; 
--# expr-done 1116
Not_Val_1 := not (Is_Null_3); 
--# expr-done 1117
   If_Result_1 := Not_Val_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 1120 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <Var self_ret>: <Var Var_Expr> <FieldAccess .matching_type(...)>>)>' Dyn_Var_Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:12688
--# expr-start 1118 '<FieldAccess .origin_node>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12688
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_9 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1118
Bound_Origin_2 := Fld_9; 
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1119 '<FieldAccess .matching_type(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:12689
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Var_Expr.Node, Expected_Type => Block_Var_1, Origin => Bound_Origin_2, E_Info => Var_Expr.Info);
--# end
--# expr-done 1119
Scope_Result := Fld_10;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Dyn_Var_Bind_Result_2 := Result_Var; 
--# expr-done 1120
   If_Result_2 := Dyn_Var_Bind_Result_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 1121
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Return_Type;
--# end
--# property-start BaseSubpSpec.match_signature /usr/src/packages/BUILD/ada/ast.py:12694
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Signature
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Subp_Spec
      ; Match_Name : Boolean
      ; Use_Entity_Info : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_name Match_Name
   --# bind use_entity_info Use_Entity_Info
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Subp_Spec;
As_Entity : Internal_Entity_Base_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Fld : Bare_Defining_Name;
Fld_1 : Bare_Defining_Name;
Cast_Result : Bare_Name;
Fld_2 : Boolean;
If_Result_2 : Boolean;
Fld_3 : Boolean;
If_Result_3 : Boolean;
Cast_Expr : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
If_Result_4 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1130 '<Block>' Let_Result None
--# scope-start
--# expr-start 1123 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12710
if Use_Entity_Info then
   If_Result_1 := Ent;
else
   --# expr-start 1122 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:12710
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Base_Subp_Spec;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1122
   If_Result_1 := If_Result;
end if;
--# expr-done 1123
Block_Var := If_Result_1; 
--# bind ent Block_Var
--# expr-start 1124 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12713
Not_Val := not (Match_Name); 
--# expr-done 1124
if Not_Val then
   If_Result_2 := True;
else
   --# expr-start 1127 '<FieldAccess .matches(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12713
--# expr-start 1125 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:12713
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1125
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1126 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12713
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1126
      Cast_Result := Fld_1;
--# property-call-start Name.matches
Fld_2 := Libadalang.Implementation.Name_P_Matches (Node => Fld, N => Cast_Result);
--# end
--# expr-done 1127
   If_Result_2 := Fld_2;
end if;
if If_Result_2 then
   --# expr-start 1128 '<FieldAccess .match_return_type(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12714
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_return_type
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Return_Type (Node => Block_Var.Node, Other => Other, E_Info => Block_Var.Info);
--# end
--# expr-done 1128
   If_Result_3 := Fld_3;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   --# expr-start 1129 '<FieldAccess .match_formal_params(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12715
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Other; 
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Block_Var.Node, Other => Cast_Result_1, Match_Names => Match_Name, E_Info => Block_Var.Info);
--# end
--# expr-done 1129
   If_Result_4 := Fld_4;
else
   If_Result_4 := False;
end if;
Let_Result := If_Result_4; 
--# end
--# expr-done 1130
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Signature;
--# end
--# property-start BaseSubpSpec.match_other /usr/src/packages/BUILD/ada/ast.py:12719
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Match_Other
  (Node : Bare_Base_Subp_Spec
      ; Other : Internal_Entity_Base_Formal_Param_Holder
      ; Match_Names : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind match_names Match_Names
   Property_Result : Boolean;
      Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1133 '<Block>' Let_Result None
--# scope-start
--# expr-start 1132 '<FieldAccess .match_signature(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:12721
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1131 '<Cast to BaseSubpSpec>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12722
Cast_Expr := Other; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1131
--# property-call-start BaseSubpSpec.match_signature
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Ent.Node, Other => Cast_Result, Match_Name => Match_Names, E_Info => Ent.Info);
--# end
--# expr-done 1132
Let_Result := Fld; 
--# end
--# expr-done 1133
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Match_Other;
--# end
--# property-start BaseSubpSpec.defining_env /usr/src/packages/BUILD/ada/ast.py:12727
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Defining_Env
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_979;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_979 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_979;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1139 '<Block>' Let_Result None
--# scope-start
--# expr-start 1138 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12731
--# expr-start 1135 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12731
--# expr-start 1134 '<FieldAccess .returns>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12731
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1134
Is_Null := Fld_2.Node = null; 
--# expr-done 1135
if Is_Null then
   If_Result := AST_Envs.Empty_Env;
else
   --# expr-start 1137 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12732
--# expr-start 1136 '<FieldAccess .return_type>' Fld /usr/src/packages/BUILD/ada/ast.py:12732
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1136
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1137
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1138
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1139
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_979;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_979;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Defining_Env;
--# end
--# property-start BaseSubpSpec.potential_dottable_type /usr/src/packages/BUILD/ada/ast.py:12735
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Potential_Dottable_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_980;
      Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Param_Spec;
Fld : Internal_Entity_Param_Spec_Array_Access;
Get_Result : Internal_Entity_Param_Spec;
Fld_1 : Internal_Entity_Type_Expr;
Scope_Result : Internal_Entity_Type_Expr;
Result_Var : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_980 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_980;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1144 '<Block>' Let_Result None
--# scope-start
--# expr-start 1141 '<at(<FieldAccess .params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12740
--# expr-start 1140 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:12740
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1140
Get_Result := Get (Fld, 0, True); 
--# expr-done 1141
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 1142 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12740
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Var_Expr_1.Node.Param_Spec_F_Type_Expr, Info => Var_Expr_1.Info);
--# expr-done 1142
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Expr;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 1143 '<FieldAccess .element_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12740
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.element_type
Fld_2 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1143
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 1144
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_980;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_980;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Potential_Dottable_Type;
--# end
--# property-start BaseSubpSpec.candidate_type_for_primitive /usr/src/packages/BUILD/ada/ast.py:12743
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Candidate_Type_For_Primitive
  (Node : Bare_Base_Subp_Spec
      ; Type_Expr : Internal_Entity_Type_Expr
      ; Canonicalize : Boolean
         := True
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind type_expr Type_Expr
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Block_Var_4 : Bare_Declarative_Part;
Match_0 : Internal_Entity_Anonymous_Type;
Match_1 : Internal_Entity_Type_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Boolean;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Incomplete_Type_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Type_Decl;
Var_Expr_6 : Bare_Declarative_Part;
Var_Expr_7 : Bare_Declarative_Part;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Var_Expr_9 : Bare_Declarative_Part;
Var_Expr_10 : Bare_Base_Package_Decl;
Var_Expr_11 : Bare_Declarative_Part;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Declarative_Part;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Not_Val : Boolean;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Expr;
Cast_Result_1 : Internal_Entity_Anonymous_Type;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_7 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_10 : Bare_Base_Type_Decl;
Fld_11 : Bare_Ada_Node;
Fld_12 : Bare_Ada_Node;
Cast_Result_4 : Bare_Declarative_Part;
Scope_Result_6 : Bare_Declarative_Part;
Result_Var_4 : Bare_Declarative_Part;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity_Type_Decl;
Fld_13 : Internal_Entity_Type_Def;
Scope_Result_7 : Internal_Entity_Type_Def;
Result_Var_5 : Internal_Entity_Type_Def;
Is_A : Boolean;
Fld_14 : Bare_Ada_Node;
Scope_Result_8 : Bare_Ada_Node;
Result_Var_6 : Bare_Ada_Node;
Is_A_1 : Boolean;
Fld_15 : Bare_Ada_Node;
Scope_Result_9 : Bare_Ada_Node;
Result_Var_7 : Bare_Ada_Node;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_16 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_8 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
Cast_Expr_2 : Bare_Declarative_Part;
Cast_Result_6 : Bare_Declarative_Part;
Is_Equal : Boolean;
Fld_17 : Bare_Ada_Node;
Scope_Result_11 : Bare_Ada_Node;
Result_Var_9 : Bare_Ada_Node;
Cast_Result_7 : Bare_Base_Package_Decl;
Fld_18 : Bare_Public_Part;
Scope_Result_12 : Bare_Public_Part;
Result_Var_10 : Bare_Public_Part;
Cast_Result_8 : Bare_Declarative_Part;
Is_Equal_1 : Boolean;
If_Result_4 : Boolean;
Scope_Result_13 : Boolean;
Result_Var_11 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1181 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1148 '<Cast to DeclarativePart>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12754
--# expr-start 1147 '<parent(<parent(<parent(<Var Self>)>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12754
--# expr-start 1146 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12754
--# expr-start 1145 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:12754
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1145
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1146
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 1147
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Fld_2;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1148
Block_Var := Cast_Result; 
--# bind decl_scope Block_Var
--# expr-start 1155 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:12756
--# expr-start 1149 '<FieldAccess .origin_node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12756
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1149
Bound_Origin := Fld_3; 
--# expr-start 1154 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:12756
      if Type_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Type_Expr; 
case Ada_Type_Expr (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Anonymous_Type
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind at Match_0
--# expr-start 1150 '<FieldAccess .element_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12757
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.element_type
Fld_4 := Libadalang.Implementation.Type_Expr_P_Element_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 1150
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind et Var_Expr
--# expr-start 1152 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12760
--# expr-start 1151 '<FieldAccess .is_classwide>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12760
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1151
Not_Val := not (Fld_5); 
--# expr-done 1152
Var_Expr_1 := Not_Val;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Enum_Lit_Synth_Type_Expr .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind other Match_1
--# expr-start 1153 '<FieldAccess .designated_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12764
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 1153
Let_Result_1 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 1154
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 1155
Block_Var_1 := Dyn_Var_Bind_Result; 
--# bind typ Block_Var_1
--# expr-start 1159 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12768
if Canonicalize then
   --# expr-start 1158 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <Var typ>: <Var Var_Expr_2> <FieldAccess .canonical_type>>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:12770
--# expr-start 1156 '<FieldAccess .origin_node>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12770
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_7 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1156
Bound_Origin_1 := Fld_7; 
Var_Expr_2 := Block_Var_1;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1157 '<FieldAccess .canonical_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12770
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr_2.Node, Origin => Bound_Origin_1, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1157
Scope_Result_4 := Fld_8;
   --# end
   Result_Var_2 := Scope_Result_4;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_2; 
--# expr-done 1158
   If_Result := Dyn_Var_Bind_Result_1;
else
   If_Result := Block_Var_1;
end if;
--# expr-done 1159
Block_Var_2 := If_Result; 
--# bind canon_type Block_Var_2
--# expr-start 1160 '<Cast to IncompleteTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:12774
Cast_Expr := Block_Var_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1160
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_3
--# expr-start 1161 '<FieldAccess .next_part>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12775
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1161
Scope_Result_5 := Fld_9;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := Block_Var_2;
end if;
Block_Var_3 := Result_Var_3; 
--# bind final_type Block_Var_3
Var_Expr_4 := Block_Var_3;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_4
--# expr-start 1165 '<Cast to DeclarativePart>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:12780
--# expr-start 1164 '<parent(<parent(<FieldAccess .node>)>)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:12780
--# expr-start 1163 '<parent(<FieldAccess .node>)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:12780
--# expr-start 1162 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:12780
Fld_10 := Var_Expr_4.Node;
--# expr-done 1162
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Fld_10.Parent; 
--# expr-done 1163
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Fld_11.Parent; 
--# expr-done 1164
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result_4 := Fld_12;
   else
         Cast_Result_4 := No_Bare_Ada_Node;
   end if;
--# expr-done 1165
Scope_Result_6 := Cast_Result_4;
   --# end
   Result_Var_4 := Scope_Result_6;
else
   Result_Var_4 := No_Bare_Ada_Node;
end if;
Block_Var_4 := Result_Var_4; 
--# bind type_scope Block_Var_4
--# expr-start 1180 '<If>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:12783
--# expr-start 1168 '<IsA DerivedTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:12795
--# expr-start 1166 '<Cast to TypeDecl>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:12795
Cast_Expr_1 := Block_Var_3; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_5 := No_Entity_Type_Decl;
   end if;
--# expr-done 1166
Var_Expr_5 := Cast_Result_5;
if Var_Expr_5 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1167 '<FieldAccess .type_def>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:12795
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Type_Def (Node => Var_Expr_5.Node.Type_Decl_F_Type_Def, Info => Var_Expr_5.Info);
--# expr-done 1167
Scope_Result_7 := Fld_13;
   --# end
   Result_Var_5 := Scope_Result_7;
else
   Result_Var_5 := No_Entity_Type_Def;
end if;
Is_A := Result_Var_5.Node /= null 
and then Result_Var_5.Node.Kind in Ada_Derived_Type_Def_Range; 
--# expr-done 1168
if Is_A then
   If_Result_2 := True;
else
--# expr-start 1170 '<IsA BasePackageDecl>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:12800
Var_Expr_6 := Block_Var_4;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1169 '<parent(<Var Var_Expr_6>)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:12800
      if Var_Expr_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr_6.Parent; 
--# expr-done 1169
Scope_Result_8 := Fld_14;
   --# end
   Result_Var_6 := Scope_Result_8;
else
   Result_Var_6 := No_Bare_Ada_Node;
end if;
Is_A_1 := Result_Var_6 /= null 
and then Result_Var_6.Kind in Ada_Base_Package_Decl; 
--# expr-done 1170
if Is_A_1 then
   --# expr-start 1172 '<IsA BasePackageDecl>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:12801
Var_Expr_7 := Block_Var;
if Var_Expr_7 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1171 '<parent(<Var Var_Expr_7>)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:12801
      if Var_Expr_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Var_Expr_7.Parent; 
--# expr-done 1171
Scope_Result_9 := Fld_15;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := No_Bare_Ada_Node;
end if;
Is_A_2 := Result_Var_7 /= null 
and then Result_Var_7.Kind in Ada_Base_Package_Decl; 
--# expr-done 1172
   If_Result_1 := Is_A_2;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
   --# expr-start 1174 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:12806
Var_Expr_8 := Block_Var_3;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1173 '<FieldAccess .is_classwide>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:12806
      if Var_Expr_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_8.Node, E_Info => Var_Expr_8.Info);
--# end
--# expr-done 1173
Scope_Result_10 := Fld_16;
   --# end
   Result_Var_8 := Scope_Result_10;
else
   Result_Var_8 := False;
end if;
Not_Val_1 := not (Result_Var_8); 
--# expr-done 1174
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
Var_Expr_9 := Block_Var_4;
if Var_Expr_9 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ds Var_Expr_9
--# expr-start 1175 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:12811
Cast_Expr_2 := Block_Var; 
      Cast_Result_6 := Cast_Expr_2;
Is_Equal := Cast_Result_6 = Var_Expr_9; 
--# expr-done 1175
if Is_Equal then
   If_Result_4 := True;
else
   --# expr-start 1179 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:12811
--# expr-start 1177 '<Cast to BasePackageDecl>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:12813
Var_Expr_11 := Block_Var;
if Var_Expr_11 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1176 '<parent(<Var Var_Expr_11>)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:12813
      if Var_Expr_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Var_Expr_11.Parent; 
--# expr-done 1176
Scope_Result_11 := Fld_17;
   --# end
   Result_Var_9 := Scope_Result_11;
else
   Result_Var_9 := No_Bare_Ada_Node;
end if;
   if Result_Var_9 = null
      or else Result_Var_9.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_7 := Result_Var_9;
   else
         Cast_Result_7 := No_Bare_Ada_Node;
   end if;
--# expr-done 1177
Var_Expr_10 := Cast_Result_7;
if Var_Expr_10 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1178 '<FieldAccess .public_part>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:12813
      if Var_Expr_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Var_Expr_10.Base_Package_Decl_F_Public_Part;
--# expr-done 1178
Scope_Result_12 := Fld_18;
   --# end
   Result_Var_10 := Scope_Result_12;
else
   Result_Var_10 := No_Bare_Ada_Node;
end if;
      Cast_Result_8 := Result_Var_10;
Is_Equal_1 := Var_Expr_9 = Cast_Result_8; 
--# expr-done 1179
   If_Result_4 := Is_Equal_1;
end if;
Scope_Result_13 := If_Result_4;
   --# end
   Result_Var_11 := Scope_Result_13;
else
   Result_Var_11 := False;
end if;
   If_Result_5 := Result_Var_11;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_6 := Block_Var_3;
else
   If_Result_6 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 1180
Let_Result_2 := If_Result_6; 
--# end
--# expr-done 1181
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Candidate_Type_For_Primitive;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_types /usr/src/packages/BUILD/ada/ast.py:12822
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_982;
            procedure Finalizer_Scope_1684;
      Block_Var : Internal_Single_Formal_Array_Access;
Block_Var_1 : Internal_Entity_Type_Expr_Array_Access;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Fld : Internal_Single_Formal_Array_Access;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_2 : Internal_Entity_Type_Expr;
Map_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_3 : Internal_Entity_Type_Expr;
Singleton : Internal_Entity_Type_Expr_Array_Access;
Scope_Result : Internal_Entity_Type_Expr_Array_Access;
Result_Var : Internal_Entity_Type_Expr_Array_Access;
Concat_Result : Internal_Entity_Type_Expr_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Incomplete_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Map_Result_3 : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_982 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_982;
            procedure Finalizer_Scope_1684 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1684;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Get_Primitive_Subp_Types,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 1197 '<Block>' Let_Result None
--# scope-start
--# expr-start 1182 '<FieldAccess .unpacked_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:12832
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1182
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind params Block_Var
--# expr-start 1187 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:12833
--# expr-start 1185 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12833
declare
   Map_Result_Vec : Internal_Entity_Type_Expr_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Block_Var;
      begin
         for Item_47 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_47
      --# expr-start 1184 '<FieldAccess .type_expression>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12833
--# expr-start 1183 '<FieldAccess .spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12833
Fld_1 := Item_47.Spec;
--# expr-done 1183
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 1184
         declare
            Item_To_Append : constant Internal_Entity_Type_Expr :=
               Fld_2;
         begin
            Internal_Entity_Type_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Type_Expr_Array
        (Items_Count => Natural (Internal_Entity_Type_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Type_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Type_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Type_Expr_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1185
--# expr-start 1186 '<FieldAccess .returns>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1186
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
                Singleton := Create_Internal_Entity_Type_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1684;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1187
Block_Var_1 := Concat_Result; Inc_Ref (Block_Var_1);
--# bind types Block_Var_1
--# expr-start 1196 '<unique(<map>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:12837
--# expr-start 1195 '<map>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:12837
--# expr-start 1192 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:12837
--# expr-start 1189 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:12837
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Type_Expr_Array_Access := Block_Var_1;
      begin
         for Item_50 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_50
      --# expr-start 1188 '<FieldAccess .candidate_type_for_primitive(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.candidate_type_for_primitive
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Type_For_Primitive (Node => Ent.Node, Type_Expr => Item_50, Canonicalize => Canonicalize);
--# end
--# expr-done 1188
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1189
declare
   Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_49 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_49
               --# expr-start 1191 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12842
--# expr-start 1190 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12842
Is_Null := Item_49.Node = null; 
--# expr-done 1190
Not_Val := not (Is_Null); 
--# expr-done 1191
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_49;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1192
declare
   Map_Result_3_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_2;
      begin
         for Item_48 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_48
--# expr-start 1193 '<Cast to IncompleteTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12844
Cast_Expr := Item_48; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Incomplete_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Incomplete_Type_Decl;
   end if;
--# expr-done 1193
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Incomplete_Type_Decl then
   --# scope-start
   --# bind i Var_Expr_1
--# expr-start 1194 '<FieldAccess .next_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12845
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1194
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Item_48;
end if;
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Result_Var_1;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1195
Unique_Array := Make_Unique (Map_Result_3); 
--# expr-done 1196
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 1197
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_982;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                     As_Internal_Entity_Base_Type_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_982;
               Finalizer_Scope_1684;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_first_type /usr/src/packages/BUILD/ada/ast.py:12851
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_983;
            procedure Finalizer_Scope_1689;
      Var_Expr : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_983 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_983;
            procedure Finalizer_Scope_1689 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_1689;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1200 '<Block>' Let_Result None
--# scope-start
--# expr-start 1198 '<FieldAccess .get_primitive_subp_types>' Fld /usr/src/packages/BUILD/ada/ast.py:12855
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1198
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Base_Type_Decl_Array_Type then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1199 '<at(<Var p>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12855
Get_Result := Get (Var_Expr, 0, True); 
--# expr-done 1199
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_1689;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1200
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_983;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_983;
               Finalizer_Scope_1689;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.get_primitive_subp_tagged_type /usr/src/packages/BUILD/ada/ast.py:12858
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Canonicalize : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind canonicalize Canonicalize
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_984;
      Cast_Expr : Bare_Base_Subp_Spec;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_984 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_984;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Subp_Spec_Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Subp_Spec,
                                As_Bare_Base_Subp_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Canonicalize);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 1207 '<Block>' Let_Result None
--# scope-start
--# expr-start 1206 '<bind(<DynamicVariable origin>, <Var Self>, <at(<filter>, 0)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:12864
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1205 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12865
--# expr-start 1204 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12865
--# expr-start 1201 '<FieldAccess .get_primitive_subp_types(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:12865
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Ent.Node, Canonicalize => Canonicalize, E_Info => Ent.Info);
--# end
--# expr-done 1201
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_51 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_51
               --# expr-start 1203 '<FieldAccess .is_tagged_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12866
--# expr-start 1202 '<FieldAccess .full_view>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12866
      if Item_51.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Item_51.Node, E_Info => Item_51.Info);
--# end
--# expr-done 1202
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1203
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_51;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1204
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1205
Dyn_Var_Bind_Result := Get_Result; 
--# expr-done 1206
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1207
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_984;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_984;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.decl_spec /usr/src/packages/BUILD/ada/ast.py:12872
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Follow_Generic : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_generic Follow_Generic
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Fld : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Not_Val : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Result_Var_2 : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1216 '<Block>' Let_Result None
--# scope-start
--# expr-start 1209 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:12884
--# expr-start 1208 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:12884
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1208
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 1209
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 1210 '<FieldAccess .basic_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12884
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1210
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var := Result_Var; 
--# bind bd Block_Var
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1211 '<FieldAccess .canonical_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12885
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1211
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr_1
--# expr-start 1215 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:12886
--# expr-start 1212 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12887
Not_Val := not (Follow_Generic); 
--# expr-done 1212
if Not_Val then
   --# expr-start 1213 '<IsA GenericSubpInternal>' Is_A /usr/src/packages/BUILD/ada/ast.py:12887
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Generic_Subp_Internal_Range; 
--# expr-done 1213
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := No_Entity_Base_Subp_Spec;
else
   --# expr-start 1214 '<FieldAccess .subp_spec_or_null(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12889
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, Follow_Generic => Follow_Generic, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1214
   If_Result_2 := Fld_3;
end if;
--# expr-done 1215
Scope_Result_2 := If_Result_2;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Ent;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1216
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Decl_Spec;
--# end
--# property-start BaseSubpSpec.primitive_decl_spec /usr/src/packages/BUILD/ada/ast.py:12896
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Decl_Spec
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Base_Subp_Spec;
Let_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1218 '<Block>' Let_Result None
--# scope-start
--# expr-start 1217 '<FieldAccess .decl_spec(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:12902
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Decl_Spec (Node => Ent.Node, Follow_Generic => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1217
Let_Result := Fld; 
--# end
--# expr-done 1218
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Decl_Spec;
--# end
--# property-start BaseSubpSpec.primitive_subp_types /usr/src/packages/BUILD/ada/ast.py:12906
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Types
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_987;
            procedure Finalizer_Scope_1694;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_987 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_987;
            procedure Finalizer_Scope_1694 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1694;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1221 '<Block>' Let_Result None
--# scope-start
--# expr-start 1219 '<FieldAccess .primitive_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:12910
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1219
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1220 '<FieldAccess .get_primitive_subp_types>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12910
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1220
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1694;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1221
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_987;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_987;
               Finalizer_Scope_1694;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_Types;
--# end
--# property-start BaseSubpSpec.primitive_subp_first_type /usr/src/packages/BUILD/ada/ast.py:12914
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_First_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1224 '<Block>' Let_Result None
--# scope-start
--# expr-start 1222 '<FieldAccess .primitive_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:12918
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1222
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1223 '<FieldAccess .get_primitive_subp_first_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12918
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1223
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1224
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_First_Type;
--# end
--# property-start BaseSubpSpec.primitive_subp_tagged_type /usr/src/packages/BUILD/ada/ast.py:12922
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Primitive_Subp_Tagged_Type
  (Node : Bare_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1227 '<Block>' Let_Result None
--# scope-start
--# expr-start 1225 '<FieldAccess .primitive_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:12927
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.primitive_decl_spec
Fld := Libadalang.Implementation.Base_Subp_Spec_P_Primitive_Decl_Spec (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1225
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1226 '<FieldAccess .get_primitive_subp_tagged_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12927
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1226
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1227
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Primitive_Subp_Tagged_Type;
--# end
--# property-start BaseSubpSpec.dottable_subp_of /usr/src/packages/BUILD/ada/ast.py:12930
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Dottable_Subp_Of
  (Node : Bare_Base_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_990;
            procedure Finalizer_Scope_1697;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Defining_Name;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Singleton_1 : Internal_Entity_Base_Type_Decl_Array_Access;
If_Result : Internal_Entity_Base_Type_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_6 : Integer;
Comp_Result : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_990 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_990;
            procedure Finalizer_Scope_1697 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_1697;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1240 '<Block>' Let_Result None
--# scope-start
--# expr-start 1239 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <If>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:12935
--# expr-start 1229 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12935
--# expr-start 1228 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:12935
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1228
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld);
--# end
--# expr-done 1229
Bound_Origin := Fld_1; 
--# expr-start 1238 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:12935
--# expr-start 1230 '<FieldAccess .nb_max_params>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Ent.Node);
--# end
--# expr-done 1230
Comp_Result := Fld_6 > 0; 
if Comp_Result then
--# expr-start 1231 '<FieldAccess .potential_dottable_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12937
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.potential_dottable_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Potential_Dottable_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 1231
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 1237 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12937
--# expr-start 1232 '<IsA ClasswideTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:12938
Is_A := Var_Expr.Node /= null 
and then Var_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range; 
--# expr-done 1232
if Is_A then
   --# expr-start 1234 '<FieldAccess .typedecl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12939
--# expr-start 1233 '<Cast to ClasswideTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12939
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 1233
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld_3 := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1234
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
   If_Result_1 := Singleton;
else
--# expr-start 1236 '<FieldAccess .is_tagged_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12946
--# expr-start 1235 '<FieldAccess .full_view>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12946
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1235
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld_4.Node, Origin => Bound_Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 1236
if Fld_5 then
                Singleton_1 := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr;
   If_Result := Singleton_1;
else
   If_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1237
Scope_Result := If_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1697;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
   If_Result_2 := Result_Var;
else
   If_Result_2 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1238
Dyn_Var_Bind_Result := If_Result_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 1239
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1240
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_990;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_990;
               Finalizer_Scope_1697;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Dottable_Subp_Of;
--# end
--# property-start BaseSubpSpec.return_type /usr/src/packages/BUILD/ada/ast.py:12956
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Return_Type
  (Node : Bare_Base_Subp_Spec
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1243 '<Block>' Let_Result None
--# scope-start
--# expr-start 1241 '<FieldAccess .returns>' Fld /usr/src/packages/BUILD/ada/ast.py:12961
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1241
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind rt Var_Expr
--# expr-start 1242 '<FieldAccess .real_designated_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12962
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.real_designated_type
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Real_Designated_Type (Node => Ent.Node, Typ => Var_Expr, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1242
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1243
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Return_Type;
--# end
--# property-start BaseSubpSpec.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:12965
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Entry_Point
  (Node : Bare_Base_Subp_Spec
  )
   return Boolean
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Xref_Entry_Point;
--# end
--# property-start BaseSubpSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:12966
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Spec_P_Xref_Equation
  (Node : Bare_Base_Subp_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Subp_Spec := Bare_Base_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Spec :=
     Internal_Entity_Base_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_993;
            procedure Finalizer_Scope_1699;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_993 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_993;
            procedure Finalizer_Scope_1699 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1699;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 1244 '<FieldAccess .returns>' Fld /usr/src/packages/BUILD/ada/ast.py:12966
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1244
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 1245 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12966
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1245
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1699;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12967") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_993;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_993;
               Finalizer_Scope_1699;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Entry_Spec
   --
      procedure Initialize_Fields_For_Entry_Spec
        (Self : Bare_Entry_Spec
         ; Entry_Spec_F_Entry_Name : Bare_Defining_Name
         ; Entry_Spec_F_Family_Type : Bare_Ada_Node
         ; Entry_Spec_F_Entry_Params : Bare_Params
        ) is
      begin
            Self.Entry_Spec_F_Entry_Name := Entry_Spec_F_Entry_Name;
            Self.Entry_Spec_F_Family_Type := Entry_Spec_F_Family_Type;
            Self.Entry_Spec_F_Entry_Params := Entry_Spec_F_Entry_Params;
      end Initialize_Fields_For_Entry_Spec;
   function Entry_Spec_F_Entry_Name
     (Node : Bare_Entry_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Spec_F_Entry_Name;
   end;
   function Entry_Spec_F_Family_Type
     (Node : Bare_Entry_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Spec_F_Family_Type;
   end;
   function Entry_Spec_F_Entry_Params
     (Node : Bare_Entry_Spec) return Bare_Params
   is
   begin
         return Node.Entry_Spec_F_Entry_Params;
   end;
--# property-start EntrySpec.name /usr/src/packages/BUILD/ada/ast.py:13034
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Name
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
      Fld : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1246 '<FieldAccess .entry_name>' Fld /usr/src/packages/BUILD/ada/ast.py:13034
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Entry_Spec_F_Entry_Name;
--# expr-done 1246
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Name;
--# end
--# property-start EntrySpec.params /usr/src/packages/BUILD/ada/ast.py:13035
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Params
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Spec :=
     Internal_Entity_Entry_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1005;
            procedure Finalizer_Scope_1700;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_52 : Internal_Entity_Param_Spec;
Bare_Item_52 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
Scope_Result : Internal_Entity_Param_Spec_Array_Access;
Result_Var : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_1005 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1005;
            procedure Finalizer_Scope_1700 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1700;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1247 '<FieldAccess .entry_params>' Fld /usr/src/packages/BUILD/ada/ast.py:13036
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Spec_F_Entry_Params, Info => Ent.Info);
--# expr-done 1247
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1249 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13037
--# expr-start 1248 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13037
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1248
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_52 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_52 := Untyped_Item_52; 
Is_Null := Bare_Item_52 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_52); 
   If_Result := As_Entity;
end if;
                  Item_52 := If_Result; 
   --# scope-start
               --# bind p Item_52
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec :=
               Item_52;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Param_Spec_Array
        (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1249
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1700;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Param_Spec_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1005;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1005;
               Finalizer_Scope_1700;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Params;
--# end
--# property-start EntrySpec.returns /usr/src/packages/BUILD/ada/ast.py:13041
pragma Warnings (Off, "is not referenced");
 function Entry_Spec_P_Returns
  (Node : Bare_Entry_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Entry_Spec := Bare_Entry_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Type_Expr;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Enum_Subp_Spec
   --
--# property-start EnumSubpSpec.enum_decl /usr/src/packages/BUILD/ada/ast.py:12978
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Enum_Decl
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Literal_Decl
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Enum_Literal_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Enum_Literal_Decl;
As_Entity : Internal_Entity_Enum_Literal_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Literal_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1252 '<as_entity(<Cast to EnumLiteralDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:12978
--# expr-start 1251 '<Cast to EnumLiteralDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12978
--# expr-start 1250 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:12978
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1250
   if Fld = null
      or else Fld.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1251
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Literal_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1252
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Enum_Decl;
--# end
--# property-start EnumSubpSpec.name /usr/src/packages/BUILD/ada/ast.py:12980
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Name
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Subp_Spec :=
     Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Bare_Defining_Name;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1255 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12980
--# expr-start 1254 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12980
--# expr-start 1253 '<FieldAccess .enum_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:12980
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1253
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Defining_Name (Node => Fld.Node.Enum_Literal_Decl_F_Name, Info => Fld.Info);
--# expr-done 1254
Fld_2 := Fld_1.Node;
--# expr-done 1255
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Name;
--# end
--# property-start EnumSubpSpec.returns /usr/src/packages/BUILD/ada/ast.py:12981
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Returns
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Subp_Spec :=
     Internal_Entity_Enum_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Cast_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 1257 '<FieldAccess .synth_type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12981
--# expr-start 1256 '<FieldAccess .enum_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:12981
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumSubpSpec.enum_decl
Fld := Libadalang.Implementation.Enum_Subp_Spec_P_Enum_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1256
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.synth_type_expr
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Synth_Type_Expr (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 1257
      Cast_Result := Create_Internal_Entity_Type_Expr
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Returns;
--# end
--# property-start EnumSubpSpec.params /usr/src/packages/BUILD/ada/ast.py:12982
pragma Warnings (Off, "is not referenced");
 function Enum_Subp_Spec_P_Params
  (Node : Bare_Enum_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Enum_Subp_Spec := Bare_Enum_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Param_Spec_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Subp_Spec_P_Params;
--# end
   --
   --  Primitives for Bare_Subp_Spec
   --
      procedure Initialize_Fields_For_Subp_Spec
        (Self : Bare_Subp_Spec
         ; Subp_Spec_F_Subp_Kind : Bare_Subp_Kind
         ; Subp_Spec_F_Subp_Name : Bare_Defining_Name
         ; Subp_Spec_F_Subp_Params : Bare_Params
         ; Subp_Spec_F_Subp_Returns : Bare_Type_Expr
        ) is
      begin
            Self.Subp_Spec_F_Subp_Kind := Subp_Spec_F_Subp_Kind;
            Self.Subp_Spec_F_Subp_Name := Subp_Spec_F_Subp_Name;
            Self.Subp_Spec_F_Subp_Params := Subp_Spec_F_Subp_Params;
            Self.Subp_Spec_F_Subp_Returns := Subp_Spec_F_Subp_Returns;
      end Initialize_Fields_For_Subp_Spec;
   function Subp_Spec_F_Subp_Kind
     (Node : Bare_Subp_Spec) return Bare_Subp_Kind
   is
   begin
         return Node.Subp_Spec_F_Subp_Kind;
   end;
   function Subp_Spec_F_Subp_Name
     (Node : Bare_Subp_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Subp_Spec_F_Subp_Name;
   end;
   function Subp_Spec_F_Subp_Params
     (Node : Bare_Subp_Spec) return Bare_Params
   is
   begin
         return Node.Subp_Spec_F_Subp_Params;
   end;
   function Subp_Spec_F_Subp_Returns
     (Node : Bare_Subp_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Subp_Spec_F_Subp_Returns;
   end;
--# property-start SubpSpec.name /usr/src/packages/BUILD/ada/ast.py:12994
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Name
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Defining_Name
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Defining_Name;
      Fld : Bare_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1258 '<FieldAccess .subp_name>' Fld /usr/src/packages/BUILD/ada/ast.py:12994
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subp_Spec_F_Subp_Name;
--# expr-done 1258
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Name;
--# end
--# property-start SubpSpec.params /usr/src/packages/BUILD/ada/ast.py:12995
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Params
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Param_Spec_Array_Access
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Spec :=
     Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_999;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Item_53 : Internal_Entity_Param_Spec;
Bare_Item_53 : Bare_Param_Spec;
Map_Result : Internal_Entity_Param_Spec_Array_Access;
            procedure Finalizer_Scope_999 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_999;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1261 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12995
--# expr-start 1259 '<FieldAccess .subp_params>' Fld /usr/src/packages/BUILD/ada/ast.py:12995
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Subp_Spec_F_Subp_Params, Info => Ent.Info);
--# expr-done 1259
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1260 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12995
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1260
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Param_Spec_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Param_Spec_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_53 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_53 := Untyped_Item_53; 
Is_Null := Bare_Item_53 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_53); 
   If_Result := As_Entity;
end if;
                  Item_53 := If_Result; 
   --# scope-start
               --# bind p Item_53
         declare
            Item_To_Append : constant Internal_Entity_Param_Spec :=
               Item_53;
         begin
            Internal_Entity_Param_Spec_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Param_Spec_Array
        (Items_Count => Natural (Internal_Entity_Param_Spec_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Param_Spec_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Param_Spec_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Param_Spec_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1261
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_999;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_999;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Params;
--# end
--# property-start SubpSpec.returns /usr/src/packages/BUILD/ada/ast.py:12997
pragma Warnings (Off, "is not referenced");
 function Subp_Spec_P_Returns
  (Node : Bare_Subp_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Spec := Bare_Subp_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Spec :=
     Internal_Entity_Subp_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1262 '<FieldAccess .subp_returns>' Fld /usr/src/packages/BUILD/ada/ast.py:12997
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Subp_Spec_F_Subp_Returns, Info => Ent.Info);
--# expr-done 1262
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Spec_P_Returns;
--# end
   --
   --  Primitives for Bare_Component_List
   --
      procedure Initialize_Fields_For_Component_List
        (Self : Bare_Component_List
         ; Component_List_F_Components : Bare_Ada_Node_List
         ; Component_List_F_Variant_Part : Bare_Variant_Part
        ) is
      begin
            Self.Component_List_F_Components := Component_List_F_Components;
            Self.Component_List_F_Variant_Part := Component_List_F_Variant_Part;
      end Initialize_Fields_For_Component_List;
   function Component_List_F_Components
     (Node : Bare_Component_List) return Bare_Ada_Node_List
   is
   begin
         return Node.Component_List_F_Components;
   end;
   function Component_List_F_Variant_Part
     (Node : Bare_Component_List) return Bare_Variant_Part
   is
   begin
         return Node.Component_List_F_Variant_Part;
   end;
--# property-start ComponentList.type_def /usr/src/packages/BUILD/ada/ast.py:3728
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Def
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Def
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Def;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Type_Def;
As_Entity : Internal_Entity_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1266 '<as_entity(<Cast to TypeDef>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3728
--# expr-start 1265 '<Cast to TypeDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3728
--# expr-start 1264 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3728
--# expr-start 1263 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:3728
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1263
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1264
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Type_Def
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1265
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Type_Def;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1266
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Type_Def;
--# end
--# property-start ComponentList.type_decl /usr/src/packages/BUILD/ada/ast.py:3729
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Type_Decl
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1269 '<Cast to TypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3729
--# expr-start 1268 '<parent(<FieldAccess .type_def>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3729
--# expr-start 1267 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:3729
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1267
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1268
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 1269
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Type_Decl;
--# end
--# property-start ComponentList.parent_component_list /usr/src/packages/BUILD/ada/ast.py:3731
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Parent_Component_List
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Var_Expr : Internal_Entity_Base_Record_Def;
Var_Expr_1 : Internal_Entity_Derived_Type_Def;
Cast_Expr : Bare_Component_List;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Component_List;
Scope_Result_1 : Internal_Entity_Component_List;
Result_Var_1 : Internal_Entity_Component_List;
Dyn_Var_Bind_Result : Internal_Entity_Component_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1275 '<bind(<DynamicVariable origin>, <Var Self>, <Then <Then <Cast to DerivedTypeDef>: <Var Var_Expr_1> <FieldAccess .record_def>>: <Var Var_Expr> <FieldAccess .comps>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:3731
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 1271 '<Cast to DerivedTypeDef>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:3733
--# expr-start 1270 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:3733
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1270
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1271
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 1273 '<FieldAccess .record_def>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3733
--# expr-start 1272 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3733
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1272
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 1273
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Record_Def then
   --# scope-start
--# expr-start 1274 '<FieldAccess .comps>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3733
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_3 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1274
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Component_List;
end if;
Dyn_Var_Bind_Result := Result_Var_1; 
--# expr-done 1275
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Parent_Component_List;
--# end
--# property-start ComponentList.abstract_formal_params_for_assocs /usr/src/packages/BUILD/ada/ast.py:3738
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_For_Assocs
  (Node : Bare_Component_List
      ; Assocs : Internal_Entity_Assoc_List
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind assocs Assocs
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_261;
            procedure Finalizer_Scope_1706;
      Block_Var : Internal_Entity_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Expr : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Entity_Base_Record_Def;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_261 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_261;
            procedure Finalizer_Scope_1706 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1706;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1291 '<Block>' Let_Result None
--# scope-start
--# expr-start 1276 '<FieldAccess .type_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:3740
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1276
Block_Var := Fld; 
--# bind td Block_Var
--# expr-start 1277 '<FieldAccess .discriminants_list>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3741
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1277
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind discriminants Block_Var_1
--# expr-start 1287 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:3744
--# expr-start 1279 '<FieldAccess .match_formals(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3744
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1278 '<FieldAccess .discriminants_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3745
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 1278
--# property-call-start AdaNode.match_formals
Fld_3 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_2, Params => Assocs, Is_Dottable_Subp => False);
--# end
--# expr-done 1279
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_3;
      begin
         for Item_54 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_54
               --# expr-start 1286 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:3747
--# expr-start 1285 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:3747
--# expr-start 1284 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:3747
--# expr-start 1283 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3747
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Formal_Param_Decl_Array_Access := Block_Var_1;
      begin
         for Item_55 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_55
               --# expr-start 1282 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:3748
Cast_Expr := Item_55; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1281 '<FieldAccess .spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3748
--# expr-start 1280 '<FieldAccess .formal>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3748
Fld_4 := Item_54.Formal;
--# expr-done 1280
Fld_5 := Fld_4.Spec;
--# expr-done 1281
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1282
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Item_55;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1283
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1284
Is_Null := Get_Result.Node = null; 
--# expr-done 1285
Not_Val := not (Is_Null); 
--# expr-done 1286
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_54;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1706;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1287
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind discriminants_matches Block_Var_2
--# expr-start 1290 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3756
--# expr-start 1289 '<FieldAccess .comps>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3756
--# expr-start 1288 '<FieldAccess .record_def>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3756
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1288
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_7 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1289
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_8 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_7.Node, Discriminants => Block_Var_2, E_Info => Fld_7.Info);
--# end
--# expr-done 1290
Let_Result := Fld_8; Inc_Ref (Let_Result);
--# end
--# expr-done 1291
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_261;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_261;
               Finalizer_Scope_1706;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params_For_Assocs;
--# end
--# property-start ComponentList.abstract_formal_params_impl /usr/src/packages/BUILD/ada/ast.py:3761
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params_Impl
  (Node : Bare_Component_List
      ; Discriminants : Internal_Param_Match_Array_Access
      ; Include_Discriminants : Boolean
         := True
      ; Recurse : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind discriminants Discriminants
   --# bind include_discriminants Include_Discriminants
   --# bind recurse Recurse
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_262;
            procedure Finalizer_Scope_1709;
            procedure Finalizer_Scope_1710;
            procedure Finalizer_Scope_1712;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Var_Expr_1 : Internal_Entity_Component_List;
Var_Expr_2 : Internal_Entity_Discriminant_Constraint;
Var_Expr_3 : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_56 : Internal_Entity;
Bare_Item_56 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Entity_Component_List;
Fld_6 : Internal_Entity_Type_Decl;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Fld_9 : Internal_Entity_Subtype_Indication;
Fld_10 : Internal_Entity_Constraint;
Cast_Result_2 : Internal_Entity_Discriminant_Constraint;
Fld_11 : Internal_Entity_Assoc_List;
Scope_Result_1 : Internal_Entity_Assoc_List;
Result_Var_1 : Internal_Entity_Assoc_List;
Fld_12 : Internal_Param_Match_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Entity_Type_Decl;
Fld_15 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_262 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_262;
            procedure Finalizer_Scope_1709 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1709;
            procedure Finalizer_Scope_1710 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1710;
            procedure Finalizer_Scope_1712 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_1712;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1316 '<Block>' Let_Result None
--# scope-start
--# expr-start 1298 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:3769
--# expr-start 1295 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3769
--# expr-start 1292 '<FieldAccess .components>' Fld /usr/src/packages/BUILD/ada/ast.py:3769
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1292
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_56 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_56 := Untyped_Item_56; 
Is_Null := Bare_Item_56 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_56); 
   If_Result := As_Entity;
end if;
                  Item_56 := If_Result; 
   --# scope-start
               --# bind e Item_56
               --# expr-start 1293 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_56.Node /= null 
and then Item_56.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1293
               if Is_A then
      --# expr-start 1294 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_56; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1294
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1295
--# expr-start 1296 '<FieldAccess .variant_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3770
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1296
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
--# expr-start 1297 '<FieldAccess .get_components(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3770
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start VariantPart.get_components
Fld_4 := Libadalang.Implementation.Variant_Part_P_Get_Components (Node => Var_Expr.Node, Discriminants => Discriminants, E_Info => Var_Expr.Info);
--# end
--# expr-done 1297
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1709;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Concat_Result := Concat (Map_Result, Result_Var); 
--# expr-done 1298
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1311 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:3777
if Recurse then
--# expr-start 1299 '<FieldAccess .parent_component_list>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3779
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.parent_component_list
Fld_5 := Libadalang.Implementation.Component_List_P_Parent_Component_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1299
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Component_List then
   --# scope-start
   --# bind pcl Var_Expr_1
--# expr-start 1310 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:3780
--# expr-start 1309 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:3780
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1308 '<FieldAccess .match_formals(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3781
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1301 '<FieldAccess .discriminants_list>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3782
--# expr-start 1300 '<FieldAccess .type_decl>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3782
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_6 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1300
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 1301
--# expr-start 1306 '<Cast to DiscriminantConstraint>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:3783
--# expr-start 1305 '<FieldAccess .constraint>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3783
--# expr-start 1304 '<FieldAccess .subtype_indication>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3783
--# expr-start 1303 '<Cast to DerivedTypeDef>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:3783
--# expr-start 1302 '<FieldAccess .type_def>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3783
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld_8 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1302
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 1303
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Subtype_Indication (Node => Cast_Result_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Cast_Result_1.Info);
--# expr-done 1304
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Constraint (Node => Fld_9.Node.Subtype_Indication_F_Constraint, Info => Fld_9.Info);
--# expr-done 1305
   if Fld_10.Node = null
      or else Fld_10.Node.Kind in Ada_Discriminant_Constraint_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Discriminant_Constraint
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
   else
         Cast_Result_2 := No_Entity_Discriminant_Constraint;
   end if;
--# expr-done 1306
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Entity_Discriminant_Constraint then
   --# scope-start
--# expr-start 1307 '<FieldAccess .constraints>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3783
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Assoc_List (Node => Var_Expr_2.Node.Discriminant_Constraint_F_Constraints, Info => Var_Expr_2.Info);
--# expr-done 1307
Scope_Result_1 := Fld_11;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Assoc_List;
end if;
--# property-call-start AdaNode.match_formals
Fld_12 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Var_Expr_1.Node, Formal_Params => Fld_7, Params => Result_Var_1, Is_Dottable_Subp => False);
--# end
--# expr-done 1308
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_13 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Var_Expr_1.Node, Discriminants => Fld_12, Include_Discriminants => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1309
Concat_Result_1 := Concat (Fld_13, Block_Var); 
--# expr-done 1310
Scope_Result_2 := Concat_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1710;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_1 := Result_Var_2;
else
   If_Result_1 := Block_Var;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1311
Block_Var_1 := If_Result_1; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 1315 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:3795
if Include_Discriminants then
   --# expr-start 1314 '<Concat>' Concat_Result_2 /usr/src/packages/BUILD/ada/ast.py:3797
--# expr-start 1312 '<FieldAccess .type_decl>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:3797
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_14 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1312
Var_Expr_3 := Fld_14;
if Var_Expr_3 /= No_Entity_Type_Decl then
   --# scope-start
--# expr-start 1313 '<FieldAccess .discriminants_list>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:3797
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_15 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1313
Scope_Result_3 := Fld_15;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1712;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_2 := Concat (Result_Var_3, Block_Var_1); 
--# expr-done 1314
   If_Result_2 := Concat_Result_2;
else
   If_Result_2 := Block_Var_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1315
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1316
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_262;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_262;
               Finalizer_Scope_1709;
               Finalizer_Scope_1710;
               Finalizer_Scope_1712;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params_Impl;
--# end
--# property-start ComponentList.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:3802
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Abstract_Formal_Params
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_263;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_263 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_263;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1318 '<Block>' Let_Result None
--# scope-start
--# expr-start 1317 '<FieldAccess .abstract_formal_params_impl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:3803
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Ent.Node, Discriminants => No_Internal_Param_Match_Array_Type, E_Info => Ent.Info);
--# end
--# expr-done 1317
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 1318
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_263;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_263;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Abstract_Formal_Params;
--# end
--# property-start ComponentList.shapes /usr/src/packages/BUILD/ada/ast.py:3806
pragma Warnings (Off, "is not referenced");
 function Component_List_P_Shapes
  (Node : Bare_Component_List
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Component_List := Bare_Component_List (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_List :=
     Internal_Entity_Component_List'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_264;
            procedure Finalizer_Scope_1714;
            procedure Finalizer_Scope_1715;
            procedure Finalizer_Scope_1716;
      Block_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Variant_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_57 : Internal_Entity;
Bare_Item_57 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Internal_Entity_Variant_Part;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Variant;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Variant;
Fld_7 : Internal_Entity_Component_List;
Fld_8 : Internal_Shape_Array_Access;
Fld_9 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Internal_Entity_Alternatives_List;
New_Struct : Internal_Discriminant_Values;
Singleton : Internal_Discriminant_Values_Array_Access;
Fld_12 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_1 : Internal_Shape_Array_Access;
Item_58 : Internal_Entity_Variant;
Bare_Item_58 : Bare_Variant;
Map_Result_2 : Internal_Shape_Array_Access;
Scope_Result : Internal_Shape_Array_Access;
New_Struct_2 : Internal_Shape := No_Shape;
Singleton_1 : Internal_Shape_Array_Access;
Result_Var : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_264 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (New_Struct_2);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_264;
            procedure Finalizer_Scope_1714 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1714;
            procedure Finalizer_Scope_1715 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1715;
            procedure Finalizer_Scope_1716 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1716;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1338 '<Block>' Let_Result None
--# scope-start
--# expr-start 1322 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3810
--# expr-start 1319 '<FieldAccess .components>' Fld /usr/src/packages/BUILD/ada/ast.py:3810
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Component_List_F_Components, Info => Ent.Info);
--# expr-done 1319
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_57 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_57 := Untyped_Item_57; 
Is_Null := Bare_Item_57 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_57); 
   If_Result := As_Entity;
end if;
                  Item_57 := If_Result; 
   --# scope-start
               --# bind e Item_57
               --# expr-start 1320 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_57.Node /= null 
and then Item_57.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1320
               if Is_A then
      --# expr-start 1321 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_57; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1321
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1322
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind self_comps Block_Var
--# expr-start 1323 '<FieldAccess .variant_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3811
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Variant_Part (Node => Ent.Node.Component_List_F_Variant_Part, Info => Ent.Info);
--# expr-done 1323
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Variant_Part then
   --# scope-start
   --# bind vpart Var_Expr
--# expr-start 1336 '<mapcat>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:3812
--# expr-start 1324 '<FieldAccess .variant>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3812
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Var_Expr.Node.Variant_Part_F_Variant, Info => Var_Expr.Info);
--# expr-done 1324
Fld_5 := Fld_4.Node;
declare
   Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_2 := Create_Internal_Shape_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_58 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_58 := Untyped_Item_58; 
Is_Null_1 := Bare_Item_58 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity_1 := (Info => Fld_6, Node => Bare_Item_58); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_58 := If_Result_1; 
   --# scope-start
               --# bind v Item_58
      --# expr-start 1335 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:3813
--# expr-start 1326 '<FieldAccess .shapes>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3813
--# expr-start 1325 '<FieldAccess .components>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3813
      if Item_58.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Component_List (Node => Item_58.Node.Variant_F_Components, Info => Item_58.Info);
--# expr-done 1325
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.shapes
Fld_8 := Libadalang.Implementation.Component_List_P_Shapes (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 1326
declare
   Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_8;
      begin
         for Item_59 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_59
      --# expr-start 1334 '<New InternalShape>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:3814
--# expr-start 1328 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:3815
--# expr-start 1327 '<FieldAccess .components>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3815
Fld_9 := Item_59.Components;
Inc_Ref (Fld_9);
--# expr-done 1327
Concat_Result := Concat (Block_Var, Fld_9); 
--# expr-done 1328
--# expr-start 1333 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:3816
--# expr-start 1331 '<New InternalDiscriminantValues>' New_Struct /usr/src/packages/BUILD/ada/ast.py:3816
--# expr-start 1329 '<FieldAccess .discr_name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3817
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Variant_Part_F_Discr_Name, Info => Var_Expr.Info);
--# expr-done 1329
--# expr-start 1330 '<FieldAccess .choices>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3818
      if Item_58.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Alternatives_List (Node => Item_58.Node.Variant_F_Choices, Info => Item_58.Info);
--# expr-done 1330
New_Struct := (Discriminant => Fld_10, Values => Fld_11); 
--# expr-done 1331
                Singleton := Create_Internal_Discriminant_Values_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
--# expr-start 1332 '<FieldAccess .discriminants_values>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3819
Fld_12 := Item_59.Discriminants_Values;
Inc_Ref (Fld_12);
--# expr-done 1332
Concat_Result_1 := Concat (Singleton, Fld_12); 
--# expr-done 1333
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct_1 := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 1334
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1716;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1335
         for Item_To_Append of
               Map_Result_1.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1715;
         end loop;
      end;
      Map_Result_2 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
      end if;
end;
--# expr-done 1336
Scope_Result := Map_Result_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1714;
   Result_Var := Scope_Result;
else
   --# expr-start 1337 '<New InternalShape>' New_Struct_2 /usr/src/packages/BUILD/ada/ast.py:3823
Inc_Ref (Block_Var);
Inc_Ref (No_Internal_Discriminant_Values_Array_Type);
New_Struct_2 := (Components => Block_Var, Discriminants_Values => No_Internal_Discriminant_Values_Array_Type); 
--# expr-done 1337
                Singleton_1 := Create_Internal_Shape_Array (Items_Count => 1);
                Singleton_1.Items (1) := New_Struct_2;
                Inc_Ref (Singleton_1.Items (1));
   Result_Var := Singleton_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1338
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_264;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_264;
               Finalizer_Scope_1714;
               Finalizer_Scope_1715;
               Finalizer_Scope_1716;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_List_P_Shapes;
--# end
   --
   --  Primitives for Bare_Discriminant_Part
   --
--# property-start DiscriminantPart.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:3381
pragma Warnings (Off, "is not referenced");
 function Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Discriminant_Part := Bare_Discriminant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_221;
      Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_221 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_221;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1339 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type; Inc_Ref (Let_Result);
--# end
--# expr-done 1339
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_221;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_221;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Known_Discriminant_Part
   --
      procedure Initialize_Fields_For_Known_Discriminant_Part
        (Self : Bare_Known_Discriminant_Part
         ; Known_Discriminant_Part_F_Discr_Specs : Bare_Discriminant_Spec_List
        ) is
      begin
            Self.Known_Discriminant_Part_F_Discr_Specs := Known_Discriminant_Part_F_Discr_Specs;
      end Initialize_Fields_For_Known_Discriminant_Part;
   function Known_Discriminant_Part_F_Discr_Specs
     (Node : Bare_Known_Discriminant_Part) return Bare_Discriminant_Spec_List
   is
   begin
         return Node.Known_Discriminant_Part_F_Discr_Specs;
   end;
--# property-start KnownDiscriminantPart.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:3393
pragma Warnings (Off, "is not referenced");
 function Known_Discriminant_Part_P_Abstract_Formal_Params
  (Node : Bare_Known_Discriminant_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Known_Discriminant_Part := Bare_Known_Discriminant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_222;
      Fld : Bare_Discriminant_Spec_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Base_Formal_Param_Decl;
As_Entity : Internal_Entity_Base_Formal_Param_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_60 : Bare_Discriminant_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_222 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_222;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1344 '<Block>' Let_Result None
--# scope-start
--# expr-start 1343 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3394
--# expr-start 1340 '<FieldAccess .discr_specs>' Fld /usr/src/packages/BUILD/ada/ast.py:3394
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Known_Discriminant_Part_F_Discr_Specs;
--# expr-done 1340
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Spec_List := Fld;
      begin
         for Untyped_Item_60 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_60 := Untyped_Item_60; 
   --# scope-start
               --# bind e Item_60
      --# expr-start 1342 '<as_entity(<Cast to BaseFormalParamDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3395
--# expr-start 1341 '<Cast to BaseFormalParamDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3395
Cast_Expr := Item_60; 
      Cast_Result := Cast_Expr;
--# expr-done 1341
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Formal_Param_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1342
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               If_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1343
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1344
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_222;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_222;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Known_Discriminant_Part_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Unknown_Discriminant_Part
   --
   --
   --  Primitives for Bare_Entry_Completion_Formal_Params
   --
      procedure Initialize_Fields_For_Entry_Completion_Formal_Params
        (Self : Bare_Entry_Completion_Formal_Params
         ; Entry_Completion_Formal_Params_F_Params : Bare_Params
        ) is
      begin
            Self.Entry_Completion_Formal_Params_F_Params := Entry_Completion_Formal_Params_F_Params;
      end Initialize_Fields_For_Entry_Completion_Formal_Params;
   function Entry_Completion_Formal_Params_F_Params
     (Node : Bare_Entry_Completion_Formal_Params) return Bare_Params
   is
   begin
         return Node.Entry_Completion_Formal_Params_F_Params;
   end;
--# property-start EntryCompletionFormalParams.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:15114
pragma Warnings (Off, "is not referenced");
 function Entry_Completion_Formal_Params_P_Abstract_Formal_Params
  (Node : Bare_Entry_Completion_Formal_Params
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Entry_Completion_Formal_Params := Bare_Entry_Completion_Formal_Params (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Completion_Formal_Params :=
     Internal_Entity_Entry_Completion_Formal_Params'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1129;
      Var_Expr : Internal_Entity_Params;
Fld : Internal_Entity_Params;
Fld_1 : Internal_Entity_Param_Spec_List;
Scope_Result : Internal_Entity_Param_Spec_List;
Result_Var : Internal_Entity_Param_Spec_List;
Fld_2 : Bare_Param_Spec_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Param_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Param_Spec;
Cast_Expr : Internal_Entity_Param_Spec;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_61 : Internal_Entity_Param_Spec;
Bare_Item_61 : Bare_Param_Spec;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1129 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1129;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1348 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:15115
--# expr-start 1345 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:15115
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Params (Node => Ent.Node.Entry_Completion_Formal_Params_F_Params, Info => Ent.Info);
--# expr-done 1345
Var_Expr := Fld;
if Var_Expr /= No_Entity_Params then
   --# scope-start
--# expr-start 1346 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15115
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Param_Spec_List (Node => Var_Expr.Node.Params_F_Params, Info => Var_Expr.Info);
--# expr-done 1346
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Param_Spec_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Param_Spec_List := Fld_2;
      begin
         for Untyped_Item_61 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_61 := Untyped_Item_61; 
Is_Null := Bare_Item_61 = null; 
if Is_Null then
   If_Result := No_Entity_Param_Spec;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_61); 
   If_Result := As_Entity;
end if;
                  Item_61 := If_Result; 
   --# scope-start
               --# bind p Item_61
      --# expr-start 1347 '<Cast to BaseFormalParamDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15115
Cast_Expr := Item_61; 
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 1347
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1348
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1129;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1129;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Completion_Formal_Params_P_Abstract_Formal_Params;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Part
   --
      procedure Initialize_Fields_For_Generic_Formal_Part
        (Self : Bare_Generic_Formal_Part
         ; Generic_Formal_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Generic_Formal_Part_F_Decls := Generic_Formal_Part_F_Decls;
      end Initialize_Fields_For_Generic_Formal_Part;
   function Generic_Formal_Part_F_Decls
     (Node : Bare_Generic_Formal_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Generic_Formal_Part_F_Decls;
   end;
--# property-start GenericFormalPart.abstract_formal_params /usr/src/packages/BUILD/ada/ast.py:8374
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Abstract_Formal_Params
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Generic_Formal_Part := Bare_Generic_Formal_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal_Part :=
     Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_732;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Formal_Param_Decl;
Item_62 : Internal_Entity;
Bare_Item_62 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_732 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_732;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1352 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:8374
--# expr-start 1349 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:8374
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 1349
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Formal_Param_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_62 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_62 := Untyped_Item_62; 
Is_Null := Bare_Item_62 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_62); 
   If_Result := As_Entity;
end if;
                  Item_62 := If_Result; 
   --# scope-start
               --# bind e Item_62
               --# expr-start 1350 '<IsA BaseFormalParamDecl>' Is_A None
Is_A := Item_62.Node /= null 
and then Item_62.Node.Kind in Ada_Base_Formal_Param_Decl; 
--# expr-done 1350
               if Is_A then
      --# expr-start 1351 '<Cast to BaseFormalParamDecl>' Cast_Result None
Cast_Expr := Item_62; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Formal_Param_Decl;
   end if;
--# expr-done 1351
         declare
            Item_To_Append : constant Internal_Entity_Base_Formal_Param_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Formal_Param_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Formal_Param_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Formal_Param_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Formal_Param_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Formal_Param_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Formal_Param_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1352
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_732;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_732;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_Part_P_Abstract_Formal_Params;
--# end
--# property-start GenericFormalPart.use_clauses_envs /usr/src/packages/BUILD/ada/ast.py:8377
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_Part_P_Use_Clauses_Envs
  (Node : Bare_Generic_Formal_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Formal_Part := Bare_Generic_Formal_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal_Part :=
     Internal_Entity_Generic_Formal_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_733;
            procedure Finalizer_Scope_1721;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_3 : Lexical_Env := Empty_Env;
Item_63 : Internal_Entity;
Bare_Item_63 : Bare_Ada_Node;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_733 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_733;
            procedure Finalizer_Scope_1721 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1721;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1359 '<Block>' Let_Result None
--# scope-start
--# expr-start 1358 '<env_group(<filter_map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:8382
--# expr-start 1357 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:8382
--# expr-start 1353 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:8382
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Generic_Formal_Part_F_Decls, Info => Ent.Info);
--# expr-done 1353
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_63 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_63 := Untyped_Item_63; 
Is_Null := Bare_Item_63 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_63); 
   If_Result := As_Entity;
end if;
                  Item_63 := If_Result; 
   --# scope-start
               --# bind u Item_63
               --# expr-start 1354 '<IsA UseClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:8384
Is_A := Item_63.Node /= null 
and then Item_63.Node.Kind in Ada_Use_Clause; 
--# expr-done 1354
               if Is_A then
      --# expr-start 1356 '<FieldAccess .used_envs>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8383
--# expr-start 1355 '<Cast to UseClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8383
Cast_Expr := Item_63; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 1355
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UseClause.used_envs
Fld_3 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1356
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1721;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1357
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 1358
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 1359
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_733;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_733;
               Finalizer_Scope_1721;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_Part_P_Use_Clauses_Envs;
--# end
   --
   --  Primitives for Bare_Base_Record_Def
   --
      procedure Initialize_Fields_For_Base_Record_Def
        (Self : Bare_Base_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Self.Base_Record_Def_F_Components := Base_Record_Def_F_Components;
      end Initialize_Fields_For_Base_Record_Def;
   function Base_Record_Def_F_Components
     (Node : Bare_Base_Record_Def) return Bare_Component_List
   is
   begin
         return Node.Base_Record_Def_F_Components;
   end;
--# property-start BaseRecordDef.comps /usr/src/packages/BUILD/ada/ast.py:3838
pragma Warnings (Off, "is not referenced");
 function Base_Record_Def_P_Comps
  (Node : Bare_Base_Record_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Component_List
is
   Self : Bare_Base_Record_Def := Bare_Base_Record_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Record_Def :=
     Internal_Entity_Base_Record_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Component_List;
      Fld : Internal_Entity_Component_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1360 '<FieldAccess .components>' Fld /usr/src/packages/BUILD/ada/ast.py:3838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_List (Node => Ent.Node.Base_Record_Def_F_Components, Info => Ent.Info);
--# expr-done 1360
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Record_Def_P_Comps;
--# end
   --
   --  Primitives for Bare_Null_Record_Def
   --
      procedure Initialize_Fields_For_Null_Record_Def
        (Self : Bare_Null_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Null_Record_Def;
   --
   --  Primitives for Bare_Record_Def
   --
      procedure Initialize_Fields_For_Record_Def
        (Self : Bare_Record_Def
         ; Base_Record_Def_F_Components : Bare_Component_List
        ) is
      begin
            Initialize_Fields_For_Base_Record_Def
              (Self, Base_Record_Def_F_Components);
      end Initialize_Fields_For_Record_Def;
   --
   --  Primitives for Bare_Basic_Assoc
   --
--# property-start '[dispatcher]BasicAssoc.expr' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Expr
  (Node : Bare_Basic_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.expr
                  Property_Result := Aggregate_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Assoc =>
                  --# property-call-start DiscriminantAssoc.expr
                  Property_Result := Discriminant_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.expr
                  Property_Result := Iterated_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.expr
                  Property_Result := Param_Assoc_P_Expr
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Assoc_P_Expr;
--# end
--# property-start '[dispatcher]BasicAssoc.names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Assoc_P_Names
  (Node : Bare_Basic_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Assoc (Self.Kind) is
               when Ada_Aggregate_Assoc .. Ada_Multi_Dim_Array_Assoc =>
                  --# property-call-start AggregateAssoc.names
                  Property_Result := Aggregate_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Discriminant_Assoc =>
                  --# property-call-start DiscriminantAssoc.names
                  Property_Result := Discriminant_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Iterated_Assoc =>
                  --# property-call-start IteratedAssoc.names
                  Property_Result := Iterated_Assoc_P_Names
                    (Self
                    );
                  --# end
               when Ada_Param_Assoc =>
                  --# property-call-start ParamAssoc.names
                  Property_Result := Param_Assoc_P_Names
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Assoc_P_Names;
--# end
--# property-start BasicAssoc.get_params /usr/src/packages/BUILD/ada/ast.py:5794
pragma Warnings (Off, "is not referenced");
 function Basic_Assoc_P_Get_Params
  (Node : Bare_Basic_Assoc
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Assoc := Bare_Basic_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Assoc :=
     Internal_Entity_Basic_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_478;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_1 : Internal_Param_Actual_Array_Access;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Let_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_478 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_478;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1369 '<Block>' Let_Result None
--# scope-start
--# expr-start 1368 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5799
--# expr-start 1363 '<FieldAccess .zip_with_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5799
--# expr-start 1362 '<Cast to AssocList>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5799
--# expr-start 1361 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:5799
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1361
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1362
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.zip_with_params
Fld_1 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Cast_Result.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 1363
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Fld_1;
      begin
         for Item_64 of
               Collection.Items
         loop
   --# scope-start
               --# bind m Item_64
               --# expr-start 1366 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:5801
--# expr-start 1364 '<FieldAccess .actual>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5801
Fld_3 := Item_64.Actual;
--# expr-done 1364
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 1365 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5801
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1365
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1366
               if Is_Equiv then
      --# expr-start 1367 '<FieldAccess .param>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5800
Fld_2 := Item_64.Param;
--# expr-done 1367
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Fld_2;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1368
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1369
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_478;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_478;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Assoc_P_Get_Params;
--# end
   --
   --  Primitives for Bare_Aggregate_Assoc
   --
      procedure Initialize_Fields_For_Aggregate_Assoc
        (Self : Bare_Aggregate_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Aggregate_Assoc_F_Designators := Aggregate_Assoc_F_Designators;
            Self.Aggregate_Assoc_F_R_Expr := Aggregate_Assoc_F_R_Expr;
      end Initialize_Fields_For_Aggregate_Assoc;
   function Aggregate_Assoc_F_Designators
     (Node : Bare_Aggregate_Assoc) return Bare_Alternatives_List
   is
   begin
         return Node.Aggregate_Assoc_F_Designators;
   end;
   function Aggregate_Assoc_F_R_Expr
     (Node : Bare_Aggregate_Assoc) return Bare_Expr
   is
   begin
         return Node.Aggregate_Assoc_F_R_Expr;
   end;
--# property-start AggregateAssoc.expr /usr/src/packages/BUILD/ada/ast.py:11000
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Expr
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1370 '<FieldAccess .r_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11000
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Aggregate_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1370
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Expr;
--# end
--# property-start AggregateAssoc.names /usr/src/packages/BUILD/ada/ast.py:11001
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Names
  (Node : Bare_Aggregate_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_873;
      Fld : Bare_Alternatives_List;
Item_65 : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_873 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_873;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1372 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11001
--# expr-start 1371 '<FieldAccess .designators>' Fld /usr/src/packages/BUILD/ada/ast.py:11001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Aggregate_Assoc_F_Designators;
--# expr-done 1371
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_65 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_65 := Untyped_Item_65; 
   --# scope-start
               --# bind d Item_65
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_65;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1372
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_873;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_873;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Names;
--# end
--# property-start AggregateAssoc.xref_stop_resolution /usr/src/packages/BUILD/ada/ast.py:11003
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Aggregate_Assoc
  )
   return Boolean
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start AggregateAssoc.base_aggregate /usr/src/packages/BUILD/ada/ast.py:11005
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Base_Aggregate
  (Node : Bare_Aggregate_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1375 '<Cast to BaseAggregate>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11006
--# expr-start 1374 '<parent(<parent(<Var Ent>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11006
--# expr-start 1373 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:11006
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1373
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1374
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1375
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Base_Aggregate;
--# end
--# property-start AggregateAssoc.xref_equation /usr/src/packages/BUILD/ada/ast.py:11010
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Xref_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_876;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Aggregate;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Sym : Symbol_Type;
Fld_6 : Boolean;
Sym_1 : Symbol_Type;
Fld_7 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Logic_Equation;
Sym_2 : Symbol_Type;
Fld_9 : Boolean;
Sym_3 : Symbol_Type;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Logic_Equation;
Sym_4 : Symbol_Type;
Fld_12 : Boolean;
Fld_13 : Logic_Equation;
Sym_5 : Symbol_Type;
Fld_14 : Boolean;
Fld_15 : Logic_Equation;
Sym_6 : Symbol_Type;
Fld_16 : Boolean;
Fld_17 : Logic_Equation;
Fld_18 : Internal_Entity;
Is_A : Boolean;
Logic_True : Logic_Equation;
Is_Null : Boolean;
Fld_19 : Logic_Equation;
Fld_20 : Logic_Equation;
If_Result_3 : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
If_Result_9 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_876 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_20);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_876;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1411 '<Block>' Let_Result None
--# scope-start
--# expr-start 1376 '<FieldAccess .base_aggregate>' Fld /usr/src/packages/BUILD/ada/ast.py:11011
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1376
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1377 '<FieldAccess .multidim_root_aggregate>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11013
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1377
Block_Var_1 := Fld_1; 
--# bind mra Block_Var_1
--# expr-start 1384 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11018
--# expr-start 1379 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11019
--# expr-start 1378 '<IsNull>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11019
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1378
Not_Val := not (Is_Equal); 
--# expr-done 1379
if Not_Val then
   --# expr-start 1380 '<FieldAccess .typ>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11020
Fld_2 := Block_Var_1.Typ;
--# expr-done 1380
   If_Result := Fld_2;
else
   --# expr-start 1383 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11021
--# expr-start 1382 '<FieldAccess .type_val>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11021
--# expr-start 1381 '<FieldAccess .base_aggregate>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11021
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld_3 := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1381
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_4 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_3.Node);
--# end
--# expr-done 1382
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1383
   If_Result := Cast_Result;
end if;
--# expr-done 1384
Block_Var_2 := If_Result; 
--# bind td Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1385 '<FieldAccess .array_def>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11024
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1385
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind atd Block_Var_3
--# expr-start 1410 '<Cond>' If_Result_9 /usr/src/packages/BUILD/ada/ast.py:11026
--# expr-start 1387 '<FieldAccess .in_aspect(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11027
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1386 '<Symbol Global>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 1386
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym);
--# end
--# expr-done 1387
if Fld_6 then
   If_Result_1 := True;
else
   --# expr-start 1389 '<FieldAccess .in_aspect(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11027
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1388 '<Symbol Refined_Global>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Global); 
--# expr-done 1388
--# property-call-start AdaNode.in_aspect
Fld_7 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_1);
--# end
--# expr-done 1389
   If_Result_1 := Fld_7;
end if;
if If_Result_1 then
   --# expr-start 1390 '<FieldAccess .globals_assoc_equation>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11028
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.globals_assoc_equation
Fld_8 := Libadalang.Implementation.Aggregate_Assoc_P_Globals_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1390
   If_Result_9 := Fld_8;
else
--# expr-start 1392 '<FieldAccess .in_aspect(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11030
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1391 '<Symbol Depends>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 1391
--# property-call-start AdaNode.in_aspect
Fld_9 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_2);
--# end
--# expr-done 1392
if Fld_9 then
   If_Result_2 := True;
else
   --# expr-start 1394 '<FieldAccess .in_aspect(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11030
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1393 '<Symbol Refined_Depends>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Depends); 
--# expr-done 1393
--# property-call-start AdaNode.in_aspect
Fld_10 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_3);
--# end
--# expr-done 1394
   If_Result_2 := Fld_10;
end if;
if If_Result_2 then
   --# expr-start 1395 '<FieldAccess .depends_assoc_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11031
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.depends_assoc_equation
Fld_11 := Libadalang.Implementation.Aggregate_Assoc_P_Depends_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1395
   If_Result_8 := Fld_11;
else
--# expr-start 1397 '<FieldAccess .in_aspect(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11033
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1396 '<Symbol Depends>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 1396
--# property-call-start AdaNode.in_aspect
Fld_12 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_4);
--# end
--# expr-done 1397
if Fld_12 then
   --# expr-start 1398 '<FieldAccess .depends_assoc_equation>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11033
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.depends_assoc_equation
Fld_13 := Libadalang.Implementation.Aggregate_Assoc_P_Depends_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1398
   If_Result_7 := Fld_13;
else
--# expr-start 1400 '<FieldAccess .in_aspect(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11035
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1399 '<Symbol Test_Case>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 1399
--# property-call-start AdaNode.in_aspect
Fld_14 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_5);
--# end
--# expr-done 1400
if Fld_14 then
   --# expr-start 1401 '<FieldAccess .test_case_assoc_equation>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11036
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.test_case_assoc_equation
Fld_15 := Libadalang.Implementation.Aggregate_Assoc_P_Test_Case_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1401
   If_Result_6 := Fld_15;
else
--# expr-start 1403 '<FieldAccess .in_aspect(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:11038
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1402 '<Symbol Refined_State>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_State); 
--# expr-done 1402
--# property-call-start AdaNode.in_aspect
Fld_16 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Block_Var.Node, Name => Sym_6);
--# end
--# expr-done 1403
if Fld_16 then
   --# expr-start 1404 '<FieldAccess .refined_state_assoc_equation>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:11039
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.refined_state_assoc_equation
Fld_17 := Libadalang.Implementation.Aggregate_Assoc_P_Refined_State_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1404
   If_Result_5 := Fld_17;
else
--# expr-start 1406 '<IsA AspectClause, AspectAssoc, PragmaArgumentAssoc>' Is_A /usr/src/packages/BUILD/ada/ast.py:11041
--# expr-start 1405 '<parent(<Var agg>)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11041
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 1405
Is_A := Fld_18.Node /= null 
and then Fld_18.Node.Kind in Ada_Aspect_Clause | Ada_Aspect_Assoc_Range | Ada_Pragma_Argument_Assoc_Range; 
--# expr-done 1406
if Is_A then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11042") else null)); 
   If_Result_4 := Logic_True;
else
--# expr-start 1407 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:11044
Is_Null := Block_Var_3.Node = null; 
--# expr-done 1407
if Is_Null then
   --# expr-start 1408 '<FieldAccess .record_assoc_equation>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11045
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.record_assoc_equation
Fld_19 := Libadalang.Implementation.Aggregate_Assoc_P_Record_Assoc_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1408
   If_Result_3 := Fld_19;
else
   --# expr-start 1409 '<FieldAccess .array_assoc_equation(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11046
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.array_assoc_equation
Fld_20 := Libadalang.Implementation.Aggregate_Assoc_P_Array_Assoc_Equation (Node => Ent.Node, Atd => Block_Var_3, Mra => Block_Var_1, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1409
   If_Result_3 := Fld_20;
end if;
      Inc_Ref (If_Result_3);
   If_Result_4 := If_Result_3;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
   If_Result_9 := If_Result_8;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 1410
Let_Result := If_Result_9; Inc_Ref (Let_Result);
--# end
--# expr-done 1411
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_876;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_876;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Xref_Equation;
--# end
--# property-start AggregateAssoc.record_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11050
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Record_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_877;
            procedure Finalizer_Scope_1728;
            procedure Finalizer_Scope_1729;
            procedure Finalizer_Scope_1730;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Param_Match_Array_Access;
Block_Var_2 : Internal_Param_Match_Array_Access;
Block_Var_3 : Boolean;
Var_Expr : Bare_Base_Id;
Var_Expr_1 : Internal_Single_Formal;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Param_Match_Array_Access;
Fld_2 : Internal_Single_Actual;
Fld_3 : Internal_Entity_Basic_Assoc;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Aggregate_Assoc;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Actual;
Fld_6 : Internal_Entity_Basic_Assoc;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Aggregate_Assoc;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Map_Result_1 : Internal_Param_Match_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Param_Match_Array_Access;
Fld_7 : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Quantifier_Result : Boolean;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Internal_Single_Formal;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Type_Expr;
Fld_15 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_16 : Logic_Equation;
Fld_17 : Internal_Single_Actual;
Fld_18 : Internal_Entity_Basic_Assoc;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_21 : Internal_Single_Actual;
Fld_22 : Bare_Base_Id;
Fld_23 : Logic_Var;
Fld_24 : Internal_Single_Formal;
Fld_25 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_26 : Internal_Single_Formal;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Base_Formal_Param_Decl;
Fld_30 : Internal_Entity_Type_Expr;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_6 : Internal_Entity;
Fld_32 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Not_Val_1 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_877 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_877;
            procedure Finalizer_Scope_1728 is
            begin
                     Dec_Ref (Fld_16);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1728;
            procedure Finalizer_Scope_1729 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1729;
            procedure Finalizer_Scope_1730 is
            begin
                     Dec_Ref (Fld_32);
            end Finalizer_Scope_1730;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1460 '<Block>' Let_Result None
--# scope-start
--# expr-start 1412 '<FieldAccess .base_aggregate>' Fld /usr/src/packages/BUILD/ada/ast.py:11055
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AggregateAssoc.base_aggregate
Fld := Libadalang.Implementation.Aggregate_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1412
Block_Var := Fld; 
--# bind agg Block_Var
--# expr-start 1417 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11058
--# expr-start 1413 '<FieldAccess .matched_discriminants>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11058
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1413
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_1;
      begin
         for Item_66 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_66
               --# expr-start 1416 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:11059
--# expr-start 1415 '<FieldAccess .assoc>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11059
--# expr-start 1414 '<FieldAccess .actual>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11059
Fld_2 := Item_66.Actual;
--# expr-done 1414
Fld_3 := Fld_2.Assoc;
--# expr-done 1415
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1416
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_66;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1417
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind discr_matches Block_Var_1
--# expr-start 1425 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11071
--# expr-start 1419 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11072
--# expr-start 1418 '<IsNull>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11072
Is_Equal := Equivalent (Block_Var_1, No_Internal_Param_Match_Array_Type); 
--# expr-done 1418
Not_Val := not (Is_Equal); 
--# expr-done 1419
if Not_Val then
   If_Result := Block_Var_1;
else
   --# expr-start 1424 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11076
--# expr-start 1420 '<FieldAccess .matched_components>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11076
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1420
declare
   Map_Result_1_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_4;
      begin
         for Item_67 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_67
               --# expr-start 1423 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:11077
--# expr-start 1422 '<FieldAccess .assoc>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11077
--# expr-start 1421 '<FieldAccess .actual>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11077
Fld_5 := Item_67.Actual;
--# expr-done 1421
Fld_6 := Fld_5.Assoc;
--# expr-done 1422
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1423
               if Is_Equiv_1 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_67;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1424
   If_Result := Map_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 1425
Block_Var_2 := If_Result; Inc_Ref (Block_Var_2);
--# bind matches Block_Var_2
--# expr-start 1426 '<FieldAccess .names>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11082
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1426
Quantifier_Result := False;
   declare
      Collection : constant Bare_Ada_Node_Array_Access := Fld_7;
   begin
      for Item_68 of
            Collection.Items
      loop
   --# scope-start
            --# bind n Item_68
         --# expr-start 1427 '<IsA OthersDesignator>' Is_A /usr/src/packages/BUILD/ada/ast.py:11083
Is_A := Item_68 /= null 
and then Item_68.Kind in Ada_Others_Designator_Range; 
--# expr-done 1427
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Block_Var_3 := Quantifier_Result; 
--# bind is_others_assoc Block_Var_3
--# expr-start 1459 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11086
--# expr-start 1428 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:11087
Not_Val_1 := not (Block_Var_3); 
--# expr-done 1428
if Not_Val_1 then
   --# expr-start 1451 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11089
--# expr-start 1450 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:11089
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Block_Var_2;
      begin
         for Item_69 of
               Collection.Items
         loop
   --# scope-start
               --# bind match Item_69
      --# expr-start 1449 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:11089
--# expr-start 1442 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11089
--# expr-start 1437 '<FieldAccess .type_bind_val(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:11090
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1432 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11091
--# expr-start 1431 '<FieldAccess .expr>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11091
--# expr-start 1430 '<FieldAccess .assoc>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11091
--# expr-start 1429 '<FieldAccess .actual>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11091
Fld_8 := Item_69.Actual;
--# expr-done 1429
Fld_9 := Fld_8.Assoc;
--# expr-done 1430
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 1431
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10.Node);
--# expr-done 1432
--# expr-start 1436 '<FieldAccess .designated_type>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11092
--# expr-start 1435 '<FieldAccess .type_expression>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11092
--# expr-start 1434 '<FieldAccess .spec>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11092
--# expr-start 1433 '<FieldAccess .formal>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11092
Fld_12 := Item_69.Formal;
--# expr-done 1433
Fld_13 := Fld_12.Spec;
--# expr-done 1434
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 1435
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_14.Node, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1436
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
--# property-call-start AdaNode.type_bind_val
Fld_16 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 1437
--# expr-start 1441 '<FieldAccess .sub_equation>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11094
--# expr-start 1440 '<FieldAccess .expr>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11094
--# expr-start 1439 '<FieldAccess .assoc>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11094
--# expr-start 1438 '<FieldAccess .actual>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:11094
Fld_17 := Item_69.Actual;
--# expr-done 1438
Fld_18 := Fld_17.Assoc;
--# expr-done 1439
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_18.Node, E_Info => Fld_18.Info);
--# end
--# expr-done 1440
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_20 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_19.Node, Env => Env, Origin => Origin, E_Info => Fld_19.Info);
--# end
--# expr-done 1441
And_Pred := Logic_And (Fld_16, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11089") else null)); 
--# expr-done 1442
--# expr-start 1444 '<FieldAccess .name>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:11095
--# expr-start 1443 '<FieldAccess .actual>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:11095
Fld_21 := Item_69.Actual;
--# expr-done 1443
Fld_22 := Fld_21.Name;
--# expr-done 1444
Var_Expr := Fld_22;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1448 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:11096
--# expr-start 1445 '<FieldAccess .ref_var>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:11096
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr);
--# end
--# expr-done 1445
Fld_23.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_23.all);
Eq_Node.Refs.Destroy (Fld_23.all);
--# expr-start 1447 '<FieldAccess .spec>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:11096
--# expr-start 1446 '<FieldAccess .formal>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:11096
Fld_24 := Item_69.Formal;
--# expr-done 1446
Fld_25 := Fld_24.Spec;
--# expr-done 1447
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_25.Node,
         Info => Fld_25.Info);
Bind_Result := Bind_Default_Default.Create (Fld_23, Cast_Result_5, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11096") else null)); 
--# expr-done 1448
Scope_Result := Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1729;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11097") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11089") else null)); 
--# expr-done 1449
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1728;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1450
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11089") else null)); 
--# expr-done 1451
   If_Result_1 := Logic_Boolean_Op;
else
--# expr-start 1452 '<FieldAccess .first_unmatched_formal>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:11105
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.first_unmatched_formal
Fld_26 := Libadalang.Implementation.Base_Aggregate_P_First_Unmatched_Formal (Node => Block_Var.Node, Origin => Origin, Env => Env, E_Info => Block_Var.Info);
--# end
--# expr-done 1452
Var_Expr_1 := Fld_26;
if Var_Expr_1 /= No_Single_Formal then
   --# scope-start
   --# bind unmatched_formal Var_Expr_1
--# expr-start 1458 '<FieldAccess .type_bind_val(...)>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:11106
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1454 '<FieldAccess .type_var>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:11107
--# expr-start 1453 '<FieldAccess .expr>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:11107
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_27 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1453
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27.Node);
--# expr-done 1454
--# expr-start 1457 '<FieldAccess .designated_type>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:11108
--# expr-start 1456 '<FieldAccess .type_expression>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:11108
--# expr-start 1455 '<FieldAccess .spec>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:11108
Fld_29 := Var_Expr_1.Spec;
--# expr-done 1455
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_29.Node, E_Info => Fld_29.Info);
--# end
--# expr-done 1456
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_31 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_30.Node, Origin => Origin, E_Info => Fld_30.Info);
--# end
--# expr-done 1457
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_28, Right => Cast_Result_6, Origin => Origin);
--# end
--# expr-done 1458
Scope_Result_1 := Fld_32;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1730;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11110") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_1 := Result_Var_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1459
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1460
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_877;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_877;
               Finalizer_Scope_1728;
               Finalizer_Scope_1729;
               Finalizer_Scope_1730;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Record_Assoc_Equation;
--# end
--# property-start AggregateAssoc.array_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11115
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Array_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Atd : Internal_Entity_Array_Type_Def
      ; Mra : Internal_Multidim_Aggregate_Info
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind atd Atd
   --# bind mra Mra
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_878;
            procedure Finalizer_Scope_1731;
            procedure Finalizer_Scope_1732;
      Var_Expr : Bare_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True : Logic_Equation;
Is_Equal : Boolean;
Fld_6 : Integer;
Fld_7 : Integer;
Arith_Result : Integer;
Is_Equal_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
Fld_8 : Bare_Ada_Node_Array_Access;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_2 : Internal_Entity;
Fld_9 : Logic_Equation;
Cast_Expr : Bare_Ada_Node;
Cast_Result_1 : Bare_Expr;
Fld_10 : Logic_Var;
Fld_11 : Integer;
Fld_12 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_13 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_878 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_878;
            procedure Finalizer_Scope_1731 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_1731;
            procedure Finalizer_Scope_1732 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_1732;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1486 '<Block>' Let_Result None
--# scope-start
--# expr-start 1485 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:11121
--# expr-start 1473 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11122
--# expr-start 1461 '<IsNull>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11125
Is_Equal := Mra = No_Multidim_Aggregate_Info; 
--# expr-done 1461
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1465 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:11125
--# expr-start 1462 '<FieldAccess .rank>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11125
Fld_6 := Mra.Rank;
--# expr-done 1462
--# expr-start 1464 '<Op ->' Arith_Result /usr/src/packages/BUILD/ada/ast.py:11125
--# expr-start 1463 '<FieldAccess .array_ndims>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11125
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_7 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1463
Arith_Result := (Fld_7 - 1); 
--# expr-done 1464
Is_Equal_1 := Fld_6 = Arith_Result; 
--# expr-done 1465
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 1472 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11128
--# expr-start 1467 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11128
--# expr-start 1466 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11128
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1466
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1467
--# expr-start 1471 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11129
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1469 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11129
--# expr-start 1468 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11129
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1468
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2.Node);
--# expr-done 1469
--# expr-start 1470 '<FieldAccess .comp_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11129
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_4 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Atd.Node, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1470
      Cast_Result := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 1471
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11128") else null)); 
--# expr-done 1472
   If_Result_1 := And_Pred;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11133") else null)); 
   If_Result_1 := Logic_True;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1473
--# expr-start 1484 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11136
--# expr-start 1483 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11136
--# expr-start 1474 '<FieldAccess .names>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11136
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1474
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Fld_8;
      begin
         for Item_70 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_70
      --# expr-start 1482 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:11138
--# expr-start 1476 '<FieldAccess .sub_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11138
--# expr-start 1475 '<as_entity(<Var n>)>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:11138
Is_Null := Item_70 = null; 
if Is_Null then
   If_Result_2 := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_70); 
   If_Result_2 := As_Entity;
end if;
--# expr-done 1475
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result_2.Node, Env => Env, Origin => Origin, E_Info => If_Result_2.Info);
--# end
--# expr-done 1476
--# expr-start 1477 '<Cast to Expr>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11139
Cast_Expr := Item_70; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Expr
   then
      Cast_Result_1 := Cast_Expr;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1477
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1481 '<FieldAccess .type_bind_val(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11140
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1478 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11140
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr);
--# expr-done 1478
--# expr-start 1480 '<FieldAccess .index_type(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11141
      if Atd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1479 '<FieldAccess .rank>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11141
Fld_11 := Mra.Rank;
--# expr-done 1479
--# property-call-start ArrayTypeDef.index_type
Fld_12 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Atd.Node, Dim => Fld_11, Origin => Origin, E_Info => Atd.Info);
--# end
--# expr-done 1480
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_10, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 1481
Scope_Result := Fld_13;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1732;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11142") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_9, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11138") else null)); 
--# expr-done 1482
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1731;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1483
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11136") else null)); 
--# expr-done 1484
And_Pred_2 := Logic_And (If_Result_1, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11121") else null)); 
--# expr-done 1485
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 1486
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_878;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_878;
               Finalizer_Scope_1731;
               Finalizer_Scope_1732;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Array_Assoc_Equation;
--# end
--# property-start AggregateAssoc.globals_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11148
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Globals_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_879;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_879 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_879;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1489 '<Block>' Let_Result None
--# scope-start
--# expr-start 1488 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11156
--# expr-start 1487 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11156
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1487
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1488
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1489
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_879;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_879;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Globals_Assoc_Equation;
--# end
--# property-start AggregateAssoc.depends_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11159
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Depends_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_880;
            procedure Finalizer_Scope_1733;
      Var_Expr : Internal_Entity_Identifier;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Identifier;
As_Entity : Internal_Entity_Identifier;
Is_Null : Boolean;
If_Result : Internal_Entity_Identifier;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_880 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_880;
            procedure Finalizer_Scope_1733 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1733;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1498 '<Block>' Let_Result None
--# scope-start
--# expr-start 1497 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11164
--# expr-start 1491 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11169
--# expr-start 1490 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11169
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1490
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1491
--# expr-start 1495 '<as_entity(<Cast to Identifier>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:11177
--# expr-start 1494 '<Cast to Identifier>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11177
--# expr-start 1493 '<at(<FieldAccess .names>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11177
--# expr-start 1492 '<FieldAccess .names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11177
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1492
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 1493
   if Get_Result = null
      or else Get_Result.Kind in Ada_Identifier_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1494
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Identifier;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1495
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Identifier then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1496 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11178
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1496
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1733;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11178") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11164") else null)); 
--# expr-done 1497
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1498
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_880;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_880;
               Finalizer_Scope_1733;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Depends_Assoc_Equation;
--# end
--# property-start AggregateAssoc.test_case_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11183
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Test_Case_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_881;
      Var_Expr : Bare_Base_Id;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_2 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Id;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_881 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_881;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1511 '<Block>' Let_Result None
--# scope-start
--# expr-start 1510 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11188
--# expr-start 1501 '<Cast to BaseId>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11191
--# expr-start 1500 '<at(<FieldAccess .names>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11191
--# expr-start 1499 '<FieldAccess .names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11191
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Names (Node => Ent.Node);
--# end
--# expr-done 1499
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 1500
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Id
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1501
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 1504 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11192
--# expr-start 1502 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11192
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 1502
--# expr-start 1503 '<Symbol Requires>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Requires); 
--# expr-done 1503
Is_Equal := Fld_3 = Sym; 
--# expr-done 1504
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1507 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:11192
--# expr-start 1505 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11192
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 1505
--# expr-start 1506 '<Symbol Ensures>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ensures); 
--# expr-done 1506
Is_Equal_1 := Fld_4 = Sym_1; 
--# expr-done 1507
   If_Result := Is_Equal_1;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 1509 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11196
--# expr-start 1508 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11196
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1508
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1509
   If_Result_1 := Fld_1;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11197") else null)); 
   If_Result_1 := Logic_True;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1510
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1511
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_881;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_881;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Test_Case_Assoc_Equation;
--# end
--# property-start AggregateAssoc.refined_state_assoc_equation /usr/src/packages/BUILD/ada/ast.py:11201
pragma Warnings (Off, "is not referenced");
 function Aggregate_Assoc_P_Refined_State_Assoc_Equation
  (Node : Bare_Aggregate_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Aggregate_Assoc := Bare_Aggregate_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Aggregate_Assoc :=
     Internal_Entity_Aggregate_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_882;
            procedure Finalizer_Scope_1735;
            procedure Finalizer_Scope_1736;
      Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Alternatives_List;
Fld_1 : Bare_Alternatives_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Logic_Equation;
Item_71 : Internal_Entity;
Bare_Item_71 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_882 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_882;
            procedure Finalizer_Scope_1735 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1735;
            procedure Finalizer_Scope_1736 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1736;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1520 '<Block>' Let_Result None
--# scope-start
--# expr-start 1519 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11207
--# expr-start 1515 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11208
--# expr-start 1514 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11208
--# expr-start 1512 '<FieldAccess .designators>' Fld /usr/src/packages/BUILD/ada/ast.py:11208
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Aggregate_Assoc_F_Designators, Info => Ent.Info);
--# expr-done 1512
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_1;
      begin
         for Untyped_Item_71 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_71 := Untyped_Item_71; 
Is_Null := Bare_Item_71 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_71); 
   If_Result := As_Entity;
end if;
                  Item_71 := If_Result; 
   --# scope-start
               --# bind d Item_71
      --# expr-start 1513 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11208
      if Item_71.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_71.Node, Env => Env, Origin => Origin, E_Info => Item_71.Info);
--# end
--# expr-done 1513
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1735;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1514
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11208") else null)); 
--# expr-done 1515
--# expr-start 1517 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11209
--# expr-start 1516 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11209
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1516
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 1517
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 1518 '<FieldAccess .sub_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11210
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1518
Scope_Result := Fld_5;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1736;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11211") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Logic_Boolean_Op, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11207") else null)); 
--# expr-done 1519
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 1520
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_882;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_882;
               Finalizer_Scope_1735;
               Finalizer_Scope_1736;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Aggregate_Assoc_P_Refined_State_Assoc_Equation;
--# end
   --
   --  Primitives for Bare_Multi_Dim_Array_Assoc
   --
      procedure Initialize_Fields_For_Multi_Dim_Array_Assoc
        (Self : Bare_Multi_Dim_Array_Assoc
         ; Aggregate_Assoc_F_Designators : Bare_Alternatives_List
         ; Aggregate_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Aggregate_Assoc
              (Self, Aggregate_Assoc_F_Designators, Aggregate_Assoc_F_R_Expr);
      end Initialize_Fields_For_Multi_Dim_Array_Assoc;
   --
   --  Primitives for Bare_Discriminant_Assoc
   --
      procedure Initialize_Fields_For_Discriminant_Assoc
        (Self : Bare_Discriminant_Assoc
         ; Discriminant_Assoc_F_Ids : Bare_Discriminant_Choice_List
         ; Discriminant_Assoc_F_Discr_Expr : Bare_Expr
        ) is
      begin
            Self.Discriminant_Assoc_F_Ids := Discriminant_Assoc_F_Ids;
            Self.Discriminant_Assoc_F_Discr_Expr := Discriminant_Assoc_F_Discr_Expr;
      end Initialize_Fields_For_Discriminant_Assoc;
   function Discriminant_Assoc_F_Ids
     (Node : Bare_Discriminant_Assoc) return Bare_Discriminant_Choice_List
   is
   begin
         return Node.Discriminant_Assoc_F_Ids;
   end;
   function Discriminant_Assoc_F_Discr_Expr
     (Node : Bare_Discriminant_Assoc) return Bare_Expr
   is
   begin
         return Node.Discriminant_Assoc_F_Discr_Expr;
   end;
--# property-start DiscriminantAssoc.expr /usr/src/packages/BUILD/ada/ast.py:5813
pragma Warnings (Off, "is not referenced");
 function Discriminant_Assoc_P_Expr
  (Node : Bare_Discriminant_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Discriminant_Assoc := Bare_Discriminant_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Assoc :=
     Internal_Entity_Discriminant_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1521 '<FieldAccess .discr_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:5813
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Assoc_F_Discr_Expr, Info => Ent.Info);
--# expr-done 1521
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Assoc_P_Expr;
--# end
--# property-start DiscriminantAssoc.names /usr/src/packages/BUILD/ada/ast.py:5814
pragma Warnings (Off, "is not referenced");
 function Discriminant_Assoc_P_Names
  (Node : Bare_Discriminant_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Discriminant_Assoc := Bare_Discriminant_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_480;
      Fld : Bare_Discriminant_Choice_List;
Cast_Expr : Bare_Identifier;
Cast_Result : Bare_Ada_Node;
Item_72 : Bare_Identifier;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_480 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_480;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1524 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5814
--# expr-start 1522 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:5814
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Assoc_F_Ids;
--# expr-done 1522
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Discriminant_Choice_List := Fld;
      begin
         for Untyped_Item_72 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_72 := Untyped_Item_72; 
   --# scope-start
               --# bind i Item_72
      --# expr-start 1523 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5814
Cast_Expr := Item_72; 
      Cast_Result := Cast_Expr;
--# expr-done 1523
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1524
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_480;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_480;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Assoc_P_Names;
--# end
   --
   --  Primitives for Bare_Iterated_Assoc
   --
      procedure Initialize_Fields_For_Iterated_Assoc
        (Self : Bare_Iterated_Assoc
         ; Iterated_Assoc_F_Spec : Bare_For_Loop_Spec
         ; Iterated_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Iterated_Assoc_F_Spec := Iterated_Assoc_F_Spec;
            Self.Iterated_Assoc_F_R_Expr := Iterated_Assoc_F_R_Expr;
      end Initialize_Fields_For_Iterated_Assoc;
   function Iterated_Assoc_F_Spec
     (Node : Bare_Iterated_Assoc) return Bare_For_Loop_Spec
   is
   begin
         return Node.Iterated_Assoc_F_Spec;
   end;
   function Iterated_Assoc_F_R_Expr
     (Node : Bare_Iterated_Assoc) return Bare_Expr
   is
   begin
         return Node.Iterated_Assoc_F_R_Expr;
   end;
--# property-start IteratedAssoc.expr /usr/src/packages/BUILD/ada/ast.py:11223
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Expr
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1525 '<FieldAccess .r_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11223
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Iterated_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1525
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Expr;
--# end
--# property-start IteratedAssoc.names /usr/src/packages/BUILD/ada/ast.py:11224
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Names
  (Node : Bare_Iterated_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Bare_Ada_Node_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Names;
--# end
--# property-start IteratedAssoc.base_aggregate /usr/src/packages/BUILD/ada/ast.py:11226
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Base_Aggregate
  (Node : Bare_Iterated_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Aggregate
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Aggregate;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Aggregate;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1528 '<Cast to BaseAggregate>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11227
--# expr-start 1527 '<parent(<parent(<Var Ent>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11227
--# expr-start 1526 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:11227
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1526
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 1527
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Base_Aggregate
   then
      Cast_Result := Create_Internal_Entity_Base_Aggregate
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1528
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Base_Aggregate;
--# end
--# property-start IteratedAssoc.xref_stop_resolution /usr/src/packages/BUILD/ada/ast.py:11230
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Stop_Resolution
  (Node : Bare_Iterated_Assoc
  )
   return Boolean
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Xref_Stop_Resolution;
--# end
--# property-start IteratedAssoc.xref_equation /usr/src/packages/BUILD/ada/ast.py:11233
pragma Warnings (Off, "is not referenced");
 function Iterated_Assoc_P_Xref_Equation
  (Node : Bare_Iterated_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Iterated_Assoc := Bare_Iterated_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Iterated_Assoc :=
     Internal_Entity_Iterated_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_887;
      Block_Var : Internal_Entity_Base_Aggregate;
Block_Var_1 : Internal_Multidim_Aggregate_Info;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Array_Type_Def;
Block_Var_4 : Boolean;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Aggregate;
Fld_1 : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_5 : Internal_Entity_For_Loop_Spec;
Fld_6 : Internal_Entity_For_Loop_Spec;
Fld_7 : Internal_Entity_For_Loop_Var_Decl;
Fld_8 : Internal_Entity_Defining_Name;
Fld_9 : Logic_Var;
Fld_10 : Integer;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_12 : Logic_Equation;
Fld_13 : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Var;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_19 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True : Logic_Equation;
Is_Equal_1 : Boolean;
Fld_20 : Integer;
Fld_21 : Integer;
Arith_Result : Integer;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_887 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_887;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1561 '<Block>' Let_Result None
--# scope-start
--# expr-start 1529 '<FieldAccess .base_aggregate>' Fld /usr/src/packages/BUILD/ada/ast.py:11234
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start IteratedAssoc.base_aggregate
Fld := Libadalang.Implementation.Iterated_Assoc_P_Base_Aggregate (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1529
Block_Var := Fld; 
--# bind aggregate Block_Var
--# expr-start 1530 '<FieldAccess .multidim_root_aggregate>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11236
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_1 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 1530
Block_Var_1 := Fld_1; 
--# bind root_agg Block_Var_1
--# expr-start 1536 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11241
--# expr-start 1532 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11242
--# expr-start 1531 '<IsNull>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11242
Is_Equal := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1531
Not_Val := not (Is_Equal); 
--# expr-done 1532
if Not_Val then
   --# expr-start 1533 '<FieldAccess .typ>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11243
Fld_2 := Block_Var_1.Typ;
--# expr-done 1533
   If_Result := Fld_2;
else
   --# expr-start 1535 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11244
--# expr-start 1534 '<FieldAccess .type_val>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11244
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Block_Var.Node);
--# end
--# expr-done 1534
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1535
   If_Result := Cast_Result;
end if;
--# expr-done 1536
Block_Var_2 := If_Result; 
--# bind type_decl Block_Var_2
Var_Expr := Block_Var_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1537 '<FieldAccess .array_def>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11247
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 1537
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var_3 := Result_Var; 
--# bind array_type_def Block_Var_3
--# expr-start 1546 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11251
--# expr-start 1538 '<FieldAccess .spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11251
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 1538
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1545 '<FieldAccess .type_bind_val(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11252
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1542 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11252
--# expr-start 1541 '<FieldAccess .id>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11252
--# expr-start 1540 '<FieldAccess .var_decl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11252
--# expr-start 1539 '<FieldAccess .spec>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11252
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Iterated_Assoc_F_Spec, Info => Ent.Info);
--# expr-done 1539
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Fld_6.Node.For_Loop_Spec_F_Var_Decl, Info => Fld_6.Info);
--# expr-done 1540
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Defining_Name (Node => Fld_7.Node.For_Loop_Var_Decl_F_Id, Info => Fld_7.Info);
--# expr-done 1541
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8.Node);
--# expr-done 1542
--# expr-start 1544 '<FieldAccess .index_type(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11253
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1543 '<FieldAccess .rank>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11253
Fld_10 := Block_Var_1.Rank;
--# expr-done 1543
--# property-call-start ArrayTypeDef.index_type
Fld_11 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Block_Var_3.Node, Dim => Fld_10, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1544
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 1545
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_13 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_5.Node, Additional_Equation => Fld_12, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 1546
Block_Var_4 := Fld_13; 
--# bind spec_success Block_Var_4
--# expr-start 1560 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:11256
if Block_Var_4 then
   --# expr-start 1559 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:11258
--# expr-start 1547 '<IsNull>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:11261
Is_Equal_1 := Block_Var_1 = No_Multidim_Aggregate_Info; 
--# expr-done 1547
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 1551 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:11262
--# expr-start 1548 '<FieldAccess .rank>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11262
Fld_20 := Block_Var_1.Rank;
--# expr-done 1548
--# expr-start 1550 '<Op ->' Arith_Result /usr/src/packages/BUILD/ada/ast.py:11262
--# expr-start 1549 '<FieldAccess .array_ndims>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:11262
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_21 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1549
Arith_Result := (Fld_21 - 1); 
--# expr-done 1550
Is_Equal_2 := Fld_20 = Arith_Result; 
--# expr-done 1551
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   --# expr-start 1558 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11265
--# expr-start 1553 '<FieldAccess .sub_equation>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11265
--# expr-start 1552 '<FieldAccess .expr>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11265
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1552
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_14.Node, Env => Env, Origin => Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1553
--# expr-start 1557 '<FieldAccess .type_bind_val(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11266
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1555 '<FieldAccess .type_var>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:11266
--# expr-start 1554 '<FieldAccess .expr>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:11266
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_16 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1554
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_16.Node);
--# expr-done 1555
--# expr-start 1556 '<FieldAccess .comp_type>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11267
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_18 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var_3.Node, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 1556
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
--# property-call-start AdaNode.type_bind_val
Fld_19 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_17, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 1557
And_Pred := Logic_And (Fld_15, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11265") else null)); 
--# expr-done 1558
   If_Result_2 := And_Pred;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11271") else null)); 
   If_Result_2 := Logic_True;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 1559
   If_Result_3 := If_Result_2;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11273") else null)); 
   If_Result_3 := Logic_False;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 1560
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1561
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_887;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_887;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Iterated_Assoc_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Param_Assoc
   --
      procedure Initialize_Fields_For_Param_Assoc
        (Self : Bare_Param_Assoc
         ; Param_Assoc_F_Designator : Bare_Ada_Node
         ; Param_Assoc_F_R_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Assoc_F_Designator := Param_Assoc_F_Designator;
            Self.Param_Assoc_F_R_Expr := Param_Assoc_F_R_Expr;
      end Initialize_Fields_For_Param_Assoc;
   function Param_Assoc_F_Designator
     (Node : Bare_Param_Assoc) return Bare_Ada_Node
   is
   begin
         return Node.Param_Assoc_F_Designator;
   end;
   function Param_Assoc_F_R_Expr
     (Node : Bare_Param_Assoc) return Bare_Expr
   is
   begin
         return Node.Param_Assoc_F_R_Expr;
   end;
--# property-start ParamAssoc.expr /usr/src/packages/BUILD/ada/ast.py:10973
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Expr
  (Node : Bare_Param_Assoc
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Assoc :=
     Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1562 '<FieldAccess .r_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:10973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Assoc_F_R_Expr, Info => Ent.Info);
--# expr-done 1562
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Expr;
--# end
--# property-start ParamAssoc.names /usr/src/packages/BUILD/ada/ast.py:10974
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Names
  (Node : Bare_Param_Assoc
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_868;
      Fld : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Fld_1 : Bare_Ada_Node;
Is_Null : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_868 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_868;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1566 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10974
--# expr-start 1564 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10974
--# expr-start 1563 '<FieldAccess .designator>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10974
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Param_Assoc_F_Designator;
--# expr-done 1563
Is_Null := Fld_1 = null; 
--# expr-done 1564
if Is_Null then
   If_Result := No_Bare_Ada_Node_Array_Type;
else
   --# expr-start 1565 '<FieldAccess .designator>' Fld /usr/src/packages/BUILD/ada/ast.py:10975
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Assoc_F_Designator;
--# expr-done 1565
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
   If_Result := Singleton;
end if;
      Inc_Ref (If_Result);
--# expr-done 1566
      Property_Result := If_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_868;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_868;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Names;
--# end
--# property-start ParamAssoc.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:10977
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Entry_Point
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1567 '<FieldAccess .is_static_attribute_assoc>' Fld /usr/src/packages/BUILD/ada/ast.py:10977
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ParamAssoc.is_static_attribute_assoc
Fld := Libadalang.Implementation.Param_Assoc_P_Is_Static_Attribute_Assoc (Node => Self);
--# end
--# expr-done 1567
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Xref_Entry_Point;
--# end
--# property-start ParamAssoc.xref_equation /usr/src/packages/BUILD/ada/ast.py:10978
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Xref_Equation
  (Node : Bare_Param_Assoc
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Assoc :=
     Internal_Entity_Param_Assoc'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_870;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_2 : Boolean;
If_Result : Logic_Equation;
            procedure Finalizer_Scope_870 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_870;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1571 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10978
--# expr-start 1568 '<FieldAccess .xref_entry_point>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_entry_point'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Entry_Point (Node => Self);
--# end
--# expr-done 1568
if Fld_2 then
   --# expr-start 1570 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10980
--# expr-start 1569 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:10980
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1569
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 1570
   If_Result := Fld_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10981") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 1571
      Property_Result := If_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_870;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_870;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Xref_Equation;
--# end
--# property-start ParamAssoc.is_static_attribute_assoc /usr/src/packages/BUILD/ada/ast.py:10985
pragma Warnings (Off, "is not referenced");
 function Param_Assoc_P_Is_Static_Attribute_Assoc
  (Node : Bare_Param_Assoc
  )
   return Boolean
is
   Self : Bare_Param_Assoc := Bare_Param_Assoc (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Attribute_Ref;
Fld_2 : Bare_Identifier;
Fld_3 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_4 : Bare_Identifier;
Fld_5 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Identifier;
Fld_7 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_8 : Bare_Identifier;
Fld_9 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1591 '<Block>' Let_Result None
--# scope-start
--# expr-start 1574 '<Cast to AttributeRef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10986
--# expr-start 1573 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10986
--# expr-start 1572 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:10986
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1572
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1573
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1574
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ar Var_Expr
--# expr-start 1578 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1576 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1575 '<FieldAccess .attribute>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10987
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1575
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2);
--# end
--# expr-done 1576
--# expr-start 1577 '<Symbol First>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 1577
Is_Equal := Fld_3 = Sym; 
--# expr-done 1578
if Is_Equal then
   If_Result := True;
else
   --# expr-start 1582 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1580 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1579 '<FieldAccess .attribute>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10987
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1579
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4);
--# end
--# expr-done 1580
--# expr-start 1581 '<Symbol Last>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 1581
Is_Equal_1 := Fld_5 = Sym_1; 
--# expr-done 1582
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1586 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1584 '<FieldAccess .name_symbol>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1583 '<FieldAccess .attribute>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10987
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1583
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6);
--# end
--# expr-done 1584
--# expr-start 1585 '<Symbol Range>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 1585
Is_Equal_2 := Fld_7 = Sym_2; 
--# expr-done 1586
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 1590 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1588 '<FieldAccess .name_symbol>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10987
--# expr-start 1587 '<FieldAccess .attribute>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10987
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr.Attribute_Ref_F_Attribute;
--# expr-done 1587
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_8);
--# end
--# expr-done 1588
--# expr-start 1589 '<Symbol Length>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 1589
Is_Equal_3 := Fld_9 = Sym_3; 
--# expr-done 1590
   If_Result_2 := Is_Equal_3;
end if;
Scope_Result := If_Result_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1591
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Assoc_P_Is_Static_Attribute_Assoc;
--# end
   --
   --  Primitives for Bare_Basic_Decl
   --
   function Basic_Decl_F_Aspects
     (Node : Bare_Basic_Decl) return Bare_Aspect_Spec
   is
         Kind : constant Ada_Basic_Decl := Node.Kind;
   begin
         case Kind is
               when Ada_Error_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Discriminant_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Component_Decl =>
         return Node.Component_Decl_F_Aspects;
               when Ada_Classwide_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Type_Decl | Ada_Anonymous_Type_Decl | Ada_Synth_Anonymous_Type_Decl =>
         return Node.Type_Decl_F_Aspects;
               when Ada_Subtype_Decl =>
         return Node.Subtype_Decl_F_Aspects;
               when Ada_Discrete_Base_Subtype_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Task_Type_Decl | Ada_Single_Task_Type_Decl =>
         return Node.Task_Type_Decl_F_Aspects;
               when Ada_Protected_Type_Decl =>
         return Node.Protected_Type_Decl_F_Aspects;
               when Ada_Param_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Subp_Decl =>
         return Node.Subp_Decl_F_Aspects;
               when Ada_Abstract_Subp_Decl =>
         return Node.Abstract_Subp_Decl_F_Aspects;
               when Ada_Single_Task_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Single_Protected_Decl =>
         return Node.Single_Protected_Decl_F_Aspects;
               when Ada_Number_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Object_Decl | Ada_Extended_Return_Stmt_Object_Decl =>
         return Node.Object_Decl_F_Aspects;
               when Ada_Anonymous_Expr_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Internal | Ada_Package_Decl =>
         return Node.Base_Package_Decl_F_Aspects;
               when Ada_Exception_Decl =>
         return Node.Exception_Decl_F_Aspects;
               when Ada_Generic_Subp_Instantiation =>
         return Node.Generic_Subp_Instantiation_F_Aspects;
               when Ada_Generic_Package_Instantiation =>
         return Node.Generic_Package_Instantiation_F_Aspects;
               when Ada_Package_Renaming_Decl =>
         return Node.Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Package_Renaming_Decl =>
         return Node.Generic_Package_Renaming_Decl_F_Aspects;
               when Ada_Generic_Subp_Renaming_Decl =>
         return Node.Generic_Subp_Renaming_Decl_F_Aspects;
               when Ada_Abstract_Formal_Subp_Decl | Ada_Concrete_Formal_Subp_Decl =>
         return Node.Formal_Subp_Decl_F_Aspects;
               when Ada_Generic_Formal_Obj_Decl | Ada_Generic_Formal_Package | Ada_Generic_Formal_Subp_Decl | Ada_Generic_Formal_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Subp_Internal =>
         return Node.Generic_Subp_Internal_F_Aspects;
               when Ada_Generic_Subp_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Generic_Package_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Enum_Literal_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Entry_Decl =>
         return Node.Entry_Decl_F_Aspects;
               when Ada_For_Loop_Var_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Expr_Function =>
         return Node.Expr_Function_F_Aspects;
               when Ada_Null_Subp_Decl =>
         return Node.Null_Subp_Decl_F_Aspects;
               when Ada_Subp_Renaming_Decl =>
         return Node.Subp_Renaming_Decl_F_Aspects;
               when Ada_Subp_Body =>
         return Node.Subp_Body_F_Aspects;
               when Ada_Exception_Handler =>
                     return No_Bare_Ada_Node;
               when Ada_Label_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Named_Stmt_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Package_Body =>
         return Node.Package_Body_F_Aspects;
               when Ada_Task_Body =>
         return Node.Task_Body_F_Aspects;
               when Ada_Protected_Body =>
         return Node.Protected_Body_F_Aspects;
               when Ada_Entry_Body =>
                     return No_Bare_Ada_Node;
               when Ada_Entry_Index_Spec =>
                     return No_Bare_Ada_Node;
               when Ada_Protected_Body_Stub =>
         return Node.Protected_Body_Stub_F_Aspects;
               when Ada_Subp_Body_Stub =>
         return Node.Subp_Body_Stub_F_Aspects;
               when Ada_Package_Body_Stub =>
         return Node.Package_Body_Stub_F_Aspects;
               when Ada_Task_Body_Stub =>
         return Node.Task_Body_Stub_F_Aspects;
               when Ada_Incomplete_Type_Decl | Ada_Incomplete_Tagged_Type_Decl =>
                     return No_Bare_Ada_Node;
               when Ada_Abstract_State_Decl =>
         return Node.Abstract_State_Decl_F_Aspects;
         end case;
   end;
--# property-start BasicDecl.env_hook_basic_decl /usr/src/packages/BUILD/ada/ast.py:1272
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Hook_Basic_Decl
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1594;
      Var_Expr : Internal_Entity_Dotted_Name;
Var_Expr_1 : Internal_Unit;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Symbol_Type_Array_Access;
Fld_4 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1594 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1594;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1601 '<Block>' Let_Result None
--# scope-start
--# expr-start 1600 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1276
--# expr-start 1592 '<IsA PackageDecl, BasicSubpDecl, PackageRenamingDecl, GenericPackageDecl, GenericPackageInstantiation, GenericSubpInstantiation, GenericSubpDecl, SubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:1279
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Decl_Range | Ada_Basic_Subp_Decl | Ada_Package_Renaming_Decl_Range | Ada_Generic_Package_Decl_Range | Ada_Generic_Package_Instantiation_Range | Ada_Generic_Subp_Instantiation_Range | Ada_Generic_Subp_Decl_Range | Ada_Subp_Body_Range; 
--# expr-done 1592
if Is_A then
--# expr-start 1596 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1283
--# expr-start 1595 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1283
--# expr-start 1594 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:1283
--# expr-start 1593 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:1283
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 1593
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 1594
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Defining_Name_F_Name, Info => Fld.Info);
--# expr-done 1595
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 1596
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 1599 '<FieldAccess .get_unit(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1285
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1598 '<FieldAccess .as_symbol_array>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1285
--# expr-start 1597 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1285
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 1597
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_2.Node);
--# end
--# expr-done 1598
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_3, Kind => Unit_Specification, Load_If_Needed => True);
--# expr-done 1599
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= null then
   --# scope-start
   --# bind _ Var_Expr_1
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
      Finalizer_Scope_1594;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
--# expr-done 1600
Let_Result := If_Result_1; 
--# end
--# expr-done 1601
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1594;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Env_Hook_Basic_Decl;
--# end
--# property-start BasicDecl.has_top_level_env_name_impl /usr/src/packages/BUILD/ada/ast.py:1293
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name_Impl
  (Node : Bare_Basic_Decl
      ; Allow_Bodies : Boolean
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind allow_bodies Allow_Bodies
   Property_Result : Boolean;
            procedure Finalizer_Scope_103;
      Block_Var : Boolean;
Block_Var_1 : Boolean;
Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Is_A_2 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Is_A_3 : Boolean;
Not_Val_1 : Boolean;
If_Result_3 : Boolean;
Fld_2 : Boolean;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
If_Result_4 : Boolean;
Scope_Result : Boolean;
Is_A_4 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result_5 : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_103 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_103;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1619 '<Block>' Let_Result None
--# scope-start
--# expr-start 1602 '<IsA BasePackageDecl, BasicSubpDecl, GenericDecl, TaskTypeDecl, ProtectedTypeDecl, SingleTaskDecl, SingleProtectedDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:1298
Is_A := Self /= null 
and then Self.Kind in Ada_Base_Package_Decl | Ada_Basic_Subp_Decl | Ada_Generic_Decl | Ada_Task_Type_Decl_Range | Ada_Protected_Type_Decl_Range | Ada_Single_Task_Decl_Range | Ada_Single_Protected_Decl_Range; 
--# expr-done 1602
Block_Var := Is_A; 
--# bind is_decl Block_Var
--# expr-start 1603 '<IsA Body>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:1304
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Body_Node; 
--# expr-done 1603
Block_Var_1 := Is_A_1; 
--# bind is_body Block_Var_1
--# expr-start 1604 '<FieldAccess .is_compilation_unit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:1306
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1604
if Fld then
   If_Result_7 := True;
else
if Block_Var then
   If_Result_1 := True;
else
if Allow_Bodies then
   If_Result := Block_Var_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
--# expr-start 1606 '<env_node(<FieldAccess .node_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:1308
--# expr-start 1605 '<FieldAccess .node_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 1605
Env_Node := AST_Envs.Env_Node (Fld_1); 
--# expr-done 1606
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind node Var_Expr
--# expr-start 1607 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1309
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1607
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 1614 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:1310
--# expr-start 1608 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1311
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Cast_Result_1 = Var_Expr_1; 
--# expr-done 1608
if Is_Equal then
   If_Result_4 := True;
else
   --# expr-start 1613 '<FieldAccess .has_top_level_env_name_impl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1313
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
if Allow_Bodies then
   --# expr-start 1610 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1316
--# expr-start 1609 '<IsA BaseSubpBody>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:1316
Is_A_2 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 1609
Not_Val := not (Is_A_2); 
--# expr-done 1610
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 1612 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:1317
--# expr-start 1611 '<IsA BaseSubpBody>' Is_A_3 /usr/src/packages/BUILD/ada/ast.py:1317
Is_A_3 := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Subp_Body; 
--# expr-done 1611
Not_Val_1 := not (Is_A_3); 
--# expr-done 1612
   If_Result_3 := Not_Val_1;
else
   If_Result_3 := False;
end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => If_Result_3);
--# end
--# expr-done 1613
   If_Result_4 := Fld_2;
end if;
--# expr-done 1614
Scope_Result := If_Result_4;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 1615 '<IsA PrivatePart>' Is_A_4 /usr/src/packages/BUILD/ada/ast.py:1322
Is_A_4 := Var_Expr /= null 
and then Var_Expr.Kind in Ada_Private_Part_Range; 
--# expr-done 1615
if Is_A_4 then
--# expr-start 1617 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:1323
--# expr-start 1616 '<parent(<Var node>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1323
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr.Parent; 
--# expr-done 1616
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1617
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 1618 '<FieldAccess .has_top_level_env_name_impl(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1325
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => Allow_Bodies);
--# end
--# expr-done 1618
Scope_Result_1 := Fld_4;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result_5 := Result_Var;
else
   If_Result_5 := False;
end if;
   Result_Var_1 := If_Result_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_6 := Result_Var_2;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 1619
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_103;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_103;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Has_Top_Level_Env_Name_Impl;
--# end
--# property-start BasicDecl.has_top_level_env_name /usr/src/packages/BUILD/ada/ast.py:1334
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_104;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Is_A : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Fld_3 : Boolean;
Scope_Result_1 : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Result_Var_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_104 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_104;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1628 '<Block>' Let_Result None
--# scope-start
--# expr-start 1621 '<env_node(<FieldAccess .children_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:1402
--# expr-start 1620 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:1402
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 1620
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 1621
Var_Expr := Env_Node;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind node Var_Expr
--# expr-start 1622 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1403
Cast_Expr := Var_Expr; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Basic_Decl
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1622
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_1
--# expr-start 1623 '<FieldAccess .has_top_level_env_name_impl(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1404
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_1, Allow_Bodies => True);
--# end
--# expr-done 1623
Scope_Result := Fld_1;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 1624 '<IsA PrivatePart>' Is_A /usr/src/packages/BUILD/ada/ast.py:1408
Is_A := Var_Expr /= null 
and then Var_Expr.Kind in Ada_Private_Part_Range; 
--# expr-done 1624
if Is_A then
--# expr-start 1626 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1409
--# expr-start 1625 '<parent(<Var node>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1409
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Parent; 
--# expr-done 1625
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Fld_2;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1626
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 1627 '<FieldAccess .has_top_level_env_name_impl(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1410
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.has_top_level_env_name_impl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name_Impl (Node => Var_Expr_2, Allow_Bodies => True);
--# end
--# expr-done 1627
Scope_Result_1 := Fld_3;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   Result_Var_1 := If_Result;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 1628
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_104;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_104;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Has_Top_Level_Env_Name;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name_impl /usr/src/packages/BUILD/ada/ast.py:1420
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl
  (Node : Bare_Basic_Decl
      ; Self_Env : Lexical_Env
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind self_env Self_Env
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_105;
            procedure Finalizer_Scope_1748;
      Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Bare_Basic_Decl;
Var_Expr_2 : Bare_Private_Part;
Fld : Boolean;
Fld_1 : Bare_Compilation_Unit;
Fld_2 : Symbol_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_3 : Character_Type_Array_Access;
Is_A : Boolean;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
Fld_5 : Bare_Ada_Node;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Character_Type_Array_Access;
Env_Node : Bare_Ada_Node;
Cast_Result_1 : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Private_Part;
Fld_8 : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_3 : Bare_Basic_Decl;
Result_Var_1 : Bare_Basic_Decl;
Fld_9 : Lexical_Env := Empty_Env;
Fld_10 : Character_Type_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Fld_11 : Symbol_Type;
Fld_12 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Scope_Result_2 : Character_Type_Array_Access;
Result_Var_2 : Character_Type_Array_Access;
If_Result : Character_Type_Array_Access;
If_Result_1 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_105 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_105;
            procedure Finalizer_Scope_1748 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1748;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1654 '<Block>' Let_Result None
--# scope-start
--# expr-start 1653 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1424
--# expr-start 1629 '<FieldAccess .is_compilation_unit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:1427
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1629
if Fld then
   --# expr-start 1633 '<FieldAccess .sym_join(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1428
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1631 '<FieldAccess .syntactic_fully_qualified_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1429
--# expr-start 1630 '<FieldAccess .enclosing_compilation_unit>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1429
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_1 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 1630
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_2 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Fld_1);
--# end
--# expr-done 1631
--# expr-start 1632 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:1430
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 1632
--# property-call-start AdaNode.sym_join
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Self, Syms => Fld_2, Sep => Array_Lit);
--# end
--# expr-done 1633
   If_Result_1 := Fld_3;
else
--# expr-start 1634 '<IsA GenericPackageInternal, GenericSubpInternal>' Is_A /usr/src/packages/BUILD/ada/ast.py:1435
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range; 
--# expr-done 1634
if Is_A then
   --# expr-start 1639 '<FieldAccess .env_spec_fully_qualified_name_impl(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1436
--# expr-start 1636 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1436
--# expr-start 1635 '<parent(<Var Self>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1436
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Parent; 
--# expr-done 1635
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Basic_Decl
   then
      Cast_Result := Fld_4;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1636
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1638 '<FieldAccess .node_env>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1437
--# expr-start 1637 '<parent(<Var Self>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1437
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 1637
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Node_Env (Node => Fld_5);
--# expr-done 1638
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Cast_Result, Self_Env => Fld_6);
--# end
--# expr-done 1639
   If_Result := Fld_7;
else
--# expr-start 1641 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1441
--# expr-start 1640 '<env_node(<Var self_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:1441
Env_Node := AST_Envs.Env_Node (Self_Env); 
--# expr-done 1640
   if Env_Node = null
      or else Env_Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Env_Node;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 1641
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result;
else
   --# expr-start 1645 '<Cast to BasicDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:1442
--# expr-start 1643 '<Cast to PrivatePart>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:1442
--# expr-start 1642 '<env_node(<Var self_env>)>' Env_Node_1 /usr/src/packages/BUILD/ada/ast.py:1442
Env_Node_1 := AST_Envs.Env_Node (Self_Env); 
--# expr-done 1642
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Private_Part_Range
   then
      Cast_Result_2 := Env_Node_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 1643
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1644 '<parent(<Var Var_Expr_2>)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1442
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr_2.Parent; 
--# expr-done 1644
Scope_Result_1 := Fld_8;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Result_Var;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 1645
   Result_Var_1 := Cast_Result_3;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 1652 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:1446
--# expr-start 1649 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:1446
--# expr-start 1647 '<FieldAccess .env_spec_fully_qualified_name_impl(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:1446
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1646 '<FieldAccess .node_env>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1446
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Node_Env (Node => Var_Expr);
--# expr-done 1646
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Var_Expr, Self_Env => Fld_9);
--# end
--# expr-done 1647
--# expr-start 1648 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:1447
Array_Lit_1 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 1648
Concat_Result := Concat (Fld_10, Array_Lit_1); 
--# expr-done 1649
--# expr-start 1651 '<FieldAccess .image>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:1448
--# expr-start 1650 '<FieldAccess .name_symbol>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:1448
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 1650
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Image (Self => Fld_11);
--# expr-done 1651
Concat_Result_1 := Concat (Concat_Result, Fld_12); 
--# expr-done 1652
Scope_Result_2 := Concat_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1748;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Character_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result := Result_Var_2;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 1653
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1654
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_105;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_105;
               Finalizer_Scope_1748;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl;
--# end
--# property-start BasicDecl.env_spec_fully_qualified_name /usr/src/packages/BUILD/ada/ast.py:1453
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Env_Spec_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_106;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_106 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_106;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1657 '<Block>' Let_Result None
--# scope-start
--# expr-start 1656 '<FieldAccess .env_spec_fully_qualified_name_impl(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1462
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1655 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:1462
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 1655
--# property-call-start BasicDecl.env_spec_fully_qualified_name_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name_Impl (Node => Self, Self_Env => Fld);
--# end
--# expr-done 1656
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1657
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_106;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_106;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Env_Spec_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.top_level_env_name /usr/src/packages/BUILD/ada/ast.py:1465
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Top_Level_Env_Name
  (Node : Bare_Basic_Decl
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_107;
      Fld : Character_Type_Array_Access;
Fld_1 : Boolean;
If_Result : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_107 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_107;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1661 '<Block>' Let_Result None
--# scope-start
--# expr-start 1660 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1471
--# expr-start 1658 '<FieldAccess .has_top_level_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1472
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 1658
if Fld_1 then
   --# expr-start 1659 '<FieldAccess .env_spec_fully_qualified_name>' Fld /usr/src/packages/BUILD/ada/ast.py:1473
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.env_spec_fully_qualified_name
Fld := Libadalang.Implementation.Basic_Decl_P_Env_Spec_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 1659
   If_Result := Fld;
else
   If_Result := No_Character_Type_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 1660
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1661
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_107;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_107;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Top_Level_Env_Name;
--# end
--# property-start '[dispatcher]BasicDecl.child_decl_initial_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Entry_Index_Spec | Ada_Exception_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.child_decl_initial_env_name
                  Property_Result := Basic_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.child_decl_initial_env_name
                  Property_Result := Error_Decl_P_Child_Decl_Initial_Env_Name
                    (Self
                        , Private_Part
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start BasicDecl.is_formal /usr/src/packages/BUILD/ada/ast.py:1511
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Formal
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1663 '<IsA GenericFormal>' Is_A /usr/src/packages/BUILD/ada/ast.py:1512
--# expr-start 1662 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1512
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1662
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal; 
--# expr-done 1663
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Formal;
--# end
--# property-start '[dispatcher]BasicDecl.previous_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.previous_part_for_decl
                  Property_Result := Basic_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Param_Spec =>
                  --# property-call-start BaseFormalParamDecl.previous_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Previous_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.canonical_part /usr/src/packages/BUILD/ada/ast.py:1569
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Part
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1666 '<Block>' Let_Result None
--# scope-start
--# expr-start 1664 '<FieldAccess .previous_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:1574
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1664
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 1665 '<FieldAccess .canonical_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1575
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 1665
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1666
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Canonical_Part;
--# end
--# property-start BasicDecl.all_previous_parts /usr/src/packages/BUILD/ada/ast.py:1580
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Previous_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_114;
            procedure Finalizer_Scope_1754;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_114 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_114;
            procedure Finalizer_Scope_1754 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1754;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1672 '<Block>' Let_Result None
--# scope-start
--# expr-start 1667 '<FieldAccess .previous_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:1585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1667
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 1671 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1586
--# expr-start 1668 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:1587
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1668
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 1670 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:1589
--# expr-start 1669 '<FieldAccess .all_previous_parts>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1589
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 1669
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Concat_Result := Concat (Fld_1, Singleton); 
--# expr-done 1670
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 1671
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1754;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1672
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_114;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_114;
               Finalizer_Scope_1754;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_All_Previous_Parts;
--# end
--# property-start BasicDecl.all_next_parts /usr/src/packages/BUILD/ada/ast.py:1595
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Next_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_115;
            procedure Finalizer_Scope_1759;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_115 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_115;
            procedure Finalizer_Scope_1759 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1759;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1678 '<Block>' Let_Result None
--# scope-start
--# expr-start 1673 '<FieldAccess .next_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:1600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1673
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 1677 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1601
--# expr-start 1674 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:1602
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 1674
if Is_Equiv then
   If_Result := No_Internal_Entity_Basic_Decl_Array_Type;
else
   --# expr-start 1676 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:1604
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
--# expr-start 1675 '<FieldAccess .all_next_parts>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1604
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 1675
Concat_Result := Concat (Singleton, Fld_1); 
--# expr-done 1676
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 1677
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1759;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 1678
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_115;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_115;
               Finalizer_Scope_1759;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_All_Next_Parts;
--# end
--# property-start BasicDecl.all_parts /usr/src/packages/BUILD/ada/ast.py:1610
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_All_Parts
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_116;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_116 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Singleton);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_116;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1683 '<Block>' Let_Result None
--# scope-start
--# expr-start 1679 '<FieldAccess .all_previous_parts>' Fld /usr/src/packages/BUILD/ada/ast.py:1615
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.all_previous_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Previous_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1679
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind prevs Block_Var
--# expr-start 1680 '<FieldAccess .all_next_parts>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1616
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.all_next_parts
Fld_1 := Libadalang.Implementation.Basic_Decl_P_All_Next_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1680
Block_Var_1 := Fld_1; Inc_Ref (Block_Var_1);
--# bind nexts Block_Var_1
--# expr-start 1682 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:1617
--# expr-start 1681 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:1617
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
Concat_Result := Concat (Block_Var, Singleton); 
--# expr-done 1681
Concat_Result_1 := Concat (Concat_Result, Block_Var_1); 
--# expr-done 1682
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 1683
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_116;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_116;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_All_Parts;
--# end
--# property-start '[dispatcher]BasicDecl.is_static_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Component_Decl .. Ada_Package_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Entry_Decl | Ada_Generic_Subp_Internal .. Ada_Named_Stmt_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_static_decl
                  Property_Result := Basic_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.is_static_decl
                  Property_Result := Anonymous_Expr_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.is_static_decl
                  Property_Result := Discrete_Base_Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.is_static_decl
                  Property_Result := Subtype_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_static_decl
                  Property_Result := Type_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.is_static_decl
                  Property_Result := Enum_Literal_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.is_static_decl
                  Property_Result := Number_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.is_static_decl
                  Property_Result := Object_Decl_P_Is_Static_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.unshed_rebindings /usr/src/packages/BUILD/ada/ast.py:1627
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unshed_Rebindings
  (Node : Bare_Basic_Decl
      ; Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind rebindings Rebindings
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_118;
      Is_Equal : Boolean;
Fld : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Basic_Decl;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Internal_Entity_Info;
Fld_4 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Is_Equal_2 : Boolean;
Fld_7 : Env_Rebindings;
Fld_8 : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Is_Equal_3 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_10 : Env_Rebindings;
Fld_11 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_118 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_118;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1705 '<Block>' Let_Result None
--# scope-start
--# expr-start 1704 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:1634
--# expr-start 1684 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1635
Is_Equal := Rebindings = null; 
--# expr-done 1684
if Is_Equal then
   If_Result_3 := Ent;
else
--# expr-start 1687 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:1638
--# expr-start 1686 '<env_node(<FieldAccess .old_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:1638
--# expr-start 1685 '<FieldAccess .old_env>' Fld /usr/src/packages/BUILD/ada/ast.py:1638
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Rebindings.Old_Env;
Inc_Ref (Fld);
--# expr-done 1685
Env_Node := AST_Envs.Env_Node (Fld); 
--# expr-done 1686
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 1687
if Is_Equal_1 then
   --# expr-start 1701 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1640
--# expr-start 1690 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:1642
--# expr-start 1689 '<FieldAccess .rebindings>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1642
--# expr-start 1688 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1642
Fld_5 := Ent.Info;
--# expr-done 1688
Fld_6 := Fld_5.Rebindings;
--# expr-done 1689
Is_Equal_2 := Fld_6 = null; 
--# expr-done 1690
if Is_Equal_2 then
   If_Result := True;
else
   --# expr-start 1694 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:1643
--# expr-start 1691 '<FieldAccess .get_parent>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1643
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Rebindings.Parent;
--# expr-done 1691
--# expr-start 1693 '<FieldAccess .rebindings>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1643
--# expr-start 1692 '<FieldAccess .info>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1643
Fld_8 := Ent.Info;
--# expr-done 1692
Fld_9 := Fld_8.Rebindings;
--# expr-done 1693
Is_Equal_3 := Fld_7 = Fld_9; 
--# expr-done 1694
   If_Result := Is_Equal_3;
end if;
if If_Result then
   --# expr-start 1700 '<New InternalEntityBasicDecl>' New_Struct_1 None
--# expr-start 1699 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:1648
--# expr-start 1696 '<FieldAccess .from_rebound>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1651
--# expr-start 1695 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1651
Fld_3 := Ent.Info;
--# expr-done 1695
Fld_4 := Fld_3.From_Rebound;
--# expr-done 1696
--# expr-start 1698 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1650
--# expr-start 1697 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1650
Fld_1 := Ent.Info;
--# expr-done 1697
Fld_2 := Fld_1.MD;
--# expr-done 1698
New_Struct := (From_Rebound => Fld_4, MD => Fld_2, Rebindings => Rebindings); 
--# expr-done 1699
New_Struct_1 := (Info => New_Struct, Node => Self); 
--# expr-done 1700
   If_Result_1 := New_Struct_1;
else
   If_Result_1 := raise Property_Error with "Incorrect rebindings";
end if;
--# expr-done 1701
   If_Result_2 := If_Result_1;
else
   --# expr-start 1703 '<FieldAccess .unshed_rebindings(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:1657
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1702 '<FieldAccess .get_parent>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:1657
      if Rebindings = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Rebindings.Parent;
--# expr-done 1702
--# property-call-start BasicDecl.unshed_rebindings
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Ent.Node, Rebindings => Fld_10, E_Info => Ent.Info);
--# end
--# expr-done 1703
   If_Result_2 := Fld_11;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1704
Let_Result := If_Result_3; 
--# end
--# expr-done 1705
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_118;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_118;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Unshed_Rebindings;
--# end
--# property-start '[dispatcher]BasicDecl.is_imported' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_imported
                  Property_Result := Basic_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.is_imported
                  Property_Result := Generic_Subp_Decl_P_Is_Imported
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_Imported;
--# end
--# property-start '[dispatcher]BasicDecl.is_library_item' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Param_Spec | Ada_Package_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_library_item
                  Property_Result := Basic_Decl_P_Is_Library_Item
                    (Self
                    );
                  --# end
               when Ada_Generic_Package_Internal =>
                  --# property-call-start GenericPackageInternal.is_library_item
                  Property_Result := Generic_Package_Internal_P_Is_Library_Item
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_Library_Item;
--# end
--# property-start BasicDecl.decl_private_part /usr/src/packages/BUILD/ada/ast.py:1678
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Decl_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Private_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Private_Part;
      Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity_Task_Type_Decl;
Match_2 : Internal_Entity_Single_Task_Decl;
Match_3 : Internal_Entity_Protected_Type_Decl;
Match_4 : Internal_Entity_Single_Protected_Decl;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Private_Part;
Fld_1 : Internal_Entity_Task_Def;
Fld_2 : Internal_Entity_Private_Part;
Fld_3 : Internal_Entity_Single_Task_Type_Decl;
Fld_4 : Internal_Entity_Task_Def;
Fld_5 : Internal_Entity_Private_Part;
Fld_6 : Internal_Entity_Protected_Def;
Fld_7 : Internal_Entity_Private_Part;
Fld_8 : Internal_Entity_Protected_Def;
Fld_9 : Internal_Entity_Private_Part;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Private_Part;
Scope_Result : Internal_Entity_Private_Part;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Private_Part;
Scope_Result_1 : Internal_Entity_Private_Part;
Cast_Result_2 : Internal_Entity_Single_Task_Decl;
Let_Result_2 : Internal_Entity_Private_Part;
Scope_Result_2 : Internal_Entity_Private_Part;
Cast_Result_3 : Internal_Entity_Protected_Type_Decl;
Let_Result_3 : Internal_Entity_Private_Part;
Scope_Result_3 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Single_Protected_Decl;
Let_Result_4 : Internal_Entity_Private_Part;
Scope_Result_4 : Internal_Entity_Private_Part;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Private_Part;
Scope_Result_5 : Internal_Entity_Private_Part;
Match_Result : Internal_Entity_Private_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1716 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:1678
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bpd Match_0
--# expr-start 1706 '<FieldAccess .private_part>' Fld /usr/src/packages/BUILD/ada/ast.py:1679
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 1706
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ttd Match_1
--# expr-start 1708 '<FieldAccess .private_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1680
--# expr-start 1707 '<FieldAccess .definition>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1680
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Task_Def (Node => Match_1.Node.Task_Type_Decl_F_Definition, Info => Match_1.Info);
--# expr-done 1707
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Private_Part (Node => Fld_1.Node.Task_Def_F_Private_Part, Info => Fld_1.Info);
--# expr-done 1708
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind td Match_2
--# expr-start 1711 '<FieldAccess .private_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1681
--# expr-start 1710 '<FieldAccess .definition>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1681
--# expr-start 1709 '<FieldAccess .task_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1681
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_2.Node.Single_Task_Decl_F_Task_Type, Info => Match_2.Info);
--# expr-done 1709
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Task_Def (Node => Fld_3.Node.Task_Type_Decl_F_Definition, Info => Fld_3.Info);
--# expr-done 1710
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Fld_4.Node.Task_Def_F_Private_Part, Info => Fld_4.Info);
--# expr-done 1711
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ptd Match_3
--# expr-start 1713 '<FieldAccess .private_part>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1682
--# expr-start 1712 '<FieldAccess .definition>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1682
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Protected_Def (Node => Match_3.Node.Protected_Type_Decl_F_Definition, Info => Match_3.Info);
--# expr-done 1712
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Private_Part (Node => Fld_6.Node.Protected_Def_F_Private_Part, Info => Fld_6.Info);
--# expr-done 1713
Let_Result_3 := Fld_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind spd Match_4
--# expr-start 1715 '<FieldAccess .private_part>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1683
--# expr-start 1714 '<FieldAccess .definition>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1683
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Protected_Def (Node => Match_4.Node.Single_Protected_Decl_F_Definition, Info => Match_4.Info);
--# expr-done 1714
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Private_Part (Node => Fld_8.Node.Protected_Def_F_Private_Part, Info => Fld_8.Info);
--# expr-done 1715
Let_Result_4 := Fld_9; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Type_Decl .. Ada_Package_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind _ Match_5
Let_Result_5 := No_Entity_Private_Part; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1716
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Decl_Private_Part;
--# end
--# property-start '[dispatcher]BasicDecl.declarative_region' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.declarative_region
                  Property_Result := Basic_Decl_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.declarative_region
                  Property_Result := Base_Package_Decl_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body =>
                  --# property-call-start SubpBody.declarative_region
                  Property_Result := Subp_Body_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.declarative_region
                  Property_Result := Package_Body_P_Declarative_Region
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Declarative_Region;
--# end
--# property-start '[dispatcher]BasicDecl.get_aspect_assoc' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.get_aspect_assoc
                  Property_Result := Basic_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.get_aspect_assoc
                  Property_Result := Classwide_Type_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericDecl.get_aspect_assoc
                  Property_Result := Generic_Decl_P_Get_Aspect_Assoc
                    (Self
                        , Name
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.get_aspect_spec_expr /usr/src/packages/BUILD/ada/ast.py:1708
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Spec_Expr
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Aspect_Assoc;
Fld : Internal_Entity_Aspect_Assoc;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1719 '<Block>' Let_Result None
--# scope-start
--# expr-start 1717 '<FieldAccess .get_aspect_assoc(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1713
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1717
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Assoc then
   --# scope-start
--# expr-start 1718 '<FieldAccess .expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1713
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Var_Expr.Node.Aspect_Assoc_F_Expr, Info => Var_Expr.Info);
--# expr-done 1718
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1719
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect_Spec_Expr;
--# end
--# property-start BasicDecl.library_item_pragmas /usr/src/packages/BUILD/ada/ast.py:1716
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Library_Item_Pragmas
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node_List
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Pragma_Node_List;
      Fld : Internal_Entity;
Is_A : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_3 : Internal_Entity_Pragma_Node_List;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity;
Is_A_1 : Boolean;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Fld_8 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_9 : Internal_Entity_Pragma_Node_List;
If_Result : Internal_Entity_Pragma_Node_List;
If_Result_1 : Internal_Entity_Pragma_Node_List;
Let_Result : Internal_Entity_Pragma_Node_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1735 '<Block>' Let_Result None
--# scope-start
--# expr-start 1734 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1720
--# expr-start 1721 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:1721
--# expr-start 1720 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1721
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1720
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 1721
if Is_A then
   --# expr-start 1725 '<FieldAccess .pragmas>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1722
--# expr-start 1724 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1722
--# expr-start 1723 '<parent(<parent(<Var Ent>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1722
--# expr-start 1722 '<parent(<Var Ent>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1722
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1722
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 1723
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Compilation_Unit;
   end if;
--# expr-done 1724
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result.Info);
--# expr-done 1725
   If_Result_1 := Fld_3;
else
--# expr-start 1728 '<IsA LibraryItem>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:1724
--# expr-start 1727 '<parent(<parent(<Var Ent>)>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1724
--# expr-start 1726 '<parent(<Var Ent>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1724
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1726
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 1727
Is_A_1 := Fld_5.Node /= null 
and then Fld_5.Node.Kind in Ada_Library_Item_Range; 
--# expr-done 1728
if Is_A_1 then
   --# expr-start 1733 '<FieldAccess .pragmas>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1725
--# expr-start 1732 '<Cast to CompilationUnit>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1725
--# expr-start 1731 '<parent(<parent(<parent(<Var Ent>)>)>)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1725
--# expr-start 1730 '<parent(<parent(<Var Ent>)>)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1725
--# expr-start 1729 '<parent(<Var Ent>)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1725
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 1729
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Parent (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# expr-done 1730
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Parent (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# expr-done 1731
   if Fld_8.Node = null
      or else Fld_8.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
   else
         Cast_Result_1 := No_Entity_Compilation_Unit;
   end if;
--# expr-done 1732
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Pragma_Node_List (Node => Cast_Result_1.Node.Compilation_Unit_F_Pragmas, Info => Cast_Result_1.Info);
--# expr-done 1733
   If_Result := Fld_9;
else
   If_Result := No_Entity_Pragma_Node_List;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 1734
Let_Result := If_Result_1; 
--# end
--# expr-done 1735
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Library_Item_Pragmas;
--# end
--# property-start BasicDecl.get_aspect_impl /usr/src/packages/BUILD/ada/ast.py:1732
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Impl
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
      Var_Expr : Internal_Aspect;
Var_Expr_1 : Internal_Aspect;
Var_Expr_2 : Internal_Entity_Pragma_Node;
Var_Expr_3 : Internal_Entity_Base_Assoc;
Var_Expr_4 : Internal_Entity_Base_Assoc_List;
Var_Expr_5 : Internal_Entity_Aspect_Assoc;
Var_Expr_6 : Internal_Entity_Attribute_Def_Clause;
Fld : Internal_Entity_Pragma_Node;
Fld_1 : Internal_Entity_Base_Assoc_List;
Saved : Internal_Entity_Base_Assoc_List;
Fld_2 : Bare_Base_Assoc_List;
Fld_3 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Assoc;
Scope_Result : Internal_Entity_Base_Assoc;
Result_Var : Internal_Entity_Base_Assoc;
Fld_4 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Pragma_Node;
Cast_Result : Internal_Entity;
New_Struct : Internal_Aspect;
Scope_Result_2 : Internal_Aspect;
Result_Var_2 : Internal_Aspect;
Scope_Result_3 : Internal_Aspect;
Fld_5 : Internal_Entity_Aspect_Assoc;
Fld_6 : Internal_Entity_Expr;
Cast_Expr_1 : Internal_Entity_Aspect_Assoc;
Cast_Result_1 : Internal_Entity;
New_Struct_1 : Internal_Aspect;
Scope_Result_4 : Internal_Aspect;
Result_Var_3 : Internal_Aspect;
Result_Var_4 : Internal_Aspect;
Scope_Result_5 : Internal_Aspect;
Fld_7 : Internal_Entity_Attribute_Def_Clause;
Fld_8 : Internal_Entity_Expr;
Cast_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Cast_Result_2 : Internal_Entity;
New_Struct_2 : Internal_Aspect;
Scope_Result_6 : Internal_Aspect;
Result_Var_5 : Internal_Aspect;
Result_Var_6 : Internal_Aspect;
Let_Result : Internal_Aspect;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1746 '<Block>' Let_Result None
--# scope-start
--# expr-start 1736 '<FieldAccess .get_pragma(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1737
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_pragma
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1736
Var_Expr_2 := Fld;
if Var_Expr_2 /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr_2
--# expr-start 1739 '<New InternalAspect>' New_Struct /usr/src/packages/BUILD/ada/ast.py:1738
Cast_Expr := Var_Expr_2; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 1737 '<FieldAccess .args>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1739
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Base_Assoc_List (Node => Var_Expr_2.Node.Pragma_Node_F_Args, Info => Var_Expr_2.Info);
--# expr-done 1737
Var_Expr_4 := Fld_1;
if Var_Expr_4 /= No_Entity_Base_Assoc_List then
   --# scope-start
Saved := Var_Expr_4; 
Fld_2 := Saved.Node;
Get_Result := Get (Fld_2, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Assoc;
else
Fld_3 := Saved.Info;
As_Entity := (Info => Fld_3, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Assoc;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 1738 '<FieldAccess .assoc_expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1739
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1738
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
New_Struct := (Exists => True, Node => Cast_Result, Value => Result_Var_1); 
--# expr-done 1739
Scope_Result_2 := New_Struct;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Aspect;
end if;
Var_Expr_1 := Result_Var_2;
if Var_Expr_1 /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_3 := Var_Expr_1;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 1740 '<FieldAccess .get_aspect_assoc(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1741
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Name, E_Info => Ent.Info);
--# end
--# expr-done 1740
Var_Expr_5 := Fld_5;
if Var_Expr_5 /= No_Entity_Aspect_Assoc then
   --# scope-start
   --# bind aa Var_Expr_5
--# expr-start 1742 '<New InternalAspect>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:1742
Cast_Expr_1 := Var_Expr_5; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 1741 '<FieldAccess .expr>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1742
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Var_Expr_5.Node.Aspect_Assoc_F_Expr, Info => Var_Expr_5.Info);
--# expr-done 1741
New_Struct_1 := (Exists => True, Node => Cast_Result_1, Value => Fld_6); 
--# expr-done 1742
Scope_Result_4 := New_Struct_1;
   --# end
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Aspect;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr := Result_Var_4;
if Var_Expr /= No_Aspect then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_5 := Var_Expr;
   --# end
   Result_Var_6 := Scope_Result_5;
else
--# expr-start 1743 '<FieldAccess .get_representation_clause(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1743
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_representation_clause
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1743
Var_Expr_6 := Fld_7;
if Var_Expr_6 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind rc Var_Expr_6
--# expr-start 1745 '<New InternalAspect>' New_Struct_2 /usr/src/packages/BUILD/ada/ast.py:1744
Cast_Expr_2 := Var_Expr_6; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 1744 '<FieldAccess .expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1744
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Var_Expr_6.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_6.Info);
--# expr-done 1744
New_Struct_2 := (Exists => True, Node => Cast_Result_2, Value => Fld_8); 
--# expr-done 1745
Scope_Result_6 := New_Struct_2;
   --# end
   Result_Var_5 := Scope_Result_6;
else
   Result_Var_5 := No_Aspect;
end if;
   Result_Var_6 := Result_Var_5;
end if;
Let_Result := Result_Var_6; 
--# end
--# expr-done 1746
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect_Impl;
--# end
--# property-start BasicDecl.get_aspect /usr/src/packages/BUILD/ada/ast.py:1749
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Aspect
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Aspect;
            procedure Finalizer_Scope_127;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Aspect;
Map_Result : Internal_Aspect_Array_Access;
Fld_2 : Boolean;
Map_Result_1 : Internal_Aspect_Array_Access;
Get_Result : Internal_Aspect;
Let_Result : Internal_Aspect;
            procedure Finalizer_Scope_127 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_127;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1756 '<Block>' Let_Result None
--# scope-start
--# expr-start 1750 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1762
--# expr-start 1748 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1765
--# expr-start 1747 '<Symbol Inline>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Inline); 
--# expr-done 1747
Is_Equal := Name = Sym; 
--# expr-done 1748
if Is_Equal then
   --# expr-start 1749 '<FieldAccess .all_parts>' Fld /usr/src/packages/BUILD/ada/ast.py:1766
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.all_parts
Fld := Libadalang.Implementation.Basic_Decl_P_All_Parts (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1749
   If_Result := Fld;
else
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
   If_Result := Singleton;
end if;
      Inc_Ref (If_Result);
--# expr-done 1750
Block_Var := If_Result; Inc_Ref (Block_Var);
--# bind parts_to_check Block_Var
--# expr-start 1755 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1769
--# expr-start 1754 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:1769
--# expr-start 1752 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1769
declare
   Map_Result_Vec : Internal_Aspect_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_75 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_75
      --# expr-start 1751 '<FieldAccess .get_aspect_impl(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1770
      if Item_75.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_aspect_impl
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Impl (Node => Item_75.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_75.Info);
--# end
--# expr-done 1751
         declare
            Item_To_Append : constant Internal_Aspect :=
               Fld_1;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Aspect_Array
        (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Aspect_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Aspect_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Aspect_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1752
declare
   Map_Result_1_Vec : Internal_Aspect_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Aspect_Array_Access := Map_Result;
      begin
         for Item_74 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_74
               --# expr-start 1753 '<FieldAccess .exists>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1772
Fld_2 := Item_74.Exists;
--# expr-done 1753
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Aspect :=
               Item_74;
         begin
            Internal_Aspect_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Aspect_Array
        (Items_Count => Natural (Internal_Aspect_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Aspect_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Aspect_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Aspect_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1754
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 1755
Let_Result := Get_Result; 
--# end
--# expr-done 1756
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_127;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_127;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect;
--# end
--# property-start BasicDecl.has_aspect /usr/src/packages/BUILD/ada/ast.py:1777
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Has_Aspect
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_1780;
      Block_Var : Internal_Aspect;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Aspect;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_5 : Big_Integer_Type;
Big_Int : Big_Integer_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1780 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Big_Int);
            end Finalizer_Scope_1780;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1769 '<Block>' Let_Result None
--# scope-start
--# expr-start 1757 '<FieldAccess .get_aspect(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1784
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Name, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1757
Block_Var := Fld; 
--# bind a Block_Var
--# expr-start 1758 '<FieldAccess .exists>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1786
Fld_1 := Block_Var.Exists;
--# expr-done 1758
if Fld_1 then
   --# expr-start 1768 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1786
--# expr-start 1759 '<FieldAccess .is_contract_aspect(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1787
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.is_contract_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_Is_Contract_Aspect (Node => Self, Name => Name);
--# end
--# expr-done 1759
if Fld_6 then
   If_Result_1 := True;
else
--# expr-start 1760 '<FieldAccess .value>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1793
Fld_2 := Block_Var.Value;
--# expr-done 1760
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind val Var_Expr
--# expr-start 1764 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1797
--# expr-start 1763 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:1797
--# expr-start 1761 '<FieldAccess .expression_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1797
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1761
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 1762 '<FieldAccess .bool_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1797
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Self);
--# end
--# expr-done 1762
Is_Equiv := Equivalent (Cast_Result, Fld_4); 
--# expr-done 1763
Not_Val := not (Is_Equiv); 
--# expr-done 1764
if Not_Val then
   If_Result := True;
else
   --# expr-start 1767 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1798
--# expr-start 1765 '<FieldAccess .eval_as_int>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1798
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_5 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1765
--# expr-start 1766 '<BigInteger 1>' Big_Int /usr/src/packages/BUILD/ada/ast.py:1798
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 1766
Is_Equal := Equivalent (Fld_5, Big_Int); 
--# expr-done 1767
   If_Result := Is_Equal;
end if;
Scope_Result := If_Result;
   --# end
      Finalizer_Scope_1780;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
   If_Result_1 := Result_Var;
end if;
--# expr-done 1768
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1769
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1780;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Has_Aspect;
--# end
--# property-start BasicDecl.is_valid_pragma_for_name /usr/src/packages/BUILD/ada/ast.py:1803
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Valid_Pragma_For_Name
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Decl : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind decl Decl
   Property_Result : Boolean;
            procedure Finalizer_Scope_1781;
      Var_Expr : Internal_Entity_Pragma_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Pragma_Node;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Pragma_Node;
Node_Comp : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1781 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1781;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1781 '<Block>' Let_Result None
--# scope-start
--# expr-start 1770 '<Cast to Pragma>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1809
Cast_Expr := Decl; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result := Create_Internal_Entity_Pragma_Node
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Pragma_Node;
   end if;
--# expr-done 1770
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1772 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1812
--# expr-start 1771 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:1812
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Var_Expr.Node.Pragma_Node_F_Id, Info => Var_Expr.Info);
--# expr-done 1771
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Name);
--# end
--# expr-done 1772
if Fld_1 then
   --# expr-start 1778 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1814
--# expr-start 1777 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:1814
--# expr-start 1776 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1814
--# expr-start 1775 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1814
--# expr-start 1773 '<FieldAccess .associated_decls>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1814
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_decls
Fld_2 := Libadalang.Implementation.Pragma_Node_P_Associated_Decls (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1773
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_2;
      begin
         for Item_76 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_76
               --# expr-start 1774 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:1814
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Item_76; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1774
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_76;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1775
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1776
Is_Null := Get_Result.Node = null; 
--# expr-done 1777
Not_Val := not (Is_Null); 
--# expr-done 1778
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 1780 '<OrderingTest '"'"'lt'"'"'>' Node_Comp /usr/src/packages/BUILD/ada/ast.py:1817
--# expr-start 1779 '<FieldAccess .node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1817
Fld_3 := Var_Expr.Node;
--# expr-done 1779
Node_Comp := Compare (Self, Fld_3, Less_Than); 
--# expr-done 1780
   If_Result_1 := Node_Comp;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_1781;
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1781
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1781;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Valid_Pragma_For_Name;
--# end
--# property-start BasicDecl.get_pragma /usr/src/packages/BUILD/ada/ast.py:1822
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Pragma
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Pragma_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_130;
            procedure Finalizer_Scope_1792;
            procedure Finalizer_Scope_1799;
      Var_Expr : Internal_Entity_Pragma_Node;
Var_Expr_1 : Internal_Entity_Pragma_Node_List;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Bare_Declarative_Part;
Var_Expr_5 : Bare_Public_Part;
Var_Expr_6 : Bare_Private_Part;
Match_0 : Bare_Base_Package_Decl;
Match_1 : Bare_Protected_Def;
Match_2 : Bare_Ada_Node;
Var_Expr_7 : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Pragma_Node_List;
Saved : Internal_Entity_Pragma_Node_List;
Fld_1 : Bare_Pragma_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Pragma_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Pragma_Node;
Item_77 : Internal_Entity_Pragma_Node;
Bare_Item_77 : Bare_Pragma_Node;
Fld_3 : Internal_Entity_Identifier;
Fld_4 : Boolean;
Map_Result : Internal_Entity_Pragma_Node_Array_Access;
Get_Result : Internal_Entity_Pragma_Node;
Scope_Result : Internal_Entity_Pragma_Node;
Result_Var : Internal_Entity_Pragma_Node;
Scope_Result_1 : Internal_Entity_Pragma_Node;
Fld_5 : Bare_Declarative_Part;
Fld_6 : Bare_Ada_Node_List;
Scope_Result_2 : Bare_Ada_Node_List;
Result_Var_1 : Bare_Ada_Node_List;
As_Entity_1 : Internal_Entity_Ada_Node_List;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Ada_Node_List;
Fld_7 : Bare_Ada_Node_List;
Fld_8 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Item_78 : Internal_Entity;
Bare_Item_78 : Bare_Ada_Node;
Fld_9 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Fld_10 : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_11 : Bare_Ada_Node;
Fld_12 : Bare_Private_Part;
Fld_13 : Bare_Private_Part;
Match_Prefix : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Let_Result : Bare_Private_Part;
Scope_Result_4 : Bare_Private_Part;
Cast_Result_2 : Bare_Protected_Def;
Let_Result_1 : Bare_Private_Part;
Scope_Result_5 : Bare_Private_Part;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Private_Part;
Scope_Result_6 : Bare_Private_Part;
Match_Result : Bare_Private_Part;
Fld_14 : Bare_Ada_Node_List;
Scope_Result_7 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
As_Entity_3 : Internal_Entity_Ada_Node_List;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Ada_Node_List;
Fld_15 : Bare_Ada_Node_List;
Fld_16 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity;
Item_79 : Internal_Entity;
Bare_Item_79 : Bare_Ada_Node;
Fld_17 : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Scope_Result_8 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Result_Var_4 : Internal_Entity;
Scope_Result_9 : Internal_Entity;
Fld_18 : Internal_Entity_Declarative_Part;
Fld_19 : Internal_Entity_Ada_Node_List;
Scope_Result_10 : Internal_Entity_Ada_Node_List;
Result_Var_5 : Internal_Entity_Ada_Node_List;
Fld_20 : Bare_Ada_Node_List;
Fld_21 : Internal_Entity_Info;
As_Entity_5 : Internal_Entity;
Is_Null_5 : Boolean;
If_Result_5 : Internal_Entity;
Item_80 : Internal_Entity;
Bare_Item_80 : Bare_Ada_Node;
Fld_22 : Boolean;
Map_Result_3 : Internal_Entity_Array_Access;
Get_Result_3 : Internal_Entity;
Result_Var_6 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Pragma_Node;
Result_Var_7 : Internal_Entity_Pragma_Node;
Let_Result_3 : Internal_Entity_Pragma_Node;
            procedure Finalizer_Scope_130 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_130;
            procedure Finalizer_Scope_1792 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1792;
            procedure Finalizer_Scope_1799 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1799;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1810 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 1782 '<FieldAccess .library_item_pragmas>' Fld /usr/src/packages/BUILD/ada/ast.py:1828
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.library_item_pragmas
Fld := Libadalang.Implementation.Basic_Decl_P_Library_Item_Pragmas (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1782
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Pragma_Node_List then
   --# scope-start
   --# bind plist Var_Expr_1
--# expr-start 1786 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1830
--# expr-start 1785 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1830
Saved := Var_Expr_1; 
Fld_1 := Saved.Node;
declare
   Map_Result_Vec : Internal_Entity_Pragma_Node_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Pragma_Node_Array (0);
      else
      declare
         Collection : constant Bare_Pragma_Node_List := Fld_1;
      begin
         for Untyped_Item_77 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_77 := Untyped_Item_77; 
Is_Null := Bare_Item_77 = null; 
if Is_Null then
   If_Result := No_Entity_Pragma_Node;
else
Fld_2 := Saved.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_77); 
   If_Result := As_Entity;
end if;
                  Item_77 := If_Result; 
   --# scope-start
               --# bind p Item_77
               --# expr-start 1784 '<FieldAccess .name_is(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1830
--# expr-start 1783 '<FieldAccess .id>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1830
      if Item_77.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Identifier (Node => Item_77.Node.Pragma_Node_F_Id, Info => Item_77.Info);
--# expr-done 1783
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Name);
--# end
--# expr-done 1784
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Entity_Pragma_Node :=
               Item_77;
         begin
            Internal_Entity_Pragma_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Pragma_Node_Array
        (Items_Count => Natural (Internal_Entity_Pragma_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Pragma_Node_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Pragma_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Pragma_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1785
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1786
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_1792;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Pragma_Node;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Pragma_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_7 := Scope_Result_1;
else
   --# expr-start 1809 '<Cast to Pragma>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:1833
--# expr-start 1792 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:1833
--# expr-start 1791 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:1833
--# expr-start 1789 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr_4> <FieldAccess .decls>>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1833
--# expr-start 1787 '<FieldAccess .declarative_scope>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1833
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_5 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1787
Var_Expr_4 := Fld_5;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1788 '<FieldAccess .decls>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1833
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Var_Expr_4.Declarative_Part_F_Decls;
--# expr-done 1788
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Is_Null_1 := Result_Var_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Ada_Node_List;
else
As_Entity_1 := (Info => E_Info, Node => Result_Var_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 1789
Fld_7 := If_Result_1.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result_1 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_7;
      begin
         for Untyped_Item_78 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_78 := Untyped_Item_78; 
Is_Null_2 := Bare_Item_78 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_8 := If_Result_1.Info;
As_Entity_2 := (Info => Fld_8, Node => Bare_Item_78); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_78 := If_Result_2; 
   --# scope-start
               --# bind d Item_78
               --# expr-start 1790 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:1834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_78, E_Info => Ent.Info);
--# end
--# expr-done 1790
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_78;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 1791
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 1792
Var_Expr_3 := Get_Result_1;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_3 := Var_Expr_3;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 1794 '<Cast to PublicPart>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1839
--# expr-start 1793 '<FieldAccess .declarative_scope>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:1839
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_10 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1793
   if Fld_10 = null
      or else Fld_10.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld_10;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1794
Var_Expr_5 := Cast_Result;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_5
--# expr-start 1803 '<at(<filter>, 0)>' Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:1840
--# expr-start 1802 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:1840
--# expr-start 1800 '<as_entity(<Then <Match>: <Var Var_Expr_6> <FieldAccess .decls>>)>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:1840
--# expr-start 1798 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:1840
--# expr-start 1795 '<parent(<Var pp>)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:1840
      if Var_Expr_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Var_Expr_5.Parent; 
--# expr-done 1795
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_11; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_0 := Cast_Result_1; 
--# bind pkg Match_0
--# expr-start 1796 '<FieldAccess .private_part>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:1841
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Match_0.Base_Package_Decl_F_Private_Part;
--# expr-done 1796
Let_Result := Fld_12; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind ptd Match_1
--# expr-start 1797 '<FieldAccess .private_part>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:1842
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Match_1.Protected_Def_F_Private_Part;
--# expr-done 1797
Let_Result_1 := Fld_13; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abort_Absent .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Private_Present | Ada_Protected_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_6 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 1798
Var_Expr_6 := Match_Result;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1799 '<FieldAccess .decls>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:1840
      if Var_Expr_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr_6.Declarative_Part_F_Decls;
--# expr-done 1799
Scope_Result_7 := Fld_14;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Is_Null_3 := Result_Var_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Ada_Node_List;
else
As_Entity_3 := (Info => E_Info, Node => Result_Var_2); 
   If_Result_3 := As_Entity_3;
end if;
--# expr-done 1800
Fld_15 := If_Result_3.Node;
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result_2 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_15;
      begin
         for Untyped_Item_79 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_79 := Untyped_Item_79; 
Is_Null_4 := Bare_Item_79 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity;
else
Fld_16 := If_Result_3.Info;
As_Entity_4 := (Info => Fld_16, Node => Bare_Item_79); 
   If_Result_4 := As_Entity_4;
end if;
                  Item_79 := If_Result_4; 
   --# scope-start
               --# bind d Item_79
               --# expr-start 1801 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:1845
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_79, E_Info => Ent.Info);
--# end
--# expr-done 1801
               if Fld_17 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_79;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end if;
end;
--# expr-done 1802
Get_Result_2 := Get (Map_Result_2, 0, True); 
--# expr-done 1803
Scope_Result_8 := Get_Result_2;
   --# end
      Finalizer_Scope_1799;
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := No_Entity;
end if;
   Result_Var_4 := Result_Var_3;
end if;
Var_Expr_2 := Result_Var_4;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_9 := Var_Expr_2;
   --# end
   Result_Var_6 := Scope_Result_9;
else
   --# expr-start 1808 '<at(<filter>, 0)>' Get_Result_3 /usr/src/packages/BUILD/ada/ast.py:1850
--# expr-start 1807 '<filter>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:1850
--# expr-start 1804 '<FieldAccess .declarative_region>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:1850
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.declarative_region'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Declarative_Region (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1804
Var_Expr_7 := Fld_18;
if Var_Expr_7 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 1805 '<FieldAccess .decls>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:1850
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_7.Node.Declarative_Part_F_Decls, Info => Var_Expr_7.Info);
--# expr-done 1805
Scope_Result_10 := Fld_19;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := No_Entity_Ada_Node_List;
end if;
Fld_20 := Result_Var_5.Node;
declare
   Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_20 = null then
         Map_Result_3 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_20;
      begin
         for Untyped_Item_80 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_80 := Untyped_Item_80; 
Is_Null_5 := Bare_Item_80 = null; 
if Is_Null_5 then
   If_Result_5 := No_Entity;
else
Fld_21 := Result_Var_5.Info;
As_Entity_5 := (Info => Fld_21, Node => Bare_Item_80); 
   If_Result_5 := As_Entity_5;
end if;
                  Item_80 := If_Result_5; 
   --# scope-start
               --# bind d Item_80
               --# expr-start 1806 '<FieldAccess .is_valid_pragma_for_name(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:1851
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_valid_pragma_for_name
Fld_22 := Libadalang.Implementation.Basic_Decl_P_Is_Valid_Pragma_For_Name (Node => Ent.Node, Name => Name, Decl => Item_80, E_Info => Ent.Info);
--# end
--# expr-done 1806
               if Fld_22 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_80;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end if;
end;
--# expr-done 1807
Get_Result_3 := Get (Map_Result_3, 0, True); 
--# expr-done 1808
   Result_Var_6 := Get_Result_3;
end if;
   if Result_Var_6.Node = null
      or else Result_Var_6.Node.Kind in Ada_Pragma_Node_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Pragma_Node
        (Node => Result_Var_6.Node,
         Info => Result_Var_6.Info);
   else
         Cast_Result_4 := No_Entity_Pragma_Node;
   end if;
--# expr-done 1809
   Result_Var_7 := Cast_Result_4;
end if;
Let_Result_3 := Result_Var_7; 
--# end
--# expr-done 1810
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_130;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_130;
               Finalizer_Scope_1792;
               Finalizer_Scope_1799;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Pragma;
--# end
--# property-start BasicDecl.get_representation_clause /usr/src/packages/BUILD/ada/ast.py:1859
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Representation_Clause
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Attribute_Def_Clause
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Attribute_Def_Clause;
            procedure Finalizer_Scope_131;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Attribute_Def_Clause;
Attr : Internal_Entity_Attribute_Ref;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_81 : Internal_Entity;
Bare_Item_81 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Attribute_Def_Clause;
Fld_4 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Attribute_Ref;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_4 : Internal_Entity_Attribute_Def_Clause;
Let_Result_1 : Internal_Entity_Attribute_Def_Clause;
            procedure Finalizer_Scope_131 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_131;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1826 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 1825 '<Cast to AttributeDefClause.entity>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:1864
--# expr-start 1824 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1864
--# expr-start 1823 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1864
--# expr-start 1813 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:1864
--# expr-start 1811 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:1864
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 1811
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1812 '<FieldAccess .decls>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1864
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 1812
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 1813
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_81 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_81 := Untyped_Item_81; 
Is_Null_1 := Bare_Item_81 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_81); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_81 := If_Result_1; 
   --# scope-start
               --# bind d Item_81
--# expr-start 1814 '<Cast to AttributeDefClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1865
Cast_Expr := Item_81; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 1814
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 1822 '<Let attr>' Let_Result /usr/src/packages/BUILD/ada/ast.py:1866
--# scope-start
--# expr-start 1816 '<Cast to AttributeRef>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1867
--# expr-start 1815 '<FieldAccess .attribute_expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1867
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Attribute_Def_Clause_F_Attribute_Expr, Info => Var_Expr_1.Info);
--# expr-done 1815
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Attribute_Ref
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1816
Attr := Cast_Result_1; 
--# bind attr Attr
--# expr-start 1818 '<FieldAccess .name_is(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1868
--# expr-start 1817 '<FieldAccess .attribute>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1868
      if Attr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Attr.Node.Attribute_Ref_F_Attribute, Info => Attr.Info);
--# expr-done 1817
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_6 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_5.Node, Sym => Name);
--# end
--# expr-done 1818
if Fld_6 then
   --# expr-start 1821 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:1869
--# expr-start 1820 '<FieldAccess .referenced_decl>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:1869
--# expr-start 1819 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1869
      if Attr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Attr.Node.Attribute_Ref_F_Prefix, Info => Attr.Info);
--# expr-done 1819
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_8 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 1820
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 1821
   If_Result_2 := Is_Equiv;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 1822
Scope_Result_1 := Let_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_81;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 1823
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 1824
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Attribute_Def_Clause_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Attribute_Def_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_4 := No_Entity_Attribute_Def_Clause;
   end if;
--# expr-done 1825
Let_Result_1 := Cast_Result_4; 
--# end
--# expr-done 1826
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_131;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_131;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Representation_Clause;
--# end
--# property-start BasicDecl.is_compilation_unit_root /usr/src/packages/BUILD/ada/ast.py:1875
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Compilation_Unit_Root
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Ada_Node;
Match_0 : Bare_Library_Item;
Match_1 : Bare_Generic_Package_Decl;
Match_2 : Bare_Subunit;
Match_3 : Bare_Ada_Node;
Var_Expr_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Generic_Package_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subunit;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1831 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 1827 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1879
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1827
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 1830 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:1879
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind gen_pkg_decl Match_1
--# expr-start 1828 '<parent(<Var gen_pkg_decl>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1882
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_1.Parent; 
--# expr-done 1828
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 1829 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:1882
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Library_Item_Range; 
--# expr-done 1829
Scope_Result := Is_A;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := False; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 1830
Scope_Result_5 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := False;
end if;
Let_Result_4 := Result_Var_1; 
--# end
--# expr-done 1831
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Compilation_Unit_Root;
--# end
--# property-start BasicDecl.populate_dependent_units /usr/src/packages/BUILD/ada/ast.py:1888
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Populate_Dependent_Units
  (Node : Bare_Basic_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_133;
      Var_Expr : Bare_Ada_Node;
Fld : Bare_Name_Array_Access;
Fld_1 : Internal_Unit;
Fld_2 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Fld_3 : Internal_Unit;
Fld_4 : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Map_Result : Bare_Ada_Node_Array_Access;
Fld_5 : Boolean;
If_Result : Bare_Ada_Node_Array_Access;
Let_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_133 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_133;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1840 '<Block>' Let_Result None
--# scope-start
--# expr-start 1839 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:1889
--# expr-start 1832 '<FieldAccess .is_compilation_unit_root>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1890
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1832
if Fld_5 then
   --# expr-start 1838 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1891
--# expr-start 1833 '<FieldAccess .top_level_with_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:1891
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 1833
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_82 of
               Collection.Items
         loop
   --# scope-start
               --# bind package_name Item_82
--# expr-start 1835 '<FieldAccess .root>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1894
--# expr-start 1834 '<FieldAccess .referenced_unit(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1894
      if Item_82 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Item_82, Kind => Unit_Specification);
--# end
--# expr-done 1834
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.AST_Root;
--# expr-done 1835
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 1837 '<FieldAccess .root>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1897
--# expr-start 1836 '<FieldAccess .referenced_unit(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1897
      if Item_82 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Item_82, Kind => Unit_Body);
--# end
--# expr-done 1836
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.AST_Root;
--# expr-done 1837
   Result_Var := Fld_4;
end if;
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Result_Var;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1838
   If_Result := Map_Result;
else
   If_Result := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 1839
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 1840
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_133;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_133;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Populate_Dependent_Units;
--# end
--# property-start BasicDecl.should_ref_generic_formals /usr/src/packages/BUILD/ada/ast.py:1903
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Should_Ref_Generic_Formals
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Boolean;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1846 '<Block>' Let_Result None
--# scope-start
--# expr-start 1842 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1913
--# expr-start 1841 '<FieldAccess .is_compilation_unit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:1913
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 1841
Not_Val := not (Fld); 
--# expr-done 1842
if Not_Val then
   If_Result := True;
else
   --# expr-start 1844 '<IsA Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:1918
--# expr-start 1843 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1918
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 1843
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Subunit_Range; 
--# expr-done 1844
   If_Result := Is_A;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 1845 '<IsA BaseSubpBody>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:1924
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Base_Subp_Body; 
--# expr-done 1845
   If_Result_1 := Is_A_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 1846
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Should_Ref_Generic_Formals;
--# end
--# property-start BasicDecl.is_in_public_part /usr/src/packages/BUILD/ada/ast.py:1927
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Public_Part
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1849 '<IsA PublicPart>' Is_A /usr/src/packages/BUILD/ada/ast.py:1927
--# expr-start 1848 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1927
--# expr-start 1847 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1927
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 1847
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 1848
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Public_Part_Range; 
--# expr-done 1849
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_In_Public_Part;
--# end
--# property-start '[dispatcher]BasicDecl.is_in_private_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.is_in_private_part
                  Property_Result := Basic_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_in_private_part
                  Property_Result := Classwide_Type_Decl_P_Is_In_Private_Part
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.is_visible /usr/src/packages/BUILD/ada/ast.py:1931
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Visible
  (Node : Bare_Basic_Decl
      ; From_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_node From_Node
   Property_Result : Boolean;
            procedure Finalizer_Scope_137;
      Var_Expr : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_137 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_137;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1863 '<Block>' Let_Result None
--# scope-start
--# expr-start 1862 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:1940
--# expr-start 1850 '<FieldAccess .is_in_private_part>' Fld /usr/src/packages/BUILD/ada/ast.py:1943
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1850
if Fld then
--# expr-start 1853 '<EnvGet(<FieldAccess .node_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:1944
--# expr-start 1851 '<FieldAccess .node_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1944
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 1851
--# expr-start 1852 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1852
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1853
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
   begin
      for Item_83 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_83
         --# expr-start 1854 '<Eq>' Is_Equiv None
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Item_83, Cast_Result); 
--# expr-done 1854
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_3 := Quantifier_Result;
else
--# expr-start 1855 '<FieldAccess .is_in_public_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1949
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 1855
if Fld_3 then
   --# expr-start 1857 '<FieldAccess .is_visible(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1950
--# expr-start 1856 '<FieldAccess .parent_basic_decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1950
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_4 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1856
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_visible
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Visible (Node => Fld_4.Node, From_Node => From_Node, E_Info => Fld_4.Info);
--# end
--# expr-done 1857
   If_Result_2 := Fld_5;
else
--# expr-start 1858 '<FieldAccess .is_compilation_unit_root>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1955
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 1858
if Fld_6 then
   --# expr-start 1861 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1956
--# expr-start 1859 '<Cast to Body>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:1956
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 1859
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 1860 '<FieldAccess .is_subunit>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1956
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_7 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Var_Expr.Node);
--# end
--# expr-done 1860
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Not_Val := not (Result_Var); 
--# expr-done 1861
   If_Result := Not_Val;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
   If_Result_1 := raise Property_Error with "Only package-level declaration support visibility checks for now.";
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 1862
Let_Result := If_Result_3; 
--# end
--# expr-done 1863
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_137;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_137;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Visible;
--# end
--# property-start BasicDecl.subp_decl_match_signature /usr/src/packages/BUILD/ada/ast.py:1966
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Decl_Match_Signature
  (Node : Bare_Basic_Decl
      ; Other : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1868 '<Block>' Let_Result None
--# scope-start
--# expr-start 1867 '<FieldAccess .match_signature(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1968
--# expr-start 1864 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:1968
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1864
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1866 '<Cast to SubpSpec>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1969
--# expr-start 1865 '<FieldAccess .subp_spec_or_null>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1969
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Other.Node, E_Info => Other.Info);
--# end
--# expr-done 1865
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1866
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result.Node,
         Info => Cast_Result.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld.Node, Other => Cast_Result_1, Match_Name => False, E_Info => Fld.Info);
--# end
--# expr-done 1867
Let_Result := Fld_2; 
--# end
--# expr-done 1868
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Subp_Decl_Match_Signature;
--# end
--# property-start BasicDecl.base_subp_declarations /usr/src/packages/BUILD/ada/ast.py:1976
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Base_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_139;
            procedure Finalizer_Scope_1815;
            procedure Finalizer_Scope_1816;
            procedure Finalizer_Scope_1817;
      Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Subp_Spec;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Result_Var : Internal_Entity_Base_Subp_Spec;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_4 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_139 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_139;
            procedure Finalizer_Scope_1815 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_1815;
            procedure Finalizer_Scope_1816 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
            end Finalizer_Scope_1816;
            procedure Finalizer_Scope_1817 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1817;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Basic_Decl_Basic_Decl_P_Base_Subp_Declarations,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Basic_Decl,
                                As_Bare_Basic_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 1883 '<Block>' Let_Result None
--# scope-start
--# expr-start 1882 '<unique(<Then <FieldAccess .is_subprogram>: <Var _> <Then <FieldAccess .subp_spec_or_null>: <Var spec> <Then <FieldAccess .get_primitive_subp_tagged_type(...)>: <Var t> <filter_map>>>>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:1985
--# expr-start 1869 '<FieldAccess .is_subprogram>' Fld /usr/src/packages/BUILD/ada/ast.py:1985
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Ent.Node);
--# end
--# expr-done 1869
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 1870 '<FieldAccess .subp_spec_or_null>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1986
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1870
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 1871 '<FieldAccess .get_primitive_subp_tagged_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1999
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Var_Expr_1.Node, Canonicalize => False, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1871
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_2
--# expr-start 1881 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2002
--# expr-start 1875 '<EnvGet(<FieldAccess .primitives_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2002
--# expr-start 1873 '<FieldAccess .primitives_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2002
--# expr-start 1872 '<FieldAccess .full_view>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2002
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 1872
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 1873
--# expr-start 1874 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2003
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1874
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_4, Key => Fld_5, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1875
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_84 of
               Collection.Items
         loop
   --# scope-start
               --# bind bd Item_84
--# expr-start 1876 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:2005
Cast_Expr_1 := Item_84; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 1876
Var_Expr_4 := Cast_Result_1;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 1877 '<FieldAccess .subp_spec_or_null>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2005
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 1877
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_3 := Result_Var;
if Var_Expr_3 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr_3
--# expr-start 1878 '<FieldAccess .match_signature(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2012
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Var_Expr_3.Node, Other => Var_Expr_1, Match_Name => False, Use_Entity_Info => True, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 1878
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
      --# expr-start 1880 '<FieldAccess .canonical_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2004
--# expr-start 1879 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2004
Cast_Expr := Item_84; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 1879
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Cast_Result.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 1880
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Fld_6;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1881
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1817;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Scope_Result_3 := Result_Var_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1816;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Scope_Result_4 := Result_Var_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1815;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Unique_Array := Make_Unique (Result_Var_4); 
--# expr-done 1882
Let_Result := Unique_Array; Inc_Ref (Let_Result);
--# end
--# expr-done 1883
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_139;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl_Array_Access,
                     As_Internal_Entity_Basic_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_139;
               Finalizer_Scope_1815;
               Finalizer_Scope_1816;
               Finalizer_Scope_1817;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Base_Subp_Declarations;
--# end
--# property-start BasicDecl.root_subp_declarations /usr/src/packages/BUILD/ada/ast.py:2023
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Root_Subp_Declarations
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_140;
            procedure Finalizer_Scope_1824;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Basic_Decl_Array_Access;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Unique_Array : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Quantifier_Result : Boolean;
Not_Val_1 : Boolean;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Bare_Ada_Node;
Fld_8 : Bare_Base_Type_Decl;
Cast_Result_3 : Bare_Ada_Node;
Is_Equal : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_140 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_140;
            procedure Finalizer_Scope_1824 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_1824;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1905 '<Block>' Let_Result None
--# scope-start
--# expr-start 1884 '<FieldAccess .base_subp_declarations>' Fld /usr/src/packages/BUILD/ada/ast.py:2041
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 1884
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind base_decls Block_Var
--# expr-start 1891 '<unique(<map>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:2044
--# expr-start 1890 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2044
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_85 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_85
      --# expr-start 1889 '<as_bare_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2045
--# expr-start 1888 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2045
--# expr-start 1887 '<FieldAccess .primitive>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2045
--# expr-start 1886 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2045
--# expr-start 1885 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2045
Fld_1 := Item_85.Info;
--# expr-done 1885
Fld_2 := Fld_1.MD;
--# expr-done 1886
Fld_3 := Fld_2.Primitive;
--# expr-done 1887
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 1888
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 1889
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               If_Result;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1890
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 1891
Block_Var_1 := Unique_Array; Inc_Ref (Block_Var_1);
--# bind base_types Block_Var_1
--# expr-start 1904 '<map>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:2051
--# expr-start 1896 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:2051
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_1;
      begin
         for Item_87 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_87
               --# expr-start 1895 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:2052
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var_1;
   begin
      for Item_88 of
            Collection.Items
      loop
   --# scope-start
            --# bind u Item_88
--# expr-start 1893 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2054
--# expr-start 1892 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:2054
Cast_Expr := Item_87; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_88; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 1892
Not_Val := not (Is_Equiv); 
--# expr-done 1893
if Not_Val then
   --# expr-start 1894 '<FieldAccess .is_derived_type(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2055
      if Item_87.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Item_87.Node, Other_Type => Item_88, Origin => Origin, E_Info => Item_87.Info);
--# end
--# expr-done 1894
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
            if If_Result_1 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val_1 := not (Quantifier_Result); 
--# expr-done 1895
               if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_87;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1896
declare
   Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_86 of
               Collection.Items
         loop
   --# scope-start
               --# bind root_type Item_86
      --# expr-start 1903 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2060
--# expr-start 1902 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:2060
declare
   Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var;
      begin
         for Item_89 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_89
               --# expr-start 1901 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2061
--# expr-start 1899 '<FieldAccess .primitive>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2061
--# expr-start 1898 '<FieldAccess .md>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2061
--# expr-start 1897 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2061
Fld_5 := Item_89.Info;
--# expr-done 1897
Fld_6 := Fld_5.MD;
--# expr-done 1898
Fld_7 := Fld_6.Primitive;
--# expr-done 1899
--# expr-start 1900 '<FieldAccess .node>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2061
Fld_8 := Item_86.Node;
--# expr-done 1900
      Cast_Result_3 := Fld_8;
Is_Equal := Fld_7 = Cast_Result_3; 
--# expr-done 1901
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_89;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1902
Get_Result := Get (Map_Result_2, 0, True); 
--# expr-done 1903
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Get_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1824;
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1904
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1905
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_140;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_140;
               Finalizer_Scope_1824;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Root_Subp_Declarations;
--# end
--# property-start BasicDecl.find_all_overrides /usr/src/packages/BUILD/ada/ast.py:2067
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Find_All_Overrides
  (Node : Bare_Basic_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_142;
            procedure Finalizer_Scope_1827;
            procedure Finalizer_Scope_1828;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Prims : Internal_Entity_Basic_Decl_Array_Access;
Base_P : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_5 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Basic_Decl;
Is_Equal : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
Fld_9 : Bare_Type_Decl;
Cast_Result_2 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Boolean;
If_Result : Boolean;
Map_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Fld_13 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Map_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Map_Result_4 : Internal_Entity_Basic_Decl_Array_Access;
Unique_Array : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_142 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_142;
            procedure Finalizer_Scope_1827 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1827;
            procedure Finalizer_Scope_1828 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Prims);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1828;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1935 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 1906 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:2073
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1906
Block_Var := Fld; 
--# bind spec Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1907 '<FieldAccess .get_primitive_subp_tagged_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2074
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_tagged_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Tagged_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 1907
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind prim_type Block_Var_1
Var_Expr_1 := Block_Var_1;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 1908 '<FieldAccess .find_all_derived_types(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2075
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.find_all_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_All_Derived_Types (Node => Var_Expr_1.Node, Units => Units, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1908
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1827;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_2 := Result_Var_1; Inc_Ref (Block_Var_2);
--# bind derivations Block_Var_2
--# expr-start 1934 '<unique(<mapcat>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:2077
--# expr-start 1933 '<mapcat>' Map_Result_4 /usr/src/packages/BUILD/ada/ast.py:2077
declare
   Map_Result_4_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Type_Decl_Array_Access := Block_Var_2;
      begin
         for Item_90 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_90
      --# expr-start 1932 '<Let prims>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:2078
--# scope-start
--# expr-start 1913 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2080
--# expr-start 1911 '<EnvGet(<FieldAccess .primitives_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2080
--# expr-start 1909 '<FieldAccess .primitives_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2080
      if Item_90.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Item_90.Node, E_Info => Item_90.Info);
--# end
--# expr-done 1909
--# expr-start 1910 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2080
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 1910
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 1911
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_91 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_91
      --# expr-start 1912 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2081
Cast_Expr := Item_91; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1912
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 1913
Prims := Map_Result; Inc_Ref (Prims);
--# bind prims Prims
--# expr-start 1931 '<Let base_p>' Let_Result /usr/src/packages/BUILD/ada/ast.py:2082
--# scope-start
--# expr-start 1917 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2085
--# expr-start 1916 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:2085
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_92 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_92
               --# expr-start 1915 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2085
--# expr-start 1914 '<FieldAccess .node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2085
Fld_5 := Item_92.Node;
--# expr-done 1914
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 1915
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_92;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 1916
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 1917
Base_P := Get_Result; 
--# bind base_p Base_P
--# expr-start 1930 '<map>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:2087
--# expr-start 1926 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:2087
declare
   Map_Result_2_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Prims;
      begin
         for Item_94 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_94
--# expr-start 1922 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:2093
--# expr-start 1920 '<FieldAccess .primitive>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2093
--# expr-start 1919 '<FieldAccess .md>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2093
--# expr-start 1918 '<FieldAccess .info>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2093
Fld_6 := Item_94.Info;
--# expr-done 1918
Fld_7 := Fld_6.MD;
--# expr-done 1919
Fld_8 := Fld_7.Primitive;
--# expr-done 1920
--# expr-start 1921 '<FieldAccess .node>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2093
Fld_9 := Item_90.Node;
--# expr-done 1921
      Cast_Result_2 := Fld_9;
Is_Equal_1 := Fld_8 = Cast_Result_2; 
--# expr-done 1922
if Is_Equal_1 then
   --# expr-start 1925 '<FieldAccess .match_signature(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:2096
--# expr-start 1923 '<FieldAccess .subp_spec_or_null>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:2096
      if Base_P.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Base_P.Node, E_Info => Base_P.Info);
--# end
--# expr-done 1923
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1924 '<FieldAccess .subp_spec_or_null>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:2097
      if Item_94.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_94.Node, E_Info => Item_94.Info);
--# end
--# expr-done 1924
--# property-call-start BaseSubpSpec.match_signature
Fld_12 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_10.Node, Other => Fld_11, Match_Name => False, Use_Entity_Info => True, E_Info => Fld_10.Info);
--# end
--# expr-done 1925
   If_Result := Fld_12;
else
   If_Result := False;
end if;
               if If_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_94;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 1926
declare
   Map_Result_3_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result_2;
      begin
         for Item_93 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_93
      --# expr-start 1929 '<Cast to BasicDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:2103
--# expr-start 1928 '<FieldAccess .without_md>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:2103
--# expr-start 1927 '<FieldAccess .canonical_part>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:2103
      if Item_93.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_13 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Item_93.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_93.Info);
--# end
--# expr-done 1927
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.without_md
Fld_14 := Libadalang.Implementation.Ada_Node_P_Without_Md (Node => Fld_13.Node, E_Info => Fld_13.Info);
--# end
--# expr-done 1928
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 1929
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_3;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 1930
Let_Result := Map_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 1931
Let_Result_1 := Let_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 1932
         for Item_To_Append of
               Let_Result_1.Items
         loop
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1828;
         end loop;
      end;
      Map_Result_4 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_4_Vec)));
      for I in Map_Result_4.Items'Range loop
         Map_Result_4.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_4_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_4.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_4_Vec);
end;
--# expr-done 1933
Unique_Array := Make_Unique (Map_Result_4); 
--# expr-done 1934
Let_Result_2 := Unique_Array; Inc_Ref (Let_Result_2);
--# end
--# expr-done 1935
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_142;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_142;
               Finalizer_Scope_1827;
               Finalizer_Scope_1828;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Find_All_Overrides;
--# end
--# property-start '[dispatcher]BasicDecl.defining_names' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Names
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.defining_names
                  Property_Result := Abstract_State_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_names
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_names
                  Property_Result := Component_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_names
                  Property_Result := Discriminant_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.defining_names
                  Property_Result := Generic_Formal_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_names
                  Property_Result := Param_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_names
                  Property_Result := Base_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.defining_names
                  Property_Result := Base_Type_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl | Ada_Subp_Decl | Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.defining_names
                  Property_Result := Basic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
                  --# property-call-start FormalSubpDecl.defining_names
                  Property_Result := Formal_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.defining_names
                  Property_Result := Entry_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.defining_names
                  Property_Result := Enum_Literal_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_names
                  Property_Result := Base_Subp_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub =>
                  --# property-call-start PackageBodyStub.defining_names
                  Property_Result := Package_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body_Stub =>
                  --# property-call-start ProtectedBodyStub.defining_names
                  Property_Result := Protected_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.defining_names
                  Property_Result := Subp_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body_Stub =>
                  --# property-call-start TaskBodyStub.defining_names
                  Property_Result := Task_Body_Stub_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Body =>
                  --# property-call-start EntryBody.defining_names
                  Property_Result := Entry_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_names
                  Property_Result := Package_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Body =>
                  --# property-call-start ProtectedBody.defining_names
                  Property_Result := Protected_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Body =>
                  --# property-call-start TaskBody.defining_names
                  Property_Result := Task_Body_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_names
                  Property_Result := Entry_Index_Spec_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Error_Decl =>
                  --# property-call-start ErrorDecl.defining_names
                  Property_Result := Error_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.defining_names
                  Property_Result := Exception_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.defining_names
                  Property_Result := Exception_Handler_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_names
                  Property_Result := For_Loop_Var_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_names
                  Property_Result := Generic_Package_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.defining_names
                  Property_Result := Generic_Subp_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_names
                  Property_Result := Generic_Package_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.defining_names
                  Property_Result := Generic_Subp_Instantiation_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_names
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.defining_names
                  Property_Result := Generic_Subp_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Label_Decl =>
                  --# property-call-start LabelDecl.defining_names
                  Property_Result := Label_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_names
                  Property_Result := Named_Stmt_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.defining_names
                  Property_Result := Number_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.defining_names
                  Property_Result := Object_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_names
                  Property_Result := Package_Renaming_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_names
                  Property_Result := Single_Protected_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_names
                  Property_Result := Single_Task_Decl_P_Defining_Names
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Defining_Names;
--# end
--# property-start BasicDecl.defining_name /usr/src/packages/BUILD/ada/ast.py:2118
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_144;
      Fld : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_144 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_144;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1937 '<at(<FieldAccess .defining_names>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2119
--# expr-start 1936 '<FieldAccess .defining_names>' Fld /usr/src/packages/BUILD/ada/ast.py:2119
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1936
Get_Result := Get (Fld, 0, True); 
--# expr-done 1937
      Property_Result := Get_Result;
   --# end
      Finalizer_Scope_144;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_144;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Defining_Name;
--# end
--# property-start '[dispatcher]BasicDecl.defining_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl | Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl | Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Handler | Ada_Generic_Subp_Decl | Ada_Generic_Subp_Instantiation | Ada_Generic_Subp_Renaming_Decl .. Ada_Label_Decl | Ada_Number_Decl =>
                  --# property-call-start BasicDecl.defining_env
                  Property_Result := Basic_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.defining_env
                  Property_Result := Anonymous_Expr_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.defining_env
                  Property_Result := Component_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.defining_env
                  Property_Result := Discriminant_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.defining_env
                  Property_Result := Param_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
                  --# property-call-start BasePackageDecl.defining_env
                  Property_Result := Base_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.defining_env
                  Property_Result := Base_Subtype_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.defining_env
                  Property_Result := Classwide_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.defining_env
                  Property_Result := Incomplete_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.defining_env
                  Property_Result := Protected_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.defining_env
                  Property_Result := Task_Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.defining_env
                  Property_Result := Type_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.defining_env
                  Property_Result := Basic_Subp_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.defining_env
                  Property_Result := Base_Subp_Body_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body =>
                  --# property-call-start PackageBody.defining_env
                  Property_Result := Package_Body_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.defining_env
                  Property_Result := Entry_Index_Spec_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.defining_env
                  Property_Result := For_Loop_Var_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.defining_env
                  Property_Result := Generic_Package_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.defining_env
                  Property_Result := Generic_Package_Instantiation_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.defining_env
                  Property_Result := Generic_Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Named_Stmt_Decl =>
                  --# property-call-start NamedStmtDecl.defining_env
                  Property_Result := Named_Stmt_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.defining_env
                  Property_Result := Object_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Package_Renaming_Decl =>
                  --# property-call-start PackageRenamingDecl.defining_env
                  Property_Result := Package_Renaming_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Protected_Decl =>
                  --# property-call-start SingleProtectedDecl.defining_env
                  Property_Result := Single_Protected_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.defining_env
                  Property_Result := Single_Task_Decl_P_Defining_Env
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.identity_type /usr/src/packages/BUILD/ada/ast.py:2136
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Identity_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Exception_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Object_Decl;
Match_3 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Exception_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Object_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_4 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1942 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 1941 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2137
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Exception_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Exception_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 1938 '<FieldAccess .exc_id_type>' Fld /usr/src/packages/BUILD/ada/ast.py:2138
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.exc_id_type
Fld := Libadalang.Implementation.Ada_Node_P_Exc_Id_Type (Node => Self);
--# end
--# expr-done 1938
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 1939 '<FieldAccess .task_id_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2139
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.task_id_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 1939
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 1940 '<FieldAccess .task_id_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2143
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.task_id_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 1940
Let_Result_2 := Fld_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abstract_State_Decl .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 1941
Let_Result_4 := Match_Result; 
--# end
--# expr-done 1942
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Identity_Type;
--# end
--# property-start '[dispatcher]BasicDecl.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.array_ndims
                  Property_Result := Basic_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.array_ndims
                  Property_Result := Component_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_ndims
                  Property_Result := Base_Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_ndims
                  Property_Result := Base_Subtype_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.array_ndims
                  Property_Result := Type_Decl_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.is_array /usr/src/packages/BUILD/ada/ast.py:2151
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Array
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Integer;
Comp_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1943 '<FieldAccess .array_ndims>' Fld /usr/src/packages/BUILD/ada/ast.py:2151
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 1943
Comp_Result := Fld > 0; 
      Property_Result := Comp_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Array;
--# end
--# property-start '[dispatcher]BasicDecl.expr_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Package_Body_Stub .. Ada_Task_Body | Ada_Error_Decl .. Ada_Exception_Decl | Ada_Generic_Package_Decl .. Ada_Named_Stmt_Decl | Ada_Object_Decl .. Ada_Single_Protected_Decl =>
                  --# property-call-start BasicDecl.expr_type
                  Property_Result := Basic_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.expr_type
                  Property_Result := Base_Type_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.expr_type
                  Property_Result := Basic_Subp_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.expr_type
                  Property_Result := Base_Subp_Body_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Index_Spec =>
                  --# property-call-start EntryIndexSpec.expr_type
                  Property_Result := Entry_Index_Spec_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Handler =>
                  --# property-call-start ExceptionHandler.expr_type
                  Property_Result := Exception_Handler_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_For_Loop_Var_Decl =>
                  --# property-call-start ForLoopVarDecl.expr_type
                  Property_Result := For_Loop_Var_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Number_Decl =>
                  --# property-call-start NumberDecl.expr_type
                  Property_Result := Number_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Single_Task_Decl =>
                  --# property-call-start SingleTaskDecl.expr_type
                  Property_Result := Single_Task_Decl_P_Expr_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Expr_Type;
--# end
--# property-start '[dispatcher]BasicDecl.type_expression' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Generic_Package_Internal .. Ada_Synth_Anonymous_Type_Decl | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.type_expression
                  Property_Result := Basic_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_State_Decl =>
                  --# property-call-start AbstractStateDecl.type_expression
                  Property_Result := Abstract_State_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Expr_Decl =>
                  --# property-call-start AnonymousExprDecl.type_expression
                  Property_Result := Anonymous_Expr_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.type_expression
                  Property_Result := Component_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discriminant_Spec =>
                  --# property-call-start DiscriminantSpec.type_expression
                  Property_Result := Discriminant_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Formal_Obj_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start GenericFormal.type_expression
                  Property_Result := Generic_Formal_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.type_expression
                  Property_Result := Param_Spec_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.type_expression
                  Property_Result := Basic_Subp_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.type_expression
                  Property_Result := Base_Subp_Body_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subp_Body_Stub =>
                  --# property-call-start SubpBodyStub.type_expression
                  Property_Result := Subp_Body_Stub_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
                  --# property-call-start ObjectDecl.type_expression
                  Property_Result := Object_Decl_P_Type_Expression
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.subp_spec_or_null /usr/src/packages/BUILD/ada/ast.py:2198
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Subp_Spec_Or_Null
  (Node : Bare_Basic_Decl
      ; Follow_Generic : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_generic Follow_Generic
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Match_0 : Internal_Entity_Basic_Subp_Decl;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity_Subp_Body_Stub;
Match_3 : Internal_Entity_Generic_Subp_Decl;
Match_4 : Internal_Entity_Generic_Subp_Instantiation;
Match_5 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Generic_Subp_Internal;
Fld_4 : Internal_Entity_Subp_Spec;
If_Result : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Cast_Expr : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Subp_Decl;
Let_Result : Internal_Entity_Base_Subp_Spec;
Scope_Result : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Body;
Cast_Result_3 : Internal_Entity_Base_Subp_Spec;
Let_Result_1 : Internal_Entity_Base_Subp_Spec;
Scope_Result_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_4 : Internal_Entity_Subp_Body_Stub;
Cast_Result_5 : Internal_Entity_Base_Subp_Spec;
Let_Result_2 : Internal_Entity_Base_Subp_Spec;
Scope_Result_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_6 : Internal_Entity_Generic_Subp_Decl;
Cast_Result_7 : Internal_Entity_Base_Subp_Spec;
Let_Result_3 : Internal_Entity_Base_Subp_Spec;
Scope_Result_3 : Internal_Entity_Base_Subp_Spec;
Cast_Result_8 : Internal_Entity_Generic_Subp_Instantiation;
Let_Result_4 : Internal_Entity_Base_Subp_Spec;
Scope_Result_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity_Subp_Spec;
Cast_Result_10 : Internal_Entity_Base_Subp_Spec;
Let_Result_5 : Internal_Entity_Base_Subp_Spec;
Scope_Result_5 : Internal_Entity_Base_Subp_Spec;
Match_Result : Internal_Entity_Base_Subp_Spec;
Let_Result_6 : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1954 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 1953 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2205
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind subp Match_0
--# expr-start 1944 '<FieldAccess .subp_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:2206
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_0.Node, E_Info => Match_0.Info);
--# end
--# expr-done 1944
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind subp Match_1
--# expr-start 1945 '<FieldAccess .subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2207
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Match_1.Node.Base_Subp_Body_F_Subp_Spec, Info => Match_1.Info);
--# expr-done 1945
      Cast_Result_3 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind subp Match_2
--# expr-start 1946 '<FieldAccess .subp_spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2208
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Match_2.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_2.Info);
--# expr-done 1946
      Cast_Result_5 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind gsp Match_3
--# expr-start 1949 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:2210
if Follow_Generic then
   --# expr-start 1948 '<FieldAccess .subp_spec>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2210
--# expr-start 1947 '<FieldAccess .subp_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2210
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Generic_Subp_Internal (Node => Match_3.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Match_3.Info);
--# expr-done 1947
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 1948
   If_Result := Fld_4;
else
   If_Result := No_Entity_Subp_Spec;
end if;
--# expr-done 1949
      Cast_Result_7 := Create_Internal_Entity_Base_Subp_Spec
        (Node => If_Result.Node,
         Info => If_Result.Info);
Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Generic_Subp_Instantiation
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_8; 
--# bind gsi Match_4
--# expr-start 1952 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2212
if Follow_Generic then
   --# expr-start 1951 '<FieldAccess .subp_spec_or_null(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2213
--# expr-start 1950 '<FieldAccess .designated_generic_decl>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2213
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_5 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 1950
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_5.Node, Follow_Generic => True, E_Info => Fld_5.Info);
--# end
--# expr-done 1951
   If_Result_1 := Fld_6;
else
Cast_Expr := No_Entity_Subp_Spec; 
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result;
end if;
--# expr-done 1952
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Synth_Anonymous_Type_Decl | Ada_Package_Body_Stub .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation | Ada_Generic_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_9; 
--# bind _ Match_5
Cast_Expr_1 := No_Entity_Subp_Spec; 
      Cast_Result_10 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_5 := Cast_Result_10; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 1953
Let_Result_6 := Match_Result; 
--# end
--# expr-done 1954
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Subp_Spec_Or_Null;
--# end
--# property-start BasicDecl.formal_param_holder_or_null /usr/src/packages/BUILD/ada/ast.py:2219
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Formal_Param_Holder_Or_Null
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Match_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1959 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 1958 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2220
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 1955 '<FieldAccess .discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:2221
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_0.Node.Type_Decl_F_Discriminants, Info => Match_0.Info);
--# expr-done 1955
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 1956 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2222
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 1956
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 1957 '<FieldAccess .subp_spec_or_null(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2223
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, Follow_Generic => True, E_Info => Ent.Info);
--# end
--# expr-done 1957
      Cast_Result_5 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 1958
Let_Result_3 := Match_Result; 
--# end
--# expr-done 1959
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Formal_Param_Holder_Or_Null;
--# end
--# property-start BasicDecl.is_subprogram /usr/src/packages/BUILD/ada/ast.py:2227
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Subprogram
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 1961 '<Block>' Let_Result None
--# scope-start
--# expr-start 1960 '<IsA BasicSubpDecl, BaseSubpBody, SubpBodyStub, EntryDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:2237
Is_A := Self /= null 
and then Self.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body | Ada_Subp_Body_Stub_Range | Ada_Entry_Decl_Range; 
--# expr-done 1960
Let_Result := Is_A; 
--# end
--# expr-done 1961
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Subprogram;
--# end
--# property-start BasicDecl.is_stream_subprogram_for_type /usr/src/packages/BUILD/ada/ast.py:2240
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Stream_Subprogram_For_Type
  (Node : Bare_Basic_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Return_Obj : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind return_obj Return_Obj
   Property_Result : Boolean;
            procedure Finalizer_Scope_154;
            procedure Finalizer_Scope_1847;
            procedure Finalizer_Scope_1848;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Single_Formal_Array_Access;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Single_Formal_Array_Access;
Scope_Result_1 : Internal_Single_Formal_Array_Access;
Result_Var_1 : Internal_Single_Formal_Array_Access;
Fld_5 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_6 : Boolean;
Get_Result : Internal_Single_Formal;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
Get_Result_1 : Internal_Single_Formal;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_154 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_1);
            end Finalizer_Scope_154;
            procedure Finalizer_Scope_1847 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1847;
            procedure Finalizer_Scope_1848 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1848;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1989 '<Block>' Let_Result None
--# scope-start
--# expr-start 1971 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:2243
--# expr-start 1970 '<FieldAccess .classwide_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2243
--# expr-start 1969 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2243
--# expr-start 1968 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Stream_Type)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2243
--# expr-start 1965 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:2243
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1964 '<ArrayLiteral>' Array_Lit None
--# expr-start 1962 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 1962
--# expr-start 1963 '<Symbol Streams>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Streams); 
--# expr-done 1963
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 1964
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 1965
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 1966 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2243
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 1966
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1847;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 1967 '<Symbol Root_Stream_Type>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Stream_Type); 
--# expr-done 1967
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 1968
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 1969
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 1970
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 1971
Block_Var := Cast_Result_1; 
--# bind root_stream_type Block_Var
--# expr-start 1972 '<FieldAccess .subp_spec_or_null>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2249
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1972
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 1973 '<FieldAccess .unpacked_formal_params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2249
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 1973
Scope_Result_1 := Fld_4;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1848;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Single_Formal_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_1 := Result_Var_1; Inc_Ref (Block_Var_1);
--# bind params Block_Var_1
--# expr-start 1988 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <And>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2251
--# expr-start 1974 '<FieldAccess .origin_node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2252
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_5 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 1974
Bound_Origin := Fld_5; 
--# expr-start 1975 '<FieldAccess .is_subprogram>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2253
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 1975
if Fld_6 then
   --# expr-start 1979 '<FieldAccess .is_access_to(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2254
--# expr-start 1978 '<FieldAccess .formal_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2254
--# expr-start 1977 '<FieldAccess .spec>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2254
--# expr-start 1976 '<at(<Var params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2254
Get_Result := Get (Block_Var_1, 0, True); 
--# expr-done 1976
Fld_7 := Get_Result.Spec;
--# expr-done 1977
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Bound_Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 1978
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_access_to
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Access_To (Node => Fld_8.Node, Typ => Block_Var, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 1979
   If_Result := Fld_9;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 1987 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2255
if Return_Obj then
   --# expr-start 1982 '<FieldAccess .matching_formal_type(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:2257
--# expr-start 1981 '<FieldAccess .return_type>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:2257
--# expr-start 1980 '<FieldAccess .subp_spec_or_null>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:2257
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1980
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_10.Node, Origin => Bound_Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 1981
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_11.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 1982
   If_Result_1 := Fld_12;
else
   --# expr-start 1986 '<FieldAccess .matching_formal_type(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:2258
--# expr-start 1985 '<FieldAccess .formal_type>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:2258
--# expr-start 1984 '<FieldAccess .spec>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:2258
--# expr-start 1983 '<at(<Var params>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:2258
Get_Result_1 := Get (Block_Var_1, 1, True); 
--# expr-done 1983
Fld_13 := Get_Result_1.Spec;
--# expr-done 1984
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_14 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_13.Node, Origin => Bound_Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 1985
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_15 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_14.Node, Formal_Type => Typ, Origin => Bound_Origin, E_Info => Fld_14.Info);
--# end
--# expr-done 1986
   If_Result_1 := Fld_15;
end if;
--# expr-done 1987
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 1988
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 1989
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_154;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_154;
               Finalizer_Scope_1847;
               Finalizer_Scope_1848;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Stream_Subprogram_For_Type;
--# end
--# property-start BasicDecl.can_be_paramless /usr/src/packages/BUILD/ada/ast.py:2263
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Can_Be_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 1995 '<Block>' Let_Result None
--# scope-start
--# expr-start 1990 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:2268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1990
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 1994 '<FieldAccess .paramless(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2269
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1993 '<FieldAccess .dottable_subp>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2270
--# expr-start 1992 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2270
--# expr-start 1991 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2270
Fld_1 := Ent.Info;
--# expr-done 1991
Fld_2 := Fld_1.MD;
--# expr-done 1992
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 1993
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => True);
--# end
--# expr-done 1994
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 1995
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Can_Be_Paramless;
--# end
--# property-start BasicDecl.is_paramless /usr/src/packages/BUILD/ada/ast.py:2276
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Paramless
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Boolean;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2001 '<Block>' Let_Result None
--# scope-start
--# expr-start 1996 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:2281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 1996
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 2000 '<FieldAccess .paramless(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2282
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 1999 '<FieldAccess .dottable_subp>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2283
--# expr-start 1998 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2283
--# expr-start 1997 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2283
Fld_1 := Ent.Info;
--# expr-done 1997
Fld_2 := Fld_1.MD;
--# expr-done 1998
Fld_3 := Fld_2.Dottable_Subp;
--# expr-done 1999
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_3, Can_Be => False);
--# end
--# expr-done 2000
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2001
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Paramless;
--# end
--# property-start '[dispatcher]BasicDecl.constrain_prefix' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Discriminant_Spec .. Ada_Synth_Anonymous_Type_Decl | Ada_Expr_Function .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.constrain_prefix
                  Property_Result := Basic_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl =>
                  --# property-call-start ComponentDecl.constrain_prefix
                  Property_Result := Component_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.constrain_prefix
                  Property_Result := Basic_Subp_Decl_P_Constrain_Prefix
                    (Self
                        , Prefix
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.relative_name /usr/src/packages/BUILD/ada/ast.py:2315
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
Scope_Result : Internal_Entity_Single_Tok_Node;
Result_Var : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2002 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2316
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2002
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2003 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2316
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2003
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Single_Tok_Node;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Relative_Name;
--# end
--# property-start BasicDecl.relative_name_text /usr/src/packages/BUILD/ada/ast.py:2322
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Relative_Name_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Symbol_Type;
      Var_Expr : Internal_Entity_Single_Tok_Node;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
Scope_Result : Symbol_Type;
Result_Var : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2004 '<FieldAccess .relative_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2323
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2004
Var_Expr := Fld;
if Var_Expr /= No_Entity_Single_Tok_Node then
   --# scope-start
--# expr-start 2005 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:2323
Fld_1 := Var_Expr.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 2005
Scope_Result := Sym;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := null;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Relative_Name_Text;
--# end
--# property-start BasicDecl.name_symbol /usr/src/packages/BUILD/ada/ast.py:2328
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Name_Symbol
  (Node : Bare_Basic_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2008 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:2328
--# expr-start 2007 '<FieldAccess .relative_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2328
--# expr-start 2006 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2328
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2006
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.relative_name
Fld := Libadalang.Implementation.Basic_Decl_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2007
Fld_1 := Fld.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 2008
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Name_Symbol;
--# end
--# property-start BasicDecl.basic_decl_next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:2331
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Basic_Decl_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_161;
      Block_Var : Internal_Unit;
Match_0 : Bare_Body_Node;
Match_1 : Bare_Basic_Decl;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Bare_Compilation_Unit;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_3 : Internal_Unit;
Scope_Result_1 : Internal_Unit;
Result_Var_1 : Internal_Unit;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Internal_Unit;
Scope_Result_2 : Internal_Unit;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Internal_Unit;
Scope_Result_3 : Internal_Unit;
Match_Result : Internal_Unit;
Fld_4 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_161 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_161;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2019 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2014 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2337
--# expr-start 2010 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2337
--# expr-start 2009 '<FieldAccess .enclosing_compilation_unit>' Fld /usr/src/packages/BUILD/ada/ast.py:2337
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 2009
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld);
--# end
--# expr-done 2010
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := null; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 2011 '<as_bare_entity(<Var b>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2340
Is_Null := Match_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Match_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2011
Var_Expr_1 := If_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2012 '<FieldAccess .defining_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2340
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2012
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2013 '<FieldAccess .referenced_unit(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2340
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_unit
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Unit (Node => Var_Expr.Node, Kind => Unit_Body);
--# end
--# expr-done 2013
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := null;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2014
Block_Var := Match_Result; 
--# expr-start 2018 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:2344
--# expr-start 2017 '<EnvGet(<FieldAccess .children_env>, __nextpart)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2344
--# expr-start 2015 '<FieldAccess .children_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2344
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2015
--# expr-start 2016 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 2016
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Minimal), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 2017
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2018
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 2019
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_161;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_161;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Basic_Decl_Next_Part_For_Decl;
--# end
--# property-start '[dispatcher]BasicDecl.next_part_for_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Decl (Self.Kind) is
               when Ada_Abstract_State_Decl .. Ada_Anonymous_Expr_Decl | Ada_Generic_Package_Internal .. Ada_Package_Decl | Ada_Expr_Function .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Error_Decl | Ada_Exception_Handler .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
                  --# property-call-start BasicDecl.next_part_for_decl
                  Property_Result := Basic_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Component_Decl .. Ada_Param_Spec =>
                  --# property-call-start BaseFormalParamDecl.next_part_for_decl
                  Property_Result := Base_Formal_Param_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.next_part_for_decl
                  Property_Result := Base_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.next_part_for_decl
                  Property_Result := Protected_Type_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal =>
                  --# property-call-start BasicSubpDecl.next_part_for_decl
                  Property_Result := Basic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
                  --# property-call-start BodyStub.next_part_for_decl
                  Property_Result := Body_Stub_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Exception_Decl =>
                  --# property-call-start ExceptionDecl.next_part_for_decl
                  Property_Result := Exception_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.next_part_for_decl
                  Property_Result := Generic_Subp_Decl_P_Next_Part_For_Decl
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicDecl.body_part_for_decl /usr/src/packages/BUILD/ada/ast.py:2363
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Body_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_2 : Internal_Entity_Body_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2024 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2023 '<Cast to Body>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:2372
--# expr-start 2020 '<FieldAccess .next_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:2372
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2020
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 2022 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2372
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 2021 '<FieldAccess .next_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2373
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 2021
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2022
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 2023
Let_Result_2 := Cast_Result_2; 
--# end
--# expr-done 2024
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Body_Part_For_Decl;
--# end
--# property-start BasicDecl.fully_qualified_name_impl /usr/src/packages/BUILD/ada/ast.py:2378
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Impl
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_164;
            procedure Finalizer_Scope_1855;
            procedure Finalizer_Scope_1856;
            procedure Finalizer_Scope_1857;
            procedure Finalizer_Scope_1858;
            procedure Finalizer_Scope_1859;
      Block_Var : Bare_Single_Tok_Node_Array_Access;
Var_Expr : Bare_Single_Tok_Node_Array_Access;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Defining_Name;
Var_Expr_3 : Bare_Single_Tok_Node_Array_Access;
Var_Expr_4 : Bare_Subunit;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Result_Var : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Internal_Entity_Defining_Name;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Result_Var_1 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Is_A : Boolean;
If_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Result_Var_2 : Bare_Single_Tok_Node_Array_Access;
Fld_6 : Boolean;
If_Result_1 : Bare_Single_Tok_Node_Array_Access;
Fld_7 : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_8 : Bare_Name;
Fld_9 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Result_Var_3 : Bare_Single_Tok_Node_Array_Access;
Concat_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_4 : Bare_Single_Tok_Node_Array_Access;
Result_Var_4 : Bare_Single_Tok_Node_Array_Access;
Let_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_164 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_164;
            procedure Finalizer_Scope_1855 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1855;
            procedure Finalizer_Scope_1856 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1856;
            procedure Finalizer_Scope_1857 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_1857;
            procedure Finalizer_Scope_1858 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_1858;
            procedure Finalizer_Scope_1859 is
            begin
                     Dec_Ref (Var_Expr_3);
            end Finalizer_Scope_1859;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2041 '<Block>' Let_Result None
--# scope-start
--# expr-start 2035 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2383
--# expr-start 2025 '<FieldAccess .is_compilation_unit_root>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2384
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Ent.Node);
--# end
--# expr-done 2025
if Fld_6 then
   --# expr-start 2027 '<FieldAccess .as_single_tok_node_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2385
--# expr-start 2026 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2385
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2026
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld.Node);
--# end
--# expr-done 2027
   If_Result_1 := Fld_1;
else
--# expr-start 2028 '<FieldAccess .parent_basic_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2387
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.parent_basic_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Parent_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2028
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2029 '<FieldAccess .fully_qualified_name_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2387
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2029
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1855;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Bare_Single_Tok_Node_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 2034 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:2388
--# expr-start 2030 '<IsA GenericPackageInternal, GenericSubpInternal>' Is_A /usr/src/packages/BUILD/ada/ast.py:2389
Is_A := Self /= null 
and then Self.Kind in Ada_Generic_Package_Internal_Range | Ada_Generic_Subp_Internal_Range; 
--# expr-done 2030
if Is_A then
   If_Result := Var_Expr;
else
   --# expr-start 2033 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2391
--# expr-start 2031 '<FieldAccess .defining_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2391
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2031
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2032 '<FieldAccess .as_single_tok_node_array>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2391
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Var_Expr_2.Node);
--# end
--# expr-done 2032
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1857;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result := Concat (Var_Expr, Result_Var_1); 
--# expr-done 2033
   If_Result := Concat_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 2034
Scope_Result_2 := If_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1856;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_1 := Result_Var_2;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 2035
Block_Var := If_Result_1; Inc_Ref (Block_Var);
--# bind fqn Block_Var
--# expr-start 2040 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:2395
--# expr-start 2037 '<Cast to Subunit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2395
--# expr-start 2036 '<parent(<Var Self>)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2395
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Parent; 
--# expr-done 2036
   if Fld_7 = null
      or else Fld_7.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld_7;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2037
Var_Expr_4 := Cast_Result;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2039 '<FieldAccess .as_single_tok_node_array>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2395
--# expr-start 2038 '<FieldAccess .name>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2395
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Var_Expr_4.Subunit_F_Name;
--# expr-done 2038
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_9 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_8);
--# end
--# expr-done 2039
Scope_Result_3 := Fld_9;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1858;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Bare_Single_Tok_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Concat_Result_1 := Concat (Result_Var_3, Block_Var); 
--# expr-done 2040
Var_Expr_3 := Concat_Result_1;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Bare_Single_Tok_Node_Array_Type then
   --# scope-start
   --# bind e Var_Expr_3
Scope_Result_4 := Var_Expr_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1859;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := Block_Var;
end if;
      Inc_Ref (Result_Var_4);
Let_Result := Result_Var_4; Inc_Ref (Let_Result);
--# end
--# expr-done 2041
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_164;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_164;
               Finalizer_Scope_1855;
               Finalizer_Scope_1856;
               Finalizer_Scope_1857;
               Finalizer_Scope_1858;
               Finalizer_Scope_1859;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name_Impl;
--# end
--# property-start BasicDecl.fully_qualified_name_array /usr/src/packages/BUILD/ada/ast.py:2399
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name_Array
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_165;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_165 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_165;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2045 '<Block>' Let_Result None
--# scope-start
--# expr-start 2044 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2404
--# expr-start 2042 '<FieldAccess .fully_qualified_name_impl>' Fld /usr/src/packages/BUILD/ada/ast.py:2404
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2042
declare
   Map_Result_Vec : Symbol_Type_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_95 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_95
      --# expr-start 2043 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:2404
Sym := Get_Symbol (Item_95); 
--# expr-done 2043
         declare
            Item_To_Append : constant Symbol_Type :=
               Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Symbol_Type_Array
        (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Symbol_Type_Vectors.Get
           (Map_Result_Vec,
            I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Symbol_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2044
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2045
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_165;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_165;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name_Array;
--# end
--# property-start BasicDecl.fully_qualified_name /usr/src/packages/BUILD/ada/ast.py:2407
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_166;
            procedure Finalizer_Scope_1861;
      Fld : Bare_Single_Tok_Node_Array_Access;
Fld_1 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_166 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_166;
            procedure Finalizer_Scope_1861 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1861;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2051 '<Block>' Let_Result None
--# scope-start
--# expr-start 2050 '<FieldAccess .string_join(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2411
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2048 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2412
--# expr-start 2046 '<FieldAccess .fully_qualified_name_impl>' Fld /usr/src/packages/BUILD/ada/ast.py:2412
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_impl
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Impl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2046
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_96 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_96
      --# expr-start 2047 '<FieldAccess .text>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2412
      if Item_96 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Item_96);
--# expr-done 2047
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1861;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2048
--# expr-start 2049 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2413
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 2049
--# property-call-start AdaNode.string_join
Fld_2 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit);
--# end
--# expr-done 2050
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2051
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_166;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_166;
               Finalizer_Scope_1861;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.canonical_fully_qualified_name /usr/src/packages/BUILD/ada/ast.py:2417
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Canonical_Fully_Qualified_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_167;
            procedure Finalizer_Scope_1862;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_167 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_167;
            procedure Finalizer_Scope_1862 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1862;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2057 '<Block>' Let_Result None
--# scope-start
--# expr-start 2056 '<FieldAccess .string_join(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2422
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2054 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2423
--# expr-start 2052 '<FieldAccess .fully_qualified_name_array>' Fld /usr/src/packages/BUILD/ada/ast.py:2423
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2052
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Symbol_Type_Array_Access := Fld;
      begin
         for Item_97 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_97
      --# expr-start 2053 '<FieldAccess .image>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2423
      if Item_97 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Image (Self => Item_97);
--# expr-done 2053
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1862;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2054
--# expr-start 2055 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2424
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 2055
--# property-call-start AdaNode.string_join
Fld_2 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit);
--# end
--# expr-done 2056
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2057
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_167;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_167;
               Finalizer_Scope_1862;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Canonical_Fully_Qualified_Name;
--# end
--# property-start BasicDecl.unique_identifying_name /usr/src/packages/BUILD/ada/ast.py:2428
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Unique_Identifying_Name
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_168;
            procedure Finalizer_Scope_1863;
            procedure Finalizer_Scope_1864;
      Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Character_Type_Array_Access;
Fld_1 : Character_Type_Array_Access;
Fld_2 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Match_Result : Character_Type_Array_Access;
Let_Result_2 : Character_Type_Array_Access;
            procedure Finalizer_Scope_168 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_168;
            procedure Finalizer_Scope_1863 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1863;
            procedure Finalizer_Scope_1864 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1864;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2063 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2062 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2439
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 2058 '<FieldAccess .custom_id_text>' Fld /usr/src/packages/BUILD/ada/ast.py:2440
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2058
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1863;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_Type_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 2061 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2441
--# expr-start 2059 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2441
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2059
--# expr-start 2060 '<FieldAccess .custom_id_text>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2442
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2060
Concat_Result := Concat (Fld_1, Fld_2); 
--# expr-done 2061
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1864;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 2062
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 2063
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_168;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_168;
               Finalizer_Scope_1863;
               Finalizer_Scope_1864;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Unique_Identifying_Name;
--# end
--# property-start BasicDecl.custom_id_text /usr/src/packages/BUILD/ada/ast.py:2447
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Custom_Id_Text
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_169;
            procedure Finalizer_Scope_1865;
            procedure Finalizer_Scope_1866;
            procedure Finalizer_Scope_1867;
            procedure Finalizer_Scope_1868;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Single_Formal_Array_Access;
Var_Expr_2 : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Single_Formal_Array_Access;
Array_Lit : Character_Type_Array_Access;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Character_Type_Array_Access;
Map_Result : Character_Type_Array_Access_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Fld_5 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Array_Lit_2 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Scope_Result : Character_Type_Array_Access;
Result_Var : Character_Type_Array_Access;
Fld_6 : Internal_Entity_Type_Expr;
Array_Lit_3 : Character_Type_Array_Access;
Fld_7 : Character_Type_Array_Access;
Concat_Result_2 : Character_Type_Array_Access;
Scope_Result_1 : Character_Type_Array_Access;
Result_Var_1 : Character_Type_Array_Access;
Concat_Result_3 : Character_Type_Array_Access;
Scope_Result_2 : Character_Type_Array_Access;
Array_Lit_4 : Character_Type_Array_Access;
Result_Var_2 : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_169 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Array_Lit_4);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_169;
            procedure Finalizer_Scope_1865 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Concat_Result_3);
            end Finalizer_Scope_1865;
            procedure Finalizer_Scope_1866 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_1866;
            procedure Finalizer_Scope_1867 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_1867;
            procedure Finalizer_Scope_1868 is
            begin
                     Dec_Ref (Array_Lit_3);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result_2);
            end Finalizer_Scope_1868;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2082 '<Block>' Let_Result None
--# scope-start
--# expr-start 2064 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:2448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2064
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 2080 '<Concat>' Concat_Result_3 /usr/src/packages/BUILD/ada/ast.py:2451
--# expr-start 2065 '<FieldAccess .unpacked_formal_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2451
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_1 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2065
Var_Expr_1 := Fld_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Single_Formal_Array_Type then
   --# scope-start
   --# bind ufp Var_Expr_1
--# expr-start 2075 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:2452
--# expr-start 2073 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2452
--# expr-start 2066 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2452
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (32), 2 => Character_Type'Val (40))); 
--# expr-done 2066
--# expr-start 2072 '<FieldAccess .string_join(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2453
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2070 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2454
declare
   Map_Result_Vec : Character_Type_Array_Access_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Var_Expr_1;
      begin
         for Item_98 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_98
      --# expr-start 2069 '<FieldAccess .custom_id_text>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2455
--# expr-start 2068 '<FieldAccess .type_expression>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2455
--# expr-start 2067 '<FieldAccess .spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2455
Fld_2 := Item_98.Spec;
--# expr-done 2067
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 2068
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# end
--# expr-done 2069
         declare
            Item_To_Append : constant Character_Type_Array_Access :=
               Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Character_Type_Array_Access_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1867;
         end loop;
      end;
      Map_Result := Create_Character_Type_Array_Access_Array
        (Items_Count => Natural (Character_Type_Array_Access_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Character_Type_Array_Access_Vectors.Get
           (Map_Result_Vec,
            I + Character_Type_Array_Access_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Character_Type_Array_Access_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2070
--# expr-start 2071 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:2457
Array_Lit_1 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (44), 2 => Character_Type'Val (32))); 
--# expr-done 2071
--# property-call-start AdaNode.string_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_String_Join (Node => Ent.Node, Strns => Map_Result, Sep => Array_Lit_1);
--# end
--# expr-done 2072
Concat_Result := Concat (Array_Lit, Fld_5); 
--# expr-done 2073
--# expr-start 2074 '<ArrayLiteral>' Array_Lit_2 /usr/src/packages/BUILD/ada/ast.py:2459
Array_Lit_2 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (41))); 
--# expr-done 2074
Concat_Result_1 := Concat (Concat_Result, Array_Lit_2); 
--# expr-done 2075
Scope_Result := Concat_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1866;
   Result_Var := Scope_Result;
else
   Result_Var := No_Character_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2076 '<FieldAccess .returns>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2460
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2076
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Type_Expr then
   --# scope-start
   --# bind r Var_Expr_2
--# expr-start 2079 '<Concat>' Concat_Result_2 /usr/src/packages/BUILD/ada/ast.py:2461
--# expr-start 2077 '<ArrayLiteral>' Array_Lit_3 /usr/src/packages/BUILD/ada/ast.py:2461
Array_Lit_3 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (32), 2 => Character_Type'Val (114), 3 => Character_Type'Val (101), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (114), 7 => Character_Type'Val (110), 8 => Character_Type'Val (32))); 
--# expr-done 2077
--# expr-start 2078 '<FieldAccess .custom_id_text>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2461
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.custom_id_text'
Fld_7 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Custom_Id_Text (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2078
Concat_Result_2 := Concat (Array_Lit_3, Fld_7); 
--# expr-done 2079
Scope_Result_1 := Concat_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1868;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Character_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Concat_Result_3 := Concat (Result_Var, Result_Var_1); 
--# expr-done 2080
Scope_Result_2 := Concat_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1865;
   Result_Var_2 := Scope_Result_2;
else
   --# expr-start 2081 '<ArrayLiteral>' Array_Lit_4 /usr/src/packages/BUILD/ada/ast.py:2463
Array_Lit_4 := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 2081
   Result_Var_2 := Array_Lit_4;
end if;
      Inc_Ref (Result_Var_2);
Let_Result := Result_Var_2; Inc_Ref (Let_Result);
--# end
--# expr-done 2082
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_169;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_169;
               Finalizer_Scope_1865;
               Finalizer_Scope_1866;
               Finalizer_Scope_1867;
               Finalizer_Scope_1868;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Custom_Id_Text;
--# end
--# property-start BasicDecl.does_aspects_make_preelaborable /usr/src/packages/BUILD/ada/ast.py:2467
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Does_Aspects_Make_Preelaborable
  (Node : Bare_Basic_Decl
      ; From_Body : Boolean
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_body From_Body
   Property_Result : Boolean;
      Bound_Imprecise_Fallback : Boolean;
Sym : Symbol_Type;
Fld : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Not_Val : Boolean;
Sym_3 : Symbol_Type;
Fld_3 : Boolean;
Sym_4 : Symbol_Type;
Fld_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2095 '<Block>' Let_Result None
--# scope-start
--# expr-start 2094 '<bind(<DynamicVariable imprecise_fallback>, False, <Or>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2476
Bound_Imprecise_Fallback := False; 
--# expr-start 2084 '<FieldAccess .has_aspect(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:2478
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2083 '<Symbol Pure>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 2083
--# property-call-start BasicDecl.has_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2084
if Fld then
   If_Result := True;
else
   --# expr-start 2086 '<FieldAccess .has_aspect(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2479
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2085 '<Symbol Preelaborate>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 2085
--# property-call-start BasicDecl.has_aspect
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2086
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2088 '<FieldAccess .has_aspect(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2480
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2087 '<Symbol Shared_Passive>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Shared_Passive); 
--# expr-done 2087
--# property-call-start BasicDecl.has_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_2, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2088
   If_Result_1 := Fld_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 2089 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2481
Not_Val := not (From_Body); 
--# expr-done 2089
if Not_Val then
--# expr-start 2091 '<FieldAccess .has_aspect(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2483
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2090 '<Symbol Remote_Types>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remote_Types); 
--# expr-done 2090
--# property-call-start BasicDecl.has_aspect
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_3, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2091
if Fld_3 then
   If_Result_2 := True;
else
   --# expr-start 2093 '<FieldAccess .has_aspect(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2484
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2092 '<Symbol Remote_Call_Interface>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remote_Call_Interface); 
--# expr-done 2092
--# property-call-start BasicDecl.has_aspect
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Has_Aspect (Node => Ent.Node, Name => Sym_4, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2093
   If_Result_2 := Fld_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
Dyn_Var_Bind_Result := If_Result_4; 
--# expr-done 2094
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2095
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Does_Aspects_Make_Preelaborable;
--# end
--# property-start BasicDecl.wrap_public_reference /usr/src/packages/BUILD/ada/ast.py:2489
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Wrap_Public_Reference
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Generic_Subp_Internal;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Generic_Subp_Internal;
Fld : Internal_Entity_Generic_Subp_Instantiation;
Scope_Result : Internal_Entity_Generic_Subp_Instantiation;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2098 '<Block>' Let_Result None
--# scope-start
--# expr-start 2096 '<Cast to GenericSubpInternal>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2509
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Generic_Subp_Internal_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Subp_Internal
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Generic_Subp_Internal;
   end if;
--# expr-done 2096
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Generic_Subp_Internal then
   --# scope-start
   --# bind g Var_Expr
--# expr-start 2097 '<FieldAccess .get_instantiation>' Fld /usr/src/packages/BUILD/ada/ast.py:2510
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericSubpInternal.get_instantiation
Fld := Libadalang.Implementation.Generic_Subp_Internal_P_Get_Instantiation (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2097
Scope_Result := Fld;
   --# end
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result_1;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2098
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Wrap_Public_Reference;
--# end
--# property-start BasicDecl.child_decl_initial_env_name /usr/src/packages/BUILD/ada/ast.py:1478
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Basic_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_108;
            procedure Finalizer_Scope_1749;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Dotted_Name;
Var_Expr : Internal_Entity_Dotted_Name;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Character_Type_Array_Access;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym_2 : Symbol_Type;
Fld_6 : Internal_Entity_Name;
Fld_7 : Character_Type_Array_Access;
Sym_3 : Symbol_Type;
If_Result_1 : Symbol_Type;
Scope_Result : Symbol_Type;
Sym_4 : Symbol_Type;
Result_Var : Symbol_Type;
If_Result_2 : Symbol_Type;
If_Result_3 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_108 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_108;
            procedure Finalizer_Scope_1749 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_1749;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2119 '<Block>' Let_Result None
--# scope-start
--# expr-start 2100 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:1486
--# expr-start 2099 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:1486
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 2099
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 2100
Block_Var := Fld; 
--# bind defining_name Block_Var
--# expr-start 2102 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1487
--# expr-start 2101 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1487
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Block_Var.Node.Defining_Name_F_Name, Info => Block_Var.Info);
--# expr-done 2101
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 2102
Block_Var_1 := Cast_Result; 
--# bind child_name Block_Var_1
--# expr-start 2118 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:1488
--# expr-start 2106 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1491
--# expr-start 2104 '<to_symbol(<FieldAccess .text>)>' Sym /usr/src/packages/BUILD/ada/ast.py:1491
--# expr-start 2103 '<FieldAccess .text>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1491
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Text (Node => Block_Var.Node);
--# expr-done 2103
Sym := String_To_Symbol (Self.Unit.Context, Fld_2); 
--# expr-done 2104
--# expr-start 2105 '<Symbol standard>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Standard_264); 
--# expr-done 2105
Is_Equal := Sym = Sym_1; 
--# expr-done 2106
if Is_Equal then
   If_Result_3 := null;
else
--# expr-start 2107 '<FieldAccess .is_library_item>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:1494
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 2107
if Fld_3 then
Var_Expr := Block_Var_1;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 2116 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:1498
if Private_Part then
   --# expr-start 2112 '<to_symbol(<Concat>)>' Sym_2 /usr/src/packages/BUILD/ada/ast.py:1500
--# expr-start 2111 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:1500
--# expr-start 2109 '<FieldAccess .text>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1500
--# expr-start 2108 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1500
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 2108
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Text (Node => Fld_4.Node);
--# expr-done 2109
--# expr-start 2110 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:1500
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46), 2 => Character_Type'Val (95), 3 => Character_Type'Val (95), 4 => Character_Type'Val (112), 5 => Character_Type'Val (114), 6 => Character_Type'Val (105), 7 => Character_Type'Val (118), 8 => Character_Type'Val (97), 9 => Character_Type'Val (116), 10 => Character_Type'Val (101), 11 => Character_Type'Val (112), 12 => Character_Type'Val (97), 13 => Character_Type'Val (114), 14 => Character_Type'Val (116))); 
--# expr-done 2110
Concat_Result := Concat (Fld_5, Array_Lit); 
--# expr-done 2111
Sym_2 := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 2112
   If_Result_1 := Sym_2;
else
   --# expr-start 2115 '<to_symbol(<FieldAccess .text>)>' Sym_3 /usr/src/packages/BUILD/ada/ast.py:1501
--# expr-start 2114 '<FieldAccess .text>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:1501
--# expr-start 2113 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1501
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 2113
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Text (Node => Fld_6.Node);
--# expr-done 2114
Sym_3 := String_To_Symbol (Self.Unit.Context, Fld_7); 
--# expr-done 2115
   If_Result_1 := Sym_3;
end if;
--# expr-done 2116
Scope_Result := If_Result_1;
   --# end
      Finalizer_Scope_1749;
   Result_Var := Scope_Result;
else
   --# expr-start 2117 '<Symbol standard>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Standard_264); 
--# expr-done 2117
   Result_Var := Sym_4;
end if;
   If_Result_2 := Result_Var;
else
   If_Result_2 := null;
end if;
   If_Result_3 := If_Result_2;
end if;
--# expr-done 2118
Let_Result := If_Result_3; 
--# end
--# expr-done 2119
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_108;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_108;
               Finalizer_Scope_1749;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Child_Decl_Initial_Env_Name;
--# end
--# property-start BasicDecl.previous_part_for_decl /usr/src/packages/BUILD/ada/ast.py:1551
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Body_Node;
Match_2 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Body_Node;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2124 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 2123 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:1560
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind btd Match_0
--# expr-start 2121 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:1562
--# expr-start 2120 '<FieldAccess .previous_part(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1562
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Match_0.Node, Go_To_Incomplete => True, E_Info => Match_0.Info);
--# end
--# expr-done 2120
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
--# expr-done 2121
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind bd Match_1
--# expr-start 2122 '<FieldAccess .previous_part_internal>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1563
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 2122
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2123
Let_Result_3 := Match_Result; 
--# end
--# expr-done 2124
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BasicDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:1620
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Static_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2125 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2125
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Static_Decl;
--# end
--# property-start BasicDecl.is_imported /usr/src/packages/BUILD/ada/ast.py:1661
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Imported
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Pragma_Node;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_2 : Internal_Entity_Pragma_Node;
Is_Null_2 : Boolean;
Not_Val_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2138 '<Block>' Let_Result None
--# scope-start
--# expr-start 2129 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:1666
--# expr-start 2128 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:1666
--# expr-start 2127 '<FieldAccess .get_aspect_assoc(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:1666
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2126 '<Symbol Import>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 2126
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2127
Is_Null := Fld.Node = null; 
--# expr-done 2128
Not_Val := not (Is_Null); 
--# expr-done 2129
if Not_Val then
   If_Result := True;
else
   --# expr-start 2133 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:1667
--# expr-start 2132 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:1667
--# expr-start 2131 '<FieldAccess .get_pragma(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1667
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2130 '<Symbol Import>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 2130
--# property-call-start BasicDecl.get_pragma
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2131
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 2132
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2133
   If_Result := Not_Val_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2137 '<Not>' Not_Val_2 /usr/src/packages/BUILD/ada/ast.py:1668
--# expr-start 2136 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:1668
--# expr-start 2135 '<FieldAccess .get_pragma(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:1668
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2134 '<Symbol Interface>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 2134
--# property-call-start BasicDecl.get_pragma
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Pragma (Node => Ent.Node, Name => Sym_2, E_Info => Ent.Info);
--# end
--# expr-done 2135
Is_Null_2 := Fld_2.Node = null; 
--# expr-done 2136
Not_Val_2 := not (Is_Null_2); 
--# expr-done 2137
   If_Result_1 := Not_Val_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2138
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Imported;
--# end
--# property-start BasicDecl.is_library_item /usr/src/packages/BUILD/ada/ast.py:1672
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_Library_Item
  (Node : Bare_Basic_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2141 '<Block>' Let_Result None
--# scope-start
--# expr-start 2140 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:1676
--# expr-start 2139 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1676
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2139
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 2140
Let_Result := Is_A; 
--# end
--# expr-done 2141
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_Library_Item;
--# end
--# property-start BasicDecl.declarative_region /usr/src/packages/BUILD/ada/ast.py:1688
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Declarative_Region
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Declarative_Part;
      Let_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2142 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Declarative_Part; 
--# end
--# expr-done 2142
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Declarative_Region;
--# end
--# property-start BasicDecl.get_aspect_assoc /usr/src/packages/BUILD/ada/ast.py:1699
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Basic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_123;
            procedure Finalizer_Scope_1767;
      Var_Expr : Internal_Entity_Aspect_Spec;
Fld : Internal_Entity_Aspect_Spec;
Fld_1 : Internal_Entity_Aspect_Assoc_List;
Scope_Result : Internal_Entity_Aspect_Assoc_List;
Result_Var : Internal_Entity_Aspect_Assoc_List;
Fld_2 : Bare_Aspect_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Aspect_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Aspect_Assoc;
Item_73 : Internal_Entity_Aspect_Assoc;
Bare_Item_73 : Bare_Aspect_Assoc;
Fld_4 : Internal_Entity_Name;
Fld_5 : Character_Type_Array_Access;
Fld_6 : Character_Type_Array_Access;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Aspect_Assoc_Array_Access;
Get_Result : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
            procedure Finalizer_Scope_123 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_123;
            procedure Finalizer_Scope_1767 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_1767;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2151 '<Block>' Let_Result None
--# scope-start
--# expr-start 2150 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:1703
--# expr-start 2149 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:1703
--# expr-start 2143 '<FieldAccess .aspects>' Fld /usr/src/packages/BUILD/ada/ast.py:1703
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Aspect_Spec (Node => Implementation.Basic_Decl_F_Aspects (Ent.Node), Info => Ent.Info);
--# expr-done 2143
Var_Expr := Fld;
if Var_Expr /= No_Entity_Aspect_Spec then
   --# scope-start
--# expr-start 2144 '<FieldAccess .aspect_assocs>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1703
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Aspect_Assoc_List (Node => Var_Expr.Node.Aspect_Spec_F_Aspect_Assocs, Info => Var_Expr.Info);
--# expr-done 2144
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Aspect_Assoc_List;
end if;
Fld_2 := Result_Var.Node;
declare
   Map_Result_Vec : Internal_Entity_Aspect_Assoc_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Aspect_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Aspect_Assoc_List := Fld_2;
      begin
         for Untyped_Item_73 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_73 := Untyped_Item_73; 
Is_Null := Bare_Item_73 = null; 
if Is_Null then
   If_Result := No_Entity_Aspect_Assoc;
else
Fld_3 := Result_Var.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_73); 
   If_Result := As_Entity;
end if;
                  Item_73 := If_Result; 
   --# scope-start
               --# bind asp Item_73
               --# expr-start 2148 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:1704
--# expr-start 2146 '<FieldAccess .aspect_name(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:1704
      if Item_73.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2145 '<FieldAccess .id>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:1704
      if Item_73.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Item_73.Node.Aspect_Assoc_F_Id, Info => Item_73.Info);
--# expr-done 2145
--# property-call-start AspectAssoc.aspect_name
Fld_5 := Libadalang.Implementation.Aspect_Assoc_P_Aspect_Name (Node => Item_73.Node, N => Fld_4);
--# end
--# expr-done 2146
--# expr-start 2147 '<FieldAccess .image>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:1704
      if Name = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Image (Self => Name);
--# expr-done 2147
Is_Equal := Equivalent (Fld_5, Fld_6); 
--# expr-done 2148
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Aspect_Assoc :=
               Item_73;
         begin
            Internal_Entity_Aspect_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1767;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Aspect_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Aspect_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Aspect_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Aspect_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Aspect_Assoc_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2149
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2150
Let_Result := Get_Result; 
--# end
--# expr-done 2151
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_123;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_123;
               Finalizer_Scope_1767;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start BasicDecl.is_in_private_part /usr/src/packages/BUILD/ada/ast.py:1928
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Is_In_Private_Part
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2154 '<IsA PrivatePart>' Is_A /usr/src/packages/BUILD/ada/ast.py:1928
--# expr-start 2153 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:1928
--# expr-start 2152 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:1928
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2152
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2153
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Private_Part_Range; 
--# expr-done 2154
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Is_In_Private_Part;
--# end
--# property-start BasicDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:2126
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Defining_Env
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := AST_Envs.Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Defining_Env;
--# end
--# property-start BasicDecl.array_ndims /usr/src/packages/BUILD/ada/ast.py:2148
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Array_Ndims
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
Let_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2157 '<Block>' Let_Result None
--# scope-start
--# expr-start 2156 '<FieldAccess .array_ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2149
--# expr-start 2155 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:2149
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2155
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2156
Let_Result := Fld_1; 
--# end
--# expr-done 2157
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Array_Ndims;
--# end
--# property-start BasicDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:2155
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Expr_Type
  (Node : Bare_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2165 '<Block>' Let_Result None
--# scope-start
--# expr-start 2158 '<FieldAccess .type_expression>' Fld /usr/src/packages/BUILD/ada/ast.py:2174
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2158
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind te Var_Expr
--# expr-start 2159 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2174
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2159
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var := Result_Var; 
--# bind ret Block_Var
--# expr-start 2164 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:2179
--# expr-start 2162 '<FieldAccess .access_entity>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2180
--# expr-start 2161 '<FieldAccess .md>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2180
--# expr-start 2160 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2180
Fld_3 := Ent.Info;
--# expr-done 2160
Fld_4 := Fld_3.MD;
--# expr-done 2161
Fld_5 := Fld_4.Access_Entity;
--# expr-done 2162
if Fld_5 then
   --# expr-start 2163 '<FieldAccess .anonymous_access_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2181
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2163
   If_Result := Fld_2;
else
   If_Result := Block_Var;
end if;
--# expr-done 2164
Block_Var_1 := If_Result; 
--# bind ret_2 Block_Var_1
Let_Result := Block_Var_1; 
--# end
--# expr-done 2165
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Expr_Type;
--# end
--# property-start BasicDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:2187
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Type_Expression
  (Node : Bare_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Node_For_Entity : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2167 '<as_entity(<No BareTypeExpr>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2188
--# expr-start 2166 '<No BareTypeExpr>' Node_For_Entity /usr/src/packages/BUILD/ada/ast.py:2188
Node_For_Entity := No_Bare_Ada_Node;
--# expr-done 2166
Is_Null := Node_For_Entity = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Node_For_Entity); 
   If_Result := As_Entity;
end if;
--# expr-done 2167
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Type_Expression;
--# end
--# property-start BasicDecl.constrain_prefix /usr/src/packages/BUILD/ada/ast.py:2289
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_157;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_157 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_157;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2168 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:2313") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 2168
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_157;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_157;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:2351
pragma Warnings (Off, "is not referenced");
 function Basic_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Decl := Bare_Basic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Decl :=
     Internal_Entity_Basic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2170 '<Block>' Let_Result None
--# scope-start
--# expr-start 2169 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:2360
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2169
Let_Result := Fld; 
--# end
--# expr-done 2170
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Decl_P_Next_Part_For_Decl;
--# end
   --
   --  Primitives for Bare_Abstract_State_Decl
   --
      procedure Abstract_State_Decl_Pre_Env_Actions
        (Self            : Bare_Abstract_State_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_268 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:15814");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Abstract_State_Decl
        (Self : Bare_Abstract_State_Decl
         ; Abstract_State_Decl_F_Name : Bare_Defining_Name
         ; Abstract_State_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Abstract_State_Decl_F_Name := Abstract_State_Decl_F_Name;
            Self.Abstract_State_Decl_F_Aspects := Abstract_State_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_State_Decl;
   function Abstract_State_Decl_F_Name
     (Node : Bare_Abstract_State_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Abstract_State_Decl_F_Name;
   end;
--# property-start AbstractStateDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:15810
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Defining_Names
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Abstract_State_Decl := Bare_Abstract_State_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Abstract_State_Decl :=
     Internal_Entity_Abstract_State_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1166;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1166 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1166;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2171 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15810
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Abstract_State_Decl_F_Name, Info => Ent.Info);
--# expr-done 2171
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1166;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1166;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_State_Decl_P_Defining_Names;
--# end
--# property-start AbstractStateDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:15811
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_P_Type_Expression
  (Node : Bare_Abstract_State_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Abstract_State_Decl := Bare_Abstract_State_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Type_Expr;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_State_Decl_P_Type_Expression;
--# end
--# property-start AbstractStateDecl._env_mappings_268 /usr/src/packages/BUILD/ada/ast.py:15814
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_268
  (Node : Bare_Abstract_State_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Abstract_State_Decl := Bare_Abstract_State_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1438;
      Fld : Bare_Defining_Name;
Fld_1 : Symbol_Type;
Cast_Expr : Bare_Abstract_State_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1438 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1438;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2174 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15814
--# expr-start 2173 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15814
--# expr-start 2172 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15814
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Abstract_State_Decl_F_Name;
--# expr-done 2172
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 2173
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2174
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1438;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1438;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_268;
--# end
   --
   --  Primitives for Bare_Anonymous_Expr_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Expr_Decl
        (Self : Bare_Anonymous_Expr_Decl
         ; Anonymous_Expr_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Anonymous_Expr_Decl_F_Expr := Anonymous_Expr_Decl_F_Expr;
      end Initialize_Fields_For_Anonymous_Expr_Decl;
   function Anonymous_Expr_Decl_F_Expr
     (Node : Bare_Anonymous_Expr_Decl) return Bare_Expr
   is
   begin
         return Node.Anonymous_Expr_Decl_F_Expr;
   end;
--# property-start AnonymousExprDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7508
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Names
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Anonymous_Expr_Decl := Bare_Anonymous_Expr_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Expr_Decl_P_Defining_Names;
--# end
--# property-start AnonymousExprDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:7509
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Defining_Env
  (Node : Bare_Anonymous_Expr_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Anonymous_Expr_Decl := Bare_Anonymous_Expr_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Expr_Decl :=
     Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_674;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_674 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_674;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2176 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7509
--# expr-start 2175 '<FieldAccess .type_expression>' Fld /usr/src/packages/BUILD/ada/ast.py:7509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2175
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2176
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_674;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_674;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Expr_Decl_P_Defining_Env;
--# end
--# property-start AnonymousExprDecl.type_expression_internal /usr/src/packages/BUILD/ada/ast.py:7512
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Type_Expression_Internal
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Anonymous_Expr_Decl := Bare_Anonymous_Expr_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Expr_Decl :=
     Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
            procedure Finalizer_Scope_675;
      Block_Var : Internal_Entity_Basic_Assoc;
Block_Var_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Assoc;
Bound_Imprecise_Fallback : Boolean;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Type_Expr;
Let_Result : Internal_Entity_Type_Expr;
            procedure Finalizer_Scope_675 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_675;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Anonymous_Expr_Decl_Anonymous_Expr_Decl_P_Type_Expression_Internal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Anonymous_Expr_Decl,
                                As_Bare_Anonymous_Expr_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Type_Expr;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2185 '<Block>' Let_Result None
--# scope-start
--# expr-start 2179 '<Cast to BasicAssoc>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7518
--# expr-start 2178 '<parent(<FieldAccess .expr>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7518
--# expr-start 2177 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7518
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 2177
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 2178
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Assoc
   then
      Cast_Result := Create_Internal_Entity_Basic_Assoc
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Assoc;
   end if;
--# expr-done 2179
Block_Var := Cast_Result; 
--# bind assoc Block_Var
--# expr-start 2183 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .basic_decl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:7519
Bound_Imprecise_Fallback := False; 
--# expr-start 2182 '<FieldAccess .basic_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7521
--# expr-start 2181 '<at(<FieldAccess .get_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:7521
--# expr-start 2180 '<FieldAccess .get_params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7521
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicAssoc.get_params
Fld_2 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Block_Var.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 2180
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 2181
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 2182
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 2183
Block_Var_1 := Dyn_Var_Bind_Result; 
--# bind formal Block_Var_1
--# expr-start 2184 '<FieldAccess .type_expression>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7523
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2184
Let_Result := Fld_4; 
--# end
--# expr-done 2185
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_675;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Type_Expr,
                     As_Internal_Entity_Type_Expr => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_675;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Expr_Decl_P_Type_Expression_Internal;
--# end
--# property-start AnonymousExprDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:7526
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Type_Expression
  (Node : Bare_Anonymous_Expr_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Anonymous_Expr_Decl := Bare_Anonymous_Expr_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Expr_Decl :=
     Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
Let_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2187 '<Block>' Let_Result None
--# scope-start
--# expr-start 2186 '<FieldAccess .type_expression_internal>' Fld /usr/src/packages/BUILD/ada/ast.py:7527
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AnonymousExprDecl.type_expression_internal
Fld := Libadalang.Implementation.Anonymous_Expr_Decl_P_Type_Expression_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2186
Let_Result := Fld; 
--# end
--# expr-done 2187
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Expr_Decl_P_Type_Expression;
--# end
--# property-start AnonymousExprDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:7530
pragma Warnings (Off, "is not referenced");
 function Anonymous_Expr_Decl_P_Is_Static_Decl
  (Node : Bare_Anonymous_Expr_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Expr_Decl := Bare_Anonymous_Expr_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Expr_Decl :=
     Internal_Entity_Anonymous_Expr_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2190 '<Block>' Let_Result None
--# scope-start
--# expr-start 2189 '<FieldAccess .is_static_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7531
--# expr-start 2188 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7531
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Anonymous_Expr_Decl_F_Expr, Info => Ent.Info);
--# expr-done 2188
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2189
Let_Result := Fld_1; 
--# end
--# expr-done 2190
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Expr_Decl_P_Is_Static_Decl;
--# end
   --
   --  Primitives for Bare_Base_Formal_Param_Decl
   --
--# property-start '[dispatcher]BaseFormalParamDecl.is_mandatory' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Formal_Param_Decl (Self.Kind) is
               when Ada_Component_Decl .. Ada_Generic_Formal_Type_Decl =>
                  --# property-call-start BaseFormalParamDecl.is_mandatory
                  Property_Result := Base_Formal_Param_Decl_P_Is_Mandatory
                    (Self
                    );
                  --# end
               when Ada_Param_Spec =>
                  --# property-call-start ParamSpec.is_mandatory
                  Property_Result := Param_Spec_P_Is_Mandatory
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
--# property-start BaseFormalParamDecl.formal_type /usr/src/packages/BUILD/ada/ast.py:3015
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Formal_Type
  (Node : Bare_Base_Formal_Param_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Type_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2193 '<Block>' Let_Result None
--# scope-start
--# expr-start 2191 '<FieldAccess .type_expression>' Fld /usr/src/packages/BUILD/ada/ast.py:3019
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2191
Var_Expr := Fld;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 2192 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3019
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2192
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2193
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Formal_Type;
--# end
--# property-start BaseFormalParamDecl.parent_decl /usr/src/packages/BUILD/ada/ast.py:3022
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Parent_Decl
  (Node : Bare_Base_Formal_Param_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2196 '<Block>' Let_Result None
--# scope-start
--# expr-start 2195 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3023
--# expr-start 2194 '<FieldAccess .semantic_parent>' Fld /usr/src/packages/BUILD/ada/ast.py:3023
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2194
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 2195
Let_Result := Cast_Result; 
--# end
--# expr-done 2196
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Parent_Decl;
--# end
--# property-start BaseFormalParamDecl.get_param /usr/src/packages/BUILD/ada/ast.py:3026
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Get_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Part : Internal_Entity_Basic_Decl
      ; Param : Internal_Entity_Defining_Name
         := No_Entity_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind part Part
   --# bind param Param
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1872;
            procedure Finalizer_Scope_1873;
      Block_Var : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Base_Formal_Param_Holder;
Scope_Result : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Single_Formal_Array_Access;
Scope_Result_1 : Internal_Single_Formal_Array_Access;
Result_Var_1 : Internal_Single_Formal_Array_Access;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_6 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_1872 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1872;
            procedure Finalizer_Scope_1873 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1873;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2206 '<Block>' Let_Result None
--# scope-start
Var_Expr := Param;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 2197 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:3029
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2197
   Result_Var := Fld;
end if;
Block_Var := Result_Var; 
--# bind p Block_Var
Var_Expr_1 := Part;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 2205 '<FieldAccess .name>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3032
--# expr-start 2204 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:3032
--# expr-start 2203 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3032
--# expr-start 2198 '<FieldAccess .formal_param_holder_or_null>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3032
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2198
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
--# expr-start 2199 '<FieldAccess .unpacked_formal_params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3032
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2199
Scope_Result_1 := Fld_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1873;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Single_Formal_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Result_Var_1;
      begin
         for Item_99 of
               Collection.Items
         loop
   --# scope-start
               --# bind sf Item_99
               --# expr-start 2202 '<FieldAccess .name_is(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3033
--# expr-start 2200 '<FieldAccess .name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3033
Fld_3 := Item_99.Name;
--# expr-done 2200
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2201 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3033
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Block_Var.Node);
--# end
--# expr-done 2201
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Fld_4);
--# end
--# expr-done 2202
               if Fld_5 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_99;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2203
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2204
Fld_6 := Get_Result.Name;
--# expr-done 2205
Scope_Result_2 := Fld_6;
   --# end
      Finalizer_Scope_1872;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 2206
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1872;
               Finalizer_Scope_1873;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Get_Param;
--# end
--# property-start BaseFormalParamDecl.decl_param /usr/src/packages/BUILD/ada/ast.py:3038
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Decl_Param
  (Node : Bare_Base_Formal_Param_Decl
      ; Param : Internal_Entity_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind param Param
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2211 '<Block>' Let_Result None
--# scope-start
--# expr-start 2210 '<FieldAccess .get_param(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3043
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2208 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3044
--# expr-start 2207 '<FieldAccess .parent_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:3044
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2207
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 2208
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 2209 '<FieldAccess .decl_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3044
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2209
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, Param => Param, E_Info => Ent.Info);
--# end
--# expr-done 2210
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Param;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2211
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Decl_Param;
--# end
--# property-start BaseFormalParamDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:3048
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2216 '<Block>' Let_Result None
--# scope-start
--# expr-start 2214 '<FieldAccess .get_param(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3049
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2212 '<FieldAccess .parent_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:3050
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2212
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2213 '<FieldAccess .next_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3050
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2213
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 2214
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2215 '<FieldAccess .basic_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3049
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2215
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2216
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.previous_part_for_decl /usr/src/packages/BUILD/ada/ast.py:3053
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Previous_Part_For_Decl
  (Node : Bare_Base_Formal_Param_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Formal_Param_Decl :=
     Internal_Entity_Base_Formal_Param_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Defining_Name;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2221 '<Block>' Let_Result None
--# scope-start
--# expr-start 2219 '<FieldAccess .get_param(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3054
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2217 '<FieldAccess .parent_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:3055
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.parent_decl
Fld := Libadalang.Implementation.Base_Formal_Param_Decl_P_Parent_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2217
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2218 '<FieldAccess .previous_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3055
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Var_Expr_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2218
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
--# property-call-start BaseFormalParamDecl.get_param
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Get_Param (Node => Ent.Node, Part => Result_Var, E_Info => Ent.Info);
--# end
--# expr-done 2219
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 2220 '<FieldAccess .basic_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3054
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2220
Scope_Result_1 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2221
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Previous_Part_For_Decl;
--# end
--# property-start BaseFormalParamDecl.is_mandatory /usr/src/packages/BUILD/ada/ast.py:3011
pragma Warnings (Off, "is not referenced");
 function Base_Formal_Param_Decl_P_Is_Mandatory
  (Node : Bare_Base_Formal_Param_Decl
  )
   return Boolean
is
   Self : Bare_Base_Formal_Param_Decl := Bare_Base_Formal_Param_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Formal_Param_Decl_P_Is_Mandatory;
--# end
   --
   --  Primitives for Bare_Component_Decl
   --
      procedure Component_Decl_Pre_Env_Actions
        (Self            : Bare_Component_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_1 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:3658");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Component_Decl
        (Self : Bare_Component_Decl
         ; Component_Decl_F_Ids : Bare_Defining_Name_List
         ; Component_Decl_F_Component_Def : Bare_Component_Def
         ; Component_Decl_F_Default_Expr : Bare_Expr
         ; Component_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Component_Decl_F_Ids := Component_Decl_F_Ids;
            Self.Component_Decl_F_Component_Def := Component_Decl_F_Component_Def;
            Self.Component_Decl_F_Default_Expr := Component_Decl_F_Default_Expr;
            Self.Component_Decl_F_Aspects := Component_Decl_F_Aspects;
      end Initialize_Fields_For_Component_Decl;
   function Component_Decl_F_Ids
     (Node : Bare_Component_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Component_Decl_F_Ids;
   end;
   function Component_Decl_F_Component_Def
     (Node : Bare_Component_Decl) return Bare_Component_Def
   is
   begin
         return Node.Component_Decl_F_Component_Def;
   end;
   function Component_Decl_F_Default_Expr
     (Node : Bare_Component_Decl) return Bare_Expr
   is
   begin
         return Node.Component_Decl_F_Default_Expr;
   end;
--# property-start ComponentDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:3660
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Env
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_249;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_249 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_249;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2224 '<FieldAccess .defining_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3661
--# expr-start 2223 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3661
--# expr-start 2222 '<FieldAccess .component_def>' Fld /usr/src/packages/BUILD/ada/ast.py:3661
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2222
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2223
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_2 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2224
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_249;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_249;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Defining_Env;
--# end
--# property-start ComponentDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:3665
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Defining_Names
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_250;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_100 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_250 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_250;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2227 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3665
--# expr-start 2225 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:3665
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 2225
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_100 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_100 := Untyped_Item_100; 
   --# scope-start
               --# bind i Item_100
      --# expr-start 2226 '<as_entity(<Var i>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3665
Is_Null := Item_100 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_100); 
   If_Result := As_Entity;
end if;
--# expr-done 2226
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2227
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_250;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_250;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Defining_Names;
--# end
--# property-start ComponentDecl.array_ndims /usr/src/packages/BUILD/ada/ast.py:3667
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Array_Ndims
  (Node : Bare_Component_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2230 '<FieldAccess .array_ndims>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3667
--# expr-start 2229 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3667
--# expr-start 2228 '<FieldAccess .component_def>' Fld /usr/src/packages/BUILD/ada/ast.py:3667
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2228
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 2229
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.array_ndims
Fld_2 := Libadalang.Implementation.Type_Expr_P_Array_Ndims (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 2230
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Array_Ndims;
--# end
--# property-start ComponentDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:3669
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Type_Expression
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Bare_Component_Def;
Fld_1 : Bare_Type_Expr;
As_Entity : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2233 '<as_entity(<FieldAccess .type_expr>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3669
--# expr-start 2232 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3669
--# expr-start 2231 '<FieldAccess .component_def>' Fld /usr/src/packages/BUILD/ada/ast.py:3669
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Component_Def;
--# expr-done 2231
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Component_Def_F_Type_Expr;
--# expr-done 2232
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Type_Expr;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2233
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Type_Expression;
--# end
--# property-start ComponentDecl.constrain_prefix /usr/src/packages/BUILD/ada/ast.py:3672
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Constrain_Prefix
  (Node : Bare_Component_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_253;
      Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_253 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_253;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2237 '<Block>' Let_Result None
--# scope-start
--# expr-start 2236 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:3674
--# expr-start 2234 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:3674
      if Prefix = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 2234
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
--# expr-start 2235 '<FieldAccess .container_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3675
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentDecl.container_type
Fld_1 := Libadalang.Implementation.Component_Decl_P_Container_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2235
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Bind_Result := Bind_Default_560.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_560'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3674") else null)); 
--# expr-done 2236
Let_Result := Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2237
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_253;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_253;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Constrain_Prefix;
--# end
--# property-start ComponentDecl.container_type /usr/src/packages/BUILD/ada/ast.py:3679
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Container_Type
  (Node : Bare_Component_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_254;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_254 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_254;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2244 '<Block>' Let_Result None
--# scope-start
--# expr-start 2243 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3683
--# expr-start 2242 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3683
--# expr-start 2241 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:3683
--# expr-start 2240 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3683
--# expr-start 2238 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:3683
Node_Parents := Parents (Self, True); 
--# expr-done 2238
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_101 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_101
               --# expr-start 2239 '<IsA BaseTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:3684
Is_A := Item_101 /= null 
and then Item_101.Kind in Ada_Base_Type_Decl; 
--# expr-done 2239
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_101;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2240
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2241
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2242
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2243
Let_Result := If_Result; 
--# end
--# expr-done 2244
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_254;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_254;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Container_Type;
--# end
--# property-start ComponentDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:3688
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Equation
  (Node : Bare_Component_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Decl :=
     Internal_Entity_Component_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_255;
            procedure Finalizer_Scope_1883;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Component_Def;
Fld_2 : Internal_Entity_Type_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_255 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_255;
            procedure Finalizer_Scope_1883 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1883;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2255 '<Block>' Let_Result None
--# scope-start
--# expr-start 2245 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3689
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2245
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 2254 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:3691
--# expr-start 2248 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3691
--# expr-start 2247 '<FieldAccess .type_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3691
--# expr-start 2246 '<FieldAccess .component_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3691
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Component_Decl_F_Component_Def, Info => Ent.Info);
--# expr-done 2246
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Type_Expr (Node => Fld_1.Node.Component_Def_F_Type_Expr, Info => Fld_1.Info);
--# expr-done 2247
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 2248
--# expr-start 2249 '<FieldAccess .default_expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3692
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 2249
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2253 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:3694
--# expr-start 2250 '<FieldAccess .sub_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3694
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2250
--# expr-start 2252 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:3695
--# expr-start 2251 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3695
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 2251
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_566.Create (Fld_6, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3695") else null)); 
--# expr-done 2252
And_Pred := Logic_And (Fld_5, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3694") else null)); 
--# expr-done 2253
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1883;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3697") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3691") else null)); 
--# expr-done 2254
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2255
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_255;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_255;
               Finalizer_Scope_1883;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Xref_Equation;
--# end
--# property-start ComponentDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:3701
pragma Warnings (Off, "is not referenced");
 function Component_Decl_P_Xref_Entry_Point
  (Node : Bare_Component_Decl
  )
   return Boolean
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Decl_P_Xref_Entry_Point;
--# end
--# property-start ComponentDecl._env_mappings_1 /usr/src/packages/BUILD/ada/ast.py:3658
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_1
  (Node : Bare_Component_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Component_Decl := Bare_Component_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1170;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Component_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1170 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1170;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2257 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3658
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2256 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:3658
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Component_Decl_F_Ids;
--# expr-done 2256
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2257
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1170;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1170;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_1;
--# end
   --
   --  Primitives for Bare_Discriminant_Spec
   --
      procedure Discriminant_Spec_Pre_Env_Actions
        (Self            : Bare_Discriminant_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_0 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:3067");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Discriminant_Spec
        (Self : Bare_Discriminant_Spec
         ; Discriminant_Spec_F_Ids : Bare_Defining_Name_List
         ; Discriminant_Spec_F_Type_Expr : Bare_Type_Expr
         ; Discriminant_Spec_F_Default_Expr : Bare_Expr
        ) is
      begin
            Self.Discriminant_Spec_F_Ids := Discriminant_Spec_F_Ids;
            Self.Discriminant_Spec_F_Type_Expr := Discriminant_Spec_F_Type_Expr;
            Self.Discriminant_Spec_F_Default_Expr := Discriminant_Spec_F_Default_Expr;
      end Initialize_Fields_For_Discriminant_Spec;
   function Discriminant_Spec_F_Ids
     (Node : Bare_Discriminant_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Discriminant_Spec_F_Ids;
   end;
   function Discriminant_Spec_F_Type_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Type_Expr;
   end;
   function Discriminant_Spec_F_Default_Expr
     (Node : Bare_Discriminant_Spec) return Bare_Expr
   is
   begin
         return Node.Discriminant_Spec_F_Default_Expr;
   end;
--# property-start DiscriminantSpec.defining_names /usr/src/packages/BUILD/ada/ast.py:3069
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Names
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_203;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_102 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_203 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_203;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2260 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3069
--# expr-start 2258 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:3069
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 2258
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_102 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_102 := Untyped_Item_102; 
   --# scope-start
               --# bind id Item_102
      --# expr-start 2259 '<as_entity(<Var id>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:3069
Is_Null := Item_102 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_102); 
   If_Result := As_Entity;
end if;
--# expr-done 2259
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2260
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_203;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_203;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Defining_Names;
--# end
--# property-start DiscriminantSpec.defining_env /usr/src/packages/BUILD/ada/ast.py:3070
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Defining_Env
  (Node : Bare_Discriminant_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_204;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_204 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_204;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2262 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3070
--# expr-start 2261 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:3070
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2261
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2262
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_204;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_204;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Defining_Env;
--# end
--# property-start DiscriminantSpec.type_expression /usr/src/packages/BUILD/ada/ast.py:3072
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Type_Expression
  (Node : Bare_Discriminant_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2263 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:3072
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2263
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Type_Expression;
--# end
--# property-start DiscriminantSpec.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:3074
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Entry_Point
  (Node : Bare_Discriminant_Spec
  )
   return Boolean
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Xref_Entry_Point;
--# end
--# property-start DiscriminantSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:3077
pragma Warnings (Off, "is not referenced");
 function Discriminant_Spec_P_Xref_Equation
  (Node : Bare_Discriminant_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Spec :=
     Internal_Entity_Discriminant_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_207;
            procedure Finalizer_Scope_1885;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_207 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_207;
            procedure Finalizer_Scope_1885 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1885;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2274 '<Block>' Let_Result None
--# scope-start
--# expr-start 2273 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:3078
--# expr-start 2265 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3079
--# expr-start 2264 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:3079
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Discriminant_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2264
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2265
--# expr-start 2266 '<FieldAccess .default_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3080
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Discriminant_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 2266
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2272 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:3082
--# expr-start 2267 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3082
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2267
--# expr-start 2271 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:3083
--# expr-start 2269 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3083
--# expr-start 2268 '<FieldAccess .node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3083
Fld_4 := Var_Expr.Node;
--# expr-done 2268
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 2269
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 2270 '<FieldAccess .expr_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3084
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2270
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Bind_Default_566.Create (Fld_5, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3083") else null)); 
--# expr-done 2271
And_Pred := Logic_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3082") else null)); 
--# expr-done 2272
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1885;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3086") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3078") else null)); 
--# expr-done 2273
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2274
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_207;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_207;
               Finalizer_Scope_1885;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Spec_P_Xref_Equation;
--# end
--# property-start DiscriminantSpec._env_mappings_0 /usr/src/packages/BUILD/ada/ast.py:3067
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_0
  (Node : Bare_Discriminant_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Discriminant_Spec := Bare_Discriminant_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1169;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Discriminant_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1169 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1169;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2276 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3067
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2275 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:3067
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Discriminant_Spec_F_Ids;
--# expr-done 2275
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2276
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1169;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1169;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_0;
--# end
   --
   --  Primitives for Bare_Generic_Formal
   --
      procedure Initialize_Fields_For_Generic_Formal
        (Self : Bare_Generic_Formal
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Self.Generic_Formal_F_Decl := Generic_Formal_F_Decl;
      end Initialize_Fields_For_Generic_Formal;
   function Generic_Formal_F_Decl
     (Node : Bare_Generic_Formal) return Bare_Basic_Decl
   is
   begin
         return Node.Generic_Formal_F_Decl;
   end;
--# property-start GenericFormal.defining_names /usr/src/packages/BUILD/ada/ast.py:8396
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Defining_Names
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_734;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_734 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_734;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2278 '<FieldAccess .defining_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8396
--# expr-start 2277 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8396
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2277
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2278
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_734;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_734;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Defining_Names;
--# end
--# property-start GenericFormal.type_expression /usr/src/packages/BUILD/ada/ast.py:8397
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Type_Expression
  (Node : Bare_Generic_Formal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2280 '<FieldAccess .type_expression>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8397
--# expr-start 2279 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8397
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2279
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2280
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Type_Expression;
--# end
--# property-start GenericFormal.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:8399
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Entry_Point
  (Node : Bare_Generic_Formal
  )
   return Boolean
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Xref_Entry_Point;
--# end
--# property-start GenericFormal.xref_equation /usr/src/packages/BUILD/ada/ast.py:8400
pragma Warnings (Off, "is not referenced");
 function Generic_Formal_P_Xref_Equation
  (Node : Bare_Generic_Formal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Formal := Bare_Generic_Formal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Formal :=
     Internal_Entity_Generic_Formal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_737;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_737 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_737;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2282 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8400
--# expr-start 2281 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8400
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Basic_Decl (Node => Ent.Node.Generic_Formal_F_Decl, Info => Ent.Info);
--# expr-done 2281
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2282
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_737;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_737;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Formal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Formal_Obj_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Obj_Decl
        (Self : Bare_Generic_Formal_Obj_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Obj_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Package
   --
      procedure Initialize_Fields_For_Generic_Formal_Package
        (Self : Bare_Generic_Formal_Package
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Package;
   --
   --  Primitives for Bare_Generic_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Subp_Decl
        (Self : Bare_Generic_Formal_Subp_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Generic_Formal_Type_Decl
   --
      procedure Initialize_Fields_For_Generic_Formal_Type_Decl
        (Self : Bare_Generic_Formal_Type_Decl
         ; Generic_Formal_F_Decl : Bare_Basic_Decl
        ) is
      begin
            Initialize_Fields_For_Generic_Formal
              (Self, Generic_Formal_F_Decl);
      end Initialize_Fields_For_Generic_Formal_Type_Decl;
   --
   --  Primitives for Bare_Param_Spec
   --
      procedure Param_Spec_Pre_Env_Actions
        (Self            : Bare_Param_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_26 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:6724");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Param_Spec
        (Self : Bare_Param_Spec
         ; Param_Spec_F_Ids : Bare_Defining_Name_List
         ; Param_Spec_F_Has_Aliased : Bare_Aliased_Node
         ; Param_Spec_F_Mode : Bare_Mode
         ; Param_Spec_F_Type_Expr : Bare_Type_Expr
         ; Param_Spec_F_Default_Expr : Bare_Expr
        ) is
      begin
            Self.Param_Spec_F_Ids := Param_Spec_F_Ids;
            Self.Param_Spec_F_Has_Aliased := Param_Spec_F_Has_Aliased;
            Self.Param_Spec_F_Mode := Param_Spec_F_Mode;
            Self.Param_Spec_F_Type_Expr := Param_Spec_F_Type_Expr;
            Self.Param_Spec_F_Default_Expr := Param_Spec_F_Default_Expr;
      end Initialize_Fields_For_Param_Spec;
   function Param_Spec_F_Ids
     (Node : Bare_Param_Spec) return Bare_Defining_Name_List
   is
   begin
         return Node.Param_Spec_F_Ids;
   end;
   function Param_Spec_F_Has_Aliased
     (Node : Bare_Param_Spec) return Bare_Aliased_Node
   is
   begin
         return Node.Param_Spec_F_Has_Aliased;
   end;
   function Param_Spec_F_Mode
     (Node : Bare_Param_Spec) return Bare_Mode
   is
   begin
         return Node.Param_Spec_F_Mode;
   end;
   function Param_Spec_F_Type_Expr
     (Node : Bare_Param_Spec) return Bare_Type_Expr
   is
   begin
         return Node.Param_Spec_F_Type_Expr;
   end;
   function Param_Spec_F_Default_Expr
     (Node : Bare_Param_Spec) return Bare_Expr
   is
   begin
         return Node.Param_Spec_F_Default_Expr;
   end;
--# property-start ParamSpec.is_mandatory /usr/src/packages/BUILD/ada/ast.py:6720
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Is_Mandatory
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Expr;
Is_Null : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2284 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:6720
--# expr-start 2283 '<FieldAccess .default_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:6720
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Default_Expr;
--# expr-done 2283
Is_Null := Fld = null; 
--# expr-done 2284
      Property_Result := Is_Null;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Is_Mandatory;
--# end
--# property-start ParamSpec.defining_names /usr/src/packages/BUILD/ada/ast.py:6721
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Names
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_618;
      Fld : Bare_Defining_Name_List;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_103 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_618 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_618;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2287 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6721
--# expr-start 2285 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:6721
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 2285
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld;
      begin
         for Untyped_Item_103 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_103 := Untyped_Item_103; 
   --# scope-start
               --# bind id Item_103
      --# expr-start 2286 '<as_entity(<Var id>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6721
Is_Null := Item_103 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Item_103); 
   If_Result := As_Entity;
end if;
--# expr-done 2286
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               If_Result;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2287
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_618;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_618;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Defining_Names;
--# end
--# property-start ParamSpec.type_expression /usr/src/packages/BUILD/ada/ast.py:6727
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Type_Expression
  (Node : Bare_Param_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2288 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:6727
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2288
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Type_Expression;
--# end
--# property-start ParamSpec.defining_env /usr/src/packages/BUILD/ada/ast.py:6730
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Defining_Env
  (Node : Bare_Param_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_620;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_620 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_620;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2291 '<Block>' Let_Result None
--# scope-start
--# expr-start 2290 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6731
--# expr-start 2289 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:6731
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2289
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2290
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2291
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_620;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_620;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Defining_Env;
--# end
--# property-start ParamSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:6734
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Equation
  (Node : Bare_Param_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Param_Spec :=
     Internal_Entity_Param_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_621;
            procedure Finalizer_Scope_1887;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_621 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_621;
            procedure Finalizer_Scope_1887 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1887;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2301 '<Block>' Let_Result None
--# scope-start
--# expr-start 2292 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6735
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2292
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 2300 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:6737
--# expr-start 2294 '<FieldAccess .sub_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6737
--# expr-start 2293 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6737
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Param_Spec_F_Type_Expr, Info => Ent.Info);
--# expr-done 2293
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 2294
--# expr-start 2295 '<FieldAccess .default_expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6739
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Param_Spec_F_Default_Expr, Info => Ent.Info);
--# expr-done 2295
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 2299 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:6740
--# expr-start 2296 '<FieldAccess .sub_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6740
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2296
--# expr-start 2298 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:6741
--# expr-start 2297 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6741
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 2297
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_566.Create (Fld_5, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6741") else null)); 
--# expr-done 2298
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6740") else null)); 
--# expr-done 2299
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1887;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6743") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6737") else null)); 
--# expr-done 2300
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2301
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_621;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_621;
               Finalizer_Scope_1887;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Xref_Equation;
--# end
--# property-start ParamSpec.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6747
pragma Warnings (Off, "is not referenced");
 function Param_Spec_P_Xref_Entry_Point
  (Node : Bare_Param_Spec
  )
   return Boolean
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Param_Spec_P_Xref_Entry_Point;
--# end
--# property-start ParamSpec._env_mappings_26 /usr/src/packages/BUILD/ada/ast.py:6724
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_26
  (Node : Bare_Param_Spec
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Param_Spec := Bare_Param_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1196;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Param_Spec;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1196 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1196;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2303 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6724
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2302 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:6724
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Param_Spec_F_Ids;
--# expr-done 2302
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 2303
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1196;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1196;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_26;
--# end
   --
   --  Primitives for Bare_Base_Package_Decl
   --
      procedure Initialize_Fields_For_Base_Package_Decl
        (Self : Bare_Base_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Package_Decl_F_Package_Name := Base_Package_Decl_F_Package_Name;
            Self.Base_Package_Decl_F_Aspects := Base_Package_Decl_F_Aspects;
            Self.Base_Package_Decl_F_Public_Part := Base_Package_Decl_F_Public_Part;
            Self.Base_Package_Decl_F_Private_Part := Base_Package_Decl_F_Private_Part;
            Self.Base_Package_Decl_F_End_Name := Base_Package_Decl_F_End_Name;
      end Initialize_Fields_For_Base_Package_Decl;
   function Base_Package_Decl_F_Package_Name
     (Node : Bare_Base_Package_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Package_Decl_F_Package_Name;
   end;
   function Base_Package_Decl_F_Public_Part
     (Node : Bare_Base_Package_Decl) return Bare_Public_Part
   is
   begin
         return Node.Base_Package_Decl_F_Public_Part;
   end;
   function Base_Package_Decl_F_Private_Part
     (Node : Bare_Base_Package_Decl) return Bare_Private_Part
   is
   begin
         return Node.Base_Package_Decl_F_Private_Part;
   end;
   function Base_Package_Decl_F_End_Name
     (Node : Bare_Base_Package_Decl) return Bare_End_Name
   is
   begin
         return Node.Base_Package_Decl_F_End_Name;
   end;
--# property-start BasePackageDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7621
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Names
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_681;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_681 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_681;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2304 '<FieldAccess .package_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7621
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Package_Decl_F_Package_Name, Info => Ent.Info);
--# expr-done 2304
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_681;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_681;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Defining_Names;
--# end
--# property-start BasePackageDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:7622
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Defining_Env
  (Node : Bare_Base_Package_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_682;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_682 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_682;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2305 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7622
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2305
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_682;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_682;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Defining_Env;
--# end
--# property-start BasePackageDecl.body_part /usr/src/packages/BUILD/ada/ast.py:7625
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Body_Part
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Package_Body;
Dyn_Var_Bind_Result : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2309 '<Block>' Let_Result None
--# scope-start
--# expr-start 2308 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to PackageBody>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:7629
Bound_Imprecise_Fallback := False; 
--# expr-start 2307 '<Cast to PackageBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7630
--# expr-start 2306 '<FieldAccess .body_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:7630
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2306
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Package_Body;
   end if;
--# expr-done 2307
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 2308
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2309
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Body_Part;
--# end
--# property-start BasePackageDecl.declarative_region /usr/src/packages/BUILD/ada/ast.py:7633
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Declarative_Region
  (Node : Bare_Base_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Package_Decl :=
     Internal_Entity_Base_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Public_Part;
Cast_Result : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 2310 '<FieldAccess .public_part>' Fld /usr/src/packages/BUILD/ada/ast.py:7633
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Public_Part (Node => Ent.Node.Base_Package_Decl_F_Public_Part, Info => Ent.Info);
--# expr-done 2310
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Declarative_Region;
--# end
--# property-start BasePackageDecl.env_names /usr/src/packages/BUILD/ada/ast.py:7636
pragma Warnings (Off, "is not referenced");
 function Base_Package_Decl_P_Env_Names
  (Node : Bare_Base_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Base_Package_Decl := Bare_Base_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_685;
            procedure Finalizer_Scope_1888;
      Block_Var : Character_Type_Array_Access;
Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Sym_1 : Symbol_Type;
Array_Lit_1 : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym_2 : Symbol_Type;
Array_Lit_2 : Symbol_Type_Array_Access;
Fld_1 : Bare_Private_Part;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_685 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_685;
            procedure Finalizer_Scope_1888 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_2);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1888;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2323 '<Block>' Let_Result None
--# scope-start
--# expr-start 2311 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7643
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 2311
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind fqn Block_Var
Var_Expr := Block_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 2322 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:7645
--# expr-start 2314 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:7646
--# expr-start 2313 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:7646
--# expr-start 2312 '<FieldAccess .private_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7646
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Base_Package_Decl_F_Private_Part;
--# expr-done 2312
Is_Null := Fld_1 = null; 
--# expr-done 2313
Not_Val := not (Is_Null); 
--# expr-done 2314
if Not_Val then
   --# expr-start 2316 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:7647
--# expr-start 2315 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:7648
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 2315
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 2316
   If_Result := Array_Lit;
else
   --# expr-start 2321 '<ArrayLiteral>' Array_Lit_2 /usr/src/packages/BUILD/ada/ast.py:7650
--# expr-start 2317 '<to_symbol(<Var fqn>)>' Sym_1 /usr/src/packages/BUILD/ada/ast.py:7651
Sym_1 := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 2317
--# expr-start 2320 '<to_symbol(<Concat>)>' Sym_2 /usr/src/packages/BUILD/ada/ast.py:7652
--# expr-start 2319 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:7652
--# expr-start 2318 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:7652
Array_Lit_1 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46), 2 => Character_Type'Val (95), 3 => Character_Type'Val (95), 4 => Character_Type'Val (112), 5 => Character_Type'Val (114), 6 => Character_Type'Val (105), 7 => Character_Type'Val (118), 8 => Character_Type'Val (97), 9 => Character_Type'Val (116), 10 => Character_Type'Val (101), 11 => Character_Type'Val (112), 12 => Character_Type'Val (97), 13 => Character_Type'Val (114), 14 => Character_Type'Val (116))); 
--# expr-done 2318
Concat_Result := Concat (Var_Expr, Array_Lit_1); 
--# expr-done 2319
Sym_2 := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 2320
Array_Lit_2 := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym_1, 2 => Sym_2)); 
--# expr-done 2321
   If_Result := Array_Lit_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 2322
Scope_Result := If_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1888;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2323
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_685;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_685;
               Finalizer_Scope_1888;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Package_Decl_P_Env_Names;
--# end
   --
   --  Primitives for Bare_Generic_Package_Internal
   --
      procedure Generic_Package_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_130 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_131 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Package_Internal
        (Self : Bare_Generic_Package_Internal
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Generic_Package_Internal;
--# property-start GenericPackageInternal.is_library_item /usr/src/packages/BUILD/ada/ast.py:8542
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Internal_P_Is_Library_Item
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal := Bare_Generic_Package_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2327 '<Block>' Let_Result None
--# scope-start
--# expr-start 2326 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:8546
--# expr-start 2325 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8546
--# expr-start 2324 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8546
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2324
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 2325
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Library_Item_Range; 
--# expr-done 2326
Let_Result := Is_A; 
--# end
--# expr-done 2327
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Internal_P_Is_Library_Item;
--# end
--# property-start GenericPackageInternal._env_trans_parent_130 /usr/src/packages/BUILD/ada/ast.py:8548
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_130
  (Node : Bare_Generic_Package_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Package_Internal := Bare_Generic_Package_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_130;
--# end
--# property-start GenericPackageInternal._env_names_131 /usr/src/packages/BUILD/ada/ast.py:8548
pragma Warnings (Off, "is not referenced");
 function Env_Names_131
  (Node : Bare_Generic_Package_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Package_Internal := Bare_Generic_Package_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1301;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1301 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1301;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2328 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:8548
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 2328
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1301;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1301;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_131;
--# end
   --
   --  Primitives for Bare_Package_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Package_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_55 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:7665" & ")";
         end if;
      return Initial_Env;
   end Bare_Package_Decl_Initial_Env_Getter_Fn;
      procedure Package_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_53 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_54 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_57 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_58 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_56 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:7670");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_59 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_60 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_61 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_63 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_62 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_65 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_64 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Decl
        (Self : Bare_Package_Decl
         ; Base_Package_Decl_F_Package_Name : Bare_Defining_Name
         ; Base_Package_Decl_F_Aspects : Bare_Aspect_Spec
         ; Base_Package_Decl_F_Public_Part : Bare_Public_Part
         ; Base_Package_Decl_F_Private_Part : Bare_Private_Part
         ; Base_Package_Decl_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Package_Decl
              (Self, Base_Package_Decl_F_Package_Name, Base_Package_Decl_F_Aspects, Base_Package_Decl_F_Public_Part, Base_Package_Decl_F_Private_Part, Base_Package_Decl_F_End_Name);
      end Initialize_Fields_For_Package_Decl;
--# property-start PackageDecl._env_do_53 /usr/src/packages/BUILD/ada/ast.py:7663
pragma Warnings (Off, "is not referenced");
 function Env_Do_53
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2329 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:7663
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 2329
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_53;
--# end
--# property-start PackageDecl._initial_env_name_54 /usr/src/packages/BUILD/ada/ast.py:7666
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_54
  (Node : Bare_Package_Decl
  )
   return Symbol_Type
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2330 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:7666
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => True);
--# end
--# expr-done 2330
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_54;
--# end
--# property-start PackageDecl._initial_env_55 /usr/src/packages/BUILD/ada/ast.py:7667
pragma Warnings (Off, "is not referenced");
 function Initial_Env_55
  (Node : Bare_Package_Decl
  )
   return Lexical_Env
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1225;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1225 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1225;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2331 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7667
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 2331
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1225;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1225;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_55;
--# end
--# property-start PackageDecl._env_mappings_56 /usr/src/packages/BUILD/ada/ast.py:7670
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_56
  (Node : Bare_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Decl :=
     Internal_Entity_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1226;
      Fld : Symbol_Type;
Cast_Expr : Bare_Package_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1226 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1226;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2333 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7670
--# expr-start 2332 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:7671
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2332
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2333
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1226;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1226;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_56;
--# end
--# property-start PackageDecl._dest_env_name_57 /usr/src/packages/BUILD/ada/ast.py:7673
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_57
  (Node : Bare_Package_Decl
  )
   return Symbol_Type
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2334 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:7673
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => False);
--# end
--# expr-done 2334
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_57;
--# end
--# property-start PackageDecl._fallback_env_58 /usr/src/packages/BUILD/ada/ast.py:7674
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_58
  (Node : Bare_Package_Decl
  )
   return Lexical_Env
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1228;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1228 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1228;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2335 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7674
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 2335
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1228;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1228;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_58;
--# end
--# property-start PackageDecl._env_trans_parent_59 /usr/src/packages/BUILD/ada/ast.py:7662
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_59
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_59;
--# end
--# property-start PackageDecl._env_names_60 /usr/src/packages/BUILD/ada/ast.py:7677
pragma Warnings (Off, "is not referenced");
 function Env_Names_60
  (Node : Bare_Package_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1230;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1230 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1230;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2336 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:7677
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.env_names
Fld := Libadalang.Implementation.Base_Package_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 2336
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1230;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1230;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_60;
--# end
--# property-start PackageDecl._env_do_61 /usr/src/packages/BUILD/ada/ast.py:7679
pragma Warnings (Off, "is not referenced");
 function Env_Do_61
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1231;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1231 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1231;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2337 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:7679
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 2337
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1231;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1231;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_61;
--# end
--# property-start PackageDecl._ref_env_nodes_62 /usr/src/packages/BUILD/ada/ast.py:7682
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_62
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1232;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1232 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1232;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2338 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:7682
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 2338
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1232;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1232;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_62;
--# end
--# property-start PackageDecl._ref_cond_63 /usr/src/packages/BUILD/ada/ast.py:7684
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_63
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2340 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:7684
--# expr-start 2339 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7684
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2339
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 2340
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_63;
--# end
--# property-start PackageDecl._ref_env_nodes_64 /usr/src/packages/BUILD/ada/ast.py:7688
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_64
  (Node : Bare_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1234;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1234 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1234;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2341 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:7688
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 2341
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1234;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1234;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_64;
--# end
--# property-start PackageDecl._ref_cond_65 /usr/src/packages/BUILD/ada/ast.py:7690
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_65
  (Node : Bare_Package_Decl
  )
   return Boolean
is
   Self : Bare_Package_Decl := Bare_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2343 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:7690
--# expr-start 2342 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7690
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2342
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 2343
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_65;
--# end
   --
   --  Primitives for Bare_Base_Type_Decl
   --
      procedure Base_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Base_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_2 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:4028");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Base_Type_Decl
        (Self : Bare_Base_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Base_Type_Decl_F_Name := Base_Type_Decl_F_Name;
      Self.LF_Present_Dottable_Subps_Env := False;
      Self.LF_Stg_Dottable_Subps_Env := Empty_Env;
      end Initialize_Fields_For_Base_Type_Decl;
   function Base_Type_Decl_F_Name
     (Node : Bare_Base_Type_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Base_Type_Decl_F_Name;
   end;
--# property-start BaseTypeDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:4030
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Defining_Names
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_313;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_313 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_313;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2344 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:4030
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Base_Type_Decl_F_Name, Info => Ent.Info);
--# expr-done 2344
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_313;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_313;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Defining_Names;
--# end
--# property-start BaseTypeDecl.base_subtype /usr/src/packages/BUILD/ada/ast.py:4035
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Base_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Base_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2348 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2347 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4040
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 2346 '<FieldAccess .base_subtype>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4041
--# expr-start 2345 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4041
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2345
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2346
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Classwide_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := Ent; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2347
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2348
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Subtype;
--# end
--# property-start BaseTypeDecl.anonymous_access_type /usr/src/packages/BUILD/ada/ast.py:4046
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Not_Null_Absent;
Cast_Result : Bare_Not_Null;
New_Node_1 : Bare_Anonymous_Type_Access_Def;
Cast_Result_1 : Bare_Type_Def;
New_Node_2 : Bare_Synth_Anonymous_Type_Decl;
Cast_Result_2 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Anonymous_Access_Type,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2355 '<Block>' Let_Result None
--# scope-start
--# expr-start 2354 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result None
--# expr-start 2353 '<Cast to BaseTypeDecl>' Cast_Result_2 None
--# expr-start 2352 '<New BareSynthAnonymousTypeDecl>' New_Node_2 None
--# expr-start 2349 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:4048
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2349
--# expr-start 2351 '<New BareAnonymousTypeAccessDef>' New_Node_1 None
--# expr-start 2350 '<New BareNotNullAbsent>' New_Node None
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Not_Null_Absent);
Initialize
  (Self => New_Node,
   Kind => Ada_Not_Null_Absent,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 2350
      Cast_Result := New_Node;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node_1 := new Root_Node_Record
  (Ada_Anonymous_Type_Access_Def);
Initialize
  (Self => New_Node_1,
   Kind => Ada_Anonymous_Type_Access_Def,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_1);
   Initialize_Fields_For_Anonymous_Type_Access_Def
     (Self => New_Node_1,
      Access_Def_F_Has_Not_Null => Cast_Result, Anonymous_Type_Access_Def_F_Type_Decl => Self);
--# expr-done 2351
      Cast_Result_1 := New_Node_1;
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node_2 := new Root_Node_Record
  (Ada_Synth_Anonymous_Type_Decl);
Initialize
  (Self => New_Node_2,
   Kind => Ada_Synth_Anonymous_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node_2);
   Initialize_Fields_For_Synth_Anonymous_Type_Decl
     (Self => New_Node_2,
      Base_Type_Decl_F_Name => Fld, Type_Decl_F_Aspects => No_Bare_Ada_Node, Type_Decl_F_Discriminants => No_Bare_Ada_Node, Type_Decl_F_Type_Def => Cast_Result_1);
--# expr-done 2352
      Cast_Result_2 := New_Node_2;
--# expr-done 2353
Is_Null := Cast_Result_2 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_2); 
   If_Result := As_Entity;
end if;
--# expr-done 2354
Let_Result := If_Result; 
--# end
--# expr-done 2355
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Anonymous_Access_Type;
--# end
--# property-start BaseTypeDecl.anonymous_access_type_or_null /usr/src/packages/BUILD/ada/ast.py:4058
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Anonymous_Access_Type_Or_Null
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2357 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2356 '<FieldAccess .anonymous_access_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4059
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2356
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2357
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Anonymous_Access_Type_Or_Null;
--# end
--# property-start BaseTypeDecl.private_completion /usr/src/packages/BUILD/ada/ast.py:4064
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Private_Completion
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_317;
            procedure Finalizer_Scope_1895;
      Var_Expr : Bare_Ada_Node_List;
Var_Expr_1 : Bare_Private_Part;
Var_Expr_2 : Bare_Base_Package_Decl;
Var_Expr_3 : Bare_Public_Part;
Var_Expr_4 : Bare_Base_Type_Decl;
Fld : Bare_Declarative_Part;
Cast_Result : Bare_Public_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result_1 : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Scope_Result_1 : Bare_Private_Part;
Result_Var_1 : Bare_Private_Part;
Fld_3 : Bare_Ada_Node_List;
Scope_Result_2 : Bare_Ada_Node_List;
Result_Var_2 : Bare_Ada_Node_List;
Item_104 : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Type_Decl;
Fld_4 : Symbol_Type;
Fld_5 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Scope_Result_4 : Bare_Ada_Node_Array_Access;
Result_Var_4 : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_3 : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_317 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
            end Finalizer_Scope_317;
            procedure Finalizer_Scope_1895 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1895;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Private_Completion,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2372 '<Block>' Let_Result None
--# scope-start
--# expr-start 2371 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4069
--# expr-start 2370 '<Cast to BaseTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:4069
--# expr-start 2369 '<at(<Then <Then <Then <Cast to BasePackageDecl>: <Var Var_Expr_2> <FieldAccess .private_part>>: <Var Var_Expr_1> <FieldAccess .decls>>: <Var Var_Expr> <filter>>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:4069
--# expr-start 2361 '<Cast to BasePackageDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:4069
--# expr-start 2359 '<Cast to PublicPart>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4069
--# expr-start 2358 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:4069
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2358
   if Fld = null
      or else Fld.Kind in Ada_Public_Part_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2359
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2360 '<parent(<Var Var_Expr_3>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4069
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr_3.Parent; 
--# expr-done 2360
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Result_Var;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 2361
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2362 '<FieldAccess .private_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4069
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr_2.Base_Package_Decl_F_Private_Part;
--# expr-done 2362
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr_1 := Result_Var_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2363 '<FieldAccess .decls>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4069
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr_1.Declarative_Part_F_Decls;
--# expr-done 2363
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2368 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4069
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Var_Expr = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Var_Expr;
      begin
         for Untyped_Item_104 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_104 := Untyped_Item_104; 
   --# scope-start
               --# bind d Item_104
--# expr-start 2364 '<Cast to BaseTypeDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:4072
Cast_Expr := Item_104; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Cast_Expr;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2364
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pp Var_Expr_4
--# expr-start 2367 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4074
--# expr-start 2365 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4074
      if Var_Expr_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Var_Expr_4);
--# end
--# expr-done 2365
--# expr-start 2366 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4074
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2366
Is_Equal := Fld_4 = Fld_5; 
--# expr-done 2367
Scope_Result_3 := Is_Equal;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
               if Result_Var_3 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_104;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2368
Scope_Result_4 := Map_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_1895;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Get_Result := Get (Result_Var_4, 0, True); 
--# expr-done 2369
   if Get_Result = null
      or else Get_Result.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Get_Result;
   else
         Cast_Result_3 := No_Bare_Ada_Node;
   end if;
--# expr-done 2370
Is_Null := Cast_Result_3 = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_3); 
   If_Result := As_Entity;
end if;
--# expr-done 2371
Let_Result := If_Result; 
--# end
--# expr-done 2372
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_317;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_317;
               Finalizer_Scope_1895;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Private_Completion;
--# end
--# property-start BaseTypeDecl.model_of_type /usr/src/packages/BUILD/ada/ast.py:4080
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Model_Of_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2377 '<Block>' Let_Result None
--# scope-start
--# expr-start 2376 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4085
--# expr-start 2375 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4085
--# expr-start 2374 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4085
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2373 '<Symbol Model_Of>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 2373
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2374
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2375
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 2376
Let_Result := Fld_1; 
--# end
--# expr-done 2377
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Model_Of_Type;
--# end
--# property-start BaseTypeDecl.modeled_type /usr/src/packages/BUILD/ada/ast.py:4090
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Modeled_Type
  (Node : Bare_Base_Type_Decl
      ; From_Unit : Internal_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_unit From_Unit
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_319;
      Block_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Result : Bare_Package_Decl;
Fld_1 : Bare_Public_Part;
Fld_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_319 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_319;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2386 '<Block>' Let_Result None
--# scope-start
--# expr-start 2381 '<FieldAccess .types_with_models>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4094
--# expr-start 2380 '<FieldAccess .public_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4094
--# expr-start 2379 '<Cast to PackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4094
--# expr-start 2378 '<FieldAccess .top_level_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4094
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_decl
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => From_Unit);
--# end
--# expr-done 2378
   if Fld = null
      or else Fld.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Fld;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2379
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 2380
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.types_with_models
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Types_With_Models (Node => Fld_1);
--# end
--# expr-done 2381
Block_Var := Fld_2; Inc_Ref (Block_Var);
--# bind types_with_models Block_Var
--# expr-start 2385 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:4098
--# expr-start 2384 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4098
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_105 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_105
               --# expr-start 2383 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4098
--# expr-start 2382 '<FieldAccess .model_of_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4098
      if Item_105.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.model_of_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Model_Of_Type (Node => Item_105.Node, E_Info => Item_105.Info);
--# end
--# expr-done 2382
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2383
               if Is_Equiv then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_105;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2384
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2385
Let_Result := Get_Result; 
--# end
--# expr-done 2386
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_319;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_319;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Modeled_Type;
--# end
--# property-start BaseTypeDecl.is_view_of_type /usr/src/packages/BUILD/ada/ast.py:4101
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_View_Of_Type
  (Node : Bare_Base_Type_Decl
      ; Comp_View : Internal_Entity_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind comp_view Comp_View
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2392 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind typ Block_Var
--# expr-start 2391 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:4107
--# expr-start 2387 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4108
Is_Null := Comp_View.Node = null; 
--# expr-done 2387
if Is_Null then
   If_Result_1 := False;
else
--# expr-start 2388 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4109
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Comp_View; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2388
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 2390 '<FieldAccess .is_view_of_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4110
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2389 '<FieldAccess .previous_part(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4110
      if Comp_View.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Comp_View.Node, Go_To_Incomplete => True, E_Info => Comp_View.Info);
--# end
--# expr-done 2389
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Block_Var.Node, Comp_View => Fld, E_Info => Block_Var.Info);
--# end
--# expr-done 2390
   If_Result := Fld_1;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2391
Let_Result := If_Result_1; 
--# end
--# expr-done 2392
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_View_Of_Type;
--# end
--# property-start BaseTypeDecl.is_array_or_rec /usr/src/packages/BUILD/ada/ast.py:4114
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Or_Rec
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2395 '<Block>' Let_Result None
--# scope-start
--# expr-start 2393 '<FieldAccess .is_array>' Fld /usr/src/packages/BUILD/ada/ast.py:4115
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2393
if Fld then
   If_Result := True;
else
   --# expr-start 2394 '<FieldAccess .is_record_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4115
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2394
   If_Result := Fld_1;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2395
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Or_Rec;
--# end
--# property-start BaseTypeDecl.is_inherited_primitive /usr/src/packages/BUILD/ada/ast.py:4118
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Inherited_Primitive
  (Node : Bare_Base_Type_Decl
      ; P : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind p P
   Property_Result : Boolean;
      Fld : Bare_Base_Type_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Internal_Metadata;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2402 '<Block>' Let_Result None
--# scope-start
--# expr-start 2401 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4123
--# expr-start 2400 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4123
--# expr-start 2396 '<FieldAccess .node>' Fld /usr/src/packages/BUILD/ada/ast.py:4123
Fld := Ent.Node;
--# expr-done 2396
      Cast_Result := Fld;
--# expr-start 2399 '<FieldAccess .primitive>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4123
--# expr-start 2398 '<FieldAccess .md>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4123
--# expr-start 2397 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4123
Fld_1 := P.Info;
--# expr-done 2397
Fld_2 := Fld_1.MD;
--# expr-done 2398
Fld_3 := Fld_2.Primitive;
--# expr-done 2399
Is_Equal := Cast_Result = Fld_3; 
--# expr-done 2400
Not_Val := not (Is_Equal); 
--# expr-done 2401
Let_Result := Not_Val; 
--# end
--# expr-done 2402
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Inherited_Primitive;
--# end
--# property-start BaseTypeDecl.get_record_representation_clause /usr/src/packages/BUILD/ada/ast.py:4127
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Record_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Record_Rep_Clause
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Record_Rep_Clause;
            procedure Finalizer_Scope_323;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Record_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_106 : Internal_Entity;
Bare_Item_106 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Record_Rep_Clause;
Let_Result : Internal_Entity_Record_Rep_Clause;
            procedure Finalizer_Scope_323 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_323;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2413 '<Block>' Let_Result None
--# scope-start
--# expr-start 2412 '<Cast to RecordRepClause.entity>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:4132
--# expr-start 2411 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:4132
--# expr-start 2410 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4132
--# expr-start 2405 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4132
--# expr-start 2403 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:4132
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2403
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2404 '<FieldAccess .decls>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4132
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 2404
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2405
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_106 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_106 := Untyped_Item_106; 
Is_Null_1 := Bare_Item_106 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_106); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_106 := If_Result_1; 
   --# scope-start
               --# bind d Item_106
--# expr-start 2406 '<Cast to RecordRepClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4133
Cast_Expr := Item_106; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 2406
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Record_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2409 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4134
--# expr-start 2408 '<FieldAccess .referenced_decl>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4134
--# expr-start 2407 '<FieldAccess .name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4134
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Record_Rep_Clause_F_Name, Info => Var_Expr_1.Info);
--# expr-done 2407
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 2408
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2409
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_106;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2410
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2411
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Record_Rep_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Record_Rep_Clause;
   end if;
--# expr-done 2412
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2413
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_323;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_323;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Record_Representation_Clause;
--# end
--# property-start BaseTypeDecl.get_enum_representation_clause /usr/src/packages/BUILD/ada/ast.py:4140
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Enum_Representation_Clause
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Rep_Clause
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Enum_Rep_Clause;
            procedure Finalizer_Scope_324;
      Var_Expr : Bare_Declarative_Part;
Var_Expr_1 : Internal_Entity_Enum_Rep_Clause;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node_List;
Scope_Result : Bare_Ada_Node_List;
Result_Var : Bare_Ada_Node_List;
As_Entity : Internal_Entity_Ada_Node_List;
Is_Null : Boolean;
If_Result : Internal_Entity_Ada_Node_List;
Fld_2 : Bare_Ada_Node_List;
Fld_3 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_107 : Internal_Entity;
Bare_Item_107 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Rep_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Enum_Rep_Clause;
Let_Result : Internal_Entity_Enum_Rep_Clause;
            procedure Finalizer_Scope_324 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_324;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2424 '<Block>' Let_Result None
--# scope-start
--# expr-start 2423 '<Cast to EnumRepClause.entity>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:4145
--# expr-start 2422 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:4145
--# expr-start 2421 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4145
--# expr-start 2416 '<as_entity(<Then <FieldAccess .declarative_scope>: <Var Var_Expr> <FieldAccess .decls>>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4145
--# expr-start 2414 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:4145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2414
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2415 '<FieldAccess .decls>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4145
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr.Declarative_Part_F_Decls;
--# expr-done 2415
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Ada_Node_List;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 2416
Fld_2 := If_Result.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_2;
      begin
         for Untyped_Item_107 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_107 := Untyped_Item_107; 
Is_Null_1 := Bare_Item_107 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_3 := If_Result.Info;
As_Entity_1 := (Info => Fld_3, Node => Bare_Item_107); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_107 := If_Result_1; 
   --# scope-start
               --# bind d Item_107
--# expr-start 2417 '<Cast to EnumRepClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4146
Cast_Expr := Item_107; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 2417
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Enum_Rep_Clause then
   --# scope-start
   --# bind p Var_Expr_1
--# expr-start 2420 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4147
--# expr-start 2419 '<FieldAccess .referenced_decl>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4147
--# expr-start 2418 '<FieldAccess .type_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4147
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Enum_Rep_Clause_F_Type_Name, Info => Var_Expr_1.Info);
--# expr-done 2418
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_5 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_4.Info);
--# end
--# expr-done 2419
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2420
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_107;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2421
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2422
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Enum_Rep_Clause_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Enum_Rep_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Enum_Rep_Clause;
   end if;
--# expr-done 2423
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2424
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_324;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_324;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Enum_Representation_Clause;
--# end
--# property-start '[dispatcher]BaseTypeDecl.primitives_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.primitives_env
                  Property_Result := Base_Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.primitives_env
                  Property_Result := Base_Subtype_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.primitives_env
                  Property_Result := Type_Decl_P_Primitives_Env
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_record_type
                  Property_Result := Base_Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_record_type
                  Property_Result := Base_Subtype_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_record_type
                  Property_Result := Type_Decl_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_array_type /usr/src/packages/BUILD/ada/ast.py:4163
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2426 '<Block>' Let_Result None
--# scope-start
--# expr-start 2425 '<FieldAccess .is_array>' Fld /usr/src/packages/BUILD/ada/ast.py:4167
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2425
Let_Result := Fld; 
--# end
--# expr-done 2426
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Type;
--# end
--# property-start BaseTypeDecl.find_derived_types /usr/src/packages/BUILD/ada/ast.py:4171
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_328;
            procedure Finalizer_Scope_1907;
            procedure Finalizer_Scope_1909;
            procedure Finalizer_Scope_1910;
            procedure Finalizer_Scope_1911;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Type_Decl;
Var_Expr_2 : Boolean;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Entity_Type_Decl_Array_Access;
Map_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Type_Decl;
Fld_3 : Boolean;
Singleton : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Type_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_328 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_328;
            procedure Finalizer_Scope_1907 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1907;
            procedure Finalizer_Scope_1909 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1909;
            procedure Finalizer_Scope_1910 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_1910;
            procedure Finalizer_Scope_1911 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1911;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2438 '<Block>' Let_Result None
--# scope-start
--# expr-start 2437 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:4176
--# expr-start 2427 '<children(<Var root>)>' Fld /usr/src/packages/BUILD/ada/ast.py:4176
      if Root.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 2427
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 2434 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:4177
--# expr-start 2432 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4177
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_109 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_109
               --# expr-start 2431 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4177
--# expr-start 2428 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4177
Is_Null := Item_109.Node = null; 
--# expr-done 2428
if Is_Null then
   If_Result := True;
else
   --# expr-start 2430 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4177
--# expr-start 2429 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4177
Fld_1 := Item_109.Node;
--# expr-done 2429
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 2430
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 2431
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_109;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2432
declare
   Map_Result_1_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_108 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_108
      --# expr-start 2433 '<FieldAccess .find_derived_types(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.find_derived_types
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => Item_108, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2433
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1909;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2434
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1907;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2435 '<Cast to TypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:4179
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 2435
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Type_Decl then
   --# scope-start
   --# bind type_decl Var_Expr_1
--# expr-start 2436 '<FieldAccess .is_derived_type(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4180
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr_1.Node, Other_Type => Ent, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2436
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
                Singleton := Create_Internal_Entity_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr_1;
Scope_Result_1 := Singleton;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1911;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1910;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 2437
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2438
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_328;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_328;
               Finalizer_Scope_1907;
               Finalizer_Scope_1909;
               Finalizer_Scope_1910;
               Finalizer_Scope_1911;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Find_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_task_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_task_type
                  Property_Result := Base_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.is_task_type
                  Property_Result := Task_Type_Decl_P_Is_Task_Type
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_real_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_real_type
                  Property_Result := Base_Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_real_type
                  Property_Result := Base_Subtype_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_real_type
                  Property_Result := Type_Decl_P_Is_Real_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_float_type
                  Property_Result := Base_Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_float_type
                  Property_Result := Base_Subtype_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_float_type
                  Property_Result := Type_Decl_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_fixed_point
                  Property_Result := Base_Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_fixed_point
                  Property_Result := Base_Subtype_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_fixed_point
                  Property_Result := Type_Decl_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_enum_type
                  Property_Result := Base_Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_enum_type
                  Property_Result := Base_Subtype_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_enum_type
                  Property_Result := Type_Decl_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_classwide' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_classwide
                  Property_Result := Base_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_classwide
                  Property_Result := Base_Subtype_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_classwide
                  Property_Result := Classwide_Type_Decl_P_Is_Classwide
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_access_type
                  Property_Result := Base_Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_access_type
                  Property_Result := Base_Subtype_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_access_type
                  Property_Result := Type_Decl_P_Is_Access_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.is_implicit_deref /usr/src/packages/BUILD/ada/ast.py:4213
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Implicit_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2439 '<FieldAccess .is_access_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4214
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2439
if Fld then
   If_Result := True;
else
   --# expr-start 2442 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4214
--# expr-start 2441 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4214
--# expr-start 2440 '<FieldAccess .get_imp_deref>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4214
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2440
Is_Null := Fld_1.Node = null; 
--# expr-done 2441
Not_Val := not (Is_Null); 
--# expr-done 2442
   If_Result := Not_Val;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Implicit_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.has_ud_indexing' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.has_ud_indexing
                  Property_Result := Base_Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.has_ud_indexing
                  Property_Result := Base_Subtype_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.has_ud_indexing
                  Property_Result := Type_Decl_P_Has_Ud_Indexing
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start '[dispatcher]BaseTypeDecl.constant_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.constant_indexing_fns
                  Property_Result := Base_Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.constant_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.constant_indexing_fns
                  Property_Result := Type_Decl_P_Constant_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.variable_indexing_fns' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.variable_indexing_fns
                  Property_Result := Base_Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.variable_indexing_fns
                  Property_Result := Base_Subtype_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.variable_indexing_fns
                  Property_Result := Type_Decl_P_Variable_Indexing_Fns
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start '[dispatcher]BaseTypeDecl.get_imp_deref' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.get_imp_deref
                  Property_Result := Base_Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.get_imp_deref
                  Property_Result := Type_Decl_P_Get_Imp_Deref
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start '[dispatcher]BaseTypeDecl.access_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.access_def
                  Property_Result := Base_Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.access_def
                  Property_Result := Base_Subtype_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.access_def
                  Property_Result := Type_Decl_P_Access_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Access_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_char_type
                  Property_Result := Base_Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_char_type
                  Property_Result := Base_Subtype_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_char_type
                  Property_Result := Type_Decl_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.classwide_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.classwide_type
                  Property_Result := Base_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.classwide_type
                  Property_Result := Classwide_Type_Decl_P_Classwide_Type
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.is_universal_type /usr/src/packages/BUILD/ada/ast.py:4260
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2449 '<Block>' Let_Result None
--# scope-start
--# expr-start 2444 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4268
--# expr-start 2443 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4268
Is_Null := Ent.Node = null; 
--# expr-done 2443
Not_Val := not (Is_Null); 
--# expr-done 2444
if Not_Val then
--# expr-start 2446 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4269
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 2445 '<FieldAccess .universal_int_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4269
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2445
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 2446
if Is_Equiv then
   If_Result := True;
else
   --# expr-start 2448 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:4270
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 2447 '<FieldAccess .universal_real_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4270
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2447
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_1); 
--# expr-done 2448
   If_Result := Is_Equiv_1;
end if;
   If_Result_1 := If_Result;
else
   If_Result_1 := False;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2449
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_not_universal_type /usr/src/packages/BUILD/ada/ast.py:4274
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Not_Universal_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2454 '<Block>' Let_Result None
--# scope-start
--# expr-start 2451 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4282
--# expr-start 2450 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4282
Is_Null := Ent.Node = null; 
--# expr-done 2450
Not_Val := not (Is_Null); 
--# expr-done 2451
if Not_Val then
   --# expr-start 2453 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4282
--# expr-start 2452 '<FieldAccess .is_universal_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4282
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_universal_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Is_Universal_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2452
Not_Val_1 := not (Fld); 
--# expr-done 2453
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2454
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Not_Universal_Type;
--# end
--# property-start BaseTypeDecl.is_access_type_predicate /usr/src/packages/BUILD/ada/ast.py:4285
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type_Predicate
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2456 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2455 '<FieldAccess .is_access_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4291
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2455
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2456
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_Type_Predicate;
--# end
--# property-start BaseTypeDecl.array_ndims /usr/src/packages/BUILD/ada/ast.py:4294
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Ndims
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Let_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2457 '<Block>' Let_Result None
--# scope-start
Let_Result := 0; 
--# end
--# expr-done 2457
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Ndims;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.discrete_range
                  Property_Result := Base_Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.discrete_range
                  Property_Result := Subtype_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.discrete_range
                  Property_Result := Type_Decl_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.static_predicate /usr/src/packages/BUILD/ada/ast.py:4306
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Static_Predicate
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Expr;
      Var_Expr : Internal_Entity_Expr;
Sym : Symbol_Type;
Fld : Internal_Aspect;
Fld_1 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Aspect;
Fld_3 : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Let_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2464 '<Block>' Let_Result None
--# scope-start
--# expr-start 2460 '<FieldAccess .value>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4311
--# expr-start 2459 '<FieldAccess .get_aspect(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4311
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2458 '<Symbol Static_Predicate>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Static_Predicate); 
--# expr-done 2458
--# property-call-start BasicDecl.get_aspect
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2459
Fld_1 := Fld.Value;
--# expr-done 2460
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 2463 '<FieldAccess .value>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4312
--# expr-start 2462 '<FieldAccess .get_aspect(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2461 '<Symbol Predicate>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Predicate); 
--# expr-done 2461
--# property-call-start BasicDecl.get_aspect
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect (Node => Ent.Node, Name => Sym_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2462
Fld_3 := Fld_2.Value;
--# expr-done 2463
   Result_Var := Fld_3;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2464
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Static_Predicate;
--# end
--# property-start BaseTypeDecl.satisfies_type_predicates /usr/src/packages/BUILD/ada/ast.py:4318
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Satisfies_Type_Predicates
  (Node : Bare_Base_Type_Decl
      ; Value : Big_Integer_Type
      ; Imprecise_Fallback : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind value Value
   --# bind imprecise_fallback Imprecise_Fallback
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_350;
            procedure Finalizer_Scope_1914;
      Block_Var : Big_Integer_Type;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subtype_Decl;
Big_Int : Big_Integer_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
New_Struct : Internal_Substitution := No_Substitution;
Singleton : Internal_Substitution_Array_Access;
Fld_1 : Big_Integer_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Subtype_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_350 is
            begin
                     Dec_Ref (Big_Int);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_350;
            procedure Finalizer_Scope_1914 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1914;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2480 '<Block>' Let_Result None
--# scope-start
--# expr-start 2465 '<BigInteger 1>' Big_Int /usr/src/packages/BUILD/ada/ast.py:4325
Big_Int := Create_Big_Integer ("1"); 
--# expr-done 2465
Block_Var := Big_Int; Inc_Ref (Block_Var);
--# bind true_val Block_Var
--# expr-start 2466 '<FieldAccess .static_predicate>' Fld /usr/src/packages/BUILD/ada/ast.py:4328
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.static_predicate
Fld := Libadalang.Implementation.Base_Type_Decl_P_Static_Predicate (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2466
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind pred Var_Expr
--# expr-start 2469 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4329
--# expr-start 2468 '<FieldAccess .eval_as_int_in_env(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4329
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2467 '<New InternalSubstitution>' New_Struct /usr/src/packages/BUILD/ada/ast.py:4330
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Inc_Ref (Value);
New_Struct := (From_Decl => Cast_Result, To_Value => Value, Value_Type => Ent); 
--# expr-done 2467
                Singleton := Create_Internal_Substitution_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Var_Expr.Node, Env => Singleton, E_Info => Var_Expr.Info);
--# expr-done 2468
Is_Equal := Equivalent (Block_Var, Fld_1); 
--# expr-done 2469
Scope_Result := Is_Equal;
   --# end
      Finalizer_Scope_1914;
   Result_Var := Scope_Result;
else
   Result_Var := True;
end if;
Block_Var_1 := Result_Var; 
--# bind satisfies_own_predicate Block_Var_1
--# expr-start 2470 '<Cast to BaseSubtypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:4340
Cast_Expr_1 := Ent; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subtype_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subtype_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subtype_Decl;
   end if;
--# expr-done 2470
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Subtype_Decl then
   --# scope-start
--# expr-start 2471 '<FieldAccess .get_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4340
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2471
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var_1; 
--# bind from_type Block_Var_2
--# expr-start 2472 '<FieldAccess .base_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4341
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2472
Block_Var_3 := Fld_3; 
--# bind base_type Block_Var_3
if Block_Var_1 then
   --# expr-start 2479 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:4343
--# expr-start 2474 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4344
--# expr-start 2473 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4344
Is_Null := Block_Var_2.Node = null; 
--# expr-done 2473
Not_Val := not (Is_Null); 
--# expr-done 2474
if Not_Val then
   --# expr-start 2475 '<FieldAccess .satisfies_type_predicates(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4345
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_2.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 2475
   If_Result_1 := Fld_4;
else
--# expr-start 2477 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4347
--# expr-start 2476 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:4347
Is_Null_1 := Block_Var_3.Node = null; 
--# expr-done 2476
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2477
if Not_Val_1 then
   --# expr-start 2478 '<FieldAccess .satisfies_type_predicates(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4348
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.satisfies_type_predicates
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Satisfies_Type_Predicates (Node => Block_Var_3.Node, Value => Value, Imprecise_Fallback => Imprecise_Fallback, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 2478
   If_Result := Fld_5;
else
   If_Result := True;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2479
   If_Result_2 := If_Result_1;
else
   If_Result_2 := False;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 2480
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_350;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_350;
               Finalizer_Scope_1914;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Satisfies_Type_Predicates;
--# end
--# property-start BaseTypeDecl.is_iterator_type /usr/src/packages/BUILD/ada/ast.py:4354
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterator_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_351;
      Block_Var : Bare_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Classwide_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Basic_Decl;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_351 is
            begin
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_351;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Is_Iterator_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2491 '<Block>' Let_Result None
--# scope-start
--# expr-start 2484 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4355
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2483 '<ArrayLiteral>' Array_Lit None
--# expr-start 2481 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 2481
--# expr-start 2482 '<Symbol Iterator_Interfaces>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Interfaces); 
--# expr-done 2482
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 2483
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 2484
Block_Var := Fld; 
--# bind iifcs Block_Var
--# expr-start 2485 '<Cast to ClasswideTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4358
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 2485
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind cw Var_Expr
--# expr-start 2486 '<FieldAccess .typedecl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4359
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld_1 := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2486
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Block_Var_1 := Result_Var; 
--# bind typ Block_Var_1
--# expr-start 2490 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4361
--# expr-start 2489 '<FieldAccess .node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4361
--# expr-start 2488 '<FieldAccess .semantic_parent>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4361
--# expr-start 2487 '<FieldAccess .semantic_parent>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4361
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_2 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2487
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_3 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 2488
Fld_4 := Fld_3.Node;
--# expr-done 2489
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_4 = Cast_Result_1; 
--# expr-done 2490
Let_Result := Is_Equal; 
--# end
--# expr-done 2491
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_351;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_351;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Iterator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_discrete_type
                  Property_Result := Base_Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_discrete_type
                  Property_Result := Base_Subtype_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_discrete_type
                  Property_Result := Type_Decl_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_int_type
                  Property_Result := Base_Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_int_type
                  Property_Result := Base_Subtype_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_int_type
                  Property_Result := Type_Decl_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.is_str_type_or_null /usr/src/packages/BUILD/ada/ast.py:4376
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Str_Type_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Fld : Integer;
Is_Equal : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2497 '<Block>' Let_Result None
--# scope-start
--# expr-start 2492 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4377
Is_Null := Self = null; 
--# expr-done 2492
if Is_Null then
   If_Result_1 := True;
else
--# expr-start 2494 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:4379
--# expr-start 2493 '<FieldAccess .array_ndims>' Fld /usr/src/packages/BUILD/ada/ast.py:4379
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2493
Is_Equal := Fld = 1; 
--# expr-done 2494
if Is_Equal then
--# expr-start 2495 '<FieldAccess .comp_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4379
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2495
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2496 '<FieldAccess .is_char_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4379
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2496
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2497
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Str_Type_Or_Null;
--# end
--# property-start '[dispatcher]BaseTypeDecl.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.accessed_type
                  Property_Result := Base_Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.accessed_type
                  Property_Result := Base_Subtype_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.accessed_type
                  Property_Result := Type_Decl_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.final_accessed_type /usr/src/packages/BUILD/ada/ast.py:4391
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Final_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; First_Call : Boolean
         := True
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind first_call First_Call
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2501 '<Block>' Let_Result None
--# scope-start
--# expr-start 2498 '<FieldAccess .accessed_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4402
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2498
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr
--# expr-start 2499 '<FieldAccess .final_accessed_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4403
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Var_Expr.Node, First_Call => False, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2499
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 2500 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:4404
if First_Call then
   If_Result := No_Entity_Base_Type_Decl;
else
   If_Result := Ent;
end if;
--# expr-done 2500
   Result_Var := If_Result;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2501
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Final_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_access_to /usr/src/packages/BUILD/ada/ast.py:4408
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_To
  (Node : Bare_Base_Type_Decl
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2504 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 2503 '<FieldAccess .matching_formal_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4410
--# expr-start 2502 '<FieldAccess .accessed_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4410
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2502
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld.Node, Formal_Type => Typ, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2503
Let_Result := Fld_1; 
--# end
--# expr-done 2504
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_To;
--# end
--# property-start BaseTypeDecl.is_subp_access_of /usr/src/packages/BUILD/ada/ast.py:4413
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Subp_Access_Of
  (Node : Bare_Base_Type_Decl
      ; Entity : Internal_Entity_Basic_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind entity Entity
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Access_To_Subp_Def;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2510 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind access_type Block_Var
--# expr-start 2506 '<Cast to AccessToSubpDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4419
--# expr-start 2505 '<FieldAccess .access_def>' Fld /usr/src/packages/BUILD/ada/ast.py:4419
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2505
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 2506
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr
--# expr-start 2507 '<FieldAccess .subp_spec_or_null>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4420
      if Entity.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Entity.Node, E_Info => Entity.Info);
--# end
--# expr-done 2507
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind se Var_Expr_1
--# expr-start 2509 '<FieldAccess .match_signature(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4421
--# expr-start 2508 '<FieldAccess .subp_spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4421
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr.Info);
--# expr-done 2508
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_3 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Var_Expr_1, Match_Name => False, E_Info => Fld_2.Info);
--# end
--# expr-done 2509
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 2510
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Subp_Access_Of;
--# end
--# property-start BaseTypeDecl.is_generic_formal /usr/src/packages/BUILD/ada/ast.py:4426
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Generic_Formal
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Base_Type_Decl;
Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2516 '<Block>' Let_Result None
--# scope-start
--# expr-start 2512 '<IsA GenericFormalTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:4431
--# expr-start 2511 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:4431
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2511
Is_A := Fld /= null 
and then Fld.Kind in Ada_Generic_Formal_Type_Decl_Range; 
--# expr-done 2512
if Is_A then
   If_Result := True;
else
--# expr-start 2514 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4432
--# expr-start 2513 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4432
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 2513
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2514
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2515 '<FieldAccess .is_generic_formal>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4432
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Var_Expr);
--# end
--# expr-done 2515
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2516
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Generic_Formal;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl | Ada_Protected_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_tagged_type
                  Property_Result := Base_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_tagged_type
                  Property_Result := Base_Subtype_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_tagged_type
                  Property_Result := Classwide_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTaggedTypeDecl.is_tagged_type
                  Property_Result := Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_tagged_type
                  Property_Result := Type_Decl_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_type
                  Property_Result := Base_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_type
                  Property_Result := Base_Subtype_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_type
                  Property_Result := Classwide_Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.base_type
                  Property_Result := Type_Decl_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Base_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_types' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_types
                  Property_Result := Base_Type_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.base_types
                  Property_Result := Base_Subtype_Decl_P_Base_Types
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Base_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.base_interfaces
                  Property_Result := Base_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.base_interfaces
                  Property_Result := Classwide_Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.base_interfaces
                  Property_Result := Type_Decl_P_Base_Interfaces
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.find_all_derived_types /usr/src/packages/BUILD/ada/ast.py:4460
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_All_Derived_Types
  (Node : Bare_Base_Type_Decl
      ; Units : Internal_Unit_Array_Access
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_364;
            procedure Finalizer_Scope_1923;
            procedure Finalizer_Scope_1924;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Unit_Array_Access;
Fld_2 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_3 : Internal_Entity_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Type_Decl_Array_Access;
Map_Result : Internal_Entity_Type_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Type_Decl_Array_Access;
            procedure Finalizer_Scope_364 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_364;
            procedure Finalizer_Scope_1923 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1923;
            procedure Finalizer_Scope_1924 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1924;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2524 '<Block>' Let_Result None
--# scope-start
--# expr-start 2523 '<bind(<DynamicVariable origin>, <Var Self>, <mapcat>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:4465
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2522 '<mapcat>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4467
--# expr-start 2518 '<FieldAccess .filter_is_imported_by(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4467
--# expr-start 2517 '<FieldAccess .canonical_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4467
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2517
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Fld.Node, Units => Units, Transitive => True);
--# expr-done 2518
declare
   Map_Result_Vec : Internal_Entity_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Unit_Array_Access := Fld_1;
      begin
         for Item_110 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_110
--# expr-start 2519 '<FieldAccess .root>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4468
      if Item_110 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Item_110.AST_Root;
--# expr-done 2519
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 2521 '<FieldAccess .find_derived_types(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4469
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2520 '<as_bare_entity(<Var r>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4469
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 2520
--# property-call-start BaseTypeDecl.find_derived_types
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Derived_Types (Node => Ent.Node, Root => If_Result, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2521
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1924;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
            Internal_Entity_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1923;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2522
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 2523
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2524
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_364;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_364;
               Finalizer_Scope_1923;
               Finalizer_Scope_1924;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Find_All_Derived_Types;
--# end
--# property-start '[dispatcher]BaseTypeDecl.record_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.record_def
                  Property_Result := Base_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.record_def
                  Property_Result := Base_Subtype_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.record_def
                  Property_Result := Classwide_Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.record_def
                  Property_Result := Type_Decl_P_Record_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Record_Def;
--# end
--# property-start '[dispatcher]BaseTypeDecl.array_def' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.array_def
                  Property_Result := Base_Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.array_def
                  Property_Result := Base_Subtype_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.array_def
                  Property_Result := Type_Decl_P_Array_Def
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.array_def_with_deref /usr/src/packages/BUILD/ada/ast.py:4477
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Array_Type_Def;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
If_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2531 '<Block>' Let_Result None
--# scope-start
--# expr-start 2530 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:4483
--# expr-start 2525 '<FieldAccess .is_array>' Fld /usr/src/packages/BUILD/ada/ast.py:4484
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2525
if Fld then
   --# expr-start 2526 '<FieldAccess .array_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4484
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2526
   If_Result_1 := Fld_1;
else
--# expr-start 2527 '<FieldAccess .is_implicit_deref>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4486
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2527
if Fld_2 then
--# expr-start 2528 '<FieldAccess .accessed_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4487
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2528
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 2529 '<FieldAccess .array_def>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4487
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2529
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
   If_Result := Result_Var;
else
   If_Result := No_Entity_Array_Type_Def;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 2530
Let_Result := If_Result_1; 
--# end
--# expr-done 2531
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.is_array_def_with_deref /usr/src/packages/BUILD/ada/ast.py:4492
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Array_Def_With_Deref
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Is_Null : Boolean;
Not_Val : Boolean;
Fld : Internal_Entity_Array_Type_Def;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2533 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4493
--# expr-start 2532 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4493
Is_Null := Self = null; 
--# expr-done 2532
Not_Val := not (Is_Null); 
--# expr-done 2533
if Not_Val then
   --# expr-start 2536 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4493
--# expr-start 2535 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:4493
--# expr-start 2534 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:4493
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2534
Is_Null_1 := Fld.Node = null; 
--# expr-done 2535
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2536
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Array_Def_With_Deref;
--# end
--# property-start BaseTypeDecl.comp_type /usr/src/packages/BUILD/ada/ast.py:4499
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Is_Subscript : Boolean
         := False
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind is_subscript Is_Subscript
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Ad : Internal_Entity_Array_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Var_Expr_3 : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Base_Type_Access_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Array_Type_Def;
If_Result : Internal_Entity_Array_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Access_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2547 '<Block>' Let_Result_3 None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 2546 '<Let ad>' Let_Result_2 /usr/src/packages/BUILD/ada/ast.py:4509
--# scope-start
--# expr-start 2539 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:4510
if Is_Subscript then
   --# expr-start 2537 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:4511
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2537
   If_Result := Fld;
else
   --# expr-start 2538 '<FieldAccess .array_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4512
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2538
   If_Result := Fld_1;
end if;
--# expr-done 2539
Ad := If_Result; 
--# bind ad Ad
Var_Expr_2 := Ad;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr_2
--# expr-start 2540 '<FieldAccess .comp_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4513
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_2 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2540
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
--# expr-start 2541 '<FieldAccess .access_def>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4514
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2541
Var_Expr_3 := Fld_3;
if Var_Expr_3 /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 2545 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4514
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_3; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind asd Match_0
--# expr-start 2543 '<FieldAccess .return_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4516
--# expr-start 2542 '<FieldAccess .subp_spec>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4516
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 2542
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2543
Let_Result := Fld_5; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tad Match_1
--# expr-start 2544 '<FieldAccess .accessed_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4517
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2544
Let_Result_1 := Fld_6; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 2545
Scope_Result_4 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_2 := Result_Var_1;
end if;
Let_Result_2 := Result_Var_2; 
--# end
--# expr-done 2546
Scope_Result_5 := Let_Result_2;
   --# end
   Result_Var_3 := Scope_Result_5;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Let_Result_3 := Result_Var_3; 
--# end
--# expr-done 2547
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Comp_Type;
--# end
--# property-start BaseTypeDecl.index_type /usr/src/packages/BUILD/ada/ast.py:4524
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Index_Type
  (Node : Bare_Base_Type_Decl
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Array_Type_Def;
Fld : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2550 '<Block>' Let_Result None
--# scope-start
--# expr-start 2548 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:4529
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2548
Var_Expr := Fld;
if Var_Expr /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind ad Var_Expr
--# expr-start 2549 '<FieldAccess .index_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4529
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.index_type
Fld_1 := Libadalang.Implementation.Array_Type_Def_P_Index_Type (Node => Var_Expr.Node, Dim => Dim, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2549
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2550
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Index_Type;
--# end
--# property-start BaseTypeDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:4533
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Expr_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Expr_Type;
--# end
--# property-start BaseTypeDecl.is_derived_type /usr/src/packages/BUILD/ada/ast.py:4537
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Derived_Type
  (Node : Bare_Base_Type_Decl
      ; Other_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other_type Other_Type
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_372;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Classwide_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_6 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Quantifier_Result : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_372 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_372;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2562 '<Block>' Let_Result None
--# scope-start
--# expr-start 2551 '<FieldAccess .canonical_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4541
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2551
Block_Var := Fld; 
--# bind entity_can Block_Var
--# expr-start 2552 '<FieldAccess .canonical_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4542
      if Other_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Other_Type.Node, Origin => Origin, E_Info => Other_Type.Info);
--# end
--# expr-done 2552
Block_Var_1 := Fld_1; 
--# bind other_can Block_Var_1
--# expr-start 2553 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4544
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 2553
if Is_Equiv then
   If_Result_1 := True;
else
--# expr-start 2556 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4545
--# expr-start 2555 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4545
--# expr-start 2554 '<FieldAccess .classwide_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4545
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2554
Is_Null := Fld_2.Node = null; 
--# expr-done 2555
Not_Val := not (Is_Null); 
--# expr-done 2556
if Not_Val then
   --# expr-start 2559 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:4546
--# expr-start 2557 '<FieldAccess .classwide_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4546
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2557
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# expr-start 2558 '<FieldAccess .classwide_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4546
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2558
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 2559
   If_Result := Is_Equiv_1;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
--# expr-start 2560 '<FieldAccess .base_types>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4547
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2560
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_5;
   begin
      for Item_111 of
            Collection.Items
      loop
   --# scope-start
            --# bind bt Item_111
Var_Expr := Item_111;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2561 '<FieldAccess .is_derived_type(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4547
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr.Node, Other_Type => Other_Type, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2561
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
            if Result_Var then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   If_Result_2 := Quantifier_Result;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 2562
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_372;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_372;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Derived_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_iterable_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_iterable_type
                  Property_Result := Base_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_iterable_type
                  Property_Result := Base_Subtype_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_iterable_type
                  Property_Result := Classwide_Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_iterable_type
                  Property_Result := Type_Decl_P_Is_Iterable_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.is_interface_type /usr/src/packages/BUILD/ada/ast.py:4560
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Interface_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Def;
Is_A : Boolean;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Scope_Result_3 : Boolean;
Result_Var : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2569 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 2563 '<FieldAccess .full_view>' Fld /usr/src/packages/BUILD/ada/ast.py:4564
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2563
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2568 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4564
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind td Match_0
--# expr-start 2565 '<IsA InterfaceTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:4565
--# expr-start 2564 '<FieldAccess .type_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4565
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_0.Node.Type_Decl_F_Type_Def, Info => Match_0.Info);
--# expr-done 2564
Is_A := Fld_1.Node /= null 
and then Fld_1.Node.Kind in Ada_Interface_Type_Def_Range; 
--# expr-done 2565
Let_Result := Is_A; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind sb Match_1
--# expr-start 2567 '<FieldAccess .is_interface_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4566
--# expr-start 2566 '<FieldAccess .get_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4566
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 2566
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_interface_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Interface_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 2567
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2568
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := False;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 2569
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Interface_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.iterable_comp_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.iterable_comp_type
                  Property_Result := Base_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.iterable_comp_type
                  Property_Result := Base_Subtype_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.iterable_comp_type
                  Property_Result := Classwide_Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.iterable_comp_type
                  Property_Result := Type_Decl_P_Iterable_Comp_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.matching_prefix_type /usr/src/packages/BUILD/ada/ast.py:4575
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Prefix_Type
  (Node : Bare_Base_Type_Decl
      ; Container_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind container_type Container_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2575 '<Block>' Let_Result None
--# scope-start
Block_Var := Container_Type; 
--# bind cont_type Block_Var
--# expr-start 2570 '<FieldAccess .matching_formal_prim_type(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4584
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2570
if Fld then
   If_Result := True;
else
--# expr-start 2571 '<FieldAccess .final_accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4587
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.final_accessed_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Final_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2571
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2572 '<FieldAccess .matching_formal_prim_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4587
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Var_Expr.Node, Formal_Type => Block_Var, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2572
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 2573 '<FieldAccess .accessed_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4591
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2573
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind at Var_Expr_1
--# expr-start 2574 '<FieldAccess .matching_formal_prim_type(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4592
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Ent.Node, Formal_Type => Var_Expr_1, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2574
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2575
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Prefix_Type;
--# end
--# property-start BaseTypeDecl.matching_access_type /usr/src/packages/BUILD/ada/ast.py:4597
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Anonymous_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Anonymous_Type_Decl;
Match_1_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Anonymous_Type_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Match_Result_1 : Boolean;
Let_Result_4 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2580 '<Block>' Let_Result_4 None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2579 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:4603
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Expected_Type; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_2; 
--# bind atd Match_0
--# expr-start 2576 '<FieldAccess .access_def_matches(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4605
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0.Node, Other => Block_Var, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2576
Let_Result_2 := Fld; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_3; 
--# bind _ Match_1
--# expr-start 2578 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4606
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Block_Var; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Anonymous_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind atd2 Match_0_1
--# expr-start 2577 '<FieldAccess .access_def_matches(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4608
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AnonymousTypeDecl.access_def_matches
Fld_1 := Libadalang.Implementation.Anonymous_Type_Decl_P_Access_Def_Matches (Node => Match_0_1.Node, Other => Expected_Type, For_Assignment => For_Assignment, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 2577
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2578
Let_Result_3 := Match_Result; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
end case;
--# expr-done 2579
Let_Result_4 := Match_Result_1; 
--# end
--# expr-done 2580
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Access_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_prim_type /usr/src/packages/BUILD/ada/ast.py:4614
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Prim_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2582 '<Block>' Let_Result None
--# scope-start
--# expr-start 2581 '<FieldAccess .matching_formal_type_impl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4615
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Accept_Derived => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2581
Let_Result := Fld; 
--# end
--# expr-done 2582
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Prim_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_inverted /usr/src/packages/BUILD/ada/ast.py:4618
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Inverted
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2584 '<Block>' Let_Result None
--# scope-start
--# expr-start 2583 '<FieldAccess .matching_formal_type_impl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4619
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Formal_Type.Node, Formal_Type => Ent, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 2583
Let_Result := Fld; 
--# end
--# expr-done 2584
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type_Inverted;
--# end
--# property-start BaseTypeDecl.matching_formal_type /usr/src/packages/BUILD/ada/ast.py:4622
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2586 '<Block>' Let_Result None
--# scope-start
--# expr-start 2585 '<FieldAccess .matching_formal_type_impl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4623
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type_impl
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type_Impl (Node => Ent.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2585
Let_Result := Fld; 
--# end
--# expr-done 2586
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type;
--# end
--# property-start BaseTypeDecl.matching_formal_type_impl /usr/src/packages/BUILD/ada/ast.py:4626
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Formal_Type_Impl
  (Node : Bare_Base_Type_Decl
      ; Formal_Type : Internal_Entity_Base_Type_Decl
      ; Accept_Derived : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind formal_type Formal_Type
   --# bind accept_derived Accept_Derived
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Boolean;
Fld_1 : Boolean;
If_Result_1 : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_4 : Boolean;
Fld_7 : Boolean;
If_Result_5 : Boolean;
Fld_8 : Boolean;
Fld_9 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_11 : Internal_Entity_Base_Type_Decl;
Fld_12 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Fld_13 : Boolean;
If_Result_11 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2603 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2587 '<FieldAccess .is_classwide>' Fld /usr/src/packages/BUILD/ada/ast.py:4630
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Formal_Type.Node, E_Info => Formal_Type.Info);
--# end
--# expr-done 2587
if Fld then
   If_Result := True;
else
   If_Result := Accept_Derived;
end if;
if If_Result then
   --# expr-start 2588 '<FieldAccess .is_derived_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4631
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Block_Var.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2588
   If_Result_1 := Fld_1;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 2589 '<FieldAccess .is_classwide>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4633
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2589
if Fld_2 then
   --# expr-start 2590 '<FieldAccess .is_derived_type(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4634
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Block_Var.Node, Other_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2590
   If_Result_2 := Fld_3;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_8 := True;
else
--# expr-start 2591 '<FieldAccess .accessed_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4637
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2591
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind actual_access Var_Expr
--# expr-start 2592 '<FieldAccess .accessed_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4639
      if Formal_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Formal_Type.Node, Origin => Origin, E_Info => Formal_Type.Info);
--# end
--# expr-done 2592
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind formal_access Var_Expr_1
--# expr-start 2593 '<FieldAccess .is_classwide>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4641
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2593
if Fld_6 then
   If_Result_4 := True;
else
   If_Result_4 := Accept_Derived;
end if;
if If_Result_4 then
   --# expr-start 2594 '<FieldAccess .is_derived_type(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4642
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr.Node, Other_Type => Var_Expr_1, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2594
   If_Result_5 := Fld_7;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 2595 '<FieldAccess .is_classwide>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4643
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2595
if Fld_8 then
   --# expr-start 2596 '<FieldAccess .is_derived_type(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4644
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => Var_Expr.Node, Other_Type => Var_Expr_1, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2596
   If_Result_6 := Fld_9;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Scope_Result := If_Result_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_8 := Result_Var_1;
end if;
if If_Result_8 then
   If_Result_10 := True;
else
--# expr-start 2599 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4649
--# expr-start 2598 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4649
--# expr-start 2597 '<FieldAccess .get_imp_deref>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:4649
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_10 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2597
Is_Null := Fld_10.Node = null; 
--# expr-done 2598
Not_Val := not (Is_Null); 
--# expr-done 2599
if Not_Val then
   --# expr-start 2601 '<FieldAccess .matching_formal_type(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:4650
--# expr-start 2600 '<FieldAccess .accessed_type>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:4650
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2600
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Fld_11.Node, Formal_Type => Formal_Type, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 2601
   If_Result_9 := Fld_12;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 2602 '<FieldAccess .matching_type(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:4653
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Block_Var.Node, Expected_Type => Formal_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2602
   If_Result_11 := Fld_13;
end if;
Let_Result := If_Result_11; 
--# end
--# expr-done 2603
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Formal_Type_Impl;
--# end
--# property-start BaseTypeDecl.matching_assign_type /usr/src/packages/BUILD/ada/ast.py:4657
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Assign_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_6 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Boolean;
Fld_9 : Boolean;
If_Result_6 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2618 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2604 '<FieldAccess .matching_type(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4660
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Ent.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2604
if Fld then
   If_Result_1 := True;
else
--# expr-start 2605 '<FieldAccess .is_classwide>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4663
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 2605
if Fld_1 then
   --# expr-start 2606 '<FieldAccess .matching_formal_prim_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4664
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_formal_prim_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Prim_Type (Node => Block_Var.Node, Formal_Type => Expected_Type, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2606
   If_Result := Fld_2;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
if If_Result_1 then
   If_Result_3 := True;
else
--# expr-start 2609 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4668
--# expr-start 2608 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4668
--# expr-start 2607 '<FieldAccess .get_imp_deref>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4668
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 2607
Is_Null := Fld_3.Node = null; 
--# expr-done 2608
Not_Val := not (Is_Null); 
--# expr-done 2609
if Not_Val then
   --# expr-start 2611 '<FieldAccess .matching_assign_type(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4669
--# expr-start 2610 '<FieldAccess .accessed_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4669
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2610
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_4.Node, Expected_Type => Expected_Type, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2611
   If_Result_2 := Fld_5;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   If_Result_5 := True;
else
--# expr-start 2614 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4674
--# expr-start 2613 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:4674
--# expr-start 2612 '<FieldAccess .get_imp_deref>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4674
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Expected_Type.Node, E_Info => Expected_Type.Info);
--# end
--# expr-done 2612
Is_Null_1 := Fld_6.Node = null; 
--# expr-done 2613
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2614
if Not_Val_1 then
   --# expr-start 2616 '<FieldAccess .matching_assign_type(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4675
--# expr-start 2615 '<FieldAccess .accessed_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4675
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2615
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Fld_7.Node, Expected_Type => Block_Var, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 2616
   If_Result_4 := Fld_8;
else
   If_Result_4 := False;
end if;
   If_Result_5 := If_Result_4;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 2617 '<FieldAccess .matching_access_type(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4679
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Ent.Node, Expected_Type => Expected_Type, For_Assignment => True, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2617
   If_Result_6 := Fld_9;
end if;
Let_Result := If_Result_6; 
--# end
--# expr-done 2618
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Assign_Type;
--# end
--# property-start BaseTypeDecl.matching_type /usr/src/packages/BUILD/ada/ast.py:4684
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Type
  (Node : Bare_Base_Type_Decl
      ; Expected_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expected_type Expected_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_5 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_7 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Cast_Result_5 : Internal_Entity;
Is_Equiv_4 : Boolean;
Fld_10 : Boolean;
If_Result_8 : Boolean;
If_Result_9 : Boolean;
If_Result_10 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2639 '<Block>' Let_Result None
--# scope-start
Block_Var := Ent; 
--# bind actual_type Block_Var
--# expr-start 2620 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:4690
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 2619 '<FieldAccess .universal_int_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4690
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2619
Is_Equiv := Equivalent (Cast_Result, Fld); 
--# expr-done 2620
if Is_Equiv then
   --# expr-start 2621 '<FieldAccess .is_int_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4691
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2621
   If_Result := Fld_1;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 2623 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:4693
Cast_Expr_1 := Expected_Type; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 2622 '<FieldAccess .universal_int_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4693
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_2 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 2622
Is_Equiv_1 := Equivalent (Cast_Result_1, Fld_2); 
--# expr-done 2623
if Is_Equiv_1 then
   --# expr-start 2624 '<FieldAccess .is_int_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4694
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2624
   If_Result_1 := Fld_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
   If_Result_4 := True;
else
--# expr-start 2626 '<Eq>' Is_Equiv_2 /usr/src/packages/BUILD/ada/ast.py:4696
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# expr-start 2625 '<FieldAccess .universal_real_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4696
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2625
Is_Equiv_2 := Equivalent (Cast_Result_2, Fld_4); 
--# expr-done 2626
if Is_Equiv_2 then
   --# expr-start 2627 '<FieldAccess .is_real_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4697
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2627
   If_Result_3 := Fld_5;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
if If_Result_4 then
   If_Result_6 := True;
else
--# expr-start 2629 '<Eq>' Is_Equiv_3 /usr/src/packages/BUILD/ada/ast.py:4699
Cast_Expr_3 := Expected_Type; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# expr-start 2628 '<FieldAccess .universal_real_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4699
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 2628
Is_Equiv_3 := Equivalent (Cast_Result_3, Fld_6); 
--# expr-done 2629
if Is_Equiv_3 then
   --# expr-start 2630 '<FieldAccess .is_real_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4700
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2630
   If_Result_5 := Fld_7;
else
   If_Result_5 := False;
end if;
   If_Result_6 := If_Result_5;
end if;
if If_Result_6 then
   If_Result_10 := True;
else
--# expr-start 2632 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4702
--# expr-start 2631 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4702
Is_Null := Expected_Type.Node = null; 
--# expr-done 2631
Not_Val := not (Is_Null); 
--# expr-done 2632
if Not_Val then
   --# expr-start 2634 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4703
--# expr-start 2633 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:4703
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 2633
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2634
   If_Result_7 := Not_Val_1;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 2637 '<Eq>' Is_Equiv_4 /usr/src/packages/BUILD/ada/ast.py:4704
--# expr-start 2635 '<FieldAccess .canonical_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4704
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2635
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 2636 '<FieldAccess .canonical_type>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4704
      if Expected_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Expected_Type.Node, Origin => Origin, E_Info => Expected_Type.Info);
--# end
--# expr-done 2636
      Cast_Result_5 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
Is_Equiv_4 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 2637
if Is_Equiv_4 then
   If_Result_8 := True;
else
   --# expr-start 2638 '<FieldAccess .matching_access_type(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:4705
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_access_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Access_Type (Node => Block_Var.Node, Expected_Type => Expected_Type, For_Assignment => False, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 2638
   If_Result_8 := Fld_10;
end if;
   If_Result_9 := If_Result_8;
else
   If_Result_9 := False;
end if;
   If_Result_10 := If_Result_9;
end if;
Let_Result := If_Result_10; 
--# end
--# expr-done 2639
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Type;
--# end
--# property-start BaseTypeDecl.matching_allocator_type /usr/src/packages/BUILD/ada/ast.py:4709
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Matching_Allocator_Type
  (Node : Bare_Base_Type_Decl
      ; Allocated_Type : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind allocated_type Allocated_Type
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2643 '<Block>' Let_Result None
--# scope-start
--# expr-start 2640 '<FieldAccess .is_access_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4711
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2640
if Fld then
   --# expr-start 2642 '<FieldAccess .matching_formal_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4712
      if Allocated_Type.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2641 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4712
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2641
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Allocated_Type.Node, Formal_Type => Fld_1, Origin => Origin, E_Info => Allocated_Type.Info);
--# end
--# expr-done 2642
   If_Result := Fld_2;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 2643
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Matching_Allocator_Type;
--# end
--# property-start '[dispatcher]BaseTypeDecl.canonical_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.canonical_type
                  Property_Result := Base_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.canonical_type
                  Property_Result := Base_Subtype_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.canonical_type
                  Property_Result := Classwide_Type_Decl_P_Canonical_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.base_subtype_or_null /usr/src/packages/BUILD/ada/ast.py:4728
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Subtype_Or_Null
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2645 '<Block>' Let_Result None
--# scope-start
Var_Expr := Ent;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2644 '<FieldAccess .base_subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:4729
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.base_subtype
Fld := Libadalang.Implementation.Base_Type_Decl_P_Base_Subtype (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2644
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2645
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Subtype_Or_Null;
--# end
--# property-start BaseTypeDecl.classwide_type_node /usr/src/packages/BUILD/ada/ast.py:4733
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Classwide_Type_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Classwide_Type_Decl;
Let_Result : Bare_Classwide_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Classwide_Type_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Classwide_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 2648 '<Block>' Let_Result None
--# scope-start
--# expr-start 2647 '<New BareClasswideTypeDecl>' New_Node None
--# expr-start 2646 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:4734
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2646
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Classwide_Type_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Classwide_Type_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Classwide_Type_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 2647
Let_Result := New_Node; 
--# end
--# expr-done 2648
      Property_Result := Let_Result;
   --# end
         Mmz_Val := (Kind => Mmz_Bare_Classwide_Type_Decl,
                     As_Bare_Classwide_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Classwide_Type_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype_node /usr/src/packages/BUILD/ada/ast.py:4739
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype_Node
  (Node : Bare_Base_Type_Decl
  )
   return Bare_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Discrete_Base_Subtype_Decl;
      Fld : Bare_Defining_Name;
New_Node : Bare_Discrete_Base_Subtype_Decl;
Let_Result : Bare_Discrete_Base_Subtype_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Scalar_Base_Subtype_Node,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Bare_Discrete_Base_Subtype_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 2651 '<Block>' Let_Result None
--# scope-start
--# expr-start 2650 '<New BareDiscreteBaseSubtypeDecl>' New_Node /usr/src/packages/BUILD/ada/ast.py:4744
--# expr-start 2649 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:4744
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2649
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Discrete_Base_Subtype_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Discrete_Base_Subtype_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Discrete_Base_Subtype_Decl
     (Self => New_Node,
      Base_Type_Decl_F_Name => Fld);
--# expr-done 2650
Let_Result := New_Node; 
--# end
--# expr-done 2651
      Property_Result := Let_Result;
   --# end
         Mmz_Val := (Kind => Mmz_Bare_Discrete_Base_Subtype_Decl,
                     As_Bare_Discrete_Base_Subtype_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Scalar_Base_Subtype_Node;
--# end
--# property-start BaseTypeDecl.scalar_base_subtype /usr/src/packages/BUILD/ada/ast.py:4747
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Scalar_Base_Subtype
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Discrete_Base_Subtype_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
      Fld : Bare_Discrete_Base_Subtype_Decl;
As_Entity : Internal_Entity_Discrete_Base_Subtype_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
Let_Result : Internal_Entity_Discrete_Base_Subtype_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2654 '<Block>' Let_Result None
--# scope-start
--# expr-start 2653 '<as_entity(<FieldAccess .scalar_base_subtype_node(...)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4752
--# expr-start 2652 '<FieldAccess .scalar_base_subtype_node(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:4752
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype_Node (Node => Self);
--# end
--# expr-done 2652
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Discrete_Base_Subtype_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 2653
Let_Result := If_Result; 
--# end
--# expr-done 2654
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Scalar_Base_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.previous_part' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl | Ada_Incomplete_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start BaseTypeDecl.previous_part
                  Property_Result := Base_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.previous_part
                  Property_Result := Classwide_Type_Decl_P_Previous_Part
                    (Self
                        , Go_To_Incomplete
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.next_part /usr/src/packages/BUILD/ada/ast.py:4776
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Boolean;
Var_Expr_3 : Internal_Entity_Package_Body;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Bare_Declarative_Part;
Fld_4 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_5 : Internal_Entity_Package_Body;
Fld_6 : Internal_Entity_Declarative_Part;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Fld_10 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Incomplete_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Next_Part,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2672 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2671 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4781
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind itd Match_0
--# expr-start 2655 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:4785
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Match_0.Node);
--# end
--# expr-done 2655
Var_Expr_1 := Fld;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind s Var_Expr_1
--# expr-start 2657 '<FieldAccess .find_next_part_in(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4786
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2656 '<as_entity(<Var s>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4786
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 2656
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_1 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => If_Result, E_Info => Match_0.Info);
--# end
--# expr-done 2657
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_1;
else
--# expr-start 2658 '<FieldAccess .is_in_private_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4792
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2658
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= False then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 2663 '<FieldAccess .body_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4794
--# expr-start 2662 '<as_entity(<Cast to BasePackageDecl>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:4794
--# expr-start 2661 '<Cast to BasePackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4794
--# expr-start 2660 '<parent(<FieldAccess .declarative_scope>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4794
--# expr-start 2659 '<FieldAccess .declarative_scope>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4794
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_3 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2659
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 2660
   if Fld_4 = null
      or else Fld_4.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_4;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2661
Is_Null_1 := Cast_Result = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 2662
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_5 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 2663
Var_Expr_3 := Fld_5;
if Var_Expr_3 /= No_Entity_Package_Body then
   --# scope-start
   --# bind p Var_Expr_3
--# expr-start 2665 '<FieldAccess .find_next_part_in(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4796
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2664 '<FieldAccess .decls>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4796
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_3.Node.Package_Body_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 2664
--# property-call-start IncompleteTypeDecl.find_next_part_in
Fld_7 := Libadalang.Implementation.Incomplete_Type_Decl_P_Find_Next_Part_In (Node => Match_0.Node, Decl_Part => Fld_6, E_Info => Match_0.Info);
--# end
--# expr-done 2665
Scope_Result_2 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Scope_Result_3 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_3;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Let_Result := Result_Var_3; 
--# end
Scope_Result_4 := Let_Result;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Classwide_Type_Decl | Ada_Protected_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 2670 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:4800
--# expr-start 2666 '<FieldAccess .is_private>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4801
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2666
if Fld_9 then
   --# expr-start 2668 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4802
--# expr-start 2667 '<FieldAccess .is_generic_formal>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:4802
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_generic_formal
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Generic_Formal (Node => Ent.Node);
--# end
--# expr-done 2667
Not_Val := not (Fld_10); 
--# expr-done 2668
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 2669 '<FieldAccess .private_completion>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4803
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.private_completion
Fld_8 := Libadalang.Implementation.Base_Type_Decl_P_Private_Completion (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2669
   If_Result_3 := Fld_8;
else
   If_Result_3 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 2670
Let_Result_1 := If_Result_3; 
--# end
Scope_Result_5 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 2671
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2672
      Property_Result := Let_Result_2;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Next_Part;
--# end
--# property-start BaseTypeDecl.full_view /usr/src/packages/BUILD/ada/ast.py:4810
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Full_View
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2675 '<Block>' Let_Result None
--# scope-start
--# expr-start 2673 '<FieldAccess .next_part>' Fld /usr/src/packages/BUILD/ada/ast.py:4814
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2673
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind np Var_Expr
--# expr-start 2674 '<FieldAccess .full_view>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4815
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.full_view
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Full_View (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2674
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2675
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Full_View;
--# end
--# property-start BaseTypeDecl.is_definite_subtype /usr/src/packages/BUILD/ada/ast.py:4821
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Definite_Subtype
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Incomplete_Type_Decl;
Match_1 : Internal_Entity_Type_Decl;
Match_2 : Internal_Entity_Subtype_Decl;
Match_3 : Internal_Entity_Classwide_Type_Decl;
Match_4 : Internal_Entity_Task_Type_Decl;
Match_5 : Internal_Entity_Protected_Type_Decl;
Match_6 : Internal_Entity_Base_Type_Decl;
Match_0_1 : Internal_Entity_Derived_Type_Def;
Match_1_1 : Internal_Entity_Array_Type_Def;
Match_2_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Discriminant_Part;
Is_Null : Boolean;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Constraint;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Array_Indices;
Is_A : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Array_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Fld_7 : Internal_Entity_Subtype_Indication;
Fld_8 : Internal_Entity_Constraint;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Boolean;
If_Result_2 : Boolean;
Fld_11 : Internal_Entity_Discriminant_Part;
Is_Null_3 : Boolean;
Fld_12 : Internal_Entity_Discriminant_Part;
Is_Null_4 : Boolean;
Match_Prefix_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Incomplete_Type_Decl;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Type_Decl;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Subtype_Decl;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Classwide_Type_Decl;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_7 : Internal_Entity_Task_Type_Decl;
Let_Result_7 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_8 : Internal_Entity_Protected_Type_Decl;
Let_Result_8 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_9 : Internal_Entity_Base_Type_Decl;
Let_Result_9 : Boolean;
Scope_Result_9 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2699 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 2698 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:4828
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Ent; 
case Ada_Base_Type_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Incomplete_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind _ Match_0
Let_Result_3 := False; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind td Match_1
--# expr-start 2677 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:4830
--# expr-start 2676 '<FieldAccess .discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:4830
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Match_1.Node.Type_Decl_F_Discriminants, Info => Match_1.Info);
--# expr-done 2676
Is_Null := Fld.Node = null; 
--# expr-done 2677
if Is_Null then
   --# expr-start 2687 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4830
--# expr-start 2678 '<FieldAccess .type_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4830
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Match_1.Node.Type_Decl_F_Type_Def, Info => Match_1.Info);
--# expr-done 2678
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind dtd Match_0_1
--# expr-start 2682 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:4832
--# expr-start 2681 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:4832
--# expr-start 2680 '<FieldAccess .constraint>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4832
--# expr-start 2679 '<FieldAccess .subtype_indication>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4832
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Match_0_1.Node.Derived_Type_Def_F_Subtype_Indication, Info => Match_0_1.Info);
--# expr-done 2679
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Constraint (Node => Fld_2.Node.Subtype_Indication_F_Constraint, Info => Fld_2.Info);
--# expr-done 2680
Is_Null_1 := Fld_3.Node = null; 
--# expr-done 2681
Not_Val := not (Is_Null_1); 
--# expr-done 2682
if Not_Val then
   If_Result := True;
else
   --# expr-start 2684 '<FieldAccess .is_definite_subtype>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4833
--# expr-start 2683 '<FieldAccess .base_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4833
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0_1.Node, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 2683
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 2684
   If_Result := Fld_5;
end if;
Let_Result := If_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind atd Match_1_1
--# expr-start 2686 '<IsA ConstrainedArrayIndices>' Is_A /usr/src/packages/BUILD/ada/ast.py:4836
--# expr-start 2685 '<FieldAccess .indices>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4836
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Array_Indices (Node => Match_1_1.Node.Array_Type_Def_F_Indices, Info => Match_1_1.Info);
--# expr-done 2685
Is_A := Fld_6.Node /= null 
and then Fld_6.Node.Kind in Ada_Constrained_Array_Indices_Range; 
--# expr-done 2686
Let_Result_1 := Is_A; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2687
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
Let_Result_4 := If_Result_1; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_5; 
--# bind st Match_2
--# expr-start 2691 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:4840
--# expr-start 2690 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:4840
--# expr-start 2689 '<FieldAccess .constraint>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4840
--# expr-start 2688 '<FieldAccess .subtype>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4840
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Subtype_Indication (Node => Match_2.Node.Subtype_Decl_F_Subtype, Info => Match_2.Info);
--# expr-done 2688
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Constraint (Node => Fld_7.Node.Subtype_Indication_F_Constraint, Info => Fld_7.Info);
--# expr-done 2689
Is_Null_2 := Fld_8.Node = null; 
--# expr-done 2690
Not_Val_1 := not (Is_Null_2); 
--# expr-done 2691
if Not_Val_1 then
   If_Result_2 := True;
else
   --# expr-start 2693 '<FieldAccess .is_definite_subtype>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:4841
--# expr-start 2692 '<FieldAccess .get_type>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4841
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_9 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_2.Node, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 2692
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_definite_subtype
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Is_Definite_Subtype (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 2693
   If_Result_2 := Fld_10;
end if;
Let_Result_5 := If_Result_2; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Classwide_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_4 := Cast_Result_7; 
--# bind ttd Match_4
--# expr-start 2695 '<IsNull>' Is_Null_3 /usr/src/packages/BUILD/ada/ast.py:4843
--# expr-start 2694 '<FieldAccess .discriminants>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:4843
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Discriminant_Part (Node => Match_4.Node.Task_Type_Decl_F_Discriminants, Info => Match_4.Info);
--# expr-done 2694
Is_Null_3 := Fld_11.Node = null; 
--# expr-done 2695
Let_Result_7 := Is_Null_3; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_7; 
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_5 := Cast_Result_8; 
--# bind ptd Match_5
--# expr-start 2697 '<IsNull>' Is_Null_4 /usr/src/packages/BUILD/ada/ast.py:4844
--# expr-start 2696 '<FieldAccess .discriminants>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:4844
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Discriminant_Part (Node => Match_5.Node.Protected_Type_Decl_F_Discriminants, Info => Match_5.Info);
--# expr-done 2696
Is_Null_4 := Fld_12.Node = null; 
--# expr-done 2697
Let_Result_8 := Is_Null_4; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Discrete_Base_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_9 := True; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_9; 
end case;
--# expr-done 2698
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 2699
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Definite_Subtype;
--# end
--# property-start '[dispatcher]BaseTypeDecl.is_private' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Incomplete_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.is_private
                  Property_Result := Base_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.is_private
                  Property_Result := Base_Subtype_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.is_private
                  Property_Result := Classwide_Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.is_private
                  Property_Result := Type_Decl_P_Is_Private
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Is_Private;
--# end
--# property-start '[dispatcher]BaseTypeDecl.discriminants_list' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Discriminants_List
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.discriminants_list
                  Property_Result := Base_Subtype_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Classwide_Type_Decl =>
                  --# property-call-start ClasswideTypeDecl.discriminants_list
                  Property_Result := Classwide_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Incomplete_Type_Decl .. Ada_Incomplete_Tagged_Type_Decl =>
                  --# property-call-start IncompleteTypeDecl.discriminants_list
                  Property_Result := Incomplete_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Protected_Type_Decl =>
                  --# property-call-start ProtectedTypeDecl.discriminants_list
                  Property_Result := Protected_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start TaskTypeDecl.discriminants_list
                  Property_Result := Task_Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.discriminants_list
                  Property_Result := Type_Decl_P_Discriminants_List
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Discriminants_List;
--# end
--# property-start '[dispatcher]BaseTypeDecl.root_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Type_Decl (Self.Kind) is
               when Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
                  --# property-call-start BaseTypeDecl.root_type
                  Property_Result := Base_Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Discrete_Base_Subtype_Decl .. Ada_Subtype_Decl =>
                  --# property-call-start BaseSubtypeDecl.root_type
                  Property_Result := Base_Subtype_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
                  --# property-call-start TypeDecl.root_type
                  Property_Result := Type_Decl_P_Root_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Type_Decl_P_Root_Type;
--# end
--# property-start BaseTypeDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:4875
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Base_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2703 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:4875
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind ttd Match_0
--# expr-start 2700 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:4876
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 2700
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
--# expr-start 2702 '<Cast to BasicDecl.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4877
--# expr-start 2701 '<FieldAccess .next_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4877
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.next_part
Fld_1 := Libadalang.Implementation.Base_Type_Decl_P_Next_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2701
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
--# expr-done 2702
Let_Result_1 := Cast_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2703
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BaseTypeDecl.shapes /usr/src/packages/BUILD/ada/ast.py:4882
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Shapes
  (Node : Bare_Base_Type_Decl
      ; Include_Discriminants : Boolean
         := True
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Shape_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind include_discriminants Include_Discriminants
   --# bind origin Origin
   Property_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_398;
            procedure Finalizer_Scope_1973;
            procedure Finalizer_Scope_1974;
            procedure Finalizer_Scope_1975;
            procedure Finalizer_Scope_1976;
      Block_Var : Internal_Entity_Base_Record_Def;
Block_Var_1 : Internal_Entity_Component_List;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Record_Def;
Block_Var_4 : Internal_Shape_Array_Access;
Block_Var_5 : Internal_Shape_Array_Access;
Block_Var_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Derived_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Record_Def;
Fld : Internal_Entity_Base_Record_Def;
Fld_1 : Internal_Entity_Component_List;
Fld_2 : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Result_Var_1 : Internal_Entity_Base_Record_Def;
Fld_5 : Internal_Shape_Array_Access;
Fld_6 : Internal_Shape_Array_Access;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_9 : Internal_Discriminant_Values_Array_Access;
Fld_10 : Internal_Discriminant_Values_Array_Access;
Concat_Result_1 : Internal_Discriminant_Values_Array_Access;
New_Struct : Internal_Shape := No_Shape;
Map_Result : Internal_Shape_Array_Access;
Map_Result_1 : Internal_Shape_Array_Access;
Scope_Result_2 : Internal_Shape_Array_Access;
Result_Var_2 : Internal_Shape_Array_Access;
Fld_11 : Internal_Entity_Type_Decl;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_13 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Concat_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_14 : Internal_Discriminant_Values_Array_Access;
New_Struct_1 : Internal_Shape := No_Shape;
Map_Result_2 : Internal_Shape_Array_Access;
If_Result : Internal_Shape_Array_Access;
Let_Result : Internal_Shape_Array_Access;
            procedure Finalizer_Scope_398 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_398;
            procedure Finalizer_Scope_1973 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_1973;
            procedure Finalizer_Scope_1974 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1974;
            procedure Finalizer_Scope_1975 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1975;
            procedure Finalizer_Scope_1976 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Concat_Result_2);
                     Dec_Ref (Fld_14);
                     Dec_Ref (New_Struct_1);
            end Finalizer_Scope_1976;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2729 '<Block>' Let_Result None
--# scope-start
--# expr-start 2704 '<FieldAccess .record_def>' Fld /usr/src/packages/BUILD/ada/ast.py:4936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2704
Block_Var := Fld; 
--# bind rdef Block_Var
--# expr-start 2705 '<FieldAccess .components>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4937
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Component_List (Node => Block_Var.Node.Base_Record_Def_F_Components, Info => Block_Var.Info);
--# expr-done 2705
Block_Var_1 := Fld_1; 
--# bind comps Block_Var_1
--# expr-start 2707 '<Cast to DerivedTypeDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4939
--# expr-start 2706 '<FieldAccess .type_def>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4939
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_def
Fld_2 := Libadalang.Implementation.Component_List_P_Type_Def (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2706
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Derived_Type_Def;
   end if;
--# expr-done 2707
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Derived_Type_Def then
   --# scope-start
--# expr-start 2708 '<FieldAccess .base_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4939
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2708
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_2 := Result_Var; 
--# bind parent_type Block_Var_2
Var_Expr_1 := Block_Var_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2709 '<FieldAccess .record_def>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4941
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2709
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Record_Def;
end if;
Block_Var_3 := Result_Var_1; 
--# bind parent_record Block_Var_3
--# expr-start 2710 '<FieldAccess .shapes>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4943
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.shapes
Fld_5 := Libadalang.Implementation.Component_List_P_Shapes (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2710
Block_Var_4 := Fld_5; Inc_Ref (Block_Var_4);
--# bind own_shapes Block_Var_4
Var_Expr_2 := Block_Var_3;
if Var_Expr_2 /= No_Entity_Base_Record_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 2720 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:4948
--# expr-start 2711 '<FieldAccess .shapes(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:4948
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.shapes
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Shapes (Node => Block_Var_2.Node, Include_Discriminants => False, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 2711
declare
   Map_Result_1_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Fld_6;
      begin
         for Item_113 of
               Collection.Items
         loop
   --# scope-start
               --# bind parent_shape Item_113
      --# expr-start 2719 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4949
declare
   Map_Result_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_4;
      begin
         for Item_114 of
               Collection.Items
         loop
   --# scope-start
               --# bind own_shape Item_114
      --# expr-start 2718 '<New InternalShape>' New_Struct /usr/src/packages/BUILD/ada/ast.py:4950
--# expr-start 2714 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:4951
--# expr-start 2712 '<FieldAccess .components>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:4951
Fld_7 := Item_113.Components;
Inc_Ref (Fld_7);
--# expr-done 2712
--# expr-start 2713 '<FieldAccess .components>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:4952
Fld_8 := Item_114.Components;
Inc_Ref (Fld_8);
--# expr-done 2713
Concat_Result := Concat (Fld_7, Fld_8); 
--# expr-done 2714
--# expr-start 2717 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:4954
--# expr-start 2715 '<FieldAccess .discriminants_values>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:4954
Fld_9 := Item_113.Discriminants_Values;
Inc_Ref (Fld_9);
--# expr-done 2715
--# expr-start 2716 '<FieldAccess .discriminants_values>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:4956
Fld_10 := Item_114.Discriminants_Values;
Inc_Ref (Fld_10);
--# expr-done 2716
Concat_Result_1 := Concat (Fld_9, Fld_10); 
--# expr-done 2717
Inc_Ref (Concat_Result);
Inc_Ref (Concat_Result_1);
New_Struct := (Components => Concat_Result, Discriminants_Values => Concat_Result_1); 
--# expr-done 2718
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1975;
         end loop;
      end;
      Map_Result := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2719
         for Item_To_Append of
               Map_Result.Items
         loop
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1974;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2720
Scope_Result_2 := Map_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1973;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var_4;
end if;
      Inc_Ref (Result_Var_2);
Block_Var_5 := Result_Var_2; Inc_Ref (Block_Var_5);
--# bind all_shapes Block_Var_5
--# expr-start 2722 '<FieldAccess .discriminants_list>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:4964
--# expr-start 2721 '<FieldAccess .type_decl>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:4964
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_11 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2721
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_12 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 2722
Block_Var_6 := Fld_12; Inc_Ref (Block_Var_6);
--# bind discrs Block_Var_6
--# expr-start 2728 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:4965
if Include_Discriminants then
   --# expr-start 2727 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:4967
declare
   Map_Result_2_Vec : Internal_Shape_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Shape_Array_Access := Block_Var_5;
      begin
         for Item_115 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_115
      --# expr-start 2726 '<New InternalShape>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:4968
--# expr-start 2724 '<Concat>' Concat_Result_2 /usr/src/packages/BUILD/ada/ast.py:4969
--# expr-start 2723 '<FieldAccess .components>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:4969
Fld_13 := Item_115.Components;
Inc_Ref (Fld_13);
--# expr-done 2723
Concat_Result_2 := Concat (Block_Var_6, Fld_13); 
--# expr-done 2724
--# expr-start 2725 '<FieldAccess .discriminants_values>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:4970
Fld_14 := Item_115.Discriminants_Values;
Inc_Ref (Fld_14);
--# expr-done 2725
Inc_Ref (Concat_Result_2);
Inc_Ref (Fld_14);
New_Struct_1 := (Components => Concat_Result_2, Discriminants_Values => Fld_14); 
--# expr-done 2726
         declare
            Item_To_Append : constant Internal_Shape :=
               New_Struct_1;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Shape_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1976;
         end loop;
      end;
      Map_Result_2 := Create_Internal_Shape_Array
        (Items_Count => Natural (Internal_Shape_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Shape_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Shape_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Shape_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 2727
   If_Result := Map_Result_2;
else
   If_Result := Block_Var_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 2728
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 2729
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_398;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_398;
               Finalizer_Scope_1973;
               Finalizer_Scope_1974;
               Finalizer_Scope_1975;
               Finalizer_Scope_1976;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Shapes;
--# end
--# property-start BaseTypeDecl.dottable_subps_env /usr/src/packages/BUILD/ada/ast.py:4977
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_F_Dottable_Subps_Env
  (Node : Bare_Base_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_399;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_399 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_399;
begin
   --# property-body-start
      if Self.LF_Present_Dottable_Subps_Env then
         Property_Result := Self.LF_Stg_Dottable_Subps_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2731 '<Block>' Let_Result None
--# scope-start
--# expr-start 2730 '<langkit.expressions.envs.DynamicLexicalEnv object at 0xf5d5dce8>' Dyn_Env /usr/src/packages/BUILD/ada/ast.py:4982
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Base_Type_Decl_P_Dottable_Subps'Access, null, False); 
--# expr-done 2730
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2731
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_399;
      Self.LF_Present_Dottable_Subps_Env := True;
      Self.LF_Stg_Dottable_Subps_Env := Property_Result;
         Inc_Ref (Property_Result);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_399;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_F_Dottable_Subps_Env;
--# end
--# property-start BaseTypeDecl.dottable_subps /usr/src/packages/BUILD/ada/ast.py:4988
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Dottable_Subps
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_401;
            procedure Finalizer_Scope_1982;
            procedure Finalizer_Scope_1985;
            procedure Finalizer_Scope_1987;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Base_Package_Decl;
Var_Expr : Bare_Package_Body;
Var_Expr_1 : Bare_Declarative_Part;
Var_Expr_2 : Bare_Declarative_Part;
Var_Expr_3 : Internal_Entity_Package_Body;
Var_Expr_4 : Internal_Entity_Declarative_Part;
Bd : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Base_Subp_Spec;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
Cast_Result : Bare_Package_Body;
Bound_Imprecise_Fallback : Boolean;
As_Entity : Internal_Entity_Package_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Package_Body;
Fld_2 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Fld_3 : Bare_Ada_Node;
Scope_Result_2 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Base_Package_Decl;
As_Entity_1 : Internal_Entity_Base_Package_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_2 : Internal_Entity_Base_Package_Decl;
Fld_4 : Internal_Entity_Public_Part;
Cast_Result_3 : Internal_Entity_Declarative_Part;
Fld_5 : Internal_Entity_Private_Part;
Cast_Result_4 : Internal_Entity_Declarative_Part;
Fld_6 : Internal_Entity_Package_Body;
Fld_7 : Internal_Entity_Declarative_Part;
Scope_Result_3 : Internal_Entity_Declarative_Part;
Result_Var_3 : Internal_Entity_Declarative_Part;
Array_Lit : Internal_Entity_Declarative_Part_Array_Access;
Fld_8 : Internal_Entity_Ada_Node_List;
Scope_Result_4 : Internal_Entity_Ada_Node_List;
Result_Var_4 : Internal_Entity_Ada_Node_List;
Fld_9 : Bare_Ada_Node_List;
Fld_10 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Item_273 : Internal_Entity;
Bare_Item_273 : Bare_Ada_Node;
Map_Result : Internal_Entity_Array_Access;
Map_Result_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Defining_Name;
Fld_12 : Symbol_Type;
Fld_13 : Bare_Basic_Decl;
New_Struct : Internal_Metadata;
Cast_Result_6 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Let_Result : Internal_Inner_Env_Assoc;
Cast_Expr_1 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity_Base_Subp_Spec;
Scope_Result_5 : Internal_Entity_Base_Subp_Spec;
Result_Var_5 : Internal_Entity_Base_Subp_Spec;
Fld_15 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_9 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Map_Result_2 : Internal_Inner_Env_Assoc_Array_Access;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_401 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_401;
            procedure Finalizer_Scope_1982 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1982;
            procedure Finalizer_Scope_1985 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
            end Finalizer_Scope_1985;
            procedure Finalizer_Scope_1987 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_1987;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Dottable_Subps,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2766 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 2732 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:4994
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 2732
Block_Var := Fld; 
--# bind scope Block_Var
--# expr-start 2734 '<Cast to PackageBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4996
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2733 '<parent(<Var Var_Expr_1>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4996
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr_1.Parent; 
--# expr-done 2733
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
   if Result_Var = null
      or else Result_Var.Kind in Ada_Package_Body_Range
   then
      Cast_Result := Result_Var;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2734
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind body Var_Expr
--# expr-start 2738 '<Cast to BasePackageDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:4997
--# expr-start 2737 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .previous_part>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:4997
Bound_Imprecise_Fallback := False; 
--# expr-start 2736 '<FieldAccess .previous_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4999
--# expr-start 2735 '<as_entity(<Var body>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4999
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Package_Body;
else
As_Entity := (Info => E_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 2735
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Previous_Part (Node => If_Result.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 2736
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 2737
   if Dyn_Var_Bind_Result.Node = null
      or else Dyn_Var_Bind_Result.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Package_Decl
        (Node => Dyn_Var_Bind_Result.Node,
         Info => Dyn_Var_Bind_Result.Info);
   else
         Cast_Result_1 := No_Entity_Base_Package_Decl;
   end if;
--# expr-done 2738
Scope_Result_1 := Cast_Result_1;
   --# end
   Result_Var_2 := Scope_Result_1;
else
   --# expr-start 2741 '<as_entity(<Cast to BasePackageDecl>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:5001
--# expr-start 2740 '<Cast to BasePackageDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:5001
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 2739 '<parent(<Var Var_Expr_2>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5001
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr_2.Parent; 
--# expr-done 2739
Scope_Result_2 := Fld_3;
   --# end
   Result_Var_1 := Scope_Result_2;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
   if Result_Var_1 = null
      or else Result_Var_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result_2 := Result_Var_1;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 2740
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Package_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 2741
   Result_Var_2 := If_Result_1;
end if;
Block_Var_1 := Result_Var_2; 
--# bind pkg Block_Var_1
--# expr-start 2765 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:5004
--# expr-start 2742 '<IsNull>' Is_Null_3 /usr/src/packages/BUILD/ada/ast.py:5005
Is_Null_3 := Block_Var_1.Node = null; 
--# expr-done 2742
if Is_Null_3 then
   If_Result_3 := No_Internal_Inner_Env_Assoc_Array_Type;
else
   --# expr-start 2764 '<filter_map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:5008
--# expr-start 2752 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:5008
--# expr-start 2749 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:5008
--# expr-start 2744 '<Cast to DeclarativePart>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:5009
--# expr-start 2743 '<FieldAccess .public_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5009
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Public_Part (Node => Block_Var_1.Node.Base_Package_Decl_F_Public_Part, Info => Block_Var_1.Info);
--# expr-done 2743
      Cast_Result_3 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-done 2744
--# expr-start 2746 '<Cast to DeclarativePart>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:5010
--# expr-start 2745 '<FieldAccess .private_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5010
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Private_Part (Node => Block_Var_1.Node.Base_Package_Decl_F_Private_Part, Info => Block_Var_1.Info);
--# expr-done 2745
      Cast_Result_4 := Create_Internal_Entity_Declarative_Part
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# expr-done 2746
--# expr-start 2747 '<FieldAccess .body_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5011
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_6 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 2747
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Package_Body then
   --# scope-start
--# expr-start 2748 '<FieldAccess .decls>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5011
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Declarative_Part (Node => Var_Expr_3.Node.Package_Body_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 2748
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Declarative_Part;
end if;
Array_Lit := Create_Internal_Entity_Declarative_Part_Array (Internal_Internal_Entity_Declarative_Part_Array'(1 => Cast_Result_3, 2 => Cast_Result_4, 3 => Result_Var_3)); 
--# expr-done 2749
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Declarative_Part_Array_Access := Array_Lit;
      begin
         for Item_117 of
               Collection.Items
         loop
   --# scope-start
               --# bind dp Item_117
      --# expr-start 2751 '<take_while>' Map_Result None
Var_Expr_4 := Item_117;
if Var_Expr_4 /= No_Entity_Declarative_Part then
   --# scope-start
--# expr-start 2750 '<FieldAccess .decls>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5013
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_4.Node.Declarative_Part_F_Decls, Info => Var_Expr_4.Info);
--# expr-done 2750
Scope_Result_4 := Fld_8;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Ada_Node_List;
end if;
Fld_9 := Result_Var_4.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_9 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_9;
      begin
         for Untyped_Item_273 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_273 := Untyped_Item_273; 
Is_Null_2 := Bare_Item_273 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_10 := Result_Var_4.Info;
As_Entity_2 := (Info => Fld_10, Node => Bare_Item_273); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_273 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_273;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2751
         for Item_To_Append of
               Map_Result.Items
         loop
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_1982;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 2752
declare
   Map_Result_2_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result_1;
      begin
         for Item_116 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_116
--# expr-start 2753 '<Cast to BasicDecl>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:5024
Cast_Expr_1 := Item_116; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2753
Var_Expr_6 := Cast_Result_7;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 2754 '<FieldAccess .subp_spec_or_null>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:5024
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 2754
Scope_Result_5 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_5 := Result_Var_5;
if Var_Expr_5 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 2755 '<FieldAccess .dottable_subp_of>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:5024
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.dottable_subp_of
Fld_15 := Libadalang.Implementation.Base_Subp_Spec_P_Dottable_Subp_Of (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 2755
Scope_Result_6 := Fld_15;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_1987;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Result_Var_6;
   begin
      for Item_118 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_118
         --# expr-start 2756 '<Eq>' Is_Equiv None
Cast_Expr_2 := Item_118; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Cast_Expr_3 := Ent; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 2756
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
      --# expr-start 2763 '<Let bd>' Let_Result /usr/src/packages/BUILD/ada/ast.py:5015
--# scope-start
--# expr-start 2757 '<Cast to BasicDecl>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:5016
Cast_Expr := Item_116; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Basic_Decl;
   end if;
--# expr-done 2757
Bd := Cast_Result_5; 
--# bind bd Bd
--# expr-start 2762 '<New InternalInnerEnvAssoc>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:5016
--# expr-start 2759 '<FieldAccess .name_symbol>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:5017
--# expr-start 2758 '<FieldAccess .defining_name>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5017
      if Bd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Bd.Node, E_Info => Bd.Info);
--# end
--# expr-done 2758
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_11.Node);
--# end
--# expr-done 2759
--# expr-start 2760 '<New InternalMetadata>' New_Struct /usr/src/packages/BUILD/ada/ast.py:5019
New_Struct := (Access_Entity => False, Dottable_Subp => True, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 2760
--# expr-start 2761 '<FieldAccess .node>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:5018
Fld_13 := Bd.Node;
--# expr-done 2761
      Cast_Result_6 := Fld_13;
New_Struct_1 := (Key => Fld_12, Metadata => New_Struct, Val => Cast_Result_6); 
--# expr-done 2762
Let_Result := New_Struct_1; 
--# end
--# expr-done 2763
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc :=
               Let_Result;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_1985;
         end loop;
      end;
      Map_Result_2 := Create_Internal_Inner_Env_Assoc_Array
        (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 2764
   If_Result_3 := Map_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 2765
Let_Result_1 := If_Result_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 2766
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_401;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                     As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_401;
               Finalizer_Scope_1982;
               Finalizer_Scope_1985;
               Finalizer_Scope_1987;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Dottable_Subps;
--# end
--# property-start BaseTypeDecl.find_base_type_rebindings /usr/src/packages/BUILD/ada/ast.py:5032
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Find_Base_Type_Rebindings
  (Node : Bare_Base_Type_Decl
      ; Target : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Env_Rebindings
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind target Target
   --# bind origin Origin
   Property_Result : Env_Rebindings;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Env_Rebindings;
Scope_Result : Env_Rebindings;
Result_Var : Env_Rebindings;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Base_Type_Decl;
Is_Equal : Boolean;
If_Result : Env_Rebindings;
Let_Result : Env_Rebindings;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Type_Decl_Base_Type_Decl_P_Find_Base_Type_Rebindings,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Type_Decl,
                                As_Bare_Base_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Base_Type_Decl,
                                      As_Bare_Base_Type_Decl => Target);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Env_Rebindings;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 2773 '<Block>' Let_Result None
--# scope-start
--# expr-start 2772 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5049
--# expr-start 2767 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:5050
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Target; 
--# expr-done 2767
if Is_Equal then
   --# expr-start 2769 '<FieldAccess .rebindings>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5051
--# expr-start 2768 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:5051
Fld := Ent.Info;
--# expr-done 2768
Fld_1 := Fld.Rebindings;
--# expr-done 2769
   If_Result := Fld_1;
else
--# expr-start 2770 '<FieldAccess .base_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5052
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2770
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 2771 '<FieldAccess .find_base_type_rebindings(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5053
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.find_base_type_rebindings
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Find_Base_Type_Rebindings (Node => Var_Expr.Node, Target => Target, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2771
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := null;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 2772
Let_Result := If_Result; 
--# end
--# expr-done 2773
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Env_Rebindings,
                     As_Env_Rebindings => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Find_Base_Type_Rebindings;
--# end
--# property-start BaseTypeDecl._env_mappings_2 /usr/src/packages/BUILD/ada/ast.py:4028
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_2
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1171;
      Fld : Symbol_Type;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1171 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1171;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2775 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:4028
--# expr-start 2774 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:4028
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2774
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2775
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1171;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1171;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_2;
--# end
--# property-start BaseTypeDecl.primitives_env /usr/src/packages/BUILD/ada/ast.py:4152
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Primitives_Env
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_325;
      Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_325 is
            begin
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_325;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2776 '<Block>' Let_Result None
--# scope-start
Let_Result := AST_Envs.Empty_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 2776
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_325;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_325;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Primitives_Env;
--# end
--# property-start BaseTypeDecl.is_record_type /usr/src/packages/BUILD/ada/ast.py:4156
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Record_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2777 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2777
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Record_Type;
--# end
--# property-start BaseTypeDecl.is_task_type /usr/src/packages/BUILD/ada/ast.py:4185
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Task_Type
  (Node : Bare_Base_Type_Decl
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Task_Type;
--# end
--# property-start BaseTypeDecl.is_real_type /usr/src/packages/BUILD/ada/ast.py:4187
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Real_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Real_Type;
--# end
--# property-start BaseTypeDecl.is_float_type /usr/src/packages/BUILD/ada/ast.py:4191
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Float_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Float_Type;
--# end
--# property-start BaseTypeDecl.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:4195
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseTypeDecl.is_enum_type /usr/src/packages/BUILD/ada/ast.py:4200
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseTypeDecl.is_classwide /usr/src/packages/BUILD/ada/ast.py:4205
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Classwide
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Classwide;
--# end
--# property-start BaseTypeDecl.is_access_type /usr/src/packages/BUILD/ada/ast.py:4207
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Access_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Access_Type;
--# end
--# property-start BaseTypeDecl.has_ud_indexing /usr/src/packages/BUILD/ada/ast.py:4219
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseTypeDecl.constant_indexing_fns /usr/src/packages/BUILD/ada/ast.py:4223
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.variable_indexing_fns /usr/src/packages/BUILD/ada/ast.py:4230
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Basic_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start BaseTypeDecl.get_imp_deref /usr/src/packages/BUILD/ada/ast.py:4238
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Expr;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start BaseTypeDecl.access_def /usr/src/packages/BUILD/ada/ast.py:4243
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Access_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Access_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Access_Def;
--# end
--# property-start BaseTypeDecl.is_char_type /usr/src/packages/BUILD/ada/ast.py:4245
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Char_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Char_Type;
--# end
--# property-start BaseTypeDecl.classwide_type /usr/src/packages/BUILD/ada/ast.py:4253
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Classwide_Type
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
      Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Bare_Classwide_Type_Decl;
As_Entity : Internal_Entity_Classwide_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Classwide_Type_Decl;
Fld_1 : Boolean;
If_Result_1 : Internal_Entity_Classwide_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2782 '<bind(<DynamicVariable origin>, <Var Self>, <If>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:4253
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 2781 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:4253
--# expr-start 2778 '<FieldAccess .is_tagged_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4254
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2778
if Fld_1 then
   --# expr-start 2780 '<as_entity(<FieldAccess .classwide_type_node>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:4255
--# expr-start 2779 '<FieldAccess .classwide_type_node>' Fld /usr/src/packages/BUILD/ada/ast.py:4255
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.classwide_type_node
Fld := Libadalang.Implementation.Base_Type_Decl_P_Classwide_Type_Node (Node => Self);
--# end
--# expr-done 2779
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Classwide_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 2780
   If_Result_1 := If_Result;
else
   If_Result_1 := No_Entity_Classwide_Type_Decl;
end if;
--# expr-done 2781
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 2782
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Classwide_Type;
--# end
--# property-start BaseTypeDecl.discrete_range /usr/src/packages/BUILD/ada/ast.py:4298
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Discrete_Range
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2783 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 2783
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Discrete_Range;
--# end
--# property-start BaseTypeDecl.is_discrete_type /usr/src/packages/BUILD/ada/ast.py:4364
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
Fld_2 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2787 '<Block>' Let_Result None
--# scope-start
--# expr-start 2784 '<FieldAccess .is_int_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4368
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2784
if Fld then
   If_Result := True;
else
   --# expr-start 2785 '<FieldAccess .is_enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4368
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2785
   If_Result := Fld_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2786 '<FieldAccess .is_char_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4368
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2786
   If_Result_1 := Fld_2;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 2787
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseTypeDecl.is_int_type /usr/src/packages/BUILD/ada/ast.py:4371
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Int_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2788 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 2788
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Int_Type;
--# end
--# property-start BaseTypeDecl.accessed_type /usr/src/packages/BUILD/ada/ast.py:4383
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Accessed_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2789 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 2789
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Accessed_Type;
--# end
--# property-start BaseTypeDecl.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:4435
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseTypeDecl.base_type /usr/src/packages/BUILD/ada/ast.py:4440
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Type;
--# end
--# property-start BaseTypeDecl.base_types /usr/src/packages/BUILD/ada/ast.py:4446
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Types
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_362;
            procedure Finalizer_Scope_1922;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_362 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_362;
            procedure Finalizer_Scope_1922 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1922;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2792 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:4447
--# expr-start 2790 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:4447
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2790
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1922;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 2791 '<FieldAccess .base_interfaces>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2791
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 2792
      Property_Result := Concat_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_362;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_362;
               Finalizer_Scope_1922;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Types;
--# end
--# property-start BaseTypeDecl.base_interfaces /usr/src/packages/BUILD/ada/ast.py:4454
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Base_Interfaces
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Base_Interfaces;
--# end
--# property-start BaseTypeDecl.record_def /usr/src/packages/BUILD/ada/ast.py:4473
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Record_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Record_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Record_Def;
--# end
--# property-start BaseTypeDecl.array_def /usr/src/packages/BUILD/ada/ast.py:4474
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Array_Def
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Array_Type_Def;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Array_Def;
--# end
--# property-start BaseTypeDecl.is_iterable_type /usr/src/packages/BUILD/ada/ast.py:4550
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseTypeDecl.iterable_comp_type /usr/src/packages/BUILD/ada/ast.py:4571
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2793 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 2793
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseTypeDecl.canonical_type /usr/src/packages/BUILD/ada/ast.py:4717
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Canonical_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2797 '<Block>' Let_Result None
--# scope-start
--# expr-start 2796 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to BaseTypeDecl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:4722
Bound_Imprecise_Fallback := False; 
--# expr-start 2795 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:4723
--# expr-start 2794 '<FieldAccess .canonical_part>' Fld /usr/src/packages/BUILD/ada/ast.py:4723
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2794
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2795
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 2796
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2797
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Canonical_Type;
--# end
--# property-start BaseTypeDecl.previous_part /usr/src/packages/BUILD/ada/ast.py:4757
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Previous_Part
  (Node : Bare_Base_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1948;
            procedure Finalizer_Scope_1949;
      Var_Expr : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Defining_Name;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Cast_Expr : Bare_Base_Type_Decl;
Cast_Result : Bare_Ada_Node;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1948 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1948;
            procedure Finalizer_Scope_1949 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1949;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2809 '<Block>' Let_Result None
--# scope-start
--# expr-start 2798 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:4761
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Base_Type_Decl_F_Name;
--# expr-done 2798
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind type_name Var_Expr
--# expr-start 2808 '<Cast to BaseTypeDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:4764
--# expr-start 2801 '<FieldAccess .env_get(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:4764
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2799 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:4764
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2799
--# expr-start 2800 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:4764
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr);
--# end
--# expr-done 2800
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_1, Symbol => Fld_2, From_Node => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 2801
Var_Expr_1 := Fld_3;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind previous_parts Var_Expr_1
--# expr-start 2807 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:4766
--# expr-start 2806 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:4766
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_112 of
               Collection.Items
         loop
   --# scope-start
               --# bind pp Item_112
--# expr-start 2802 '<FieldAccess .is_in_private_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:4767
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2802
if Fld_4 then
--# expr-start 2803 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:4768
Cast_Expr_1 := Item_112; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2803
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 2804 '<FieldAccess .is_private>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:4768
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 2804
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_2 := True;
else
if Go_To_Incomplete then
   --# expr-start 2805 '<IsA IncompleteTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:4770
Is_A := Item_112.Node /= null 
and then Item_112.Node.Kind in Ada_Incomplete_Type_Decl_Range; 
--# expr-done 2805
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
               if If_Result_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_112;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 2806
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2807
Scope_Result_1 := Get_Result;
   --# end
      Finalizer_Scope_1949;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
   if Result_Var_1.Node = null
      or else Result_Var_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2808
Scope_Result_2 := Cast_Result_2;
   --# end
      Finalizer_Scope_1948;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 2809
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1948;
               Finalizer_Scope_1949;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Previous_Part;
--# end
--# property-start BaseTypeDecl.is_private /usr/src/packages/BUILD/ada/ast.py:4848
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Is_Private
  (Node : Bare_Base_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Is_Private;
--# end
--# property-start BaseTypeDecl.root_type /usr/src/packages/BUILD/ada/ast.py:4865
pragma Warnings (Off, "is not referenced");
 function Base_Type_Decl_P_Root_Type
  (Node : Bare_Base_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Type_Decl := Bare_Base_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Type_Decl :=
     Internal_Entity_Base_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Type_Decl_P_Root_Type;
--# end
function Base_Type_Decl_P_Dottable_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Internal_Inner_Env_Assoc_Array_Access :=
      Base_Type_Decl_P_Dottable_Subps (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Base_Subtype_Decl
        (Self : Bare_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Base_Subtype_Decl;
--# property-start BaseSubtypeDecl.from_type_bound /usr/src/packages/BUILD/ada/ast.py:6116
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_From_Type_Bound
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2813 '<Block>' Let_Result None
--# scope-start
--# expr-start 2812 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .get_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6121
--# expr-start 2810 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:6121
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 2810
Bound_Origin := Fld; 
--# expr-start 2811 '<FieldAccess .get_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6121
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 2811
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 2812
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 2813
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_From_Type_Bound;
--# end
--# property-start '[dispatcher]BaseSubtypeDecl.get_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Base_Subtype_Decl (Self.Kind) is
               when Ada_Discrete_Base_Subtype_Decl =>
                  --# property-call-start DiscreteBaseSubtypeDecl.get_type
                  Property_Result := Discrete_Base_Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Decl =>
                  --# property-call-start SubtypeDecl.get_type
                  Property_Result := Subtype_Decl_P_Get_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Base_Subtype_Decl_P_Get_Type;
--# end
--# property-start BaseSubtypeDecl.primitives_env /usr/src/packages/BUILD/ada/ast.py:6133
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Primitives_Env
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_532;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_532 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_532;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2815 '<FieldAccess .primitives_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6133
--# expr-start 2814 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6133
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2814
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2815
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_532;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_532;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Primitives_Env;
--# end
--# property-start BaseSubtypeDecl.array_ndims /usr/src/packages/BUILD/ada/ast.py:6135
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Ndims
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2817 '<FieldAccess .array_ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6135
--# expr-start 2816 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6135
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2816
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2817
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Array_Ndims;
--# end
--# property-start BaseSubtypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:6136
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Defining_Env
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_534;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_534 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_534;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2819 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6136
--# expr-start 2818 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6136
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2818
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2819
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_534;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_534;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Defining_Env;
--# end
--# property-start BaseSubtypeDecl.canonical_type /usr/src/packages/BUILD/ada/ast.py:6138
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Canonical_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2821 '<FieldAccess .canonical_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6138
--# expr-start 2820 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6138
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2820
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2821
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Canonical_Type;
--# end
--# property-start BaseSubtypeDecl.record_def /usr/src/packages/BUILD/ada/ast.py:6139
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Record_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2823 '<FieldAccess .record_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6139
--# expr-start 2822 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6139
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2822
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2823
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Record_Def;
--# end
--# property-start BaseSubtypeDecl.accessed_type /usr/src/packages/BUILD/ada/ast.py:6140
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Accessed_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2825 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6140
--# expr-start 2824 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6140
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2824
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2825
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Accessed_Type;
--# end
--# property-start BaseSubtypeDecl.is_int_type /usr/src/packages/BUILD/ada/ast.py:6141
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Int_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2827 '<FieldAccess .is_int_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6141
--# expr-start 2826 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6141
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2826
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2827
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Int_Type;
--# end
--# property-start BaseSubtypeDecl.is_discrete_type /usr/src/packages/BUILD/ada/ast.py:6142
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Discrete_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2829 '<FieldAccess .is_discrete_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6142
--# expr-start 2828 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6142
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2828
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2829
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Discrete_Type;
--# end
--# property-start BaseSubtypeDecl.is_real_type /usr/src/packages/BUILD/ada/ast.py:6144
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Real_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2831 '<FieldAccess .is_real_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6144
--# expr-start 2830 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6144
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2830
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2831
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Real_Type;
--# end
--# property-start BaseSubtypeDecl.is_float_type /usr/src/packages/BUILD/ada/ast.py:6145
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Float_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2833 '<FieldAccess .is_float_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6145
--# expr-start 2832 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6145
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2832
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2833
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Float_Type;
--# end
--# property-start BaseSubtypeDecl.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:6146
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Fixed_Point
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2835 '<FieldAccess .is_fixed_point>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6146
--# expr-start 2834 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6146
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2834
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2835
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Fixed_Point;
--# end
--# property-start BaseSubtypeDecl.is_enum_type /usr/src/packages/BUILD/ada/ast.py:6147
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Enum_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2837 '<FieldAccess .is_enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6147
--# expr-start 2836 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6147
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2836
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2837
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Enum_Type;
--# end
--# property-start BaseSubtypeDecl.is_access_type /usr/src/packages/BUILD/ada/ast.py:6148
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Access_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2839 '<FieldAccess .is_access_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6148
--# expr-start 2838 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6148
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2838
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2839
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Access_Type;
--# end
--# property-start BaseSubtypeDecl.access_def /usr/src/packages/BUILD/ada/ast.py:6149
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Access_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2841 '<FieldAccess .access_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6149
--# expr-start 2840 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6149
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2840
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2841
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Access_Def;
--# end
--# property-start BaseSubtypeDecl.is_char_type /usr/src/packages/BUILD/ada/ast.py:6150
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Char_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2843 '<FieldAccess .is_char_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6150
--# expr-start 2842 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6150
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2842
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2843
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Char_Type;
--# end
--# property-start BaseSubtypeDecl.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:6151
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Tagged_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2845 '<FieldAccess .is_tagged_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6151
--# expr-start 2844 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6151
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2844
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2845
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Tagged_Type;
--# end
--# property-start BaseSubtypeDecl.base_type /usr/src/packages/BUILD/ada/ast.py:6152
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2847 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6152
--# expr-start 2846 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6152
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2846
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2847
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Base_Type;
--# end
--# property-start BaseSubtypeDecl.base_types /usr/src/packages/BUILD/ada/ast.py:6153
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Base_Types
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_549;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_549 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_549;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2849 '<FieldAccess .base_types>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6153
--# expr-start 2848 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2848
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2849
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_549;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_549;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Base_Types;
--# end
--# property-start BaseSubtypeDecl.array_def /usr/src/packages/BUILD/ada/ast.py:6154
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Array_Def
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2851 '<FieldAccess .array_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6154
--# expr-start 2850 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6154
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2850
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2851
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Array_Def;
--# end
--# property-start BaseSubtypeDecl.is_classwide /usr/src/packages/BUILD/ada/ast.py:6155
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Classwide
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2853 '<FieldAccess .is_classwide>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6155
--# expr-start 2852 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6155
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2852
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2853
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Classwide;
--# end
--# property-start BaseSubtypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:6156
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Discriminants_List
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_552;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_552 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_552;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2855 '<FieldAccess .discriminants_list>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6156
--# expr-start 2854 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6156
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2854
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2855
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_552;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_552;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Discriminants_List;
--# end
--# property-start BaseSubtypeDecl.is_iterable_type /usr/src/packages/BUILD/ada/ast.py:6157
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Iterable_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2857 '<FieldAccess .is_iterable_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6157
--# expr-start 2856 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6157
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2856
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2857
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Iterable_Type;
--# end
--# property-start BaseSubtypeDecl.iterable_comp_type /usr/src/packages/BUILD/ada/ast.py:6158
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Iterable_Comp_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2859 '<FieldAccess .iterable_comp_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6158
--# expr-start 2858 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6158
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2858
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2859
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Iterable_Comp_Type;
--# end
--# property-start BaseSubtypeDecl.is_record_type /usr/src/packages/BUILD/ada/ast.py:6159
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Record_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2861 '<FieldAccess .is_record_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6159
--# expr-start 2860 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6159
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2860
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2861
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Record_Type;
--# end
--# property-start BaseSubtypeDecl.is_private /usr/src/packages/BUILD/ada/ast.py:6160
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Is_Private
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2863 '<FieldAccess .is_private>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6160
--# expr-start 2862 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6160
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2862
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2863
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Is_Private;
--# end
--# property-start BaseSubtypeDecl.root_type /usr/src/packages/BUILD/ada/ast.py:6161
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Root_Type
  (Node : Bare_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2865 '<FieldAccess .root_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6161
--# expr-start 2864 '<FieldAccess .get_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6161
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2864
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2865
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Root_Type;
--# end
--# property-start BaseSubtypeDecl.has_ud_indexing /usr/src/packages/BUILD/ada/ast.py:6163
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Has_Ud_Indexing
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2867 '<FieldAccess .has_ud_indexing>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6164
--# expr-start 2866 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6164
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2866
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2867
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Has_Ud_Indexing;
--# end
--# property-start BaseSubtypeDecl.constant_indexing_fns /usr/src/packages/BUILD/ada/ast.py:6166
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_559;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_559 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_559;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2869 '<FieldAccess .constant_indexing_fns>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6167
--# expr-start 2868 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6167
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2868
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2869
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_559;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_559;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start BaseSubtypeDecl.variable_indexing_fns /usr/src/packages/BUILD/ada/ast.py:6169
pragma Warnings (Off, "is not referenced");
 function Base_Subtype_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Base_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Base_Subtype_Decl := Bare_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subtype_Decl :=
     Internal_Entity_Base_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_560;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_560 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_560;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2871 '<FieldAccess .variable_indexing_fns>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6170
--# expr-start 2870 '<FieldAccess .from_type_bound>' Fld /usr/src/packages/BUILD/ada/ast.py:6170
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubtypeDecl.from_type_bound
Fld := Libadalang.Implementation.Base_Subtype_Decl_P_From_Type_Bound (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2870
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2871
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_560;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_560;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subtype_Decl_P_Variable_Indexing_Fns;
--# end
   --
   --  Primitives for Bare_Discrete_Base_Subtype_Decl
   --
      procedure Initialize_Fields_For_Discrete_Base_Subtype_Decl
        (Self : Bare_Discrete_Base_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Discrete_Base_Subtype_Decl;
--# property-start DiscreteBaseSubtypeDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:6208
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Discrete_Base_Subtype_Decl := Bare_Discrete_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discrete_Base_Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start DiscreteBaseSubtypeDecl.get_type /usr/src/packages/BUILD/ada/ast.py:6212
pragma Warnings (Off, "is not referenced");
 function Discrete_Base_Subtype_Decl_P_Get_Type
  (Node : Bare_Discrete_Base_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Discrete_Base_Subtype_Decl := Bare_Discrete_Base_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2874 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6213
--# expr-start 2873 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6213
--# expr-start 2872 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6213
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2872
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 2873
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2874
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discrete_Base_Subtype_Decl_P_Get_Type;
--# end
   --
   --  Primitives for Bare_Subtype_Decl
   --
      procedure Initialize_Fields_For_Subtype_Decl
        (Self : Bare_Subtype_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Subtype_Decl_F_Subtype : Bare_Subtype_Indication
         ; Subtype_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subtype_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Subtype_Decl_F_Subtype := Subtype_Decl_F_Subtype;
            Self.Subtype_Decl_F_Aspects := Subtype_Decl_F_Aspects;
      end Initialize_Fields_For_Subtype_Decl;
   function Subtype_Decl_F_Subtype
     (Node : Bare_Subtype_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.Subtype_Decl_F_Subtype;
   end;
--# property-start SubtypeDecl.get_type /usr/src/packages/BUILD/ada/ast.py:6182
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Get_Type
  (Node : Bare_Subtype_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Decl;
Match_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Subtype_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2879 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2878 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:6183
--# expr-start 2876 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6183
--# expr-start 2875 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:6183
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2875
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2876
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind st Match_0
--# expr-start 2877 '<FieldAccess .get_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6184
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2877
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 2878
Let_Result_2 := Match_Result; 
--# end
--# expr-done 2879
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Get_Type;
--# end
--# property-start SubtypeDecl.discrete_range /usr/src/packages/BUILD/ada/ast.py:6189
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Discrete_Range
  (Node : Bare_Subtype_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2882 '<Block>' Let_Result None
--# scope-start
--# expr-start 2881 '<FieldAccess .discrete_range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6190
--# expr-start 2880 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:6190
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2880
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2881
Let_Result := Fld_1; 
--# end
--# expr-done 2882
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Discrete_Range;
--# end
--# property-start SubtypeDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:6193
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Equation
  (Node : Bare_Subtype_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_563;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_563 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_563;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2885 '<Block>' Let_Result None
--# scope-start
--# expr-start 2884 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6194
--# expr-start 2883 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:6194
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2883
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2884
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 2885
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_563;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_563;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Xref_Equation;
--# end
--# property-start SubtypeDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:6196
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Is_Static_Decl
  (Node : Bare_Subtype_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Decl :=
     Internal_Entity_Subtype_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2887 '<FieldAccess .is_static_subtype>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6196
--# expr-start 2886 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:6196
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Subtype_Decl_F_Subtype, Info => Ent.Info);
--# expr-done 2886
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 2887
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Is_Static_Decl;
--# end
--# property-start SubtypeDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6197
pragma Warnings (Off, "is not referenced");
 function Subtype_Decl_P_Xref_Entry_Point
  (Node : Bare_Subtype_Decl
  )
   return Boolean
is
   Self : Bare_Subtype_Decl := Bare_Subtype_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Classwide_Type_Decl
   --
      procedure Initialize_Fields_For_Classwide_Type_Decl
        (Self : Bare_Classwide_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
      end Initialize_Fields_For_Classwide_Type_Decl;
--# property-start ClasswideTypeDecl.typedecl /usr/src/packages/BUILD/ada/ast.py:5070
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Typedecl
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Type_Decl;
As_Entity : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2890 '<as_entity(<Cast to BaseTypeDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5070
--# expr-start 2889 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5070
--# expr-start 2888 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:5070
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 2888
   if Fld = null
      or else Fld.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 2889
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Base_Type_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 2890
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Typedecl;
--# end
--# property-start ClasswideTypeDecl.is_classwide /usr/src/packages/BUILD/ada/ast.py:5072
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Classwide
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Classwide;
--# end
--# property-start ClasswideTypeDecl.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:5074
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start ClasswideTypeDecl.base_type /usr/src/packages/BUILD/ada/ast.py:5075
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2892 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5075
--# expr-start 2891 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5075
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2891
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2892
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Base_Type;
--# end
--# property-start ClasswideTypeDecl.base_interfaces /usr/src/packages/BUILD/ada/ast.py:5076
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Base_Interfaces
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_407;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_407 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_407;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2894 '<FieldAccess .base_interfaces>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5076
--# expr-start 2893 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5076
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2893
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Interfaces (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2894
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_407;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_407;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Base_Interfaces;
--# end
--# property-start ClasswideTypeDecl.record_def /usr/src/packages/BUILD/ada/ast.py:5077
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Record_Def
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2896 '<FieldAccess .record_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5077
--# expr-start 2895 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5077
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2895
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2896
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Record_Def;
--# end
--# property-start ClasswideTypeDecl.classwide_type /usr/src/packages/BUILD/ada/ast.py:5078
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Classwide_Type
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Classwide_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Classwide_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Classwide_Type;
--# end
--# property-start ClasswideTypeDecl.is_iterable_type /usr/src/packages/BUILD/ada/ast.py:5079
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2898 '<FieldAccess .is_iterable_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5079
--# expr-start 2897 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5079
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2897
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2898
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start ClasswideTypeDecl.iterable_comp_type /usr/src/packages/BUILD/ada/ast.py:5080
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2900 '<FieldAccess .iterable_comp_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5080
--# expr-start 2899 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5080
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2899
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2900
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start ClasswideTypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:5081
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Defining_Env
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_412;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_412 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_412;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2902 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5081
--# expr-start 2901 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5081
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2901
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2902
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_412;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_412;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Defining_Env;
--# end
--# property-start ClasswideTypeDecl.is_private /usr/src/packages/BUILD/ada/ast.py:5082
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_Private
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2904 '<FieldAccess .is_private>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5082
--# expr-start 2903 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5082
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2903
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_private'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Private (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2904
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_Private;
--# end
--# property-start ClasswideTypeDecl.is_in_private_part /usr/src/packages/BUILD/ada/ast.py:5083
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Is_In_Private_Part
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2906 '<FieldAccess .is_in_private_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5083
--# expr-start 2905 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5083
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2905
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2906
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Is_In_Private_Part;
--# end
--# property-start ClasswideTypeDecl.get_aspect_assoc /usr/src/packages/BUILD/ada/ast.py:5086
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Classwide_Type_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2909 '<Block>' Let_Result None
--# scope-start
--# expr-start 2908 '<FieldAccess .get_aspect_assoc(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5087
--# expr-start 2907 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5087
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2907
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld.Node, Name => Name, E_Info => Fld.Info);
--# end
--# expr-done 2908
Let_Result := Fld_1; 
--# end
--# expr-done 2909
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Get_Aspect_Assoc;
--# end
--# property-start ClasswideTypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:5089
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Discriminants_List
  (Node : Bare_Classwide_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_416;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_416 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_416;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2911 '<FieldAccess .discriminants_list>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5089
--# expr-start 2910 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5089
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2910
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2911
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_416;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_416;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Discriminants_List;
--# end
--# property-start ClasswideTypeDecl.previous_part /usr/src/packages/BUILD/ada/ast.py:5093
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Previous_Part
  (Node : Bare_Classwide_Type_Decl
      ; Go_To_Incomplete : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind go_to_incomplete Go_To_Incomplete
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Let_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Classwide_Type_Decl_Classwide_Type_Decl_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Classwide_Type_Decl,
                                As_Bare_Classwide_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Go_To_Incomplete);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 2915 '<Block>' Let_Result None
--# scope-start
--# expr-start 2913 '<FieldAccess .previous_part(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5094
--# expr-start 2912 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5094
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2912
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => Go_To_Incomplete, E_Info => Fld.Info);
--# end
--# expr-done 2913
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 2914 '<FieldAccess .classwide_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5095
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2914
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 2915
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
      Property_Result := Cast_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Previous_Part;
--# end
--# property-start ClasswideTypeDecl.canonical_type /usr/src/packages/BUILD/ada/ast.py:5098
pragma Warnings (Off, "is not referenced");
 function Classwide_Type_Decl_P_Canonical_Type
  (Node : Bare_Classwide_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Classwide_Type_Decl := Bare_Classwide_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classwide_Type_Decl :=
     Internal_Entity_Classwide_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Classwide_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2917 '<FieldAccess .canonical_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5098
--# expr-start 2916 '<FieldAccess .typedecl>' Fld /usr/src/packages/BUILD/ada/ast.py:5098
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 2916
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 2917
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 2918 '<FieldAccess .classwide_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5102
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 2918
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Classwide_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result := Var_Expr_1;
   --# end
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Scope_Result.Node,
         Info => Scope_Result.Info);
   Result_Var := Cast_Result;
else
   Result_Var := Var_Expr;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
      Property_Result := Result_Var_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classwide_Type_Decl_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Incomplete_Type_Decl
   --
      procedure Incomplete_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Incomplete_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_266 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:15713");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_267 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Incomplete_Type_Decl
        (Self : Bare_Incomplete_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Incomplete_Type_Decl_F_Discriminants := Incomplete_Type_Decl_F_Discriminants;
      end Initialize_Fields_For_Incomplete_Type_Decl;
   function Incomplete_Type_Decl_F_Discriminants
     (Node : Bare_Incomplete_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Incomplete_Type_Decl_F_Discriminants;
   end;
--# property-start IncompleteTypeDecl.find_next_part_in /usr/src/packages/BUILD/ada/ast.py:15699
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Find_Next_Part_In
  (Node : Bare_Incomplete_Type_Decl
      ; Decl_Part : Internal_Entity_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind decl_part Decl_Part
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1161;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Item_119 : Internal_Entity;
Bare_Item_119 : Bare_Ada_Node;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type;
Fld_5 : Boolean;
Cast_Expr_1 : Internal_Entity_Incomplete_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Not_Val : Boolean;
If_Result_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1161 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1161;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2929 '<Block>' Let_Result None
--# scope-start
--# expr-start 2928 '<Cast to BaseTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:15705
--# expr-start 2927 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:15705
--# expr-start 2926 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:15705
--# expr-start 2919 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:15705
      if Decl_Part.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Decl_Part.Node.Declarative_Part_F_Decls, Info => Decl_Part.Info);
--# expr-done 2919
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_119 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_119 := Untyped_Item_119; 
Is_Null := Bare_Item_119 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_119); 
   If_Result := As_Entity;
end if;
                  Item_119 := If_Result; 
   --# scope-start
               --# bind t Item_119
--# expr-start 2920 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15706
Cast_Expr := Item_119; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2920
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind btd Var_Expr
--# expr-start 2923 '<FieldAccess .name_is(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:15708
--# expr-start 2921 '<FieldAccess .name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15708
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Defining_Name (Node => Var_Expr.Node.Base_Type_Decl_F_Name, Info => Var_Expr.Info);
--# expr-done 2921
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2922 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15708
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 2922
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_3.Node, Sym => Fld_4);
--# end
--# expr-done 2923
if Fld_5 then
   --# expr-start 2925 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:15708
--# expr-start 2924 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:15708
Cast_Expr_1 := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Var_Expr; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 2924
Not_Val := not (Is_Equiv); 
--# expr-done 2925
   If_Result_1 := Not_Val;
else
   If_Result_1 := False;
end if;
Scope_Result := If_Result_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_119;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2926
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 2927
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 2928
Let_Result := Cast_Result_3; 
--# end
--# expr-done 2929
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1161;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1161;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Find_Next_Part_In;
--# end
--# property-start IncompleteTypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:15717
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Defining_Env
  (Node : Bare_Incomplete_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1162;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1162 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_1162;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2933 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:15718
--# expr-start 2932 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:15718
--# expr-start 2930 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 2930
--# expr-start 2931 '<FieldAccess .dottable_subps_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15718
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Self);
--# end
--# expr-done 2931
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1)); 
--# expr-done 2932
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 2933
      Property_Result := Group_Env;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1162;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1162;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Defining_Env;
--# end
--# property-start IncompleteTypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:15722
pragma Warnings (Off, "is not referenced");
 function Incomplete_Type_Decl_P_Discriminants_List
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1163;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_1163 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1163;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2935 '<FieldAccess .abstract_formal_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15722
--# expr-start 2934 '<FieldAccess .discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:15722
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Incomplete_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2934
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2935
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1163;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1163;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Type_Decl_P_Discriminants_List;
--# end
--# property-start IncompleteTypeDecl._env_mappings_266 /usr/src/packages/BUILD/ada/ast.py:15713
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_266
  (Node : Bare_Incomplete_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Incomplete_Type_Decl :=
     Internal_Entity_Incomplete_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1436;
      Fld : Symbol_Type;
Cast_Expr : Bare_Incomplete_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1436 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1436;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2937 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15713
--# expr-start 2936 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:15713
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2936
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2937
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1436;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1436;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_266;
--# end
--# property-start IncompleteTypeDecl._env_trans_parent_267 /usr/src/packages/BUILD/ada/ast.py:15712
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_267
  (Node : Bare_Incomplete_Type_Decl
  )
   return Boolean
is
   Self : Bare_Incomplete_Type_Decl := Bare_Incomplete_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_267;
--# end
   --
   --  Primitives for Bare_Incomplete_Tagged_Type_Decl
   --
      procedure Initialize_Fields_For_Incomplete_Tagged_Type_Decl
        (Self : Bare_Incomplete_Tagged_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Incomplete_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Incomplete_Tagged_Type_Decl_F_Has_Abstract : Bare_Abstract_Node
        ) is
      begin
            Initialize_Fields_For_Incomplete_Type_Decl
              (Self, Base_Type_Decl_F_Name, Incomplete_Type_Decl_F_Discriminants);
            Self.Incomplete_Tagged_Type_Decl_F_Has_Abstract := Incomplete_Tagged_Type_Decl_F_Has_Abstract;
      end Initialize_Fields_For_Incomplete_Tagged_Type_Decl;
   function Incomplete_Tagged_Type_Decl_F_Has_Abstract
     (Node : Bare_Incomplete_Tagged_Type_Decl) return Bare_Abstract_Node
   is
   begin
         return Node.Incomplete_Tagged_Type_Decl_F_Has_Abstract;
   end;
--# property-start IncompleteTaggedTypeDecl.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:15732
pragma Warnings (Off, "is not referenced");
 function Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Incomplete_Tagged_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Incomplete_Tagged_Type_Decl := Bare_Incomplete_Tagged_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Incomplete_Tagged_Type_Decl_P_Is_Tagged_Type;
--# end
   --
   --  Primitives for Bare_Protected_Type_Decl
   --
      procedure Protected_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Protected_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_12 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:6314");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_13 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_14 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Type_Decl
        (Self : Bare_Protected_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Protected_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Protected_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Protected_Type_Decl_F_Interfaces : Bare_Parent_List
         ; Protected_Type_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Protected_Type_Decl_F_Discriminants := Protected_Type_Decl_F_Discriminants;
            Self.Protected_Type_Decl_F_Aspects := Protected_Type_Decl_F_Aspects;
            Self.Protected_Type_Decl_F_Interfaces := Protected_Type_Decl_F_Interfaces;
            Self.Protected_Type_Decl_F_Definition := Protected_Type_Decl_F_Definition;
      end Initialize_Fields_For_Protected_Type_Decl;
   function Protected_Type_Decl_F_Discriminants
     (Node : Bare_Protected_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Protected_Type_Decl_F_Discriminants;
   end;
   function Protected_Type_Decl_F_Interfaces
     (Node : Bare_Protected_Type_Decl) return Bare_Parent_List
   is
   begin
         return Node.Protected_Type_Decl_F_Interfaces;
   end;
   function Protected_Type_Decl_F_Definition
     (Node : Bare_Protected_Type_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Protected_Type_Decl_F_Definition;
   end;
--# property-start ProtectedTypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:6295
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Discriminants_List
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_575;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_575 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_575;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2939 '<FieldAccess .abstract_formal_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6295
--# expr-start 2938 '<FieldAccess .discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:6295
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Protected_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2938
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2939
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_575;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_575;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Discriminants_List;
--# end
--# property-start ProtectedTypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:6297
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Defining_Env
  (Node : Bare_Protected_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_576;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_576 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_576;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2940 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6297
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2940
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_576;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_576;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Defining_Env;
--# end
--# property-start ProtectedTypeDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:6299
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Next_Part_For_Decl
  (Node : Bare_Protected_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2941 '<FieldAccess .basic_decl_next_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6299
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 2941
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ProtectedTypeDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6301
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start ProtectedTypeDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:6304
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Xref_Equation
  (Node : Bare_Protected_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_579;
            procedure Finalizer_Scope_1997;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_120 : Internal_Entity_Name;
Bare_Item_120 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_579 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_579;
            procedure Finalizer_Scope_1997 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1997;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2946 '<Block>' Let_Result None
--# scope-start
--# expr-start 2945 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6305
--# expr-start 2944 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6305
--# expr-start 2942 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:6305
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Protected_Type_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 2942
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_120 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_120 := Untyped_Item_120; 
Is_Null := Bare_Item_120 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_120); 
   If_Result := As_Entity;
end if;
                  Item_120 := If_Result; 
   --# scope-start
               --# bind ifc Item_120
      --# expr-start 2943 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6305
      if Item_120.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_120.Node, Env => Env, Origin => Origin, E_Info => Item_120.Info);
--# end
--# expr-done 2943
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_1997;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 2944
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6305") else null)); 
--# expr-done 2945
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 2946
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_579;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_579;
               Finalizer_Scope_1997;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Xref_Equation;
--# end
--# property-start ProtectedTypeDecl.env_names /usr/src/packages/BUILD/ada/ast.py:6308
pragma Warnings (Off, "is not referenced");
 function Protected_Type_Decl_P_Env_Names
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_580;
            procedure Finalizer_Scope_1998;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_580 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_580;
            procedure Finalizer_Scope_1998 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1998;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2949 '<Block>' Let_Result None
--# scope-start
--# expr-start 2947 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:6309
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 2947
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 2948 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:6310
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 2948
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1998;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2949
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_580;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_580;
               Finalizer_Scope_1998;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Type_Decl_P_Env_Names;
--# end
--# property-start ProtectedTypeDecl._env_mappings_12 /usr/src/packages/BUILD/ada/ast.py:6314
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_12
  (Node : Bare_Protected_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Type_Decl :=
     Internal_Entity_Protected_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1182;
      Fld : Symbol_Type;
Cast_Expr : Bare_Protected_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1182 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1182;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2951 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:6314
--# expr-start 2950 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:6314
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2950
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2951
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1182;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1182;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_12;
--# end
--# property-start ProtectedTypeDecl._env_trans_parent_13 /usr/src/packages/BUILD/ada/ast.py:6313
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_13
  (Node : Bare_Protected_Type_Decl
  )
   return Boolean
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_13;
--# end
--# property-start ProtectedTypeDecl._env_names_14 /usr/src/packages/BUILD/ada/ast.py:6315
pragma Warnings (Off, "is not referenced");
 function Env_Names_14
  (Node : Bare_Protected_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Type_Decl := Bare_Protected_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1184;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1184 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1184;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2952 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:6315
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ProtectedTypeDecl.env_names
Fld := Libadalang.Implementation.Protected_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 2952
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1184;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1184;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_14;
--# end
   --
   --  Primitives for Bare_Task_Type_Decl
   --
      procedure Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_8 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:6256");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_9 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_10 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Type_Decl
        (Self : Bare_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Task_Type_Decl_F_Discriminants := Task_Type_Decl_F_Discriminants;
            Self.Task_Type_Decl_F_Aspects := Task_Type_Decl_F_Aspects;
            Self.Task_Type_Decl_F_Definition := Task_Type_Decl_F_Definition;
      end Initialize_Fields_For_Task_Type_Decl;
   function Task_Type_Decl_F_Discriminants
     (Node : Bare_Task_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Task_Type_Decl_F_Discriminants;
   end;
   function Task_Type_Decl_F_Definition
     (Node : Bare_Task_Type_Decl) return Bare_Task_Def
   is
   begin
         return Node.Task_Type_Decl_F_Definition;
   end;
--# property-start TaskTypeDecl.is_task_type /usr/src/packages/BUILD/ada/ast.py:6247
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Is_Task_Type
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Is_Task_Type;
--# end
--# property-start TaskTypeDecl.env_names /usr/src/packages/BUILD/ada/ast.py:6250
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Env_Names
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_570;
            procedure Finalizer_Scope_1999;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_570 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_570;
            procedure Finalizer_Scope_1999 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1999;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2955 '<Block>' Let_Result None
--# scope-start
--# expr-start 2953 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:6251
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 2953
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 2954 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:6252
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 2954
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1999;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2955
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_570;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_570;
               Finalizer_Scope_1999;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Env_Names;
--# end
--# property-start TaskTypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:6260
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Defining_Env
  (Node : Bare_Task_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_571;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_571 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_571;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2956 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6260
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 2956
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_571;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_571;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Defining_Env;
--# end
--# property-start TaskTypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:6262
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Discriminants_List
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_572;
      Fld : Internal_Entity_Discriminant_Part;
Fld_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_572 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_572;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2958 '<FieldAccess .abstract_formal_params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6262
--# expr-start 2957 '<FieldAccess .discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:6262
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Task_Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 2957
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 2958
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_572;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_572;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Discriminants_List;
--# end
--# property-start TaskTypeDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6264
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TaskTypeDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:6267
pragma Warnings (Off, "is not referenced");
 function Task_Type_Decl_P_Xref_Equation
  (Node : Bare_Task_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_574;
            procedure Finalizer_Scope_2000;
      Var_Expr : Internal_Entity_Task_Def;
Fld : Internal_Entity_Task_Def;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_574 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_574;
            procedure Finalizer_Scope_2000 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2000;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2961 '<Block>' Let_Result None
--# scope-start
--# expr-start 2959 '<FieldAccess .definition>' Fld /usr/src/packages/BUILD/ada/ast.py:6268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Task_Def (Node => Ent.Node.Task_Type_Decl_F_Definition, Info => Ent.Info);
--# expr-done 2959
Var_Expr := Fld;
if Var_Expr /= No_Entity_Task_Def then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 2960 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6269
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2960
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2000;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6270") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 2961
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_574;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_574;
               Finalizer_Scope_2000;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Type_Decl_P_Xref_Equation;
--# end
--# property-start TaskTypeDecl._env_mappings_8 /usr/src/packages/BUILD/ada/ast.py:6256
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_8
  (Node : Bare_Task_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Type_Decl :=
     Internal_Entity_Task_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1178;
      Fld : Symbol_Type;
Cast_Expr : Bare_Task_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1178 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1178;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 2963 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:6256
--# expr-start 2962 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:6256
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 2962
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 2963
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1178;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1178;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_8;
--# end
--# property-start TaskTypeDecl._env_trans_parent_9 /usr/src/packages/BUILD/ada/ast.py:6255
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_9
  (Node : Bare_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_9;
--# end
--# property-start TaskTypeDecl._env_names_10 /usr/src/packages/BUILD/ada/ast.py:6257
pragma Warnings (Off, "is not referenced");
 function Env_Names_10
  (Node : Bare_Task_Type_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Type_Decl := Bare_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1180;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1180 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1180;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 2964 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:6257
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TaskTypeDecl.env_names
Fld := Libadalang.Implementation.Task_Type_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 2964
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1180;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1180;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_10;
--# end
   --
   --  Primitives for Bare_Single_Task_Type_Decl
   --
      procedure Single_Task_Type_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_11 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Type_Decl
        (Self : Bare_Single_Task_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Task_Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Task_Type_Decl_F_Aspects : Bare_Aspect_Spec
         ; Task_Type_Decl_F_Definition : Bare_Task_Def
        ) is
      begin
            Initialize_Fields_For_Task_Type_Decl
              (Self, Base_Type_Decl_F_Name, Task_Type_Decl_F_Discriminants, Task_Type_Decl_F_Aspects, Task_Type_Decl_F_Definition);
      end Initialize_Fields_For_Single_Task_Type_Decl;
--# property-start SingleTaskTypeDecl._env_trans_parent_11 /usr/src/packages/BUILD/ada/ast.py:6278
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_11
  (Node : Bare_Single_Task_Type_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Type_Decl := Bare_Single_Task_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_11;
--# end
   --
   --  Primitives for Bare_Type_Decl
   --
      procedure Type_Decl_Pre_Env_Actions
        (Self            : Bare_Type_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_3 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:5324");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_4 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Type_Decl_Post_Env_Actions
        (Self : Bare_Type_Decl; State : in out PLE_Node_State) is
      begin
      if Libadalang.Implementation.Ref_Cond_7 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_5 (Node => 
Self);
            Env : Lexical_Env :=
              Libadalang.Implementation.Env_Dest_6 (Node => 
Self);
         begin
               if Is_Foreign_Strict (Env, Self) then
                  raise Property_Error with
                     "unsound foreign environment in RefEnvs ("
                     & "ast.py:5327)";
               end if;
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Type_Decl_P_Parent_Primitives_Env'Access,
               Transitive,
               (Inherited_Primitives => True, others => False),
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Type_Decl
        (Self : Bare_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Type_Decl
              (Self, Base_Type_Decl_F_Name);
            Self.Type_Decl_F_Discriminants := Type_Decl_F_Discriminants;
            Self.Type_Decl_F_Type_Def := Type_Decl_F_Type_Def;
            Self.Type_Decl_F_Aspects := Type_Decl_F_Aspects;
      Self.LF_Present_Direct_Primitives_Env := False;
      Self.LF_Stg_Direct_Primitives_Env := Empty_Env;
      end Initialize_Fields_For_Type_Decl;
   function Type_Decl_F_Discriminants
     (Node : Bare_Type_Decl) return Bare_Discriminant_Part
   is
   begin
         return Node.Type_Decl_F_Discriminants;
   end;
   function Type_Decl_F_Type_Def
     (Node : Bare_Type_Decl) return Bare_Type_Def
   is
   begin
         return Node.Type_Decl_F_Type_Def;
   end;
--# property-start TypeDecl.is_iterable_type /usr/src/packages/BUILD/ada/ast.py:5114
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Iterable_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Sym_1 : Symbol_Type;
Fld_2 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Internal_Entity_Type_Def;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result_2 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 2965 '<FieldAccess .is_array>' Fld /usr/src/packages/BUILD/ada/ast.py:5119
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2965
if Fld then
   If_Result := True;
else
   --# expr-start 2969 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5120
--# expr-start 2968 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5120
--# expr-start 2967 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2966 '<Symbol Iterator_Element>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Element); 
--# expr-done 2966
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2967
Is_Null := Fld_1.Node = null; 
--# expr-done 2968
Not_Val := not (Is_Null); 
--# expr-done 2969
   If_Result := Not_Val;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 2973 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:5121
--# expr-start 2972 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:5121
--# expr-start 2971 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5121
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2970 '<Symbol Iterable>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 2970
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2971
Is_Null_1 := Fld_2.Node = null; 
--# expr-done 2972
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2973
   If_Result_1 := Not_Val_1;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 2977 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5122
--# expr-start 2974 '<FieldAccess .type_def>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5122
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 2974
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_3; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 2975 '<FieldAccess .base_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5124
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 2975
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 2976 '<FieldAccess .is_iterable_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5124
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 2976
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 2977
   If_Result_2 := Match_Result;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
--# expr-start 2978 '<FieldAccess .previous_part(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5127
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 2978
Var_Expr_1 := Fld_6;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 2979 '<FieldAccess .is_iterable_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5127
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2979
Scope_Result_3 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_3;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
end if;
      Property_Result := If_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Iterable_Type;
--# end
--# property-start TypeDecl.iterable_comp_type /usr/src/packages/BUILD/ada/ast.py:5136
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Iterable_Comp_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_420;
            procedure Finalizer_Scope_2005;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Bound_Imprecise_Fallback : Boolean;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Aggregate;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Internal_Single_Actual_Array_Access;
Fld_8 : Bare_Base_Id;
Sym_2 : Symbol_Type;
Fld_9 : Boolean;
Map_Result : Internal_Single_Actual_Array_Access;
Get_Result : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Fld_12 : Internal_Entity_Basic_Decl;
Fld_13 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Type_Def;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result_3 : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Type_Def;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_17 : Internal_Entity_Base_Type_Decl;
Fld_18 : Internal_Entity_Base_Type_Decl;
Scope_Result_5 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_420 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_420;
            procedure Finalizer_Scope_2005 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_2005;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3015 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 2981 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5137
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2980 '<Symbol Iterator_Element>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterator_Element); 
--# expr-done 2980
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 2981
Block_Var := Fld; 
--# bind ie Block_Var
--# expr-start 2983 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5138
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2982 '<Symbol Iterable>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Iterable); 
--# expr-done 2982
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 2983
Block_Var_1 := Fld_1; 
--# bind it Block_Var_1
--# expr-start 3014 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <Cond>: <Var e> <Var e>>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:5140
Bound_Imprecise_Fallback := False; 
--# expr-start 3011 '<Cond>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:5140
--# expr-start 2984 '<FieldAccess .is_array>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5141
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2984
if Fld_2 then
   --# expr-start 2985 '<FieldAccess .comp_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5141
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_3 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 2985
   If_Result_2 := Fld_3;
else
--# expr-start 2987 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5143
--# expr-start 2986 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5143
Is_Null := Block_Var.Node = null; 
--# expr-done 2986
Not_Val := not (Is_Null); 
--# expr-done 2987
if Not_Val then
--# expr-start 2988 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5143
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 2988
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 2991 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5144
--# expr-start 2989 '<FieldAccess .node_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5144
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Node_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 2989
Bound_Env := Fld_4; Inc_Ref (Bound_Env);
--# expr-start 2990 '<FieldAccess .designated_type_impl>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5144
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Var_Expr_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 2990
Dyn_Var_Bind_Result := Fld_5; 
--# expr-done 2991
Scope_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_2005;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := Result_Var;
else
--# expr-start 2993 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:5147
--# expr-start 2992 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:5147
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 2992
Not_Val_1 := not (Is_Null_1); 
--# expr-done 2993
if Not_Val_1 then
   --# expr-start 3006 '<FieldAccess .expr_type>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3005 '<FieldAccess .referenced_decl>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3004 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3003 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3002 '<FieldAccess .assoc>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3001 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 3000 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 2996 '<FieldAccess .unpacked_params>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 2995 '<FieldAccess .assocs>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5148
--# expr-start 2994 '<Cast to Aggregate>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5148
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 2994
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Assoc_List (Node => Cast_Result_1.Node.Base_Aggregate_F_Assocs, Info => Cast_Result_1.Info);
--# expr-done 2995
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_7 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 2996
declare
   Map_Result_Vec : Internal_Single_Actual_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Actual_Array_Access := Fld_7;
      begin
         for Item_121 of
               Collection.Items
         loop
   --# scope-start
               --# bind sa Item_121
               --# expr-start 2999 '<FieldAccess .name_is(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:5149
--# expr-start 2997 '<FieldAccess .name>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5149
Fld_8 := Item_121.Name;
--# expr-done 2997
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 2998 '<Symbol Element>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Element); 
--# expr-done 2998
--# property-call-start Name.name_is
Fld_9 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_8, Sym => Sym_2);
--# end
--# expr-done 2999
               if Fld_9 then
         declare
            Item_To_Append : constant Internal_Single_Actual :=
               Item_121;
         begin
            Internal_Single_Actual_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Single_Actual_Array
        (Items_Count => Natural (Internal_Single_Actual_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Actual_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Actual_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Actual_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3000
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3001
Fld_10 := Get_Result.Assoc;
--# expr-done 3002
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 3003
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3004
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_2.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 3005
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Fld_12.Node, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 3006
   If_Result := Fld_13;
else
   --# expr-start 3010 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5153
--# expr-start 3007 '<FieldAccess .type_def>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:5153
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3007
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_14; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind dtd Match_0
--# expr-start 3008 '<FieldAccess .base_type>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:5155
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3008
Var_Expr_2 := Fld_15;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr_2
--# expr-start 3009 '<FieldAccess .iterable_comp_type>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:5155
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_2.Node, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 3009
Scope_Result_1 := Fld_16;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 3010
   If_Result := Match_Result;
end if;
   If_Result_1 := If_Result;
end if;
   If_Result_2 := If_Result_1;
end if;
--# expr-done 3011
Var_Expr := If_Result_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_4 := Var_Expr;
   --# end
   Result_Var_3 := Scope_Result_4;
else
--# expr-start 3012 '<FieldAccess .previous_part(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:5158
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_17 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 3012
Var_Expr_3 := Fld_17;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr_3
--# expr-start 3013 '<FieldAccess .iterable_comp_type>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:5159
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_18 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 3013
Scope_Result_5 := Fld_18;
   --# end
   Result_Var_2 := Scope_Result_5;
else
   Result_Var_2 := No_Entity_Base_Type_Decl;
end if;
   Result_Var_3 := Result_Var_2;
end if;
Dyn_Var_Bind_Result_1 := Result_Var_3; 
--# expr-done 3014
Let_Result_2 := Dyn_Var_Bind_Result_1; 
--# end
--# expr-done 3015
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_420;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_420;
               Finalizer_Scope_2005;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Iterable_Comp_Type;
--# end
--# property-start TypeDecl.discrete_range /usr/src/packages/BUILD/ada/ast.py:5162
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discrete_Range
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3018 '<Block>' Let_Result None
--# scope-start
--# expr-start 3017 '<FieldAccess .discrete_range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5163
--# expr-start 3016 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5163
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3016
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.discrete_range'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3017
Let_Result := Fld_1; 
--# end
--# expr-done 3018
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Discrete_Range;
--# end
--# property-start TypeDecl.discriminants_list /usr/src/packages/BUILD/ada/ast.py:5166
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Discriminants_List
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_422;
            procedure Finalizer_Scope_2012;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Var_Expr : Internal_Entity_Discriminant_Part;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Discriminant_Part;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_3 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Len : Integer;
Comp_Result : Boolean;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
If_Result_2 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_422 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_422;
            procedure Finalizer_Scope_2012 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2012;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3034 '<Block>' Let_Result None
--# scope-start
--# expr-start 3020 '<bind(<DynamicVariable origin>, <Var Self>, <FieldAccess .base_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5168
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 3019 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5168
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3019
Dyn_Var_Bind_Result := Fld; 
--# expr-done 3020
Block_Var := Dyn_Var_Bind_Result; 
--# bind base_type Block_Var
--# expr-start 3021 '<FieldAccess .discriminants>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5169
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Discriminant_Part (Node => Ent.Node.Type_Decl_F_Discriminants, Info => Ent.Info);
--# expr-done 3021
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Discriminant_Part then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 3022 '<FieldAccess .abstract_formal_params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5170
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3022
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2012;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind self_discs Block_Var_1
--# expr-start 3033 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cond>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:5173
--# expr-start 3023 '<FieldAccess .origin_node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5173
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3023
Bound_Origin_1 := Fld_3; 
--# expr-start 3032 '<Cond>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:5173
--# expr-start 3024 '<FieldAccess .is_access_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5174
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 3024
if Fld_4 then
   --# expr-start 3026 '<FieldAccess .discriminants_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5175
--# expr-start 3025 '<FieldAccess .accessed_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5175
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 3025
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 3026
   If_Result_2 := Fld_6;
else
--# expr-start 3027 '<length(<Var self_discs>)>' Len /usr/src/packages/BUILD/ada/ast.py:5177
Len := Length (Block_Var_1); 
--# expr-done 3027
Comp_Result := Len > 0; 
if Comp_Result then
   If_Result_1 := Block_Var_1;
else
--# expr-start 3029 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5178
--# expr-start 3028 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5178
Is_Null := Block_Var.Node = null; 
--# expr-done 3028
Not_Val := not (Is_Null); 
--# expr-done 3029
if Not_Val then
   --# expr-start 3031 '<FieldAccess .discriminants_list>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5178
--# expr-start 3030 '<FieldAccess .base_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Type (Node => Ent.Node, Origin => Bound_Origin_1, E_Info => Ent.Info);
--# end
--# expr-done 3030
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 3031
   If_Result := Fld_8;
else
   If_Result := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 3032
Dyn_Var_Bind_Result_1 := If_Result_2; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 3033
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3034
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_422;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_422;
               Finalizer_Scope_2012;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Discriminants_List;
--# end
--# property-start TypeDecl.direct_primitives_env /usr/src/packages/BUILD/ada/ast.py:5183
pragma Warnings (Off, "is not referenced");
 function Type_Decl_F_Direct_Primitives_Env
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_423;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_423 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_423;
begin
   --# property-body-start
      if Self.LF_Present_Direct_Primitives_Env then
         Property_Result := Self.LF_Stg_Direct_Primitives_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3036 '<Block>' Let_Result None
--# scope-start
--# expr-start 3035 '<langkit.expressions.envs.DynamicLexicalEnv object at 0xf5d088e0>' Dyn_Env /usr/src/packages/BUILD/ada/ast.py:5188
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Type_Decl_P_Direct_Primitive_Subps'Access, null, False); 
--# expr-done 3035
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3036
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_423;
      Self.LF_Present_Direct_Primitives_Env := True;
      Self.LF_Stg_Direct_Primitives_Env := Property_Result;
         Inc_Ref (Property_Result);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_423;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_F_Direct_Primitives_Env;
--# end
--# property-start TypeDecl.direct_primitive_subps /usr/src/packages/BUILD/ada/ast.py:5195
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Direct_Primitive_Subps
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_424;
            procedure Finalizer_Scope_2014;
            procedure Finalizer_Scope_2015;
            procedure Finalizer_Scope_2019;
            procedure Finalizer_Scope_2021;
            procedure Finalizer_Scope_2022;
            procedure Finalizer_Scope_2024;
            procedure Finalizer_Scope_2026;
            procedure Finalizer_Scope_2028;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Array_Access;
Block_Var_2 : Internal_Inner_Env_Assoc_Array_Access;
Block_Var_3 : Internal_Inner_Env_Assoc_Array_Access;
Var_Expr : Internal_Entity;
Var_Expr_1 : Bare_Declarative_Part;
Match_0 : Internal_Entity_Base_Package_Decl;
Match_1 : Internal_Entity_Package_Body;
Match_2 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Private_Part;
Var_Expr_3 : Internal_Entity_Declarative_Part;
Var_Expr_4 : Internal_Entity_Enum_Type_Def;
Bd : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Base_Subp_Spec;
Var_Expr_6 : Internal_Entity_Basic_Decl;
Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Scope_Result : Bare_Ada_Node;
Result_Var : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_2 : Internal_Entity_Public_Part;
Fld_3 : Internal_Entity_Ada_Node_List;
Fld_4 : Bare_Ada_Node_List;
Fld_5 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Item_274 : Internal_Entity;
Bare_Item_274 : Bare_Ada_Node;
Map_Result : Internal_Entity_Array_Access;
Fld_6 : Internal_Entity_Private_Part;
Fld_7 : Internal_Entity_Ada_Node_List;
Scope_Result_1 : Internal_Entity_Ada_Node_List;
Result_Var_1 : Internal_Entity_Ada_Node_List;
Fld_8 : Bare_Ada_Node_List;
Fld_9 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Item_275 : Internal_Entity;
Bare_Item_275 : Bare_Ada_Node;
Map_Result_1 : Internal_Entity_Array_Access;
Concat_Result : Internal_Entity_Array_Access;
Fld_10 : Internal_Entity_Declarative_Part;
Fld_11 : Internal_Entity_Ada_Node_List;
Fld_12 : Bare_Ada_Node_List;
Fld_13 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity;
Item_276 : Internal_Entity;
Bare_Item_276 : Bare_Ada_Node;
Map_Result_2 : Internal_Entity_Array_Access;
Fld_14 : Internal_Entity;
Fld_15 : Internal_Entity;
Fld_16 : Internal_Entity;
Cast_Result : Internal_Entity_Declarative_Part;
Fld_17 : Internal_Entity_Ada_Node_List;
Fld_18 : Bare_Ada_Node_List;
Fld_19 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity;
Item_277 : Internal_Entity;
Bare_Item_277 : Bare_Ada_Node;
Map_Result_3 : Internal_Entity_Array_Access;
Scope_Result_2 : Internal_Entity_Array_Access;
Result_Var_2 : Internal_Entity_Array_Access;
Fld_20 : Internal_Entity_Type_Def;
Is_A : Boolean;
If_Result_5 : Internal_Entity_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Cast_Result_2 : Internal_Entity_Package_Body;
Let_Result_1 : Internal_Entity_Array_Access;
Scope_Result_4 : Internal_Entity_Array_Access;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Array_Access;
Scope_Result_5 : Internal_Entity_Array_Access;
Match_Result : Internal_Entity_Array_Access;
Scope_Result_6 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
Fld_21 : Internal_Entity_Type_Def;
Cast_Result_4 : Internal_Entity_Enum_Type_Def;
Fld_22 : Internal_Entity_Enum_Literal_Decl_List;
Fld_23 : Bare_Enum_Literal_Decl_List;
Fld_24 : Internal_Entity_Info;
As_Entity_5 : Internal_Entity_Enum_Literal_Decl;
Is_Null_5 : Boolean;
If_Result_6 : Internal_Entity_Enum_Literal_Decl;
Fld_25 : Internal_Entity_Defining_Name;
Fld_26 : Symbol_Type;
Fld_27 : Bare_Enum_Literal_Decl;
Cast_Expr : Bare_Type_Decl;
Cast_Result_5 : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Cast_Result_6 : Bare_Ada_Node;
New_Struct_1 : Internal_Inner_Env_Assoc;
Item_122 : Internal_Entity_Enum_Literal_Decl;
Bare_Item_122 : Bare_Enum_Literal_Decl;
Map_Result_4 : Internal_Inner_Env_Assoc_Array_Access;
Scope_Result_7 : Internal_Inner_Env_Assoc_Array_Access;
Result_Var_4 : Internal_Inner_Env_Assoc_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Fld_28 : Internal_Entity_Defining_Name;
Fld_29 : Symbol_Type;
Fld_30 : Bare_Basic_Decl;
Cast_Expr_2 : Bare_Type_Decl;
Cast_Result_8 : Bare_Ada_Node;
New_Struct_2 : Internal_Metadata;
Cast_Result_9 : Bare_Ada_Node;
New_Struct_3 : Internal_Inner_Env_Assoc;
Let_Result_3 : Internal_Inner_Env_Assoc;
Cast_Expr_3 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Basic_Decl;
Fld_31 : Internal_Entity_Base_Subp_Spec;
Scope_Result_8 : Internal_Entity_Base_Subp_Spec;
Result_Var_5 : Internal_Entity_Base_Subp_Spec;
Fld_32 : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result_9 : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_11 : Internal_Entity;
Cast_Expr_5 : Internal_Entity_Type_Decl;
Cast_Result_12 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Map_Result_5 : Internal_Inner_Env_Assoc_Array_Access;
Concat_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result_4 : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_424 is
            begin
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_424;
            procedure Finalizer_Scope_2014 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2014;
            procedure Finalizer_Scope_2015 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2015;
            procedure Finalizer_Scope_2019 is
            begin
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2019;
            procedure Finalizer_Scope_2021 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_2021;
            procedure Finalizer_Scope_2022 is
            begin
                     Dec_Ref (Map_Result_3);
            end Finalizer_Scope_2022;
            procedure Finalizer_Scope_2024 is
            begin
                     Dec_Ref (Map_Result_4);
            end Finalizer_Scope_2024;
            procedure Finalizer_Scope_2026 is
            begin
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Result_Var_6);
            end Finalizer_Scope_2026;
            procedure Finalizer_Scope_2028 is
            begin
                     Dec_Ref (Fld_32);
            end Finalizer_Scope_2028;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Direct_Primitive_Subps,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 3082 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 3037 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:5201
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3037
Block_Var := Fld; 
--# bind scope Block_Var
--# expr-start 3039 '<as_entity(<Then <Var scope>: <Var Var_Expr_1> <parent(<Var Var_Expr_1>)>>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5202
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3038 '<parent(<Var Var_Expr_1>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5202
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Var_Expr_1.Parent; 
--# expr-done 3038
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 3039
Var_Expr := If_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 3059 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5202
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Internal .. Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Base_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind pkg_decl Match_0
--# expr-start 3046 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:5204
--# expr-start 3042 '<take_while>' Map_Result None
--# expr-start 3041 '<FieldAccess .decls>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5204
--# expr-start 3040 '<FieldAccess .public_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5204
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Public_Part (Node => Match_0.Node.Base_Package_Decl_F_Public_Part, Info => Match_0.Info);
--# expr-done 3040
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Ada_Node_List (Node => Fld_2.Node.Declarative_Part_F_Decls, Info => Fld_2.Info);
--# expr-done 3041
Fld_4 := Fld_3.Node;
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_4 = null then
         Map_Result := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_4;
      begin
         for Untyped_Item_274 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_274 := Untyped_Item_274; 
Is_Null_1 := Bare_Item_274 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_5 := Fld_3.Info;
As_Entity_1 := (Info => Fld_5, Node => Bare_Item_274); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_274 := If_Result_1; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_274;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 3042
--# expr-start 3045 '<take_while>' Map_Result_1 None
--# expr-start 3043 '<FieldAccess .private_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5205
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Private_Part (Node => Match_0.Node.Base_Package_Decl_F_Private_Part, Info => Match_0.Info);
--# expr-done 3043
Var_Expr_2 := Fld_6;
if Var_Expr_2 /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 3044 '<FieldAccess .decls>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5205
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_2.Node.Declarative_Part_F_Decls, Info => Var_Expr_2.Info);
--# expr-done 3044
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Ada_Node_List;
end if;
Fld_8 := Result_Var_1.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result_1 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_8;
      begin
         for Untyped_Item_275 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_275 := Untyped_Item_275; 
Is_Null_2 := Bare_Item_275 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
Fld_9 := Result_Var_1.Info;
As_Entity_2 := (Info => Fld_9, Node => Bare_Item_275); 
   If_Result_2 := As_Entity_2;
end if;
                  Item_275 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_275;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 3045
Concat_Result := Concat (Map_Result, Map_Result_1); 
--# expr-done 3046
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2015;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind pkg_body Match_1
--# expr-start 3049 '<take_while>' Map_Result_2 None
--# expr-start 3048 '<FieldAccess .decls>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5207
--# expr-start 3047 '<FieldAccess .decls>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:5207
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Declarative_Part (Node => Match_1.Node.Package_Body_F_Decls, Info => Match_1.Info);
--# expr-done 3047
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Ada_Node_List (Node => Fld_10.Node.Declarative_Part_F_Decls, Info => Fld_10.Info);
--# expr-done 3048
Fld_12 := Fld_11.Node;
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_12 = null then
         Map_Result_2 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_12;
      begin
         for Untyped_Item_276 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_276 := Untyped_Item_276; 
Is_Null_3 := Bare_Item_276 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity;
else
Fld_13 := Fld_11.Info;
As_Entity_3 := (Info => Fld_13, Node => Bare_Item_276); 
   If_Result_3 := As_Entity_3;
end if;
                  Item_276 := If_Result_3; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_276;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
      end if;
end;
--# expr-done 3049
Let_Result_1 := Map_Result_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_4 := Let_Result_1;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2019;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Spec | Ada_Discrete_Base_Subtype_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
--# expr-start 3058 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:5208
--# expr-start 3051 '<IsA DerivedTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:5211
--# expr-start 3050 '<FieldAccess .type_def>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:5211
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3050
Is_A := Fld_20.Node /= null 
and then Fld_20.Node.Kind in Ada_Derived_Type_Def_Range; 
--# expr-done 3051
if Is_A then
--# expr-start 3055 '<Cast to DeclarativePart>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5212
--# expr-start 3054 '<parent(<parent(<parent(<Var Ent>)>)>)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:5212
--# expr-start 3053 '<parent(<parent(<Var Ent>)>)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:5212
--# expr-start 3052 '<parent(<Var Ent>)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:5212
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3052
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Parent (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# expr-done 3053
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Parent (Node => Fld_15.Node, E_Info => Fld_15.Info);
--# expr-done 3054
   if Fld_16.Node = null
      or else Fld_16.Node.Kind in Ada_Declarative_Part_Range
   then
      Cast_Result := Create_Internal_Entity_Declarative_Part
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
   else
         Cast_Result := No_Entity_Declarative_Part;
   end if;
--# expr-done 3055
Var_Expr_3 := Cast_Result;
if Var_Expr_3 /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind dp Var_Expr_3
--# expr-start 3057 '<take_while>' Map_Result_3 None
--# expr-start 3056 '<FieldAccess .decls>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:5213
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Ada_Node_List (Node => Var_Expr_3.Node.Declarative_Part_F_Decls, Info => Var_Expr_3.Info);
--# expr-done 3056
Fld_18 := Fld_17.Node;
declare
   Map_Result_3_Vec : Internal_Entity_Vectors.Vector;
begin
      if Fld_18 = null then
         Map_Result_3 := Create_Internal_Entity_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_18;
      begin
         for Untyped_Item_277 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_277 := Untyped_Item_277; 
Is_Null_4 := Bare_Item_277 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity;
else
Fld_19 := Fld_17.Info;
As_Entity_4 := (Info => Fld_19, Node => Bare_Item_277); 
   If_Result_4 := As_Entity_4;
end if;
                  Item_277 := If_Result_4; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_277;
         begin
            Internal_Entity_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_3 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_3_Vec);
      end if;
end;
--# expr-done 3057
Scope_Result_2 := Map_Result_3;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2022;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_5 := Result_Var_2;
else
   If_Result_5 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 3058
Let_Result_2 := If_Result_5; Inc_Ref (Let_Result_2);
--# end
Scope_Result_5 := Let_Result_2;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2021;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 3059
Scope_Result_6 := Match_Result;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2014;
   Result_Var_3 := Scope_Result_6;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_1 := Result_Var_3; Inc_Ref (Block_Var_1);
--# bind pkg_decls Block_Var_1
--# expr-start 3061 '<Cast to EnumTypeDef>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:5218
--# expr-start 3060 '<FieldAccess .type_def>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:5218
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3060
   if Fld_21.Node = null
      or else Fld_21.Node.Kind in Ada_Enum_Type_Def_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Enum_Type_Def
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
   else
         Cast_Result_4 := No_Entity_Enum_Type_Def;
   end if;
--# expr-done 3061
Var_Expr_4 := Cast_Result_4;
if Var_Expr_4 /= No_Entity_Enum_Type_Def then
   --# scope-start
   --# bind etf Var_Expr_4
--# expr-start 3068 '<map>' Map_Result_4 /usr/src/packages/BUILD/ada/ast.py:5219
--# expr-start 3062 '<FieldAccess .enum_literals>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:5219
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Enum_Literal_Decl_List (Node => Var_Expr_4.Node.Enum_Type_Def_F_Enum_Literals, Info => Var_Expr_4.Info);
--# expr-done 3062
Fld_23 := Fld_22.Node;
declare
   Map_Result_4_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
begin
      if Fld_23 = null then
         Map_Result_4 := Create_Internal_Inner_Env_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Enum_Literal_Decl_List := Fld_23;
      begin
         for Untyped_Item_122 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_122 := Untyped_Item_122; 
Is_Null_5 := Bare_Item_122 = null; 
if Is_Null_5 then
   If_Result_6 := No_Entity_Enum_Literal_Decl;
else
Fld_24 := Fld_22.Info;
As_Entity_5 := (Info => Fld_24, Node => Bare_Item_122); 
   If_Result_6 := As_Entity_5;
end if;
                  Item_122 := If_Result_6; 
   --# scope-start
               --# bind lit Item_122
      --# expr-start 3067 '<New InternalInnerEnvAssoc>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:5220
--# expr-start 3064 '<FieldAccess .name_symbol>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:5221
--# expr-start 3063 '<FieldAccess .defining_name>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:5221
      if Item_122.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_122.Node, E_Info => Item_122.Info);
--# end
--# expr-done 3063
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_25.Node);
--# end
--# expr-done 3064
--# expr-start 3065 '<New InternalMetadata>' New_Struct /usr/src/packages/BUILD/ada/ast.py:5223
Cast_Expr := Self; 
      Cast_Result_5 := Cast_Expr;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => Cast_Result_5, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3065
--# expr-start 3066 '<FieldAccess .node>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:5222
Fld_27 := Item_122.Node;
--# expr-done 3066
      Cast_Result_6 := Fld_27;
New_Struct_1 := (Key => Fld_26, Metadata => New_Struct, Val => Cast_Result_6); 
--# expr-done 3067
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc :=
               New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_4 := Create_Internal_Inner_Env_Assoc_Array
        (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_4_Vec)));
      for I in Map_Result_4.Items'Range loop
         Map_Result_4.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
           (Map_Result_4_Vec,
            I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_4.Items'First);
      end loop;
      Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_4_Vec);
      end if;
end;
--# expr-done 3068
Scope_Result_7 := Map_Result_4;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2024;
   Result_Var_4 := Scope_Result_7;
else
   Result_Var_4 := No_Internal_Inner_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var_4);
Block_Var_2 := Result_Var_4; Inc_Ref (Block_Var_2);
--# bind enum_lits Block_Var_2
--# expr-start 3080 '<filter_map>' Map_Result_5 /usr/src/packages/BUILD/ada/ast.py:5227
declare
   Map_Result_5_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_123 of
               Collection.Items
         loop
   --# scope-start
               --# bind decl Item_123
--# expr-start 3069 '<Cast to BasicDecl>' Cast_Result_10 /usr/src/packages/BUILD/ada/ast.py:5237
Cast_Expr_3 := Item_123; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_10 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3069
Var_Expr_6 := Cast_Result_10;
if Var_Expr_6 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3070 '<FieldAccess .subp_spec_or_null>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:5237
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_31 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 3070
Scope_Result_8 := Fld_31;
   --# end
   Result_Var_5 := Scope_Result_8;
else
   Result_Var_5 := No_Entity_Base_Subp_Spec;
end if;
Var_Expr_5 := Result_Var_5;
if Var_Expr_5 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3071 '<FieldAccess .get_primitive_subp_types>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:5237
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_types
Fld_32 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_Types (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 3071
Scope_Result_9 := Fld_32;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2028;
   Result_Var_6 := Scope_Result_9;
else
   Result_Var_6 := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Result_Var_6;
   begin
      for Item_124 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_124
         --# expr-start 3072 '<Eq>' Is_Equiv None
Cast_Expr_4 := Item_124; 
      Cast_Result_11 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
Cast_Expr_5 := Ent; 
      Cast_Result_12 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
Is_Equiv := Equivalent (Cast_Result_11, Cast_Result_12); 
--# expr-done 3072
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
      --# expr-start 3079 '<Let bd>' Let_Result_3 /usr/src/packages/BUILD/ada/ast.py:5228
--# scope-start
--# expr-start 3073 '<Cast to BasicDecl>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:5229
Cast_Expr_1 := Item_123; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_7 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3073
Bd := Cast_Result_7; 
--# bind bd Bd
--# expr-start 3078 '<New InternalInnerEnvAssoc>' New_Struct_3 /usr/src/packages/BUILD/ada/ast.py:5229
--# expr-start 3075 '<FieldAccess .name_symbol>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:5230
--# expr-start 3074 '<FieldAccess .defining_name>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:5230
      if Bd.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_28 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Bd.Node, E_Info => Bd.Info);
--# end
--# expr-done 3074
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_28.Node);
--# end
--# expr-done 3075
--# expr-start 3076 '<New InternalMetadata>' New_Struct_2 /usr/src/packages/BUILD/ada/ast.py:5232
Cast_Expr_2 := Self; 
      Cast_Result_8 := Cast_Expr_2;
New_Struct_2 := (Access_Entity => False, Dottable_Subp => False, Primitive => Cast_Result_8, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3076
--# expr-start 3077 '<FieldAccess .node>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:5231
Fld_30 := Bd.Node;
--# expr-done 3077
      Cast_Result_9 := Fld_30;
New_Struct_3 := (Key => Fld_29, Metadata => New_Struct_2, Val => Cast_Result_9); 
--# expr-done 3078
Let_Result_3 := New_Struct_3; 
--# end
--# expr-done 3079
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc :=
               Let_Result_3;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2026;
         end loop;
      end;
      Map_Result_5 := Create_Internal_Inner_Env_Assoc_Array
        (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_5_Vec)));
      for I in Map_Result_5.Items'Range loop
         Map_Result_5.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
           (Map_Result_5_Vec,
            I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result_5.Items'First);
      end loop;
      Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_5_Vec);
end;
--# expr-done 3080
Block_Var_3 := Map_Result_5; Inc_Ref (Block_Var_3);
--# bind prim_subps Block_Var_3
--# expr-start 3081 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:5241
Concat_Result_1 := Concat (Block_Var_2, Block_Var_3); 
--# expr-done 3081
Let_Result_4 := Concat_Result_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 3082
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_424;
         Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                     As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_424;
               Finalizer_Scope_2014;
               Finalizer_Scope_2015;
               Finalizer_Scope_2019;
               Finalizer_Scope_2021;
               Finalizer_Scope_2022;
               Finalizer_Scope_2024;
               Finalizer_Scope_2026;
               Finalizer_Scope_2028;
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Direct_Primitive_Subps;
--# end
--# property-start TypeDecl.array_ndims /usr/src/packages/BUILD/ada/ast.py:5243
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Ndims
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3084 '<FieldAccess .array_ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5243
--# expr-start 3083 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5243
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3083
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3084
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Array_Ndims;
--# end
--# property-start TypeDecl.is_record_type /usr/src/packages/BUILD/ada/ast.py:5245
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Record_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3086 '<FieldAccess .is_record_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5245
--# expr-start 3085 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5245
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3085
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_record_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Record_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3086
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Record_Type;
--# end
--# property-start TypeDecl.is_real_type /usr/src/packages/BUILD/ada/ast.py:5246
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Real_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3088 '<FieldAccess .is_real_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5246
--# expr-start 3087 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5246
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3087
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.is_real_type
Fld_1 := Libadalang.Implementation.Type_Def_P_Is_Real_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3088
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Real_Type;
--# end
--# property-start TypeDecl.is_float_type /usr/src/packages/BUILD/ada/ast.py:5247
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Float_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3090 '<FieldAccess .is_float_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5247
--# expr-start 3089 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5247
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3089
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3090
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Float_Type;
--# end
--# property-start TypeDecl.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:5248
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Fixed_Point
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3092 '<FieldAccess .is_fixed_point>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5248
--# expr-start 3091 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5248
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3091
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3092
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Fixed_Point;
--# end
--# property-start TypeDecl.is_int_type /usr/src/packages/BUILD/ada/ast.py:5249
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Int_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3094 '<FieldAccess .is_int_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5249
--# expr-start 3093 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5249
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3093
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3094
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Int_Type;
--# end
--# property-start TypeDecl.is_access_type /usr/src/packages/BUILD/ada/ast.py:5250
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Access_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3097 '<FieldAccess .is_access_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5250
--# expr-start 3096 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5250
--# expr-start 3095 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5250
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3095
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 3096
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Access_Type (Node => Fld.Node, Origin => Origin);
--# end
--# expr-done 3097
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Access_Type;
--# end
--# property-start TypeDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:5251
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Static_Decl
  (Node : Bare_Type_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3100 '<FieldAccess .is_static>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5251
--# expr-start 3099 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5251
--# expr-start 3098 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5251
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Type_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3098
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => If_Result.Node.Type_Decl_F_Type_Def, Info => If_Result.Info);
--# expr-done 3099
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 3100
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Static_Decl;
--# end
--# property-start TypeDecl.accessed_type /usr/src/packages/BUILD/ada/ast.py:5254
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Accessed_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_433;
      Block_Var : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Fld_4 : Lexical_Env := Empty_Env;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_5 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_433 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_433;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3114 '<Block>' Let_Result None
--# scope-start
--# expr-start 3101 '<FieldAccess .get_imp_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:5255
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3101
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 3113 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5257
--# expr-start 3102 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5258
Is_Null := Block_Var.Node = null; 
--# expr-done 3102
if Is_Null then
   --# expr-start 3104 '<FieldAccess .accessed_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5259
--# expr-start 3103 '<FieldAccess .type_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5259
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3103
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3104
   If_Result := Fld_2;
else
   --# expr-start 3112 '<FieldAccess .accessed_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5263
--# expr-start 3111 '<FieldAccess .formal_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5263
--# expr-start 3110 '<Cast to BaseFormalParamDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5263
--# expr-start 3109 '<EnvGet(<FieldAccess .defining_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:5263
--# expr-start 3106 '<FieldAccess .defining_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5263
--# expr-start 3105 '<FieldAccess .type_def>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5263
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3105
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 3106
--# expr-start 3108 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5264
--# expr-start 3107 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5264
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 3107
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Cast_Result.Node);
--# end
--# expr-done 3108
Env_Get_Result := AST_Envs.Get_First (Self => Fld_4, Key => Fld_5, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 3109
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Formal_Param_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3110
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Cast_Result_1.Node, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 3111
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_6.Node, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 3112
   If_Result := Fld_7;
end if;
--# expr-done 3113
Let_Result := If_Result; 
--# end
--# expr-done 3114
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_433;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_433;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Accessed_Type;
--# end
--# property-start TypeDecl.access_def /usr/src/packages/BUILD/ada/ast.py:5273
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Access_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Access_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Access_Def;
      Match_0 : Internal_Entity_Access_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Access_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Let_Result : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Access_Def;
Scope_Result_1 : Internal_Entity_Access_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Access_Def;
Scope_Result_2 : Internal_Entity_Access_Def;
Match_Result : Internal_Entity_Access_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3118 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5273
--# expr-start 3115 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5273
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3115
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ad Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 3117 '<FieldAccess .access_def>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5275
--# expr-start 3116 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5275
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3116
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3117
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Access_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3118
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Access_Def;
--# end
--# property-start TypeDecl.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:5279
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Tagged_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3120 '<FieldAccess .is_tagged_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5279
--# expr-start 3119 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5279
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3119
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3120
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Tagged_Type;
--# end
--# property-start TypeDecl.base_type /usr/src/packages/BUILD/ada/ast.py:5280
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3122 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5280
--# expr-start 3121 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5280
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3121
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3122
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Base_Type;
--# end
--# property-start TypeDecl.base_interfaces /usr/src/packages/BUILD/ada/ast.py:5281
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Base_Interfaces
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_437;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_437 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_437;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3124 '<FieldAccess .base_interfaces>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5281
--# expr-start 3123 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5281
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3123
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3124
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_437;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_437;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Base_Interfaces;
--# end
--# property-start TypeDecl.is_char_type /usr/src/packages/BUILD/ada/ast.py:5282
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Char_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3126 '<FieldAccess .is_char_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5282
--# expr-start 3125 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5282
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3125
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3126
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Char_Type;
--# end
--# property-start TypeDecl.is_enum_type /usr/src/packages/BUILD/ada/ast.py:5283
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Enum_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3128 '<FieldAccess .is_enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5283
--# expr-start 3127 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5283
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3127
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3128
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Enum_Type;
--# end
--# property-start TypeDecl.is_private /usr/src/packages/BUILD/ada/ast.py:5284
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Private
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Derived_Type_Def;
Fld : Bare_Type_Def;
Is_A : Boolean;
Fld_1 : Bare_Type_Def;
Cast_Result : Bare_Derived_Type_Def;
Fld_2 : Bare_With_Private;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3130 '<IsA PrivateTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:5285
--# expr-start 3129 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5285
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 3129
Is_A := Fld /= null 
and then Fld.Kind in Ada_Private_Type_Def_Range; 
--# expr-done 3130
if Is_A then
   If_Result := True;
else
--# expr-start 3132 '<Cast to DerivedTypeDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5286
--# expr-start 3131 '<FieldAccess .type_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5286
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Type_Decl_F_Type_Def;
--# expr-done 3131
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Derived_Type_Def_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3132
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind dtd Var_Expr
--# expr-start 3134 '<FieldAccess .as_bool>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5287
--# expr-start 3133 '<FieldAccess .has_with_private>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5287
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Derived_Type_Def_F_Has_With_Private;
--# expr-done 3133
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_3 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_2);
--# end
--# expr-done 3134
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Private;
--# end
--# property-start TypeDecl.array_def /usr/src/packages/BUILD/ada/ast.py:5291
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Array_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Type_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Type_Def;
      Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Array_Type_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Array_Type_Def;
Scope_Result_1 : Internal_Entity_Array_Type_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Array_Type_Def;
Scope_Result_2 : Internal_Entity_Array_Type_Def;
Match_Result : Internal_Entity_Array_Type_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3138 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5291
--# expr-start 3135 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5291
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3135
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind atd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind dtd Match_1
--# expr-start 3137 '<FieldAccess .array_def>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5293
--# expr-start 3136 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5293
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3136
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.array_def'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Array_Def (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3137
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Array_Type_Def; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3138
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Array_Def;
--# end
--# property-start TypeDecl.root_type /usr/src/packages/BUILD/ada/ast.py:5297
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Root_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Derived_Type_Def;
Match_1 : Internal_Entity_Type_Def;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Derived_Type_Def;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Def;
Cast_Expr : Internal_Entity_Type_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3142 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5297
--# expr-start 3139 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5297
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3139
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind dtd Match_0
--# expr-start 3141 '<FieldAccess .root_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5298
--# expr-start 3140 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5298
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3140
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3141
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Cast_Expr := Ent; 
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3142
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Root_Type;
--# end
--# property-start TypeDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:5303
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Defining_Env
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_443;
            procedure Finalizer_Scope_2047;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Expr;
Fld_1 : Internal_Entity_Type_Def;
Fld_2 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Basic_Decl_Array_Access;
Fld_7 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Array_Lit_1 : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_443 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_443;
            procedure Finalizer_Scope_2047 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2047;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3162 '<Block>' Let_Result None
--# scope-start
--# expr-start 3143 '<FieldAccess .get_imp_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:5304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.get_imp_deref'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Get_Imp_Deref (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3143
Block_Var := Fld; 
--# bind imp_deref Block_Var
--# expr-start 3145 '<FieldAccess .defining_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5308
--# expr-start 3144 '<FieldAccess .type_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5308
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3144
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.defining_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3145
Block_Var_1 := Fld_2; Inc_Ref (Block_Var_1);
--# bind self_env Block_Var_1
--# expr-start 3161 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:5310
--# expr-start 3147 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5311
--# expr-start 3146 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5311
Is_Null := Block_Var.Node = null; 
--# expr-done 3146
Not_Val := not (Is_Null); 
--# expr-done 3147
if Not_Val then
   --# expr-start 3151 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:5312
--# expr-start 3150 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:5312
--# expr-start 3149 '<FieldAccess .defining_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5312
--# expr-start 3148 '<FieldAccess .accessed_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5312
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3148
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 3149
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_1, 2 => Fld_4)); 
--# expr-done 3150
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3151
   If_Result_1 := Group_Env;
else
--# expr-start 3152 '<FieldAccess .has_ud_indexing>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5314
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3152
if Fld_5 then
   --# expr-start 3160 '<env_group(<Concat>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:5315
--# expr-start 3159 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:5315
--# expr-start 3157 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5315
--# expr-start 3155 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:5315
--# expr-start 3153 '<FieldAccess .constant_indexing_fns>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5315
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3153
--# expr-start 3154 '<FieldAccess .variable_indexing_fns>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5316
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3154
Concat_Result := Concat (Fld_6, Fld_7); 
--# expr-done 3155
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_125 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_125
      --# expr-start 3156 '<FieldAccess .defining_env>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5317
      if Item_125.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_125.Node, Origin => Origin, E_Info => Item_125.Info);
--# end
--# expr-done 3156
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_8;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2047;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3157
--# expr-start 3158 '<ArrayLiteral>' Array_Lit_1 None
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_1)); 
--# expr-done 3158
Concat_Result_1 := Concat (Map_Result, Array_Lit_1); 
--# expr-done 3159
Group_Env_1 := Group (Concat_Result_1, No_Metadata); 
--# expr-done 3160
   If_Result := Group_Env_1;
else
   If_Result := Block_Var_1;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3161
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3162
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_443;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_443;
               Finalizer_Scope_2047;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Defining_Env;
--# end
--# property-start TypeDecl.record_def /usr/src/packages/BUILD/ada/ast.py:5337
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Record_Def
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Record_Def
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Record_Def;
      Match_0 : Internal_Entity_Record_Type_Def;
Match_1 : Internal_Entity_Derived_Type_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Type_Def;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Base_Record_Def;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Record_Def;
Scope_Result : Internal_Entity_Base_Record_Def;
Result_Var : Internal_Entity_Base_Record_Def;
Fld_5 : Boolean;
If_Result : Internal_Entity_Base_Record_Def;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Record_Type_Def;
Let_Result : Internal_Entity_Base_Record_Def;
Scope_Result_1 : Internal_Entity_Base_Record_Def;
Cast_Result_1 : Internal_Entity_Derived_Type_Def;
Let_Result_1 : Internal_Entity_Base_Record_Def;
Scope_Result_2 : Internal_Entity_Base_Record_Def;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Internal_Entity_Base_Record_Def;
Scope_Result_3 : Internal_Entity_Base_Record_Def;
Match_Result : Internal_Entity_Base_Record_Def;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3170 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5338
--# expr-start 3163 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5338
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3163
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Record_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Record_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind r Match_0
--# expr-start 3164 '<FieldAccess .record_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5339
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Base_Record_Def (Node => Match_0.Node.Record_Type_Def_F_Record_Def, Info => Match_0.Info);
--# expr-done 3164
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Derived_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Derived_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind d Match_1
--# expr-start 3169 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5342
--# expr-start 3165 '<FieldAccess .is_tagged_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5343
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_tagged_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3165
if Fld_5 then
   --# expr-start 3166 '<FieldAccess .record_extension>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5344
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Record_Def (Node => Match_1.Node.Derived_Type_Def_F_Record_Extension, Info => Match_1.Info);
--# expr-done 3166
   If_Result := Fld_2;
else
--# expr-start 3167 '<FieldAccess .base_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5345
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3167
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 3168 '<FieldAccess .record_def>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5345
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3168
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Record_Def;
end if;
   If_Result := Result_Var;
end if;
--# expr-done 3169
Let_Result_1 := If_Result; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Record_Def; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 3170
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Record_Def;
--# end
--# property-start TypeDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:5351
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Xref_Entry_Point;
--# end
--# property-start TypeDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:5354
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Xref_Equation
  (Node : Bare_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_446;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_446 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_446;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3173 '<Block>' Let_Result None
--# scope-start
--# expr-start 3172 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5356
--# expr-start 3171 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5356
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3171
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3172
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3173
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_446;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_446;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Xref_Equation;
--# end
--# property-start TypeDecl.is_discrete_type /usr/src/packages/BUILD/ada/ast.py:5358
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Is_Discrete_Type
  (Node : Bare_Type_Decl
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Type_Def;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3175 '<FieldAccess .is_discrete_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5358
--# expr-start 3174 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5358
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3174
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Discrete_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3175
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Is_Discrete_Type;
--# end
--# property-start TypeDecl.own_primitives_env /usr/src/packages/BUILD/ada/ast.py:5361
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Env
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_448;
      Fld : Lexical_Env := Empty_Env;
Rebound_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_448 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Rebound_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_448;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3178 '<Block>' Let_Result None
--# scope-start
--# expr-start 3177 '<rebind_env(<FieldAccess .direct_primitives_env>, <Var with_rebindings>)>' Rebound_Env /usr/src/packages/BUILD/ada/ast.py:5366
--# expr-start 3176 '<FieldAccess .direct_primitives_env>' Fld /usr/src/packages/BUILD/ada/ast.py:5366
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.direct_primitives_env
Fld := Libadalang.Implementation.Type_Decl_F_Direct_Primitives_Env (Node => Ent.Node);
--# end
--# expr-done 3176
Rebound_Env := Rebind_Env (Fld, With_Rebindings); 
--# expr-done 3177
Let_Result := Rebound_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3178
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_448;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_448;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Own_Primitives_Env;
--# end
--# property-start TypeDecl.own_primitives_envs /usr/src/packages/BUILD/ada/ast.py:5369
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Own_Primitives_Envs
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_449;
            procedure Finalizer_Scope_2053;
      Var_Expr : Internal_Entity_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Scope_Result : Lexical_Env_Array_Access;
Fld_3 : Lexical_Env := Empty_Env;
Singleton : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_449 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_449;
            procedure Finalizer_Scope_2053 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
            end Finalizer_Scope_2053;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3185 '<Block>' Let_Result None
--# scope-start
--# expr-start 3180 '<Cast to TypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5376
--# expr-start 3179 '<FieldAccess .previous_part(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5376
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Ent.Node, Go_To_Incomplete => False, E_Info => Ent.Info);
--# end
--# expr-done 3179
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3180
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind pp Var_Expr
--# expr-start 3183 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:5377
--# expr-start 3181 '<FieldAccess .own_primitives_env(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5378
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_1 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3181
--# expr-start 3182 '<FieldAccess .own_primitives_env(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5379
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_2 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr.Info);
--# end
--# expr-done 3182
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_2)); 
--# expr-done 3183
Scope_Result := Array_Lit;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2053;
   Result_Var := Scope_Result;
else
   --# expr-start 3184 '<FieldAccess .own_primitives_env(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5381
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_env
Fld_3 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Env (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3184
                Singleton := Create_Lexical_Env_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
                Inc_Ref (Singleton.Items (1));
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3185
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_449;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_449;
               Finalizer_Scope_2053;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Own_Primitives_Envs;
--# end
--# property-start TypeDecl.primitives_envs /usr/src/packages/BUILD/ada/ast.py:5385
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Envs
  (Node : Bare_Type_Decl
      ; With_Rebindings : Env_Rebindings
      ; Include_Self : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind with_rebindings With_Rebindings
   --# bind include_self Include_Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_450;
            procedure Finalizer_Scope_2054;
            procedure Finalizer_Scope_2058;
      Var_Expr : Internal_Entity_Type_Decl;
Match_0 : Internal_Entity_Type_Decl;
Match_1 : Internal_Entity_Subtype_Decl;
Match_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Bare_Subtype_Decl;
Fld_2 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Type_Decl;
Match_Prefix : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
Scope_Result : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Subtype_Decl;
Let_Result_1 : Internal_Entity_Type_Decl;
Scope_Result_1 : Internal_Entity_Type_Decl;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Let_Result_2 : Internal_Entity_Type_Decl;
Scope_Result_2 : Internal_Entity_Type_Decl;
Match_Result : Internal_Entity_Type_Decl;
Fld_4 : Lexical_Env_Array_Access;
Fld_5 : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Scope_Result_3 : Lexical_Env_Array_Access;
Result_Var : Lexical_Env_Array_Access;
Map_Result : Lexical_Env_Array_Access;
Fld_6 : Lexical_Env_Array_Access;
If_Result : Lexical_Env_Array_Access;
Concat_Result_1 : Lexical_Env_Array_Access;
Dyn_Var_Bind_Result_1 : Lexical_Env_Array_Access;
Let_Result_3 : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_450 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (If_Result);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_450;
            procedure Finalizer_Scope_2054 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2054;
            procedure Finalizer_Scope_2058 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2058;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3201 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3200 '<bind(<DynamicVariable origin>, <Var Self>, <Concat>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:5394
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 3199 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:5394
--# expr-start 3196 '<mapcat>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5394
--# expr-start 3186 '<FieldAccess .base_types>' Fld /usr/src/packages/BUILD/ada/ast.py:5394
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.base_types'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Base_Types (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 3186
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld;
      begin
         for Item_126 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_126
--# expr-start 3192 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5394
      if Item_126.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_126; 
case Ada_Base_Type_Decl (Match_Prefix.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind td Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subtype_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind std Match_1
--# expr-start 3191 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cast to TypeDecl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5396
--# expr-start 3188 '<FieldAccess .origin_node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5397
--# expr-start 3187 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5397
Fld_1 := Match_1.Node;
--# expr-done 3187
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld_1);
--# end
--# expr-done 3188
Bound_Origin_1 := Fld_2; 
--# expr-start 3190 '<Cast to TypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5397
--# expr-start 3189 '<FieldAccess .get_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5397
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubtypeDecl.get_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subtype_Decl_P_Get_Type (Node => Match_1.Node, Origin => Bound_Origin_1, E_Info => Match_1.Info);
--# end
--# expr-done 3189
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Type_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         Cast_Result_1 := No_Entity_Type_Decl;
   end if;
--# expr-done 3190
Dyn_Var_Bind_Result := Cast_Result_1; 
--# expr-done 3191
Let_Result_1 := Dyn_Var_Bind_Result; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Discrete_Base_Subtype_Decl | Ada_Classwide_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3192
Var_Expr := Match_Result;
if Var_Expr /= No_Entity_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 3195 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:5402
--# expr-start 3193 '<FieldAccess .own_primitives_envs(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5402
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_4 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr.Info);
--# end
--# expr-done 3193
--# expr-start 3194 '<FieldAccess .primitives_envs(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5403
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.primitives_envs
Fld_5 := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Var_Expr.Node, With_Rebindings => With_Rebindings, E_Info => Var_Expr.Info);
--# end
--# expr-done 3194
Concat_Result := Concat (Fld_4, Fld_5); 
--# expr-done 3195
Scope_Result_3 := Concat_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2058;
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (Result_Var);
         for Item_To_Append of
               Result_Var.Items
         loop
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2054;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3196
--# expr-start 3198 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5405
if Include_Self then
   --# expr-start 3197 '<FieldAccess .own_primitives_envs(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5406
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.own_primitives_envs
Fld_6 := Libadalang.Implementation.Type_Decl_P_Own_Primitives_Envs (Node => Ent.Node, With_Rebindings => With_Rebindings, E_Info => Ent.Info);
--# end
--# expr-done 3197
   If_Result := Fld_6;
else
   If_Result := No_Lexical_Env_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 3198
Concat_Result_1 := Concat (Map_Result, If_Result); 
--# expr-done 3199
Dyn_Var_Bind_Result_1 := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 3200
Let_Result_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 3201
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_450;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_450;
               Finalizer_Scope_2054;
               Finalizer_Scope_2058;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Primitives_Envs;
--# end
--# property-start TypeDecl.compute_primitives_env /usr/src/packages/BUILD/ada/ast.py:5411
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Compute_Primitives_Env
  (Node : Bare_Type_Decl
      ; Include_Self : Boolean
         := True
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind include_self Include_Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_451;
      Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env_Array_Access;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_451 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_451;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Type_Decl_Type_Decl_P_Compute_Primitives_Env,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Type_Decl,
                                As_Bare_Type_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Include_Self);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3207 '<Block>' Let_Result None
--# scope-start
--# expr-start 3206 '<env_group(<FieldAccess .primitives_envs(...)>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:5416
--# expr-start 3204 '<FieldAccess .primitives_envs(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5416
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3203 '<FieldAccess .rebindings>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5417
--# expr-start 3202 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:5417
Fld := Ent.Info;
--# expr-done 3202
Fld_1 := Fld.Rebindings;
--# expr-done 3203
--# property-call-start TypeDecl.primitives_envs
Fld_2 := Libadalang.Implementation.Type_Decl_P_Primitives_Envs (Node => Ent.Node, With_Rebindings => Fld_1, Include_Self => Include_Self, E_Info => Ent.Info);
--# end
--# expr-done 3204
--# expr-start 3205 '<New InternalMetadata>' New_Struct /usr/src/packages/BUILD/ada/ast.py:5420
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => Cast_Result); 
--# expr-done 3205
Group_Env := Group (Fld_2, New_Struct); 
--# expr-done 3206
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3207
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_451;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_451;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Compute_Primitives_Env;
--# end
--# property-start TypeDecl.parent_primitives_env /usr/src/packages/BUILD/ada/ast.py:5424
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Parent_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_452;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Bare_Type_Def;
Is_A : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_452 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_452;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3213 '<Block>' Let_Result None
--# scope-start
--# expr-start 3212 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5425
--# expr-start 3209 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:5426
--# expr-start 3208 '<FieldAccess .type_def>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5426
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Type_Decl_F_Type_Def;
--# expr-done 3208
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 3209
if Is_A then
   --# expr-start 3210 '<FieldAccess .compute_primitives_env(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5427
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => False, E_Info => Ent.Info);
--# end
--# expr-done 3210
   If_Result := Fld;
else
   --# expr-start 3211 '<FieldAccess .empty_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5428
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.empty_env
Fld_1 := Libadalang.Implementation.Ada_Node_P_Empty_Env (Node => Self);
--# end
--# expr-done 3211
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 3212
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3213
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_452;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_452;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Parent_Primitives_Env;
--# end
--# property-start TypeDecl.primitives_env /usr/src/packages/BUILD/ada/ast.py:5432
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Primitives_Env
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_453;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_453 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_453;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3215 '<Block>' Let_Result None
--# scope-start
--# expr-start 3214 '<FieldAccess .compute_primitives_env(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5433
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.compute_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Compute_Primitives_Env (Node => Ent.Node, Include_Self => True, E_Info => Ent.Info);
--# end
--# expr-done 3214
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 3215
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_453;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_453;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Primitives_Env;
--# end
--# property-start TypeDecl.get_primitives /usr/src/packages/BUILD/ada/ast.py:5436
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Primitives
  (Node : Bare_Type_Decl
      ; Only_Inherited : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind only_inherited Only_Inherited
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_454;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
A_Prim : Internal_Entity_Base_Type_Decl;
B_Prim : Internal_Entity_Base_Type_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Bare_Ada_Node;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Base_Subp_Spec;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Fld_11 : Boolean;
Not_Val : Boolean;
Fld_12 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
Quantifier_Result : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_454 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_454;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3241 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 3218 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5443
if Only_Inherited then
   --# expr-start 3216 '<FieldAccess .parent_primitives_env>' Fld /usr/src/packages/BUILD/ada/ast.py:5445
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDecl.parent_primitives_env
Fld := Libadalang.Implementation.Type_Decl_P_Parent_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3216
   If_Result := Fld;
else
   --# expr-start 3217 '<FieldAccess .primitives_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5446
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3217
   If_Result := Fld_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 3218
Block_Var := If_Result; Inc_Ref (Block_Var);
--# bind prim_env Block_Var
--# expr-start 3221 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5449
--# expr-start 3219 '<EnvGet(<Var prim_env>, <No SymbolType>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:5449
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 3219
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_127 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_127
      --# expr-start 3220 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5450
Cast_Expr := Item_127; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3220
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3221
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind bds Block_Var_1
--# expr-start 3240 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <filter>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5455
--# expr-start 3222 '<FieldAccess .origin_node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5455
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3222
Bound_Origin := Fld_2; 
--# expr-start 3239 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:5455
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_128 of
               Collection.Items
         loop
   --# scope-start
               --# bind a Item_128
Quantifier_Result := True;
   declare
      Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
   begin
      for Item_129 of
            Collection.Items
      loop
   --# scope-start
            --# bind b Item_129
         --# expr-start 3238 '<Let a_prim, b_prim>' Let_Result /usr/src/packages/BUILD/ada/ast.py:5456
--# scope-start
--# expr-start 3227 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5458
--# expr-start 3226 '<as_bare_entity(<FieldAccess .primitive>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:5458
--# expr-start 3225 '<FieldAccess .primitive>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5458
--# expr-start 3224 '<FieldAccess .md>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5458
--# expr-start 3223 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5458
Fld_3 := Item_128.Info;
--# expr-done 3223
Fld_4 := Fld_3.MD;
--# expr-done 3224
Fld_5 := Fld_4.Primitive;
--# expr-done 3225
Is_Null := Fld_5 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_5); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 3226
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3227
A_Prim := Cast_Result_1; 
--# bind a_prim A_Prim
--# expr-start 3232 '<Cast to BaseTypeDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:5459
--# expr-start 3231 '<as_bare_entity(<FieldAccess .primitive>)>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:5459
--# expr-start 3230 '<FieldAccess .primitive>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5459
--# expr-start 3229 '<FieldAccess .md>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5459
--# expr-start 3228 '<FieldAccess .info>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5459
Fld_6 := Item_129.Info;
--# expr-done 3228
Fld_7 := Fld_6.MD;
--# expr-done 3229
Fld_8 := Fld_7.Primitive;
--# expr-done 3230
Is_Null_1 := Fld_8 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_8); 
   If_Result_2 := As_Entity_1;
end if;
--# expr-done 3231
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 3232
B_Prim := Cast_Result_2; 
--# bind b_prim B_Prim
--# expr-start 3236 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5464
--# expr-start 3235 '<FieldAccess .match_signature(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5464
--# expr-start 3233 '<FieldAccess .subp_spec_or_null>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:5464
      if Item_128.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_128.Node, E_Info => Item_128.Info);
--# end
--# expr-done 3233
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3234 '<FieldAccess .subp_spec_or_null>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:5465
      if Item_129.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_129.Node, E_Info => Item_129.Info);
--# end
--# expr-done 3234
--# property-call-start BaseSubpSpec.match_signature
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_9.Node, Other => Fld_10, Match_Name => True, Use_Entity_Info => True, E_Info => Fld_9.Info);
--# end
--# expr-done 3235
Not_Val := not (Fld_11); 
--# expr-done 3236
if Not_Val then
   If_Result_3 := True;
else
   --# expr-start 3237 '<FieldAccess .is_derived_type(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:5468
      if A_Prim.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_derived_type
Fld_12 := Libadalang.Implementation.Base_Type_Decl_P_Is_Derived_Type (Node => A_Prim.Node, Other_Type => B_Prim, Origin => Bound_Origin, E_Info => A_Prim.Info);
--# end
--# expr-done 3237
   If_Result_3 := Fld_12;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 3238
            if not (Let_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
               if Quantifier_Result then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_128;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3239
Dyn_Var_Bind_Result := Map_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3240
Let_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_1);
--# end
--# expr-done 3241
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_454;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_454;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Get_Primitives;
--# end
--# property-start TypeDecl.get_imp_deref /usr/src/packages/BUILD/ada/ast.py:5473
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Get_Imp_Deref
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3243 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5474
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3242 '<Symbol Implicit_Dereference>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Implicit_Dereference); 
--# expr-done 3242
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3243
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Get_Imp_Deref;
--# end
--# property-start TypeDecl.has_ud_indexing /usr/src/packages/BUILD/ada/ast.py:5477
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Has_Ud_Indexing
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
Sym_1 : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3247 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5478
--# expr-start 3246 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5478
--# expr-start 3245 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5478
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3244 '<Symbol Constant_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant_Indexing); 
--# expr-done 3244
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3245
Is_Null := Fld.Node = null; 
--# expr-done 3246
Not_Val := not (Is_Null); 
--# expr-done 3247
if Not_Val then
   If_Result := True;
else
   --# expr-start 3251 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:5479
--# expr-start 3250 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:5479
--# expr-start 3249 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5479
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3248 '<Symbol Variable_Indexing>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Variable_Indexing); 
--# expr-done 3248
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym_1, E_Info => Ent.Info);
--# end
--# expr-done 3249
Is_Null_1 := Fld_1.Node = null; 
--# expr-done 3250
Not_Val_1 := not (Is_Null_1); 
--# expr-done 3251
   If_Result := Not_Val_1;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Has_Ud_Indexing;
--# end
--# property-start TypeDecl.constant_indexing_fns /usr/src/packages/BUILD/ada/ast.py:5483
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Constant_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_457;
            procedure Finalizer_Scope_2062;
            procedure Finalizer_Scope_2064;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Single_Formal;
Sym : Symbol_Type;
Fld : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_4 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_457 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_457;
            procedure Finalizer_Scope_2062 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2062;
            procedure Finalizer_Scope_2064 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2064;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3267 '<Block>' Let_Result None
--# scope-start
--# expr-start 3266 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5485
--# expr-start 3253 '<FieldAccess .get_aspect_spec_expr(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:5485
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3252 '<Symbol Constant_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Constant_Indexing); 
--# expr-done 3252
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3253
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
--# expr-start 3255 '<FieldAccess .all_env_elements_internal(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5485
--# expr-start 3254 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5485
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3254
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 3255
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2062;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_130 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_130
--# expr-start 3257 '<FieldAccess .subp_spec_or_null>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5490
--# expr-start 3256 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:5490
Cast_Expr_2 := Item_130; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3256
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 3257
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 3264 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .matching_formal_type(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5492
--# expr-start 3258 '<FieldAccess .origin_node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5493
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_3 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3258
Bound_Origin := Fld_3; 
--# expr-start 3263 '<FieldAccess .matching_formal_type(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5494
--# expr-start 3260 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:5494
--# expr-start 3259 '<FieldAccess .unpacked_formal_params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5494
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3259
Get_Result := Get (Fld_4, 0, True); 
--# expr-done 3260
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Single_Formal then
   --# scope-start
--# expr-start 3262 '<FieldAccess .formal_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5494
--# expr-start 3261 '<FieldAccess .spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5494
Fld_5 := Var_Expr_2.Spec;
--# expr-done 3261
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 3262
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
      if Result_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var_1.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var_1.Info);
--# end
--# expr-done 3263
Dyn_Var_Bind_Result := Fld_7; 
--# expr-done 3264
Scope_Result_2 := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_2064;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
               if Result_Var_2 then
      --# expr-start 3265 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5488
Cast_Expr_1 := Item_130; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3265
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3266
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3267
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_457;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_457;
               Finalizer_Scope_2062;
               Finalizer_Scope_2064;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Constant_Indexing_Fns;
--# end
--# property-start TypeDecl.variable_indexing_fns /usr/src/packages/BUILD/ada/ast.py:5502
pragma Warnings (Off, "is not referenced");
 function Type_Decl_P_Variable_Indexing_Fns
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_458;
            procedure Finalizer_Scope_2066;
            procedure Finalizer_Scope_2068;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Var_Expr_2 : Internal_Single_Formal;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Sym : Symbol_Type;
Fld_1 : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_3 : Internal_Entity_Type_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Boolean;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_458 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_458;
            procedure Finalizer_Scope_2066 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2066;
            procedure Finalizer_Scope_2068 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2068;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3285 '<Block>' Let_Result None
--# scope-start
--# expr-start 3284 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <FieldAccess .get_aspect_spec_expr(...)>: <Var a> <filter_map>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5503
--# expr-start 3268 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:5504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3268
Bound_Origin := Fld; 
--# expr-start 3270 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5505
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3269 '<Symbol Variable_Indexing>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Variable_Indexing); 
--# expr-done 3269
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Ent.Node, Name => Sym, E_Info => Ent.Info);
--# end
--# expr-done 3270
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 3283 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5506
--# expr-start 3272 '<FieldAccess .all_env_elements_internal(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5506
--# expr-start 3271 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5506
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3271
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Cast_Result.Node, Seq => False, E_Info => Cast_Result.Info);
--# end
--# expr-done 3272
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_131 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_131
--# expr-start 3274 '<FieldAccess .subp_spec_or_null>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5510
--# expr-start 3273 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:5510
Cast_Expr_2 := Item_131; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3273
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 3274
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_1
--# expr-start 3279 '<FieldAccess .matching_formal_type(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5512
--# expr-start 3276 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:5512
--# expr-start 3275 '<FieldAccess .unpacked_formal_params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5512
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3275
Get_Result := Get (Fld_4, 0, True); 
--# expr-done 3276
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Single_Formal then
   --# scope-start
--# expr-start 3278 '<FieldAccess .formal_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5512
--# expr-start 3277 '<FieldAccess .spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5512
Fld_5 := Var_Expr_2.Spec;
--# expr-done 3277
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_6 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_5.Node, Origin => Bound_Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 3278
Scope_Result := Fld_6;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      if Result_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_3 := Ent; 
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start BaseTypeDecl.matching_formal_type
Fld_7 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Formal_Type (Node => Result_Var.Node, Formal_Type => Cast_Result_3, Origin => Bound_Origin, E_Info => Result_Var.Info);
--# end
--# expr-done 3279
if Fld_7 then
   --# expr-start 3281 '<FieldAccess .is_implicit_deref>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:5514
--# expr-start 3280 '<FieldAccess .return_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5514
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3280
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Fld_8.Node, Origin => Bound_Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 3281
   If_Result := Fld_9;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
      Finalizer_Scope_2068;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
      --# expr-start 3282 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5508
Cast_Expr_1 := Item_131; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3282
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result_1;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3283
Scope_Result_2 := Map_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2066;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Dyn_Var_Bind_Result := Result_Var_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3284
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3285
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_458;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_458;
               Finalizer_Scope_2066;
               Finalizer_Scope_2068;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Decl_P_Variable_Indexing_Fns;
--# end
--# property-start TypeDecl._env_mappings_3 /usr/src/packages/BUILD/ada/ast.py:5324
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_3
  (Node : Bare_Type_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Decl :=
     Internal_Entity_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1172;
      Fld : Symbol_Type;
Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1172 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1172;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3287 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:5324
--# expr-start 3286 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:5324
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3286
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3287
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1172;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1172;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_3;
--# end
--# property-start TypeDecl._env_trans_parent_4 /usr/src/packages/BUILD/ada/ast.py:5323
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_4
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_4;
--# end
--# property-start TypeDecl._ref_env_nodes_5 /usr/src/packages/BUILD/ada/ast.py:5328
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_5
  (Node : Bare_Type_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1175;
      Cast_Expr : Bare_Type_Decl;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1175 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1175;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3288 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5328
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3288
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1175;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1175;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_5;
--# end
--# property-start TypeDecl._env_dest_6 /usr/src/packages/BUILD/ada/ast.py:5331
pragma Warnings (Off, "is not referenced");
 function Env_Dest_6
  (Node : Bare_Type_Decl
  )
   return Lexical_Env
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1176;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1176 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1176;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3289 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:5331
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 3289
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1176;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1176;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Dest_6;
--# end
--# property-start TypeDecl._ref_cond_7 /usr/src/packages/BUILD/ada/ast.py:5332
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_7
  (Node : Bare_Type_Decl
  )
   return Boolean
is
   Self : Bare_Type_Decl := Bare_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Type_Def;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3291 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:5332
--# expr-start 3290 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5332
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Type_Decl_F_Type_Def;
--# expr-done 3290
Is_A := Fld /= null 
and then Fld.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 3291
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_7;
--# end
function Type_Decl_P_Direct_Primitive_Subps
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Internal_Inner_Env_Assoc_Array_Access :=
      Type_Decl_P_Direct_Primitive_Subps (E.Node, E_Info);
begin
      return Result;
end;
function Type_Decl_P_Parent_Primitives_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Type_Decl_P_Parent_Primitives_Env (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Anonymous_Type_Decl
        (Self : Bare_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def, Type_Decl_F_Aspects);
      end Initialize_Fields_For_Anonymous_Type_Decl;
--# property-start AnonymousTypeDecl.access_def_matches /usr/src/packages/BUILD/ada/ast.py:5527
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Access_Def_Matches
  (Node : Bare_Anonymous_Type_Decl
      ; Other : Internal_Entity_Base_Type_Decl
      ; For_Assignment : Boolean
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl := Bare_Anonymous_Type_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type_Decl :=
     Internal_Entity_Anonymous_Type_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind other Other
   --# bind for_assignment For_Assignment
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Access_Def;
Match_0 : Internal_Entity_Access_To_Subp_Def;
Match_1 : Internal_Entity_Access_Def;
Var_Expr_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Exp : Internal_Entity_Base_Type_Decl;
Act : Internal_Entity_Base_Type_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Classwide_Type_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Internal_Entity_Classwide_Type_Decl;
Fld : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Access_Def;
Fld_1 : Internal_Entity_Access_Def;
Cast_Result_1 : Internal_Entity_Access_To_Subp_Def;
Fld_2 : Internal_Entity_Subp_Spec;
Fld_3 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity_Classwide_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Result_Var_2 : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity_Classwide_Type_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Fld_10 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Scope_Result_6 : Boolean;
Result_Var_6 : Boolean;
Match_Prefix : Internal_Entity_Access_Def;
Cast_Result_5 : Internal_Entity_Access_To_Subp_Def;
Let_Result_1 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_6 : Internal_Entity_Access_Def;
Let_Result_2 : Boolean;
Scope_Result_8 : Boolean;
Match_Result : Boolean;
Scope_Result_9 : Boolean;
Result_Var_7 : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3310 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3293 '<Cast to AccessDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5539
--# expr-start 3292 '<FieldAccess .type_def>' Fld /usr/src/packages/BUILD/ada/ast.py:5539
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Def (Node => Ent.Node.Type_Decl_F_Type_Def, Info => Ent.Info);
--# expr-done 3292
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Access_Def
   then
      Cast_Result := Create_Internal_Entity_Access_Def
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Access_Def;
   end if;
--# expr-done 3293
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Access_Def then
   --# scope-start
--# expr-start 3309 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5539
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Access_Def (Match_Prefix.Node.Kind) is
         when Ada_Access_To_Subp_Def =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_5; 
--# bind asp Match_0
--# expr-start 3295 '<Cast to AccessToSubpDef>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:5541
--# expr-start 3294 '<FieldAccess .access_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5541
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3294
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result_1 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 3295
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_1
--# expr-start 3298 '<FieldAccess .match_signature(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5542
--# expr-start 3296 '<FieldAccess .subp_spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5542
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 3296
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3297 '<FieldAccess .subp_spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5543
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Match_0.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Match_0.Info);
--# expr-done 3297
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_4 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_2.Node, Other => Cast_Result_2, Match_Name => False, E_Info => Fld_2.Info);
--# end
--# expr-done 3298
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_7 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Anonymous_Type_Access_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_6; 
--# bind ad Match_1
--# expr-start 3299 '<FieldAccess .accessed_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5547
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 3299
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind ast Var_Expr_2
--# expr-start 3300 '<FieldAccess .accessed_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5548
      if Other.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Other.Node, Origin => Origin, E_Info => Other.Info);
--# end
--# expr-done 3300
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind oat Var_Expr_3
--# expr-start 3308 '<Let exp, act>' Let_Result /usr/src/packages/BUILD/ada/ast.py:5549
--# scope-start
--# expr-start 3301 '<Cast to ClasswideTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:5554
Cast_Expr := Var_Expr_2; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_3 := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 3301
Var_Expr_5 := Cast_Result_3;
if Var_Expr_5 /= No_Entity_Classwide_Type_Decl then
   --# scope-start
--# expr-start 3302 '<FieldAccess .typedecl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5554
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld_7 := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 3302
Scope_Result_1 := Fld_7;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Var_Expr_4 := Result_Var_1;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_4
Scope_Result_2 := Var_Expr_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Var_Expr_2;
end if;
Exp := Result_Var_2; 
--# bind exp Exp
--# expr-start 3303 '<Cast to ClasswideTypeDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:5555
Cast_Expr_1 := Var_Expr_3; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Classwide_Type_Decl_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Classwide_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_4 := No_Entity_Classwide_Type_Decl;
   end if;
--# expr-done 3303
Var_Expr_7 := Cast_Result_4;
if Var_Expr_7 /= No_Entity_Classwide_Type_Decl then
   --# scope-start
--# expr-start 3304 '<FieldAccess .typedecl>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5555
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ClasswideTypeDecl.typedecl
Fld_8 := Libadalang.Implementation.Classwide_Type_Decl_P_Typedecl (Node => Var_Expr_7.Node, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 3304
Scope_Result_3 := Fld_8;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Var_Expr_6 := Result_Var_3;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind e Var_Expr_6
Scope_Result_4 := Var_Expr_6;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := Var_Expr_3;
end if;
Act := Result_Var_4; 
--# bind act Act
--# expr-start 3307 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:5557
if For_Assignment then
   --# expr-start 3305 '<FieldAccess .matching_assign_type(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:5563
      if Act.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_assign_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Assign_Type (Node => Act.Node, Expected_Type => Var_Expr_2, Origin => Origin, E_Info => Act.Info);
--# end
--# expr-done 3305
   If_Result := Fld_9;
else
   --# expr-start 3306 '<FieldAccess .matching_type(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:5565
      if Act.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_10 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Act.Node, Expected_Type => Exp, Origin => Origin, E_Info => Act.Info);
--# end
--# expr-done 3306
   If_Result := Fld_10;
end if;
--# expr-done 3307
Let_Result := If_Result; 
--# end
--# expr-done 3308
Scope_Result_5 := Let_Result;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
Scope_Result_6 := Result_Var_5;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := False;
end if;
Let_Result_2 := Result_Var_6; 
--# end
Scope_Result_8 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 3309
Scope_Result_9 := Match_Result;
   --# end
   Result_Var_7 := Scope_Result_9;
else
   Result_Var_7 := False;
end if;
Let_Result_3 := Result_Var_7; 
--# end
--# expr-done 3310
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Decl_P_Access_Def_Matches;
--# end
--# property-start AnonymousTypeDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:5572
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Decl_P_Xref_Entry_Point
  (Node : Bare_Anonymous_Type_Decl
  )
   return Boolean
is
   Self : Bare_Anonymous_Type_Decl := Bare_Anonymous_Type_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Decl_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Synth_Anonymous_Type_Decl
   --
      procedure Initialize_Fields_For_Synth_Anonymous_Type_Decl
        (Self : Bare_Synth_Anonymous_Type_Decl
         ; Base_Type_Decl_F_Name : Bare_Defining_Name
         ; Type_Decl_F_Discriminants : Bare_Discriminant_Part
         ; Type_Decl_F_Type_Def : Bare_Type_Def
         ; Type_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Anonymous_Type_Decl
              (Self, Base_Type_Decl_F_Name, Type_Decl_F_Discriminants, Type_Decl_F_Type_Def, Type_Decl_F_Aspects);
      end Initialize_Fields_For_Synth_Anonymous_Type_Decl;
   --
   --  Primitives for Bare_Basic_Subp_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Basic_Subp_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_29 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:6880" & ")";
         end if;
      return Initial_Env;
   end Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn;
      procedure Basic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Basic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_27 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_28 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_31 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_32 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_30 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:6885");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_33 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Basic_Subp_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_34 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_35 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_37 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_36 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_39 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_38 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
--# property-start BasicSubpDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:6771
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Names
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_623;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_623 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_623;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3313 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6771
--# expr-start 3312 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6771
--# expr-start 3311 '<FieldAccess .subp_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:6771
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3311
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3312
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3313
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_623;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_623;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Defining_Names;
--# end
--# property-start BasicSubpDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:6773
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Defining_Env
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_624;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_624 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_624;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3315 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6773
--# expr-start 3314 '<FieldAccess .subp_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:6773
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3314
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3315
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_624;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_624;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Defining_Env;
--# end
--# property-start BasicSubpDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:6775
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Type_Expression
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3317 '<FieldAccess .returns>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6776
--# expr-start 3316 '<FieldAccess .subp_decl_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:6776
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3316
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3317
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Type_Expression;
--# end
--# property-start BasicSubpDecl.get_body_in_env /usr/src/packages/BUILD/ada/ast.py:6783
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Get_Body_In_Env
  (Node : Bare_Basic_Subp_Decl
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Body_Node
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_626;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Body_Node;
Block_Var_2 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Body_Node;
Fld : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Fld_1 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Body_Node;
Fld_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Bare_Base_Subp_Spec;
As_Entity_1 : Internal_Entity_Base_Subp_Spec;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Body_Node;
Is_A : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Internal_Entity_Body_Node;
Fld_6 : Bare_Body_Node;
As_Entity_2 : Internal_Entity_Body_Node;
Is_Null_3 : Boolean;
If_Result_4 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Body_Node;
            procedure Finalizer_Scope_626 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_626;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3339 '<Block>' Let_Result None
--# scope-start
--# expr-start 3319 '<EnvGet(<Var env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:6785
--# expr-start 3318 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:6785
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3318
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3319
Block_Var := Env_Get_Result; Inc_Ref (Block_Var);
--# bind elements Block_Var
--# expr-start 3330 '<Cast to Body>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:6787
--# expr-start 3329 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:6787
--# expr-start 3328 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6787
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_132 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_132
--# expr-start 3322 '<Cast to Body>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6791
--# expr-start 3321 '<as_bare_entity(<FieldAccess .node>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6791
--# expr-start 3320 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6791
Fld_1 := Item_132.Node;
--# expr-done 3320
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3321
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result := Create_Internal_Entity_Body_Node
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Body_Node;
   end if;
--# expr-done 3322
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Body_Node then
   --# scope-start
--# expr-start 3327 '<FieldAccess .match_other(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6791
--# expr-start 3323 '<FieldAccess .formal_param_holder_or_null>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6791
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.formal_param_holder_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Formal_Param_Holder_Or_Null (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3323
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3326 '<as_bare_entity(<FieldAccess .node>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:6793
--# expr-start 3325 '<FieldAccess .node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6793
--# expr-start 3324 '<FieldAccess .subp_decl_spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6793
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3324
Fld_4 := Fld_3.Node;
--# expr-done 3325
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Base_Subp_Spec;
else
As_Entity_1 := (Info => No_Entity_Info, Node => Fld_4); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3326
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
--# property-call-start '[dispatcher]BaseFormalParamHolder.match_other'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Match_Other (Node => Fld_2.Node, Other => Cast_Result_1, Match_Names => True, E_Info => Fld_2.Info);
--# end
--# expr-done 3327
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_132;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3328
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3329
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Body_Node
   then
      Cast_Result_2 := Create_Internal_Entity_Body_Node
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Body_Node;
   end if;
--# expr-done 3330
Block_Var_1 := Cast_Result_2; 
--# bind precise Block_Var_1
--# expr-start 3336 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:6797
--# expr-start 3331 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:6798
Is_Null_2 := Block_Var_1.Node = null; 
--# expr-done 3331
if Is_Null_2 then
   If_Result_2 := Imprecise_Fallback;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 3335 '<Cast to Body>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:6799
--# expr-start 3334 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:6799
--# expr-start 3333 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:6799
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_133 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_133
               --# expr-start 3332 '<IsA Body>' Is_A /usr/src/packages/BUILD/ada/ast.py:6799
Is_A := Item_133.Node /= null 
and then Item_133.Node.Kind in Ada_Body_Node; 
--# expr-done 3332
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_133;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3333
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 3334
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_3 := Create_Internal_Entity_Body_Node
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         Cast_Result_3 := No_Entity_Body_Node;
   end if;
--# expr-done 3335
   If_Result_3 := Cast_Result_3;
else
   If_Result_3 := Block_Var_1;
end if;
--# expr-done 3336
Block_Var_2 := If_Result_3; 
--# bind result Block_Var_2
--# expr-start 3338 '<as_entity(<FieldAccess .node>)>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:6804
--# expr-start 3337 '<FieldAccess .node>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6804
Fld_6 := Block_Var_2.Node;
--# expr-done 3337
Is_Null_3 := Fld_6 = null; 
if Is_Null_3 then
   If_Result_4 := No_Entity_Body_Node;
else
As_Entity_2 := (Info => E_Info, Node => Fld_6); 
   If_Result_4 := As_Entity_2;
end if;
--# expr-done 3338
Let_Result := If_Result_4; 
--# end
--# expr-done 3339
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_626;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_626;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Get_Body_In_Env;
--# end
--# property-start BasicSubpDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:6807
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Basic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_627;
            procedure Finalizer_Scope_2084;
            procedure Finalizer_Scope_2085;
            procedure Finalizer_Scope_2087;
      Block_Var : Bare_Declarative_Part;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Declarative_Part;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Body_Node;
Var_Expr_3 : Internal_Entity_Private_Part;
Var_Expr_4 : Internal_Entity_Body_Node;
Fld : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Is_A : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Internal_Entity_Body_Node;
Scope_Result_1 : Internal_Entity_Body_Node;
Result_Var_1 : Internal_Entity_Body_Node;
Is_A_1 : Boolean;
Fld_6 : Internal_Entity_Private_Part;
Fld_7 : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Body_Node;
Scope_Result_2 : Internal_Entity_Body_Node;
Result_Var_2 : Internal_Entity_Body_Node;
Scope_Result_3 : Internal_Entity_Body_Node;
Fld_9 : Internal_Entity_Body_Node;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Internal_Entity_Body_Node;
Scope_Result_4 : Internal_Entity_Body_Node;
Result_Var_3 : Internal_Entity_Body_Node;
Result_Var_4 : Internal_Entity_Body_Node;
Is_Null_2 : Boolean;
Fld_12 : Lexical_Env := Empty_Env;
Fld_13 : Internal_Entity_Body_Node;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result_2 : Internal_Entity_Basic_Decl;
If_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
If_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
If_Result_3 : Internal_Entity_Basic_Decl;
If_Result_4 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_627 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_627;
            procedure Finalizer_Scope_2084 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2084;
            procedure Finalizer_Scope_2085 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2085;
            procedure Finalizer_Scope_2087 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2087;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3362 '<Block>' Let_Result None
--# scope-start
--# expr-start 3340 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:6808
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 3340
Block_Var := Fld; 
--# bind decl_scope Block_Var
--# expr-start 3341 '<as_entity(<Var decl_scope>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6809
Is_Null := Block_Var = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Block_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 3341
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Declarative_Part then
   --# scope-start
   --# bind ds Var_Expr
--# expr-start 3343 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6810
--# expr-start 3342 '<FieldAccess .semantic_parent>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6810
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.semantic_parent
Fld_1 := Libadalang.Implementation.Ada_Node_P_Semantic_Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3342
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3343
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind parent_decl Block_Var_1
--# expr-start 3344 '<FieldAccess .basic_decl_next_part_for_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6813
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.basic_decl_next_part_for_decl
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Basic_Decl_Next_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3344
Block_Var_2 := Fld_2; 
--# bind default_next_part Block_Var_2
--# expr-start 3361 '<Cond>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:6815
--# expr-start 3346 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:6818
--# expr-start 3345 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:6818
Is_Null_1 := Block_Var_2.Node = null; 
--# expr-done 3345
Not_Val := not (Is_Null_1); 
--# expr-done 3346
if Not_Val then
   If_Result_4 := Block_Var_2;
else
--# expr-start 3347 '<IsA PrivatePart>' Is_A /usr/src/packages/BUILD/ada/ast.py:6822
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Private_Part_Range; 
--# expr-done 3347
if Is_A then
--# expr-start 3348 '<FieldAccess .next_part_for_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6823
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3348
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind np Var_Expr_1
--# expr-start 3350 '<FieldAccess .get_body_in_env(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6824
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3349 '<FieldAccess .children_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6824
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 3349
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_5 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_4, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3350
Scope_Result_1 := Fld_5;
   --# end
      Finalizer_Scope_2084;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Body_Node;
end if;
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_1.Node,
         Info => Result_Var_1.Info);
   If_Result_3 := Cast_Result_4;
else
--# expr-start 3351 '<IsA PublicPart>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:6828
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Public_Part_Range; 
--# expr-done 3351
if Is_A_1 then
--# expr-start 3352 '<FieldAccess .decl_private_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6831
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.decl_private_part
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Decl_Private_Part (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3352
Var_Expr_3 := Fld_6;
if Var_Expr_3 /= No_Entity_Private_Part then
   --# scope-start
   --# bind dpp Var_Expr_3
--# expr-start 3354 '<FieldAccess .get_body_in_env(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:6832
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3353 '<FieldAccess .children_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:6832
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 3353
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_8 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_7, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3354
Scope_Result_2 := Fld_8;
   --# end
      Finalizer_Scope_2085;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Body_Node;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Body_Node then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_3 := Var_Expr_2;
   --# end
   Result_Var_4 := Scope_Result_3;
else
--# expr-start 3355 '<FieldAccess .body_part_for_decl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:6835
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3355
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Body_Node then
   --# scope-start
   --# bind np Var_Expr_4
--# expr-start 3357 '<FieldAccess .get_body_in_env(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:6836
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3356 '<FieldAccess .children_env>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:6836
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# expr-done 3356
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_11 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_10, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3357
Scope_Result_4 := Fld_11;
   --# end
      Finalizer_Scope_2087;
   Result_Var_3 := Scope_Result_4;
else
   Result_Var_3 := No_Entity_Body_Node;
end if;
   Result_Var_4 := Result_Var_3;
end if;
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
   If_Result_2 := Cast_Result_3;
else
--# expr-start 3358 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:6840
Is_Null_2 := Block_Var = null; 
--# expr-done 3358
if Is_Null_2 then
Cast_Expr := No_Entity_Body_Node; 
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_1 := Cast_Result_2;
else
--# expr-start 3360 '<FieldAccess .get_body_in_env(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:6844
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3359 '<FieldAccess .children_env>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:6844
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Children_Env (Node => Block_Var);
--# expr-done 3359
--# property-call-start BasicSubpDecl.get_body_in_env
Fld_13 := Libadalang.Implementation.Basic_Subp_Decl_P_Get_Body_In_Env (Node => Ent.Node, Env => Fld_12, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3360
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   If_Result_1 := Cast_Result_1;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 3361
Let_Result := If_Result_4; 
--# end
--# expr-done 3362
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_627;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_627;
               Finalizer_Scope_2084;
               Finalizer_Scope_2085;
               Finalizer_Scope_2087;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start BasicSubpDecl.constrain_prefix /usr/src/packages/BUILD/ada/ast.py:6848
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Constrain_Prefix
  (Node : Bare_Basic_Subp_Decl
      ; Prefix : Bare_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind prefix Prefix
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_628;
      Var_Expr : Internal_Single_Formal;
Fld : Logic_Var;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Logic_True : Logic_Equation;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_628 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_628;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3374 '<Block>' Let_Result None
--# scope-start
--# expr-start 3373 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:6849
--# expr-start 3365 '<FieldAccess .dottable_subp>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:6853
--# expr-start 3364 '<FieldAccess .md>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6853
--# expr-start 3363 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6853
Fld_5 := Ent.Info;
--# expr-done 3363
Fld_6 := Fld_5.MD;
--# expr-done 3364
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 3365
if Fld_7 then
   --# expr-start 3372 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:6854
--# expr-start 3366 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:6854
      if Prefix = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Prefix);
--# expr-done 3366
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
--# expr-start 3369 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:6855
--# expr-start 3368 '<FieldAccess .unpacked_formal_params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6855
--# expr-start 3367 '<FieldAccess .subp_decl_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6855
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3367
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3368
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 3369
Var_Expr := Get_Result;
if Var_Expr /= No_Single_Formal then
   --# scope-start
--# expr-start 3371 '<FieldAccess .formal_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6855
--# expr-start 3370 '<FieldAccess .spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6855
Fld_3 := Var_Expr.Spec;
--# expr-done 3370
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_3.Node, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 3371
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Bind_Result := Bind_Default_560.Create (Fld, Cast_Result, No_Logic_Converter_Default, Equals_Data_560'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6854") else null)); 
--# expr-done 3372
   If_Result := Bind_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6858") else null)); 
   If_Result := Logic_True;
end if;
      Inc_Ref (If_Result);
--# expr-done 3373
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3374
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_628;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_628;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Constrain_Prefix;
--# end
--# property-start BasicSubpDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:6862
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Expr_Type
  (Node : Bare_Basic_Subp_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3377 '<Block>' Let_Result None
--# scope-start
--# expr-start 3375 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:6863
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3375
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3376 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6863
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3376
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3377
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Expr_Type;
--# end
--# property-start BasicSubpDecl.env_names /usr/src/packages/BUILD/ada/ast.py:6866
pragma Warnings (Off, "is not referenced");
 function Basic_Subp_Decl_P_Env_Names
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_630;
            procedure Finalizer_Scope_2090;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_630 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_630;
            procedure Finalizer_Scope_2090 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2090;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3380 '<Block>' Let_Result None
--# scope-start
--# expr-start 3378 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:6867
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3378
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 3379 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:6868
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 3379
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2090;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3380
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_630;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_630;
               Finalizer_Scope_2090;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Basic_Subp_Decl_P_Env_Names;
--# end
--# property-start '[dispatcher]BasicSubpDecl.subp_decl_spec' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Basic_Subp_Decl (Self.Kind) is
               when Ada_Abstract_Subp_Decl .. Ada_Subp_Decl =>
                  --# property-call-start ClassicSubpDecl.subp_decl_spec
                  Property_Result := Classic_Subp_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Entry_Decl =>
                  --# property-call-start EntryDecl.subp_decl_spec
                  Property_Result := Entry_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Literal_Decl =>
                  --# property-call-start EnumLiteralDecl.subp_decl_spec
                  Property_Result := Enum_Literal_Decl_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Internal =>
                  --# property-call-start GenericSubpInternal.subp_decl_spec
                  Property_Result := Generic_Subp_Internal_P_Subp_Decl_Spec
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start BasicSubpDecl._env_do_27 /usr/src/packages/BUILD/ada/ast.py:6878
pragma Warnings (Off, "is not referenced");
 function Env_Do_27
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3381 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:6878
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3381
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_27;
--# end
--# property-start BasicSubpDecl._initial_env_name_28 /usr/src/packages/BUILD/ada/ast.py:6881
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_28
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3382 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:6881
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => True);
--# end
--# expr-done 3382
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_28;
--# end
--# property-start BasicSubpDecl._initial_env_29 /usr/src/packages/BUILD/ada/ast.py:6882
pragma Warnings (Off, "is not referenced");
 function Initial_Env_29
  (Node : Bare_Basic_Subp_Decl
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1199;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1199 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1199;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3383 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6882
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3383
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1199;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1199;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_29;
--# end
--# property-start BasicSubpDecl._env_mappings_30 /usr/src/packages/BUILD/ada/ast.py:6885
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_30
  (Node : Bare_Basic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Basic_Subp_Decl :=
     Internal_Entity_Basic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1200;
      Fld : Symbol_Type;
Cast_Expr : Bare_Basic_Subp_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1200 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1200;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3385 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:6885
--# expr-start 3384 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:6886
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3384
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3385
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1200;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1200;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_30;
--# end
--# property-start BasicSubpDecl._dest_env_name_31 /usr/src/packages/BUILD/ada/ast.py:6888
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_31
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3386 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:6888
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => False);
--# end
--# expr-done 3386
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_31;
--# end
--# property-start BasicSubpDecl._fallback_env_32 /usr/src/packages/BUILD/ada/ast.py:6889
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_32
  (Node : Bare_Basic_Subp_Decl
  )
   return Lexical_Env
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1202;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1202 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1202;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3387 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6889
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 3387
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1202;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1202;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_32;
--# end
--# property-start BasicSubpDecl._env_trans_parent_33 /usr/src/packages/BUILD/ada/ast.py:6876
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_33
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_33;
--# end
--# property-start BasicSubpDecl._env_names_34 /usr/src/packages/BUILD/ada/ast.py:6892
pragma Warnings (Off, "is not referenced");
 function Env_Names_34
  (Node : Bare_Basic_Subp_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1204;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1204 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1204;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3388 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:6892
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3388
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1204;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1204;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_34;
--# end
--# property-start BasicSubpDecl._env_do_35 /usr/src/packages/BUILD/ada/ast.py:6894
pragma Warnings (Off, "is not referenced");
 function Env_Do_35
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1205;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1205 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1205;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3389 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:6894
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3389
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1205;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1205;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_35;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_36 /usr/src/packages/BUILD/ada/ast.py:6897
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_36
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1206;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1206 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1206;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3390 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:6897
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3390
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1206;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1206;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_36;
--# end
--# property-start BasicSubpDecl._ref_cond_37 /usr/src/packages/BUILD/ada/ast.py:6899
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_37
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3392 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:6899
--# expr-start 3391 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6899
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3391
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3392
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_37;
--# end
--# property-start BasicSubpDecl._ref_env_nodes_38 /usr/src/packages/BUILD/ada/ast.py:6903
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_38
  (Node : Bare_Basic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1208;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1208 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1208;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3393 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:6903
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3393
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1208;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1208;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_38;
--# end
--# property-start BasicSubpDecl._ref_cond_39 /usr/src/packages/BUILD/ada/ast.py:6905
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_39
  (Node : Bare_Basic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Basic_Subp_Decl := Bare_Basic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3395 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:6905
--# expr-start 3394 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6905
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3394
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3395
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_39;
--# end
   --
   --  Primitives for Bare_Classic_Subp_Decl
   --
      procedure Initialize_Fields_For_Classic_Subp_Decl
        (Self : Bare_Classic_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Classic_Subp_Decl_F_Overriding := Classic_Subp_Decl_F_Overriding;
            Self.Classic_Subp_Decl_F_Subp_Spec := Classic_Subp_Decl_F_Subp_Spec;
      end Initialize_Fields_For_Classic_Subp_Decl;
   function Classic_Subp_Decl_F_Overriding
     (Node : Bare_Classic_Subp_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Classic_Subp_Decl_F_Overriding;
   end;
   function Classic_Subp_Decl_F_Subp_Spec
     (Node : Bare_Classic_Subp_Decl) return Bare_Subp_Spec
   is
   begin
         return Node.Classic_Subp_Decl_F_Subp_Spec;
   end;
--# property-start ClassicSubpDecl.subp_decl_spec /usr/src/packages/BUILD/ada/ast.py:6920
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Subp_Decl_Spec
  (Node : Bare_Classic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Classic_Subp_Decl := Bare_Classic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classic_Subp_Decl :=
     Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3396 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:6920
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3396
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classic_Subp_Decl_P_Subp_Decl_Spec;
--# end
--# property-start ClassicSubpDecl.body_part /usr/src/packages/BUILD/ada/ast.py:6924
pragma Warnings (Off, "is not referenced");
 function Classic_Subp_Decl_P_Body_Part
  (Node : Bare_Classic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Classic_Subp_Decl := Bare_Classic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Classic_Subp_Decl :=
     Internal_Entity_Classic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3399 '<Block>' Let_Result None
--# scope-start
--# expr-start 3398 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6928
--# expr-start 3397 '<FieldAccess .body_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6928
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3397
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3398
Let_Result := Cast_Result; 
--# end
--# expr-done 3399
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Classic_Subp_Decl_P_Body_Part;
--# end
   --
   --  Primitives for Bare_Abstract_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Subp_Decl
        (Self : Bare_Abstract_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Abstract_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Abstract_Subp_Decl_F_Aspects := Abstract_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Abstract_Subp_Decl;
   --
   --  Primitives for Bare_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Formal_Subp_Decl
        (Self : Bare_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Formal_Subp_Decl_F_Default_Expr := Formal_Subp_Decl_F_Default_Expr;
            Self.Formal_Subp_Decl_F_Aspects := Formal_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Formal_Subp_Decl;
   function Formal_Subp_Decl_F_Default_Expr
     (Node : Bare_Formal_Subp_Decl) return Bare_Expr
   is
   begin
         return Node.Formal_Subp_Decl_F_Default_Expr;
   end;
--# property-start FormalSubpDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8273
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Defining_Names
  (Node : Bare_Formal_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Formal_Subp_Decl := Bare_Formal_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Formal_Subp_Decl :=
     Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_729;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_729 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_729;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3402 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:8273
--# expr-start 3401 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8273
--# expr-start 3400 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:8273
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Classic_Subp_Decl_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3400
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3401
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3402
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_729;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_729;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Subp_Decl_P_Defining_Names;
--# end
--# property-start FormalSubpDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:8275
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Xref_Equation
  (Node : Bare_Formal_Subp_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Subp_Decl := Bare_Formal_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Formal_Subp_Decl :=
     Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_730;
            procedure Finalizer_Scope_2091;
            procedure Finalizer_Scope_2092;
            procedure Finalizer_Scope_2093;
            procedure Finalizer_Scope_2094;
      Var_Expr : Internal_Entity_Expr;
Match_0 : Internal_Entity_Null_Literal;
Match_1 : Internal_Entity_Box_Expr;
Match_2 : Internal_Entity_Name;
Match_3 : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Logic_True : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Logic_True_2 : Logic_Equation;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Null_Literal;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Box_Expr;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Expr;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Result_Var : Logic_Equation;
            procedure Finalizer_Scope_730 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_730;
            procedure Finalizer_Scope_2091 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2091;
            procedure Finalizer_Scope_2092 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2092;
            procedure Finalizer_Scope_2093 is
            begin
                     Dec_Ref (Logic_True_1);
            end Finalizer_Scope_2093;
            procedure Finalizer_Scope_2094 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2094;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 3403 '<FieldAccess .default_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:8276
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Formal_Subp_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 3403
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 3412 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:8276
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Null_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Null_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind _ Match_0
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8277") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2092;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Box_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Box_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8279") else null)); 
Let_Result_1 := Logic_True_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2093;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_String_Literal | Ada_Int_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind n Match_2
--# expr-start 3411 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:8281
--# expr-start 3408 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:8282
--# expr-start 3404 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8282
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Match_2.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 3404
--# expr-start 3407 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /usr/src/packages/BUILD/ada/ast.py:8283
--# expr-start 3405 '<FieldAccess .ref_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8284
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 3405
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 3406 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8284
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 3406
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_2, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 3407
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8282") else null)); 
--# expr-done 3408
--# expr-start 3410 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:8289
--# expr-start 3409 '<IsA AttributeRef>' Is_A /usr/src/packages/BUILD/ada/ast.py:8289
Is_A := Match_2.Node /= null 
and then Match_2.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 3409
if Is_A then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8289") else null)); 
   If_Result := Logic_False;
else
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8289") else null)); 
   If_Result := Logic_True_2;
end if;
      Inc_Ref (If_Result);
--# expr-done 3410
And_Pred_1 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8281") else null)); 
--# expr-done 3411
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2094;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl_Expr .. Ada_Relation_Op | Ada_Case_Expr_Alternative .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 3412
Scope_Result_4 := Match_Result;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2091;
   Result_Var := Scope_Result_4;
else
   Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:8293") else null)); 
   Result_Var := Logic_True_3;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_730;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_730;
               Finalizer_Scope_2091;
               Finalizer_Scope_2092;
               Finalizer_Scope_2093;
               Finalizer_Scope_2094;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Subp_Decl_P_Xref_Equation;
--# end
--# property-start FormalSubpDecl.designated_subprogram_from /usr/src/packages/BUILD/ada/ast.py:8297
pragma Warnings (Off, "is not referenced");
 function Formal_Subp_Decl_P_Designated_Subprogram_From
  (Node : Bare_Formal_Subp_Decl
      ; Inst : Internal_Entity_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Formal_Subp_Decl := Bare_Formal_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Formal_Subp_Decl :=
     Internal_Entity_Formal_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind inst Inst
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_731;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Subp_Spec;
Block_Var_2 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Symbol_Type;
Fld_3 : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Subp_Spec;
Fld_6 : Internal_Entity_Info;
Fld_7 : Internal_Metadata;
Fld_8 : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Fld_10 : Bare_Ada_Node;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
New_Struct_1 : Internal_Entity_Subp_Spec;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_15 : Internal_Entity_Base_Subp_Spec;
Fld_16 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_17 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_731 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_731;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3438 '<Block>' Let_Result None
--# scope-start
--# expr-start 3417 '<FieldAccess .env_get(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8303
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3413 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8304
      if Inst.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Inst.Node, E_Info => Inst.Info);
--# expr-done 3413
--# expr-start 3415 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8305
--# expr-start 3414 '<FieldAccess .defining_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8305
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3414
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_1.Node);
--# end
--# expr-done 3415
--# expr-start 3416 '<FieldAccess .node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8306
Fld_3 := Inst.Node;
--# expr-done 3416
      Cast_Result := Fld_3;
--# property-call-start AdaNode.env_get
Fld_4 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld, Symbol => Fld_2, From_Node => Cast_Result);
--# end
--# expr-done 3417
Block_Var := Fld_4; Inc_Ref (Block_Var);
--# bind subps Block_Var
--# expr-start 3430 '<New InternalEntitySubpSpec>' New_Struct_1 None
--# expr-start 3428 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8324
--# expr-start 3419 '<FieldAccess .from_rebound>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:8331
--# expr-start 3418 '<FieldAccess .info>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:8331
Fld_13 := Ent.Info;
--# expr-done 3418
Fld_14 := Fld_13.From_Rebound;
--# expr-done 3419
--# expr-start 3421 '<FieldAccess .md>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:8325
--# expr-start 3420 '<FieldAccess .info>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8325
Fld_6 := Ent.Info;
--# expr-done 3420
Fld_7 := Fld_6.MD;
--# expr-done 3421
--# expr-start 3427 '<append_rebinding(<FieldAccess .rebindings>, <FieldAccess .node_env>, <FieldAccess .instantiation_env>)>' Rebinding /usr/src/packages/BUILD/ada/ast.py:8326
--# expr-start 3423 '<FieldAccess .rebindings>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:8326
--# expr-start 3422 '<FieldAccess .info>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:8326
Fld_8 := Ent.Info;
--# expr-done 3422
Fld_9 := Fld_8.Rebindings;
--# expr-done 3423
--# expr-start 3425 '<FieldAccess .node_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:8329
--# expr-start 3424 '<parent(<Var Self>)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:8329
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Self.Parent; 
--# expr-done 3424
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Node_Env (Node => Fld_10);
--# expr-done 3425
--# expr-start 3426 '<FieldAccess .instantiation_env>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:8329
      if Inst.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Inst.Node);
--# end
--# expr-done 3426
Rebinding := AST_Envs.Append_Rebinding (Fld_9, Fld_11, Fld_12); 
--# expr-done 3427
New_Struct := (From_Rebound => Fld_14, MD => Fld_7, Rebindings => Rebinding); 
--# expr-done 3428
--# expr-start 3429 '<FieldAccess .subp_spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8323
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Classic_Subp_Decl_F_Subp_Spec;
--# expr-done 3429
New_Struct_1 := (Info => New_Struct, Node => Fld_5); 
--# expr-done 3430
Block_Var_1 := New_Struct_1; 
--# bind actual_spec Block_Var_1
--# expr-start 3436 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:8336
--# expr-start 3435 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:8336
--# expr-start 3434 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:8336
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_134 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_134
--# expr-start 3432 '<FieldAccess .subp_spec_or_null>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:8337
--# expr-start 3431 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:8337
Cast_Expr := Item_134; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3431
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 3432
Var_Expr := Fld_15;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 3433 '<FieldAccess .match_signature(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:8338
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.match_signature
Fld_16 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Block_Var_1.Node, Other => Var_Expr, Match_Name => False, E_Info => Block_Var_1.Info);
--# end
--# expr-done 3433
Scope_Result := Fld_16;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_134;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3434
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3435
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 3436
Block_Var_2 := Cast_Result_2; 
--# bind found Block_Var_2
Var_Expr_1 := Block_Var_2;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3437 '<FieldAccess .wrap_public_reference>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:8349
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_17 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 3437
Scope_Result_1 := Fld_17;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 3438
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_731;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_731;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Subp_Decl_P_Designated_Subprogram_From;
--# end
   --
   --  Primitives for Bare_Abstract_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Abstract_Formal_Subp_Decl
        (Self : Bare_Abstract_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Abstract_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Concrete_Formal_Subp_Decl
   --
      procedure Initialize_Fields_For_Concrete_Formal_Subp_Decl
        (Self : Bare_Concrete_Formal_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Formal_Subp_Decl_F_Default_Expr : Bare_Expr
         ; Formal_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Formal_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec, Formal_Subp_Decl_F_Default_Expr, Formal_Subp_Decl_F_Aspects);
      end Initialize_Fields_For_Concrete_Formal_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Decl
   --
      procedure Initialize_Fields_For_Subp_Decl
        (Self : Bare_Subp_Decl
         ; Classic_Subp_Decl_F_Overriding : Bare_Overriding_Node
         ; Classic_Subp_Decl_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Classic_Subp_Decl
              (Self, Classic_Subp_Decl_F_Overriding, Classic_Subp_Decl_F_Subp_Spec);
            Self.Subp_Decl_F_Aspects := Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Decl;
   --
   --  Primitives for Bare_Entry_Decl
   --
      procedure Entry_Decl_Pre_Env_Actions
        (Self            : Bare_Entry_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_147 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:13021");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_148 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Decl
        (Self : Bare_Entry_Decl
         ; Entry_Decl_F_Overriding : Bare_Overriding_Node
         ; Entry_Decl_F_Spec : Bare_Entry_Spec
         ; Entry_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Entry_Decl_F_Overriding := Entry_Decl_F_Overriding;
            Self.Entry_Decl_F_Spec := Entry_Decl_F_Spec;
            Self.Entry_Decl_F_Aspects := Entry_Decl_F_Aspects;
      end Initialize_Fields_For_Entry_Decl;
   function Entry_Decl_F_Overriding
     (Node : Bare_Entry_Decl) return Bare_Overriding_Node
   is
   begin
         return Node.Entry_Decl_F_Overriding;
   end;
   function Entry_Decl_F_Spec
     (Node : Bare_Entry_Decl) return Bare_Entry_Spec
   is
   begin
         return Node.Entry_Decl_F_Spec;
   end;
--# property-start EntryDecl.subp_decl_spec /usr/src/packages/BUILD/ada/ast.py:13008
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Subp_Decl_Spec
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Entry_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3439 '<FieldAccess .spec>' Fld /usr/src/packages/BUILD/ada/ast.py:13008
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 3439
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EntryDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:13010
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Defining_Names
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1002;
      Fld : Internal_Entity_Entry_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1002 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1002;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3442 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13010
--# expr-start 3441 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13010
--# expr-start 3440 '<FieldAccess .spec>' Fld /usr/src/packages/BUILD/ada/ast.py:13010
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Spec (Node => Ent.Node.Entry_Decl_F_Spec, Info => Ent.Info);
--# expr-done 3440
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3441
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3442
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1002;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1002;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Defining_Names;
--# end
--# property-start EntryDecl.body_part /usr/src/packages/BUILD/ada/ast.py:13014
pragma Warnings (Off, "is not referenced");
 function Entry_Decl_P_Body_Part
  (Node : Bare_Entry_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Body
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Entry_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Let_Result : Internal_Entity_Entry_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3445 '<Block>' Let_Result None
--# scope-start
--# expr-start 3444 '<Cast to EntryBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13018
--# expr-start 3443 '<FieldAccess .body_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:13018
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3443
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3444
Let_Result := Cast_Result; 
--# end
--# expr-done 3445
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Decl_P_Body_Part;
--# end
--# property-start EntryDecl._env_mappings_147 /usr/src/packages/BUILD/ada/ast.py:13021
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_147
  (Node : Bare_Entry_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Decl :=
     Internal_Entity_Entry_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1317;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1317 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1317;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3447 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:13021
--# expr-start 3446 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:13021
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3446
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3447
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1317;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1317;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_147;
--# end
--# property-start EntryDecl._env_trans_parent_148 /usr/src/packages/BUILD/ada/ast.py:13020
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_148
  (Node : Bare_Entry_Decl
  )
   return Boolean
is
   Self : Bare_Entry_Decl := Bare_Entry_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_148;
--# end
   --
   --  Primitives for Bare_Enum_Literal_Decl
   --
      procedure Enum_Literal_Decl_Pre_Env_Actions
        (Self            : Bare_Enum_Literal_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_145 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:12541");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_146 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Enum_Literal_Decl
        (Self : Bare_Enum_Literal_Decl
         ; Enum_Literal_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Enum_Literal_Decl_F_Name := Enum_Literal_Decl_F_Name;
      end Initialize_Fields_For_Enum_Literal_Decl;
   function Enum_Literal_Decl_F_Name
     (Node : Bare_Enum_Literal_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Enum_Literal_Decl_F_Name;
   end;
--# property-start EnumLiteralDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:12518
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Is_Static_Decl
  (Node : Bare_Enum_Literal_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Is_Static_Decl;
--# end
--# property-start EnumLiteralDecl.enum_type /usr/src/packages/BUILD/ada/ast.py:12521
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Enum_Type
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_962;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
Let_Result : Internal_Entity_Type_Decl;
            procedure Finalizer_Scope_962 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_962;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3454 '<Block>' Let_Result None
--# scope-start
--# expr-start 3453 '<Cast to TypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12525
--# expr-start 3452 '<as_entity(<at(<filter>, 0)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:12525
--# expr-start 3451 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12525
--# expr-start 3450 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12525
--# expr-start 3448 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:12525
Node_Parents := Parents (Self, True); 
--# expr-done 3448
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_135 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_135
               --# expr-start 3449 '<IsA TypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:12526
Is_A := Item_135 /= null 
and then Item_135.Kind in Ada_Type_Decl_Range; 
--# expr-done 3449
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_135;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3450
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3451
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 3452
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Type_Decl;
   end if;
--# expr-done 3453
Let_Result := Cast_Result; 
--# end
--# expr-done 3454
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_962;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_962;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Enum_Type;
--# end
--# property-start EnumLiteralDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:12529
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Defining_Names
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Literal_Decl :=
     Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_963;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_963 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_963;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3455 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:12529
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Enum_Literal_Decl_F_Name, Info => Ent.Info);
--# expr-done 3455
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_963;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_963;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Defining_Names;
--# end
--# property-start EnumLiteralDecl.synth_type_expr /usr/src/packages/BUILD/ada/ast.py:12532
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Synth_Type_Expr
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Enum_Lit_Synth_Type_Expr
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      New_Node : Bare_Enum_Lit_Synth_Type_Expr;
As_Entity : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
Let_Result : Internal_Entity_Enum_Lit_Synth_Type_Expr;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Synth_Type_Expr,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Enum_Lit_Synth_Type_Expr;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3458 '<Block>' Let_Result None
--# scope-start
--# expr-start 3457 '<as_entity(<New BareEnumLitSynthTypeExpr>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:12533
--# expr-start 3456 '<New BareEnumLitSynthTypeExpr>' New_Node /usr/src/packages/BUILD/ada/ast.py:12533
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Lit_Synth_Type_Expr);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Lit_Synth_Type_Expr,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 3456
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Lit_Synth_Type_Expr;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 3457
Let_Result := If_Result; 
--# end
--# expr-done 3458
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Enum_Lit_Synth_Type_Expr,
                     As_Internal_Entity_Enum_Lit_Synth_Type_Expr => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Synth_Type_Expr;
--# end
--# property-start EnumLiteralDecl.subp_decl_spec /usr/src/packages/BUILD/ada/ast.py:12536
pragma Warnings (Off, "is not referenced");
 function Enum_Literal_Decl_P_Subp_Decl_Spec
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Subp_Spec;
      New_Node : Bare_Enum_Subp_Spec;
As_Entity : Internal_Entity_Enum_Subp_Spec;
Is_Null : Boolean;
If_Result : Internal_Entity_Enum_Subp_Spec;
Let_Result : Internal_Entity_Enum_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Enum_Literal_Decl_Enum_Literal_Decl_P_Subp_Decl_Spec,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Enum_Literal_Decl,
                                As_Bare_Enum_Literal_Decl => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Subp_Spec;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 3461 '<Block>' Let_Result None
--# scope-start
--# expr-start 3460 '<as_entity(<New BareEnumSubpSpec>)>' If_Result None
--# expr-start 3459 '<New BareEnumSubpSpec>' New_Node None
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Enum_Subp_Spec);
Initialize
  (Self => New_Node,
   Kind => Ada_Enum_Subp_Spec,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
--# expr-done 3459
Is_Null := New_Node = null; 
if Is_Null then
   If_Result := No_Entity_Enum_Subp_Spec;
else
As_Entity := (Info => E_Info, Node => New_Node); 
   If_Result := As_Entity;
end if;
--# expr-done 3460
Let_Result := If_Result; 
--# end
--# expr-done 3461
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Let_Result.Node,
         Info => Let_Result.Info);
      Property_Result := Cast_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Subp_Spec,
                     As_Internal_Entity_Base_Subp_Spec => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Literal_Decl_P_Subp_Decl_Spec;
--# end
--# property-start EnumLiteralDecl._env_mappings_145 /usr/src/packages/BUILD/ada/ast.py:12541
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_145
  (Node : Bare_Enum_Literal_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Literal_Decl :=
     Internal_Entity_Enum_Literal_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1315;
      Fld : Symbol_Type;
Fld_1 : Internal_Entity_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Enum_Literal_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1315 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1315;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3465 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:12541
--# expr-start 3463 '<FieldAccess .node_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12542
--# expr-start 3462 '<FieldAccess .enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12542
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3462
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 3463
--# expr-start 3464 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:12541
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3464
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_2);
New_Struct := (Dest_Env => Fld_2, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3465
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1315;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1315;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_145;
--# end
--# property-start EnumLiteralDecl._env_trans_parent_146 /usr/src/packages/BUILD/ada/ast.py:12539
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_146
  (Node : Bare_Enum_Literal_Decl
  )
   return Boolean
is
   Self : Bare_Enum_Literal_Decl := Bare_Enum_Literal_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_146;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Internal
   --
      procedure Generic_Subp_Internal_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Internal;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_118 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_119 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Internal
        (Self : Bare_Generic_Subp_Internal
         ; Generic_Subp_Internal_F_Subp_Spec : Bare_Subp_Spec
         ; Generic_Subp_Internal_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Internal_F_Subp_Spec := Generic_Subp_Internal_F_Subp_Spec;
            Self.Generic_Subp_Internal_F_Aspects := Generic_Subp_Internal_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Internal;
   function Generic_Subp_Internal_F_Subp_Spec
     (Node : Bare_Generic_Subp_Internal) return Bare_Subp_Spec
   is
   begin
         return Node.Generic_Subp_Internal_F_Subp_Spec;
   end;
--# property-start GenericSubpInternal.subp_decl_spec /usr/src/packages/BUILD/ada/ast.py:8441
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Internal_P_Subp_Decl_Spec
  (Node : Bare_Generic_Subp_Internal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Spec
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Internal :=
     Internal_Entity_Generic_Subp_Internal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Subp_Spec;
      Fld : Internal_Entity_Subp_Spec;
Cast_Result : Internal_Entity_Base_Subp_Spec;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3466 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:8441
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3466
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Internal_P_Subp_Decl_Spec;
--# end
--# property-start GenericSubpInternal.get_instantiation /usr/src/packages/BUILD/ada/ast.py:8446
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Internal_P_Get_Instantiation
  (Node : Bare_Generic_Subp_Internal
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Subp_Instantiation
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Internal :=
     Internal_Entity_Generic_Subp_Internal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Subp_Instantiation;
            procedure Finalizer_Scope_739;
      Block_Var : Bare_Ada_Node;
Fld : Internal_Entity_Info;
Fld_1 : Env_Rebindings;
Fld_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Generic_Subp_Instantiation;
Fld_3 : Internal_Entity_Info;
Fld_4 : Env_Rebindings;
Fld_5 : Env_Rebindings;
Fld_6 : Internal_Entity_Info;
Fld_7 : Boolean;
New_Struct : Internal_Metadata;
New_Struct_1 : Internal_Entity_Info;
New_Struct_2 : Internal_Entity_Generic_Subp_Instantiation;
Let_Result : Internal_Entity_Generic_Subp_Instantiation;
            procedure Finalizer_Scope_739 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_739;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3480 '<Block>' Let_Result None
--# scope-start
--# expr-start 3470 '<env_node(<FieldAccess .new_env>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:8451
--# expr-start 3469 '<FieldAccess .new_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8451
--# expr-start 3468 '<FieldAccess .rebindings>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8451
--# expr-start 3467 '<FieldAccess .info>' Fld /usr/src/packages/BUILD/ada/ast.py:8451
Fld := Ent.Info;
--# expr-done 3467
Fld_1 := Fld.Rebindings;
--# expr-done 3468
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.New_Env;
Inc_Ref (Fld_2);
--# expr-done 3469
Env_Node := AST_Envs.Env_Node (Fld_2); 
--# expr-done 3470
Block_Var := Env_Node; 
--# bind inst_node Block_Var
--# expr-start 3479 '<New InternalEntityGenericSubpInstantiation>' New_Struct_2 None
--# expr-start 3477 '<New InternalEntityInfo>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:8454
--# expr-start 3472 '<FieldAccess .from_rebound>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:8458
--# expr-start 3471 '<FieldAccess .info>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8458
Fld_6 := Ent.Info;
--# expr-done 3471
Fld_7 := Fld_6.From_Rebound;
--# expr-done 3472
--# expr-start 3473 '<New InternalMetadata>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8459
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 3473
--# expr-start 3476 '<FieldAccess .get_parent>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8457
--# expr-start 3475 '<FieldAccess .rebindings>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8457
--# expr-start 3474 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8457
Fld_3 := Ent.Info;
--# expr-done 3474
Fld_4 := Fld_3.Rebindings;
--# expr-done 3475
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Fld_4.Parent;
--# expr-done 3476
New_Struct_1 := (From_Rebound => Fld_7, MD => New_Struct, Rebindings => Fld_5); 
--# expr-done 3477
--# expr-start 3478 '<Cast to GenericSubpInstantiation>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8453
Cast_Expr := Block_Var; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Generic_Subp_Instantiation_Range
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3478
New_Struct_2 := (Info => New_Struct_1, Node => Cast_Result); 
--# expr-done 3479
Let_Result := New_Struct_2; 
--# end
--# expr-done 3480
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_739;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_739;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Internal_P_Get_Instantiation;
--# end
--# property-start GenericSubpInternal._env_trans_parent_118 /usr/src/packages/BUILD/ada/ast.py:8443
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_118
  (Node : Bare_Generic_Subp_Internal
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_118;
--# end
--# property-start GenericSubpInternal._env_names_119 /usr/src/packages/BUILD/ada/ast.py:8443
pragma Warnings (Off, "is not referenced");
 function Env_Names_119
  (Node : Bare_Generic_Subp_Internal
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Subp_Internal := Bare_Generic_Subp_Internal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1289;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1289 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1289;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3481 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:8443
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicSubpDecl.env_names
Fld := Libadalang.Implementation.Basic_Subp_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 3481
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1289;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1289;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_119;
--# end
   --
   --  Primitives for Bare_Body_Node
   --
--# property-start Body.env_hook_body /usr/src/packages/BUILD/ada/ast.py:2541
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Env_Hook_Body
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_174;
      Unused_1 : Internal_Unit;
Var_Expr : Bare_Subp_Body;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Internal_Unit;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Subp_Body;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
Is_A : Boolean;
If_Result_1 : Boolean;
Let_Result_1 : Boolean;
            procedure Finalizer_Scope_174 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_174;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3491 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 3490 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2545
--# expr-start 3482 '<IsA PackageBody, SubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:2548
Is_A := Self /= null 
and then Self.Kind in Ada_Package_Body_Range | Ada_Subp_Body_Range; 
--# expr-done 3482
if Is_A then
   --# expr-start 3489 '<Let _>' Let_Result /usr/src/packages/BUILD/ada/ast.py:2550
--# scope-start
--# expr-start 3486 '<FieldAccess .get_unit(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2551
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3485 '<FieldAccess .as_symbol_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2552
--# expr-start 3484 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2552
--# expr-start 3483 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2552
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3483
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3484
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 3485
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification, Load_If_Needed => True);
--# expr-done 3486
Unused_1 := Fld_2; 
--# bind _ Unused_1
--# expr-start 3487 '<Cast to SubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2559
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3487
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind subp_body Var_Expr
--# expr-start 3488 '<FieldAccess .env_hook_basic_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2560
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.env_hook_basic_decl
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Env_Hook_Basic_Decl (Node => Var_Expr);
--# end
--# expr-done 3488
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3489
   If_Result_1 := Let_Result;
else
   If_Result_1 := False;
end if;
--# expr-done 3490
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 3491
      Property_Result := Let_Result_1;
   --# end
      Finalizer_Scope_174;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_174;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Env_Hook_Body;
--# end
--# property-start Body.subunit_decl_env /usr/src/packages/BUILD/ada/ast.py:2568
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Decl_Env
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_175;
            procedure Finalizer_Scope_2104;
            procedure Finalizer_Scope_2105;
            procedure Finalizer_Scope_2107;
            procedure Finalizer_Scope_2108;
      Var_Expr : Lexical_Env := Empty_Env;
Match_0 : Internal_Entity_Generic_Package_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Generic_Package_Internal;
Fld_5 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_175 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_175;
            procedure Finalizer_Scope_2104 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2104;
            procedure Finalizer_Scope_2105 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2105;
            procedure Finalizer_Scope_2107 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2107;
            procedure Finalizer_Scope_2108 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_2108;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3511 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3510 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <Then <Match>: <Var public_part> <Then <at(<EnvGet(<Var public_part>, __privatepart)>, 0)>: <Var pp> <env_group(<ArrayLiteral>)>>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2569
--# expr-start 3492 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:2570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3492
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3503 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2571
--# expr-start 3496 '<at(<EnvGet(<FieldAccess .body_scope(...)>, <FieldAccess .name_symbol>)>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2571
--# expr-start 3495 '<EnvGet(<FieldAccess .body_scope(...)>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2571
--# expr-start 3493 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2571
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3493
--# expr-start 3494 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2571
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3494
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_1, Key => Fld_2, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3495
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 3496
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gpd Match_0
--# expr-start 3501 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:2578
--# expr-start 3500 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2578
--# expr-start 3497 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2578
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Match_0.Node, E_Info => Match_0.Info);
--# expr-done 3497
--# expr-start 3499 '<FieldAccess .children_env>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2578
--# expr-start 3498 '<FieldAccess .package_decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2578
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_0.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_0.Info);
--# expr-done 3498
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Children_Env (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# expr-done 3499
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_5)); 
--# expr-done 3500
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3501
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2104;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abstract_State_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind pd Match_1
--# expr-start 3502 '<FieldAccess .children_env>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2580
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 3502
Let_Result_1 := Fld_6; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2105;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
end case;
--# expr-done 3503
Var_Expr := Match_Result;
      Inc_Ref (Var_Expr);
if Var_Expr /= Empty_Env then
   --# scope-start
   --# bind public_part Var_Expr
--# expr-start 3506 '<at(<EnvGet(<Var public_part>, __privatepart)>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:2582
--# expr-start 3505 '<EnvGet(<Var public_part>, __privatepart)>' Env_Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:2582
--# expr-start 3504 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 3504
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Var_Expr, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3505
Get_Result_1 := Get (Env_Get_Result_1, 0, True); 
--# expr-done 3506
Var_Expr_1 := Get_Result_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 3509 '<env_group(<ArrayLiteral>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:2586
--# expr-start 3508 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:2586
--# expr-start 3507 '<FieldAccess .children_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2586
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 3507
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_7, 2 => Var_Expr)); 
--# expr-done 3508
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 3509
Scope_Result_3 := Group_Env_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2108;
   Result_Var := Scope_Result_3;
else
   Result_Var := Var_Expr;
end if;
      Inc_Ref (Result_Var);
Scope_Result_4 := Result_Var;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2107;
   Result_Var_1 := Scope_Result_4;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Dyn_Var_Bind_Result := Result_Var_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3510
Let_Result_3 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 3511
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_175;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_175;
               Finalizer_Scope_2104;
               Finalizer_Scope_2105;
               Finalizer_Scope_2107;
               Finalizer_Scope_2108;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subunit_Decl_Env;
--# end
--# property-start Body.in_scope /usr/src/packages/BUILD/ada/ast.py:2592
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_In_Scope
  (Node : Bare_Body_Node
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
            procedure Finalizer_Scope_176;
      Fld : Internal_Unit;
Fld_1 : Internal_Unit;
Is_Equal : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_176 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_176;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3521 '<Block>' Let_Result None
--# scope-start
--# expr-start 3514 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2597
--# expr-start 3512 '<FieldAccess .unit>' Fld /usr/src/packages/BUILD/ada/ast.py:2597
      if Origin = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Unit (Node => Origin);
--# expr-done 3512
--# expr-start 3513 '<FieldAccess .unit>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2597
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 3513
Is_Equal := Fld = Fld_1; 
--# expr-done 3514
if Is_Equal then
   --# expr-start 3520 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2598
--# expr-start 3519 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:2598
--# expr-start 3518 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2598
--# expr-start 3517 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2598
--# expr-start 3515 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:2598
Node_Parents := Parents (Origin, True); 
--# expr-done 3515
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_136 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_136
               --# expr-start 3516 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:2598
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Cast_Result = Item_136; 
--# expr-done 3516
               if Is_Equal_1 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_136;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3517
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3518
Is_Null := Get_Result = null; 
--# expr-done 3519
Not_Val := not (Is_Null); 
--# expr-done 3520
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 3521
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_176;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_176;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_In_Scope;
--# end
--# property-start Body.body_decl_scope /usr/src/packages/BUILD/ada/ast.py:2602
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Decl_Scope
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_177;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_177 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_177;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3525 '<Block>' Let_Result None
--# scope-start
--# expr-start 3524 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2606
--# expr-start 3522 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:2607
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3522
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3523 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => True, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3523
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3524
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 3525
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_177;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_177;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Body_Decl_Scope;
--# end
--# property-start Body.body_initial_env_name /usr/src/packages/BUILD/ada/ast.py:2612
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Initial_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_178;
      Fld : Boolean;
Fld_1 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym : Symbol_Type;
Fld_2 : Boolean;
Fld_3 : Character_Type_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Sym_1 : Symbol_Type;
If_Result : Symbol_Type;
If_Result_1 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_178 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Concat_Result_1);
            end Finalizer_Scope_178;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3537 '<Block>' Let_Result None
--# scope-start
--# expr-start 3536 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2618
--# expr-start 3526 '<FieldAccess .is_library_item>' Fld /usr/src/packages/BUILD/ada/ast.py:2619
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 3526
if Fld then
   --# expr-start 3530 '<to_symbol(<Concat>)>' Sym /usr/src/packages/BUILD/ada/ast.py:2620
--# expr-start 3529 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2620
--# expr-start 3527 '<FieldAccess .top_level_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2620
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3527
--# expr-start 3528 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2620
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46), 2 => Character_Type'Val (95), 3 => Character_Type'Val (95), 4 => Character_Type'Val (112), 5 => Character_Type'Val (114), 6 => Character_Type'Val (105), 7 => Character_Type'Val (118), 8 => Character_Type'Val (97), 9 => Character_Type'Val (116), 10 => Character_Type'Val (101), 11 => Character_Type'Val (112), 12 => Character_Type'Val (97), 13 => Character_Type'Val (114), 14 => Character_Type'Val (116))); 
--# expr-done 3528
Concat_Result := Concat (Fld_1, Array_Lit); 
--# expr-done 3529
Sym := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 3530
   If_Result_1 := Sym;
else
--# expr-start 3531 '<FieldAccess .is_subunit>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2622
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3531
if Fld_2 then
   --# expr-start 3535 '<to_symbol(<Concat>)>' Sym_1 /usr/src/packages/BUILD/ada/ast.py:2623
--# expr-start 3534 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:2623
--# expr-start 3532 '<FieldAccess .top_level_env_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2623
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3532
--# expr-start 3533 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:2623
Array_Lit_1 := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (95), 2 => Character_Type'Val (95), 3 => Character_Type'Val (115), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (98))); 
--# expr-done 3533
Concat_Result_1 := Concat (Fld_3, Array_Lit_1); 
--# expr-done 3534
Sym_1 := String_To_Symbol (Self.Unit.Context, Concat_Result_1); 
--# expr-done 3535
   If_Result := Sym_1;
else
   If_Result := null;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 3536
Let_Result := If_Result_1; 
--# end
--# expr-done 3537
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_178;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_178;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Body_Initial_Env_Name;
--# end
--# property-start '[dispatcher]Body.previous_part_env_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Body_Node (Self.Kind) is
               when Ada_Package_Body_Stub .. Ada_Task_Body =>
                  --# property-call-start Body.previous_part_env_name
                  Property_Result := Body_Node_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
               when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
                  --# property-call-start BaseSubpBody.previous_part_env_name
                  Property_Result := Base_Subp_Body_P_Previous_Part_Env_Name
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Body_Node_P_Previous_Part_Env_Name;
--# end
--# property-start Body.subp_previous_part /usr/src/packages/BUILD/ada/ast.py:2643
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subp_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_180;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Internal_Entity_Array_Access;
Block_Var_2 : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Generic_Subp_Decl;
Match_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Basic_Subp_Decl;
Match_3 : Internal_Entity_Subp_Body_Stub;
Match_4 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Base_Subp_Spec;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Subp_Spec;
Fld_7 : Internal_Entity_Base_Subp_Spec;
Cast_Result_3 : Internal_Entity_Subp_Spec;
Cast_Result_4 : Internal_Entity_Base_Subp_Spec;
Fld_8 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_5 : Internal_Entity_Generic_Subp_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_6 : Internal_Entity_Formal_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_7 : Internal_Entity_Basic_Subp_Decl;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_8 : Internal_Entity_Subp_Body_Stub;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_9 : Internal_Entity;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_10 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
Not_Val_2 : Boolean;
Fld_9 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_11 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_3 : Boolean;
If_Result_2 : Boolean;
Is_A : Boolean;
If_Result_3 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Cast_Result_12 : Internal_Entity_Basic_Decl;
Is_Null_2 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Internal_Entity_Basic_Decl;
Let_Result_5 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_180 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_180;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3570 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 3539 '<env_parent(<FieldAccess .children_env>)>' Env_Parent /usr/src/packages/BUILD/ada/ast.py:2648
--# expr-start 3538 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:2648
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3538
Env_Parent := AST_Envs.Parent (Fld); 
--# expr-done 3539
Block_Var := Env_Parent; Inc_Ref (Block_Var);
--# bind env Block_Var
--# expr-start 3541 '<EnvGet(<Var env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2650
--# expr-start 3540 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2650
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3540
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3541
Block_Var_1 := Env_Get_Result; Inc_Ref (Block_Var_1);
--# bind elements Block_Var_1
--# expr-start 3558 '<Cast to BasicDecl.entity>' Cast_Result_10 /usr/src/packages/BUILD/ada/ast.py:2652
--# expr-start 3557 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2652
--# expr-start 3556 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2652
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_137 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_137
--# expr-start 3543 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2654
--# expr-start 3542 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:2654
Is_Null := Item_137.Node = null; 
--# expr-done 3542
Not_Val := not (Is_Null); 
--# expr-done 3543
if Not_Val then
   --# expr-start 3546 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:2655
--# expr-start 3545 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2655
--# expr-start 3544 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2655
Fld_2 := Item_137.Node;
--# expr-done 3544
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Fld_2 = Cast_Result; 
--# expr-done 3545
Not_Val_1 := not (Is_Equal); 
--# expr-done 3546
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3555 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2656
      if Item_137.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_137; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_5; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_6; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Generic_Subp_Internal =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Basic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind subp_decl Match_2
--# expr-start 3550 '<FieldAccess .match_signature(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2666
--# expr-start 3547 '<FieldAccess .subp_decl_spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2666
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicSubpDecl.subp_decl_spec'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Subp_Decl_P_Subp_Decl_Spec (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 3547
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3549 '<Cast to SubpSpec>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:2667
--# expr-start 3548 '<FieldAccess .subp_spec_or_null>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2667
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3548
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Subp_Spec;
   end if;
--# expr-done 3549
      Cast_Result_2 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_3.Node, Other => Cast_Result_2, Match_Name => True, Use_Entity_Info => False, E_Info => Fld_3.Info);
--# end
--# expr-done 3550
Let_Result_2 := Fld_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_8; 
--# bind subp_stub Match_3
--# expr-start 3554 '<FieldAccess .match_signature(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2674
--# expr-start 3551 '<FieldAccess .subp_spec>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2674
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Subp_Spec (Node => Match_3.Node.Subp_Body_Stub_F_Subp_Spec, Info => Match_3.Info);
--# expr-done 3551
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3553 '<Cast to SubpSpec>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:2675
--# expr-start 3552 '<FieldAccess .subp_spec_or_null>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2675
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3552
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Subp_Spec_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subp_Spec
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_3 := No_Entity_Subp_Spec;
   end if;
--# expr-done 3553
      Cast_Result_4 := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Result_3.Node,
         Info => Cast_Result_3.Info);
--# property-call-start BaseSubpSpec.match_signature
Fld_8 := Libadalang.Implementation.Base_Subp_Spec_P_Match_Signature (Node => Fld_6.Node, Other => Cast_Result_4, Match_Name => True, Use_Entity_Info => False, E_Info => Fld_6.Info);
--# end
--# expr-done 3554
Let_Result_3 := Fld_8; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Synth_Anonymous_Type_Decl | Ada_Expr_Function .. Ada_Protected_Body_Stub | Ada_Task_Body_Stub .. Ada_Generic_Package_Decl | Ada_Generic_Package_Instantiation .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_9; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 3555
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
               if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_137;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3556
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3557
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_10 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3558
Block_Var_2 := Cast_Result_10; 
--# bind precise Block_Var_2
--# expr-start 3569 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:2686
--# expr-start 3559 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:2687
Is_Null_2 := Block_Var_2.Node = null; 
--# expr-done 3559
if Is_Null_2 then
   If_Result_4 := Imprecise_Fallback;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 3568 '<Cast to BasicDecl.entity>' Cast_Result_12 /usr/src/packages/BUILD/ada/ast.py:2688
--# expr-start 3567 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:2688
--# expr-start 3566 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:2688
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_1;
      begin
         for Item_138 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_138
--# expr-start 3561 '<Not>' Not_Val_2 /usr/src/packages/BUILD/ada/ast.py:2690
--# expr-start 3560 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:2690
Is_Null_1 := Item_138.Node = null; 
--# expr-done 3560
Not_Val_2 := not (Is_Null_1); 
--# expr-done 3561
if Not_Val_2 then
   --# expr-start 3564 '<Not>' Not_Val_3 /usr/src/packages/BUILD/ada/ast.py:2691
--# expr-start 3563 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:2691
--# expr-start 3562 '<FieldAccess .node>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2691
Fld_9 := Item_138.Node;
--# expr-done 3562
Cast_Expr_1 := Self; 
      Cast_Result_11 := Cast_Expr_1;
Is_Equal_1 := Fld_9 = Cast_Result_11; 
--# expr-done 3563
Not_Val_3 := not (Is_Equal_1); 
--# expr-done 3564
   If_Result_2 := Not_Val_3;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 3565 '<IsA BasicSubpDecl, SubpBodyStub>' Is_A /usr/src/packages/BUILD/ada/ast.py:2692
Is_A := Item_138.Node /= null 
and then Item_138.Node.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Stub_Range; 
--# expr-done 3565
   If_Result_3 := Is_A;
else
   If_Result_3 := False;
end if;
               if If_Result_3 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_138;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3566
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 3567
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_12 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3568
   If_Result_5 := Cast_Result_12;
else
   If_Result_5 := Block_Var_2;
end if;
--# expr-done 3569
Let_Result_5 := If_Result_5; 
--# end
--# expr-done 3570
      Property_Result := Let_Result_5;
   --# end
      Finalizer_Scope_180;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_180;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subp_Previous_Part;
--# end
--# property-start Body.entry_previous_part /usr/src/packages/BUILD/ada/ast.py:2699
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Entry_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_181;
      Block_Var : Internal_Entity_Entry_Completion_Formal_Params;
Match_0 : Internal_Entity_Entry_Decl;
Match_1 : Internal_Entity;
Cast_Expr : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Entry_Body;
Fld : Internal_Entity_Entry_Completion_Formal_Params;
Fld_1 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Cast_Expr_1 : Bare_Body_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Internal_Entity_Entry_Spec;
Cast_Expr_2 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Match_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Entry_Decl;
Let_Result_2 : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_181 is
            begin
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_181;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3586 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 3572 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:2703
--# expr-start 3571 '<Cast to EntryBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2703
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Body_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3571
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Cast_Result.Node.Entry_Body_F_Params, Info => Cast_Result.Info);
--# expr-done 3572
Block_Var := Fld; 
--# bind spec Block_Var
--# expr-start 3585 '<Cast to EntryDecl.entity>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:2704
--# expr-start 3584 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2704
--# expr-start 3583 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2704
--# expr-start 3574 '<EnvGet(<DynamicVariable env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2704
--# expr-start 3573 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2704
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3573
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env, Key => Fld_1, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3574
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_139 of
               Collection.Items
         loop
   --# scope-start
               --# bind sp Item_139
--# expr-start 3576 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2706
--# expr-start 3575 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:2706
Is_Null := Item_139.Node = null; 
--# expr-done 3575
Not_Val := not (Is_Null); 
--# expr-done 3576
if Not_Val then
   --# expr-start 3579 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:2707
--# expr-start 3578 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2707
--# expr-start 3577 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2707
Fld_2 := Item_139.Node;
--# expr-done 3577
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Is_Equal := Fld_2 = Cast_Result_1; 
--# expr-done 3578
Not_Val_1 := not (Is_Equal); 
--# expr-done 3579
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3582 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2708
      if Item_139.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_139; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Entry_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind entry_decl Match_0
--# expr-start 3581 '<FieldAccess .match_formal_params(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2710
--# expr-start 3580 '<FieldAccess .spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2710
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Entry_Spec (Node => Match_0.Node.Entry_Decl_F_Spec, Info => Match_0.Info);
--# expr-done 3580
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_3.Node, Other => Cast_Result_2, E_Info => Fld_3.Info);
--# end
--# expr-done 3581
Let_Result := Fld_4; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Subp_Decl | Ada_Enum_Literal_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind _ Match_1
Let_Result_1 := False; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3582
   If_Result_1 := Match_Result;
else
   If_Result_1 := False;
end if;
               if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_139;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3583
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 3584
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Entry_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3585
Let_Result_2 := Cast_Result_5; 
--# end
--# expr-done 3586
      Property_Result := Let_Result_2;
   --# end
      Finalizer_Scope_181;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_181;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Entry_Previous_Part;
--# end
--# property-start Body.package_previous_part /usr/src/packages/BUILD/ada/ast.py:2718
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Package_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_182;
      Match_0 : Internal_Entity_Package_Decl;
Match_1 : Internal_Entity_Generic_Package_Decl;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Fld_3 : Internal_Entity_Generic_Package_Internal;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Package_Decl;
Cast_Expr : Internal_Entity_Package_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Get_Result : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_182 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_182;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3598 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3597 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2725
--# expr-start 3596 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:2725
--# expr-start 3593 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:2725
--# expr-start 3590 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2725
--# expr-start 3587 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:2726
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3587
Bound_Origin := Fld; 
--# expr-start 3589 '<FieldAccess .all_env_els_impl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2726
--# expr-start 3588 '<FieldAccess .defining_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2726
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3588
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3589
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3590
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Dyn_Var_Bind_Result;
      begin
         for Item_141 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_141
      --# expr-start 3592 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2728
      if Item_141.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_141; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind pkg_decl Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind gen_pkg_decl Match_1
--# expr-start 3591 '<FieldAccess .package_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2731
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_1.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_1.Info);
--# expr-done 3591
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_For_Loop_Var_Decl | Ada_Generic_Subp_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3592
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Match_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 3593
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_140 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_140
               --# expr-start 3595 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2735
--# expr-start 3594 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:2735
Is_Null := Item_140.Node = null; 
--# expr-done 3594
Not_Val := not (Is_Null); 
--# expr-done 3595
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_140;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 3596
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 3597
Let_Result_3 := Get_Result; 
--# end
--# expr-done 3598
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_182;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_182;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Package_Previous_Part;
--# end
--# property-start Body.task_previous_part /usr/src/packages/BUILD/ada/ast.py:2739
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Task_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_183;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_183 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_183;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3605 '<Block>' Let_Result None
--# scope-start
--# expr-start 3604 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2743
--# expr-start 3603 '<at(<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2743
--# expr-start 3602 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .all_env_els_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2743
--# expr-start 3599 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:2744
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 3599
Bound_Origin := Fld; 
--# expr-start 3601 '<FieldAccess .all_env_els_impl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2744
--# expr-start 3600 '<FieldAccess .defining_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2744
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3600
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3601
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3602
Get_Result := Get (Dyn_Var_Bind_Result, 0, True); 
--# expr-done 3603
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 3604
Let_Result := Cast_Result; 
--# end
--# expr-done 3605
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_183;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_183;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Task_Previous_Part;
--# end
--# property-start Body.protected_previous_part /usr/src/packages/BUILD/ada/ast.py:2748
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Protected_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_184;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Protected_Type_Decl;
Match_1 : Internal_Entity_Single_Protected_Decl;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Protected_Type_Decl;
Cast_Expr : Internal_Entity_Protected_Type_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Single_Protected_Decl;
Cast_Expr_1 : Internal_Entity_Single_Protected_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_184 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_184;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3610 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3608 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2752
--# expr-start 3607 '<FieldAccess .env_elements>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2752
--# expr-start 3606 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2752
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3606
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 3607
Get_Result := Get (Fld_1, 0, True); 
--# expr-done 3608
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 3609 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2752
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Protected_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Protected_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind prot_type Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Single_Protected_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind prot_decl Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Incomplete_Tagged_Type_Decl | Ada_Task_Type_Decl .. Ada_Package_Renaming_Decl | Ada_Single_Task_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Basic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3609
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_3 := Result_Var; 
--# end
--# expr-done 3610
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_184;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_184;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Protected_Previous_Part;
--# end
--# property-start Body.unbound_previous_part /usr/src/packages/BUILD/ada/ast.py:2760
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Unbound_Previous_Part
  (Node : Bare_Body_Node
      ; Env : Lexical_Env
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Base_Subp_Body;
Match_1 : Internal_Entity_Subp_Body_Stub;
Match_2 : Internal_Entity_Entry_Body;
Match_3 : Internal_Entity_Package_Body;
Match_4 : Internal_Entity_Package_Body_Stub;
Match_5 : Internal_Entity_Protected_Body;
Match_6 : Internal_Entity_Protected_Body_Stub;
Match_7 : Internal_Entity_Task_Body;
Match_8 : Internal_Entity_Task_Body_Stub;
Pp_Next_Part : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Entry_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Internal_Entity_Basic_Decl;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Subp_Body_Stub;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Entry_Body;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Package_Body;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity_Package_Body_Stub;
Let_Result_4 : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity_Protected_Body;
Let_Result_5 : Internal_Entity_Basic_Decl;
Scope_Result_5 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Protected_Body_Stub;
Let_Result_6 : Internal_Entity_Basic_Decl;
Scope_Result_6 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Task_Body;
Let_Result_7 : Internal_Entity_Basic_Decl;
Scope_Result_7 : Internal_Entity_Basic_Decl;
Cast_Result_9 : Internal_Entity_Task_Body_Stub;
Let_Result_8 : Internal_Entity_Basic_Decl;
Scope_Result_8 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Fld_9 : Internal_Entity_Basic_Decl;
Scope_Result_9 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Is_A : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result_9 : Internal_Entity_Basic_Decl;
Is_A_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_10 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3627 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 3620 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2765
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Body_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 3611 '<FieldAccess .subp_previous_part>' Fld /usr/src/packages/BUILD/ada/ast.py:2766
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subp_previous_part
Fld := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3611
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subp_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subp_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 3612 '<FieldAccess .subp_previous_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2767
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subp_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Subp_Previous_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3612
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 3613 '<FieldAccess .entry_previous_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2768
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.entry_previous_part
Fld_2 := Libadalang.Implementation.Body_Node_P_Entry_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3613
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Let_Result_2 := Cast_Result_3; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_4; 
--# bind _ Match_3
--# expr-start 3614 '<FieldAccess .package_previous_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2769
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.package_previous_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3614
Let_Result_3 := Fld_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Package_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_5; 
--# bind _ Match_4
--# expr-start 3615 '<FieldAccess .package_previous_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2770
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.package_previous_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Package_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3615
Let_Result_4 := Fld_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Protected_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_6; 
--# bind _ Match_5
--# expr-start 3616 '<FieldAccess .protected_previous_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2771
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.protected_previous_part
Fld_5 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3616
Let_Result_5 := Fld_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Protected_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Protected_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_7; 
--# bind _ Match_6
--# expr-start 3617 '<FieldAccess .protected_previous_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2772
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.protected_previous_part
Fld_6 := Libadalang.Implementation.Body_Node_P_Protected_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3617
Let_Result_6 := Fld_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_8; 
--# bind _ Match_7
--# expr-start 3618 '<FieldAccess .task_previous_part>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2773
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.task_previous_part
Fld_7 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3618
Let_Result_7 := Fld_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Task_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_9; 
--# bind _ Match_8
--# expr-start 3619 '<FieldAccess .task_previous_part>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2774
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.task_previous_part
Fld_8 := Libadalang.Implementation.Body_Node_P_Task_Previous_Part (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 3619
Let_Result_8 := Fld_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
end case;
--# expr-done 3620
Block_Var := Match_Result; 
--# bind pp Block_Var
--# expr-start 3626 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2781
--# expr-start 3621 '<IsA BodyStub>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:2782
Is_A_1 := Ent.Node /= null 
and then Ent.Node.Kind in Ada_Body_Stub; 
--# expr-done 3621
if Is_A_1 then
   If_Result_1 := Block_Var;
else
   --# expr-start 3625 '<Let pp_next_part>' Let_Result_9 /usr/src/packages/BUILD/ada/ast.py:2784
--# scope-start
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 3622 '<FieldAccess .next_part_for_decl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2784
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 3622
Scope_Result_9 := Fld_9;
   --# end
   Result_Var := Scope_Result_9;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Pp_Next_Part := Result_Var; 
--# bind pp_next_part Pp_Next_Part
--# expr-start 3624 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:2784
--# expr-start 3623 '<IsA BodyStub>' Is_A /usr/src/packages/BUILD/ada/ast.py:2785
Is_A := Pp_Next_Part.Node /= null 
and then Pp_Next_Part.Node.Kind in Ada_Body_Stub; 
--# expr-done 3623
if Is_A then
   If_Result := Pp_Next_Part;
else
   If_Result := Block_Var;
end if;
--# expr-done 3624
Let_Result_9 := If_Result; 
--# end
--# expr-done 3625
   If_Result_1 := Let_Result_9;
end if;
--# expr-done 3626
Let_Result_10 := If_Result_1; 
--# end
--# expr-done 3627
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Unbound_Previous_Part;
--# end
--# property-start Body.previous_part_internal /usr/src/packages/BUILD/ada/ast.py:2793
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Internal
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_186;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
As_Entity : Internal_Entity_Body_Node;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_186 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_186;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3634 '<Block>' Let_Result None
--# scope-start
--# expr-start 3633 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <as_entity(<FieldAccess .node>)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2807
--# expr-start 3628 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:2808
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 3628
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3632 '<as_entity(<FieldAccess .node>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:2809
--# expr-start 3631 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2809
--# expr-start 3630 '<FieldAccess .unbound_previous_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2809
--# expr-start 3629 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2809
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Node;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3629
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.unbound_previous_part
Fld_1 := Libadalang.Implementation.Body_Node_P_Unbound_Previous_Part (Node => If_Result.Node, Env => Bound_Env, Imprecise_Fallback => Imprecise_Fallback, E_Info => If_Result.Info);
--# end
--# expr-done 3630
Fld_2 := Fld_1.Node;
--# expr-done 3631
Is_Null_1 := Fld_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Fld_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 3632
Dyn_Var_Bind_Result := If_Result_1; 
--# expr-done 3633
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3634
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_186;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_186;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Previous_Part_Internal;
--# end
--# property-start Body.previous_part /usr/src/packages/BUILD/ada/ast.py:2815
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Body_Node_Body_Node_P_Previous_Part,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Body_Node,
                                As_Bare_Body_Node => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Boolean,
                                      As_Boolean => Imprecise_Fallback);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 3636 '<Block>' Let_Result None
--# scope-start
--# expr-start 3635 '<FieldAccess .previous_part_internal>' Fld /usr/src/packages/BUILD/ada/ast.py:2820
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3635
Let_Result := Fld; 
--# end
--# expr-done 3636
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Previous_Part;
--# end
--# property-start Body.decl_part /usr/src/packages/BUILD/ada/ast.py:2823
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Decl_Part
  (Node : Bare_Body_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Internal_Entity_Basic_Decl;
Match_0 : Internal_Entity_Body_Stub;
Match_1 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Body_Stub;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3640 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 3637 '<FieldAccess .previous_part_internal>' Fld /usr/src/packages/BUILD/ada/ast.py:2827
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3637
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind prev_part Var_Expr
--# expr-start 3639 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2828
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Package_Body_Stub .. Ada_Task_Body_Stub =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Body_Stub
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind stub Match_0
--# expr-start 3638 '<FieldAccess .previous_part_internal>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2831
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.previous_part_internal
Fld_1 := Libadalang.Implementation.Body_Node_P_Previous_Part_Internal (Node => Match_0.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3638
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Subp_Renaming_Decl | Ada_Entry_Body .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind other Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3639
Scope_Result_2 := Match_Result;
   --# end
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result_2 := Result_Var; 
--# end
--# expr-done 3640
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Decl_Part;
--# end
--# property-start Body.safe_generic_decl_part /usr/src/packages/BUILD/ada/ast.py:2837
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Safe_Generic_Decl_Part
  (Node : Bare_Body_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_2141;
            procedure Finalizer_Scope_2142;
      Match_0 : Internal_Entity_Package_Body;
Match_1 : Internal_Entity_Base_Subp_Body;
Match_2 : Internal_Entity_Body_Node;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Var_Expr_2 : Internal_Entity_Generic_Subp_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Package_Decl;
Scope_Result : Internal_Entity_Generic_Package_Decl;
Result_Var : Internal_Entity_Generic_Package_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Env_Parent : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Generic_Subp_Decl;
Scope_Result_1 : Internal_Entity_Generic_Subp_Decl;
Get_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Generic_Subp_Decl;
Result_Var_1 : Internal_Entity_Generic_Subp_Decl;
Scope_Result_2 : Internal_Entity_Generic_Subp_Decl;
Result_Var_2 : Internal_Entity_Generic_Subp_Decl;
Match_Prefix : Internal_Entity_Body_Node;
Cast_Result_3 : Internal_Entity_Package_Body;
Cast_Result_4 : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Cast_Result_5 : Internal_Entity_Base_Subp_Body;
Cast_Result_6 : Internal_Entity_Generic_Decl;
Let_Result_1 : Internal_Entity_Generic_Decl;
Scope_Result_4 : Internal_Entity_Generic_Decl;
Cast_Result_7 : Internal_Entity_Body_Node;
Let_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result_5 : Internal_Entity_Generic_Decl;
Match_Result : Internal_Entity_Generic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Generic_Decl;
Let_Result_3 : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_2141 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Parent);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_2141;
            procedure Finalizer_Scope_2142 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2142;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3654 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3653 '<bind(<DynamicVariable imprecise_fallback>, False, <Match>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:2842
Bound_Imprecise_Fallback := False; 
--# expr-start 3652 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2844
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Body_Node (Match_Prefix.Node.Kind) is
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind pkg_body Match_0
--# expr-start 3641 '<FieldAccess .decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:2846
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 3641
Var_Expr := Fld;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 3643 '<Cast to GenericPackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2847
--# expr-start 3642 '<parent(<Var d>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2847
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 3642
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Generic_Package_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 3643
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Generic_Package_Decl;
end if;
      Cast_Result_4 := Create_Internal_Entity_Generic_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
Let_Result := Cast_Result_4; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Expr_Function .. Ada_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Base_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_5; 
--# bind bod Match_1
--# expr-start 3647 '<EnvGet(<env_parent(<FieldAccess .children_env>)>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2855
--# expr-start 3645 '<env_parent(<FieldAccess .children_env>)>' Env_Parent /usr/src/packages/BUILD/ada/ast.py:2855
--# expr-start 3644 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2855
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Match_1.Node, E_Info => Match_1.Info);
--# expr-done 3644
Env_Parent := AST_Envs.Parent (Fld_2); 
--# expr-done 3645
--# expr-start 3646 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2856
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 3646
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Env_Parent, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3647
Var_Expr_1 := Env_Get_Result;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind results Var_Expr_1
--# expr-start 3649 '<Cast to GenericSubpDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:2859
--# expr-start 3648 '<at(<Var results>, 1)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2859
Get_Result := Get (Var_Expr_1, 1, True); 
--# expr-done 3648
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 3649
Var_Expr_2 := Cast_Result_1;
if Var_Expr_2 /= No_Entity_Generic_Subp_Decl then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_1 := Var_Expr_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 3651 '<Cast to GenericSubpDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:2860
--# expr-start 3650 '<at(<Var results>, 2)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:2860
Get_Result_1 := Get (Var_Expr_1, 2, True); 
--# expr-done 3650
   if Get_Result_1.Node = null
      or else Get_Result_1.Node.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Get_Result_1.Node,
         Info => Get_Result_1.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Subp_Decl;
   end if;
--# expr-done 3651
   Result_Var_1 := Cast_Result_2;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
      Finalizer_Scope_2142;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Generic_Subp_Decl;
end if;
      Cast_Result_6 := Create_Internal_Entity_Generic_Decl
        (Node => Result_Var_2.Node,
         Info => Result_Var_2.Info);
Let_Result_1 := Cast_Result_6; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
      Finalizer_Scope_2141;
            Match_Result := Scope_Result_4; 
         when Ada_Package_Body_Stub .. Ada_Entry_Body | Ada_Protected_Body .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_7; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Generic_Decl; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 3652
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 3653
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3654
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2141;
               Finalizer_Scope_2142;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Safe_Generic_Decl_Part;
--# end
--# property-start Body.is_subunit /usr/src/packages/BUILD/ada/ast.py:2868
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Is_Subunit
  (Node : Bare_Body_Node
  )
   return Boolean
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3657 '<Block>' Let_Result None
--# scope-start
--# expr-start 3656 '<IsA Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:2869
--# expr-start 3655 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:2869
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3655
Is_A := Fld /= null 
and then Fld.Kind in Ada_Subunit_Range; 
--# expr-done 3656
Let_Result := Is_A; 
--# end
--# expr-done 3657
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Is_Subunit;
--# end
--# property-start Body.subunit_root /usr/src/packages/BUILD/ada/ast.py:2872
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Subunit_Root
  (Node : Bare_Body_Node
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Var_Expr : Bare_Subunit;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Subunit;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3661 '<Block>' Let_Result None
--# scope-start
--# expr-start 3659 '<Cast to Subunit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:2876
--# expr-start 3658 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:2876
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3658
   if Fld = null
      or else Fld.Kind in Ada_Subunit_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 3659
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind su Var_Expr
--# expr-start 3660 '<FieldAccess .body_root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2876
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Var_Expr);
--# end
--# expr-done 3660
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3661
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Subunit_Root;
--# end
--# property-start Body.body_scope /usr/src/packages/BUILD/ada/ast.py:2879
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Body_Scope
  (Node : Bare_Body_Node
      ; Follow_Private : Boolean
      ; Force_Decl : Boolean
         := False
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Node :=
     Internal_Entity_Body_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind follow_private Follow_Private
   --# bind force_decl Force_Decl
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_192;
            procedure Finalizer_Scope_2101;
            procedure Finalizer_Scope_2102;
      Block_Var : Lexical_Env := Empty_Env;
Block_Var_1 : Lexical_Env := Empty_Env;
Dns : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity;
Fld : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
Fld_5 : Internal_Entity_Defining_Name;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Env_Node : Bare_Ada_Node;
Cast_Expr : Bare_Body_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
Fld_9 : Boolean;
If_Result_3 : Boolean;
Fld_10 : Internal_Entity_Defining_Name;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Bare_Ada_Node;
Fld_13 : Lexical_Env := Empty_Env;
If_Result_4 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
Env_Node_1 : Bare_Ada_Node;
Cast_Result_1 : Bare_Generic_Package_Decl;
Fld_14 : Bare_Generic_Package_Internal;
Fld_15 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_16 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Env_Node_2 : Bare_Ada_Node;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Lexical_Env := Empty_Env;
Let_Result_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_192 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Dns);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_192;
            procedure Finalizer_Scope_2101 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2101;
            procedure Finalizer_Scope_2102 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_2102;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3695 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 3683 '<Cond>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:2888
--# expr-start 3664 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:2893
--# expr-start 3663 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:2893
--# expr-start 3662 '<FieldAccess .subunit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:2893
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_root
Fld := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 3662
Is_Null := Fld.Node = null; 
--# expr-done 3663
Not_Val := not (Is_Null); 
--# expr-done 3664
if Not_Val then
   --# expr-start 3666 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2893
--# expr-start 3665 '<FieldAccess .subunit_root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2893
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.subunit_root
Fld_1 := Libadalang.Implementation.Body_Node_P_Subunit_Root (Node => Self);
--# end
--# expr-done 3665
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 3666
   If_Result_6 := Fld_2;
else
--# expr-start 3667 '<FieldAccess .is_subprogram>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2898
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Self);
--# end
--# expr-done 3667
if Fld_3 then
   --# expr-start 3668 '<FieldAccess .is_compilation_unit_root>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2898
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3668
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 3677 '<Let dns>' Let_Result /usr/src/packages/BUILD/ada/ast.py:2899
--# scope-start
--# expr-start 3670 '<FieldAccess .scope>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2899
--# expr-start 3669 '<FieldAccess .defining_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2899
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3669
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_5.Node, Env => Env);
--# end
--# expr-done 3670
Dns := Fld_6; Inc_Ref (Dns);
--# bind dns Dns
--# expr-start 3676 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:2902
--# expr-start 3671 '<IsNull>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:2902
Is_Equal := Equivalent (Dns, Empty_Env); 
--# expr-done 3671
if Is_Equal then
   If_Result_1 := True;
else
   --# expr-start 3673 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:2902
--# expr-start 3672 '<env_node(<Var dns>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:2902
Env_Node := AST_Envs.Env_Node (Dns); 
--# expr-done 3672
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal_1 := Env_Node = Cast_Result; 
--# expr-done 3673
   If_Result_1 := Is_Equal_1;
end if;
if If_Result_1 then
   --# expr-start 3675 '<FieldAccess .parent_scope>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2903
--# expr-start 3674 '<FieldAccess .defining_name>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2903
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3674
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld_7.Node, Env => Env);
--# end
--# expr-done 3675
   If_Result_2 := Fld_8;
else
   If_Result_2 := Dns;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 3676
Let_Result := If_Result_2; Inc_Ref (Let_Result);
--# end
--# expr-done 3677
   If_Result_5 := Let_Result;
else
--# expr-start 3678 '<FieldAccess .is_compilation_unit_root>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2908
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3678
if Fld_9 then
   If_Result_3 := True;
else
   If_Result_3 := Force_Decl;
end if;
if If_Result_3 then
   --# expr-start 3680 '<FieldAccess .scope>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:2909
--# expr-start 3679 '<FieldAccess .defining_name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:2909
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3679
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld_10.Node, Env => Env);
--# end
--# expr-done 3680
   If_Result_4 := Fld_11;
else
   --# expr-start 3682 '<FieldAccess .children_env>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:2913
--# expr-start 3681 '<parent(<Var Self>)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:2913
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Parent; 
--# expr-done 3681
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12);
--# expr-done 3682
   If_Result_4 := Fld_13;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 3683
Block_Var := If_Result_6; Inc_Ref (Block_Var);
--# bind scope Block_Var
--# expr-start 3685 '<Cast to GenericPackageDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:2918
--# expr-start 3684 '<env_node(<Var scope>)>' Env_Node_1 /usr/src/packages/BUILD/ada/ast.py:2918
Env_Node_1 := AST_Envs.Env_Node (Block_Var); 
--# expr-done 3684
   if Env_Node_1 = null
      or else Env_Node_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_1 := Env_Node_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 3685
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 3687 '<FieldAccess .children_env>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:2919
--# expr-start 3686 '<FieldAccess .package_decl>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:2919
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 3686
      if Fld_14 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Children_Env (Node => Fld_14);
--# expr-done 3687
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2101;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
      Inc_Ref (Result_Var);
Block_Var_1 := Result_Var; Inc_Ref (Block_Var_1);
--# bind public_scope Block_Var_1
--# expr-start 3694 '<If>' If_Result_8 /usr/src/packages/BUILD/ada/ast.py:2925
if Follow_Private then
--# expr-start 3688 '<env_node(<Var public_scope>)>' Env_Node_2 /usr/src/packages/BUILD/ada/ast.py:2928
Env_Node_2 := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 3688
Var_Expr_1 := Env_Node_2;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3689 '<IsA BasePackageDecl, SingleProtectedDecl, ProtectedTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:2928
Is_A := Var_Expr_1 /= null 
and then Var_Expr_1.Kind in Ada_Base_Package_Decl | Ada_Single_Protected_Decl_Range | Ada_Protected_Type_Decl_Range; 
--# expr-done 3689
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_7 := Result_Var_2;
else
   If_Result_7 := False;
end if;
if If_Result_7 then
--# expr-start 3692 '<at(<EnvGet(<Var public_scope>, __privatepart)>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:2933
--# expr-start 3691 '<EnvGet(<Var public_scope>, __privatepart)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:2933
--# expr-start 3690 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 3690
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_1, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 3691
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 3692
Var_Expr_2 := Get_Result;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_2
--# expr-start 3693 '<FieldAccess .children_env>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:2936
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 3693
Scope_Result_1 := Fld_16;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2102;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_1;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_8 := Result_Var_1;
else
   If_Result_8 := Block_Var_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 3694
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 3695
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_192;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_192;
               Finalizer_Scope_2101;
               Finalizer_Scope_2102;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Body_Scope;
--# end
--# property-start Body.previous_part_env_name /usr/src/packages/BUILD/ada/ast.py:2629
pragma Warnings (Off, "is not referenced");
 function Body_Node_P_Previous_Part_Env_Name
  (Node : Bare_Body_Node
  )
   return Symbol_Type
is
   Self : Bare_Body_Node := Bare_Body_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_179;
      Fld : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym : Symbol_Type;
Fld_1 : Character_Type_Array_Access;
Sym_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_179 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_179;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3704 '<Block>' Let_Result None
--# scope-start
--# expr-start 3703 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:2636
--# expr-start 3696 '<FieldAccess .is_subunit>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2637
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld_2 := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3696
if Fld_2 then
   --# expr-start 3700 '<to_symbol(<Concat>)>' Sym /usr/src/packages/BUILD/ada/ast.py:2638
--# expr-start 3699 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2638
--# expr-start 3697 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2638
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3697
--# expr-start 3698 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2638
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (95), 2 => Character_Type'Val (95), 3 => Character_Type'Val (115), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (98))); 
--# expr-done 3698
Concat_Result := Concat (Fld, Array_Lit); 
--# expr-done 3699
Sym := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 3700
   If_Result := Sym;
else
   --# expr-start 3702 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym_1 /usr/src/packages/BUILD/ada/ast.py:2639
--# expr-start 3701 '<FieldAccess .top_level_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2639
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3701
Sym_1 := String_To_Symbol (Self.Unit.Context, Fld_1); 
--# expr-done 3702
   If_Result := Sym_1;
end if;
--# expr-done 3703
Let_Result := If_Result; 
--# end
--# expr-done 3704
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_179;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_179;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Node_P_Previous_Part_Env_Name;
--# end
function Body_Node_P_Subunit_Decl_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Body_Node_P_Subunit_Decl_Env (E.Node, E_Info);
begin
      return Result;
end;
function Body_Node_P_Body_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Body_Node_P_Body_Decl_Scope (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Base_Subp_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Base_Subp_Body_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Base_Subp_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_152 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:14403" & ")";
         end if;
      return Initial_Env;
   end Bare_Base_Subp_Body_Initial_Env_Getter_Fn;
      procedure Base_Subp_Body_Pre_Env_Actions
        (Self            : Bare_Base_Subp_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_150 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_151 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Base_Subp_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_154 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_155 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_153 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:14408");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_156 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Base_Subp_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_158 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_159 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_157 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:14417");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_160 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_162 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_161 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_164 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_163 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_166 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_165 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Spec_Of_Subp_Body'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_168 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_167 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_170 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_169 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Base_Subp_Body
        (Self : Bare_Base_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Self.Base_Subp_Body_F_Overriding := Base_Subp_Body_F_Overriding;
            Self.Base_Subp_Body_F_Subp_Spec := Base_Subp_Body_F_Subp_Spec;
      end Initialize_Fields_For_Base_Subp_Body;
   function Base_Subp_Body_F_Overriding
     (Node : Bare_Base_Subp_Body) return Bare_Overriding_Node
   is
   begin
         return Node.Base_Subp_Body_F_Overriding;
   end;
   function Base_Subp_Body_F_Subp_Spec
     (Node : Bare_Base_Subp_Body) return Bare_Subp_Spec
   is
   begin
         return Node.Base_Subp_Body_F_Subp_Spec;
   end;
--# property-start BaseSubpBody.defining_names /usr/src/packages/BUILD/ada/ast.py:14350
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Names
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1086;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1086 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1086;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3707 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14350
--# expr-start 3706 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14350
--# expr-start 3705 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:14350
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3705
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3706
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3707
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1086;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1086;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Defining_Names;
--# end
--# property-start BaseSubpBody.defining_env /usr/src/packages/BUILD/ada/ast.py:14353
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Defining_Env
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1087;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Internal_Entity_Info;
Fld_6 : Internal_Metadata;
Fld_7 : Boolean;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Lexical_Env := Empty_Env;
Fld_9 : Internal_Entity_Subp_Spec;
Fld_10 : Lexical_Env := Empty_Env;
Fld_11 : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1087 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1087;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3724 '<Block>' Let_Result None
--# scope-start
--# expr-start 3723 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:14354
--# expr-start 3708 '<FieldAccess .in_scope>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:14355
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.in_scope
Fld_11 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 3708
if Fld_11 then
   --# expr-start 3720 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14357
--# expr-start 3709 '<FieldAccess .subp_spec_or_null>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14358
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3709
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3713 '<FieldAccess .paramless(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14358
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3712 '<FieldAccess .dottable_subp>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14359
--# expr-start 3711 '<FieldAccess .md>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14359
--# expr-start 3710 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14359
Fld_5 := Ent.Info;
--# expr-done 3710
Fld_6 := Fld_5.MD;
--# expr-done 3711
Fld_7 := Fld_6.Dottable_Subp;
--# expr-done 3712
--# property-call-start BaseFormalParamHolder.paramless
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr.Node, Dottable_Subp => Fld_7, Can_Be => True);
--# end
--# expr-done 3713
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 3718 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:14360
--# expr-start 3717 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:14360
--# expr-start 3714 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:14361
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3714
--# expr-start 3716 '<FieldAccess .defining_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14361
--# expr-start 3715 '<FieldAccess .subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14361
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3715
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_2 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3716
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_2)); 
--# expr-done 3717
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3718
   If_Result := Group_Env;
else
   --# expr-start 3719 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14363
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3719
   If_Result := Fld_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 3720
   If_Result_1 := If_Result;
else
   --# expr-start 3722 '<FieldAccess .defining_env>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:14366
--# expr-start 3721 '<FieldAccess .subp_spec>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:14366
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3721
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.defining_env
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 3722
   If_Result_1 := Fld_10;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3723
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3724
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1087;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1087;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Defining_Env;
--# end
--# property-start BaseSubpBody.type_expression /usr/src/packages/BUILD/ada/ast.py:14369
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Type_Expression
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3726 '<FieldAccess .returns>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14369
--# expr-start 3725 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:14369
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3725
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3726
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Type_Expression;
--# end
--# property-start BaseSubpBody.expr_type /usr/src/packages/BUILD/ada/ast.py:14372
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Expr_Type
  (Node : Bare_Base_Subp_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3729 '<Block>' Let_Result None
--# scope-start
--# expr-start 3727 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:14373
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3727
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 3728 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14373
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 3728
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 3729
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Expr_Type;
--# end
--# property-start BaseSubpBody.initial_env_name /usr/src/packages/BUILD/ada/ast.py:14376
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Initial_Env_Name
  (Node : Bare_Base_Subp_Body
      ; Follow_Private : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind follow_private Follow_Private
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Fld_2 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3734 '<Block>' Let_Result None
--# scope-start
--# expr-start 3733 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14377
--# expr-start 3730 '<FieldAccess .is_library_item>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14378
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 3730
if Fld_2 then
   --# expr-start 3731 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14379
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => Follow_Private);
--# end
--# expr-done 3731
   If_Result := Fld;
else
   --# expr-start 3732 '<FieldAccess .body_initial_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14380
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_initial_env_name
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 3732
   If_Result := Fld_1;
end if;
--# expr-done 3733
Let_Result := If_Result; 
--# end
--# expr-done 3734
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Initial_Env_Name;
--# end
--# property-start BaseSubpBody.previous_part_env_name /usr/src/packages/BUILD/ada/ast.py:14384
pragma Warnings (Off, "is not referenced");
 function Base_Subp_Body_P_Previous_Part_Env_Name
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1091;
      Fld : Boolean;
Fld_1 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym : Symbol_Type;
Fld_2 : Boolean;
Fld_3 : Character_Type_Array_Access;
Sym_1 : Symbol_Type;
If_Result : Symbol_Type;
If_Result_1 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_1091 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1091;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3744 '<Block>' Let_Result None
--# scope-start
--# expr-start 3743 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:14390
--# expr-start 3735 '<FieldAccess .is_subunit>' Fld /usr/src/packages/BUILD/ada/ast.py:14391
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3735
if Fld then
   --# expr-start 3739 '<to_symbol(<Concat>)>' Sym /usr/src/packages/BUILD/ada/ast.py:14392
--# expr-start 3738 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:14392
--# expr-start 3736 '<FieldAccess .top_level_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14392
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3736
--# expr-start 3737 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:14392
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (95), 2 => Character_Type'Val (95), 3 => Character_Type'Val (115), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (98))); 
--# expr-done 3737
Concat_Result := Concat (Fld_1, Array_Lit); 
--# expr-done 3738
Sym := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 3739
   If_Result_1 := Sym;
else
--# expr-start 3740 '<FieldAccess .is_library_item>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14394
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 3740
if Fld_2 then
   --# expr-start 3742 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym_1 /usr/src/packages/BUILD/ada/ast.py:14395
--# expr-start 3741 '<FieldAccess .top_level_env_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14395
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3741
Sym_1 := String_To_Symbol (Self.Unit.Context, Fld_3); 
--# expr-done 3742
   If_Result := Sym_1;
else
   If_Result := null;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 3743
Let_Result := If_Result_1; 
--# end
--# expr-done 3744
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1091;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1091;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Subp_Body_P_Previous_Part_Env_Name;
--# end
--# property-start BaseSubpBody._env_do_150 /usr/src/packages/BUILD/ada/ast.py:14401
pragma Warnings (Off, "is not referenced");
 function Env_Do_150
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3745 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:14401
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3745
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_150;
--# end
--# property-start BaseSubpBody._initial_env_name_151 /usr/src/packages/BUILD/ada/ast.py:14404
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_151
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3746 '<FieldAccess .initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14404
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => True);
--# end
--# expr-done 3746
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_151;
--# end
--# property-start BaseSubpBody._initial_env_152 /usr/src/packages/BUILD/ada/ast.py:14405
pragma Warnings (Off, "is not referenced");
 function Initial_Env_152
  (Node : Bare_Base_Subp_Body
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1322;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1322 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1322;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3747 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:14405
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3747
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1322;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1322;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_152;
--# end
--# property-start BaseSubpBody._env_mappings_153 /usr/src/packages/BUILD/ada/ast.py:14408
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_153
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1323;
      Fld : Symbol_Type;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1323 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1323;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3749 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14408
--# expr-start 3748 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:14409
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3748
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3749
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1323;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1323;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_153;
--# end
--# property-start BaseSubpBody._dest_env_name_154 /usr/src/packages/BUILD/ada/ast.py:14411
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_154
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3750 '<FieldAccess .initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14411
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpBody.initial_env_name
Fld := Libadalang.Implementation.Base_Subp_Body_P_Initial_Env_Name (Node => Self, Follow_Private => False);
--# end
--# expr-done 3750
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_154;
--# end
--# property-start BaseSubpBody._fallback_env_155 /usr/src/packages/BUILD/ada/ast.py:14412
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_155
  (Node : Bare_Base_Subp_Body
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1325;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1325 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1325;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3751 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:14412
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 3751
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1325;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1325;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_155;
--# end
--# property-start BaseSubpBody._env_trans_parent_156 /usr/src/packages/BUILD/ada/ast.py:14400
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_156
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_156;
--# end
--# property-start BaseSubpBody._env_mappings_157 /usr/src/packages/BUILD/ada/ast.py:14417
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_157
  (Node : Bare_Base_Subp_Body
  )
   return Internal_Env_Assoc
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1327;
      Sym : Symbol_Type;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1327 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1327;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3753 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14417
--# expr-start 3752 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3752
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3753
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1327;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1327;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_157;
--# end
--# property-start BaseSubpBody._dest_env_name_158 /usr/src/packages/BUILD/ada/ast.py:14420
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_158
  (Node : Bare_Base_Subp_Body
  )
   return Symbol_Type
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3754 '<FieldAccess .previous_part_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14420
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 3754
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_158;
--# end
--# property-start BaseSubpBody._fallback_env_159 /usr/src/packages/BUILD/ada/ast.py:14421
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_159
  (Node : Bare_Base_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Subp_Body :=
     Internal_Entity_Base_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1329;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
If_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1329 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result);
            end Finalizer_Scope_1329;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3760 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14421
--# expr-start 3755 '<FieldAccess .has_top_level_env_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14426
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.has_top_level_env_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Has_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3755
if Fld_3 then
   If_Result := Empty_Env;
else
   --# expr-start 3759 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14428
--# expr-start 3756 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:14429
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3756
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3758 '<FieldAccess .initial_env(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14430
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3757 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14431
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3757
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3758
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3759
   If_Result := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result);
--# expr-done 3760
      Property_Result := If_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1329;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1329;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_159;
--# end
--# property-start BaseSubpBody._env_do_160 /usr/src/packages/BUILD/ada/ast.py:14438
pragma Warnings (Off, "is not referenced");
 function Env_Do_160
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1330;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1330 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1330;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3761 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:14438
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3761
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1330;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1330;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_160;
--# end
--# property-start BaseSubpBody._ref_env_nodes_161 /usr/src/packages/BUILD/ada/ast.py:14441
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_161
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1331;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1331 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1331;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3762 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:14441
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3762
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1331;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1331;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_161;
--# end
--# property-start BaseSubpBody._ref_cond_162 /usr/src/packages/BUILD/ada/ast.py:14443
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_162
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3764 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:14443
--# expr-start 3763 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:14443
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3763
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3764
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_162;
--# end
--# property-start BaseSubpBody._ref_env_nodes_163 /usr/src/packages/BUILD/ada/ast.py:14447
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_163
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1333;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1333 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1333;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3765 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:14447
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3765
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1333;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1333;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_163;
--# end
--# property-start BaseSubpBody._ref_cond_164 /usr/src/packages/BUILD/ada/ast.py:14449
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_164
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3767 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:14449
--# expr-start 3766 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:14449
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3766
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3767
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_164;
--# end
--# property-start BaseSubpBody._ref_env_nodes_165 /usr/src/packages/BUILD/ada/ast.py:14456
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_165
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1335;
            procedure Finalizer_Scope_2147;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1335 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1335;
            procedure Finalizer_Scope_2147 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2147;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3768 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14456
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3768
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2147;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1335;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1335;
               Finalizer_Scope_2147;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_165;
--# end
--# property-start BaseSubpBody._ref_cond_166 /usr/src/packages/BUILD/ada/ast.py:14458
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_166
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3770 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:14458
--# expr-start 3769 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:14458
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3769
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range; 
--# expr-done 3770
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_166;
--# end
--# property-start BaseSubpBody._ref_env_nodes_167 /usr/src/packages/BUILD/ada/ast.py:14462
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_167
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1337;
            procedure Finalizer_Scope_2148;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1337 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1337;
            procedure Finalizer_Scope_2148 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2148;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3771 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14462
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3771
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2148;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1337;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1337;
               Finalizer_Scope_2148;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_167;
--# end
--# property-start BaseSubpBody._ref_cond_168 /usr/src/packages/BUILD/ada/ast.py:14464
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_168
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3772 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:14464
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3772
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_168;
--# end
--# property-start BaseSubpBody._ref_env_nodes_169 /usr/src/packages/BUILD/ada/ast.py:14472
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_169
  (Node : Bare_Base_Subp_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1339;
            procedure Finalizer_Scope_2149;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Base_Subp_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1339 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1339;
            procedure Finalizer_Scope_2149 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2149;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3773 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14472
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3773
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2149;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1339;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1339;
               Finalizer_Scope_2149;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_169;
--# end
--# property-start BaseSubpBody._ref_cond_170 /usr/src/packages/BUILD/ada/ast.py:14474
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_170
  (Node : Bare_Base_Subp_Body
  )
   return Boolean
is
   Self : Bare_Base_Subp_Body := Bare_Base_Subp_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3774 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:14474
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3774
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_170;
--# end
   --
   --  Primitives for Bare_Expr_Function
   --
      procedure Initialize_Fields_For_Expr_Function
        (Self : Bare_Expr_Function
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Expr_Function_F_Expr : Bare_Expr
         ; Expr_Function_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Expr_Function_F_Expr := Expr_Function_F_Expr;
            Self.Expr_Function_F_Aspects := Expr_Function_F_Aspects;
      end Initialize_Fields_For_Expr_Function;
   function Expr_Function_F_Expr
     (Node : Bare_Expr_Function) return Bare_Expr
   is
   begin
         return Node.Expr_Function_F_Expr;
   end;
--# property-start ExprFunction.xref_equation /usr/src/packages/BUILD/ada/ast.py:14488
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Equation
  (Node : Bare_Expr_Function
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Expr_Function := Bare_Expr_Function (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr_Function :=
     Internal_Entity_Expr_Function'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1092;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_1092 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_1092;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3782 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14489
--# expr-start 3776 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14489
--# expr-start 3775 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14489
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 3775
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3776
--# expr-start 3781 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:14490
--# expr-start 3778 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14490
--# expr-start 3777 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14490
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Expr_Function_F_Expr, Info => Ent.Info);
--# expr-done 3777
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2.Node);
--# expr-done 3778
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 3780 '<FieldAccess .return_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14490
--# expr-start 3779 '<FieldAccess .subp_spec>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14490
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Base_Subp_Body_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3779
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_5 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_4.Node, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 3780
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Bind_Default_566.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14490") else null)); 
--# expr-done 3781
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14489") else null)); 
--# expr-done 3782
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1092;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1092;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_Function_P_Xref_Equation;
--# end
--# property-start ExprFunction.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:14494
pragma Warnings (Off, "is not referenced");
 function Expr_Function_P_Xref_Entry_Point
  (Node : Bare_Expr_Function
  )
   return Boolean
is
   Self : Bare_Expr_Function := Bare_Expr_Function (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_Function_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Null_Subp_Decl
   --
      procedure Initialize_Fields_For_Null_Subp_Decl
        (Self : Bare_Null_Subp_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Null_Subp_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Null_Subp_Decl_F_Aspects := Null_Subp_Decl_F_Aspects;
      end Initialize_Fields_For_Null_Subp_Decl;
   --
   --  Primitives for Bare_Subp_Body
   --
      procedure Initialize_Fields_For_Subp_Body
        (Self : Bare_Subp_Body
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_F_Aspects : Bare_Aspect_Spec
         ; Subp_Body_F_Decls : Bare_Declarative_Part
         ; Subp_Body_F_Stmts : Bare_Handled_Stmts
         ; Subp_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Body_F_Aspects := Subp_Body_F_Aspects;
            Self.Subp_Body_F_Decls := Subp_Body_F_Decls;
            Self.Subp_Body_F_Stmts := Subp_Body_F_Stmts;
            Self.Subp_Body_F_End_Name := Subp_Body_F_End_Name;
      end Initialize_Fields_For_Subp_Body;
   function Subp_Body_F_Decls
     (Node : Bare_Subp_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Subp_Body_F_Decls;
   end;
   function Subp_Body_F_Stmts
     (Node : Bare_Subp_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Subp_Body_F_Stmts;
   end;
   function Subp_Body_F_End_Name
     (Node : Bare_Subp_Body) return Bare_End_Name
   is
   begin
         return Node.Subp_Body_F_End_Name;
   end;
--# property-start SubpBody.declarative_region /usr/src/packages/BUILD/ada/ast.py:14547
pragma Warnings (Off, "is not referenced");
 function Subp_Body_P_Declarative_Region
  (Node : Bare_Subp_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Subp_Body := Bare_Subp_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body :=
     Internal_Entity_Subp_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3783 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:14547
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Subp_Body_F_Decls, Info => Ent.Info);
--# expr-done 3783
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_P_Declarative_Region;
--# end
   --
   --  Primitives for Bare_Subp_Renaming_Decl
   --
      procedure Initialize_Fields_For_Subp_Renaming_Decl
        (Self : Bare_Subp_Renaming_Decl
         ; Base_Subp_Body_F_Overriding : Bare_Overriding_Node
         ; Base_Subp_Body_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Base_Subp_Body
              (Self, Base_Subp_Body_F_Overriding, Base_Subp_Body_F_Subp_Spec);
            Self.Subp_Renaming_Decl_F_Renames := Subp_Renaming_Decl_F_Renames;
            Self.Subp_Renaming_Decl_F_Aspects := Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Subp_Renaming_Decl;
   function Subp_Renaming_Decl_F_Renames
     (Node : Bare_Subp_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Subp_Renaming_Decl_F_Renames;
   end;
--# property-start SubpRenamingDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:14513
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Subp_Renaming_Decl := Bare_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start SubpRenamingDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:14514
pragma Warnings (Off, "is not referenced");
 function Subp_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Subp_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subp_Renaming_Decl := Bare_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Renaming_Decl :=
     Internal_Entity_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1095;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Renaming_Clause;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Subp_Renaming_Decl;
Cast_Result : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Logic_True : Logic_Equation;
Fld_6 : Internal_Entity_Renaming_Clause;
Fld_7 : Internal_Entity_Name;
Is_A : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Fld_8 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Boolean;
If_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
            procedure Finalizer_Scope_1095 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_1095;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3802 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:14514
--# expr-start 3797 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14515
--# expr-start 3792 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14515
--# expr-start 3786 '<FieldAccess .xref_no_overloading(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14516
--# expr-start 3785 '<FieldAccess .renamed_object>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14516
--# expr-start 3784 '<FieldAccess .renames>' Fld /usr/src/packages/BUILD/ada/ast.py:14516
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3784
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 3785
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3786
--# expr-start 3791 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /usr/src/packages/BUILD/ada/ast.py:14517
--# expr-start 3789 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14518
--# expr-start 3788 '<FieldAccess .renamed_object>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14518
--# expr-start 3787 '<FieldAccess .renames>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14518
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3787
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Fld_3.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_3.Info);
--# expr-done 3788
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4.Node);
--# end
--# expr-done 3789
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 3790 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14519
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 3790
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_5, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 3791
And_Pred := Logic_And (Fld_2, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14515") else null)); 
--# expr-done 3792
--# expr-start 3796 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14527
--# expr-start 3795 '<IsA AttributeRef>' Is_A /usr/src/packages/BUILD/ada/ast.py:14527
--# expr-start 3794 '<FieldAccess .renamed_object>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14527
--# expr-start 3793 '<FieldAccess .renames>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14527
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3793
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Fld_6.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_6.Info);
--# expr-done 3794
Is_A := Fld_7.Node /= null 
and then Fld_7.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 3795
if Is_A then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14528") else null)); 
   If_Result := Logic_False;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14528") else null)); 
   If_Result := Logic_True;
end if;
      Inc_Ref (If_Result);
--# expr-done 3796
And_Pred_1 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14515") else null)); 
--# expr-done 3797
--# expr-start 3801 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:14532
--# expr-start 3800 '<FieldAccess .is_operator_name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:14532
--# expr-start 3799 '<FieldAccess .renamed_object>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:14532
--# expr-start 3798 '<FieldAccess .renames>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14532
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3798
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Fld_8.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_8.Info);
--# expr-done 3799
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_operator_name
Fld_10 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 3800
if Fld_10 then
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14533") else null)); 
   If_Result_1 := Logic_True_1;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14533") else null)); 
   If_Result_1 := Logic_False_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 3801
Or_Pred := Logic_Or (And_Pred_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14514") else null)); 
--# expr-done 3802
      Property_Result := Or_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1095;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1095;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Body_Stub
   --
--# property-start BodyStub.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:2950
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Next_Part_For_Decl
  (Node : Bare_Body_Stub
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Body_Stub := Bare_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Body_Stub :=
     Internal_Entity_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_193;
      Fld : Symbol_Type_Array_Access;
Fld_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_193 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_193;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3806 '<Block>' Let_Result None
--# scope-start
--# expr-start 3805 '<as_entity(<FieldAccess .get_unit_root_decl(...)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2951
--# expr-start 3804 '<FieldAccess .get_unit_root_decl(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2951
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3803 '<FieldAccess .fully_qualified_name_array>' Fld /usr/src/packages/BUILD/ada/ast.py:2952
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.fully_qualified_name_array
Fld := Libadalang.Implementation.Basic_Decl_P_Fully_Qualified_Name_Array (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3803
--# property-call-start AdaNode.get_unit_root_decl
Fld_1 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld, Kind => Unit_Body);
--# end
--# expr-done 3804
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3805
Let_Result := If_Result; 
--# end
--# expr-done 3806
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_193;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_193;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Stub_P_Next_Part_For_Decl;
--# end
--# property-start BodyStub.syntactic_fully_qualified_name /usr/src/packages/BUILD/ada/ast.py:2956
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub := Bare_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_194;
      Block_Var : Symbol_Type_Array_Access;
Block_Var_1 : Bare_Body_Node;
Block_Var_2 : Bare_Compilation_Unit;
Match_0 : Bare_Subp_Body;
Match_1 : Bare_Protected_Body;
Match_2 : Bare_Package_Body;
Match_3 : Bare_Task_Body;
Match_4 : Bare_Ada_Node;
As_Entity : Internal_Entity_Body_Stub;
Is_Null : Boolean;
If_Result : Internal_Entity_Body_Stub;
Fld : Internal_Entity_Defining_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Ada_Node;
Fld_4 : Bare_Ada_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
Cast_Expr : Bare_Subp_Body;
Cast_Result_1 : Bare_Body_Node;
Let_Result : Bare_Body_Node;
Scope_Result : Bare_Body_Node;
Cast_Result_2 : Bare_Protected_Body;
Cast_Expr_1 : Bare_Protected_Body;
Cast_Result_3 : Bare_Body_Node;
Let_Result_1 : Bare_Body_Node;
Scope_Result_1 : Bare_Body_Node;
Cast_Result_4 : Bare_Package_Body;
Cast_Expr_2 : Bare_Package_Body;
Cast_Result_5 : Bare_Body_Node;
Let_Result_2 : Bare_Body_Node;
Scope_Result_2 : Bare_Body_Node;
Cast_Result_6 : Bare_Task_Body;
Cast_Expr_3 : Bare_Task_Body;
Cast_Result_7 : Bare_Body_Node;
Let_Result_3 : Bare_Body_Node;
Scope_Result_3 : Bare_Body_Node;
Cast_Result_8 : Bare_Ada_Node;
Let_Result_4 : Bare_Body_Node;
Scope_Result_4 : Bare_Body_Node;
Match_Result : Bare_Body_Node;
Fld_5 : Bare_Ada_Node;
Fld_6 : Bare_Ada_Node;
Cast_Result_9 : Bare_Compilation_Unit;
Fld_7 : Bare_Ada_Node;
Is_A : Boolean;
Fld_8 : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Bare_Compilation_Unit;
Fld_9 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Let_Result_5 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_194 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_194;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3824 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 3809 '<FieldAccess .as_symbol_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:2964
--# expr-start 3808 '<FieldAccess .defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2964
--# expr-start 3807 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:2964
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Body_Stub;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 3807
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 3808
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld.Node);
--# end
--# expr-done 3809
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind rel_name Block_Var
--# expr-start 3813 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:2975
--# expr-start 3812 '<parent(<parent(<parent(<Var Self>)>)>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:2975
--# expr-start 3811 '<parent(<parent(<Var Self>)>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:2975
--# expr-start 3810 '<parent(<Var Self>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:2975
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Parent; 
--# expr-done 3810
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Fld_2.Parent; 
--# expr-done 3811
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.Parent; 
--# expr-done 3812
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_4; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind b Match_0
Cast_Expr := Match_0; 
      Cast_Result_1 := Cast_Expr;
Let_Result := Cast_Result_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Protected_Body =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_1 := Cast_Result_2; 
--# bind b Match_1
Cast_Expr_1 := Match_1; 
      Cast_Result_3 := Cast_Expr_1;
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind b Match_2
Cast_Expr_2 := Match_2; 
      Cast_Result_5 := Cast_Expr_2;
Let_Result_2 := Cast_Result_5; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_3 := Cast_Result_6; 
--# bind b Match_3
Cast_Expr_3 := Match_3; 
      Cast_Result_7 := Cast_Expr_3;
Let_Result_3 := Cast_Result_7; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := raise Property_Error with "invalid body stub"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 3813
Block_Var_1 := Match_Result; 
--# bind top_body Block_Var_1
--# expr-start 3821 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:2982
--# expr-start 3815 '<IsA LibraryItem>' Is_A /usr/src/packages/BUILD/ada/ast.py:2983
--# expr-start 3814 '<parent(<Var top_body>)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:2983
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Block_Var_1.Parent; 
--# expr-done 3814
Is_A := Fld_7 /= null 
and then Fld_7.Kind in Ada_Library_Item_Range; 
--# expr-done 3815
if Is_A then
   If_Result_1 := True;
else
   --# expr-start 3817 '<IsA Subunit>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:2984
--# expr-start 3816 '<parent(<Var top_body>)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:2984
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Block_Var_1.Parent; 
--# expr-done 3816
Is_A_1 := Fld_8 /= null 
and then Fld_8.Kind in Ada_Subunit_Range; 
--# expr-done 3817
   If_Result_1 := Is_A_1;
end if;
if If_Result_1 then
   --# expr-start 3820 '<Cast to CompilationUnit>' Cast_Result_9 /usr/src/packages/BUILD/ada/ast.py:2986
--# expr-start 3819 '<parent(<parent(<Var top_body>)>)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:2986
--# expr-start 3818 '<parent(<Var top_body>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:2986
      if Block_Var_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Block_Var_1.Parent; 
--# expr-done 3818
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Fld_5.Parent; 
--# expr-done 3819
   if Fld_6 = null
      or else Fld_6.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_9 := Fld_6;
   else
         Cast_Result_9 := No_Bare_Ada_Node;
   end if;
--# expr-done 3820
   If_Result_2 := Cast_Result_9;
else
   If_Result_2 := raise Property_Error with "invalid body stub";
end if;
--# expr-done 3821
Block_Var_2 := If_Result_2; 
--# bind cu Block_Var_2
--# expr-start 3823 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2990
--# expr-start 3822 '<FieldAccess .syntactic_fully_qualified_name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:2990
      if Block_Var_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld_9 := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Block_Var_2);
--# end
--# expr-done 3822
Concat_Result := Concat (Fld_9, Block_Var); 
--# expr-done 3823
Let_Result_5 := Concat_Result; Inc_Ref (Let_Result_5);
--# end
--# expr-done 3824
      Property_Result := Let_Result_5;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_194;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_194;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Stub_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start BodyStub.env_names /usr/src/packages/BUILD/ada/ast.py:2993
pragma Warnings (Off, "is not referenced");
 function Body_Stub_P_Env_Names
  (Node : Bare_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Body_Stub := Bare_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_195;
      Fld : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym : Symbol_Type;
Array_Lit_1 : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_195 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_195;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3830 '<Block>' Let_Result None
--# scope-start
--# expr-start 3829 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:2998
--# expr-start 3828 '<to_symbol(<Concat>)>' Sym /usr/src/packages/BUILD/ada/ast.py:2999
--# expr-start 3827 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:2999
--# expr-start 3825 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:2999
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3825
--# expr-start 3826 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:2999
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (95), 2 => Character_Type'Val (95), 3 => Character_Type'Val (115), 4 => Character_Type'Val (116), 5 => Character_Type'Val (117), 6 => Character_Type'Val (98))); 
--# expr-done 3826
Concat_Result := Concat (Fld, Array_Lit); 
--# expr-done 3827
Sym := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 3828
Array_Lit_1 := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 3829
Let_Result := Array_Lit_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3830
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_195;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_195;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Body_Stub_P_Env_Names;
--# end
   --
   --  Primitives for Bare_Package_Body_Stub
   --
      procedure Package_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Package_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_257 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_258 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_256 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15640");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_259 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_260 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Package_Body_Stub
        (Self : Bare_Package_Body_Stub
         ; Package_Body_Stub_F_Name : Bare_Defining_Name
         ; Package_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Body_Stub_F_Name := Package_Body_Stub_F_Name;
            Self.Package_Body_Stub_F_Aspects := Package_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Package_Body_Stub;
   function Package_Body_Stub_F_Name
     (Node : Bare_Package_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_Stub_F_Name;
   end;
--# property-start PackageBodyStub.defining_names /usr/src/packages/BUILD/ada/ast.py:15637
pragma Warnings (Off, "is not referenced");
 function Package_Body_Stub_P_Defining_Names
  (Node : Bare_Package_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body_Stub :=
     Internal_Entity_Package_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1158;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1158 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1158;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3831 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15637
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3831
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1158;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1158;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_Stub_P_Defining_Names;
--# end
--# property-start PackageBodyStub._env_mappings_256 /usr/src/packages/BUILD/ada/ast.py:15640
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_256
  (Node : Bare_Package_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1426;
      Sym : Symbol_Type;
Cast_Expr : Bare_Package_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1426 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1426;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3833 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15640
--# expr-start 3832 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3832
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3833
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1426;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1426;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_256;
--# end
--# property-start PackageBodyStub._dest_env_name_257 /usr/src/packages/BUILD/ada/ast.py:15643
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_257
  (Node : Bare_Package_Body_Stub
  )
   return Symbol_Type
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1427;
      Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
            procedure Finalizer_Scope_1427 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1427;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3835 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym /usr/src/packages/BUILD/ada/ast.py:15643
--# expr-start 3834 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15643
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3834
Sym := String_To_Symbol (Self.Unit.Context, Fld); 
--# expr-done 3835
      Property_Result := Sym;
   --# end
      Finalizer_Scope_1427;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1427;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_257;
--# end
--# property-start PackageBodyStub._fallback_env_258 /usr/src/packages/BUILD/ada/ast.py:15644
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_258
  (Node : Bare_Package_Body_Stub
  )
   return Lexical_Env
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_258;
--# end
--# property-start PackageBodyStub._env_trans_parent_259 /usr/src/packages/BUILD/ada/ast.py:15639
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_259
  (Node : Bare_Package_Body_Stub
  )
   return Boolean
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_259;
--# end
--# property-start PackageBodyStub._env_names_260 /usr/src/packages/BUILD/ada/ast.py:15646
pragma Warnings (Off, "is not referenced");
 function Env_Names_260
  (Node : Bare_Package_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Package_Body_Stub := Bare_Package_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1430;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1430 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1430;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3836 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:15646
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 3836
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1430;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1430;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_260;
--# end
   --
   --  Primitives for Bare_Protected_Body_Stub
   --
      procedure Protected_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Protected_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_246 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_247 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_245 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15589");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_248 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_249 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Protected_Body_Stub
        (Self : Bare_Protected_Body_Stub
         ; Protected_Body_Stub_F_Name : Bare_Defining_Name
         ; Protected_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Protected_Body_Stub_F_Name := Protected_Body_Stub_F_Name;
            Self.Protected_Body_Stub_F_Aspects := Protected_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Protected_Body_Stub;
   function Protected_Body_Stub_F_Name
     (Node : Bare_Protected_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_Stub_F_Name;
   end;
--# property-start ProtectedBodyStub.defining_names /usr/src/packages/BUILD/ada/ast.py:15586
pragma Warnings (Off, "is not referenced");
 function Protected_Body_Stub_P_Defining_Names
  (Node : Bare_Protected_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body_Stub :=
     Internal_Entity_Protected_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1155;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1155 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1155;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3837 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15586
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3837
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1155;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1155;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Body_Stub_P_Defining_Names;
--# end
--# property-start ProtectedBodyStub._env_mappings_245 /usr/src/packages/BUILD/ada/ast.py:15589
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_245
  (Node : Bare_Protected_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1415;
      Sym : Symbol_Type;
Cast_Expr : Bare_Protected_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1415 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1415;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3839 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15589
--# expr-start 3838 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3838
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3839
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1415;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1415;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_245;
--# end
--# property-start ProtectedBodyStub._dest_env_name_246 /usr/src/packages/BUILD/ada/ast.py:15592
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_246
  (Node : Bare_Protected_Body_Stub
  )
   return Symbol_Type
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1416;
      Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
            procedure Finalizer_Scope_1416 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1416;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3841 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym /usr/src/packages/BUILD/ada/ast.py:15592
--# expr-start 3840 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15592
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3840
Sym := String_To_Symbol (Self.Unit.Context, Fld); 
--# expr-done 3841
      Property_Result := Sym;
   --# end
      Finalizer_Scope_1416;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1416;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_246;
--# end
--# property-start ProtectedBodyStub._fallback_env_247 /usr/src/packages/BUILD/ada/ast.py:15593
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_247
  (Node : Bare_Protected_Body_Stub
  )
   return Lexical_Env
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_247;
--# end
--# property-start ProtectedBodyStub._env_trans_parent_248 /usr/src/packages/BUILD/ada/ast.py:15588
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_248
  (Node : Bare_Protected_Body_Stub
  )
   return Boolean
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_248;
--# end
--# property-start ProtectedBodyStub._env_names_249 /usr/src/packages/BUILD/ada/ast.py:15595
pragma Warnings (Off, "is not referenced");
 function Env_Names_249
  (Node : Bare_Protected_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Protected_Body_Stub := Bare_Protected_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1419;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1419 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1419;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3842 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:15595
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 3842
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1419;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1419;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_249;
--# end
   --
   --  Primitives for Bare_Subp_Body_Stub
   --
      procedure Subp_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Subp_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_250 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:15613");
         Dec_Ref (Mapping.Dest_Env);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_252 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_253 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_251 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15617");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_254 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_255 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Subp_Body_Stub
        (Self : Bare_Subp_Body_Stub
         ; Subp_Body_Stub_F_Overriding : Bare_Overriding_Node
         ; Subp_Body_Stub_F_Subp_Spec : Bare_Subp_Spec
         ; Subp_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Subp_Body_Stub_F_Overriding := Subp_Body_Stub_F_Overriding;
            Self.Subp_Body_Stub_F_Subp_Spec := Subp_Body_Stub_F_Subp_Spec;
            Self.Subp_Body_Stub_F_Aspects := Subp_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Subp_Body_Stub;
   function Subp_Body_Stub_F_Overriding
     (Node : Bare_Subp_Body_Stub) return Bare_Overriding_Node
   is
   begin
         return Node.Subp_Body_Stub_F_Overriding;
   end;
   function Subp_Body_Stub_F_Subp_Spec
     (Node : Bare_Subp_Body_Stub) return Bare_Subp_Spec
   is
   begin
         return Node.Subp_Body_Stub_F_Subp_Spec;
   end;
--# property-start SubpBodyStub.defining_names /usr/src/packages/BUILD/ada/ast.py:15608
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Defining_Names
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body_Stub :=
     Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1156;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1156 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1156;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3845 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:15608
--# expr-start 3844 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15608
--# expr-start 3843 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:15608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3843
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3844
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 3845
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1156;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1156;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_Stub_P_Defining_Names;
--# end
--# property-start SubpBodyStub.type_expression /usr/src/packages/BUILD/ada/ast.py:15626
pragma Warnings (Off, "is not referenced");
 function Subp_Body_Stub_P_Type_Expression
  (Node : Bare_Subp_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subp_Body_Stub :=
     Internal_Entity_Subp_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3847 '<FieldAccess .returns>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15626
--# expr-start 3846 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:15626
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Subp_Body_Stub_F_Subp_Spec, Info => Ent.Info);
--# expr-done 3846
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 3847
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subp_Body_Stub_P_Type_Expression;
--# end
--# property-start SubpBodyStub._env_mappings_250 /usr/src/packages/BUILD/ada/ast.py:15613
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_250
  (Node : Bare_Subp_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1420;
      Fld : Symbol_Type;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1420 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1420;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3849 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15613
--# expr-start 3848 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:15614
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 3848
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3849
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1420;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1420;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_250;
--# end
--# property-start SubpBodyStub._env_mappings_251 /usr/src/packages/BUILD/ada/ast.py:15617
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_251
  (Node : Bare_Subp_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1421;
      Sym : Symbol_Type;
Cast_Expr : Bare_Subp_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1421 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1421;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3851 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15617
--# expr-start 3850 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3850
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3851
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1421;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1421;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_251;
--# end
--# property-start SubpBodyStub._dest_env_name_252 /usr/src/packages/BUILD/ada/ast.py:15620
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_252
  (Node : Bare_Subp_Body_Stub
  )
   return Symbol_Type
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1422;
      Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
            procedure Finalizer_Scope_1422 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1422;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3853 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym /usr/src/packages/BUILD/ada/ast.py:15620
--# expr-start 3852 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15620
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3852
Sym := String_To_Symbol (Self.Unit.Context, Fld); 
--# expr-done 3853
      Property_Result := Sym;
   --# end
      Finalizer_Scope_1422;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1422;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_252;
--# end
--# property-start SubpBodyStub._fallback_env_253 /usr/src/packages/BUILD/ada/ast.py:15621
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_253
  (Node : Bare_Subp_Body_Stub
  )
   return Lexical_Env
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_253;
--# end
--# property-start SubpBodyStub._env_trans_parent_254 /usr/src/packages/BUILD/ada/ast.py:15612
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_254
  (Node : Bare_Subp_Body_Stub
  )
   return Boolean
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_254;
--# end
--# property-start SubpBodyStub._env_names_255 /usr/src/packages/BUILD/ada/ast.py:15623
pragma Warnings (Off, "is not referenced");
 function Env_Names_255
  (Node : Bare_Subp_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Subp_Body_Stub := Bare_Subp_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1425;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1425 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1425;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3854 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:15623
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 3854
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1425;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1425;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_255;
--# end
   --
   --  Primitives for Bare_Task_Body_Stub
   --
      procedure Task_Body_Stub_Pre_Env_Actions
        (Self            : Bare_Task_Body_Stub;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_262 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_263 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_261 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15661");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_264 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_265 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Task_Body_Stub
        (Self : Bare_Task_Body_Stub
         ; Task_Body_Stub_F_Name : Bare_Defining_Name
         ; Task_Body_Stub_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Task_Body_Stub_F_Name := Task_Body_Stub_F_Name;
            Self.Task_Body_Stub_F_Aspects := Task_Body_Stub_F_Aspects;
      end Initialize_Fields_For_Task_Body_Stub;
   function Task_Body_Stub_F_Name
     (Node : Bare_Task_Body_Stub) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_Stub_F_Name;
   end;
--# property-start TaskBodyStub.defining_names /usr/src/packages/BUILD/ada/ast.py:15658
pragma Warnings (Off, "is not referenced");
 function Task_Body_Stub_P_Defining_Names
  (Node : Bare_Task_Body_Stub
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body_Stub :=
     Internal_Entity_Task_Body_Stub'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1159;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1159 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1159;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3855 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15658
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_Stub_F_Name, Info => Ent.Info);
--# expr-done 3855
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1159;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1159;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_Stub_P_Defining_Names;
--# end
--# property-start TaskBodyStub._env_mappings_261 /usr/src/packages/BUILD/ada/ast.py:15661
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_261
  (Node : Bare_Task_Body_Stub
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1431;
      Sym : Symbol_Type;
Cast_Expr : Bare_Task_Body_Stub;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1431 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1431;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3857 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15661
--# expr-start 3856 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3856
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3857
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1431;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1431;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_261;
--# end
--# property-start TaskBodyStub._dest_env_name_262 /usr/src/packages/BUILD/ada/ast.py:15664
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_262
  (Node : Bare_Task_Body_Stub
  )
   return Symbol_Type
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_1432;
      Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
            procedure Finalizer_Scope_1432 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1432;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3859 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym /usr/src/packages/BUILD/ada/ast.py:15664
--# expr-start 3858 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15664
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 3858
Sym := String_To_Symbol (Self.Unit.Context, Fld); 
--# expr-done 3859
      Property_Result := Sym;
   --# end
      Finalizer_Scope_1432;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1432;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_262;
--# end
--# property-start TaskBodyStub._fallback_env_263 /usr/src/packages/BUILD/ada/ast.py:15665
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_263
  (Node : Bare_Task_Body_Stub
  )
   return Lexical_Env
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_263;
--# end
--# property-start TaskBodyStub._env_trans_parent_264 /usr/src/packages/BUILD/ada/ast.py:15660
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_264
  (Node : Bare_Task_Body_Stub
  )
   return Boolean
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_264;
--# end
--# property-start TaskBodyStub._env_names_265 /usr/src/packages/BUILD/ada/ast.py:15667
pragma Warnings (Off, "is not referenced");
 function Env_Names_265
  (Node : Bare_Task_Body_Stub
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Task_Body_Stub := Bare_Task_Body_Stub (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1435;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1435 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1435;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3860 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:15667
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BodyStub.env_names
Fld := Libadalang.Implementation.Body_Stub_P_Env_Names (Node => Self);
--# end
--# expr-done 3860
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1435;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1435;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_265;
--# end
   --
   --  Primitives for Bare_Entry_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Entry_Body_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Entry_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_241 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:15502" & ")";
         end if;
      return Initial_Env;
   end Bare_Entry_Body_Initial_Env_Getter_Fn;
      procedure Entry_Body_Pre_Env_Actions
        (Self            : Bare_Entry_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_240 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            null;
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Entry_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_242 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:15507");
         end loop;
         Dec_Ref (Mappings);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_243 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Entry_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Body
        (Self : Bare_Entry_Body
         ; Entry_Body_F_Entry_Name : Bare_Defining_Name
         ; Entry_Body_F_Index_Spec : Bare_Entry_Index_Spec
         ; Entry_Body_F_Params : Bare_Entry_Completion_Formal_Params
         ; Entry_Body_F_Barrier : Bare_Expr
         ; Entry_Body_F_Decls : Bare_Declarative_Part
         ; Entry_Body_F_Stmts : Bare_Handled_Stmts
         ; Entry_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Entry_Body_F_Entry_Name := Entry_Body_F_Entry_Name;
            Self.Entry_Body_F_Index_Spec := Entry_Body_F_Index_Spec;
            Self.Entry_Body_F_Params := Entry_Body_F_Params;
            Self.Entry_Body_F_Barrier := Entry_Body_F_Barrier;
            Self.Entry_Body_F_Decls := Entry_Body_F_Decls;
            Self.Entry_Body_F_Stmts := Entry_Body_F_Stmts;
            Self.Entry_Body_F_End_Name := Entry_Body_F_End_Name;
      end Initialize_Fields_For_Entry_Body;
   function Entry_Body_F_Entry_Name
     (Node : Bare_Entry_Body) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Body_F_Entry_Name;
   end;
   function Entry_Body_F_Index_Spec
     (Node : Bare_Entry_Body) return Bare_Entry_Index_Spec
   is
   begin
         return Node.Entry_Body_F_Index_Spec;
   end;
   function Entry_Body_F_Params
     (Node : Bare_Entry_Body) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Entry_Body_F_Params;
   end;
   function Entry_Body_F_Barrier
     (Node : Bare_Entry_Body) return Bare_Expr
   is
   begin
         return Node.Entry_Body_F_Barrier;
   end;
   function Entry_Body_F_Decls
     (Node : Bare_Entry_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Entry_Body_F_Decls;
   end;
   function Entry_Body_F_Stmts
     (Node : Bare_Entry_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Entry_Body_F_Stmts;
   end;
   function Entry_Body_F_End_Name
     (Node : Bare_Entry_Body) return Bare_End_Name
   is
   begin
         return Node.Entry_Body_F_End_Name;
   end;
--# property-start EntryBody.defining_names /usr/src/packages/BUILD/ada/ast.py:15497
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Defining_Names
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1143;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1143 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1143;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3861 '<FieldAccess .entry_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15497
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Body_F_Entry_Name, Info => Ent.Info);
--# expr-done 3861
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1143;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1143;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Defining_Names;
--# end
--# property-start EntryBody.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:15514
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Entry_Point
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Xref_Entry_Point;
--# end
--# property-start EntryBody.xref_equation /usr/src/packages/BUILD/ada/ast.py:15517
pragma Warnings (Off, "is not referenced");
 function Entry_Body_P_Xref_Equation
  (Node : Bare_Entry_Body
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1145;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1145 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1145;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3864 '<Block>' Let_Result None
--# scope-start
--# expr-start 3863 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15518
--# expr-start 3862 '<FieldAccess .barrier>' Fld /usr/src/packages/BUILD/ada/ast.py:15518
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Entry_Body_F_Barrier, Info => Ent.Info);
--# expr-done 3862
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3863
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3864
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1145;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1145;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Body_P_Xref_Equation;
--# end
--# property-start EntryBody._env_do_240 /usr/src/packages/BUILD/ada/ast.py:15500
pragma Warnings (Off, "is not referenced");
 function Env_Do_240
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3865 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:15500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3865
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_240;
--# end
--# property-start EntryBody._initial_env_241 /usr/src/packages/BUILD/ada/ast.py:15503
pragma Warnings (Off, "is not referenced");
 function Initial_Env_241
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1411;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1411 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1411;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3868 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .body_scope(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15503
--# expr-start 3866 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15503
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3866
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3867 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15503
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3867
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3868
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1411;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1411;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_241;
--# end
--# property-start EntryBody._env_mappings_242 /usr/src/packages/BUILD/ada/ast.py:15508
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_242
  (Node : Bare_Entry_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Body :=
     Internal_Entity_Entry_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1412;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Singleton : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1412 is
            begin
                     Dec_Ref (New_Struct);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1412;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3870 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15508
--# expr-start 3869 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:15508
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3869
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3870
                Singleton := Create_Internal_Env_Assoc_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
                Inc_Ref (Singleton.Items (1));
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1412;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1412;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_242;
--# end
--# property-start EntryBody._env_trans_parent_243 /usr/src/packages/BUILD/ada/ast.py:15499
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_243
  (Node : Bare_Entry_Body
  )
   return Boolean
is
   Self : Bare_Entry_Body := Bare_Entry_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_243;
--# end
   --
   --  Primitives for Bare_Package_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Body_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Package_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_182 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:15206" & ")";
         end if;
      return Initial_Env;
   end Bare_Package_Body_Initial_Env_Getter_Fn;
      procedure Package_Body_Pre_Env_Actions
        (Self            : Bare_Package_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_180 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_181 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_184 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_185 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_183 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15211");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_186 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_187 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_189 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_188 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_191 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_190 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_193 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_192 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_195 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_194 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_197 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_196 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_199 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_198 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_201 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_200 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Package_Body_P_Package_Decl_Uses_Clauses_Envs'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Body
        (Self : Bare_Package_Body
         ; Package_Body_F_Package_Name : Bare_Defining_Name
         ; Package_Body_F_Aspects : Bare_Aspect_Spec
         ; Package_Body_F_Decls : Bare_Declarative_Part
         ; Package_Body_F_Stmts : Bare_Handled_Stmts
         ; Package_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Package_Body_F_Package_Name := Package_Body_F_Package_Name;
            Self.Package_Body_F_Aspects := Package_Body_F_Aspects;
            Self.Package_Body_F_Decls := Package_Body_F_Decls;
            Self.Package_Body_F_Stmts := Package_Body_F_Stmts;
            Self.Package_Body_F_End_Name := Package_Body_F_End_Name;
      end Initialize_Fields_For_Package_Body;
   function Package_Body_F_Package_Name
     (Node : Bare_Package_Body) return Bare_Defining_Name
   is
   begin
         return Node.Package_Body_F_Package_Name;
   end;
   function Package_Body_F_Decls
     (Node : Bare_Package_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Package_Body_F_Decls;
   end;
   function Package_Body_F_Stmts
     (Node : Bare_Package_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Package_Body_F_Stmts;
   end;
   function Package_Body_F_End_Name
     (Node : Bare_Package_Body) return Bare_End_Name
   is
   begin
         return Node.Package_Body_F_End_Name;
   end;
--# property-start PackageBody.defining_names /usr/src/packages/BUILD/ada/ast.py:15298
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Names
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1135;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1135 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1135;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3871 '<FieldAccess .package_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15298
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Body_F_Package_Name, Info => Ent.Info);
--# expr-done 3871
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1135;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1135;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Defining_Names;
--# end
--# property-start PackageBody.defining_env /usr/src/packages/BUILD/ada/ast.py:15299
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Defining_Env
  (Node : Bare_Package_Body
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1136;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1136 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1136;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3872 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15299
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 3872
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1136;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1136;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Defining_Env;
--# end
--# property-start PackageBody.declarative_region /usr/src/packages/BUILD/ada/ast.py:15301
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Declarative_Region
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Declarative_Part
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Declarative_Part;
      Fld : Internal_Entity_Declarative_Part;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3873 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:15301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Declarative_Part (Node => Ent.Node.Package_Body_F_Decls, Info => Ent.Info);
--# expr-done 3873
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Declarative_Region;
--# end
--# property-start PackageBody.package_decl_uses_clauses_envs /usr/src/packages/BUILD/ada/ast.py:15304
pragma Warnings (Off, "is not referenced");
 function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1138;
            procedure Finalizer_Scope_2156;
      Block_Var : Internal_Entity_Base_Package_Decl;
Var_Expr : Internal_Entity_Private_Part;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Base_Package_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity_Public_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Private_Part;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1138 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1138;
            procedure Finalizer_Scope_2156 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2156;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3883 '<Block>' Let_Result None
--# scope-start
--# expr-start 3876 '<bind(<DynamicVariable imprecise_fallback>, False, <Cast to BasePackageDecl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15309
Bound_Imprecise_Fallback := False; 
--# expr-start 3875 '<Cast to BasePackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15310
--# expr-start 3874 '<FieldAccess .decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:15310
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3874
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Package_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3875
Dyn_Var_Bind_Result := Cast_Result; 
--# expr-done 3876
Block_Var := Dyn_Var_Bind_Result; 
--# bind pd Block_Var
--# expr-start 3882 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:15313
--# expr-start 3881 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:15313
--# expr-start 3878 '<FieldAccess .use_clauses_envs>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15313
--# expr-start 3877 '<FieldAccess .public_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15313
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Public_Part (Node => Block_Var.Node.Base_Package_Decl_F_Public_Part, Info => Block_Var.Info);
--# expr-done 3877
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_2 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 3878
--# expr-start 3879 '<FieldAccess .private_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15314
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Private_Part (Node => Block_Var.Node.Base_Package_Decl_F_Private_Part, Info => Block_Var.Info);
--# expr-done 3879
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Private_Part then
   --# scope-start
--# expr-start 3880 '<FieldAccess .use_clauses_envs>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15314
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.use_clauses_envs
Fld_4 := Libadalang.Implementation.Declarative_Part_P_Use_Clauses_Envs (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 3880
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2156;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_2, 2 => Result_Var)); 
--# expr-done 3881
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 3882
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 3883
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1138;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1138;
               Finalizer_Scope_2156;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Body_P_Package_Decl_Uses_Clauses_Envs;
--# end
--# property-start PackageBody._env_do_180 /usr/src/packages/BUILD/ada/ast.py:15203
pragma Warnings (Off, "is not referenced");
 function Env_Do_180
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3884 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:15203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3884
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_180;
--# end
--# property-start PackageBody._initial_env_name_181 /usr/src/packages/BUILD/ada/ast.py:15207
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_181
  (Node : Bare_Package_Body
  )
   return Symbol_Type
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3885 '<FieldAccess .body_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15207
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_initial_env_name
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 3885
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_181;
--# end
--# property-start PackageBody._initial_env_182 /usr/src/packages/BUILD/ada/ast.py:15208
pragma Warnings (Off, "is not referenced");
 function Initial_Env_182
  (Node : Bare_Package_Body
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1352;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1352 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1352;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3886 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15208
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3886
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1352;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1352;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_182;
--# end
--# property-start PackageBody._env_mappings_183 /usr/src/packages/BUILD/ada/ast.py:15211
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_183
  (Node : Bare_Package_Body
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1353;
      Sym : Symbol_Type;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1353 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1353;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3888 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15211
--# expr-start 3887 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3887
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3888
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1353;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1353;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_183;
--# end
--# property-start PackageBody._dest_env_name_184 /usr/src/packages/BUILD/ada/ast.py:15214
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_184
  (Node : Bare_Package_Body
  )
   return Symbol_Type
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3889 '<FieldAccess .previous_part_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15214
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 3889
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_184;
--# end
--# property-start PackageBody._fallback_env_185 /usr/src/packages/BUILD/ada/ast.py:15215
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_185
  (Node : Bare_Package_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Body :=
     Internal_Entity_Package_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1355;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1355 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1355;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3893 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15215
--# expr-start 3890 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15216
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3890
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3892 '<FieldAccess .initial_env(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15217
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3891 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15218
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3891
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3892
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3893
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1355;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1355;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_185;
--# end
--# property-start PackageBody._env_trans_parent_186 /usr/src/packages/BUILD/ada/ast.py:15226
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_186
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3894 '<FieldAccess .is_library_item>' Fld /usr/src/packages/BUILD/ada/ast.py:15226
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 3894
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_186;
--# end
--# property-start PackageBody._env_do_187 /usr/src/packages/BUILD/ada/ast.py:15228
pragma Warnings (Off, "is not referenced");
 function Env_Do_187
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1357;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1357 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1357;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3895 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:15228
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3895
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1357;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1357;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_187;
--# end
--# property-start PackageBody._ref_env_nodes_188 /usr/src/packages/BUILD/ada/ast.py:15231
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_188
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1358;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1358 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1358;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3896 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3896
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1358;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1358;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_188;
--# end
--# property-start PackageBody._ref_cond_189 /usr/src/packages/BUILD/ada/ast.py:15233
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_189
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3898 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15233
--# expr-start 3897 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15233
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3897
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3898
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_189;
--# end
--# property-start PackageBody._ref_env_nodes_190 /usr/src/packages/BUILD/ada/ast.py:15237
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_190
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1360;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1360 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1360;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3899 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15237
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3899
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1360;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1360;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_190;
--# end
--# property-start PackageBody._ref_cond_191 /usr/src/packages/BUILD/ada/ast.py:15239
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_191
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3901 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15239
--# expr-start 3900 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15239
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3900
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3901
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_191;
--# end
--# property-start PackageBody._ref_env_nodes_192 /usr/src/packages/BUILD/ada/ast.py:15243
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_192
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1362;
            procedure Finalizer_Scope_2157;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1362 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1362;
            procedure Finalizer_Scope_2157 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2157;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3902 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15243
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3902
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2157;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1362;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1362;
               Finalizer_Scope_2157;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_192;
--# end
--# property-start PackageBody._ref_cond_193 /usr/src/packages/BUILD/ada/ast.py:15245
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_193
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3903 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15245
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3903
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_193;
--# end
--# property-start PackageBody._ref_env_nodes_194 /usr/src/packages/BUILD/ada/ast.py:15253
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_194
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1364;
            procedure Finalizer_Scope_2158;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1364 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1364;
            procedure Finalizer_Scope_2158 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2158;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3904 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15253
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3904
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2158;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1364;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1364;
               Finalizer_Scope_2158;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_194;
--# end
--# property-start PackageBody._ref_cond_195 /usr/src/packages/BUILD/ada/ast.py:15255
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_195
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3905 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15255
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3905
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_195;
--# end
--# property-start PackageBody._ref_env_nodes_196 /usr/src/packages/BUILD/ada/ast.py:15266
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_196
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1366;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1366 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1366;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3906 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15266
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3906
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1366;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1366;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_196;
--# end
--# property-start PackageBody._ref_cond_197 /usr/src/packages/BUILD/ada/ast.py:15268
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_197
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3907 '<FieldAccess .is_subunit>' Fld /usr/src/packages/BUILD/ada/ast.py:15268
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3907
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_197;
--# end
--# property-start PackageBody._ref_env_nodes_198 /usr/src/packages/BUILD/ada/ast.py:15280
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_198
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1368;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1368 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1368;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3908 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15280
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3908
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1368;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1368;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_198;
--# end
--# property-start PackageBody._ref_cond_199 /usr/src/packages/BUILD/ada/ast.py:15282
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_199
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3910 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:15282
--# expr-start 3909 '<FieldAccess .is_compilation_unit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:15282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3909
Not_Val := not (Fld); 
--# expr-done 3910
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_199;
--# end
--# property-start PackageBody._ref_env_nodes_200 /usr/src/packages/BUILD/ada/ast.py:15287
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_200
  (Node : Bare_Package_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1370;
      Cast_Expr : Bare_Package_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1370 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1370;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3911 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15287
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3911
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1370;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1370;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_200;
--# end
--# property-start PackageBody._ref_cond_201 /usr/src/packages/BUILD/ada/ast.py:15289
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_201
  (Node : Bare_Package_Body
  )
   return Boolean
is
   Self : Bare_Package_Body := Bare_Package_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3913 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:15289
--# expr-start 3912 '<FieldAccess .is_compilation_unit_root>' Fld /usr/src/packages/BUILD/ada/ast.py:15289
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_compilation_unit_root
Fld := Libadalang.Implementation.Basic_Decl_P_Is_Compilation_Unit_Root (Node => Self);
--# end
--# expr-done 3912
Not_Val := not (Fld); 
--# expr-done 3913
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_201;
--# end
function Package_Body_P_Package_Decl_Uses_Clauses_Envs
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Package_Body_P_Package_Decl_Uses_Clauses_Envs (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Protected_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Protected_Body_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Protected_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_223 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:15414" & ")";
         end if;
      return Initial_Env;
   end Bare_Protected_Body_Initial_Env_Getter_Fn;
      procedure Protected_Body_Pre_Env_Actions
        (Self            : Bare_Protected_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_221 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_222 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Protected_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_225 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_226 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_224 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15419");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_227 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Protected_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_228 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_230 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_229 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_232 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_231 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_234 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_233 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_236 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_235 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_237 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Body_Decl_Scope'Access,
               Transitive,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_239 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_238 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Protected_Body
        (Self : Bare_Protected_Body
         ; Protected_Body_F_Name : Bare_Defining_Name
         ; Protected_Body_F_Aspects : Bare_Aspect_Spec
         ; Protected_Body_F_Decls : Bare_Declarative_Part
         ; Protected_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Body_F_Name := Protected_Body_F_Name;
            Self.Protected_Body_F_Aspects := Protected_Body_F_Aspects;
            Self.Protected_Body_F_Decls := Protected_Body_F_Decls;
            Self.Protected_Body_F_End_Name := Protected_Body_F_End_Name;
      end Initialize_Fields_For_Protected_Body;
   function Protected_Body_F_Name
     (Node : Bare_Protected_Body) return Bare_Defining_Name
   is
   begin
         return Node.Protected_Body_F_Name;
   end;
   function Protected_Body_F_Decls
     (Node : Bare_Protected_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Protected_Body_F_Decls;
   end;
   function Protected_Body_F_End_Name
     (Node : Bare_Protected_Body) return Bare_End_Name
   is
   begin
         return Node.Protected_Body_F_End_Name;
   end;
--# property-start ProtectedBody.defining_names /usr/src/packages/BUILD/ada/ast.py:15479
pragma Warnings (Off, "is not referenced");
 function Protected_Body_P_Defining_Names
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body :=
     Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1142;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1142 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1142;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3914 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15479
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Protected_Body_F_Name, Info => Ent.Info);
--# expr-done 3914
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1142;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1142;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Body_P_Defining_Names;
--# end
--# property-start ProtectedBody._env_do_221 /usr/src/packages/BUILD/ada/ast.py:15412
pragma Warnings (Off, "is not referenced");
 function Env_Do_221
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3915 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:15412
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3915
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_221;
--# end
--# property-start ProtectedBody._initial_env_name_222 /usr/src/packages/BUILD/ada/ast.py:15415
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_222
  (Node : Bare_Protected_Body
  )
   return Symbol_Type
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3916 '<FieldAccess .body_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15415
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_initial_env_name
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 3916
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_222;
--# end
--# property-start ProtectedBody._initial_env_223 /usr/src/packages/BUILD/ada/ast.py:15416
pragma Warnings (Off, "is not referenced");
 function Initial_Env_223
  (Node : Bare_Protected_Body
  )
   return Lexical_Env
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1393;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1393 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1393;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3917 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15416
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3917
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1393;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1393;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_223;
--# end
--# property-start ProtectedBody._env_mappings_224 /usr/src/packages/BUILD/ada/ast.py:15419
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_224
  (Node : Bare_Protected_Body
  )
   return Internal_Env_Assoc
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1394;
      Sym : Symbol_Type;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1394 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1394;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3919 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15419
--# expr-start 3918 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3918
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3919
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1394;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1394;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_224;
--# end
--# property-start ProtectedBody._dest_env_name_225 /usr/src/packages/BUILD/ada/ast.py:15422
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_225
  (Node : Bare_Protected_Body
  )
   return Symbol_Type
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3920 '<FieldAccess .previous_part_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15422
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 3920
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_225;
--# end
--# property-start ProtectedBody._fallback_env_226 /usr/src/packages/BUILD/ada/ast.py:15423
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_226
  (Node : Bare_Protected_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Protected_Body :=
     Internal_Entity_Protected_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1396;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1396 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1396;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3924 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15423
--# expr-start 3921 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15424
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3921
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3923 '<FieldAccess .initial_env(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15425
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3922 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15426
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3922
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3923
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3924
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1396;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1396;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_226;
--# end
--# property-start ProtectedBody._env_trans_parent_227 /usr/src/packages/BUILD/ada/ast.py:15411
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_227
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_227;
--# end
--# property-start ProtectedBody._env_do_228 /usr/src/packages/BUILD/ada/ast.py:15434
pragma Warnings (Off, "is not referenced");
 function Env_Do_228
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1398;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1398 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1398;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3925 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:15434
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3925
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1398;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1398;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_228;
--# end
--# property-start ProtectedBody._ref_env_nodes_229 /usr/src/packages/BUILD/ada/ast.py:15437
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_229
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1399;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1399 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1399;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3926 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15437
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3926
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1399;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1399;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_229;
--# end
--# property-start ProtectedBody._ref_cond_230 /usr/src/packages/BUILD/ada/ast.py:15439
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_230
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3928 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15439
--# expr-start 3927 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15439
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3927
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3928
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_230;
--# end
--# property-start ProtectedBody._ref_env_nodes_231 /usr/src/packages/BUILD/ada/ast.py:15442
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_231
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1401;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1401 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1401;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3929 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15442
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3929
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1401;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1401;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_231;
--# end
--# property-start ProtectedBody._ref_cond_232 /usr/src/packages/BUILD/ada/ast.py:15444
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_232
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3931 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15444
--# expr-start 3930 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15444
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3930
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3931
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_232;
--# end
--# property-start ProtectedBody._ref_env_nodes_233 /usr/src/packages/BUILD/ada/ast.py:15447
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_233
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1403;
            procedure Finalizer_Scope_2159;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1403 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1403;
            procedure Finalizer_Scope_2159 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2159;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3932 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15447
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3932
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2159;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1403;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1403;
               Finalizer_Scope_2159;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_233;
--# end
--# property-start ProtectedBody._ref_cond_234 /usr/src/packages/BUILD/ada/ast.py:15449
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_234
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3933 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15449
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3933
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_234;
--# end
--# property-start ProtectedBody._ref_env_nodes_235 /usr/src/packages/BUILD/ada/ast.py:15457
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_235
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1405;
            procedure Finalizer_Scope_2160;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1405 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1405;
            procedure Finalizer_Scope_2160 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2160;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3934 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15457
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3934
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2160;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1405;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1405;
               Finalizer_Scope_2160;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_235;
--# end
--# property-start ProtectedBody._ref_cond_236 /usr/src/packages/BUILD/ada/ast.py:15459
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_236
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3935 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15459
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3935
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_236;
--# end
--# property-start ProtectedBody._ref_env_nodes_237 /usr/src/packages/BUILD/ada/ast.py:15463
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_237
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1407;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1407 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1407;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3936 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15463
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3936
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1407;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1407;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_237;
--# end
--# property-start ProtectedBody._ref_env_nodes_238 /usr/src/packages/BUILD/ada/ast.py:15468
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_238
  (Node : Bare_Protected_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1408;
      Cast_Expr : Bare_Protected_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1408 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1408;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3937 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15468
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3937
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1408;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1408;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_238;
--# end
--# property-start ProtectedBody._ref_cond_239 /usr/src/packages/BUILD/ada/ast.py:15470
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_239
  (Node : Bare_Protected_Body
  )
   return Boolean
is
   Self : Bare_Protected_Body := Bare_Protected_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3938 '<FieldAccess .is_subunit>' Fld /usr/src/packages/BUILD/ada/ast.py:15470
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3938
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_239;
--# end
   --
   --  Primitives for Bare_Task_Body
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Task_Body_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Task_Body := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_204 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:15333" & ")";
         end if;
      return Initial_Env;
   end Bare_Task_Body_Initial_Env_Getter_Fn;
      procedure Task_Body_Pre_Env_Actions
        (Self            : Bare_Task_Body;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_202 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_203 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Task_Body_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_206 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_207 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_205 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:15338");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_208 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Task_Body_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_209 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_211 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_210 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_213 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_212 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_215 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_214 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Nested_Generic_Formal_Part'Access,
               Prioritary,
               All_Cats,
               True);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_217 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_216 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Ada_Node_P_Use_Clauses_In_Generic_Formal_Part'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_218 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Task_Body_P_Task_Type_Decl_Scope'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      if Libadalang.Implementation.Ref_Cond_220 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_219 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Body_Node_P_Subunit_Decl_Env'Access,
               Prioritary,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Task_Body
        (Self : Bare_Task_Body
         ; Task_Body_F_Name : Bare_Defining_Name
         ; Task_Body_F_Aspects : Bare_Aspect_Spec
         ; Task_Body_F_Decls : Bare_Declarative_Part
         ; Task_Body_F_Stmts : Bare_Handled_Stmts
         ; Task_Body_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Body_F_Name := Task_Body_F_Name;
            Self.Task_Body_F_Aspects := Task_Body_F_Aspects;
            Self.Task_Body_F_Decls := Task_Body_F_Decls;
            Self.Task_Body_F_Stmts := Task_Body_F_Stmts;
            Self.Task_Body_F_End_Name := Task_Body_F_End_Name;
      end Initialize_Fields_For_Task_Body;
   function Task_Body_F_Name
     (Node : Bare_Task_Body) return Bare_Defining_Name
   is
   begin
         return Node.Task_Body_F_Name;
   end;
   function Task_Body_F_Decls
     (Node : Bare_Task_Body) return Bare_Declarative_Part
   is
   begin
         return Node.Task_Body_F_Decls;
   end;
   function Task_Body_F_Stmts
     (Node : Bare_Task_Body) return Bare_Handled_Stmts
   is
   begin
         return Node.Task_Body_F_Stmts;
   end;
   function Task_Body_F_End_Name
     (Node : Bare_Task_Body) return Bare_End_Name
   is
   begin
         return Node.Task_Body_F_End_Name;
   end;
--# property-start TaskBody.defining_names /usr/src/packages/BUILD/ada/ast.py:15328
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Defining_Names
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1139;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1139 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1139;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3939 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15328
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Task_Body_F_Name, Info => Ent.Info);
--# expr-done 3939
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1139;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1139;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Defining_Names;
--# end
--# property-start TaskBody.task_type_decl_scope /usr/src/packages/BUILD/ada/ast.py:15395
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type_Decl_Scope
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1140;
      Fld : Internal_Entity_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1140 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1140;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3941 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15395
--# expr-start 3940 '<FieldAccess .task_type>' Fld /usr/src/packages/BUILD/ada/ast.py:15395
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 3940
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 3941
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1140;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1140;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Task_Type_Decl_Scope;
--# end
--# property-start TaskBody.task_type /usr/src/packages/BUILD/ada/ast.py:15398
pragma Warnings (Off, "is not referenced");
 function Task_Body_P_Task_Type
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Task_Type_Decl
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Task_Type_Decl;
      Match_0 : Internal_Entity_Task_Type_Decl;
Match_1 : Internal_Entity_Single_Task_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Bound_Imprecise_Fallback : Boolean;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Single_Task_Type_Decl;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Task_Type_Decl;
Let_Result : Internal_Entity_Task_Type_Decl;
Scope_Result : Internal_Entity_Task_Type_Decl;
Cast_Result_1 : Internal_Entity_Single_Task_Decl;
Cast_Result_2 : Internal_Entity_Task_Type_Decl;
Let_Result_1 : Internal_Entity_Task_Type_Decl;
Scope_Result_1 : Internal_Entity_Task_Type_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Task_Type_Decl;
Scope_Result_2 : Internal_Entity_Task_Type_Decl;
Match_Result : Internal_Entity_Task_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Task_Type_Decl;
Let_Result_3 : Internal_Entity_Task_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3946 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 3945 '<bind(<DynamicVariable imprecise_fallback>, False, <Match>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15399
Bound_Imprecise_Fallback := False; 
--# expr-start 3944 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:15399
--# expr-start 3942 '<FieldAccess .decl_part>' Fld /usr/src/packages/BUILD/ada/ast.py:15399
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Ent.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 3942
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Task_Type_Decl .. Ada_Single_Task_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Task_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Single_Task_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 3943 '<FieldAccess .task_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15401
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Single_Task_Type_Decl (Node => Match_1.Node.Single_Task_Decl_F_Task_Type, Info => Match_1.Info);
--# expr-done 3943
      Cast_Result_2 := Create_Internal_Entity_Task_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Protected_Type_Decl | Ada_Type_Decl .. Ada_Single_Protected_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Should not happen"; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 3944
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 3945
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 3946
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Body_P_Task_Type;
--# end
--# property-start TaskBody._env_do_202 /usr/src/packages/BUILD/ada/ast.py:15331
pragma Warnings (Off, "is not referenced");
 function Env_Do_202
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3947 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:15331
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 3947
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_202;
--# end
--# property-start TaskBody._initial_env_name_203 /usr/src/packages/BUILD/ada/ast.py:15334
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_203
  (Node : Bare_Task_Body
  )
   return Symbol_Type
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3948 '<FieldAccess .body_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15334
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_initial_env_name
Fld := Libadalang.Implementation.Body_Node_P_Body_Initial_Env_Name (Node => Self);
--# end
--# expr-done 3948
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_203;
--# end
--# property-start TaskBody._initial_env_204 /usr/src/packages/BUILD/ada/ast.py:15335
pragma Warnings (Off, "is not referenced");
 function Initial_Env_204
  (Node : Bare_Task_Body
  )
   return Lexical_Env
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1374;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1374 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1374;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3949 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15335
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3949
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1374;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1374;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_204;
--# end
--# property-start TaskBody._env_mappings_205 /usr/src/packages/BUILD/ada/ast.py:15338
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_205
  (Node : Bare_Task_Body
  )
   return Internal_Env_Assoc
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1375;
      Sym : Symbol_Type;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1375 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1375;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3951 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15338
--# expr-start 3950 '<Symbol __nextpart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 3950
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3951
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1375;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1375;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_205;
--# end
--# property-start TaskBody._dest_env_name_206 /usr/src/packages/BUILD/ada/ast.py:15341
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_206
  (Node : Bare_Task_Body
  )
   return Symbol_Type
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3952 '<FieldAccess .previous_part_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:15341
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Body.previous_part_env_name'
Fld := Libadalang.Implementation.Dispatcher_Body_Node_P_Previous_Part_Env_Name (Node => Self);
--# end
--# expr-done 3952
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_206;
--# end
--# property-start TaskBody._fallback_env_207 /usr/src/packages/BUILD/ada/ast.py:15342
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_207
  (Node : Bare_Task_Body
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Body :=
     Internal_Entity_Task_Body'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1377;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1377 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_1377;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3956 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <FieldAccess .initial_env(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:15342
--# expr-start 3953 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:15343
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 3953
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 3955 '<FieldAccess .initial_env(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15344
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3954 '<FieldAccess .body_scope(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15345
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.body_scope
Fld_1 := Libadalang.Implementation.Body_Node_P_Body_Scope (Node => Ent.Node, Follow_Private => False, Force_Decl => True, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 3954
--# property-call-start AdaNode.initial_env
Fld_2 := Libadalang.Implementation.Ada_Node_P_Initial_Env (Node => Self, Scope => Fld_1, Env => Bound_Env);
--# end
--# expr-done 3955
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 3956
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1377;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1377;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_207;
--# end
--# property-start TaskBody._env_trans_parent_208 /usr/src/packages/BUILD/ada/ast.py:15330
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_208
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_208;
--# end
--# property-start TaskBody._env_do_209 /usr/src/packages/BUILD/ada/ast.py:15353
pragma Warnings (Off, "is not referenced");
 function Env_Do_209
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1379;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1379 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1379;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3957 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:15353
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 3957
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1379;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1379;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_209;
--# end
--# property-start TaskBody._ref_env_nodes_210 /usr/src/packages/BUILD/ada/ast.py:15356
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_210
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1380;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1380 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1380;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3958 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15356
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 3958
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1380;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1380;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_210;
--# end
--# property-start TaskBody._ref_cond_211 /usr/src/packages/BUILD/ada/ast.py:15358
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_211
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3960 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15358
--# expr-start 3959 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15358
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3959
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3960
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_211;
--# end
--# property-start TaskBody._ref_env_nodes_212 /usr/src/packages/BUILD/ada/ast.py:15362
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_212
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1382;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1382 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1382;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3961 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:15362
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 3961
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1382;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1382;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_212;
--# end
--# property-start TaskBody._ref_cond_213 /usr/src/packages/BUILD/ada/ast.py:15364
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_213
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3963 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:15364
--# expr-start 3962 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15364
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 3962
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 3963
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_213;
--# end
--# property-start TaskBody._ref_env_nodes_214 /usr/src/packages/BUILD/ada/ast.py:15368
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_214
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1384;
            procedure Finalizer_Scope_2164;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1384 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1384;
            procedure Finalizer_Scope_2164 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2164;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3964 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15368
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3964
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2164;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1384;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1384;
               Finalizer_Scope_2164;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_214;
--# end
--# property-start TaskBody._ref_cond_215 /usr/src/packages/BUILD/ada/ast.py:15370
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_215
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3965 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15370
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3965
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_215;
--# end
--# property-start TaskBody._ref_env_nodes_216 /usr/src/packages/BUILD/ada/ast.py:15378
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_216
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1386;
            procedure Finalizer_Scope_2165;
      Var_Expr : Bare_Ada_Node;
Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
Scope_Result : Bare_Ada_Node_Array_Access;
Result_Var : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1386 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1386;
            procedure Finalizer_Scope_2165 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2165;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 3966 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15378
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3966
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2165;
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1386;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1386;
               Finalizer_Scope_2165;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_216;
--# end
--# property-start TaskBody._ref_cond_217 /usr/src/packages/BUILD/ada/ast.py:15380
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_217
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3967 '<FieldAccess .should_ref_generic_formals>' Fld /usr/src/packages/BUILD/ada/ast.py:15380
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.should_ref_generic_formals
Fld := Libadalang.Implementation.Basic_Decl_P_Should_Ref_Generic_Formals (Node => Self);
--# end
--# expr-done 3967
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_217;
--# end
--# property-start TaskBody._ref_env_nodes_218 /usr/src/packages/BUILD/ada/ast.py:15384
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_218
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1388;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1388 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1388;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3968 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15384
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3968
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1388;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1388;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_218;
--# end
--# property-start TaskBody._ref_env_nodes_219 /usr/src/packages/BUILD/ada/ast.py:15389
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_219
  (Node : Bare_Task_Body
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1389;
      Cast_Expr : Bare_Task_Body;
Cast_Result : Bare_Ada_Node;
Singleton : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1389 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1389;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3969 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15389
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# expr-done 3969
                Singleton := Create_Bare_Ada_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Cast_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1389;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1389;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_219;
--# end
--# property-start TaskBody._ref_cond_220 /usr/src/packages/BUILD/ada/ast.py:15391
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_220
  (Node : Bare_Task_Body
  )
   return Boolean
is
   Self : Bare_Task_Body := Bare_Task_Body (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3970 '<FieldAccess .is_subunit>' Fld /usr/src/packages/BUILD/ada/ast.py:15391
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.is_subunit
Fld := Libadalang.Implementation.Body_Node_P_Is_Subunit (Node => Self);
--# end
--# expr-done 3970
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_220;
--# end
function Task_Body_P_Task_Type_Decl_Scope
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Task_Body_P_Task_Type_Decl_Scope (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Entry_Index_Spec
   --
      procedure Entry_Index_Spec_Pre_Env_Actions
        (Self            : Bare_Entry_Index_Spec;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_244 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:15530");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Entry_Index_Spec
        (Self : Bare_Entry_Index_Spec
         ; Entry_Index_Spec_F_Id : Bare_Defining_Name
         ; Entry_Index_Spec_F_Subtype : Bare_Ada_Node
        ) is
      begin
            Self.Entry_Index_Spec_F_Id := Entry_Index_Spec_F_Id;
            Self.Entry_Index_Spec_F_Subtype := Entry_Index_Spec_F_Subtype;
      end Initialize_Fields_For_Entry_Index_Spec;
   function Entry_Index_Spec_F_Id
     (Node : Bare_Entry_Index_Spec) return Bare_Defining_Name
   is
   begin
         return Node.Entry_Index_Spec_F_Id;
   end;
   function Entry_Index_Spec_F_Subtype
     (Node : Bare_Entry_Index_Spec) return Bare_Ada_Node
   is
   begin
         return Node.Entry_Index_Spec_F_Subtype;
   end;
--# property-start EntryIndexSpec.defining_names /usr/src/packages/BUILD/ada/ast.py:15532
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Names
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1146;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1146 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1146;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3971 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:15532
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Entry_Index_Spec_F_Id, Info => Ent.Info);
--# expr-done 3971
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1146;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1146;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Defining_Names;
--# end
--# property-start EntryIndexSpec.defining_env /usr/src/packages/BUILD/ada/ast.py:15533
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Defining_Env
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1147;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1147 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1147;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3973 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15533
--# expr-start 3972 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:15533
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 3972
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3973
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1147;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1147;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Defining_Env;
--# end
--# property-start EntryIndexSpec.expr_type /usr/src/packages/BUILD/ada/ast.py:15534
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Expr_Type
  (Node : Bare_Entry_Index_Spec
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Base_Type_Decl;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3978 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:15534
--# expr-start 3974 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:15534
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 3974
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind subt Match_0
--# expr-start 3975 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15535
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 3975
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abort_Absent .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind e Match_1
--# expr-start 3977 '<FieldAccess .expression_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15536
--# expr-start 3976 '<Cast to Expr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15536
Cast_Expr := Match_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 3976
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 3977
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 3978
      Property_Result := Match_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Expr_Type;
--# end
--# property-start EntryIndexSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:15540
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Equation
  (Node : Bare_Entry_Index_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1149;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1149 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1149;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3981 '<Block>' Let_Result None
--# scope-start
--# expr-start 3980 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15541
--# expr-start 3979 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:15541
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Entry_Index_Spec_F_Subtype, Info => Ent.Info);
--# expr-done 3979
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 3980
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 3981
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1149;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1149;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Xref_Equation;
--# end
--# property-start EntryIndexSpec.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:15543
pragma Warnings (Off, "is not referenced");
 function Entry_Index_Spec_P_Xref_Entry_Point
  (Node : Bare_Entry_Index_Spec
  )
   return Boolean
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Entry_Index_Spec_P_Xref_Entry_Point;
--# end
--# property-start EntryIndexSpec._env_mappings_244 /usr/src/packages/BUILD/ada/ast.py:15530
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_244
  (Node : Bare_Entry_Index_Spec
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Entry_Index_Spec := Bare_Entry_Index_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Entry_Index_Spec :=
     Internal_Entity_Entry_Index_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1414;
      Fld : Symbol_Type;
Cast_Expr : Bare_Entry_Index_Spec;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1414 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1414;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3983 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:15530
--# expr-start 3982 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:15530
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 3982
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 3983
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1414;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1414;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_244;
--# end
   --
   --  Primitives for Bare_Error_Decl
   --
--# property-start ErrorDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:2521
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Defining_Names
  (Node : Bare_Error_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Error_Decl := Bare_Error_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Defining_Name_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Error_Decl_P_Defining_Names;
--# end
--# property-start ErrorDecl.child_decl_initial_env_name /usr/src/packages/BUILD/ada/ast.py:2524
pragma Warnings (Off, "is not referenced");
 function Error_Decl_P_Child_Decl_Initial_Env_Name
  (Node : Bare_Error_Decl
      ; Private_Part : Boolean
         := False
  )
   return Symbol_Type
is
   Self : Bare_Error_Decl := Bare_Error_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind private_part Private_Part
   Property_Result : Symbol_Type;
      Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3984 '<Block>' Let_Result None
--# scope-start
Let_Result := null; 
--# end
--# expr-done 3984
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Error_Decl_P_Child_Decl_Initial_Env_Name;
--# end
   --
   --  Primitives for Bare_Exception_Decl
   --
      procedure Exception_Decl_Pre_Env_Actions
        (Self            : Bare_Exception_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_66 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7711");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Decl
        (Self : Bare_Exception_Decl
         ; Exception_Decl_F_Ids : Bare_Defining_Name_List
         ; Exception_Decl_F_Renames : Bare_Renaming_Clause
         ; Exception_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Exception_Decl_F_Ids := Exception_Decl_F_Ids;
            Self.Exception_Decl_F_Renames := Exception_Decl_F_Renames;
            Self.Exception_Decl_F_Aspects := Exception_Decl_F_Aspects;
      end Initialize_Fields_For_Exception_Decl;
   function Exception_Decl_F_Ids
     (Node : Bare_Exception_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Exception_Decl_F_Ids;
   end;
   function Exception_Decl_F_Renames
     (Node : Bare_Exception_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Exception_Decl_F_Renames;
   end;
--# property-start ExceptionDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7702
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Defining_Names
  (Node : Bare_Exception_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Decl :=
     Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_686;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_142 : Internal_Entity_Defining_Name;
Bare_Item_142 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_686 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_686;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3986 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7702
--# expr-start 3985 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7702
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Exception_Decl_F_Ids, Info => Ent.Info);
--# expr-done 3985
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_142 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_142 := Untyped_Item_142; 
Is_Null := Bare_Item_142 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_142); 
   If_Result := As_Entity;
end if;
                  Item_142 := If_Result; 
   --# scope-start
               --# bind id Item_142
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_142;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 3986
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_686;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_686;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Defining_Names;
--# end
--# property-start ExceptionDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:7705
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Next_Part_For_Decl
  (Node : Bare_Exception_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3987 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 3987
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Next_Part_For_Decl;
--# end
--# property-start ExceptionDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7713
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Entry_Point
  (Node : Bare_Exception_Decl
  )
   return Boolean
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Xref_Entry_Point;
--# end
--# property-start ExceptionDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:7716
pragma Warnings (Off, "is not referenced");
 function Exception_Decl_P_Xref_Equation
  (Node : Bare_Exception_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Decl :=
     Internal_Entity_Exception_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_689;
            procedure Finalizer_Scope_2169;
      Var_Expr : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_689 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_689;
            procedure Finalizer_Scope_2169 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2169;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 3991 '<Block>' Let_Result None
--# scope-start
--# expr-start 3988 '<FieldAccess .renames>' Fld /usr/src/packages/BUILD/ada/ast.py:7717
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Exception_Decl_F_Renames, Info => Ent.Info);
--# expr-done 3988
Var_Expr := Fld;
if Var_Expr /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 3990 '<FieldAccess .xref_no_overloading>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7718
--# expr-start 3989 '<FieldAccess .renamed_object>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7718
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr.Info);
--# expr-done 3989
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 3990
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2169;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7719") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 3991
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_689;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_689;
               Finalizer_Scope_2169;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Decl_P_Xref_Equation;
--# end
--# property-start ExceptionDecl._env_mappings_66 /usr/src/packages/BUILD/ada/ast.py:7711
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_66
  (Node : Bare_Exception_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Decl := Bare_Exception_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1236;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Exception_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1236 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1236;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3993 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7711
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3992 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7711
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Exception_Decl_F_Ids;
--# expr-done 3992
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 3993
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1236;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1236;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_66;
--# end
   --
   --  Primitives for Bare_Exception_Handler
   --
      procedure Exception_Handler_Pre_Env_Actions
        (Self            : Bare_Exception_Handler;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_171 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_172 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:14573");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Exception_Handler
        (Self : Bare_Exception_Handler
         ; Exception_Handler_F_Exception_Name : Bare_Defining_Name
         ; Exception_Handler_F_Handled_Exceptions : Bare_Alternatives_List
         ; Exception_Handler_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Exception_Handler_F_Exception_Name := Exception_Handler_F_Exception_Name;
            Self.Exception_Handler_F_Handled_Exceptions := Exception_Handler_F_Handled_Exceptions;
            Self.Exception_Handler_F_Stmts := Exception_Handler_F_Stmts;
      end Initialize_Fields_For_Exception_Handler;
   function Exception_Handler_F_Exception_Name
     (Node : Bare_Exception_Handler) return Bare_Defining_Name
   is
   begin
         return Node.Exception_Handler_F_Exception_Name;
   end;
   function Exception_Handler_F_Handled_Exceptions
     (Node : Bare_Exception_Handler) return Bare_Alternatives_List
   is
   begin
         return Node.Exception_Handler_F_Handled_Exceptions;
   end;
   function Exception_Handler_F_Stmts
     (Node : Bare_Exception_Handler) return Bare_Stmt_List
   is
   begin
         return Node.Exception_Handler_F_Stmts;
   end;
--# property-start ExceptionHandler.defining_names /usr/src/packages/BUILD/ada/ast.py:14583
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Defining_Names
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1097;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1097 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1097;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 3994 '<FieldAccess .exception_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14583
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 3994
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1097;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1097;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Defining_Names;
--# end
--# property-start ExceptionHandler.expr_type /usr/src/packages/BUILD/ada/ast.py:14586
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Expr_Type
  (Node : Bare_Exception_Handler
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1098;
            procedure Finalizer_Scope_2170;
      Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1098 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1098;
            procedure Finalizer_Scope_2170 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2170;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4003 '<Block>' Let_Result None
--# scope-start
--# expr-start 4002 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14588
--# expr-start 4001 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Exception_Occurrence)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:14588
--# expr-start 3998 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14588
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 3997 '<ArrayLiteral>' Array_Lit None
--# expr-start 3995 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 3995
--# expr-start 3996 '<Symbol Exceptions>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exceptions); 
--# expr-done 3996
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 3997
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 3998
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 3999 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14588
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 3999
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2170;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 4000 '<Symbol Exception_Occurrence>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exception_Occurrence); 
--# expr-done 4000
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 4001
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4002
Let_Result := Cast_Result; 
--# end
--# expr-done 4003
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1098;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1098;
               Finalizer_Scope_2170;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Expr_Type;
--# end
--# property-start ExceptionHandler.xref_equation /usr/src/packages/BUILD/ada/ast.py:14594
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Equation
  (Node : Bare_Exception_Handler
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1099;
            procedure Finalizer_Scope_2171;
      Fld : Bare_Alternatives_List;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_1 : Logic_Equation;
Item_143 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_1099 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_1099;
            procedure Finalizer_Scope_2171 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2171;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4008 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:14595
--# expr-start 4007 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14595
--# expr-start 4004 '<FieldAccess .handled_exceptions>' Fld /usr/src/packages/BUILD/ada/ast.py:14595
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Exception_Handler_F_Handled_Exceptions;
--# expr-done 4004
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld;
      begin
         for Untyped_Item_143 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_143 := Untyped_Item_143; 
   --# scope-start
               --# bind he Item_143
      --# expr-start 4006 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14595
--# expr-start 4005 '<as_entity(<Var he>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14595
Is_Null := Item_143 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Item_143); 
   If_Result := As_Entity;
end if;
--# expr-done 4005
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => If_Result.Node, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4006
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2171;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4007
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14595") else null)); 
--# expr-done 4008
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1099;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1099;
               Finalizer_Scope_2171;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Xref_Equation;
--# end
--# property-start ExceptionHandler.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:14598
pragma Warnings (Off, "is not referenced");
 function Exception_Handler_P_Xref_Entry_Point
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exception_Handler_P_Xref_Entry_Point;
--# end
--# property-start ExceptionHandler._env_trans_parent_171 /usr/src/packages/BUILD/ada/ast.py:14571
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_171
  (Node : Bare_Exception_Handler
  )
   return Boolean
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_171;
--# end
--# property-start ExceptionHandler._env_mappings_172 /usr/src/packages/BUILD/ada/ast.py:14574
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_172
  (Node : Bare_Exception_Handler
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Exception_Handler := Bare_Exception_Handler (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exception_Handler :=
     Internal_Entity_Exception_Handler'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1342;
            procedure Finalizer_Scope_2172;
            procedure Finalizer_Scope_2173;
      Var_Expr : Internal_Entity_Defining_Name;
Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Symbol_Type;
Fld_2 : Lexical_Env := Empty_Env;
Cast_Expr : Bare_Exception_Handler;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
Map_Result : Internal_Env_Assoc_Array_Access;
Scope_Result : Internal_Env_Assoc_Array_Access;
Result_Var : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1342 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1342;
            procedure Finalizer_Scope_2172 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2172;
            procedure Finalizer_Scope_2173 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_2173;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 4009 '<FieldAccess .exception_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14574
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Exception_Handler_F_Exception_Name, Info => Ent.Info);
--# expr-done 4009
Var_Expr := Fld;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 4013 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14574
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
declare
   Map_Result_Vec : Internal_Env_Assoc_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Singleton;
      begin
         for Item_144 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_144
      --# expr-start 4012 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14576
--# expr-start 4010 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14578
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Self);
--# expr-done 4010
--# expr-start 4011 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14576
      if Item_144.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_144.Node);
--# end
--# expr-done 4011
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Fld_2);
New_Struct := (Dest_Env => Fld_2, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4012
         declare
            Item_To_Append : constant Internal_Env_Assoc :=
               New_Struct;
         begin
               Inc_Ref (Item_To_Append);
            Internal_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2173;
         end loop;
      end;
      Map_Result := Create_Internal_Env_Assoc_Array
        (Items_Count => Natural (Internal_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4013
Scope_Result := Map_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2172;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Env_Assoc_Array_Type;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1342;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1342;
               Finalizer_Scope_2172;
               Finalizer_Scope_2173;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_172;
--# end
   --
   --  Primitives for Bare_For_Loop_Var_Decl
   --
      procedure For_Loop_Var_Decl_Pre_Env_Actions
        (Self            : Bare_For_Loop_Var_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_149 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:13095");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Var_Decl
        (Self : Bare_For_Loop_Var_Decl
         ; For_Loop_Var_Decl_F_Id : Bare_Defining_Name
         ; For_Loop_Var_Decl_F_Id_Type : Bare_Subtype_Indication
        ) is
      begin
            Self.For_Loop_Var_Decl_F_Id := For_Loop_Var_Decl_F_Id;
            Self.For_Loop_Var_Decl_F_Id_Type := For_Loop_Var_Decl_F_Id_Type;
      end Initialize_Fields_For_For_Loop_Var_Decl;
   function For_Loop_Var_Decl_F_Id
     (Node : Bare_For_Loop_Var_Decl) return Bare_Defining_Name
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id;
   end;
   function For_Loop_Var_Decl_F_Id_Type
     (Node : Bare_For_Loop_Var_Decl) return Bare_Subtype_Indication
   is
   begin
         return Node.For_Loop_Var_Decl_F_Id_Type;
   end;
--# property-start ForLoopVarDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:13077
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Names
  (Node : Bare_For_Loop_Var_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1007;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1007 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1007;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4014 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:13077
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 4014
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1007;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1007;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Defining_Names;
--# end
--# property-start ForLoopVarDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:13079
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Defining_Env
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1008;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1008 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1008;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4016 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13079
--# expr-start 4015 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:13079
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4015
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4016
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1008;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1008;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Defining_Env;
--# end
--# property-start ForLoopVarDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:13082
pragma Warnings (Off, "is not referenced");
 function For_Loop_Var_Decl_P_Expr_Type
  (Node : Bare_For_Loop_Var_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Var_Decl :=
     Internal_Entity_For_Loop_Var_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Defining_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Subtype_Indication;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Var_Decl_For_Loop_Var_Decl_P_Expr_Type,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Var_Decl,
                                As_Bare_For_Loop_Var_Decl => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4024 '<Block>' Let_Result None
--# scope-start
--# expr-start 4023 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:13083
--# expr-start 4018 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:13084
--# expr-start 4017 '<FieldAccess .id_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13084
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 4017
Is_Null := Fld_4 = null; 
--# expr-done 4018
if Is_Null then
   --# expr-start 4020 '<FieldAccess .expression_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13089
--# expr-start 4019 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:13089
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.For_Loop_Var_Decl_F_Id, Info => Ent.Info);
--# expr-done 4019
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4020
   If_Result := Fld_1;
else
   --# expr-start 4022 '<FieldAccess .designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13092
--# expr-start 4021 '<FieldAccess .id_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13092
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.For_Loop_Var_Decl_F_Id_Type, Info => Ent.Info);
--# expr-done 4021
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_2.Node, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4022
   If_Result := Fld_3;
end if;
--# expr-done 4023
Let_Result := If_Result; 
--# end
--# expr-done 4024
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Var_Decl_P_Expr_Type;
--# end
--# property-start ForLoopVarDecl._env_mappings_149 /usr/src/packages/BUILD/ada/ast.py:13095
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_149
  (Node : Bare_For_Loop_Var_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_For_Loop_Var_Decl := Bare_For_Loop_Var_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1319;
      Fld : Symbol_Type;
Cast_Expr : Bare_For_Loop_Var_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1319 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1319;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4026 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:13095
--# expr-start 4025 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:13095
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4025
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4026
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1319;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1319;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_149;
--# end
   --
   --  Primitives for Bare_Generic_Decl
   --
      procedure Initialize_Fields_For_Generic_Decl
        (Self : Bare_Generic_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
        ) is
      begin
            Self.Generic_Decl_F_Formal_Part := Generic_Decl_F_Formal_Part;
      end Initialize_Fields_For_Generic_Decl;
   function Generic_Decl_F_Formal_Part
     (Node : Bare_Generic_Decl) return Bare_Generic_Formal_Part
   is
   begin
         return Node.Generic_Decl_F_Formal_Part;
   end;
--# property-start '[dispatcher]GenericDecl.decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Decl_P_Decl
  (Node : Bare_Generic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Decl := Bare_Generic_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Decl (Self.Kind) is
               when Ada_Generic_Package_Decl =>
                  --# property-call-start GenericPackageDecl.decl
                  Property_Result := Generic_Package_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Decl =>
                  --# property-call-start GenericSubpDecl.decl
                  Property_Result := Generic_Subp_Decl_P_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Decl_P_Decl;
--# end
--# property-start GenericDecl.get_aspect_assoc /usr/src/packages/BUILD/ada/ast.py:8475
pragma Warnings (Off, "is not referenced");
 function Generic_Decl_P_Get_Aspect_Assoc
  (Node : Bare_Generic_Decl
      ; Name : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Aspect_Assoc
is
   Self : Bare_Generic_Decl := Bare_Generic_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Decl :=
     Internal_Entity_Generic_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind name Name
   Property_Result : Internal_Entity_Aspect_Assoc;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Aspect_Assoc;
Let_Result : Internal_Entity_Aspect_Assoc;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4029 '<Block>' Let_Result None
--# scope-start
--# expr-start 4028 '<FieldAccess .get_aspect_assoc(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8478
--# expr-start 4027 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8478
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4027
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.get_aspect_assoc'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Get_Aspect_Assoc (Node => Fld.Node, Name => Name, E_Info => Fld.Info);
--# end
--# expr-done 4028
Let_Result := Fld_1; 
--# end
--# expr-done 4029
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Decl_P_Get_Aspect_Assoc;
--# end
   --
   --  Primitives for Bare_Generic_Package_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Package_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_134 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8558" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Package_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Package_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_132 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_133 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            Libadalang.Implementation.Dest_Env_Name_136 (Node => 
Self);
         Fallback_Env : constant Lexical_Env :=
           (if Dest_Env_Name = null
            then Libadalang.Implementation.Fallback_Env_137 (Node => 
Self)
            else Empty_Env);
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_135 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Fallback_Env,
            DSL_Location => "ast.py:8563");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_138 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_139 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_141 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_140 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_143 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_142 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Decl
        (Self : Bare_Generic_Package_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Package_Decl_F_Package_Decl : Bare_Generic_Package_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Package_Decl_F_Package_Decl := Generic_Package_Decl_F_Package_Decl;
      end Initialize_Fields_For_Generic_Package_Decl;
   function Generic_Package_Decl_F_Package_Decl
     (Node : Bare_Generic_Package_Decl) return Bare_Generic_Package_Internal
   is
   begin
         return Node.Generic_Package_Decl_F_Package_Decl;
   end;
--# property-start GenericPackageDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:8588
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_748;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_748 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_748;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4031 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8588
--# expr-start 4030 '<FieldAccess .package_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8588
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 4030
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4031
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_748;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_748;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8590
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_749;
      Fld : Bare_Generic_Package_Internal;
Fld_1 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_749 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_749;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4034 '<as_entity(<FieldAccess .package_name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:8591
--# expr-start 4033 '<FieldAccess .package_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8591
--# expr-start 4032 '<FieldAccess .package_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8591
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Generic_Package_Decl_F_Package_Decl;
--# expr-done 4032
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Base_Package_Decl_F_Package_Name;
--# expr-done 4033
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 4034
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_749;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_749;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageDecl.body_part /usr/src/packages/BUILD/ada/ast.py:8595
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Body_Part
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Package_Body
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Package_Body;
      Fld : Internal_Entity_Generic_Package_Internal;
Fld_1 : Internal_Entity_Package_Body;
Let_Result : Internal_Entity_Package_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4037 '<Block>' Let_Result None
--# scope-start
--# expr-start 4036 '<FieldAccess .body_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8600
--# expr-start 4035 '<FieldAccess .package_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 4035
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasePackageDecl.body_part
Fld_1 := Libadalang.Implementation.Base_Package_Decl_P_Body_Part (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4036
Let_Result := Fld_1; 
--# end
--# expr-done 4037
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Body_Part;
--# end
--# property-start GenericPackageDecl.decl /usr/src/packages/BUILD/ada/ast.py:8602
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Decl_P_Decl
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Package_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 4038 '<FieldAccess .package_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8602
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Package_Internal (Node => Ent.Node.Generic_Package_Decl_F_Package_Decl, Info => Ent.Info);
--# expr-done 4038
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Decl_P_Decl;
--# end
--# property-start GenericPackageDecl._env_do_132 /usr/src/packages/BUILD/ada/ast.py:8556
pragma Warnings (Off, "is not referenced");
 function Env_Do_132
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4039 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8556
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4039
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_132;
--# end
--# property-start GenericPackageDecl._initial_env_name_133 /usr/src/packages/BUILD/ada/ast.py:8559
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_133
  (Node : Bare_Generic_Package_Decl
  )
   return Symbol_Type
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4040 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:8559
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => True);
--# end
--# expr-done 4040
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_133;
--# end
--# property-start GenericPackageDecl._initial_env_134 /usr/src/packages/BUILD/ada/ast.py:8560
pragma Warnings (Off, "is not referenced");
 function Initial_Env_134
  (Node : Bare_Generic_Package_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1304;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1304 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1304;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4041 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8560
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4041
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1304;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1304;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_134;
--# end
--# property-start GenericPackageDecl._env_mappings_135 /usr/src/packages/BUILD/ada/ast.py:8563
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_135
  (Node : Bare_Generic_Package_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Decl :=
     Internal_Entity_Generic_Package_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1305;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Package_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1305 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1305;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4043 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8563
--# expr-start 4042 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8564
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4042
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4043
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1305;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1305;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_135;
--# end
--# property-start GenericPackageDecl._dest_env_name_136 /usr/src/packages/BUILD/ada/ast.py:8566
pragma Warnings (Off, "is not referenced");
 function Dest_Env_Name_136
  (Node : Bare_Generic_Package_Decl
  )
   return Symbol_Type
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4044 '<FieldAccess .child_decl_initial_env_name(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:8566
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self, Private_Part => False);
--# end
--# expr-done 4044
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dest_Env_Name_136;
--# end
--# property-start GenericPackageDecl._fallback_env_137 /usr/src/packages/BUILD/ada/ast.py:8567
pragma Warnings (Off, "is not referenced");
 function Fallback_Env_137
  (Node : Bare_Generic_Package_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1307;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1307 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1307;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4045 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8567
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4045
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1307;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1307;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Fallback_Env_137;
--# end
--# property-start GenericPackageDecl._env_trans_parent_138 /usr/src/packages/BUILD/ada/ast.py:8555
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_138
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_138;
--# end
--# property-start GenericPackageDecl._env_do_139 /usr/src/packages/BUILD/ada/ast.py:8572
pragma Warnings (Off, "is not referenced");
 function Env_Do_139
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1309;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1309 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1309;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4046 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8572
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4046
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1309;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1309;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_139;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_140 /usr/src/packages/BUILD/ada/ast.py:8574
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_140
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1310;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1310 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1310;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4047 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8574
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4047
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1310;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1310;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_140;
--# end
--# property-start GenericPackageDecl._ref_cond_141 /usr/src/packages/BUILD/ada/ast.py:8576
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_141
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4049 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8576
--# expr-start 4048 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8576
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4048
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4049
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_141;
--# end
--# property-start GenericPackageDecl._ref_env_nodes_142 /usr/src/packages/BUILD/ada/ast.py:8579
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_142
  (Node : Bare_Generic_Package_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1312;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1312 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1312;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4050 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8579
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4050
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1312;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1312;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_142;
--# end
--# property-start GenericPackageDecl._ref_cond_143 /usr/src/packages/BUILD/ada/ast.py:8581
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_143
  (Node : Bare_Generic_Package_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Decl := Bare_Generic_Package_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4052 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8581
--# expr-start 4051 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8581
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4051
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4052
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_143;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Subp_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_122 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8503" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_120 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_121 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_123 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:8508");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_124 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_125 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_127 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_126 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_129 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_128 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Decl
        (Self : Bare_Generic_Subp_Decl
         ; Generic_Decl_F_Formal_Part : Bare_Generic_Formal_Part
         ; Generic_Subp_Decl_F_Subp_Decl : Bare_Generic_Subp_Internal
        ) is
      begin
            Initialize_Fields_For_Generic_Decl
              (Self, Generic_Decl_F_Formal_Part);
            Self.Generic_Subp_Decl_F_Subp_Decl := Generic_Subp_Decl_F_Subp_Decl;
      end Initialize_Fields_For_Generic_Subp_Decl;
   function Generic_Subp_Decl_F_Subp_Decl
     (Node : Bare_Generic_Subp_Decl) return Bare_Generic_Subp_Internal
   is
   begin
         return Node.Generic_Subp_Decl_F_Subp_Decl;
   end;
--# property-start GenericSubpDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8489
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_742;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_742 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_742;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4056 '<as_entity(<FieldAccess .name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:8490
--# expr-start 4055 '<FieldAccess .name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8490
--# expr-start 4054 '<FieldAccess .subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8490
--# expr-start 4053 '<FieldAccess .subp_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8490
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 4053
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Fld.Node.Generic_Subp_Internal_F_Subp_Spec, Info => Fld.Info);
--# expr-done 4054
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.name'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4055
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
As_Entity := (Info => E_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 4056
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_742;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_742;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpDecl.body_part /usr/src/packages/BUILD/ada/ast.py:8493
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Body_Part
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Subp_Body
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Base_Subp_Body;
      Fld : Internal_Entity_Body_Node;
Cast_Result : Internal_Entity_Base_Subp_Body;
Let_Result : Internal_Entity_Base_Subp_Body;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4059 '<Block>' Let_Result None
--# scope-start
--# expr-start 4058 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8497
--# expr-start 4057 '<FieldAccess .body_part_for_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8497
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4057
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 4058
Let_Result := Cast_Result; 
--# end
--# expr-done 4059
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Body_Part;
--# end
--# property-start GenericSubpDecl.decl /usr/src/packages/BUILD/ada/ast.py:8527
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Decl
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 4060 '<FieldAccess .subp_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8527
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 4060
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Decl;
--# end
--# property-start GenericSubpDecl.is_imported /usr/src/packages/BUILD/ada/ast.py:8530
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Is_Imported
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4062 '<FieldAccess .is_imported>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8530
--# expr-start 4061 '<FieldAccess .subp_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8530
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 4061
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_imported'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Imported (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4062
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Is_Imported;
--# end
--# property-start GenericSubpDecl.next_part_for_decl /usr/src/packages/BUILD/ada/ast.py:8531
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Decl_P_Next_Part_For_Decl
  (Node : Bare_Generic_Subp_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Generic_Subp_Internal;
Fld_1 : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4064 '<FieldAccess .next_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8531
--# expr-start 4063 '<FieldAccess .subp_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:8531
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Generic_Subp_Internal (Node => Ent.Node.Generic_Subp_Decl_F_Subp_Decl, Info => Ent.Info);
--# expr-done 4063
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 4064
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Decl_P_Next_Part_For_Decl;
--# end
--# property-start GenericSubpDecl._env_do_120 /usr/src/packages/BUILD/ada/ast.py:8501
pragma Warnings (Off, "is not referenced");
 function Env_Do_120
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4065 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8501
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4065
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_120;
--# end
--# property-start GenericSubpDecl._initial_env_name_121 /usr/src/packages/BUILD/ada/ast.py:8504
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_121
  (Node : Bare_Generic_Subp_Decl
  )
   return Symbol_Type
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4066 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4066
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_121;
--# end
--# property-start GenericSubpDecl._initial_env_122 /usr/src/packages/BUILD/ada/ast.py:8505
pragma Warnings (Off, "is not referenced");
 function Initial_Env_122
  (Node : Bare_Generic_Subp_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1292;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1292 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1292;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4067 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8505
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4067
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1292;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1292;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_122;
--# end
--# property-start GenericSubpDecl._env_mappings_123 /usr/src/packages/BUILD/ada/ast.py:8508
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_123
  (Node : Bare_Generic_Subp_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Decl :=
     Internal_Entity_Generic_Subp_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1293;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Subp_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1293 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1293;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4069 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8508
--# expr-start 4068 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8509
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4068
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4069
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1293;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1293;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_123;
--# end
--# property-start GenericSubpDecl._env_trans_parent_124 /usr/src/packages/BUILD/ada/ast.py:8499
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_124
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_124;
--# end
--# property-start GenericSubpDecl._env_do_125 /usr/src/packages/BUILD/ada/ast.py:8514
pragma Warnings (Off, "is not referenced");
 function Env_Do_125
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1295;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1295 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1295;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4070 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8514
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4070
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1295;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1295;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_125;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_126 /usr/src/packages/BUILD/ada/ast.py:8516
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_126
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1296;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1296 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1296;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4071 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8516
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4071
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1296;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1296;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_126;
--# end
--# property-start GenericSubpDecl._ref_cond_127 /usr/src/packages/BUILD/ada/ast.py:8518
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_127
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4073 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8518
--# expr-start 4072 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8518
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4072
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4073
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_127;
--# end
--# property-start GenericSubpDecl._ref_env_nodes_128 /usr/src/packages/BUILD/ada/ast.py:8521
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_128
  (Node : Bare_Generic_Subp_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1298;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1298 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1298;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4074 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8521
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4074
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1298;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1298;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_128;
--# end
--# property-start GenericSubpDecl._ref_cond_129 /usr/src/packages/BUILD/ada/ast.py:8523
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_129
  (Node : Bare_Generic_Subp_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Decl := Bare_Generic_Subp_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4076 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8523
--# expr-start 4075 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8523
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4075
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4076
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_129;
--# end
   --
   --  Primitives for Bare_Generic_Instantiation
   --
      procedure Initialize_Fields_For_Generic_Instantiation
        (Self : Bare_Generic_Instantiation
        ) is
      begin
      Self.Generic_Instantiation_Inst_Env := Empty_Env;
      Self.LF_Present_Instantiation_Env := False;
      Self.LF_Stg_Instantiation_Env := Empty_Env;
      Self.LF_Present_Actual_Expr_Decls := False;
      Self.LF_Stg_Actual_Expr_Decls := No_Bare_Anonymous_Expr_Decl_Array_Type;
      end Initialize_Fields_For_Generic_Instantiation;
--# property-start GenericInstantiation.instantiation_env /usr/src/packages/BUILD/ada/ast.py:7732
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Instantiation_Env
  (Node : Bare_Generic_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_690;
      Dyn_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_690 is
            begin
                     Dec_Ref (Dyn_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_690;
begin
   --# property-body-start
      if Self.LF_Present_Instantiation_Env then
         Property_Result := Self.LF_Stg_Instantiation_Env;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4078 '<Block>' Let_Result None
--# scope-start
--# expr-start 4077 '<langkit.expressions.envs.DynamicLexicalEnv object at 0xf5c9a658>' Dyn_Env /usr/src/packages/BUILD/ada/ast.py:7733
Dyn_Env := Create_Dynamic_Lexical_Env (Self, Generic_Instantiation_P_Instantiation_Bindings'Access, Ada_Node_P_Resolve_Generic_Actual'Access, False); 
--# expr-done 4077
Let_Result := Dyn_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 4078
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_690;
      Self.LF_Present_Instantiation_Env := True;
      Self.LF_Stg_Instantiation_Env := Property_Result;
         Inc_Ref (Property_Result);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_690;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_F_Instantiation_Env;
--# end
--# property-start GenericInstantiation.instantiation_bindings /usr/src/packages/BUILD/ada/ast.py:7741
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Instantiation_Bindings
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Inner_Env_Assoc_Array_Access
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Instantiation :=
     Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_691;
            procedure Finalizer_Scope_2174;
            procedure Finalizer_Scope_2175;
      Var_Expr : Internal_Entity_Generic_Decl;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Generic_Decl;
Fld_2 : Internal_Entity_Generic_Formal_Part;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Param_Match_Array_Access;
Scope_Result : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_5 : Internal_Single_Formal;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Symbol_Type;
Fld_8 : Bare_Anonymous_Expr_Decl_Array_Access;
Get_Result : Bare_Anonymous_Expr_Decl;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Bare_Expr;
Cast_Result : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Fld_13 : Internal_Single_Formal;
Fld_14 : Internal_Entity_Base_Formal_Param_Decl;
Is_A : Boolean;
If_Result : Bare_Ada_Node;
New_Struct : Internal_Metadata;
New_Struct_1 : Internal_Inner_Env_Assoc;
I : Integer;
Map_Result : Internal_Inner_Env_Assoc_Array_Access;
Dyn_Var_Bind_Result : Internal_Inner_Env_Assoc_Array_Access;
Fld_15 : Boolean;
If_Result_1 : Internal_Inner_Env_Assoc_Array_Access;
Let_Result : Internal_Inner_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_691 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_691;
            procedure Finalizer_Scope_2174 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2174;
            procedure Finalizer_Scope_2175 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2175;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Generic_Instantiation_Generic_Instantiation_P_Instantiation_Bindings,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Generic_Instantiation,
                                As_Bare_Generic_Instantiation => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Inner_Env_Assoc_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
   --# scope-start
      --# expr-start 4103 '<Block>' Let_Result None
--# scope-start
--# expr-start 4102 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7742
--# expr-start 4079 '<FieldAccess .is_any_formal>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:7743
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_15 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4079
if Fld_15 then
   If_Result_1 := No_Internal_Inner_Env_Assoc_Array_Type;
else
   --# expr-start 4101 '<bind(<DynamicVariable env>, <FieldAccess .default_initial_env>, <map>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:7745
--# expr-start 4080 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7746
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4080
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4100 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7747
--# expr-start 4081 '<FieldAccess .nonbound_generic_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7747
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_1 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 4081
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 4084 '<FieldAccess .match_param_list(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7747
--# expr-start 4082 '<FieldAccess .formal_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7747
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 4082
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4083 '<FieldAccess .generic_inst_params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7748
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4083
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_2.Node, Params => Fld_3, Is_Dottable_Subp => False, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 4084
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2174;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Inner_Env_Assoc_Vectors.Vector;
begin
         I := 0;
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_145 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_145
               --# bind i I
      --# expr-start 4099 '<New InternalInnerEnvAssoc>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:7750
--# expr-start 4087 '<FieldAccess .name_symbol>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7751
--# expr-start 4086 '<FieldAccess .name>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7751
--# expr-start 4085 '<FieldAccess .formal>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7751
Fld_5 := Item_145.Formal;
--# expr-done 4085
Fld_6 := Fld_5.Name;
--# expr-done 4086
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 4087
--# expr-start 4088 '<New InternalMetadata>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7757
New_Struct := (Access_Entity => False, Dottable_Subp => False, Primitive => No_Bare_Ada_Node, Primitive_Real_Type => No_Bare_Ada_Node); 
--# expr-done 4088
--# expr-start 4098 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:7752
--# expr-start 4091 '<IsA GenericFormalObjDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:7753
--# expr-start 4090 '<FieldAccess .spec>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7753
--# expr-start 4089 '<FieldAccess .formal>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7753
Fld_13 := Item_145.Formal;
--# expr-done 4089
Fld_14 := Fld_13.Spec;
--# expr-done 4090
Is_A := Fld_14.Node /= null 
and then Fld_14.Node.Kind in Ada_Generic_Formal_Obj_Decl_Range; 
--# expr-done 4091
if Is_A then
--# expr-start 4093 '<at(<FieldAccess .actual_expr_decls>, <Var i>)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:7754
--# expr-start 4092 '<FieldAccess .actual_expr_decls>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7754
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.actual_expr_decls
Fld_8 := Libadalang.Implementation.Generic_Instantiation_F_Actual_Expr_Decls (Node => Ent.Node);
--# end
--# expr-done 4092
Get_Result := Get (Fld_8, I, True); 
--# expr-done 4093
      Cast_Result := Get_Result;
   If_Result := Cast_Result;
else
--# expr-start 4097 '<FieldAccess .node>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7755
--# expr-start 4096 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7755
--# expr-start 4095 '<FieldAccess .assoc>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7755
--# expr-start 4094 '<FieldAccess .actual>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7755
Fld_9 := Item_145.Actual;
--# expr-done 4094
Fld_10 := Fld_9.Assoc;
--# expr-done 4095
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 4096
Fld_12 := Fld_11.Node;
--# expr-done 4097
      Cast_Result_1 := Fld_12;
   If_Result := Cast_Result_1;
end if;
--# expr-done 4098
New_Struct_1 := (Key => Fld_7, Metadata => New_Struct, Val => If_Result); 
--# expr-done 4099
         declare
            Item_To_Append : constant Internal_Inner_Env_Assoc :=
               New_Struct_1;
         begin
            Internal_Inner_Env_Assoc_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
      Finalizer_Scope_2175;
         end loop;
      end;
      Map_Result := Create_Internal_Inner_Env_Assoc_Array
        (Items_Count => Natural (Internal_Inner_Env_Assoc_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Inner_Env_Assoc_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Inner_Env_Assoc_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Inner_Env_Assoc_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4100
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4101
   If_Result_1 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4102
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4103
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_691;
         Mmz_Val := (Kind => Mmz_Internal_Inner_Env_Assoc_Array_Access,
                     As_Internal_Inner_Env_Assoc_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_691;
               Finalizer_Scope_2174;
               Finalizer_Scope_2175;
         if Self /= null then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Instantiation_Bindings;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_entity_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_entity_name
                  Property_Result := Generic_Package_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_entity_name
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Entity_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start '[dispatcher]GenericInstantiation.generic_inst_params' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.generic_inst_params
                  Property_Result := Generic_Package_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.generic_inst_params
                  Property_Result := Generic_Subp_Instantiation_P_Generic_Inst_Params
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericInstantiation.is_any_formal /usr/src/packages/BUILD/ada/ast.py:7776
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Is_Any_Formal
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Instantiation :=
     Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Assoc;
Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4106 '<IsA BoxExpr>' Is_A /usr/src/packages/BUILD/ada/ast.py:7777
--# expr-start 4104 '<FieldAccess .generic_inst_params>' Fld /usr/src/packages/BUILD/ada/ast.py:7777
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4104
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 4105 '<FieldAccess .expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7777
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4105
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Is_A := Result_Var.Node /= null 
and then Result_Var.Node.Kind in Ada_Box_Expr_Range; 
--# expr-done 4106
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Is_Any_Formal;
--# end
--# property-start GenericInstantiation.nonbound_generic_decl /usr/src/packages/BUILD/ada/ast.py:7780
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Nonbound_Generic_Decl
  (Node : Bare_Generic_Instantiation
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_695;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity;
Match_0 : Internal_Entity_Body_Node;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity_Basic_Decl;
Match_3 : Internal_Entity;
As_Entity : Internal_Entity_Generic_Instantiation;
Is_Null : Boolean;
If_Result : Internal_Entity_Generic_Instantiation;
Fld : Internal_Entity_Name;
Cast_Expr : Bare_Generic_Instantiation;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity_Array_Access;
Fld_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Bound_Imprecise_Fallback : Boolean;
Fld_3 : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result_1 : Internal_Entity_Body_Node;
Let_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Renaming_Decl;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Let_Result_1 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Let_Result_2 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Cast_Result_5 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Generic_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Let_Result_3 : Internal_Entity_Basic_Decl;
Scope_Result_3 : Internal_Entity_Basic_Decl;
Match_Result : Internal_Entity_Basic_Decl;
Scope_Result_4 : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Generic_Decl;
Scope_Result_5 : Internal_Entity_Generic_Decl;
Result_Var_1 : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_695 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_695;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 4112 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:7781
--# expr-start 4111 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7781
--# expr-start 4109 '<FieldAccess .all_env_elements_internal(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7781
--# expr-start 4108 '<FieldAccess .generic_entity_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7781
--# expr-start 4107 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:7781
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Generic_Instantiation;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4107
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4108
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start Name.all_env_elements_internal
Fld_1 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Fld.Node, Seq => True, Seq_From => Cast_Result, Categories => (Inherited_Primitives => False, Nocat => True), E_Info => Fld.Info);
--# end
--# expr-done 4109
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_146 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_146
               --# expr-start 4110 '<FieldAccess .has_visibility(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7784
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_2 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_146);
--# end
--# expr-done 4110
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_146;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4111
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4112
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
--# expr-start 4116 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:7781
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind b Match_0
--# expr-start 4114 '<bind(<DynamicVariable imprecise_fallback>, False, <FieldAccess .decl_part>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:7786
Bound_Imprecise_Fallback := False; 
--# expr-start 4113 '<FieldAccess .decl_part>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7786
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_3 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_0.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Match_0.Info);
--# end
--# expr-done 4113
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 4114
Let_Result := Dyn_Var_Bind_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind rd Match_1
--# expr-start 4115 '<FieldAccess .resolve>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7787
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_4 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4115
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
Let_Result_1 := Cast_Result_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind d Match_2
Let_Result_2 := Match_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Cast_Expr_1 := No_Entity_Generic_Decl; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Let_Result_3 := Cast_Result_6; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 4116
Scope_Result_4 := Match_Result;
   --# end
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4117 '<Cast to GenericDecl>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:7781
Cast_Expr_2 := Var_Expr; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_7 := Create_Internal_Entity_Generic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_7 := No_Entity_Generic_Decl;
   end if;
--# expr-done 4117
Scope_Result_5 := Cast_Result_7;
   --# end
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Entity_Generic_Decl;
end if;
      Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_695;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_695;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Nonbound_Generic_Decl;
--# end
--# property-start '[dispatcher]GenericInstantiation.designated_generic_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Instantiation (Self.Kind) is
               when Ada_Generic_Package_Instantiation =>
                  --# property-call-start GenericPackageInstantiation.designated_generic_decl
                  Property_Result := Generic_Package_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Instantiation =>
                  --# property-call-start GenericSubpInstantiation.designated_generic_decl
                  Property_Result := Generic_Subp_Instantiation_P_Designated_Generic_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericInstantiation.actual_expr_decls /usr/src/packages/BUILD/ada/ast.py:7807
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_F_Actual_Expr_Decls
  (Node : Bare_Generic_Instantiation
  )
   return Bare_Anonymous_Expr_Decl_Array_Access
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_697;
      Match_0 : Bare_Generic_Subp_Instantiation;
Match_1 : Bare_Generic_Package_Instantiation;
Fld : Bare_Assoc_List;
Fld_1 : Bare_Assoc_List;
Match_Prefix : Bare_Generic_Instantiation;
Cast_Result : Bare_Generic_Subp_Instantiation;
Let_Result : Bare_Assoc_List;
Scope_Result : Bare_Assoc_List;
Cast_Result_1 : Bare_Generic_Package_Instantiation;
Let_Result_1 : Bare_Assoc_List;
Scope_Result_1 : Bare_Assoc_List;
Match_Result : Bare_Assoc_List;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Bare_Expr;
New_Node : Bare_Anonymous_Expr_Decl;
Item_147 : Bare_Basic_Assoc;
Map_Result : Bare_Anonymous_Expr_Decl_Array_Access;
Let_Result_2 : Bare_Anonymous_Expr_Decl_Array_Access;
            procedure Finalizer_Scope_697 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_697;
begin
   --# property-body-start
      if Self.LF_Present_Actual_Expr_Decls then
         Property_Result := Self.LF_Stg_Actual_Expr_Decls;
            Inc_Ref (Property_Result);
         return Property_Result;
      end if;
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4126 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 4125 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7808
--# expr-start 4120 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:7808
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Generic_Instantiation (Match_Prefix.Kind) is
         when Ada_Generic_Subp_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind subp Match_0
--# expr-start 4118 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:7809
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_0.Generic_Subp_Instantiation_F_Params;
--# expr-done 4118
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Instantiation =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind pkg Match_1
--# expr-start 4119 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7810
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_1.Generic_Package_Instantiation_F_Params;
--# expr-done 4119
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4120
declare
   Map_Result_Vec : Bare_Anonymous_Expr_Decl_Vectors.Vector;
begin
      if Match_Result = null then
         Map_Result := Create_Bare_Anonymous_Expr_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Match_Result;
      begin
         for Untyped_Item_147 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_147 := Untyped_Item_147; 
   --# scope-start
               --# bind assoc Item_147
      --# expr-start 4124 '<New BareAnonymousExprDecl>' New_Node /usr/src/packages/BUILD/ada/ast.py:7812
--# expr-start 4123 '<FieldAccess .node>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7813
--# expr-start 4122 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7813
--# expr-start 4121 '<as_bare_entity(<Var assoc>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:7813
Is_Null := Item_147 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
As_Entity := (Info => No_Entity_Info, Node => Item_147); 
   If_Result := As_Entity;
end if;
--# expr-done 4121
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 4122
Fld_3 := Fld_2.Node;
--# expr-done 4123
if Is_Foreign_Strict (Self.Self_Env, Self) then
   raise Property_Error with
      "synthetic nodes cannot have foreign lexical envs";
end if;
New_Node := new Root_Node_Record
  (Ada_Anonymous_Expr_Decl);
Initialize
  (Self => New_Node,
   Kind => Ada_Anonymous_Expr_Decl,
   Unit => Self.Unit,
   Token_Start_Index => No_Token_Index,
   Token_End_Index   => No_Token_Index,
   Parent => Self,
   Self_Env => Self.Self_Env);
Register_Destroyable (Self.Unit, New_Node);
   Initialize_Fields_For_Anonymous_Expr_Decl
     (Self => New_Node,
      Anonymous_Expr_Decl_F_Expr => Fld_3);
--# expr-done 4124
         declare
            Item_To_Append : constant Bare_Anonymous_Expr_Decl :=
               New_Node;
         begin
            Bare_Anonymous_Expr_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Anonymous_Expr_Decl_Array
        (Items_Count => Natural (Bare_Anonymous_Expr_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Anonymous_Expr_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Anonymous_Expr_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Anonymous_Expr_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4125
Let_Result_2 := Map_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 4126
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_697;
      Self.LF_Present_Actual_Expr_Decls := True;
      Self.LF_Stg_Actual_Expr_Decls := Property_Result;
         Inc_Ref (Property_Result);
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_697;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_F_Actual_Expr_Decls;
--# end
--# property-start GenericInstantiation.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7817
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Entry_Point
  (Node : Bare_Generic_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Xref_Entry_Point;
--# end
--# property-start GenericInstantiation.xref_equation /usr/src/packages/BUILD/ada/ast.py:7819
pragma Warnings (Off, "is not referenced");
 function Generic_Instantiation_P_Xref_Equation
  (Node : Bare_Generic_Instantiation
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Instantiation := Bare_Generic_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Instantiation :=
     Internal_Entity_Generic_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_699;
            procedure Finalizer_Scope_2180;
            procedure Finalizer_Scope_2181;
            procedure Finalizer_Scope_2182;
            procedure Finalizer_Scope_2183;
            procedure Finalizer_Scope_2184;
            procedure Finalizer_Scope_2185;
            procedure Finalizer_Scope_2186;
            procedure Finalizer_Scope_2187;
            procedure Finalizer_Scope_2188;
            procedure Finalizer_Scope_2189;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Name;
Var_Expr : Internal_Entity_Generic_Decl;
Actual_Name : Internal_Entity_Name;
Match_0_1 : Internal_Entity_Type_Decl;
Match_1_1 : Internal_Entity_Formal_Subp_Decl;
Match_2 : Internal_Entity_Object_Decl;
Match_3 : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Name;
Var_Expr_2 : Bare_Base_Id;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Generic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Name;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_2 : Internal_Entity_Name;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Match_Result : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_6 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Generic_Decl;
Fld_7 : Internal_Entity_Generic_Formal_Part;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Internal_Param_Match_Array_Access;
Scope_Result_2 : Internal_Param_Match_Array_Access;
Result_Var : Internal_Param_Match_Array_Access;
Fld_10 : Internal_Single_Actual;
Fld_11 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Cast_Result_4 : Internal_Entity_Name;
Fld_13 : Internal_Single_Formal;
Fld_14 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_5 : Internal_Entity_Generic_Formal;
Fld_15 : Internal_Entity_Basic_Decl;
Fld_16 : Logic_Equation;
Fld_17 : Logic_Equation;
Fld_18 : Logic_Var;
Cast_Expr : Internal_Entity_Formal_Subp_Decl;
Cast_Result_6 : Internal_Entity_Basic_Decl;
Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Logic_True_2 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Fld_21 : Internal_Entity_Single_Tok_Node;
Fld_22 : Boolean;
If_Result : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
Fld_27 : Internal_Single_Actual;
Fld_28 : Internal_Entity_Basic_Assoc;
Fld_29 : Internal_Entity_Expr;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Logic_True_4 : Logic_Equation;
Match_Prefix_1 : Internal_Entity_Basic_Decl;
Cast_Result_8 : Internal_Entity_Type_Decl;
Let_Result_2 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_9 : Internal_Entity_Formal_Subp_Decl;
Let_Result_3 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Object_Decl;
Let_Result_4 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Basic_Decl;
Let_Result_5 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Fld_32 : Internal_Single_Actual;
Fld_33 : Bare_Base_Id;
Fld_34 : Logic_Var;
Fld_35 : Internal_Single_Formal;
Fld_36 : Internal_Entity_Defining_Name;
Fld_37 : Bare_Defining_Name;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Defining_Name;
Fld_38 : Internal_Entity_Basic_Decl;
Cast_Result_12 : Internal_Entity;
Bind_Result_2 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Logic_True_5 : Logic_Equation;
Result_Var_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result_6 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_39 : Boolean;
If_Result_2 : Logic_Equation;
And_Pred_4 : Logic_Equation;
            procedure Finalizer_Scope_699 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_699;
            procedure Finalizer_Scope_2180 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2180;
            procedure Finalizer_Scope_2181 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2181;
            procedure Finalizer_Scope_2182 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2182;
            procedure Finalizer_Scope_2183 is
            begin
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_5);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Logic_True_5);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_2183;
            procedure Finalizer_Scope_2184 is
            begin
                     Dec_Ref (Fld_16);
            end Finalizer_Scope_2184;
            procedure Finalizer_Scope_2185 is
            begin
                     Dec_Ref (Fld_17);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2185;
            procedure Finalizer_Scope_2186 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2186;
            procedure Finalizer_Scope_2187 is
            begin
                     Dec_Ref (Fld_26);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2187;
            procedure Finalizer_Scope_2188 is
            begin
                     Dec_Ref (Logic_True_4);
            end Finalizer_Scope_2188;
            procedure Finalizer_Scope_2189 is
            begin
                     Dec_Ref (Bind_Result_2);
            end Finalizer_Scope_2189;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4188 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:7820
--# expr-start 4135 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7820
--# expr-start 4130 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:7820
--# expr-start 4128 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7820
--# expr-start 4127 '<FieldAccess .generic_entity_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7820
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4127
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld.Node);
--# end
--# expr-done 4128
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 4129 '<FieldAccess .nonbound_generic_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7821
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld_2 := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Ent.Node);
--# end
--# expr-done 4129
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7820") else null)); 
--# expr-done 4130
--# expr-start 4134 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:7822
--# expr-start 4131 '<FieldAccess .generic_entity_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7822
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_entity_name'
Fld_3 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4131
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_3; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_1; 
--# bind dn Match_0
--# expr-start 4133 '<FieldAccess .xref_no_overloading>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7823
--# expr-start 4132 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7823
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 4132
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4133
Let_Result := Fld_5; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2180;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_2; 
--# bind _ Match_1
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7824") else null)); 
Let_Result_1 := Logic_True; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2181;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 4134
And_Pred := Logic_And (Bind_Result, Match_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7820") else null)); 
--# expr-done 4135
--# expr-start 4187 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:7825
--# expr-start 4136 '<FieldAccess .is_any_formal>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:7826
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.is_any_formal
Fld_39 := Libadalang.Implementation.Generic_Instantiation_P_Is_Any_Formal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4136
if Fld_39 then
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7827") else null)); 
   If_Result_2 := Logic_True_1;
else
   --# expr-start 4186 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:7829
--# expr-start 4185 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7829
--# expr-start 4138 '<Cast to GenericDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:7829
--# expr-start 4137 '<FieldAccess .designated_generic_decl>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7829
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.designated_generic_decl'
Fld_6 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Designated_Generic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4137
   if Fld_6.Node = null
      or else Fld_6.Node.Kind in Ada_Generic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Generic_Decl
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4138
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 4141 '<FieldAccess .match_param_list(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7829
--# expr-start 4139 '<FieldAccess .formal_part>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7829
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr.Info);
--# expr-done 4139
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4140 '<FieldAccess .generic_inst_params>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7831
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericInstantiation.generic_inst_params'
Fld_8 := Libadalang.Implementation.Dispatcher_Generic_Instantiation_P_Generic_Inst_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4140
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Fld_7.Node, Params => Fld_8, Is_Dottable_Subp => False, Env => Env, E_Info => Fld_7.Info);
--# end
--# expr-done 4141
Scope_Result_2 := Fld_9;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2182;
   Result_Var := Scope_Result_2;
else
   Result_Var := No_Internal_Param_Match_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Result_Var;
      begin
         for Item_148 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_148
      --# expr-start 4184 '<Let actual_name>' Let_Result_6 /usr/src/packages/BUILD/ada/ast.py:7832
--# scope-start
--# expr-start 4145 '<Cast to Name>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:7833
--# expr-start 4144 '<FieldAccess .expr>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7833
--# expr-start 4143 '<FieldAccess .assoc>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7833
--# expr-start 4142 '<FieldAccess .actual>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7833
Fld_10 := Item_148.Actual;
--# expr-done 4142
Fld_11 := Fld_10.Assoc;
--# expr-done 4143
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# end
--# expr-done 4144
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Name
   then
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result_4 := No_Entity_Name;
   end if;
--# expr-done 4145
Actual_Name := Cast_Result_4; 
--# bind actual_name Actual_Name
--# expr-start 4183 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:7834
--# expr-start 4173 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:7834
--# expr-start 4149 '<FieldAccess .decl>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:7834
--# expr-start 4148 '<Cast to GenericFormal>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:7834
--# expr-start 4147 '<FieldAccess .spec>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7834
--# expr-start 4146 '<FieldAccess .formal>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7834
Fld_13 := Item_148.Formal;
--# expr-done 4146
Fld_14 := Fld_13.Spec;
--# expr-done 4147
   if Fld_14.Node = null
      or else Fld_14.Node.Kind in Ada_Generic_Formal
   then
      Cast_Result_5 := Create_Internal_Entity_Generic_Formal
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
   else
         Cast_Result_5 := No_Entity_Generic_Formal;
   end if;
--# expr-done 4148
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Basic_Decl (Node => Cast_Result_5.Node.Generic_Formal_F_Decl, Info => Cast_Result_5.Info);
--# expr-done 4149
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld_15; 
case Ada_Basic_Decl (Match_Prefix_1.Node.Kind) is
         when Ada_Type_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Type_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0_1 := Cast_Result_8; 
--# bind _ Match_0_1
--# expr-start 4150 '<FieldAccess .xref_no_overloading>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:7835
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_16 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 4150
Let_Result_2 := Fld_16; Inc_Ref (Let_Result_2);
--# end
Scope_Result_4 := Let_Result_2;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2184;
            Match_Result_1 := Scope_Result_4; Inc_Ref (Match_Result_1);
         when Ada_Abstract_Formal_Subp_Decl .. Ada_Concrete_Formal_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Formal_Subp_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1_1 := Cast_Result_9; 
--# bind subp_decl Match_1_1
--# expr-start 4161 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:7838
--# expr-start 4155 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:7839
--# expr-start 4151 '<FieldAccess .xref_no_overloading(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:7839
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_17 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Actual_Name.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Actual_Name.Info);
--# end
--# expr-done 4151
--# expr-start 4154 '<Predicate on BasicDecl.subp_decl_match_signature>' Pred /usr/src/packages/BUILD/ada/ast.py:7840
--# expr-start 4152 '<FieldAccess .ref_var>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:7841
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Actual_Name.Node);
--# end
--# expr-done 4152
Fld_18.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_18.all);
Eq_Node.Refs.Destroy (Fld_18.all);
--# expr-start 4153 '<Cast to BasicDecl>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:7842
Cast_Expr := Match_1_1; 
      Cast_Result_6 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 4153
Pred := Basic_Decl_P_Subp_Decl_Match_Signature_0_Pred.Create (Fld_18, Create_Basic_Decl_P_Subp_Decl_Match_Signature_0_Predicate (Cast_Result_6, Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Subp_Decl_Match_Signature")            else null))); 
--# expr-done 4154
And_Pred_1 := Logic_And (Fld_17, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7839") else null)); 
--# expr-done 4155
--# expr-start 4160 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:7843
--# expr-start 4157 '<FieldAccess .is_operator_name>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:7847
--# expr-start 4156 '<FieldAccess .relative_name>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:7847
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Actual_Name.Node, E_Info => Actual_Name.Info);
--# end
--# expr-done 4156
      if Fld_21.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_operator_name
Fld_22 := Libadalang.Implementation.Name_P_Is_Operator_Name (Node => Fld_21.Node, E_Info => Fld_21.Info);
--# end
--# expr-done 4157
if Fld_22 then
--# expr-start 4158 '<FieldAccess .base_name>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:7848
      if Actual_Name.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.base_name'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Base_Name (Node => Actual_Name.Node, E_Info => Actual_Name.Info);
--# end
--# expr-done 4158
Var_Expr_1 := Fld_19;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 4159 '<FieldAccess .xref_no_overloading>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:7849
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_20 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4159
Scope_Result_3 := Fld_20;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2186;
   Result_Var_1 := Scope_Result_3;
else
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7850") else null)); 
   Result_Var_1 := Logic_True_2;
end if;
      Inc_Ref (Result_Var_1);
   If_Result := Result_Var_1;
else
   Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7852") else null)); 
   If_Result := Logic_True_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 4160
Or_Pred := Logic_Or (And_Pred_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7838") else null)); 
--# expr-done 4161
Let_Result_3 := Or_Pred; Inc_Ref (Let_Result_3);
--# end
Scope_Result_5 := Let_Result_3;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2185;
            Match_Result_1 := Scope_Result_5; Inc_Ref (Match_Result_1);
         when Ada_Object_Decl .. Ada_Extended_Return_Stmt_Object_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Object_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_10; 
--# bind obj_decl Match_2
--# expr-start 4172 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:7857
--# expr-start 4165 '<FieldAccess .sub_equation>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:7857
--# expr-start 4164 '<FieldAccess .expr>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:7857
--# expr-start 4163 '<FieldAccess .assoc>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:7857
--# expr-start 4162 '<FieldAccess .actual>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:7857
Fld_23 := Item_148.Actual;
--# expr-done 4162
Fld_24 := Fld_23.Assoc;
--# expr-done 4163
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_24.Node, E_Info => Fld_24.Info);
--# end
--# expr-done 4164
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 4165
--# expr-start 4171 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:7858
--# expr-start 4169 '<FieldAccess .type_var>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:7858
--# expr-start 4168 '<FieldAccess .expr>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:7858
--# expr-start 4167 '<FieldAccess .assoc>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:7858
--# expr-start 4166 '<FieldAccess .actual>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:7858
Fld_27 := Item_148.Actual;
--# expr-done 4166
Fld_28 := Fld_27.Assoc;
--# expr-done 4167
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_28.Node, E_Info => Fld_28.Info);
--# end
--# expr-done 4168
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29.Node);
--# expr-done 4169
Fld_30.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_30.all);
Eq_Node.Refs.Destroy (Fld_30.all);
--# expr-start 4170 '<FieldAccess .expr_type>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:7858
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_31 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_2.Node, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 4170
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
Bind_Result_1 := Bind_Default_566.Create (Fld_30, Cast_Result_7, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7858") else null)); 
--# expr-done 4171
And_Pred_2 := Logic_And (Fld_26, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7857") else null)); 
--# expr-done 4172
Let_Result_4 := And_Pred_2; Inc_Ref (Let_Result_4);
--# end
Scope_Result_6 := Let_Result_4;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2187;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Type_Decl | Ada_Abstract_Subp_Decl | Ada_Subp_Decl .. Ada_Number_Decl | Ada_Package_Renaming_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_11; 
--# bind _ Match_3
Logic_True_4 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7861") else null)); 
Let_Result_5 := Logic_True_4; Inc_Ref (Let_Result_5);
--# end
Scope_Result_7 := Let_Result_5;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2188;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 4173
--# expr-start 4175 '<FieldAccess .name>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:7862
--# expr-start 4174 '<FieldAccess .actual>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:7862
Fld_32 := Item_148.Actual;
--# expr-done 4174
Fld_33 := Fld_32.Name;
--# expr-done 4175
Var_Expr_2 := Fld_33;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 4182 '<Bind>' Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:7864
--# expr-start 4176 '<FieldAccess .ref_var>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:7864
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_34 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr_2);
--# end
--# expr-done 4176
Fld_34.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_34.all);
Eq_Node.Refs.Destroy (Fld_34.all);
--# expr-start 4181 '<FieldAccess .basic_decl>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:7865
--# expr-start 4180 '<as_bare_entity(<FieldAccess .node>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7865
--# expr-start 4179 '<FieldAccess .node>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:7865
--# expr-start 4178 '<FieldAccess .name>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:7865
--# expr-start 4177 '<FieldAccess .formal>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:7865
Fld_35 := Item_148.Formal;
--# expr-done 4177
Fld_36 := Fld_35.Name;
--# expr-done 4178
Fld_37 := Fld_36.Node;
--# expr-done 4179
Is_Null := Fld_37 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Defining_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_37); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 4180
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_38 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 4181
      Cast_Result_12 := Create_Internal_Entity
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
Bind_Result_2 := Bind_Default_Default.Create (Fld_34, Cast_Result_12, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7864") else null)); 
--# expr-done 4182
Scope_Result_8 := Bind_Result_2;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2189;
   Result_Var_2 := Scope_Result_8;
else
   Logic_True_5 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7866") else null)); 
   Result_Var_2 := Logic_True_5;
end if;
      Inc_Ref (Result_Var_2);
And_Pred_3 := Logic_And (Match_Result_1, Result_Var_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7834") else null)); 
--# expr-done 4183
Let_Result_6 := And_Pred_3; Inc_Ref (Let_Result_6);
--# end
--# expr-done 4184
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2183;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4185
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7829") else null)); 
--# expr-done 4186
   If_Result_2 := Logic_Boolean_Op;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 4187
And_Pred_4 := Logic_And (And_Pred, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7820") else null)); 
--# expr-done 4188
      Property_Result := And_Pred_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_699;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_699;
               Finalizer_Scope_2180;
               Finalizer_Scope_2181;
               Finalizer_Scope_2182;
               Finalizer_Scope_2183;
               Finalizer_Scope_2184;
               Finalizer_Scope_2185;
               Finalizer_Scope_2186;
               Finalizer_Scope_2187;
               Finalizer_Scope_2188;
               Finalizer_Scope_2189;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Instantiation_P_Xref_Equation;
--# end
function Generic_Instantiation_P_Instantiation_Bindings
  (E : Entity
  ) return Internal_Inner_Env_Assoc_Array_Access
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Internal_Inner_Env_Assoc_Array_Access :=
      Generic_Instantiation_P_Instantiation_Bindings (E.Node, E_Info);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Generic_Package_Instantiation
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Package_Instantiation := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_79 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8043" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn;
      procedure Generic_Package_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_77 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_78 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_80 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:8048");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_81 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Instantiation_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_82 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_83 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_85 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_84 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_87 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_86 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Instantiation
        (Self : Bare_Generic_Package_Instantiation
         ; Generic_Package_Instantiation_F_Name : Bare_Defining_Name
         ; Generic_Package_Instantiation_F_Generic_Pkg_Name : Bare_Name
         ; Generic_Package_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Package_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Package_Instantiation_F_Name := Generic_Package_Instantiation_F_Name;
            Self.Generic_Package_Instantiation_F_Generic_Pkg_Name := Generic_Package_Instantiation_F_Generic_Pkg_Name;
            Self.Generic_Package_Instantiation_F_Params := Generic_Package_Instantiation_F_Params;
            Self.Generic_Package_Instantiation_F_Aspects := Generic_Package_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Instantiation;
   function Generic_Package_Instantiation_F_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Name;
   end;
   function Generic_Package_Instantiation_F_Generic_Pkg_Name
     (Node : Bare_Generic_Package_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Package_Instantiation_F_Generic_Pkg_Name;
   end;
   function Generic_Package_Instantiation_F_Params
     (Node : Bare_Generic_Package_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Package_Instantiation_F_Params;
   end;
--# property-start GenericPackageInstantiation.generic_entity_name /usr/src/packages/BUILD/ada/ast.py:7956
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4189 '<FieldAccess .generic_pkg_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7956
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Generic_Pkg_Name, Info => Ent.Info);
--# expr-done 4189
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericPackageInstantiation.generic_inst_params /usr/src/packages/BUILD/ada/ast.py:7957
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4190 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:7957
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Package_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 4190
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericPackageInstantiation.designated_package /usr/src/packages/BUILD/ada/ast.py:7960
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Package
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Package_Decl
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_1485;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Package_Decl;
Fld_2 : Bare_Generic_Package_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Base_Package_Decl;
New_Struct_1 : Internal_Entity_Base_Package_Decl;
Scope_Result_1 : Internal_Entity_Base_Package_Decl;
Result_Var_1 : Internal_Entity_Base_Package_Decl;
Let_Result : Internal_Entity_Base_Package_Decl;
            procedure Finalizer_Scope_1485 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1485;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4211 '<Block>' Let_Result None
--# scope-start
--# expr-start 4191 '<FieldAccess .nonbound_generic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:7961
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 4191
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 4210 '<New InternalEntityBasePackageDecl>' New_Struct_1 None
--# expr-start 4206 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7964
--# expr-start 4193 '<FieldAccess .from_rebound>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7981
--# expr-start 4192 '<FieldAccess .info>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7981
Fld_13 := Var_Expr.Info;
--# expr-done 4192
Fld_14 := Fld_13.From_Rebound;
--# expr-done 4193
--# expr-start 4195 '<FieldAccess .md>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7965
--# expr-start 4194 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7965
Fld_3 := Var_Expr.Info;
--# expr-done 4194
Fld_4 := Fld_3.MD;
--# expr-done 4195
--# expr-start 4205 '<append_rebinding(<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>, <FieldAccess .children_env>, <FieldAccess .instantiation_env>)>' Rebinding_1 /usr/src/packages/BUILD/ada/ast.py:7968
--# expr-start 4201 '<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>' Rebinding /usr/src/packages/BUILD/ada/ast.py:7968
--# expr-start 4197 '<FieldAccess .rebindings>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7968
--# expr-start 4196 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7968
Fld_5 := Ent.Info;
--# expr-done 4196
Fld_6 := Fld_5.Rebindings;
--# expr-done 4197
--# expr-start 4200 '<FieldAccess .rebindings>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7971
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 4199 '<FieldAccess .info>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7971
--# expr-start 4198 '<FieldAccess .decl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7971
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4198
Fld_8 := Fld_7.Info;
--# expr-done 4199
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 4200
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 4201
--# expr-start 4203 '<FieldAccess .children_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7979
--# expr-start 4202 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7979
Fld_10 := Var_Expr.Node;
--# expr-done 4202
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 4203
--# expr-start 4204 '<FieldAccess .instantiation_env>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7980
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 4204
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 4205
New_Struct := (From_Rebound => Fld_14, MD => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 4206
--# expr-start 4209 '<FieldAccess .package_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7963
--# expr-start 4208 '<Cast to GenericPackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7963
--# expr-start 4207 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7963
Fld_1 := Var_Expr.Node;
--# expr-done 4207
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4208
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Generic_Package_Decl_F_Package_Decl;
--# expr-done 4209
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 4210
Scope_Result_1 := New_Struct_1;
   --# end
      Finalizer_Scope_1485;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Package_Decl;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 4211
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1485;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Designated_Package;
--# end
--# property-start GenericPackageInstantiation.designated_generic_decl /usr/src/packages/BUILD/ada/ast.py:7986
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4214 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7987
--# expr-start 4213 '<parent(<FieldAccess .designated_package>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7987
--# expr-start 4212 '<FieldAccess .designated_package>' Fld /usr/src/packages/BUILD/ada/ast.py:7987
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4212
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4213
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4214
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericPackageInstantiation.defining_env_impl /usr/src/packages/BUILD/ada/ast.py:7991
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env_Impl
  (Node : Bare_Generic_Package_Instantiation
      ; Inst_From_Formal : Boolean
         := False
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind inst_from_formal Inst_From_Formal
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_709;
      Block_Var : Internal_Entity_Base_Package_Decl;
Fld : Internal_Entity_Base_Package_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity;
Fld_3 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
If_Result : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Fld_6 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_709 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_709;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4227 '<Block>' Let_Result None
--# scope-start
--# expr-start 4215 '<FieldAccess .designated_package>' Fld /usr/src/packages/BUILD/ada/ast.py:8000
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.designated_package
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Designated_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4215
Block_Var := Fld; 
--# bind dp Block_Var
--# expr-start 4226 '<env_group(<ArrayLiteral>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:8001
--# expr-start 4225 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:8001
--# expr-start 4223 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:8002
--# expr-start 4216 '<FieldAccess .is_formal>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8003
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_formal
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Self);
--# end
--# expr-done 4216
if Fld_5 then
   If_Result := True;
else
   If_Result := Inst_From_Formal;
end if;
if If_Result then
   --# expr-start 4221 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:8004
--# expr-start 4220 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:8004
--# expr-start 4217 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8004
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 4217
--# expr-start 4219 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8004
--# expr-start 4218 '<parent(<Var dp>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8004
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 4218
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4219
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_1, 2 => Fld_3)); 
--# expr-done 4220
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 4221
   If_Result_1 := Group_Env;
else
   --# expr-start 4222 '<FieldAccess .children_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8005
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 4222
   If_Result_1 := Fld_4;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4223
--# expr-start 4224 '<FieldAccess .children_env>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8017
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4224
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => If_Result_1, 2 => Fld_6)); 
--# expr-done 4225
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 4226
Let_Result := Group_Env_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4227
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_709;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_709;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Env_Impl;
--# end
--# property-start GenericPackageInstantiation.defining_env /usr/src/packages/BUILD/ada/ast.py:8021
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Env
  (Node : Bare_Generic_Package_Instantiation
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_710;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_710 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_710;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4229 '<Block>' Let_Result None
--# scope-start
--# expr-start 4228 '<FieldAccess .defining_env_impl>' Fld /usr/src/packages/BUILD/ada/ast.py:8022
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4228
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4229
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_710;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_710;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Env;
--# end
--# property-start GenericPackageInstantiation.initial_env_name /usr/src/packages/BUILD/ada/ast.py:8025
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Initial_Env_Name
  (Node : Bare_Generic_Package_Instantiation
  )
   return Symbol_Type
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4233 '<Block>' Let_Result None
--# scope-start
--# expr-start 4232 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:8026
--# expr-start 4230 '<FieldAccess .is_library_item>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8027
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_library_item'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Library_Item (Node => Self);
--# end
--# expr-done 4230
if Fld_1 then
   --# expr-start 4231 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8028
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4231
   If_Result := Fld;
else
   If_Result := null;
end if;
--# expr-done 4232
Let_Result := If_Result; 
--# end
--# expr-done 4233
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Initial_Env_Name;
--# end
--# property-start GenericPackageInstantiation.env_names /usr/src/packages/BUILD/ada/ast.py:8033
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Env_Names
  (Node : Bare_Generic_Package_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_712;
            procedure Finalizer_Scope_2190;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_712 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_712;
            procedure Finalizer_Scope_2190 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2190;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4236 '<Block>' Let_Result None
--# scope-start
--# expr-start 4234 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8034
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4234
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 4235 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:8035
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 4235
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2190;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4236
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_712;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_712;
               Finalizer_Scope_2190;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Env_Names;
--# end
--# property-start GenericPackageInstantiation.defining_names /usr/src/packages/BUILD/ada/ast.py:8038
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_713;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_713 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_713;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4237 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:8038
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Instantiation_F_Name, Info => Ent.Info);
--# expr-done 4237
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_713;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_713;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Instantiation_P_Defining_Names;
--# end
--# property-start GenericPackageInstantiation._env_do_77 /usr/src/packages/BUILD/ada/ast.py:8041
pragma Warnings (Off, "is not referenced");
 function Env_Do_77
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4238 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8041
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4238
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_77;
--# end
--# property-start GenericPackageInstantiation._initial_env_name_78 /usr/src/packages/BUILD/ada/ast.py:8044
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_78
  (Node : Bare_Generic_Package_Instantiation
  )
   return Symbol_Type
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4239 '<FieldAccess .initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8044
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.initial_env_name
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4239
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_78;
--# end
--# property-start GenericPackageInstantiation._initial_env_79 /usr/src/packages/BUILD/ada/ast.py:8045
pragma Warnings (Off, "is not referenced");
 function Initial_Env_79
  (Node : Bare_Generic_Package_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1249;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1249 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1249;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4240 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8045
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4240
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1249;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1249;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_79;
--# end
--# property-start GenericPackageInstantiation._env_mappings_80 /usr/src/packages/BUILD/ada/ast.py:8048
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_80
  (Node : Bare_Generic_Package_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Instantiation :=
     Internal_Entity_Generic_Package_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1250;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Package_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1250 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1250;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4242 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8048
--# expr-start 4241 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8048
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4241
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4242
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1250;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1250;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_80;
--# end
--# property-start GenericPackageInstantiation._env_trans_parent_81 /usr/src/packages/BUILD/ada/ast.py:8040
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_81
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_81;
--# end
--# property-start GenericPackageInstantiation._env_names_82 /usr/src/packages/BUILD/ada/ast.py:8050
pragma Warnings (Off, "is not referenced");
 function Env_Names_82
  (Node : Bare_Generic_Package_Instantiation
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1252;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1252 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1252;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4243 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:8050
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.env_names
Fld := Libadalang.Implementation.Generic_Package_Instantiation_P_Env_Names (Node => Self);
--# end
--# expr-done 4243
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1252;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1252;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_82;
--# end
--# property-start GenericPackageInstantiation._env_do_83 /usr/src/packages/BUILD/ada/ast.py:8052
pragma Warnings (Off, "is not referenced");
 function Env_Do_83
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1253;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1253 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1253;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4244 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8052
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4244
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1253;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1253;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_83;
--# end
--# property-start GenericPackageInstantiation._ref_env_nodes_84 /usr/src/packages/BUILD/ada/ast.py:8054
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_84
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1254;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1254 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1254;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4245 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8054
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4245
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1254;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1254;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_84;
--# end
--# property-start GenericPackageInstantiation._ref_cond_85 /usr/src/packages/BUILD/ada/ast.py:8056
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_85
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4247 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8056
--# expr-start 4246 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8056
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4246
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4247
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_85;
--# end
--# property-start GenericPackageInstantiation._ref_env_nodes_86 /usr/src/packages/BUILD/ada/ast.py:8059
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_86
  (Node : Bare_Generic_Package_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1256;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1256 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1256;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4248 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8059
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4248
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1256;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1256;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_86;
--# end
--# property-start GenericPackageInstantiation._ref_cond_87 /usr/src/packages/BUILD/ada/ast.py:8061
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_87
  (Node : Bare_Generic_Package_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Package_Instantiation := Bare_Generic_Package_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4250 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8061
--# expr-start 4249 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8061
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4249
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4250
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_87;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Instantiation
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Subp_Instantiation := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_69 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:7918" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Instantiation_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Instantiation;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_67 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_68 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            Generic_Subp_Instantiation_P_Designated_Subp'Access;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_70 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7923");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_71 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Instantiation_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_72 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_74 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_73 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_76 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_75 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Instantiation
        (Self : Bare_Generic_Subp_Instantiation
         ; Generic_Subp_Instantiation_F_Overriding : Bare_Overriding_Node
         ; Generic_Subp_Instantiation_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Instantiation_F_Subp_Name : Bare_Defining_Name
         ; Generic_Subp_Instantiation_F_Generic_Subp_Name : Bare_Name
         ; Generic_Subp_Instantiation_F_Params : Bare_Assoc_List
         ; Generic_Subp_Instantiation_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Generic_Instantiation
              (Self);
            Self.Generic_Subp_Instantiation_F_Overriding := Generic_Subp_Instantiation_F_Overriding;
            Self.Generic_Subp_Instantiation_F_Kind := Generic_Subp_Instantiation_F_Kind;
            Self.Generic_Subp_Instantiation_F_Subp_Name := Generic_Subp_Instantiation_F_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Generic_Subp_Name := Generic_Subp_Instantiation_F_Generic_Subp_Name;
            Self.Generic_Subp_Instantiation_F_Params := Generic_Subp_Instantiation_F_Params;
            Self.Generic_Subp_Instantiation_F_Aspects := Generic_Subp_Instantiation_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Instantiation;
   function Generic_Subp_Instantiation_F_Overriding
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Overriding_Node
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Overriding;
   end;
   function Generic_Subp_Instantiation_F_Kind
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Kind;
   end;
   function Generic_Subp_Instantiation_F_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Generic_Subp_Name
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Generic_Subp_Name;
   end;
   function Generic_Subp_Instantiation_F_Params
     (Node : Bare_Generic_Subp_Instantiation) return Bare_Assoc_List
   is
   begin
         return Node.Generic_Subp_Instantiation_F_Params;
   end;
--# property-start GenericSubpInstantiation.defining_names /usr/src/packages/BUILD/ada/ast.py:7885
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Defining_Names
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_700;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_700 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_700;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4251 '<FieldAccess .subp_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7885
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Subp_Name, Info => Ent.Info);
--# expr-done 4251
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_700;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_700;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Defining_Names;
--# end
--# property-start GenericSubpInstantiation.generic_entity_name /usr/src/packages/BUILD/ada/ast.py:7887
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Entity_Name
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4252 '<FieldAccess .generic_subp_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7887
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Instantiation_F_Generic_Subp_Name, Info => Ent.Info);
--# expr-done 4252
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Generic_Entity_Name;
--# end
--# property-start GenericSubpInstantiation.generic_inst_params /usr/src/packages/BUILD/ada/ast.py:7888
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Generic_Inst_Params
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4253 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:7888
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Generic_Subp_Instantiation_F_Params, Info => Ent.Info);
--# expr-done 4253
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Generic_Inst_Params;
--# end
--# property-start GenericSubpInstantiation.designated_subp /usr/src/packages/BUILD/ada/ast.py:7891
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Subp
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_1488;
      Var_Expr : Internal_Entity_Generic_Decl;
Var_Expr_1 : Internal_Entity_Generic_Decl;
Fld : Internal_Entity_Generic_Decl;
Fld_1 : Bare_Generic_Decl;
Cast_Result : Bare_Generic_Subp_Decl;
Fld_2 : Bare_Generic_Subp_Internal;
Fld_3 : Internal_Entity_Info;
Fld_4 : Internal_Metadata;
Fld_5 : Internal_Entity_Info;
Fld_6 : Env_Rebindings;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Info;
Scope_Result : Internal_Entity_Info;
Result_Var : Internal_Entity_Info;
Fld_9 : Env_Rebindings;
Rebinding : Env_Rebindings;
Fld_10 : Bare_Generic_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Fld_12 : Lexical_Env := Empty_Env;
Rebinding_1 : Env_Rebindings;
Fld_13 : Internal_Entity_Info;
Fld_14 : Boolean;
New_Struct : Internal_Entity_Info;
Cast_Result_1 : Bare_Basic_Subp_Decl;
New_Struct_1 : Internal_Entity_Basic_Subp_Decl;
Cast_Result_2 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_1488 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_1488;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4275 '<Block>' Let_Result None
--# scope-start
--# expr-start 4254 '<FieldAccess .nonbound_generic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:7895
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.nonbound_generic_decl
Fld := Libadalang.Implementation.Generic_Instantiation_P_Nonbound_Generic_Decl (Node => Self);
--# end
--# expr-done 4254
Var_Expr := Fld;
if Var_Expr /= No_Entity_Generic_Decl then
   --# scope-start
   --# bind p Var_Expr
--# expr-start 4274 '<Cast to AdaNode.entity>' Cast_Result_2 None
--# expr-start 4273 '<New InternalEntityBasicSubpDecl>' New_Struct_1 None
--# expr-start 4269 '<New InternalEntityInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7898
--# expr-start 4256 '<FieldAccess .from_rebound>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7906
--# expr-start 4255 '<FieldAccess .info>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7906
Fld_13 := Var_Expr.Info;
--# expr-done 4255
Fld_14 := Fld_13.From_Rebound;
--# expr-done 4256
--# expr-start 4258 '<FieldAccess .md>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7899
--# expr-start 4257 '<FieldAccess .info>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7899
Fld_3 := Var_Expr.Info;
--# expr-done 4257
Fld_4 := Fld_3.MD;
--# expr-done 4258
--# expr-start 4268 '<append_rebinding(<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>, <FieldAccess .children_env>, <FieldAccess .instantiation_env>)>' Rebinding_1 /usr/src/packages/BUILD/ada/ast.py:7900
--# expr-start 4264 '<concat_rebindings(<FieldAccess .rebindings>, <FieldAccess .rebindings>)>' Rebinding /usr/src/packages/BUILD/ada/ast.py:7900
--# expr-start 4260 '<FieldAccess .rebindings>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7900
--# expr-start 4259 '<FieldAccess .info>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7900
Fld_5 := Ent.Info;
--# expr-done 4259
Fld_6 := Fld_5.Rebindings;
--# expr-done 4260
--# expr-start 4263 '<FieldAccess .rebindings>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7902
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Generic_Decl then
   --# scope-start
--# expr-start 4262 '<FieldAccess .info>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7902
--# expr-start 4261 '<FieldAccess .decl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7902
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4261
Fld_8 := Fld_7.Info;
--# expr-done 4262
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Info;
end if;
Fld_9 := Result_Var.Rebindings;
--# expr-done 4263
Rebinding := AST_Envs.Combine (Fld_6, Fld_9); 
--# expr-done 4264
--# expr-start 4266 '<FieldAccess .children_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7904
--# expr-start 4265 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7904
Fld_10 := Var_Expr.Node;
--# expr-done 4265
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Fld_10);
--# expr-done 4266
--# expr-start 4267 '<FieldAccess .instantiation_env>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7904
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericInstantiation.instantiation_env
Fld_12 := Libadalang.Implementation.Generic_Instantiation_F_Instantiation_Env (Node => Self);
--# end
--# expr-done 4267
Rebinding_1 := AST_Envs.Append_Rebinding (Rebinding, Fld_11, Fld_12); 
--# expr-done 4268
New_Struct := (From_Rebound => Fld_14, MD => Fld_4, Rebindings => Rebinding_1); 
--# expr-done 4269
--# expr-start 4272 '<FieldAccess .subp_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7897
--# expr-start 4271 '<Cast to GenericSubpDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7897
--# expr-start 4270 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7897
Fld_1 := Var_Expr.Node;
--# expr-done 4270
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Generic_Subp_Decl_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4271
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Generic_Subp_Decl_F_Subp_Decl;
--# expr-done 4272
      Cast_Result_1 := Fld_2;
New_Struct_1 := (Info => New_Struct, Node => Cast_Result_1); 
--# expr-done 4273
      Cast_Result_2 := Create_Internal_Entity
        (Node => New_Struct_1.Node,
         Info => New_Struct_1.Info);
--# expr-done 4274
Scope_Result_1 := Cast_Result_2;
   --# end
      Finalizer_Scope_1488;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 4275
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1488;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Designated_Subp;
--# end
--# property-start GenericSubpInstantiation.designated_generic_decl /usr/src/packages/BUILD/ada/ast.py:7911
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Instantiation_P_Designated_Generic_Decl
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4278 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7912
--# expr-start 4277 '<parent(<FieldAccess .designated_subp>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7912
--# expr-start 4276 '<FieldAccess .designated_subp>' Fld /usr/src/packages/BUILD/ada/ast.py:7912
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericSubpInstantiation.designated_subp
Fld := Libadalang.Implementation.Generic_Subp_Instantiation_P_Designated_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4276
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4277
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4278
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Instantiation_P_Designated_Generic_Decl;
--# end
--# property-start GenericSubpInstantiation._env_do_67 /usr/src/packages/BUILD/ada/ast.py:7916
pragma Warnings (Off, "is not referenced");
 function Env_Do_67
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4279 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:7916
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4279
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_67;
--# end
--# property-start GenericSubpInstantiation._initial_env_name_68 /usr/src/packages/BUILD/ada/ast.py:7919
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_68
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Symbol_Type
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4280 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7919
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4280
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_68;
--# end
--# property-start GenericSubpInstantiation._initial_env_69 /usr/src/packages/BUILD/ada/ast.py:7920
pragma Warnings (Off, "is not referenced");
 function Initial_Env_69
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1239;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1239 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1239;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4281 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7920
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4281
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1239;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1239;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_69;
--# end
--# property-start GenericSubpInstantiation._env_mappings_70 /usr/src/packages/BUILD/ada/ast.py:7923
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_70
  (Node : Bare_Generic_Subp_Instantiation
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Instantiation :=
     Internal_Entity_Generic_Subp_Instantiation'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1240;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Subp_Instantiation;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1240 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1240;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4283 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7923
--# expr-start 4282 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:7924
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4282
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4283
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1240;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1240;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_70;
--# end
--# property-start GenericSubpInstantiation._env_trans_parent_71 /usr/src/packages/BUILD/ada/ast.py:7915
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_71
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_71;
--# end
--# property-start GenericSubpInstantiation._env_do_72 /usr/src/packages/BUILD/ada/ast.py:7931
pragma Warnings (Off, "is not referenced");
 function Env_Do_72
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1242;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1242 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1242;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4284 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:7931
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4284
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1242;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1242;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_72;
--# end
--# property-start GenericSubpInstantiation._ref_env_nodes_73 /usr/src/packages/BUILD/ada/ast.py:7934
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_73
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1243;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1243 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1243;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4285 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:7934
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4285
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1243;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1243;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_73;
--# end
--# property-start GenericSubpInstantiation._ref_cond_74 /usr/src/packages/BUILD/ada/ast.py:7936
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_74
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4287 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:7936
--# expr-start 4286 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7936
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4286
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4287
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_74;
--# end
--# property-start GenericSubpInstantiation._ref_env_nodes_75 /usr/src/packages/BUILD/ada/ast.py:7939
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_75
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1245;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1245 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1245;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4288 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:7939
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4288
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1245;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1245;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_75;
--# end
--# property-start GenericSubpInstantiation._ref_cond_76 /usr/src/packages/BUILD/ada/ast.py:7941
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_76
  (Node : Bare_Generic_Subp_Instantiation
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Instantiation := Bare_Generic_Subp_Instantiation (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4290 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:7941
--# expr-start 4289 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7941
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4289
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4290
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_76;
--# end
function Generic_Subp_Instantiation_P_Designated_Subp
  (E : Entity
  ) return Internal_Entity
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Internal_Entity :=
      Generic_Subp_Instantiation_P_Designated_Subp (E.Node, E_Info);
begin
      return (Node => Result.Node, Info => Result.Info);
end;
   --
   --  Primitives for Bare_Generic_Renaming_Decl
   --
--# property-start '[dispatcher]GenericRenamingDecl.renaming_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Generic_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Generic_Renaming_Decl (Self.Kind) is
               when Ada_Generic_Package_Renaming_Decl =>
                  --# property-call-start GenericPackageRenamingDecl.renaming_name
                  Property_Result := Generic_Package_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Generic_Subp_Renaming_Decl =>
                  --# property-call-start GenericSubpRenamingDecl.renaming_name
                  Property_Result := Generic_Subp_Renaming_Decl_P_Renaming_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Generic_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericRenamingDecl.resolve /usr/src/packages/BUILD/ada/ast.py:8158
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Resolve
  (Node : Bare_Generic_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Generic_Decl
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Renaming_Decl :=
     Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_721;
      Var_Expr : Internal_Entity;
Match_0 : Internal_Entity_Generic_Decl;
Match_1 : Internal_Entity_Generic_Renaming_Decl;
Match_2 : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_3 : Internal_Entity_Generic_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Generic_Decl;
Let_Result : Internal_Entity_Generic_Decl;
Scope_Result : Internal_Entity_Generic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Renaming_Decl;
Let_Result_1 : Internal_Entity_Generic_Decl;
Scope_Result_1 : Internal_Entity_Generic_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Generic_Decl;
Scope_Result_2 : Internal_Entity_Generic_Decl;
Match_Result : Internal_Entity_Generic_Decl;
Scope_Result_3 : Internal_Entity_Generic_Decl;
Result_Var : Internal_Entity_Generic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Generic_Decl;
            procedure Finalizer_Scope_721 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_721;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4297 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <at(<FieldAccess .env_elements>, 0)>: <Var Var_Expr> <Match>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:8158
--# expr-start 4291 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8159
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4291
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4294 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:8160
--# expr-start 4293 '<FieldAccess .env_elements>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8160
--# expr-start 4292 '<FieldAccess .renaming_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8160
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4292
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_2 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_1.Node, Env => Bound_Env, E_Info => Fld_1.Info);
--# end
--# expr-done 4293
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 4294
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 4296 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:8160
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Var_Expr; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Generic_Package_Decl .. Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Generic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind gd Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Generic_Package_Renaming_Decl .. Ada_Generic_Subp_Renaming_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Generic_Renaming_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind grd Match_1
--# expr-start 4295 '<FieldAccess .resolve>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8162
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld_3 := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4295
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Generic_Subp_Instantiation | Ada_Label_Decl .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Generic_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4296
Scope_Result_3 := Match_Result;
   --# end
   Result_Var := Scope_Result_3;
else
   Result_Var := No_Entity_Generic_Decl;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 4297
      Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_721;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_721;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Resolve;
--# end
--# property-start GenericRenamingDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:8169
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Generic_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start GenericRenamingDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:8170
pragma Warnings (Off, "is not referenced");
 function Generic_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Generic_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Generic_Renaming_Decl := Bare_Generic_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Renaming_Decl :=
     Internal_Entity_Generic_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_723;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_723 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_723;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4299 '<FieldAccess .xref_no_overloading>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8170
--# expr-start 4298 '<FieldAccess .renaming_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8170
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericRenamingDecl.renaming_name'
Fld := Libadalang.Implementation.Dispatcher_Generic_Renaming_Decl_P_Renaming_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4298
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4299
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_723;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_723;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Renaming_Decl_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Generic_Package_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Package_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_100 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8188" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_98 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_99 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_101 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:8193");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_102 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_103 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_105 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_104 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_107 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_106 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Package_Renaming_Decl
        (Self : Bare_Generic_Package_Renaming_Decl
         ; Generic_Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Package_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Package_Renaming_Decl_F_Name := Generic_Package_Renaming_Decl_F_Name;
            Self.Generic_Package_Renaming_Decl_F_Renames := Generic_Package_Renaming_Decl_F_Renames;
            Self.Generic_Package_Renaming_Decl_F_Aspects := Generic_Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Package_Renaming_Decl;
   function Generic_Package_Renaming_Decl_F_Name
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Name;
   end;
   function Generic_Package_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Package_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Package_Renaming_Decl_F_Renames;
   end;
--# property-start GenericPackageRenamingDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8181
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_724;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_724 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_724;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4300 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:8181
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 4300
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_724;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_724;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericPackageRenamingDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:8182
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Generic_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_725;
      Fld : Internal_Entity_Generic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_725 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_725;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4302 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8182
--# expr-start 4301 '<FieldAccess .resolve>' Fld /usr/src/packages/BUILD/ada/ast.py:8182
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericRenamingDecl.resolve
Fld := Libadalang.Implementation.Generic_Renaming_Decl_P_Resolve (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4301
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4302
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_725;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_725;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start GenericPackageRenamingDecl.renaming_name /usr/src/packages/BUILD/ada/ast.py:8183
pragma Warnings (Off, "is not referenced");
 function Generic_Package_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4303 '<FieldAccess .renames>' Fld /usr/src/packages/BUILD/ada/ast.py:8183
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4303
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Package_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericPackageRenamingDecl._env_do_98 /usr/src/packages/BUILD/ada/ast.py:8186
pragma Warnings (Off, "is not referenced");
 function Env_Do_98
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4304 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4304
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_98;
--# end
--# property-start GenericPackageRenamingDecl._initial_env_name_99 /usr/src/packages/BUILD/ada/ast.py:8189
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_99
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Symbol_Type
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4305 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8189
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4305
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_99;
--# end
--# property-start GenericPackageRenamingDecl._initial_env_100 /usr/src/packages/BUILD/ada/ast.py:8190
pragma Warnings (Off, "is not referenced");
 function Initial_Env_100
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1270;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1270 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1270;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4306 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8190
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4306
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1270;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1270;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_100;
--# end
--# property-start GenericPackageRenamingDecl._env_mappings_101 /usr/src/packages/BUILD/ada/ast.py:8193
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_101
  (Node : Bare_Generic_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Package_Renaming_Decl :=
     Internal_Entity_Generic_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1271;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1271 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1271;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4308 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8193
--# expr-start 4307 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8194
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4307
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4308
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1271;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1271;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_101;
--# end
--# property-start GenericPackageRenamingDecl._env_trans_parent_102 /usr/src/packages/BUILD/ada/ast.py:8185
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_102
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_102;
--# end
--# property-start GenericPackageRenamingDecl._env_do_103 /usr/src/packages/BUILD/ada/ast.py:8200
pragma Warnings (Off, "is not referenced");
 function Env_Do_103
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1273;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1273 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1273;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4309 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8200
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4309
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1273;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1273;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_103;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_104 /usr/src/packages/BUILD/ada/ast.py:8203
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_104
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1274;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1274 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1274;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4310 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4310
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1274;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1274;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_104;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_105 /usr/src/packages/BUILD/ada/ast.py:8205
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_105
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4312 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8205
--# expr-start 4311 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8205
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4311
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4312
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_105;
--# end
--# property-start GenericPackageRenamingDecl._ref_env_nodes_106 /usr/src/packages/BUILD/ada/ast.py:8208
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_106
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1276;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1276 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1276;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4313 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8208
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4313
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1276;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1276;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_106;
--# end
--# property-start GenericPackageRenamingDecl._ref_cond_107 /usr/src/packages/BUILD/ada/ast.py:8210
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_107
  (Node : Bare_Generic_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Package_Renaming_Decl := Bare_Generic_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4315 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8210
--# expr-start 4314 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8210
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4314
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4315
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_107;
--# end
   --
   --  Primitives for Bare_Generic_Subp_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Generic_Subp_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_110 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8230" & ")";
         end if;
      return Initial_Env;
   end Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Generic_Subp_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Generic_Subp_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_108 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_109 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_111 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:8235");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_112 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Generic_Subp_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_113 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_115 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_114 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_117 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_116 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Generic_Subp_Renaming_Decl
        (Self : Bare_Generic_Subp_Renaming_Decl
         ; Generic_Subp_Renaming_Decl_F_Kind : Bare_Subp_Kind
         ; Generic_Subp_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Generic_Subp_Renaming_Decl_F_Renames : Bare_Name
         ; Generic_Subp_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Generic_Subp_Renaming_Decl_F_Kind := Generic_Subp_Renaming_Decl_F_Kind;
            Self.Generic_Subp_Renaming_Decl_F_Name := Generic_Subp_Renaming_Decl_F_Name;
            Self.Generic_Subp_Renaming_Decl_F_Renames := Generic_Subp_Renaming_Decl_F_Renames;
            Self.Generic_Subp_Renaming_Decl_F_Aspects := Generic_Subp_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Generic_Subp_Renaming_Decl;
   function Generic_Subp_Renaming_Decl_F_Kind
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Subp_Kind
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Kind;
   end;
   function Generic_Subp_Renaming_Decl_F_Name
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Name;
   end;
   function Generic_Subp_Renaming_Decl_F_Renames
     (Node : Bare_Generic_Subp_Renaming_Decl) return Bare_Name
   is
   begin
         return Node.Generic_Subp_Renaming_Decl_F_Renames;
   end;
--# property-start GenericSubpRenamingDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8261
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Defining_Names
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_727;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_727 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_727;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4316 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:8261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 4316
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_727;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_727;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Renaming_Decl_P_Defining_Names;
--# end
--# property-start GenericSubpRenamingDecl.renaming_name /usr/src/packages/BUILD/ada/ast.py:8262
pragma Warnings (Off, "is not referenced");
 function Generic_Subp_Renaming_Decl_P_Renaming_Name
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4317 '<FieldAccess .renames>' Fld /usr/src/packages/BUILD/ada/ast.py:8262
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Generic_Subp_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4317
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Generic_Subp_Renaming_Decl_P_Renaming_Name;
--# end
--# property-start GenericSubpRenamingDecl._env_do_108 /usr/src/packages/BUILD/ada/ast.py:8228
pragma Warnings (Off, "is not referenced");
 function Env_Do_108
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4318 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8228
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4318
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_108;
--# end
--# property-start GenericSubpRenamingDecl._initial_env_name_109 /usr/src/packages/BUILD/ada/ast.py:8231
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_109
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Symbol_Type
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4319 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8231
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4319
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_109;
--# end
--# property-start GenericSubpRenamingDecl._initial_env_110 /usr/src/packages/BUILD/ada/ast.py:8232
pragma Warnings (Off, "is not referenced");
 function Initial_Env_110
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Lexical_Env
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1280;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1280 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1280;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4320 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8232
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4320
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1280;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1280;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_110;
--# end
--# property-start GenericSubpRenamingDecl._env_mappings_111 /usr/src/packages/BUILD/ada/ast.py:8235
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_111
  (Node : Bare_Generic_Subp_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Generic_Subp_Renaming_Decl :=
     Internal_Entity_Generic_Subp_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1281;
      Fld : Symbol_Type;
Cast_Expr : Bare_Generic_Subp_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1281 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1281;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4322 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8235
--# expr-start 4321 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8236
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4321
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4322
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1281;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1281;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_111;
--# end
--# property-start GenericSubpRenamingDecl._env_trans_parent_112 /usr/src/packages/BUILD/ada/ast.py:8227
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_112
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_112;
--# end
--# property-start GenericSubpRenamingDecl._env_do_113 /usr/src/packages/BUILD/ada/ast.py:8242
pragma Warnings (Off, "is not referenced");
 function Env_Do_113
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1283;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1283 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1283;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4323 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8242
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4323
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1283;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1283;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_113;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_114 /usr/src/packages/BUILD/ada/ast.py:8245
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_114
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1284;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1284 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1284;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4324 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8245
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4324
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1284;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1284;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_114;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_115 /usr/src/packages/BUILD/ada/ast.py:8247
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_115
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4326 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8247
--# expr-start 4325 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8247
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4325
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4326
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_115;
--# end
--# property-start GenericSubpRenamingDecl._ref_env_nodes_116 /usr/src/packages/BUILD/ada/ast.py:8250
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_116
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1286;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1286 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1286;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4327 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8250
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4327
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1286;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1286;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_116;
--# end
--# property-start GenericSubpRenamingDecl._ref_cond_117 /usr/src/packages/BUILD/ada/ast.py:8252
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_117
  (Node : Bare_Generic_Subp_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Generic_Subp_Renaming_Decl := Bare_Generic_Subp_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4329 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8252
--# expr-start 4328 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8252
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4328
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4329
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_117;
--# end
   --
   --  Primitives for Bare_Label_Decl
   --
      procedure Label_Decl_Pre_Env_Actions
        (Self            : Bare_Label_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_173 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:14913");
         Dec_Ref (Mapping.Dest_Env);
      end;
      end;
      procedure Initialize_Fields_For_Label_Decl
        (Self : Bare_Label_Decl
         ; Label_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Label_Decl_F_Name := Label_Decl_F_Name;
      end Initialize_Fields_For_Label_Decl;
   function Label_Decl_F_Name
     (Node : Bare_Label_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Label_Decl_F_Name;
   end;
--# property-start LabelDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:14911
pragma Warnings (Off, "is not referenced");
 function Label_Decl_P_Defining_Names
  (Node : Bare_Label_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Label_Decl := Bare_Label_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Label_Decl :=
     Internal_Entity_Label_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1117;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1117 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1117;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4330 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:14911
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Label_Decl_F_Name, Info => Ent.Info);
--# expr-done 4330
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1117;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1117;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Label_Decl_P_Defining_Names;
--# end
--# property-start LabelDecl._env_mappings_173 /usr/src/packages/BUILD/ada/ast.py:14913
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_173
  (Node : Bare_Label_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Label_Decl := Bare_Label_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1343;
      Fld : Symbol_Type;
Cast_Expr : Bare_Label_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1343 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1343;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4332 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14913
--# expr-start 4331 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:14913
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4331
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4332
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1343;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1343;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_173;
--# end
   --
   --  Primitives for Bare_Named_Stmt_Decl
   --
      procedure Initialize_Fields_For_Named_Stmt_Decl
        (Self : Bare_Named_Stmt_Decl
         ; Named_Stmt_Decl_F_Name : Bare_Defining_Name
        ) is
      begin
            Self.Named_Stmt_Decl_F_Name := Named_Stmt_Decl_F_Name;
      end Initialize_Fields_For_Named_Stmt_Decl;
   function Named_Stmt_Decl_F_Name
     (Node : Bare_Named_Stmt_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Named_Stmt_Decl_F_Name;
   end;
--# property-start NamedStmtDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:14949
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Names
  (Node : Bare_Named_Stmt_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Named_Stmt_Decl := Bare_Named_Stmt_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Named_Stmt_Decl :=
     Internal_Entity_Named_Stmt_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1120;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_1120 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1120;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4333 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:14949
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Named_Stmt_Decl_F_Name, Info => Ent.Info);
--# expr-done 4333
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1120;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1120;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_Decl_P_Defining_Names;
--# end
--# property-start NamedStmtDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:14950
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_Decl_P_Defining_Env
  (Node : Bare_Named_Stmt_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Named_Stmt_Decl := Bare_Named_Stmt_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1121;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld_1 : Bare_Composite_Stmt;
Fld_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1121 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_1121;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4337 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14950
--# expr-start 4336 '<FieldAccess .stmt>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14950
--# expr-start 4335 '<Cast to NamedStmt>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14950
--# expr-start 4334 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:14950
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4334
   if Fld = null
      or else Fld.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4335
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Cast_Result.Named_Stmt_F_Stmt;
--# expr-done 4336
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => Fld_1);
--# expr-done 4337
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1121;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1121;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_Decl_P_Defining_Env;
--# end
   --
   --  Primitives for Bare_Number_Decl
   --
      procedure Number_Decl_Pre_Env_Actions
        (Self            : Bare_Number_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_48 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7379");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Number_Decl
        (Self : Bare_Number_Decl
         ; Number_Decl_F_Ids : Bare_Defining_Name_List
         ; Number_Decl_F_Expr : Bare_Expr
        ) is
      begin
            Self.Number_Decl_F_Ids := Number_Decl_F_Ids;
            Self.Number_Decl_F_Expr := Number_Decl_F_Expr;
      end Initialize_Fields_For_Number_Decl;
   function Number_Decl_F_Ids
     (Node : Bare_Number_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Number_Decl_F_Ids;
   end;
   function Number_Decl_F_Expr
     (Node : Bare_Number_Decl) return Bare_Expr
   is
   begin
         return Node.Number_Decl_F_Expr;
   end;
--# property-start NumberDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7377
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Defining_Names
  (Node : Bare_Number_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_660;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_149 : Internal_Entity_Defining_Name;
Bare_Item_149 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_660 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_660;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4339 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7377
--# expr-start 4338 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7377
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Number_Decl_F_Ids, Info => Ent.Info);
--# expr-done 4338
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_149 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_149 := Untyped_Item_149; 
Is_Null := Bare_Item_149 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_149); 
   If_Result := As_Entity;
end if;
                  Item_149 := If_Result; 
   --# scope-start
               --# bind id Item_149
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_149;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4339
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_660;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_660;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Defining_Names;
--# end
--# property-start NumberDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:7382
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Expr_Type
  (Node : Bare_Number_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
Fld_2 : Bare_Expr;
Fld_3 : Internal_Entity;
Is_Null : Boolean;
If_Result : Boolean;
Fld_4 : Bare_Expr;
Fld_5 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_6 : Internal_Entity;
Fld_7 : Internal_Entity;
Fld_8 : Boolean;
If_Result_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4355 '<Block>' Let_Result None
--# scope-start
--# expr-start 4345 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:7383
--# expr-start 4342 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:7383
--# expr-start 4341 '<FieldAccess .type_val>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7383
--# expr-start 4340 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7383
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Number_Decl_F_Expr;
--# expr-done 4340
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_3 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_2);
--# end
--# expr-done 4341
Is_Null := Fld_3.Node = null; 
--# expr-done 4342
if Is_Null then
   --# expr-start 4344 '<FieldAccess .resolve_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7384
--# expr-start 4343 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7384
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 4343
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4344
   If_Result := Fld_1;
else
   If_Result := True;
end if;
--# expr-done 4345
Block_Var := If_Result; 
--# bind p Block_Var
--# expr-start 4349 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7387
if Block_Var then
   --# expr-start 4348 '<Cast to BaseTypeDecl.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7388
--# expr-start 4347 '<FieldAccess .type_val>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7388
--# expr-start 4346 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7388
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Number_Decl_F_Expr;
--# expr-done 4346
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_5 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_4);
--# end
--# expr-done 4347
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4348
   If_Result_1 := Cast_Result;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 4349
Block_Var_1 := If_Result_1; 
--# bind typ Block_Var_1
--# expr-start 4354 '<Cast to BaseTypeDecl.entity>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7391
--# expr-start 4353 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:7391
--# expr-start 4350 '<FieldAccess .is_int_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7391
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_8 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4350
if Fld_8 then
   --# expr-start 4351 '<FieldAccess .universal_int_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7392
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_6 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 4351
   If_Result_2 := Fld_6;
else
   --# expr-start 4352 '<FieldAccess .universal_real_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7393
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_real_type
Fld_7 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Type (Node => Self);
--# end
--# expr-done 4352
   If_Result_2 := Fld_7;
end if;
--# expr-done 4353
   if If_Result_2.Node = null
      or else If_Result_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result_2.Node,
         Info => If_Result_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4354
Let_Result := Cast_Result_1; 
--# end
--# expr-done 4355
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Expr_Type;
--# end
--# property-start NumberDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7395
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Entry_Point
  (Node : Bare_Number_Decl
  )
   return Boolean
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Xref_Entry_Point;
--# end
--# property-start NumberDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:7397
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Is_Static_Decl
  (Node : Bare_Number_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Is_Static_Decl;
--# end
--# property-start NumberDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:7399
pragma Warnings (Off, "is not referenced");
 function Number_Decl_P_Xref_Equation
  (Node : Bare_Number_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Number_Decl :=
     Internal_Entity_Number_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_664;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_664 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_664;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4357 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7399
--# expr-start 4356 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7399
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Number_Decl_F_Expr, Info => Ent.Info);
--# expr-done 4356
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4357
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_664;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_664;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Number_Decl_P_Xref_Equation;
--# end
--# property-start NumberDecl._env_mappings_48 /usr/src/packages/BUILD/ada/ast.py:7379
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_48
  (Node : Bare_Number_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Number_Decl := Bare_Number_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1218;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Number_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1218 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1218;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4359 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7379
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4358 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7379
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Number_Decl_F_Ids;
--# expr-done 4358
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 4359
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1218;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1218;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_48;
--# end
   --
   --  Primitives for Bare_Object_Decl
   --
      procedure Object_Decl_Pre_Env_Actions
        (Self            : Bare_Object_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mappings : Internal_Env_Assoc_Array_Access :=
            Libadalang.Implementation.Env_Mappings_49 (Node => 
Self);
      begin
         for Mapping of Mappings.Items loop
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7418");
         end loop;
         Dec_Ref (Mappings);
      end;
      end;
      procedure Initialize_Fields_For_Object_Decl
        (Self : Bare_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Object_Decl_F_Ids := Object_Decl_F_Ids;
            Self.Object_Decl_F_Has_Aliased := Object_Decl_F_Has_Aliased;
            Self.Object_Decl_F_Has_Constant := Object_Decl_F_Has_Constant;
            Self.Object_Decl_F_Mode := Object_Decl_F_Mode;
            Self.Object_Decl_F_Type_Expr := Object_Decl_F_Type_Expr;
            Self.Object_Decl_F_Default_Expr := Object_Decl_F_Default_Expr;
            Self.Object_Decl_F_Renaming_Clause := Object_Decl_F_Renaming_Clause;
            Self.Object_Decl_F_Aspects := Object_Decl_F_Aspects;
      end Initialize_Fields_For_Object_Decl;
   function Object_Decl_F_Ids
     (Node : Bare_Object_Decl) return Bare_Defining_Name_List
   is
   begin
         return Node.Object_Decl_F_Ids;
   end;
   function Object_Decl_F_Has_Aliased
     (Node : Bare_Object_Decl) return Bare_Aliased_Node
   is
   begin
         return Node.Object_Decl_F_Has_Aliased;
   end;
   function Object_Decl_F_Has_Constant
     (Node : Bare_Object_Decl) return Bare_Constant_Node
   is
   begin
         return Node.Object_Decl_F_Has_Constant;
   end;
   function Object_Decl_F_Mode
     (Node : Bare_Object_Decl) return Bare_Mode
   is
   begin
         return Node.Object_Decl_F_Mode;
   end;
   function Object_Decl_F_Type_Expr
     (Node : Bare_Object_Decl) return Bare_Type_Expr
   is
   begin
         return Node.Object_Decl_F_Type_Expr;
   end;
   function Object_Decl_F_Default_Expr
     (Node : Bare_Object_Decl) return Bare_Expr
   is
   begin
         return Node.Object_Decl_F_Default_Expr;
   end;
   function Object_Decl_F_Renaming_Clause
     (Node : Bare_Object_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Object_Decl_F_Renaming_Clause;
   end;
--# property-start ObjectDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7420
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Names
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_665;
      Fld : Internal_Entity_Defining_Name_List;
Fld_1 : Bare_Defining_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Defining_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Defining_Name;
Item_150 : Internal_Entity_Defining_Name;
Bare_Item_150 : Bare_Defining_Name;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_665 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_665;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4361 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7420
--# expr-start 4360 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7420
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name_List (Node => Ent.Node.Object_Decl_F_Ids, Info => Ent.Info);
--# expr-done 4360
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Defining_Name_Array (0);
      else
      declare
         Collection : constant Bare_Defining_Name_List := Fld_1;
      begin
         for Untyped_Item_150 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_150 := Untyped_Item_150; 
Is_Null := Bare_Item_150 = null; 
if Is_Null then
   If_Result := No_Entity_Defining_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_150); 
   If_Result := As_Entity;
end if;
                  Item_150 := If_Result; 
   --# scope-start
               --# bind id Item_150
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_150;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4361
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_665;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_665;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Defining_Names;
--# end
--# property-start ObjectDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:7421
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Defining_Env
  (Node : Bare_Object_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_666;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_666 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_666;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4363 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7421
--# expr-start 4362 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7421
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4362
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.defining_env
Fld_1 := Libadalang.Implementation.Type_Expr_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4363
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_666;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_666;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Defining_Env;
--# end
--# property-start ObjectDecl.type_expression /usr/src/packages/BUILD/ada/ast.py:7422
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Type_Expression
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Expr
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Expr;
      Fld : Internal_Entity_Type_Expr;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4364 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:7422
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4364
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Type_Expression;
--# end
--# property-start ObjectDecl.is_static_decl /usr/src/packages/BUILD/ada/ast.py:7425
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Is_Static_Decl
  (Node : Bare_Object_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Bare_Constant_Node;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Renaming_Clause;
Fld_5 : Internal_Entity_Name;
Fld_6 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4372 '<Block>' Let_Result None
--# scope-start
--# expr-start 4366 '<FieldAccess .as_bool>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7427
--# expr-start 4365 '<FieldAccess .has_constant>' Fld /usr/src/packages/BUILD/ada/ast.py:7427
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Object_Decl_F_Has_Constant;
--# expr-done 4365
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 4366
if Fld_1 then
--# expr-start 4367 '<FieldAccess .default_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7427
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4367
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr
--# expr-start 4368 '<FieldAccess .is_static_expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7428
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_3 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 4368
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_1 := True;
else
--# expr-start 4369 '<FieldAccess .renaming_clause>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7430
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 4369
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
--# expr-start 4371 '<FieldAccess .is_static_expr>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7430
--# expr-start 4370 '<FieldAccess .renamed_object>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7430
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4370
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_6 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_5.Info);
--# end
--# expr-done 4371
Scope_Result_1 := Fld_6;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 4372
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Is_Static_Decl;
--# end
--# property-start ObjectDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:7434
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Equation
  (Node : Bare_Object_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_669;
            procedure Finalizer_Scope_2199;
            procedure Finalizer_Scope_2200;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Renaming_Clause;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Bare_Expr;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_7 : Internal_Entity_Renaming_Clause;
Fld_8 : Internal_Entity_Name;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Name;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_669 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_669;
            procedure Finalizer_Scope_2199 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2199;
            procedure Finalizer_Scope_2200 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2200;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4392 '<Block>' Let_Result None
--# scope-start
--# expr-start 4373 '<FieldAccess .expr_type>' Fld /usr/src/packages/BUILD/ada/ast.py:7435
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4373
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4391 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:7437
--# expr-start 4382 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:7437
--# expr-start 4375 '<FieldAccess .sub_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7437
--# expr-start 4374 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7437
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Object_Decl_F_Type_Expr, Info => Ent.Info);
--# expr-done 4374
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4375
--# expr-start 4376 '<FieldAccess .default_expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7438
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Object_Decl_F_Default_Expr, Info => Ent.Info);
--# expr-done 4376
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind de Var_Expr
--# expr-start 4381 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7440
--# expr-start 4377 '<FieldAccess .sub_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7440
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4377
--# expr-start 4380 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:7441
--# expr-start 4379 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7441
--# expr-start 4378 '<FieldAccess .node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7441
Fld_5 := Var_Expr.Node;
--# expr-done 4378
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_5);
--# expr-done 4379
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_566.Create (Fld_6, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7441") else null)); 
--# expr-done 4380
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7440") else null)); 
--# expr-done 4381
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2199;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7444") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_2, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7437") else null)); 
--# expr-done 4382
--# expr-start 4383 '<FieldAccess .renaming_clause>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7446
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Object_Decl_F_Renaming_Clause, Info => Ent.Info);
--# expr-done 4383
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind rc Var_Expr_1
--# expr-start 4390 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:7448
--# expr-start 4385 '<FieldAccess .sub_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7448
--# expr-start 4384 '<FieldAccess .renamed_object>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7448
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4384
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4385
--# expr-start 4389 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:7449
--# expr-start 4388 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7449
--# expr-start 4387 '<FieldAccess .node>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7449
--# expr-start 4386 '<FieldAccess .renamed_object>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:7449
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_1.Info);
--# expr-done 4386
Fld_11 := Fld_10.Node;
--# expr-done 4387
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 4388
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Bind_Default_566.Create (Fld_12, Cast_Result_1, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7449") else null)); 
--# expr-done 4389
And_Pred_2 := Logic_And (Fld_9, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7448") else null)); 
--# expr-done 4390
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2200;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7451") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_3 := Logic_And (And_Pred_1, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7437") else null)); 
--# expr-done 4391
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 4392
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_669;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_669;
               Finalizer_Scope_2199;
               Finalizer_Scope_2200;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Xref_Equation;
--# end
--# property-start ObjectDecl.public_part_decl /usr/src/packages/BUILD/ada/ast.py:7456
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Public_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_670;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_2 : Bare_Public_Part;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Bare_Constant_Node;
Fld_7 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_670 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_670;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4405 '<Block>' Let_Result None
--# scope-start
--# expr-start 4404 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7461
--# expr-start 4393 '<FieldAccess .is_in_private_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4393
if Fld_5 then
   --# expr-start 4395 '<FieldAccess .as_bool>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7462
--# expr-start 4394 '<FieldAccess .has_constant>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7462
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Object_Decl_F_Has_Constant;
--# expr-done 4394
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 4395
   If_Result := Fld_7;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4403 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4402 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4400 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4399 '<FieldAccess .public_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4398 '<Cast to BasePackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4397 '<parent(<FieldAccess .declarative_scope>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7463
--# expr-start 4396 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:7463
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 4396
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 4397
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4398
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 4399
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 4400
--# expr-start 4401 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7465
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4401
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4402
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4403
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Basic_Decl;
end if;
--# expr-done 4404
Let_Result := If_Result_1; 
--# end
--# expr-done 4405
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_670;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_670;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Public_Part_Decl;
--# end
--# property-start ObjectDecl.private_part_decl /usr/src/packages/BUILD/ada/ast.py:7471
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Private_Part_Decl
  (Node : Bare_Object_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Object_Decl :=
     Internal_Entity_Object_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_671;
      Fld : Bare_Declarative_Part;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_2 : Bare_Private_Part;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_5 : Boolean;
Fld_6 : Bare_Constant_Node;
Fld_7 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_671 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_671;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4418 '<Block>' Let_Result None
--# scope-start
--# expr-start 4417 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7476
--# expr-start 4406 '<FieldAccess .is_in_public_part>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7477
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_in_public_part
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Is_In_Public_Part (Node => Ent.Node);
--# end
--# expr-done 4406
if Fld_5 then
   --# expr-start 4408 '<FieldAccess .as_bool>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7477
--# expr-start 4407 '<FieldAccess .has_constant>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7477
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Object_Decl_F_Has_Constant;
--# expr-done 4407
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Constant.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Constant_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 4408
   If_Result := Fld_7;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 4416 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4415 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4413 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4412 '<FieldAccess .private_part>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4411 '<Cast to BasePackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4410 '<parent(<FieldAccess .declarative_scope>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7478
--# expr-start 4409 '<FieldAccess .declarative_scope>' Fld /usr/src/packages/BUILD/ada/ast.py:7478
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Self);
--# end
--# expr-done 4409
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 4410
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4411
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Base_Package_Decl_F_Private_Part;
--# expr-done 4412
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Fld_2);
--# expr-done 4413
--# expr-start 4414 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7480
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4414
Env_Get_Result := AST_Envs.Get_First (Self => Fld_3, Key => Fld_4, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 4415
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 4416
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Basic_Decl;
end if;
--# expr-done 4417
Let_Result := If_Result_1; 
--# end
--# expr-done 4418
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_671;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_671;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Private_Part_Decl;
--# end
--# property-start ObjectDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7485
pragma Warnings (Off, "is not referenced");
 function Object_Decl_P_Xref_Entry_Point
  (Node : Bare_Object_Decl
  )
   return Boolean
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Object_Decl_P_Xref_Entry_Point;
--# end
--# property-start ObjectDecl._env_mappings_49 /usr/src/packages/BUILD/ada/ast.py:7418
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_49
  (Node : Bare_Object_Decl
  )
   return Internal_Env_Assoc_Array_Access
is
   Self : Bare_Object_Decl := Bare_Object_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1219;
      Fld : Bare_Defining_Name_List;
Cast_Expr : Bare_Object_Decl;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Env_Assoc_Array_Access;
            procedure Finalizer_Scope_1219 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1219;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4420 '<FieldAccess .env_mappings(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7418
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4419 '<FieldAccess .ids>' Fld /usr/src/packages/BUILD/ada/ast.py:7418
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Object_Decl_F_Ids;
--# expr-done 4419
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start AdaNode.env_mappings
Fld_1 := Libadalang.Implementation.Ada_Node_P_Env_Mappings (Node => Self, Defining_Names => Fld, Value => Cast_Result);
--# end
--# expr-done 4420
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1219;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1219;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_49;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt_Object_Decl
   --
      procedure Initialize_Fields_For_Extended_Return_Stmt_Object_Decl
        (Self : Bare_Extended_Return_Stmt_Object_Decl
         ; Object_Decl_F_Ids : Bare_Defining_Name_List
         ; Object_Decl_F_Has_Aliased : Bare_Aliased_Node
         ; Object_Decl_F_Has_Constant : Bare_Constant_Node
         ; Object_Decl_F_Mode : Bare_Mode
         ; Object_Decl_F_Type_Expr : Bare_Type_Expr
         ; Object_Decl_F_Default_Expr : Bare_Expr
         ; Object_Decl_F_Renaming_Clause : Bare_Renaming_Clause
         ; Object_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Initialize_Fields_For_Object_Decl
              (Self, Object_Decl_F_Ids, Object_Decl_F_Has_Aliased, Object_Decl_F_Has_Constant, Object_Decl_F_Mode, Object_Decl_F_Type_Expr, Object_Decl_F_Default_Expr, Object_Decl_F_Renaming_Clause, Object_Decl_F_Aspects);
      end Initialize_Fields_For_Extended_Return_Stmt_Object_Decl;
   --
   --  Primitives for Bare_Package_Renaming_Decl
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Package_Renaming_Decl := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_90 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:8116" & ")";
         end if;
      return Initial_Env;
   end Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn;
      procedure Package_Renaming_Decl_Pre_Env_Actions
        (Self            : Bare_Package_Renaming_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dummy : Boolean;
      begin
         Dummy := Libadalang.Implementation.Env_Do_88 (Node => 
Self);
      end;
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_89 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access);
      end;
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_91 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:8121");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_92 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            Bare_Package_Renaming_Decl_Initial_Env_Getter_Fn'Access;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      declare
         Dummy : Bare_Ada_Node_Array_Access;
      begin
         Dummy := Libadalang.Implementation.Env_Do_93 (Node => 
Self);
         Dec_Ref (Dummy);
      end;
      if Libadalang.Implementation.Ref_Cond_95 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_94 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      if Libadalang.Implementation.Ref_Cond_97 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_96 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Package_Renaming_Decl
        (Self : Bare_Package_Renaming_Decl
         ; Package_Renaming_Decl_F_Name : Bare_Defining_Name
         ; Package_Renaming_Decl_F_Renames : Bare_Renaming_Clause
         ; Package_Renaming_Decl_F_Aspects : Bare_Aspect_Spec
        ) is
      begin
            Self.Package_Renaming_Decl_F_Name := Package_Renaming_Decl_F_Name;
            Self.Package_Renaming_Decl_F_Renames := Package_Renaming_Decl_F_Renames;
            Self.Package_Renaming_Decl_F_Aspects := Package_Renaming_Decl_F_Aspects;
      end Initialize_Fields_For_Package_Renaming_Decl;
   function Package_Renaming_Decl_F_Name
     (Node : Bare_Package_Renaming_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Package_Renaming_Decl_F_Name;
   end;
   function Package_Renaming_Decl_F_Renames
     (Node : Bare_Package_Renaming_Decl) return Bare_Renaming_Clause
   is
   begin
         return Node.Package_Renaming_Decl_F_Renames;
   end;
--# property-start PackageRenamingDecl.renamed_package /usr/src/packages/BUILD/ada/ast.py:8092
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_714;
      Var_Expr : Internal_Entity;
Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Renaming_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_714 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_714;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4428 '<Block>' Let_Result None
--# scope-start
--# expr-start 4427 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <Then <at(<FieldAccess .env_elements>, 0)>: <Var Var_Expr> <Cast to BasicDecl>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:8096
--# expr-start 4421 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8097
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4421
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4425 '<at(<FieldAccess .env_elements>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:8098
--# expr-start 4424 '<FieldAccess .env_elements>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8098
--# expr-start 4423 '<FieldAccess .renamed_object>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8098
--# expr-start 4422 '<FieldAccess .renames>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8098
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4422
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Fld_1.Node.Renaming_Clause_F_Renamed_Object, Info => Fld_1.Info);
--# expr-done 4423
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld_2.Node, Env => Bound_Env, E_Info => Fld_2.Info);
--# end
--# expr-done 4424
Get_Result := Get (Fld_3, 0, True); 
--# expr-done 4425
Var_Expr := Get_Result;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 4426 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8098
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 4426
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 4427
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4428
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_714;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_714;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.final_renamed_package /usr/src/packages/BUILD/ada/ast.py:8102
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Final_Renamed_Package
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4432 '<Block>' Let_Result None
--# scope-start
--# expr-start 4429 '<FieldAccess .renamed_package>' Fld /usr/src/packages/BUILD/ada/ast.py:8107
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4429
Block_Var := Fld; 
--# bind pkg Block_Var
--# expr-start 4430 '<Cast to PackageRenamingDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8108
Cast_Expr := Block_Var; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 4430
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 4431 '<FieldAccess .final_renamed_package>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8109
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld_1 := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4431
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 4432
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Final_Renamed_Package;
--# end
--# property-start PackageRenamingDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:8142
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Names
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_716;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_716 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_716;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4433 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:8142
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Package_Renaming_Decl_F_Name, Info => Ent.Info);
--# expr-done 4433
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_716;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_716;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Defining_Names;
--# end
--# property-start PackageRenamingDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:8143
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Defining_Env
  (Node : Bare_Package_Renaming_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_717;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_717 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_717;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4435 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8143
--# expr-start 4434 '<FieldAccess .renamed_package>' Fld /usr/src/packages/BUILD/ada/ast.py:8143
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Renamed_Package (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4434
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4435
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_717;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_717;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Defining_Env;
--# end
--# property-start PackageRenamingDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:8145
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Entry_Point
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Xref_Entry_Point;
--# end
--# property-start PackageRenamingDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:8146
pragma Warnings (Off, "is not referenced");
 function Package_Renaming_Decl_P_Xref_Equation
  (Node : Bare_Package_Renaming_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_719;
      Fld : Internal_Entity_Renaming_Clause;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
            procedure Finalizer_Scope_719 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_719;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4438 '<FieldAccess .xref_no_overloading>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8147
--# expr-start 4437 '<FieldAccess .renamed_object>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8147
--# expr-start 4436 '<FieldAccess .renames>' Fld /usr/src/packages/BUILD/ada/ast.py:8147
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Renaming_Clause (Node => Ent.Node.Package_Renaming_Decl_F_Renames, Info => Ent.Info);
--# expr-done 4436
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Fld.Node.Renaming_Clause_F_Renamed_Object, Info => Fld.Info);
--# expr-done 4437
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_2 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 4438
      Property_Result := Fld_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_719;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_719;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Package_Renaming_Decl_P_Xref_Equation;
--# end
--# property-start PackageRenamingDecl._env_do_88 /usr/src/packages/BUILD/ada/ast.py:8114
pragma Warnings (Off, "is not referenced");
 function Env_Do_88
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4439 '<FieldAccess .env_hook>' Fld /usr/src/packages/BUILD/ada/ast.py:8114
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.env_hook
Fld := Libadalang.Implementation.Ada_Node_P_Env_Hook (Node => Self);
--# end
--# expr-done 4439
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_88;
--# end
--# property-start PackageRenamingDecl._initial_env_name_89 /usr/src/packages/BUILD/ada/ast.py:8117
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_89
  (Node : Bare_Package_Renaming_Decl
  )
   return Symbol_Type
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4440 '<FieldAccess .child_decl_initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:8117
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.child_decl_initial_env_name'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Child_Decl_Initial_Env_Name (Node => Self);
--# end
--# expr-done 4440
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_89;
--# end
--# property-start PackageRenamingDecl._initial_env_90 /usr/src/packages/BUILD/ada/ast.py:8118
pragma Warnings (Off, "is not referenced");
 function Initial_Env_90
  (Node : Bare_Package_Renaming_Decl
  )
   return Lexical_Env
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1260;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1260 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1260;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4441 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8118
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 4441
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1260;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1260;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_90;
--# end
--# property-start PackageRenamingDecl._env_mappings_91 /usr/src/packages/BUILD/ada/ast.py:8121
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_91
  (Node : Bare_Package_Renaming_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Package_Renaming_Decl :=
     Internal_Entity_Package_Renaming_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1261;
      Fld : Symbol_Type;
Cast_Expr : Bare_Package_Renaming_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1261 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1261;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4443 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8121
--# expr-start 4442 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:8122
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4442
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4443
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1261;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1261;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_91;
--# end
--# property-start PackageRenamingDecl._env_trans_parent_92 /usr/src/packages/BUILD/ada/ast.py:8113
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_92
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_92;
--# end
--# property-start PackageRenamingDecl._env_do_93 /usr/src/packages/BUILD/ada/ast.py:8128
pragma Warnings (Off, "is not referenced");
 function Env_Do_93
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1263;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1263 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1263;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4444 '<FieldAccess .populate_dependent_units>' Fld /usr/src/packages/BUILD/ada/ast.py:8128
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.populate_dependent_units
Fld := Libadalang.Implementation.Basic_Decl_P_Populate_Dependent_Units (Node => Self);
--# end
--# expr-done 4444
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1263;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1263;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Do_93;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_94 /usr/src/packages/BUILD/ada/ast.py:8131
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_94
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1264;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1264 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1264;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4445 '<FieldAccess .top_level_use_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8131
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Package_Clauses (Node => Self);
--# end
--# expr-done 4445
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1264;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1264;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_94;
--# end
--# property-start PackageRenamingDecl._ref_cond_95 /usr/src/packages/BUILD/ada/ast.py:8133
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_95
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4447 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8133
--# expr-start 4446 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8133
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4446
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4447
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_95;
--# end
--# property-start PackageRenamingDecl._ref_env_nodes_96 /usr/src/packages/BUILD/ada/ast.py:8136
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_96
  (Node : Bare_Package_Renaming_Decl
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1266;
      Fld : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1266 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1266;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4448 '<FieldAccess .top_level_use_type_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:8136
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_use_type_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_Use_Type_Clauses (Node => Self);
--# end
--# expr-done 4448
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1266;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1266;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_96;
--# end
--# property-start PackageRenamingDecl._ref_cond_97 /usr/src/packages/BUILD/ada/ast.py:8138
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_97
  (Node : Bare_Package_Renaming_Decl
  )
   return Boolean
is
   Self : Bare_Package_Renaming_Decl := Bare_Package_Renaming_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4450 '<IsA LibraryItem, Subunit>' Is_A /usr/src/packages/BUILD/ada/ast.py:8138
--# expr-start 4449 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:8138
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4449
Is_A := Fld /= null 
and then Fld.Kind in Ada_Library_Item_Range | Ada_Subunit_Range; 
--# expr-done 4450
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_97;
--# end
   --
   --  Primitives for Bare_Single_Protected_Decl
   --
      procedure Single_Protected_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Protected_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_45 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7273");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_46 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_47 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Protected_Decl
        (Self : Bare_Single_Protected_Decl
         ; Single_Protected_Decl_F_Name : Bare_Defining_Name
         ; Single_Protected_Decl_F_Aspects : Bare_Aspect_Spec
         ; Single_Protected_Decl_F_Interfaces : Bare_Parent_List
         ; Single_Protected_Decl_F_Definition : Bare_Protected_Def
        ) is
      begin
            Self.Single_Protected_Decl_F_Name := Single_Protected_Decl_F_Name;
            Self.Single_Protected_Decl_F_Aspects := Single_Protected_Decl_F_Aspects;
            Self.Single_Protected_Decl_F_Interfaces := Single_Protected_Decl_F_Interfaces;
            Self.Single_Protected_Decl_F_Definition := Single_Protected_Decl_F_Definition;
      end Initialize_Fields_For_Single_Protected_Decl;
   function Single_Protected_Decl_F_Name
     (Node : Bare_Single_Protected_Decl) return Bare_Defining_Name
   is
   begin
         return Node.Single_Protected_Decl_F_Name;
   end;
   function Single_Protected_Decl_F_Interfaces
     (Node : Bare_Single_Protected_Decl) return Bare_Parent_List
   is
   begin
         return Node.Single_Protected_Decl_F_Interfaces;
   end;
   function Single_Protected_Decl_F_Definition
     (Node : Bare_Single_Protected_Decl) return Bare_Protected_Def
   is
   begin
         return Node.Single_Protected_Decl_F_Definition;
   end;
--# property-start SingleProtectedDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7256
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Names
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_652;
      Fld : Internal_Entity_Defining_Name;
Singleton : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_652 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_652;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4451 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:7256
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Defining_Name (Node => Ent.Node.Single_Protected_Decl_F_Name, Info => Ent.Info);
--# expr-done 4451
                Singleton := Create_Internal_Entity_Defining_Name_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
      Property_Result := Singleton;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_652;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_652;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Defining_Names;
--# end
--# property-start SingleProtectedDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:7258
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Defining_Env
  (Node : Bare_Single_Protected_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_653;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_653 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_653;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4452 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7258
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4452
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_653;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_653;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Defining_Env;
--# end
--# property-start SingleProtectedDecl.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7260
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Entry_Point
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Xref_Entry_Point;
--# end
--# property-start SingleProtectedDecl.xref_equation /usr/src/packages/BUILD/ada/ast.py:7263
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Xref_Equation
  (Node : Bare_Single_Protected_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_655;
            procedure Finalizer_Scope_2203;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_151 : Internal_Entity_Name;
Bare_Item_151 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_655 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_655;
            procedure Finalizer_Scope_2203 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2203;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4457 '<Block>' Let_Result None
--# scope-start
--# expr-start 4456 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:7264
--# expr-start 4455 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7264
--# expr-start 4453 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:7264
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Single_Protected_Decl_F_Interfaces, Info => Ent.Info);
--# expr-done 4453
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_151 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_151 := Untyped_Item_151; 
Is_Null := Bare_Item_151 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_151); 
   If_Result := As_Entity;
end if;
                  Item_151 := If_Result; 
   --# scope-start
               --# bind ifc Item_151
      --# expr-start 4454 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7264
      if Item_151.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_151.Node, Env => Env, Origin => Origin, E_Info => Item_151.Info);
--# end
--# expr-done 4454
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2203;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4455
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7264") else null)); 
--# expr-done 4456
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 4457
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_655;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_655;
               Finalizer_Scope_2203;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Xref_Equation;
--# end
--# property-start SingleProtectedDecl.env_names /usr/src/packages/BUILD/ada/ast.py:7267
pragma Warnings (Off, "is not referenced");
 function Single_Protected_Decl_P_Env_Names
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_656;
            procedure Finalizer_Scope_2204;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_656 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_656;
            procedure Finalizer_Scope_2204 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2204;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4460 '<Block>' Let_Result None
--# scope-start
--# expr-start 4458 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7268
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4458
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 4459 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:7269
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 4459
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2204;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4460
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_656;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_656;
               Finalizer_Scope_2204;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Protected_Decl_P_Env_Names;
--# end
--# property-start SingleProtectedDecl._env_mappings_45 /usr/src/packages/BUILD/ada/ast.py:7273
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_45
  (Node : Bare_Single_Protected_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Protected_Decl :=
     Internal_Entity_Single_Protected_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1215;
      Fld : Symbol_Type;
Cast_Expr : Bare_Single_Protected_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1215 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1215;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4462 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7273
--# expr-start 4461 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:7273
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 4461
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4462
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1215;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1215;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_45;
--# end
--# property-start SingleProtectedDecl._env_trans_parent_46 /usr/src/packages/BUILD/ada/ast.py:7272
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_46
  (Node : Bare_Single_Protected_Decl
  )
   return Boolean
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_46;
--# end
--# property-start SingleProtectedDecl._env_names_47 /usr/src/packages/BUILD/ada/ast.py:7274
pragma Warnings (Off, "is not referenced");
 function Env_Names_47
  (Node : Bare_Single_Protected_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Protected_Decl := Bare_Single_Protected_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1217;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1217 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1217;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4463 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:7274
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleProtectedDecl.env_names
Fld := Libadalang.Implementation.Single_Protected_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4463
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1217;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1217;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_47;
--# end
   --
   --  Primitives for Bare_Single_Task_Decl
   --
      procedure Single_Task_Decl_Pre_Env_Actions
        (Self            : Bare_Single_Task_Decl;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_42 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7242");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_43 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_44 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Single_Task_Decl
        (Self : Bare_Single_Task_Decl
         ; Single_Task_Decl_F_Task_Type : Bare_Single_Task_Type_Decl
        ) is
      begin
            Self.Single_Task_Decl_F_Task_Type := Single_Task_Decl_F_Task_Type;
      end Initialize_Fields_For_Single_Task_Decl;
   function Single_Task_Decl_F_Task_Type
     (Node : Bare_Single_Task_Decl) return Bare_Single_Task_Type_Decl
   is
   begin
         return Node.Single_Task_Decl_F_Task_Type;
   end;
--# property-start SingleTaskDecl.defining_names /usr/src/packages/BUILD/ada/ast.py:7230
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Names
  (Node : Bare_Single_Task_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name_Array_Access
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_648;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Internal_Entity_Defining_Name_Array_Access;
            procedure Finalizer_Scope_648 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_648;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4465 '<FieldAccess .defining_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7230
--# expr-start 4464 '<FieldAccess .task_type>' Fld /usr/src/packages/BUILD/ada/ast.py:7230
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4464
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4465
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_648;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_648;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Defining_Names;
--# end
--# property-start SingleTaskDecl.expr_type /usr/src/packages/BUILD/ada/ast.py:7231
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Expr_Type
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 4466 '<FieldAccess .task_type>' Fld /usr/src/packages/BUILD/ada/ast.py:7231
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4466
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Expr_Type;
--# end
--# property-start SingleTaskDecl.defining_env /usr/src/packages/BUILD/ada/ast.py:7233
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Defining_Env
  (Node : Bare_Single_Task_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Task_Decl :=
     Internal_Entity_Single_Task_Decl'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_650;
      Fld : Internal_Entity_Single_Task_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_650 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_650;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4468 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7233
--# expr-start 4467 '<FieldAccess .task_type>' Fld /usr/src/packages/BUILD/ada/ast.py:7233
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Single_Task_Type_Decl (Node => Ent.Node.Single_Task_Decl_F_Task_Type, Info => Ent.Info);
--# expr-done 4467
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4468
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_650;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_650;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Defining_Env;
--# end
--# property-start SingleTaskDecl.env_names /usr/src/packages/BUILD/ada/ast.py:7236
pragma Warnings (Off, "is not referenced");
 function Single_Task_Decl_P_Env_Names
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_651;
            procedure Finalizer_Scope_2205;
      Var_Expr : Character_Type_Array_Access;
Fld : Character_Type_Array_Access;
Sym : Symbol_Type;
Singleton : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_651 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_651;
            procedure Finalizer_Scope_2205 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2205;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4471 '<Block>' Let_Result None
--# scope-start
--# expr-start 4469 '<FieldAccess .top_level_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7237
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Self);
--# end
--# expr-done 4469
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind fqn Var_Expr
--# expr-start 4470 '<to_symbol(<Var fqn>)>' Sym /usr/src/packages/BUILD/ada/ast.py:7238
Sym := String_To_Symbol (Self.Unit.Context, Var_Expr); 
--# expr-done 4470
                Singleton := Create_Symbol_Type_Array (Items_Count => 1);
                Singleton.Items (1) := Sym;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2205;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 4471
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_651;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_651;
               Finalizer_Scope_2205;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Task_Decl_P_Env_Names;
--# end
--# property-start SingleTaskDecl._env_mappings_42 /usr/src/packages/BUILD/ada/ast.py:7242
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_42
  (Node : Bare_Single_Task_Decl
  )
   return Internal_Env_Assoc
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1212;
      Fld : Symbol_Type;
Cast_Expr : Bare_Single_Task_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1212 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1212;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4473 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7242
--# expr-start 4472 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:7242
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Self);
--# end
--# expr-done 4472
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4473
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1212;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1212;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_42;
--# end
--# property-start SingleTaskDecl._env_trans_parent_43 /usr/src/packages/BUILD/ada/ast.py:7241
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_43
  (Node : Bare_Single_Task_Decl
  )
   return Boolean
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_43;
--# end
--# property-start SingleTaskDecl._env_names_44 /usr/src/packages/BUILD/ada/ast.py:7243
pragma Warnings (Off, "is not referenced");
 function Env_Names_44
  (Node : Bare_Single_Task_Decl
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Single_Task_Decl := Bare_Single_Task_Decl (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1214;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1214 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1214;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4474 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:7243
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTaskDecl.env_names
Fld := Libadalang.Implementation.Single_Task_Decl_P_Env_Names (Node => Self);
--# end
--# expr-done 4474
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1214;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1214;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_44;
--# end
   --
   --  Primitives for Bare_Case_Stmt_Alternative
   --
      procedure Initialize_Fields_For_Case_Stmt_Alternative
        (Self : Bare_Case_Stmt_Alternative
         ; Case_Stmt_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Stmt_Alternative_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Case_Stmt_Alternative_F_Choices := Case_Stmt_Alternative_F_Choices;
            Self.Case_Stmt_Alternative_F_Stmts := Case_Stmt_Alternative_F_Stmts;
      end Initialize_Fields_For_Case_Stmt_Alternative;
   function Case_Stmt_Alternative_F_Choices
     (Node : Bare_Case_Stmt_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Choices;
   end;
   function Case_Stmt_Alternative_F_Stmts
     (Node : Bare_Case_Stmt_Alternative) return Bare_Stmt_List
   is
   begin
         return Node.Case_Stmt_Alternative_F_Stmts;
   end;
--# property-start CaseStmtAlternative.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:15083
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Entry_Point
  (Node : Bare_Case_Stmt_Alternative
  )
   return Boolean
is
   Self : Bare_Case_Stmt_Alternative := Bare_Case_Stmt_Alternative (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_Alternative_P_Xref_Entry_Point;
--# end
--# property-start CaseStmtAlternative.xref_equation /usr/src/packages/BUILD/ada/ast.py:15086
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_Alternative_P_Xref_Equation
  (Node : Bare_Case_Stmt_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt_Alternative := Bare_Case_Stmt_Alternative (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Stmt_Alternative :=
     Internal_Entity_Case_Stmt_Alternative'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1128;
            procedure Finalizer_Scope_2206;
            procedure Finalizer_Scope_2207;
            procedure Finalizer_Scope_2208;
            procedure Finalizer_Scope_2209;
      Block_Var : Internal_Entity_Case_Stmt;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Case_Stmt;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Alternatives_List;
Fld_5 : Bare_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_152 : Internal_Entity;
Bare_Item_152 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_1128 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_1128;
            procedure Finalizer_Scope_2206 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2206;
            procedure Finalizer_Scope_2207 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2207;
            procedure Finalizer_Scope_2208 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2208;
            procedure Finalizer_Scope_2209 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2209;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4489 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4477 '<Cast to CaseStmt>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15087
--# expr-start 4476 '<parent(<parent(<Var Ent>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15087
--# expr-start 4475 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:15087
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4475
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4476
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Case_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Case_Stmt
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4477
Block_Var := Cast_Result; 
--# bind case_stmt Block_Var
--# expr-start 4479 '<FieldAccess .expression_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15090
--# expr-start 4478 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15090
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Block_Var.Node.Case_Stmt_F_Expr, Info => Block_Var.Info);
--# expr-done 4478
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4479
Block_Var_1 := Fld_3; 
--# bind selected_type Block_Var_1
--# expr-start 4488 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:15092
--# expr-start 4487 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:15092
--# expr-start 4480 '<FieldAccess .choices>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15092
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Case_Stmt_Alternative_F_Choices, Info => Ent.Info);
--# expr-done 4480
Fld_5 := Fld_4.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_152 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_152 := Untyped_Item_152; 
Is_Null := Bare_Item_152 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_152); 
   If_Result := As_Entity;
end if;
                  Item_152 := If_Result; 
   --# scope-start
               --# bind c Item_152
      --# expr-start 4486 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:15092
      if Item_152.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_152; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind e Match_0
--# expr-start 4484 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:15095
--# expr-start 4482 '<FieldAccess .type_bind_val(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:15095
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4481 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:15095
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 4481
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 4482
--# expr-start 4483 '<FieldAccess .sub_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:15096
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4483
And_Pred := Logic_And (Fld_8, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15095") else null)); 
--# expr-done 4484
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2207;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind t Match_1
--# expr-start 4485 '<FieldAccess .xref_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:15099
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_10 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 4485
Let_Result_1 := Fld_10; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2208;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15101") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2209;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4486
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2206;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4487
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15092") else null)); 
--# expr-done 4488
Let_Result_4 := Logic_Boolean_Op; Inc_Ref (Let_Result_4);
--# end
--# expr-done 4489
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1128;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1128;
               Finalizer_Scope_2206;
               Finalizer_Scope_2207;
               Finalizer_Scope_2208;
               Finalizer_Scope_2209;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Compilation_Unit
   --
      procedure Initialize_Fields_For_Compilation_Unit
        (Self : Bare_Compilation_Unit
         ; Compilation_Unit_F_Prelude : Bare_Ada_Node_List
         ; Compilation_Unit_F_Body : Bare_Ada_Node
         ; Compilation_Unit_F_Pragmas : Bare_Pragma_Node_List
        ) is
      begin
            Self.Compilation_Unit_F_Prelude := Compilation_Unit_F_Prelude;
            Self.Compilation_Unit_F_Body := Compilation_Unit_F_Body;
            Self.Compilation_Unit_F_Pragmas := Compilation_Unit_F_Pragmas;
      Self.Compilation_Unit_No_Env := Empty_Env;
      end Initialize_Fields_For_Compilation_Unit;
   function Compilation_Unit_F_Prelude
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Prelude;
   end;
   function Compilation_Unit_F_Body
     (Node : Bare_Compilation_Unit) return Bare_Ada_Node
   is
   begin
         return Node.Compilation_Unit_F_Body;
   end;
   function Compilation_Unit_F_Pragmas
     (Node : Bare_Compilation_Unit) return Bare_Pragma_Node_List
   is
   begin
         return Node.Compilation_Unit_F_Pragmas;
   end;
--# property-start CompilationUnit.syntactic_fully_qualified_name /usr/src/packages/BUILD/ada/ast.py:14145
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Syntactic_Fully_Qualified_Name
  (Node : Bare_Compilation_Unit
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1076;
            procedure Finalizer_Scope_1495;
            procedure Finalizer_Scope_1496;
            procedure Finalizer_Scope_1497;
      Match_0 : Internal_Entity_Library_Item;
Match_1 : Internal_Entity_Subunit;
Match_2 : Internal_Entity;
Var_Expr : Internal_Entity_Defining_Name;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
Fld_3 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Symbol_Type_Array_Access;
Fld_6 : Internal_Entity_Body_Node;
Fld_7 : Internal_Entity_Defining_Name;
Fld_8 : Symbol_Type_Array_Access;
Concat_Result : Symbol_Type_Array_Access;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Library_Item;
Let_Result : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Cast_Result_1 : Internal_Entity_Subunit;
Let_Result_1 : Symbol_Type_Array_Access;
Scope_Result_2 : Symbol_Type_Array_Access;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Symbol_Type_Array_Access;
Scope_Result_3 : Symbol_Type_Array_Access;
Match_Result : Symbol_Type_Array_Access;
Let_Result_3 : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1076 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_1076;
            procedure Finalizer_Scope_1495 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1495;
            procedure Finalizer_Scope_1496 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_1496;
            procedure Finalizer_Scope_1497 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_1497;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4502 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4501 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:14149
--# expr-start 4491 '<FieldAccess .body>' Fld /usr/src/packages/BUILD/ada/ast.py:14149
--# expr-start 4490 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14149
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4490
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => If_Result.Node.Compilation_Unit_F_Body, Info => If_Result.Info);
--# expr-done 4491
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 4493 '<FieldAccess .defining_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14150
--# expr-start 4492 '<FieldAccess .item>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14150
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Basic_Decl (Node => Match_0.Node.Library_Item_F_Item, Info => Match_0.Info);
--# expr-done 4492
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4493
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 4494 '<FieldAccess .as_symbol_array>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14150
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Var_Expr.Node);
--# end
--# expr-done 4494
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1496;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1495;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 4500 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:14151
--# expr-start 4496 '<FieldAccess .as_symbol_array>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14151
--# expr-start 4495 '<FieldAccess .name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14151
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Match_1.Node.Subunit_F_Name, Info => Match_1.Info);
--# expr-done 4495
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_5 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_4.Node);
--# end
--# expr-done 4496
--# expr-start 4499 '<FieldAccess .as_symbol_array>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14152
--# expr-start 4498 '<FieldAccess .defining_name>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14152
--# expr-start 4497 '<FieldAccess .body>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14152
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Body_Node (Node => Match_1.Node.Subunit_F_Body, Info => Match_1.Info);
--# expr-done 4497
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 4498
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_8 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_7.Node);
--# end
--# expr-done 4499
Concat_Result := Concat (Fld_5, Fld_8); 
--# expr-done 4500
Let_Result_1 := Concat_Result; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1497;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 4501
Let_Result_3 := Match_Result; Inc_Ref (Let_Result_3);
--# end
--# expr-done 4502
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1076;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1076;
               Finalizer_Scope_1495;
               Finalizer_Scope_1496;
               Finalizer_Scope_1497;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Syntactic_Fully_Qualified_Name;
--# end
--# property-start CompilationUnit.unit_kind /usr/src/packages/BUILD/ada/ast.py:14160
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Kind
  (Node : Bare_Compilation_Unit
  )
   return Analysis_Unit_Kind
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Analysis_Unit_Kind;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Match_0_1 : Bare_Body_Node;
Match_1_1 : Bare_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Match_Prefix : Bare_Basic_Decl;
Cast_Result : Bare_Body_Node;
Let_Result : Analysis_Unit_Kind;
Scope_Result : Analysis_Unit_Kind;
Cast_Result_1 : Bare_Basic_Decl;
Let_Result_1 : Analysis_Unit_Kind;
Scope_Result_1 : Analysis_Unit_Kind;
Match_Result : Analysis_Unit_Kind;
Match_Prefix_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Library_Item;
Let_Result_2 : Analysis_Unit_Kind;
Scope_Result_2 : Analysis_Unit_Kind;
Cast_Result_3 : Bare_Subunit;
Let_Result_3 : Analysis_Unit_Kind;
Scope_Result_3 : Analysis_Unit_Kind;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Analysis_Unit_Kind;
Scope_Result_4 : Analysis_Unit_Kind;
Match_Result_1 : Analysis_Unit_Kind;
Let_Result_5 : Analysis_Unit_Kind;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4507 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 4506 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:14164
--# expr-start 4503 '<FieldAccess .body>' Fld /usr/src/packages/BUILD/ada/ast.py:14164
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 4503
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix_1;
Match_0 := Cast_Result_2; 
--# bind li Match_0
--# expr-start 4505 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:14165
--# expr-start 4504 '<FieldAccess .item>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14165
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 4504
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Basic_Decl (Match_Prefix.Kind) is
         when Ada_Expr_Function .. Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0_1 := Cast_Result; 
--# bind _ Match_0_1
Let_Result := Unit_Body; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Subp_Internal | Ada_Entry_Index_Spec .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1_1 := Cast_Result_1; 
--# bind _ Match_1_1
Let_Result_1 := Unit_Specification; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
end case;
--# expr-done 4505
Let_Result_2 := Match_Result; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result_1 := Scope_Result_2; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix_1;
Match_1 := Cast_Result_3; 
--# bind _ Match_1
Let_Result_3 := Unit_Body; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix_1;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_4 := raise Property_Error with "Unexpected CompilationUnit.f_body attribute"; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
end case;
--# expr-done 4506
Let_Result_5 := Match_Result_1; 
--# end
--# expr-done 4507
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Kind;
--# end
--# property-start CompilationUnit.withed_units /usr/src/packages/BUILD/ada/ast.py:14177
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Withed_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1078;
            procedure Finalizer_Scope_2216;
      Var_Expr : Bare_Compilation_Unit;
Fld : Bare_Name_Array_Access;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Compilation_Unit;
Scope_Result : Bare_Compilation_Unit;
Fld_3 : Symbol_Type_Array_Access;
Fld_4 : Bare_Compilation_Unit;
Result_Var : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1078 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1078;
            procedure Finalizer_Scope_2216 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2216;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Withed_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4515 '<Block>' Let_Result None
--# scope-start
--# expr-start 4514 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14182
--# expr-start 4508 '<FieldAccess .top_level_with_package_clauses>' Fld /usr/src/packages/BUILD/ada/ast.py:14182
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.top_level_with_package_clauses
Fld := Libadalang.Implementation.Ada_Node_P_Top_Level_With_Package_Clauses (Node => Self);
--# end
--# expr-done 4508
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Name_Array_Access := Fld;
      begin
         for Item_153 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_153
      --# expr-start 4513 '<as_bare_entity(<Then <FieldAccess .designated_compilation_unit(...)>: <Var e> <Var e>>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14186
--# expr-start 4510 '<FieldAccess .designated_compilation_unit(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4509 '<FieldAccess .as_symbol_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14187
      if Item_153 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Item_153);
--# end
--# expr-done 4509
--# property-call-start AdaNode.designated_compilation_unit
Fld_2 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_1, Kind => Unit_Specification);
--# end
--# expr-done 4510
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 4512 '<FieldAccess .designated_compilation_unit(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14189
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4511 '<FieldAccess .as_symbol_array>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14190
      if Item_153 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_3 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Item_153);
--# end
--# expr-done 4511
--# property-call-start AdaNode.designated_compilation_unit
Fld_4 := Libadalang.Implementation.Ada_Node_P_Designated_Compilation_Unit (Node => Self, Name => Fld_3, Kind => Unit_Body);
--# end
--# expr-done 4512
   Result_Var := Fld_4;
end if;
Is_Null := Result_Var = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Result_Var); 
   If_Result := As_Entity;
end if;
--# expr-done 4513
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               If_Result;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2216;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4514
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4515
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1078;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1078;
               Finalizer_Scope_2216;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Withed_Units;
--# end
--# property-start CompilationUnit.imported_units /usr/src/packages/BUILD/ada/ast.py:14197
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Imported_Units
  (Node : Bare_Compilation_Unit
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1079;
            procedure Finalizer_Scope_2218;
            procedure Finalizer_Scope_2219;
            procedure Finalizer_Scope_2220;
      Var_Expr : Bare_Ada_Node;
Var_Expr_1 : Lexical_Env := Empty_Env;
Var_Expr_2 : Bare_Basic_Decl;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Scope_Result_1 : Bare_Ada_Node;
Result_Var_1 : Bare_Ada_Node;
Fld_3 : Bare_Compilation_Unit;
As_Entity : Internal_Entity_Compilation_Unit;
Is_Null : Boolean;
If_Result : Internal_Entity_Compilation_Unit;
Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Scope_Result_2 : Internal_Entity_Compilation_Unit_Array_Access;
Result_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1079 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1079;
            procedure Finalizer_Scope_2218 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2218;
            procedure Finalizer_Scope_2219 is
            begin
                     Dec_Ref (Var_Expr_1);
            end Finalizer_Scope_2219;
            procedure Finalizer_Scope_2220 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2220;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Imported_Units,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4523 '<Block>' Let_Result None
--# scope-start
--# expr-start 4522 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:14202
--# expr-start 4516 '<FieldAccess .withed_units>' Fld /usr/src/packages/BUILD/ada/ast.py:14202
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.withed_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Withed_Units (Node => Self);
--# end
--# expr-done 4516
--# expr-start 4517 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14203
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Self);
--# end
--# expr-done 4517
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 4518 '<FieldAccess .node_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14203
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Node_Env (Node => Var_Expr_2);
--# expr-done 4518
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2218;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Var_Expr_1 := Result_Var;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= Empty_Env then
   --# scope-start
--# expr-start 4519 '<env_node(<Var Var_Expr_1>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:14203
Env_Node := AST_Envs.Env_Node (Var_Expr_1); 
--# expr-done 4519
Scope_Result_1 := Env_Node;
   --# end
      Finalizer_Scope_2219;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Var_Expr := Result_Var_1;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 4521 '<as_bare_entity(<FieldAccess .enclosing_compilation_unit>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14205
--# expr-start 4520 '<FieldAccess .enclosing_compilation_unit>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14205
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Var_Expr);
--# end
--# expr-done 4520
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Compilation_Unit;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 4521
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
Scope_Result_2 := Singleton;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2220;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Entity_Compilation_Unit_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Fld, Result_Var_2); 
--# expr-done 4522
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4523
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1079;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1079;
               Finalizer_Scope_2218;
               Finalizer_Scope_2219;
               Finalizer_Scope_2220;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Imported_Units;
--# end
--# property-start CompilationUnit.unit_dependencies_helper /usr/src/packages/BUILD/ada/ast.py:14210
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies_Helper
  (Node : Bare_Compilation_Unit
      ; Visited : Internal_Entity_Compilation_Unit_Array_Access
      ; To_Visit : Internal_Entity_Compilation_Unit_Array_Access
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind visited Visited
   --# bind to_visit To_Visit
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1080;
            procedure Finalizer_Scope_2221;
      Block_Var : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_1 : Internal_Entity_Compilation_Unit_Array_Access;
Block_Var_2 : Internal_Entity_Compilation_Unit_Array_Access;
Concat_Result : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Cast_Expr : Internal_Entity_Compilation_Unit;
Cast_Result : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Compilation_Unit;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result_1 : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Internal_Entity_Compilation_Unit_Array_Access;
Len : Integer;
Comp_Result : Boolean;
If_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1080 is
            begin
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1080;
            procedure Finalizer_Scope_2221 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2221;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4534 '<Block>' Let_Result None
--# scope-start
--# expr-start 4524 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:14219
Concat_Result := Concat (Visited, To_Visit); 
--# expr-done 4524
Block_Var := Concat_Result; Inc_Ref (Block_Var);
--# bind now_visited Block_Var
--# expr-start 4527 '<unique(<mapcat>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:14220
--# expr-start 4526 '<mapcat>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14220
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := To_Visit;
      begin
         for Item_154 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_154
      --# expr-start 4525 '<FieldAccess .imported_units>' Fld /usr/src/packages/BUILD/ada/ast.py:14220
      if Item_154.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.imported_units
Fld := Libadalang.Implementation.Compilation_Unit_P_Imported_Units (Node => Item_154.Node);
--# end
--# expr-done 4525
         for Item_To_Append of
               Fld.Items
         loop
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2221;
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4526
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 4527
Block_Var_1 := Unique_Array; Inc_Ref (Block_Var_1);
--# bind new_imports Block_Var_1
--# expr-start 4530 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:14222
declare
   Map_Result_1_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var_1;
      begin
         for Item_155 of
               Collection.Items
         loop
   --# scope-start
               --# bind c Item_155
               --# expr-start 4529 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:14222
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Block_Var;
   begin
      for Item_156 of
            Collection.Items
      loop
   --# scope-start
            --# bind item Item_156
         --# expr-start 4528 '<Eq>' Is_Equiv None
Cast_Expr := Item_156; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Cast_Expr_1 := Item_155; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 4528
            if Is_Equiv then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result); 
--# expr-done 4529
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               Item_155;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4530
Block_Var_2 := Map_Result_1; Inc_Ref (Block_Var_2);
--# bind to_visit_next Block_Var_2
--# expr-start 4533 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14224
--# expr-start 4531 '<length(<Var to_visit_next>)>' Len /usr/src/packages/BUILD/ada/ast.py:14225
Len := Length (Block_Var_2); 
--# expr-done 4531
Comp_Result := Len > 0; 
if Comp_Result then
   --# expr-start 4532 '<FieldAccess .unit_dependencies_helper(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14226
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld_1 := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => Block_Var, To_Visit => Block_Var_2);
--# end
--# expr-done 4532
   If_Result := Fld_1;
else
   If_Result := Block_Var;
end if;
      Inc_Ref (If_Result);
--# expr-done 4533
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4534
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1080;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1080;
               Finalizer_Scope_2221;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Dependencies_Helper;
--# end
--# property-start CompilationUnit.unit_dependencies /usr/src/packages/BUILD/ada/ast.py:14232
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Unit_Dependencies
  (Node : Bare_Compilation_Unit
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Compilation_Unit_Array_Access
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1081;
      Singleton : Internal_Entity_Compilation_Unit_Array_Access;
Fld : Internal_Entity_Compilation_Unit_Array_Access;
Unique_Array : Internal_Entity_Compilation_Unit_Array_Access;
Fld_1 : Bare_Compilation_Unit;
Cast_Result : Bare_Compilation_Unit;
Is_Equal : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Compilation_Unit_Array_Access;
Let_Result : Internal_Entity_Compilation_Unit_Array_Access;
            procedure Finalizer_Scope_1081 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Fld);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1081;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Compilation_Unit_Compilation_Unit_P_Unit_Dependencies,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Compilation_Unit,
                                As_Bare_Compilation_Unit => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Compilation_Unit_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4541 '<Block>' Let_Result None
--# scope-start
--# expr-start 4540 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14237
--# expr-start 4536 '<unique(<FieldAccess .unit_dependencies_helper(...)>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:14237
--# expr-start 4535 '<FieldAccess .unit_dependencies_helper(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14237
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
                Singleton := Create_Internal_Entity_Compilation_Unit_Array (Items_Count => 1);
                Singleton.Items (1) := Ent;
--# property-call-start CompilationUnit.unit_dependencies_helper
Fld := Libadalang.Implementation.Compilation_Unit_P_Unit_Dependencies_Helper (Node => Self, Visited => No_Internal_Entity_Compilation_Unit_Array_Type, To_Visit => Singleton);
--# end
--# expr-done 4535
Unique_Array := Make_Unique (Fld); 
--# expr-done 4536
declare
   Map_Result_Vec : Internal_Entity_Compilation_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Compilation_Unit_Array_Access := Unique_Array;
      begin
         for Item_157 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_157
               --# expr-start 4539 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:14242
--# expr-start 4538 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:14242
--# expr-start 4537 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14242
Fld_1 := Item_157.Node;
--# expr-done 4537
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Self; 
--# expr-done 4538
Not_Val := not (Is_Equal); 
--# expr-done 4539
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Compilation_Unit :=
               Item_157;
         begin
            Internal_Entity_Compilation_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Compilation_Unit_Array
        (Items_Count => Natural (Internal_Entity_Compilation_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Compilation_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Compilation_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Compilation_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4540
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4541
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1081;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Compilation_Unit_Array_Access,
                     As_Internal_Entity_Compilation_Unit_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1081;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Unit_Dependencies;
--# end
--# property-start CompilationUnit.decl /usr/src/packages/BUILD/ada/ast.py:14247
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Decl
  (Node : Bare_Compilation_Unit
  )
   return Bare_Basic_Decl
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Basic_Decl;
      Match_0 : Bare_Library_Item;
Match_1 : Bare_Subunit;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Basic_Decl;
Fld_2 : Bare_Body_Node;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Library_Item;
Let_Result : Bare_Basic_Decl;
Scope_Result : Bare_Basic_Decl;
Cast_Result_1 : Bare_Subunit;
Cast_Result_2 : Bare_Basic_Decl;
Let_Result_1 : Bare_Basic_Decl;
Scope_Result_1 : Bare_Basic_Decl;
Cast_Result_3 : Bare_Ada_Node;
Let_Result_2 : Bare_Basic_Decl;
Scope_Result_2 : Bare_Basic_Decl;
Match_Result : Bare_Basic_Decl;
Let_Result_3 : Bare_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4546 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4545 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:14251
--# expr-start 4542 '<FieldAccess .body>' Fld /usr/src/packages/BUILD/ada/ast.py:14251
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Compilation_Unit_F_Body;
--# expr-done 4542
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind li Match_0
--# expr-start 4543 '<FieldAccess .item>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14252
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_0.Library_Item_F_Item;
--# expr-done 4543
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind su Match_1
--# expr-start 4544 '<FieldAccess .body>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14253
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Match_1.Subunit_F_Body;
--# expr-done 4544
      Cast_Result_2 := Fld_2;
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4545
Let_Result_3 := Match_Result; 
--# end
--# expr-done 4546
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Decl;
--# end
--# property-start CompilationUnit.is_preelaborable_impl /usr/src/packages/BUILD/ada/ast.py:14259
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable_Impl
  (Node : Bare_Compilation_Unit
      ; From_Body : Boolean
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_body From_Body
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Subunit;
Match_1 : Internal_Entity_Library_Item;
Match_2 : Internal_Entity;
Match_0_1 : Internal_Entity_Subp_Decl;
Match_1_1 : Internal_Entity_Generic_Subp_Decl;
Match_2_1 : Internal_Entity_Subp_Body;
Match_3 : Internal_Entity_Package_Decl;
Match_4 : Internal_Entity_Generic_Package_Decl;
Match_5 : Internal_Entity_Package_Body;
Match_6 : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Cast_Result : Internal_Entity_Compilation_Unit;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Basic_Decl;
Fld_6 : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Fld_9 : Boolean;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity;
Fld_12 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Compilation_Unit;
Fld_13 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_14 : Boolean;
Fld_15 : Internal_Entity_Generic_Package_Internal;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Internal_Unit;
Fld_19 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Compilation_Unit;
Fld_20 : Boolean;
Match_Prefix : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Subp_Decl;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity_Generic_Subp_Decl;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_5 : Internal_Entity_Subp_Body;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_6 : Internal_Entity_Package_Decl;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_7 : Internal_Entity_Generic_Package_Decl;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_8 : Internal_Entity_Package_Body;
Let_Result_5 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_9 : Internal_Entity_Basic_Decl;
Let_Result_6 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Match_Prefix_1 : Internal_Entity;
Cast_Result_10 : Internal_Entity_Subunit;
Let_Result_7 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_11 : Internal_Entity_Library_Item;
Let_Result_8 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_12 : Internal_Entity;
Let_Result_9 : Boolean;
Scope_Result_10 : Boolean;
Match_Result_1 : Boolean;
Let_Result_10 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4574 '<Block>' Let_Result_10 None
--# scope-start
--# expr-start 4573 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:14267
--# expr-start 4547 '<FieldAccess .body>' Fld /usr/src/packages/BUILD/ada/ast.py:14267
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Compilation_Unit_F_Body, Info => Ent.Info);
--# expr-done 4547
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Ada_Node (Match_Prefix_1.Node.Kind) is
         when Ada_Subunit =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Subunit
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_10; 
--# bind su Match_0
--# expr-start 4552 '<FieldAccess .is_preelaborable_impl(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14271
--# expr-start 4551 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14271
--# expr-start 4550 '<parent(<parent(<FieldAccess .body_root>)>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14271
--# expr-start 4549 '<parent(<FieldAccess .body_root>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14271
--# expr-start 4548 '<FieldAccess .body_root>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14271
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_1 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Match_0.Node);
--# end
--# expr-done 4548
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 4549
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4550
   if Fld_3.Node = null
      or else Fld_3.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4551
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 4552
Let_Result_7 := Fld_4; 
--# end
Scope_Result_8 := Let_Result_7;
   --# end
            Match_Result_1 := Scope_Result_8; 
         when Ada_Library_Item =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Library_Item
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_11; 
--# bind li Match_1
--# expr-start 4572 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:14275
--# expr-start 4553 '<FieldAccess .item>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14275
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Basic_Decl (Node => Match_1.Node.Library_Item_F_Item, Info => Match_1.Info);
--# expr-done 4553
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_5; 
case Ada_Basic_Decl (Match_Prefix.Node.Kind) is
         when Ada_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_3; 
--# bind subp_decl Match_0_1
--# expr-start 4554 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14277
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_0_1.Node, From_Body => From_Body, E_Info => Match_0_1.Info);
--# end
--# expr-done 4554
Let_Result := Fld_6; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Generic_Subp_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Generic_Subp_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_4; 
--# bind gen_subp_decl Match_1_1
--# expr-start 4556 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14280
--# expr-start 4555 '<FieldAccess .decl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14280
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]GenericDecl.decl'
Fld_7 := Libadalang.Implementation.Dispatcher_Generic_Decl_P_Decl (Node => Match_1_1.Node, E_Info => Match_1_1.Info);
--# end
--# expr-done 4555
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_7.Node, From_Body => From_Body, E_Info => Fld_7.Info);
--# end
--# expr-done 4556
Let_Result_1 := Fld_8; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subp_Body =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Subp_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_5; 
--# bind subp_body Match_2_1
--# expr-start 4557 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:14286
      if Match_2_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_2_1.Node, From_Body => False, E_Info => Match_2_1.Info);
--# end
--# expr-done 4557
if Fld_9 then
   If_Result := True;
else
--# expr-start 4558 '<FieldAccess .decl_part>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:14290
      if Match_2_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_10 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_2_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2_1.Info);
--# end
--# expr-done 4558
Var_Expr := Fld_10;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind dp Var_Expr
--# expr-start 4562 '<FieldAccess .is_preelaborable_impl(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:14291
--# expr-start 4561 '<Cast to CompilationUnit>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:14291
--# expr-start 4560 '<parent(<parent(<Var dp>)>)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:14291
--# expr-start 4559 '<parent(<Var dp>)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:14291
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Parent (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# expr-done 4559
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Fld_11.Node, E_Info => Fld_11.Info);
--# expr-done 4560
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Compilation_Unit
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4561
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_13 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_1.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 4562
Scope_Result := Fld_13;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
Let_Result_2 := If_Result; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind pkg_decl Match_3
--# expr-start 4563 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:14297
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Match_3.Node, From_Body => From_Body, E_Info => Match_3.Info);
--# end
--# expr-done 4563
Let_Result_3 := Fld_14; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Generic_Package_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind gen_pkg_decl Match_4
--# expr-start 4565 '<FieldAccess .does_aspects_make_preelaborable(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:14300
--# expr-start 4564 '<FieldAccess .package_decl>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:14300
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Generic_Package_Internal (Node => Match_4.Node.Generic_Package_Decl_F_Package_Decl, Info => Match_4.Info);
--# expr-done 4564
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.does_aspects_make_preelaborable
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Does_Aspects_Make_Preelaborable (Node => Fld_15.Node, From_Body => From_Body, E_Info => Fld_15.Info);
--# end
--# expr-done 4565
Let_Result_4 := Fld_16; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Package_Body =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Package_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind pkg_body Match_5
--# expr-start 4571 '<FieldAccess .is_preelaborable_impl(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:14306
--# expr-start 4570 '<Cast to CompilationUnit>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:14306
--# expr-start 4569 '<as_bare_entity(<FieldAccess .root>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:14306
--# expr-start 4568 '<FieldAccess .root>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:14306
--# expr-start 4567 '<FieldAccess .unit>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:14306
--# expr-start 4566 '<FieldAccess .decl_part>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:14306
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_17 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 4566
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Unit (Node => Fld_17.Node);
--# expr-done 4567
      if Fld_18 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Fld_18.AST_Root;
--# expr-done 4568
Is_Null := Fld_19 = null; 
if Is_Null then
   If_Result_1 := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_19); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 4569
   if If_Result_1.Node = null
      or else If_Result_1.Node.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Compilation_Unit
        (Node => If_Result_1.Node,
         Info => If_Result_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4570
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld_20 := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Cast_Result_2.Node, From_Body => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 4571
Let_Result_5 := Fld_20; 
--# end
Scope_Result_6 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl .. Ada_Generic_Package_Internal | Ada_Discrete_Base_Subtype_Decl .. Ada_Concrete_Formal_Subp_Decl | Ada_Entry_Decl .. Ada_Null_Subp_Decl | Ada_Subp_Renaming_Decl .. Ada_Entry_Body | Ada_Protected_Body .. Ada_For_Loop_Var_Decl | Ada_Generic_Package_Instantiation .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_9; 
--# bind _ Match_6
Let_Result_6 := False; 
--# end
Scope_Result_7 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 4572
Let_Result_8 := Match_Result; 
--# end
Scope_Result_9 := Let_Result_8;
   --# end
            Match_Result_1 := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iter_Type_Of | Ada_Limited_Absent .. Ada_Subp_Kind_Procedure | Ada_Synchronized_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_12; 
--# bind _ Match_2
Let_Result_9 := False; 
--# end
Scope_Result_10 := Let_Result_9;
   --# end
            Match_Result_1 := Scope_Result_10; 
end case;
--# expr-done 4573
Let_Result_10 := Match_Result_1; 
--# end
--# expr-done 4574
      Property_Result := Let_Result_10;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Preelaborable_Impl;
--# end
--# property-start CompilationUnit.is_preelaborable /usr/src/packages/BUILD/ada/ast.py:14319
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Preelaborable
  (Node : Bare_Compilation_Unit
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Compilation_Unit :=
     Internal_Entity_Compilation_Unit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4576 '<Block>' Let_Result None
--# scope-start
--# expr-start 4575 '<FieldAccess .is_preelaborable_impl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:14323
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.is_preelaborable_impl
Fld := Libadalang.Implementation.Compilation_Unit_P_Is_Preelaborable_Impl (Node => Ent.Node, From_Body => False, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4575
Let_Result := Fld; 
--# end
--# expr-done 4576
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Preelaborable;
--# end
--# property-start CompilationUnit.is_text_io_child /usr/src/packages/BUILD/ada/ast.py:14326
pragma Warnings (Off, "is not referenced");
 function Compilation_Unit_P_Is_Text_Io_Child
  (Node : Bare_Compilation_Unit
  )
   return Boolean
is
   Self : Bare_Compilation_Unit := Bare_Compilation_Unit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1085;
      Block_Var : Symbol_Type_Array_Access;
Fld : Symbol_Type_Array_Access;
Len : Integer;
Is_Equal : Boolean;
Get_Result : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Get_Result_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
Get_Result_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Get_Result_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1085 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_1085;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4592 '<Block>' Let_Result None
--# scope-start
--# expr-start 4577 '<FieldAccess .syntactic_fully_qualified_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14331
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.syntactic_fully_qualified_name
Fld := Libadalang.Implementation.Compilation_Unit_P_Syntactic_Fully_Qualified_Name (Node => Self);
--# end
--# expr-done 4577
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind name_parts Block_Var
--# expr-start 4579 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:14333
--# expr-start 4578 '<length(<Var name_parts>)>' Len /usr/src/packages/BUILD/ada/ast.py:14333
Len := Length (Block_Var); 
--# expr-done 4578
Is_Equal := Len = 3; 
--# expr-done 4579
if Is_Equal then
   --# expr-start 4582 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:14334
--# expr-start 4580 '<at(<Var name_parts>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:14334
Get_Result := Get (Block_Var, 0, True); 
--# expr-done 4580
--# expr-start 4581 '<Symbol ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada_208); 
--# expr-done 4581
Is_Equal_1 := Get_Result = Sym; 
--# expr-done 4582
   If_Result := Is_Equal_1;
else
   If_Result := False;
end if;
if If_Result then
--# expr-start 4585 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:14335
--# expr-start 4583 '<at(<Var name_parts>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:14335
Get_Result_1 := Get (Block_Var, 1, True); 
--# expr-done 4583
--# expr-start 4584 '<Symbol text_io>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Text_Io); 
--# expr-done 4584
Is_Equal_2 := Get_Result_1 = Sym_1; 
--# expr-done 4585
if Is_Equal_2 then
   If_Result_1 := True;
else
   --# expr-start 4588 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:14335
--# expr-start 4586 '<at(<Var name_parts>, 1)>' Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:14335
Get_Result_2 := Get (Block_Var, 1, True); 
--# expr-done 4586
--# expr-start 4587 '<Symbol wide_text_io>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Text_Io); 
--# expr-done 4587
Is_Equal_3 := Get_Result_2 = Sym_2; 
--# expr-done 4588
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4591 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:14335
--# expr-start 4589 '<at(<Var name_parts>, 1)>' Get_Result_3 /usr/src/packages/BUILD/ada/ast.py:14335
Get_Result_3 := Get (Block_Var, 1, True); 
--# expr-done 4589
--# expr-start 4590 '<Symbol wide_wide_text_io>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Text_Io); 
--# expr-done 4590
Is_Equal_4 := Get_Result_3 = Sym_3; 
--# expr-done 4591
   If_Result_2 := Is_Equal_4;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
Let_Result := If_Result_3; 
--# end
--# expr-done 4592
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1085;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1085;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Compilation_Unit_P_Is_Text_Io_Child;
--# end
   --
   --  Primitives for Bare_Component_Clause
   --
      procedure Initialize_Fields_For_Component_Clause
        (Self : Bare_Component_Clause
         ; Component_Clause_F_Id : Bare_Identifier
         ; Component_Clause_F_Position : Bare_Expr
         ; Component_Clause_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Component_Clause_F_Id := Component_Clause_F_Id;
            Self.Component_Clause_F_Position := Component_Clause_F_Position;
            Self.Component_Clause_F_Range := Component_Clause_F_Range;
      end Initialize_Fields_For_Component_Clause;
   function Component_Clause_F_Id
     (Node : Bare_Component_Clause) return Bare_Identifier
   is
   begin
         return Node.Component_Clause_F_Id;
   end;
   function Component_Clause_F_Position
     (Node : Bare_Component_Clause) return Bare_Expr
   is
   begin
         return Node.Component_Clause_F_Position;
   end;
   function Component_Clause_F_Range
     (Node : Bare_Component_Clause) return Bare_Range_Spec
   is
   begin
         return Node.Component_Clause_F_Range;
   end;
--# property-start ComponentClause.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:7172
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Entry_Point
  (Node : Bare_Component_Clause
  )
   return Boolean
is
   Self : Bare_Component_Clause := Bare_Component_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Clause_P_Xref_Entry_Point;
--# end
--# property-start ComponentClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:7175
pragma Warnings (Off, "is not referenced");
 function Component_Clause_P_Xref_Equation
  (Node : Bare_Component_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Clause := Bare_Component_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Clause :=
     Internal_Entity_Component_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_645;
      Block_Var : Internal_Entity_Record_Rep_Clause;
Block_Var_1 : Lexical_Env := Empty_Env;
Fld : Internal_Entity;
Fld_1 : Internal_Entity;
Cast_Result : Internal_Entity_Record_Rep_Clause;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Range_Spec;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_645 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_645;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4607 '<Block>' Let_Result None
--# scope-start
--# expr-start 4595 '<Cast to RecordRepClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7178
--# expr-start 4594 '<parent(<parent(<Var Ent>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7178
--# expr-start 4593 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7178
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4593
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4594
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Record_Rep_Clause_Range
   then
      Cast_Result := Create_Internal_Entity_Record_Rep_Clause
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4595
Block_Var := Cast_Result; 
--# bind rep_clause Block_Var
--# expr-start 4597 '<FieldAccess .designated_env_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7182
--# expr-start 4596 '<FieldAccess .name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7182
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Block_Var.Node.Record_Rep_Clause_F_Name, Info => Block_Var.Info);
--# expr-done 4596
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4597
Block_Var_1 := Fld_3; Inc_Ref (Block_Var_1);
--# bind record_env Block_Var_1
--# expr-start 4606 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:7184
--# expr-start 4603 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:7184
--# expr-start 4600 '<bind(<DynamicVariable env>, <Var record_env>, <FieldAccess .xref_equation>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:7186
Bound_Env := Block_Var_1; Inc_Ref (Bound_Env);
--# expr-start 4599 '<FieldAccess .xref_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7186
--# expr-start 4598 '<FieldAccess .id>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7186
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Component_Clause_F_Id, Info => Ent.Info);
--# expr-done 4598
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_5 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4599
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4600
--# expr-start 4602 '<FieldAccess .sub_equation>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7187
--# expr-start 4601 '<FieldAccess .position>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7187
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Ent.Node.Component_Clause_F_Position, Info => Ent.Info);
--# expr-done 4601
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 4602
And_Pred := Logic_And (Dyn_Var_Bind_Result, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7184") else null)); 
--# expr-done 4603
--# expr-start 4605 '<FieldAccess .xref_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:7188
--# expr-start 4604 '<FieldAccess .range>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7188
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Component_Clause_F_Range, Info => Ent.Info);
--# expr-done 4604
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_9 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 4605
And_Pred_1 := Logic_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7184") else null)); 
--# expr-done 4606
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4607
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_645;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_645;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Clause_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Component_Def
   --
      procedure Initialize_Fields_For_Component_Def
        (Self : Bare_Component_Def
         ; Component_Def_F_Has_Aliased : Bare_Aliased_Node
         ; Component_Def_F_Has_Constant : Bare_Constant_Node
         ; Component_Def_F_Type_Expr : Bare_Type_Expr
        ) is
      begin
            Self.Component_Def_F_Has_Aliased := Component_Def_F_Has_Aliased;
            Self.Component_Def_F_Has_Constant := Component_Def_F_Has_Constant;
            Self.Component_Def_F_Type_Expr := Component_Def_F_Type_Expr;
      end Initialize_Fields_For_Component_Def;
   function Component_Def_F_Has_Aliased
     (Node : Bare_Component_Def) return Bare_Aliased_Node
   is
   begin
         return Node.Component_Def_F_Has_Aliased;
   end;
   function Component_Def_F_Has_Constant
     (Node : Bare_Component_Def) return Bare_Constant_Node
   is
   begin
         return Node.Component_Def_F_Has_Constant;
   end;
   function Component_Def_F_Type_Expr
     (Node : Bare_Component_Def) return Bare_Type_Expr
   is
   begin
         return Node.Component_Def_F_Type_Expr;
   end;
--# property-start ComponentDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6049
pragma Warnings (Off, "is not referenced");
 function Component_Def_P_Xref_Equation
  (Node : Bare_Component_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Component_Def := Bare_Component_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Component_Def :=
     Internal_Entity_Component_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_520;
      Fld : Internal_Entity_Type_Expr;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_520 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_520;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4610 '<Block>' Let_Result None
--# scope-start
--# expr-start 4609 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6050
--# expr-start 4608 '<FieldAccess .type_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:6050
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Type_Expr (Node => Ent.Node.Component_Def_F_Type_Expr, Info => Ent.Info);
--# expr-done 4608
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4609
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4610
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_520;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_520;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Component_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Constant_Node
   --
--# property-start '[dispatcher]Constant.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Constant_Node_P_As_Bool
  (Node : Bare_Constant_Node
  )
   return Boolean
is
   Self : Bare_Constant_Node := Bare_Constant_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Constant_Node (Self.Kind) is
               when Ada_Constant_Absent =>
                  --# property-call-start Constant.Absent.as_bool
                  Property_Result := Constant_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Constant_Present =>
                  --# property-call-start Constant.Present.as_bool
                  Property_Result := Constant_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Constant_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Absent
   --
--# property-start Constant.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3903
pragma Warnings (Off, "is not referenced");
 function Constant_Absent_P_As_Bool
  (Node : Bare_Constant_Absent
  )
   return Boolean
is
   Self : Bare_Constant_Absent := Bare_Constant_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constant_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constant_Present
   --
--# property-start Constant.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3903
pragma Warnings (Off, "is not referenced");
 function Constant_Present_P_As_Bool
  (Node : Bare_Constant_Present
  )
   return Boolean
is
   Self : Bare_Constant_Present := Bare_Constant_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constant_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Constraint
   --
--# property-start Constraint.subtype /usr/src/packages/BUILD/ada/ast.py:5651
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Subtype
  (Node : Bare_Constraint
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Constraint := Bare_Constraint (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Subtype_Indication;
As_Entity : Internal_Entity_Subtype_Indication;
Is_Null : Boolean;
If_Result : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4616 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5651
--# expr-start 4611 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:5652
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4611
Bound_Origin := Fld; 
--# expr-start 4615 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5653
--# expr-start 4614 '<as_entity(<Cast to SubtypeIndication>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5653
--# expr-start 4613 '<Cast to SubtypeIndication>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5653
--# expr-start 4612 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5653
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 4612
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Fld_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4613
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subtype_Indication;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 4614
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => If_Result.Node, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 4615
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 4616
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constraint_P_Subtype;
--# end
--# property-start Constraint.is_static /usr/src/packages/BUILD/ada/ast.py:5658
pragma Warnings (Off, "is not referenced");
 function Constraint_P_Is_Static
  (Node : Bare_Constraint
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Constraint := Bare_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Constraint :=
     Internal_Entity_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Range_Constraint;
Match_1 : Internal_Entity_Index_Constraint;
Match_2 : Internal_Entity_Discriminant_Constraint;
Match_3 : Internal_Entity_Constraint;
Match_0_1 : Internal_Entity_Subtype_Indication;
Match_1_1 : Internal_Entity_Expr;
Match_2_1 : Internal_Entity;
Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Constraint_List;
Fld_4 : Bare_Constraint_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Boolean;
Fld_7 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Internal_Entity_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Item_158 : Internal_Entity;
Bare_Item_158 : Bare_Ada_Node;
Quantifier_Result : Boolean;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Bare_Assoc_List;
Fld_10 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Boolean;
Item_159 : Internal_Entity_Basic_Assoc;
Bare_Item_159 : Bare_Basic_Assoc;
Quantifier_Result_1 : Boolean;
Match_Prefix_1 : Internal_Entity_Constraint;
Cast_Result_3 : Internal_Entity_Range_Constraint;
Let_Result_3 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_4 : Internal_Entity_Index_Constraint;
Let_Result_4 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_5 : Internal_Entity_Discriminant_Constraint;
Let_Result_5 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_6 : Internal_Entity_Constraint;
Let_Result_6 : Boolean;
Scope_Result_6 : Boolean;
Match_Result_1 : Boolean;
Let_Result_7 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4628 '<Block>' Let_Result_7 None
--# scope-start
--# expr-start 4627 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:5659
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Ent; 
case Ada_Constraint (Match_Prefix_1.Node.Kind) is
         when Ada_Range_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Range_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_0 := Cast_Result_3; 
--# bind rc Match_0
--# expr-start 4619 '<FieldAccess .is_static_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5660
--# expr-start 4618 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5660
--# expr-start 4617 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:5660
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Match_0.Node.Range_Constraint_F_Range, Info => Match_0.Info);
--# expr-done 4617
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 4618
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 4619
Let_Result_3 := Fld_2; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result_1 := Scope_Result_3; 
         when Ada_Index_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Index_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_1 := Cast_Result_4; 
--# bind ic Match_1
--# expr-start 4620 '<FieldAccess .constraints>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5662
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Constraint_List (Node => Match_1.Node.Index_Constraint_F_Constraints, Info => Match_1.Info);
--# expr-done 4620
Fld_4 := Fld_3.Node;
Quantifier_Result := True;
   if Fld_4 /= null then
   declare
      Collection : constant Bare_Constraint_List := Fld_4;
   begin
      for Untyped_Item_158 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_158 := Untyped_Item_158; 
Is_Null := Bare_Item_158 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_158); 
   If_Result := As_Entity;
end if;
               Item_158 := If_Result; 
   --# scope-start
            --# bind c Item_158
         --# expr-start 4623 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:5662
      if Item_158.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_158; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result; 
--# bind st Match_0_1
--# expr-start 4621 '<FieldAccess .is_static_subtype>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5663
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_6 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Match_0_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_0_1.Info);
--# end
--# expr-done 4621
Let_Result := Fld_6; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_1; 
--# bind e Match_1_1
--# expr-start 4622 '<FieldAccess .is_static_expr>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5664
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_7 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Match_1_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1_1.Info);
--# end
--# expr-done 4622
Let_Result_1 := Fld_7; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2_1 := Cast_Result_2; 
--# bind _ Match_2_1
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4623
            if not (Match_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result_4 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result_1 := Scope_Result_4; 
         when Ada_Discriminant_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Discriminant_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_2 := Cast_Result_5; 
--# bind dc Match_2
--# expr-start 4624 '<FieldAccess .constraints>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5667
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Assoc_List (Node => Match_2.Node.Discriminant_Constraint_F_Constraints, Info => Match_2.Info);
--# expr-done 4624
Fld_9 := Fld_8.Node;
Quantifier_Result_1 := True;
   if Fld_9 /= null then
   declare
      Collection : constant Bare_Assoc_List := Fld_9;
   begin
      for Untyped_Item_159 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_159 := Untyped_Item_159; 
Is_Null_1 := Bare_Item_159 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity_1 := (Info => Fld_10, Node => Bare_Item_159); 
   If_Result_1 := As_Entity_1;
end if;
               Item_159 := If_Result_1; 
   --# scope-start
            --# bind c Item_159
         --# expr-start 4626 '<FieldAccess .is_static_expr>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:5668
--# expr-start 4625 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5668
      if Item_159.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_159.Node, E_Info => Item_159.Info);
--# end
--# expr-done 4625
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_12 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_11.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_11.Info);
--# end
--# expr-done 4626
            if not (Fld_12) then
               Quantifier_Result_1 := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result_5 := Quantifier_Result_1; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result_1 := Scope_Result_5; 
         when Ada_Delta_Constraint .. Ada_Digits_Constraint =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Constraint
        (Node => Match_Prefix_1.Node,
         Info => Match_Prefix_1.Info);
Match_3 := Cast_Result_6; 
--# bind _ Match_3
Let_Result_6 := False; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result_1 := Scope_Result_6; 
end case;
--# expr-done 4627
Let_Result_7 := Match_Result_1; 
--# end
--# expr-done 4628
      Property_Result := Let_Result_7;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Constraint_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Delta_Constraint
   --
      procedure Initialize_Fields_For_Delta_Constraint
        (Self : Bare_Delta_Constraint
         ; Delta_Constraint_F_Digits : Bare_Expr
         ; Delta_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Delta_Constraint_F_Digits := Delta_Constraint_F_Digits;
            Self.Delta_Constraint_F_Range := Delta_Constraint_F_Range;
      end Initialize_Fields_For_Delta_Constraint;
   function Delta_Constraint_F_Digits
     (Node : Bare_Delta_Constraint) return Bare_Expr
   is
   begin
         return Node.Delta_Constraint_F_Digits;
   end;
   function Delta_Constraint_F_Range
     (Node : Bare_Delta_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Delta_Constraint_F_Range;
   end;
--# property-start DeltaConstraint.xref_equation /usr/src/packages/BUILD/ada/ast.py:5708
pragma Warnings (Off, "is not referenced");
 function Delta_Constraint_P_Xref_Equation
  (Node : Bare_Delta_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delta_Constraint := Bare_Delta_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Delta_Constraint :=
     Internal_Entity_Delta_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_473;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_473 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_473;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4633 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5709
--# expr-start 4630 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5709
--# expr-start 4629 '<FieldAccess .digits>' Fld /usr/src/packages/BUILD/ada/ast.py:5709
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delta_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 4629
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4630
--# expr-start 4632 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5709
--# expr-start 4631 '<FieldAccess .range>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5709
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Delta_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4631
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4632
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5709") else null)); 
--# expr-done 4633
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_473;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_473;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Delta_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Digits_Constraint
   --
      procedure Initialize_Fields_For_Digits_Constraint
        (Self : Bare_Digits_Constraint
         ; Digits_Constraint_F_Digits : Bare_Expr
         ; Digits_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Digits_Constraint_F_Digits := Digits_Constraint_F_Digits;
            Self.Digits_Constraint_F_Range := Digits_Constraint_F_Range;
      end Initialize_Fields_For_Digits_Constraint;
   function Digits_Constraint_F_Digits
     (Node : Bare_Digits_Constraint) return Bare_Expr
   is
   begin
         return Node.Digits_Constraint_F_Digits;
   end;
   function Digits_Constraint_F_Range
     (Node : Bare_Digits_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Digits_Constraint_F_Range;
   end;
--# property-start DigitsConstraint.xref_equation /usr/src/packages/BUILD/ada/ast.py:5696
pragma Warnings (Off, "is not referenced");
 function Digits_Constraint_P_Xref_Equation
  (Node : Bare_Digits_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Digits_Constraint := Bare_Digits_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Digits_Constraint :=
     Internal_Entity_Digits_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_472;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Range_Spec;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_472 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_472;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4638 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5697
--# expr-start 4635 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5697
--# expr-start 4634 '<FieldAccess .digits>' Fld /usr/src/packages/BUILD/ada/ast.py:5697
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Digits_Constraint_F_Digits, Info => Ent.Info);
--# expr-done 4634
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4635
--# expr-start 4637 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5697
--# expr-start 4636 '<FieldAccess .range>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5697
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Digits_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4636
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 4637
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5697") else null)); 
--# expr-done 4638
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_472;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_472;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Digits_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Discriminant_Constraint
   --
      procedure Initialize_Fields_For_Discriminant_Constraint
        (Self : Bare_Discriminant_Constraint
         ; Discriminant_Constraint_F_Constraints : Bare_Assoc_List
        ) is
      begin
            Self.Discriminant_Constraint_F_Constraints := Discriminant_Constraint_F_Constraints;
      end Initialize_Fields_For_Discriminant_Constraint;
   function Discriminant_Constraint_F_Constraints
     (Node : Bare_Discriminant_Constraint) return Bare_Assoc_List
   is
   begin
         return Node.Discriminant_Constraint_F_Constraints;
   end;
--# property-start DiscriminantConstraint.xref_equation /usr/src/packages/BUILD/ada/ast.py:5751
pragma Warnings (Off, "is not referenced");
 function Discriminant_Constraint_P_Xref_Equation
  (Node : Bare_Discriminant_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Discriminant_Constraint := Bare_Discriminant_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Discriminant_Constraint :=
     Internal_Entity_Discriminant_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_475;
            procedure Finalizer_Scope_2248;
            procedure Finalizer_Scope_2249;
            procedure Finalizer_Scope_2250;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Base_Id;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Item_160 : Internal_Entity_Basic_Assoc;
Bare_Item_160 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_6 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Param_Match_Array_Access;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
Fld_13 : Internal_Single_Actual;
Fld_14 : Internal_Entity_Basic_Assoc;
Fld_15 : Internal_Entity_Expr;
Fld_16 : Logic_Var;
Fld_17 : Internal_Single_Formal;
Fld_18 : Internal_Entity_Base_Formal_Param_Decl;
Fld_19 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_20 : Internal_Single_Actual;
Fld_21 : Bare_Base_Id;
Fld_22 : Logic_Var;
Fld_23 : Internal_Single_Formal;
Fld_24 : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_25 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_475 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_475;
            procedure Finalizer_Scope_2248 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2248;
            procedure Finalizer_Scope_2249 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2249;
            procedure Finalizer_Scope_2250 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2250;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4672 '<Block>' Let_Result None
--# scope-start
--# expr-start 4639 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:5752
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4639
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4671 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:5754
--# expr-start 4640 '<FieldAccess .is_array>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:5757
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4640
if Fld_25 then
   --# expr-start 4645 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:5760
--# expr-start 4644 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5760
--# expr-start 4641 '<FieldAccess .constraints>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5760
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Discriminant_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4641
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_2;
      begin
         for Untyped_Item_160 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_160 := Untyped_Item_160; 
Is_Null := Bare_Item_160 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_160); 
   If_Result := As_Entity;
end if;
                  Item_160 := If_Result; 
   --# scope-start
               --# bind c Item_160
      --# expr-start 4643 '<FieldAccess .sub_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5761
--# expr-start 4642 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5761
      if Item_160.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_160.Node, E_Info => Item_160.Info);
--# end
--# expr-done 4642
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 4643
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2248;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4644
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5760") else null)); 
--# expr-done 4645
   If_Result_1 := Logic_Boolean_Op;
else
   --# expr-start 4670 '<LogicAll>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:5765
--# expr-start 4669 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:5765
--# expr-start 4648 '<FieldAccess .match_formals(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:5765
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4646 '<FieldAccess .discriminants_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5766
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 4646
--# expr-start 4647 '<FieldAccess .constraints>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5766
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Discriminant_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4647
--# property-call-start AdaNode.match_formals
Fld_8 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld_6, Params => Fld_7, Is_Dottable_Subp => False);
--# end
--# expr-done 4648
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_8;
      begin
         for Item_161 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_161
      --# expr-start 4668 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:5769
--# expr-start 4661 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5769
--# expr-start 4652 '<FieldAccess .xref_equation>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:5769
--# expr-start 4651 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:5769
--# expr-start 4650 '<FieldAccess .assoc>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:5769
--# expr-start 4649 '<FieldAccess .actual>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:5769
Fld_9 := Item_161.Actual;
--# expr-done 4649
Fld_10 := Fld_9.Assoc;
--# expr-done 4650
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 4651
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_12 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_11.Node, Env => Env, Origin => Origin, E_Info => Fld_11.Info);
--# end
--# expr-done 4652
--# expr-start 4660 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:5770
--# expr-start 4656 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:5771
--# expr-start 4655 '<FieldAccess .expr>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:5771
--# expr-start 4654 '<FieldAccess .assoc>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:5771
--# expr-start 4653 '<FieldAccess .actual>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:5771
Fld_13 := Item_161.Actual;
--# expr-done 4653
Fld_14 := Fld_13.Assoc;
--# expr-done 4654
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_15 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 4655
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_15.Node);
--# expr-done 4656
Fld_16.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_16.all);
Eq_Node.Refs.Destroy (Fld_16.all);
--# expr-start 4659 '<FieldAccess .formal_type>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:5771
--# expr-start 4658 '<FieldAccess .spec>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:5771
--# expr-start 4657 '<FieldAccess .formal>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:5771
Fld_17 := Item_161.Formal;
--# expr-done 4657
Fld_18 := Fld_17.Spec;
--# expr-done 4658
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_19 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_18.Node, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 4659
      Cast_Result := Create_Internal_Entity
        (Node => Fld_19.Node,
         Info => Fld_19.Info);
Bind_Result := Bind_Default_564.Create (Fld_16, Cast_Result, No_Logic_Converter_Default, Equals_Data_564'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5770") else null)); 
--# expr-done 4660
And_Pred := Logic_And (Fld_12, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5769") else null)); 
--# expr-done 4661
--# expr-start 4663 '<FieldAccess .name>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:5774
--# expr-start 4662 '<FieldAccess .actual>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:5774
Fld_20 := Item_161.Actual;
--# expr-done 4662
Fld_21 := Fld_20.Name;
--# expr-done 4663
Var_Expr := Fld_21;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 4667 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:5775
--# expr-start 4664 '<FieldAccess .ref_var>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:5775
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Var_Expr);
--# end
--# expr-done 4664
Fld_22.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_22.all);
Eq_Node.Refs.Destroy (Fld_22.all);
--# expr-start 4666 '<FieldAccess .spec>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:5775
--# expr-start 4665 '<FieldAccess .formal>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:5775
Fld_23 := Item_161.Formal;
--# expr-done 4665
Fld_24 := Fld_23.Spec;
--# expr-done 4666
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_24.Node,
         Info => Fld_24.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_22, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5775") else null)); 
--# expr-done 4667
Scope_Result := Bind_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2250;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5776") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (And_Pred, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5769") else null)); 
--# expr-done 4668
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2249;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4669
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5765") else null)); 
--# expr-done 4670
   If_Result_1 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 4671
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4672
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_475;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_475;
               Finalizer_Scope_2248;
               Finalizer_Scope_2249;
               Finalizer_Scope_2250;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Discriminant_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Index_Constraint
   --
      procedure Initialize_Fields_For_Index_Constraint
        (Self : Bare_Index_Constraint
         ; Index_Constraint_F_Constraints : Bare_Constraint_List
        ) is
      begin
            Self.Index_Constraint_F_Constraints := Index_Constraint_F_Constraints;
      end Initialize_Fields_For_Index_Constraint;
   function Index_Constraint_F_Constraints
     (Node : Bare_Index_Constraint) return Bare_Constraint_List
   is
   begin
         return Node.Index_Constraint_F_Constraints;
   end;
--# property-start IndexConstraint.xref_equation /usr/src/packages/BUILD/ada/ast.py:5720
pragma Warnings (Off, "is not referenced");
 function Index_Constraint_P_Xref_Equation
  (Node : Bare_Index_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Index_Constraint := Bare_Index_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Index_Constraint :=
     Internal_Entity_Index_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_474;
            procedure Finalizer_Scope_2251;
            procedure Finalizer_Scope_2252;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Constraint_List;
Fld_2 : Bare_Constraint_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_7 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Item_162 : Internal_Entity;
Bare_Item_162 : Bare_Ada_Node;
I : Integer;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_474 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_474;
            procedure Finalizer_Scope_2251 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2251;
            procedure Finalizer_Scope_2252 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2252;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4683 '<Block>' Let_Result None
--# scope-start
--# expr-start 4673 '<FieldAccess .subtype>' Fld /usr/src/packages/BUILD/ada/ast.py:5721
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4673
Block_Var := Fld; 
--# bind typ Block_Var
--# expr-start 4682 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:5722
--# expr-start 4681 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5722
--# expr-start 4674 '<FieldAccess .constraints>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5722
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Constraint_List (Node => Ent.Node.Index_Constraint_F_Constraints, Info => Ent.Info);
--# expr-done 4674
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
         I := 0;
      declare
         Collection : constant Bare_Constraint_List := Fld_2;
      begin
         for Untyped_Item_162 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_162 := Untyped_Item_162; 
Is_Null := Bare_Item_162 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_162); 
   If_Result := As_Entity;
end if;
                  Item_162 := If_Result; 
   --# scope-start
               --# bind c Item_162
               --# bind i I
      --# expr-start 4680 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5724
--# expr-start 4675 '<FieldAccess .xref_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5724
      if Item_162.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_162.Node, Env => Env, Origin => Origin, E_Info => Item_162.Info);
--# end
--# expr-done 4675
--# expr-start 4676 '<Cast to Expr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:5737
Cast_Expr := Item_162; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Expr;
   end if;
--# expr-done 4676
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 4679 '<FieldAccess .type_bind_val(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:5738
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4677 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5738
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 4677
--# expr-start 4678 '<FieldAccess .index_type(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5738
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var.Node, Dim => I, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4678
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
--# property-call-start AdaNode.type_bind_val
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 4679
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2252;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5739") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_4, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5724") else null)); 
--# expr-done 4680
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
      Finalizer_Scope_2251;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4681
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5722") else null)); 
--# expr-done 4682
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 4683
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_474;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_474;
               Finalizer_Scope_2251;
               Finalizer_Scope_2252;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Index_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Range_Constraint
   --
      procedure Initialize_Fields_For_Range_Constraint
        (Self : Bare_Range_Constraint
         ; Range_Constraint_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Range_Constraint_F_Range := Range_Constraint_F_Range;
      end Initialize_Fields_For_Range_Constraint;
   function Range_Constraint_F_Range
     (Node : Bare_Range_Constraint) return Bare_Range_Spec
   is
   begin
         return Node.Range_Constraint_F_Range;
   end;
--# property-start RangeConstraint.xref_equation /usr/src/packages/BUILD/ada/ast.py:5682
pragma Warnings (Off, "is not referenced");
 function Range_Constraint_P_Xref_Equation
  (Node : Bare_Range_Constraint
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Constraint := Bare_Range_Constraint (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Range_Constraint :=
     Internal_Entity_Range_Constraint'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_471;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_471 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_471;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4692 '<Block>' Let_Result None
--# scope-start
--# expr-start 4691 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5683
--# expr-start 4688 '<FieldAccess .type_bind_val(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5684
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4686 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5684
--# expr-start 4685 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5684
--# expr-start 4684 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:5684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4684
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 4685
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1.Node);
--# expr-done 4686
--# expr-start 4687 '<FieldAccess .subtype>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.subtype
Fld_3 := Libadalang.Implementation.Constraint_P_Subtype (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4687
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 4688
--# expr-start 4690 '<FieldAccess .sub_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5685
--# expr-start 4689 '<FieldAccess .range>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5685
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Range_Constraint_F_Range, Info => Ent.Info);
--# expr-done 4689
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 4690
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5683") else null)); 
--# expr-done 4691
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4692
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_471;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_471;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Range_Constraint_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Declarative_Part
   --
      procedure Initialize_Fields_For_Declarative_Part
        (Self : Bare_Declarative_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Self.Declarative_Part_F_Decls := Declarative_Part_F_Decls;
      end Initialize_Fields_For_Declarative_Part;
   function Declarative_Part_F_Decls
     (Node : Bare_Declarative_Part) return Bare_Ada_Node_List
   is
   begin
         return Node.Declarative_Part_F_Decls;
   end;
--# property-start DeclarativePart.types_with_models /usr/src/packages/BUILD/ada/ast.py:7550
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Types_With_Models
  (Node : Bare_Declarative_Part
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Declarative_Part := Bare_Declarative_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_678;
      Var_Expr : Internal_Entity_Base_Type_Decl;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Bare_Ada_Node_List;
Fld_2 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Item_163 : Internal_Entity;
Bare_Item_163 : Bare_Ada_Node;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Fld_3 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Is_Null_2 : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_678 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_678;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Declarative_Part_Declarative_Part_P_Types_With_Models,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Declarative_Part,
                                As_Bare_Declarative_Part => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4702 '<Block>' Let_Result None
--# scope-start
--# expr-start 4701 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7551
--# expr-start 4694 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:7551
--# expr-start 4693 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:7551
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 4693
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => If_Result.Node.Declarative_Part_F_Decls, Info => If_Result.Info);
--# expr-done 4694
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Ada_Node_List := Fld_1;
      begin
         for Untyped_Item_163 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_163 := Untyped_Item_163; 
Is_Null_1 := Bare_Item_163 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_2 := Fld.Info;
As_Entity_1 := (Info => Fld_2, Node => Bare_Item_163); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_163 := If_Result_1; 
   --# scope-start
               --# bind d Item_163
               --# expr-start 4699 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:7554
--# expr-start 4698 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:7554
--# expr-start 4695 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7554
Cast_Expr_1 := Item_163; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4695
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4697 '<FieldAccess .get_aspect_spec_expr(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7554
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4696 '<Symbol Model_Of>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Of); 
--# expr-done 4696
--# property-call-start BasicDecl.get_aspect_spec_expr
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Get_Aspect_Spec_Expr (Node => Var_Expr.Node, Name => Sym, E_Info => Var_Expr.Info);
--# end
--# expr-done 4697
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Is_Null_2 := Result_Var.Node = null; 
--# expr-done 4698
Not_Val := not (Is_Null_2); 
--# expr-done 4699
               if Not_Val then
      --# expr-start 4700 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7552
Cast_Expr := Item_163; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4700
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4701
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4702
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_678;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl_Array_Access,
                     As_Internal_Entity_Base_Type_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_678;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Declarative_Part_P_Types_With_Models;
--# end
--# property-start DeclarativePart.use_clauses_envs /usr/src/packages/BUILD/ada/ast.py:7559
pragma Warnings (Off, "is not referenced");
 function Declarative_Part_P_Use_Clauses_Envs
  (Node : Bare_Declarative_Part
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Declarative_Part := Bare_Declarative_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Declarative_Part :=
     Internal_Entity_Declarative_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_679;
            procedure Finalizer_Scope_2155;
      Fld : Internal_Entity_Ada_Node_List;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Use_Clause;
Fld_2 : Lexical_Env := Empty_Env;
Is_A : Boolean;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_679 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_679;
            procedure Finalizer_Scope_2155 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2155;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4710 '<Block>' Let_Result None
--# scope-start
--# expr-start 4709 '<env_group(<filter_map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:7564
--# expr-start 4708 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7564
--# expr-start 4704 '<children(<FieldAccess .decls>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7564
--# expr-start 4703 '<FieldAccess .decls>' Fld /usr/src/packages/BUILD/ada/ast.py:7564
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Ada_Node_List (Node => Ent.Node.Declarative_Part_F_Decls, Info => Ent.Info);
--# expr-done 4703
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children (Node => Fld.Node, E_Info => Fld.Info);
--# expr-done 4704
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_164 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_164
               --# expr-start 4705 '<IsA UseClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:7566
Is_A := Item_164.Node /= null 
and then Item_164.Node.Kind in Ada_Use_Clause; 
--# expr-done 4705
               if Is_A then
      --# expr-start 4707 '<FieldAccess .used_envs>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7565
--# expr-start 4706 '<Cast to UseClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7565
Cast_Expr := Item_164; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Use_Clause
   then
      Cast_Result := Create_Internal_Entity_Use_Clause
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Use_Clause;
   end if;
--# expr-done 4706
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UseClause.used_envs
Fld_2 := Libadalang.Implementation.Use_Clause_P_Used_Envs (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 4707
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_2;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2155;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4708
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 4709
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 4710
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_679;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_679;
               Finalizer_Scope_2155;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Declarative_Part_P_Use_Clauses_Envs;
--# end
   --
   --  Primitives for Bare_Private_Part
   --
      procedure Private_Part_Pre_Env_Actions
        (Self            : Bare_Private_Part;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_50 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:7590");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_51 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            Libadalang.Implementation.Env_Names_52 (Node => 
Self);
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Private_Part
        (Self : Bare_Private_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Private_Part;
--# property-start PrivatePart.env_names /usr/src/packages/BUILD/ada/ast.py:7575
pragma Warnings (Off, "is not referenced");
 function Private_Part_P_Env_Names
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_680;
            procedure Finalizer_Scope_2253;
            procedure Finalizer_Scope_2254;
      Var_Expr : Bare_Base_Package_Decl;
Var_Expr_1 : Character_Type_Array_Access;
Fld : Bare_Ada_Node;
Cast_Result : Bare_Base_Package_Decl;
Fld_1 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Sym : Symbol_Type;
Array_Lit_1 : Symbol_Type_Array_Access;
Scope_Result : Symbol_Type_Array_Access;
Result_Var : Symbol_Type_Array_Access;
Scope_Result_1 : Symbol_Type_Array_Access;
Result_Var_1 : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_680 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_680;
            procedure Finalizer_Scope_2253 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2253;
            procedure Finalizer_Scope_2254 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Array_Lit_1);
            end Finalizer_Scope_2254;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4718 '<Block>' Let_Result None
--# scope-start
--# expr-start 4712 '<Cast to BasePackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7581
--# expr-start 4711 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7581
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 4711
   if Fld = null
      or else Fld.Kind in Ada_Base_Package_Decl
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 4712
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind pkg Var_Expr
--# expr-start 4713 '<FieldAccess .top_level_env_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7582
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Var_Expr);
--# end
--# expr-done 4713
Var_Expr_1 := Fld_1;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Character_Type_Array_Type then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 4717 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:7583
--# expr-start 4716 '<to_symbol(<Concat>)>' Sym /usr/src/packages/BUILD/ada/ast.py:7584
--# expr-start 4715 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:7584
--# expr-start 4714 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:7584
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46), 2 => Character_Type'Val (95), 3 => Character_Type'Val (95), 4 => Character_Type'Val (112), 5 => Character_Type'Val (114), 6 => Character_Type'Val (105), 7 => Character_Type'Val (118), 8 => Character_Type'Val (97), 9 => Character_Type'Val (116), 10 => Character_Type'Val (101), 11 => Character_Type'Val (112), 12 => Character_Type'Val (97), 13 => Character_Type'Val (114), 14 => Character_Type'Val (116))); 
--# expr-done 4714
Concat_Result := Concat (Var_Expr_1, Array_Lit); 
--# expr-done 4715
Sym := String_To_Symbol (Self.Unit.Context, Concat_Result); 
--# expr-done 4716
Array_Lit_1 := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 4717
Scope_Result := Array_Lit_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2254;
   Result_Var := Scope_Result;
else
   Result_Var := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2253;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Symbol_Type_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4718
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_680;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_680;
               Finalizer_Scope_2253;
               Finalizer_Scope_2254;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Part_P_Env_Names;
--# end
--# property-start PrivatePart._env_mappings_50 /usr/src/packages/BUILD/ada/ast.py:7590
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_50
  (Node : Bare_Private_Part
  )
   return Internal_Env_Assoc
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1220;
      Sym : Symbol_Type;
Cast_Expr : Bare_Private_Part;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1220 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1220;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4720 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:7590
--# expr-start 4719 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 4719
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Sym, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 4720
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1220;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1220;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_50;
--# end
--# property-start PrivatePart._env_trans_parent_51 /usr/src/packages/BUILD/ada/ast.py:7589
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_51
  (Node : Bare_Private_Part
  )
   return Boolean
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_51;
--# end
--# property-start PrivatePart._env_names_52 /usr/src/packages/BUILD/ada/ast.py:7591
pragma Warnings (Off, "is not referenced");
 function Env_Names_52
  (Node : Bare_Private_Part
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Private_Part := Bare_Private_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1222;
      Fld : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_1222 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1222;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4721 '<FieldAccess .env_names>' Fld /usr/src/packages/BUILD/ada/ast.py:7591
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PrivatePart.env_names
Fld := Libadalang.Implementation.Private_Part_P_Env_Names (Node => Self);
--# end
--# expr-done 4721
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1222;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1222;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Names_52;
--# end
   --
   --  Primitives for Bare_Public_Part
   --
      procedure Initialize_Fields_For_Public_Part
        (Self : Bare_Public_Part
         ; Declarative_Part_F_Decls : Bare_Ada_Node_List
        ) is
      begin
            Initialize_Fields_For_Declarative_Part
              (Self, Declarative_Part_F_Decls);
      end Initialize_Fields_For_Public_Part;
   --
   --  Primitives for Bare_Elsif_Expr_Part
   --
      procedure Initialize_Fields_For_Elsif_Expr_Part
        (Self : Bare_Elsif_Expr_Part
         ; Elsif_Expr_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Expr_Part_F_Then_Expr : Bare_Expr
        ) is
      begin
            Self.Elsif_Expr_Part_F_Cond_Expr := Elsif_Expr_Part_F_Cond_Expr;
            Self.Elsif_Expr_Part_F_Then_Expr := Elsif_Expr_Part_F_Then_Expr;
      end Initialize_Fields_For_Elsif_Expr_Part;
   function Elsif_Expr_Part_F_Cond_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Cond_Expr;
   end;
   function Elsif_Expr_Part_F_Then_Expr
     (Node : Bare_Elsif_Expr_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Expr_Part_F_Then_Expr;
   end;
   --
   --  Primitives for Bare_Elsif_Stmt_Part
   --
      procedure Initialize_Fields_For_Elsif_Stmt_Part
        (Self : Bare_Elsif_Stmt_Part
         ; Elsif_Stmt_Part_F_Cond_Expr : Bare_Expr
         ; Elsif_Stmt_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Elsif_Stmt_Part_F_Cond_Expr := Elsif_Stmt_Part_F_Cond_Expr;
            Self.Elsif_Stmt_Part_F_Stmts := Elsif_Stmt_Part_F_Stmts;
      end Initialize_Fields_For_Elsif_Stmt_Part;
   function Elsif_Stmt_Part_F_Cond_Expr
     (Node : Bare_Elsif_Stmt_Part) return Bare_Expr
   is
   begin
         return Node.Elsif_Stmt_Part_F_Cond_Expr;
   end;
   function Elsif_Stmt_Part_F_Stmts
     (Node : Bare_Elsif_Stmt_Part) return Bare_Stmt_List
   is
   begin
         return Node.Elsif_Stmt_Part_F_Stmts;
   end;
--# property-start ElsifStmtPart.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:14893
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Entry_Point
  (Node : Bare_Elsif_Stmt_Part
  )
   return Boolean
is
   Self : Bare_Elsif_Stmt_Part := Bare_Elsif_Stmt_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Elsif_Stmt_Part_P_Xref_Entry_Point;
--# end
--# property-start ElsifStmtPart.xref_equation /usr/src/packages/BUILD/ada/ast.py:14896
pragma Warnings (Off, "is not referenced");
 function Elsif_Stmt_Part_P_Xref_Equation
  (Node : Bare_Elsif_Stmt_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Elsif_Stmt_Part := Bare_Elsif_Stmt_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Elsif_Stmt_Part :=
     Internal_Entity_Elsif_Stmt_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1116;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1116 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1116;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4728 '<Block>' Let_Result None
--# scope-start
--# expr-start 4727 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14898
--# expr-start 4723 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14898
--# expr-start 4722 '<FieldAccess .cond_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14898
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Elsif_Stmt_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 4722
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4723
--# expr-start 4726 '<FieldAccess .bool_bind(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14899
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4725 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14899
--# expr-start 4724 '<FieldAccess .cond_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14899
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Elsif_Stmt_Part_F_Cond_Expr;
--# expr-done 4724
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 4725
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 4726
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14898") else null)); 
--# expr-done 4727
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4728
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1116;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1116;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Elsif_Stmt_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Expr
   --
      procedure Initialize_Fields_For_Expr
        (Self : Bare_Expr
        ) is
      begin
      Self.Expr_Logic_Vars := System.Null_Address;
      end Initialize_Fields_For_Expr;
--# property-start Expr.type_val /usr/src/packages/BUILD/ada/ast.py:8640
pragma Warnings (Off, "is not referenced");
 function Expr_P_Type_Val
  (Node : Bare_Expr
  )
   return Internal_Entity
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity;
      Fld : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4730 '<get_value(<FieldAccess .type_var>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:8640
--# expr-start 4729 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:8640
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4729
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 4730
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Type_Val;
--# end
--# property-start Expr.expression_type /usr/src/packages/BUILD/ada/ast.py:8642
pragma Warnings (Off, "is not referenced");
 function Expr_P_Expression_Type
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4734 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:8643
--# expr-start 4733 '<FieldAccess .value>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8643
--# expr-start 4732 '<FieldAccess .logic_val(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8643
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 4731 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:8643
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4731
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 4732
Fld_2 := Fld_1.Value;
--# expr-done 4733
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 4734
      Property_Result := Cast_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Expression_Type;
--# end
--# property-start Expr.is_dynamically_tagged /usr/src/packages/BUILD/ada/ast.py:8654
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dynamically_Tagged
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2259;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Qual_Expr;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Cond_Expr;
Match_3 : Internal_Entity_Decl_Expr;
Match_4 : Internal_Entity_Paren_Expr;
Match_5 : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
Fld_6 : Internal_Entity_Expr;
Fld_7 : Boolean;
Fld_8 : Boolean;
Fld_9 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Fld_12 : Internal_Entity_Expr_Array_Access;
Fld_13 : Boolean;
Quantifier_Result : Boolean;
Fld_14 : Internal_Entity_Expr;
Fld_15 : Boolean;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result_3 : Internal_Entity_Qual_Expr;
Let_Result : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_4 : Internal_Entity_Name;
Let_Result_1 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_5 : Internal_Entity_Cond_Expr;
Let_Result_2 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_6 : Internal_Entity_Decl_Expr;
Let_Result_3 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Internal_Entity_Paren_Expr;
Let_Result_4 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Internal_Entity_Expr;
Let_Result_5 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
If_Result_2 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_6 : Boolean;
            procedure Finalizer_Scope_2259 is
            begin
                     Dec_Ref (Fld_12);
            end Finalizer_Scope_2259;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4757 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 4756 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Or>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:8659
--# expr-start 4735 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:8659
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4735
Bound_Origin := Fld; 
--# expr-start 4737 '<FieldAccess .is_classwide>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8660
--# expr-start 4736 '<FieldAccess .expression_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8660
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_1 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4736
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4737
if Fld_2 then
   If_Result := True;
else
--# expr-start 4739 '<FieldAccess .accessed_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8661
--# expr-start 4738 '<FieldAccess .expression_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8661
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_3 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4738
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld_3.Node, Origin => Bound_Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 4739
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4740 '<FieldAccess .is_classwide>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8661
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_classwide'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Classwide (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4740
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
end if;
if If_Result then
   If_Result_2 := True;
else
   --# expr-start 4755 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:8663
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_3; 
--# bind qual_expr Match_0
--# expr-start 4742 '<FieldAccess .is_dynamically_tagged>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:8665
--# expr-start 4741 '<FieldAccess .suffix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8665
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Expr (Node => Match_0.Node.Qual_Expr_F_Suffix, Info => Match_0.Info);
--# expr-done 4741
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_7 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_6.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_6.Info);
--# end
--# expr-done 4742
Let_Result := Fld_7; 
--# end
Scope_Result_2 := Let_Result;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_4; 
--# bind n Match_1
--# expr-start 4743 '<FieldAccess .is_dispatching_call>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:8670
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_8 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 4743
if Fld_8 then
--# expr-start 4745 '<Cast to BaseSubpSpec>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8671
--# expr-start 4744 '<FieldAccess .called_subp_spec>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:8671
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_9 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4744
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 4745
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_1
--# expr-start 4748 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:8675
--# expr-start 4746 '<FieldAccess .get_primitive_subp_first_type>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:8675
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.get_primitive_subp_first_type
Fld_10 := Libadalang.Implementation.Base_Subp_Spec_P_Get_Primitive_Subp_First_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4746
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
--# expr-start 4747 '<FieldAccess .return_type>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:8675
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_11 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4747
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 4748
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Let_Result_1 := If_Result_1; 
--# end
Scope_Result_3 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Case_Expr .. Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Cond_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_5; 
--# bind cond_expr Match_2
--# expr-start 4749 '<FieldAccess .dependent_exprs>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:8679
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]CondExpr.dependent_exprs'
Fld_12 := Libadalang.Implementation.Dispatcher_Cond_Expr_P_Dependent_Exprs (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4749
Quantifier_Result := True;
   declare
      Collection : constant Internal_Entity_Expr_Array_Access := Fld_12;
   begin
      for Item_165 of
            Collection.Items
      loop
   --# scope-start
            --# bind e Item_165
         --# expr-start 4750 '<FieldAccess .is_dynamically_tagged>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:8680
      if Item_165.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_13 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Item_165.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_165.Info);
--# end
--# expr-done 4750
            if not (Fld_13) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result_2 := Quantifier_Result; 
--# end
Scope_Result_4 := Let_Result_2;
   --# end
      Finalizer_Scope_2259;
            Match_Result := Scope_Result_4; 
         when Ada_Decl_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Decl_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_6; 
--# bind decl_expr Match_3
--# expr-start 4752 '<FieldAccess .is_dynamically_tagged>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:8684
--# expr-start 4751 '<FieldAccess .expr>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:8684
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Expr (Node => Match_3.Node.Decl_Expr_F_Expr, Info => Match_3.Info);
--# expr-done 4751
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_15 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_14.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 4752
Let_Result_3 := Fld_15; 
--# end
Scope_Result_5 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_7; 
--# bind paren_expr Match_4
--# expr-start 4754 '<FieldAccess .is_dynamically_tagged>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:8687
--# expr-start 4753 '<FieldAccess .expr>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:8687
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Match_4.Node.Paren_Expr_F_Expr, Info => Match_4.Info);
--# expr-done 4753
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_17 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 4754
Let_Result_4 := Fld_17; 
--# end
Scope_Result_6 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Case_Expr_Alternative | Ada_Contract_Cases | Ada_Membership_Expr | Ada_Quantified_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_8; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_7 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 4755
   If_Result_2 := Match_Result;
end if;
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 4756
Let_Result_6 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4757
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2259;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Dynamically_Tagged;
--# end
--# property-start '[dispatcher]Expr.potential_actuals_for_dispatch' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.potential_actuals_for_dispatch
                  Property_Result := Expr_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.potential_actuals_for_dispatch
                  Property_Result := Bin_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Target_Name =>
                  --# property-call-start Name.potential_actuals_for_dispatch
                  Property_Result := Name_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.potential_actuals_for_dispatch
                  Property_Result := Un_Op_P_Potential_Actuals_For_Dispatch
                    (Self
                        , Spec
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Expr.is_dispatching_call_impl /usr/src/packages/BUILD/ada/ast.py:8712
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call_Impl
  (Node : Bare_Expr
      ; Decl : Internal_Entity_Basic_Decl
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind decl Decl
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_757;
            procedure Finalizer_Scope_2264;
            procedure Finalizer_Scope_2265;
      Block_Var : Internal_Entity_Base_Subp_Spec;
Block_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_2 : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_3 : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr : Internal_Entity_Type_Expr;
Var_Expr_1 : Internal_Entity_Assign_Stmt;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Base_Subp_Spec;
Fld_2 : Internal_Expected_Type_For_Expr_Array_Access;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity;
Cast_Result : Internal_Entity_Assign_Stmt;
Fld_5 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_1 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result : Internal_Expected_Type_For_Expr_Array_Access;
Fld_6 : Internal_Entity_Type_Expr;
Fld_7 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_757 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Block_Var_3);
            end Finalizer_Scope_757;
            procedure Finalizer_Scope_2264 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2264;
            procedure Finalizer_Scope_2265 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2265;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4773 '<Block>' Let_Result None
--# scope-start
--# expr-start 4759 '<FieldAccess .subp_spec_or_null(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8719
--# expr-start 4758 '<FieldAccess .canonical_part>' Fld /usr/src/packages/BUILD/ada/ast.py:8719
      if Decl.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Decl.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Decl.Info);
--# end
--# expr-done 4758
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld.Node, Follow_Generic => True, E_Info => Fld.Info);
--# end
--# expr-done 4759
Block_Var := Fld_1; 
--# bind spec Block_Var
--# expr-start 4760 '<FieldAccess .potential_actuals_for_dispatch(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8724
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.potential_actuals_for_dispatch'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Potential_Actuals_For_Dispatch (Node => Ent.Node, Spec => Block_Var, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 4760
Block_Var_1 := Fld_2; Inc_Ref (Block_Var_1);
--# bind candidate_actuals Block_Var_1
--# expr-start 4761 '<FieldAccess .returns>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8730
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 4761
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Type_Expr then
   --# scope-start
   --# bind rt Var_Expr
--# expr-start 4763 '<Cast to AssignStmt>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:8730
--# expr-start 4762 '<parent(<Var Ent>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8730
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4762
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Create_Internal_Entity_Assign_Stmt
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Assign_Stmt;
   end if;
--# expr-done 4763
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Assign_Stmt then
   --# scope-start
   --# bind a Var_Expr_1
--# expr-start 4765 '<New InternalExpectedTypeForExpr>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8731
--# expr-start 4764 '<FieldAccess .dest>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8733
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Assign_Stmt_F_Dest, Info => Var_Expr_1.Info);
--# expr-done 4764
      Cast_Result_1 := Create_Internal_Entity_Expr
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
New_Struct := (Expected_Type => Var_Expr, Expr => Cast_Result_1); 
--# expr-done 4765
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2265;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_1 := Result_Var;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2264;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_2 := Result_Var_1; Inc_Ref (Block_Var_2);
--# bind candidate_ret Block_Var_2
--# expr-start 4766 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:8738
Concat_Result := Concat (Block_Var_1, Block_Var_2); 
--# expr-done 4766
Block_Var_3 := Concat_Result; Inc_Ref (Block_Var_3);
--# bind candidates Block_Var_3
Quantifier_Result := False;
   declare
      Collection : constant Internal_Expected_Type_For_Expr_Array_Access := Block_Var_3;
   begin
      for Item_166 of
            Collection.Items
      loop
   --# scope-start
            --# bind c Item_166
--# expr-start 4770 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:8742
--# expr-start 4769 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:8742
--# expr-start 4768 '<FieldAccess .candidate_type_for_primitive(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:8742
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4767 '<FieldAccess .expected_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8743
Fld_6 := Item_166.Expected_Type;
--# expr-done 4767
--# property-call-start BaseSubpSpec.candidate_type_for_primitive
Fld_7 := Libadalang.Implementation.Base_Subp_Spec_P_Candidate_Type_For_Primitive (Node => Block_Var.Node, Type_Expr => Fld_6);
--# end
--# expr-done 4768
Is_Null := Fld_7.Node = null; 
--# expr-done 4769
Not_Val := not (Is_Null); 
--# expr-done 4770
if Not_Val then
   --# expr-start 4772 '<FieldAccess .is_dynamically_tagged>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:8753
--# expr-start 4771 '<FieldAccess .expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:8753
Fld_8 := Item_166.Expr;
--# expr-done 4771
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dynamically_tagged
Fld_9 := Libadalang.Implementation.Expr_P_Is_Dynamically_Tagged (Node => Fld_8.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_8.Info);
--# end
--# expr-done 4772
   If_Result := Fld_9;
else
   If_Result := False;
end if;
            if If_Result then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 4773
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_757;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_757;
               Finalizer_Scope_2264;
               Finalizer_Scope_2265;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Dispatching_Call_Impl;
--# end
--# property-start '[dispatcher]Expr.is_dispatching_call' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Raise_Expr =>
                  --# property-call-start Expr.is_dispatching_call
                  Property_Result := Expr_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Bin_Op .. Ada_Relation_Op =>
                  --# property-call-start BinOp.is_dispatching_call
                  Property_Result := Bin_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Target_Name =>
                  --# property-call-start Name.is_dispatching_call
                  Property_Result := Name_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Un_Op =>
                  --# property-call-start UnOp.is_dispatching_call
                  Property_Result := Un_Op_P_Is_Dispatching_Call
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Is_Dispatching_Call;
--# end
--# property-start Expr.is_static_expr /usr/src/packages/BUILD/ada/ast.py:8778
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Static_Expr
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Match_0 : Internal_Entity_Num_Literal;
Match_1 : Internal_Entity_String_Literal;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Call_Expr;
Match_4 : Internal_Entity_Qual_Expr;
Match_5 : Internal_Entity_Name;
Match_6 : Internal_Entity_Membership_Expr;
Match_7 : Internal_Entity_Bin_Op;
Match_8 : Internal_Entity_Un_Op;
Match_9 : Internal_Entity_If_Expr;
Match_10 : Internal_Entity_Paren_Expr;
Match_11 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Var_Expr_5 : Internal_Entity_Base_Type_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Identifier;
Fld_6 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_10 : Internal_Entity_Identifier;
Fld_11 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Boolean;
Fld_16 : Internal_Entity_Name;
Fld_17 : Internal_Entity_Basic_Decl;
Fld_18 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Fld_19 : Internal_Entity_Identifier;
Fld_20 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_21 : Internal_Entity_Identifier;
Fld_22 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_5 : Boolean;
Fld_23 : Internal_Entity_Identifier;
Fld_24 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_6 : Boolean;
Fld_25 : Internal_Entity_Identifier;
Fld_26 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_7 : Boolean;
If_Result_8 : Boolean;
Fld_27 : Internal_Entity_Assoc_List;
Fld_28 : Bare_Assoc_List;
Fld_29 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_9 : Internal_Entity_Basic_Assoc;
Fld_30 : Internal_Entity_Expr;
Fld_31 : Boolean;
If_Result_10 : Boolean;
If_Result_11 : Boolean;
Fld_32 : Internal_Entity_Name;
Fld_33 : Internal_Entity_Base_Type_Decl;
Fld_34 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Fld_35 : Internal_Entity_Assoc_List;
Fld_36 : Bare_Assoc_List;
Fld_37 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_12 : Internal_Entity_Basic_Assoc;
Fld_38 : Internal_Entity_Expr;
Fld_39 : Boolean;
If_Result_13 : Boolean;
Fld_40 : Internal_Entity_Name;
Fld_41 : Internal_Entity_Base_Type_Decl;
Fld_42 : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Fld_43 : Internal_Entity_Expr;
Fld_44 : Boolean;
If_Result_14 : Boolean;
Fld_45 : Internal_Entity_Basic_Decl;
Fld_46 : Boolean;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Boolean;
Fld_49 : Internal_Entity_Expr_Alternatives_List;
Fld_50 : Bare_Expr_Alternatives_List;
Fld_51 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Expr;
Is_Null_2 : Boolean;
If_Result_15 : Internal_Entity_Expr;
Fld_52 : Boolean;
Item_167 : Internal_Entity_Expr;
Bare_Item_167 : Bare_Expr;
Quantifier_Result : Boolean;
If_Result_16 : Boolean;
Fld_53 : Internal_Entity_Expr;
Fld_54 : Boolean;
Fld_55 : Internal_Entity_Expr;
Fld_56 : Boolean;
If_Result_17 : Boolean;
Fld_57 : Internal_Entity_Op;
Fld_58 : Internal_Entity_Basic_Decl;
Is_Null_3 : Boolean;
If_Result_18 : Boolean;
Fld_59 : Internal_Entity_Expr;
Fld_60 : Boolean;
Fld_61 : Internal_Entity_Op;
Fld_62 : Internal_Entity_Basic_Decl;
Is_Null_4 : Boolean;
If_Result_19 : Boolean;
Fld_63 : Internal_Entity_Expr;
Fld_64 : Boolean;
Fld_65 : Internal_Entity_Expr;
Fld_66 : Boolean;
If_Result_20 : Boolean;
Fld_67 : Internal_Entity_Elsif_Expr_Part_List;
Fld_68 : Bare_Elsif_Expr_Part_List;
Fld_69 : Internal_Entity_Info;
As_Entity_3 : Internal_Entity_Elsif_Expr_Part;
Is_Null_5 : Boolean;
If_Result_21 : Internal_Entity_Elsif_Expr_Part;
Fld_70 : Internal_Entity_Expr;
Fld_71 : Boolean;
Fld_72 : Internal_Entity_Expr;
Fld_73 : Boolean;
If_Result_22 : Boolean;
Item_168 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_168 : Bare_Elsif_Expr_Part;
Quantifier_Result_1 : Boolean;
If_Result_23 : Boolean;
Fld_74 : Internal_Entity_Expr;
Fld_75 : Boolean;
If_Result_24 : Boolean;
Fld_76 : Internal_Entity_Expr;
Fld_77 : Boolean;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Num_Literal;
Let_Result : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_1 : Internal_Entity_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_2 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_3 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_4 : Internal_Entity_Qual_Expr;
Let_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_5 : Boolean;
Scope_Result_11 : Boolean;
Cast_Result_6 : Internal_Entity_Membership_Expr;
Let_Result_6 : Boolean;
Scope_Result_12 : Boolean;
Cast_Result_7 : Internal_Entity_Bin_Op;
Let_Result_7 : Boolean;
Scope_Result_13 : Boolean;
Cast_Result_8 : Internal_Entity_Un_Op;
Let_Result_8 : Boolean;
Scope_Result_14 : Boolean;
Cast_Result_9 : Internal_Entity_If_Expr;
Let_Result_9 : Boolean;
Scope_Result_15 : Boolean;
Cast_Result_10 : Internal_Entity_Paren_Expr;
Let_Result_10 : Boolean;
Scope_Result_16 : Boolean;
Cast_Result_11 : Internal_Entity_Expr;
Let_Result_11 : Boolean;
Scope_Result_17 : Boolean;
Match_Result : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_12 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4865 '<Block>' Let_Result_12 None
--# scope-start
--# expr-start 4864 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:8783
--# expr-start 4774 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:8783
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4774
Bound_Origin := Fld; 
--# expr-start 4863 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:8783
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Int_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Num_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result_6 := Let_Result;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_String_Literal
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := True; 
--# end
Scope_Result_7 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind ar Match_2
--# expr-start 4779 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:8788
--# expr-start 4776 '<FieldAccess .name_designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8788
--# expr-start 4775 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8788
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4775
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4776
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4777 '<FieldAccess .root_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8788
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.root_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Root_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4777
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4778 '<FieldAccess .is_formal>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8788
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_formal
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Is_Formal (Node => Var_Expr.Node);
--# end
--# expr-done 4778
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Not_Val := not (Result_Var_1); 
--# expr-done 4779
if Not_Val then
   --# expr-start 4783 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:8790
--# expr-start 4781 '<FieldAccess .name_symbol>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:8790
--# expr-start 4780 '<FieldAccess .attribute>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8790
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4780
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_5.Node);
--# end
--# expr-done 4781
--# expr-start 4782 '<Symbol Base>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Base); 
--# expr-done 4782
Is_Equal := Fld_6 = Sym; 
--# expr-done 4783
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   If_Result_4 := True;
else
--# expr-start 4785 '<FieldAccess .name_designated_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:8792
--# expr-start 4784 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:8792
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4784
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 4785
Var_Expr_2 := Fld_8;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4786 '<FieldAccess .is_static_decl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:8792
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 4786
Scope_Result_2 := Fld_9;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
--# expr-start 4790 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4788 '<FieldAccess .name_symbol>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4787 '<FieldAccess .attribute>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:8793
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4787
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_10.Node);
--# end
--# expr-done 4788
--# expr-start 4789 '<Symbol First>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 4789
Is_Equal_1 := Fld_11 = Sym_1; 
--# expr-done 4790
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 4794 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4792 '<FieldAccess .name_symbol>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4791 '<FieldAccess .attribute>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:8793
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4791
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 4792
--# expr-start 4793 '<Symbol Last>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 4793
Is_Equal_2 := Fld_13 = Sym_2; 
--# expr-done 4794
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4798 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4796 '<FieldAccess .name_symbol>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:8793
--# expr-start 4795 '<FieldAccess .attribute>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:8793
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4795
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 4796
--# expr-start 4797 '<Symbol Range>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 4797
Is_Equal_3 := Fld_15 = Sym_3; 
--# expr-done 4798
   If_Result_2 := Is_Equal_3;
end if;
   If_Result_3 := If_Result_2;
else
   If_Result_3 := False;
end if;
   If_Result_4 := If_Result_3;
end if;
if If_Result_4 then
   If_Result_11 := True;
else
--# expr-start 4800 '<FieldAccess .referenced_decl>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:8796
--# expr-start 4799 '<FieldAccess .prefix>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:8796
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 4799
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_17 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_16.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_16.Info);
--# end
--# expr-done 4800
Var_Expr_3 := Fld_17;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 4801 '<FieldAccess .is_array>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:8796
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_18 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Var_Expr_3.Node, Origin => Bound_Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 4801
Scope_Result_3 := Fld_18;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 4805 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4803 '<FieldAccess .name_symbol>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4802 '<FieldAccess .attribute>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:8797
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4802
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_19.Node);
--# end
--# expr-done 4803
--# expr-start 4804 '<Symbol First>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 4804
Is_Equal_4 := Fld_20 = Sym_4; 
--# expr-done 4805
if Is_Equal_4 then
   If_Result_5 := True;
else
   --# expr-start 4809 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4807 '<FieldAccess .name_symbol>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4806 '<FieldAccess .attribute>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:8797
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4806
      if Fld_21.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_22 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_21.Node);
--# end
--# expr-done 4807
--# expr-start 4808 '<Symbol Last>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 4808
Is_Equal_5 := Fld_22 = Sym_5; 
--# expr-done 4809
   If_Result_5 := Is_Equal_5;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 4813 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4811 '<FieldAccess .name_symbol>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4810 '<FieldAccess .attribute>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:8797
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4810
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_24 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_23.Node);
--# end
--# expr-done 4811
--# expr-start 4812 '<Symbol Length>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 4812
Is_Equal_6 := Fld_24 = Sym_6; 
--# expr-done 4813
   If_Result_6 := Is_Equal_6;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 4817 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4815 '<FieldAccess .name_symbol>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:8797
--# expr-start 4814 '<FieldAccess .attribute>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:8797
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 4814
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_26 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_25.Node);
--# end
--# expr-done 4815
--# expr-start 4816 '<Symbol Range>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 4816
Is_Equal_7 := Fld_26 = Sym_7; 
--# expr-done 4817
   If_Result_7 := Is_Equal_7;
end if;
   If_Result_8 := If_Result_7;
else
   If_Result_8 := False;
end if;
if If_Result_8 then
   --# expr-start 4820 '<FieldAccess .is_static_expr>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:8799
--# expr-start 4819 '<FieldAccess .expr>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:8799
--# expr-start 4818 '<FieldAccess .args_list>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:8799
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_27 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Match_2.Node, E_Info => Match_2.Info);
--# end
--# expr-done 4818
Fld_28 := Fld_27.Node;
Get_Result := Get (Fld_28, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_9 := No_Entity_Basic_Assoc;
else
Fld_29 := Fld_27.Info;
As_Entity := (Info => Fld_29, Node => Get_Result); 
   If_Result_9 := As_Entity;
end if;
      if If_Result_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_30 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_9.Node, E_Info => If_Result_9.Info);
--# end
--# expr-done 4819
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_31 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_30.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_30.Info);
--# end
--# expr-done 4820
   If_Result_10 := Fld_31;
else
   If_Result_10 := False;
end if;
   If_Result_11 := If_Result_10;
end if;
Let_Result_2 := If_Result_11; 
--# end
Scope_Result_8 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind ce Match_3
--# expr-start 4822 '<FieldAccess .name_designated_type>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:8802
--# expr-start 4821 '<FieldAccess .name>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:8802
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Name (Node => Match_3.Node.Call_Expr_F_Name, Info => Match_3.Info);
--# expr-done 4821
      if Fld_32.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_33 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_32.Node, E_Info => Fld_32.Info);
--# end
--# expr-done 4822
Var_Expr_4 := Fld_33;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4823 '<FieldAccess .is_static_decl>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:8802
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_4.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 4823
Scope_Result_4 := Fld_34;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
if Result_Var_4 then
   --# expr-start 4826 '<FieldAccess .is_static_expr>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:8803
--# expr-start 4825 '<FieldAccess .expr>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:8803
--# expr-start 4824 '<FieldAccess .params>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:8803
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_35 := Libadalang.Implementation.Call_Expr_P_Params (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 4824
Fld_36 := Fld_35.Node;
Get_Result_1 := Get (Fld_36, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_12 := No_Entity_Basic_Assoc;
else
Fld_37 := Fld_35.Info;
As_Entity_1 := (Info => Fld_37, Node => Get_Result_1); 
   If_Result_12 := As_Entity_1;
end if;
      if If_Result_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_12.Node, E_Info => If_Result_12.Info);
--# end
--# expr-done 4825
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_39 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_38.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_38.Info);
--# end
--# expr-done 4826
   If_Result_13 := Fld_39;
else
   If_Result_13 := False;
end if;
Let_Result_3 := If_Result_13; 
--# end
Scope_Result_9 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_4; 
--# bind qe Match_4
--# expr-start 4828 '<FieldAccess .name_designated_type>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:8805
--# expr-start 4827 '<FieldAccess .prefix>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:8805
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Name (Node => Match_4.Node.Qual_Expr_F_Prefix, Info => Match_4.Info);
--# expr-done 4827
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_41 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_40.Node, E_Info => Fld_40.Info);
--# end
--# expr-done 4828
Var_Expr_5 := Fld_41;
if Var_Expr_5 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 4829 '<FieldAccess .is_static_decl>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:8805
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_42 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Var_Expr_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 4829
Scope_Result_5 := Fld_42;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 4831 '<FieldAccess .is_static_expr>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:8806
--# expr-start 4830 '<FieldAccess .suffix>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:8806
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_Expr (Node => Match_4.Node.Qual_Expr_F_Suffix, Info => Match_4.Info);
--# expr-done 4830
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_44 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_43.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_43.Info);
--# end
--# expr-done 4831
   If_Result_14 := Fld_44;
else
   If_Result_14 := False;
end if;
Let_Result_4 := If_Result_14; 
--# end
Scope_Result_10 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Defining_Name .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Op_Xor | Ada_Null_Literal | Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_5; 
--# bind n Match_5
--# expr-start 4833 '<FieldAccess .is_static_decl>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:8807
--# expr-start 4832 '<FieldAccess .referenced_decl>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:8807
      if Match_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_45 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Match_5.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_5.Info);
--# end
--# expr-done 4832
      if Fld_45.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_46 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_45.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_45.Info);
--# end
--# expr-done 4833
Let_Result_5 := Fld_46; 
--# end
Scope_Result_11 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Membership_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Membership_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_6 := Cast_Result_6; 
--# bind me Match_6
--# expr-start 4835 '<FieldAccess .is_static_expr>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:8809
--# expr-start 4834 '<FieldAccess .expr>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:8809
      if Match_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Create_Internal_Entity_Expr (Node => Match_6.Node.Membership_Expr_F_Expr, Info => Match_6.Info);
--# expr-done 4834
      if Fld_47.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_48 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_47.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_47.Info);
--# end
--# expr-done 4835
if Fld_48 then
--# expr-start 4836 '<FieldAccess .membership_exprs>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:8809
      if Match_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Create_Internal_Entity_Expr_Alternatives_List (Node => Match_6.Node.Membership_Expr_F_Membership_Exprs, Info => Match_6.Info);
--# expr-done 4836
Fld_50 := Fld_49.Node;
Quantifier_Result := True;
   if Fld_50 /= null then
   declare
      Collection : constant Bare_Expr_Alternatives_List := Fld_50;
   begin
      for Untyped_Item_167 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_167 := Untyped_Item_167; 
Is_Null_2 := Bare_Item_167 = null; 
if Is_Null_2 then
   If_Result_15 := No_Entity_Expr;
else
Fld_51 := Fld_49.Info;
As_Entity_2 := (Info => Fld_51, Node => Bare_Item_167); 
   If_Result_15 := As_Entity_2;
end if;
               Item_167 := If_Result_15; 
   --# scope-start
            --# bind e Item_167
         --# expr-start 4837 '<FieldAccess .is_static_expr>' Fld_52 /usr/src/packages/BUILD/ada/ast.py:8810
      if Item_167.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_52 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Item_167.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Item_167.Info);
--# end
--# expr-done 4837
            if not (Fld_52) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
   If_Result_16 := Quantifier_Result;
else
   If_Result_16 := False;
end if;
Let_Result_6 := If_Result_16; 
--# end
Scope_Result_12 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_7 := Cast_Result_7; 
--# bind bo Match_7
--# expr-start 4839 '<FieldAccess .is_static_expr>' Fld_54 /usr/src/packages/BUILD/ada/ast.py:8813
--# expr-start 4838 '<FieldAccess .left>' Fld_53 /usr/src/packages/BUILD/ada/ast.py:8813
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Left, Info => Match_7.Info);
--# expr-done 4838
      if Fld_53.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_54 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_53.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_53.Info);
--# end
--# expr-done 4839
if Fld_54 then
   --# expr-start 4841 '<FieldAccess .is_static_expr>' Fld_56 /usr/src/packages/BUILD/ada/ast.py:8814
--# expr-start 4840 '<FieldAccess .right>' Fld_55 /usr/src/packages/BUILD/ada/ast.py:8814
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Create_Internal_Entity_Expr (Node => Match_7.Node.Bin_Op_F_Right, Info => Match_7.Info);
--# expr-done 4840
      if Fld_55.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_56 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_55.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_55.Info);
--# end
--# expr-done 4841
   If_Result_17 := Fld_56;
else
   If_Result_17 := False;
end if;
if If_Result_17 then
   --# expr-start 4844 '<IsNull>' Is_Null_3 /usr/src/packages/BUILD/ada/ast.py:8815
--# expr-start 4843 '<FieldAccess .referenced_decl>' Fld_58 /usr/src/packages/BUILD/ada/ast.py:8815
--# expr-start 4842 '<FieldAccess .op>' Fld_57 /usr/src/packages/BUILD/ada/ast.py:8815
      if Match_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_57 := Create_Internal_Entity_Op (Node => Match_7.Node.Bin_Op_F_Op, Info => Match_7.Info);
--# expr-done 4842
      if Fld_57.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_58 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_57.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_57.Info);
--# end
--# expr-done 4843
Is_Null_3 := Fld_58.Node = null; 
--# expr-done 4844
   If_Result_18 := Is_Null_3;
else
   If_Result_18 := False;
end if;
Let_Result_7 := If_Result_18; 
--# end
Scope_Result_13 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity_Un_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_8 := Cast_Result_8; 
--# bind uo Match_8
--# expr-start 4846 '<FieldAccess .is_static_expr>' Fld_60 /usr/src/packages/BUILD/ada/ast.py:8817
--# expr-start 4845 '<FieldAccess .expr>' Fld_59 /usr/src/packages/BUILD/ada/ast.py:8817
      if Match_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Create_Internal_Entity_Expr (Node => Match_8.Node.Un_Op_F_Expr, Info => Match_8.Info);
--# expr-done 4845
      if Fld_59.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_60 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_59.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_59.Info);
--# end
--# expr-done 4846
if Fld_60 then
   --# expr-start 4849 '<IsNull>' Is_Null_4 /usr/src/packages/BUILD/ada/ast.py:8818
--# expr-start 4848 '<FieldAccess .referenced_decl>' Fld_62 /usr/src/packages/BUILD/ada/ast.py:8818
--# expr-start 4847 '<FieldAccess .op>' Fld_61 /usr/src/packages/BUILD/ada/ast.py:8818
      if Match_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Create_Internal_Entity_Op (Node => Match_8.Node.Un_Op_F_Op, Info => Match_8.Info);
--# expr-done 4847
      if Fld_61.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_62 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld_61.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_61.Info);
--# end
--# expr-done 4848
Is_Null_4 := Fld_62.Node = null; 
--# expr-done 4849
   If_Result_19 := Is_Null_4;
else
   If_Result_19 := False;
end if;
Let_Result_8 := If_Result_19; 
--# end
Scope_Result_14 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_If_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_If_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_9 := Cast_Result_9; 
--# bind i Match_9
--# expr-start 4851 '<FieldAccess .is_static_expr>' Fld_64 /usr/src/packages/BUILD/ada/ast.py:8820
--# expr-start 4850 '<FieldAccess .cond_expr>' Fld_63 /usr/src/packages/BUILD/ada/ast.py:8820
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_63 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Cond_Expr, Info => Match_9.Info);
--# expr-done 4850
      if Fld_63.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_64 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_63.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_63.Info);
--# end
--# expr-done 4851
if Fld_64 then
   --# expr-start 4853 '<FieldAccess .is_static_expr>' Fld_66 /usr/src/packages/BUILD/ada/ast.py:8820
--# expr-start 4852 '<FieldAccess .then_expr>' Fld_65 /usr/src/packages/BUILD/ada/ast.py:8820
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_65 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Then_Expr, Info => Match_9.Info);
--# expr-done 4852
      if Fld_65.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_66 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_65.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_65.Info);
--# end
--# expr-done 4853
   If_Result_20 := Fld_66;
else
   If_Result_20 := False;
end if;
if If_Result_20 then
--# expr-start 4854 '<FieldAccess .alternatives>' Fld_67 /usr/src/packages/BUILD/ada/ast.py:8821
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_67 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Match_9.Node.If_Expr_F_Alternatives, Info => Match_9.Info);
--# expr-done 4854
Fld_68 := Fld_67.Node;
Quantifier_Result_1 := True;
   if Fld_68 /= null then
   declare
      Collection : constant Bare_Elsif_Expr_Part_List := Fld_68;
   begin
      for Untyped_Item_168 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_168 := Untyped_Item_168; 
Is_Null_5 := Bare_Item_168 = null; 
if Is_Null_5 then
   If_Result_21 := No_Entity_Elsif_Expr_Part;
else
Fld_69 := Fld_67.Info;
As_Entity_3 := (Info => Fld_69, Node => Bare_Item_168); 
   If_Result_21 := As_Entity_3;
end if;
               Item_168 := If_Result_21; 
   --# scope-start
            --# bind a Item_168
--# expr-start 4856 '<FieldAccess .is_static_expr>' Fld_71 /usr/src/packages/BUILD/ada/ast.py:8823
--# expr-start 4855 '<FieldAccess .cond_expr>' Fld_70 /usr/src/packages/BUILD/ada/ast.py:8823
      if Item_168.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_70 := Create_Internal_Entity_Expr (Node => Item_168.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_168.Info);
--# expr-done 4855
      if Fld_70.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_71 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_70.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_70.Info);
--# end
--# expr-done 4856
if Fld_71 then
   --# expr-start 4858 '<FieldAccess .is_static_expr>' Fld_73 /usr/src/packages/BUILD/ada/ast.py:8824
--# expr-start 4857 '<FieldAccess .then_expr>' Fld_72 /usr/src/packages/BUILD/ada/ast.py:8824
      if Item_168.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_72 := Create_Internal_Entity_Expr (Node => Item_168.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_168.Info);
--# expr-done 4857
      if Fld_72.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_73 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_72.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_72.Info);
--# end
--# expr-done 4858
   If_Result_22 := Fld_73;
else
   If_Result_22 := False;
end if;
            if not (If_Result_22) then
               Quantifier_Result_1 := False;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
   If_Result_23 := Quantifier_Result_1;
else
   If_Result_23 := False;
end if;
if If_Result_23 then
   --# expr-start 4860 '<FieldAccess .is_static_expr>' Fld_75 /usr/src/packages/BUILD/ada/ast.py:8826
--# expr-start 4859 '<FieldAccess .else_expr>' Fld_74 /usr/src/packages/BUILD/ada/ast.py:8826
      if Match_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_74 := Create_Internal_Entity_Expr (Node => Match_9.Node.If_Expr_F_Else_Expr, Info => Match_9.Info);
--# expr-done 4859
      if Fld_74.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_75 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_74.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_74.Info);
--# end
--# expr-done 4860
   If_Result_24 := Fld_75;
else
   If_Result_24 := False;
end if;
Let_Result_9 := If_Result_24; 
--# end
Scope_Result_15 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Paren_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Paren_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_10 := Cast_Result_10; 
--# bind pe Match_10
--# expr-start 4862 '<FieldAccess .is_static_expr>' Fld_77 /usr/src/packages/BUILD/ada/ast.py:8827
--# expr-start 4861 '<FieldAccess .expr>' Fld_76 /usr/src/packages/BUILD/ada/ast.py:8827
      if Match_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_76 := Create_Internal_Entity_Expr (Node => Match_10.Node.Paren_Expr_F_Expr, Info => Match_10.Info);
--# expr-done 4861
      if Fld_76.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_77 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_76.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_76.Info);
--# end
--# expr-done 4862
Let_Result_10 := Fld_77; 
--# end
Scope_Result_16 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Case_Expr | Ada_Contract_Cases .. Ada_Decl_Expr | Ada_Quantified_Expr .. Ada_Raise_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_11 := Cast_Result_11; 
--# bind _ Match_11
Let_Result_11 := False; 
--# end
Scope_Result_17 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_17; 
end case;
--# expr-done 4863
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 4864
Let_Result_12 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4865
      Property_Result := Let_Result_12;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Static_Expr;
--# end
--# property-start '[dispatcher]Expr.first_corresponding_decl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
                  --# property-call-start Expr.first_corresponding_decl
                  Property_Result := Expr_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Target_Name =>
                  --# property-call-start Name.first_corresponding_decl
                  Property_Result := Name_P_First_Corresponding_Decl
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.eval_as_int /usr/src/packages/BUILD/ada/ast.py:8840
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_Int
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Big_Integer_Type
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Big_Integer_Type;
            procedure Finalizer_Scope_761;
      Fld : Big_Integer_Type;
Let_Result : Big_Integer_Type;
            procedure Finalizer_Scope_761 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_761;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4867 '<Block>' Let_Result None
--# scope-start
--# expr-start 4866 '<FieldAccess .eval_as_int_in_env(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:8856
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_Int_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 4866
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4867
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_761;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_761;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Eval_As_Int;
--# end
--# property-start Expr.eval_as_string /usr/src/packages/BUILD/ada/ast.py:8880
pragma Warnings (Off, "is not referenced");
 function Expr_P_Eval_As_String
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_764;
      Fld : Character_Type_Array_Access;
Let_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_764 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_764;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4869 '<Block>' Let_Result None
--# scope-start
--# expr-start 4868 '<FieldAccess .eval_as_string_in_env(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:8896
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Eval_As_String_In_Env (Node => Ent.Node, Env => No_Internal_Substitution_Array_Type, E_Info => Ent.Info);
--# expr-done 4868
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4869
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_764;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_764;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Eval_As_String;
--# end
--# property-start Expr.discrete_range /usr/src/packages/BUILD/ada/ast.py:8920
pragma Warnings (Off, "is not referenced");
 function Expr_P_Discrete_Range
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Match_0 : Internal_Entity_Attribute_Ref;
Match_1 : Internal_Entity_Name;
Match_2 : Internal_Entity_Bin_Op;
Match_3 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Discrete_Range;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Match_Prefix : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Attribute_Ref;
Let_Result : Internal_Discrete_Range;
Scope_Result_1 : Internal_Discrete_Range;
Cast_Result_1 : Internal_Entity_Name;
Let_Result_1 : Internal_Discrete_Range;
Scope_Result_2 : Internal_Discrete_Range;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result_2 : Internal_Discrete_Range;
Scope_Result_3 : Internal_Discrete_Range;
Cast_Result_3 : Internal_Entity_Expr;
Let_Result_3 : Internal_Discrete_Range;
Scope_Result_4 : Internal_Discrete_Range;
Match_Result : Internal_Discrete_Range;
Let_Result_4 : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4878 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 4877 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:8924
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Expr (Match_Prefix.Node.Kind) is
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 4871 '<FieldAccess .discrete_range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8926
--# expr-start 4870 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:8926
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Attribute_Ref_F_Prefix, Info => Match_0.Info);
--# expr-done 4870
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_1 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4871
Let_Result := Fld_1; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Call_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind n Match_1
--# expr-start 4872 '<FieldAccess .name_designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:8928
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4872
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind dt Var_Expr
--# expr-start 4873 '<FieldAccess .discrete_range>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:8929
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 4873
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Discrete_Range;
end if;
Let_Result_1 := Result_Var; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind bo Match_2
--# expr-start 4876 '<New InternalDiscreteRange>' New_Struct /usr/src/packages/BUILD/ada/ast.py:8932
--# expr-start 4874 '<FieldAccess .right>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:8933
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Right, Info => Match_2.Info);
--# expr-done 4874
--# expr-start 4875 '<FieldAccess .left>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:8933
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Match_2.Node.Bin_Op_F_Left, Info => Match_2.Info);
--# expr-done 4875
New_Struct := (High_Bound => Fld_5, Low_Bound => Fld_4); 
--# expr-done 4876
Let_Result_2 := New_Struct; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Abstract_State_Decl_Expr .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := No_Discrete_Range; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
end case;
--# expr-done 4877
Let_Result_4 := Match_Result; 
--# end
--# expr-done 4878
      Property_Result := Let_Result_4;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Discrete_Range;
--# end
--# property-start '[dispatcher]Expr.designated_env_no_overloading' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env_no_overloading
                  Property_Result := Expr_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env_no_overloading
                  Property_Result := Dotted_Name_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env_no_overloading
                  Property_Result := Base_Id_P_Designated_Env_No_Overloading
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Designated_Env_No_Overloading;
--# end
--# property-start '[dispatcher]Expr.designated_env' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.designated_env
                  Property_Result := Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_env
                  Property_Result := Attribute_Ref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_env
                  Property_Result := Call_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_env
                  Property_Result := Dotted_Name_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.designated_env
                  Property_Result := Explicit_Deref_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.designated_env
                  Property_Result := Qual_Expr_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_env
                  Property_Result := Base_Id_P_Designated_Env
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Designated_Env;
--# end
--# property-start Expr.env_elements /usr/src/packages/BUILD/ada/ast.py:8957
pragma Warnings (Off, "is not referenced");
 function Expr_P_Env_Elements
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_769;
      Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_769 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_769;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4881 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:8958
--# expr-start 4879 '<FieldAccess .env_elements_impl>' Fld /usr/src/packages/BUILD/ada/ast.py:8958
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4879
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_169 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_169
               --# expr-start 4880 '<FieldAccess .has_visibility(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8958
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_169);
--# end
--# expr-done 4880
               if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_169;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4881
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_769;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_769;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Env_Elements;
--# end
--# property-start '[dispatcher]Expr.env_elements_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Expr (Self.Kind) is
               when Ada_Abstract_State_Decl_Expr .. Ada_Membership_Expr | Ada_Discrete_Subtype_Name | Ada_Null_Literal .. Ada_Un_Op =>
                  --# property-call-start Expr.env_elements_impl
                  Property_Result := Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.env_elements_impl
                  Property_Result := Attribute_Ref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.env_elements_impl
                  Property_Result := Call_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.env_elements_impl
                  Property_Result := Defining_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.env_elements_impl
                  Property_Result := Dotted_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.env_elements_impl
                  Property_Result := End_Name_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.env_elements_impl
                  Property_Result := Explicit_Deref_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.env_elements_impl
                  Property_Result := Qual_Expr_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.env_elements_impl
                  Property_Result := Base_Id_P_Env_Elements_Impl
                    (Self
                        , Env
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Expr_P_Env_Elements_Impl;
--# end
--# property-start Expr.matching_nodes /usr/src/packages/BUILD/ada/ast.py:8973
pragma Warnings (Off, "is not referenced");
 function Expr_P_Matching_Nodes
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_771;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_771 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_771;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4885 '<Block>' Let_Result None
--# scope-start
--# expr-start 4884 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .env_elements>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:8978
--# expr-start 4882 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8978
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 4882
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 4883 '<FieldAccess .env_elements>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:8978
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Bound_Env, E_Info => Ent.Info);
--# end
--# expr-done 4883
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4884
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 4885
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_771;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_771;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Matching_Nodes;
--# end
 function Expr_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Expr
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is (raise Property_Error
    with "Property Expr.potential_actuals_for_dispatch not implemented on type "
    & Kind_Name (Node));
--# property-start Expr.is_dispatching_call /usr/src/packages/BUILD/ada/ast.py:8759
pragma Warnings (Off, "is not referenced");
 function Expr_P_Is_Dispatching_Call
  (Node : Bare_Expr
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4886 '<Block>' Let_Result None
--# scope-start
Let_Result := raise Property_Error with "Invalid node type: expected Name, UnOp or BinOp"; 
--# end
--# expr-done 4886
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Is_Dispatching_Call;
--# end
--# property-start Expr.first_corresponding_decl /usr/src/packages/BUILD/ada/ast.py:8832
pragma Warnings (Off, "is not referenced");
 function Expr_P_First_Corresponding_Decl
  (Node : Bare_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
      Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 4887 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Basic_Decl; 
--# end
--# expr-done 4887
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_First_Corresponding_Decl;
--# end
--# property-start Expr.designated_env_no_overloading /usr/src/packages/BUILD/ada/ast.py:8939
pragma Warnings (Off, "is not referenced");
 function Expr_P_Designated_Env_No_Overloading
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Expr := Bare_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Expr :=
     Internal_Entity_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_767;
      Fld : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_767 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_767;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4889 '<Block>' Let_Result None
--# scope-start
--# expr-start 4888 '<FieldAccess .designated_env>' Fld /usr/src/packages/BUILD/ada/ast.py:8944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4888
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 4889
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_767;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_767;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Expr_P_Designated_Env_No_Overloading;
--# end
 function Expr_P_Designated_Env
  (Node : Bare_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Expr.designated_env not implemented on type "
    & Kind_Name (Node));
 function Expr_P_Env_Elements_Impl
  (Node : Bare_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Expr.env_elements_impl not implemented on type "
    & Kind_Name (Node));
   --
   --  Primitives for Bare_Abstract_State_Decl_Expr
   --
      procedure Initialize_Fields_For_Abstract_State_Decl_Expr
        (Self : Bare_Abstract_State_Decl_Expr
         ; Abstract_State_Decl_Expr_F_State_Decl : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Abstract_State_Decl_Expr_F_State_Decl := Abstract_State_Decl_Expr_F_State_Decl;
      end Initialize_Fields_For_Abstract_State_Decl_Expr;
   function Abstract_State_Decl_Expr_F_State_Decl
     (Node : Bare_Abstract_State_Decl_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Abstract_State_Decl_Expr_F_State_Decl;
   end;
--# property-start AbstractStateDeclExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:15855
pragma Warnings (Off, "is not referenced");
 function Abstract_State_Decl_Expr_P_Xref_Equation
  (Node : Bare_Abstract_State_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abstract_State_Decl_Expr := Bare_Abstract_State_Decl_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1168;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1168 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1168;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15855") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1168;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1168;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abstract_State_Decl_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Allocator
   --
      procedure Initialize_Fields_For_Allocator
        (Self : Bare_Allocator
         ; Allocator_F_Subpool : Bare_Name
         ; Allocator_F_Type_Or_Expr : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Allocator_F_Subpool := Allocator_F_Subpool;
            Self.Allocator_F_Type_Or_Expr := Allocator_F_Type_Or_Expr;
      end Initialize_Fields_For_Allocator;
   function Allocator_F_Subpool
     (Node : Bare_Allocator) return Bare_Name
   is
   begin
         return Node.Allocator_F_Subpool;
   end;
   function Allocator_F_Type_Or_Expr
     (Node : Bare_Allocator) return Bare_Ada_Node
   is
   begin
         return Node.Allocator_F_Type_Or_Expr;
   end;
--# property-start Allocator.get_allocated_type /usr/src/packages/BUILD/ada/ast.py:13255
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Get_Allocated_Type
  (Node : Bare_Allocator
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Allocator := Bare_Allocator (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Allocator :=
     Internal_Entity_Allocator'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Subtype_Indication;
Match_1 : Internal_Entity_Qual_Expr;
Match_2 : Internal_Entity;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Subtype_Indication;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Qual_Expr;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4896 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 4895 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Match>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:13259
--# expr-start 4890 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:13259
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4890
Bound_Origin := Fld; 
--# expr-start 4894 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:13259
--# expr-start 4891 '<FieldAccess .type_or_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13259
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 4891
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
--# expr-start 4892 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13260
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_0.Node, Origin => Bound_Origin, E_Info => Match_0.Info);
--# end
--# expr-done 4892
Let_Result := Fld_2; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind q Match_1
--# expr-start 4893 '<FieldAccess .designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13261
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.designated_type
Fld_3 := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 4893
Let_Result_1 := Fld_3; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Explicit_Deref | Ada_Char_Literal .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 4894
Dyn_Var_Bind_Result := Match_Result; 
--# expr-done 4895
Let_Result_3 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 4896
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Allocator_P_Get_Allocated_Type;
--# end
--# property-start Allocator.xref_equation /usr/src/packages/BUILD/ada/ast.py:13266
pragma Warnings (Off, "is not referenced");
 function Allocator_P_Xref_Equation
  (Node : Bare_Allocator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Allocator := Bare_Allocator (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Allocator :=
     Internal_Entity_Allocator'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1016;
      Fld : Internal_Entity;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Base_Type_Decl;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1016 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1016;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4903 '<Block>' Let_Result None
--# scope-start
--# expr-start 4902 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13268
--# expr-start 4898 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13268
--# expr-start 4897 '<FieldAccess .type_or_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:13268
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Allocator_F_Type_Or_Expr, Info => Ent.Info);
--# expr-done 4897
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 4898
--# expr-start 4901 '<Predicate on BaseTypeDecl.matching_allocator_type>' Pred /usr/src/packages/BUILD/ada/ast.py:13269
--# expr-start 4899 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13270
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4899
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 4900 '<FieldAccess .get_allocated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13270
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Allocator.get_allocated_type
Fld_3 := Libadalang.Implementation.Allocator_P_Get_Allocated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 4900
Pred := Base_Type_Decl_P_Matching_Allocator_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Matching_Allocator_Type_0_Predicate (Fld_3, Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Matching_Allocator_Type")            else null))); 
--# expr-done 4901
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13268") else null)); 
--# expr-done 4902
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 4903
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1016;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1016;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Allocator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Base_Aggregate
   --
      procedure Initialize_Fields_For_Base_Aggregate
        (Self : Bare_Base_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Base_Aggregate_F_Ancestor_Expr := Base_Aggregate_F_Ancestor_Expr;
            Self.Base_Aggregate_F_Assocs := Base_Aggregate_F_Assocs;
      end Initialize_Fields_For_Base_Aggregate;
   function Base_Aggregate_F_Ancestor_Expr
     (Node : Bare_Base_Aggregate) return Bare_Expr
   is
   begin
         return Node.Base_Aggregate_F_Ancestor_Expr;
   end;
   function Base_Aggregate_F_Assocs
     (Node : Bare_Base_Aggregate) return Bare_Assoc_List
   is
   begin
         return Node.Base_Aggregate_F_Assocs;
   end;
--# property-start BaseAggregate.xref_equation /usr/src/packages/BUILD/ada/ast.py:9335
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Xref_Equation
  (Node : Bare_Base_Aggregate
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_784;
            procedure Finalizer_Scope_2296;
            procedure Finalizer_Scope_2297;
      Block_Var : Logic_Equation;
Var_Expr : Internal_Entity_Expr;
Logic_True : Logic_Equation;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Sym : Symbol_Type;
Fld_2 : Boolean;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Fld_5 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Fld_6 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Fld_7 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Internal_Entity_Assoc_List;
Fld_11 : Bare_Assoc_List;
Fld_12 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_6 : Internal_Entity_Basic_Assoc;
Fld_13 : Logic_Equation;
Item_170 : Internal_Entity_Basic_Assoc;
Bare_Item_170 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_784 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_784;
            procedure Finalizer_Scope_2296 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2296;
            procedure Finalizer_Scope_2297 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2297;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4929 '<Block>' Let_Result None
--# scope-start
--# expr-start 4920 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:9341
--# expr-start 4905 '<FieldAccess .in_aspect(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9342
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4904 '<Symbol Global>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Global); 
--# expr-done 4904
--# property-call-start AdaNode.in_aspect
Fld_2 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 4905
if Fld_2 then
   If_Result := True;
else
   --# expr-start 4907 '<FieldAccess .in_aspect(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9342
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4906 '<Symbol Refined_Global>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Global); 
--# expr-done 4906
--# property-call-start AdaNode.in_aspect
Fld_3 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 4907
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 4909 '<FieldAccess .in_aspect(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9343
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4908 '<Symbol Depends>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 4908
--# property-call-start AdaNode.in_aspect
Fld_4 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_2);
--# end
--# expr-done 4909
   If_Result_1 := Fld_4;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 4911 '<FieldAccess .in_aspect(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9343
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4910 '<Symbol Refined_Depends>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Depends); 
--# expr-done 4910
--# property-call-start AdaNode.in_aspect
Fld_5 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_3);
--# end
--# expr-done 4911
   If_Result_2 := Fld_5;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 4913 '<FieldAccess .in_aspect(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9344
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4912 '<Symbol Test_Case>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Test_Case); 
--# expr-done 4912
--# property-call-start AdaNode.in_aspect
Fld_6 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_4);
--# end
--# expr-done 4913
   If_Result_3 := Fld_6;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 4915 '<FieldAccess .in_aspect(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9344
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4914 '<Symbol Refined_State>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_State); 
--# expr-done 4914
--# property-call-start AdaNode.in_aspect
Fld_7 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_5);
--# end
--# expr-done 4915
   If_Result_4 := Fld_7;
end if;
if If_Result_4 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9345") else null)); 
   If_Result_5 := Logic_True;
else
   --# expr-start 4919 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Predicate on BaseTypeDecl.is_array_or_rec>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:9346
--# expr-start 4916 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:9346
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 4916
Bound_Origin := Fld; 
--# expr-start 4918 '<Predicate on BaseTypeDecl.is_array_or_rec>' Pred /usr/src/packages/BUILD/ada/ast.py:9347
--# expr-start 4917 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9347
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 4917
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Base_Type_Decl_P_Is_Array_Or_Rec_0_Pred.Create (Fld_1, Create_Base_Type_Decl_P_Is_Array_Or_Rec_0_Predicate (Bound_Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Or_Rec")            else null))); 
--# expr-done 4918
Dyn_Var_Bind_Result := Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 4919
   If_Result_5 := Dyn_Var_Bind_Result;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 4920
Block_Var := If_Result_5; Inc_Ref (Block_Var);
--# bind type_constraint Block_Var
--# expr-start 4928 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9350
--# expr-start 4923 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9350
--# expr-start 4921 '<FieldAccess .ancestor_expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9350
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Expr (Node => Ent.Node.Base_Aggregate_F_Ancestor_Expr, Info => Ent.Info);
--# expr-done 4921
Var_Expr := Fld_8;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind ae Var_Expr
--# expr-start 4922 '<FieldAccess .sub_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9351
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4922
Scope_Result := Fld_9;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2296;
   Result_Var := Scope_Result;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9351") else null)); 
   Result_Var := Logic_True_1;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Block_Var, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9350") else null)); 
--# expr-done 4923
--# expr-start 4927 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:9352
--# expr-start 4926 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9352
--# expr-start 4924 '<FieldAccess .assocs>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9352
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4924
Fld_11 := Fld_10.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_11 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_11;
      begin
         for Untyped_Item_170 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_170 := Untyped_Item_170; 
Is_Null := Bare_Item_170 = null; 
if Is_Null then
   If_Result_6 := No_Entity_Basic_Assoc;
else
Fld_12 := Fld_10.Info;
As_Entity := (Info => Fld_12, Node => Bare_Item_170); 
   If_Result_6 := As_Entity;
end if;
                  Item_170 := If_Result_6; 
   --# scope-start
               --# bind assoc Item_170
      --# expr-start 4925 '<FieldAccess .sub_equation>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9353
      if Item_170.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_170.Node, Env => Env, Origin => Origin, E_Info => Item_170.Info);
--# end
--# expr-done 4925
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_13;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2297;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 4926
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9352") else null)); 
--# expr-done 4927
And_Pred_1 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9350") else null)); 
--# expr-done 4928
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 4929
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_784;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_784;
               Finalizer_Scope_2296;
               Finalizer_Scope_2297;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Xref_Equation;
--# end
--# property-start BaseAggregate.multidim_root_aggregate /usr/src/packages/BUILD/ada/ast.py:9358
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Multidim_Root_Aggregate
  (Node : Bare_Base_Aggregate
      ; R : Integer
         := 0
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Multidim_Aggregate_Info
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind r R
   --# bind origin Origin
   Property_Result : Internal_Multidim_Aggregate_Info;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Aggregate;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
New_Struct : Internal_Multidim_Aggregate_Info;
Fld_1 : Integer;
Comp_Result : Boolean;
If_Result : Internal_Multidim_Aggregate_Info;
Scope_Result : Internal_Multidim_Aggregate_Info;
Fld_2 : Internal_Entity;
Fld_3 : Internal_Entity;
Fld_4 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Aggregate;
Arith_Result : Integer;
Fld_5 : Internal_Multidim_Aggregate_Info;
Scope_Result_1 : Internal_Multidim_Aggregate_Info;
Result_Var : Internal_Multidim_Aggregate_Info;
Result_Var_1 : Internal_Multidim_Aggregate_Info;
Let_Result : Internal_Multidim_Aggregate_Info;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Multidim_Root_Aggregate,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Integer,
                                      As_Integer => R);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Multidim_Aggregate_Info;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4941 '<Block>' Let_Result None
--# scope-start
--# expr-start 4931 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9365
--# expr-start 4930 '<FieldAccess .type_val>' Fld /usr/src/packages/BUILD/ada/ast.py:9365
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Ent.Node);
--# end
--# expr-done 4930
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4931
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind tv Var_Expr
--# expr-start 4934 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:9366
--# expr-start 4932 '<FieldAccess .array_ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9369
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 4932
Comp_Result := Fld_1 > 1; 
if Comp_Result then
   --# expr-start 4933 '<New InternalMultidimAggregateInfo>' New_Struct /usr/src/packages/BUILD/ada/ast.py:9370
New_Struct := (Agg => Ent, Rank => R, Typ => Var_Expr); 
--# expr-done 4933
   If_Result := New_Struct;
else
   If_Result := No_Multidim_Aggregate_Info;
end if;
--# expr-done 4934
Scope_Result := If_Result;
   --# end
   Result_Var_1 := Scope_Result;
else
--# expr-start 4938 '<Cast to Aggregate>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:9378
--# expr-start 4937 '<parent(<parent(<parent(<Var Ent>)>)>)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9378
--# expr-start 4936 '<parent(<parent(<Var Ent>)>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9378
--# expr-start 4935 '<parent(<Var Ent>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9378
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 4935
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# expr-done 4936
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Parent (Node => Fld_3.Node, E_Info => Fld_3.Info);
--# expr-done 4937
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Aggregate
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result_1 := No_Entity_Aggregate;
   end if;
--# expr-done 4938
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Aggregate then
   --# scope-start
--# expr-start 4940 '<FieldAccess .multidim_root_aggregate(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9378
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4939 '<Op +>' Arith_Result /usr/src/packages/BUILD/ada/ast.py:9379
Arith_Result := (R + 1); 
--# expr-done 4939
--# property-call-start BaseAggregate.multidim_root_aggregate
Fld_5 := Libadalang.Implementation.Base_Aggregate_P_Multidim_Root_Aggregate (Node => Var_Expr_1.Node, R => Arith_Result, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 4940
Scope_Result_1 := Fld_5;
   --# end
   Result_Var := Scope_Result_1;
else
   Result_Var := No_Multidim_Aggregate_Info;
end if;
   Result_Var_1 := Result_Var;
end if;
Let_Result := Result_Var_1; 
--# end
--# expr-done 4941
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Multidim_Aggregate_Info,
                     As_Internal_Multidim_Aggregate_Info => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Multidim_Root_Aggregate;
--# end
--# property-start BaseAggregate.all_discriminants /usr/src/packages/BUILD/ada/ast.py:9385
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_786;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Component_List;
Fld_3 : Internal_Entity_Type_Decl;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_786 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_786;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4948 '<Block>' Let_Result None
--# scope-start
--# expr-start 4943 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9390
--# expr-start 4942 '<FieldAccess .type_val>' Fld /usr/src/packages/BUILD/ada/ast.py:9390
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 4942
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4943
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 4946 '<FieldAccess .type_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9391
--# expr-start 4945 '<FieldAccess .comps>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9391
--# expr-start 4944 '<FieldAccess .record_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9391
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4944
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_2 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4945
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.type_decl
Fld_3 := Libadalang.Implementation.Component_List_P_Type_Decl (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 4946
Block_Var_1 := Fld_3; 
--# bind record_decl Block_Var_1
--# expr-start 4947 '<FieldAccess .discriminants_list>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9392
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4947
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4948
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_786;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                     As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_786;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_All_Discriminants;
--# end
--# property-start BaseAggregate.all_components /usr/src/packages/BUILD/ada/ast.py:9397
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_All_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_787;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Component_List;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Record_Def;
Fld_2 : Internal_Entity_Component_List;
Fld_3 : Internal_Entity_Assoc_List;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_787 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_787;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_All_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Formal_Param_Decl_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4955 '<Block>' Let_Result None
--# scope-start
--# expr-start 4950 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9402
--# expr-start 4949 '<FieldAccess .type_val>' Fld /usr/src/packages/BUILD/ada/ast.py:9402
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld := Libadalang.Implementation.Expr_P_Type_Val (Node => Self);
--# end
--# expr-done 4949
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 4950
Block_Var := Cast_Result; 
--# bind td Block_Var
--# expr-start 4952 '<FieldAccess .comps>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9403
--# expr-start 4951 '<FieldAccess .record_def>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9403
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.record_def'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Record_Def (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 4951
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseRecordDef.comps
Fld_2 := Libadalang.Implementation.Base_Record_Def_P_Comps (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 4952
Block_Var_1 := Fld_2; 
--# bind comp_list Block_Var_1
--# expr-start 4954 '<FieldAccess .abstract_formal_params_for_assocs(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9404
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4953 '<FieldAccess .assocs>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9404
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4953
--# property-call-start ComponentList.abstract_formal_params_for_assocs
Fld_4 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_For_Assocs (Node => Block_Var_1.Node, Assocs => Fld_3, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 4954
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
--# expr-done 4955
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_787;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Formal_Param_Decl_Array_Access,
                     As_Internal_Entity_Base_Formal_Param_Decl_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_787;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_All_Components;
--# end
--# property-start BaseAggregate.matched_discriminants /usr/src/packages/BUILD/ada/ast.py:9408
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Discriminants
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_788;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_788 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_788;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Discriminants,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4959 '<Block>' Let_Result None
--# scope-start
--# expr-start 4958 '<FieldAccess .match_formals(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9413
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4956 '<FieldAccess .all_discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:9414
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 4956
--# expr-start 4957 '<FieldAccess .assocs>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9414
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4957
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 4958
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4959
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_788;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                     As_Internal_Param_Match_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_788;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Matched_Discriminants;
--# end
--# property-start BaseAggregate.matched_components /usr/src/packages/BUILD/ada/ast.py:9419
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_Matched_Components
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Param_Match_Array_Access
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_789;
      Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Entity_Assoc_List;
Fld_2 : Internal_Param_Match_Array_Access;
Let_Result : Internal_Param_Match_Array_Access;
            procedure Finalizer_Scope_789 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_789;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Aggregate_Base_Aggregate_P_Matched_Components,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Aggregate,
                                As_Bare_Base_Aggregate => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (3) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (3).As_Lexical_Env);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Param_Match_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 4963 '<Block>' Let_Result None
--# scope-start
--# expr-start 4962 '<FieldAccess .match_formals(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9424
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4960 '<FieldAccess .all_components>' Fld /usr/src/packages/BUILD/ada/ast.py:9425
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_components
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4960
--# expr-start 4961 '<FieldAccess .assocs>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9425
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Assoc_List (Node => Ent.Node.Base_Aggregate_F_Assocs, Info => Ent.Info);
--# expr-done 4961
--# property-call-start AdaNode.match_formals
Fld_2 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Fld, Params => Fld_1, Is_Dottable_Subp => False);
--# end
--# expr-done 4962
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 4963
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_789;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Param_Match_Array_Access,
                     As_Internal_Param_Match_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_789;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_Matched_Components;
--# end
--# property-start BaseAggregate.first_unmatched_formal /usr/src/packages/BUILD/ada/ast.py:9429
pragma Warnings (Off, "is not referenced");
 function Base_Aggregate_P_First_Unmatched_Formal
  (Node : Bare_Base_Aggregate
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Single_Formal
is
   Self : Bare_Base_Aggregate := Bare_Base_Aggregate (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Aggregate :=
     Internal_Entity_Base_Aggregate'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Single_Formal;
            procedure Finalizer_Scope_790;
            procedure Finalizer_Scope_2300;
            procedure Finalizer_Scope_2302;
      Block_Var : Internal_Single_Formal;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
Fld_2 : Internal_Param_Match_Array_Access;
Fld_3 : Internal_Single_Formal;
Is_Equal : Boolean;
Quantifier_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_5 : Internal_Single_Formal_Array_Access;
Fld_6 : Internal_Param_Match_Array_Access;
Fld_7 : Internal_Single_Formal;
Is_Equal_1 : Boolean;
Quantifier_Result_1 : Boolean;
Not_Val_1 : Boolean;
Map_Result_1 : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
Is_Equal_2 : Boolean;
Not_Val_2 : Boolean;
If_Result : Internal_Single_Formal;
Let_Result : Internal_Single_Formal;
            procedure Finalizer_Scope_790 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_790;
            procedure Finalizer_Scope_2300 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2300;
            procedure Finalizer_Scope_2302 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2302;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 4983 '<Block>' Let_Result None
--# scope-start
--# expr-start 4971 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9436
--# expr-start 4970 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9436
--# expr-start 4965 '<FieldAccess .unpack_formals(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9436
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4964 '<FieldAccess .all_discriminants>' Fld /usr/src/packages/BUILD/ada/ast.py:9436
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_discriminants
Fld := Libadalang.Implementation.Base_Aggregate_P_All_Discriminants (Node => Ent.Node, Origin => Origin);
--# end
--# expr-done 4964
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 4965
declare
   Map_Result_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_1;
      begin
         for Item_171 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_171
               --# expr-start 4969 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9437
--# expr-start 4966 '<FieldAccess .matched_discriminants>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9437
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_discriminants
Fld_2 := Libadalang.Implementation.Base_Aggregate_P_Matched_Discriminants (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 4966
Quantifier_Result := False;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_2;
   begin
      for Item_172 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_172
         --# expr-start 4968 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9438
--# expr-start 4967 '<FieldAccess .formal>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9438
Fld_3 := Item_172.Formal;
--# expr-done 4967
Is_Equal := Fld_3 = Item_171; 
--# expr-done 4968
            if Is_Equal then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val := not (Quantifier_Result); 
--# expr-done 4969
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_171;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2300;
         end loop;
      end;
      Map_Result := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4970
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 4971
Block_Var := Get_Result; 
--# bind unmatched_discr Block_Var
--# expr-start 4982 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:9442
--# expr-start 4973 '<Not>' Not_Val_2 /usr/src/packages/BUILD/ada/ast.py:9443
--# expr-start 4972 '<IsNull>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:9443
Is_Equal_2 := Block_Var = No_Single_Formal; 
--# expr-done 4972
Not_Val_2 := not (Is_Equal_2); 
--# expr-done 4973
if Not_Val_2 then
   If_Result := Block_Var;
else
   --# expr-start 4981 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:9452
--# expr-start 4980 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:9452
--# expr-start 4975 '<FieldAccess .unpack_formals(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9452
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4974 '<FieldAccess .all_components>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9452
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.all_components
Fld_4 := Libadalang.Implementation.Base_Aggregate_P_All_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4974
--# property-call-start AdaNode.unpack_formals
Fld_5 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld_4);
--# end
--# expr-done 4975
declare
   Map_Result_1_Vec : Internal_Single_Formal_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Single_Formal_Array_Access := Fld_5;
      begin
         for Item_173 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_173
               --# expr-start 4979 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:9453
--# expr-start 4976 '<FieldAccess .matched_components>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9453
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseAggregate.matched_components
Fld_6 := Libadalang.Implementation.Base_Aggregate_P_Matched_Components (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 4976
Quantifier_Result_1 := False;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_6;
   begin
      for Item_174 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_174
         --# expr-start 4978 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:9454
--# expr-start 4977 '<FieldAccess .formal>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9454
Fld_7 := Item_174.Formal;
--# expr-done 4977
Is_Equal_1 := Fld_7 = Item_173; 
--# expr-done 4978
            if Is_Equal_1 then
               Quantifier_Result_1 := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Not_Val_1 := not (Quantifier_Result_1); 
--# expr-done 4979
               if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Single_Formal :=
               Item_173;
         begin
            Internal_Single_Formal_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2302;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Single_Formal_Array
        (Items_Count => Natural (Internal_Single_Formal_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Single_Formal_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Single_Formal_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Single_Formal_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 4980
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 4981
   If_Result := Get_Result_1;
end if;
--# expr-done 4982
Let_Result := If_Result; 
--# end
--# expr-done 4983
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_790;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_790;
               Finalizer_Scope_2300;
               Finalizer_Scope_2302;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Aggregate_P_First_Unmatched_Formal;
--# end
   --
   --  Primitives for Bare_Aggregate
   --
      procedure Initialize_Fields_For_Aggregate
        (Self : Bare_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Aggregate;
   --
   --  Primitives for Bare_Bracket_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Aggregate
        (Self : Bare_Bracket_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Aggregate;
   --
   --  Primitives for Bare_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Delta_Aggregate
        (Self : Bare_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Delta_Aggregate;
   --
   --  Primitives for Bare_Bracket_Delta_Aggregate
   --
      procedure Initialize_Fields_For_Bracket_Delta_Aggregate
        (Self : Bare_Bracket_Delta_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Delta_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Bracket_Delta_Aggregate;
   --
   --  Primitives for Bare_Null_Record_Aggregate
   --
      procedure Initialize_Fields_For_Null_Record_Aggregate
        (Self : Bare_Null_Record_Aggregate
         ; Base_Aggregate_F_Ancestor_Expr : Bare_Expr
         ; Base_Aggregate_F_Assocs : Bare_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Base_Aggregate
              (Self, Base_Aggregate_F_Ancestor_Expr, Base_Aggregate_F_Assocs);
      end Initialize_Fields_For_Null_Record_Aggregate;
   --
   --  Primitives for Bare_Bin_Op
   --
      procedure Initialize_Fields_For_Bin_Op
        (Self : Bare_Bin_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Bin_Op_F_Left := Bin_Op_F_Left;
            Self.Bin_Op_F_Op := Bin_Op_F_Op;
            Self.Bin_Op_F_Right := Bin_Op_F_Right;
      end Initialize_Fields_For_Bin_Op;
   function Bin_Op_F_Left
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Left;
   end;
   function Bin_Op_F_Op
     (Node : Bare_Bin_Op) return Bare_Op
   is
   begin
         return Node.Bin_Op_F_Op;
   end;
   function Bin_Op_F_Right
     (Node : Bare_Bin_Op) return Bare_Expr
   is
   begin
         return Node.Bin_Op_F_Right;
   end;
--# property-start BinOp.xref_equation /usr/src/packages/BUILD/ada/ast.py:9089
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Xref_Equation
  (Node : Bare_Bin_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Bin_Op :=
     Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_778;
            procedure Finalizer_Scope_2306;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Ps : Internal_Single_Formal_Array_Access;
Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Integer;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Len : Integer;
Is_Equal_1 : Boolean;
Fld_4 : Bare_Op;
Fld_5 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result : Boolean;
Fld_6 : Bare_Op;
Sym_1 : Symbol_Type;
Cast_Expr : Internal_Entity_Bin_Op;
Cast_Result : Internal_Entity;
Fld_7 : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Integer;
Is_Equal_3 : Boolean;
Map_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
If_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_14 : Internal_Entity_Base_Subp_Spec;
Fld_15 : Internal_Single_Formal_Array_Access;
Fld_16 : Internal_Entity_Base_Subp_Spec;
Get_Result : Internal_Single_Formal;
Fld_17 : Internal_Entity_Base_Formal_Param_Decl;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
Fld_20 : Internal_Entity_Base_Subp_Spec;
Get_Result_1 : Internal_Single_Formal;
Fld_21 : Internal_Entity_Base_Formal_Param_Decl;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_24 : Logic_Var;
Fld_25 : Internal_Entity_Base_Subp_Spec;
Fld_26 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_27 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_28 : Bare_Op;
Fld_29 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_30 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_778 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_778;
            procedure Finalizer_Scope_2306 is
            begin
                     Dec_Ref (Fld_15);
                     Dec_Ref (Ps);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2306;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5037 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 4989 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9090
--# expr-start 4985 '<FieldAccess .subprograms>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9090
--# expr-start 4984 '<FieldAccess .op>' Fld /usr/src/packages/BUILD/ada/ast.py:9090
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 4984
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprograms
Fld_1 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 4985
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_1;
      begin
         for Item_175 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_175
               --# expr-start 4988 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9091
--# expr-start 4987 '<FieldAccess .nb_max_params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9091
--# expr-start 4986 '<FieldAccess .subp_spec_or_null>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9091
      if Item_175.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_175.Node, E_Info => Item_175.Info);
--# end
--# expr-done 4986
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_2.Node);
--# end
--# expr-done 4987
Is_Equal := Fld_3 = 2; 
--# expr-done 4988
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_175;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 4989
Block_Var := Map_Result; Inc_Ref (Block_Var);
--# bind subps Block_Var
--# expr-start 4991 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:9098
--# expr-start 4990 '<length(<Var subps>)>' Len /usr/src/packages/BUILD/ada/ast.py:9098
Len := Length (Block_Var); 
--# expr-done 4990
Is_Equal_1 := Len = 0; 
--# expr-done 4991
if Is_Equal_1 then
   --# expr-start 4995 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:9099
--# expr-start 4993 '<FieldAccess .subprogram_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9099
--# expr-start 4992 '<FieldAccess .op>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9099
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Bin_Op_F_Op;
--# expr-done 4992
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld_5 := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Fld_4);
--# end
--# expr-done 4993
--# expr-start 4994 '<Symbol "/=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 4994
Is_Equal_2 := Fld_5 = Sym; 
--# expr-done 4995
   If_Result := Is_Equal_2;
else
   If_Result := False;
end if;
Block_Var_1 := If_Result; 
--# bind refers_to_synthetic_neq Block_Var_1
--# expr-start 5003 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9103
if Block_Var_1 then
   --# expr-start 5002 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:9105
--# expr-start 4998 '<FieldAccess .subprograms_for_symbol(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9105
--# expr-start 4996 '<FieldAccess .op>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9105
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Bin_Op_F_Op;
--# expr-done 4996
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 4997 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 4997
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_7 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Fld_6, Sym => Sym_1, From_Node => Cast_Result);
--# end
--# expr-done 4998
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_7;
      begin
         for Item_176 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_176
               --# expr-start 5001 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:9106
--# expr-start 5000 '<FieldAccess .nb_max_params>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9106
--# expr-start 4999 '<FieldAccess .subp_spec_or_null>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9106
      if Item_176.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_176.Node, E_Info => Item_176.Info);
--# end
--# expr-done 4999
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_8.Node);
--# end
--# expr-done 5000
Is_Equal_3 := Fld_9 = 2; 
--# expr-done 5001
               if Is_Equal_3 then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_176;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 5002
   If_Result_1 := Map_Result_1;
else
   If_Result_1 := Block_Var;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 5003
Block_Var_2 := If_Result_1; Inc_Ref (Block_Var_2);
--# bind refined_subps Block_Var_2
--# expr-start 5036 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:9111
--# expr-start 5008 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9112
--# expr-start 5005 '<FieldAccess .sub_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9112
--# expr-start 5004 '<FieldAccess .left>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9112
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 5004
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 5005
--# expr-start 5007 '<FieldAccess .sub_equation>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9113
--# expr-start 5006 '<FieldAccess .right>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9113
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 5006
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_12.Node, Env => Env, Origin => Origin, E_Info => Fld_12.Info);
--# end
--# expr-done 5007
And_Pred := Logic_And (Fld_11, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9112") else null)); 
--# expr-done 5008
--# expr-start 5035 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:9114
--# expr-start 5033 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:9114
--# expr-start 5032 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:9114
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_2;
      begin
         for Item_177 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_177
      --# expr-start 5031 '<Let ps>' Let_Result /usr/src/packages/BUILD/ada/ast.py:9114
--# scope-start
--# expr-start 5010 '<FieldAccess .unpacked_formal_params>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:9115
--# expr-start 5009 '<FieldAccess .subp_spec_or_null>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:9115
      if Item_177.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_177.Node, E_Info => Item_177.Info);
--# end
--# expr-done 5009
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_15 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_14.Node, E_Info => Fld_14.Info);
--# end
--# expr-done 5010
Ps := Fld_15; Inc_Ref (Ps);
--# bind ps Ps
--# expr-start 5030 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:9119
--# expr-start 5026 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:9119
--# expr-start 5021 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9119
--# expr-start 5015 '<FieldAccess .call_argument_equation(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:9119
--# expr-start 5011 '<FieldAccess .subp_spec_or_null>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:9119
      if Item_177.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_177.Node, E_Info => Item_177.Info);
--# end
--# expr-done 5011
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5013 '<FieldAccess .spec>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:9120
--# expr-start 5012 '<at(<Var ps>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9120
Get_Result := Get (Ps, 0, True); 
--# expr-done 5012
Fld_17 := Get_Result.Spec;
--# expr-done 5013
--# expr-start 5014 '<FieldAccess .left>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:9120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 5014
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_19 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Fld_16.Node, Param => Fld_17, Arg => Fld_18, Env => Env, Origin => Origin, E_Info => Fld_16.Info);
--# end
--# expr-done 5015
--# expr-start 5020 '<FieldAccess .call_argument_equation(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:9125
--# expr-start 5016 '<FieldAccess .subp_spec_or_null>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:9125
      if Item_177.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_20 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_177.Node, E_Info => Item_177.Info);
--# end
--# expr-done 5016
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5018 '<FieldAccess .spec>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:9126
--# expr-start 5017 '<at(<Var ps>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:9126
Get_Result_1 := Get (Ps, 1, True); 
--# expr-done 5017
Fld_21 := Get_Result_1.Spec;
--# expr-done 5018
--# expr-start 5019 '<FieldAccess .right>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:9126
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 5019
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_23 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Fld_20.Node, Param => Fld_21, Arg => Fld_22, Env => Env, Origin => Origin, E_Info => Fld_20.Info);
--# end
--# expr-done 5020
And_Pred_1 := Logic_And (Fld_19, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9119") else null)); 
--# expr-done 5021
--# expr-start 5025 '<FieldAccess .type_bind_val(...)>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:9130
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5022 '<FieldAccess .type_var>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:9130
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5022
--# expr-start 5024 '<FieldAccess .return_type>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:9131
--# expr-start 5023 '<FieldAccess .subp_spec_or_null>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:9131
      if Item_177.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_25 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_177.Node, E_Info => Item_177.Info);
--# end
--# expr-done 5023
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_26 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_25.Node, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 5024
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_26.Node,
         Info => Fld_26.Info);
--# property-call-start AdaNode.type_bind_val
Fld_27 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_24, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 5025
And_Pred_2 := Logic_And (And_Pred_1, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9119") else null)); 
--# expr-done 5026
--# expr-start 5029 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:9136
--# expr-start 5028 '<FieldAccess .ref_var>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:9136
--# expr-start 5027 '<FieldAccess .op>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:9136
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Self.Bin_Op_F_Op;
--# expr-done 5027
      if Fld_28 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_28);
--# end
--# expr-done 5028
Fld_29.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_29.all);
Eq_Node.Refs.Destroy (Fld_29.all);
Cast_Expr_1 := Item_177; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Bind_Default_Default.Create (Fld_29, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9136") else null)); 
--# expr-done 5029
And_Pred_3 := Logic_And (And_Pred_2, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9119") else null)); 
--# expr-done 5030
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 5031
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2306;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 5032
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9114") else null)); 
--# expr-done 5033
--# expr-start 5034 '<FieldAccess .no_overload_equation>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:9137
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BinOp.no_overload_equation'
Fld_30 := Libadalang.Implementation.Dispatcher_Bin_Op_P_No_Overload_Equation (Node => Self, Origin => Origin);
--# end
--# expr-done 5034
Or_Pred := Logic_Or (Logic_Boolean_Op, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9114") else null)); 
--# expr-done 5035
And_Pred_4 := Logic_And (And_Pred, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9111") else null)); 
--# expr-done 5036
Let_Result_1 := And_Pred_4; Inc_Ref (Let_Result_1);
--# end
--# expr-done 5037
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_778;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_778;
               Finalizer_Scope_2306;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_Xref_Equation;
--# end
--# property-start '[dispatcher]BinOp.no_overload_equation' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Bin_Op_Range (Self.Kind) is
               when Ada_Bin_Op =>
                  --# property-call-start BinOp.no_overload_equation
                  Property_Result := Bin_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Relation_Op =>
                  --# property-call-start RelationOp.no_overload_equation
                  Property_Result := Relation_Op_P_No_Overload_Equation
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Bin_Op_P_No_Overload_Equation;
--# end
--# property-start BinOp.potential_actuals_for_dispatch /usr/src/packages/BUILD/ada/ast.py:9256
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Bin_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Bin_Op :=
     Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_780;
      Block_Var : Internal_Single_Formal_Array_Access;
Fld : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_1 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_2 : Internal_Entity_Base_Formal_Param_Decl;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Get_Result_1 : Internal_Single_Formal;
Fld_5 : Internal_Entity_Base_Formal_Param_Decl;
Fld_6 : Internal_Entity_Type_Expr;
Fld_7 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Array_Lit : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_780 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_780;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5051 '<Block>' Let_Result None
--# scope-start
--# expr-start 5039 '<FieldAccess .unpack_formals(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9257
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5038 '<FieldAccess .abstract_formal_params>' Fld /usr/src/packages/BUILD/ada/ast.py:9257
      if Spec.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 5038
--# property-call-start AdaNode.unpack_formals
Fld_1 := Libadalang.Implementation.Ada_Node_P_Unpack_Formals (Node => Self, Formal_Params => Fld);
--# end
--# expr-done 5039
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind params Block_Var
--# expr-start 5050 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:9258
--# expr-start 5044 '<New InternalExpectedTypeForExpr>' New_Struct /usr/src/packages/BUILD/ada/ast.py:9259
--# expr-start 5042 '<FieldAccess .type_expression>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9260
--# expr-start 5041 '<FieldAccess .spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9260
--# expr-start 5040 '<at(<Var params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9260
Get_Result := Get (Block_Var, 0, True); 
--# expr-done 5040
Fld_2 := Get_Result.Spec;
--# expr-done 5041
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 5042
--# expr-start 5043 '<FieldAccess .left>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9261
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Left, Info => Ent.Info);
--# expr-done 5043
New_Struct := (Expected_Type => Fld_3, Expr => Fld_4); 
--# expr-done 5044
--# expr-start 5049 '<New InternalExpectedTypeForExpr>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:9263
--# expr-start 5047 '<FieldAccess .type_expression>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9264
--# expr-start 5046 '<FieldAccess .spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9264
--# expr-start 5045 '<at(<Var params>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:9264
Get_Result_1 := Get (Block_Var, 1, True); 
--# expr-done 5045
Fld_5 := Get_Result_1.Spec;
--# expr-done 5046
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 5047
--# expr-start 5048 '<FieldAccess .right>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9265
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Expr (Node => Ent.Node.Bin_Op_F_Right, Info => Ent.Info);
--# expr-done 5048
New_Struct_1 := (Expected_Type => Fld_6, Expr => Fld_7); 
--# expr-done 5049
Array_Lit := Create_Internal_Expected_Type_For_Expr_Array (Internal_Internal_Expected_Type_For_Expr_Array'(1 => New_Struct, 2 => New_Struct_1)); 
--# expr-done 5050
Let_Result := Array_Lit; Inc_Ref (Let_Result);
--# end
--# expr-done 5051
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_780;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_780;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start BinOp.is_dispatching_call /usr/src/packages/BUILD/ada/ast.py:9270
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_Is_Dispatching_Call
  (Node : Bare_Bin_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Bin_Op :=
     Internal_Entity_Bin_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5055 '<Block>' Let_Result None
--# scope-start
--# expr-start 5053 '<FieldAccess .referenced_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9271
--# expr-start 5052 '<FieldAccess .op>' Fld /usr/src/packages/BUILD/ada/ast.py:9271
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Bin_Op_F_Op, Info => Ent.Info);
--# expr-done 5052
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 5053
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr
--# expr-start 5054 '<FieldAccess .is_dispatching_call_impl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9272
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dispatching_call_impl
Fld_2 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Ent.Node, Decl => Var_Expr, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5054
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5055
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_Is_Dispatching_Call;
--# end
--# property-start BinOp.no_overload_equation /usr/src/packages/BUILD/ada/ast.py:9140
pragma Warnings (Off, "is not referenced");
 function Bin_Op_P_No_Overload_Equation
  (Node : Bare_Bin_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Bin_Op := Bare_Bin_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_779;
            procedure Finalizer_Scope_2307;
            procedure Finalizer_Scope_2308;
            procedure Finalizer_Scope_2309;
            procedure Finalizer_Scope_2310;
      Match_0 : Bare_Op_Pow;
Match_1 : Bare_Op_Concat;
Match_2 : Bare_Op_Double_Dot;
Match_3 : Bare_Op;
Fld : Bare_Op;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Pred : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Bare_Expr;
Fld_16 : Logic_Var;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Bare_Expr;
Fld_19 : Logic_Var;
Pred_1 : Logic_Equation;
Fld_20 : Logic_Var;
Fld_21 : Bare_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_24 : Bare_Expr;
Fld_25 : Logic_Var;
Fld_26 : Bare_Expr;
Fld_27 : Logic_Var;
Fld_28 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_29 : Bare_Expr;
Fld_30 : Logic_Var;
Pred_2 : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Bare_Expr;
Fld_33 : Logic_Var;
Fld_34 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_35 : Bare_Expr;
Fld_36 : Logic_Var;
Fld_37 : Bare_Expr;
Fld_38 : Logic_Var;
Fld_39 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Fld_40 : Bare_Expr;
Fld_41 : Logic_Var;
Fld_42 : Bare_Expr;
Fld_43 : Logic_Var;
Fld_44 : Logic_Equation;
Fld_45 : Logic_Var;
Fld_46 : Bare_Expr;
Fld_47 : Logic_Var;
Fld_48 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_49 : Logic_Var;
Pred_3 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Or_Pred_2 : Logic_Equation;
Fld_50 : Logic_Var;
Fld_51 : Bare_Expr;
Fld_52 : Logic_Var;
Fld_53 : Logic_Equation;
Fld_54 : Logic_Var;
Fld_55 : Bare_Expr;
Fld_56 : Logic_Var;
Fld_57 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_58 : Bare_Expr;
Fld_59 : Logic_Var;
Pred_4 : Logic_Equation;
Fld_60 : Bare_Expr;
Fld_61 : Logic_Var;
Pred_5 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_62 : Bare_Expr;
Fld_63 : Logic_Var;
Fld_64 : Logic_Equation;
Fld_65 : Bare_Expr;
Fld_66 : Logic_Var;
Fld_67 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Fld_68 : Bare_Expr;
Fld_69 : Logic_Var;
Fld_70 : Logic_Equation;
Fld_71 : Bare_Expr;
Fld_72 : Logic_Var;
Fld_73 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Or_Pred_3 : Logic_Equation;
Fld_74 : Logic_Var;
Fld_75 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Fld_76 : Bare_Expr;
Fld_77 : Logic_Var;
Fld_78 : Logic_Var;
Bind_Result : Logic_Equation;
Or_Pred_4 : Logic_Equation;
And_Pred_14 : Logic_Equation;
Fld_79 : Bare_Expr;
Fld_80 : Logic_Var;
Pred_6 : Logic_Equation;
Fld_81 : Bare_Expr;
Fld_82 : Logic_Var;
Pred_7 : Logic_Equation;
And_Pred_15 : Logic_Equation;
Fld_83 : Bare_Expr;
Fld_84 : Logic_Var;
Fld_85 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred_16 : Logic_Equation;
Or_Pred_5 : Logic_Equation;
Fld_86 : Bare_Expr;
Fld_87 : Logic_Var;
Pred_8 : Logic_Equation;
Fld_88 : Bare_Expr;
Fld_89 : Logic_Var;
Pred_9 : Logic_Equation;
And_Pred_17 : Logic_Equation;
Fld_90 : Bare_Expr;
Fld_91 : Logic_Var;
Fld_92 : Logic_Var;
Bind_Result_2 : Logic_Equation;
And_Pred_18 : Logic_Equation;
Or_Pred_6 : Logic_Equation;
Fld_93 : Bare_Expr;
Fld_94 : Logic_Var;
Pred_10 : Logic_Equation;
Fld_95 : Bare_Expr;
Fld_96 : Logic_Var;
Pred_11 : Logic_Equation;
And_Pred_19 : Logic_Equation;
Fld_97 : Bare_Expr;
Fld_98 : Logic_Var;
Fld_99 : Logic_Var;
Bind_Result_3 : Logic_Equation;
And_Pred_20 : Logic_Equation;
Fld_100 : Bare_Expr;
Fld_101 : Logic_Var;
Fld_102 : Logic_Var;
Bind_Result_4 : Logic_Equation;
And_Pred_21 : Logic_Equation;
Or_Pred_7 : Logic_Equation;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_Pow;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Bare_Op_Concat;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Bare_Op_Double_Dot;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Bare_Op;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_779 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_779;
            procedure Finalizer_Scope_2307 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2307;
            procedure Finalizer_Scope_2308 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Pred_1);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_28);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Pred_2);
                     Dec_Ref (Fld_34);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_39);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Fld_44);
                     Dec_Ref (Fld_48);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Pred_3);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Or_Pred_2);
            end Finalizer_Scope_2308;
            procedure Finalizer_Scope_2309 is
            begin
                     Dec_Ref (Fld_53);
                     Dec_Ref (Fld_57);
                     Dec_Ref (And_Pred_9);
            end Finalizer_Scope_2309;
            procedure Finalizer_Scope_2310 is
            begin
                     Dec_Ref (Pred_4);
                     Dec_Ref (Pred_5);
                     Dec_Ref (And_Pred_10);
                     Dec_Ref (Fld_64);
                     Dec_Ref (Fld_67);
                     Dec_Ref (And_Pred_11);
                     Dec_Ref (Fld_70);
                     Dec_Ref (Fld_73);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Or_Pred_3);
                     Dec_Ref (Fld_75);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Or_Pred_4);
                     Dec_Ref (And_Pred_14);
                     Dec_Ref (Pred_6);
                     Dec_Ref (Pred_7);
                     Dec_Ref (And_Pred_15);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_16);
                     Dec_Ref (Or_Pred_5);
                     Dec_Ref (Pred_8);
                     Dec_Ref (Pred_9);
                     Dec_Ref (And_Pred_17);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (And_Pred_18);
                     Dec_Ref (Or_Pred_6);
                     Dec_Ref (Pred_10);
                     Dec_Ref (Pred_11);
                     Dec_Ref (And_Pred_19);
                     Dec_Ref (Bind_Result_3);
                     Dec_Ref (And_Pred_20);
                     Dec_Ref (Bind_Result_4);
                     Dec_Ref (And_Pred_21);
                     Dec_Ref (Or_Pred_7);
            end Finalizer_Scope_2310;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5207 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5206 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9145
--# expr-start 5056 '<FieldAccess .op>' Fld /usr/src/packages/BUILD/ada/ast.py:9145
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Bin_Op_F_Op;
--# expr-done 5056
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 5064 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9148
--# expr-start 5059 '<FieldAccess .universal_int_bind(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9148
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5058 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9148
--# expr-start 5057 '<FieldAccess .right>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9148
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Bin_Op_F_Right;
--# expr-done 5057
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 5058
--# property-call-start AdaNode.universal_int_bind
Fld_3 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_2, Origin => Origin);
--# end
--# expr-done 5059
--# expr-start 5063 '<FieldAccess .type_bind_var(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9149
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5061 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9149
--# expr-start 5060 '<FieldAccess .left>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9149
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Bin_Op_F_Left;
--# expr-done 5060
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 5061
--# expr-start 5062 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9149
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5062
--# property-call-start AdaNode.type_bind_var
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_5, Right => Fld_6, Origin => Origin);
--# end
--# expr-done 5063
And_Pred := Logic_And (Fld_3, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9148") else null)); 
--# expr-done 5064
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2307;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 5121 '<Or>' Or_Pred_2 /usr/src/packages/BUILD/ada/ast.py:9151
--# expr-start 5107 '<Or>' Or_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9151
--# expr-start 5092 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:9151
--# expr-start 5077 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:9152
--# expr-start 5072 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9152
--# expr-start 5067 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred /usr/src/packages/BUILD/ada/ast.py:9152
--# expr-start 5066 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9153
--# expr-start 5065 '<FieldAccess .left>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.Bin_Op_F_Left;
--# expr-done 5065
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 5066
Fld_9.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_9.all);
Eq_Node.Refs.Destroy (Fld_9.all);
Pred := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_9, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 5067
--# expr-start 5071 '<FieldAccess .type_bind_var(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5068 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5068
--# expr-start 5070 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9154
--# expr-start 5069 '<FieldAccess .left>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9154
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Bin_Op_F_Left;
--# expr-done 5069
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 5070
--# property-call-start AdaNode.type_bind_var
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_10, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 5071
And_Pred_1 := Logic_And (Pred, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9152") else null)); 
--# expr-done 5072
--# expr-start 5076 '<FieldAccess .type_bind_var(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:9155
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5073 '<FieldAccess .type_var>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:9155
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5073
--# expr-start 5075 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:9155
--# expr-start 5074 '<FieldAccess .right>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:9155
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Self.Bin_Op_F_Right;
--# expr-done 5074
      if Fld_15 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_15);
--# expr-done 5075
--# property-call-start AdaNode.type_bind_var
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_14, Right => Fld_16, Origin => Origin);
--# end
--# expr-done 5076
And_Pred_2 := Logic_And (And_Pred_1, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9152") else null)); 
--# expr-done 5077
--# expr-start 5091 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:9157
--# expr-start 5085 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:9157
--# expr-start 5080 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred_1 /usr/src/packages/BUILD/ada/ast.py:9157
--# expr-start 5079 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:9158
--# expr-start 5078 '<FieldAccess .left>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:9158
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Self.Bin_Op_F_Left;
--# expr-done 5078
      if Fld_18 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18);
--# expr-done 5079
Fld_19.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_19.all);
Eq_Node.Refs.Destroy (Fld_19.all);
Pred_1 := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_19, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 5080
--# expr-start 5084 '<FieldAccess .type_bind_var(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:9159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5081 '<FieldAccess .type_var>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:9159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5081
--# expr-start 5083 '<FieldAccess .type_var>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:9159
--# expr-start 5082 '<FieldAccess .left>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:9159
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Self.Bin_Op_F_Left;
--# expr-done 5082
      if Fld_21 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_21);
--# expr-done 5083
--# property-call-start AdaNode.type_bind_var
Fld_23 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_20, Right => Fld_22, Origin => Origin);
--# end
--# expr-done 5084
And_Pred_3 := Logic_And (Pred_1, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9157") else null)); 
--# expr-done 5085
--# expr-start 5090 '<FieldAccess .comp_bind(...)>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:9160
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5087 '<FieldAccess .type_var>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:9160
--# expr-start 5086 '<FieldAccess .left>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:9160
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Self.Bin_Op_F_Left;
--# expr-done 5086
      if Fld_24 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24);
--# expr-done 5087
--# expr-start 5089 '<FieldAccess .type_var>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:9160
--# expr-start 5088 '<FieldAccess .right>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:9160
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Bin_Op_F_Right;
--# expr-done 5088
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_26);
--# expr-done 5089
--# property-call-start AdaNode.comp_bind
Fld_28 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_25, Right => Fld_27, Origin => Origin);
--# end
--# expr-done 5090
And_Pred_4 := Logic_And (And_Pred_3, Fld_28, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9157") else null)); 
--# expr-done 5091
Or_Pred := Logic_Or (And_Pred_2, And_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9151") else null)); 
--# expr-done 5092
--# expr-start 5106 '<And>' And_Pred_6 /usr/src/packages/BUILD/ada/ast.py:9162
--# expr-start 5100 '<And>' And_Pred_5 /usr/src/packages/BUILD/ada/ast.py:9162
--# expr-start 5095 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred_2 /usr/src/packages/BUILD/ada/ast.py:9162
--# expr-start 5094 '<FieldAccess .type_var>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:9163
--# expr-start 5093 '<FieldAccess .right>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:9163
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Self.Bin_Op_F_Right;
--# expr-done 5093
      if Fld_29 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29);
--# expr-done 5094
Fld_30.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_30.all);
Eq_Node.Refs.Destroy (Fld_30.all);
Pred_2 := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_30, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 5095
--# expr-start 5099 '<FieldAccess .type_bind_var(...)>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:9164
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5096 '<FieldAccess .type_var>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:9164
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5096
--# expr-start 5098 '<FieldAccess .type_var>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:9164
--# expr-start 5097 '<FieldAccess .right>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:9164
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Self.Bin_Op_F_Right;
--# expr-done 5097
      if Fld_32 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_32);
--# expr-done 5098
--# property-call-start AdaNode.type_bind_var
Fld_34 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_31, Right => Fld_33, Origin => Origin);
--# end
--# expr-done 5099
And_Pred_5 := Logic_And (Pred_2, Fld_34, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9162") else null)); 
--# expr-done 5100
--# expr-start 5105 '<FieldAccess .comp_bind(...)>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:9165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5102 '<FieldAccess .type_var>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:9165
--# expr-start 5101 '<FieldAccess .right>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:9165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Self.Bin_Op_F_Right;
--# expr-done 5101
      if Fld_35 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_35);
--# expr-done 5102
--# expr-start 5104 '<FieldAccess .type_var>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:9165
--# expr-start 5103 '<FieldAccess .left>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:9165
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Self.Bin_Op_F_Left;
--# expr-done 5103
      if Fld_37 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_38 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_37);
--# expr-done 5104
--# property-call-start AdaNode.comp_bind
Fld_39 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_36, Right => Fld_38, Origin => Origin);
--# end
--# expr-done 5105
And_Pred_6 := Logic_And (And_Pred_5, Fld_39, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9162") else null)); 
--# expr-done 5106
Or_Pred_1 := Logic_Or (Or_Pred, And_Pred_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9151") else null)); 
--# expr-done 5107
--# expr-start 5120 '<And>' And_Pred_8 /usr/src/packages/BUILD/ada/ast.py:9167
--# expr-start 5117 '<And>' And_Pred_7 /usr/src/packages/BUILD/ada/ast.py:9167
--# expr-start 5112 '<FieldAccess .type_bind_var(...)>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:9167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5109 '<FieldAccess .type_var>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:9167
--# expr-start 5108 '<FieldAccess .right>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:9167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Self.Bin_Op_F_Right;
--# expr-done 5108
      if Fld_40 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_40);
--# expr-done 5109
--# expr-start 5111 '<FieldAccess .type_var>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:9167
--# expr-start 5110 '<FieldAccess .left>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:9167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_42 := Self.Bin_Op_F_Left;
--# expr-done 5110
      if Fld_42 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_42);
--# expr-done 5111
--# property-call-start AdaNode.type_bind_var
Fld_44 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_41, Right => Fld_43, Origin => Origin);
--# end
--# expr-done 5112
--# expr-start 5116 '<FieldAccess .comp_bind(...)>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:9168
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5113 '<FieldAccess .type_var>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:9168
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5113
--# expr-start 5115 '<FieldAccess .type_var>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:9168
--# expr-start 5114 '<FieldAccess .right>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:9168
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_46 := Self.Bin_Op_F_Right;
--# expr-done 5114
      if Fld_46 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_46);
--# expr-done 5115
--# property-call-start AdaNode.comp_bind
Fld_48 := Libadalang.Implementation.Ada_Node_P_Comp_Bind (Node => Self, Left => Fld_45, Right => Fld_47, Origin => Origin);
--# end
--# expr-done 5116
And_Pred_7 := Logic_And (Fld_44, Fld_48, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9167") else null)); 
--# expr-done 5117
--# expr-start 5119 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred_3 /usr/src/packages/BUILD/ada/ast.py:9169
--# expr-start 5118 '<FieldAccess .type_var>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:9170
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5118
Fld_49.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_49.all);
Eq_Node.Refs.Destroy (Fld_49.all);
Pred_3 := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_49, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 5119
And_Pred_8 := Logic_And (And_Pred_7, Pred_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9167") else null)); 
--# expr-done 5120
Or_Pred_2 := Logic_Or (Or_Pred_1, And_Pred_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9151") else null)); 
--# expr-done 5121
Let_Result_1 := Or_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2308;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Op_Double_Dot =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 5130 '<And>' And_Pred_9 /usr/src/packages/BUILD/ada/ast.py:9176
--# expr-start 5125 '<FieldAccess .type_bind_var(...)>' Fld_53 /usr/src/packages/BUILD/ada/ast.py:9177
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5122 '<FieldAccess .type_var>' Fld_50 /usr/src/packages/BUILD/ada/ast.py:9177
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_50 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5122
--# expr-start 5124 '<FieldAccess .type_var>' Fld_52 /usr/src/packages/BUILD/ada/ast.py:9177
--# expr-start 5123 '<FieldAccess .left>' Fld_51 /usr/src/packages/BUILD/ada/ast.py:9177
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Self.Bin_Op_F_Left;
--# expr-done 5123
      if Fld_51 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_52 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_51);
--# expr-done 5124
--# property-call-start AdaNode.type_bind_var
Fld_53 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_50, Right => Fld_52, Origin => Origin);
--# end
--# expr-done 5125
--# expr-start 5129 '<FieldAccess .type_bind_var(...)>' Fld_57 /usr/src/packages/BUILD/ada/ast.py:9178
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5126 '<FieldAccess .type_var>' Fld_54 /usr/src/packages/BUILD/ada/ast.py:9178
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_54 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5126
--# expr-start 5128 '<FieldAccess .type_var>' Fld_56 /usr/src/packages/BUILD/ada/ast.py:9178
--# expr-start 5127 '<FieldAccess .right>' Fld_55 /usr/src/packages/BUILD/ada/ast.py:9178
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Self.Bin_Op_F_Right;
--# expr-done 5127
      if Fld_55 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_56 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_55);
--# expr-done 5128
--# property-call-start AdaNode.type_bind_var
Fld_57 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_54, Right => Fld_56, Origin => Origin);
--# end
--# expr-done 5129
And_Pred_9 := Logic_And (Fld_53, Fld_57, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9176") else null)); 
--# expr-done 5130
Let_Result_2 := And_Pred_9; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2309;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Op_Abs .. Ada_Op_And_Then | Ada_Op_Div | Ada_Op_Eq .. Ada_Op_Plus | Ada_Op_Rem .. Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 5205 '<Or>' Or_Pred_7 /usr/src/packages/BUILD/ada/ast.py:9182
--# expr-start 5187 '<Or>' Or_Pred_6 /usr/src/packages/BUILD/ada/ast.py:9182
--# expr-start 5174 '<Or>' Or_Pred_5 /usr/src/packages/BUILD/ada/ast.py:9182
--# expr-start 5161 '<And>' And_Pred_14 /usr/src/packages/BUILD/ada/ast.py:9195
--# expr-start 5137 '<And>' And_Pred_10 /usr/src/packages/BUILD/ada/ast.py:9195
--# expr-start 5133 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_4 /usr/src/packages/BUILD/ada/ast.py:9195
--# expr-start 5132 '<FieldAccess .type_var>' Fld_59 /usr/src/packages/BUILD/ada/ast.py:9196
--# expr-start 5131 '<FieldAccess .left>' Fld_58 /usr/src/packages/BUILD/ada/ast.py:9196
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_58 := Self.Bin_Op_F_Left;
--# expr-done 5131
      if Fld_58 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_58);
--# expr-done 5132
Fld_59.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_59.all);
Eq_Node.Refs.Destroy (Fld_59.all);
Pred_4 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_59, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 5133
--# expr-start 5136 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_5 /usr/src/packages/BUILD/ada/ast.py:9197
--# expr-start 5135 '<FieldAccess .type_var>' Fld_61 /usr/src/packages/BUILD/ada/ast.py:9198
--# expr-start 5134 '<FieldAccess .right>' Fld_60 /usr/src/packages/BUILD/ada/ast.py:9198
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_60 := Self.Bin_Op_F_Right;
--# expr-done 5134
      if Fld_60 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_60);
--# expr-done 5135
Fld_61.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_61.all);
Eq_Node.Refs.Destroy (Fld_61.all);
Pred_5 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_61, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 5136
And_Pred_10 := Logic_And (Pred_4, Pred_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9195") else null)); 
--# expr-done 5137
--# expr-start 5160 '<Or>' Or_Pred_4 /usr/src/packages/BUILD/ada/ast.py:9199
--# expr-start 5155 '<And>' And_Pred_13 /usr/src/packages/BUILD/ada/ast.py:9205
--# expr-start 5152 '<Or>' Or_Pred_3 /usr/src/packages/BUILD/ada/ast.py:9205
--# expr-start 5144 '<And>' And_Pred_11 /usr/src/packages/BUILD/ada/ast.py:9206
--# expr-start 5140 '<FieldAccess .universal_int_bind(...)>' Fld_64 /usr/src/packages/BUILD/ada/ast.py:9206
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5139 '<FieldAccess .type_var>' Fld_63 /usr/src/packages/BUILD/ada/ast.py:9206
--# expr-start 5138 '<FieldAccess .left>' Fld_62 /usr/src/packages/BUILD/ada/ast.py:9206
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_62 := Self.Bin_Op_F_Left;
--# expr-done 5138
      if Fld_62 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_63 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_62);
--# expr-done 5139
--# property-call-start AdaNode.universal_int_bind
Fld_64 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_63, Origin => Origin);
--# end
--# expr-done 5140
--# expr-start 5143 '<FieldAccess .universal_real_bind(...)>' Fld_67 /usr/src/packages/BUILD/ada/ast.py:9207
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5142 '<FieldAccess .type_var>' Fld_66 /usr/src/packages/BUILD/ada/ast.py:9207
--# expr-start 5141 '<FieldAccess .right>' Fld_65 /usr/src/packages/BUILD/ada/ast.py:9207
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_65 := Self.Bin_Op_F_Right;
--# expr-done 5141
      if Fld_65 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_66 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_65);
--# expr-done 5142
--# property-call-start AdaNode.universal_real_bind
Fld_67 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_66, Origin => Origin);
--# end
--# expr-done 5143
And_Pred_11 := Logic_And (Fld_64, Fld_67, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9206") else null)); 
--# expr-done 5144
--# expr-start 5151 '<And>' And_Pred_12 /usr/src/packages/BUILD/ada/ast.py:9209
--# expr-start 5147 '<FieldAccess .universal_real_bind(...)>' Fld_70 /usr/src/packages/BUILD/ada/ast.py:9209
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5146 '<FieldAccess .type_var>' Fld_69 /usr/src/packages/BUILD/ada/ast.py:9209
--# expr-start 5145 '<FieldAccess .left>' Fld_68 /usr/src/packages/BUILD/ada/ast.py:9209
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_68 := Self.Bin_Op_F_Left;
--# expr-done 5145
      if Fld_68 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_69 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_68);
--# expr-done 5146
--# property-call-start AdaNode.universal_real_bind
Fld_70 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_69, Origin => Origin);
--# end
--# expr-done 5147
--# expr-start 5150 '<FieldAccess .universal_int_bind(...)>' Fld_73 /usr/src/packages/BUILD/ada/ast.py:9210
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5149 '<FieldAccess .type_var>' Fld_72 /usr/src/packages/BUILD/ada/ast.py:9210
--# expr-start 5148 '<FieldAccess .right>' Fld_71 /usr/src/packages/BUILD/ada/ast.py:9210
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_71 := Self.Bin_Op_F_Right;
--# expr-done 5148
      if Fld_71 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_72 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_71);
--# expr-done 5149
--# property-call-start AdaNode.universal_int_bind
Fld_73 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_72, Origin => Origin);
--# end
--# expr-done 5150
And_Pred_12 := Logic_And (Fld_70, Fld_73, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9209") else null)); 
--# expr-done 5151
Or_Pred_3 := Logic_Or (And_Pred_11, And_Pred_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9205") else null)); 
--# expr-done 5152
--# expr-start 5154 '<FieldAccess .universal_real_bind(...)>' Fld_75 /usr/src/packages/BUILD/ada/ast.py:9211
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5153 '<FieldAccess .type_var>' Fld_74 /usr/src/packages/BUILD/ada/ast.py:9211
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_74 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5153
--# property-call-start AdaNode.universal_real_bind
Fld_75 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld_74, Origin => Origin);
--# end
--# expr-done 5154
And_Pred_13 := Logic_And (Or_Pred_3, Fld_75, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9205") else null)); 
--# expr-done 5155
--# expr-start 5159 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:9215
--# expr-start 5157 '<FieldAccess .type_var>' Fld_77 /usr/src/packages/BUILD/ada/ast.py:9215
--# expr-start 5156 '<FieldAccess .left>' Fld_76 /usr/src/packages/BUILD/ada/ast.py:9215
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_76 := Self.Bin_Op_F_Left;
--# expr-done 5156
      if Fld_76 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_77 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_76);
--# expr-done 5157
Fld_77.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_77.all);
Eq_Node.Refs.Destroy (Fld_77.all);
--# expr-start 5158 '<FieldAccess .type_var>' Fld_78 /usr/src/packages/BUILD/ada/ast.py:9215
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_78 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5158
Fld_78.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_78.all);
Eq_Node.Refs.Destroy (Fld_78.all);
Bind_Result := Bind_Default_567.Create (Fld_77, Fld_78, No_Logic_Converter_Default, Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9215") else null)); 
--# expr-done 5159
Or_Pred_4 := Logic_Or (And_Pred_13, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9199") else null)); 
--# expr-done 5160
And_Pred_14 := Logic_And (And_Pred_10, Or_Pred_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9195") else null)); 
--# expr-done 5161
--# expr-start 5173 '<And>' And_Pred_16 /usr/src/packages/BUILD/ada/ast.py:9221
--# expr-start 5168 '<And>' And_Pred_15 /usr/src/packages/BUILD/ada/ast.py:9221
--# expr-start 5164 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_6 /usr/src/packages/BUILD/ada/ast.py:9221
--# expr-start 5163 '<FieldAccess .type_var>' Fld_80 /usr/src/packages/BUILD/ada/ast.py:9222
--# expr-start 5162 '<FieldAccess .left>' Fld_79 /usr/src/packages/BUILD/ada/ast.py:9222
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_79 := Self.Bin_Op_F_Left;
--# expr-done 5162
      if Fld_79 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_80 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_79);
--# expr-done 5163
Fld_80.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_80.all);
Eq_Node.Refs.Destroy (Fld_80.all);
Pred_6 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_80, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 5164
--# expr-start 5167 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_7 /usr/src/packages/BUILD/ada/ast.py:9223
--# expr-start 5166 '<FieldAccess .type_var>' Fld_82 /usr/src/packages/BUILD/ada/ast.py:9224
--# expr-start 5165 '<FieldAccess .right>' Fld_81 /usr/src/packages/BUILD/ada/ast.py:9224
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_81 := Self.Bin_Op_F_Right;
--# expr-done 5165
      if Fld_81 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_82 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_81);
--# expr-done 5166
Fld_82.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_82.all);
Eq_Node.Refs.Destroy (Fld_82.all);
Pred_7 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_82, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 5167
And_Pred_15 := Logic_And (Pred_6, Pred_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9221") else null)); 
--# expr-done 5168
--# expr-start 5172 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:9225
--# expr-start 5170 '<FieldAccess .type_var>' Fld_84 /usr/src/packages/BUILD/ada/ast.py:9225
--# expr-start 5169 '<FieldAccess .right>' Fld_83 /usr/src/packages/BUILD/ada/ast.py:9225
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_83 := Self.Bin_Op_F_Right;
--# expr-done 5169
      if Fld_83 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_84 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_83);
--# expr-done 5170
Fld_84.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_84.all);
Eq_Node.Refs.Destroy (Fld_84.all);
--# expr-start 5171 '<FieldAccess .type_var>' Fld_85 /usr/src/packages/BUILD/ada/ast.py:9225
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_85 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5171
Fld_85.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_85.all);
Eq_Node.Refs.Destroy (Fld_85.all);
Bind_Result_1 := Bind_570_567.Create (Fld_84, Fld_85, Logic_Converter_570'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9225") else null)); 
--# expr-done 5172
And_Pred_16 := Logic_And (And_Pred_15, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9221") else null)); 
--# expr-done 5173
Or_Pred_5 := Logic_Or (And_Pred_14, And_Pred_16, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9182") else null)); 
--# expr-done 5174
--# expr-start 5186 '<And>' And_Pred_18 /usr/src/packages/BUILD/ada/ast.py:9231
--# expr-start 5181 '<And>' And_Pred_17 /usr/src/packages/BUILD/ada/ast.py:9231
--# expr-start 5177 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_8 /usr/src/packages/BUILD/ada/ast.py:9231
--# expr-start 5176 '<FieldAccess .type_var>' Fld_87 /usr/src/packages/BUILD/ada/ast.py:9232
--# expr-start 5175 '<FieldAccess .left>' Fld_86 /usr/src/packages/BUILD/ada/ast.py:9232
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_86 := Self.Bin_Op_F_Left;
--# expr-done 5175
      if Fld_86 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_87 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_86);
--# expr-done 5176
Fld_87.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_87.all);
Eq_Node.Refs.Destroy (Fld_87.all);
Pred_8 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_87, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 5177
--# expr-start 5180 '<Predicate on BaseTypeDecl.is_universal_type>' Pred_9 /usr/src/packages/BUILD/ada/ast.py:9233
--# expr-start 5179 '<FieldAccess .type_var>' Fld_89 /usr/src/packages/BUILD/ada/ast.py:9234
--# expr-start 5178 '<FieldAccess .right>' Fld_88 /usr/src/packages/BUILD/ada/ast.py:9234
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_88 := Self.Bin_Op_F_Right;
--# expr-done 5178
      if Fld_88 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_89 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_88);
--# expr-done 5179
Fld_89.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_89.all);
Eq_Node.Refs.Destroy (Fld_89.all);
Pred_9 := Base_Type_Decl_P_Is_Universal_Type_0_Pred.Create (Fld_89, Create_Base_Type_Decl_P_Is_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Universal_Type")            else null))); 
--# expr-done 5180
And_Pred_17 := Logic_And (Pred_8, Pred_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9231") else null)); 
--# expr-done 5181
--# expr-start 5185 '<Bind>' Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:9235
--# expr-start 5183 '<FieldAccess .type_var>' Fld_91 /usr/src/packages/BUILD/ada/ast.py:9235
--# expr-start 5182 '<FieldAccess .left>' Fld_90 /usr/src/packages/BUILD/ada/ast.py:9235
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_90 := Self.Bin_Op_F_Left;
--# expr-done 5182
      if Fld_90 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_91 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_90);
--# expr-done 5183
Fld_91.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_91.all);
Eq_Node.Refs.Destroy (Fld_91.all);
--# expr-start 5184 '<FieldAccess .type_var>' Fld_92 /usr/src/packages/BUILD/ada/ast.py:9235
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_92 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5184
Fld_92.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_92.all);
Eq_Node.Refs.Destroy (Fld_92.all);
Bind_Result_2 := Bind_570_567.Create (Fld_91, Fld_92, Logic_Converter_570'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9235") else null)); 
--# expr-done 5185
And_Pred_18 := Logic_And (And_Pred_17, Bind_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9231") else null)); 
--# expr-done 5186
Or_Pred_6 := Logic_Or (Or_Pred_5, And_Pred_18, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9182") else null)); 
--# expr-done 5187
--# expr-start 5204 '<And>' And_Pred_21 /usr/src/packages/BUILD/ada/ast.py:9242
--# expr-start 5199 '<And>' And_Pred_20 /usr/src/packages/BUILD/ada/ast.py:9242
--# expr-start 5194 '<And>' And_Pred_19 /usr/src/packages/BUILD/ada/ast.py:9242
--# expr-start 5190 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_10 /usr/src/packages/BUILD/ada/ast.py:9242
--# expr-start 5189 '<FieldAccess .type_var>' Fld_94 /usr/src/packages/BUILD/ada/ast.py:9243
--# expr-start 5188 '<FieldAccess .left>' Fld_93 /usr/src/packages/BUILD/ada/ast.py:9243
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_93 := Self.Bin_Op_F_Left;
--# expr-done 5188
      if Fld_93 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_94 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_93);
--# expr-done 5189
Fld_94.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_94.all);
Eq_Node.Refs.Destroy (Fld_94.all);
Pred_10 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_94, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 5190
--# expr-start 5193 '<Predicate on BaseTypeDecl.is_not_universal_type>' Pred_11 /usr/src/packages/BUILD/ada/ast.py:9244
--# expr-start 5192 '<FieldAccess .type_var>' Fld_96 /usr/src/packages/BUILD/ada/ast.py:9245
--# expr-start 5191 '<FieldAccess .right>' Fld_95 /usr/src/packages/BUILD/ada/ast.py:9245
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_95 := Self.Bin_Op_F_Right;
--# expr-done 5191
      if Fld_95 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_96 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_95);
--# expr-done 5192
Fld_96.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_96.all);
Eq_Node.Refs.Destroy (Fld_96.all);
Pred_11 := Base_Type_Decl_P_Is_Not_Universal_Type_0_Pred.Create (Fld_96, Create_Base_Type_Decl_P_Is_Not_Universal_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Not_Universal_Type")            else null))); 
--# expr-done 5193
And_Pred_19 := Logic_And (Pred_10, Pred_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9242") else null)); 
--# expr-done 5194
--# expr-start 5198 '<Bind>' Bind_Result_3 /usr/src/packages/BUILD/ada/ast.py:9246
--# expr-start 5196 '<FieldAccess .type_var>' Fld_98 /usr/src/packages/BUILD/ada/ast.py:9246
--# expr-start 5195 '<FieldAccess .left>' Fld_97 /usr/src/packages/BUILD/ada/ast.py:9246
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_97 := Self.Bin_Op_F_Left;
--# expr-done 5195
      if Fld_97 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_98 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_97);
--# expr-done 5196
Fld_98.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_98.all);
Eq_Node.Refs.Destroy (Fld_98.all);
--# expr-start 5197 '<FieldAccess .type_var>' Fld_99 /usr/src/packages/BUILD/ada/ast.py:9246
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_99 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5197
Fld_99.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_99.all);
Eq_Node.Refs.Destroy (Fld_99.all);
Bind_Result_3 := Bind_570_567.Create (Fld_98, Fld_99, Logic_Converter_570'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9246") else null)); 
--# expr-done 5198
And_Pred_20 := Logic_And (And_Pred_19, Bind_Result_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9242") else null)); 
--# expr-done 5199
--# expr-start 5203 '<Bind>' Bind_Result_4 /usr/src/packages/BUILD/ada/ast.py:9249
--# expr-start 5201 '<FieldAccess .type_var>' Fld_101 /usr/src/packages/BUILD/ada/ast.py:9249
--# expr-start 5200 '<FieldAccess .right>' Fld_100 /usr/src/packages/BUILD/ada/ast.py:9249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_100 := Self.Bin_Op_F_Right;
--# expr-done 5200
      if Fld_100 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_101 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_100);
--# expr-done 5201
Fld_101.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_101.all);
Eq_Node.Refs.Destroy (Fld_101.all);
--# expr-start 5202 '<FieldAccess .type_var>' Fld_102 /usr/src/packages/BUILD/ada/ast.py:9249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_102 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5202
Fld_102.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_102.all);
Eq_Node.Refs.Destroy (Fld_102.all);
Bind_Result_4 := Bind_570_567.Create (Fld_101, Fld_102, Logic_Converter_570'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9249") else null)); 
--# expr-done 5203
And_Pred_21 := Logic_And (And_Pred_20, Bind_Result_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9242") else null)); 
--# expr-done 5204
Or_Pred_7 := Logic_Or (Or_Pred_6, And_Pred_21, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9182") else null)); 
--# expr-done 5205
Let_Result_3 := Or_Pred_7; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2310;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5206
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5207
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_779;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_779;
               Finalizer_Scope_2307;
               Finalizer_Scope_2308;
               Finalizer_Scope_2309;
               Finalizer_Scope_2310;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Bin_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Relation_Op
   --
      procedure Initialize_Fields_For_Relation_Op
        (Self : Bare_Relation_Op
         ; Bin_Op_F_Left : Bare_Expr
         ; Bin_Op_F_Op : Bare_Op
         ; Bin_Op_F_Right : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Bin_Op
              (Self, Bin_Op_F_Left, Bin_Op_F_Op, Bin_Op_F_Right);
      end Initialize_Fields_For_Relation_Op;
--# property-start RelationOp.no_overload_equation /usr/src/packages/BUILD/ada/ast.py:9280
pragma Warnings (Off, "is not referenced");
 function Relation_Op_P_No_Overload_Equation
  (Node : Bare_Relation_Op
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Relation_Op := Bare_Relation_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_782;
      Fld : Bare_Expr;
Fld_1 : Logic_Var;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_782 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_782;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5215 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9281
--# expr-start 5212 '<FieldAccess .type_bind_var(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9281
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5209 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9281
--# expr-start 5208 '<FieldAccess .left>' Fld /usr/src/packages/BUILD/ada/ast.py:9281
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Bin_Op_F_Left;
--# expr-done 5208
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 5209
--# expr-start 5211 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9281
--# expr-start 5210 '<FieldAccess .right>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9281
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Bin_Op_F_Right;
--# expr-done 5210
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 5211
--# property-call-start AdaNode.type_bind_var
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_1, Right => Fld_3, Origin => Origin);
--# end
--# expr-done 5212
--# expr-start 5214 '<FieldAccess .bool_bind(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5213 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5213
--# property-call-start AdaNode.bool_bind
Fld_6 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_5, Origin => Origin);
--# end
--# expr-done 5214
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9281") else null)); 
--# expr-done 5215
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_782;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_782;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Relation_Op_P_No_Overload_Equation;
--# end
   --
   --  Primitives for Bare_Box_Expr
   --
      procedure Initialize_Fields_For_Box_Expr
        (Self : Bare_Box_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Box_Expr;
--# property-start BoxExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:11498
pragma Warnings (Off, "is not referenced");
 function Box_Expr_P_Xref_Equation
  (Node : Bare_Box_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Box_Expr := Bare_Box_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_899;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_899 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_899;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11498") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_899;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_899;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Box_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Case_Expr_Alternative
   --
      procedure Initialize_Fields_For_Case_Expr_Alternative
        (Self : Bare_Case_Expr_Alternative
         ; Case_Expr_Alternative_F_Choices : Bare_Alternatives_List
         ; Case_Expr_Alternative_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Case_Expr_Alternative_F_Choices := Case_Expr_Alternative_F_Choices;
            Self.Case_Expr_Alternative_F_Expr := Case_Expr_Alternative_F_Expr;
      end Initialize_Fields_For_Case_Expr_Alternative;
   function Case_Expr_Alternative_F_Choices
     (Node : Bare_Case_Expr_Alternative) return Bare_Alternatives_List
   is
   begin
         return Node.Case_Expr_Alternative_F_Choices;
   end;
   function Case_Expr_Alternative_F_Expr
     (Node : Bare_Case_Expr_Alternative) return Bare_Expr
   is
   begin
         return Node.Case_Expr_Alternative_F_Expr;
   end;
   --
   --  Primitives for Bare_Cond_Expr
   --
      procedure Initialize_Fields_For_Cond_Expr
        (Self : Bare_Cond_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Cond_Expr;
--# property-start '[dispatcher]CondExpr.dependent_exprs' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Cond_Expr_P_Dependent_Exprs
  (Node : Bare_Cond_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Cond_Expr := Bare_Cond_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Expr_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Cond_Expr (Self.Kind) is
               when Ada_Case_Expr =>
                  --# property-call-start CaseExpr.dependent_exprs
                  Property_Result := Case_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_If_Expr =>
                  --# property-call-start IfExpr.dependent_exprs
                  Property_Result := If_Expr_P_Dependent_Exprs
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Cond_Expr_P_Dependent_Exprs;
--# end
   --
   --  Primitives for Bare_Case_Expr
   --
      procedure Initialize_Fields_For_Case_Expr
        (Self : Bare_Case_Expr
         ; Case_Expr_F_Expr : Bare_Expr
         ; Case_Expr_F_Cases : Bare_Case_Expr_Alternative_List
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.Case_Expr_F_Expr := Case_Expr_F_Expr;
            Self.Case_Expr_F_Cases := Case_Expr_F_Cases;
      end Initialize_Fields_For_Case_Expr;
   function Case_Expr_F_Expr
     (Node : Bare_Case_Expr) return Bare_Expr
   is
   begin
         return Node.Case_Expr_F_Expr;
   end;
   function Case_Expr_F_Cases
     (Node : Bare_Case_Expr) return Bare_Case_Expr_Alternative_List
   is
   begin
         return Node.Case_Expr_F_Cases;
   end;
--# property-start CaseExpr.dependent_exprs /usr/src/packages/BUILD/ada/ast.py:11591
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Dependent_Exprs
  (Node : Bare_Case_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_Case_Expr := Bare_Case_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Expr :=
     Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_904;
      Fld : Internal_Entity_Case_Expr_Alternative_List;
Fld_1 : Bare_Case_Expr_Alternative_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_3 : Internal_Entity_Expr;
Item_178 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_178 : Bare_Case_Expr_Alternative;
Map_Result : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_904 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_904;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5219 '<Block>' Let_Result None
--# scope-start
--# expr-start 5218 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11592
--# expr-start 5216 '<FieldAccess .cases>' Fld /usr/src/packages/BUILD/ada/ast.py:11592
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 5216
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_1;
      begin
         for Untyped_Item_178 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_178 := Untyped_Item_178; 
Is_Null := Bare_Item_178 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_178); 
   If_Result := As_Entity;
end if;
                  Item_178 := If_Result; 
   --# scope-start
               --# bind c Item_178
      --# expr-start 5217 '<FieldAccess .expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11592
      if Item_178.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Item_178.Node.Case_Expr_Alternative_F_Expr, Info => Item_178.Info);
--# expr-done 5217
         declare
            Item_To_Append : constant Internal_Entity_Expr :=
               Fld_3;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Expr_Array
        (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5218
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5219
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_904;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_904;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Expr_P_Dependent_Exprs;
--# end
--# property-start CaseExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:11595
pragma Warnings (Off, "is not referenced");
 function Case_Expr_P_Xref_Equation
  (Node : Bare_Case_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Expr := Bare_Case_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Expr :=
     Internal_Entity_Case_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_905;
            procedure Finalizer_Scope_2313;
            procedure Finalizer_Scope_2314;
            procedure Finalizer_Scope_2315;
            procedure Finalizer_Scope_2316;
            procedure Finalizer_Scope_2317;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Expr;
Fld_1 : Bare_Expr;
Fld_2 : Logic_Var;
Pred : Logic_Equation;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Case_Expr_Alternative_List;
Fld_5 : Bare_Case_Expr_Alternative_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Case_Expr_Alternative;
Is_Null : Boolean;
If_Result : Internal_Entity_Case_Expr_Alternative;
Fld_7 : Internal_Entity_Alternatives_List;
Fld_8 : Bare_Alternatives_List;
Fld_9 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_10 : Logic_Var;
Fld_11 : Bare_Expr;
Fld_12 : Internal_Entity;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_15 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_180 : Internal_Entity;
Bare_Item_180 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_16 : Internal_Entity_Expr;
Fld_17 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Internal_Entity_Expr;
Fld_20 : Logic_Var;
Fld_21 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_179 : Internal_Entity_Case_Expr_Alternative;
Bare_Item_179 : Bare_Case_Expr_Alternative;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_905 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_905;
            procedure Finalizer_Scope_2313 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2313;
            procedure Finalizer_Scope_2314 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2314;
            procedure Finalizer_Scope_2315 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2315;
            procedure Finalizer_Scope_2316 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2316;
            procedure Finalizer_Scope_2317 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2317;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5247 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5224 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11598
--# expr-start 5220 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11598
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Expr_F_Expr, Info => Ent.Info);
--# expr-done 5220
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5223 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred /usr/src/packages/BUILD/ada/ast.py:11599
--# expr-start 5222 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11599
--# expr-start 5221 '<FieldAccess .expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11599
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Case_Expr_F_Expr;
--# expr-done 5221
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1);
--# expr-done 5222
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
Pred := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 5223
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5224
Block_Var := Fld_3; 
--# expr-start 5246 '<LogicAll>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:11602
--# expr-start 5245 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11602
--# expr-start 5225 '<FieldAccess .cases>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11602
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Case_Expr_Alternative_List (Node => Ent.Node.Case_Expr_F_Cases, Info => Ent.Info);
--# expr-done 5225
Fld_5 := Fld_4.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Case_Expr_Alternative_List := Fld_5;
      begin
         for Untyped_Item_179 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_179 := Untyped_Item_179; 
Is_Null := Bare_Item_179 = null; 
if Is_Null then
   If_Result := No_Entity_Case_Expr_Alternative;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_179); 
   If_Result := As_Entity;
end if;
                  Item_179 := If_Result; 
   --# scope-start
               --# bind alt Item_179
      --# expr-start 5244 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:11603
--# expr-start 5239 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:11603
--# expr-start 5236 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11603
--# expr-start 5235 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11603
--# expr-start 5226 '<FieldAccess .choices>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11603
      if Item_179.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Alternatives_List (Node => Item_179.Node.Case_Expr_Alternative_F_Choices, Info => Item_179.Info);
--# expr-done 5226
Fld_8 := Fld_7.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_8;
      begin
         for Untyped_Item_180 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_180 := Untyped_Item_180; 
Is_Null_1 := Bare_Item_180 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_9 := Fld_7.Info;
As_Entity_1 := (Info => Fld_9, Node => Bare_Item_180); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_180 := If_Result_1; 
   --# scope-start
               --# bind c Item_180
      --# expr-start 5234 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:11603
      if Item_180.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_180; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind e Match_0
--# expr-start 5232 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11606
--# expr-start 5230 '<FieldAccess .type_bind_val(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11606
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5227 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11606
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 5227
--# expr-start 5229 '<FieldAccess .type_val>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11606
--# expr-start 5228 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11606
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Case_Expr_F_Expr;
--# expr-done 5228
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_12 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_11);
--# end
--# expr-done 5229
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_10, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 5230
--# expr-start 5231 '<FieldAccess .sub_equation>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11607
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5231
And_Pred := Logic_And (Fld_13, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11606") else null)); 
--# expr-done 5232
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2315;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 5233 '<FieldAccess .xref_equation>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11610
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_15 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 5233
Let_Result_1 := Fld_15; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2316;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11612") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2317;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5234
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2314;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5235
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11603") else null)); 
--# expr-done 5236
--# expr-start 5238 '<FieldAccess .sub_equation>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:11618
--# expr-start 5237 '<FieldAccess .expr>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:11618
      if Item_179.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Expr (Node => Item_179.Node.Case_Expr_Alternative_F_Expr, Info => Item_179.Info);
--# expr-done 5237
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_17 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_16.Node, Env => Env, Origin => Origin, E_Info => Fld_16.Info);
--# end
--# expr-done 5238
And_Pred_1 := Logic_And (Logic_Boolean_Op, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11603") else null)); 
--# expr-done 5239
--# expr-start 5243 '<FieldAccess .type_bind_var(...)>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:11623
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5240 '<FieldAccess .type_var>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11623
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5240
--# expr-start 5242 '<FieldAccess .type_var>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11623
--# expr-start 5241 '<FieldAccess .expr>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11623
      if Item_179.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Expr (Node => Item_179.Node.Case_Expr_Alternative_F_Expr, Info => Item_179.Info);
--# expr-done 5241
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_19.Node);
--# expr-done 5242
--# property-call-start AdaNode.type_bind_var
Fld_21 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_20, Origin => Origin);
--# end
--# expr-done 5243
And_Pred_2 := Logic_And (And_Pred_1, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11603") else null)); 
--# expr-done 5244
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2313;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 5245
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11602") else null)); 
--# expr-done 5246
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5247
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_905;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_905;
               Finalizer_Scope_2313;
               Finalizer_Scope_2314;
               Finalizer_Scope_2315;
               Finalizer_Scope_2316;
               Finalizer_Scope_2317;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_If_Expr
   --
      procedure Initialize_Fields_For_If_Expr
        (Self : Bare_If_Expr
         ; If_Expr_F_Cond_Expr : Bare_Expr
         ; If_Expr_F_Then_Expr : Bare_Expr
         ; If_Expr_F_Alternatives : Bare_Elsif_Expr_Part_List
         ; If_Expr_F_Else_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Cond_Expr
              (Self);
            Self.If_Expr_F_Cond_Expr := If_Expr_F_Cond_Expr;
            Self.If_Expr_F_Then_Expr := If_Expr_F_Then_Expr;
            Self.If_Expr_F_Alternatives := If_Expr_F_Alternatives;
            Self.If_Expr_F_Else_Expr := If_Expr_F_Else_Expr;
      end Initialize_Fields_For_If_Expr;
   function If_Expr_F_Cond_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Cond_Expr;
   end;
   function If_Expr_F_Then_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Then_Expr;
   end;
   function If_Expr_F_Alternatives
     (Node : Bare_If_Expr) return Bare_Elsif_Expr_Part_List
   is
   begin
         return Node.If_Expr_F_Alternatives;
   end;
   function If_Expr_F_Else_Expr
     (Node : Bare_If_Expr) return Bare_Expr
   is
   begin
         return Node.If_Expr_F_Else_Expr;
   end;
--# property-start IfExpr.dependent_exprs /usr/src/packages/BUILD/ada/ast.py:11535
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Dependent_Exprs
  (Node : Bare_If_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Expr_Array_Access
is
   Self : Bare_If_Expr := Bare_If_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Expr :=
     Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_902;
      Fld : Internal_Entity_Expr;
Singleton : Internal_Entity_Expr_Array_Access;
Fld_1 : Internal_Entity_Elsif_Expr_Part_List;
Fld_2 : Bare_Elsif_Expr_Part_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Elsif_Expr_Part;
Fld_4 : Internal_Entity_Expr;
Item_181 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_181 : Bare_Elsif_Expr_Part;
Map_Result : Internal_Entity_Expr_Array_Access;
Concat_Result : Internal_Entity_Expr_Array_Access;
Fld_5 : Internal_Entity_Expr;
Singleton_1 : Internal_Entity_Expr_Array_Access;
Concat_Result_1 : Internal_Entity_Expr_Array_Access;
Let_Result : Internal_Entity_Expr_Array_Access;
            procedure Finalizer_Scope_902 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Singleton_1);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_902;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5255 '<Block>' Let_Result None
--# scope-start
--# expr-start 5254 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:11536
--# expr-start 5252 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:11536
--# expr-start 5248 '<FieldAccess .then_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 5248
                Singleton := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := Fld;
--# expr-start 5251 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11537
--# expr-start 5249 '<FieldAccess .alternatives>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11537
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 5249
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Internal_Entity_Expr_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Internal_Entity_Expr_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_2;
      begin
         for Untyped_Item_181 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_181 := Untyped_Item_181; 
Is_Null := Bare_Item_181 = null; 
if Is_Null then
   If_Result := No_Entity_Elsif_Expr_Part;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_181); 
   If_Result := As_Entity;
end if;
                  Item_181 := If_Result; 
   --# scope-start
               --# bind a Item_181
      --# expr-start 5250 '<FieldAccess .then_expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11537
      if Item_181.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Item_181.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_181.Info);
--# expr-done 5250
         declare
            Item_To_Append : constant Internal_Entity_Expr :=
               Fld_4;
         begin
            Internal_Entity_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Expr_Array
        (Items_Count => Natural (Internal_Entity_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Expr_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5251
Concat_Result := Concat (Singleton, Map_Result); 
--# expr-done 5252
--# expr-start 5253 '<FieldAccess .else_expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11538
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 5253
                Singleton_1 := Create_Internal_Entity_Expr_Array (Items_Count => 1);
                Singleton_1.Items (1) := Fld_5;
Concat_Result_1 := Concat (Concat_Result, Singleton_1); 
--# expr-done 5254
Let_Result := Concat_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5255
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_902;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_902;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Expr_P_Dependent_Exprs;
--# end
--# property-start IfExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:11541
pragma Warnings (Off, "is not referenced");
 function If_Expr_P_Xref_Equation
  (Node : Bare_If_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Expr := Bare_If_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Expr :=
     Internal_Entity_If_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_903;
            procedure Finalizer_Scope_2320;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Equation;
Fld_6 : Bare_Expr;
Fld_7 : Logic_Var;
Fld_8 : Bare_Expr;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Fld_14 : Bare_Expr;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_15 : Internal_Entity_Elsif_Expr_Part_List;
Fld_16 : Bare_Elsif_Expr_Part_List;
Fld_17 : Internal_Entity_Info;
As_Entity : Internal_Entity_Elsif_Expr_Part;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Elsif_Expr_Part;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Equation;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_22 : Internal_Entity_Expr;
Fld_23 : Logic_Var;
Fld_24 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_25 : Bare_Expr;
Fld_26 : Logic_Var;
Fld_27 : Internal_Entity_Expr;
Fld_28 : Logic_Var;
Fld_29 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Item_182 : Internal_Entity_Elsif_Expr_Part;
Bare_Item_182 : Bare_Elsif_Expr_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_30 : Bare_Expr;
Fld_31 : Logic_Var;
Fld_32 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_33 : Bare_Expr;
Fld_34 : Logic_Var;
Fld_35 : Logic_Var;
Fld_36 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_903 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_36);
                     Dec_Ref (And_Pred_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_903;
            procedure Finalizer_Scope_2320 is
            begin
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_21);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_24);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_5);
            end Finalizer_Scope_2320;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5305 '<Block>' Let_Result None
--# scope-start
--# expr-start 5304 '<And>' And_Pred_8 /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5299 '<And>' And_Pred_7 /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5295 '<And>' And_Pred_6 /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5276 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5260 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5257 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11544
--# expr-start 5256 '<FieldAccess .cond_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:11544
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Cond_Expr, Info => Ent.Info);
--# expr-done 5256
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5257
--# expr-start 5259 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11545
--# expr-start 5258 '<FieldAccess .then_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11545
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Then_Expr, Info => Ent.Info);
--# expr-done 5258
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5259
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11544") else null)); 
--# expr-done 5260
--# expr-start 5275 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11547
--# expr-start 5263 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11548
--# expr-start 5262 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:11548
--# expr-start 5261 '<FieldAccess .else_expr>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11548
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Self.If_Expr_F_Else_Expr;
--# expr-done 5261
Is_Null := Fld_14 = null; 
--# expr-done 5262
Not_Val := not (Is_Null); 
--# expr-done 5263
if Not_Val then
   --# expr-start 5271 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:11551
--# expr-start 5265 '<FieldAccess .sub_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11551
--# expr-start 5264 '<FieldAccess .else_expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11551
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.If_Expr_F_Else_Expr, Info => Ent.Info);
--# expr-done 5264
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5265
--# expr-start 5270 '<FieldAccess .type_bind_var(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11553
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5267 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11553
--# expr-start 5266 '<FieldAccess .then_expr>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11553
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.If_Expr_F_Then_Expr;
--# expr-done 5266
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 5267
--# expr-start 5269 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11554
--# expr-start 5268 '<FieldAccess .else_expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11554
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.If_Expr_F_Else_Expr;
--# expr-done 5268
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 5269
--# property-call-start AdaNode.type_bind_var
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_7, Right => Fld_9, Origin => Origin);
--# end
--# expr-done 5270
And_Pred_1 := Logic_And (Fld_5, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11551") else null)); 
--# expr-done 5271
   If_Result := And_Pred_1;
else
   --# expr-start 5274 '<FieldAccess .bool_bind(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11557
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5273 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11557
--# expr-start 5272 '<FieldAccess .then_expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11557
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.If_Expr_F_Then_Expr;
--# expr-done 5272
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 5273
--# property-call-start AdaNode.bool_bind
Fld_13 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_12, Origin => Origin);
--# end
--# expr-done 5274
   If_Result := Fld_13;
end if;
      Inc_Ref (If_Result);
--# expr-done 5275
And_Pred_2 := Logic_And (And_Pred, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11544") else null)); 
--# expr-done 5276
--# expr-start 5294 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11558
--# expr-start 5293 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11558
--# expr-start 5277 '<FieldAccess .alternatives>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11558
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Elsif_Expr_Part_List (Node => Ent.Node.If_Expr_F_Alternatives, Info => Ent.Info);
--# expr-done 5277
Fld_16 := Fld_15.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_16 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Elsif_Expr_Part_List := Fld_16;
      begin
         for Untyped_Item_182 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_182 := Untyped_Item_182; 
Is_Null_1 := Bare_Item_182 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Elsif_Expr_Part;
else
Fld_17 := Fld_15.Info;
As_Entity := (Info => Fld_17, Node => Bare_Item_182); 
   If_Result_1 := As_Entity;
end if;
                  Item_182 := If_Result_1; 
   --# scope-start
               --# bind elsif Item_182
      --# expr-start 5292 '<And>' And_Pred_5 /usr/src/packages/BUILD/ada/ast.py:11560
--# expr-start 5286 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:11560
--# expr-start 5282 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:11560
--# expr-start 5279 '<FieldAccess .sub_equation>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:11560
--# expr-start 5278 '<FieldAccess .cond_expr>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:11560
      if Item_182.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Expr (Node => Item_182.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_182.Info);
--# expr-done 5278
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_19 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_18.Node, Env => Env, Origin => Origin, E_Info => Fld_18.Info);
--# end
--# expr-done 5279
--# expr-start 5281 '<FieldAccess .sub_equation>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:11561
--# expr-start 5280 '<FieldAccess .then_expr>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:11561
      if Item_182.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Expr (Node => Item_182.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_182.Info);
--# expr-done 5280
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_21 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_20.Node, Env => Env, Origin => Origin, E_Info => Fld_20.Info);
--# end
--# expr-done 5281
And_Pred_3 := Logic_And (Fld_19, Fld_21, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11560") else null)); 
--# expr-done 5282
--# expr-start 5285 '<FieldAccess .bool_bind(...)>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:11564
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5284 '<FieldAccess .type_var>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:11564
--# expr-start 5283 '<FieldAccess .cond_expr>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:11564
      if Item_182.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Expr (Node => Item_182.Node.Elsif_Expr_Part_F_Cond_Expr, Info => Item_182.Info);
--# expr-done 5283
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_22.Node);
--# expr-done 5284
--# property-call-start AdaNode.bool_bind
Fld_24 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_23, Origin => Origin);
--# end
--# expr-done 5285
And_Pred_4 := Logic_And (And_Pred_3, Fld_24, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11560") else null)); 
--# expr-done 5286
--# expr-start 5291 '<FieldAccess .type_bind_var(...)>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:11568
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5288 '<FieldAccess .type_var>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:11568
--# expr-start 5287 '<FieldAccess .then_expr>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:11568
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Self.If_Expr_F_Then_Expr;
--# expr-done 5287
      if Fld_25 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_25);
--# expr-done 5288
--# expr-start 5290 '<FieldAccess .type_var>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:11569
--# expr-start 5289 '<FieldAccess .then_expr>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:11569
      if Item_182.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Expr (Node => Item_182.Node.Elsif_Expr_Part_F_Then_Expr, Info => Item_182.Info);
--# expr-done 5289
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27.Node);
--# expr-done 5290
--# property-call-start AdaNode.type_bind_var
Fld_29 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_26, Right => Fld_28, Origin => Origin);
--# end
--# expr-done 5291
And_Pred_5 := Logic_And (And_Pred_4, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11560") else null)); 
--# expr-done 5292
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2320;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5293
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11558") else null)); 
--# expr-done 5294
And_Pred_6 := Logic_And (And_Pred_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11544") else null)); 
--# expr-done 5295
--# expr-start 5298 '<FieldAccess .bool_bind(...)>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:11570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5297 '<FieldAccess .type_var>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:11570
--# expr-start 5296 '<FieldAccess .cond_expr>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:11570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Self.If_Expr_F_Cond_Expr;
--# expr-done 5296
      if Fld_30 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_30);
--# expr-done 5297
--# property-call-start AdaNode.bool_bind
Fld_32 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_31, Origin => Origin);
--# end
--# expr-done 5298
And_Pred_7 := Logic_And (And_Pred_6, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11544") else null)); 
--# expr-done 5299
--# expr-start 5303 '<FieldAccess .type_bind_var(...)>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:11571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5301 '<FieldAccess .type_var>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:11571
--# expr-start 5300 '<FieldAccess .then_expr>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:11571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Self.If_Expr_F_Then_Expr;
--# expr-done 5300
      if Fld_33 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_33);
--# expr-done 5301
--# expr-start 5302 '<FieldAccess .type_var>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:11571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5302
--# property-call-start AdaNode.type_bind_var
Fld_36 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_34, Right => Fld_35, Origin => Origin);
--# end
--# expr-done 5303
And_Pred_8 := Logic_And (And_Pred_7, Fld_36, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11544") else null)); 
--# expr-done 5304
Let_Result := And_Pred_8; Inc_Ref (Let_Result);
--# end
--# expr-done 5305
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_903;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_903;
               Finalizer_Scope_2320;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Contract_Cases
   --
      procedure Initialize_Fields_For_Contract_Cases
        (Self : Bare_Contract_Cases
         ; Contract_Cases_F_Contract_Cases : Bare_Contract_Case_Assoc_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Contract_Cases_F_Contract_Cases := Contract_Cases_F_Contract_Cases;
      end Initialize_Fields_For_Contract_Cases;
   function Contract_Cases_F_Contract_Cases
     (Node : Bare_Contract_Cases) return Bare_Contract_Case_Assoc_List
   is
   begin
         return Node.Contract_Cases_F_Contract_Cases;
   end;
   --
   --  Primitives for Bare_Decl_Expr
   --
      procedure Decl_Expr_Pre_Env_Actions
        (Self            : Bare_Decl_Expr;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_144 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Decl_Expr
        (Self : Bare_Decl_Expr
         ; Decl_Expr_F_Decls : Bare_Basic_Decl_List
         ; Decl_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Decl_Expr_F_Decls := Decl_Expr_F_Decls;
            Self.Decl_Expr_F_Expr := Decl_Expr_F_Expr;
      end Initialize_Fields_For_Decl_Expr;
   function Decl_Expr_F_Decls
     (Node : Bare_Decl_Expr) return Bare_Basic_Decl_List
   is
   begin
         return Node.Decl_Expr_F_Decls;
   end;
   function Decl_Expr_F_Expr
     (Node : Bare_Decl_Expr) return Bare_Expr
   is
   begin
         return Node.Decl_Expr_F_Expr;
   end;
--# property-start DeclExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:9004
pragma Warnings (Off, "is not referenced");
 function Decl_Expr_P_Xref_Equation
  (Node : Bare_Decl_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Decl_Expr := Bare_Decl_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Decl_Expr :=
     Internal_Entity_Decl_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_773;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Fld_3 : Bare_Expr;
Fld_4 : Logic_Var;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_773 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_773;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5315 '<Block>' Let_Result None
--# scope-start
--# expr-start 5314 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9005
--# expr-start 5309 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <FieldAccess .sub_equation>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:9005
--# expr-start 5306 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:9005
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5306
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5308 '<FieldAccess .sub_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9005
--# expr-start 5307 '<FieldAccess .expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9005
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Ent.Node.Decl_Expr_F_Expr, Info => Ent.Info);
--# expr-done 5307
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 5308
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5309
--# expr-start 5313 '<FieldAccess .type_bind_var(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9006
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5311 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9006
--# expr-start 5310 '<FieldAccess .expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9006
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Decl_Expr_F_Expr;
--# expr-done 5310
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_3);
--# expr-done 5311
--# expr-start 5312 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9006
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5312
--# property-call-start AdaNode.type_bind_var
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_4, Right => Fld_5, Origin => Origin);
--# end
--# expr-done 5313
And_Pred := Logic_And (Dyn_Var_Bind_Result, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9005") else null)); 
--# expr-done 5314
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 5315
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_773;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_773;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Decl_Expr_P_Xref_Equation;
--# end
--# property-start DeclExpr._env_trans_parent_144 /usr/src/packages/BUILD/ada/ast.py:8999
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_144
  (Node : Bare_Decl_Expr
  )
   return Boolean
is
   Self : Bare_Decl_Expr := Bare_Decl_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_144;
--# end
   --
   --  Primitives for Bare_Membership_Expr
   --
      procedure Initialize_Fields_For_Membership_Expr
        (Self : Bare_Membership_Expr
         ; Membership_Expr_F_Expr : Bare_Expr
         ; Membership_Expr_F_Op : Bare_Op
         ; Membership_Expr_F_Membership_Exprs : Bare_Expr_Alternatives_List
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Membership_Expr_F_Expr := Membership_Expr_F_Expr;
            Self.Membership_Expr_F_Op := Membership_Expr_F_Op;
            Self.Membership_Expr_F_Membership_Exprs := Membership_Expr_F_Membership_Exprs;
      end Initialize_Fields_For_Membership_Expr;
   function Membership_Expr_F_Expr
     (Node : Bare_Membership_Expr) return Bare_Expr
   is
   begin
         return Node.Membership_Expr_F_Expr;
   end;
   function Membership_Expr_F_Op
     (Node : Bare_Membership_Expr) return Bare_Op
   is
   begin
         return Node.Membership_Expr_F_Op;
   end;
   function Membership_Expr_F_Membership_Exprs
     (Node : Bare_Membership_Expr) return Bare_Expr_Alternatives_List
   is
   begin
         return Node.Membership_Expr_F_Membership_Exprs;
   end;
--# property-start MembershipExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:9297
pragma Warnings (Off, "is not referenced");
 function Membership_Expr_P_Xref_Equation
  (Node : Bare_Membership_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Membership_Expr := Bare_Membership_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Membership_Expr :=
     Internal_Entity_Membership_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_783;
            procedure Finalizer_Scope_2321;
            procedure Finalizer_Scope_2323;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Name;
Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Expr_Alternatives_List;
Fld_5 : Bare_Expr_Alternatives_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Expr;
Cast_Expr : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_7 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Expr_1 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Fld_9 : Logic_Equation;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Item_183 : Internal_Entity_Expr;
Bare_Item_183 : Bare_Expr;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
            procedure Finalizer_Scope_783 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_783;
            procedure Finalizer_Scope_2321 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2321;
            procedure Finalizer_Scope_2323 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2323;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5334 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:9298
--# expr-start 5320 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9298
--# expr-start 5317 '<FieldAccess .bool_bind(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9298
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5316 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:9298
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5316
--# property-call-start AdaNode.bool_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 5317
--# expr-start 5319 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9299
--# expr-start 5318 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9299
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 5318
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5319
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9298") else null)); 
--# expr-done 5320
--# expr-start 5333 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:9301
--# expr-start 5332 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9301
--# expr-start 5321 '<FieldAccess .membership_exprs>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9301
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr_Alternatives_List (Node => Ent.Node.Membership_Expr_F_Membership_Exprs, Info => Ent.Info);
--# expr-done 5321
Fld_5 := Fld_4.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Expr_Alternatives_List := Fld_5;
      begin
         for Untyped_Item_183 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_183 := Untyped_Item_183; 
Is_Null := Bare_Item_183 = null; 
if Is_Null then
   If_Result := No_Entity_Expr;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_183); 
   If_Result := As_Entity;
end if;
                  Item_183 := If_Result; 
   --# scope-start
               --# bind m Item_183
--# expr-start 5322 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9302
Cast_Expr := Item_183; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 5322
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
--# expr-start 5323 '<FieldAccess .name_designated_type>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9302
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_7 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5323
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 5325 '<FieldAccess .xref_no_overloading>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9304
--# expr-start 5324 '<Cast to Name>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:9304
Cast_Expr_1 := Item_183; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 5324
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5325
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2323;
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5331 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9307
--# expr-start 5326 '<FieldAccess .sub_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9307
      if Item_183.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_9 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_183.Node, Env => Env, Origin => Origin, E_Info => Item_183.Info);
--# end
--# expr-done 5326
--# expr-start 5330 '<FieldAccess .type_bind_var(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9307
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5328 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9308
--# expr-start 5327 '<FieldAccess .expr>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9308
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Expr (Node => Ent.Node.Membership_Expr_F_Expr, Info => Ent.Info);
--# expr-done 5327
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10.Node);
--# expr-done 5328
--# expr-start 5329 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9308
      if Item_183.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_183.Node);
--# expr-done 5329
--# property-call-start AdaNode.type_bind_var
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 5330
And_Pred_1 := Logic_And (Fld_9, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9307") else null)); 
--# expr-done 5331
   Result_Var_1 := And_Pred_1;
end if;
      Inc_Ref (Result_Var_1);
         declare
            Item_To_Append : constant Logic_Equation :=
               Result_Var_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2321;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 5332
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9301") else null)); 
--# expr-done 5333
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9298") else null)); 
--# expr-done 5334
      Property_Result := And_Pred_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_783;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_783;
               Finalizer_Scope_2321;
               Finalizer_Scope_2323;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Membership_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Name
   --
      procedure Initialize_Fields_For_Name
        (Self : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
      end Initialize_Fields_For_Name;
--# property-start Name.enclosing_defining_name /usr/src/packages/BUILD/ada/ast.py:9549
pragma Warnings (Off, "is not referenced");
 function Name_P_Enclosing_Defining_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_791;
      Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_791 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_791;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5339 '<Cast to DefiningName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9550
--# expr-start 5338 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9550
--# expr-start 5337 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9550
--# expr-start 5335 '<FieldAccess .parents>' Fld /usr/src/packages/BUILD/ada/ast.py:9550
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 5335
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_184 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_184
               --# expr-start 5336 '<IsA DefiningName>' Is_A /usr/src/packages/BUILD/ada/ast.py:9550
Is_A := Item_184.Node /= null 
and then Item_184.Node.Kind in Ada_Defining_Name_Range; 
--# expr-done 5336
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_184;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5337
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5338
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Defining_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Defining_Name
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result := No_Entity_Defining_Name;
   end if;
--# expr-done 5339
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_791;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_791;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Enclosing_Defining_Name;
--# end
--# property-start Name.is_defining /usr/src/packages/BUILD/ada/ast.py:9559
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Defining
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Dotted_Name;
Is_A : Boolean;
Fld : Bare_Ada_Node;
Is_A_1 : Boolean;
If_Result : Boolean;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Is_A_2 : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_4 : Bare_Base_Id;
Scope_Result : Bare_Base_Id;
Result_Var : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5350 '<Block>' Let_Result None
--# scope-start
--# expr-start 5340 '<IsA DefiningName>' Is_A /usr/src/packages/BUILD/ada/ast.py:9565
Is_A := Self /= null 
and then Self.Kind in Ada_Defining_Name_Range; 
--# expr-done 5340
if Is_A then
   If_Result := True;
else
   --# expr-start 5342 '<IsA DefiningName>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:9569
--# expr-start 5341 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:9569
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5341
Is_A_1 := Fld /= null 
and then Fld.Kind in Ada_Defining_Name_Range; 
--# expr-done 5342
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 5345 '<IsA DefiningName>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:9573
--# expr-start 5344 '<parent(<parent(<Var Self>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9573
--# expr-start 5343 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9573
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5343
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 5344
Is_A_2 := Fld_2 /= null 
and then Fld_2.Kind in Ada_Defining_Name_Range; 
--# expr-done 5345
if Is_A_2 then
   --# expr-start 5349 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9574
--# expr-start 5347 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9574
--# expr-start 5346 '<parent(<Var Self>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9574
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5346
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_3;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5347
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 5348 '<FieldAccess .suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9574
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr.Dotted_Name_F_Suffix;
--# expr-done 5348
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Bare_Ada_Node;
end if;
      Cast_Result_1 := Result_Var;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 5349
   If_Result_1 := Is_Equal;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 5350
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Defining;
--# end
--# property-start '[dispatcher]Name.parent_scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.parent_scope
                  Property_Result := Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.parent_scope
                  Property_Result := Defining_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.parent_scope
                  Property_Result := Dotted_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.parent_scope
                  Property_Result := End_Name_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.parent_scope
                  Property_Result := Base_Id_P_Parent_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Parent_Scope;
--# end
--# property-start Name.name_is /usr/src/packages/BUILD/ada/ast.py:9587
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Is
  (Node : Bare_Name
      ; Sym : Symbol_Type
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   Property_Result : Boolean;
      Var_Expr : Symbol_Type;
Fld : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5353 '<Block>' Let_Result None
--# scope-start
--# expr-start 5351 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:9591
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 5351
Var_Expr := Fld;
if Var_Expr /= null then
   --# scope-start
   --# bind ns Var_Expr
--# expr-start 5352 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9591
Is_Equal := Var_Expr = Sym; 
--# expr-done 5352
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5353
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Is;
--# end
--# property-start Name.is_direct_call /usr/src/packages/BUILD/ada/ast.py:9594
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Direct_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5359 '<Block>' Let_Result None
--# scope-start
--# expr-start 5354 '<FieldAccess .is_call>' Fld /usr/src/packages/BUILD/ada/ast.py:9600
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5354
if Fld then
   --# expr-start 5358 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9601
--# expr-start 5357 '<IsA AccessToSubpDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:9601
--# expr-start 5356 '<parent(<FieldAccess .called_subp_spec>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9601
--# expr-start 5355 '<FieldAccess .called_subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9601
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5355
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 5356
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 5357
Not_Val := not (Is_A); 
--# expr-done 5358
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5359
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Direct_Call;
--# end
--# property-start Name.is_access_call /usr/src/packages/BUILD/ada/ast.py:9605
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Access_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_2 : Internal_Entity;
Is_A : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5364 '<Block>' Let_Result None
--# scope-start
--# expr-start 5360 '<FieldAccess .is_call>' Fld /usr/src/packages/BUILD/ada/ast.py:9611
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5360
if Fld then
   --# expr-start 5363 '<IsA AccessToSubpDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:9612
--# expr-start 5362 '<parent(<FieldAccess .called_subp_spec>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9612
--# expr-start 5361 '<FieldAccess .called_subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9612
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5361
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# expr-done 5362
Is_A := Fld_2.Node /= null 
and then Fld_2.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 5363
   If_Result := Is_A;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5364
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Access_Call;
--# end
--# property-start Name.is_call /usr/src/packages/BUILD/ada/ast.py:9616
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Call
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Base_Formal_Param_Holder;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5370 '<Block>' Let_Result None
--# scope-start
--# expr-start 5366 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9621
--# expr-start 5365 '<FieldAccess .is_defining>' Fld /usr/src/packages/BUILD/ada/ast.py:9621
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5365
Not_Val := not (Fld); 
--# expr-done 5366
if Not_Val then
   --# expr-start 5369 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:9622
--# expr-start 5368 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:9622
--# expr-start 5367 '<FieldAccess .called_subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9622
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_1 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5367
Is_Null := Fld_1.Node = null; 
--# expr-done 5368
Not_Val_1 := not (Is_Null); 
--# expr-done 5369
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5370
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Call;
--# end
--# property-start Name.is_dot_call /usr/src/packages/BUILD/ada/ast.py:9627
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dot_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Info;
Fld_3 : Internal_Metadata;
Fld_4 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5377 '<Block>' Let_Result None
--# scope-start
--# expr-start 5372 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9632
--# expr-start 5371 '<FieldAccess .is_defining>' Fld /usr/src/packages/BUILD/ada/ast.py:9632
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5371
Not_Val := not (Fld); 
--# expr-done 5372
if Not_Val then
   --# expr-start 5376 '<FieldAccess .dottable_subp>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9633
--# expr-start 5375 '<FieldAccess .md>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9633
--# expr-start 5374 '<FieldAccess .info>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9633
--# expr-start 5373 '<FieldAccess .referenced_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9633
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5373
Fld_2 := Fld_1.Info;
--# expr-done 5374
Fld_3 := Fld_2.MD;
--# expr-done 5375
Fld_4 := Fld_3.Dottable_Subp;
--# expr-done 5376
   If_Result := Fld_4;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5377
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Dot_Call;
--# end
--# property-start Name.failsafe_referenced_def_name /usr/src/packages/BUILD/ada/ast.py:9638
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Def_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Def
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2327;
      Block_Var : Internal_Refd_Decl;
Block_Var_1 : Internal_Entity_Defining_Name;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Symbol_Type;
Fld_3 : Internal_Entity_Defining_Name_Array_Access;
Fld_4 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_5 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Fld_6 : Ref_Result_Kind;
New_Struct : Internal_Refd_Def;
Let_Result : Internal_Refd_Def;
            procedure Finalizer_Scope_2327 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2327;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5388 '<Block>' Let_Result None
--# scope-start
--# expr-start 5378 '<FieldAccess .failsafe_referenced_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:9643
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.failsafe_referenced_decl
Fld := Libadalang.Implementation.Name_P_Failsafe_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5378
Block_Var := Fld; 
--# bind ref_decl Block_Var
--# expr-start 5379 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9645
Fld_1 := Block_Var.Decl;
--# expr-done 5379
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 5380 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9646
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5380
Var_Expr_2 := Fld_2;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 5384 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9647
--# expr-start 5383 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9647
--# expr-start 5381 '<FieldAccess .defining_names>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9647
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5381
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_3;
      begin
         for Item_185 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_185
               --# expr-start 5382 '<FieldAccess .name_is(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9648
      if Item_185.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_4 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_185.Node, Sym => Var_Expr_2);
--# end
--# expr-done 5382
               if Fld_4 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_185;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5383
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5384
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2327;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5385 '<FieldAccess .defining_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9650
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5385
   Result_Var_1 := Fld_5;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Block_Var_1 := Result_Var_2; 
--# bind def_name Block_Var_1
--# expr-start 5387 '<New InternalRefdDef>' New_Struct /usr/src/packages/BUILD/ada/ast.py:9654
--# expr-start 5386 '<FieldAccess .kind>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9654
Fld_6 := Block_Var.Kind;
--# expr-done 5386
New_Struct := (Def_Name => Block_Var_1, Kind => Fld_6); 
--# expr-done 5387
Let_Result := New_Struct; 
--# end
--# expr-done 5388
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2327;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Failsafe_Referenced_Def_Name;
--# end
--# property-start Name.referenced_defining_name /usr/src/packages/BUILD/ada/ast.py:9658
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Defining_Name
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2331;
      Block_Var : Internal_Entity_Basic_Decl;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Symbol_Type;
Fld : Internal_Entity_Basic_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Defining_Name_Array_Access;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Scope_Result_1 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Result_Var_1 : Internal_Entity_Defining_Name;
Scope_Result_2 : Internal_Entity_Defining_Name;
Result_Var_2 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_2331 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_2331;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5396 '<Block>' Let_Result None
--# scope-start
--# expr-start 5389 '<FieldAccess .referenced_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:9663
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5389
Block_Var := Fld; 
--# bind ref_decl Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind ref_decl Var_Expr
--# expr-start 5390 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9664
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5390
Var_Expr_2 := Fld_1;
if Var_Expr_2 /= null then
   --# scope-start
   --# bind rel_name Var_Expr_2
--# expr-start 5394 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9665
--# expr-start 5393 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9665
--# expr-start 5391 '<FieldAccess .defining_names>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9665
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5391
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_2;
      begin
         for Item_186 of
               Collection.Items
         loop
   --# scope-start
               --# bind dn Item_186
               --# expr-start 5392 '<FieldAccess .name_is(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9666
      if Item_186.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Item_186.Node, Sym => Var_Expr_2);
--# end
--# expr-done 5392
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_186;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5393
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5394
Scope_Result := Get_Result;
   --# end
      Finalizer_Scope_2331;
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_1
Scope_Result_1 := Var_Expr_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5395 '<FieldAccess .defining_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9668
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5395
   Result_Var_1 := Fld_4;
end if;
Scope_Result_2 := Result_Var_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Defining_Name;
end if;
Let_Result := Result_Var_2; 
--# end
--# expr-done 5396
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2331;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Defining_Name;
--# end
--# property-start Name.gnat_xref_decl /usr/src/packages/BUILD/ada/ast.py:9672
pragma Warnings (Off, "is not referenced");
 function Name_P_Gnat_Xref_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Basic_Decl;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Var_Expr_5 : Internal_Entity_Basic_Decl;
Var_Expr_6 : Internal_Entity_Base_Type_Decl;
Var_Expr_7 : Internal_Entity_Basic_Decl;
Var_Expr_8 : Internal_Entity_Basic_Decl;
Var_Expr_9 : Internal_Entity_Basic_Decl;
Var_Expr_10 : Internal_Entity_Defining_Name;
Var_Expr_11 : Internal_Entity_Body_Node;
Fld : Boolean;
Fld_1 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_2 : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Param_Spec;
Fld_3 : Internal_Entity_Defining_Name;
Is_A_1 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Body_Node;
Fld_4 : Internal_Entity_Basic_Decl;
Fld_5 : Internal_Entity_Defining_Name;
Scope_Result_4 : Internal_Entity_Defining_Name;
Result_Var_4 : Internal_Entity_Defining_Name;
Is_A_2 : Boolean;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result : Boolean;
Scope_Result_5 : Boolean;
Result_Var_5 : Boolean;
Cast_Expr_3 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_7 : Internal_Entity_Base_Type_Decl;
Fld_8 : Internal_Entity_Defining_Name;
Scope_Result_6 : Internal_Entity_Defining_Name;
Result_Var_6 : Internal_Entity_Defining_Name;
Is_A_3 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_7 : Boolean;
Is_A_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_8 : Boolean;
Is_A_5 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_9 : Boolean;
Fld_9 : Internal_Entity_Body_Node;
Fld_10 : Internal_Entity_Defining_Name;
Scope_Result_10 : Internal_Entity_Defining_Name;
Result_Var_10 : Internal_Entity_Defining_Name;
Scope_Result_11 : Internal_Entity_Defining_Name;
Fld_11 : Internal_Entity_Defining_Name;
Result_Var_11 : Internal_Entity_Defining_Name;
Fld_12 : Internal_Entity_Defining_Name;
If_Result_1 : Internal_Entity_Defining_Name;
If_Result_2 : Internal_Entity_Defining_Name;
If_Result_3 : Internal_Entity_Defining_Name;
If_Result_4 : Internal_Entity_Defining_Name;
If_Result_5 : Internal_Entity_Defining_Name;
If_Result_6 : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5421 '<Block>' Let_Result None
--# scope-start
--# expr-start 5397 '<FieldAccess .is_defining>' Fld /usr/src/packages/BUILD/ada/ast.py:9677
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5397
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 5398 '<FieldAccess .enclosing_defining_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9678
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld_1 := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5398
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Block_Var := Result_Var; 
--# bind dn Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind dn Var_Expr_1
--# expr-start 5399 '<FieldAccess .basic_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9680
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_2 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 5399
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Basic_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind bd Block_Var_1
--# expr-start 5420 '<Cond>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:9682
Var_Expr_2 := Block_Var_1;
if Var_Expr_2 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_2
--# expr-start 5400 '<IsA ParamSpec>' Is_A /usr/src/packages/BUILD/ada/ast.py:9683
Is_A := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Param_Spec_Range; 
--# expr-done 5400
Scope_Result_2 := Is_A;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   --# expr-start 5402 '<FieldAccess .decl_param(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9684
--# expr-start 5401 '<Cast to ParamSpec>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9684
Cast_Expr := Block_Var_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result := Create_Internal_Entity_Param_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Param_Spec;
   end if;
--# expr-done 5401
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.decl_param
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Decl_Param (Node => Cast_Result.Node, Param => Block_Var, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result.Info);
--# end
--# expr-done 5402
   If_Result_6 := Fld_3;
else
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 5403 '<IsA Body>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:9686
Is_A_1 := Var_Expr_3.Node /= null 
and then Var_Expr_3.Node.Kind in Ada_Body_Node; 
--# expr-done 5403
Scope_Result_3 := Is_A_1;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
if Result_Var_3 then
--# expr-start 5405 '<FieldAccess .decl_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9687
--# expr-start 5404 '<Cast to Body>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:9687
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Body_Node
   then
      Cast_Result_1 := Create_Internal_Entity_Body_Node
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Body_Node;
   end if;
--# expr-done 5404
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.decl_part
Fld_4 := Libadalang.Implementation.Body_Node_P_Decl_Part (Node => Cast_Result_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 5405
Var_Expr_4 := Fld_4;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5406 '<FieldAccess .defining_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9687
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 5406
Scope_Result_4 := Fld_5;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Defining_Name;
end if;
   If_Result_5 := Result_Var_4;
else
Var_Expr_5 := Block_Var_1;
if Var_Expr_5 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_5
--# expr-start 5407 '<IsA BaseTypeDecl>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:9689
Is_A_2 := Var_Expr_5.Node /= null 
and then Var_Expr_5.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 5407
if Is_A_2 then
   --# expr-start 5409 '<FieldAccess .is_in_private_part>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9690
--# expr-start 5408 '<Cast to BaseTypeDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:9690
Cast_Expr_2 := Var_Expr_5; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5408
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_in_private_part'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_In_Private_Part (Node => Cast_Result_2.Node, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 5409
   If_Result := Fld_6;
else
   If_Result := False;
end if;
Scope_Result_5 := If_Result;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
--# expr-start 5411 '<FieldAccess .previous_part(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9691
--# expr-start 5410 '<Cast to BaseTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:9691
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 5410
      if Cast_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Cast_Result_3.Node, Go_To_Incomplete => True, E_Info => Cast_Result_3.Info);
--# end
--# expr-done 5411
Var_Expr_6 := Fld_7;
if Var_Expr_6 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5412 '<FieldAccess .defining_name>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9691
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 5412
Scope_Result_6 := Fld_8;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Defining_Name;
end if;
   If_Result_4 := Result_Var_6;
else
Var_Expr_7 := Block_Var_1;
if Var_Expr_7 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_7
--# expr-start 5413 '<IsA ObjectDecl>' Is_A_3 /usr/src/packages/BUILD/ada/ast.py:9693
Is_A_3 := Var_Expr_7.Node /= null 
and then Var_Expr_7.Node.Kind in Ada_Object_Decl_Range; 
--# expr-done 5413
Scope_Result_7 := Is_A_3;
   --# end
   Result_Var_7 := Scope_Result_7;
else
   Result_Var_7 := False;
end if;
if Result_Var_7 then
   If_Result_3 := No_Entity_Defining_Name;
else
Var_Expr_8 := Block_Var_1;
if Var_Expr_8 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_8
--# expr-start 5414 '<IsA NumberDecl>' Is_A_4 /usr/src/packages/BUILD/ada/ast.py:9698
Is_A_4 := Var_Expr_8.Node /= null 
and then Var_Expr_8.Node.Kind in Ada_Number_Decl_Range; 
--# expr-done 5414
Scope_Result_8 := Is_A_4;
   --# end
   Result_Var_8 := Scope_Result_8;
else
   Result_Var_8 := False;
end if;
if Result_Var_8 then
   If_Result_2 := No_Entity_Defining_Name;
else
Var_Expr_9 := Block_Var_1;
if Var_Expr_9 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_9
--# expr-start 5415 '<IsA BasicDecl>' Is_A_5 /usr/src/packages/BUILD/ada/ast.py:9702
Is_A_5 := Var_Expr_9.Node /= null 
and then Var_Expr_9.Node.Kind in Ada_Basic_Decl; 
--# expr-done 5415
Scope_Result_9 := Is_A_5;
   --# end
   Result_Var_9 := Scope_Result_9;
else
   Result_Var_9 := False;
end if;
if Result_Var_9 then
--# expr-start 5416 '<FieldAccess .body_part_for_decl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9703
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.body_part_for_decl
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Body_Part_For_Decl (Node => Block_Var_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var_1.Info);
--# end
--# expr-done 5416
Var_Expr_11 := Fld_9;
if Var_Expr_11 /= No_Entity_Body_Node then
   --# scope-start
   --# bind bpe Var_Expr_11
--# expr-start 5417 '<FieldAccess .defining_name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9703
      if Var_Expr_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_11.Node, E_Info => Var_Expr_11.Info);
--# end
--# expr-done 5417
Scope_Result_10 := Fld_10;
   --# end
   Result_Var_10 := Scope_Result_10;
else
   Result_Var_10 := No_Entity_Defining_Name;
end if;
Var_Expr_10 := Result_Var_10;
if Var_Expr_10 /= No_Entity_Defining_Name then
   --# scope-start
   --# bind e Var_Expr_10
Scope_Result_11 := Var_Expr_10;
   --# end
   Result_Var_11 := Scope_Result_11;
else
   --# expr-start 5418 '<FieldAccess .defining_name>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9704
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 5418
   Result_Var_11 := Fld_11;
end if;
   If_Result_1 := Result_Var_11;
else
   --# expr-start 5419 '<FieldAccess .referenced_defining_name>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9706
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_defining_name
Fld_12 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5419
   If_Result_1 := Fld_12;
end if;
   If_Result_2 := If_Result_1;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
   If_Result_5 := If_Result_4;
end if;
   If_Result_6 := If_Result_5;
end if;
--# expr-done 5420
Let_Result := If_Result_6; 
--# end
--# expr-done 5421
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Gnat_Xref_Decl;
--# end
--# property-start '[dispatcher]Name.all_env_els_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.all_env_els_impl
                  Property_Result := Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.all_env_els_impl
                  Property_Result := Defining_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.all_env_els_impl
                  Property_Result := Dotted_Name_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.all_env_els_impl
                  Property_Result := Base_Id_P_All_Env_Els_Impl
                    (Self
                        , Seq
                        , Seq_From
                        , Categories
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_All_Env_Els_Impl;
--# end
--# property-start Name.all_env_elements_internal /usr/src/packages/BUILD/ada/ast.py:9720
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements_Internal
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_803;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_803 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_803;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5427 '<Block>' Let_Result None
--# scope-start
--# expr-start 5426 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .all_env_els_impl(...)>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:9729
--# expr-start 5422 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:9730
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5422
Bound_Origin := Fld; 
--# expr-start 5425 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .all_env_els_impl(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:9731
--# expr-start 5423 '<FieldAccess .node_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9732
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5423
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 5424 '<FieldAccess .all_env_els_impl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9733
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 5424
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5425
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 5426
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 5427
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_803;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_803;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_All_Env_Elements_Internal;
--# end
--# property-start Name.all_env_elements /usr/src/packages/BUILD/ada/ast.py:9742
pragma Warnings (Off, "is not referenced");
 function Name_P_All_Env_Elements
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_804;
      Fld : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_804 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_804;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5429 '<Block>' Let_Result None
--# scope-start
--# expr-start 5428 '<FieldAccess .all_env_elements_internal(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:9749
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, Seq => Seq, Seq_From => Seq_From, E_Info => Ent.Info);
--# end
--# expr-done 5428
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 5429
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_804;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_804;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_All_Env_Elements;
--# end
--# property-start Name.first_corresponding_decl /usr/src/packages/BUILD/ada/ast.py:9752
pragma Warnings (Off, "is not referenced");
 function Name_P_First_Corresponding_Decl
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_805;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Bare_Ada_Node;
Is_A : Boolean;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_805 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_805;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5440 '<Block>' Let_Result None
--# scope-start
--# expr-start 5439 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9753
--# expr-start 5431 '<IsA DottedName>' Is_A /usr/src/packages/BUILD/ada/ast.py:9754
--# expr-start 5430 '<parent(<Var Self>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9754
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5430
Is_A := Fld_3 /= null 
and then Fld_3.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5431
if Is_A then
   --# expr-start 5432 '<FieldAccess .is_suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9754
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 5432
   If_Result := Fld_4;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5435 '<FieldAccess .first_corresponding_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9755
--# expr-start 5434 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9755
--# expr-start 5433 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:9755
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5433
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 5434
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5435
   If_Result_1 := Fld_1;
else
   --# expr-start 5438 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:9756
--# expr-start 5437 '<at(<FieldAccess .all_env_elements_internal>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9756
--# expr-start 5436 '<FieldAccess .all_env_elements_internal>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9756
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_2 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5436
Get_Result := Get (Fld_2, 0, True); 
--# expr-done 5437
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5438
   If_Result_1 := Cast_Result_1;
end if;
--# expr-done 5439
Let_Result := If_Result_1; 
--# end
--# expr-done 5440
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_805;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_805;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_First_Corresponding_Decl;
--# end
--# property-start Name.bottom_up_name_equation /usr/src/packages/BUILD/ada/ast.py:9760
pragma Warnings (Off, "is not referenced");
 function Name_P_Bottom_Up_Name_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_806;
            procedure Finalizer_Scope_2346;
            procedure Finalizer_Scope_2347;
            procedure Finalizer_Scope_2348;
            procedure Finalizer_Scope_2349;
      Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Explicit_Deref;
Match_2 : Internal_Entity_Qual_Expr;
Match_3 : Internal_Entity_Name;
Fld : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Equation;
Logic_False : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Qual_Expr;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_806 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_806;
            procedure Finalizer_Scope_2346 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2346;
            procedure Finalizer_Scope_2347 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2347;
            procedure Finalizer_Scope_2348 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2348;
            procedure Finalizer_Scope_2349 is
            begin
                     Dec_Ref (Logic_False);
            end Finalizer_Scope_2349;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5447 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5446 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9761
--# expr-start 5442 '<as_entity(<FieldAccess .innermost_name>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:9761
--# expr-start 5441 '<FieldAccess .innermost_name>' Fld /usr/src/packages/BUILD/ada/ast.py:9761
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.innermost_name
Fld := Libadalang.Implementation.Name_P_Innermost_Name (Node => Self);
--# end
--# expr-done 5441
Is_Null := Fld = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld); 
   If_Result := As_Entity;
end if;
--# expr-done 5442
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := If_Result; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 5443 '<FieldAccess .general_xref_equation(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9762
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.general_xref_equation
Fld_1 := Libadalang.Implementation.Call_Expr_P_General_Xref_Equation (Node => Match_0.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 5443
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2346;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Explicit_Deref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 5444 '<FieldAccess .general_xref_equation(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9763
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ExplicitDeref.general_xref_equation
Fld_2 := Libadalang.Implementation.Explicit_Deref_P_General_Xref_Equation (Node => Match_1.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 5444
Let_Result_1 := Fld_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2347;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Qual_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Qual_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind qe Match_2
--# expr-start 5445 '<FieldAccess .general_xref_equation(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9764
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.general_xref_equation
Fld_3 := Libadalang.Implementation.Qual_Expr_P_General_Xref_Equation (Node => Match_2.Node, Root => Self, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 5445
Let_Result_2 := Fld_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2348;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Char_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9765") else null)); 
Let_Result_3 := Logic_False; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2349;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5446
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5447
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_806;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_806;
               Finalizer_Scope_2346;
               Finalizer_Scope_2347;
               Finalizer_Scope_2348;
               Finalizer_Scope_2349;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Bottom_Up_Name_Equation;
--# end
--# property-start Name.innermost_name /usr/src/packages/BUILD/ada/ast.py:9769
pragma Warnings (Off, "is not referenced");
 function Name_P_Innermost_Name
  (Node : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Name;
      Block_Var : Bare_Name;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Name;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Call_Expr;
Let_Result : Bare_Name;
Scope_Result : Bare_Name;
Cast_Result_1 : Bare_Explicit_Deref;
Let_Result_1 : Bare_Name;
Scope_Result_1 : Bare_Name;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Bare_Name;
Scope_Result_2 : Bare_Name;
Match_Result : Bare_Name;
Is_A : Boolean;
Fld_2 : Bare_Name;
Is_A_1 : Boolean;
If_Result : Bare_Name;
If_Result_1 : Bare_Name;
Let_Result_3 : Bare_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5455 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5450 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9782
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind ce Match_0
--# expr-start 5448 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:9783
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_0.Call_Expr_F_Name;
--# expr-done 5448
Let_Result := Fld; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind ed Match_1
--# expr-start 5449 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9784
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Match_1.Explicit_Deref_F_Prefix;
--# expr-done 5449
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Bare_Ada_Node; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5450
Block_Var := Match_Result; 
--# bind name Block_Var
--# expr-start 5454 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9789
--# expr-start 5451 '<IsA CallExpr, ExplicitDeref>' Is_A /usr/src/packages/BUILD/ada/ast.py:9790
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Call_Expr_Range | Ada_Explicit_Deref_Range; 
--# expr-done 5451
if Is_A then
   --# expr-start 5452 '<FieldAccess .innermost_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9790
      if Block_Var = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.innermost_name
Fld_2 := Libadalang.Implementation.Name_P_Innermost_Name (Node => Block_Var);
--# end
--# expr-done 5452
   If_Result_1 := Fld_2;
else
--# expr-start 5453 '<IsA QualExpr>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:9791
Is_A_1 := Block_Var /= null 
and then Block_Var.Kind in Ada_Qual_Expr_Range; 
--# expr-done 5453
if Is_A_1 then
   If_Result := Block_Var;
else
   If_Result := Self;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 5454
Let_Result_3 := If_Result_1; 
--# end
--# expr-done 5455
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Innermost_Name;
--# end
--# property-start Name.parent_name_equation /usr/src/packages/BUILD/ada/ast.py:9796
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name_Equation
  (Node : Bare_Name
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_808;
            procedure Finalizer_Scope_2357;
            procedure Finalizer_Scope_2358;
            procedure Finalizer_Scope_2360;
            procedure Finalizer_Scope_2361;
      Block_Var : Internal_Entity_Access_To_Subp_Def;
Block_Var_1 : Boolean;
Block_Var_2 : Boolean;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Base_Type_Decl;
Match_0 : Bare_Call_Expr;
Match_1 : Bare_Explicit_Deref;
Match_2 : Bare_Name;
Var_Expr_4 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_5 : Internal_Entity_Name;
Var_Expr_6 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Access_Def;
Scope_Result : Internal_Entity_Access_Def;
Result_Var : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_1 : Internal_Entity_Subp_Spec;
Fld_2 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_3 : Internal_Entity_Subp_Spec;
Fld_4 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Is_A : Boolean;
Not_Val : Boolean;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_3 : Internal_Entity_Base_Type_Decl;
Result_Var_3 : Internal_Entity_Base_Type_Decl;
Logic_False : Logic_Equation;
As_Entity : Internal_Entity_Call_Expr;
Is_Null : Boolean;
If_Result : Internal_Entity_Call_Expr;
Fld_6 : Logic_Equation;
As_Entity_1 : Internal_Entity_Explicit_Deref;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Explicit_Deref;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Internal_Entity_Subp_Spec;
Scope_Result_4 : Internal_Entity_Subp_Spec;
Result_Var_4 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Bare_Ada_Node;
As_Entity_2 : Internal_Entity;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Match_Prefix : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
Let_Result : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Cast_Result_3 : Bare_Explicit_Deref;
Let_Result_1 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_4 : Bare_Name;
Let_Result_2 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_12 : Bare_Name;
As_Entity_3 : Internal_Entity_Name;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Name;
Is_A_1 : Boolean;
Is_Null_4 : Boolean;
Not_Val_1 : Boolean;
If_Result_4 : Boolean;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_16 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
Cast_Expr : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_17 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_5 : Boolean;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Equation;
If_Result_7 : Logic_Equation;
If_Result_8 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null_5 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_3 : Logic_Equation;
            procedure Finalizer_Scope_808 is
            begin
                     Dec_Ref (Logic_False);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_808;
            procedure Finalizer_Scope_2357 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2357;
            procedure Finalizer_Scope_2358 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2358;
            procedure Finalizer_Scope_2360 is
            begin
                     Dec_Ref (Bind_Result_1);
            end Finalizer_Scope_2360;
            procedure Finalizer_Scope_2361 is
            begin
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_16);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_19);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (If_Result_8);
            end Finalizer_Scope_2361;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5496 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5457 '<Cast to AccessToSubpDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9800
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5456 '<FieldAccess .access_def>' Fld /usr/src/packages/BUILD/ada/ast.py:9800
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 5456
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Access_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 5457
Block_Var := Cast_Result; 
--# bind as_subp_access Block_Var
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5459 '<FieldAccess .paramless(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9801
--# expr-start 5458 '<FieldAccess .subp_spec>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9801
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_1.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_1.Info);
--# expr-done 5458
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_2 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_1.Node, Dottable_Subp => False, Can_Be => False);
--# end
--# expr-done 5459
Scope_Result_1 := Fld_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Block_Var_1 := Result_Var_1; 
--# bind is_paramless Block_Var_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5461 '<FieldAccess .paramless(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9805
--# expr-start 5460 '<FieldAccess .subp_spec>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9805
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 5460
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.paramless
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Fld_3.Node, Dottable_Subp => False, Can_Be => True);
--# end
--# expr-done 5461
Scope_Result_2 := Fld_4;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Block_Var_2 := Result_Var_2; 
--# bind can_be_paramless Block_Var_2
Var_Expr_3 := Typ;
if Var_Expr_3 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5464 '<FieldAccess .comp_type(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9810
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5463 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9810
--# expr-start 5462 '<IsA ExplicitDeref>' Is_A /usr/src/packages/BUILD/ada/ast.py:9810
Is_A := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 5462
Not_Val := not (Is_A); 
--# expr-done 5463
--# property-call-start BaseTypeDecl.comp_type
Fld_5 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_3.Node, Is_Subscript => Not_Val, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 5464
Scope_Result_3 := Fld_5;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Base_Type_Decl;
end if;
Block_Var_3 := Result_Var_3; 
--# bind comp_type Block_Var_3
--# expr-start 5495 '<If>' If_Result_9 /usr/src/packages/BUILD/ada/ast.py:9812
--# expr-start 5465 '<IsNull>' Is_Null_5 /usr/src/packages/BUILD/ada/ast.py:9813
Is_Null_5 := Typ.Node = null; 
--# expr-done 5465
if Is_Null_5 then
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9814") else null)); 
   If_Result_9 := Logic_False;
else
   --# expr-start 5494 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9816
--# expr-start 5477 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9816
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_0 := Cast_Result_2; 
--# bind ce Match_0
--# expr-start 5467 '<FieldAccess .subscriptable_type_equation(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9818
--# expr-start 5466 '<as_entity(<Var ce>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:9818
Is_Null := Match_0 = null; 
if Is_Null then
   If_Result := No_Entity_Call_Expr;
else
As_Entity := (Info => E_Info, Node => Match_0); 
   If_Result := As_Entity;
end if;
--# expr-done 5466
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.subscriptable_type_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subscriptable_Type_Equation (Node => If_Result.Node, Typ => Typ, Env => Env, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 5467
Let_Result := Fld_6; Inc_Ref (Let_Result);
--# end
Scope_Result_5 := Let_Result;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2357;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
         when Ada_Explicit_Deref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_1 := Cast_Result_3; 
--# bind ed Match_1
--# expr-start 5473 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9821
--# expr-start 5469 '<FieldAccess .eq_for_type(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9821
--# expr-start 5468 '<as_entity(<Var ed>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9821
Is_Null_1 := Match_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Explicit_Deref;
else
As_Entity_1 := (Info => E_Info, Node => Match_1); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 5468
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ExplicitDeref.eq_for_type
Fld_7 := Libadalang.Implementation.Explicit_Deref_P_Eq_For_Type (Node => If_Result_1.Node, Typ => Typ, Env => Env, Origin => Origin);
--# end
--# expr-done 5469
--# expr-start 5472 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:9826
--# expr-start 5470 '<FieldAccess .subp_spec_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5470
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Var_Expr_4 := Block_Var;
if Var_Expr_4 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
--# expr-start 5471 '<FieldAccess .subp_spec>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9826
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_4.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_4.Info);
--# expr-done 5471
Scope_Result_4 := Fld_9;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Subp_Spec;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
Bind_Result := Bind_Default_Default.Create (Fld_8, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9826") else null)); 
--# expr-done 5472
And_Pred := Logic_And (Fld_7, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9821") else null)); 
--# expr-done 5473
Let_Result_1 := And_Pred; Inc_Ref (Let_Result_1);
--# end
Scope_Result_6 := Let_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2358;
            Match_Result := Scope_Result_6; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_End_Name | Ada_Qual_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_2 := Cast_Result_4; 
--# bind _ Match_2
--# expr-start 5476 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:9828
--# expr-start 5474 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9828
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 5474
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
--# expr-start 5475 '<FieldAccess .node>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9828
Fld_11 := No_Entity.Node;
--# expr-done 5475
Is_Null_2 := Fld_11 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity;
else
As_Entity_2 := (Info => E_Info, Node => Fld_11); 
   If_Result_2 := As_Entity_2;
end if;
Bind_Result_1 := Bind_Default_Default.Create (Fld_10, If_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9828") else null)); 
--# expr-done 5476
Let_Result_2 := Bind_Result_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_7 := Let_Result_2;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2360;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
end case;
--# expr-done 5477
--# expr-start 5479 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:9829
--# expr-start 5478 '<FieldAccess .parent_name(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9829
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_12 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 5478
Is_Null_3 := Fld_12 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Name;
else
As_Entity_3 := (Info => E_Info, Node => Fld_12); 
   If_Result_3 := As_Entity_3;
end if;
--# expr-done 5479
Var_Expr_5 := If_Result_3;
if Var_Expr_5 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_5
--# expr-start 5493 '<Cond>' If_Result_8 /usr/src/packages/BUILD/ada/ast.py:9830
--# expr-start 5480 '<IsA ExplicitDeref>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:9831
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Explicit_Deref_Range; 
--# expr-done 5480
if Is_A_1 then
   --# expr-start 5482 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:9831
--# expr-start 5481 '<IsNull>' Is_Null_4 /usr/src/packages/BUILD/ada/ast.py:9831
Is_Null_4 := Block_Var.Node = null; 
--# expr-done 5481
Not_Val_1 := not (Is_Null_4); 
--# expr-done 5482
   If_Result_4 := Not_Val_1;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 5488 '<Cond>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:9835
if Block_Var_1 then
   --# expr-start 5483 '<FieldAccess .parent_name_equation(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9841
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5483
   If_Result_6 := Fld_13;
else
if Block_Var_2 then
   --# expr-start 5486 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:9848
--# expr-start 5484 '<FieldAccess .parent_name_equation(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:9849
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_14 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5484
--# expr-start 5485 '<FieldAccess .parent_name_equation(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:9850
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5485
Or_Pred := Logic_Or (Fld_14, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9848") else null)); 
--# expr-done 5486
   If_Result_5 := Or_Pred;
else
   --# expr-start 5487 '<FieldAccess .parent_name_equation(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:9856
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_16 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5487
   If_Result_5 := Fld_16;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 5488
   If_Result_8 := If_Result_6;
else
--# expr-start 5489 '<Cast to CallExpr>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:9860
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 5489
Var_Expr_6 := Cast_Result_5;
if Var_Expr_6 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 5490 '<FieldAccess .check_array_slice(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:9860
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_17 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Var_Expr_6.Node, Typ => Typ, Origin => Origin, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 5490
Scope_Result_8 := Fld_17;
   --# end
   Result_Var_5 := Scope_Result_8;
else
   Result_Var_5 := False;
end if;
if Result_Var_5 then
   --# expr-start 5491 '<FieldAccess .parent_name_equation(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:9861
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_18 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Typ, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5491
   If_Result_7 := Fld_18;
else
   --# expr-start 5492 '<FieldAccess .parent_name_equation(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:9865
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_19 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_5.Node, Typ => Block_Var_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 5492
   If_Result_7 := Fld_19;
end if;
      Inc_Ref (If_Result_7);
   If_Result_8 := If_Result_7;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 5493
Scope_Result_9 := If_Result_8;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2361;
   Result_Var_6 := Scope_Result_9;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9867") else null)); 
   Result_Var_6 := Logic_True;
end if;
      Inc_Ref (Result_Var_6);
And_Pred_1 := Logic_And (Match_Result, Result_Var_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9816") else null)); 
--# expr-done 5494
   If_Result_9 := And_Pred_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 5495
Let_Result_3 := If_Result_9; Inc_Ref (Let_Result_3);
--# end
--# expr-done 5496
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_808;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_808;
               Finalizer_Scope_2357;
               Finalizer_Scope_2358;
               Finalizer_Scope_2360;
               Finalizer_Scope_2361;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Name_Equation;
--# end
--# property-start Name.subtype_indication_equation /usr/src/packages/BUILD/ada/ast.py:9872
pragma Warnings (Off, "is not referenced");
 function Name_P_Subtype_Indication_Equation
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_809;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_809 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_809;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5498 '<Block>' Let_Result None
--# scope-start
--# expr-start 5497 '<FieldAccess .xref_no_overloading>' Fld /usr/src/packages/BUILD/ada/ast.py:9873
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5497
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 5498
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_809;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_809;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Subtype_Indication_Equation;
--# end
--# property-start Name.can_designate_primitive /usr/src/packages/BUILD/ada/ast.py:9876
pragma Warnings (Off, "is not referenced");
 function Name_P_Can_Designate_Primitive
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Name;
Match_1 : Bare_Renaming_Clause;
Match_2 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Renaming_Clause;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Ada_Node;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5505 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5504 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9885
--# expr-start 5499 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:9885
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5499
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Attribute_Ref .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind n Match_0
--# expr-start 5500 '<FieldAccess .can_designate_primitive>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9886
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.can_designate_primitive
Fld_1 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Match_0);
--# end
--# expr-done 5500
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Renaming_Clause .. Ada_Synthetic_Renaming_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind r Match_1
--# expr-start 5503 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:9887
--# expr-start 5502 '<IsA PackageRenamingDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:9887
--# expr-start 5501 '<parent(<Var r>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9887
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Match_1.Parent; 
--# expr-done 5501
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Package_Renaming_Decl_Range; 
--# expr-done 5502
Not_Val := not (Is_A); 
--# expr-done 5503
Let_Result_1 := Not_Val; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Range_Spec | Ada_Reverse_Absent .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := True; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5504
Let_Result_3 := Match_Result; 
--# end
--# expr-done 5505
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Can_Designate_Primitive;
--# end
--# property-start Name.parent_name /usr/src/packages/BUILD/ada/ast.py:9892
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Name
  (Node : Bare_Name
      ; Stop_At : Bare_Name
  )
   return Bare_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind stop_at Stop_At
   Property_Result : Bare_Name;
      Fld : Bare_Ada_Node;
Cast_Result : Bare_Name;
Is_Null : Boolean;
Cast_Expr : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Bare_Name;
Let_Result : Bare_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5511 '<Block>' Let_Result None
--# scope-start
--# expr-start 5510 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9896
--# expr-start 5506 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:9896
Is_Null := Stop_At = null; 
--# expr-done 5506
if Is_Null then
   If_Result := True;
else
   --# expr-start 5507 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9896
Cast_Expr := Self; 
      Cast_Result_1 := Cast_Expr;
Is_Equal := Cast_Result_1 = Stop_At; 
--# expr-done 5507
   If_Result := Is_Equal;
end if;
if If_Result then
   If_Result_1 := No_Bare_Ada_Node;
else
   --# expr-start 5509 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9898
--# expr-start 5508 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:9898
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5508
   if Fld = null
      or else Fld.Kind in Ada_Name
   then
      Cast_Result := Fld;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5509
   If_Result_1 := Cast_Result;
end if;
--# expr-done 5510
Let_Result := If_Result_1; 
--# end
--# expr-done 5511
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Name;
--# end
--# property-start Name.parent_callexpr /usr/src/packages/BUILD/ada/ast.py:9901
pragma Warnings (Off, "is not referenced");
 function Name_P_Parent_Callexpr
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Call_Expr
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_812;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Is_A_1 : Boolean;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Id;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Fld_3 : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv_1 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_3 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_4 : Internal_Entity;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Is_A_2 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
Let_Result_3 : Internal_Entity_Call_Expr;
            procedure Finalizer_Scope_812 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_812;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5526 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5525 '<Cast to CallExpr>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:9918
--# expr-start 5524 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9918
--# expr-start 5523 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:9918
--# expr-start 5521 '<take_while>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9918
--# expr-start 5512 '<FieldAccess .parents>' Fld /usr/src/packages/BUILD/ada/ast.py:9918
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 5512
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_188 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_188
--# expr-start 5513 '<IsA CallExpr>' Is_A /usr/src/packages/BUILD/ada/ast.py:9919
Is_A := Item_188.Node /= null 
and then Item_188.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 5513
if Is_A then
   If_Result_1 := True;
else
--# expr-start 5514 '<IsA DottedName, BaseId>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:9920
Is_A_1 := Item_188.Node /= null 
and then Item_188.Node.Kind in Ada_Dotted_Name_Range | Ada_Base_Id; 
--# expr-done 5514
if Is_A_1 then
   --# expr-start 5520 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9920
--# expr-start 5515 '<parent(<Var p>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9920
      if Item_188.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Parent (Node => Item_188.Node, E_Info => Item_188.Info);
--# expr-done 5515
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind pfx Match_0
--# expr-start 5517 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:9921
--# expr-start 5516 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9921
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 5516
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Is_Equiv := Equivalent (Cast_Result, Item_188); 
--# expr-done 5517
Let_Result := Is_Equiv; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind ce Match_1
--# expr-start 5519 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:9922
--# expr-start 5518 '<FieldAccess .name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9922
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 5518
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Is_Equiv_1 := Equivalent (Cast_Result_1, Item_188); 
--# expr-done 5519
Let_Result_1 := Is_Equiv_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5520
   If_Result := Match_Result;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
      exit when not (If_Result_1);
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_188;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5521
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_187 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_187
               --# expr-start 5522 '<IsA CallExpr>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:9925
Is_A_2 := Item_187.Node /= null 
and then Item_187.Node.Kind in Ada_Call_Expr_Range; 
--# expr-done 5522
               if Is_A_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_187;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 5523
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 5524
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 5525
Let_Result_3 := Cast_Result_5; 
--# end
--# expr-done 5526
      Property_Result := Let_Result_3;
   --# end
      Finalizer_Scope_812;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_812;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Parent_Callexpr;
--# end
--# property-start Name.is_range_attribute /usr/src/packages/BUILD/ada/ast.py:9928
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Range_Attribute
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Var_Expr : Bare_Attribute_Ref;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Attribute_Ref;
As_Entity : Internal_Entity_Attribute_Ref;
Is_Null : Boolean;
If_Result : Internal_Entity_Attribute_Ref;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5532 '<Block>' Let_Result None
--# scope-start
--# expr-start 5527 '<Cast to AttributeRef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:9932
Cast_Expr := Self; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Attribute_Ref_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5527
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind attr_ref Var_Expr
--# expr-start 5531 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9934
--# expr-start 5529 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:9934
--# expr-start 5528 '<as_bare_entity(<Var attr_ref>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:9934
Is_Null := Var_Expr = null; 
if Is_Null then
   If_Result := No_Entity_Attribute_Ref;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr); 
   If_Result := As_Entity;
end if;
--# expr-done 5528
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => If_Result.Node.Attribute_Ref_F_Attribute, Info => If_Result.Info);
--# expr-done 5529
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5530 '<Symbol Range>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 5530
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 5531
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 5532
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Range_Attribute;
--# end
--# property-start '[dispatcher]Name.scope' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_Explicit_Deref .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.scope
                  Property_Result := Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.scope
                  Property_Result := Defining_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.scope
                  Property_Result := Dotted_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.scope
                  Property_Result := End_Name_P_Scope
                    (Self
                        , Env
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.scope
                  Property_Result := Base_Id_P_Scope
                    (Self
                        , Env
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Scope;
--# end
--# property-start Name.is_simple_name /usr/src/packages/BUILD/ada/ast.py:9948
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Simple_Name
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Match_0 : Bare_Base_Id;
Match_1 : Bare_Dotted_Name;
Match_2 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Boolean;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Base_Id;
Let_Result : Boolean;
Scope_Result : Boolean;
Cast_Result_1 : Bare_Dotted_Name;
Let_Result_1 : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_2 : Bare_Name;
Let_Result_2 : Boolean;
Scope_Result_2 : Boolean;
Match_Result : Boolean;
Let_Result_3 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5536 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 5535 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:9953
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
Let_Result := True; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind dt Match_1
--# expr-start 5534 '<FieldAccess .is_simple_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9955
--# expr-start 5533 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:9955
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_1.Dotted_Name_F_Prefix;
--# expr-done 5533
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_simple_name
Fld_1 := Libadalang.Implementation.Name_P_Is_Simple_Name (Node => Fld);
--# end
--# expr-done 5534
Let_Result_1 := Fld_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := False; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 5535
Let_Result_3 := Match_Result; 
--# end
--# expr-done 5536
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Simple_Name;
--# end
--# property-start '[dispatcher]Name.ref_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name =>
                  --# property-call-start Name.ref_var
                  Property_Result := Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.ref_var
                  Property_Result := Attribute_Ref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.ref_var
                  Property_Result := Call_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.ref_var
                  Property_Result := Defining_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.ref_var
                  Property_Result := Dotted_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.ref_var
                  Property_Result := End_Name_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.ref_var
                  Property_Result := Explicit_Deref_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.ref_var
                  Property_Result := Qual_Expr_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.ref_var
                  Property_Result := Single_Tok_Node_P_Ref_Var
                    (Self
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.ref_var
                  Property_Result := Target_Name_P_Ref_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Ref_Var;
--# end
--# property-start '[dispatcher]Name.subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr | Ada_Target_Name =>
                  --# property-call-start Name.subp_spec_var
                  Property_Result := Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.subp_spec_var
                  Property_Result := Call_Expr_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.subp_spec_var
                  Property_Result := Dotted_Name_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.subp_spec_var
                  Property_Result := Explicit_Deref_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.subp_spec_var
                  Property_Result := Single_Tok_Node_P_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Subp_Spec_Var;
--# end
--# property-start '[dispatcher]Name.defines_subp_spec_var' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name | Ada_Qual_Expr | Ada_Target_Name =>
                  --# property-call-start Name.defines_subp_spec_var
                  Property_Result := Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.defines_subp_spec_var
                  Property_Result := Call_Expr_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.defines_subp_spec_var
                  Property_Result := Dotted_Name_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.defines_subp_spec_var
                  Property_Result := Explicit_Deref_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.defines_subp_spec_var
                  Property_Result := Single_Tok_Node_P_Defines_Subp_Spec_Var
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.called_subp_spec /usr/src/packages/BUILD/ada/ast.py:9992
pragma Warnings (Off, "is not referenced");
 function Name_P_Called_Subp_Spec
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
      Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_3 : Boolean;
If_Result : Internal_Entity_Base_Formal_Param_Holder;
Let_Result : Internal_Entity_Base_Formal_Param_Holder;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5543 '<Block>' Let_Result None
--# scope-start
--# expr-start 5542 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:9997
--# expr-start 5537 '<FieldAccess .defines_subp_spec_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9998
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.defines_subp_spec_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Defines_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5537
if Fld_3 then
   --# expr-start 5541 '<Cast to BaseFormalParamHolder>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:9999
--# expr-start 5540 '<FieldAccess .value>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9999
--# expr-start 5539 '<FieldAccess .logic_val(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9999
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 5538 '<FieldAccess .subp_spec_var>' Fld /usr/src/packages/BUILD/ada/ast.py:10001
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 5538
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 5539
Fld_2 := Fld_1.Value;
--# expr-done 5540
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Base_Formal_Param_Holder
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Base_Formal_Param_Holder;
   end if;
--# expr-done 5541
   If_Result := Cast_Result_1;
else
   If_Result := No_Entity_Base_Formal_Param_Holder;
end if;
--# expr-done 5542
Let_Result := If_Result; 
--# end
--# expr-done 5543
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Called_Subp_Spec;
--# end
--# property-start Name.referenced_decl /usr/src/packages/BUILD/ada/ast.py:10008
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Basic_Decl;
      Fld : Internal_Refd_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5546 '<Block>' Let_Result None
--# scope-start
--# expr-start 5545 '<FieldAccess .decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10015
--# expr-start 5544 '<FieldAccess .referenced_decl_internal>' Fld /usr/src/packages/BUILD/ada/ast.py:10015
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5544
Fld_1 := Fld.Decl;
--# expr-done 5545
Let_Result := Fld_1; 
--# end
--# expr-done 5546
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Decl;
--# end
--# property-start Name.failsafe_referenced_decl /usr/src/packages/BUILD/ada/ast.py:10019
pragma Warnings (Off, "is not referenced");
 function Name_P_Failsafe_Referenced_Decl
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Fld : Internal_Refd_Decl;
New_Struct : Internal_Refd_Decl;
Try_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5550 '<Block>' Let_Result None
--# scope-start
--# expr-start 5549 '<Try>' Try_Result /usr/src/packages/BUILD/ada/ast.py:10024
begin
   --# expr-start 5547 '<FieldAccess .referenced_decl_internal>' Fld /usr/src/packages/BUILD/ada/ast.py:10025
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl_internal
Fld := Libadalang.Implementation.Name_P_Referenced_Decl_Internal (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5547
   Try_Result := Fld;
exception
   when Property_Error =>
      --# expr-start 5548 '<New InternalRefdDecl>' New_Struct /usr/src/packages/BUILD/ada/ast.py:10026
New_Struct := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 5548
      Try_Result := New_Struct;
end;
--# expr-done 5549
Let_Result := Try_Result; 
--# end
--# expr-done 5550
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Failsafe_Referenced_Decl;
--# end
--# property-start Name.referenced_decl_internal /usr/src/packages/BUILD/ada/ast.py:10031
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Decl_Internal
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Refd_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Refd_Decl;
      Var_Expr : Internal_Refd_Decl;
V : Internal_Logic_Val_Result;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Name;
V_1 : Internal_Logic_Val_Result;
Var_Expr_3 : Internal_Refd_Decl;
Var_Expr_4 : Internal_Entity_Basic_Decl;
Cast_Expr : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Fld : Logic_Var;
Fld_1 : Internal_Logic_Val_Result;
New_Struct : Internal_Logic_Val_Result;
Try_Result : Internal_Logic_Val_Result;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
New_Struct_1 : Internal_Refd_Decl;
Fld_3 : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
New_Struct_2 : Internal_Refd_Decl;
Scope_Result_1 : Internal_Refd_Decl;
New_Struct_3 : Internal_Refd_Decl;
Result_Var_1 : Internal_Refd_Decl;
Fld_4 : Boolean;
If_Result : Internal_Refd_Decl;
Let_Result : Internal_Refd_Decl;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Fld_5 : Logic_Var;
Fld_6 : Internal_Logic_Val_Result;
Fld_7 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
New_Struct_4 : Internal_Refd_Decl;
New_Struct_5 : Internal_Refd_Decl;
Fld_8 : Boolean;
If_Result_1 : Internal_Refd_Decl;
Let_Result_1 : Internal_Refd_Decl;
If_Result_2 : Internal_Refd_Decl;
Fld_9 : Internal_Entity_Basic_Decl;
Fld_10 : Internal_Entity_Basic_Decl;
Scope_Result_2 : Internal_Entity_Basic_Decl;
Result_Var_2 : Internal_Entity_Basic_Decl;
Fld_11 : Ref_Result_Kind;
New_Struct_6 : Internal_Refd_Decl;
Fld_12 : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
Fld_13 : Ref_Result_Kind;
Is_Equal : Boolean;
If_Result_3 : Boolean;
If_Result_4 : Internal_Refd_Decl;
Scope_Result_3 : Internal_Refd_Decl;
Result_Var_3 : Internal_Refd_Decl;
Scope_Result_4 : Internal_Refd_Decl;
Result_Var_4 : Internal_Refd_Decl;
Fld_14 : Boolean;
If_Result_5 : Internal_Refd_Decl;
Let_Result_2 : Internal_Refd_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5585 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 5584 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:10036
--# expr-start 5551 '<FieldAccess .is_defining>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10039
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld_14 := Libadalang.Implementation.Name_P_Is_Defining (Node => Ent.Node);
--# end
--# expr-done 5551
if Fld_14 then
   If_Result_5 := No_Refd_Decl;
else
--# expr-start 5574 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:10042
if Imprecise_Fallback then
   --# expr-start 5564 '<Let v>' Let_Result /usr/src/packages/BUILD/ada/ast.py:10046
--# scope-start
--# expr-start 5555 '<Try>' Try_Result /usr/src/packages/BUILD/ada/ast.py:10046
begin
   --# expr-start 5553 '<FieldAccess .logic_val(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10047
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-start 5552 '<FieldAccess .ref_var>' Fld /usr/src/packages/BUILD/ada/ast.py:10047
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 5552
--# property-call-start AdaNode.logic_val
Fld_1 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result, Lvar => Fld);
--# end
--# expr-done 5553
   Try_Result := Fld_1;
exception
   when Property_Error =>
      --# expr-start 5554 '<New InternalLogicValResult>' New_Struct /usr/src/packages/BUILD/ada/ast.py:10048
New_Struct := (Success => False, Value => No_Entity); 
--# expr-done 5554
      Try_Result := New_Struct;
end;
--# expr-done 5555
V := Try_Result; 
--# bind v V
--# expr-start 5563 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10049
--# expr-start 5556 '<FieldAccess .success>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10050
Fld_4 := V.Success;
--# expr-done 5556
if Fld_4 then
   --# expr-start 5559 '<New InternalRefdDecl>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:10053
--# expr-start 5558 '<Cast to BasicDecl.entity>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:10054
--# expr-start 5557 '<FieldAccess .value>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10054
Fld_2 := V.Value;
--# expr-done 5557
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Basic_Decl;
   end if;
--# expr-done 5558
New_Struct_1 := (Decl => Cast_Result_1, Kind => Precise); 
--# expr-done 5559
   If_Result := New_Struct_1;
else
Var_Expr_2 := Ent;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 5560 '<FieldAccess .first_corresponding_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10060
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.first_corresponding_decl'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_First_Corresponding_Decl (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 5560
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind fcd Var_Expr_1
--# expr-start 5561 '<New InternalRefdDecl>' New_Struct_2 /usr/src/packages/BUILD/ada/ast.py:10062
New_Struct_2 := (Decl => Var_Expr_1, Kind => Imprecise); 
--# expr-done 5561
Scope_Result_1 := New_Struct_2;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 5562 '<New InternalRefdDecl>' New_Struct_3 /usr/src/packages/BUILD/ada/ast.py:10063
New_Struct_3 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 5562
   Result_Var_1 := New_Struct_3;
end if;
   If_Result := Result_Var_1;
end if;
--# expr-done 5563
Let_Result := If_Result; 
--# end
--# expr-done 5564
   If_Result_2 := Let_Result;
else
   --# expr-start 5573 '<Let v>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:10068
--# scope-start
--# expr-start 5566 '<FieldAccess .logic_val(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_1 := Ent; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# expr-start 5565 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10068
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 5565
--# property-call-start AdaNode.logic_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Logic_Val (Node => Self, From_Node => Cast_Result_2, Lvar => Fld_5);
--# end
--# expr-done 5566
V_1 := Fld_6; 
--# bind v V_1
--# expr-start 5572 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10068
--# expr-start 5567 '<FieldAccess .success>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10069
Fld_8 := V_1.Success;
--# expr-done 5567
if Fld_8 then
   --# expr-start 5570 '<New InternalRefdDecl>' New_Struct_4 /usr/src/packages/BUILD/ada/ast.py:10070
--# expr-start 5569 '<Cast to BasicDecl.entity>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:10071
--# expr-start 5568 '<FieldAccess .value>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10071
Fld_7 := V_1.Value;
--# expr-done 5568
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5569
New_Struct_4 := (Decl => Cast_Result_3, Kind => Precise); 
--# expr-done 5570
   If_Result_1 := New_Struct_4;
else
   --# expr-start 5571 '<New InternalRefdDecl>' New_Struct_5 /usr/src/packages/BUILD/ada/ast.py:10074
New_Struct_5 := (Decl => No_Entity_Basic_Decl, Kind => Error); 
--# expr-done 5571
   If_Result_1 := New_Struct_5;
end if;
--# expr-done 5572
Let_Result_1 := If_Result_1; 
--# end
--# expr-done 5573
   If_Result_2 := Let_Result_1;
end if;
--# expr-done 5574
Var_Expr := If_Result_2;
if Var_Expr /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr
--# expr-start 5578 '<New InternalRefdDecl>' New_Struct_6 /usr/src/packages/BUILD/ada/ast.py:10079
--# expr-start 5575 '<FieldAccess .decl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10080
Fld_9 := Var_Expr.Decl;
--# expr-done 5575
Var_Expr_4 := Fld_9;
if Var_Expr_4 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 5576 '<FieldAccess .wrap_public_reference>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10080
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.wrap_public_reference
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Wrap_Public_Reference (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 5576
Scope_Result_2 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity_Basic_Decl;
end if;
--# expr-start 5577 '<FieldAccess .kind>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10081
Fld_11 := Var_Expr.Kind;
--# expr-done 5577
New_Struct_6 := (Decl => Result_Var_2, Kind => Fld_11); 
--# expr-done 5578
Var_Expr_3 := New_Struct_6;
if Var_Expr_3 /= No_Refd_Decl then
   --# scope-start
   --# bind res Var_Expr_3
--# expr-start 5583 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:10082
--# expr-start 5580 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10085
--# expr-start 5579 '<FieldAccess .decl>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10085
Fld_12 := Var_Expr_3.Decl;
--# expr-done 5579
Is_Null := Fld_12.Node = null; 
--# expr-done 5580
if Is_Null then
   --# expr-start 5582 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10085
--# expr-start 5581 '<FieldAccess .kind>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10085
Fld_13 := Var_Expr_3.Kind;
--# expr-done 5581
Is_Equal := Fld_13 = Precise; 
--# expr-done 5582
   If_Result_3 := Is_Equal;
else
   If_Result_3 := False;
end if;
if If_Result_3 then
   If_Result_4 := No_Refd_Decl;
else
   If_Result_4 := Var_Expr_3;
end if;
--# expr-done 5583
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Refd_Decl;
end if;
Scope_Result_4 := Result_Var_3;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Refd_Decl;
end if;
   If_Result_5 := Result_Var_4;
end if;
--# expr-done 5584
Let_Result_2 := If_Result_5; 
--# end
--# expr-done 5585
      Property_Result := Let_Result_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Decl_Internal;
--# end
--# property-start '[dispatcher]Name.designated_type_impl' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.designated_type_impl
                  Property_Result := Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.designated_type_impl
                  Property_Result := Attribute_Ref_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.designated_type_impl
                  Property_Result := Call_Expr_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.designated_type_impl
                  Property_Result := Dotted_Name_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_String_Literal =>
                  --# property-call-start BaseId.designated_type_impl
                  Property_Result := Base_Id_P_Designated_Type_Impl
                    (Self
                        , Env
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Designated_Type_Impl;
--# end
--# property-start Name.name_designated_type /usr/src/packages/BUILD/ada/ast.py:10102
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_824;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_824 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_824;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5590 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:10103
--# expr-start 5586 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:10103
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5586
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 5589 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:10104
--# expr-start 5587 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10104
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 5587
Bound_Origin := Fld_1; 
--# expr-start 5588 '<FieldAccess .designated_type_impl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10104
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 5588
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 5589
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 5590
      Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_824;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_824;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Designated_Type;
--# end
--# property-start Name.is_static_subtype /usr/src/packages/BUILD/ada/ast.py:10116
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Subtype
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5593 '<Block>' Let_Result None
--# scope-start
--# expr-start 5592 '<FieldAccess .is_static_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10120
--# expr-start 5591 '<FieldAccess .name_designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:10120
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5591
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 5592
Let_Result := Fld_1; 
--# end
--# expr-done 5593
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Static_Subtype;
--# end
--# property-start Name.name_designated_type_env /usr/src/packages/BUILD/ada/ast.py:10123
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Designated_Type_Env
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_826;
            procedure Finalizer_Scope_2380;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_826 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_826;
            procedure Finalizer_Scope_2380 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2380;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Name_Designated_Type_Env,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5596 '<Block>' Let_Result None
--# scope-start
--# expr-start 5594 '<FieldAccess .name_designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:10124
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5594
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5595 '<FieldAccess .primitives_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10124
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.primitives_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Primitives_Env (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5595
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2380;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 5596
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_826;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_826;
               Finalizer_Scope_2380;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Designated_Type_Env;
--# end
--# property-start Name.referenced_unit /usr/src/packages/BUILD/ada/ast.py:10147
pragma Warnings (Off, "is not referenced");
 function Name_P_Referenced_Unit
  (Node : Bare_Name
      ; Kind : Analysis_Unit_Kind
  )
   return Internal_Unit
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind kind Kind
   Property_Result : Internal_Unit;
      Fld : Internal_Unit;
Let_Result : Internal_Unit;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5598 '<Block>' Let_Result None
--# scope-start
--# expr-start 5597 '<FieldAccess .internal_referenced_unit(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:10151
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Name_P_Internal_Referenced_Unit (Node => Self, Kind => Kind, Load_If_Needed => True);
--# expr-done 5597
Let_Result := Fld; 
--# end
--# expr-done 5598
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Referenced_Unit;
--# end
--# property-start Name.matches /usr/src/packages/BUILD/ada/ast.py:10154
pragma Warnings (Off, "is not referenced");
 function Name_P_Matches
  (Node : Bare_Name
      ; N : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Match_0 : Bare_Identifier;
Match_1 : Bare_String_Literal;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Defining_Name;
Match_4 : Bare_Name;
Var_Expr : Bare_Identifier;
Var_Expr_1 : Bare_String_Literal;
Var_Expr_2 : Bare_Dotted_Name;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Identifier;
Fld : Symbol_Type;
Fld_1 : Symbol_Type;
Is_Equal : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Cast_Expr_1 : Bare_Name;
Cast_Result_1 : Bare_String_Literal;
Fld_2 : Symbol_Type;
Fld_3 : Symbol_Type;
Is_Equal_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_2 : Bare_Name;
Cast_Result_2 : Bare_Dotted_Name;
Fld_4 : Bare_Name;
Fld_5 : Bare_Name;
Fld_6 : Boolean;
Fld_7 : Bare_Base_Id;
Fld_8 : Bare_Base_Id;
Cast_Result_3 : Bare_Name;
Fld_9 : Boolean;
If_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Fld_10 : Bare_Name;
Fld_11 : Boolean;
Match_Prefix : Bare_Name;
Cast_Result_4 : Bare_Identifier;
Let_Result : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_5 : Bare_String_Literal;
Let_Result_1 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_6 : Bare_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_7 : Bare_Defining_Name;
Let_Result_3 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_8 : Bare_Name;
Let_Result_4 : Boolean;
Scope_Result_7 : Boolean;
Match_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5617 '<Block>' Let_Result_5 None
--# scope-start
--# expr-start 5616 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10161
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_0 := Cast_Result_4; 
--# bind id Match_0
--# expr-start 5599 '<Cast to Identifier>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10163
Cast_Expr := N; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Identifier_Range
   then
      Cast_Result := Cast_Expr;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5599
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_id Var_Expr
--# expr-start 5602 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10164
--# expr-start 5600 '<FieldAccess .sym>' Fld /usr/src/packages/BUILD/ada/ast.py:10164
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_0);
--# end
--# expr-done 5600
--# expr-start 5601 '<FieldAccess .sym>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10164
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr);
--# end
--# expr-done 5601
Is_Equal := Fld = Fld_1; 
--# expr-done 5602
Scope_Result := Is_Equal;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
Scope_Result_3 := Let_Result;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_1 := Cast_Result_5; 
--# bind sl Match_1
--# expr-start 5603 '<Cast to StringLiteral>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:10167
Cast_Expr_1 := N; 
   if Cast_Expr_1 = null
      or else Cast_Expr_1.Kind in Ada_String_Literal_Range
   then
      Cast_Result_1 := Cast_Expr_1;
   else
         Cast_Result_1 := No_Bare_Ada_Node;
   end if;
--# expr-done 5603
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind other_sl Var_Expr_1
--# expr-start 5606 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:10168
--# expr-start 5604 '<FieldAccess .sym>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10168
      if Match_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Match_1);
--# end
--# expr-done 5604
--# expr-start 5605 '<FieldAccess .sym>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10168
      if Var_Expr_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_3 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Var_Expr_1);
--# end
--# expr-done 5605
Is_Equal_1 := Fld_2 = Fld_3; 
--# expr-done 5606
Scope_Result_1 := Is_Equal_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
Let_Result_1 := Result_Var_1; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_2 := Cast_Result_6; 
--# bind dn Match_2
--# expr-start 5607 '<Cast to DottedName>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:10171
Cast_Expr_2 := N; 
   if Cast_Expr_2 = null
      or else Cast_Expr_2.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_2 := Cast_Expr_2;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5607
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sn Var_Expr_2
--# expr-start 5610 '<FieldAccess .matches(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10173
--# expr-start 5608 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10173
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Var_Expr_2.Dotted_Name_F_Prefix;
--# expr-done 5608
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5609 '<FieldAccess .prefix>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10173
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 5609
--# property-call-start Name.matches
Fld_6 := Libadalang.Implementation.Name_P_Matches (Node => Fld_4, N => Fld_5);
--# end
--# expr-done 5610
if Fld_6 then
   --# expr-start 5613 '<FieldAccess .matches(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10174
--# expr-start 5611 '<FieldAccess .suffix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10174
      if Var_Expr_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Var_Expr_2.Dotted_Name_F_Suffix;
--# expr-done 5611
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5612 '<FieldAccess .suffix>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10174
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 5612
      Cast_Result_3 := Fld_8;
--# property-call-start Name.matches
Fld_9 := Libadalang.Implementation.Name_P_Matches (Node => Fld_7, N => Cast_Result_3);
--# end
--# expr-done 5613
   If_Result := Fld_9;
else
   If_Result := False;
end if;
Scope_Result_2 := If_Result;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
Let_Result_2 := Result_Var_2; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_3 := Cast_Result_7; 
--# bind di Match_3
--# expr-start 5615 '<FieldAccess .matches(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10177
      if N = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5614 '<FieldAccess .name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10177
      if Match_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Match_3.Defining_Name_F_Name;
--# expr-done 5614
--# property-call-start Name.matches
Fld_11 := Libadalang.Implementation.Name_P_Matches (Node => N, N => Fld_10);
--# end
--# expr-done 5615
Let_Result_3 := Fld_11; 
--# end
Scope_Result_6 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Op_Xor | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_4 := Cast_Result_8; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_7 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_7; 
end case;
--# expr-done 5616
Let_Result_5 := Match_Result; 
--# end
--# expr-done 5617
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Matches;
--# end
--# property-start Name.name_matches /usr/src/packages/BUILD/ada/ast.py:10182
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Matches
  (Node : Bare_Name
      ; N : Internal_Entity_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Boolean;
      Fld : Bare_Name;
Fld_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5620 '<Block>' Let_Result None
--# scope-start
--# expr-start 5619 '<FieldAccess .matches(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10189
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5618 '<FieldAccess .node>' Fld /usr/src/packages/BUILD/ada/ast.py:10189
Fld := N.Node;
--# expr-done 5618
--# property-call-start Name.matches
Fld_1 := Libadalang.Implementation.Name_P_Matches (Node => Self, N => Fld);
--# end
--# expr-done 5619
Let_Result := Fld_1; 
--# end
--# expr-done 5620
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Matches;
--# end
--# property-start Name.use_package_name_designated_env /usr/src/packages/BUILD/ada/ast.py:10192
pragma Warnings (Off, "is not referenced");
 function Name_P_Use_Package_Name_Designated_Env
  (Node : Bare_Name
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_831;
      Fld : Integer;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Cast_Result : Bare_Use_Package_Clause;
Fld_3 : Lexical_Env_Array_Access;
Get_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_831 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Get_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_831;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5627 '<Block>' Let_Result None
--# scope-start
--# expr-start 5626 '<at(<FieldAccess .designated_envs>, <FieldAccess .child_index>)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:10198
--# expr-start 5624 '<FieldAccess .designated_envs>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10198
--# expr-start 5623 '<Cast to UsePackageClause>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10198
--# expr-start 5622 '<parent(<parent(<Var Self>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10198
--# expr-start 5621 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10198
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5621
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 5622
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Use_Package_Clause_Range
   then
      Cast_Result := Fld_2;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5623
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld_3 := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Cast_Result);
--# end
--# expr-done 5624
--# expr-start 5625 '<FieldAccess .child_index>' Fld /usr/src/packages/BUILD/ada/ast.py:10199
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Child_Index (Node => Self);
--# expr-done 5625
Get_Result := Get (Fld_3, Fld, True); 
--# expr-done 5626
Let_Result := Get_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5627
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_831;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_831;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Use_Package_Name_Designated_Env;
--# end
--# property-start '[dispatcher]Name.relative_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Discrete_Subtype_Name =>
                  --# property-call-start Name.relative_name
                  Property_Result := Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
                  --# property-call-start AttributeRef.relative_name
                  Property_Result := Attribute_Ref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Call_Expr =>
                  --# property-call-start CallExpr.relative_name
                  Property_Result := Call_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Defining_Name =>
                  --# property-call-start DefiningName.relative_name
                  Property_Result := Defining_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.relative_name
                  Property_Result := Dotted_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_End_Name =>
                  --# property-call-start EndName.relative_name
                  Property_Result := End_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Explicit_Deref =>
                  --# property-call-start ExplicitDeref.relative_name
                  Property_Result := Explicit_Deref_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Qual_Expr =>
                  --# property-call-start QualExpr.relative_name
                  Property_Result := Qual_Expr_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Char_Literal .. Ada_Real_Literal =>
                  --# property-call-start SingleTokNode.relative_name
                  Property_Result := Single_Tok_Node_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Target_Name =>
                  --# property-call-start TargetName.relative_name
                  Property_Result := Target_Name_P_Relative_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Relative_Name;
--# end
--# property-start '[dispatcher]Name.name_symbol' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Identifier | Ada_String_Literal .. Ada_Target_Name =>
                  --# property-call-start Name.name_symbol
                  Property_Result := Name_P_Name_Symbol
                    (Self
                    );
                  --# end
               when Ada_Op_Abs .. Ada_Op_Xor =>
                  --# property-call-start Op.name_symbol
                  Property_Result := Op_P_Name_Symbol
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Name_Symbol;
--# end
--# property-start '[dispatcher]Name.base_name' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Name_P_Base_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Name (Self.Kind) is
               when Ada_Attribute_Ref .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Target_Name =>
                  --# property-call-start Name.base_name
                  Property_Result := Name_P_Base_Name
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Dotted_Name =>
                  --# property-call-start DottedName.base_name
                  Property_Result := Dotted_Name_P_Base_Name
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Name_P_Base_Name;
--# end
--# property-start Name.xref_no_overloading /usr/src/packages/BUILD/ada/ast.py:10220
pragma Warnings (Off, "is not referenced");
 function Name_P_Xref_No_Overloading
  (Node : Bare_Name
      ; Sequential : Boolean
         := True
      ; All_Els : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind sequential Sequential
   --# bind all_els All_Els
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_835;
            procedure Finalizer_Scope_2389;
            procedure Finalizer_Scope_2390;
            procedure Finalizer_Scope_2391;
            procedure Finalizer_Scope_2392;
      Match_0 : Internal_Entity_Dotted_Name;
Match_1 : Internal_Entity_Base_Id;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Symbol_Type;
Fld_7 : Boolean;
If_Result : Lookup_Kind;
Fld_8 : Bare_Name;
If_Result_1 : Bare_Name;
Cast_Result : Bare_Ada_Node;
Fld_9 : Internal_Entity_Array_Access;
Fld_10 : Logic_Var;
Domain_Equation : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_2 : Lookup_Kind;
Fld_14 : Bare_Name;
If_Result_3 : Bare_Name;
Cast_Result_1 : Bare_Ada_Node;
Fld_15 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_16 : Boolean;
If_Result_4 : Boolean;
If_Result_5 : Logic_Equation;
Fld_17 : Internal_Entity_Name;
Fld_18 : Logic_Equation;
Fld_19 : Internal_Entity_Name;
Fld_20 : Logic_Var;
Fld_21 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result_6 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Dotted_Name;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_3 : Internal_Entity_Base_Id;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_835 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_835;
            procedure Finalizer_Scope_2389 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2389;
            procedure Finalizer_Scope_2390 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (If_Result_5);
            end Finalizer_Scope_2390;
            procedure Finalizer_Scope_2391 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2391;
            procedure Finalizer_Scope_2392 is
            begin
                     Dec_Ref (Logic_False);
            end Finalizer_Scope_2392;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5667 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5666 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10229
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind dn Match_0
--# expr-start 5635 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10232
--# expr-start 5629 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10232
--# expr-start 5628 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:10232
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 5628
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 5629
--# expr-start 5634 '<bind(<DynamicVariable env>, <FieldAccess .designated_env>, <FieldAccess .xref_no_overloading(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:10233
--# expr-start 5631 '<FieldAccess .designated_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10234
--# expr-start 5630 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10234
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_0.Node.Dotted_Name_F_Prefix, Info => Match_0.Info);
--# expr-done 5630
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5631
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 5633 '<FieldAccess .xref_no_overloading(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10235
--# expr-start 5632 '<FieldAccess .suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10235
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Match_0.Node.Dotted_Name_F_Suffix, Info => Match_0.Info);
--# expr-done 5632
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Sequential => Sequential, All_Els => All_Els, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5633
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 5634
And_Pred := Logic_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10232") else null)); 
--# expr-done 5635
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2389;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_String_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Base_Id
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind i Match_1
--# expr-start 5653 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:10237
if All_Els then
   --# expr-start 5636 '<FieldAccess .is_suffix>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10238
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_16 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 5636
   If_Result_4 := Fld_16;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 5644 '<domain(<FieldAccess .ref_var>, <FieldAccess .env_get(...)>)>' Domain_Equation /usr/src/packages/BUILD/ada/ast.py:10239
--# expr-start 5642 '<FieldAccess .env_get(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10240
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5637 '<FieldAccess .name_symbol>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10242
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 5637
--# expr-start 5639 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10244
--# expr-start 5638 '<FieldAccess .is_prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10244
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_7 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 5638
if Fld_7 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 5639
--# expr-start 5641 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10243
if Sequential then
   --# expr-start 5640 '<FieldAccess .node>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10243
Fld_8 := Ent.Node;
--# expr-done 5640
   If_Result_1 := Fld_8;
else
   If_Result_1 := No_Bare_Ada_Node;
end if;
--# expr-done 5641
      Cast_Result := If_Result_1;
--# property-call-start AdaNode.env_get
Fld_9 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld_6, Lookup => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 5642
--# expr-start 5643 '<FieldAccess .ref_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10239
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 5643
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
declare
   Dom : Internal_Entity_Array_Access := Fld_9;
   A   : Bind_Default_Default.Impl.Unify_Left.R_Type_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Bind_Default_Default.Impl.Member
     (Fld_10, A);
end;
--# expr-done 5644
   If_Result_5 := Domain_Equation;
else
   --# expr-start 5652 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10247
--# expr-start 5645 '<FieldAccess .ref_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10248
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_1.Node);
--# end
--# expr-done 5645
Fld_11.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_11.all);
Eq_Node.Refs.Destroy (Fld_11.all);
--# expr-start 5651 '<FieldAccess .env_get_first(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10249
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5646 '<FieldAccess .name_symbol>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10251
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Match_1.Node);
--# end
--# expr-done 5646
--# expr-start 5648 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:10253
--# expr-start 5647 '<FieldAccess .is_prefix>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10253
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_13 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 5647
if Fld_13 then
   If_Result_2 := Recursive;
else
   If_Result_2 := Flat;
end if;
--# expr-done 5648
--# expr-start 5650 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:10252
if Sequential then
   --# expr-start 5649 '<FieldAccess .node>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10252
Fld_14 := Ent.Node;
--# expr-done 5649
   If_Result_3 := Fld_14;
else
   If_Result_3 := No_Bare_Ada_Node;
end if;
--# expr-done 5650
      Cast_Result_1 := If_Result_3;
--# property-call-start AdaNode.env_get_first
Fld_15 := Libadalang.Implementation.Ada_Node_P_Env_Get_First (Node => Self, Env => Env, Symbol => Fld_12, Lookup => If_Result_2, From_Node => Cast_Result_1, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 5651
Bind_Result := Bind_Default_Default.Create (Fld_11, Fld_15, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10247") else null)); 
--# expr-done 5652
   If_Result_5 := Bind_Result;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 5653
Let_Result_1 := If_Result_5; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2390;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind ar Match_2
--# expr-start 5665 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10263
--# expr-start 5655 '<FieldAccess .xref_no_overloading(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10263
--# expr-start 5654 '<FieldAccess .prefix>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10263
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 5654
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_17.Node, Sequential => Sequential, All_Els => All_Els, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 5655
--# expr-start 5664 '<If>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:10264
--# expr-start 5659 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10264
--# expr-start 5657 '<FieldAccess .sym>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:10264
--# expr-start 5656 '<FieldAccess .attribute>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10264
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Match_2.Node.Attribute_Ref_F_Attribute, Info => Match_2.Info);
--# expr-done 5656
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_23 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_22.Node);
--# end
--# expr-done 5657
--# expr-start 5658 '<Symbol Class>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 5658
Is_Equal := Fld_23 = Sym; 
--# expr-done 5659
if Is_Equal then
   --# expr-start 5663 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:10265
--# expr-start 5661 '<FieldAccess .ref_var>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10265
--# expr-start 5660 '<FieldAccess .prefix>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10265
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Create_Internal_Entity_Name (Node => Match_2.Node.Attribute_Ref_F_Prefix, Info => Match_2.Info);
--# expr-done 5660
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_19.Node);
--# end
--# expr-done 5661
Fld_20.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_20.all);
Eq_Node.Refs.Destroy (Fld_20.all);
--# expr-start 5662 '<FieldAccess .ref_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10265
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Match_2.Node);
--# end
--# expr-done 5662
Fld_21.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_21.all);
Eq_Node.Refs.Destroy (Fld_21.all);
Bind_Result_1 := Bind_527_Default.Create (Fld_20, Fld_21, Logic_Converter_527'(null record), No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10265") else null)); 
--# expr-done 5663
   If_Result_6 := Bind_Result_1;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10267") else null)); 
   If_Result_6 := Logic_True;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 5664
And_Pred_1 := Logic_And (Fld_18, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10263") else null)); 
--# expr-done 5665
Let_Result_2 := And_Pred_1; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2391;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Null_Literal .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10269") else null)); 
Let_Result_3 := Logic_False; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2392;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5666
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5667
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_835;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_835;
               Finalizer_Scope_2389;
               Finalizer_Scope_2390;
               Finalizer_Scope_2391;
               Finalizer_Scope_2392;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Xref_No_Overloading;
--# end
--# property-start Name.is_prefix /usr/src/packages/BUILD/ada/ast.py:10273
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Prefix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Name;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Prefix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5680 '<Block>' Let_Result None
--# scope-start
--# expr-start 5669 '<IsA DottedName>' Is_A /usr/src/packages/BUILD/ada/ast.py:10280
--# expr-start 5668 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:10280
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5668
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5669
if Is_A then
   --# expr-start 5673 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10281
--# expr-start 5672 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10281
--# expr-start 5671 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10281
--# expr-start 5670 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10281
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5670
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5671
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Prefix;
--# expr-done 5672
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 5673
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5676 '<FieldAccess .is_prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10282
--# expr-start 5675 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:10282
--# expr-start 5674 '<parent(<Var Self>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5674
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5675
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_4 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Cast_Result_2);
--# end
--# expr-done 5676
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5679 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10283
--# expr-start 5678 '<IsA DottedName>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:10283
--# expr-start 5677 '<parent(<Var Self>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10283
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 5677
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5678
Not_Val := not (Is_A_1); 
--# expr-done 5679
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 5680
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Prefix;
--# end
--# property-start Name.is_suffix /usr/src/packages/BUILD/ada/ast.py:10287
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Suffix
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Is_A : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Dotted_Name;
Fld_2 : Bare_Base_Id;
Cast_Result_1 : Bare_Name;
Is_Equal : Boolean;
If_Result : Boolean;
Fld_3 : Bare_Ada_Node;
Cast_Result_2 : Bare_Name;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Fld_5 : Bare_Ada_Node;
Is_A_1 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Name_Name_P_Is_Suffix,
            Items    => new Mmz_Key_Array (1 ..  1))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Name,
                                As_Bare_Name => Self);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 5693 '<Block>' Let_Result None
--# scope-start
--# expr-start 5682 '<IsA DottedName>' Is_A /usr/src/packages/BUILD/ada/ast.py:10292
--# expr-start 5681 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:10292
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 5681
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5682
if Is_A then
   --# expr-start 5686 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10293
--# expr-start 5685 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10293
--# expr-start 5684 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10293
--# expr-start 5683 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10293
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 5683
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 5684
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Cast_Result.Dotted_Name_F_Suffix;
--# expr-done 5685
      Cast_Result_1 := Fld_2;
Is_Equal := Cast_Result_1 = Self; 
--# expr-done 5686
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 5689 '<FieldAccess .is_suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10294
--# expr-start 5688 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:10294
--# expr-start 5687 '<parent(<Var Self>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10294
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Parent; 
--# expr-done 5687
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Name
   then
      Cast_Result_2 := Fld_3;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 5688
      if Cast_Result_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_4 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Cast_Result_2);
--# end
--# expr-done 5689
   If_Result_1 := Fld_4;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5692 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10296
--# expr-start 5691 '<IsA DottedName>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:10296
--# expr-start 5690 '<parent(<Var Self>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10296
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 5690
Is_A_1 := Fld_5 /= null 
and then Fld_5.Kind in Ada_Dotted_Name_Range; 
--# expr-done 5691
Not_Val := not (Is_A_1); 
--# expr-done 5692
   If_Result_2 := Not_Val;
end if;
Let_Result := If_Result_2; 
--# end
--# expr-done 5693
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Suffix;
--# end
--# property-start Name.is_operator_name /usr/src/packages/BUILD/ada/ast.py:10300
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Operator_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_1 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_2 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_4 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Fld_5 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_6 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_7 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_8 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_9 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Fld_10 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Fld_11 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Fld_12 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Fld_13 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Fld_14 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Fld_15 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Fld_16 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Fld_17 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Fld_18 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Fld_19 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Fld_20 : Symbol_Type;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5757 '<Block>' Let_Result None
--# scope-start
--# expr-start 5696 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5694 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5694
--# expr-start 5695 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 5695
Is_Equal := Fld = Sym; 
--# expr-done 5696
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5699 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5697 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5697
--# expr-start 5698 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 5698
Is_Equal_1 := Fld_1 = Sym_1; 
--# expr-done 5699
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 5702 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5700 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5700
--# expr-start 5701 '<Symbol "/=">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 5701
Is_Equal_2 := Fld_2 = Sym_2; 
--# expr-done 5702
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 5705 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5703 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5703
--# expr-start 5704 '<Symbol "<">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 5704
Is_Equal_3 := Fld_3 = Sym_3; 
--# expr-done 5705
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 5708 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5706 '<FieldAccess .name_symbol>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5706
--# expr-start 5707 '<Symbol "<=">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 5707
Is_Equal_4 := Fld_4 = Sym_4; 
--# expr-done 5708
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 5711 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5709 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5709
--# expr-start 5710 '<Symbol ">">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 5710
Is_Equal_5 := Fld_5 = Sym_5; 
--# expr-done 5711
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 5714 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5712 '<FieldAccess .name_symbol>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5712
--# expr-start 5713 '<Symbol ">=">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 5713
Is_Equal_6 := Fld_6 = Sym_6; 
--# expr-done 5714
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 5717 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5715 '<FieldAccess .name_symbol>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5715
--# expr-start 5716 '<Symbol "and">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 5716
Is_Equal_7 := Fld_7 = Sym_7; 
--# expr-done 5717
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 5720 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5718 '<FieldAccess .name_symbol>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5718
--# expr-start 5719 '<Symbol "or">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 5719
Is_Equal_8 := Fld_8 = Sym_8; 
--# expr-done 5720
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 5723 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5721 '<FieldAccess .name_symbol>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5721
--# expr-start 5722 '<Symbol "xor">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 5722
Is_Equal_9 := Fld_9 = Sym_9; 
--# expr-done 5723
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 5726 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5724 '<FieldAccess .name_symbol>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_10 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5724
--# expr-start 5725 '<Symbol "abs">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 5725
Is_Equal_10 := Fld_10 = Sym_10; 
--# expr-done 5726
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 5729 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5727 '<FieldAccess .name_symbol>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_11 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5727
--# expr-start 5728 '<Symbol "*">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 5728
Is_Equal_11 := Fld_11 = Sym_11; 
--# expr-done 5729
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 5732 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5730 '<FieldAccess .name_symbol>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5730
--# expr-start 5731 '<Symbol "/">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 5731
Is_Equal_12 := Fld_12 = Sym_12; 
--# expr-done 5732
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 5735 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5733 '<FieldAccess .name_symbol>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5733
--# expr-start 5734 '<Symbol "mod">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 5734
Is_Equal_13 := Fld_13 = Sym_13; 
--# expr-done 5735
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 5738 '<Eq>' Is_Equal_14 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5736 '<FieldAccess .name_symbol>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5736
--# expr-start 5737 '<Symbol "rem">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 5737
Is_Equal_14 := Fld_14 = Sym_14; 
--# expr-done 5738
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 5741 '<Eq>' Is_Equal_15 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5739 '<FieldAccess .name_symbol>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5739
--# expr-start 5740 '<Symbol "+">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 5740
Is_Equal_15 := Fld_15 = Sym_15; 
--# expr-done 5741
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 5744 '<Eq>' Is_Equal_16 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5742 '<FieldAccess .name_symbol>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_16 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5742
--# expr-start 5743 '<Symbol "-">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 5743
Is_Equal_16 := Fld_16 = Sym_16; 
--# expr-done 5744
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 5747 '<Eq>' Is_Equal_17 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5745 '<FieldAccess .name_symbol>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5745
--# expr-start 5746 '<Symbol "&""+">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_1); 
--# expr-done 5746
Is_Equal_17 := Fld_17 = Sym_17; 
--# expr-done 5747
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 5750 '<Eq>' Is_Equal_18 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5748 '<FieldAccess .name_symbol>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5748
--# expr-start 5749 '<Symbol "-">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 5749
Is_Equal_18 := Fld_18 = Sym_18; 
--# expr-done 5750
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 5753 '<Eq>' Is_Equal_19 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5751 '<FieldAccess .name_symbol>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5751
--# expr-start 5752 '<Symbol "not">' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 5752
Is_Equal_19 := Fld_19 = Sym_19; 
--# expr-done 5753
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 5756 '<Eq>' Is_Equal_20 /usr/src/packages/BUILD/ada/ast.py:10304
--# expr-start 5754 '<FieldAccess .name_symbol>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10304
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_20 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Ent.Node);
--# end
--# expr-done 5754
--# expr-start 5755 '<Symbol "abs">' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 5755
Is_Equal_20 := Fld_20 = Sym_20; 
--# expr-done 5756
   If_Result_19 := Is_Equal_20;
end if;
Let_Result := If_Result_19; 
--# end
--# expr-done 5757
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Operator_Name;
--# end
--# property-start Name.is_write_reference /usr/src/packages/BUILD/ada/ast.py:10312
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Write_Reference
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
            procedure Finalizer_Scope_2396;
            procedure Finalizer_Scope_2399;
      Match_0 : Internal_Entity_Assign_Stmt;
Match_1 : Internal_Entity_Call_Expr;
Match_2 : Internal_Entity_Dotted_Name;
Match_3 : Internal_Entity_Param_Assoc;
Match_4 : Internal_Entity_Attribute_Ref;
Match_5 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Mode;
Var_Expr_2 : Internal_Entity_Param_Spec;
Var_Expr_3 : Internal_Entity_Mode;
Var_Expr_4 : Internal_Entity_Param_Spec;
Fld : Internal_Entity;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity;
Cast_Expr : Internal_Entity_Name;
Cast_Result_1 : Internal_Entity;
Is_Equiv : Boolean;
Fld_2 : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity;
Cast_Expr_1 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity;
Is_Equiv_1 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
Fld_5 : Internal_Entity_Name;
Cast_Result_4 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Name;
Cast_Result_5 : Internal_Entity;
Is_Equiv_2 : Boolean;
Fld_6 : Internal_Entity_Base_Id;
Fld_7 : Internal_Entity_Base_Formal_Param_Holder;
Fld_8 : Internal_Entity_Info;
Fld_9 : Internal_Metadata;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Formal_Param_Decl;
Cast_Result_6 : Internal_Entity_Param_Spec;
Fld_12 : Internal_Entity_Mode;
Scope_Result : Internal_Entity_Mode;
Result_Var : Internal_Entity_Mode;
Fld_13 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_14 : Internal_Entity_Defining_Name_Array_Access;
Fld_15 : Internal_Entity_Basic_Decl;
Cast_Result_7 : Internal_Entity_Param_Spec;
Fld_16 : Internal_Entity_Mode;
Scope_Result_3 : Internal_Entity_Mode;
Result_Var_3 : Internal_Entity_Mode;
Fld_17 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
Quantifier_Result : Boolean;
Fld_18 : Internal_Entity_Name;
Cast_Result_8 : Internal_Entity;
Cast_Expr_3 : Internal_Entity_Name;
Cast_Result_9 : Internal_Entity;
Is_Equiv_3 : Boolean;
Fld_19 : Boolean;
If_Result_4 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_10 : Internal_Entity_Assign_Stmt;
Let_Result : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_11 : Internal_Entity_Call_Expr;
Let_Result_1 : Boolean;
Scope_Result_6 : Boolean;
Cast_Result_12 : Internal_Entity_Dotted_Name;
Let_Result_2 : Boolean;
Scope_Result_7 : Boolean;
Cast_Result_13 : Internal_Entity_Param_Assoc;
Let_Result_3 : Boolean;
Scope_Result_8 : Boolean;
Cast_Result_14 : Internal_Entity_Attribute_Ref;
Let_Result_4 : Boolean;
Scope_Result_9 : Boolean;
Cast_Result_15 : Internal_Entity;
Let_Result_5 : Boolean;
Scope_Result_10 : Boolean;
Match_Result : Boolean;
Let_Result_6 : Boolean;
            procedure Finalizer_Scope_2396 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2396;
            procedure Finalizer_Scope_2399 is
            begin
                     Dec_Ref (Fld_14);
            end Finalizer_Scope_2399;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5786 '<Block>' Let_Result_6 None
--# scope-start
--# expr-start 5785 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10328
--# expr-start 5758 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:10328
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 5758
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assign_Stmt =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Assign_Stmt
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_10; 
--# bind a Match_0
--# expr-start 5760 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:10331
--# expr-start 5759 '<FieldAccess .dest>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10331
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Assign_Stmt_F_Dest, Info => Match_0.Info);
--# expr-done 5759
      Cast_Result := Create_Internal_Entity
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
Cast_Expr := Ent; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Is_Equiv := Equivalent (Cast_Result, Cast_Result_1); 
--# expr-done 5760
Let_Result := Is_Equiv; 
--# end
Scope_Result_5 := Let_Result;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_11; 
--# bind c Match_1
--# expr-start 5762 '<Eq>' Is_Equiv_1 /usr/src/packages/BUILD/ada/ast.py:10336
--# expr-start 5761 '<FieldAccess .name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10336
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Match_1.Node.Call_Expr_F_Name, Info => Match_1.Info);
--# expr-done 5761
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Cast_Expr_1 := Ent; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Is_Equiv_1 := Equivalent (Cast_Result_2, Cast_Result_3); 
--# expr-done 5762
if Is_Equiv_1 then
   --# expr-start 5763 '<FieldAccess .is_write_reference>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10337
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_write_reference
Fld_3 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_1.Info);
--# end
--# expr-done 5763
   If_Result := Fld_3;
else
   If_Result := False;
end if;
Let_Result_1 := If_Result; 
--# end
Scope_Result_6 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_12; 
--# bind d Match_2
--# expr-start 5764 '<FieldAccess .is_write_reference>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10347
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_write_reference
Fld_4 := Libadalang.Implementation.Name_P_Is_Write_Reference (Node => Match_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_2.Info);
--# end
--# expr-done 5764
if Fld_4 then
   If_Result_3 := True;
else
--# expr-start 5766 '<Eq>' Is_Equiv_2 /usr/src/packages/BUILD/ada/ast.py:10350
--# expr-start 5765 '<FieldAccess .prefix>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10350
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Match_2.Node.Dotted_Name_F_Prefix, Info => Match_2.Info);
--# expr-done 5765
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv_2 := Equivalent (Cast_Result_4, Cast_Result_5); 
--# expr-done 5766
if Is_Equiv_2 then
--# expr-start 5768 '<FieldAccess .called_subp_spec>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10350
--# expr-start 5767 '<FieldAccess .suffix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10350
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Base_Id (Node => Match_2.Node.Dotted_Name_F_Suffix, Info => Match_2.Info);
--# expr-done 5767
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.called_subp_spec
Fld_7 := Libadalang.Implementation.Name_P_Called_Subp_Spec (Node => Fld_6.Node, E_Info => Fld_6.Info);
--# end
--# expr-done 5768
Var_Expr := Fld_7;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind spec Var_Expr
--# expr-start 5771 '<FieldAccess .dottable_subp>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10352
--# expr-start 5770 '<FieldAccess .md>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10352
--# expr-start 5769 '<FieldAccess .info>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10352
Fld_8 := Var_Expr.Info;
--# expr-done 5769
Fld_9 := Fld_8.MD;
--# expr-done 5770
Fld_10 := Fld_9.Dottable_Subp;
--# expr-done 5771
if Fld_10 then
--# expr-start 5774 '<Cast to ParamSpec>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:10353
--# expr-start 5773 '<at(<FieldAccess .abstract_formal_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:10353
--# expr-start 5772 '<FieldAccess .abstract_formal_params>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10353
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseFormalParamHolder.abstract_formal_params'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Formal_Param_Holder_P_Abstract_Formal_Params (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5772
Get_Result := Get (Fld_11, 0, True); 
--# expr-done 5773
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_6 := Create_Internal_Entity_Param_Spec
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_6 := No_Entity_Param_Spec;
   end if;
--# expr-done 5774
Var_Expr_2 := Cast_Result_6;
if Var_Expr_2 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 5775 '<FieldAccess .mode>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10353
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Mode (Node => Var_Expr_2.Node.Param_Spec_F_Mode, Info => Var_Expr_2.Info);
--# expr-done 5775
Scope_Result := Fld_12;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Mode;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Mode then
   --# scope-start
--# expr-start 5776 '<FieldAccess .is_writable>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10353
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Mode.is_writable
Fld_13 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_1.Node);
--# end
--# expr-done 5776
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_1 := Result_Var_1;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_2396;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
Let_Result_2 := If_Result_3; 
--# end
Scope_Result_7 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Param_Assoc =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity_Param_Assoc
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_13; 
--# bind p Match_3
--# expr-start 5777 '<FieldAccess .get_params>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10359
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicAssoc.get_params
Fld_14 := Libadalang.Implementation.Basic_Assoc_P_Get_Params (Node => Match_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Match_3.Info);
--# end
--# expr-done 5777
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Defining_Name_Array_Access := Fld_14;
   begin
      for Item_189 of
            Collection.Items
      loop
   --# scope-start
            --# bind m Item_189
--# expr-start 5779 '<Cast to ParamSpec>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:10360
--# expr-start 5778 '<FieldAccess .basic_decl>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10360
      if Item_189.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_15 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Item_189.Node, E_Info => Item_189.Info);
--# end
--# expr-done 5778
   if Fld_15.Node = null
      or else Fld_15.Node.Kind in Ada_Param_Spec_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Param_Spec
        (Node => Fld_15.Node,
         Info => Fld_15.Info);
   else
         Cast_Result_7 := No_Entity_Param_Spec;
   end if;
--# expr-done 5779
Var_Expr_4 := Cast_Result_7;
if Var_Expr_4 /= No_Entity_Param_Spec then
   --# scope-start
--# expr-start 5780 '<FieldAccess .mode>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10360
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Mode (Node => Var_Expr_4.Node.Param_Spec_F_Mode, Info => Var_Expr_4.Info);
--# expr-done 5780
Scope_Result_3 := Fld_16;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Mode;
end if;
Var_Expr_3 := Result_Var_3;
if Var_Expr_3 /= No_Entity_Mode then
   --# scope-start
--# expr-start 5781 '<FieldAccess .is_writable>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10360
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Mode.is_writable
Fld_17 := Libadalang.Implementation.Mode_P_Is_Writable (Node => Var_Expr_3.Node);
--# end
--# expr-done 5781
Scope_Result_4 := Fld_17;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
            if Result_Var_4 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
Let_Result_3 := Quantifier_Result; 
--# end
Scope_Result_8 := Let_Result_3;
   --# end
      Finalizer_Scope_2399;
            Match_Result := Scope_Result_8; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_14; 
--# bind a Match_4
--# expr-start 5783 '<Eq>' Is_Equiv_3 /usr/src/packages/BUILD/ada/ast.py:10364
--# expr-start 5782 '<FieldAccess .prefix>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10364
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Name (Node => Match_4.Node.Attribute_Ref_F_Prefix, Info => Match_4.Info);
--# expr-done 5782
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
Cast_Expr_3 := Ent; 
      Cast_Result_9 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
Is_Equiv_3 := Equivalent (Cast_Result_8, Cast_Result_9); 
--# expr-done 5783
if Is_Equiv_3 then
   --# expr-start 5784 '<FieldAccess .is_access_attr>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10364
      if Match_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Match_4.Node, E_Info => Match_4.Info);
--# end
--# expr-done 5784
   If_Result_4 := Fld_19;
else
   If_Result_4 := False;
end if;
Let_Result_4 := If_Result_4; 
--# end
Scope_Result_9 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Abort_Absent .. Ada_Iterated_Assoc | Ada_Abstract_State_Decl .. Ada_Membership_Expr | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Abort_Stmt | Ada_Call_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_5 := Cast_Result_15; 
--# bind _ Match_5
Let_Result_5 := False; 
--# end
Scope_Result_10 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_10; 
end case;
--# expr-done 5785
Let_Result_6 := Match_Result; 
--# end
--# expr-done 5786
      Property_Result := Let_Result_6;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2396;
               Finalizer_Scope_2399;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Write_Reference;
--# end
--# property-start Name.potential_actuals_for_dispatch /usr/src/packages/BUILD/ada/ast.py:10370
pragma Warnings (Off, "is not referenced");
 function Name_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Name
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_840;
            procedure Finalizer_Scope_2405;
            procedure Finalizer_Scope_2410;
            procedure Finalizer_Scope_2412;
      Block_Var : Internal_Expected_Type_For_Expr_Array_Access;
Block_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Var_Expr : Boolean;
Var_Expr_1 : Internal_Entity_Dotted_Name;
Match_0 : Internal_Entity_Call_Expr;
Match_1 : Internal_Entity_Dotted_Name;
Match_2 : Internal_Entity_Identifier;
Match_3 : Internal_Entity_Name;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Call_Expr;
Fld : Boolean;
Fld_1 : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Dotted_Name;
Match_Prefix : Internal_Entity_Name;
Cast_Result_2 : Internal_Entity_Call_Expr;
Let_Result : Internal_Entity_Dotted_Name;
Scope_Result : Internal_Entity_Dotted_Name;
Cast_Result_3 : Internal_Entity_Dotted_Name;
Let_Result_1 : Internal_Entity_Dotted_Name;
Scope_Result_1 : Internal_Entity_Dotted_Name;
Cast_Result_4 : Internal_Entity_Identifier;
Let_Result_2 : Internal_Entity_Dotted_Name;
Scope_Result_2 : Internal_Entity_Dotted_Name;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Internal_Entity_Dotted_Name;
Scope_Result_3 : Internal_Entity_Dotted_Name;
Match_Result : Internal_Entity_Dotted_Name;
Fld_3 : Internal_Single_Formal_Array_Access;
Get_Result : Internal_Single_Formal;
Fld_4 : Internal_Entity_Base_Formal_Param_Decl;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Name;
Cast_Result_6 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var : Internal_Expected_Type_For_Expr_Array_Access;
Scope_Result_5 : Internal_Expected_Type_For_Expr_Array_Access;
Result_Var_1 : Internal_Expected_Type_For_Expr_Array_Access;
Cast_Expr : Internal_Entity_Name;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_7 : Internal_Entity_Assoc_List;
Scope_Result_6 : Internal_Entity_Assoc_List;
Result_Var_2 : Internal_Entity_Assoc_List;
Fld_8 : Internal_Param_Actual_Array_Access;
Scope_Result_7 : Internal_Param_Actual_Array_Access;
Result_Var_3 : Internal_Param_Actual_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Internal_Entity_Basic_Decl;
Fld_11 : Internal_Entity_Type_Expr;
Fld_12 : Internal_Entity_Expr;
New_Struct_1 : Internal_Expected_Type_For_Expr;
Map_Result : Internal_Expected_Type_For_Expr_Array_Access;
Concat_Result : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result_4 : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_840 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_840;
            procedure Finalizer_Scope_2405 is
            begin
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2405;
            procedure Finalizer_Scope_2410 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2410;
            procedure Finalizer_Scope_2412 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2412;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5809 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5787 '<FieldAccess .is_dot_call>' Fld /usr/src/packages/BUILD/ada/ast.py:10380
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_dot_call
Fld := Libadalang.Implementation.Name_P_Is_Dot_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5787
Var_Expr := Fld;
if Var_Expr /= False then
   --# scope-start
   --# bind _ Var_Expr
--# expr-start 5792 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10380
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Name (Match_Prefix.Node.Kind) is
         when Ada_Call_Expr =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Call_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_2; 
--# bind c Match_0
--# expr-start 5789 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10381
--# expr-start 5788 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10381
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Match_0.Node.Call_Expr_F_Name, Info => Match_0.Info);
--# expr-done 5788
   if Fld_1.Node = null
      or else Fld_1.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5789
Let_Result := Cast_Result; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Dotted_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_3; 
--# bind d Match_1
Let_Result_1 := Match_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Identifier =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Identifier
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind i Match_2
--# expr-start 5791 '<Cast to DottedName>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:10383
--# expr-start 5790 '<parent(<Var i>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10383
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Match_2.Node, E_Info => Match_2.Info);
--# expr-done 5790
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Dotted_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5791
Let_Result_2 := Cast_Result_1; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref | Ada_Defining_Name .. Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Char_Literal | Ada_Op_Abs .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind _ Match_3
Let_Result_3 := No_Entity_Dotted_Name; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
end case;
--# expr-done 5792
Var_Expr_1 := Match_Result;
if Var_Expr_1 /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind d Var_Expr_1
--# expr-start 5798 '<New InternalExpectedTypeForExpr>' New_Struct /usr/src/packages/BUILD/ada/ast.py:10385
--# expr-start 5796 '<FieldAccess .type_expression>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10386
--# expr-start 5795 '<FieldAccess .spec>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10386
--# expr-start 5794 '<at(<FieldAccess .unpacked_formal_params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:10386
--# expr-start 5793 '<FieldAccess .unpacked_formal_params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10386
      if Spec.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_3 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 5793
Get_Result := Get (Fld_3, 0, True); 
--# expr-done 5794
Fld_4 := Get_Result.Spec;
--# expr-done 5795
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 5796
--# expr-start 5797 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10388
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Var_Expr_1.Node.Dotted_Name_F_Prefix, Info => Var_Expr_1.Info);
--# expr-done 5797
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
New_Struct := (Expected_Type => Fld_5, Expr => Cast_Result_6); 
--# expr-done 5798
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Scope_Result_4 := Singleton;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2410;
   Result_Var := Scope_Result_4;
else
   Result_Var := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var);
Scope_Result_5 := Result_Var;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2405;
   Result_Var_1 := Scope_Result_5;
else
   Result_Var_1 := No_Internal_Expected_Type_For_Expr_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Block_Var := Result_Var_1; Inc_Ref (Block_Var);
--# bind prefix Block_Var
--# expr-start 5807 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10395
--# expr-start 5799 '<Cast to CallExpr>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:10395
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 5799
Var_Expr_3 := Cast_Result_7;
if Var_Expr_3 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 5800 '<FieldAccess .params>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10395
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 5800
Scope_Result_6 := Fld_7;
   --# end
   Result_Var_2 := Scope_Result_6;
else
   Result_Var_2 := No_Entity_Assoc_List;
end if;
Var_Expr_2 := Result_Var_2;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 5801 '<FieldAccess .zip_with_params>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10395
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.zip_with_params
Fld_8 := Libadalang.Implementation.Assoc_List_P_Zip_With_Params (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 5801
Scope_Result_7 := Fld_8;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2412;
   Result_Var_3 := Scope_Result_7;
else
   Result_Var_3 := No_Internal_Param_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
declare
   Map_Result_Vec : Internal_Expected_Type_For_Expr_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Actual_Array_Access := Result_Var_3;
      begin
         for Item_190 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_190
      --# expr-start 5806 '<New InternalExpectedTypeForExpr>' New_Struct_1 /usr/src/packages/BUILD/ada/ast.py:10396
--# expr-start 5804 '<FieldAccess .type_expression>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10397
--# expr-start 5803 '<FieldAccess .basic_decl>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10397
--# expr-start 5802 '<FieldAccess .param>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10397
Fld_9 := Item_190.Param;
--# expr-done 5802
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_10 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 5803
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 5804
--# expr-start 5805 '<FieldAccess .actual>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10398
Fld_12 := Item_190.Actual;
--# expr-done 5805
New_Struct_1 := (Expected_Type => Fld_11, Expr => Fld_12); 
--# expr-done 5806
         declare
            Item_To_Append : constant Internal_Expected_Type_For_Expr :=
               New_Struct_1;
         begin
            Internal_Expected_Type_For_Expr_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Expected_Type_For_Expr_Array
        (Items_Count => Natural (Internal_Expected_Type_For_Expr_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Expected_Type_For_Expr_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Expected_Type_For_Expr_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Expected_Type_For_Expr_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5807
Block_Var_1 := Map_Result; Inc_Ref (Block_Var_1);
--# bind args Block_Var_1
--# expr-start 5808 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:10403
Concat_Result := Concat (Block_Var, Block_Var_1); 
--# expr-done 5808
Let_Result_4 := Concat_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5809
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_840;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_840;
               Finalizer_Scope_2405;
               Finalizer_Scope_2410;
               Finalizer_Scope_2412;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start Name.is_dispatching_call /usr/src/packages/BUILD/ada/ast.py:10406
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Dispatching_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Name;
Fld : Boolean;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Call_Expr;
Cast_Result : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Basic_Decl;
Fld_4 : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5816 '<Block>' Let_Result None
--# scope-start
--# expr-start 5810 '<FieldAccess .is_access_call>' Fld /usr/src/packages/BUILD/ada/ast.py:10408
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_access_call
Fld := Libadalang.Implementation.Name_P_Is_Access_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5810
if Fld then
   If_Result_1 := True;
else
--# expr-start 5811 '<FieldAccess .is_direct_call>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10409
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_direct_call
Fld_1 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5811
if Fld_1 then
   --# expr-start 5815 '<FieldAccess .is_dispatching_call_impl(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10410
--# expr-start 5813 '<Cast to Name.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10410
--# expr-start 5812 '<FieldAccess .parent_callexpr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10410
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_callexpr
Fld_2 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5812
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
--# expr-done 5813
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := Ent;
end if;
      if Result_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5814 '<FieldAccess .referenced_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10411
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_3 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5814
--# property-call-start Expr.is_dispatching_call_impl
Fld_4 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Result_Var.Node, Decl => Fld_3, Imprecise_Fallback => Imprecise_Fallback, E_Info => Result_Var.Info);
--# end
--# expr-done 5815
   If_Result := Fld_4;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 5816
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Dispatching_Call;
--# end
--# property-start Name.is_static_call /usr/src/packages/BUILD/ada/ast.py:10416
pragma Warnings (Off, "is not referenced");
 function Name_P_Is_Static_Call
  (Node : Bare_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Name :=
     Internal_Entity_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Not_Val : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5820 '<Block>' Let_Result None
--# scope-start
--# expr-start 5817 '<FieldAccess .is_call>' Fld /usr/src/packages/BUILD/ada/ast.py:10425
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_call
Fld := Libadalang.Implementation.Name_P_Is_Call (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5817
if Fld then
   --# expr-start 5819 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10425
--# expr-start 5818 '<FieldAccess .is_dispatching_call>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10425
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 5818
Not_Val := not (Fld_1); 
--# expr-done 5819
   If_Result := Not_Val;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 5820
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Is_Static_Call;
--# end
--# property-start Name.as_single_tok_node_array /usr/src/packages/BUILD/ada/ast.py:10428
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Single_Tok_Node_Array
  (Node : Bare_Name
  )
   return Bare_Single_Tok_Node_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_843;
            procedure Finalizer_Scope_2415;
            procedure Finalizer_Scope_2416;
            procedure Finalizer_Scope_2417;
      Match_0 : Bare_Defining_Name;
Match_1 : Bare_Single_Tok_Node;
Match_2 : Bare_Dotted_Name;
Match_3 : Bare_Name;
Fld : Bare_Name;
Fld_1 : Bare_Single_Tok_Node_Array_Access;
Singleton : Bare_Single_Tok_Node_Array_Access;
Fld_2 : Bare_Name;
Fld_3 : Bare_Single_Tok_Node_Array_Access;
Fld_4 : Bare_Base_Id;
Fld_5 : Bare_Single_Tok_Node_Array_Access;
Concat_Result : Bare_Single_Tok_Node_Array_Access;
Match_Prefix : Bare_Name;
Cast_Result : Bare_Defining_Name;
Let_Result : Bare_Single_Tok_Node_Array_Access;
Scope_Result : Bare_Single_Tok_Node_Array_Access;
Cast_Result_1 : Bare_Single_Tok_Node;
Let_Result_1 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_1 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_2 : Bare_Dotted_Name;
Let_Result_2 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_2 : Bare_Single_Tok_Node_Array_Access;
Cast_Result_3 : Bare_Name;
Let_Result_3 : Bare_Single_Tok_Node_Array_Access;
Scope_Result_3 : Bare_Single_Tok_Node_Array_Access;
Match_Result : Bare_Single_Tok_Node_Array_Access;
Let_Result_4 : Bare_Single_Tok_Node_Array_Access;
            procedure Finalizer_Scope_843 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_843;
            procedure Finalizer_Scope_2415 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2415;
            procedure Finalizer_Scope_2416 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2416;
            procedure Finalizer_Scope_2417 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_2417;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5829 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 5828 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10432
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Name (Match_Prefix.Kind) is
         when Ada_Defining_Name =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind dname Match_0
--# expr-start 5822 '<FieldAccess .as_single_tok_node_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10433
--# expr-start 5821 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10433
      if Match_0 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Match_0.Defining_Name_F_Name;
--# expr-done 5821
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_1 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld);
--# end
--# expr-done 5822
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2415;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Char_Literal .. Ada_Real_Literal =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind tok Match_1
                Singleton := Create_Bare_Single_Tok_Node_Array (Items_Count => 1);
                Singleton.Items (1) := Match_1;
Let_Result_1 := Singleton; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2416;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Dotted_Name =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind dot Match_2
--# expr-start 5827 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:10436
--# expr-start 5824 '<FieldAccess .as_single_tok_node_array>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10436
--# expr-start 5823 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10436
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Match_2.Dotted_Name_F_Prefix;
--# expr-done 5823
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_3 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_2);
--# end
--# expr-done 5824
--# expr-start 5826 '<FieldAccess .as_single_tok_node_array>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10437
--# expr-start 5825 '<FieldAccess .suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10437
      if Match_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Match_2.Dotted_Name_F_Suffix;
--# expr-done 5825
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld_5 := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Fld_4);
--# end
--# expr-done 5826
Concat_Result := Concat (Fld_3, Fld_5); 
--# expr-done 5827
Let_Result_2 := Concat_Result; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2417;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Call_Expr | Ada_Discrete_Subtype_Name | Ada_End_Name .. Ada_Qual_Expr | Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 5828
Let_Result_4 := Match_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 5829
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_843;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_843;
               Finalizer_Scope_2415;
               Finalizer_Scope_2416;
               Finalizer_Scope_2417;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_As_Single_Tok_Node_Array;
--# end
--# property-start Name.as_symbol_array /usr/src/packages/BUILD/ada/ast.py:10443
pragma Warnings (Off, "is not referenced");
 function Name_P_As_Symbol_Array
  (Node : Bare_Name
  )
   return Symbol_Type_Array_Access
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_844;
      Fld : Bare_Single_Tok_Node_Array_Access;
Sym : Symbol_Type;
Map_Result : Symbol_Type_Array_Access;
Let_Result : Symbol_Type_Array_Access;
            procedure Finalizer_Scope_844 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_844;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5833 '<Block>' Let_Result None
--# scope-start
--# expr-start 5832 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10450
--# expr-start 5830 '<FieldAccess .as_single_tok_node_array>' Fld /usr/src/packages/BUILD/ada/ast.py:10450
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_single_tok_node_array
Fld := Libadalang.Implementation.Name_P_As_Single_Tok_Node_Array (Node => Self);
--# end
--# expr-done 5830
declare
   Map_Result_Vec : Symbol_Type_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Single_Tok_Node_Array_Access := Fld;
      begin
         for Item_191 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_191
      --# expr-start 5831 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:10450
Sym := Get_Symbol (Item_191); 
--# expr-done 5831
         declare
            Item_To_Append : constant Symbol_Type :=
               Sym;
         begin
            Symbol_Type_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Symbol_Type_Array
        (Items_Count => Natural (Symbol_Type_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Symbol_Type_Vectors.Get
           (Map_Result_Vec,
            I + Symbol_Type_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Symbol_Type_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5832
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5833
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_844;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_844;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_As_Symbol_Array;
--# end
 function Name_P_Parent_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is (raise Property_Error
    with "Property Name.parent_scope not implemented on type "
    & Kind_Name (Node));
 function Name_P_All_Env_Els_Impl
  (Node : Bare_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is (raise Property_Error
    with "Property Name.all_env_els_impl not implemented on type "
    & Kind_Name (Node));
--# property-start Name.scope /usr/src/packages/BUILD/ada/ast.py:9937
pragma Warnings (Off, "is not referenced");
 function Name_P_Scope
  (Node : Bare_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := AST_Envs.Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Scope;
--# end
 function Name_P_Ref_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.ref_var not implemented on type "
    & Kind_Name (Node));
 function Name_P_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Logic_Var
is (raise Property_Error
    with "Property Name.subp_spec_var not implemented on type "
    & Kind_Name (Node));
--# property-start Name.defines_subp_spec_var /usr/src/packages/BUILD/ada/ast.py:9982
pragma Warnings (Off, "is not referenced");
 function Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Name
  )
   return Boolean
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5834 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 5834
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start Name.designated_type_impl /usr/src/packages/BUILD/ada/ast.py:10091
pragma Warnings (Off, "is not referenced");
 function Name_P_Designated_Type_Impl
  (Node : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Designated_Type_Impl;
--# end
 function Name_P_Relative_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is (raise Property_Error
    with "Property Name.relative_name not implemented on type "
    & Kind_Name (Node));
--# property-start Name.name_symbol /usr/src/packages/BUILD/ada/ast.py:10209
pragma Warnings (Off, "is not referenced");
 function Name_P_Name_Symbol
  (Node : Bare_Name
  )
   return Symbol_Type
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld : Internal_Entity_Single_Tok_Node;
Fld_1 : Bare_Single_Tok_Node;
Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5837 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:10209
--# expr-start 5836 '<FieldAccess .relative_name>' Fld /usr/src/packages/BUILD/ada/ast.py:10209
--# expr-start 5835 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:10209
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 5835
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 5836
Fld_1 := Fld.Node; 
Sym := Get_Symbol (Fld_1); 
--# expr-done 5837
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Name_Symbol;
--# end
--# property-start Name.base_name /usr/src/packages/BUILD/ada/ast.py:10211
pragma Warnings (Off, "is not referenced");
 function Name_P_Base_Name
  (Node : Bare_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Name := Bare_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Name;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Name_P_Base_Name;
--# end
function Name_P_Name_Designated_Type_Env
  (E : Entity
  ) return Lexical_Env
is
      E_Info : Internal_Entity_Info :=
         Shed_Rebindings (E.Info, Node_Env (E.Node));
   Result : constant Lexical_Env :=
      Name_P_Name_Designated_Type_Env (E.Node, E_Info);
begin
      return Result;
end;
function Name_P_Use_Package_Name_Designated_Env
  (E : Entity
  ) return Lexical_Env
is
   Result : constant Lexical_Env :=
      Name_P_Use_Package_Name_Designated_Env (E.Node);
begin
      return Result;
end;
   --
   --  Primitives for Bare_Attribute_Ref
   --
      procedure Initialize_Fields_For_Attribute_Ref
        (Self : Bare_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Attribute_Ref_F_Prefix := Attribute_Ref_F_Prefix;
            Self.Attribute_Ref_F_Attribute := Attribute_Ref_F_Attribute;
            Self.Attribute_Ref_F_Args := Attribute_Ref_F_Args;
      Self.Attribute_Ref_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Attribute_Ref;
   function Attribute_Ref_F_Prefix
     (Node : Bare_Attribute_Ref) return Bare_Name
   is
   begin
         return Node.Attribute_Ref_F_Prefix;
   end;
   function Attribute_Ref_F_Attribute
     (Node : Bare_Attribute_Ref) return Bare_Identifier
   is
   begin
         return Node.Attribute_Ref_F_Attribute;
   end;
   function Attribute_Ref_F_Args
     (Node : Bare_Attribute_Ref) return Bare_Ada_Node
   is
   begin
         return Node.Attribute_Ref_F_Args;
   end;
--# property-start AttributeRef.ref_var /usr/src/packages/BUILD/ada/ast.py:13339
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Ref_Var
  (Node : Bare_Attribute_Ref
  )
   return Logic_Var
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5838 '<FieldAccess .r_ref_var>' Fld /usr/src/packages/BUILD/ada/ast.py:13339
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_R_Ref_Var'Unrestricted_Access;
--# expr-done 5838
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Ref_Var;
--# end
--# property-start AttributeRef.relative_name /usr/src/packages/BUILD/ada/ast.py:13342
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Relative_Name
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 5840 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13342
--# expr-start 5839 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13342
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5839
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 5840
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Relative_Name;
--# end
--# property-start AttributeRef.designated_type_impl /usr/src/packages/BUILD/ada/ast.py:13344
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Type_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Scope_Result : Internal_Entity_Classwide_Type_Decl;
Result_Var : Internal_Entity_Classwide_Type_Decl;
Fld_5 : Bare_Identifier;
Fld_6 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
Fld_7 : Internal_Entity_Name;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Entity_Discrete_Base_Subtype_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
If_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5855 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:13344
--# expr-start 5844 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:13345
--# expr-start 5842 '<FieldAccess .sym>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13345
--# expr-start 5841 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13345
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 5841
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 5842
--# expr-start 5843 '<Symbol Class>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 5843
Is_Equal := Fld_1 = Sym; 
--# expr-done 5844
if Is_Equal then
--# expr-start 5846 '<FieldAccess .designated_type_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13346
--# expr-start 5845 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13346
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5845
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 5846
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 5847 '<FieldAccess .classwide_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13346
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 5847
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Classwide_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   If_Result_1 := Cast_Result_1;
else
--# expr-start 5851 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:13348
--# expr-start 5849 '<FieldAccess .sym>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13348
--# expr-start 5848 '<FieldAccess .attribute>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13348
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Attribute_Ref_F_Attribute;
--# expr-done 5848
      if Fld_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_6 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld_5);
--# end
--# expr-done 5849
--# expr-start 5850 '<Symbol Base>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Base); 
--# expr-done 5850
Is_Equal_1 := Fld_6 = Sym_1; 
--# expr-done 5851
if Is_Equal_1 then
--# expr-start 5854 '<FieldAccess .scalar_base_subtype>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13349
--# expr-start 5853 '<FieldAccess .name_designated_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13349
--# expr-start 5852 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13349
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5852
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_8 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 5853
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.scalar_base_subtype
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Scalar_Base_Subtype (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 5854
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   If_Result := Cast_Result;
else
   If_Result := No_Entity_Base_Type_Decl;
end if;
   If_Result_1 := If_Result;
end if;
--# expr-done 5855
      Property_Result := If_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Type_Impl;
--# end
--# property-start AttributeRef.args_list /usr/src/packages/BUILD/ada/ast.py:13354
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Args_List
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Var_Expr : Internal_Entity;
Fld : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Scope_Result : Internal_Entity_Assoc_List;
Result_Var : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 5856 '<FieldAccess .args>' Fld /usr/src/packages/BUILD/ada/ast.py:13354
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 5856
Var_Expr := Fld;
if Var_Expr /= No_Entity then
   --# scope-start
--# expr-start 5857 '<Cast to AssocList>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13354
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5857
Scope_Result := Cast_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Assoc_List;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Args_List;
--# end
--# property-start AttributeRef.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:13357
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Env_Elements_Impl
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1028;
      Fld : Bare_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
If_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1028 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1028;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5868 '<Block>' Let_Result None
--# scope-start
--# expr-start 5867 '<Cond>' If_Result /usr/src/packages/BUILD/ada/ast.py:13358
--# expr-start 5861 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:13359
--# expr-start 5859 '<FieldAccess .sym>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13359
--# expr-start 5858 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13359
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 5858
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Fld);
--# end
--# expr-done 5859
--# expr-start 5860 '<Symbol Unrestricted_Access>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5860
Is_Equal := Fld_1 = Sym; 
--# expr-done 5861
if Is_Equal then
   --# expr-start 5866 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13360
--# expr-start 5863 '<FieldAccess .env_elements_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13360
--# expr-start 5862 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13360
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5862
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_2.Node, Env => Env, E_Info => Fld_2.Info);
--# end
--# expr-done 5863
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_192 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_192
      --# expr-start 5865 '<FieldAccess .trigger_access_entity(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13364
--# expr-start 5864 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13364
Cast_Expr := Item_192; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5864
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.trigger_access_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Trigger_Access_Entity (Node => Cast_Result.Node, Val => True, E_Info => Cast_Result.Info);
--# end
--# expr-done 5865
         declare
            Item_To_Append : constant Internal_Entity :=
               Fld_4;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5866
   If_Result := Map_Result;
else
   If_Result := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (If_Result);
--# expr-done 5867
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 5868
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1028;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1028;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Env_Elements_Impl;
--# end
--# property-start AttributeRef.is_access_attr /usr/src/packages/BUILD/ada/ast.py:13370
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Is_Access_Attr
  (Node : Bare_Attribute_Ref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 5872 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5870 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5869 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13371
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5869
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5870
--# expr-start 5871 '<Symbol Access>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access); 
--# expr-done 5871
Is_Equal := Fld_1 = Sym; 
--# expr-done 5872
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5876 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5874 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5873 '<FieldAccess .attribute>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13371
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5873
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 5874
--# expr-start 5875 '<Symbol Unchecked_Access>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Access); 
--# expr-done 5875
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 5876
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 5880 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5878 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13371
--# expr-start 5877 '<FieldAccess .attribute>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13371
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5877
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 5878
--# expr-start 5879 '<Symbol Unrestricted_Access>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5879
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 5880
   If_Result_1 := Is_Equal_2;
end if;
      Property_Result := If_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Is_Access_Attr;
--# end
--# property-start AttributeRef.designated_env /usr/src/packages/BUILD/ada/ast.py:13377
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1030;
      Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Boolean;
Fld_4 : Internal_Entity_Name;
Fld_5 : Lexical_Env := Empty_Env;
Fld_6 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_7 : Boolean;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Entity_Base_Type_Decl;
Fld_10 : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
If_Result_2 : Lexical_Env := Empty_Env;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1030 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1030;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5901 '<Block>' Let_Result None
--# scope-start
--# expr-start 5900 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:13378
--# expr-start 5883 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13379
--# expr-start 5881 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13379
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5881
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5882 '<Symbol Model>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model); 
--# expr-done 5882
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 5883
if Fld_1 then
   --# expr-start 5884 '<FieldAccess .designated_env_model_attr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13380
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.designated_env_model_attr
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Designated_Env_Model_Attr (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5884
   If_Result_3 := Fld_2;
else
--# expr-start 5885 '<FieldAccess .is_access_attr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13382
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.is_access_attr
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Is_Access_Attr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 5885
if Fld_3 then
   --# expr-start 5887 '<FieldAccess .designated_env>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13383
--# expr-start 5886 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13383
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5886
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 5887
   If_Result_2 := Fld_5;
else
--# expr-start 5890 '<FieldAccess .name_is(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13385
--# expr-start 5888 '<FieldAccess .attribute>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13385
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5888
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5889 '<Symbol Result>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Result); 
--# expr-done 5889
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_6.Node, Sym => Sym_1);
--# end
--# expr-done 5890
if Fld_7 then
   --# expr-start 5899 '<FieldAccess .defining_env>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5898 '<FieldAccess .return_type>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5897 '<FieldAccess .subp_spec_or_null>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5896 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5895 '<as_entity(<at(<filter>, 0)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5894 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5893 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13386
--# expr-start 5891 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:13386
Node_Parents := Parents (Self, True); 
--# expr-done 5891
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_193 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_193
               --# expr-start 5892 '<IsA BasicSubpDecl, SubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:13386
Is_A := Item_193 /= null 
and then Item_193.Kind in Ada_Basic_Subp_Decl | Ada_Subp_Body_Range; 
--# expr-done 5892
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_193;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5893
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 5894
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 5895
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 5896
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 5897
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_9 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 5898
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_9.Node, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 5899
   If_Result_1 := Fld_10;
else
   If_Result_1 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (If_Result_1);
   If_Result_2 := If_Result_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 5900
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 5901
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1030;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1030;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Env;
--# end
--# property-start AttributeRef.designated_env_model_attr /usr/src/packages/BUILD/ada/ast.py:13394
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Designated_Env_Model_Attr
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1031;
            procedure Finalizer_Scope_2426;
      Block_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_3 : Internal_Unit;
Fld_4 : Internal_Entity_Base_Type_Decl;
Map_Result_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Is_Null : Boolean;
Not_Val : Boolean;
Map_Result_2 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_5 : Lexical_Env := Empty_Env;
Map_Result_3 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1031 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1031;
            procedure Finalizer_Scope_2426 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2426;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 5916 '<Block>' Let_Result None
--# scope-start
--# expr-start 5912 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:13396
--# expr-start 5909 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:13396
--# expr-start 5906 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13396
--# expr-start 5903 '<FieldAccess .env_elements>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13396
--# expr-start 5902 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13396
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 5902
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_1 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 5903
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_196 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_196
      --# expr-start 5905 '<FieldAccess .expr_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13397
--# expr-start 5904 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13397
Cast_Expr := Item_196; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 5904
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 5905
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_2;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 5906
declare
   Map_Result_1_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result;
      begin
         for Item_195 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_195
      --# expr-start 5908 '<FieldAccess .modeled_type(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13398
      if Item_195.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5907 '<FieldAccess .unit>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13398
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 5907
--# property-call-start BaseTypeDecl.modeled_type
Fld_4 := Libadalang.Implementation.Base_Type_Decl_P_Modeled_Type (Node => Item_195.Node, From_Unit => Fld_3, E_Info => Item_195.Info);
--# end
--# expr-done 5908
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_4;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 5909
declare
   Map_Result_2_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Map_Result_1;
      begin
         for Item_194 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_194
               --# expr-start 5911 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:13399
--# expr-start 5910 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:13399
Is_Null := Item_194.Node = null; 
--# expr-done 5910
Not_Val := not (Is_Null); 
--# expr-done 5911
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Item_194;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 5912
Block_Var := Map_Result_2; Inc_Ref (Block_Var);
--# bind model_types Block_Var
--# expr-start 5915 '<env_group(<map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:13402
--# expr-start 5914 '<map>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:13402
declare
   Map_Result_3_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Block_Var;
      begin
         for Item_197 of
               Collection.Items
         loop
   --# scope-start
               --# bind mt Item_197
      --# expr-start 5913 '<FieldAccess .defining_env>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13402
      if Item_197.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Item_197.Node, Origin => Origin, E_Info => Item_197.Info);
--# end
--# expr-done 5913
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_5;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2426;
         end loop;
      end;
      Map_Result_3 := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_3_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 5914
Group_Env := Group (Map_Result_3, No_Metadata); 
--# expr-done 5915
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
--# expr-done 5916
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1031;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1031;
               Finalizer_Scope_2426;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Designated_Env_Model_Attr;
--# end
--# property-start AttributeRef.xref_equation /usr/src/packages/BUILD/ada/ast.py:13405
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1032;
      Block_Var : Symbol_Type;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_2 : Logic_Equation;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Fld_3 : Logic_Equation;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_2 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_3 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_4 : Boolean;
Fld_4 : Logic_Equation;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_5 : Boolean;
Fld_5 : Logic_Equation;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
Fld_6 : Logic_Equation;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_6 : Boolean;
Fld_7 : Logic_Equation;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_7 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_8 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_9 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_10 : Boolean;
Fld_8 : Logic_Equation;
Sym_18 : Symbol_Type;
Is_Equal_18 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_11 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_12 : Boolean;
Fld_9 : Logic_Equation;
Sym_21 : Symbol_Type;
Is_Equal_21 : Boolean;
Sym_22 : Symbol_Type;
Fld_10 : Internal_Entity;
Fld_11 : Logic_Equation;
Sym_23 : Symbol_Type;
Is_Equal_22 : Boolean;
Sym_24 : Symbol_Type;
Fld_12 : Internal_Entity;
Fld_13 : Logic_Equation;
Sym_25 : Symbol_Type;
Is_Equal_23 : Boolean;
Sym_26 : Symbol_Type;
Fld_14 : Internal_Entity;
Fld_15 : Logic_Equation;
Sym_27 : Symbol_Type;
Is_Equal_24 : Boolean;
Sym_28 : Symbol_Type;
Fld_16 : Internal_Entity;
Fld_17 : Logic_Equation;
Sym_29 : Symbol_Type;
Is_Equal_25 : Boolean;
Sym_30 : Symbol_Type;
Fld_18 : Internal_Entity;
Fld_19 : Logic_Equation;
Sym_31 : Symbol_Type;
Is_Equal_26 : Boolean;
Sym_32 : Symbol_Type;
Fld_20 : Internal_Entity;
Fld_21 : Logic_Equation;
Sym_33 : Symbol_Type;
Is_Equal_27 : Boolean;
Fld_22 : Logic_Equation;
Sym_34 : Symbol_Type;
Is_Equal_28 : Boolean;
Fld_23 : Logic_Equation;
Sym_35 : Symbol_Type;
Is_Equal_29 : Boolean;
Fld_24 : Logic_Equation;
Sym_36 : Symbol_Type;
Is_Equal_30 : Boolean;
Sym_37 : Symbol_Type;
Is_Equal_31 : Boolean;
If_Result_13 : Boolean;
Sym_38 : Symbol_Type;
Is_Equal_32 : Boolean;
If_Result_14 : Boolean;
Sym_39 : Symbol_Type;
Is_Equal_33 : Boolean;
If_Result_15 : Boolean;
Sym_40 : Symbol_Type;
Is_Equal_34 : Boolean;
If_Result_16 : Boolean;
Sym_41 : Symbol_Type;
Is_Equal_35 : Boolean;
If_Result_17 : Boolean;
Sym_42 : Symbol_Type;
Is_Equal_36 : Boolean;
If_Result_18 : Boolean;
Sym_43 : Symbol_Type;
Is_Equal_37 : Boolean;
If_Result_19 : Boolean;
Fld_25 : Logic_Equation;
Sym_44 : Symbol_Type;
Is_Equal_38 : Boolean;
Sym_45 : Symbol_Type;
Fld_26 : Internal_Entity;
Fld_27 : Logic_Equation;
Sym_46 : Symbol_Type;
Is_Equal_39 : Boolean;
Sym_47 : Symbol_Type;
Is_Equal_40 : Boolean;
If_Result_20 : Boolean;
Sym_48 : Symbol_Type;
Is_Equal_41 : Boolean;
If_Result_21 : Boolean;
Fld_28 : Logic_Equation;
Sym_49 : Symbol_Type;
Is_Equal_42 : Boolean;
Fld_29 : Logic_Equation;
Sym_50 : Symbol_Type;
Is_Equal_43 : Boolean;
Fld_30 : Logic_Equation;
Sym_51 : Symbol_Type;
Is_Equal_44 : Boolean;
Fld_31 : Logic_Equation;
Sym_52 : Symbol_Type;
Is_Equal_45 : Boolean;
Sym_53 : Symbol_Type;
Is_Equal_46 : Boolean;
If_Result_22 : Boolean;
Fld_32 : Logic_Equation;
Sym_54 : Symbol_Type;
Is_Equal_47 : Boolean;
Fld_33 : Internal_Entity_Name;
Fld_34 : Logic_Equation;
Sym_55 : Symbol_Type;
Is_Equal_48 : Boolean;
Fld_35 : Internal_Entity_Name;
Fld_36 : Logic_Equation;
Fld_37 : Logic_Var;
Fld_38 : Logic_Equation;
And_Pred : Logic_Equation;
Sym_56 : Symbol_Type;
Is_Equal_49 : Boolean;
Fld_39 : Logic_Equation;
Sym_57 : Symbol_Type;
Is_Equal_50 : Boolean;
Sym_58 : Symbol_Type;
Is_Equal_51 : Boolean;
If_Result_23 : Boolean;
Sym_59 : Symbol_Type;
Is_Equal_52 : Boolean;
If_Result_24 : Boolean;
Sym_60 : Symbol_Type;
Is_Equal_53 : Boolean;
If_Result_25 : Boolean;
Sym_61 : Symbol_Type;
Is_Equal_54 : Boolean;
If_Result_26 : Boolean;
Sym_62 : Symbol_Type;
Is_Equal_55 : Boolean;
If_Result_27 : Boolean;
Sym_63 : Symbol_Type;
Is_Equal_56 : Boolean;
If_Result_28 : Boolean;
Sym_64 : Symbol_Type;
Is_Equal_57 : Boolean;
If_Result_29 : Boolean;
Sym_65 : Symbol_Type;
Is_Equal_58 : Boolean;
If_Result_30 : Boolean;
Sym_66 : Symbol_Type;
Is_Equal_59 : Boolean;
If_Result_31 : Boolean;
Sym_67 : Symbol_Type;
Is_Equal_60 : Boolean;
If_Result_32 : Boolean;
Sym_68 : Symbol_Type;
Is_Equal_61 : Boolean;
If_Result_33 : Boolean;
Sym_69 : Symbol_Type;
Is_Equal_62 : Boolean;
If_Result_34 : Boolean;
Sym_70 : Symbol_Type;
Is_Equal_63 : Boolean;
If_Result_35 : Boolean;
Sym_71 : Symbol_Type;
Is_Equal_64 : Boolean;
If_Result_36 : Boolean;
Sym_72 : Symbol_Type;
Is_Equal_65 : Boolean;
If_Result_37 : Boolean;
Sym_73 : Symbol_Type;
Is_Equal_66 : Boolean;
If_Result_38 : Boolean;
Sym_74 : Symbol_Type;
Is_Equal_67 : Boolean;
If_Result_39 : Boolean;
Sym_75 : Symbol_Type;
Is_Equal_68 : Boolean;
If_Result_40 : Boolean;
Sym_76 : Symbol_Type;
Is_Equal_69 : Boolean;
If_Result_41 : Boolean;
Sym_77 : Symbol_Type;
Is_Equal_70 : Boolean;
If_Result_42 : Boolean;
Sym_78 : Symbol_Type;
Is_Equal_71 : Boolean;
If_Result_43 : Boolean;
Fld_40 : Internal_Entity_Name;
Fld_41 : Logic_Equation;
Fld_42 : Logic_Var;
Fld_43 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Sym_79 : Symbol_Type;
Is_Equal_72 : Boolean;
Fld_44 : Logic_Var;
Sym_80 : Symbol_Type;
Fld_45 : Internal_Entity;
Fld_46 : Logic_Equation;
Sym_81 : Symbol_Type;
Is_Equal_73 : Boolean;
Fld_47 : Logic_Equation;
Sym_82 : Symbol_Type;
Is_Equal_74 : Boolean;
Fld_48 : Logic_Equation;
Sym_83 : Symbol_Type;
Is_Equal_75 : Boolean;
Sym_84 : Symbol_Type;
Is_Equal_76 : Boolean;
If_Result_44 : Boolean;
Fld_49 : Internal_Entity_Name;
Fld_50 : Logic_Equation;
Fld_51 : Logic_Var;
Fld_52 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Sym_85 : Symbol_Type;
Is_Equal_77 : Boolean;
Fld_53 : Internal_Entity_Name;
Fld_54 : Logic_Equation;
Fld_55 : Logic_Var;
Fld_56 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Sym_86 : Symbol_Type;
Is_Equal_78 : Boolean;
Fld_57 : Internal_Entity_Name;
Fld_58 : Logic_Equation;
Fld_59 : Logic_Var;
Fld_60 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_4 : Logic_Equation;
Sym_87 : Symbol_Type;
Is_Equal_79 : Boolean;
Sym_88 : Symbol_Type;
Is_Equal_80 : Boolean;
If_Result_45 : Boolean;
Sym_89 : Symbol_Type;
Is_Equal_81 : Boolean;
If_Result_46 : Boolean;
Sym_90 : Symbol_Type;
Is_Equal_82 : Boolean;
If_Result_47 : Boolean;
Sym_91 : Symbol_Type;
Is_Equal_83 : Boolean;
If_Result_48 : Boolean;
Sym_92 : Symbol_Type;
Is_Equal_84 : Boolean;
If_Result_49 : Boolean;
Sym_93 : Symbol_Type;
Is_Equal_85 : Boolean;
If_Result_50 : Boolean;
Sym_94 : Symbol_Type;
Is_Equal_86 : Boolean;
If_Result_51 : Boolean;
Fld_61 : Logic_Equation;
Sym_95 : Symbol_Type;
Is_Equal_87 : Boolean;
Sym_96 : Symbol_Type;
Is_Equal_88 : Boolean;
If_Result_52 : Boolean;
Fld_62 : Logic_Equation;
Sym_97 : Symbol_Type;
Is_Equal_89 : Boolean;
Fld_63 : Logic_Equation;
Sym_98 : Symbol_Type;
Is_Equal_90 : Boolean;
Sym_99 : Symbol_Type;
Is_Equal_91 : Boolean;
If_Result_53 : Boolean;
Fld_64 : Logic_Equation;
If_Result_54 : Logic_Equation;
If_Result_55 : Logic_Equation;
If_Result_56 : Logic_Equation;
If_Result_57 : Logic_Equation;
If_Result_58 : Logic_Equation;
If_Result_59 : Logic_Equation;
If_Result_60 : Logic_Equation;
If_Result_61 : Logic_Equation;
If_Result_62 : Logic_Equation;
If_Result_63 : Logic_Equation;
If_Result_64 : Logic_Equation;
If_Result_65 : Logic_Equation;
If_Result_66 : Logic_Equation;
If_Result_67 : Logic_Equation;
If_Result_68 : Logic_Equation;
If_Result_69 : Logic_Equation;
If_Result_70 : Logic_Equation;
If_Result_71 : Logic_Equation;
If_Result_72 : Logic_Equation;
If_Result_73 : Logic_Equation;
If_Result_74 : Logic_Equation;
If_Result_75 : Logic_Equation;
If_Result_76 : Logic_Equation;
If_Result_77 : Logic_Equation;
If_Result_78 : Logic_Equation;
If_Result_79 : Logic_Equation;
If_Result_80 : Logic_Equation;
If_Result_81 : Logic_Equation;
If_Result_82 : Logic_Equation;
If_Result_83 : Logic_Equation;
If_Result_84 : Logic_Equation;
If_Result_85 : Logic_Equation;
If_Result_86 : Logic_Equation;
If_Result_87 : Logic_Equation;
If_Result_88 : Logic_Equation;
If_Result_89 : Logic_Equation;
If_Result_90 : Logic_Equation;
If_Result_91 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1032 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_9);
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (Fld_17);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Fld_21);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Fld_24);
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_27);
                     Dec_Ref (Fld_28);
                     Dec_Ref (Fld_29);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_34);
                     Dec_Ref (Fld_36);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_39);
                     Dec_Ref (Fld_41);
                     Dec_Ref (Fld_43);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_46);
                     Dec_Ref (Fld_47);
                     Dec_Ref (Fld_48);
                     Dec_Ref (Fld_50);
                     Dec_Ref (Fld_52);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_54);
                     Dec_Ref (Fld_56);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_61);
                     Dec_Ref (Fld_62);
                     Dec_Ref (Fld_63);
                     Dec_Ref (Fld_64);
                     Dec_Ref (If_Result_54);
                     Dec_Ref (If_Result_55);
                     Dec_Ref (If_Result_56);
                     Dec_Ref (If_Result_57);
                     Dec_Ref (If_Result_58);
                     Dec_Ref (If_Result_59);
                     Dec_Ref (If_Result_60);
                     Dec_Ref (If_Result_61);
                     Dec_Ref (If_Result_62);
                     Dec_Ref (If_Result_63);
                     Dec_Ref (If_Result_64);
                     Dec_Ref (If_Result_65);
                     Dec_Ref (If_Result_66);
                     Dec_Ref (If_Result_67);
                     Dec_Ref (If_Result_68);
                     Dec_Ref (If_Result_69);
                     Dec_Ref (If_Result_70);
                     Dec_Ref (If_Result_71);
                     Dec_Ref (If_Result_72);
                     Dec_Ref (If_Result_73);
                     Dec_Ref (If_Result_74);
                     Dec_Ref (If_Result_75);
                     Dec_Ref (If_Result_76);
                     Dec_Ref (If_Result_77);
                     Dec_Ref (If_Result_78);
                     Dec_Ref (If_Result_79);
                     Dec_Ref (If_Result_80);
                     Dec_Ref (If_Result_81);
                     Dec_Ref (If_Result_82);
                     Dec_Ref (If_Result_83);
                     Dec_Ref (If_Result_84);
                     Dec_Ref (If_Result_85);
                     Dec_Ref (If_Result_86);
                     Dec_Ref (If_Result_87);
                     Dec_Ref (If_Result_88);
                     Dec_Ref (If_Result_89);
                     Dec_Ref (If_Result_90);
                     Dec_Ref (If_Result_91);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1032;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6181 '<Block>' Let_Result None
--# scope-start
--# expr-start 5918 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13406
--# expr-start 5917 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13406
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 5917
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 5918
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 6180 '<Cond>' If_Result_91 /usr/src/packages/BUILD/ada/ast.py:13407
--# expr-start 5920 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:13408
--# expr-start 5919 '<Symbol Succ>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Succ); 
--# expr-done 5919
Is_Equal := Block_Var = Sym; 
--# expr-done 5920
if Is_Equal then
   If_Result := True;
else
   --# expr-start 5922 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:13408
--# expr-start 5921 '<Symbol Pred>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pred); 
--# expr-done 5921
Is_Equal_1 := Block_Var = Sym_1; 
--# expr-done 5922
   If_Result := Is_Equal_1;
end if;
if If_Result then
   --# expr-start 5923 '<FieldAccess .succpred_xref_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13408
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.succpred_xref_equation
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Succpred_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5923
   If_Result_91 := Fld_2;
else
--# expr-start 5925 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:13409
--# expr-start 5924 '<Symbol Min>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Min); 
--# expr-done 5924
Is_Equal_2 := Block_Var = Sym_2; 
--# expr-done 5925
if Is_Equal_2 then
   If_Result_1 := True;
else
   --# expr-start 5927 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:13409
--# expr-start 5926 '<Symbol Max>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max); 
--# expr-done 5926
Is_Equal_3 := Block_Var = Sym_3; 
--# expr-done 5927
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   --# expr-start 5928 '<FieldAccess .minmax_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13409
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.minmax_equation
Fld_3 := Libadalang.Implementation.Attribute_Ref_P_Minmax_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5928
   If_Result_90 := Fld_3;
else
--# expr-start 5930 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:13411
--# expr-start 5929 '<Symbol First>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 5929
Is_Equal_4 := Block_Var = Sym_4; 
--# expr-done 5930
if Is_Equal_4 then
   If_Result_2 := True;
else
   --# expr-start 5932 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:13411
--# expr-start 5931 '<Symbol Last>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 5931
Is_Equal_5 := Block_Var = Sym_5; 
--# expr-done 5932
   If_Result_2 := Is_Equal_5;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 5934 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:13411
--# expr-start 5933 '<Symbol Range>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 5933
Is_Equal_6 := Block_Var = Sym_6; 
--# expr-done 5934
   If_Result_3 := Is_Equal_6;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 5936 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:13411
--# expr-start 5935 '<Symbol Length>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 5935
Is_Equal_7 := Block_Var = Sym_7; 
--# expr-done 5936
   If_Result_4 := Is_Equal_7;
end if;
if If_Result_4 then
   --# expr-start 5937 '<FieldAccess .array_attr_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13412
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.array_attr_equation
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Array_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5937
   If_Result_89 := Fld_4;
else
--# expr-start 5939 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:13414
--# expr-start 5938 '<Symbol Size>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Size); 
--# expr-done 5938
Is_Equal_8 := Block_Var = Sym_8; 
--# expr-done 5939
if Is_Equal_8 then
   If_Result_5 := True;
else
   --# expr-start 5941 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:13414
--# expr-start 5940 '<Symbol VADS_Size>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Vads_Size); 
--# expr-done 5940
Is_Equal_9 := Block_Var = Sym_9; 
--# expr-done 5941
   If_Result_5 := Is_Equal_9;
end if;
if If_Result_5 then
   --# expr-start 5942 '<FieldAccess .size_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13414
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.size_equation
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Size_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5942
   If_Result_88 := Fld_5;
else
--# expr-start 5944 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:13415
--# expr-start 5943 '<Symbol Pos>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pos); 
--# expr-done 5943
Is_Equal_10 := Block_Var = Sym_10; 
--# expr-done 5944
if Is_Equal_10 then
   --# expr-start 5945 '<FieldAccess .pos_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13415
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.pos_equation
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Pos_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5945
   If_Result_87 := Fld_6;
else
--# expr-start 5947 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:13416
--# expr-start 5946 '<Symbol Val>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Val); 
--# expr-done 5946
Is_Equal_11 := Block_Var = Sym_11; 
--# expr-done 5947
if Is_Equal_11 then
   If_Result_6 := True;
else
   --# expr-start 5949 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:13416
--# expr-start 5948 '<Symbol Enum_Val>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Val); 
--# expr-done 5948
Is_Equal_12 := Block_Var = Sym_12; 
--# expr-done 5949
   If_Result_6 := Is_Equal_12;
end if;
if If_Result_6 then
   --# expr-start 5950 '<FieldAccess .val_equation>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13416
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.val_equation
Fld_7 := Libadalang.Implementation.Attribute_Ref_P_Val_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5950
   If_Result_86 := Fld_7;
else
--# expr-start 5952 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:13418
--# expr-start 5951 '<Symbol Max_Size_In_Storage_Elements>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max_Size_In_Storage_Elements); 
--# expr-done 5951
Is_Equal_13 := Block_Var = Sym_13; 
--# expr-done 5952
if Is_Equal_13 then
   If_Result_7 := True;
else
   --# expr-start 5954 '<Eq>' Is_Equal_14 /usr/src/packages/BUILD/ada/ast.py:13418
--# expr-start 5953 '<Symbol Aft>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aft); 
--# expr-done 5953
Is_Equal_14 := Block_Var = Sym_14; 
--# expr-done 5954
   If_Result_7 := Is_Equal_14;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 5956 '<Eq>' Is_Equal_15 /usr/src/packages/BUILD/ada/ast.py:13418
--# expr-start 5955 '<Symbol Object_Size>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Object_Size); 
--# expr-done 5955
Is_Equal_15 := Block_Var = Sym_15; 
--# expr-done 5956
   If_Result_8 := Is_Equal_15;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 5958 '<Eq>' Is_Equal_16 /usr/src/packages/BUILD/ada/ast.py:13418
--# expr-start 5957 '<Symbol Value_Size>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value_Size); 
--# expr-done 5957
Is_Equal_16 := Block_Var = Sym_16; 
--# expr-done 5958
   If_Result_9 := Is_Equal_16;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 5960 '<Eq>' Is_Equal_17 /usr/src/packages/BUILD/ada/ast.py:13418
--# expr-start 5959 '<Symbol Storage_Size>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Size); 
--# expr-done 5959
Is_Equal_17 := Block_Var = Sym_17; 
--# expr-done 5960
   If_Result_10 := Is_Equal_17;
end if;
if If_Result_10 then
   --# expr-start 5961 '<FieldAccess .subtype_attr_equation>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13420
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.subtype_attr_equation
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Subtype_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5961
   If_Result_85 := Fld_8;
else
--# expr-start 5963 '<Eq>' Is_Equal_18 /usr/src/packages/BUILD/ada/ast.py:13422
--# expr-start 5962 '<Symbol Access>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Access); 
--# expr-done 5962
Is_Equal_18 := Block_Var = Sym_18; 
--# expr-done 5963
if Is_Equal_18 then
   If_Result_11 := True;
else
   --# expr-start 5965 '<Eq>' Is_Equal_19 /usr/src/packages/BUILD/ada/ast.py:13422
--# expr-start 5964 '<Symbol Unchecked_Access>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Access); 
--# expr-done 5964
Is_Equal_19 := Block_Var = Sym_19; 
--# expr-done 5965
   If_Result_11 := Is_Equal_19;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 5967 '<Eq>' Is_Equal_20 /usr/src/packages/BUILD/ada/ast.py:13422
--# expr-start 5966 '<Symbol Unrestricted_Access>' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unrestricted_Access); 
--# expr-done 5966
Is_Equal_20 := Block_Var = Sym_20; 
--# expr-done 5967
   If_Result_12 := Is_Equal_20;
end if;
if If_Result_12 then
   --# expr-start 5968 '<FieldAccess .access_equation>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13424
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.access_equation
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Access_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5968
   If_Result_84 := Fld_9;
else
--# expr-start 5970 '<Eq>' Is_Equal_21 /usr/src/packages/BUILD/ada/ast.py:13426
--# expr-start 5969 '<Symbol Image>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Image); 
--# expr-done 5969
Is_Equal_21 := Block_Var = Sym_21; 
--# expr-done 5970
if Is_Equal_21 then
   --# expr-start 5973 '<FieldAccess .image_equation(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13427
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5972 '<FieldAccess .std_entity(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13427
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5971 '<Symbol String>' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5971
--# property-call-start AdaNode.std_entity
Fld_10 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_22);
--# end
--# expr-done 5972
--# property-call-start AttributeRef.image_equation
Fld_11 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_10, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5973
   If_Result_83 := Fld_11;
else
--# expr-start 5975 '<Eq>' Is_Equal_22 /usr/src/packages/BUILD/ada/ast.py:13429
--# expr-start 5974 '<Symbol Wide_Image>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Image); 
--# expr-done 5974
Is_Equal_22 := Block_Var = Sym_23; 
--# expr-done 5975
if Is_Equal_22 then
   --# expr-start 5978 '<FieldAccess .image_equation(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13430
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5977 '<FieldAccess .std_entity(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13430
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5976 '<Symbol Wide_String>' Sym_24 None
Sym_24 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_String); 
--# expr-done 5976
--# property-call-start AdaNode.std_entity
Fld_12 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_24);
--# end
--# expr-done 5977
--# property-call-start AttributeRef.image_equation
Fld_13 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_12, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5978
   If_Result_82 := Fld_13;
else
--# expr-start 5980 '<Eq>' Is_Equal_23 /usr/src/packages/BUILD/ada/ast.py:13432
--# expr-start 5979 '<Symbol Wide_Wide_Image>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Image); 
--# expr-done 5979
Is_Equal_23 := Block_Var = Sym_25; 
--# expr-done 5980
if Is_Equal_23 then
   --# expr-start 5983 '<FieldAccess .image_equation(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13433
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5982 '<FieldAccess .std_entity(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13433
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5981 '<Symbol Wide_Wide_String>' Sym_26 None
Sym_26 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_String); 
--# expr-done 5981
--# property-call-start AdaNode.std_entity
Fld_14 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_26);
--# end
--# expr-done 5982
--# property-call-start AttributeRef.image_equation
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Image_Equation (Node => Ent.Node, Str_Type => Fld_14, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5983
   If_Result_81 := Fld_15;
else
--# expr-start 5985 '<Eq>' Is_Equal_24 /usr/src/packages/BUILD/ada/ast.py:13435
--# expr-start 5984 '<Symbol Value>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value); 
--# expr-done 5984
Is_Equal_24 := Block_Var = Sym_27; 
--# expr-done 5985
if Is_Equal_24 then
   --# expr-start 5988 '<FieldAccess .value_equation(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13436
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5987 '<FieldAccess .std_entity(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13436
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5986 '<Symbol String>' Sym_28 None
Sym_28 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 5986
--# property-call-start AdaNode.std_entity
Fld_16 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_28);
--# end
--# expr-done 5987
--# property-call-start AttributeRef.value_equation
Fld_17 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_16, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5988
   If_Result_80 := Fld_17;
else
--# expr-start 5990 '<Eq>' Is_Equal_25 /usr/src/packages/BUILD/ada/ast.py:13438
--# expr-start 5989 '<Symbol Wide_Value>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Value); 
--# expr-done 5989
Is_Equal_25 := Block_Var = Sym_29; 
--# expr-done 5990
if Is_Equal_25 then
   --# expr-start 5993 '<FieldAccess .value_equation(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13439
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5992 '<FieldAccess .std_entity(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13439
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5991 '<Symbol Wide_String>' Sym_30 None
Sym_30 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_String); 
--# expr-done 5991
--# property-call-start AdaNode.std_entity
Fld_18 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_30);
--# end
--# expr-done 5992
--# property-call-start AttributeRef.value_equation
Fld_19 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_18, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5993
   If_Result_79 := Fld_19;
else
--# expr-start 5995 '<Eq>' Is_Equal_26 /usr/src/packages/BUILD/ada/ast.py:13441
--# expr-start 5994 '<Symbol Wide_Wide_Value>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Value); 
--# expr-done 5994
Is_Equal_26 := Block_Var = Sym_31; 
--# expr-done 5995
if Is_Equal_26 then
   --# expr-start 5998 '<FieldAccess .value_equation(...)>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13442
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5997 '<FieldAccess .std_entity(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13442
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 5996 '<Symbol Wide_Wide_String>' Sym_32 None
Sym_32 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_String); 
--# expr-done 5996
--# property-call-start AdaNode.std_entity
Fld_20 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_32);
--# end
--# expr-done 5997
--# property-call-start AttributeRef.value_equation
Fld_21 := Libadalang.Implementation.Attribute_Ref_P_Value_Equation (Node => Ent.Node, Str_Type => Fld_20, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 5998
   If_Result_78 := Fld_21;
else
--# expr-start 6000 '<Eq>' Is_Equal_27 /usr/src/packages/BUILD/ada/ast.py:13444
--# expr-start 5999 '<Symbol Invalid_Value>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Invalid_Value); 
--# expr-done 5999
Is_Equal_27 := Block_Var = Sym_33; 
--# expr-done 6000
if Is_Equal_27 then
   --# expr-start 6001 '<FieldAccess .invalid_value_equation>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13445
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.invalid_value_equation
Fld_22 := Libadalang.Implementation.Attribute_Ref_P_Invalid_Value_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6001
   If_Result_77 := Fld_22;
else
--# expr-start 6003 '<Eq>' Is_Equal_28 /usr/src/packages/BUILD/ada/ast.py:13447
--# expr-start 6002 '<Symbol Identity>' Sym_34 None
Sym_34 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Identity); 
--# expr-done 6002
Is_Equal_28 := Block_Var = Sym_34; 
--# expr-done 6003
if Is_Equal_28 then
   --# expr-start 6004 '<FieldAccess .identity_equation>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:13447
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.identity_equation
Fld_23 := Libadalang.Implementation.Attribute_Ref_P_Identity_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6004
   If_Result_76 := Fld_23;
else
--# expr-start 6006 '<Eq>' Is_Equal_29 /usr/src/packages/BUILD/ada/ast.py:13448
--# expr-start 6005 '<Symbol Address>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address); 
--# expr-done 6005
Is_Equal_29 := Block_Var = Sym_35; 
--# expr-done 6006
if Is_Equal_29 then
   --# expr-start 6007 '<FieldAccess .address_equation>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:13448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.address_equation
Fld_24 := Libadalang.Implementation.Attribute_Ref_P_Address_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6007
   If_Result_75 := Fld_24;
else
--# expr-start 6009 '<Eq>' Is_Equal_30 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6008 '<Symbol Small>' Sym_36 None
Sym_36 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Small); 
--# expr-done 6008
Is_Equal_30 := Block_Var = Sym_36; 
--# expr-done 6009
if Is_Equal_30 then
   If_Result_13 := True;
else
   --# expr-start 6011 '<Eq>' Is_Equal_31 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6010 '<Symbol Model_Small>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Small); 
--# expr-done 6010
Is_Equal_31 := Block_Var = Sym_37; 
--# expr-done 6011
   If_Result_13 := Is_Equal_31;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 6013 '<Eq>' Is_Equal_32 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6012 '<Symbol Safe_Small>' Sym_38 None
Sym_38 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Safe_Small); 
--# expr-done 6012
Is_Equal_32 := Block_Var = Sym_38; 
--# expr-done 6013
   If_Result_14 := Is_Equal_32;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 6015 '<Eq>' Is_Equal_33 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6014 '<Symbol Epsilon>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Epsilon); 
--# expr-done 6014
Is_Equal_33 := Block_Var = Sym_39; 
--# expr-done 6015
   If_Result_15 := Is_Equal_33;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 6017 '<Eq>' Is_Equal_34 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6016 '<Symbol Model_Epsilon>' Sym_40 None
Sym_40 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Epsilon); 
--# expr-done 6016
Is_Equal_34 := Block_Var = Sym_40; 
--# expr-done 6017
   If_Result_16 := Is_Equal_34;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 6019 '<Eq>' Is_Equal_35 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6018 '<Symbol Large>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Large); 
--# expr-done 6018
Is_Equal_35 := Block_Var = Sym_41; 
--# expr-done 6019
   If_Result_17 := Is_Equal_35;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 6021 '<Eq>' Is_Equal_36 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6020 '<Symbol Safe_Large>' Sym_42 None
Sym_42 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Safe_Large); 
--# expr-done 6020
Is_Equal_36 := Block_Var = Sym_42; 
--# expr-done 6021
   If_Result_18 := Is_Equal_36;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 6023 '<Eq>' Is_Equal_37 /usr/src/packages/BUILD/ada/ast.py:13450
--# expr-start 6022 '<Symbol Delta>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Delta); 
--# expr-done 6022
Is_Equal_37 := Block_Var = Sym_43; 
--# expr-done 6023
   If_Result_19 := Is_Equal_37;
end if;
if If_Result_19 then
   --# expr-start 6024 '<FieldAccess .universal_real_equation>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:13454
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.universal_real_equation
Fld_25 := Libadalang.Implementation.Attribute_Ref_P_Universal_Real_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6024
   If_Result_74 := Fld_25;
else
--# expr-start 6026 '<Eq>' Is_Equal_38 /usr/src/packages/BUILD/ada/ast.py:13456
--# expr-start 6025 '<Symbol Img>' Sym_44 None
Sym_44 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Img); 
--# expr-done 6025
Is_Equal_38 := Block_Var = Sym_44; 
--# expr-done 6026
if Is_Equal_38 then
   --# expr-start 6029 '<FieldAccess .img_equation(...)>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:13457
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6028 '<FieldAccess .std_entity(...)>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:13457
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6027 '<Symbol String>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 6027
--# property-call-start AdaNode.std_entity
Fld_26 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_45);
--# end
--# expr-done 6028
--# property-call-start AttributeRef.img_equation
Fld_27 := Libadalang.Implementation.Attribute_Ref_P_Img_Equation (Node => Ent.Node, Str_Type => Fld_26, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6029
   If_Result_73 := Fld_27;
else
--# expr-start 6031 '<Eq>' Is_Equal_39 /usr/src/packages/BUILD/ada/ast.py:13459
--# expr-start 6030 '<Symbol Write>' Sym_46 None
Sym_46 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 6030
Is_Equal_39 := Block_Var = Sym_46; 
--# expr-done 6031
if Is_Equal_39 then
   If_Result_20 := True;
else
   --# expr-start 6033 '<Eq>' Is_Equal_40 /usr/src/packages/BUILD/ada/ast.py:13459
--# expr-start 6032 '<Symbol Read>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 6032
Is_Equal_40 := Block_Var = Sym_47; 
--# expr-done 6033
   If_Result_20 := Is_Equal_40;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 6035 '<Eq>' Is_Equal_41 /usr/src/packages/BUILD/ada/ast.py:13459
--# expr-start 6034 '<Symbol Output>' Sym_48 None
Sym_48 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 6034
Is_Equal_41 := Block_Var = Sym_48; 
--# expr-done 6035
   If_Result_21 := Is_Equal_41;
end if;
if If_Result_21 then
   --# expr-start 6036 '<FieldAccess .stream_attrs_equation(...)>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:13460
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.stream_attrs_equation
Fld_28 := Libadalang.Implementation.Attribute_Ref_P_Stream_Attrs_Equation (Node => Ent.Node, Return_Obj => False, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6036
   If_Result_72 := Fld_28;
else
--# expr-start 6038 '<Eq>' Is_Equal_42 /usr/src/packages/BUILD/ada/ast.py:13462
--# expr-start 6037 '<Symbol Input>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 6037
Is_Equal_42 := Block_Var = Sym_49; 
--# expr-done 6038
if Is_Equal_42 then
   --# expr-start 6039 '<FieldAccess .stream_attrs_equation(...)>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:13462
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.stream_attrs_equation
Fld_29 := Libadalang.Implementation.Attribute_Ref_P_Stream_Attrs_Equation (Node => Ent.Node, Return_Obj => True, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6039
   If_Result_71 := Fld_29;
else
--# expr-start 6041 '<Eq>' Is_Equal_43 /usr/src/packages/BUILD/ada/ast.py:13464
--# expr-start 6040 '<Symbol Tag>' Sym_50 None
Sym_50 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tag); 
--# expr-done 6040
Is_Equal_43 := Block_Var = Sym_50; 
--# expr-done 6041
if Is_Equal_43 then
   --# expr-start 6042 '<FieldAccess .tag_attr_equation>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:13464
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.tag_attr_equation
Fld_30 := Libadalang.Implementation.Attribute_Ref_P_Tag_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6042
   If_Result_70 := Fld_30;
else
--# expr-start 6044 '<Eq>' Is_Equal_44 /usr/src/packages/BUILD/ada/ast.py:13466
--# expr-start 6043 '<Symbol Result>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Result); 
--# expr-done 6043
Is_Equal_44 := Block_Var = Sym_51; 
--# expr-done 6044
if Is_Equal_44 then
   --# expr-start 6045 '<FieldAccess .result_attr_equation>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:13466
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.result_attr_equation
Fld_31 := Libadalang.Implementation.Attribute_Ref_P_Result_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6045
   If_Result_69 := Fld_31;
else
--# expr-start 6047 '<Eq>' Is_Equal_45 /usr/src/packages/BUILD/ada/ast.py:13468
--# expr-start 6046 '<Symbol Old>' Sym_52 None
Sym_52 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Old); 
--# expr-done 6046
Is_Equal_45 := Block_Var = Sym_52; 
--# expr-done 6047
if Is_Equal_45 then
   If_Result_22 := True;
else
   --# expr-start 6049 '<Eq>' Is_Equal_46 /usr/src/packages/BUILD/ada/ast.py:13468
--# expr-start 6048 '<Symbol Loop_Entry>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop_Entry); 
--# expr-done 6048
Is_Equal_46 := Block_Var = Sym_53; 
--# expr-done 6049
   If_Result_22 := Is_Equal_46;
end if;
if If_Result_22 then
   --# expr-start 6050 '<FieldAccess .bind_to_prefix_eq>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:13469
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.bind_to_prefix_eq
Fld_32 := Libadalang.Implementation.Attribute_Ref_P_Bind_To_Prefix_Eq (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6050
   If_Result_68 := Fld_32;
else
--# expr-start 6052 '<Eq>' Is_Equal_47 /usr/src/packages/BUILD/ada/ast.py:13471
--# expr-start 6051 '<Symbol Class>' Sym_54 None
Sym_54 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Class); 
--# expr-done 6051
Is_Equal_47 := Block_Var = Sym_54; 
--# expr-done 6052
if Is_Equal_47 then
   --# expr-start 6054 '<FieldAccess .sub_equation>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:13471
--# expr-start 6053 '<FieldAccess .prefix>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:13471
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6053
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 6054
   If_Result_67 := Fld_34;
else
--# expr-start 6056 '<Eq>' Is_Equal_48 /usr/src/packages/BUILD/ada/ast.py:13473
--# expr-start 6055 '<Symbol Valid>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Valid); 
--# expr-done 6055
Is_Equal_48 := Block_Var = Sym_55; 
--# expr-done 6056
if Is_Equal_48 then
   --# expr-start 6061 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13474
--# expr-start 6058 '<FieldAccess .sub_equation>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:13474
--# expr-start 6057 '<FieldAccess .prefix>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:13474
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6057
      if Fld_35.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_36 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_35.Node, Env => Env, Origin => Origin, E_Info => Fld_35.Info);
--# end
--# expr-done 6058
--# expr-start 6060 '<FieldAccess .bool_bind(...)>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:13475
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6059 '<FieldAccess .type_var>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:13475
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6059
--# property-call-start AdaNode.bool_bind
Fld_38 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_37, Origin => Origin);
--# end
--# expr-done 6060
And_Pred := Logic_And (Fld_36, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13474") else null)); 
--# expr-done 6061
   If_Result_66 := And_Pred;
else
--# expr-start 6063 '<Eq>' Is_Equal_49 /usr/src/packages/BUILD/ada/ast.py:13478
--# expr-start 6062 '<Symbol Model>' Sym_56 None
Sym_56 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model); 
--# expr-done 6062
Is_Equal_49 := Block_Var = Sym_56; 
--# expr-done 6063
if Is_Equal_49 then
   --# expr-start 6064 '<FieldAccess .model_attr_equation>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:13478
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.model_attr_equation
Fld_39 := Libadalang.Implementation.Attribute_Ref_P_Model_Attr_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6064
   If_Result_65 := Fld_39;
else
--# expr-start 6066 '<Eq>' Is_Equal_50 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6065 '<Symbol Width>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Width); 
--# expr-done 6065
Is_Equal_50 := Block_Var = Sym_57; 
--# expr-done 6066
if Is_Equal_50 then
   If_Result_23 := True;
else
   --# expr-start 6068 '<Eq>' Is_Equal_51 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6067 '<Symbol Component_Size>' Sym_58 None
Sym_58 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Component_Size); 
--# expr-done 6067
Is_Equal_51 := Block_Var = Sym_58; 
--# expr-done 6068
   If_Result_23 := Is_Equal_51;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 6070 '<Eq>' Is_Equal_52 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6069 '<Symbol Position>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Position); 
--# expr-done 6069
Is_Equal_52 := Block_Var = Sym_59; 
--# expr-done 6070
   If_Result_24 := Is_Equal_52;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 6072 '<Eq>' Is_Equal_53 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6071 '<Symbol Mantissa>' Sym_60 None
Sym_60 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mantissa); 
--# expr-done 6071
Is_Equal_53 := Block_Var = Sym_60; 
--# expr-done 6072
   If_Result_25 := Is_Equal_53;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 6074 '<Eq>' Is_Equal_54 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6073 '<Symbol Model_Mantissa>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Model_Mantissa); 
--# expr-done 6073
Is_Equal_54 := Block_Var = Sym_61; 
--# expr-done 6074
   If_Result_26 := Is_Equal_54;
end if;
if If_Result_26 then
   If_Result_27 := True;
else
   --# expr-start 6076 '<Eq>' Is_Equal_55 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6075 '<Symbol Machine_Mantissa>' Sym_62 None
Sym_62 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Mantissa); 
--# expr-done 6075
Is_Equal_55 := Block_Var = Sym_62; 
--# expr-done 6076
   If_Result_27 := Is_Equal_55;
end if;
if If_Result_27 then
   If_Result_28 := True;
else
   --# expr-start 6078 '<Eq>' Is_Equal_56 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6077 '<Symbol Fore>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Fore); 
--# expr-done 6077
Is_Equal_56 := Block_Var = Sym_63; 
--# expr-done 6078
   If_Result_28 := Is_Equal_56;
end if;
if If_Result_28 then
   If_Result_29 := True;
else
   --# expr-start 6080 '<Eq>' Is_Equal_57 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6079 '<Symbol Aft>' Sym_64 None
Sym_64 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aft); 
--# expr-done 6079
Is_Equal_57 := Block_Var = Sym_64; 
--# expr-done 6080
   If_Result_29 := Is_Equal_57;
end if;
if If_Result_29 then
   If_Result_30 := True;
else
   --# expr-start 6082 '<Eq>' Is_Equal_58 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6081 '<Symbol Digits>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Digits); 
--# expr-done 6081
Is_Equal_58 := Block_Var = Sym_65; 
--# expr-done 6082
   If_Result_30 := Is_Equal_58;
end if;
if If_Result_30 then
   If_Result_31 := True;
else
   --# expr-start 6084 '<Eq>' Is_Equal_59 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6083 '<Symbol Modulus>' Sym_66 None
Sym_66 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Modulus); 
--# expr-done 6083
Is_Equal_59 := Block_Var = Sym_66; 
--# expr-done 6084
   If_Result_31 := Is_Equal_59;
end if;
if If_Result_31 then
   If_Result_32 := True;
else
   --# expr-start 6086 '<Eq>' Is_Equal_60 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6085 '<Symbol Word_Size>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Word_Size); 
--# expr-done 6085
Is_Equal_60 := Block_Var = Sym_67; 
--# expr-done 6086
   If_Result_32 := Is_Equal_60;
end if;
if If_Result_32 then
   If_Result_33 := True;
else
   --# expr-start 6088 '<Eq>' Is_Equal_61 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6087 '<Symbol Max_Integer_Size>' Sym_68 None
Sym_68 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max_Integer_Size); 
--# expr-done 6087
Is_Equal_61 := Block_Var = Sym_68; 
--# expr-done 6088
   If_Result_33 := Is_Equal_61;
end if;
if If_Result_33 then
   If_Result_34 := True;
else
   --# expr-start 6090 '<Eq>' Is_Equal_62 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6089 '<Symbol Address_Size>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address_Size); 
--# expr-done 6089
Is_Equal_62 := Block_Var = Sym_69; 
--# expr-done 6090
   If_Result_34 := Is_Equal_62;
end if;
if If_Result_34 then
   If_Result_35 := True;
else
   --# expr-start 6092 '<Eq>' Is_Equal_63 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6091 '<Symbol Maximum_Alignment>' Sym_70 None
Sym_70 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Maximum_Alignment); 
--# expr-done 6091
Is_Equal_63 := Block_Var = Sym_70; 
--# expr-done 6092
   If_Result_35 := Is_Equal_63;
end if;
if If_Result_35 then
   If_Result_36 := True;
else
   --# expr-start 6094 '<Eq>' Is_Equal_64 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6093 '<Symbol System_Allocator_Alignment>' Sym_71 None
Sym_71 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System_Allocator_Alignment); 
--# expr-done 6093
Is_Equal_64 := Block_Var = Sym_71; 
--# expr-done 6094
   If_Result_36 := Is_Equal_64;
end if;
if If_Result_36 then
   If_Result_37 := True;
else
   --# expr-start 6096 '<Eq>' Is_Equal_65 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6095 '<Symbol Finalization_Size>' Sym_72 None
Sym_72 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Finalization_Size); 
--# expr-done 6095
Is_Equal_65 := Block_Var = Sym_72; 
--# expr-done 6096
   If_Result_37 := Is_Equal_65;
end if;
if If_Result_37 then
   If_Result_38 := True;
else
   --# expr-start 6098 '<Eq>' Is_Equal_66 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6097 '<Symbol Descriptor_Size>' Sym_73 None
Sym_73 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Descriptor_Size); 
--# expr-done 6097
Is_Equal_66 := Block_Var = Sym_73; 
--# expr-done 6098
   If_Result_38 := Is_Equal_66;
end if;
if If_Result_38 then
   If_Result_39 := True;
else
   --# expr-start 6100 '<Eq>' Is_Equal_67 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6099 '<Symbol Alignment>' Sym_74 None
Sym_74 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Alignment); 
--# expr-done 6099
Is_Equal_67 := Block_Var = Sym_74; 
--# expr-done 6100
   If_Result_39 := Is_Equal_67;
end if;
if If_Result_39 then
   If_Result_40 := True;
else
   --# expr-start 6102 '<Eq>' Is_Equal_68 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6101 '<Symbol First_Bit>' Sym_75 None
Sym_75 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First_Bit); 
--# expr-done 6101
Is_Equal_68 := Block_Var = Sym_75; 
--# expr-done 6102
   If_Result_40 := Is_Equal_68;
end if;
if If_Result_40 then
   If_Result_41 := True;
else
   --# expr-start 6104 '<Eq>' Is_Equal_69 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6103 '<Symbol Last_Bit>' Sym_76 None
Sym_76 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last_Bit); 
--# expr-done 6103
Is_Equal_69 := Block_Var = Sym_76; 
--# expr-done 6104
   If_Result_41 := Is_Equal_69;
end if;
if If_Result_41 then
   If_Result_42 := True;
else
   --# expr-start 6106 '<Eq>' Is_Equal_70 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6105 '<Symbol Default_Bit_Order>' Sym_77 None
Sym_77 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Default_Bit_Order); 
--# expr-done 6105
Is_Equal_70 := Block_Var = Sym_77; 
--# expr-done 6106
   If_Result_42 := Is_Equal_70;
end if;
if If_Result_42 then
   If_Result_43 := True;
else
   --# expr-start 6108 '<Eq>' Is_Equal_71 /usr/src/packages/BUILD/ada/ast.py:13480
--# expr-start 6107 '<Symbol Enum_Rep>' Sym_78 None
Sym_78 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Rep); 
--# expr-done 6107
Is_Equal_71 := Block_Var = Sym_78; 
--# expr-done 6108
   If_Result_43 := Is_Equal_71;
end if;
if If_Result_43 then
   --# expr-start 6113 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13488
--# expr-start 6110 '<FieldAccess .sub_equation>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:13488
--# expr-start 6109 '<FieldAccess .prefix>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:13488
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6109
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_41 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_40.Node, Env => Env, Origin => Origin, E_Info => Fld_40.Info);
--# end
--# expr-done 6110
--# expr-start 6112 '<FieldAccess .universal_int_bind(...)>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:13489
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6111 '<FieldAccess .type_var>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:13489
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6111
--# property-call-start AdaNode.universal_int_bind
Fld_43 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_42, Origin => Origin);
--# end
--# expr-done 6112
And_Pred_1 := Logic_And (Fld_41, Fld_43, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13488") else null)); 
--# expr-done 6113
   If_Result_64 := And_Pred_1;
else
--# expr-start 6115 '<Eq>' Is_Equal_72 /usr/src/packages/BUILD/ada/ast.py:13491
--# expr-start 6114 '<Symbol Target_Name>' Sym_79 None
Sym_79 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Target_Name); 
--# expr-done 6114
Is_Equal_72 := Block_Var = Sym_79; 
--# expr-done 6115
if Is_Equal_72 then
   --# expr-start 6119 '<FieldAccess .type_bind_val(...)>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:13492
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6116 '<FieldAccess .type_var>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:13492
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6116
--# expr-start 6118 '<FieldAccess .std_entity(...)>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:13492
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6117 '<Symbol String>' Sym_80 None
Sym_80 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 6117
--# property-call-start AdaNode.std_entity
Fld_45 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_80);
--# end
--# expr-done 6118
--# property-call-start AdaNode.type_bind_val
Fld_46 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_44, Right => Fld_45, Origin => Origin);
--# end
--# expr-done 6119
   If_Result_63 := Fld_46;
else
--# expr-start 6121 '<Eq>' Is_Equal_73 /usr/src/packages/BUILD/ada/ast.py:13494
--# expr-start 6120 '<Symbol Storage_Pool>' Sym_81 None
Sym_81 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Pool); 
--# expr-done 6120
Is_Equal_73 := Block_Var = Sym_81; 
--# expr-done 6121
if Is_Equal_73 then
   --# expr-start 6122 '<FieldAccess .storage_pool_equation>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:13494
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.storage_pool_equation
Fld_47 := Libadalang.Implementation.Attribute_Ref_P_Storage_Pool_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6122
   If_Result_62 := Fld_47;
else
--# expr-start 6124 '<Eq>' Is_Equal_74 /usr/src/packages/BUILD/ada/ast.py:13496
--# expr-start 6123 '<Symbol Type_Class>' Sym_82 None
Sym_82 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Class); 
--# expr-done 6123
Is_Equal_74 := Block_Var = Sym_82; 
--# expr-done 6124
if Is_Equal_74 then
   --# expr-start 6125 '<FieldAccess .type_class_equation>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:13496
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.type_class_equation
Fld_48 := Libadalang.Implementation.Attribute_Ref_P_Type_Class_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6125
   If_Result_61 := Fld_48;
else
--# expr-start 6127 '<Eq>' Is_Equal_75 /usr/src/packages/BUILD/ada/ast.py:13499
--# expr-start 6126 '<Symbol Callable>' Sym_83 None
Sym_83 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Callable); 
--# expr-done 6126
Is_Equal_75 := Block_Var = Sym_83; 
--# expr-done 6127
if Is_Equal_75 then
   If_Result_44 := True;
else
   --# expr-start 6129 '<Eq>' Is_Equal_76 /usr/src/packages/BUILD/ada/ast.py:13499
--# expr-start 6128 '<Symbol Terminated>' Sym_84 None
Sym_84 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Terminated); 
--# expr-done 6128
Is_Equal_76 := Block_Var = Sym_84; 
--# expr-done 6129
   If_Result_44 := Is_Equal_76;
end if;
if If_Result_44 then
   --# expr-start 6134 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13500
--# expr-start 6131 '<FieldAccess .sub_equation>' Fld_50 /usr/src/packages/BUILD/ada/ast.py:13500
--# expr-start 6130 '<FieldAccess .prefix>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:13500
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_49 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6130
      if Fld_49.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_50 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_49.Node, Env => Env, Origin => Origin, E_Info => Fld_49.Info);
--# end
--# expr-done 6131
--# expr-start 6133 '<FieldAccess .bool_bind(...)>' Fld_52 /usr/src/packages/BUILD/ada/ast.py:13500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6132 '<FieldAccess .type_var>' Fld_51 /usr/src/packages/BUILD/ada/ast.py:13500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6132
--# property-call-start AdaNode.bool_bind
Fld_52 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_51, Origin => Origin);
--# end
--# expr-done 6133
And_Pred_2 := Logic_And (Fld_50, Fld_52, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13500") else null)); 
--# expr-done 6134
   If_Result_60 := And_Pred_2;
else
--# expr-start 6136 '<Eq>' Is_Equal_77 /usr/src/packages/BUILD/ada/ast.py:13503
--# expr-start 6135 '<Symbol Count>' Sym_85 None
Sym_85 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Count); 
--# expr-done 6135
Is_Equal_77 := Block_Var = Sym_85; 
--# expr-done 6136
if Is_Equal_77 then
   --# expr-start 6141 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13504
--# expr-start 6138 '<FieldAccess .xref_no_overloading>' Fld_54 /usr/src/packages/BUILD/ada/ast.py:13504
--# expr-start 6137 '<FieldAccess .prefix>' Fld_53 /usr/src/packages/BUILD/ada/ast.py:13504
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6137
      if Fld_53.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_54 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_53.Node, Env => Env, Origin => Origin, E_Info => Fld_53.Info);
--# end
--# expr-done 6138
--# expr-start 6140 '<FieldAccess .universal_int_bind(...)>' Fld_56 /usr/src/packages/BUILD/ada/ast.py:13505
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6139 '<FieldAccess .type_var>' Fld_55 /usr/src/packages/BUILD/ada/ast.py:13505
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6139
--# property-call-start AdaNode.universal_int_bind
Fld_56 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_55, Origin => Origin);
--# end
--# expr-done 6140
And_Pred_3 := Logic_And (Fld_54, Fld_56, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13504") else null)); 
--# expr-done 6141
   If_Result_59 := And_Pred_3;
else
--# expr-start 6143 '<Eq>' Is_Equal_78 /usr/src/packages/BUILD/ada/ast.py:13507
--# expr-start 6142 '<Symbol Caller>' Sym_86 None
Sym_86 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Caller); 
--# expr-done 6142
Is_Equal_78 := Block_Var = Sym_86; 
--# expr-done 6143
if Is_Equal_78 then
   --# expr-start 6149 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:13508
--# expr-start 6145 '<FieldAccess .xref_no_overloading>' Fld_58 /usr/src/packages/BUILD/ada/ast.py:13508
--# expr-start 6144 '<FieldAccess .prefix>' Fld_57 /usr/src/packages/BUILD/ada/ast.py:13508
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_57 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6144
      if Fld_57.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_58 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_57.Node, Env => Env, Origin => Origin, E_Info => Fld_57.Info);
--# end
--# expr-done 6145
--# expr-start 6148 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13509
--# expr-start 6146 '<FieldAccess .type_var>' Fld_59 /usr/src/packages/BUILD/ada/ast.py:13509
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6146
Fld_59.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_59.all);
Eq_Node.Refs.Destroy (Fld_59.all);
--# expr-start 6147 '<FieldAccess .task_id_type>' Fld_60 /usr/src/packages/BUILD/ada/ast.py:13509
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.task_id_type
Fld_60 := Libadalang.Implementation.Ada_Node_P_Task_Id_Type (Node => Self);
--# end
--# expr-done 6147
      Cast_Result := Create_Internal_Entity
        (Node => Fld_60.Node,
         Info => Fld_60.Info);
Bind_Result := Bind_Default_Default.Create (Fld_59, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13509") else null)); 
--# expr-done 6148
And_Pred_4 := Logic_And (Fld_58, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13508") else null)); 
--# expr-done 6149
   If_Result_58 := And_Pred_4;
else
--# expr-start 6151 '<Eq>' Is_Equal_79 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6150 '<Symbol Ceiling>' Sym_87 None
Sym_87 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ceiling); 
--# expr-done 6150
Is_Equal_79 := Block_Var = Sym_87; 
--# expr-done 6151
if Is_Equal_79 then
   If_Result_45 := True;
else
   --# expr-start 6153 '<Eq>' Is_Equal_80 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6152 '<Symbol Floor>' Sym_88 None
Sym_88 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Floor); 
--# expr-done 6152
Is_Equal_80 := Block_Var = Sym_88; 
--# expr-done 6153
   If_Result_45 := Is_Equal_80;
end if;
if If_Result_45 then
   If_Result_46 := True;
else
   --# expr-start 6155 '<Eq>' Is_Equal_81 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6154 '<Symbol Rounding>' Sym_89 None
Sym_89 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rounding); 
--# expr-done 6154
Is_Equal_81 := Block_Var = Sym_89; 
--# expr-done 6155
   If_Result_46 := Is_Equal_81;
end if;
if If_Result_46 then
   If_Result_47 := True;
else
   --# expr-start 6157 '<Eq>' Is_Equal_82 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6156 '<Symbol Truncation>' Sym_90 None
Sym_90 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Truncation); 
--# expr-done 6156
Is_Equal_82 := Block_Var = Sym_90; 
--# expr-done 6157
   If_Result_47 := Is_Equal_82;
end if;
if If_Result_47 then
   If_Result_48 := True;
else
   --# expr-start 6159 '<Eq>' Is_Equal_83 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6158 '<Symbol Copy_Sign>' Sym_91 None
Sym_91 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Copy_Sign); 
--# expr-done 6158
Is_Equal_83 := Block_Var = Sym_91; 
--# expr-done 6159
   If_Result_48 := Is_Equal_83;
end if;
if If_Result_48 then
   If_Result_49 := True;
else
   --# expr-start 6161 '<Eq>' Is_Equal_84 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6160 '<Symbol Remainder>' Sym_92 None
Sym_92 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remainder); 
--# expr-done 6160
Is_Equal_84 := Block_Var = Sym_92; 
--# expr-done 6161
   If_Result_49 := Is_Equal_84;
end if;
if If_Result_49 then
   If_Result_50 := True;
else
   --# expr-start 6163 '<Eq>' Is_Equal_85 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6162 '<Symbol Adjacent>' Sym_93 None
Sym_93 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Adjacent); 
--# expr-done 6162
Is_Equal_85 := Block_Var = Sym_93; 
--# expr-done 6163
   If_Result_50 := Is_Equal_85;
end if;
if If_Result_50 then
   If_Result_51 := True;
else
   --# expr-start 6165 '<Eq>' Is_Equal_86 /usr/src/packages/BUILD/ada/ast.py:13513
--# expr-start 6164 '<Symbol Machine_Rounding>' Sym_94 None
Sym_94 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Rounding); 
--# expr-done 6164
Is_Equal_86 := Block_Var = Sym_94; 
--# expr-done 6165
   If_Result_51 := Is_Equal_86;
end if;
if If_Result_51 then
   --# expr-start 6166 '<FieldAccess .float_funcs_equation>' Fld_61 /usr/src/packages/BUILD/ada/ast.py:13516
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.float_funcs_equation
Fld_61 := Libadalang.Implementation.Attribute_Ref_P_Float_Funcs_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6166
   If_Result_57 := Fld_61;
else
--# expr-start 6168 '<Eq>' Is_Equal_87 /usr/src/packages/BUILD/ada/ast.py:13518
--# expr-start 6167 '<Symbol Exponent>' Sym_95 None
Sym_95 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exponent); 
--# expr-done 6167
Is_Equal_87 := Block_Var = Sym_95; 
--# expr-done 6168
if Is_Equal_87 then
   If_Result_52 := True;
else
   --# expr-start 6170 '<Eq>' Is_Equal_88 /usr/src/packages/BUILD/ada/ast.py:13518
--# expr-start 6169 '<Symbol Machine_Radix>' Sym_96 None
Sym_96 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Radix); 
--# expr-done 6169
Is_Equal_88 := Block_Var = Sym_96; 
--# expr-done 6170
   If_Result_52 := Is_Equal_88;
end if;
if If_Result_52 then
   --# expr-start 6171 '<FieldAccess .universal_int_equation>' Fld_62 /usr/src/packages/BUILD/ada/ast.py:13519
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.universal_int_equation
Fld_62 := Libadalang.Implementation.Attribute_Ref_P_Universal_Int_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6171
   If_Result_56 := Fld_62;
else
--# expr-start 6173 '<Eq>' Is_Equal_89 /usr/src/packages/BUILD/ada/ast.py:13521
--# expr-start 6172 '<Symbol Mod>' Sym_97 None
Sym_97 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_110); 
--# expr-done 6172
Is_Equal_89 := Block_Var = Sym_97; 
--# expr-done 6173
if Is_Equal_89 then
   --# expr-start 6174 '<FieldAccess .mod_equation>' Fld_63 /usr/src/packages/BUILD/ada/ast.py:13522
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.mod_equation
Fld_63 := Libadalang.Implementation.Attribute_Ref_P_Mod_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6174
   If_Result_55 := Fld_63;
else
--# expr-start 6176 '<Eq>' Is_Equal_90 /usr/src/packages/BUILD/ada/ast.py:13524
--# expr-start 6175 '<Symbol Asm_Input>' Sym_98 None
Sym_98 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 6175
Is_Equal_90 := Block_Var = Sym_98; 
--# expr-done 6176
if Is_Equal_90 then
   If_Result_53 := True;
else
   --# expr-start 6178 '<Eq>' Is_Equal_91 /usr/src/packages/BUILD/ada/ast.py:13524
--# expr-start 6177 '<Symbol Asm_Output>' Sym_99 None
Sym_99 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output); 
--# expr-done 6177
Is_Equal_91 := Block_Var = Sym_99; 
--# expr-done 6178
   If_Result_53 := Is_Equal_91;
end if;
if If_Result_53 then
   --# expr-start 6179 '<FieldAccess .inline_asm_equation>' Fld_64 /usr/src/packages/BUILD/ada/ast.py:13525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.inline_asm_equation
Fld_64 := Libadalang.Implementation.Attribute_Ref_P_Inline_Asm_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6179
   If_Result_54 := Fld_64;
else
   If_Result_54 := raise Property_Error with "Unhandled attribute";
end if;
      Inc_Ref (If_Result_54);
   If_Result_55 := If_Result_54;
end if;
      Inc_Ref (If_Result_55);
   If_Result_56 := If_Result_55;
end if;
      Inc_Ref (If_Result_56);
   If_Result_57 := If_Result_56;
end if;
      Inc_Ref (If_Result_57);
   If_Result_58 := If_Result_57;
end if;
      Inc_Ref (If_Result_58);
   If_Result_59 := If_Result_58;
end if;
      Inc_Ref (If_Result_59);
   If_Result_60 := If_Result_59;
end if;
      Inc_Ref (If_Result_60);
   If_Result_61 := If_Result_60;
end if;
      Inc_Ref (If_Result_61);
   If_Result_62 := If_Result_61;
end if;
      Inc_Ref (If_Result_62);
   If_Result_63 := If_Result_62;
end if;
      Inc_Ref (If_Result_63);
   If_Result_64 := If_Result_63;
end if;
      Inc_Ref (If_Result_64);
   If_Result_65 := If_Result_64;
end if;
      Inc_Ref (If_Result_65);
   If_Result_66 := If_Result_65;
end if;
      Inc_Ref (If_Result_66);
   If_Result_67 := If_Result_66;
end if;
      Inc_Ref (If_Result_67);
   If_Result_68 := If_Result_67;
end if;
      Inc_Ref (If_Result_68);
   If_Result_69 := If_Result_68;
end if;
      Inc_Ref (If_Result_69);
   If_Result_70 := If_Result_69;
end if;
      Inc_Ref (If_Result_70);
   If_Result_71 := If_Result_70;
end if;
      Inc_Ref (If_Result_71);
   If_Result_72 := If_Result_71;
end if;
      Inc_Ref (If_Result_72);
   If_Result_73 := If_Result_72;
end if;
      Inc_Ref (If_Result_73);
   If_Result_74 := If_Result_73;
end if;
      Inc_Ref (If_Result_74);
   If_Result_75 := If_Result_74;
end if;
      Inc_Ref (If_Result_75);
   If_Result_76 := If_Result_75;
end if;
      Inc_Ref (If_Result_76);
   If_Result_77 := If_Result_76;
end if;
      Inc_Ref (If_Result_77);
   If_Result_78 := If_Result_77;
end if;
      Inc_Ref (If_Result_78);
   If_Result_79 := If_Result_78;
end if;
      Inc_Ref (If_Result_79);
   If_Result_80 := If_Result_79;
end if;
      Inc_Ref (If_Result_80);
   If_Result_81 := If_Result_80;
end if;
      Inc_Ref (If_Result_81);
   If_Result_82 := If_Result_81;
end if;
      Inc_Ref (If_Result_82);
   If_Result_83 := If_Result_82;
end if;
      Inc_Ref (If_Result_83);
   If_Result_84 := If_Result_83;
end if;
      Inc_Ref (If_Result_84);
   If_Result_85 := If_Result_84;
end if;
      Inc_Ref (If_Result_85);
   If_Result_86 := If_Result_85;
end if;
      Inc_Ref (If_Result_86);
   If_Result_87 := If_Result_86;
end if;
      Inc_Ref (If_Result_87);
   If_Result_88 := If_Result_87;
end if;
      Inc_Ref (If_Result_88);
   If_Result_89 := If_Result_88;
end if;
      Inc_Ref (If_Result_89);
   If_Result_90 := If_Result_89;
end if;
      Inc_Ref (If_Result_90);
   If_Result_91 := If_Result_90;
end if;
      Inc_Ref (If_Result_91);
--# expr-done 6180
Let_Result := If_Result_91; Inc_Ref (Let_Result);
--# end
--# expr-done 6181
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1032;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1032;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Xref_Equation;
--# end
--# property-start AttributeRef.float_funcs_equation /usr/src/packages/BUILD/ada/ast.py:13531
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Float_Funcs_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1033;
            procedure Finalizer_Scope_2427;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Item_198 : Internal_Entity_Basic_Assoc;
Bare_Item_198 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1033 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1033;
            procedure Finalizer_Scope_2427 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2427;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6199 '<Block>' Let_Result None
--# scope-start
--# expr-start 6183 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13536
--# expr-start 6182 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6182
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6183
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6198 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13539
--# expr-start 6188 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13539
--# expr-start 6185 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13539
--# expr-start 6184 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13539
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6184
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6185
--# expr-start 6187 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13540
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6186 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13540
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6186
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6187
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13539") else null)); 
--# expr-done 6188
--# expr-start 6197 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:13541
--# expr-start 6196 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13541
--# expr-start 6189 '<FieldAccess .args_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13541
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6189
Fld_7 := Fld_6.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_198 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_198 := Untyped_Item_198; 
Is_Null := Bare_Item_198 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_198); 
   If_Result := As_Entity;
end if;
                  Item_198 := If_Result; 
   --# scope-start
               --# bind arg Item_198
      --# expr-start 6195 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13543
--# expr-start 6191 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13543
--# expr-start 6190 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13543
      if Item_198.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_198.Node, E_Info => Item_198.Info);
--# end
--# expr-done 6190
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 6191
--# expr-start 6194 '<FieldAccess .type_bind_val(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13544
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6193 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13544
--# expr-start 6192 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13544
      if Item_198.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_198.Node, E_Info => Item_198.Info);
--# end
--# expr-done 6192
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6193
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6194
And_Pred_1 := Logic_And (Fld_10, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13543") else null)); 
--# expr-done 6195
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2427;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6196
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13541") else null)); 
--# expr-done 6197
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13539") else null)); 
--# expr-done 6198
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6199
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1033;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1033;
               Finalizer_Scope_2427;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Float_Funcs_Equation;
--# end
--# property-start AttributeRef.mod_equation /usr/src/packages/BUILD/ada/ast.py:13549
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Mod_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1034;
            procedure Finalizer_Scope_2428;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Item_199 : Internal_Entity_Basic_Assoc;
Bare_Item_199 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1034 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1034;
            procedure Finalizer_Scope_2428 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2428;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6213 '<Block>' Let_Result None
--# scope-start
--# expr-start 6201 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13554
--# expr-start 6200 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13554
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6200
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6201
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6212 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13555
--# expr-start 6206 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13555
--# expr-start 6203 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13556
--# expr-start 6202 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13556
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6202
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6203
--# expr-start 6205 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13557
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6204 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13557
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6204
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6205
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13555") else null)); 
--# expr-done 6206
--# expr-start 6211 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:13558
--# expr-start 6210 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13558
--# expr-start 6207 '<FieldAccess .args_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13558
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6207
Fld_7 := Fld_6.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_199 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_199 := Untyped_Item_199; 
Is_Null := Bare_Item_199 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_199); 
   If_Result := As_Entity;
end if;
                  Item_199 := If_Result; 
   --# scope-start
               --# bind arg Item_199
      --# expr-start 6209 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13559
--# expr-start 6208 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13559
      if Item_199.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_199.Node, E_Info => Item_199.Info);
--# end
--# expr-done 6208
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 6209
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_10;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2428;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6210
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13558") else null)); 
--# expr-done 6211
And_Pred_1 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13555") else null)); 
--# expr-done 6212
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6213
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1034;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1034;
               Finalizer_Scope_2428;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Mod_Equation;
--# end
--# property-start AttributeRef.type_class_equation /usr/src/packages/BUILD/ada/ast.py:13564
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Type_Class_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1035;
            procedure Finalizer_Scope_2429;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1035 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1035;
            procedure Finalizer_Scope_2429 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2429;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6227 '<Block>' Let_Result None
--# scope-start
--# expr-start 6221 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13570
--# expr-start 6220 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Type_Class)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13570
--# expr-start 6217 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:13570
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6216 '<ArrayLiteral>' Array_Lit None
--# expr-start 6214 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 6214
--# expr-start 6215 '<Symbol Aux_DEC>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Aux_Dec); 
--# expr-done 6215
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 6216
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6217
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6218 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13570
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6218
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2429;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6219 '<Symbol Type_Class>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Type_Class); 
--# expr-done 6219
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6220
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6221
Block_Var := Cast_Result; 
--# bind typ Block_Var
--# expr-start 6226 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13576
--# expr-start 6223 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13576
--# expr-start 6222 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13576
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6222
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6223
--# expr-start 6225 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13577
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6224 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13577
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6224
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6225
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13576") else null)); 
--# expr-done 6226
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6227
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1035;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1035;
               Finalizer_Scope_2429;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Type_Class_Equation;
--# end
--# property-start AttributeRef.storage_pool_equation /usr/src/packages/BUILD/ada/ast.py:13580
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Storage_Pool_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1036;
            procedure Finalizer_Scope_2430;
      Block_Var : Internal_Entity_Classwide_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Classwide_Type_Decl;
Fld_3 : Internal_Entity_Name;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Cast_Expr : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1036 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1036;
            procedure Finalizer_Scope_2430 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2430;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6242 '<Block>' Let_Result None
--# scope-start
--# expr-start 6236 '<FieldAccess .classwide_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13585
--# expr-start 6235 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13585
--# expr-start 6234 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Storage_Pool)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13585
--# expr-start 6231 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:13585
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6230 '<ArrayLiteral>' Array_Lit None
--# expr-start 6228 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 6228
--# expr-start 6229 '<Symbol Storage_Pools>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Storage_Pools); 
--# expr-done 6229
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 6230
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6231
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6232 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13585
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6232
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2430;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6233 '<Symbol Root_Storage_Pool>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Storage_Pool); 
--# expr-done 6233
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6234
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6235
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 6236
Block_Var := Fld_2; 
--# bind typ Block_Var
--# expr-start 6241 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13591
--# expr-start 6238 '<FieldAccess .xref_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13591
--# expr-start 6237 '<FieldAccess .prefix>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13591
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6237
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_4 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6238
--# expr-start 6240 '<FieldAccess .type_bind_val(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13592
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6239 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13592
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6239
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6240
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13591") else null)); 
--# expr-done 6241
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6242
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1036;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1036;
               Finalizer_Scope_2430;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Storage_Pool_Equation;
--# end
--# property-start AttributeRef.model_attr_equation /usr/src/packages/BUILD/ada/ast.py:13595
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Model_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1037;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Unit;
Fld_3 : Bare_Basic_Decl;
Cast_Result : Bare_Package_Decl;
Fld_4 : Bare_Public_Part;
Fld_5 : Internal_Entity_Base_Type_Decl_Array_Access;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Map_Result : Internal_Entity_Array_Access;
Fld_6 : Logic_Var;
Domain_Equation : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Name;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1037 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Domain_Equation);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1037;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6260 '<Block>' Let_Result None
--# scope-start
--# expr-start 6259 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13597
--# expr-start 6254 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13597
--# expr-start 6244 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13597
--# expr-start 6243 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13597
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6243
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6244
--# expr-start 6253 '<domain(<FieldAccess .type_var>, <map>)>' Domain_Equation /usr/src/packages/BUILD/ada/ast.py:13598
--# expr-start 6251 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13599
--# expr-start 6249 '<FieldAccess .types_with_models>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13599
--# expr-start 6248 '<FieldAccess .public_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13599
--# expr-start 6247 '<Cast to PackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13599
--# expr-start 6246 '<FieldAccess .top_level_decl(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13599
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6245 '<FieldAccess .unit>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13599
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Unit (Node => Self);
--# expr-done 6245
--# property-call-start AdaNode.top_level_decl
Fld_3 := Libadalang.Implementation.Ada_Node_P_Top_Level_Decl (Node => Self, Unit => Fld_2);
--# end
--# expr-done 6246
   if Fld_3 = null
      or else Fld_3.Kind in Ada_Package_Decl_Range
   then
      Cast_Result := Fld_3;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6247
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Cast_Result.Base_Package_Decl_F_Public_Part;
--# expr-done 6248
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DeclarativePart.types_with_models
Fld_5 := Libadalang.Implementation.Declarative_Part_P_Types_With_Models (Node => Fld_4);
--# end
--# expr-done 6249
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_5;
      begin
         for Item_200 of
               Collection.Items
         loop
   --# scope-start
               --# bind t Item_200
      --# expr-start 6250 '<Cast to AdaNode>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:13601
Cast_Expr := Item_200; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# expr-done 6250
         declare
            Item_To_Append : constant Internal_Entity :=
               Cast_Result_1;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6251
--# expr-start 6252 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13598
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6252
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
declare
   Dom : Internal_Entity_Array_Access := Map_Result;
   A   : Bind_Default_Default.Impl.Unify_Left.R_Type_Array (1 .. Length (Dom));
begin
   for J in 0 .. Length (Dom) - 1 loop
      declare
         Item : constant Internal_Entity := Get (Dom, J);
      begin
         A (J + 1) := (Node => Item.Node, Info => Item.Info);
      end;
   end loop;
   Domain_Equation := Bind_Default_Default.Impl.Member
     (Fld_6, A);
end;
--# expr-done 6253
And_Pred := Logic_And (Fld_1, Domain_Equation, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13597") else null)); 
--# expr-done 6254
--# expr-start 6258 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13605
--# expr-start 6255 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6255
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
--# expr-start 6257 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13605
--# expr-start 6256 '<FieldAccess .prefix>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6256
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 6257
Fld_9.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_9.all);
Eq_Node.Refs.Destroy (Fld_9.all);
Bind_Result := Bind_502_567.Create (Fld_7, Fld_9, Logic_Converter_502'(null record), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13605") else null)); 
--# expr-done 6258
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13597") else null)); 
--# expr-done 6259
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6260
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1037;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1037;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Model_Attr_Equation;
--# end
--# property-start AttributeRef.bind_to_prefix_eq /usr/src/packages/BUILD/ada/ast.py:13611
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Bind_To_Prefix_Eq
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1038;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1038 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1038;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6268 '<Block>' Let_Result None
--# scope-start
--# expr-start 6267 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13612
--# expr-start 6262 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13613
--# expr-start 6261 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13613
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6261
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6262
--# expr-start 6266 '<FieldAccess .type_bind_var(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13614
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6263 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13614
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6263
--# expr-start 6265 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13614
--# expr-start 6264 '<FieldAccess .prefix>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13614
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6264
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_3);
--# expr-done 6265
--# property-call-start AdaNode.type_bind_var
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_2, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 6266
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13612") else null)); 
--# expr-done 6267
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6268
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1038;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1038;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Bind_To_Prefix_Eq;
--# end
--# property-start AttributeRef.result_attr_equation /usr/src/packages/BUILD/ada/ast.py:13618
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Result_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1039;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Subp_Spec;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_2 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1039 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1039;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6283 '<Block>' Let_Result None
--# scope-start
--# expr-start 6274 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13619
--# expr-start 6273 '<as_entity(<at(<filter>, 0)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13619
--# expr-start 6272 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:13619
--# expr-start 6271 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13619
--# expr-start 6269 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:13619
Node_Parents := Parents (Self, True); 
--# expr-done 6269
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_201 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_201
               --# expr-start 6270 '<IsA BasicSubpDecl, BaseSubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:13620
Is_A := Item_201 /= null 
and then Item_201.Kind in Ada_Basic_Subp_Decl | Ada_Base_Subp_Body; 
--# expr-done 6270
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_201;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6271
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 6272
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => E_Info, Node => Get_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 6273
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 6274
Block_Var := Cast_Result; 
--# bind containing_subp Block_Var
--# expr-start 6275 '<FieldAccess .subp_spec_or_null>' Fld /usr/src/packages/BUILD/ada/ast.py:13623
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 6275
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr
--# expr-start 6276 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13624
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6276
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var; 
--# bind returns Block_Var_1
--# expr-start 6282 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13627
--# expr-start 6278 '<FieldAccess .type_bind_val(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13628
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6277 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13628
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6277
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6278
--# expr-start 6281 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13629
--# expr-start 6280 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13629
--# expr-start 6279 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13629
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6279
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4.Node);
--# end
--# expr-done 6280
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Bind_Default_Default.Create (Fld_5, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13629") else null)); 
--# expr-done 6281
And_Pred := Logic_And (Fld_3, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13627") else null)); 
--# expr-done 6282
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6283
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1039;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1039;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Result_Attr_Equation;
--# end
--# property-start AttributeRef.tag_attr_equation /usr/src/packages/BUILD/ada/ast.py:13633
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Tag_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1040;
            procedure Finalizer_Scope_2434;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1040 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1040;
            procedure Finalizer_Scope_2434 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2434;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6297 '<Block>' Let_Result None
--# scope-start
--# expr-start 6291 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13635
--# expr-start 6290 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Tag)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13635
--# expr-start 6287 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:13635
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6286 '<ArrayLiteral>' Array_Lit None
--# expr-start 6284 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 6284
--# expr-start 6285 '<Symbol Tags>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tags); 
--# expr-done 6285
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 6286
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6287
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6288 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13635
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6288
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2434;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6289 '<Symbol Tag>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Tag); 
--# expr-done 6289
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6290
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6291
Block_Var := Cast_Result; 
--# bind tag_type Block_Var
--# expr-start 6296 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13643
--# expr-start 6293 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13643
--# expr-start 6292 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13643
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6292
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6293
--# expr-start 6295 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13646
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6294 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13646
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6294
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6295
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13643") else null)); 
--# expr-done 6296
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6297
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1040;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1040;
               Finalizer_Scope_2434;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Tag_Attr_Equation;
--# end
--# property-start AttributeRef.stream_attrs_equation /usr/src/packages/BUILD/ada/ast.py:13650
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Stream_Attrs_Equation
  (Node : Bare_Attribute_Ref
      ; Return_Obj : Boolean
         := False
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind return_obj Return_Obj
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1041;
            procedure Finalizer_Scope_2435;
            procedure Finalizer_Scope_2437;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Expr;
Block_Var_3 : Internal_Entity_Expr;
Var_Expr : Bare_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Assoc;
Var_Expr_2 : Internal_Entity_Attribute_Def_Clause;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Classwide_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity_Assoc_List;
Fld_6 : Bare_Assoc_List;
Fld_7 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Fld_11 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_12 : Internal_Entity_Expr;
Scope_Result_1 : Internal_Entity_Expr;
Result_Var_1 : Internal_Entity_Expr;
Fld_13 : Internal_Entity_Name;
Fld_14 : Logic_Equation;
Fld_15 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_18 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_20 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_22 : Logic_Equation;
Fld_23 : Logic_Equation;
And_Pred_2 : Logic_Equation;
If_Result_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Bound_Imprecise_Fallback : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Fld_26 : Internal_Entity_Attribute_Def_Clause;
Fld_27 : Internal_Entity_Identifier;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Expr;
Cast_Result_5 : Internal_Entity_Name;
Fld_30 : Internal_Entity_Basic_Decl;
Cast_Result_6 : Internal_Entity;
Bind_Result : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_2 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1041 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_18);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1041;
            procedure Finalizer_Scope_2435 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2435;
            procedure Finalizer_Scope_2437 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_2437;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6341 '<Block>' Let_Result None
--# scope-start
--# expr-start 6299 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13651
--# expr-start 6298 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13651
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6298
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6299
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6309 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:13654
--# expr-start 6308 '<FieldAccess .classwide_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13654
--# expr-start 6307 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13654
--# expr-start 6306 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Root_Stream_Type)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13654
--# expr-start 6303 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13654
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6302 '<ArrayLiteral>' Array_Lit None
--# expr-start 6300 '<Symbol Ada>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ada); 
--# expr-done 6300
--# expr-start 6301 '<Symbol Streams>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Streams); 
--# expr-done 6301
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym, 2 => Sym_1)); 
--# expr-done 6302
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6303
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6304 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13654
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6304
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2435;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6305 '<Symbol Root_Stream_Type>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Root_Stream_Type); 
--# expr-done 6305
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6306
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6307
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.classwide_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Classwide_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 6308
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# expr-done 6309
Block_Var_1 := Cast_Result_1; 
--# bind root_stream_type Block_Var_1
--# expr-start 6311 '<FieldAccess .expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13660
--# expr-start 6310 '<FieldAccess .args_list>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13660
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_5 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6310
Fld_6 := Fld_5.Node;
Get_Result := Get (Fld_6, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_7 := Fld_5.Info;
As_Entity := (Info => Fld_7, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6311
Block_Var_2 := Fld_8; 
--# bind stream_arg Block_Var_2
--# expr-start 6312 '<FieldAccess .args_list>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13661
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_9 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6312
Fld_10 := Fld_9.Node;
Get_Result_1 := Get (Fld_10, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_11 := Fld_9.Info;
As_Entity_1 := (Info => Fld_11, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
Var_Expr_1 := If_Result_1;
if Var_Expr_1 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 6313 '<FieldAccess .expr>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13661
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_12 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6313
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Expr;
end if;
Block_Var_3 := Result_Var_1; 
--# bind obj_arg Block_Var_3
--# expr-start 6340 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:13664
--# expr-start 6329 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13664
--# expr-start 6321 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13664
--# expr-start 6317 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13664
--# expr-start 6315 '<FieldAccess .sub_equation>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13664
--# expr-start 6314 '<FieldAccess .prefix>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13664
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6314
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_14 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 6315
--# expr-start 6316 '<FieldAccess .sub_equation>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13665
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_15 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_2.Node, Env => Env, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6316
And_Pred := Logic_And (Fld_14, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13664") else null)); 
--# expr-done 6317
--# expr-start 6320 '<FieldAccess .type_bind_val(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13666
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6318 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13666
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_2.Node);
--# expr-done 6318
--# expr-start 6319 '<FieldAccess .anonymous_access_type>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13667
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.anonymous_access_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Anonymous_Access_Type (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6319
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
--# property-call-start AdaNode.type_bind_val
Fld_18 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6320
And_Pred_1 := Logic_And (And_Pred, Fld_18, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13664") else null)); 
--# expr-done 6321
--# expr-start 6328 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:13668
if Return_Obj then
   --# expr-start 6323 '<FieldAccess .type_bind_val(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13670
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6322 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13670
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6322
Cast_Expr := Block_Var; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 6323
   If_Result_2 := Fld_20;
else
   --# expr-start 6327 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13671
--# expr-start 6325 '<FieldAccess .type_bind_val(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13671
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6324 '<FieldAccess .type_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13671
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_3.Node);
--# expr-done 6324
Cast_Expr_1 := Block_Var; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 6325
--# expr-start 6326 '<FieldAccess .sub_equation>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:13672
      if Block_Var_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_23 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_3.Node, Env => Env, Origin => Origin, E_Info => Block_Var_3.Info);
--# end
--# expr-done 6326
And_Pred_2 := Logic_And (Fld_22, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13671") else null)); 
--# expr-done 6327
   If_Result_2 := And_Pred_2;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 6328
And_Pred_3 := Logic_And (And_Pred_1, If_Result_2, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13664") else null)); 
--# expr-done 6329
--# expr-start 6339 '<bind(<DynamicVariable imprecise_fallback>, False, <Then <FieldAccess .get_representation_clause(...)>: <Var x> <Bind>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:13678
Bound_Imprecise_Fallback := False; 
--# expr-start 6332 '<FieldAccess .get_representation_clause(...)>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:13680
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6331 '<FieldAccess .name_symbol>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:13681
--# expr-start 6330 '<FieldAccess .attribute>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:13681
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 6330
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 6331
--# property-call-start BasicDecl.get_representation_clause
Fld_26 := Libadalang.Implementation.Basic_Decl_P_Get_Representation_Clause (Node => Block_Var.Node, Name => Fld_25, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 6332
Var_Expr_2 := Fld_26;
if Var_Expr_2 /= No_Entity_Attribute_Def_Clause then
   --# scope-start
   --# bind x Var_Expr_2
--# expr-start 6338 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13683
--# expr-start 6334 '<FieldAccess .ref_var>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:13684
--# expr-start 6333 '<FieldAccess .attribute>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:13684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 6333
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_28 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_27.Node);
--# end
--# expr-done 6334
Fld_28.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_28.all);
Eq_Node.Refs.Destroy (Fld_28.all);
--# expr-start 6337 '<FieldAccess .referenced_decl>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:13685
--# expr-start 6336 '<Cast to Name>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:13685
--# expr-start 6335 '<FieldAccess .expr>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:13685
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Create_Internal_Entity_Expr (Node => Var_Expr_2.Node.Attribute_Def_Clause_F_Expr, Info => Var_Expr_2.Info);
--# expr-done 6335
   if Fld_29.Node = null
      or else Fld_29.Node.Kind in Ada_Name
   then
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6336
      if Cast_Result_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_30 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Cast_Result_5.Node, Imprecise_Fallback => Bound_Imprecise_Fallback, E_Info => Cast_Result_5.Info);
--# end
--# expr-done 6337
      Cast_Result_6 := Create_Internal_Entity
        (Node => Fld_30.Node,
         Info => Fld_30.Info);
Bind_Result := Bind_Default_Default.Create (Fld_28, Cast_Result_6, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13683") else null)); 
--# expr-done 6338
Scope_Result_2 := Bind_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2437;
   Result_Var_2 := Scope_Result_2;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13687") else null)); 
   Result_Var_2 := Logic_True;
end if;
      Inc_Ref (Result_Var_2);
Dyn_Var_Bind_Result := Result_Var_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 6339
And_Pred_4 := Logic_And (And_Pred_3, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13664") else null)); 
--# expr-done 6340
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 6341
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1041;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1041;
               Finalizer_Scope_2435;
               Finalizer_Scope_2437;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Stream_Attrs_Equation;
--# end
--# property-start AttributeRef.address_equation /usr/src/packages/BUILD/ada/ast.py:13693
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Address_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1042;
            procedure Finalizer_Scope_2438;
      Block_Var : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld : Bare_Basic_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Name;
Fld_5 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1042 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1042;
            procedure Finalizer_Scope_2438 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2438;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6361 '<Block>' Let_Result None
--# scope-start
--# expr-start 6348 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13695
--# expr-start 6347 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, Address)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13695
--# expr-start 6344 '<FieldAccess .get_unit_root_decl(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:13695
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6343 '<ArrayLiteral>' Array_Lit None
--# expr-start 6342 '<Symbol System>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 6342
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym)); 
--# expr-done 6343
--# property-call-start AdaNode.get_unit_root_decl
Fld := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6344
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6345 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13695
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6345
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2438;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
--# expr-start 6346 '<Symbol Address>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Address); 
--# expr-done 6346
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6347
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6348
Block_Var := Cast_Result; 
--# bind address_type Block_Var
--# expr-start 6360 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13703
--# expr-start 6357 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:13703
--# expr-start 6354 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13704
--# expr-start 6350 '<FieldAccess .xref_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13704
--# expr-start 6349 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13704
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6349
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6350
--# expr-start 6353 '<Predicate on BasicDecl.is_subprogram>' Pred /usr/src/packages/BUILD/ada/ast.py:13705
--# expr-start 6352 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13705
--# expr-start 6351 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13705
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6351
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_4);
--# end
--# expr-done 6352
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Pred := Basic_Decl_P_Is_Subprogram_0_Pred.Create (Fld_5, Create_Basic_Decl_P_Is_Subprogram_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Basic_Decl.Basic_Decl_P_Is_Subprogram")            else null))); 
--# expr-done 6353
And_Pred := Logic_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13704") else null)); 
--# expr-done 6354
--# expr-start 6356 '<FieldAccess .sub_equation>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13707
--# expr-start 6355 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13707
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6355
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6356
Or_Pred := Logic_Or (And_Pred, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13703") else null)); 
--# expr-done 6357
--# expr-start 6359 '<FieldAccess .type_bind_val(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13708
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6358 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13708
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6358
Cast_Expr := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6359
And_Pred_1 := Logic_And (Or_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13703") else null)); 
--# expr-done 6360
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6361
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1042;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1042;
               Finalizer_Scope_2438;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Address_Equation;
--# end
--# property-start AttributeRef.identity_equation /usr/src/packages/BUILD/ada/ast.py:13711
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Identity_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1043;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1043 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1043;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6369 '<Block>' Let_Result None
--# scope-start
--# expr-start 6368 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13714
--# expr-start 6363 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13714
--# expr-start 6362 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13714
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6362
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6363
--# expr-start 6367 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13717
--# expr-start 6365 '<FieldAccess .ref_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13717
--# expr-start 6364 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13717
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6364
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6365
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 6366 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13717
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6366
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Bind_Result := Bind_170_567.Create (Fld_3, Fld_4, Logic_Converter_170'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13717") else null)); 
--# expr-done 6367
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13714") else null)); 
--# expr-done 6368
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6369
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1043;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1043;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Identity_Equation;
--# end
--# property-start AttributeRef.universal_real_equation /usr/src/packages/BUILD/ada/ast.py:13722
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Real_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1044;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1044 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1044;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6375 '<Block>' Let_Result None
--# scope-start
--# expr-start 6374 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13724
--# expr-start 6371 '<FieldAccess .universal_real_bind(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13724
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6370 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:13724
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6370
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 6371
--# expr-start 6373 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13725
--# expr-start 6372 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13725
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6372
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6373
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13724") else null)); 
--# expr-done 6374
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6375
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1044;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1044;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Universal_Real_Equation;
--# end
--# property-start AttributeRef.universal_int_equation /usr/src/packages/BUILD/ada/ast.py:13729
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Universal_Int_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1045;
            procedure Finalizer_Scope_2439;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_13 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Item_202 : Internal_Entity_Basic_Assoc;
Bare_Item_202 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1045 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1045;
            procedure Finalizer_Scope_2439 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2439;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6393 '<Block>' Let_Result None
--# scope-start
--# expr-start 6377 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13730
--# expr-start 6376 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13730
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6376
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6377
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6392 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13733
--# expr-start 6382 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13733
--# expr-start 6379 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13733
--# expr-start 6378 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13733
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6378
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 6379
--# expr-start 6381 '<FieldAccess .universal_int_bind(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13734
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6380 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13734
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6380
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 6381
And_Pred := Logic_And (Fld_3, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13733") else null)); 
--# expr-done 6382
--# expr-start 6391 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:13735
--# expr-start 6390 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:13735
--# expr-start 6383 '<FieldAccess .args_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13735
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6383
Fld_7 := Fld_6.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_7 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_7;
      begin
         for Untyped_Item_202 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_202 := Untyped_Item_202; 
Is_Null := Bare_Item_202 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity := (Info => Fld_8, Node => Bare_Item_202); 
   If_Result := As_Entity;
end if;
                  Item_202 := If_Result; 
   --# scope-start
               --# bind arg Item_202
      --# expr-start 6389 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13737
--# expr-start 6385 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13737
--# expr-start 6384 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13737
      if Item_202.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_202.Node, E_Info => Item_202.Info);
--# end
--# expr-done 6384
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 6385
--# expr-start 6388 '<FieldAccess .type_bind_val(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13738
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6387 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13738
--# expr-start 6386 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13738
      if Item_202.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_202.Node, E_Info => Item_202.Info);
--# end
--# expr-done 6386
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6387
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6388
And_Pred_1 := Logic_And (Fld_10, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13737") else null)); 
--# expr-done 6389
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2439;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6390
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13735") else null)); 
--# expr-done 6391
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13733") else null)); 
--# expr-done 6392
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6393
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1045;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1045;
               Finalizer_Scope_2439;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Universal_Int_Equation;
--# end
--# property-start AttributeRef.succpred_xref_equation /usr/src/packages/BUILD/ada/ast.py:13743
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Succpred_Xref_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1046;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_8 : Logic_Equation;
Fld_9 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_10 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_12 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1046 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_13);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1046;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6409 '<Block>' Let_Result None
--# scope-start
--# expr-start 6395 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13744
--# expr-start 6394 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13744
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6394
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6395
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6397 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13745
--# expr-start 6396 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13745
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6396
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6397
Block_Var_1 := Fld_5; 
--# bind arg Block_Var_1
--# expr-start 6408 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13748
--# expr-start 6406 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13748
--# expr-start 6403 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13748
--# expr-start 6400 '<FieldAccess .type_bind_val(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13748
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6399 '<FieldAccess .ref_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13748
--# expr-start 6398 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13748
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6398
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 6399
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6400
--# expr-start 6402 '<FieldAccess .type_bind_val(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13749
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6401 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13749
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6401
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6402
And_Pred := Logic_And (Fld_8, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13748") else null)); 
--# expr-done 6403
--# expr-start 6405 '<FieldAccess .type_bind_val(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13750
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6404 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13750
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6404
Cast_Expr_2 := Block_Var; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6405
And_Pred_1 := Logic_And (And_Pred, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13748") else null)); 
--# expr-done 6406
--# expr-start 6407 '<FieldAccess .sub_equation>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13751
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_13 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6407
And_Pred_2 := Logic_And (And_Pred_1, Fld_13, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13748") else null)); 
--# expr-done 6408
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6409
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1046;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1046;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Succpred_Xref_Equation;
--# end
--# property-start AttributeRef.minmax_equation /usr/src/packages/BUILD/ada/ast.py:13755
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Minmax_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1047;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Block_Var_2 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Assoc_List;
Fld_7 : Bare_Assoc_List;
Fld_8 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Logic_Equation;
Fld_11 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_12 : Bare_Name;
Fld_13 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Var;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Logic_Var;
Fld_19 : Logic_Var;
Fld_20 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_21 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_22 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1047 is
            begin
                     Dec_Ref (Fld_10);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1047;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6434 '<Block>' Let_Result None
--# scope-start
--# expr-start 6411 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13756
--# expr-start 6410 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13756
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6410
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6411
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6413 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13757
--# expr-start 6412 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13757
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6412
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6413
Block_Var_1 := Fld_5; 
--# bind left Block_Var_1
--# expr-start 6415 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13758
--# expr-start 6414 '<FieldAccess .args_list>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13758
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_6 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6414
Fld_7 := Fld_6.Node;
Get_Result_1 := Get (Fld_7, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_8 := Fld_6.Info;
As_Entity_1 := (Info => Fld_8, Node => Get_Result_1); 
   If_Result_1 := As_Entity_1;
end if;
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 6415
Block_Var_2 := Fld_9; 
--# bind right Block_Var_2
--# expr-start 6433 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:13761
--# expr-start 6430 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13761
--# expr-start 6426 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13761
--# expr-start 6422 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13761
--# expr-start 6418 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13761
--# expr-start 6416 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13761
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6416
--# expr-start 6417 '<FieldAccess .sub_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13761
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_2.Node, Env => Env, Origin => Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 6417
And_Pred := Logic_And (Fld_10, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13761") else null)); 
--# expr-done 6418
--# expr-start 6421 '<FieldAccess .type_bind_val(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13763
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6420 '<FieldAccess .ref_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13763
--# expr-start 6419 '<FieldAccess .prefix>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13763
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6419
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 6420
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_13, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6421
And_Pred_1 := Logic_And (And_Pred, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13761") else null)); 
--# expr-done 6422
--# expr-start 6425 '<FieldAccess .type_bind_var(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13764
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6423 '<FieldAccess .type_var>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13764
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6423
--# expr-start 6424 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13764
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_2.Node);
--# expr-done 6424
--# property-call-start AdaNode.type_bind_var
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_15, Right => Fld_16, Origin => Origin);
--# end
--# expr-done 6425
And_Pred_2 := Logic_And (And_Pred_1, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13761") else null)); 
--# expr-done 6426
--# expr-start 6429 '<FieldAccess .type_bind_var(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13765
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6427 '<FieldAccess .type_var>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13765
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6427
--# expr-start 6428 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13765
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6428
--# property-call-start AdaNode.type_bind_var
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_19, Origin => Origin);
--# end
--# expr-done 6429
And_Pred_3 := Logic_And (And_Pred_2, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13761") else null)); 
--# expr-done 6430
--# expr-start 6432 '<FieldAccess .type_bind_val(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13766
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6431 '<FieldAccess .type_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13766
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6431
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6432
And_Pred_4 := Logic_And (And_Pred_3, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13761") else null)); 
--# expr-done 6433
Let_Result := And_Pred_4; Inc_Ref (Let_Result);
--# end
--# expr-done 6434
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1047;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1047;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Minmax_Equation;
--# end
--# property-start AttributeRef.value_equation /usr/src/packages/BUILD/ada/ast.py:13770
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Value_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1048;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Logic_Equation;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1048 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1048;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6450 '<Block>' Let_Result None
--# scope-start
--# expr-start 6436 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13771
--# expr-start 6435 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13771
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6435
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6436
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6438 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13772
--# expr-start 6437 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13772
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6437
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6438
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6449 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13775
--# expr-start 6446 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13775
--# expr-start 6443 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13775
--# expr-start 6439 '<FieldAccess .sub_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13775
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6439
--# expr-start 6442 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13778
--# expr-start 6441 '<FieldAccess .ref_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13778
--# expr-start 6440 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13778
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6440
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 6441
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_8, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13778") else null)); 
--# expr-done 6442
And_Pred := Logic_And (Fld_6, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13775") else null)); 
--# expr-done 6443
--# expr-start 6445 '<FieldAccess .type_bind_val(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13781
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6444 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13781
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6444
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_9, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6445
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13775") else null)); 
--# expr-done 6446
--# expr-start 6448 '<FieldAccess .type_bind_val(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13784
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6447 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13784
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6447
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_12 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_11, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6448
And_Pred_2 := Logic_And (And_Pred_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13775") else null)); 
--# expr-done 6449
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6450
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1048;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1048;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Value_Equation;
--# end
--# property-start AttributeRef.image_equation /usr/src/packages/BUILD/ada/ast.py:13788
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Image_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1049;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Assoc_List;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Equation;
Fld_11 : Bare_Name;
Fld_12 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_13 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_15 : Logic_Var;
Fld_16 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Is_Null_1 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1049 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_16);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1049;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6473 '<Block>' Let_Result None
--# scope-start
--# expr-start 6452 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13789
--# expr-start 6451 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13789
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6451
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6452
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6453 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13790
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6453
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr
--# expr-start 6454 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13790
Saved := Var_Expr; 
Fld_3 := Saved.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6454
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
Block_Var_1 := Result_Var; 
--# bind expr Block_Var_1
--# expr-start 6472 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:13792
--# expr-start 6455 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:13793
Is_Null_1 := Block_Var.Node = null; 
--# expr-done 6455
if Is_Null_1 then
   --# expr-start 6460 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13796
--# expr-start 6457 '<FieldAccess .sub_equation>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13796
--# expr-start 6456 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13796
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6456
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6457
--# expr-start 6459 '<FieldAccess .type_bind_val(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13797
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6458 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13797
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6458
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6459
And_Pred := Logic_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13796") else null)); 
--# expr-done 6460
   If_Result_1 := And_Pred;
else
   --# expr-start 6471 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13799
--# expr-start 6468 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13799
--# expr-start 6465 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13799
--# expr-start 6461 '<FieldAccess .sub_equation>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13799
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_10 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6461
--# expr-start 6464 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13801
--# expr-start 6463 '<FieldAccess .ref_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13801
--# expr-start 6462 '<FieldAccess .prefix>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13801
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6462
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_11);
--# end
--# expr-done 6463
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_12, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13801") else null)); 
--# expr-done 6464
And_Pred_1 := Logic_And (Fld_10, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13799") else null)); 
--# expr-done 6465
--# expr-start 6467 '<FieldAccess .type_bind_val(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13803
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6466 '<FieldAccess .type_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13803
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6466
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_13, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6467
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13799") else null)); 
--# expr-done 6468
--# expr-start 6470 '<FieldAccess .type_bind_val(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13805
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6469 '<FieldAccess .type_var>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13805
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6469
--# property-call-start AdaNode.type_bind_val
Fld_16 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_15, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6470
And_Pred_3 := Logic_And (And_Pred_2, Fld_16, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13799") else null)); 
--# expr-done 6471
   If_Result_1 := And_Pred_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 6472
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6473
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1049;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1049;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Image_Equation;
--# end
--# property-start AttributeRef.img_equation /usr/src/packages/BUILD/ada/ast.py:13809
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Img_Equation
  (Node : Bare_Attribute_Ref
      ; Str_Type : Internal_Entity
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind str_type Str_Type
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1050;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1050 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1050;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6479 '<Block>' Let_Result None
--# scope-start
--# expr-start 6478 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13812
--# expr-start 6475 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13812
--# expr-start 6474 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13812
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6474
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6475
--# expr-start 6477 '<FieldAccess .type_bind_val(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13815
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6476 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13815
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6476
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Str_Type, Origin => Origin);
--# end
--# expr-done 6477
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13812") else null)); 
--# expr-done 6478
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6479
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1050;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1050;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Img_Equation;
--# end
--# property-start AttributeRef.pos_equation /usr/src/packages/BUILD/ada/ast.py:13819
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Pos_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1051;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1051 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1051;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6495 '<Block>' Let_Result None
--# scope-start
--# expr-start 6481 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13820
--# expr-start 6480 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13820
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6480
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6481
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6483 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13821
--# expr-start 6482 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13821
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6482
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6483
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6494 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13825
--# expr-start 6492 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13825
--# expr-start 6489 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13825
--# expr-start 6486 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13825
--# expr-start 6485 '<FieldAccess .ref_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13825
--# expr-start 6484 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13825
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6484
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 6485
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_7, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13825") else null)); 
--# expr-done 6486
--# expr-start 6488 '<FieldAccess .universal_int_bind(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6487 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13826
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6487
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 6488
And_Pred := Logic_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13825") else null)); 
--# expr-done 6489
--# expr-start 6491 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:13827
--# expr-start 6490 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13827
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6490
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_10, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13827") else null)); 
--# expr-done 6491
And_Pred_1 := Logic_And (And_Pred, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13825") else null)); 
--# expr-done 6492
--# expr-start 6493 '<FieldAccess .sub_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13828
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6493
And_Pred_2 := Logic_And (And_Pred_1, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13825") else null)); 
--# expr-done 6494
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6495
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1051;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1051;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Pos_Equation;
--# end
--# property-start AttributeRef.val_equation /usr/src/packages/BUILD/ada/ast.py:13832
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Val_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1052;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_8 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_9 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_10 : Logic_Var;
Fld_11 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1052 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_11);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1052;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6511 '<Block>' Let_Result None
--# scope-start
--# expr-start 6497 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13833
--# expr-start 6496 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13833
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6496
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6497
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6499 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13834
--# expr-start 6498 '<FieldAccess .args_list>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_2 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6498
Fld_3 := Fld_2.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6499
Block_Var_1 := Fld_5; 
--# bind expr Block_Var_1
--# expr-start 6510 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13837
--# expr-start 6508 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13837
--# expr-start 6505 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13837
--# expr-start 6502 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13837
--# expr-start 6501 '<FieldAccess .ref_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13837
--# expr-start 6500 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13837
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6500
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_6);
--# end
--# expr-done 6501
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_7, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13837") else null)); 
--# expr-done 6502
--# expr-start 6504 '<FieldAccess .type_bind_val(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13838
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6503 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13838
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6503
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_9 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6504
And_Pred := Logic_And (Bind_Result, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13837") else null)); 
--# expr-done 6505
--# expr-start 6507 '<FieldAccess .universal_int_bind(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13839
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6506 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13839
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var_1.Node);
--# expr-done 6506
--# property-call-start AdaNode.universal_int_bind
Fld_11 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_10, Origin => Origin);
--# end
--# expr-done 6507
And_Pred_1 := Logic_And (And_Pred, Fld_11, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13837") else null)); 
--# expr-done 6508
--# expr-start 6509 '<FieldAccess .sub_equation>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13840
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6509
And_Pred_2 := Logic_And (And_Pred_1, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13837") else null)); 
--# expr-done 6510
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6511
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1052;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1052;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Val_Equation;
--# end
--# property-start AttributeRef.invalid_value_equation /usr/src/packages/BUILD/ada/ast.py:13844
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Invalid_Value_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1053;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1053 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1053;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6520 '<Block>' Let_Result None
--# scope-start
--# expr-start 6513 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13845
--# expr-start 6512 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13845
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6512
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6513
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6519 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13846
--# expr-start 6516 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13847
--# expr-start 6515 '<FieldAccess .ref_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13847
--# expr-start 6514 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13847
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6514
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6515
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13847") else null)); 
--# expr-done 6516
--# expr-start 6518 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13848
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6517 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13848
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6517
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_4, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6518
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13846") else null)); 
--# expr-done 6519
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6520
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1053;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1053;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Invalid_Value_Equation;
--# end
--# property-start AttributeRef.access_equation /usr/src/packages/BUILD/ada/ast.py:13852
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Access_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1054;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Bare_Name;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Bare_Name;
Fld_9 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_10 : Bare_Name;
Fld_11 : Logic_Var;
Fld_12 : Logic_Var;
Bind_Result_1 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Or_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1054 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Or_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1054;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6541 '<Block>' Let_Result None
--# scope-start
--# expr-start 6540 '<Or>' Or_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13853
--# expr-start 6527 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13855
--# expr-start 6522 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13855
--# expr-start 6521 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13855
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6521
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, All_Els => True, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6522
--# expr-start 6526 '<Predicate on BaseTypeDecl.is_subp_access_of>' Pred /usr/src/packages/BUILD/ada/ast.py:13856
--# expr-start 6523 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13857
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6523
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 6525 '<FieldAccess .ref_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13858
--# expr-start 6524 '<FieldAccess .prefix>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13858
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6524
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_3);
--# end
--# expr-done 6525
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Subp_Access_Of_0_Pred.Create (Fld_2, Fld_4, Create_Base_Type_Decl_P_Is_Subp_Access_Of_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Subp_Access_Of")            else null))); 
--# expr-done 6526
And_Pred := Logic_And (Fld_1, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13855") else null)); 
--# expr-done 6527
--# expr-start 6539 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13860
--# expr-start 6529 '<FieldAccess .xref_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13860
--# expr-start 6528 '<FieldAccess .prefix>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13860
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6528
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 6529
--# expr-start 6538 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:13861
--# expr-start 6533 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13864
--# expr-start 6530 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13864
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6530
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
--# expr-start 6532 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13865
--# expr-start 6531 '<FieldAccess .prefix>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13865
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6531
      if Fld_8 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8);
--# expr-done 6532
Fld_9.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_9.all);
Eq_Node.Refs.Destroy (Fld_9.all);
Bind_Result := Bind_539_563.Create (Fld_7, Fld_9, Logic_Converter_539'(Origin => Origin), Equals_Data_563'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13864") else null)); 
--# expr-done 6533
--# expr-start 6537 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:13871
--# expr-start 6535 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13871
--# expr-start 6534 '<FieldAccess .prefix>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13871
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6534
      if Fld_10 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_10);
--# expr-done 6535
Fld_11.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_11.all);
Eq_Node.Refs.Destroy (Fld_11.all);
--# expr-start 6536 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13872
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6536
Fld_12.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_12.all);
Eq_Node.Refs.Destroy (Fld_12.all);
Bind_Result_1 := Bind_500_560.Create (Fld_11, Fld_12, Logic_Converter_500'(null record), Equals_Data_560'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13871") else null)); 
--# expr-done 6537
Or_Pred := Logic_Or (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13861") else null)); 
--# expr-done 6538
And_Pred_1 := Logic_And (Fld_6, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13860") else null)); 
--# expr-done 6539
Or_Pred_1 := Logic_Or (And_Pred, And_Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13853") else null)); 
--# expr-done 6540
Let_Result := Or_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6541
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1054;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1054;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Access_Equation;
--# end
--# property-start AttributeRef.size_equation /usr/src/packages/BUILD/ada/ast.py:13879
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Size_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1055;
      Block_Var : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Name;
Fld_7 : Logic_Equation;
Fld_8 : Logic_Var;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1055 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1055;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6558 '<Block>' Let_Result None
--# scope-start
--# expr-start 6543 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13880
--# expr-start 6542 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6542
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6543
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6557 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:13881
--# expr-start 6545 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:13882
--# expr-start 6544 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:13882
Is_Null := Block_Var.Node = null; 
--# expr-done 6544
Not_Val := not (Is_Null); 
--# expr-done 6545
if Not_Val then
   --# expr-start 6551 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13884
--# expr-start 6548 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13884
--# expr-start 6547 '<FieldAccess .ref_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13884
--# expr-start 6546 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13884
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Attribute_Ref_F_Prefix;
--# expr-done 6546
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 6547
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr := Block_Var; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13884") else null)); 
--# expr-done 6548
--# expr-start 6550 '<FieldAccess .universal_int_bind(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13885
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6549 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13885
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6549
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 6550
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13884") else null)); 
--# expr-done 6551
   If_Result := And_Pred;
else
   --# expr-start 6556 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13887
--# expr-start 6553 '<FieldAccess .sub_equation>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13887
--# expr-start 6552 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13887
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6552
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_7 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_6.Node, Env => Env, Origin => Origin, E_Info => Fld_6.Info);
--# end
--# expr-done 6553
--# expr-start 6555 '<FieldAccess .universal_int_bind(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13888
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6554 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13888
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6554
--# property-call-start AdaNode.universal_int_bind
Fld_9 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_8, Origin => Origin);
--# end
--# expr-done 6555
And_Pred_1 := Logic_And (Fld_7, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13887") else null)); 
--# expr-done 6556
   If_Result := And_Pred_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 6557
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 6558
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1055;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1055;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Size_Equation;
--# end
--# property-start AttributeRef.array_attr_equation /usr/src/packages/BUILD/ada/ast.py:13892
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Array_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1056;
            procedure Finalizer_Scope_2442;
      Block_Var : Boolean;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Integer;
Var_Expr : Internal_Entity_Assoc_List;
Var_Expr_1 : Internal_Entity_Expr;
Unused_2 : Boolean;
Res : Boolean;
Pfx_Typ : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_5 : Bare_Assoc_List;
Fld_6 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_7 : Internal_Entity_Expr;
Fld_8 : Boolean;
Fld_9 : Big_Integer_Type;
Small_Int : Integer;
Let_Result : Integer;
Scope_Result : Integer;
Result_Var : Integer;
Scope_Result_1 : Integer;
Result_Var_1 : Integer;
Arith_Result : Integer;
Fld_10 : Internal_Entity_Name;
Fld_11 : Logic_Equation;
Fld_12 : Boolean;
If_Result_1 : Boolean;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
Fld_15 : Boolean;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_18 : Logic_Equation;
Fld_19 : Boolean;
Fld_20 : Boolean;
Not_Val : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Fld_21 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_22 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_4 : Logic_Equation;
If_Result_5 : Logic_Equation;
If_Result_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_23 : Internal_Entity_Name;
Fld_24 : Internal_Entity_Name;
Fld_25 : Logic_Var;
Pred : Logic_Equation;
Fld_26 : Boolean;
Fld_27 : Internal_Entity_Name;
Fld_28 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Fld_29 : Logic_Var;
Fld_30 : Logic_Equation;
Fld_31 : Logic_Var;
Fld_32 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_33 : Logic_Equation;
If_Result_7 : Logic_Equation;
Fld_34 : Internal_Entity_Name;
Fld_35 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_36 : Internal_Entity_Name;
Fld_37 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Logic_False_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Is_Null_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_9 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_1056 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Fld_33);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Fld_35);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_1056;
            procedure Finalizer_Scope_2442 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2442;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6613 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6561 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13893
--# expr-start 6559 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13893
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Attribute_Ref_F_Attribute, Info => Ent.Info);
--# expr-done 6559
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6560 '<Symbol Length>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 6560
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 6561
Block_Var := Fld_1; 
--# bind is_length Block_Var
--# expr-start 6563 '<FieldAccess .name_designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13894
--# expr-start 6562 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13894
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6562
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6563
Block_Var_1 := Fld_3; 
--# bind typ Block_Var_1
--# expr-start 6570 '<Op ->' Arith_Result /usr/src/packages/BUILD/ada/ast.py:13898
--# expr-start 6564 '<FieldAccess .args_list>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13898
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_4 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6564
Var_Expr := Fld_4;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind a Var_Expr
--# expr-start 6565 '<FieldAccess .expr>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13898
Saved := Var_Expr; 
Fld_5 := Saved.Node;
Get_Result := Get (Fld_5, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_6 := Saved.Info;
As_Entity := (Info => Fld_6, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6565
Var_Expr_1 := Fld_7;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind expr Var_Expr_1
--# expr-start 6569 '<Let _>' Let_Result /usr/src/packages/BUILD/ada/ast.py:13899
--# scope-start
--# expr-start 6566 '<FieldAccess .resolve_names_internal>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13900
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_8 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6566
Unused_2 := Fld_8; 
--# bind _ Unused_2
--# expr-start 6568 '<as_int(<FieldAccess .eval_as_int>)>' Small_Int /usr/src/packages/BUILD/ada/ast.py:13901
--# expr-start 6567 '<FieldAccess .eval_as_int>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13901
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld_9 := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6567
Small_Int := To_Integer (Fld_9); 
--# expr-done 6568
Let_Result := Small_Int; 
--# end
--# expr-done 6569
Scope_Result := Let_Result;
   --# end
      Finalizer_Scope_2442;
   Result_Var := Scope_Result;
else
   Result_Var := 1;
end if;
Scope_Result_1 := Result_Var;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := 1;
end if;
Arith_Result := (Result_Var_1 - 1); 
--# expr-done 6570
Block_Var_2 := Arith_Result; 
--# bind dim Block_Var_2
--# expr-start 6612 '<If>' If_Result_9 /usr/src/packages/BUILD/ada/ast.py:13905
--# expr-start 6572 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:13906
--# expr-start 6571 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:13906
Is_Null_1 := Block_Var_1.Node = null; 
--# expr-done 6571
Not_Val_1 := not (Is_Null_1); 
--# expr-done 6572
if Not_Val_1 then
   --# expr-start 6588 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13909
--# expr-start 6574 '<FieldAccess .xref_no_overloading>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13909
--# expr-start 6573 '<FieldAccess .prefix>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13909
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6573
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_11 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 6574
--# expr-start 6587 '<Cond>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:13909
--# expr-start 6575 '<FieldAccess .is_array>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13910
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_12 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6575
if Fld_12 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 6577 '<FieldAccess .universal_int_bind(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13911
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6576 '<FieldAccess .type_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13911
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6576
--# property-call-start AdaNode.universal_int_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_13, Origin => Origin);
--# end
--# expr-done 6577
   If_Result_6 := Fld_14;
else
--# expr-start 6578 '<FieldAccess .is_array>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13914
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_array
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Is_Array (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6578
if Fld_15 then
   --# expr-start 6581 '<FieldAccess .type_bind_val(...)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13915
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6579 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13915
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6579
--# expr-start 6580 '<FieldAccess .index_type(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13915
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_17 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Block_Var_1.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6580
      Cast_Result := Create_Internal_Entity
        (Node => Fld_17.Node,
         Info => Fld_17.Info);
--# property-call-start AdaNode.type_bind_val
Fld_18 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 6581
   If_Result_5 := Fld_18;
else
--# expr-start 6582 '<FieldAccess .is_discrete_type>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13918
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6582
if Fld_19 then
   If_Result_3 := True;
else
--# expr-start 6583 '<FieldAccess .is_real_type>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13918
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_real_type'
Fld_20 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Real_Type (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 6583
if Fld_20 then
   --# expr-start 6584 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:13918
Not_Val := not (Block_Var); 
--# expr-done 6584
   If_Result_2 := Not_Val;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
if If_Result_3 then
   --# expr-start 6586 '<FieldAccess .type_bind_val(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13920
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6585 '<FieldAccess .type_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13920
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6585
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_21, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6586
   If_Result_4 := Fld_22;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13922") else null)); 
   If_Result_4 := Logic_False;
end if;
      Inc_Ref (If_Result_4);
   If_Result_5 := If_Result_4;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 6587
And_Pred := Logic_And (Fld_11, If_Result_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13909") else null)); 
--# expr-done 6588
   If_Result_9 := And_Pred;
else
   --# expr-start 6611 '<Let res, pfx_typ>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:13927
--# scope-start
--# expr-start 6593 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:13928
--# expr-start 6589 '<FieldAccess .prefix>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:13928
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6589
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6592 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred /usr/src/packages/BUILD/ada/ast.py:13929
--# expr-start 6591 '<FieldAccess .type_var>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:13930
--# expr-start 6590 '<FieldAccess .prefix>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:13930
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6590
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24.Node);
--# expr-done 6591
Fld_25.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_25.all);
Eq_Node.Refs.Destroy (Fld_25.all);
Pred := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_25, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 6592
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_26 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_23.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld_23.Info);
--# end
--# expr-done 6593
Res := Fld_26; 
--# bind res Res
--# expr-start 6596 '<Cast to BaseTypeDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:13932
--# expr-start 6595 '<FieldAccess .type_val>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:13932
--# expr-start 6594 '<FieldAccess .prefix>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:13932
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6594
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_28 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_27.Node);
--# end
--# expr-done 6595
   if Fld_28.Node = null
      or else Fld_28.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_28.Node,
         Info => Fld_28.Info);
   else
         Cast_Result_2 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6596
Pfx_Typ := Cast_Result_2; 
--# bind pfx_typ Pfx_Typ
--# expr-start 6610 '<If>' If_Result_8 /usr/src/packages/BUILD/ada/ast.py:13934
if Res then
   --# expr-start 6609 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13935
--# expr-start 6605 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13935
--# expr-start 6602 '<If>' If_Result_7 /usr/src/packages/BUILD/ada/ast.py:13935
if Block_Var then
   --# expr-start 6598 '<FieldAccess .universal_int_bind(...)>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:13936
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6597 '<FieldAccess .type_var>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:13936
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_29 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6597
--# property-call-start AdaNode.universal_int_bind
Fld_30 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_29, Origin => Origin);
--# end
--# expr-done 6598
   If_Result_7 := Fld_30;
else
   --# expr-start 6601 '<FieldAccess .type_bind_val(...)>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:13937
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6599 '<FieldAccess .type_var>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:13937
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6599
--# expr-start 6600 '<FieldAccess .index_type(...)>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:13938
      if Pfx_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.index_type
Fld_32 := Libadalang.Implementation.Base_Type_Decl_P_Index_Type (Node => Pfx_Typ.Node, Dim => Block_Var_2, Origin => Origin, E_Info => Pfx_Typ.Info);
--# end
--# expr-done 6600
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_32.Node,
         Info => Fld_32.Info);
--# property-call-start AdaNode.type_bind_val
Fld_33 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_31, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 6601
   If_Result_7 := Fld_33;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 6602
--# expr-start 6604 '<FieldAccess .xref_equation>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:13939
--# expr-start 6603 '<FieldAccess .prefix>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:13939
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6603
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_35 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_34.Node, Env => Env, Origin => Origin, E_Info => Fld_34.Info);
--# end
--# expr-done 6604
And_Pred_1 := Logic_And (If_Result_7, Fld_35, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13935") else null)); 
--# expr-done 6605
--# expr-start 6608 '<Predicate on BaseTypeDecl.is_array_def_with_deref>' Pred_1 /usr/src/packages/BUILD/ada/ast.py:13940
--# expr-start 6607 '<FieldAccess .type_var>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:13941
--# expr-start 6606 '<FieldAccess .prefix>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:13941
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6606
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_36.Node);
--# expr-done 6607
Fld_37.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_37.all);
Eq_Node.Refs.Destroy (Fld_37.all);
Pred_1 := Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Pred.Create (Fld_37, Create_Base_Type_Decl_P_Is_Array_Def_With_Deref_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Array_Def_With_Deref")            else null))); 
--# expr-done 6608
And_Pred_2 := Logic_And (And_Pred_1, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13935") else null)); 
--# expr-done 6609
   If_Result_8 := And_Pred_2;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13942") else null)); 
   If_Result_8 := Logic_False_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 6610
Let_Result_1 := If_Result_8; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6611
   If_Result_9 := Let_Result_1;
end if;
      Inc_Ref (If_Result_9);
--# expr-done 6612
Let_Result_2 := If_Result_9; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6613
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1056;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1056;
               Finalizer_Scope_2442;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Array_Attr_Equation;
--# end
--# property-start AttributeRef.subtype_attr_equation /usr/src/packages/BUILD/ada/ast.py:13946
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Subtype_Attr_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1057;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1057 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1057;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6622 '<Block>' Let_Result None
--# scope-start
--# expr-start 6621 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13952
--# expr-start 6618 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13952
--# expr-start 6615 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13952
--# expr-start 6614 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13952
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Prefix;
--# expr-done 6614
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6615
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 6617 '<FieldAccess .name_designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13952
--# expr-start 6616 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13952
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6616
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 6617
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13952") else null)); 
--# expr-done 6618
--# expr-start 6620 '<FieldAccess .universal_int_bind(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13953
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6619 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13953
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6619
--# property-call-start AdaNode.universal_int_bind
Fld_5 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld_4, Origin => Origin);
--# end
--# expr-done 6620
And_Pred := Logic_And (Bind_Result, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13952") else null)); 
--# expr-done 6621
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 6622
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1057;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1057;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Subtype_Attr_Equation;
--# end
--# property-start AttributeRef.inline_asm_equation /usr/src/packages/BUILD/ada/ast.py:13957
pragma Warnings (Off, "is not referenced");
 function Attribute_Ref_P_Inline_Asm_Equation
  (Node : Bare_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Attribute_Ref := Bare_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Attribute_Ref :=
     Internal_Entity_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1058;
            procedure Finalizer_Scope_2443;
      Block_Var : Symbol_Type;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Bare_Basic_Decl;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Fld : Bare_Identifier;
Sym_2 : Symbol_Type;
Fld_1 : Boolean;
If_Result : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Array_Lit : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_7 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Assoc_List;
Fld_9 : Bare_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Sym_5 : Symbol_Type;
Fld_13 : Internal_Entity;
Fld_14 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_15 : Internal_Entity_Assoc_List;
Fld_16 : Bare_Assoc_List;
Fld_17 : Internal_Entity_Info;
Get_Result_1 : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_18 : Internal_Entity_Expr;
Fld_19 : Logic_Var;
Fld_20 : Internal_Entity_Name;
Fld_21 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_22 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1058 is
            begin
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_7);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1058;
            procedure Finalizer_Scope_2443 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2443;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6655 '<Block>' Let_Result None
--# scope-start
--# expr-start 6628 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:13962
--# expr-start 6625 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13963
--# expr-start 6623 '<FieldAccess .attribute>' Fld /usr/src/packages/BUILD/ada/ast.py:13963
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Attribute_Ref_F_Attribute;
--# expr-done 6623
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6624 '<Symbol Asm_Input>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 6624
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld, Sym => Sym_2);
--# end
--# expr-done 6625
if Fld_1 then
   --# expr-start 6626 '<Symbol Asm_Input_Operand>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input_Operand); 
--# expr-done 6626
   If_Result := Sym;
else
   --# expr-start 6627 '<Symbol Asm_Output_Operand>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output_Operand); 
--# expr-done 6627
   If_Result := Sym_1;
end if;
--# expr-done 6628
Block_Var := If_Result; 
--# bind return_type_name Block_Var
--# expr-start 6635 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13971
--# expr-start 6634 '<EnvGet(<Then <FieldAccess .get_unit_root_decl(...)>: <Var Var_Expr> <FieldAccess .children_env>>, <Var return_type_name>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13971
--# expr-start 6632 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13971
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6631 '<ArrayLiteral>' Array_Lit None
--# expr-start 6629 '<Symbol System>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_System); 
--# expr-done 6629
--# expr-start 6630 '<Symbol Machine_Code>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Code); 
--# expr-done 6630
Array_Lit := Create_Symbol_Type_Array (Internal_Symbol_Type_Array'(1 => Sym_3, 2 => Sym_4)); 
--# expr-done 6631
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Ent.Node, Name => Array_Lit, Kind => Unit_Specification);
--# end
--# expr-done 6632
Var_Expr := Fld_2;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
--# expr-start 6633 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13971
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Var_Expr);
--# expr-done 6633
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2443;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
Env_Get_Result := AST_Envs.Get_First (Self => Result_Var, Key => Block_Var, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 6634
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6635
Block_Var_1 := Cast_Result; 
--# bind return_type Block_Var_1
--# expr-start 6654 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13977
--# expr-start 6647 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13977
--# expr-start 6640 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13977
--# expr-start 6637 '<FieldAccess .xref_no_overloading>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13978
--# expr-start 6636 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13978
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6636
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 6637
--# expr-start 6639 '<FieldAccess .type_bind_val(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6638 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13979
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6638
Cast_Expr := Block_Var_1; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_7 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_6, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 6639
And_Pred := Logic_And (Fld_5, Fld_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13977") else null)); 
--# expr-done 6640
--# expr-start 6646 '<FieldAccess .type_bind_val(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13982
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6643 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13983
--# expr-start 6642 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13983
--# expr-start 6641 '<FieldAccess .args_list>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13983
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_8 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6641
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_1 := As_Entity;
end if;
      if If_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_1.Node, E_Info => If_Result_1.Info);
--# end
--# expr-done 6642
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6643
--# expr-start 6645 '<FieldAccess .std_entity(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13984
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6644 '<Symbol String>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 6644
--# property-call-start AdaNode.std_entity
Fld_13 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym_5);
--# end
--# expr-done 6645
--# property-call-start AdaNode.type_bind_val
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 6646
And_Pred_1 := Logic_And (And_Pred, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13977") else null)); 
--# expr-done 6647
--# expr-start 6653 '<FieldAccess .type_bind_val(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13988
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6650 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13989
--# expr-start 6649 '<FieldAccess .expr>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13989
--# expr-start 6648 '<FieldAccess .args_list>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13989
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AttributeRef.args_list
Fld_15 := Libadalang.Implementation.Attribute_Ref_P_Args_List (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6648
Fld_16 := Fld_15.Node;
Get_Result_1 := Get (Fld_16, 1, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_17 := Fld_15.Info;
As_Entity_1 := (Info => Fld_17, Node => Get_Result_1); 
   If_Result_2 := As_Entity_1;
end if;
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_18 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 6649
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_18.Node);
--# expr-done 6650
--# expr-start 6652 '<FieldAccess .name_designated_type>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13990
--# expr-start 6651 '<FieldAccess .prefix>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13990
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6651
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_21 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_20.Node, E_Info => Fld_20.Info);
--# end
--# expr-done 6652
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_21.Node,
         Info => Fld_21.Info);
--# property-call-start AdaNode.type_bind_val
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6653
And_Pred_2 := Logic_And (And_Pred_1, Fld_22, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13977") else null)); 
--# expr-done 6654
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6655
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1058;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1058;
               Finalizer_Scope_2443;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Attribute_Ref_P_Inline_Asm_Equation;
--# end
   --
   --  Primitives for Bare_Update_Attribute_Ref
   --
      procedure Initialize_Fields_For_Update_Attribute_Ref
        (Self : Bare_Update_Attribute_Ref
         ; Attribute_Ref_F_Prefix : Bare_Name
         ; Attribute_Ref_F_Attribute : Bare_Identifier
         ; Attribute_Ref_F_Args : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Attribute_Ref
              (Self, Attribute_Ref_F_Prefix, Attribute_Ref_F_Attribute, Attribute_Ref_F_Args);
      end Initialize_Fields_For_Update_Attribute_Ref;
--# property-start UpdateAttributeRef.xref_equation /usr/src/packages/BUILD/ada/ast.py:14000
pragma Warnings (Off, "is not referenced");
 function Update_Attribute_Ref_P_Xref_Equation
  (Node : Bare_Update_Attribute_Ref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Update_Attribute_Ref := Bare_Update_Attribute_Ref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Update_Attribute_Ref :=
     Internal_Entity_Update_Attribute_Ref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1059;
      Block_Var : Boolean;
Fld : Internal_Entity_Name;
Fld_1 : Boolean;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Aggregate;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_6 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1059 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1059;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6665 '<Block>' Let_Result None
--# scope-start
--# expr-start 6657 '<FieldAccess .resolve_names_internal>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14005
--# expr-start 6656 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14005
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6656
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6657
Block_Var := Fld_1; 
--# expr-start 6664 '<FieldAccess .type_bind_val(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14006
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6660 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14007
--# expr-start 6659 '<Cast to Aggregate>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14007
--# expr-start 6658 '<FieldAccess .args>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14007
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity (Node => Ent.Node.Attribute_Ref_F_Args, Info => Ent.Info);
--# expr-done 6658
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Aggregate_Range
   then
      Cast_Result := Create_Internal_Entity_Aggregate
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6659
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result.Node);
--# expr-done 6660
--# expr-start 6663 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:14008
--# expr-start 6662 '<FieldAccess .type_val>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14008
--# expr-start 6661 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14008
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Attribute_Ref_F_Prefix, Info => Ent.Info);
--# expr-done 6661
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_5 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_4.Node);
--# end
--# expr-done 6662
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6663
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_6 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 6664
Let_Result := Fld_6; Inc_Ref (Let_Result);
--# end
--# expr-done 6665
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1059;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1059;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Update_Attribute_Ref_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Call_Expr
   --
      procedure Initialize_Fields_For_Call_Expr
        (Self : Bare_Call_Expr
         ; Call_Expr_F_Name : Bare_Name
         ; Call_Expr_F_Suffix : Bare_Ada_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Call_Expr_F_Name := Call_Expr_F_Name;
            Self.Call_Expr_F_Suffix := Call_Expr_F_Suffix;
      Self.Call_Expr_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Call_Expr;
   function Call_Expr_F_Name
     (Node : Bare_Call_Expr) return Bare_Name
   is
   begin
         return Node.Call_Expr_F_Name;
   end;
   function Call_Expr_F_Suffix
     (Node : Bare_Call_Expr) return Bare_Ada_Node
   is
   begin
         return Node.Call_Expr_F_Suffix;
   end;
--# property-start CallExpr.ref_var /usr/src/packages/BUILD/ada/ast.py:10500
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Ref_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6667 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10500
--# expr-start 6666 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10500
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 6666
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6667
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Ref_Var;
--# end
--# property-start CallExpr.subp_spec_var /usr/src/packages/BUILD/ada/ast.py:10504
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Logic_Var
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6668 '<FieldAccess .r_called_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:10504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_R_Called_Spec'Unrestricted_Access;
--# expr-done 6668
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subp_Spec_Var;
--# end
--# property-start CallExpr.defines_subp_spec_var /usr/src/packages/BUILD/ada/ast.py:10505
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Defines_Subp_Spec_Var
  (Node : Bare_Call_Expr
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Defines_Subp_Spec_Var;
--# end
--# property-start CallExpr.relative_name /usr/src/packages/BUILD/ada/ast.py:10507
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Relative_Name
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6670 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10507
--# expr-start 6669 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10507
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6669
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6670
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Relative_Name;
--# end
--# property-start CallExpr.designated_env /usr/src/packages/BUILD/ada/ast.py:10510
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Env
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_853;
            procedure Finalizer_Scope_2444;
            procedure Finalizer_Scope_2445;
      Block_Var : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Basic_Decl;
Match_1 : Internal_Entity;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_853 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_853;
            procedure Finalizer_Scope_2444 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2444;
            procedure Finalizer_Scope_2445 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2445;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6682 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6672 '<FieldAccess .name_designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10511
--# expr-start 6671 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10511
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6671
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_1 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 6672
Block_Var := Fld_1; 
--# bind typ Block_Var
--# expr-start 6681 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10513
--# expr-start 6674 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10514
--# expr-start 6673 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10514
Is_Null := Block_Var.Node = null; 
--# expr-done 6673
Not_Val := not (Is_Null); 
--# expr-done 6674
if Not_Val then
   --# expr-start 6675 '<FieldAccess .defining_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10515
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6675
   If_Result := Fld_2;
else
   --# expr-start 6680 '<env_group(<map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:10516
--# expr-start 6679 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10516
--# expr-start 6676 '<FieldAccess .env_elements>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10516
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_3 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6676
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_203 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_203
      --# expr-start 6678 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10516
      if Item_203.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_203; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind bd Match_0
--# expr-start 6677 '<FieldAccess .defining_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10517
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 6677
Let_Result := Fld_4; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2445;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind _ Match_1
Let_Result_1 := AST_Envs.Empty_Env; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 6678
         declare
            Item_To_Append : constant Lexical_Env :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2444;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 6679
Group_Env := Group (Map_Result, No_Metadata); 
--# expr-done 6680
   If_Result := Group_Env;
end if;
      Inc_Ref (If_Result);
--# expr-done 6681
Let_Result_2 := If_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6682
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_853;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_853;
               Finalizer_Scope_2444;
               Finalizer_Scope_2445;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Designated_Env;
--# end
--# property-start CallExpr.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:10523
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Env_Elements_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_854;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_854 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_854;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6685 '<Block>' Let_Result None
--# scope-start
--# expr-start 6684 '<FieldAccess .env_elements_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10524
--# expr-start 6683 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10524
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6683
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 6684
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 6685
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_854;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_854;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Env_Elements_Impl;
--# end
--# property-start CallExpr.designated_type_impl /usr/src/packages/BUILD/ada/ast.py:10529
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Designated_Type_Impl
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2447;
            procedure Finalizer_Scope_2448;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Boolean;
Var_Expr : Internal_Entity_Assoc_List;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Fld_4 : Internal_Param_Match_Array_Access;
Fld_5 : Boolean;
Fld_6 : Internal_Single_Formal;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_13 : Boolean;
If_Result : Boolean;
Quantifier_Result : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_2447 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2447;
            procedure Finalizer_Scope_2448 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2448;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6702 '<Block>' Let_Result None
--# scope-start
--# expr-start 6687 '<FieldAccess .designated_type_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10531
--# expr-start 6686 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10531
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6686
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 6687
Block_Var := Fld_1; 
--# bind prefix_tpe Block_Var
--# expr-start 6688 '<FieldAccess .params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10536
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6688
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Assoc_List then
   --# scope-start
   --# bind ps Var_Expr
--# expr-start 6690 '<FieldAccess .match_formals(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10537
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Var_Expr_1 := Block_Var;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 6689 '<FieldAccess .discriminants_list>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10538
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discriminants_list'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discriminants_List (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6689
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2448;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Formal_Param_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# property-call-start AdaNode.match_formals
Fld_4 := Libadalang.Implementation.Ada_Node_P_Match_Formals (Node => Self, Formal_Params => Result_Var, Params => Var_Expr, Is_Dottable_Subp => False);
--# end
--# expr-done 6690
Quantifier_Result := True;
   declare
      Collection : constant Internal_Param_Match_Array_Access := Fld_4;
   begin
      for Item_204 of
            Collection.Items
      loop
   --# scope-start
            --# bind pm Item_204
--# expr-start 6691 '<FieldAccess .has_matched>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10541
Fld_5 := Item_204.Has_Matched;
--# expr-done 6691
if Fld_5 then
   --# expr-start 6700 '<FieldAccess .matching_type(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10542
--# expr-start 6694 '<FieldAccess .formal_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10542
--# expr-start 6693 '<FieldAccess .spec>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10542
--# expr-start 6692 '<FieldAccess .formal>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10542
Fld_6 := Item_204.Formal;
--# expr-done 6692
Fld_7 := Fld_6.Spec;
--# expr-done 6693
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_8 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_7.Node, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 6694
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6698 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10543
--# expr-start 6697 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10543
--# expr-start 6696 '<FieldAccess .assoc>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10543
--# expr-start 6695 '<FieldAccess .actual>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10543
Fld_9 := Item_204.Actual;
--# expr-done 6695
Fld_10 := Fld_9.Assoc;
--# expr-done 6696
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 6697
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         Cast_Result := No_Entity_Name;
   end if;
--# expr-done 6698
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
--# expr-start 6699 '<FieldAccess .name_designated_type>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10543
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_12 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6699
Scope_Result_1 := Fld_12;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
--# property-call-start BaseTypeDecl.matching_type
Fld_13 := Libadalang.Implementation.Base_Type_Decl_P_Matching_Type (Node => Fld_8.Node, Expected_Type => Result_Var_1, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 6700
   If_Result := Fld_13;
else
   If_Result := False;
end if;
            if not (If_Result) then
               Quantifier_Result := False;
               exit;
            end if;
   --# end
      end loop;
   end;
Scope_Result_2 := Quantifier_Result;
   --# end
      Finalizer_Scope_2447;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
Block_Var_1 := Result_Var_2; 
--# bind matches_formals Block_Var_1
--# expr-start 6701 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10552
if Block_Var_1 then
   If_Result_1 := Block_Var;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 6701
Let_Result := If_Result_1; 
--# end
--# expr-done 6702
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2447;
               Finalizer_Scope_2448;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Designated_Type_Impl;
--# end
--# property-start CallExpr.params /usr/src/packages/BUILD/ada/ast.py:10554
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Params
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Assoc_List
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Assoc_List;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 6704 '<Cast to AssocList>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10554
--# expr-start 6703 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:10554
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6703
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 6704
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Params;
--# end
--# property-start CallExpr.check_array_slice /usr/src/packages/BUILD/ada/ast.py:10557
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_Array_Slice
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Assoc_List;
Var_Expr_3 : Internal_Entity_Name;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
Is_A_2 : Boolean;
If_Result_1 : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Assoc_List;
Fld_2 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Saved : Internal_Entity_Assoc_List;
Fld_3 : Bare_Assoc_List;
Fld_4 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_5 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Is_Null_2 : Boolean;
Not_Val_1 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result_3 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_4 : Boolean;
Scope_Result_3 : Boolean;
Result_Var_3 : Boolean;
If_Result_5 : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6720 '<Block>' Let_Result None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6705 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:10562
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6705
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 6707 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10564
--# expr-start 6706 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10564
Is_Null := Block_Var.Node = null; 
--# expr-done 6706
Not_Val := not (Is_Null); 
--# expr-done 6707
if Not_Val then
--# expr-start 6708 '<FieldAccess .suffix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10565
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6708
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind sfx Var_Expr_1
--# expr-start 6709 '<IsA BinOp>' Is_A /usr/src/packages/BUILD/ada/ast.py:10568
Is_A := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Bin_Op_Range; 
--# expr-done 6709
if Is_A then
   If_Result := True;
else
   --# expr-start 6710 '<IsA AttributeRef>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:10570
Is_A_1 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 6710
   If_Result := Is_A_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6711 '<IsA SubtypeIndication>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:10572
Is_A_2 := Var_Expr_1.Node /= null 
and then Var_Expr_1.Node.Kind in Ada_Subtype_Indication_Range; 
--# expr-done 6711
   If_Result_1 := Is_A_2;
end if;
if If_Result_1 then
   If_Result_4 := True;
else
--# expr-start 6712 '<Cast to AssocList>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10574
Cast_Expr := Var_Expr_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result := Create_Internal_Entity_Assoc_List
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Assoc_List;
   end if;
--# expr-done 6712
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind al Var_Expr_2
--# expr-start 6714 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10576
--# expr-start 6713 '<length(<Var al>)>' Len /usr/src/packages/BUILD/ada/ast.py:10576
Fld_2 := Var_Expr_2.Node; 
Len := Length (Fld_2); 
--# expr-done 6713
Is_Equal := Len = 1; 
--# expr-done 6714
if Is_Equal then
--# expr-start 6716 '<Cast to Name>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:10577
--# expr-start 6715 '<FieldAccess .expr>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10577
Saved := Var_Expr_2; 
Fld_3 := Saved.Node;
Get_Result := Get (Fld_3, 0, True); 
Is_Null_1 := Get_Result = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_4 := Saved.Info;
As_Entity := (Info => Fld_4, Node => Get_Result); 
   If_Result_2 := As_Entity;
end if;
      if If_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_5 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_2.Node, E_Info => If_Result_2.Info);
--# end
--# expr-done 6715
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Name
   then
      Cast_Result_1 := Create_Internal_Entity_Name
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Name;
   end if;
--# expr-done 6716
Var_Expr_3 := Cast_Result_1;
if Var_Expr_3 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_3
--# expr-start 6719 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:10578
--# expr-start 6718 '<IsNull>' Is_Null_2 /usr/src/packages/BUILD/ada/ast.py:10578
--# expr-start 6717 '<FieldAccess .name_designated_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10578
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 6717
Is_Null_2 := Fld_6.Node = null; 
--# expr-done 6718
Not_Val_1 := not (Is_Null_2); 
--# expr-done 6719
Scope_Result_1 := Not_Val_1;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result_3 := Result_Var_1;
else
   If_Result_3 := False;
end if;
Scope_Result_2 := If_Result_3;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_4 := Result_Var_2;
end if;
Scope_Result_3 := If_Result_4;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := False;
end if;
   If_Result_5 := Result_Var_3;
else
   If_Result_5 := False;
end if;
Let_Result := If_Result_5; 
--# end
--# expr-done 6720
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Check_Array_Slice;
--# end
--# property-start CallExpr.is_array_slice /usr/src/packages/BUILD/ada/ast.py:10587
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Is_Array_Slice
  (Node : Bare_Call_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6726 '<Block>' Let_Result None
--# scope-start
--# expr-start 6725 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .check_array_slice(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:10592
--# expr-start 6721 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:10593
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 6721
Bound_Origin := Fld; 
--# expr-start 6724 '<FieldAccess .check_array_slice(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10594
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6723 '<FieldAccess .expression_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10594
--# expr-start 6722 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10594
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6722
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.expression_type
Fld_2 := Libadalang.Implementation.Expr_P_Expression_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6723
--# property-call-start CallExpr.check_array_slice
Fld_3 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Fld_2, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 6724
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 6725
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 6726
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Is_Array_Slice;
--# end
--# property-start CallExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:10598
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_859;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_859 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_859;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6728 '<Block>' Let_Result None
--# scope-start
--# expr-start 6727 '<FieldAccess .bottom_up_name_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:10599
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6727
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 6728
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_859;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_859;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Xref_Equation;
--# end
--# property-start CallExpr.type_conv_xref_equation /usr/src/packages/BUILD/ada/ast.py:10602
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Type_Conv_Xref_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_860;
      Fld : Internal_Entity_Assoc_List;
Fld_1 : Bare_Assoc_List;
Fld_2 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Assoc;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Fld_9 : Bare_Name;
Fld_10 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Name;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_860 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_860;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6746 '<Block>' Let_Result None
--# scope-start
--# expr-start 6745 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10607
--# expr-start 6740 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10607
--# expr-start 6734 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10607
--# expr-start 6731 '<FieldAccess .sub_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10608
--# expr-start 6730 '<FieldAccess .expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10608
--# expr-start 6729 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:10608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6729
Fld_1 := Fld.Node;
Get_Result := Get (Fld_1, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Assoc;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Get_Result); 
   If_Result := As_Entity;
end if;
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 6730
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_3.Node, Env => Env, Origin => Origin, E_Info => Fld_3.Info);
--# end
--# expr-done 6731
--# expr-start 6733 '<FieldAccess .subtype_indication_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10609
--# expr-start 6732 '<FieldAccess .name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10609
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6732
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_6 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 6733
And_Pred := Logic_And (Fld_4, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10607") else null)); 
--# expr-done 6734
--# expr-start 6739 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10610
--# expr-start 6736 '<FieldAccess .ref_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10610
--# expr-start 6735 '<FieldAccess .name>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10610
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Call_Expr_F_Name;
--# expr-done 6735
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 6736
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
--# expr-start 6738 '<FieldAccess .type_var>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10610
--# expr-start 6737 '<FieldAccess .name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10610
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Call_Expr_F_Name;
--# expr-done 6737
      if Fld_9 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_9);
--# expr-done 6738
Fld_10.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_10.all);
Eq_Node.Refs.Destroy (Fld_10.all);
Bind_Result := Bind_Default_Default.Create (Fld_8, Fld_10, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10610") else null)); 
--# expr-done 6739
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10607") else null)); 
--# expr-done 6740
--# expr-start 6744 '<FieldAccess .type_bind_var(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6741 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6741
--# expr-start 6743 '<FieldAccess .ref_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10611
--# expr-start 6742 '<FieldAccess .name>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10611
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Call_Expr_F_Name;
--# expr-done 6742
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_12);
--# end
--# expr-done 6743
--# property-call-start AdaNode.type_bind_var
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 6744
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10607") else null)); 
--# expr-done 6745
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6746
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_860;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_860;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Type_Conv_Xref_Equation;
--# end
--# property-start CallExpr.entity_equation /usr/src/packages/BUILD/ada/ast.py:10615
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Entity_Equation
  (Node : Bare_Call_Expr
      ; S : Internal_Entity_Basic_Decl
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind s S
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_861;
            procedure Finalizer_Scope_2455;
            procedure Finalizer_Scope_2456;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Name;
Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Equation;
Fld_5 : Boolean;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Equation;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Internal_Entity_Info;
Fld_10 : Internal_Metadata;
Fld_11 : Boolean;
Fld_12 : Logic_Equation;
Fld_13 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_16 : Internal_Entity_Base_Subp_Spec;
Cast_Result_2 : Internal_Entity_Base_Formal_Param_Holder;
Fld_17 : Internal_Entity_Info;
Fld_18 : Internal_Metadata;
Fld_19 : Boolean;
Fld_20 : Logic_Equation;
Fld_21 : Bare_Name;
As_Entity_1 : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_22 : Internal_Entity_Base_Type_Decl;
Fld_23 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
If_Result_2 : Logic_Equation;
If_Result_3 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_861 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_20);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_861;
            procedure Finalizer_Scope_2455 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2455;
            procedure Finalizer_Scope_2456 is
            begin
                     Dec_Ref (Fld_23);
            end Finalizer_Scope_2456;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6779 '<Block>' Let_Result None
--# scope-start
--# expr-start 6778 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10617
--# expr-start 6749 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10617
--# expr-start 6748 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10617
--# expr-start 6747 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10617
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Call_Expr_F_Name;
--# expr-done 6747
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 6748
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Cast_Expr := S; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10617") else null)); 
--# expr-done 6749
--# expr-start 6777 '<Cond>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:10617
--# expr-start 6750 '<FieldAccess .is_paramless>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10622
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_paramless
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Is_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6750
if Fld_2 then
   --# expr-start 6752 '<FieldAccess .parent_name_equation(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10622
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6751 '<FieldAccess .expr_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10623
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6751
--# property-call-start Name.parent_name_equation
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_3, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6752
   If_Result_3 := Fld_4;
else
--# expr-start 6753 '<FieldAccess .can_be_paramless>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10628
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6753
if Fld_5 then
   --# expr-start 6766 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:10628
--# expr-start 6755 '<FieldAccess .parent_name_equation(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10629
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6754 '<FieldAccess .expr_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10630
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6754
--# property-call-start Name.parent_name_equation
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Fld_6, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6755
--# expr-start 6765 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10633
--# expr-start 6760 '<FieldAccess .subprogram_equation(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10634
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6756 '<FieldAccess .subp_spec_or_null>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10635
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6756
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_8.Node,
         Info => Fld_8.Info);
--# expr-start 6759 '<FieldAccess .dottable_subp>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10636
--# expr-start 6758 '<FieldAccess .md>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10636
--# expr-start 6757 '<FieldAccess .info>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10636
Fld_9 := S.Info;
--# expr-done 6757
Fld_10 := Fld_9.MD;
--# expr-done 6758
Fld_11 := Fld_10.Dottable_Subp;
--# expr-done 6759
--# property-call-start CallExpr.subprogram_equation
Fld_12 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => Fld_11, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6760
--# expr-start 6762 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:10638
--# expr-start 6761 '<FieldAccess .parent_name(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10638
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6761
Is_Null := Fld_13 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_13); 
   If_Result := As_Entity;
end if;
--# expr-done 6762
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 6764 '<FieldAccess .parent_name_equation(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10640
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6763 '<FieldAccess .expr_type>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10640
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_14 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6763
--# property-call-start Name.parent_name_equation
Fld_15 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_14, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6764
Scope_Result := Fld_15;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2455;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10641") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_12, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10633") else null)); 
--# expr-done 6765
Or_Pred := Logic_Or (Fld_7, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10628") else null)); 
--# expr-done 6766
   If_Result_2 := Or_Pred;
else
   --# expr-start 6776 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10646
--# expr-start 6771 '<FieldAccess .subprogram_equation(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10647
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6767 '<FieldAccess .subp_spec_or_null>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10648
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_16 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => S.Node, E_Info => S.Info);
--# end
--# expr-done 6767
      Cast_Result_2 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
--# expr-start 6770 '<FieldAccess .dottable_subp>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10649
--# expr-start 6769 '<FieldAccess .md>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10649
--# expr-start 6768 '<FieldAccess .info>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10649
Fld_17 := S.Info;
--# expr-done 6768
Fld_18 := Fld_17.MD;
--# expr-done 6769
Fld_19 := Fld_18.Dottable_Subp;
--# expr-done 6770
--# property-call-start CallExpr.subprogram_equation
Fld_20 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_2, Dottable_Subp => Fld_19, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6771
--# expr-start 6773 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10651
--# expr-start 6772 '<FieldAccess .parent_name(...)>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10651
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_21 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6772
Is_Null_1 := Fld_21 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity_1 := (Info => E_Info, Node => Fld_21); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 6773
Var_Expr_1 := If_Result_1;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr_1
--# expr-start 6775 '<FieldAccess .parent_name_equation(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:10653
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6774 '<FieldAccess .expr_type>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10653
      if S.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_22 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => S.Node, Origin => Origin, E_Info => S.Info);
--# end
--# expr-done 6774
--# property-call-start Name.parent_name_equation
Fld_23 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr_1.Node, Typ => Fld_22, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6775
Scope_Result_1 := Fld_23;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2456;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10654") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Fld_20, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10646") else null)); 
--# expr-done 6776
   If_Result_2 := And_Pred_1;
end if;
      Inc_Ref (If_Result_2);
   If_Result_3 := If_Result_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 6777
And_Pred_2 := Logic_And (Bind_Result, If_Result_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10617") else null)); 
--# expr-done 6778
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 6779
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_861;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_861;
               Finalizer_Scope_2455;
               Finalizer_Scope_2456;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Entity_Equation;
--# end
--# property-start CallExpr.general_xref_equation /usr/src/packages/BUILD/ada/ast.py:10660
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_General_Xref_Equation
  (Node : Bare_Call_Expr
      ; Root : Bare_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_862;
            procedure Finalizer_Scope_2457;
            procedure Finalizer_Scope_2458;
            procedure Finalizer_Scope_2459;
            procedure Finalizer_Scope_2460;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Boolean;
Subps : Internal_Entity_Array_Access;
S : Internal_Entity_Basic_Decl;
Var_Expr_2 : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Name;
Is_A : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val_1 : Boolean;
If_Result : Boolean;
Fld_3 : Logic_Equation;
Fld_4 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Internal_Entity_Base_Type_Decl;
Fld_7 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Fld_8 : Internal_Entity_Name;
Is_A_1 : Boolean;
Fld_9 : Internal_Entity_Name;
Fld_10 : Boolean;
Fld_11 : Internal_Entity_Name;
Fld_12 : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Fld_13 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var_1 : Logic_Equation;
Fld_14 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Item_205 : Internal_Entity_Basic_Assoc;
Bare_Item_205 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_19 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_20 : Logic_Equation;
Fld_21 : Bare_Name;
Cast_Result_2 : Bare_Call_Expr;
As_Entity_2 : Internal_Entity_Call_Expr;
Is_Null_3 : Boolean;
If_Result_3 : Internal_Entity_Call_Expr;
Fld_22 : Logic_Equation;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_3 : Internal_Entity_Entry_Decl;
Fld_23 : Internal_Entity_Entry_Spec;
Fld_24 : Internal_Entity;
Scope_Result_2 : Internal_Entity;
Result_Var_2 : Internal_Entity;
Is_Null_4 : Boolean;
If_Result_4 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_25 : Logic_Var;
Fld_26 : Bare_Name;
Fld_27 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_28 : Internal_Entity_Name;
Fld_29 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
Fld_30 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Fld_31 : Internal_Entity_Name;
Fld_32 : Boolean;
If_Result_5 : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred_3 : Logic_Equation;
If_Result_6 : Logic_Equation;
If_Result_7 : Logic_Equation;
Let_Result_2 : Logic_Equation;
            procedure Finalizer_Scope_862 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_19);
                     Dec_Ref (Subps);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Fld_30);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (If_Result_7);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_862;
            procedure Finalizer_Scope_2457 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2457;
            procedure Finalizer_Scope_2458 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2458;
            procedure Finalizer_Scope_2459 is
            begin
                     Dec_Ref (Fld_18);
            end Finalizer_Scope_2459;
            procedure Finalizer_Scope_2460 is
            begin
                     Dec_Ref (Fld_20);
                     Dec_Ref (Fld_22);
                     Dec_Ref (If_Result_4);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2460;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6838 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 6837 '<Cond>' If_Result_7 /usr/src/packages/BUILD/ada/ast.py:10665
--# expr-start 6782 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10666
--# expr-start 6781 '<IsA QualExpr>' Is_A /usr/src/packages/BUILD/ada/ast.py:10666
--# expr-start 6780 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10666
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6780
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Qual_Expr_Range; 
--# expr-done 6781
Not_Val := not (Is_A); 
--# expr-done 6782
if Not_Val then
   --# expr-start 6786 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:10667
--# expr-start 6785 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10667
--# expr-start 6784 '<FieldAccess .name_designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10667
--# expr-start 6783 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10667
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6783
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_2 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 6784
Is_Null := Fld_2.Node = null; 
--# expr-done 6785
Not_Val_1 := not (Is_Null); 
--# expr-done 6786
   If_Result := Not_Val_1;
else
   If_Result := False;
end if;
if If_Result then
   --# expr-start 6793 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10670
--# expr-start 6787 '<FieldAccess .type_conv_xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10670
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.type_conv_xref_equation
Fld_3 := Libadalang.Implementation.Call_Expr_P_Type_Conv_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6787
--# expr-start 6789 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10671
--# expr-start 6788 '<FieldAccess .parent_name(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10671
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name (Node => Ent.Node, Stop_At => Root);
--# end
--# expr-done 6788
Is_Null_1 := Fld_4 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_4); 
   If_Result_1 := As_Entity;
end if;
--# expr-done 6789
Var_Expr := If_Result_1;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 6792 '<FieldAccess .parent_name_equation(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10673
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6791 '<FieldAccess .name_designated_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10674
--# expr-start 6790 '<FieldAccess .name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10674
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6790
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_6 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Fld_5.Node, E_Info => Fld_5.Info);
--# end
--# expr-done 6791
--# property-call-start Name.parent_name_equation
Fld_7 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_6, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6792
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2457;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10675") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_3, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10670") else null)); 
--# expr-done 6793
   If_Result_7 := And_Pred;
else
--# expr-start 6795 '<IsA AttributeRef>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:10681
--# expr-start 6794 '<FieldAccess .name>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10681
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6794
Is_A_1 := Fld_8.Node /= null 
and then Fld_8.Node.Kind in Ada_Attribute_Ref_Range; 
--# expr-done 6795
if Is_A_1 then
--# expr-start 6797 '<FieldAccess .resolve_names_internal>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10682
--# expr-start 6796 '<FieldAccess .name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10682
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6796
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_10 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld_9.Node, Env => Env, Origin => Origin, E_Info => Fld_9.Info);
--# end
--# expr-done 6797
Var_Expr_1 := Fld_10;
if Var_Expr_1 /= False then
   --# scope-start
   --# bind _ Var_Expr_1
--# expr-start 6801 '<FieldAccess .parent_name_equation(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10683
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6800 '<Cast to BaseTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10684
--# expr-start 6799 '<FieldAccess .type_val>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10684
--# expr-start 6798 '<FieldAccess .name>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10684
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6798
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_12 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_11.Node);
--# end
--# expr-done 6799
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         Cast_Result := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 6800
--# property-call-start Name.parent_name_equation
Fld_13 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Cast_Result, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6801
Scope_Result_1 := Fld_13;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2458;
   Result_Var_1 := Scope_Result_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10686") else null)); 
   Result_Var_1 := Logic_False;
end if;
      Inc_Ref (Result_Var_1);
   If_Result_6 := Result_Var_1;
else
   --# expr-start 6836 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:10689
--# expr-start 6806 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:10690
--# expr-start 6805 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10690
--# expr-start 6802 '<FieldAccess .params>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10690
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6802
Fld_15 := Fld_14.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_15;
      begin
         for Untyped_Item_205 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_205 := Untyped_Item_205; 
Is_Null_2 := Bare_Item_205 = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_1 := (Info => Fld_16, Node => Bare_Item_205); 
   If_Result_2 := As_Entity_1;
end if;
                  Item_205 := If_Result_2; 
   --# scope-start
               --# bind pa Item_205
      --# expr-start 6804 '<FieldAccess .sub_equation>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10690
--# expr-start 6803 '<FieldAccess .expr>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10690
      if Item_205.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_205.Node, E_Info => Item_205.Info);
--# end
--# expr-done 6803
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 6804
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_18;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2459;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6805
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10690") else null)); 
--# expr-done 6806
--# expr-start 6835 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:10694
--# expr-start 6830 '<Let subps>' Let_Result_1 /usr/src/packages/BUILD/ada/ast.py:10694
--# scope-start
--# expr-start 6807 '<FieldAccess .env_elements>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10694
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_19 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 6807
Subps := Fld_19; Inc_Ref (Subps);
--# bind subps Subps
--# expr-start 6829 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10694
--# expr-start 6826 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10694
--# expr-start 6821 '<LogicAny>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:10695
--# expr-start 6820 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:10695
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Subps;
      begin
         for Item_206 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_206
      --# expr-start 6819 '<Let s>' Let_Result /usr/src/packages/BUILD/ada/ast.py:10695
--# scope-start
--# expr-start 6808 '<Cast to BasicDecl.entity>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:10696
Cast_Expr := Item_206; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6808
S := Cast_Result_1; 
--# bind s S
--# expr-start 6818 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:10697
--# expr-start 6812 '<IsNull>' Is_Null_4 /usr/src/packages/BUILD/ada/ast.py:10698
--# expr-start 6809 '<Cast to EntryDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:10698
Cast_Expr_1 := S; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_3 := No_Entity_Entry_Decl;
   end if;
--# expr-done 6809
Var_Expr_2 := Cast_Result_3;
if Var_Expr_2 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 6811 '<FieldAccess .family_type>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:10698
--# expr-start 6810 '<FieldAccess .spec>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:10698
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_23 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_2.Node.Entry_Decl_F_Spec, Info => Var_Expr_2.Info);
--# expr-done 6810
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity (Node => Fld_23.Node.Entry_Spec_F_Family_Type, Info => Fld_23.Info);
--# expr-done 6811
Scope_Result_2 := Fld_24;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Entity;
end if;
Is_Null_4 := Result_Var_2.Node = null; 
--# expr-done 6812
if Is_Null_4 then
   --# expr-start 6813 '<FieldAccess .entity_equation(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10699
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.entity_equation
Fld_20 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => Ent.Node, S => S, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6813
   If_Result_4 := Fld_20;
else
   --# expr-start 6817 '<FieldAccess .entity_equation(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10700
--# expr-start 6816 '<as_entity(<Cast to CallExpr>)>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:10700
--# expr-start 6815 '<Cast to CallExpr>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:10700
--# expr-start 6814 '<FieldAccess .parent_name(...)>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10700
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_21 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 6814
   if Fld_21 = null
      or else Fld_21.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_2 := Fld_21;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 6815
Is_Null_3 := Cast_Result_2 = null; 
if Is_Null_3 then
   If_Result_3 := No_Entity_Call_Expr;
else
As_Entity_2 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_3 := As_Entity_2;
end if;
--# expr-done 6816
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.entity_equation
Fld_22 := Libadalang.Implementation.Call_Expr_P_Entity_Equation (Node => If_Result_3.Node, S => S, Root => Root, Env => Env, Origin => Origin, E_Info => If_Result_3.Info);
--# end
--# expr-done 6817
   If_Result_4 := Fld_22;
end if;
      Inc_Ref (If_Result_4);
--# expr-done 6818
Let_Result := If_Result_4; Inc_Ref (Let_Result);
--# end
--# expr-done 6819
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2460;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 6820
Logic_Boolean_Op_1 := Logic_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10695") else null)); 
--# expr-done 6821
--# expr-start 6825 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10704
--# expr-start 6822 '<FieldAccess .ref_var>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:10704
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 6822
Fld_25.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_25.all);
Eq_Node.Refs.Destroy (Fld_25.all);
--# expr-start 6824 '<FieldAccess .ref_var>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:10704
--# expr-start 6823 '<FieldAccess .name>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:10704
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.Call_Expr_F_Name;
--# expr-done 6823
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_26);
--# end
--# expr-done 6824
Fld_27.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_27.all);
Eq_Node.Refs.Destroy (Fld_27.all);
Bind_Result := Bind_Default_Default.Create (Fld_25, Fld_27, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10704") else null)); 
--# expr-done 6825
And_Pred_1 := Logic_And (Logic_Boolean_Op_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10694") else null)); 
--# expr-done 6826
--# expr-start 6828 '<FieldAccess .sub_equation>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:10705
--# expr-start 6827 '<FieldAccess .name>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:10705
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6827
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_29 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_28.Node, Env => Env, Origin => Origin, E_Info => Fld_28.Info);
--# end
--# expr-done 6828
And_Pred_2 := Logic_And (And_Pred_1, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10694") else null)); 
--# expr-done 6829
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6830
--# expr-start 6834 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:10706
--# expr-start 6832 '<FieldAccess .is_simple_name>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:10706
--# expr-start 6831 '<FieldAccess .name>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:10706
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_31 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6831
      if Fld_31.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_simple_name
Fld_32 := Libadalang.Implementation.Name_P_Is_Simple_Name (Node => Fld_31.Node);
--# end
--# expr-done 6832
if Fld_32 then
   --# expr-start 6833 '<FieldAccess .operator_equation>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:10707
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.operator_equation
Fld_30 := Libadalang.Implementation.Call_Expr_P_Operator_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6833
   If_Result_5 := Fld_30;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10708") else null)); 
   If_Result_5 := Logic_False_1;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 6834
Or_Pred := Logic_Or (Let_Result_1, If_Result_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10694") else null)); 
--# expr-done 6835
And_Pred_3 := Logic_And (Logic_Boolean_Op, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10689") else null)); 
--# expr-done 6836
   If_Result_6 := And_Pred_3;
end if;
      Inc_Ref (If_Result_6);
   If_Result_7 := If_Result_6;
end if;
      Inc_Ref (If_Result_7);
--# expr-done 6837
Let_Result_2 := If_Result_7; Inc_Ref (Let_Result_2);
--# end
--# expr-done 6838
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_862;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_862;
               Finalizer_Scope_2457;
               Finalizer_Scope_2458;
               Finalizer_Scope_2459;
               Finalizer_Scope_2460;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_General_Xref_Equation;
--# end
--# property-start CallExpr.operator_equation /usr/src/packages/BUILD/ada/ast.py:10715
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Operator_Equation
  (Node : Bare_Call_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_863;
            procedure Finalizer_Scope_2462;
            procedure Finalizer_Scope_2463;
            procedure Finalizer_Scope_2464;
      Block_Var : Symbol_Type;
Var_Expr : Internal_Single_Actual_Array_Access;
Var_Expr_1 : Internal_Entity_Assoc_List;
Base_Name_Eq : Logic_Equation;
Var_Expr_2 : Internal_Entity_Name;
Fld : Internal_Entity_Name;
Fld_1 : Symbol_Type;
Fld_2 : Internal_Entity_Assoc_List;
Fld_3 : Internal_Single_Actual_Array_Access;
Scope_Result : Internal_Single_Actual_Array_Access;
Result_Var : Internal_Single_Actual_Array_Access;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_1 : Logic_Equation;
Len : Integer;
Is_Equal : Boolean;
Sym : Symbol_Type;
Is_Equal_1 : Boolean;
Sym_1 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result : Boolean;
Sym_2 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_1 : Boolean;
Sym_3 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_2 : Boolean;
Sym_4 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_3 : Boolean;
Sym_5 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_4 : Boolean;
Sym_6 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_5 : Boolean;
If_Result_6 : Boolean;
Get_Result : Internal_Single_Actual;
Fld_7 : Internal_Entity_Basic_Assoc;
Fld_8 : Internal_Entity_Expr;
Fld_9 : Logic_Var;
Get_Result_1 : Internal_Single_Actual;
Fld_10 : Internal_Entity_Basic_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Var;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Len_1 : Integer;
Is_Equal_8 : Boolean;
Sym_7 : Symbol_Type;
Is_Equal_9 : Boolean;
Sym_8 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_7 : Boolean;
Sym_9 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_8 : Boolean;
Sym_10 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_9 : Boolean;
Sym_11 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_10 : Boolean;
Sym_12 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_11 : Boolean;
Sym_13 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_12 : Boolean;
Sym_14 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_13 : Boolean;
Sym_15 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_14 : Boolean;
Sym_16 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_15 : Boolean;
Sym_17 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_16 : Boolean;
If_Result_17 : Boolean;
Get_Result_2 : Internal_Single_Actual;
Fld_16 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Fld_18 : Logic_Var;
Get_Result_3 : Internal_Single_Actual;
Fld_19 : Internal_Entity_Basic_Assoc;
Fld_20 : Internal_Entity_Expr;
Fld_21 : Logic_Var;
Fld_22 : Logic_Equation;
Get_Result_4 : Internal_Single_Actual;
Fld_23 : Internal_Entity_Basic_Assoc;
Fld_24 : Internal_Entity_Expr;
Fld_25 : Logic_Var;
Fld_26 : Logic_Var;
Fld_27 : Logic_Equation;
And_Pred_2 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Len_2 : Integer;
Is_Equal_20 : Boolean;
Sym_18 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_18 : Boolean;
Get_Result_5 : Internal_Single_Actual;
Fld_28 : Internal_Entity_Basic_Assoc;
Fld_29 : Internal_Entity_Expr;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity;
Fld_32 : Logic_Equation;
Get_Result_6 : Internal_Single_Actual;
Fld_33 : Internal_Entity_Basic_Assoc;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Var;
Fld_36 : Logic_Var;
Fld_37 : Logic_Equation;
And_Pred_4 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Len_3 : Integer;
Is_Equal_22 : Boolean;
Sym_19 : Symbol_Type;
Is_Equal_23 : Boolean;
Sym_20 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_19 : Boolean;
Sym_21 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_20 : Boolean;
Sym_22 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_21 : Boolean;
If_Result_22 : Boolean;
Get_Result_7 : Internal_Single_Actual;
Fld_38 : Internal_Entity_Basic_Assoc;
Fld_39 : Internal_Entity_Expr;
Fld_40 : Logic_Var;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_23 : Logic_Equation;
If_Result_24 : Logic_Equation;
If_Result_25 : Logic_Equation;
If_Result_26 : Logic_Equation;
Let_Result : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Result_Var_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_863 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_863;
            procedure Finalizer_Scope_2462 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2462;
            procedure Finalizer_Scope_2463 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Base_Name_Eq);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_22);
                     Dec_Ref (Fld_27);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Fld_32);
                     Dec_Ref (Fld_37);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_42);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (If_Result_26);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2463;
            procedure Finalizer_Scope_2464 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2464;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 6953 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 6840 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10719
--# expr-start 6839 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:10719
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6839
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 6840
Block_Var := Fld_1; 
--# bind rel_name Block_Var
--# expr-start 6841 '<FieldAccess .params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10721
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_2 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6841
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 6842 '<FieldAccess .unpacked_params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10721
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AssocList.unpacked_params
Fld_3 := Libadalang.Implementation.Assoc_List_P_Unpacked_Params (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6842
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2462;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Single_Actual_Array_Type;
end if;
      Inc_Ref (Result_Var);
Var_Expr := Result_Var;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Single_Actual_Array_Type then
   --# scope-start
   --# bind params Var_Expr
--# expr-start 6952 '<Let base_name_eq>' Let_Result /usr/src/packages/BUILD/ada/ast.py:10722
--# scope-start
--# expr-start 6844 '<FieldAccess .base_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10723
--# expr-start 6843 '<FieldAccess .name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10723
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Call_Expr_F_Name, Info => Ent.Info);
--# expr-done 6843
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.base_name'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Base_Name (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 6844
Var_Expr_2 := Fld_5;
if Var_Expr_2 /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr_2
--# expr-start 6845 '<FieldAccess .sub_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10724
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_2.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 6845
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2464;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10725") else null)); 
   Result_Var_1 := Logic_True;
end if;
      Inc_Ref (Result_Var_1);
Base_Name_Eq := Result_Var_1; Inc_Ref (Base_Name_Eq);
--# bind base_name_eq Base_Name_Eq
--# expr-start 6951 '<Cond>' If_Result_26 /usr/src/packages/BUILD/ada/ast.py:10726
--# expr-start 6847 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10727
--# expr-start 6846 '<length(<Var params>)>' Len /usr/src/packages/BUILD/ada/ast.py:10727
Len := Length (Var_Expr); 
--# expr-done 6846
Is_Equal := Len = 2; 
--# expr-done 6847
if Is_Equal then
--# expr-start 6849 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6848 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 6848
Is_Equal_1 := Block_Var = Sym; 
--# expr-done 6849
if Is_Equal_1 then
   If_Result := True;
else
   --# expr-start 6851 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6850 '<Symbol "=">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 6850
Is_Equal_2 := Block_Var = Sym_1; 
--# expr-done 6851
   If_Result := Is_Equal_2;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 6853 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6852 '<Symbol "/=">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 6852
Is_Equal_3 := Block_Var = Sym_2; 
--# expr-done 6853
   If_Result_1 := Is_Equal_3;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 6855 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6854 '<Symbol "<">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 6854
Is_Equal_4 := Block_Var = Sym_3; 
--# expr-done 6855
   If_Result_2 := Is_Equal_4;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 6857 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6856 '<Symbol "<=">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 6856
Is_Equal_5 := Block_Var = Sym_4; 
--# expr-done 6857
   If_Result_3 := Is_Equal_5;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 6859 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6858 '<Symbol ">">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 6858
Is_Equal_6 := Block_Var = Sym_5; 
--# expr-done 6859
   If_Result_4 := Is_Equal_6;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 6861 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:10728
--# expr-start 6860 '<Symbol ">=">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 6860
Is_Equal_7 := Block_Var = Sym_6; 
--# expr-done 6861
   If_Result_5 := Is_Equal_7;
end if;
   If_Result_6 := If_Result_5;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 6874 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10730
--# expr-start 6873 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10730
--# expr-start 6870 '<FieldAccess .type_bind_var(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10730
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6865 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10730
--# expr-start 6864 '<FieldAccess .expr>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10730
--# expr-start 6863 '<FieldAccess .assoc>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10730
--# expr-start 6862 '<at(<Var params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:10730
Get_Result := Get (Var_Expr, 0, True); 
--# expr-done 6862
Fld_7 := Get_Result.Assoc;
--# expr-done 6863
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_8 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_7.Node, E_Info => Fld_7.Info);
--# end
--# expr-done 6864
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_8.Node);
--# expr-done 6865
--# expr-start 6869 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10731
--# expr-start 6868 '<FieldAccess .expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10731
--# expr-start 6867 '<FieldAccess .assoc>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10731
--# expr-start 6866 '<at(<Var params>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:10731
Get_Result_1 := Get (Var_Expr, 1, True); 
--# expr-done 6866
Fld_10 := Get_Result_1.Assoc;
--# expr-done 6867
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_10.Node, E_Info => Fld_10.Info);
--# end
--# expr-done 6868
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11.Node);
--# expr-done 6869
--# property-call-start AdaNode.type_bind_var
Fld_13 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_9, Right => Fld_12, Origin => Origin);
--# end
--# expr-done 6870
--# expr-start 6872 '<FieldAccess .bool_bind(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10732
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6871 '<FieldAccess .type_var>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10732
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6871
--# property-call-start AdaNode.bool_bind
Fld_15 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_14, Origin => Origin);
--# end
--# expr-done 6872
And_Pred := Logic_And (Fld_13, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10730") else null)); 
--# expr-done 6873
And_Pred_1 := Logic_And (And_Pred, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10730") else null)); 
--# expr-done 6874
   If_Result_26 := And_Pred_1;
else
--# expr-start 6876 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:10736
--# expr-start 6875 '<length(<Var params>)>' Len_1 /usr/src/packages/BUILD/ada/ast.py:10736
Len_1 := Length (Var_Expr); 
--# expr-done 6875
Is_Equal_8 := Len_1 = 2; 
--# expr-done 6876
if Is_Equal_8 then
--# expr-start 6878 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6877 '<Symbol "and">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 6877
Is_Equal_9 := Block_Var = Sym_7; 
--# expr-done 6878
if Is_Equal_9 then
   If_Result_7 := True;
else
   --# expr-start 6880 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6879 '<Symbol "or">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 6879
Is_Equal_10 := Block_Var = Sym_8; 
--# expr-done 6880
   If_Result_7 := Is_Equal_10;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 6882 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6881 '<Symbol "xor">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 6881
Is_Equal_11 := Block_Var = Sym_9; 
--# expr-done 6882
   If_Result_8 := Is_Equal_11;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 6884 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6883 '<Symbol "abs">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 6883
Is_Equal_12 := Block_Var = Sym_10; 
--# expr-done 6884
   If_Result_9 := Is_Equal_12;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 6886 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6885 '<Symbol "*">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 6885
Is_Equal_13 := Block_Var = Sym_11; 
--# expr-done 6886
   If_Result_10 := Is_Equal_13;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 6888 '<Eq>' Is_Equal_14 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6887 '<Symbol "/">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 6887
Is_Equal_14 := Block_Var = Sym_12; 
--# expr-done 6888
   If_Result_11 := Is_Equal_14;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 6890 '<Eq>' Is_Equal_15 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6889 '<Symbol "mod">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 6889
Is_Equal_15 := Block_Var = Sym_13; 
--# expr-done 6890
   If_Result_12 := Is_Equal_15;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 6892 '<Eq>' Is_Equal_16 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6891 '<Symbol "rem">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 6891
Is_Equal_16 := Block_Var = Sym_14; 
--# expr-done 6892
   If_Result_13 := Is_Equal_16;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 6894 '<Eq>' Is_Equal_17 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6893 '<Symbol "+">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 6893
Is_Equal_17 := Block_Var = Sym_15; 
--# expr-done 6894
   If_Result_14 := Is_Equal_17;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 6896 '<Eq>' Is_Equal_18 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6895 '<Symbol "-">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 6895
Is_Equal_18 := Block_Var = Sym_16; 
--# expr-done 6896
   If_Result_15 := Is_Equal_18;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 6898 '<Eq>' Is_Equal_19 /usr/src/packages/BUILD/ada/ast.py:10737
--# expr-start 6897 '<Symbol "&">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol); 
--# expr-done 6897
Is_Equal_19 := Block_Var = Sym_17; 
--# expr-done 6898
   If_Result_16 := Is_Equal_19;
end if;
   If_Result_17 := If_Result_16;
else
   If_Result_17 := False;
end if;
if If_Result_17 then
   --# expr-start 6915 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:10741
--# expr-start 6914 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10741
--# expr-start 6907 '<FieldAccess .type_bind_var(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10741
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6902 '<FieldAccess .type_var>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10741
--# expr-start 6901 '<FieldAccess .expr>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10741
--# expr-start 6900 '<FieldAccess .assoc>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10741
--# expr-start 6899 '<at(<Var params>, 0)>' Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:10741
Get_Result_2 := Get (Var_Expr, 0, True); 
--# expr-done 6899
Fld_16 := Get_Result_2.Assoc;
--# expr-done 6900
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_16.Node, E_Info => Fld_16.Info);
--# end
--# expr-done 6901
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_17.Node);
--# expr-done 6902
--# expr-start 6906 '<FieldAccess .type_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10742
--# expr-start 6905 '<FieldAccess .expr>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10742
--# expr-start 6904 '<FieldAccess .assoc>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10742
--# expr-start 6903 '<at(<Var params>, 1)>' Get_Result_3 /usr/src/packages/BUILD/ada/ast.py:10742
Get_Result_3 := Get (Var_Expr, 1, True); 
--# expr-done 6903
Fld_19 := Get_Result_3.Assoc;
--# expr-done 6904
      if Fld_19.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_19.Node, E_Info => Fld_19.Info);
--# end
--# expr-done 6905
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20.Node);
--# expr-done 6906
--# property-call-start AdaNode.type_bind_var
Fld_22 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_18, Right => Fld_21, Origin => Origin);
--# end
--# expr-done 6907
--# expr-start 6913 '<FieldAccess .type_bind_var(...)>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:10743
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6911 '<FieldAccess .type_var>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:10743
--# expr-start 6910 '<FieldAccess .expr>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:10743
--# expr-start 6909 '<FieldAccess .assoc>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:10743
--# expr-start 6908 '<at(<Var params>, 0)>' Get_Result_4 /usr/src/packages/BUILD/ada/ast.py:10743
Get_Result_4 := Get (Var_Expr, 0, True); 
--# expr-done 6908
Fld_23 := Get_Result_4.Assoc;
--# expr-done 6909
      if Fld_23.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_24 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_23.Node, E_Info => Fld_23.Info);
--# end
--# expr-done 6910
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_25 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_24.Node);
--# expr-done 6911
--# expr-start 6912 '<FieldAccess .type_var>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:10744
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6912
--# property-call-start AdaNode.type_bind_var
Fld_27 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_25, Right => Fld_26, Origin => Origin);
--# end
--# expr-done 6913
And_Pred_2 := Logic_And (Fld_22, Fld_27, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10741") else null)); 
--# expr-done 6914
And_Pred_3 := Logic_And (And_Pred_2, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10741") else null)); 
--# expr-done 6915
   If_Result_25 := And_Pred_3;
else
--# expr-start 6917 '<Eq>' Is_Equal_20 /usr/src/packages/BUILD/ada/ast.py:10748
--# expr-start 6916 '<length(<Var params>)>' Len_2 /usr/src/packages/BUILD/ada/ast.py:10748
Len_2 := Length (Var_Expr); 
--# expr-done 6916
Is_Equal_20 := Len_2 = 2; 
--# expr-done 6917
if Is_Equal_20 then
   --# expr-start 6919 '<Eq>' Is_Equal_21 /usr/src/packages/BUILD/ada/ast.py:10748
--# expr-start 6918 '<Symbol "**">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_3); 
--# expr-done 6918
Is_Equal_21 := Block_Var = Sym_18; 
--# expr-done 6919
   If_Result_18 := Is_Equal_21;
else
   If_Result_18 := False;
end if;
if If_Result_18 then
   --# expr-start 6933 '<And>' And_Pred_5 /usr/src/packages/BUILD/ada/ast.py:10749
--# expr-start 6932 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:10749
--# expr-start 6925 '<FieldAccess .type_bind_val(...)>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:10749
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6923 '<FieldAccess .type_var>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:10749
--# expr-start 6922 '<FieldAccess .expr>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:10749
--# expr-start 6921 '<FieldAccess .assoc>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:10749
--# expr-start 6920 '<at(<Var params>, 1)>' Get_Result_5 /usr/src/packages/BUILD/ada/ast.py:10749
Get_Result_5 := Get (Var_Expr, 1, True); 
--# expr-done 6920
Fld_28 := Get_Result_5.Assoc;
--# expr-done 6921
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_28.Node, E_Info => Fld_28.Info);
--# end
--# expr-done 6922
      if Fld_29.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_29.Node);
--# expr-done 6923
--# expr-start 6924 '<FieldAccess .universal_int_type>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:10750
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.universal_int_type
Fld_31 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Type (Node => Self);
--# end
--# expr-done 6924
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_30, Right => Fld_31, Origin => Origin);
--# end
--# expr-done 6925
--# expr-start 6931 '<FieldAccess .type_bind_var(...)>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:10751
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6929 '<FieldAccess .type_var>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:10751
--# expr-start 6928 '<FieldAccess .expr>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:10751
--# expr-start 6927 '<FieldAccess .assoc>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:10751
--# expr-start 6926 '<at(<Var params>, 0)>' Get_Result_6 /usr/src/packages/BUILD/ada/ast.py:10751
Get_Result_6 := Get (Var_Expr, 0, True); 
--# expr-done 6926
Fld_33 := Get_Result_6.Assoc;
--# expr-done 6927
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_34 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_33.Node, E_Info => Fld_33.Info);
--# end
--# expr-done 6928
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_34.Node);
--# expr-done 6929
--# expr-start 6930 '<FieldAccess .type_var>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:10752
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6930
--# property-call-start AdaNode.type_bind_var
Fld_37 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_35, Right => Fld_36, Origin => Origin);
--# end
--# expr-done 6931
And_Pred_4 := Logic_And (Fld_32, Fld_37, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10749") else null)); 
--# expr-done 6932
And_Pred_5 := Logic_And (And_Pred_4, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10749") else null)); 
--# expr-done 6933
   If_Result_24 := And_Pred_5;
else
--# expr-start 6935 '<Eq>' Is_Equal_22 /usr/src/packages/BUILD/ada/ast.py:10756
--# expr-start 6934 '<length(<Var params>)>' Len_3 /usr/src/packages/BUILD/ada/ast.py:10756
Len_3 := Length (Var_Expr); 
--# expr-done 6934
Is_Equal_22 := Len_3 = 1; 
--# expr-done 6935
if Is_Equal_22 then
--# expr-start 6937 '<Eq>' Is_Equal_23 /usr/src/packages/BUILD/ada/ast.py:10757
--# expr-start 6936 '<Symbol "+">' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 6936
Is_Equal_23 := Block_Var = Sym_19; 
--# expr-done 6937
if Is_Equal_23 then
   If_Result_19 := True;
else
   --# expr-start 6939 '<Eq>' Is_Equal_24 /usr/src/packages/BUILD/ada/ast.py:10757
--# expr-start 6938 '<Symbol "-">' Sym_20 None
Sym_20 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 6938
Is_Equal_24 := Block_Var = Sym_20; 
--# expr-done 6939
   If_Result_19 := Is_Equal_24;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 6941 '<Eq>' Is_Equal_25 /usr/src/packages/BUILD/ada/ast.py:10757
--# expr-start 6940 '<Symbol "not">' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 6940
Is_Equal_25 := Block_Var = Sym_21; 
--# expr-done 6941
   If_Result_20 := Is_Equal_25;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 6943 '<Eq>' Is_Equal_26 /usr/src/packages/BUILD/ada/ast.py:10757
--# expr-start 6942 '<Symbol "abs">' Sym_22 None
Sym_22 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 6942
Is_Equal_26 := Block_Var = Sym_22; 
--# expr-done 6943
   If_Result_21 := Is_Equal_26;
end if;
   If_Result_22 := If_Result_21;
else
   If_Result_22 := False;
end if;
if If_Result_22 then
   --# expr-start 6950 '<And>' And_Pred_6 /usr/src/packages/BUILD/ada/ast.py:10758
--# expr-start 6949 '<FieldAccess .type_bind_var(...)>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:10758
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6947 '<FieldAccess .type_var>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:10758
--# expr-start 6946 '<FieldAccess .expr>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:10758
--# expr-start 6945 '<FieldAccess .assoc>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:10758
--# expr-start 6944 '<at(<Var params>, 0)>' Get_Result_7 /usr/src/packages/BUILD/ada/ast.py:10758
Get_Result_7 := Get (Var_Expr, 0, True); 
--# expr-done 6944
Fld_38 := Get_Result_7.Assoc;
--# expr-done 6945
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_39 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_38.Node, E_Info => Fld_38.Info);
--# end
--# expr-done 6946
      if Fld_39.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_39.Node);
--# expr-done 6947
--# expr-start 6948 '<FieldAccess .type_var>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:10759
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6948
--# property-call-start AdaNode.type_bind_var
Fld_42 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_40, Right => Fld_41, Origin => Origin);
--# end
--# expr-done 6949
And_Pred_6 := Logic_And (Fld_42, Base_Name_Eq, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10758") else null)); 
--# expr-done 6950
   If_Result_23 := And_Pred_6;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10762") else null)); 
   If_Result_23 := Logic_False;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
   If_Result_26 := If_Result_25;
end if;
      Inc_Ref (If_Result_26);
--# expr-done 6951
Let_Result := If_Result_26; Inc_Ref (Let_Result);
--# end
--# expr-done 6952
Scope_Result_2 := Let_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2463;
   Result_Var_2 := Scope_Result_2;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10765") else null)); 
   Result_Var_2 := Logic_False_1;
end if;
      Inc_Ref (Result_Var_2);
Let_Result_1 := Result_Var_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 6953
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_863;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_863;
               Finalizer_Scope_2462;
               Finalizer_Scope_2463;
               Finalizer_Scope_2464;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Operator_Equation;
--# end
--# property-start CallExpr.subscriptable_type_equation /usr/src/packages/BUILD/ada/ast.py:10769
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subscriptable_Type_Equation
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Constrain_Params : Boolean
         := True
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind constrain_params Constrain_Params
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_864;
            procedure Finalizer_Scope_2467;
            procedure Finalizer_Scope_2468;
            procedure Finalizer_Scope_2470;
            procedure Finalizer_Scope_2472;
            procedure Finalizer_Scope_2473;
            procedure Finalizer_Scope_2474;
            procedure Finalizer_Scope_2475;
            procedure Finalizer_Scope_2476;
            procedure Finalizer_Scope_2477;
            procedure Finalizer_Scope_2478;
            procedure Finalizer_Scope_2479;
      Block_Var : Internal_Entity_Array_Type_Def;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_3 : Internal_Entity_Array_Type_Def;
Match_0 : Internal_Entity_Assoc_List;
Match_1 : Internal_Entity_Bin_Op;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Subtype_Indication;
Match_4 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Name;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Assoc_List;
Formal : Internal_Single_Formal;
Ret_Type : Internal_Entity_Base_Type_Decl;
Param : Internal_Entity_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Fld_3 : Internal_Entity_Access_Def;
Is_A : Boolean;
Fld_4 : Internal_Entity_Access_Def;
Cast_Result : Internal_Entity_Access_To_Subp_Def;
Fld_5 : Internal_Entity_Subp_Spec;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_6 : Logic_Equation;
Fld_7 : Internal_Entity_Assoc_List;
Fld_8 : Bare_Assoc_List;
Fld_9 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null : Boolean;
If_Result_1 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
Item_207 : Internal_Entity_Basic_Assoc;
Bare_Item_207 : Bare_Basic_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var_2 : Logic_Equation;
Fld_12 : Internal_Entity_Array_Indices;
Scope_Result_3 : Internal_Entity_Array_Indices;
Result_Var_3 : Internal_Entity_Array_Indices;
Is_Null_1 : Boolean;
Not_Val : Boolean;
Fld_13 : Internal_Entity;
Fld_14 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
Get_Result : Bare_Basic_Assoc;
As_Entity_1 : Internal_Entity_Basic_Assoc;
Is_Null_2 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_4 : Internal_Entity_Expr;
Result_Var_4 : Internal_Entity_Expr;
Cast_Result_2 : Internal_Entity_Name;
Logic_False_1 : Logic_Equation;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_20 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_21 : Internal_Entity_Base_Type_Decl;
Is_Null_3 : Boolean;
If_Result_3 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Logic_False_2 : Logic_Equation;
Result_Var_5 : Logic_Equation;
Fld_22 : Internal_Entity_Assoc_List;
Saved : Internal_Entity_Assoc_List;
Fld_23 : Bare_Assoc_List;
Fld_24 : Internal_Entity_Info;
As_Entity_2 : Internal_Entity_Basic_Assoc;
Is_Null_4 : Boolean;
If_Result_4 : Internal_Entity_Basic_Assoc;
Fld_25 : Internal_Entity_Expr;
Fld_26 : Logic_Equation;
Logic_True : Logic_Equation;
If_Result_5 : Logic_Equation;
Fld_27 : Internal_Entity_Array_Indices;
Fld_28 : Internal_Entity_Expr;
Fld_29 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Item_208 : Internal_Entity_Basic_Assoc;
Bare_Item_208 : Bare_Basic_Assoc;
I : Integer;
Map_Result_1 : Logic_Equation_Array_Access;
Scope_Result_6 : Logic_Equation_Array_Access;
Result_Var_6 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Fld_30 : Logic_Var;
Fld_31 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_32 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_33 : Internal_Entity_Array_Indices;
Fld_34 : Internal_Entity_Expr;
Fld_35 : Logic_Equation;
Fld_36 : Internal_Entity_Array_Indices;
Fld_37 : Internal_Entity_Expr;
Fld_38 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Fld_39 : Logic_Var;
Fld_40 : Internal_Entity_Expr;
Fld_41 : Logic_Var;
Fld_42 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Fld_43 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_5 : Internal_Entity;
Fld_44 : Logic_Equation;
And_Pred_6 : Logic_Equation;
Fld_45 : Internal_Entity_Expr;
Fld_46 : Logic_Equation;
And_Pred_7 : Logic_Equation;
Fld_47 : Internal_Entity_Expr;
Fld_48 : Logic_Equation;
And_Pred_8 : Logic_Equation;
Fld_49 : Logic_Equation;
Fld_50 : Internal_Entity_Array_Indices;
Cast_Expr_2 : Internal_Entity_Attribute_Ref;
Cast_Result_6 : Internal_Entity_Expr;
Fld_51 : Logic_Equation;
And_Pred_9 : Logic_Equation;
Fld_52 : Logic_Var;
Cast_Expr_3 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Fld_53 : Logic_Equation;
And_Pred_10 : Logic_Equation;
Fld_54 : Logic_Equation;
Fld_55 : Logic_Var;
Cast_Expr_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Fld_56 : Logic_Equation;
And_Pred_11 : Logic_Equation;
Logic_False_3 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_9 : Internal_Entity_Assoc_List;
Let_Result : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Cast_Result_10 : Internal_Entity_Bin_Op;
Let_Result_1 : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Cast_Result_11 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_9 : Logic_Equation;
Cast_Result_12 : Internal_Entity_Subtype_Indication;
Let_Result_3 : Logic_Equation;
Scope_Result_10 : Logic_Equation;
Cast_Result_13 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_11 : Logic_Equation;
Match_Result : Logic_Equation;
Is_Null_5 : Boolean;
Not_Val_1 : Boolean;
Fld_57 : Boolean;
If_Result_6 : Boolean;
Fld_58 : Internal_Entity_Basic_Decl_Array_Access;
Fld_59 : Internal_Entity_Basic_Decl_Array_Access;
Concat_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_60 : Internal_Entity_Base_Subp_Spec;
Fld_61 : Internal_Single_Formal_Array_Access;
Get_Result_1 : Internal_Single_Formal;
Fld_62 : Internal_Entity_Base_Subp_Spec;
Fld_63 : Internal_Entity_Base_Type_Decl;
Fld_64 : Internal_Entity_Assoc_List;
Fld_65 : Bare_Assoc_List;
Fld_66 : Internal_Entity_Info;
Get_Result_2 : Bare_Basic_Assoc;
As_Entity_3 : Internal_Entity_Basic_Assoc;
Is_Null_6 : Boolean;
If_Result_7 : Internal_Entity_Basic_Assoc;
Fld_67 : Internal_Entity_Expr;
Fld_68 : Logic_Var;
Cast_Expr_5 : Internal_Entity_Base_Type_Decl;
Cast_Result_14 : Internal_Entity;
Fld_69 : Logic_Equation;
Fld_70 : Logic_Equation;
Logic_True_1 : Logic_Equation;
If_Result_8 : Logic_Equation;
And_Pred_12 : Logic_Equation;
Fld_71 : Logic_Var;
Fld_72 : Internal_Entity_Base_Formal_Param_Decl;
Fld_73 : Internal_Entity_Base_Type_Decl;
Cast_Result_15 : Internal_Entity;
Fld_74 : Logic_Equation;
And_Pred_13 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op_2 : Logic_Equation;
Logic_False_4 : Logic_Equation;
If_Result_9 : Logic_Equation;
If_Result_10 : Logic_Equation;
If_Result_11 : Logic_Equation;
Let_Result_6 : Logic_Equation;
            procedure Finalizer_Scope_864 is
            begin
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_9);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_10);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_11);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Fld_58);
                     Dec_Ref (Fld_59);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op_2);
                     Dec_Ref (Logic_False_4);
                     Dec_Ref (If_Result_9);
                     Dec_Ref (If_Result_10);
                     Dec_Ref (If_Result_11);
                     Dec_Ref (Let_Result_6);
            end Finalizer_Scope_864;
            procedure Finalizer_Scope_2467 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2467;
            procedure Finalizer_Scope_2468 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2468;
            procedure Finalizer_Scope_2470 is
            begin
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Logic_False_2);
                     Dec_Ref (Result_Var_5);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Result_Var_6);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Fld_32);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2470;
            procedure Finalizer_Scope_2472 is
            begin
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_20);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_2472;
            procedure Finalizer_Scope_2473 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2473;
            procedure Finalizer_Scope_2474 is
            begin
                     Dec_Ref (Fld_26);
                     Dec_Ref (Logic_True);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (Fld_29);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2474;
            procedure Finalizer_Scope_2475 is
            begin
                     Dec_Ref (Fld_35);
                     Dec_Ref (Fld_38);
                     Dec_Ref (And_Pred_4);
                     Dec_Ref (Fld_42);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Fld_44);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Fld_46);
                     Dec_Ref (And_Pred_7);
                     Dec_Ref (Fld_48);
                     Dec_Ref (And_Pred_8);
            end Finalizer_Scope_2475;
            procedure Finalizer_Scope_2476 is
            begin
                     Dec_Ref (Fld_49);
                     Dec_Ref (Fld_51);
                     Dec_Ref (And_Pred_9);
                     Dec_Ref (Fld_53);
                     Dec_Ref (And_Pred_10);
            end Finalizer_Scope_2476;
            procedure Finalizer_Scope_2477 is
            begin
                     Dec_Ref (Fld_54);
                     Dec_Ref (Fld_56);
                     Dec_Ref (And_Pred_11);
            end Finalizer_Scope_2477;
            procedure Finalizer_Scope_2478 is
            begin
                     Dec_Ref (Logic_False_3);
            end Finalizer_Scope_2478;
            procedure Finalizer_Scope_2479 is
            begin
                     Dec_Ref (Fld_61);
                     Dec_Ref (Fld_69);
                     Dec_Ref (Fld_70);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (And_Pred_12);
                     Dec_Ref (Fld_74);
                     Dec_Ref (And_Pred_13);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_2479;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7059 '<Block>' Let_Result_6 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 6954 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:10775
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 6954
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr_1
--# expr-start 6957 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10777
--# expr-start 6955 '<FieldAccess .is_implicit_deref>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10777
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_2 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6955
if Fld_2 then
   --# expr-start 6956 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10777
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 6956
   If_Result := Fld_1;
else
   If_Result := Var_Expr_1;
end if;
--# expr-done 6957
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind real_typ Block_Var_1
--# expr-start 7058 '<Cond>' If_Result_11 /usr/src/packages/BUILD/ada/ast.py:10780
--# expr-start 6959 '<IsA AccessToSubpDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:10782
--# expr-start 6958 '<FieldAccess .access_def>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10782
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6958
Is_A := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Access_To_Subp_Def_Range; 
--# expr-done 6959
if Is_A then
--# expr-start 6961 '<Cast to AccessToSubpDef>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10783
--# expr-start 6960 '<FieldAccess .access_def>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10783
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 6960
   if Fld_4.Node = null
      or else Fld_4.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
   else
         Cast_Result := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 6961
Var_Expr_2 := Cast_Result;
if Var_Expr_2 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind asd Var_Expr_2
--# expr-start 6969 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10785
--# expr-start 6963 '<FieldAccess .subprogram_equation(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10785
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6962 '<FieldAccess .subp_spec>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10785
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_2.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_2.Info);
--# expr-done 6962
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
--# property-call-start CallExpr.subprogram_equation
Fld_6 := Libadalang.Implementation.Call_Expr_P_Subprogram_Equation (Node => Ent.Node, Subp_Spec => Cast_Result_1, Dottable_Subp => False, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 6963
--# expr-start 6968 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:10786
--# expr-start 6967 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10786
--# expr-start 6964 '<FieldAccess .params>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10786
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_7 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6964
Fld_8 := Fld_7.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_8 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_8;
      begin
         for Untyped_Item_207 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_207 := Untyped_Item_207; 
Is_Null := Bare_Item_207 = null; 
if Is_Null then
   If_Result_1 := No_Entity_Basic_Assoc;
else
Fld_9 := Fld_7.Info;
As_Entity := (Info => Fld_9, Node => Bare_Item_207); 
   If_Result_1 := As_Entity;
end if;
                  Item_207 := If_Result_1; 
   --# scope-start
               --# bind pa Item_207
      --# expr-start 6966 '<FieldAccess .sub_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10787
--# expr-start 6965 '<FieldAccess .expr>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10787
      if Item_207.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_207.Node, E_Info => Item_207.Info);
--# end
--# expr-done 6965
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_10.Node, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 6966
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_11;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2468;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 6967
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10786") else null)); 
--# expr-done 6968
And_Pred := Logic_And (Fld_6, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10785") else null)); 
--# expr-done 6969
Scope_Result_2 := And_Pred;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2467;
   Result_Var_2 := Scope_Result_2;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10789") else null)); 
   Result_Var_2 := Logic_False;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_11 := Result_Var_2;
else
--# expr-start 6972 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:10792
--# expr-start 6971 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:10792
Var_Expr_3 := Block_Var;
if Var_Expr_3 /= No_Entity_Array_Type_Def then
   --# scope-start
--# expr-start 6970 '<FieldAccess .indices>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10792
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Array_Indices (Node => Var_Expr_3.Node.Array_Type_Def_F_Indices, Info => Var_Expr_3.Info);
--# expr-done 6970
Scope_Result_3 := Fld_12;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity_Array_Indices;
end if;
Is_Null_1 := Result_Var_3.Node = null; 
--# expr-done 6971
Not_Val := not (Is_Null_1); 
--# expr-done 6972
if Not_Val then
   --# expr-start 7031 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10792
--# expr-start 6973 '<FieldAccess .suffix>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10792
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity (Node => Ent.Node.Call_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 6973
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_13; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Create_Internal_Entity_Assoc_List
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 6998 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:10793
--# expr-start 6976 '<Cast to Name>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:10795
--# expr-start 6974 '<FieldAccess .params>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10795
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6974
Fld_15 := Fld_14.Node;
Get_Result := Get (Fld_15, 0, True); 
Is_Null_2 := Get_Result = null; 
if Is_Null_2 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Fld_14.Info;
As_Entity_1 := (Info => Fld_16, Node => Get_Result); 
   If_Result_2 := As_Entity_1;
end if;
Var_Expr_5 := If_Result_2;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 6975 '<FieldAccess .expr>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10795
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 6975
Scope_Result_4 := Fld_17;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Expr;
end if;
   if Result_Var_4.Node = null
      or else Result_Var_4.Node.Kind in Ada_Name
   then
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Result_Var_4.Node,
         Info => Result_Var_4.Info);
   else
         Cast_Result_2 := No_Entity_Name;
   end if;
--# expr-done 6976
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_4
--# expr-start 6983 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:10796
--# expr-start 6978 '<IsNull>' Is_Null_3 /usr/src/packages/BUILD/ada/ast.py:10797
--# expr-start 6977 '<FieldAccess .name_designated_type>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10797
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_21 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Var_Expr_4.Node, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 6977
Is_Null_3 := Fld_21.Node = null; 
--# expr-done 6978
if Is_Null_3 then
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10798") else null)); 
   If_Result_3 := Logic_False_1;
else
   --# expr-start 6982 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10799
--# expr-start 6979 '<FieldAccess .xref_no_overloading>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10800
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_18 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_4.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 6979
--# expr-start 6981 '<FieldAccess .type_bind_val(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10801
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6980 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10801
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6980
Cast_Expr := Block_Var_1; 
      Cast_Result_3 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 6981
And_Pred_1 := Logic_And (Fld_18, Fld_20, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10799") else null)); 
--# expr-done 6982
   If_Result_3 := And_Pred_1;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 6983
Scope_Result_5 := If_Result_3;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2472;
   Result_Var_5 := Scope_Result_5;
else
   Logic_False_2 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10804") else null)); 
   Result_Var_5 := Logic_False_2;
end if;
      Inc_Ref (Result_Var_5);
--# expr-start 6997 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:10808
--# expr-start 6993 '<LogicAll>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:10808
--# expr-start 6984 '<FieldAccess .params>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10808
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_22 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 6984
Var_Expr_6 := Fld_22;
if Var_Expr_6 /= No_Entity_Assoc_List then
   --# scope-start
--# expr-start 6992 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:10808
Saved := Var_Expr_6; 
Fld_23 := Saved.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_23 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
         I := 0;
      declare
         Collection : constant Bare_Assoc_List := Fld_23;
      begin
         for Untyped_Item_208 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_208 := Untyped_Item_208; 
Is_Null_4 := Bare_Item_208 = null; 
if Is_Null_4 then
   If_Result_4 := No_Entity_Basic_Assoc;
else
Fld_24 := Saved.Info;
As_Entity_2 := (Info => Fld_24, Node => Bare_Item_208); 
   If_Result_4 := As_Entity_2;
end if;
                  Item_208 := If_Result_4; 
   --# scope-start
               --# bind pa Item_208
               --# bind i I
      --# expr-start 6991 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10809
--# expr-start 6987 '<If>' If_Result_5 /usr/src/packages/BUILD/ada/ast.py:10809
if Constrain_Params then
   --# expr-start 6986 '<FieldAccess .sub_equation>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:10811
--# expr-start 6985 '<FieldAccess .expr>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:10811
      if Item_208.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_25 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_208.Node, E_Info => Item_208.Info);
--# end
--# expr-done 6985
      if Fld_25.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_26 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_25.Node, Env => Env, Origin => Origin, E_Info => Fld_25.Info);
--# end
--# expr-done 6986
   If_Result_5 := Fld_26;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10812") else null)); 
   If_Result_5 := Logic_True;
end if;
      Inc_Ref (If_Result_5);
--# expr-done 6987
--# expr-start 6990 '<FieldAccess .constrain_index_expr(...)>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:10814
--# expr-start 6988 '<FieldAccess .indices>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:10814
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6988
      if Fld_27.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6989 '<FieldAccess .expr>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:10814
      if Item_208.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_28 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Item_208.Node, E_Info => Item_208.Info);
--# end
--# expr-done 6989
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_29 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_27.Node, Index_Expr => Fld_28, Dim => I, Origin => Origin, E_Info => Fld_27.Info);
--# end
--# expr-done 6990
And_Pred_2 := Logic_And (If_Result_5, Fld_29, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10809") else null)); 
--# expr-done 6991
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               I := I + 1;
   --# end
      Finalizer_Scope_2474;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 6992
Scope_Result_6 := Map_Result_1;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2473;
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Logic_Equation_Array_Type;
end if;
      Inc_Ref (Result_Var_6);
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Result_Var_6.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10808") else null)); 
--# expr-done 6993
--# expr-start 6996 '<FieldAccess .type_bind_val(...)>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:10816
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 6994 '<FieldAccess .type_var>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:10816
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 6994
--# expr-start 6995 '<FieldAccess .comp_type>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:10816
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_31 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Block_Var.Node, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 6995
      Cast_Result_4 := Create_Internal_Entity
        (Node => Fld_31.Node,
         Info => Fld_31.Info);
--# property-call-start AdaNode.type_bind_val
Fld_32 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_30, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 6996
And_Pred_3 := Logic_And (Logic_Boolean_Op_1, Fld_32, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10808") else null)); 
--# expr-done 6997
Or_Pred := Logic_Or (Result_Var_5, And_Pred_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10793") else null)); 
--# expr-done 6998
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result_7 := Let_Result;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2470;
            Match_Result := Scope_Result_7; Inc_Ref (Match_Result);
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_10; 
--# bind bo Match_1
--# expr-start 7019 '<And>' And_Pred_8 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 7016 '<And>' And_Pred_7 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 7013 '<And>' And_Pred_6 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 7010 '<And>' And_Pred_5 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 7005 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 7001 '<FieldAccess .constrain_index_expr(...)>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:10821
--# expr-start 6999 '<FieldAccess .indices>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:10821
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 6999
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7000 '<FieldAccess .left>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:10821
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 7000
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_33.Node, Index_Expr => Fld_34, Dim => 0, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 7001
--# expr-start 7004 '<FieldAccess .constrain_index_expr(...)>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:10822
--# expr-start 7002 '<FieldAccess .indices>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:10822
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 7002
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7003 '<FieldAccess .right>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:10822
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 7003
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_38 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_36.Node, Index_Expr => Fld_37, Dim => 0, Origin => Origin, E_Info => Fld_36.Info);
--# end
--# expr-done 7004
And_Pred_4 := Logic_And (Fld_35, Fld_38, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10821") else null)); 
--# expr-done 7005
--# expr-start 7009 '<FieldAccess .type_bind_var(...)>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:10823
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7006 '<FieldAccess .type_var>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:10823
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_39 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_1.Node);
--# expr-done 7006
--# expr-start 7008 '<FieldAccess .type_var>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:10823
--# expr-start 7007 '<FieldAccess .right>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:10823
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 7007
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_40.Node);
--# expr-done 7008
--# property-call-start AdaNode.type_bind_var
Fld_42 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_39, Right => Fld_41, Origin => Origin);
--# end
--# expr-done 7009
And_Pred_5 := Logic_And (And_Pred_4, Fld_42, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10821") else null)); 
--# expr-done 7010
--# expr-start 7012 '<FieldAccess .type_bind_val(...)>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:10824
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7011 '<FieldAccess .type_var>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:10824
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7011
Cast_Expr_1 := Block_Var_1; 
      Cast_Result_5 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_44 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_43, Right => Cast_Result_5, Origin => Origin);
--# end
--# expr-done 7012
And_Pred_6 := Logic_And (And_Pred_5, Fld_44, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10821") else null)); 
--# expr-done 7013
--# expr-start 7015 '<FieldAccess .sub_equation>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:10825
--# expr-start 7014 '<FieldAccess .left>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:10825
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_45 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Left, Info => Match_1.Info);
--# expr-done 7014
      if Fld_45.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_46 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_45.Node, Env => Env, Origin => Origin, E_Info => Fld_45.Info);
--# end
--# expr-done 7015
And_Pred_7 := Logic_And (And_Pred_6, Fld_46, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10821") else null)); 
--# expr-done 7016
--# expr-start 7018 '<FieldAccess .sub_equation>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:10826
--# expr-start 7017 '<FieldAccess .right>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:10826
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Create_Internal_Entity_Expr (Node => Match_1.Node.Bin_Op_F_Right, Info => Match_1.Info);
--# expr-done 7017
      if Fld_47.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_48 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_47.Node, Env => Env, Origin => Origin, E_Info => Fld_47.Info);
--# end
--# expr-done 7018
And_Pred_8 := Logic_And (And_Pred_7, Fld_48, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10821") else null)); 
--# expr-done 7019
Let_Result_1 := And_Pred_8; Inc_Ref (Let_Result_1);
--# end
Scope_Result_8 := Let_Result_1;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2475;
            Match_Result := Scope_Result_8; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_11; 
--# bind ar Match_2
--# expr-start 7026 '<And>' And_Pred_10 /usr/src/packages/BUILD/ada/ast.py:10830
--# expr-start 7023 '<And>' And_Pred_9 /usr/src/packages/BUILD/ada/ast.py:10830
--# expr-start 7020 '<FieldAccess .sub_equation>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:10830
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_49 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 7020
--# expr-start 7022 '<FieldAccess .constrain_index_expr(...)>' Fld_51 /usr/src/packages/BUILD/ada/ast.py:10831
--# expr-start 7021 '<FieldAccess .indices>' Fld_50 /usr/src/packages/BUILD/ada/ast.py:10831
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_50 := Create_Internal_Entity_Array_Indices (Node => Block_Var.Node.Array_Type_Def_F_Indices, Info => Block_Var.Info);
--# expr-done 7021
      if Fld_50.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Cast_Expr_2 := Match_2; 
      Cast_Result_6 := Create_Internal_Entity_Expr
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start '[dispatcher]ArrayIndices.constrain_index_expr'
Fld_51 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Constrain_Index_Expr (Node => Fld_50.Node, Index_Expr => Cast_Result_6, Dim => 0, Origin => Origin, E_Info => Fld_50.Info);
--# end
--# expr-done 7022
And_Pred_9 := Logic_And (Fld_49, Fld_51, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10830") else null)); 
--# expr-done 7023
--# expr-start 7025 '<FieldAccess .type_bind_val(...)>' Fld_53 /usr/src/packages/BUILD/ada/ast.py:10832
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7024 '<FieldAccess .type_var>' Fld_52 /usr/src/packages/BUILD/ada/ast.py:10832
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_52 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7024
Cast_Expr_3 := Block_Var_1; 
      Cast_Result_7 := Create_Internal_Entity
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_53 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_52, Right => Cast_Result_7, Origin => Origin);
--# end
--# expr-done 7025
And_Pred_10 := Logic_And (And_Pred_9, Fld_53, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10830") else null)); 
--# expr-done 7026
Let_Result_2 := And_Pred_10; Inc_Ref (Let_Result_2);
--# end
Scope_Result_9 := Let_Result_2;
   Inc_Ref (Scope_Result_9);
   --# end
      Finalizer_Scope_2476;
            Match_Result := Scope_Result_9; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_12; 
--# bind st Match_3
--# expr-start 7030 '<And>' And_Pred_11 /usr/src/packages/BUILD/ada/ast.py:10836
--# expr-start 7027 '<FieldAccess .sub_equation>' Fld_54 /usr/src/packages/BUILD/ada/ast.py:10836
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_54 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 7027
--# expr-start 7029 '<FieldAccess .type_bind_val(...)>' Fld_56 /usr/src/packages/BUILD/ada/ast.py:10837
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7028 '<FieldAccess .type_var>' Fld_55 /usr/src/packages/BUILD/ada/ast.py:10837
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_55 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7028
Cast_Expr_4 := Block_Var_1; 
      Cast_Result_8 := Create_Internal_Entity
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_56 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_55, Right => Cast_Result_8, Origin => Origin);
--# end
--# expr-done 7029
And_Pred_11 := Logic_And (Fld_54, Fld_56, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10836") else null)); 
--# expr-done 7030
Let_Result_3 := And_Pred_11; Inc_Ref (Let_Result_3);
--# end
Scope_Result_10 := Let_Result_3;
   Inc_Ref (Scope_Result_10);
   --# end
      Finalizer_Scope_2477;
            Match_Result := Scope_Result_10; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_13; 
--# bind _ Match_4
Logic_False_3 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10839") else null)); 
Let_Result_4 := Logic_False_3; Inc_Ref (Let_Result_4);
--# end
Scope_Result_11 := Let_Result_4;
   Inc_Ref (Scope_Result_11);
   --# end
      Finalizer_Scope_2478;
            Match_Result := Scope_Result_11; Inc_Ref (Match_Result);
end case;
--# expr-done 7031
   If_Result_10 := Match_Result;
else
--# expr-start 7033 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:10843
--# expr-start 7032 '<IsNull>' Is_Null_5 /usr/src/packages/BUILD/ada/ast.py:10843
Is_Null_5 := Typ.Node = null; 
--# expr-done 7032
Not_Val_1 := not (Is_Null_5); 
--# expr-done 7033
if Not_Val_1 then
   --# expr-start 7034 '<FieldAccess .has_ud_indexing>' Fld_57 /usr/src/packages/BUILD/ada/ast.py:10843
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_57 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 7034
   If_Result_6 := Fld_57;
else
   If_Result_6 := False;
end if;
if If_Result_6 then
   --# expr-start 7057 '<LogicAny>' Logic_Boolean_Op_2 /usr/src/packages/BUILD/ada/ast.py:10844
--# expr-start 7056 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:10844
--# expr-start 7037 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:10844
--# expr-start 7035 '<FieldAccess .constant_indexing_fns>' Fld_58 /usr/src/packages/BUILD/ada/ast.py:10844
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.constant_indexing_fns'
Fld_58 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Constant_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 7035
--# expr-start 7036 '<FieldAccess .variable_indexing_fns>' Fld_59 /usr/src/packages/BUILD/ada/ast.py:10844
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.variable_indexing_fns'
Fld_59 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Variable_Indexing_Fns (Node => Typ.Node, E_Info => Typ.Info);
--# end
--# expr-done 7036
Concat_Result := Concat (Fld_58, Fld_59); 
--# expr-done 7037
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Concat_Result;
      begin
         for Item_209 of
               Collection.Items
         loop
   --# scope-start
               --# bind fn Item_209
      --# expr-start 7055 '<Let formal, ret_type, param>' Let_Result_5 /usr/src/packages/BUILD/ada/ast.py:10845
--# scope-start
--# expr-start 7040 '<at(<FieldAccess .unpacked_formal_params>, 1)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:10847
--# expr-start 7039 '<FieldAccess .unpacked_formal_params>' Fld_61 /usr/src/packages/BUILD/ada/ast.py:10847
--# expr-start 7038 '<FieldAccess .subp_spec_or_null>' Fld_60 /usr/src/packages/BUILD/ada/ast.py:10847
      if Item_209.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_60 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_209.Node, E_Info => Item_209.Info);
--# end
--# expr-done 7038
      if Fld_60.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_61 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_60.Node, E_Info => Fld_60.Info);
--# end
--# expr-done 7039
Get_Result_1 := Get (Fld_61, 1, True); 
--# expr-done 7040
Formal := Get_Result_1; 
--# bind formal Formal
--# expr-start 7042 '<FieldAccess .return_type>' Fld_63 /usr/src/packages/BUILD/ada/ast.py:10848
--# expr-start 7041 '<FieldAccess .subp_spec_or_null>' Fld_62 /usr/src/packages/BUILD/ada/ast.py:10848
      if Item_209.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_62 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_209.Node, E_Info => Item_209.Info);
--# end
--# expr-done 7041
      if Fld_62.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_63 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_62.Node, Origin => Origin, E_Info => Fld_62.Info);
--# end
--# expr-done 7042
Ret_Type := Fld_63; 
--# bind ret_type Ret_Type
--# expr-start 7044 '<FieldAccess .expr>' Fld_67 /usr/src/packages/BUILD/ada/ast.py:10849
--# expr-start 7043 '<FieldAccess .params>' Fld_64 /usr/src/packages/BUILD/ada/ast.py:10849
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_64 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7043
Fld_65 := Fld_64.Node;
Get_Result_2 := Get (Fld_65, 0, True); 
Is_Null_6 := Get_Result_2 = null; 
if Is_Null_6 then
   If_Result_7 := No_Entity_Basic_Assoc;
else
Fld_66 := Fld_64.Info;
As_Entity_3 := (Info => Fld_66, Node => Get_Result_2); 
   If_Result_7 := As_Entity_3;
end if;
      if If_Result_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_67 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => If_Result_7.Node, E_Info => If_Result_7.Info);
--# end
--# expr-done 7044
Param := Fld_67; 
--# bind param Param
--# expr-start 7054 '<And>' And_Pred_13 /usr/src/packages/BUILD/ada/ast.py:10851
--# expr-start 7049 '<And>' And_Pred_12 /usr/src/packages/BUILD/ada/ast.py:10851
--# expr-start 7046 '<FieldAccess .type_bind_val(...)>' Fld_69 /usr/src/packages/BUILD/ada/ast.py:10851
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7045 '<FieldAccess .type_var>' Fld_68 /usr/src/packages/BUILD/ada/ast.py:10851
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_68 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7045
Cast_Expr_5 := Ret_Type; 
      Cast_Result_14 := Create_Internal_Entity
        (Node => Cast_Expr_5.Node,
         Info => Cast_Expr_5.Info);
--# property-call-start AdaNode.type_bind_val
Fld_69 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_68, Right => Cast_Result_14, Origin => Origin);
--# end
--# expr-done 7046
--# expr-start 7048 '<If>' If_Result_8 /usr/src/packages/BUILD/ada/ast.py:10852
if Constrain_Params then
   --# expr-start 7047 '<FieldAccess .sub_equation>' Fld_70 /usr/src/packages/BUILD/ada/ast.py:10853
      if Param.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_70 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Param.Node, Env => Env, Origin => Origin, E_Info => Param.Info);
--# end
--# expr-done 7047
   If_Result_8 := Fld_70;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10853") else null)); 
   If_Result_8 := Logic_True_1;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 7048
And_Pred_12 := Logic_And (Fld_69, If_Result_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10851") else null)); 
--# expr-done 7049
--# expr-start 7053 '<FieldAccess .type_bind_val(...)>' Fld_74 /usr/src/packages/BUILD/ada/ast.py:10854
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7050 '<FieldAccess .type_var>' Fld_71 /usr/src/packages/BUILD/ada/ast.py:10854
      if Param.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_71 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Param.Node);
--# expr-done 7050
--# expr-start 7052 '<FieldAccess .formal_type>' Fld_73 /usr/src/packages/BUILD/ada/ast.py:10854
--# expr-start 7051 '<FieldAccess .spec>' Fld_72 /usr/src/packages/BUILD/ada/ast.py:10854
Fld_72 := Formal.Spec;
--# expr-done 7051
      if Fld_72.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamDecl.formal_type
Fld_73 := Libadalang.Implementation.Base_Formal_Param_Decl_P_Formal_Type (Node => Fld_72.Node, Origin => Origin, E_Info => Fld_72.Info);
--# end
--# expr-done 7052
      Cast_Result_15 := Create_Internal_Entity
        (Node => Fld_73.Node,
         Info => Fld_73.Info);
--# property-call-start AdaNode.type_bind_val
Fld_74 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_71, Right => Cast_Result_15, Origin => Origin);
--# end
--# expr-done 7053
And_Pred_13 := Logic_And (And_Pred_12, Fld_74, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10851") else null)); 
--# expr-done 7054
Let_Result_5 := And_Pred_13; Inc_Ref (Let_Result_5);
--# end
--# expr-done 7055
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result_5;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2479;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7056
Logic_Boolean_Op_2 := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10844") else null)); 
--# expr-done 7057
   If_Result_9 := Logic_Boolean_Op_2;
else
   Logic_False_4 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10857") else null)); 
   If_Result_9 := Logic_False_4;
end if;
      Inc_Ref (If_Result_9);
   If_Result_10 := If_Result_9;
end if;
      Inc_Ref (If_Result_10);
   If_Result_11 := If_Result_10;
end if;
      Inc_Ref (If_Result_11);
--# expr-done 7058
Let_Result_6 := If_Result_11; Inc_Ref (Let_Result_6);
--# end
--# expr-done 7059
      Property_Result := Let_Result_6;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_864;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_864;
               Finalizer_Scope_2467;
               Finalizer_Scope_2468;
               Finalizer_Scope_2470;
               Finalizer_Scope_2472;
               Finalizer_Scope_2473;
               Finalizer_Scope_2474;
               Finalizer_Scope_2475;
               Finalizer_Scope_2476;
               Finalizer_Scope_2477;
               Finalizer_Scope_2478;
               Finalizer_Scope_2479;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subscriptable_Type_Equation;
--# end
--# property-start CallExpr.subprogram_equation /usr/src/packages/BUILD/ada/ast.py:10861
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Subprogram_Equation
  (Node : Bare_Call_Expr
      ; Subp_Spec : Internal_Entity_Base_Formal_Param_Holder
      ; Dottable_Subp : Boolean
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind subp_spec Subp_Spec
   --# bind dottable_subp Dottable_Subp
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_865;
            procedure Finalizer_Scope_2480;
            procedure Finalizer_Scope_2482;
      Var_Expr : Internal_Entity_Base_Formal_Param_Holder;
Var_Expr_1 : Internal_Entity_Base_Subp_Spec;
N : Internal_Entity_Base_Formal_Param_Decl;
Fld : Logic_Var;
Cast_Expr : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result : Internal_Entity_Base_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Cast_Expr_1 : Internal_Entity_Base_Formal_Param_Holder;
Cast_Result_2 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Internal_Entity_Assoc_List;
Fld_5 : Internal_Param_Match_Array_Access;
Fld_6 : Internal_Single_Formal;
Fld_7 : Internal_Entity_Base_Formal_Param_Decl;
Fld_8 : Internal_Single_Actual;
Fld_9 : Internal_Entity_Basic_Assoc;
Fld_10 : Internal_Entity_Expr;
Fld_11 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_12 : Internal_Single_Actual;
Fld_13 : Bare_Base_Id;
Fld_14 : Logic_Var;
Fld_15 : Internal_Single_Formal;
Fld_16 : Internal_Entity_Base_Formal_Param_Decl;
Fld_17 : Bare_Base_Formal_Param_Decl;
Cast_Result_3 : Bare_Ada_Node;
Fld_18 : Internal_Entity_Info;
Fld_19 : Env_Rebindings;
Fld_20 : Internal_Entity_Info;
Fld_21 : Boolean;
Fld_22 : Internal_Entity;
Let_Result : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_23 : Internal_Single_Actual;
Fld_24 : Bare_Base_Id;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_25 : Boolean;
If_Result_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_False_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_865 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_False_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_865;
            procedure Finalizer_Scope_2480 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2480;
            procedure Finalizer_Scope_2482 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2482;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7098 '<Block>' Let_Result_1 None
--# scope-start
Var_Expr := Subp_Spec;
if Var_Expr /= No_Entity_Base_Formal_Param_Holder then
   --# scope-start
   --# bind subp_spec Var_Expr
--# expr-start 7097 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:10867
--# expr-start 7066 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10867
--# expr-start 7063 '<FieldAccess .type_bind_val(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10867
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7060 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:10867
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7060
--# expr-start 7061 '<Cast to BaseSubpSpec>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10868
Cast_Expr := Var_Expr; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Spec
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Spec
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Spec;
   end if;
--# expr-done 7061
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Base_Subp_Spec then
   --# scope-start
--# expr-start 7062 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10868
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7062
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result_1 := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7063
--# expr-start 7065 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10872
--# expr-start 7064 '<FieldAccess .subp_spec_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10872
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 7064
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Cast_Expr_1 := Var_Expr; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10872") else null)); 
--# expr-done 7065
And_Pred := Logic_And (Fld_2, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10867") else null)); 
--# expr-done 7066
--# expr-start 7096 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:10876
--# expr-start 7095 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10876
--# expr-start 7068 '<FieldAccess .match_param_list(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10876
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7067 '<FieldAccess .params>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10877
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_4 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7067
--# property-call-start BaseFormalParamHolder.match_param_list
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Param_List (Node => Var_Expr.Node, Params => Fld_4, Is_Dottable_Subp => Dottable_Subp, Env => Env, E_Info => Var_Expr.Info);
--# end
--# expr-done 7068
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Fld_5;
      begin
         for Item_210 of
               Collection.Items
         loop
   --# scope-start
               --# bind pm Item_210
      --# expr-start 7094 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:10879
--# expr-start 7069 '<FieldAccess .has_matched>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:10880
Fld_25 := Item_210.Has_Matched;
--# expr-done 7069
if Fld_25 then
   --# expr-start 7093 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:10881
--# expr-start 7075 '<FieldAccess .call_argument_equation(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10881
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7071 '<FieldAccess .spec>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10882
--# expr-start 7070 '<FieldAccess .formal>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10882
Fld_6 := Item_210.Formal;
--# expr-done 7070
Fld_7 := Fld_6.Spec;
--# expr-done 7071
--# expr-start 7074 '<FieldAccess .expr>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10883
--# expr-start 7073 '<FieldAccess .assoc>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10883
--# expr-start 7072 '<FieldAccess .actual>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10883
Fld_8 := Item_210.Actual;
--# expr-done 7072
Fld_9 := Fld_8.Assoc;
--# expr-done 7073
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_9.Node, E_Info => Fld_9.Info);
--# end
--# expr-done 7074
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_11 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Var_Expr.Node, Param => Fld_7, Arg => Fld_10, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7075
--# expr-start 7092 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:10884
--# expr-start 7078 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:10887
--# expr-start 7077 '<FieldAccess .name>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:10887
--# expr-start 7076 '<FieldAccess .actual>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:10887
Fld_23 := Item_210.Actual;
--# expr-done 7076
Fld_24 := Fld_23.Name;
--# expr-done 7077
Is_Null := Fld_24 = null; 
--# expr-done 7078
if Is_Null then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10888") else null)); 
   If_Result := Logic_True;
else
   --# expr-start 7091 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:10889
--# expr-start 7081 '<FieldAccess .ref_var>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10890
--# expr-start 7080 '<FieldAccess .name>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10890
--# expr-start 7079 '<FieldAccess .actual>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10890
Fld_12 := Item_210.Actual;
--# expr-done 7079
Fld_13 := Fld_12.Name;
--# expr-done 7080
      if Fld_13 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_13);
--# end
--# expr-done 7081
Fld_14.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_14.all);
Eq_Node.Refs.Destroy (Fld_14.all);
--# expr-start 7090 '<Let n>' Let_Result /usr/src/packages/BUILD/ada/ast.py:10891
--# scope-start
--# expr-start 7083 '<FieldAccess .spec>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10891
--# expr-start 7082 '<FieldAccess .formal>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10891
Fld_15 := Item_210.Formal;
--# expr-done 7082
Fld_16 := Fld_15.Spec;
--# expr-done 7083
N := Fld_16; 
--# bind n N
--# expr-start 7089 '<FieldAccess .entity_no_md(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:10891
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7084 '<FieldAccess .node>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:10892
Fld_17 := N.Node;
--# expr-done 7084
      Cast_Result_3 := Fld_17;
--# expr-start 7086 '<FieldAccess .rebindings>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:10893
--# expr-start 7085 '<FieldAccess .info>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:10893
Fld_18 := N.Info;
--# expr-done 7085
Fld_19 := Fld_18.Rebindings;
--# expr-done 7086
--# expr-start 7088 '<FieldAccess .from_rebound>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:10894
--# expr-start 7087 '<FieldAccess .info>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:10894
Fld_20 := N.Info;
--# expr-done 7087
Fld_21 := Fld_20.From_Rebound;
--# expr-done 7088
--# property-call-start AdaNode.entity_no_md
Fld_22 := Libadalang.Implementation.Ada_Node_P_Entity_No_Md (Node => Ent.Node, N => Cast_Result_3, Rebindings => Fld_19, From_Rebound => Fld_21);
--# end
--# expr-done 7089
Let_Result := Fld_22; 
--# end
--# expr-done 7090
Bind_Result_1 := Bind_Default_Default.Create (Fld_14, Let_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10889") else null)); 
--# expr-done 7091
   If_Result := Bind_Result_1;
end if;
      Inc_Ref (If_Result);
--# expr-done 7092
And_Pred_1 := Logic_And (Fld_11, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10881") else null)); 
--# expr-done 7093
   If_Result_1 := And_Pred_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10898") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7094
         declare
            Item_To_Append : constant Logic_Equation :=
               If_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2482;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7095
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10876") else null)); 
--# expr-done 7096
And_Pred_2 := Logic_And (And_Pred, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10867") else null)); 
--# expr-done 7097
Scope_Result_1 := And_Pred_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2480;
   Result_Var_1 := Scope_Result_1;
else
   Logic_False_1 := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10901") else null)); 
   Result_Var_1 := Logic_False_1;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_1 := Result_Var_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7098
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_865;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_865;
               Finalizer_Scope_2480;
               Finalizer_Scope_2482;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Subprogram_Equation;
--# end
--# property-start CallExpr.check_for_type /usr/src/packages/BUILD/ada/ast.py:10905
pragma Warnings (Off, "is not referenced");
 function Call_Expr_P_Check_For_Type
  (Node : Bare_Call_Expr
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Call_Expr := Bare_Call_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Expr :=
     Internal_Entity_Call_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Boolean;
      Block_Var : Internal_Entity_Array_Type_Def;
Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Var_Expr_2 : Internal_Entity_Array_Type_Def;
Match_0 : Bare_Assoc_List;
Match_1 : Bare_Bin_Op;
Match_2 : Bare_Subtype_Indication;
Match_3 : Bare_Attribute_Ref;
Match_4 : Bare_Ada_Node;
Var_Expr_3 : Internal_Entity_Access_To_Subp_Def;
Var_Expr_4 : Bare_Assoc_List;
Var_Expr_5 : Internal_Entity_Call_Expr;
Fld : Internal_Entity_Array_Type_Def;
Scope_Result : Internal_Entity_Array_Type_Def;
Result_Var : Internal_Entity_Array_Type_Def;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Integer;
Len : Integer;
Is_Equal : Boolean;
Fld_4 : Integer;
Is_Equal_1 : Boolean;
Fld_5 : Integer;
Is_Equal_2 : Boolean;
Fld_6 : Integer;
Is_Equal_3 : Boolean;
Match_Prefix : Bare_Ada_Node;
Cast_Result : Bare_Assoc_List;
Let_Result : Boolean;
Scope_Result_1 : Boolean;
Cast_Result_1 : Bare_Bin_Op;
Let_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Cast_Result_2 : Bare_Subtype_Indication;
Let_Result_2 : Boolean;
Scope_Result_3 : Boolean;
Cast_Result_3 : Bare_Attribute_Ref;
Let_Result_3 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_4 : Bare_Ada_Node;
Let_Result_4 : Boolean;
Scope_Result_5 : Boolean;
Match_Result : Boolean;
Scope_Result_6 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Access_Def;
Cast_Result_5 : Internal_Entity_Access_To_Subp_Def;
Fld_8 : Internal_Entity_Subp_Spec;
Fld_9 : Internal_Entity_Assoc_List;
Fld_10 : Boolean;
Scope_Result_7 : Boolean;
Result_Var_2 : Boolean;
If_Result : Boolean;
Fld_11 : Boolean;
Fld_12 : Bare_Ada_Node;
Cast_Result_6 : Bare_Assoc_List;
Len_1 : Integer;
Is_Equal_4 : Boolean;
Scope_Result_8 : Boolean;
Result_Var_3 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Boolean;
Fld_13 : Internal_Entity;
Cast_Result_7 : Internal_Entity_Call_Expr;
Fld_14 : Internal_Entity_Base_Type_Decl;
Fld_15 : Boolean;
If_Result_3 : Internal_Entity_Base_Type_Decl;
Fld_16 : Boolean;
Scope_Result_9 : Boolean;
Result_Var_4 : Boolean;
If_Result_4 : Boolean;
Scope_Result_10 : Boolean;
Result_Var_5 : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result_5 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7129 '<Block>' Let_Result_5 None
--# scope-start
Var_Expr := Typ;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 7099 '<FieldAccess .array_def_with_deref>' Fld /usr/src/packages/BUILD/ada/ast.py:10916
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.array_def_with_deref
Fld := Libadalang.Implementation.Base_Type_Decl_P_Array_Def_With_Deref (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7099
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Array_Type_Def;
end if;
Block_Var := Result_Var; 
--# bind atd Block_Var
--# expr-start 7128 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <Var typ>: <Var typ> <And>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:10918
--# expr-start 7100 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10918
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7100
Bound_Origin := Fld_1; 
Var_Expr_1 := Typ;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr_1
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Array_Type_Def then
   --# scope-start
   --# bind _ Var_Expr_2
--# expr-start 7111 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:10921
--# expr-start 7101 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10921
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Call_Expr_F_Suffix;
--# expr-done 7101
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_2; 
case Ada_Ada_Node (Match_Prefix.Kind) is
         when Ada_Assoc_List =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind al Match_0
--# expr-start 7104 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:10923
--# expr-start 7102 '<FieldAccess .array_ndims>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10923
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 7102
--# expr-start 7103 '<length(<Var al>)>' Len /usr/src/packages/BUILD/ada/ast.py:10923
Len := Length (Match_0); 
--# expr-done 7103
Is_Equal := Fld_3 = Len; 
--# expr-done 7104
Let_Result := Is_Equal; 
--# end
Scope_Result_1 := Let_Result;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 7106 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:10926
--# expr-start 7105 '<FieldAccess .array_ndims>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10926
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 7105
Is_Equal_1 := Fld_4 = 1; 
--# expr-done 7106
Let_Result_1 := Is_Equal_1; 
--# end
Scope_Result_2 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 7108 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:10927
--# expr-start 7107 '<FieldAccess .array_ndims>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10927
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_5 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 7107
Is_Equal_2 := Fld_5 = 1; 
--# expr-done 7108
Let_Result_2 := Is_Equal_2; 
--# end
Scope_Result_3 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 7110 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:10928
--# expr-start 7109 '<FieldAccess .array_ndims>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10928
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.array_ndims'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Def_P_Array_Ndims (Node => Block_Var.Node, Origin => Bound_Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 7109
Is_Equal_3 := Fld_6 = 1; 
--# expr-done 7110
Let_Result_3 := Is_Equal_3; 
--# end
Scope_Result_4 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abort_Absent .. Ada_Base_Assoc_List | Ada_Basic_Decl_List .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Call_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
Let_Result_4 := False; 
--# end
Scope_Result_5 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_5; 
end case;
--# expr-done 7111
Scope_Result_6 := Match_Result;
   --# end
   Result_Var_1 := Scope_Result_6;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result := True;
else
--# expr-start 7113 '<Cast to AccessToSubpDef>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:10934
--# expr-start 7112 '<FieldAccess .access_def>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10934
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.access_def'
Fld_7 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Access_Def (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7112
   if Fld_7.Node = null
      or else Fld_7.Node.Kind in Ada_Access_To_Subp_Def_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Access_To_Subp_Def
        (Node => Fld_7.Node,
         Info => Fld_7.Info);
   else
         Cast_Result_5 := No_Entity_Access_To_Subp_Def;
   end if;
--# expr-done 7113
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Access_To_Subp_Def then
   --# scope-start
   --# bind sa Var_Expr_3
--# expr-start 7116 '<FieldAccess .is_matching_param_list(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:10936
--# expr-start 7114 '<FieldAccess .subp_spec>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10936
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Subp_Spec (Node => Var_Expr_3.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Var_Expr_3.Info);
--# expr-done 7114
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7115 '<FieldAccess .params>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:10936
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_9 := Libadalang.Implementation.Call_Expr_P_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7115
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_10 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Fld_8.Node, Params => Fld_9, Is_Dottable_Subp => False, Env => Env);
--# end
--# expr-done 7116
Scope_Result_7 := Fld_10;
   --# end
   Result_Var_2 := Scope_Result_7;
else
   Result_Var_2 := False;
end if;
   If_Result := Result_Var_2;
end if;
if If_Result then
   If_Result_2 := True;
else
--# expr-start 7117 '<FieldAccess .has_ud_indexing>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:10940
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.has_ud_indexing'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Has_Ud_Indexing (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7117
if Fld_11 then
--# expr-start 7119 '<Cast to AssocList>' Cast_Result_6 /usr/src/packages/BUILD/ada/ast.py:10941
--# expr-start 7118 '<FieldAccess .suffix>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:10941
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Call_Expr_F_Suffix;
--# expr-done 7118
   if Fld_12 = null
      or else Fld_12.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_6 := Fld_12;
   else
         Cast_Result_6 := No_Bare_Ada_Node;
   end if;
--# expr-done 7119
Var_Expr_4 := Cast_Result_6;
if Var_Expr_4 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind al Var_Expr_4
--# expr-start 7121 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:10941
--# expr-start 7120 '<length(<Var al>)>' Len_1 /usr/src/packages/BUILD/ada/ast.py:10941
Len_1 := Length (Var_Expr_4); 
--# expr-done 7120
Is_Equal_4 := Len_1 = 1; 
--# expr-done 7121
Scope_Result_8 := Is_Equal_4;
   --# end
   Result_Var_3 := Scope_Result_8;
else
   Result_Var_3 := False;
end if;
   If_Result_1 := Result_Var_3;
else
   If_Result_1 := False;
end if;
   If_Result_2 := If_Result_1;
end if;
if If_Result_2 then
--# expr-start 7123 '<Cast to CallExpr>' Cast_Result_7 /usr/src/packages/BUILD/ada/ast.py:10944
--# expr-start 7122 '<parent(<Var Ent>)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:10944
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7122
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_7 := Create_Internal_Entity_Call_Expr
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         Cast_Result_7 := No_Entity_Call_Expr;
   end if;
--# expr-done 7123
Var_Expr_5 := Cast_Result_7;
if Var_Expr_5 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_5
--# expr-start 7127 '<FieldAccess .check_for_type(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:10950
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7126 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:10950
--# expr-start 7124 '<FieldAccess .check_array_slice(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:10951
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.check_array_slice
Fld_15 := Libadalang.Implementation.Call_Expr_P_Check_Array_Slice (Node => Ent.Node, Typ => Var_Expr_1, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 7124
if Fld_15 then
   If_Result_3 := Var_Expr_1;
else
   --# expr-start 7125 '<FieldAccess .comp_type(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:10953
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.comp_type
Fld_14 := Libadalang.Implementation.Base_Type_Decl_P_Comp_Type (Node => Var_Expr_1.Node, Is_Subscript => True, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7125
   If_Result_3 := Fld_14;
end if;
--# expr-done 7126
--# property-call-start CallExpr.check_for_type
Fld_16 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_5.Node, Typ => If_Result_3, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 7127
Scope_Result_9 := Fld_16;
   --# end
   Result_Var_4 := Scope_Result_9;
else
   Result_Var_4 := True;
end if;
   If_Result_4 := Result_Var_4;
else
   If_Result_4 := False;
end if;
Scope_Result_10 := If_Result_4;
   --# end
   Result_Var_5 := Scope_Result_10;
else
   Result_Var_5 := False;
end if;
Dyn_Var_Bind_Result := Result_Var_5; 
--# expr-done 7128
Let_Result_5 := Dyn_Var_Bind_Result; 
--# end
--# expr-done 7129
      Property_Result := Let_Result_5;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Expr_P_Check_For_Type;
--# end
   --
   --  Primitives for Bare_Defining_Name
   --
      procedure Initialize_Fields_For_Defining_Name
        (Self : Bare_Defining_Name
         ; Defining_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Defining_Name_F_Name := Defining_Name_F_Name;
      end Initialize_Fields_For_Defining_Name;
   function Defining_Name_F_Name
     (Node : Bare_Defining_Name) return Bare_Name
   is
   begin
         return Node.Defining_Name_F_Name;
   end;
--# property-start DefiningName.parent_scope /usr/src/packages/BUILD/ada/ast.py:11696
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Parent_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_913;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_913 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_913;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7131 '<FieldAccess .parent_scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11696
--# expr-start 7130 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11696
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 7130
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 7131
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_913;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_913;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Parent_Scope;
--# end
--# property-start DefiningName.scope /usr/src/packages/BUILD/ada/ast.py:11697
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Scope
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_914;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_914 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_914;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7133 '<FieldAccess .scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11697
--# expr-start 7132 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11697
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 7132
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 7133
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_914;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_914;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Scope;
--# end
--# property-start DefiningName.relative_name /usr/src/packages/BUILD/ada/ast.py:11698
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Relative_Name
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7135 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11698
--# expr-start 7134 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11698
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 7134
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7135
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Relative_Name;
--# end
--# property-start DefiningName.ref_var /usr/src/packages/BUILD/ada/ast.py:11699
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Ref_Var
  (Node : Bare_Defining_Name
  )
   return Logic_Var
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7137 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11699
--# expr-start 7136 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11699
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 7136
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7137
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Ref_Var;
--# end
--# property-start DefiningName.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:11700
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Env_Elements_Impl
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_917;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_917 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_917;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7139 '<FieldAccess .env_elements_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11700
--# expr-start 7138 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11700
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 7138
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 7139
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_917;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_917;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Env_Elements_Impl;
--# end
--# property-start DefiningName.all_env_els_impl /usr/src/packages/BUILD/ada/ast.py:11703
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_All_Env_Els_Impl
  (Node : Bare_Defining_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_918;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_918 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_918;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7142 '<Block>' Let_Result None
--# scope-start
--# expr-start 7141 '<FieldAccess .all_env_els_impl(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11708
--# expr-start 7140 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11708
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 7140
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7141
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7142
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_918;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_918;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_All_Env_Els_Impl;
--# end
--# property-start DefiningName.basic_decl /usr/src/packages/BUILD/ada/ast.py:11710
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Basic_Decl
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_919;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_919 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_919;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7148 '<as_entity(<Cast to BasicDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:11711
--# expr-start 7147 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11711
--# expr-start 7146 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11711
--# expr-start 7145 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11711
--# expr-start 7143 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:11711
Node_Parents := Parents (Self, True); 
--# expr-done 7143
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_211 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_211
               --# expr-start 7144 '<IsA BasicDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:11711
Is_A := Item_211 /= null 
and then Item_211.Kind in Ada_Basic_Decl; 
--# expr-done 7144
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_211;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7145
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7146
   if Get_Result = null
      or else Get_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7147
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 7148
      Property_Result := If_Result;
   --# end
      Finalizer_Scope_919;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_919;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Basic_Decl;
--# end
--# property-start DefiningName.find_refs /usr/src/packages/BUILD/ada/ast.py:11719
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Refs
  (Node : Bare_Defining_Name
      ; Root : Internal_Entity
      ; Origin : Bare_Ada_Node
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind origin Origin
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_921;
            procedure Finalizer_Scope_2494;
            procedure Finalizer_Scope_2496;
            procedure Finalizer_Scope_2497;
            procedure Finalizer_Scope_2498;
      Var_Expr : Internal_Entity_Array_Access;
Var_Expr_1 : Internal_Entity_Base_Id;
Var_Expr_2 : Ref_Result_Kind;
Fld : Internal_Entity_Array_Access;
Is_Null : Boolean;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Ada_Node;
Is_Equal : Boolean;
If_Result : Boolean;
Not_Val : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_2 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Scope_Result : Internal_Ref_Result_Array_Access;
Result_Var : Internal_Ref_Result_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_3 : Ref_Result_Kind;
New_Struct : Internal_Ref_Result;
Singleton : Internal_Ref_Result_Array_Access;
Is_Equal_1 : Boolean;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Scope_Result_2 : Internal_Ref_Result_Array_Access;
Result_Var_2 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_921 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_921;
            procedure Finalizer_Scope_2494 is
            begin
                     Dec_Ref (Var_Expr);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2494;
            procedure Finalizer_Scope_2496 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2496;
            procedure Finalizer_Scope_2497 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2497;
            procedure Finalizer_Scope_2498 is
            begin
                     Dec_Ref (Singleton);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_2498;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7164 '<Block>' Let_Result None
--# scope-start
--# expr-start 7163 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:11725
--# expr-start 7149 '<children(<Var root>)>' Fld /usr/src/packages/BUILD/ada/ast.py:11725
      if Root.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children (Node => Root.Node, E_Info => Root.Info);
--# expr-done 7149
Var_Expr := Fld;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 7156 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11726
--# expr-start 7154 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11726
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr;
      begin
         for Item_213 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_213
               --# expr-start 7153 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:11726
--# expr-start 7150 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:11726
Is_Null := Item_213.Node = null; 
--# expr-done 7150
if Is_Null then
   If_Result := True;
else
   --# expr-start 7152 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11726
--# expr-start 7151 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11726
Fld_1 := Item_213.Node;
--# expr-done 7151
      Cast_Result := Fld_1;
Is_Equal := Cast_Result = Origin; 
--# expr-done 7152
   If_Result := Is_Equal;
end if;
Not_Val := not (If_Result); 
--# expr-done 7153
               if Not_Val then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_213;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7154
declare
   Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Map_Result;
      begin
         for Item_212 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_212
      --# expr-start 7155 '<FieldAccess .find_refs(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11727
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.find_refs
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Refs (Node => Ent.Node, Root => Item_212, Origin => Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7155
         for Item_To_Append of
               Fld_2.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2496;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7156
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2494;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 7157 '<Cast to BaseId>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11729
Cast_Expr := Root; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 7157
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Base_Id then
   --# scope-start
   --# bind id Var_Expr_1
--# expr-start 7158 '<FieldAccess .is_referenced_by(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11730
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.is_referenced_by
Fld_3 := Libadalang.Implementation.Defining_Name_P_Is_Referenced_By (Node => Ent.Node, Id => Var_Expr_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7158
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= Noref then
   --# scope-start
   --# bind ref_kind Var_Expr_2
--# expr-start 7162 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:11731
--# expr-start 7159 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:11732
Is_Equal_1 := Var_Expr_2 = Precise; 
--# expr-done 7159
if Is_Equal_1 then
   If_Result_1 := True;
else
   --# expr-start 7160 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:11732
Is_Equal_2 := Var_Expr_2 = Imprecise; 
--# expr-done 7160
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   --# expr-start 7161 '<New InternalRefResult>' New_Struct /usr/src/packages/BUILD/ada/ast.py:11735
New_Struct := (Kind => Var_Expr_2, Ref => Var_Expr_1); 
--# expr-done 7161
                Singleton := Create_Internal_Ref_Result_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
   If_Result_2 := Singleton;
else
   If_Result_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7162
Scope_Result_1 := If_Result_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2498;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Scope_Result_2 := Result_Var_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2497;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_2);
Concat_Result := Concat (Result_Var, Result_Var_2); 
--# expr-done 7163
Let_Result := Concat_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7164
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_921;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_921;
               Finalizer_Scope_2494;
               Finalizer_Scope_2496;
               Finalizer_Scope_2497;
               Finalizer_Scope_2498;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_Refs;
--# end
--# property-start DefiningName.is_derivable_equal /usr/src/packages/BUILD/ada/ast.py:11742
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Derivable_Equal
  (Node : Bare_Defining_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Subp_Spec;
Var_Expr_1 : Internal_Entity_Type_Expr;
Sym : Symbol_Type;
Fld : Boolean;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Base_Subp_Spec;
Fld_3 : Internal_Entity_Type_Expr;
Fld_4 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Entity;
Cast_Result : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
If_Result : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Derivable_Equal,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7173 '<Block>' Let_Result None
--# scope-start
--# expr-start 7166 '<FieldAccess .name_is(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:11749
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7165 '<Symbol "=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 7165
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Sym);
--# end
--# expr-done 7166
if Fld then
--# expr-start 7168 '<FieldAccess .subp_spec_or_null>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11750
--# expr-start 7167 '<FieldAccess .basic_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11750
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7167
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 7168
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 7172 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:11751
--# expr-start 7169 '<FieldAccess .returns>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11751
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_3 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7169
Var_Expr_1 := Fld_3;
if Var_Expr_1 /= No_Entity_Type_Expr then
   --# scope-start
--# expr-start 7170 '<FieldAccess .designated_type_decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11751
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7170
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
      Cast_Result := Create_Internal_Entity
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
--# expr-start 7171 '<FieldAccess .bool_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11751
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.bool_type
Fld_5 := Libadalang.Implementation.Ada_Node_P_Bool_Type (Node => Ent.Node);
--# end
--# expr-done 7171
Is_Equiv := Equivalent (Cast_Result, Fld_5); 
--# expr-done 7172
Scope_Result_1 := Is_Equiv;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
   If_Result := Result_Var_1;
else
   If_Result := False;
end if;
Let_Result := If_Result; 
--# end
--# expr-done 7173
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Derivable_Equal;
--# end
--# property-start DefiningName.is_potential_reference /usr/src/packages/BUILD/ada/ast.py:11756
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Potential_Reference
  (Node : Bare_Defining_Name
      ; Symbol : Symbol_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind symbol Symbol
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
Sym : Symbol_Type;
Is_Equal : Boolean;
If_Result : Boolean;
If_Result_1 : Boolean;
Let_Result : Boolean;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Defining_Name_Defining_Name_P_Is_Potential_Reference,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Defining_Name,
                                As_Bare_Defining_Name => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Symbol_Type,
                                      As_Symbol_Type => Symbol);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Boolean;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7178 '<Block>' Let_Result None
--# scope-start
--# expr-start 7174 '<FieldAccess .name_is(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:11762
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_is
Fld := Libadalang.Implementation.Name_P_Name_Is (Node => Self, Sym => Symbol);
--# end
--# expr-done 7174
if Fld then
   If_Result_1 := True;
else
--# expr-start 7175 '<FieldAccess .is_derivable_equal>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11763
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.is_derivable_equal
Fld_1 := Libadalang.Implementation.Defining_Name_P_Is_Derivable_Equal (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7175
if Fld_1 then
   --# expr-start 7177 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11763
--# expr-start 7176 '<Symbol "/=">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 7176
Is_Equal := Symbol = Sym; 
--# expr-done 7177
   If_Result := Is_Equal;
else
   If_Result := False;
end if;
   If_Result_1 := If_Result;
end if;
Let_Result := If_Result_1; 
--# end
--# expr-done 7178
      Property_Result := Let_Result;
   --# end
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Boolean,
                     As_Boolean => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Potential_Reference;
--# end
--# property-start DefiningName.is_referenced_by /usr/src/packages/BUILD/ada/ast.py:11768
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Is_Referenced_By
  (Node : Bare_Defining_Name
      ; Id : Internal_Entity_Base_Id
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Ref_Result_Kind
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind id Id
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Ref_Result_Kind;
            procedure Finalizer_Scope_2501;
            procedure Finalizer_Scope_2506;
      Var_Expr : Internal_Refd_Def;
Canon : Bare_Defining_Name;
Var_Expr_1 : Internal_Entity_Defining_Name;
Var_Expr_2 : Internal_Entity_Defining_Name;
Var_Expr_3 : Internal_Entity_Basic_Decl_Array_Access;
Fld : Internal_Entity_Defining_Name;
New_Struct : Internal_Refd_Def;
Fld_1 : Internal_Refd_Def;
Fld_2 : Boolean;
If_Result : Internal_Refd_Def;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Internal_Entity_Defining_Name;
Scope_Result : Internal_Entity_Defining_Name;
Result_Var : Internal_Entity_Defining_Name;
Fld_5 : Bare_Defining_Name;
Scope_Result_1 : Bare_Defining_Name;
Result_Var_1 : Bare_Defining_Name;
Fld_6 : Ref_Result_Kind;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Defining_Name;
Is_Equal : Boolean;
Fld_7 : Internal_Entity_Basic_Decl;
Fld_8 : Internal_Entity_Basic_Decl_Array_Access;
Fld_9 : Internal_Entity_Defining_Name;
Fld_10 : Bare_Defining_Name;
Cast_Result_1 : Bare_Defining_Name;
Is_Equal_1 : Boolean;
Quantifier_Result : Boolean;
Fld_11 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Ref_Result_Kind;
Let_Result : Ref_Result_Kind;
Scope_Result_3 : Ref_Result_Kind;
Result_Var_3 : Ref_Result_Kind;
Fld_12 : Symbol_Type;
Fld_13 : Boolean;
If_Result_4 : Ref_Result_Kind;
Let_Result_1 : Ref_Result_Kind;
            procedure Finalizer_Scope_2501 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2501;
            procedure Finalizer_Scope_2506 is
            begin
                     Dec_Ref (Var_Expr_3);
            end Finalizer_Scope_2506;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7200 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7199 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:11780
--# expr-start 7180 '<FieldAccess .is_potential_reference(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11781
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7179 '<FieldAccess .name_symbol>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11781
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_12 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Id.Node);
--# end
--# expr-done 7179
--# property-call-start DefiningName.is_potential_reference
Fld_13 := Libadalang.Implementation.Defining_Name_P_Is_Potential_Reference (Node => Ent.Node, Symbol => Fld_12, E_Info => Ent.Info);
--# end
--# expr-done 7180
if Fld_13 then
--# expr-start 7185 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11783
--# expr-start 7181 '<FieldAccess .is_defining>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11783
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_defining
Fld_2 := Libadalang.Implementation.Name_P_Is_Defining (Node => Id.Node);
--# end
--# expr-done 7181
if Fld_2 then
   --# expr-start 7183 '<New InternalRefdDef>' New_Struct /usr/src/packages/BUILD/ada/ast.py:11784
--# expr-start 7182 '<FieldAccess .enclosing_defining_name>' Fld /usr/src/packages/BUILD/ada/ast.py:11784
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.enclosing_defining_name
Fld := Libadalang.Implementation.Name_P_Enclosing_Defining_Name (Node => Id.Node, E_Info => Id.Info);
--# end
--# expr-done 7182
New_Struct := (Def_Name => Fld, Kind => Precise); 
--# expr-done 7183
   If_Result := New_Struct;
else
   --# expr-start 7184 '<FieldAccess .failsafe_referenced_def_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11786
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.failsafe_referenced_def_name
Fld_1 := Libadalang.Implementation.Name_P_Failsafe_Referenced_Def_Name (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 7184
   If_Result := Fld_1;
end if;
--# expr-done 7185
Var_Expr := If_Result;
if Var_Expr /= No_Refd_Def then
   --# scope-start
   --# bind def_res Var_Expr
--# expr-start 7198 '<Let canon>' Let_Result /usr/src/packages/BUILD/ada/ast.py:11789
--# scope-start
--# expr-start 7186 '<FieldAccess .def_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11790
Fld_3 := Var_Expr.Def_Name;
--# expr-done 7186
Var_Expr_2 := Fld_3;
if Var_Expr_2 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 7187 '<FieldAccess .canonical_part>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11790
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.canonical_part
Fld_4 := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Var_Expr_2.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7187
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Defining_Name;
end if;
Var_Expr_1 := Result_Var;
if Var_Expr_1 /= No_Entity_Defining_Name then
   --# scope-start
--# expr-start 7188 '<FieldAccess .node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11790
Fld_5 := Var_Expr_1.Node;
--# expr-done 7188
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Bare_Ada_Node;
end if;
Canon := Result_Var_1; 
--# bind canon Canon
--# expr-start 7197 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:11792
--# expr-start 7189 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:11795
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Is_Equal := Cast_Result = Canon; 
--# expr-done 7189
if Is_Equal then
   If_Result_2 := True;
else
--# expr-start 7191 '<FieldAccess .base_subp_declarations>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11800
--# expr-start 7190 '<FieldAccess .basic_decl>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11800
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_7 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7190
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.base_subp_declarations
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Base_Subp_Declarations (Node => Fld_7.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_7.Info);
--# end
--# expr-done 7191
Var_Expr_3 := Fld_8;
      Inc_Ref (Var_Expr_3);
if Var_Expr_3 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind decls Var_Expr_3
Quantifier_Result := False;
   declare
      Collection : constant Internal_Entity_Basic_Decl_Array_Access := Var_Expr_3;
   begin
      for Item_214 of
            Collection.Items
      loop
   --# scope-start
            --# bind d Item_214
         --# expr-start 7194 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:11802
--# expr-start 7193 '<FieldAccess .node>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11802
--# expr-start 7192 '<FieldAccess .defining_name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11802
      if Item_214.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_9 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Item_214.Node, E_Info => Item_214.Info);
--# end
--# expr-done 7192
Fld_10 := Fld_9.Node;
--# expr-done 7193
      Cast_Result_1 := Fld_10;
Is_Equal_1 := Cast_Result_1 = Canon; 
--# expr-done 7194
            if Is_Equal_1 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
if Quantifier_Result then
   --# expr-start 7195 '<FieldAccess .is_dispatching_call>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11804
      if Id.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.is_dispatching_call'
Fld_11 := Libadalang.Implementation.Dispatcher_Expr_P_Is_Dispatching_Call (Node => Id.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Id.Info);
--# end
--# expr-done 7195
   If_Result_1 := Fld_11;
else
   If_Result_1 := False;
end if;
Scope_Result_2 := If_Result_1;
   --# end
      Finalizer_Scope_2506;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
   If_Result_2 := Result_Var_2;
end if;
if If_Result_2 then
   --# expr-start 7196 '<FieldAccess .kind>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11807
Fld_6 := Var_Expr.Kind;
--# expr-done 7196
   If_Result_3 := Fld_6;
else
   If_Result_3 := Noref;
end if;
--# expr-done 7197
Let_Result := If_Result_3; 
--# end
--# expr-done 7198
Scope_Result_3 := Let_Result;
   --# end
      Finalizer_Scope_2501;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := Noref;
end if;
   If_Result_4 := Result_Var_3;
else
   If_Result_4 := Noref;
end if;
--# expr-done 7199
Let_Result_1 := If_Result_4; 
--# end
--# expr-done 7200
      Property_Result := Let_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_2501;
               Finalizer_Scope_2506;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Is_Referenced_By;
--# end
--# property-start DefiningName.find_all_references /usr/src/packages/BUILD/ada/ast.py:11818
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_References
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_925;
            procedure Finalizer_Scope_2508;
            procedure Finalizer_Scope_2509;
            procedure Finalizer_Scope_2510;
            procedure Finalizer_Scope_2511;
            procedure Finalizer_Scope_2512;
            procedure Finalizer_Scope_2515;
            procedure Finalizer_Scope_2517;
      Block_Var : Internal_Entity_Defining_Name;
Block_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_2 : Internal_Unit_Array_Access;
Block_Var_3 : Internal_Ref_Result_Array_Access;
Var_Expr : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_1 : Bare_Ada_Node;
Var_Expr_2 : Internal_Entity_Renaming_Clause;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name;
Cast_Expr : Bare_Defining_Name;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_3 : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_4 : Internal_Unit_Array_Access;
Map_Result : Internal_Unit_Array_Access;
Unique_Array : Internal_Unit_Array_Access;
Cast_Expr_1 : Bare_Defining_Name;
Cast_Result_1 : Bare_Ada_Node;
Bound_Origin_1 : Bare_Ada_Node;
Fld_5 : Bare_Ada_Node;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_6 : Internal_Ref_Result_Array_Access;
Scope_Result_1 : Internal_Ref_Result_Array_Access;
Result_Var_1 : Internal_Ref_Result_Array_Access;
Map_Result_1 : Internal_Ref_Result_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Ref_Result_Array_Access;
Fld_7 : Internal_Entity_Base_Id;
Fld_8 : Internal_Entity_Array_Access;
Is_A : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Renaming_Clause;
Fld_9 : Internal_Entity_Name;
Fld_10 : Internal_Entity_Defining_Name;
Cast_Result_3 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Defining_Name;
Cast_Result_4 : Internal_Entity;
Is_Equiv : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result_3 : Internal_Ref_Result_Array_Access;
Fld_11 : Internal_Entity_Base_Id;
Fld_12 : Internal_Entity_Array_Access;
Is_A_1 : Boolean;
Map_Result_4 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_13 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Basic_Decl;
Fld_14 : Internal_Entity_Defining_Name;
Fld_15 : Internal_Ref_Result_Array_Access;
Scope_Result_3 : Internal_Ref_Result_Array_Access;
Result_Var_3 : Internal_Ref_Result_Array_Access;
Map_Result_5 : Internal_Ref_Result_Array_Access;
Concat_Result : Internal_Ref_Result_Array_Access;
If_Result_1 : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_925 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Singleton);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Unique_Array);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Map_Result_3);
                     Dec_Ref (Map_Result_5);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_925;
            procedure Finalizer_Scope_2508 is
            begin
                     Dec_Ref (Var_Expr);
            end Finalizer_Scope_2508;
            procedure Finalizer_Scope_2509 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2509;
            procedure Finalizer_Scope_2510 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2510;
            procedure Finalizer_Scope_2511 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2511;
            procedure Finalizer_Scope_2512 is
            begin
                     Dec_Ref (Fld_8);
                     Dec_Ref (Map_Result_2);
            end Finalizer_Scope_2512;
            procedure Finalizer_Scope_2515 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Map_Result_4);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
            end Finalizer_Scope_2515;
            procedure Finalizer_Scope_2517 is
            begin
                     Dec_Ref (Fld_15);
            end Finalizer_Scope_2517;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7236 '<Block>' Let_Result None
--# scope-start
--# expr-start 7201 '<FieldAccess .canonical_part>' Fld /usr/src/packages/BUILD/ada/ast.py:11828
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.canonical_part
Fld := Libadalang.Implementation.Defining_Name_P_Canonical_Part (Node => Ent.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7201
Block_Var := Fld; 
--# bind dn Block_Var
--# expr-start 7205 '<bind(<DynamicVariable origin>, <Var Self>, <Then <FieldAccess .root_subp_declarations>: <Var e> <Var e>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:11838
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 7203 '<FieldAccess .root_subp_declarations>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11840
--# expr-start 7202 '<FieldAccess .basic_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11840
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_1 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 7202
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.root_subp_declarations
Fld_2 := Libadalang.Implementation.Basic_Decl_P_Root_Subp_Declarations (Node => Fld_1.Node, Origin => Bound_Origin, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 7203
Var_Expr := Fld_2;
      Inc_Ref (Var_Expr);
if Var_Expr /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr
Scope_Result := Var_Expr;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2508;
   Result_Var := Scope_Result;
else
   --# expr-start 7204 '<FieldAccess .basic_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11840
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld_3 := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# end
--# expr-done 7204
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Fld_3;
   Result_Var := Singleton;
end if;
      Inc_Ref (Result_Var);
Dyn_Var_Bind_Result := Result_Var; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7205
Block_Var_1 := Dyn_Var_Bind_Result; Inc_Ref (Block_Var_1);
--# bind bases Block_Var_1
--# expr-start 7208 '<unique(<mapcat>)>' Unique_Array /usr/src/packages/BUILD/ada/ast.py:11843
--# expr-start 7207 '<mapcat>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11843
declare
   Map_Result_Vec : Internal_Unit_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Block_Var_1;
      begin
         for Item_215 of
               Collection.Items
         loop
   --# scope-start
               --# bind base Item_215
      --# expr-start 7206 '<FieldAccess .filter_is_imported_by(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11844
      if Item_215.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Ada_Node_P_Filter_Is_Imported_By (Node => Item_215.Node, Units => Units, Transitive => True);
--# expr-done 7206
         for Item_To_Append of
               Fld_4.Items
         loop
            Internal_Unit_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2509;
         end loop;
      end;
      Map_Result := Create_Internal_Unit_Array
        (Items_Count => Natural (Internal_Unit_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Unit_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Unit_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Unit_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7207
Unique_Array := Make_Unique (Map_Result); 
--# expr-done 7208
Block_Var_2 := Unique_Array; Inc_Ref (Block_Var_2);
--# bind all_units Block_Var_2
--# expr-start 7213 '<bind(<DynamicVariable origin>, <Var Self>, <mapcat>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:11847
Cast_Expr_1 := Self; 
      Cast_Result_1 := Cast_Expr_1;
Bound_Origin_1 := Cast_Result_1; 
--# expr-start 7212 '<mapcat>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11847
declare
   Map_Result_1_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Unit_Array_Access := Block_Var_2;
      begin
         for Item_216 of
               Collection.Items
         loop
   --# scope-start
               --# bind u Item_216
--# expr-start 7209 '<FieldAccess .root>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11847
      if Item_216 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Item_216.AST_Root;
--# expr-done 7209
Var_Expr_1 := Fld_5;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind r Var_Expr_1
--# expr-start 7211 '<FieldAccess .find_refs(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11848
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7210 '<as_bare_entity(<Var r>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:11848
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity;
else
As_Entity := (Info => No_Entity_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7210
--# property-call-start DefiningName.find_refs
Fld_6 := Libadalang.Implementation.Defining_Name_P_Find_Refs (Node => Block_Var.Node, Root => If_Result, Origin => Bound_Origin_1, Imprecise_Fallback => Imprecise_Fallback, E_Info => Block_Var.Info);
--# end
--# expr-done 7211
Scope_Result_1 := Fld_6;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2511;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
         for Item_To_Append of
               Result_Var_1.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2510;
         end loop;
      end;
      Map_Result_1 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7212
Dyn_Var_Bind_Result_1 := Map_Result_1; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 7213
Block_Var_3 := Dyn_Var_Bind_Result_1; Inc_Ref (Block_Var_3);
--# bind refs Block_Var_3
--# expr-start 7235 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11851
if Follow_Renamings then
   --# expr-start 7234 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:11853
--# expr-start 7233 '<mapcat>' Map_Result_5 /usr/src/packages/BUILD/ada/ast.py:11854
--# expr-start 7223 '<filter>' Map_Result_3 /usr/src/packages/BUILD/ada/ast.py:11854
declare
   Map_Result_3_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Block_Var_3;
      begin
         for Item_218 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_218
--# expr-start 7219 '<Cast to RenamingClause>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:11860
--# expr-start 7218 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11860
--# expr-start 7217 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:11860
--# expr-start 7215 '<FieldAccess .parents>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11860
--# expr-start 7214 '<FieldAccess .ref>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11860
Fld_7 := Item_218.Ref;
--# expr-done 7214
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Parents (Node => Fld_7.Node, With_Self => True, E_Info => Fld_7.Info);
--# expr-done 7215
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_8;
      begin
         for Item_219 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_219
               --# expr-start 7216 '<IsA RenamingClause>' Is_A /usr/src/packages/BUILD/ada/ast.py:11861
Is_A := Item_219.Node /= null 
and then Item_219.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 7216
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_219;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7217
Get_Result := Get (Map_Result_2, 0, True); 
--# expr-done 7218
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Renaming_Clause_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Renaming_Clause
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Renaming_Clause;
   end if;
--# expr-done 7219
Var_Expr_2 := Cast_Result_2;
if Var_Expr_2 /= No_Entity_Renaming_Clause then
   --# scope-start
   --# bind r Var_Expr_2
--# expr-start 7222 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:11864
--# expr-start 7221 '<FieldAccess .referenced_defining_name>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:11864
--# expr-start 7220 '<FieldAccess .renamed_object>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11864
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity_Name (Node => Var_Expr_2.Node.Renaming_Clause_F_Renamed_Object, Info => Var_Expr_2.Info);
--# expr-done 7220
      if Fld_9.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_defining_name
Fld_10 := Libadalang.Implementation.Name_P_Referenced_Defining_Name (Node => Fld_9.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_9.Info);
--# end
--# expr-done 7221
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_10.Node,
         Info => Fld_10.Info);
Cast_Expr_2 := Ent; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_3, Cast_Result_4); 
--# expr-done 7222
Scope_Result_2 := Is_Equiv;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
               if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result :=
               Item_218;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_3_Vec, Item_To_Append);
         end;
               end if;
   --# end
      Finalizer_Scope_2512;
         end loop;
      end;
      Map_Result_3 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_3_Vec)));
      for I in Map_Result_3.Items'Range loop
         Map_Result_3.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_3_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_3.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_3_Vec);
end;
--# expr-done 7223
declare
   Map_Result_5_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Map_Result_3;
      begin
         for Item_217 of
               Collection.Items
         loop
   --# scope-start
               --# bind f Item_217
--# expr-start 7230 '<Cast to BasicDecl>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:11870
--# expr-start 7229 '<parent(<at(<filter>, 0)>)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:11870
--# expr-start 7228 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:11870
--# expr-start 7227 '<filter>' Map_Result_4 /usr/src/packages/BUILD/ada/ast.py:11870
--# expr-start 7225 '<FieldAccess .parents>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:11870
--# expr-start 7224 '<FieldAccess .ref>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:11870
Fld_11 := Item_217.Ref;
--# expr-done 7224
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Parents (Node => Fld_11.Node, With_Self => True, E_Info => Fld_11.Info);
--# expr-done 7225
declare
   Map_Result_4_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_12;
      begin
         for Item_220 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_220
               --# expr-start 7226 '<IsA RenamingClause>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:11871
Is_A_1 := Item_220.Node /= null 
and then Item_220.Node.Kind in Ada_Renaming_Clause_Range; 
--# expr-done 7226
               if Is_A_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_220;
         begin
            Internal_Entity_Vectors.Append (Map_Result_4_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_4 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_4_Vec)));
      for I in Map_Result_4.Items'Range loop
         Map_Result_4.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_4_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_4.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_4_Vec);
end;
--# expr-done 7227
Get_Result_1 := Get (Map_Result_4, 0, True); 
--# expr-done 7228
      if Get_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Parent (Node => Get_Result_1.Node, E_Info => Get_Result_1.Info);
--# expr-done 7229
   if Fld_13.Node = null
      or else Fld_13.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_5 := Create_Internal_Entity_Basic_Decl
        (Node => Fld_13.Node,
         Info => Fld_13.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7230
Var_Expr_3 := Cast_Result_5;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr_3
--# expr-start 7232 '<FieldAccess .find_all_references(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:11873
--# expr-start 7231 '<FieldAccess .defining_name>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:11873
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_14 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 7231
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.find_all_references
Fld_15 := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Fld_14.Node, Units => Units, Follow_Renamings => True, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_14.Info);
--# end
--# expr-done 7232
Scope_Result_3 := Fld_15;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2517;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Ref_Result_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
         for Item_To_Append of
               Result_Var_3.Items
         loop
            Internal_Ref_Result_Vectors.Append (Map_Result_5_Vec, Item_To_Append);
         end loop;
   --# end
      Finalizer_Scope_2515;
         end loop;
      end;
      Map_Result_5 := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_5_Vec)));
      for I in Map_Result_5.Items'Range loop
         Map_Result_5.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_5_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result_5.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_5_Vec);
end;
--# expr-done 7233
Concat_Result := Concat (Block_Var_3, Map_Result_5); 
--# expr-done 7234
   If_Result_1 := Concat_Result;
else
   If_Result_1 := Block_Var_3;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7235
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7236
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_925;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_925;
               Finalizer_Scope_2508;
               Finalizer_Scope_2509;
               Finalizer_Scope_2510;
               Finalizer_Scope_2511;
               Finalizer_Scope_2512;
               Finalizer_Scope_2515;
               Finalizer_Scope_2517;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_All_References;
--# end
--# property-start DefiningName.find_matching_name /usr/src/packages/BUILD/ada/ast.py:11884
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_Matching_Name
  (Node : Bare_Defining_Name
      ; Bd : Internal_Entity_Basic_Decl
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind bd Bd
   Property_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_926;
            procedure Finalizer_Scope_2503;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Defining_Name_Array_Access;
Scope_Result : Internal_Entity_Defining_Name_Array_Access;
Result_Var : Internal_Entity_Defining_Name_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Defining_Name_Array_Access;
Get_Result : Internal_Entity_Defining_Name;
Let_Result : Internal_Entity_Defining_Name;
            procedure Finalizer_Scope_926 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_926;
            procedure Finalizer_Scope_2503 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_2503;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7243 '<Block>' Let_Result None
--# scope-start
--# expr-start 7242 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11889
--# expr-start 7241 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11889
Var_Expr := Bd;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7237 '<FieldAccess .defining_names>' Fld /usr/src/packages/BUILD/ada/ast.py:11889
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_names'
Fld := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Names (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7237
Scope_Result := Fld;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2503;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Defining_Name_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Defining_Name_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Defining_Name_Array_Access := Result_Var;
      begin
         for Item_221 of
               Collection.Items
         loop
   --# scope-start
               --# bind di Item_221
               --# expr-start 7240 '<FieldAccess .name_is(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11890
--# expr-start 7238 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11890
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 7238
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7239 '<FieldAccess .name_symbol>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11890
      if Item_221.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Item_221.Node);
--# end
--# expr-done 7239
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Fld_2);
--# end
--# expr-done 7240
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Entity_Defining_Name :=
               Item_221;
         begin
            Internal_Entity_Defining_Name_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Defining_Name_Array
        (Items_Count => Natural (Internal_Entity_Defining_Name_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Defining_Name_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Defining_Name_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Defining_Name_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7241
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7242
Let_Result := Get_Result; 
--# end
--# expr-done 7243
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_926;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_926;
               Finalizer_Scope_2503;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_Matching_Name;
--# end
--# property-start DefiningName.find_all_calls /usr/src/packages/BUILD/ada/ast.py:11895
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Find_All_Calls
  (Node : Bare_Defining_Name
      ; Units : Internal_Unit_Array_Access
      ; Follow_Renamings : Boolean
         := False
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Ref_Result_Array_Access
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind units Units
   --# bind follow_renamings Follow_Renamings
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_927;
      Fld : Internal_Ref_Result_Array_Access;
Fld_1 : Internal_Entity_Base_Id;
Fld_2 : Boolean;
Map_Result : Internal_Ref_Result_Array_Access;
Let_Result : Internal_Ref_Result_Array_Access;
            procedure Finalizer_Scope_927 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_927;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7248 '<Block>' Let_Result None
--# scope-start
--# expr-start 7247 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11907
--# expr-start 7244 '<FieldAccess .find_all_references(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:11907
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.find_all_references
Fld := Libadalang.Implementation.Defining_Name_P_Find_All_References (Node => Ent.Node, Units => Units, Follow_Renamings => Follow_Renamings, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7244
declare
   Map_Result_Vec : Internal_Ref_Result_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Ref_Result_Array_Access := Fld;
      begin
         for Item_222 of
               Collection.Items
         loop
   --# scope-start
               --# bind r Item_222
               --# expr-start 7246 '<FieldAccess .is_direct_call>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11908
--# expr-start 7245 '<FieldAccess .ref>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11908
Fld_1 := Item_222.Ref;
--# expr-done 7245
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_direct_call
Fld_2 := Libadalang.Implementation.Name_P_Is_Direct_Call (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 7246
               if Fld_2 then
         declare
            Item_To_Append : constant Internal_Ref_Result :=
               Item_222;
         begin
            Internal_Ref_Result_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Ref_Result_Array
        (Items_Count => Natural (Internal_Ref_Result_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Ref_Result_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Ref_Result_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Ref_Result_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7247
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7248
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_927;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_927;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Find_All_Calls;
--# end
--# property-start DefiningName.next_part /usr/src/packages/BUILD/ada/ast.py:11911
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Next_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7251 '<FieldAccess .find_matching_name(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11912
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7250 '<FieldAccess .next_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11912
--# expr-start 7249 '<FieldAccess .basic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:11912
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7249
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.next_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Next_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7250
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 7251
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Next_Part;
--# end
--# property-start DefiningName.previous_part /usr/src/packages/BUILD/ada/ast.py:11918
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Previous_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7254 '<FieldAccess .find_matching_name(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11919
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7253 '<FieldAccess .previous_part_for_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11919
--# expr-start 7252 '<FieldAccess .basic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:11919
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7252
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.previous_part_for_decl'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Previous_Part_For_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7253
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 7254
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Previous_Part;
--# end
--# property-start DefiningName.canonical_part /usr/src/packages/BUILD/ada/ast.py:11925
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Canonical_Part
  (Node : Bare_Defining_Name
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Defining_Name
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Entity_Defining_Name;
      Fld : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Internal_Entity_Defining_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7257 '<FieldAccess .find_matching_name(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11926
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7256 '<FieldAccess .canonical_part>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11926
--# expr-start 7255 '<FieldAccess .basic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:11926
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start DefiningName.basic_decl
Fld := Libadalang.Implementation.Defining_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7255
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_part
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Part (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7256
--# property-call-start DefiningName.find_matching_name
Fld_2 := Libadalang.Implementation.Defining_Name_P_Find_Matching_Name (Node => Ent.Node, Bd => Fld_1, E_Info => Ent.Info);
--# end
--# expr-done 7257
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Canonical_Part;
--# end
--# property-start DefiningName.xref_equation /usr/src/packages/BUILD/ada/ast.py:11933
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Equation
  (Node : Bare_Defining_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Defining_Name :=
     Internal_Entity_Defining_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_931;
            procedure Finalizer_Scope_2519;
      Var_Expr : Internal_Entity_Dotted_Name;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Dotted_Name;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_931 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_931;
            procedure Finalizer_Scope_2519 is
            begin
                     Dec_Ref (Fld_2);
            end Finalizer_Scope_2519;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7262 '<Block>' Let_Result None
--# scope-start
--# expr-start 7259 '<Cast to DottedName>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11937
--# expr-start 7258 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11937
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Defining_Name_F_Name, Info => Ent.Info);
--# expr-done 7258
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Dotted_Name_Range
   then
      Cast_Result := Create_Internal_Entity_Dotted_Name
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Dotted_Name;
   end if;
--# expr-done 7259
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Dotted_Name then
   --# scope-start
   --# bind dn Var_Expr
--# expr-start 7261 '<FieldAccess .xref_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11938
--# expr-start 7260 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11938
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Dotted_Name_F_Prefix, Info => Var_Expr.Info);
--# expr-done 7260
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 7261
Scope_Result := Fld_2;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2519;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11939") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7262
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_931;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_931;
               Finalizer_Scope_2519;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Xref_Equation;
--# end
--# property-start DefiningName.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:11944
pragma Warnings (Off, "is not referenced");
 function Defining_Name_P_Xref_Entry_Point
  (Node : Bare_Defining_Name
  )
   return Boolean
is
   Self : Bare_Defining_Name := Bare_Defining_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Name;
Is_A : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7264 '<IsA DottedName>' Is_A /usr/src/packages/BUILD/ada/ast.py:11944
--# expr-start 7263 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11944
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Defining_Name_F_Name;
--# expr-done 7263
Is_A := Fld /= null 
and then Fld.Kind in Ada_Dotted_Name_Range; 
--# expr-done 7264
      Property_Result := Is_A;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Defining_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Discrete_Subtype_Name
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Name
        (Self : Bare_Discrete_Subtype_Name
         ; Discrete_Subtype_Name_F_Subtype : Bare_Discrete_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Discrete_Subtype_Name_F_Subtype := Discrete_Subtype_Name_F_Subtype;
      end Initialize_Fields_For_Discrete_Subtype_Name;
   function Discrete_Subtype_Name_F_Subtype
     (Node : Bare_Discrete_Subtype_Name) return Bare_Discrete_Subtype_Indication
   is
   begin
         return Node.Discrete_Subtype_Name_F_Subtype;
   end;
   --
   --  Primitives for Bare_Dotted_Name
   --
      procedure Initialize_Fields_For_Dotted_Name
        (Self : Bare_Dotted_Name
         ; Dotted_Name_F_Prefix : Bare_Name
         ; Dotted_Name_F_Suffix : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Dotted_Name_F_Prefix := Dotted_Name_F_Prefix;
            Self.Dotted_Name_F_Suffix := Dotted_Name_F_Suffix;
      end Initialize_Fields_For_Dotted_Name;
   function Dotted_Name_F_Prefix
     (Node : Bare_Dotted_Name) return Bare_Name
   is
   begin
         return Node.Dotted_Name_F_Prefix;
   end;
   function Dotted_Name_F_Suffix
     (Node : Bare_Dotted_Name) return Bare_Base_Id
   is
   begin
         return Node.Dotted_Name_F_Suffix;
   end;
--# property-start DottedName.ref_var /usr/src/packages/BUILD/ada/ast.py:14043
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Ref_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7266 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14043
--# expr-start 7265 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:14043
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 7265
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7266
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Ref_Var;
--# end
--# property-start DottedName.subp_spec_var /usr/src/packages/BUILD/ada/ast.py:14045
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Logic_Var
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Base_Id;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7268 '<FieldAccess .subp_spec_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14045
--# expr-start 7267 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:14045
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 7267
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Fld);
--# end
--# expr-done 7268
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Subp_Spec_Var;
--# end
--# property-start DottedName.defines_subp_spec_var /usr/src/packages/BUILD/ada/ast.py:14046
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Defines_Subp_Spec_Var
  (Node : Bare_Dotted_Name
  )
   return Boolean
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Defines_Subp_Spec_Var;
--# end
--# property-start DottedName.complete /usr/src/packages/BUILD/ada/ast.py:14049
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Complete
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Completion_Item_Iterator_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_1064;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Env_Get_Result : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Info;
Fld_5 : Internal_Metadata;
Fld_6 : Boolean;
Fld_7 : Internal_Unit;
Fld_8 : Boolean;
New_Struct : Internal_Completion_Item;
Map_Result : Internal_Completion_Item_Array_Access;
Iter : Internal_Completion_Item_Iterator_Access;
Dyn_Var_Bind_Result : Internal_Completion_Item_Iterator_Access;
Dyn_Var_Bind_Result_1 : Internal_Completion_Item_Iterator_Access;
Let_Result : Internal_Completion_Item_Iterator_Access;
            procedure Finalizer_Scope_1064 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Iter);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1064;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7285 '<Block>' Let_Result None
--# scope-start
--# expr-start 7284 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .to_iterator>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:14050
--# expr-start 7269 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:14050
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7269
Bound_Origin := Fld; 
--# expr-start 7283 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .to_iterator>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14050
--# expr-start 7270 '<FieldAccess .node_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14051
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 7270
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 7282 '<FieldAccess .to_iterator>' Iter /usr/src/packages/BUILD/ada/ast.py:14052
--# expr-start 7281 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14052
--# expr-start 7273 '<EnvGet(<FieldAccess .designated_env>, <No SymbolType>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:14052
--# expr-start 7272 '<FieldAccess .designated_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14052
--# expr-start 7271 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14052
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7271
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7272
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_3, Key => null, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => True, Nocat => True))); 
--# expr-done 7273
declare
   Map_Result_Vec : Internal_Completion_Item_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Env_Get_Result;
      begin
         for Item_223 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_223
      --# expr-start 7280 '<New InternalCompletionItem>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14053
--# expr-start 7274 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14054
Cast_Expr := Item_223; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7274
--# expr-start 7277 '<FieldAccess .dottable_subp>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14055
--# expr-start 7276 '<FieldAccess .md>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14055
--# expr-start 7275 '<FieldAccess .info>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14055
Fld_4 := Item_223.Info;
--# expr-done 7275
Fld_5 := Fld_4.MD;
--# expr-done 7276
Fld_6 := Fld_5.Dottable_Subp;
--# expr-done 7277
--# expr-start 7279 '<FieldAccess .has_with_visibility(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14056
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7278 '<FieldAccess .unit>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14056
      if Item_223.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Unit (Node => Item_223.Node);
--# expr-done 7278
--# property-call-start AdaNode.has_with_visibility
Fld_8 := Libadalang.Implementation.Ada_Node_P_Has_With_Visibility (Node => Self, Refd_Unit => Fld_7);
--# end
--# expr-done 7279
New_Struct := (Decl => Cast_Result, Is_Dot_Call => Fld_6, Is_Visible => Fld_8); 
--# expr-done 7280
         declare
            Item_To_Append : constant Internal_Completion_Item :=
               New_Struct;
         begin
            Internal_Completion_Item_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Completion_Item_Array
        (Items_Count => Natural (Internal_Completion_Item_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Completion_Item_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Completion_Item_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Completion_Item_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7281
Iter := To_Iterator (Map_Result, Self.Unit.Context); 
--# expr-done 7282
Dyn_Var_Bind_Result := Iter; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7283
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 7284
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7285
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1064;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1064;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Complete;
--# end
--# property-start DottedName.designated_env_no_overloading /usr/src/packages/BUILD/ada/ast.py:14062
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env_No_Overloading
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1065;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1065 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1065;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7291 '<Block>' Let_Result None
--# scope-start
--# expr-start 7287 '<FieldAccess .designated_env_no_overloading>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14063
--# expr-start 7286 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14063
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7286
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7287
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 7290 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .designated_env_no_overloading>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14064
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 7289 '<FieldAccess .designated_env_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14065
--# expr-start 7288 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14065
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7288
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7289
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7290
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7291
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1065;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1065;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Env_No_Overloading;
--# end
--# property-start DottedName.designated_env /usr/src/packages/BUILD/ada/ast.py:14068
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Env
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1066;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1066 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1066;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7297 '<Block>' Let_Result None
--# scope-start
--# expr-start 7293 '<FieldAccess .designated_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14069
--# expr-start 7292 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14069
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7292
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7293
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 7296 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14070
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 7295 '<FieldAccess .designated_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14070
--# expr-start 7294 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14070
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7294
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7295
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7296
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7297
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1066;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1066;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Env;
--# end
--# property-start DottedName.all_env_els_impl /usr/src/packages/BUILD/ada/ast.py:14073
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_All_Env_Els_Impl
  (Node : Bare_Dotted_Name
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1067;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1067 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1067;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7303 '<Block>' Let_Result None
--# scope-start
--# expr-start 7299 '<FieldAccess .designated_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14078
--# expr-start 7298 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14078
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7298
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7299
Block_Var := Fld_1; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 7302 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .all_env_els_impl(...)>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14079
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 7301 '<FieldAccess .all_env_els_impl(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14081
--# expr-start 7300 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14081
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7300
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld_2.Node, Seq => Seq, Seq_From => Seq_From, Categories => Categories, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7301
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7302
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7303
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1067;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1067;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_All_Env_Els_Impl;
--# end
--# property-start DottedName.scope /usr/src/packages/BUILD/ada/ast.py:14084
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1068;
            procedure Finalizer_Scope_2521;
      Var_Expr : Bare_Base_Id;
Fld : Bare_Base_Id;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1068 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_1068;
            procedure Finalizer_Scope_2521 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_2521;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 7304 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:14084
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Suffix;
--# expr-done 7304
Var_Expr := Fld;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind sfx Var_Expr
--# expr-start 7307 '<bind(<DynamicVariable env>, <FieldAccess .parent_scope>, <FieldAccess .scope>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14085
--# expr-start 7305 '<FieldAccess .parent_scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14085
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Self, Env => Env);
--# end
--# expr-done 7305
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 7306 '<FieldAccess .scope>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14085
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Var_Expr, Env => Bound_Env);
--# end
--# expr-done 7306
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7307
Scope_Result := Dyn_Var_Bind_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2521;
   Result_Var := Scope_Result;
else
   Result_Var := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1068;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1068;
               Finalizer_Scope_2521;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Scope;
--# end
--# property-start DottedName.parent_scope /usr/src/packages/BUILD/ada/ast.py:14089
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Parent_Scope
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1069;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1069 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1069;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7309 '<FieldAccess .scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14089
--# expr-start 7308 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14089
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Dotted_Name_F_Prefix;
--# expr-done 7308
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 7309
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1069;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1069;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Parent_Scope;
--# end
--# property-start DottedName.relative_name /usr/src/packages/BUILD/ada/ast.py:14091
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Relative_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Base_Id;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7311 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14091
--# expr-start 7310 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:14091
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7310
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7311
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Relative_Name;
--# end
--# property-start DottedName.base_name /usr/src/packages/BUILD/ada/ast.py:14092
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Base_Name
  (Node : Bare_Dotted_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Fld : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7312 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14092
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7312
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Base_Name;
--# end
--# property-start DottedName.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:14095
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Env_Elements_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1072;
      Block_Var : Lexical_Env := Empty_Env;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_3 : Internal_Entity_Base_Id;
Fld_4 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1072 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1072;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7320 '<Block>' Let_Result None
--# scope-start
--# expr-start 7316 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14096
--# expr-start 7313 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:14096
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7313
Bound_Origin := Fld; 
--# expr-start 7315 '<FieldAccess .designated_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14097
--# expr-start 7314 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14097
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7314
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_1.Node, Env => Env, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 7315
Dyn_Var_Bind_Result := Fld_2; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7316
Block_Var := Dyn_Var_Bind_Result; Inc_Ref (Block_Var);
--# bind pfx_env Block_Var
--# expr-start 7319 '<bind(<DynamicVariable env>, <Var pfx_env>, <FieldAccess .env_elements_baseid>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:14098
Bound_Env := Block_Var; Inc_Ref (Bound_Env);
--# expr-start 7318 '<FieldAccess .env_elements_baseid>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14098
--# expr-start 7317 '<FieldAccess .suffix>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14098
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7317
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Fld_3.Node, Env => Bound_Env, E_Info => Fld_3.Info);
--# end
--# expr-done 7318
Dyn_Var_Bind_Result_1 := Fld_4; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 7319
Let_Result := Dyn_Var_Bind_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7320
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1072;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1072;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Env_Elements_Impl;
--# end
--# property-start DottedName.designated_type_impl /usr/src/packages/BUILD/ada/ast.py:14101
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Designated_Type_Impl
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1073;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Base_Id;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1073 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1073;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7326 '<Block>' Let_Result None
--# scope-start
--# expr-start 7325 '<bind(<DynamicVariable env>, <FieldAccess .designated_env_no_overloading>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14102
--# expr-start 7322 '<FieldAccess .designated_env_no_overloading>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14102
--# expr-start 7321 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14102
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7321
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env_no_overloading'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env_No_Overloading (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7322
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 7324 '<FieldAccess .designated_type_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14103
--# expr-start 7323 '<FieldAccess .suffix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14103
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7323
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7324
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 7325
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 7326
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1073;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1073;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Designated_Type_Impl;
--# end
--# property-start DottedName.xref_equation /usr/src/packages/BUILD/ada/ast.py:14106
pragma Warnings (Off, "is not referenced");
 function Dotted_Name_P_Xref_Equation
  (Node : Bare_Dotted_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Dotted_Name := Bare_Dotted_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Dotted_Name :=
     Internal_Entity_Dotted_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1074;
            procedure Finalizer_Scope_2533;
      Block_Var : Logic_Equation;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Base_Id;
Fld_5 : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Internal_Entity_Array_Access;
Fld_7 : Bare_Base_Id;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Name;
Cast_Result_1 : Bare_Expr;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Var;
Fld_12 : Bare_Base_Id;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_15 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1074 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1074;
            procedure Finalizer_Scope_2533 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2533;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7355 '<Block>' Let_Result None
--# scope-start
--# expr-start 7334 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14107
--# expr-start 7328 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14107
--# expr-start 7327 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:14107
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7327
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7328
--# expr-start 7333 '<bind(<DynamicVariable env>, <FieldAccess .designated_env>, <FieldAccess .sub_equation>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:14108
--# expr-start 7330 '<FieldAccess .designated_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14108
--# expr-start 7329 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14108
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Dotted_Name_F_Prefix, Info => Ent.Info);
--# expr-done 7329
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7330
Bound_Env := Fld_3; Inc_Ref (Bound_Env);
--# expr-start 7332 '<FieldAccess .sub_equation>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14109
--# expr-start 7331 '<FieldAccess .suffix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14109
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Base_Id (Node => Ent.Node.Dotted_Name_F_Suffix, Info => Ent.Info);
--# expr-done 7331
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_4.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 7332
Dyn_Var_Bind_Result := Fld_5; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7333
And_Pred := Logic_And (Fld_1, Dyn_Var_Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14107") else null)); 
--# expr-done 7334
Block_Var := And_Pred; Inc_Ref (Block_Var);
--# bind base Block_Var
--# expr-start 7354 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14111
--# expr-start 7337 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:14112
--# expr-start 7336 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:14112
--# expr-start 7335 '<FieldAccess .designated_type_impl>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:14112
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7335
Is_Null := Fld_15.Node = null; 
--# expr-done 7336
Not_Val := not (Is_Null); 
--# expr-done 7337
if Not_Val then
   If_Result := Block_Var;
else
   --# expr-start 7353 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:14114
--# expr-start 7352 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:14114
--# expr-start 7351 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14114
--# expr-start 7338 '<FieldAccess .env_elements>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14114
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld_6 := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7338
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_6;
      begin
         for Item_224 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_224
      --# expr-start 7350 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:14115
--# expr-start 7345 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14115
--# expr-start 7341 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:14115
--# expr-start 7340 '<FieldAccess .ref_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14115
--# expr-start 7339 '<FieldAccess .suffix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14115
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Dotted_Name_F_Suffix;
--# expr-done 7339
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 7340
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result := Bind_Default_Default.Create (Fld_8, Item_224, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14115") else null)); 
--# expr-done 7341
--# expr-start 7344 '<FieldAccess .constrain_prefix(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:14116
--# expr-start 7342 '<Cast to BasicDecl.entity>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14116
Cast_Expr := Item_224; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7342
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7343 '<FieldAccess .prefix>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:14116
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Dotted_Name_F_Prefix;
--# expr-done 7343
      Cast_Result_1 := Fld_9;
--# property-call-start '[dispatcher]BasicDecl.constrain_prefix'
Fld_10 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Constrain_Prefix (Node => Cast_Result.Node, Prefix => Cast_Result_1, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 7344
And_Pred_1 := Logic_And (Bind_Result, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14115") else null)); 
--# expr-done 7345
--# expr-start 7349 '<FieldAccess .type_bind_var(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:14117
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7346 '<FieldAccess .type_var>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:14117
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7346
--# expr-start 7348 '<FieldAccess .type_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:14117
--# expr-start 7347 '<FieldAccess .suffix>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:14117
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.Dotted_Name_F_Suffix;
--# expr-done 7347
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_12);
--# expr-done 7348
--# property-call-start AdaNode.type_bind_var
Fld_14 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_11, Right => Fld_13, Origin => Origin);
--# end
--# expr-done 7349
And_Pred_2 := Logic_And (And_Pred_1, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14115") else null)); 
--# expr-done 7350
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_2;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2533;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7351
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14114") else null)); 
--# expr-done 7352
And_Pred_3 := Logic_And (Block_Var, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14114") else null)); 
--# expr-done 7353
   If_Result := And_Pred_3;
end if;
      Inc_Ref (If_Result);
--# expr-done 7354
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7355
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1074;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1074;
               Finalizer_Scope_2533;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dotted_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_End_Name
   --
      procedure Initialize_Fields_For_End_Name
        (Self : Bare_End_Name
         ; End_Name_F_Name : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.End_Name_F_Name := End_Name_F_Name;
      end Initialize_Fields_For_End_Name;
   function End_Name_F_Name
     (Node : Bare_End_Name) return Bare_Name
   is
   begin
         return Node.End_Name_F_Name;
   end;
--# property-start EndName.parent_scope /usr/src/packages/BUILD/ada/ast.py:11954
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Parent_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_933;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_933 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_933;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7357 '<FieldAccess .parent_scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11954
--# expr-start 7356 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11954
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 7356
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.parent_scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Parent_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 7357
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_933;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_933;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Parent_Scope;
--# end
--# property-start EndName.scope /usr/src/packages/BUILD/ada/ast.py:11955
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Scope
  (Node : Bare_End_Name
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_934;
      Fld : Bare_Name;
Fld_1 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_934 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_934;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7359 '<FieldAccess .scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11955
--# expr-start 7358 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11955
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 7358
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.scope'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Scope (Node => Fld, Env => Env);
--# end
--# expr-done 7359
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_934;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_934;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Scope;
--# end
--# property-start EndName.relative_name /usr/src/packages/BUILD/ada/ast.py:11956
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Relative_Name
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7361 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11956
--# expr-start 7360 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11956
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 7360
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7361
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Relative_Name;
--# end
--# property-start EndName.ref_var /usr/src/packages/BUILD/ada/ast.py:11957
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Ref_Var
  (Node : Bare_End_Name
  )
   return Logic_Var
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7363 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11957
--# expr-start 7362 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11957
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.End_Name_F_Name;
--# expr-done 7362
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7363
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Ref_Var;
--# end
--# property-start EndName.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:11958
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Env_Elements_Impl
  (Node : Bare_End_Name
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_937;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_937 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_937;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7365 '<FieldAccess .env_elements_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11958
--# expr-start 7364 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:11958
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 7364
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 7365
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_937;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_937;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Env_Elements_Impl;
--# end
--# property-start EndName.basic_decl /usr/src/packages/BUILD/ada/ast.py:11960
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Basic_Decl
  (Node : Bare_End_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_938;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Bare_Ada_Node;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Expr : Bare_Ada_Node;
Cast_Result : Bare_Named_Stmt;
Fld : Bare_Named_Stmt_Decl;
Cast_Result_1 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Scope_Result_1 : Internal_Entity_Basic_Decl;
Node_Parents_1 : Bare_Ada_Node_Array_Access;
Is_A_1 : Boolean;
Map_Result_1 : Bare_Ada_Node_Array_Access;
Get_Result_1 : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Result_Var_1 : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_938 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Node_Parents_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_938;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_End_Name_End_Name_P_Basic_Decl,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_End_Name,
                                As_Bare_End_Name => Self);
               Mmz_K.Items (2) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Basic_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
--# expr-start 7369 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11961
--# expr-start 7368 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11961
--# expr-start 7366 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:11961
Node_Parents := Parents (Self, True); 
--# expr-done 7366
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_225 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_225
               --# expr-start 7367 '<IsA NamedStmt>' Is_A /usr/src/packages/BUILD/ada/ast.py:11961
Is_A := Item_225 /= null 
and then Item_225.Kind in Ada_Named_Stmt_Range; 
--# expr-done 7367
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_225;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7368
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7369
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind ns Var_Expr_1
--# expr-start 7373 '<as_entity(<Cast to BasicDecl>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:11963
--# expr-start 7372 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11963
--# expr-start 7371 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:11963
--# expr-start 7370 '<Cast to NamedStmt>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11963
Cast_Expr := Var_Expr_1; 
   if Cast_Expr = null
      or else Cast_Expr.Kind in Ada_Named_Stmt_Range
   then
      Cast_Result := Cast_Expr;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7370
      if Cast_Result = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Cast_Result.Named_Stmt_F_Decl;
--# expr-done 7371
      Cast_Result_1 := Fld;
--# expr-done 7372
Is_Null := Cast_Result_1 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Cast_Result_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7373
Scope_Result := If_Result;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Basic_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind e Var_Expr
Scope_Result_1 := Var_Expr;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   --# expr-start 7379 '<as_entity(<Cast to BasicDecl>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11965
--# expr-start 7378 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:11965
--# expr-start 7377 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:11965
--# expr-start 7376 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:11965
--# expr-start 7374 '<FieldAccess .parents>' Node_Parents_1 /usr/src/packages/BUILD/ada/ast.py:11965
Node_Parents_1 := Parents (Self, True); 
--# expr-done 7374
declare
   Map_Result_1_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents_1;
      begin
         for Item_226 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_226
               --# expr-start 7375 '<IsA BasicDecl>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:11965
Is_A_1 := Item_226 /= null 
and then Item_226.Kind in Ada_Basic_Decl; 
--# expr-done 7375
               if Is_A_1 then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_226;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_1_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7376
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 7377
   if Get_Result_1 = null
      or else Get_Result_1.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Get_Result_1;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7378
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 7379
   Result_Var_1 := If_Result_1;
end if;
      Property_Result := Result_Var_1;
   --# end
      Finalizer_Scope_938;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Basic_Decl,
                     As_Internal_Entity_Basic_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_938;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Basic_Decl;
--# end
--# property-start EndName.xref_equation /usr/src/packages/BUILD/ada/ast.py:11972
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Equation
  (Node : Bare_End_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_End_Name :=
     Internal_Entity_End_Name'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_939;
      Fld : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity;
Is_A_1 : Boolean;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Accept_Stmt;
Fld_6 : Internal_Entity_Entry_Decl;
Cast_Result_2 : Internal_Entity;
Bind_Result_1 : Logic_Equation;
Fld_7 : Internal_Entity_Name;
Fld_8 : Logic_Equation;
If_Result : Logic_Equation;
If_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_939 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (Fld_8);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_939;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7394 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:11972
--# expr-start 7381 '<IsA SubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:11973
--# expr-start 7380 '<FieldAccess .basic_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:11973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EndName.basic_decl
Fld := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7380
Is_A := Fld.Node /= null 
and then Fld.Node.Kind in Ada_Subp_Body_Range; 
--# expr-done 7381
if Is_A then
   --# expr-start 7384 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:11974
--# expr-start 7382 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11974
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7382
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 7383 '<FieldAccess .basic_decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11974
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EndName.basic_decl
Fld_2 := Libadalang.Implementation.End_Name_P_Basic_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7383
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11974") else null)); 
--# expr-done 7384
   If_Result_1 := Bind_Result;
else
--# expr-start 7386 '<IsA AcceptStmtWithStmts>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:11976
--# expr-start 7385 '<parent(<Var Ent>)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11976
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7385
Is_A_1 := Fld_3.Node /= null 
and then Fld_3.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range; 
--# expr-done 7386
if Is_A_1 then
   --# expr-start 7391 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:11977
--# expr-start 7387 '<FieldAccess .ref_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11977
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7387
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
--# expr-start 7390 '<FieldAccess .designated_entry>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11977
--# expr-start 7389 '<Cast to AcceptStmt>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:11977
--# expr-start 7388 '<parent(<Var Ent>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11977
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7388
   if Fld_5.Node = null
      or else Fld_5.Node.Kind in Ada_Accept_Stmt_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Accept_Stmt
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
   else
         Cast_Result_1 := No_Entity_Accept_Stmt;
   end if;
--# expr-done 7389
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_6 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Cast_Result_1.Node, Origin => Origin, Env => Env, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7390
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result_1 := Bind_Default_Default.Create (Fld_4, Cast_Result_2, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11977") else null)); 
--# expr-done 7391
   If_Result := Bind_Result_1;
else
   --# expr-start 7393 '<FieldAccess .xref_no_overloading>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11979
--# expr-start 7392 '<FieldAccess .name>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11979
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Name (Node => Ent.Node.End_Name_F_Name, Info => Ent.Info);
--# expr-done 7392
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_8 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld_7.Node, Env => Env, Origin => Origin, E_Info => Fld_7.Info);
--# end
--# expr-done 7393
   If_Result := Fld_8;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7394
      Property_Result := If_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_939;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_939;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Xref_Equation;
--# end
--# property-start EndName.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:11982
pragma Warnings (Off, "is not referenced");
 function End_Name_P_Xref_Entry_Point
  (Node : Bare_End_Name
  )
   return Boolean
is
   Self : Bare_End_Name := Bare_End_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end End_Name_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Explicit_Deref
   --
      procedure Initialize_Fields_For_Explicit_Deref
        (Self : Bare_Explicit_Deref
         ; Explicit_Deref_F_Prefix : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Explicit_Deref_F_Prefix := Explicit_Deref_F_Prefix;
      Self.Explicit_Deref_R_Called_Spec := Null_Var_Record;
      end Initialize_Fields_For_Explicit_Deref;
   function Explicit_Deref_F_Prefix
     (Node : Bare_Explicit_Deref) return Bare_Name
   is
   begin
         return Node.Explicit_Deref_F_Prefix;
   end;
--# property-start ExplicitDeref.ref_var /usr/src/packages/BUILD/ada/ast.py:11433
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Ref_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7396 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11433
--# expr-start 7395 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:11433
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 7395
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7396
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Ref_Var;
--# end
--# property-start ExplicitDeref.subp_spec_var /usr/src/packages/BUILD/ada/ast.py:11437
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Logic_Var
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7397 '<FieldAccess .r_called_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:11437
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_R_Called_Spec'Unrestricted_Access;
--# expr-done 7397
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.defines_subp_spec_var /usr/src/packages/BUILD/ada/ast.py:11438
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Defines_Subp_Spec_Var
  (Node : Bare_Explicit_Deref
  )
   return Boolean
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Defines_Subp_Spec_Var;
--# end
--# property-start ExplicitDeref.relative_name /usr/src/packages/BUILD/ada/ast.py:11440
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Relative_Name
  (Node : Bare_Explicit_Deref
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7399 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11440
--# expr-start 7398 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:11440
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 7398
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7399
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Relative_Name;
--# end
--# property-start ExplicitDeref.designated_env /usr/src/packages/BUILD/ada/ast.py:11443
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Designated_Env
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_894;
      Fld : Internal_Entity_Name;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_894 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_894;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7402 '<Block>' Let_Result None
--# scope-start
--# expr-start 7401 '<FieldAccess .designated_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11446
--# expr-start 7400 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:11446
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 7400
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7401
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7402
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_894;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_894;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Designated_Env;
--# end
--# property-start ExplicitDeref.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:11449
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Env_Elements_Impl
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_895;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_895 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_895;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7411 '<Block>' Let_Result None
--# scope-start
--# expr-start 7410 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <filter>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:11450
--# expr-start 7403 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:11451
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7403
Bound_Origin := Fld; 
--# expr-start 7409 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11452
--# expr-start 7405 '<FieldAccess .env_elements_impl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11452
--# expr-start 7404 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11452
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 7404
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld_1.Node, Env => Env, E_Info => Fld_1.Info);
--# end
--# expr-done 7405
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_227 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_227
--# expr-start 7406 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11455
Cast_Expr := Item_227; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7406
Var_Expr_1 := Cast_Result;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7407 '<FieldAccess .expr_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11455
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Var_Expr_1.Node, Origin => Bound_Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 7407
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Var_Expr := Result_Var;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind t Var_Expr
--# expr-start 7408 '<FieldAccess .is_access_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11455
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Var_Expr.Node, Origin => Bound_Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7408
Scope_Result_1 := Fld_4;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
               if Result_Var_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_227;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7409
Dyn_Var_Bind_Result := Map_Result; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7410
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7411
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_895;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_895;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Env_Elements_Impl;
--# end
--# property-start ExplicitDeref.eq_for_type /usr/src/packages/BUILD/ada/ast.py:11460
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Eq_For_Type
  (Node : Bare_Explicit_Deref
      ; Typ : Internal_Entity_Base_Type_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind typ Typ
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_896;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Cast_Expr : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_False : Logic_Equation;
Fld_6 : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_896 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_896;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7421 '<Block>' Let_Result None
--# scope-start
--# expr-start 7420 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11461
--# expr-start 7412 '<FieldAccess .is_access_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11462
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 7412
if Fld_6 then
   --# expr-start 7419 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11463
--# expr-start 7415 '<FieldAccess .type_bind_val(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11464
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7414 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11464
--# expr-start 7413 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:11464
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Explicit_Deref_F_Prefix;
--# expr-done 7413
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld);
--# expr-done 7414
Cast_Expr := Typ; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start AdaNode.type_bind_val
Fld_2 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_1, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 7415
--# expr-start 7418 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11465
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7416 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11465
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7416
--# expr-start 7417 '<FieldAccess .accessed_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11465
      if Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Typ.Node, Origin => Origin, E_Info => Typ.Info);
--# end
--# expr-done 7417
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_4.Node,
         Info => Fld_4.Info);
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7418
And_Pred := Logic_And (Fld_2, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11463") else null)); 
--# expr-done 7419
   If_Result := And_Pred;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11467") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 7420
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7421
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_896;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_896;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Eq_For_Type;
--# end
--# property-start ExplicitDeref.xref_equation /usr/src/packages/BUILD/ada/ast.py:11471
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_897;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_897 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_897;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7423 '<Block>' Let_Result None
--# scope-start
--# expr-start 7422 '<FieldAccess .bottom_up_name_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:11472
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.bottom_up_name_equation
Fld := Libadalang.Implementation.Name_P_Bottom_Up_Name_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7422
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7423
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_897;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_897;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_Xref_Equation;
--# end
--# property-start ExplicitDeref.general_xref_equation /usr/src/packages/BUILD/ada/ast.py:11475
pragma Warnings (Off, "is not referenced");
 function Explicit_Deref_P_General_Xref_Equation
  (Node : Bare_Explicit_Deref
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Explicit_Deref := Bare_Explicit_Deref (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Explicit_Deref :=
     Internal_Entity_Explicit_Deref'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_898;
            procedure Finalizer_Scope_2541;
            procedure Finalizer_Scope_2542;
      Block_Var : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Array_Access;
Fld_1 : Internal_Entity_Name;
Fld_2 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Var;
Fld_5 : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_6 : Logic_Var;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_9 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_False : Logic_Equation;
Result_Var : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_898 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_898;
            procedure Finalizer_Scope_2541 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_False);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2541;
            procedure Finalizer_Scope_2542 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_9);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2542;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7442 '<Block>' Let_Result None
--# scope-start
--# expr-start 7424 '<FieldAccess .env_elements>' Fld /usr/src/packages/BUILD/ada/ast.py:11476
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7424
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind env_els Block_Var
--# expr-start 7441 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:11478
--# expr-start 7426 '<FieldAccess .sub_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:11478
--# expr-start 7425 '<FieldAccess .prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11478
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Explicit_Deref_F_Prefix, Info => Ent.Info);
--# expr-done 7425
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_1.Node, Env => Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 7426
--# expr-start 7440 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:11478
--# expr-start 7439 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11478
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_228 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_228
--# expr-start 7428 '<FieldAccess .expr_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:11479
--# expr-start 7427 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:11479
Cast_Expr := Item_228; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7427
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Cast_Result.Node, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 7428
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 7438 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:11485
--# expr-start 7436 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:11485
--# expr-start 7431 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:11485
--# expr-start 7429 '<FieldAccess .ref_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:11485
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_4 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7429
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
--# expr-start 7430 '<FieldAccess .trigger_access_entity(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:11485
      if Item_228.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.trigger_access_entity
Fld_5 := Libadalang.Implementation.Ada_Node_P_Trigger_Access_Entity (Node => Item_228.Node, Val => False, E_Info => Item_228.Info);
--# end
--# expr-done 7430
Bind_Result := Bind_Default_Default.Create (Fld_4, Fld_5, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11485") else null)); 
--# expr-done 7431
--# expr-start 7435 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:11487
--# expr-start 7432 '<FieldAccess .ref_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:11487
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7432
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
--# expr-start 7434 '<FieldAccess .ref_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:11487
--# expr-start 7433 '<FieldAccess .prefix>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:11487
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Self.Explicit_Deref_F_Prefix;
--# expr-done 7433
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 7434
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result_1 := Bind_Default_Default.Create (Fld_6, Fld_8, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11487") else null)); 
--# expr-done 7435
And_Pred := Logic_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11485") else null)); 
--# expr-done 7436
--# expr-start 7437 '<FieldAccess .parent_name_equation(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:11488
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name_equation
Fld_9 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Ent.Node, Typ => Var_Expr, Root => Root, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7437
And_Pred_1 := Logic_And (And_Pred, Fld_9, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11485") else null)); 
--# expr-done 7438
Scope_Result := And_Pred_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2542;
   Result_Var := Scope_Result;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11489") else null)); 
   Result_Var := Logic_False;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Logic_Equation :=
               Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2541;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7439
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11478") else null)); 
--# expr-done 7440
And_Pred_2 := Logic_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11478") else null)); 
--# expr-done 7441
Let_Result := And_Pred_2; Inc_Ref (Let_Result);
--# end
--# expr-done 7442
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_898;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_898;
               Finalizer_Scope_2541;
               Finalizer_Scope_2542;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Explicit_Deref_P_General_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Qual_Expr
   --
      procedure Initialize_Fields_For_Qual_Expr
        (Self : Bare_Qual_Expr
         ; Qual_Expr_F_Prefix : Bare_Name
         ; Qual_Expr_F_Suffix : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
            Self.Qual_Expr_F_Prefix := Qual_Expr_F_Prefix;
            Self.Qual_Expr_F_Suffix := Qual_Expr_F_Suffix;
      end Initialize_Fields_For_Qual_Expr;
   function Qual_Expr_F_Prefix
     (Node : Bare_Qual_Expr) return Bare_Name
   is
   begin
         return Node.Qual_Expr_F_Prefix;
   end;
   function Qual_Expr_F_Suffix
     (Node : Bare_Qual_Expr) return Bare_Expr
   is
   begin
         return Node.Qual_Expr_F_Suffix;
   end;
--# property-start QualExpr.ref_var /usr/src/packages/BUILD/ada/ast.py:13282
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Ref_Var
  (Node : Bare_Qual_Expr
  )
   return Logic_Var
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7444 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13282
--# expr-start 7443 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13282
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Qual_Expr_F_Prefix;
--# expr-done 7443
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 7444
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Ref_Var;
--# end
--# property-start QualExpr.relative_name /usr/src/packages/BUILD/ada/ast.py:13284
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Relative_Name
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7446 '<FieldAccess .relative_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13284
--# expr-start 7445 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13284
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7445
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7446
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Relative_Name;
--# end
--# property-start QualExpr.general_xref_equation /usr/src/packages/BUILD/ada/ast.py:13287
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_General_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Root : Bare_Name
         := No_Bare_Ada_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind root Root
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1019;
            procedure Finalizer_Scope_2543;
      Var_Expr : Internal_Entity_Name;
Fld : Logic_Equation;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1019 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1019;
            procedure Finalizer_Scope_2543 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2543;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7454 '<Block>' Let_Result None
--# scope-start
--# expr-start 7453 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13288
--# expr-start 7447 '<FieldAccess .xref_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:13289
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7447
--# expr-start 7449 '<as_entity(<FieldAccess .parent_name(...)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13290
--# expr-start 7448 '<FieldAccess .parent_name(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13290
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_name
Fld_1 := Libadalang.Implementation.Name_P_Parent_Name (Node => Self, Stop_At => Root);
--# end
--# expr-done 7448
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7449
Var_Expr := If_Result;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind pn Var_Expr
--# expr-start 7452 '<FieldAccess .parent_name_equation(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13292
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7451 '<FieldAccess .designated_type_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13293
--# expr-start 7450 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13293
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7450
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7451
--# property-call-start Name.parent_name_equation
Fld_4 := Libadalang.Implementation.Name_P_Parent_Name_Equation (Node => Var_Expr.Node, Typ => Fld_3, Root => Root, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7452
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2543;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13296") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13288") else null)); 
--# expr-done 7453
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7454
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1019;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1019;
               Finalizer_Scope_2543;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_General_Xref_Equation;
--# end
--# property-start QualExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:13301
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Xref_Equation
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1020;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Fld_8 : Internal_Entity_Name;
Fld_9 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Bare_Expr;
Fld_12 : Logic_Var;
Fld_13 : Internal_Entity_Name;
Fld_14 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_15 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_16 : Logic_Var;
Fld_17 : Internal_Entity_Name;
Fld_18 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Fld_19 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1020 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Fld_15);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Fld_19);
                     Dec_Ref (And_Pred_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1020;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7480 '<Block>' Let_Result None
--# scope-start
--# expr-start 7479 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13305
--# expr-start 7474 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13305
--# expr-start 7468 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13305
--# expr-start 7462 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13305
--# expr-start 7456 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13305
--# expr-start 7455 '<FieldAccess .suffix>' Fld /usr/src/packages/BUILD/ada/ast.py:13305
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Qual_Expr_F_Suffix, Info => Ent.Info);
--# expr-done 7455
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7456
--# expr-start 7461 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:13306
--# expr-start 7458 '<FieldAccess .ref_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13306
--# expr-start 7457 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13306
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Qual_Expr_F_Prefix;
--# expr-done 7457
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_2);
--# end
--# expr-done 7458
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 7460 '<FieldAccess .designated_type_impl>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13302
--# expr-start 7459 '<FieldAccess .prefix>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13302
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7459
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_4.Node, Env => Env, Origin => Origin, E_Info => Fld_4.Info);
--# end
--# expr-done 7460
      Cast_Result := Create_Internal_Entity
        (Node => Fld_5.Node,
         Info => Fld_5.Info);
Bind_Result := Bind_Default_Default.Create (Fld_3, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13306") else null)); 
--# expr-done 7461
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13305") else null)); 
--# expr-done 7462
--# expr-start 7467 '<FieldAccess .type_bind_val(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13307
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7464 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13307
--# expr-start 7463 '<FieldAccess .prefix>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13307
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Qual_Expr_F_Prefix;
--# expr-done 7463
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 7464
--# expr-start 7466 '<FieldAccess .designated_type_impl>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13302
--# expr-start 7465 '<FieldAccess .prefix>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13302
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7465
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_9 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_8.Node, Env => Env, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 7466
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
--# property-call-start AdaNode.type_bind_val
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_7, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 7467
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13305") else null)); 
--# expr-done 7468
--# expr-start 7473 '<FieldAccess .type_bind_val(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7470 '<FieldAccess .type_var>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13308
--# expr-start 7469 '<FieldAccess .suffix>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13308
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Self.Qual_Expr_F_Suffix;
--# expr-done 7469
      if Fld_11 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_11);
--# expr-done 7470
--# expr-start 7472 '<FieldAccess .designated_type_impl>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13302
--# expr-start 7471 '<FieldAccess .prefix>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13302
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7471
      if Fld_13.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_14 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_13.Node, Env => Env, Origin => Origin, E_Info => Fld_13.Info);
--# end
--# expr-done 7472
      Cast_Result_2 := Create_Internal_Entity
        (Node => Fld_14.Node,
         Info => Fld_14.Info);
--# property-call-start AdaNode.type_bind_val
Fld_15 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_12, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 7473
And_Pred_2 := Logic_And (And_Pred_1, Fld_15, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13305") else null)); 
--# expr-done 7474
--# expr-start 7478 '<FieldAccess .type_bind_val(...)>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13309
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7475 '<FieldAccess .type_var>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13309
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7475
--# expr-start 7477 '<FieldAccess .designated_type_impl>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13302
--# expr-start 7476 '<FieldAccess .prefix>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13302
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7476
      if Fld_17.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_18 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_17.Node, Env => Env, Origin => Origin, E_Info => Fld_17.Info);
--# end
--# expr-done 7477
      Cast_Result_3 := Create_Internal_Entity
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
--# property-call-start AdaNode.type_bind_val
Fld_19 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_16, Right => Cast_Result_3, Origin => Origin);
--# end
--# expr-done 7478
And_Pred_3 := Logic_And (And_Pred_2, Fld_19, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13305") else null)); 
--# expr-done 7479
Let_Result := And_Pred_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7480
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1020;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1020;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Xref_Equation;
--# end
--# property-start QualExpr.designated_type /usr/src/packages/BUILD/ada/ast.py:13315
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Type
  (Node : Bare_Qual_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1021;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result_1 : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1021 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_1021;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7486 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:13316
--# expr-start 7481 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:13316
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 7481
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 7485 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:13317
--# expr-start 7482 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13317
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7482
Bound_Origin := Fld_1; 
--# expr-start 7484 '<FieldAccess .designated_type_impl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13318
--# expr-start 7483 '<FieldAccess .prefix>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13318
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7483
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_2.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7484
Dyn_Var_Bind_Result := Fld_3; 
--# expr-done 7485
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; 
--# expr-done 7486
      Property_Result := Dyn_Var_Bind_Result_1;
   --# end
      Finalizer_Scope_1021;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1021;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Designated_Type;
--# end
--# property-start QualExpr.designated_env /usr/src/packages/BUILD/ada/ast.py:13322
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Designated_Env
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1022;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1022 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1022;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7489 '<Block>' Let_Result None
--# scope-start
--# expr-start 7488 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13323
--# expr-start 7487 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:13323
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start QualExpr.designated_type
Fld := Libadalang.Implementation.Qual_Expr_P_Designated_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7487
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7488
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7489
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1022;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1022;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Designated_Env;
--# end
--# property-start QualExpr.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:13326
pragma Warnings (Off, "is not referenced");
 function Qual_Expr_P_Env_Elements_Impl
  (Node : Bare_Qual_Expr
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Qual_Expr := Bare_Qual_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Qual_Expr :=
     Internal_Entity_Qual_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1023;
      Fld : Internal_Entity_Name;
Fld_1 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_1023 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1023;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7492 '<Block>' Let_Result None
--# scope-start
--# expr-start 7491 '<FieldAccess .env_elements_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13327
--# expr-start 7490 '<FieldAccess .prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:13327
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Qual_Expr_F_Prefix, Info => Ent.Info);
--# expr-done 7490
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.env_elements_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Expr_P_Env_Elements_Impl (Node => Fld.Node, Env => Env, E_Info => Fld.Info);
--# end
--# expr-done 7491
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7492
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1023;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1023;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Qual_Expr_P_Env_Elements_Impl;
--# end
   --
   --  Primitives for Bare_Single_Tok_Node
   --
      procedure Initialize_Fields_For_Single_Tok_Node
        (Self : Bare_Single_Tok_Node
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      end Initialize_Fields_For_Single_Tok_Node;
--# property-start SingleTokNode.relative_name /usr/src/packages/BUILD/ada/ast.py:11643
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Relative_Name
  (Node : Bare_Single_Tok_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Single_Tok_Node :=
     Internal_Entity_Single_Tok_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Ent;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Relative_Name;
--# end
--# property-start SingleTokNode.defines_subp_spec_var /usr/src/packages/BUILD/ada/ast.py:11655
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Defines_Subp_Spec_Var
  (Node : Bare_Single_Tok_Node
  )
   return Boolean
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Defines_Subp_Spec_Var;
--# end
--# property-start SingleTokNode.sym /usr/src/packages/BUILD/ada/ast.py:11657
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Sym
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7493 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:11658
Sym := Get_Symbol (Self); 
--# expr-done 7493
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Sym;
--# end
--# property-start SingleTokNode.canonical_text /usr/src/packages/BUILD/ada/ast.py:11667
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Canonical_Text
  (Node : Bare_Single_Tok_Node
  )
   return Symbol_Type
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7495 '<Block>' Let_Result None
--# scope-start
--# expr-start 7494 '<FieldAccess .sym>' Fld /usr/src/packages/BUILD/ada/ast.py:11671
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SingleTokNode.sym
Fld := Libadalang.Implementation.Single_Tok_Node_P_Sym (Node => Self);
--# end
--# expr-done 7494
Let_Result := Fld; 
--# end
--# expr-done 7495
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Canonical_Text;
--# end
--# property-start SingleTokNode.env_get_first_visible /usr/src/packages/BUILD/ada/ast.py:11674
pragma Warnings (Off, "is not referenced");
 function Single_Tok_Node_P_Env_Get_First_Visible
  (Node : Bare_Single_Tok_Node
      ; Lex_Env : Lexical_Env
      ; Lookup_Type : Lookup_Kind
      ; From_Node : Bare_Ada_Node
  )
   return Internal_Entity
is
   Self : Bare_Single_Tok_Node := Bare_Single_Tok_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind lex_env Lex_Env
   --# bind lookup_type Lookup_Type
   --# bind from_node From_Node
   Property_Result : Internal_Entity;
            procedure Finalizer_Scope_912;
      Sym : Symbol_Type;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Let_Result : Internal_Entity;
            procedure Finalizer_Scope_912 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_912;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7501 '<Block>' Let_Result None
--# scope-start
--# expr-start 7500 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:11681
--# expr-start 7499 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:11681
--# expr-start 7497 '<FieldAccess .env_get(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:11681
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7496 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:11683
Sym := Get_Symbol (Self); 
--# expr-done 7496
--# property-call-start AdaNode.env_get
Fld := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Lex_Env, Symbol => Sym, Lookup => Lookup_Type, From_Node => From_Node, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 7497
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_229 of
               Collection.Items
         loop
   --# scope-start
               --# bind el Item_229
               --# expr-start 7498 '<FieldAccess .has_visibility(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:11687
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_1 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_229);
--# end
--# expr-done 7498
               if Fld_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_229;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7499
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7500
Let_Result := Get_Result; 
--# end
--# expr-done 7501
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_912;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_912;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Single_Tok_Node_P_Env_Get_First_Visible;
--# end
   --
   --  Primitives for Bare_Base_Id
   --
      procedure Initialize_Fields_For_Base_Id
        (Self : Bare_Base_Id
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Base_Id;
--# property-start BaseId.scope /usr/src/packages/BUILD/ada/ast.py:11994
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_941;
            procedure Finalizer_Scope_2545;
      Block_Var : Internal_Entity;
Block_Var_1 : Lexical_Env := Empty_Env;
Var_Expr : Bare_Generic_Package_Decl;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Env_Get_Result : Internal_Entity;
Fld_1 : Lexical_Env := Empty_Env;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_2 : Bare_Ada_Node;
Is_A : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Env_Node : Bare_Ada_Node;
Cast_Result : Bare_Generic_Package_Decl;
Fld_3 : Bare_Generic_Package_Internal;
Fld_4 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_941 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_941;
            procedure Finalizer_Scope_2545 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2545;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Scope,
            Items    => new Mmz_Key_Array (1 ..  2))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Lexical_Env;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7515 '<Block>' Let_Result None
--# scope-start
--# expr-start 7504 '<EnvGet(<DynamicVariable env>, <Var Self>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:11995
Sym := Get_Symbol (Self); 
--# expr-start 7503 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:11997
--# expr-start 7502 '<FieldAccess .is_prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:11997
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7502
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7503
Env_Get_Result := AST_Envs.Get_First (Self => Env, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (If_Result), Categories => (Inherited_Primitives => False, Nocat => True)); 
--# expr-done 7504
Block_Var := Env_Get_Result; 
--# bind elt Block_Var
--# expr-start 7510 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:12000
--# expr-start 7506 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12001
--# expr-start 7505 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12001
Is_Null := Block_Var.Node = null; 
--# expr-done 7505
Not_Val := not (Is_Null); 
--# expr-done 7506
if Not_Val then
   --# expr-start 7508 '<IsA BasicDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:12001
--# expr-start 7507 '<FieldAccess .node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12001
Fld_2 := Block_Var.Node;
--# expr-done 7507
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Basic_Decl; 
--# expr-done 7508
   If_Result_1 := Is_A;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 7509 '<FieldAccess .children_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12004
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Children_Env (Node => Block_Var.Node, E_Info => Block_Var.Info);
--# expr-done 7509
   If_Result_2 := Fld_1;
else
   If_Result_2 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7510
Block_Var_1 := If_Result_2; Inc_Ref (Block_Var_1);
--# bind ret Block_Var_1
--# expr-start 7512 '<Cast to GenericPackageDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12010
--# expr-start 7511 '<env_node(<Var ret>)>' Env_Node /usr/src/packages/BUILD/ada/ast.py:12010
Env_Node := AST_Envs.Env_Node (Block_Var_1); 
--# expr-done 7511
   if Env_Node = null
      or else Env_Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result := Env_Node;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 7512
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind gen_pkg_decl Var_Expr
--# expr-start 7514 '<FieldAccess .children_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12011
--# expr-start 7513 '<FieldAccess .package_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12011
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Var_Expr.Generic_Package_Decl_F_Package_Decl;
--# expr-done 7513
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Children_Env (Node => Fld_3);
--# expr-done 7514
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2545;
   Result_Var := Scope_Result;
else
   Result_Var := Block_Var_1;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 7515
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_941;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Lexical_Env,
                     As_Lexical_Env => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_941;
               Finalizer_Scope_2545;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Scope;
--# end
--# property-start BaseId.designated_env_no_overloading /usr/src/packages/BUILD/ada/ast.py:12016
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env_No_Overloading
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_942;
            procedure Finalizer_Scope_2546;
      Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Basic_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_942 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_942;
            procedure Finalizer_Scope_2546 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (If_Result_1);
            end Finalizer_Scope_2546;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7525 '<Block>' Let_Result None
--# scope-start
--# expr-start 7520 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12017
--# expr-start 7519 '<FieldAccess .env_get_first_visible(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12017
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7517 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12019
--# expr-start 7516 '<FieldAccess .is_prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:12019
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7516
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7517
--# expr-start 7518 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12020
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7518
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1);
--# end
--# expr-done 7519
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7520
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind bd Var_Expr
--# expr-start 7524 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12022
Var_Expr_1 := Var_Expr;
if Var_Expr_1 /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7521 '<FieldAccess .is_package>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12023
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.is_package
Fld_5 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr_1.Node);
--# end
--# expr-done 7521
Scope_Result := Fld_5;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
if Result_Var then
   --# expr-start 7522 '<FieldAccess .pkg_env(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12023
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Var_Expr, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7522
   If_Result_1 := Fld_3;
else
   --# expr-start 7523 '<FieldAccess .defining_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12023
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7523
   If_Result_1 := Fld_4;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7524
Scope_Result_1 := If_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2546;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7525
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_942;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_942;
               Finalizer_Scope_2546;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Env_No_Overloading;
--# end
--# property-start BaseId.designated_env /usr/src/packages/BUILD/ada/ast.py:12028
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Env
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_943;
            procedure Finalizer_Scope_2556;
            procedure Finalizer_Scope_2558;
      Block_Var : Bare_Ada_Node;
Block_Var_1 : Internal_Entity;
Var_Expr : Internal_Entity_Basic_Decl;
Var_Expr_1 : Internal_Entity_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Internal_Entity_Array_Access;
Fld_5 : Bare_Ada_Node;
Cast_Result_1 : Bare_Ada_Node;
Is_Equal : Boolean;
Not_Val : Boolean;
Fld_6 : Boolean;
If_Result_1 : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Basic_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Map_Result_2 : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Basic_Decl;
Fld_8 : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Cast_Expr_3 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Basic_Decl;
Fld_9 : Bare_Basic_Decl;
Cast_Result_5 : Bare_Ada_Node;
Is_Equal_1 : Boolean;
Not_Val_1 : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_943 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Group_Env);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_943;
            procedure Finalizer_Scope_2556 is
            begin
                     Dec_Ref (Var_Expr_1);
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2556;
            procedure Finalizer_Scope_2558 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2558;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7553 '<Block>' Let_Result None
--# scope-start
--# expr-start 7529 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12033
--# expr-start 7528 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12033
--# expr-start 7526 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:12033
Node_Parents := Parents (Self, True); 
--# expr-done 7526
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_230 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_230
               --# expr-start 7527 '<IsA GenericPackageInstantiation>' Is_A /usr/src/packages/BUILD/ada/ast.py:12034
Is_A := Item_230 /= null 
and then Item_230.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7527
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_230;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7528
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7529
Block_Var := Get_Result; 
--# bind bd Block_Var
--# expr-start 7533 '<FieldAccess .env_get_first_visible(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12037
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7531 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12039
--# expr-start 7530 '<FieldAccess .is_prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:12039
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7530
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7531
--# expr-start 7532 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12040
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7532
--# property-call-start SingleTokNode.env_get_first_visible
Fld_2 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Fld_1);
--# end
--# expr-done 7533
Block_Var_1 := Fld_2; 
--# expr-start 7552 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:12043
--# expr-start 7534 '<Cast to BasicDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:12037
Cast_Expr_2 := Block_Var_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_3 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7534
Var_Expr := Cast_Result_3;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
--# expr-start 7535 '<FieldAccess .is_package>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12045
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.is_package
Fld_8 := Libadalang.Implementation.Ada_Node_P_Is_Package (Node => Var_Expr.Node);
--# end
--# expr-done 7535
Scope_Result_1 := Fld_8;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   --# expr-start 7539 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:12045
--# expr-start 7538 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:12045
--# expr-start 7537 '<FieldAccess .node>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12045
--# expr-start 7536 '<Cast to BasicDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:12037
Cast_Expr_3 := Block_Var_1; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_4 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7536
Fld_9 := Cast_Result_4.Node;
--# expr-done 7537
      Cast_Result_5 := Fld_9;
Is_Equal_1 := Cast_Result_5 = Block_Var; 
--# expr-done 7538
Not_Val_1 := not (Is_Equal_1); 
--# expr-done 7539
   If_Result_2 := Not_Val_1;
else
   If_Result_2 := False;
end if;
if If_Result_2 then
   --# expr-start 7541 '<FieldAccess .pkg_env(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12046
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7540 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12037
Cast_Expr := Block_Var_1; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 7540
--# property-call-start BaseId.pkg_env
Fld_3 := Libadalang.Implementation.Base_Id_P_Pkg_Env (Node => Ent.Node, From_Pkg => Cast_Result, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7541
   If_Result_3 := Fld_3;
else
   --# expr-start 7551 '<env_group(<map>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:12048
--# expr-start 7550 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:12048
--# expr-start 7542 '<FieldAccess .env_elements_baseid>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12048
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld_4 := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7542
Var_Expr_1 := Fld_4;
      Inc_Ref (Var_Expr_1);
if Var_Expr_1 /= No_Internal_Entity_Array_Type then
   --# scope-start
   --# bind all_env_els Var_Expr_1
--# expr-start 7547 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:12050
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Var_Expr_1;
      begin
         for Item_232 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_232
--# expr-start 7545 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12052
--# expr-start 7544 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:12052
--# expr-start 7543 '<FieldAccess .node>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12052
Fld_5 := Item_232.Node;
--# expr-done 7543
      Cast_Result_1 := Fld_5;
Is_Equal := Cast_Result_1 = Block_Var; 
--# expr-done 7544
Not_Val := not (Is_Equal); 
--# expr-done 7545
if Not_Val then
   --# expr-start 7546 '<FieldAccess .has_visibility(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12054
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.has_visibility
Fld_6 := Libadalang.Implementation.Ada_Node_P_Has_Visibility (Node => Self, Other_Entity => Item_232);
--# end
--# expr-done 7546
   If_Result_1 := Fld_6;
else
   If_Result_1 := False;
end if;
               if If_Result_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_232;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7547
Scope_Result := Map_Result_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2556;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_2_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_231 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_231
      --# expr-start 7549 '<FieldAccess .defining_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12056
--# expr-start 7548 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:12056
Cast_Expr_1 := Item_231; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Basic_Decl;
   end if;
--# expr-done 7548
      if Cast_Result_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Cast_Result_2.Node, Origin => Origin, E_Info => Cast_Result_2.Info);
--# end
--# expr-done 7549
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_7;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2558;
         end loop;
      end;
      Map_Result_2 := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_2_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7550
Group_Env := Group (Map_Result_2, No_Metadata); 
--# expr-done 7551
   If_Result_3 := Group_Env;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 7552
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 7553
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_943;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_943;
               Finalizer_Scope_2556;
               Finalizer_Scope_2558;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Env;
--# end
--# property-start BaseId.pkg_env /usr/src/packages/BUILD/ada/ast.py:12060
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Pkg_Env
  (Node : Bare_Base_Id
      ; From_Pkg : Internal_Entity_Basic_Decl
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind from_pkg From_Pkg
   --# bind env Env
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_944;
            procedure Finalizer_Scope_2548;
            procedure Finalizer_Scope_2549;
            procedure Finalizer_Scope_2550;
            procedure Finalizer_Scope_2551;
      Block_Var : Internal_Entity_Basic_Decl;
Block_Var_1 : Internal_Entity_Basic_Decl;
Block_Var_2 : Boolean;
Block_Var_3 : Lexical_Env := Empty_Env;
Block_Var_4 : Lexical_Env := Empty_Env;
Block_Var_5 : Lexical_Env := Empty_Env;
Block_Var_6 : Lexical_Env := Empty_Env;
Var_Expr : Internal_Entity_Package_Renaming_Decl;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity;
Var_Expr_3 : Internal_Entity;
Var_Expr_4 : Internal_Entity_Generic_Package_Decl;
Var_Expr_5 : Bare_Ada_Node;
Var_Expr_6 : Bare_Ada_Node;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity_Package_Renaming_Decl;
Fld : Internal_Entity_Basic_Decl;
Scope_Result : Internal_Entity_Basic_Decl;
Result_Var : Internal_Entity_Basic_Decl;
Fld_1 : Internal_Entity_Info;
Fld_2 : Env_Rebindings;
Fld_3 : Internal_Entity_Basic_Decl;
Is_A : Boolean;
Fld_4 : Internal_Entity_Info;
Fld_5 : Boolean;
If_Result : Boolean;
Cast_Expr_1 : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity_Generic_Package_Instantiation;
Fld_6 : Lexical_Env := Empty_Env;
Fld_7 : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
If_Result_1 : Boolean;
If_Result_2 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_8 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Sym_1 : Symbol_Type;
Env_Get_Result_1 : Internal_Entity_Array_Access;
Get_Result_1 : Internal_Entity;
Fld_9 : Lexical_Env := Empty_Env;
Sym_2 : Symbol_Type;
Env_Get_Result_2 : Internal_Entity_Array_Access;
Get_Result_2 : Internal_Entity;
Fld_10 : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Result_Var_2 : Lexical_Env := Empty_Env;
Fld_11 : Lexical_Env := Empty_Env;
Is_A_2 : Boolean;
If_Result_3 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Result_Var_3 : Lexical_Env := Empty_Env;
Cast_Expr_2 : Internal_Entity_Basic_Decl;
Cast_Result_2 : Internal_Entity_Generic_Package_Decl;
Fld_12 : Internal_Entity_Generic_Formal_Part;
Fld_13 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Result_Var_4 : Lexical_Env := Empty_Env;
Is_Equal : Boolean;
Not_Val : Boolean;
Scope_Result_5 : Bare_Ada_Node;
Cast_Expr_3 : Bare_Base_Id;
Cast_Result_3 : Bare_Ada_Node;
Result_Var_5 : Bare_Ada_Node;
Fld_14 : Lexical_Env := Empty_Env;
Fld_15 : Boolean;
If_Result_4 : Boolean;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Scope_Result_6 : Bare_Ada_Node;
Cast_Expr_4 : Bare_Base_Id;
Cast_Result_4 : Bare_Ada_Node;
Result_Var_6 : Bare_Ada_Node;
Fld_16 : Lexical_Env := Empty_Env;
Fld_17 : Boolean;
Array_Lit_1 : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
If_Result_5 : Lexical_Env := Empty_Env;
If_Result_6 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_944 is
            begin
                     Dec_Ref (Fld_6);
                     Dec_Ref (Fld_7);
                     Dec_Ref (If_Result_2);
                     Dec_Ref (Block_Var_3);
                     Dec_Ref (Env_Get_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Block_Var_4);
                     Dec_Ref (Env_Get_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Block_Var_5);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (Block_Var_6);
                     Dec_Ref (Fld_14);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_16);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (If_Result_5);
                     Dec_Ref (If_Result_6);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_944;
            procedure Finalizer_Scope_2548 is
            begin
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2548;
            procedure Finalizer_Scope_2549 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Env_Get_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (Fld_11);
                     Dec_Ref (If_Result_3);
            end Finalizer_Scope_2549;
            procedure Finalizer_Scope_2550 is
            begin
                     Dec_Ref (Fld_10);
            end Finalizer_Scope_2550;
            procedure Finalizer_Scope_2551 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2551;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7596 '<Block>' Let_Result None
--# scope-start
--# expr-start 7554 '<Cast to PackageRenamingDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12075
Cast_Expr := From_Pkg; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Package_Renaming_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Package_Renaming_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Package_Renaming_Decl;
   end if;
--# expr-done 7554
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Package_Renaming_Decl then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 7555 '<FieldAccess .final_renamed_package>' Fld /usr/src/packages/BUILD/ada/ast.py:12076
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start PackageRenamingDecl.final_renamed_package
Fld := Libadalang.Implementation.Package_Renaming_Decl_P_Final_Renamed_Package (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 7555
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := From_Pkg;
end if;
Block_Var := Result_Var; 
--# bind pkg Block_Var
--# expr-start 7558 '<FieldAccess .unshed_rebindings(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12087
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7557 '<FieldAccess .rebindings>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12087
--# expr-start 7556 '<FieldAccess .info>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12087
Fld_1 := Ent.Info;
--# expr-done 7556
Fld_2 := Fld_1.Rebindings;
--# expr-done 7557
--# property-call-start BasicDecl.unshed_rebindings
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Unshed_Rebindings (Node => Block_Var.Node, Rebindings => Fld_2, E_Info => Block_Var.Info);
--# end
--# expr-done 7558
Block_Var_1 := Fld_3; 
--# bind bd Block_Var_1
--# expr-start 7559 '<IsA GenericPackageInstantiation>' Is_A /usr/src/packages/BUILD/ada/ast.py:12089
Is_A := Block_Var.Node /= null 
and then Block_Var.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7559
if Is_A then
   --# expr-start 7561 '<FieldAccess .from_rebound>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12090
--# expr-start 7560 '<FieldAccess .info>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12090
Fld_4 := From_Pkg.Info;
--# expr-done 7560
Fld_5 := Fld_4.From_Rebound;
--# expr-done 7561
   If_Result := Fld_5;
else
   If_Result := False;
end if;
Block_Var_2 := If_Result; 
--# bind is_inst_from_formal Block_Var_2
--# expr-start 7566 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:12092
--# expr-start 7562 '<IsA GenericPackageInstantiation>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:12093
Is_A_1 := Block_Var_1.Node /= null 
and then Block_Var_1.Node.Kind in Ada_Generic_Package_Instantiation_Range; 
--# expr-done 7562
if Is_A_1 then
   If_Result_1 := Block_Var_2;
else
   If_Result_1 := False;
end if;
if If_Result_1 then
   --# expr-start 7564 '<FieldAccess .defining_env_impl(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12094
--# expr-start 7563 '<Cast to GenericPackageInstantiation>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:12094
Cast_Expr_1 := Block_Var_1; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Generic_Package_Instantiation_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Generic_Package_Instantiation
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Generic_Package_Instantiation;
   end if;
--# expr-done 7563
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start GenericPackageInstantiation.defining_env_impl
Fld_6 := Libadalang.Implementation.Generic_Package_Instantiation_P_Defining_Env_Impl (Node => Cast_Result_1.Node, Inst_From_Formal => True, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 7564
   If_Result_2 := Fld_6;
else
   --# expr-start 7565 '<FieldAccess .defining_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12095
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Block_Var_1.Node, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7565
   If_Result_2 := Fld_7;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 7566
Block_Var_3 := If_Result_2; Inc_Ref (Block_Var_3);
--# bind env Block_Var_3
--# expr-start 7569 '<at(<EnvGet(<Var env>, __privatepart)>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12101
--# expr-start 7568 '<EnvGet(<Var env>, __privatepart)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:12101
--# expr-start 7567 '<Symbol __privatepart>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Privatepart); 
--# expr-done 7567
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_3, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7568
Get_Result := Get (Env_Get_Result, 0, True); 
--# expr-done 7569
Var_Expr_1 := Get_Result;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind pp Var_Expr_1
--# expr-start 7570 '<FieldAccess .children_env>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12102
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Children_Env (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# expr-done 7570
Scope_Result_1 := Fld_8;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2548;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var_3;
end if;
      Inc_Ref (Result_Var_1);
Block_Var_4 := Result_Var_1; Inc_Ref (Block_Var_4);
--# bind private_part_env Block_Var_4
--# expr-start 7573 '<at(<EnvGet(<Var private_part_env>, __nextpart)>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:12107
--# expr-start 7572 '<EnvGet(<Var private_part_env>, __nextpart)>' Env_Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:12107
--# expr-start 7571 '<Symbol __nextpart>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 7571
Env_Get_Result_1 := Construct_Entity_Array (AST_Envs.Get (Self => Block_Var_4, Key => Sym_1, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7572
Get_Result_1 := Get (Env_Get_Result_1, 0, True); 
--# expr-done 7573
Var_Expr_2 := Get_Result_1;
if Var_Expr_2 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_2
--# expr-start 7581 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:12109
--# expr-start 7574 '<IsA PackageBodyStub>' Is_A_2 /usr/src/packages/BUILD/ada/ast.py:12112
Is_A_2 := Var_Expr_2.Node /= null 
and then Var_Expr_2.Node.Kind in Ada_Package_Body_Stub_Range; 
--# expr-done 7574
if Is_A_2 then
--# expr-start 7578 '<at(<EnvGet(<FieldAccess .children_env>, __nextpart)>, 0)>' Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:12114
--# expr-start 7577 '<EnvGet(<FieldAccess .children_env>, __nextpart)>' Env_Get_Result_2 /usr/src/packages/BUILD/ada/ast.py:12114
--# expr-start 7575 '<FieldAccess .children_env>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12114
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 7575
--# expr-start 7576 '<Symbol __nextpart>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Nextpart); 
--# expr-done 7576
Env_Get_Result_2 := Construct_Entity_Array (AST_Envs.Get (Self => Fld_9, Key => Sym_2, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 7577
Get_Result_2 := Get (Env_Get_Result_2, 0, True); 
--# expr-done 7578
Var_Expr_3 := Get_Result_2;
if Var_Expr_3 /= No_Entity then
   --# scope-start
   --# bind pb Var_Expr_3
--# expr-start 7579 '<FieldAccess .children_env>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:12116
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_10 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# expr-done 7579
Scope_Result_2 := Fld_10;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2550;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Empty_Env;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_3 := Result_Var_2;
else
   --# expr-start 7580 '<FieldAccess .children_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:12118
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Libadalang.Implementation.Children_Env (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# expr-done 7580
   If_Result_3 := Fld_11;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 7581
Scope_Result_3 := If_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2549;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var_3);
Block_Var_5 := Result_Var_3; Inc_Ref (Block_Var_5);
--# bind package_body_env Block_Var_5
--# expr-start 7582 '<Cast to GenericPackageDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:12123
Cast_Expr_2 := Block_Var_1; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Generic_Package_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Generic_Package_Decl
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_2 := No_Entity_Generic_Package_Decl;
   end if;
--# expr-done 7582
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Generic_Package_Decl then
   --# scope-start
   --# bind pkg_g Var_Expr_4
--# expr-start 7584 '<FieldAccess .children_env>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:12124
--# expr-start 7583 '<FieldAccess .formal_part>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:12124
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Generic_Formal_Part (Node => Var_Expr_4.Node.Generic_Decl_F_Formal_Part, Info => Var_Expr_4.Info);
--# expr-done 7583
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Children_Env (Node => Fld_12.Node, E_Info => Fld_12.Info);
--# expr-done 7584
Scope_Result_4 := Fld_13;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2551;
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := AST_Envs.Empty_Env;
end if;
      Inc_Ref (Result_Var_4);
Block_Var_6 := Result_Var_4; Inc_Ref (Block_Var_6);
--# bind formals_env Block_Var_6
--# expr-start 7595 '<Cond>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:12128
--# expr-start 7586 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12132
--# expr-start 7585 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:12132
Is_Equal := Equivalent (Block_Var_5, AST_Envs.Empty_Env); 
--# expr-done 7585
Not_Val := not (Is_Equal); 
--# expr-done 7586
if Not_Val then
   --# expr-start 7588 '<FieldAccess .is_children_env(...)>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:12133
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7587 '<FieldAccess .node_env>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:12134
Var_Expr_5 := Origin;
if Var_Expr_5 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_5
Scope_Result_5 := Var_Expr_5;
   --# end
   Result_Var_5 := Scope_Result_5;
else
Cast_Expr_3 := Self; 
      Cast_Result_3 := Cast_Expr_3;
   Result_Var_5 := Cast_Result_3;
end if;
      if Result_Var_5 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Node_Env (Node => Result_Var_5);
--# expr-done 7587
--# property-call-start AdaNode.is_children_env
Fld_15 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_5, Current_Env => Fld_14);
--# end
--# expr-done 7588
   If_Result_4 := Fld_15;
else
   If_Result_4 := False;
end if;
if If_Result_4 then
   --# expr-start 7590 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:12135
--# expr-start 7589 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:12135
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_5, 2 => Block_Var_4, 3 => Block_Var_3, 4 => Block_Var_6)); 
--# expr-done 7589
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 7590
   If_Result_6 := Group_Env;
else
--# expr-start 7592 '<FieldAccess .is_children_env(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:12141
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7591 '<FieldAccess .node_env>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:12142
Var_Expr_6 := Origin;
if Var_Expr_6 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind e Var_Expr_6
Scope_Result_6 := Var_Expr_6;
   --# end
   Result_Var_6 := Scope_Result_6;
else
Cast_Expr_4 := Self; 
      Cast_Result_4 := Cast_Expr_4;
   Result_Var_6 := Cast_Result_4;
end if;
      if Result_Var_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Libadalang.Implementation.Node_Env (Node => Result_Var_6);
--# expr-done 7591
--# property-call-start AdaNode.is_children_env
Fld_17 := Libadalang.Implementation.Ada_Node_P_Is_Children_Env (Node => Self, Parent => Block_Var_4, Current_Env => Fld_16);
--# end
--# expr-done 7592
if Fld_17 then
   --# expr-start 7594 '<env_group(<ArrayLiteral>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:12143
--# expr-start 7593 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:12143
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Block_Var_4, 2 => Block_Var_3, 3 => Block_Var_6)); 
--# expr-done 7593
Group_Env_1 := Group (Array_Lit_1, No_Metadata); 
--# expr-done 7594
   If_Result_5 := Group_Env_1;
else
   If_Result_5 := Block_Var_3;
end if;
      Inc_Ref (If_Result_5);
   If_Result_6 := If_Result_5;
end if;
      Inc_Ref (If_Result_6);
--# expr-done 7595
Let_Result := If_Result_6; Inc_Ref (Let_Result);
--# end
--# expr-done 7596
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_944;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_944;
               Finalizer_Scope_2548;
               Finalizer_Scope_2549;
               Finalizer_Scope_2550;
               Finalizer_Scope_2551;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Pkg_Env;
--# end
--# property-start BaseId.parent_scope /usr/src/packages/BUILD/ada/ast.py:12152
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Parent_Scope
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
  )
   return Lexical_Env
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Parent_Scope;
--# end
--# property-start BaseId.designated_type_impl_get_real_type /usr/src/packages/BUILD/ada/ast.py:12155
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl_Get_Real_Type
  (Node : Bare_Base_Id
      ; N : Internal_Entity
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind n N
   Property_Result : Internal_Entity_Base_Type_Decl;
      Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Task_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Task_Type_Decl;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity_Task_Body;
Cast_Result_2 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_3 : Internal_Entity;
Let_Result_2 : Internal_Entity_Base_Type_Decl;
Scope_Result_2 : Internal_Entity_Base_Type_Decl;
Match_Result : Internal_Entity_Base_Type_Decl;
Let_Result_3 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7599 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 7598 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:12160
      if N.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := N; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind t Match_0
Let_Result := Match_0; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Task_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Task_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind tb Match_1
--# expr-start 7597 '<FieldAccess .task_type>' Fld /usr/src/packages/BUILD/ada/ast.py:12162
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TaskBody.task_type
Fld := Libadalang.Implementation.Task_Body_P_Task_Type (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7597
      Cast_Result_2 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
Let_Result_1 := Cast_Result_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Protected_Body | Ada_Entry_Index_Spec .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_3; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Base_Type_Decl; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 7598
Let_Result_3 := Match_Result; 
--# end
--# expr-done 7599
      Property_Result := Let_Result_3;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Type_Impl_Get_Real_Type;
--# end
--# property-start BaseId.designated_type_impl /usr/src/packages/BUILD/ada/ast.py:12167
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Designated_Type_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_947;
            procedure Finalizer_Scope_2566;
      Block_Var : Internal_Entity_Base_Type_Decl;
Block_Var_1 : Internal_Entity_Base_Type_Decl;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Block_Var_3 : Internal_Entity_Base_Type_Decl;
Var_Expr : Internal_Entity;
Var_Expr_1 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Type_Decl;
Var_Expr_3 : Bare_Ada_Node;
Var_Expr_4 : Internal_Entity_Base_Type_Decl;
Fld : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Fld_1 : Internal_Entity;
Fld_2 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Fld_3 : Boolean;
If_Result_1 : Lookup_Kind;
Fld_4 : Internal_Entity;
Fld_5 : Internal_Entity_Base_Type_Decl;
Scope_Result_1 : Internal_Entity_Base_Type_Decl;
Result_Var_1 : Internal_Entity_Base_Type_Decl;
Is_Null : Boolean;
Cast_Expr_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Is_Equiv : Boolean;
Fld_6 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
If_Result_3 : Internal_Entity_Base_Type_Decl;
If_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_7 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Fld_8 : Internal_Entity_Array_Access;
Scope_Result_3 : Internal_Entity_Array_Access;
Result_Var_3 : Internal_Entity_Array_Access;
Cast_Expr_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Cast_Expr_4 : Internal_Entity;
Cast_Result_4 : Internal_Entity_Base_Type_Decl;
Fld_9 : Boolean;
Scope_Result_4 : Boolean;
Result_Var_4 : Boolean;
If_Result_5 : Boolean;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Get_Result : Internal_Entity_Base_Type_Decl;
Is_Null_1 : Boolean;
Not_Val : Boolean;
If_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_947 is
            begin
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_947;
            procedure Finalizer_Scope_2566 is
            begin
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
            end Finalizer_Scope_2566;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7624 '<Block>' Let_Result None
--# scope-start
--# expr-start 7602 '<FieldAccess .env_get_first_visible(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12169
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7601 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12172
--# expr-start 7600 '<FieldAccess .is_prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:12172
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7600
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7601
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
--# property-call-start SingleTokNode.env_get_first_visible
Fld_1 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result, From_Node => Cast_Result);
--# end
--# expr-done 7602
Var_Expr := Fld_1;
if Var_Expr /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr
--# expr-start 7603 '<FieldAccess .designated_type_impl_get_real_type(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12174
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_2 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr);
--# end
--# expr-done 7603
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Block_Var := Result_Var; 
--# bind des_type_1 Block_Var
--# expr-start 7606 '<FieldAccess .env_get_first_visible(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12178
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7605 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12181
--# expr-start 7604 '<FieldAccess .is_prefix>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12181
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_3 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7604
if Fld_3 then
   If_Result_1 := Recursive;
else
   If_Result_1 := Flat;
end if;
--# expr-done 7605
--# property-call-start SingleTokNode.env_get_first_visible
Fld_4 := Libadalang.Implementation.Single_Tok_Node_P_Env_Get_First_Visible (Node => Self, Lex_Env => Env, Lookup_Type => If_Result_1, From_Node => Origin);
--# end
--# expr-done 7606
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity then
   --# scope-start
   --# bind env_el Var_Expr_1
--# expr-start 7607 '<FieldAccess .designated_type_impl_get_real_type(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12183
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.designated_type_impl_get_real_type
Fld_5 := Libadalang.Implementation.Base_Id_P_Designated_Type_Impl_Get_Real_Type (Node => Self, N => Var_Expr_1);
--# end
--# expr-done 7607
Scope_Result_1 := Fld_5;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity_Base_Type_Decl;
end if;
Block_Var_1 := Result_Var_1; 
--# bind des_type_2 Block_Var_1
--# expr-start 7611 '<Cond>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:12186
--# expr-start 7608 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12188
Is_Null := Block_Var.Node = null; 
--# expr-done 7608
if Is_Null then
   If_Result_4 := Block_Var_1;
else
--# expr-start 7609 '<Eq>' Is_Equiv /usr/src/packages/BUILD/ada/ast.py:12192
Cast_Expr_1 := Block_Var; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
Cast_Expr_2 := Block_Var_1; 
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
Is_Equiv := Equivalent (Cast_Result_1, Cast_Result_2); 
--# expr-done 7609
if Is_Equiv then
   If_Result_3 := Block_Var;
else
Var_Expr_2 := Block_Var;
if Var_Expr_2 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_2
--# expr-start 7610 '<FieldAccess .is_view_of_type(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12196
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_2.Node, Comp_View => Block_Var_1, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 7610
Scope_Result_2 := Fld_6;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
if Result_Var_2 then
   If_Result_2 := Block_Var_1;
else
   If_Result_2 := Block_Var;
end if;
   If_Result_3 := If_Result_2;
end if;
   If_Result_4 := If_Result_3;
end if;
--# expr-done 7611
Block_Var_2 := If_Result_4; 
--# bind des_type Block_Var_2
--# expr-start 7620 '<at(<filter_map>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:12206
--# expr-start 7619 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12206
Var_Expr_3 := Origin;
if Var_Expr_3 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind o Var_Expr_3
--# expr-start 7614 '<FieldAccess .env_get(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12206
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7612 '<FieldAccess .children_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12207
      if Var_Expr_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => Var_Expr_3);
--# expr-done 7612
--# expr-start 7613 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:12207
Sym := Get_Symbol (Self); 
--# expr-done 7613
--# property-call-start AdaNode.env_get
Fld_8 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld_7, Symbol => Sym, From_Node => Origin, Categories => (Inherited_Primitives => False, Nocat => True));
--# end
--# expr-done 7614
Scope_Result_3 := Fld_8;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2566;
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var_3);
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var_3;
      begin
         for Item_233 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_233
--# expr-start 7615 '<IsA BaseTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:12211
Is_A := Item_233.Node /= null 
and then Item_233.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 7615
if Is_A then
Var_Expr_4 := Block_Var_2;
if Var_Expr_4 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind d Var_Expr_4
--# expr-start 7617 '<FieldAccess .is_view_of_type(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12212
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7616 '<Cast to BaseTypeDecl>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:12212
Cast_Expr_4 := Item_233; 
   if Cast_Expr_4.Node = null
      or else Cast_Expr_4.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_4 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_4.Node,
         Info => Cast_Expr_4.Info);
   else
         Cast_Result_4 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7616
--# property-call-start BaseTypeDecl.is_view_of_type
Fld_9 := Libadalang.Implementation.Base_Type_Decl_P_Is_View_Of_Type (Node => Var_Expr_4.Node, Comp_View => Cast_Result_4, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7617
Scope_Result_4 := Fld_9;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := False;
end if;
   If_Result_5 := Result_Var_4;
else
   If_Result_5 := False;
end if;
               if If_Result_5 then
      --# expr-start 7618 '<Cast to BaseTypeDecl>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:12209
Cast_Expr_3 := Item_233; 
   if Cast_Expr_3.Node = null
      or else Cast_Expr_3.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_3 := Create_Internal_Entity_Base_Type_Decl
        (Node => Cast_Expr_3.Node,
         Info => Cast_Expr_3.Info);
   else
         Cast_Result_3 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7618
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Cast_Result_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7619
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7620
Block_Var_3 := Get_Result; 
--# bind completer_view Block_Var_3
--# expr-start 7623 '<If>' If_Result_6 /usr/src/packages/BUILD/ada/ast.py:12217
--# expr-start 7622 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12218
--# expr-start 7621 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:12218
Is_Null_1 := Block_Var_3.Node = null; 
--# expr-done 7621
Not_Val := not (Is_Null_1); 
--# expr-done 7622
if Not_Val then
   If_Result_6 := Block_Var_3;
else
   If_Result_6 := Block_Var_2;
end if;
--# expr-done 7623
Let_Result := If_Result_6; 
--# end
--# expr-done 7624
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_947;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_947;
               Finalizer_Scope_2566;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Designated_Type_Impl;
--# end
--# property-start BaseId.env_elements_impl /usr/src/packages/BUILD/ada/ast.py:12224
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Impl
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_948;
      Fld : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_948 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_948;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7626 '<Block>' Let_Result None
--# scope-start
--# expr-start 7625 '<FieldAccess .env_elements_baseid>' Fld /usr/src/packages/BUILD/ada/ast.py:12225
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.env_elements_baseid
Fld := Libadalang.Implementation.Base_Id_P_Env_Elements_Baseid (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7625
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7626
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_948;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_948;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Env_Elements_Impl;
--# end
--# property-start BaseId.all_env_els_impl /usr/src/packages/BUILD/ada/ast.py:12228
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_All_Env_Els_Impl
  (Node : Bare_Base_Id
      ; Seq : Boolean
         := True
      ; Seq_From : Bare_Ada_Node
         := No_Bare_Ada_Node
      ; Categories : Ref_Categories
         := (Inherited_Primitives => True, Nocat => True)
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind seq Seq
   --# bind seq_from Seq_From
   --# bind categories Categories
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_949;
      Fld : Symbol_Type;
Fld_1 : Boolean;
If_Result : Lookup_Kind;
Cast_Expr : Bare_Base_Id;
Cast_Result : Bare_Ada_Node;
Is_Null : Boolean;
Not_Val : Boolean;
If_Result_1 : Bare_Ada_Node;
If_Result_2 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Let_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_949 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_949;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7635 '<Block>' Let_Result None
--# scope-start
--# expr-start 7634 '<FieldAccess .env_get(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12233
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7627 '<FieldAccess .name_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:12235
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Self);
--# end
--# expr-done 7627
--# expr-start 7629 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12236
--# expr-start 7628 '<FieldAccess .is_prefix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12236
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld_1 := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7628
if Fld_1 then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7629
--# expr-start 7633 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:12237
if Seq then
   --# expr-start 7632 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12237
--# expr-start 7631 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12237
--# expr-start 7630 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12237
Is_Null := Seq_From = null; 
--# expr-done 7630
Not_Val := not (Is_Null); 
--# expr-done 7631
if Not_Val then
   If_Result_1 := Seq_From;
else
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
   If_Result_1 := Cast_Result;
end if;
--# expr-done 7632
   If_Result_2 := If_Result_1;
else
   If_Result_2 := No_Bare_Ada_Node;
end if;
--# expr-done 7633
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Fld, Lookup => If_Result, From_Node => If_Result_2, Categories => Categories);
--# end
--# expr-done 7634
Let_Result := Fld_2; Inc_Ref (Let_Result);
--# end
--# expr-done 7635
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_949;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_949;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_All_Env_Els_Impl;
--# end
--# property-start BaseId.env_elements_baseid /usr/src/packages/BUILD/ada/ast.py:12243
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Env_Elements_Baseid
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Array_Access
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   Property_Result : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_950;
            procedure Finalizer_Scope_2524;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Internal_Entity_Call_Expr;
Block_Var_2 : Boolean;
Var_Expr : Internal_Entity_Base_Subp_Body;
Var_Expr_1 : Internal_Entity_Assoc_List;
Match_0 : Internal_Entity_Base_Type_Decl;
Match_1 : Internal_Entity_Basic_Decl;
Match_2 : Internal_Entity;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Real_Pc : Internal_Entity_Call_Expr;
Var_Expr_3 : Internal_Entity_Entry_Spec;
Var_Expr_4 : Internal_Entity_Call_Expr;
Sym : Symbol_Type;
Fld : Boolean;
If_Result : Lookup_Kind;
Fld_1 : Bare_Ada_Node;
Fld_2 : Boolean;
If_Result_1 : Ref_Categories;
Fld_3 : Internal_Entity_Array_Access;
Fld_4 : Internal_Entity_Call_Expr;
Fld_5 : Boolean;
Not_Val : Boolean;
Fld_6 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Is_Null : Boolean;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_7 : Boolean;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Subp_Body;
Fld_8 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result_2 : Boolean;
If_Result_3 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_9 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Assoc_List;
Fld_10 : Bare_Assoc_List;
Len : Integer;
Is_Equal : Boolean;
Fld_11 : Internal_Entity_Base_Subp_Spec;
Fld_12 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Call_Expr;
Cast_Expr_2 : Internal_Entity_Base_Subp_Spec;
Cast_Result_4 : Internal_Entity_Entry_Spec;
Fld_13 : Internal_Entity;
Scope_Result_1 : Internal_Entity;
Result_Var_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Call_Expr;
Fld_14 : Internal_Entity_Info;
Fld_15 : Internal_Metadata;
Fld_16 : Boolean;
Fld_17 : Boolean;
Fld_18 : Internal_Entity;
Cast_Result_5 : Internal_Entity_Call_Expr;
Fld_19 : Internal_Entity_Base_Type_Decl;
Fld_20 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
If_Result_5 : Boolean;
Fld_21 : Internal_Entity_Base_Type_Decl;
Fld_22 : Boolean;
Fld_23 : Internal_Entity_Info;
Fld_24 : Internal_Metadata;
Fld_25 : Boolean;
Fld_26 : Boolean;
If_Result_6 : Boolean;
If_Result_7 : Boolean;
Let_Result : Boolean;
Scope_Result_3 : Boolean;
Fld_27 : Internal_Entity_Base_Type_Decl;
Fld_28 : Boolean;
Result_Var_3 : Boolean;
Match_Prefix : Internal_Entity;
Cast_Result_6 : Internal_Entity_Base_Type_Decl;
Let_Result_1 : Boolean;
Scope_Result_4 : Boolean;
Cast_Result_7 : Internal_Entity_Basic_Decl;
Let_Result_2 : Boolean;
Scope_Result_5 : Boolean;
Cast_Result_8 : Internal_Entity;
Let_Result_3 : Boolean;
Scope_Result_6 : Boolean;
Match_Result : Boolean;
Map_Result_1 : Internal_Entity_Array_Access;
Scope_Result_7 : Internal_Entity_Array_Access;
Is_A : Boolean;
Not_Val_1 : Boolean;
Map_Result_2 : Internal_Entity_Array_Access;
Result_Var_4 : Internal_Entity_Array_Access;
If_Result_8 : Internal_Entity_Array_Access;
Dyn_Var_Bind_Result : Internal_Entity_Array_Access;
Let_Result_4 : Internal_Entity_Array_Access;
            procedure Finalizer_Scope_950 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Result_Var_4);
                     Dec_Ref (If_Result_8);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_950;
            procedure Finalizer_Scope_2524 is
            begin
                     Dec_Ref (Map_Result_1);
            end Finalizer_Scope_2524;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_Base_Id_Base_Id_P_Env_Elements_Baseid,
            Items    => new Mmz_Key_Array (1 ..  3))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_Base_Id,
                                As_Bare_Base_Id => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Array_Access;
                  Inc_Ref (Property_Result);
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7688 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 7642 '<FieldAccess .env_get(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12248
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7636 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:12250
Sym := Get_Symbol (Self); 
--# expr-done 7636
--# expr-start 7638 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12251
--# expr-start 7637 '<FieldAccess .is_prefix>' Fld /usr/src/packages/BUILD/ada/ast.py:12251
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_prefix
Fld := Libadalang.Implementation.Name_P_Is_Prefix (Node => Self);
--# end
--# expr-done 7637
if Fld then
   If_Result := Recursive;
else
   If_Result := Flat;
end if;
--# expr-done 7638
--# expr-start 7639 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12253
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7639
--# expr-start 7641 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12254
--# expr-start 7640 '<FieldAccess .can_designate_primitive>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12255
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.can_designate_primitive
Fld_2 := Libadalang.Implementation.Name_P_Can_Designate_Primitive (Node => Self);
--# end
--# expr-done 7640
if Fld_2 then
   If_Result_1 := (Inherited_Primitives => True, Nocat => True);
else
   If_Result_1 := (Inherited_Primitives => False, Nocat => True);
end if;
--# expr-done 7641
--# property-call-start AdaNode.env_get
Fld_3 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Env, Symbol => Sym, Lookup => If_Result, From_Node => Fld_1, Categories => If_Result_1);
--# end
--# expr-done 7642
Block_Var := Fld_3; Inc_Ref (Block_Var);
--# bind items Block_Var
--# expr-start 7643 '<FieldAccess .parent_callexpr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12264
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.parent_callexpr
Fld_4 := Libadalang.Implementation.Name_P_Parent_Callexpr (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 7643
Block_Var_1 := Fld_4; 
--# bind pc Block_Var_1
--# expr-start 7645 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12265
--# expr-start 7644 '<FieldAccess .is_suffix>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12265
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_5 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 7644
Not_Val := not (Fld_5); 
--# expr-done 7645
Block_Var_2 := Not_Val; 
--# bind is_prefix Block_Var_2
--# expr-start 7687 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Cond>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:12267
--# expr-start 7646 '<FieldAccess .origin_node>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12267
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_6 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7646
Bound_Origin := Fld_6; 
--# expr-start 7686 '<Cond>' If_Result_8 /usr/src/packages/BUILD/ada/ast.py:12267
--# expr-start 7647 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:12268
Is_Null := Block_Var_1.Node = null; 
--# expr-done 7647
if Is_Null then
   --# expr-start 7652 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12274
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_234 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_234
--# expr-start 7649 '<FieldAccess .can_be_paramless>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12279
--# expr-start 7648 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12279
Cast_Expr := Item_234; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7648
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.can_be_paramless
Fld_7 := Libadalang.Implementation.Basic_Decl_P_Can_Be_Paramless (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 7649
if Fld_7 then
   If_Result_3 := True;
else
if Block_Var_2 then
--# expr-start 7650 '<Cast to BaseSubpBody>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:12284
Cast_Expr_1 := Item_234; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_1 := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 7650
Var_Expr := Cast_Result_1;
if Var_Expr /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 7651 '<FieldAccess .in_scope>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:12284
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.in_scope
Fld_8 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Var_Expr.Node, Origin => Bound_Origin);
--# end
--# expr-done 7651
Scope_Result := Fld_8;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result_2 := Result_Var;
else
   If_Result_2 := False;
end if;
   If_Result_3 := If_Result_2;
end if;
               if If_Result_3 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_234;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7652
   If_Result_8 := Map_Result;
else
--# expr-start 7654 '<Cast to AssocList>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:12294
--# expr-start 7653 '<FieldAccess .suffix>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:12294
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Create_Internal_Entity (Node => Block_Var_1.Node.Call_Expr_F_Suffix, Info => Block_Var_1.Info);
--# expr-done 7653
   if Fld_9.Node = null
      or else Fld_9.Node.Kind in Ada_Assoc_List_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Assoc_List
        (Node => Fld_9.Node,
         Info => Fld_9.Info);
   else
         Cast_Result_2 := No_Entity_Assoc_List;
   end if;
--# expr-done 7654
Var_Expr_1 := Cast_Result_2;
if Var_Expr_1 /= No_Entity_Assoc_List then
   --# scope-start
   --# bind params Var_Expr_1
--# expr-start 7682 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:12295
declare
   Map_Result_1_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_236 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_236
               --# expr-start 7681 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:12295
      if Item_236.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_236; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Discrete_Base_Subtype_Decl .. Ada_Synth_Anonymous_Type_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity_Base_Type_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result_6; 
--# bind _ Match_0
--# expr-start 7656 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:12297
--# expr-start 7655 '<length(<Var params>)>' Len /usr/src/packages/BUILD/ada/ast.py:12297
Fld_10 := Var_Expr_1.Node; 
Len := Length (Fld_10); 
--# expr-done 7655
Is_Equal := Len = 1; 
--# expr-done 7656
Let_Result_1 := Is_Equal; 
--# end
Scope_Result_4 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Abstract_State_Decl .. Ada_Package_Decl | Ada_Abstract_Subp_Decl .. Ada_Single_Task_Decl =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Create_Internal_Entity_Basic_Decl
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_7; 
--# bind b Match_1
--# expr-start 7657 '<FieldAccess .subp_spec_or_null>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:12300
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_11 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Match_1.Node, E_Info => Match_1.Info);
--# end
--# expr-done 7657
Var_Expr_2 := Fld_11;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind spec Var_Expr_2
--# expr-start 7678 '<Let real_pc>' Let_Result /usr/src/packages/BUILD/ada/ast.py:12301
--# scope-start
--# expr-start 7663 '<If>' If_Result_4 /usr/src/packages/BUILD/ada/ast.py:12302
--# expr-start 7660 '<IsNull>' Is_Null_1 /usr/src/packages/BUILD/ada/ast.py:12303
--# expr-start 7658 '<Cast to EntrySpec>' Cast_Result_4 /usr/src/packages/BUILD/ada/ast.py:12303
Cast_Expr_2 := Var_Expr_2; 
   if Cast_Expr_2.Node = null
      or else Cast_Expr_2.Node.Kind in Ada_Entry_Spec_Range
   then
      Cast_Result_4 := Create_Internal_Entity_Entry_Spec
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
   else
         Cast_Result_4 := No_Entity_Entry_Spec;
   end if;
--# expr-done 7658
Var_Expr_3 := Cast_Result_4;
if Var_Expr_3 /= No_Entity_Entry_Spec then
   --# scope-start
--# expr-start 7659 '<FieldAccess .family_type>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:12303
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Create_Internal_Entity (Node => Var_Expr_3.Node.Entry_Spec_F_Family_Type, Info => Var_Expr_3.Info);
--# expr-done 7659
Scope_Result_1 := Fld_13;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Entity;
end if;
Is_Null_1 := Result_Var_1.Node = null; 
--# expr-done 7660
if Is_Null_1 then
   If_Result_4 := Block_Var_1;
else
   --# expr-start 7662 '<Cast to CallExpr>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:12304
--# expr-start 7661 '<parent(<Var pc>)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:12304
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Libadalang.Implementation.Parent (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# expr-done 7661
   if Fld_12.Node = null
      or else Fld_12.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Call_Expr
        (Node => Fld_12.Node,
         Info => Fld_12.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7662
   If_Result_4 := Cast_Result_3;
end if;
--# expr-done 7663
Real_Pc := If_Result_4; 
--# bind real_pc Real_Pc
--# expr-start 7667 '<FieldAccess .is_matching_param_list(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:12310
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7666 '<FieldAccess .dottable_subp>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:12311
--# expr-start 7665 '<FieldAccess .md>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:12311
--# expr-start 7664 '<FieldAccess .info>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:12311
Fld_14 := Match_1.Info;
--# expr-done 7664
Fld_15 := Fld_14.MD;
--# expr-done 7665
Fld_16 := Fld_15.Dottable_Subp;
--# expr-done 7666
--# property-call-start BaseFormalParamHolder.is_matching_param_list
Fld_17 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Is_Matching_Param_List (Node => Var_Expr_2.Node, Params => Var_Expr_1, Is_Dottable_Subp => Fld_16, Env => Env);
--# end
--# expr-done 7667
if Fld_17 then
--# expr-start 7669 '<Cast to CallExpr>' Cast_Result_5 /usr/src/packages/BUILD/ada/ast.py:12313
--# expr-start 7668 '<parent(<Var real_pc>)>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:12313
      if Real_Pc.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Libadalang.Implementation.Parent (Node => Real_Pc.Node, E_Info => Real_Pc.Info);
--# expr-done 7668
   if Fld_18.Node = null
      or else Fld_18.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result_5 := Create_Internal_Entity_Call_Expr
        (Node => Fld_18.Node,
         Info => Fld_18.Info);
   else
         Cast_Result_5 := No_Entity_Call_Expr;
   end if;
--# expr-done 7669
Var_Expr_4 := Cast_Result_5;
if Var_Expr_4 /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr_4
--# expr-start 7671 '<FieldAccess .check_for_type(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:12314
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7670 '<FieldAccess .expr_type>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:12314
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_19 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7670
--# property-call-start CallExpr.check_for_type
Fld_20 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Var_Expr_4.Node, Typ => Fld_19, Env => Env, Origin => Bound_Origin, E_Info => Var_Expr_4.Info);
--# end
--# expr-done 7671
Scope_Result_2 := Fld_20;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := True;
end if;
   If_Result_5 := Result_Var_2;
else
   If_Result_5 := False;
end if;
if If_Result_5 then
   If_Result_7 := True;
else
--# expr-start 7673 '<FieldAccess .check_for_type(...)>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:12321
      if Real_Pc.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7672 '<FieldAccess .expr_type>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:12321
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_21 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7672
--# property-call-start CallExpr.check_for_type
Fld_22 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Real_Pc.Node, Typ => Fld_21, Env => Env, Origin => Bound_Origin, E_Info => Real_Pc.Info);
--# end
--# expr-done 7673
if Fld_22 then
   --# expr-start 7677 '<FieldAccess .paramless(...)>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:12322
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7676 '<FieldAccess .dottable_subp>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:12322
--# expr-start 7675 '<FieldAccess .md>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:12322
--# expr-start 7674 '<FieldAccess .info>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:12322
Fld_23 := Match_1.Info;
--# expr-done 7674
Fld_24 := Fld_23.MD;
--# expr-done 7675
Fld_25 := Fld_24.Dottable_Subp;
--# expr-done 7676
--# property-call-start BaseFormalParamHolder.paramless
Fld_26 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Paramless (Node => Var_Expr_2.Node, Dottable_Subp => Fld_25);
--# end
--# expr-done 7677
   If_Result_6 := Fld_26;
else
   If_Result_6 := False;
end if;
   If_Result_7 := If_Result_6;
end if;
Let_Result := If_Result_7; 
--# end
--# expr-done 7678
Scope_Result_3 := Let_Result;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   --# expr-start 7680 '<FieldAccess .check_for_type(...)>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:12328
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7679 '<FieldAccess .expr_type>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:12328
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.expr_type'
Fld_27 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Expr_Type (Node => Match_1.Node, Origin => Bound_Origin, E_Info => Match_1.Info);
--# end
--# expr-done 7679
--# property-call-start CallExpr.check_for_type
Fld_28 := Libadalang.Implementation.Call_Expr_P_Check_For_Type (Node => Block_Var_1.Node, Typ => Fld_27, Env => Env, Origin => Bound_Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 7680
   Result_Var_3 := Fld_28;
end if;
Let_Result_2 := Result_Var_3; 
--# end
Scope_Result_5 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Abort_Absent .. Ada_Param_Assoc | Ada_Case_Stmt_Alternative .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_8; 
--# bind _ Match_2
Let_Result_3 := False; 
--# end
Scope_Result_6 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_6; 
end case;
--# expr-done 7681
               if Match_Result then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_236;
         begin
            Internal_Entity_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7682
Scope_Result_7 := Map_Result_1;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2524;
   Result_Var_4 := Scope_Result_7;
else
   --# expr-start 7685 '<filter>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:12336
declare
   Map_Result_2_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_235 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_235
               --# expr-start 7684 '<Not>' Not_Val_1 /usr/src/packages/BUILD/ada/ast.py:12336
--# expr-start 7683 '<IsA BaseTypeDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:12336
Is_A := Item_235.Node /= null 
and then Item_235.Node.Kind in Ada_Base_Type_Decl; 
--# expr-done 7683
Not_Val_1 := not (Is_A); 
--# expr-done 7684
               if Not_Val_1 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_235;
         begin
            Internal_Entity_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_2 := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_2_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 7685
   Result_Var_4 := Map_Result_2;
end if;
      Inc_Ref (Result_Var_4);
   If_Result_8 := Result_Var_4;
end if;
      Inc_Ref (If_Result_8);
--# expr-done 7686
Dyn_Var_Bind_Result := If_Result_8; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 7687
Let_Result_4 := Dyn_Var_Bind_Result; Inc_Ref (Let_Result_4);
--# end
--# expr-done 7688
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_950;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Array_Access,
                     As_Internal_Entity_Array_Access => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
            if Mmz_Stored then
               Inc_Ref (Property_Result);
            end if;
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_950;
               Finalizer_Scope_2524;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Env_Elements_Baseid;
--# end
--# property-start BaseId.xref_equation /usr/src/packages/BUILD/ada/ast.py:12341
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_951;
      Fld : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_951 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_951;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7690 '<Block>' Let_Result None
--# scope-start
--# expr-start 7689 '<FieldAccess .base_id_xref_equation(...)>' Fld /usr/src/packages/BUILD/ada/ast.py:12342
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7689
Let_Result := Fld; Inc_Ref (Let_Result);
--# end
--# expr-done 7690
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_951;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_951;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Xref_Equation;
--# end
--# property-start BaseId.base_id_xref_equation /usr/src/packages/BUILD/ada/ast.py:12345
pragma Warnings (Off, "is not referenced");
 function Base_Id_P_Base_Id_Xref_Equation
  (Node : Bare_Base_Id
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Id := Bare_Base_Id (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Id :=
     Internal_Entity_Base_Id'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_952;
            procedure Finalizer_Scope_2569;
      Block_Var : Internal_Entity_Array_Access;
Block_Var_1 : Boolean;
Var_Expr : Internal_Entity_Base_Subp_Body;
Fld : Internal_Entity_Array_Access;
Fld_1 : Boolean;
Not_Val : Boolean;
Fld_2 : Logic_Var;
Bind_Result : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Bind_Result_1 : Logic_Equation;
And_Pred : Logic_Equation;
Logic_True : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Bind_Result_2 : Logic_Equation;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Base_Subp_Body;
Fld_7 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_952 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_952;
            procedure Finalizer_Scope_2569 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Bind_Result_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Bind_Result_2);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2569;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7709 '<Block>' Let_Result None
--# scope-start
--# expr-start 7691 '<FieldAccess .env_elements>' Fld /usr/src/packages/BUILD/ada/ast.py:12346
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.env_elements
Fld := Libadalang.Implementation.Expr_P_Env_Elements (Node => Ent.Node, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 7691
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind env_els Block_Var
--# expr-start 7693 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:12347
--# expr-start 7692 '<FieldAccess .is_suffix>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12347
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.is_suffix
Fld_1 := Libadalang.Implementation.Name_P_Is_Suffix (Node => Self);
--# end
--# expr-done 7692
Not_Val := not (Fld_1); 
--# expr-done 7693
Block_Var_1 := Not_Val; 
--# bind is_prefix Block_Var_1
--# expr-start 7708 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:12349
--# expr-start 7707 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12349
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var;
      begin
         for Item_237 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_237
      --# expr-start 7706 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:12351
--# expr-start 7699 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:12351
--# expr-start 7695 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:12351
--# expr-start 7694 '<FieldAccess .ref_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12351
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7694
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
Bind_Result := Bind_Default_Default.Create (Fld_2, Item_237, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12351") else null)); 
--# expr-done 7695
--# expr-start 7698 '<Bind>' Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:12352
--# expr-start 7696 '<FieldAccess .ref_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12352
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7696
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
--# expr-start 7697 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12352
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7697
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Bind_Result_1 := Bind_173_567.Create (Fld_3, Fld_4, Logic_Converter_173'(Origin => Origin), Equals_Data_567'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12352") else null)); 
--# expr-done 7698
And_Pred := Logic_And (Bind_Result, Bind_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12351") else null)); 
--# expr-done 7699
--# expr-start 7705 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:12354
if Block_Var_1 then
--# expr-start 7700 '<Cast to BaseSubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12357
Cast_Expr := Item_237; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Base_Subp_Body
   then
      Cast_Result := Create_Internal_Entity_Base_Subp_Body
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Base_Subp_Body;
   end if;
--# expr-done 7700
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Base_Subp_Body then
   --# scope-start
--# expr-start 7701 '<FieldAccess .in_scope>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:12357
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Body.in_scope
Fld_7 := Libadalang.Implementation.Body_Node_P_In_Scope (Node => Var_Expr.Node, Origin => Origin);
--# end
--# expr-done 7701
Scope_Result := Fld_7;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
   If_Result := Result_Var;
else
   If_Result := False;
end if;
if If_Result then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12359") else null)); 
   If_Result_1 := Logic_True;
else
   --# expr-start 7704 '<Bind>' Bind_Result_2 /usr/src/packages/BUILD/ada/ast.py:12366
--# expr-start 7702 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7702
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 7703 '<FieldAccess .subp_spec_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:12366
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.subp_spec_var'
Fld_6 := Libadalang.Implementation.Dispatcher_Name_P_Subp_Spec_Var (Node => Self);
--# end
--# expr-done 7703
Fld_6.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_6.all);
Eq_Node.Refs.Destroy (Fld_6.all);
Bind_Result_2 := Bind_175_Default.Create (Fld_5, Fld_6, Logic_Converter_175'(null record), No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12366") else null)); 
--# expr-done 7704
   If_Result_1 := Bind_Result_2;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7705
And_Pred_1 := Logic_And (And_Pred, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12351") else null)); 
--# expr-done 7706
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred_1;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2569;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7707
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12349") else null)); 
--# expr-done 7708
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 7709
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_952;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_952;
               Finalizer_Scope_2569;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Id_P_Base_Id_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Char_Literal
   --
      procedure Initialize_Fields_For_Char_Literal
        (Self : Bare_Char_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Char_Literal;
--# property-start CharLiteral.xref_equation /usr/src/packages/BUILD/ada/ast.py:12567
pragma Warnings (Off, "is not referenced");
 function Char_Literal_P_Xref_Equation
  (Node : Bare_Char_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Char_Literal := Bare_Char_Literal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Char_Literal :=
     Internal_Entity_Char_Literal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_967;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_967 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_967;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7717 '<Block>' Let_Result None
--# scope-start
--# expr-start 7716 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:12568
--# expr-start 7710 '<FieldAccess .base_id_xref_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:12569
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7710
--# expr-start 7715 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:12570
--# expr-start 7712 '<Predicate on AdaNode.is_not_null>' Pred /usr/src/packages/BUILD/ada/ast.py:12570
--# expr-start 7711 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7711
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Ada_Node_P_Is_Not_Null_0_Pred.Create (Fld_1, Create_Ada_Node_P_Is_Not_Null_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Ada_Node.Ada_Node_P_Is_Not_Null")            else null))); 
--# expr-done 7712
--# expr-start 7714 '<Predicate on BaseTypeDecl.is_char_type>' Pred_1 /usr/src/packages/BUILD/ada/ast.py:12571
--# expr-start 7713 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7713
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
Pred_1 := Base_Type_Decl_P_Is_Char_Type_0_Pred.Create (Fld_2, Create_Base_Type_Decl_P_Is_Char_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Char_Type")            else null))); 
--# expr-done 7714
And_Pred := Logic_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12570") else null)); 
--# expr-done 7715
Or_Pred := Logic_Or (Fld, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12568") else null)); 
--# expr-done 7716
Let_Result := Or_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7717
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_967;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_967;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Char_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Identifier
   --
      procedure Initialize_Fields_For_Identifier
        (Self : Bare_Identifier
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Identifier;
--# property-start Identifier.is_attr_with_args /usr/src/packages/BUILD/ada/ast.py:12460
pragma Warnings (Off, "is not referenced");
 function Identifier_P_Is_Attr_With_Args
  (Node : Bare_Identifier
  )
   return Boolean
is
   Self : Bare_Identifier := Bare_Identifier (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal : Boolean;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_3 : Boolean;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Sym_20 : Symbol_Type;
Sym_21 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Sym_22 : Symbol_Type;
Sym_23 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Sym_24 : Symbol_Type;
Sym_25 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Sym_26 : Symbol_Type;
Sym_27 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Sym_28 : Symbol_Type;
Sym_29 : Symbol_Type;
Is_Equal_14 : Boolean;
If_Result_13 : Boolean;
Sym_30 : Symbol_Type;
Sym_31 : Symbol_Type;
Is_Equal_15 : Boolean;
If_Result_14 : Boolean;
Sym_32 : Symbol_Type;
Sym_33 : Symbol_Type;
Is_Equal_16 : Boolean;
If_Result_15 : Boolean;
Sym_34 : Symbol_Type;
Sym_35 : Symbol_Type;
Is_Equal_17 : Boolean;
If_Result_16 : Boolean;
Sym_36 : Symbol_Type;
Sym_37 : Symbol_Type;
Is_Equal_18 : Boolean;
If_Result_17 : Boolean;
Sym_38 : Symbol_Type;
Sym_39 : Symbol_Type;
Is_Equal_19 : Boolean;
If_Result_18 : Boolean;
Sym_40 : Symbol_Type;
Sym_41 : Symbol_Type;
Is_Equal_20 : Boolean;
If_Result_19 : Boolean;
Sym_42 : Symbol_Type;
Sym_43 : Symbol_Type;
Is_Equal_21 : Boolean;
If_Result_20 : Boolean;
Sym_44 : Symbol_Type;
Sym_45 : Symbol_Type;
Is_Equal_22 : Boolean;
If_Result_21 : Boolean;
Sym_46 : Symbol_Type;
Sym_47 : Symbol_Type;
Is_Equal_23 : Boolean;
If_Result_22 : Boolean;
Sym_48 : Symbol_Type;
Sym_49 : Symbol_Type;
Is_Equal_24 : Boolean;
If_Result_23 : Boolean;
Sym_50 : Symbol_Type;
Sym_51 : Symbol_Type;
Is_Equal_25 : Boolean;
If_Result_24 : Boolean;
Sym_52 : Symbol_Type;
Sym_53 : Symbol_Type;
Is_Equal_26 : Boolean;
If_Result_25 : Boolean;
Sym_54 : Symbol_Type;
Sym_55 : Symbol_Type;
Is_Equal_27 : Boolean;
If_Result_26 : Boolean;
Sym_56 : Symbol_Type;
Sym_57 : Symbol_Type;
Is_Equal_28 : Boolean;
If_Result_27 : Boolean;
Sym_58 : Symbol_Type;
Sym_59 : Symbol_Type;
Is_Equal_29 : Boolean;
If_Result_28 : Boolean;
Sym_60 : Symbol_Type;
Sym_61 : Symbol_Type;
Is_Equal_30 : Boolean;
If_Result_29 : Boolean;
Sym_62 : Symbol_Type;
Sym_63 : Symbol_Type;
Is_Equal_31 : Boolean;
If_Result_30 : Boolean;
Sym_64 : Symbol_Type;
Sym_65 : Symbol_Type;
Is_Equal_32 : Boolean;
If_Result_31 : Boolean;
Sym_66 : Symbol_Type;
Sym_67 : Symbol_Type;
Is_Equal_33 : Boolean;
If_Result_32 : Boolean;
Sym_68 : Symbol_Type;
Sym_69 : Symbol_Type;
Is_Equal_34 : Boolean;
If_Result_33 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 7720 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7718 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:12461
Sym := Get_Symbol (Self); 
--# expr-done 7718
--# expr-start 7719 '<Symbol Write>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Write); 
--# expr-done 7719
Is_Equal := Sym = Sym_1; 
--# expr-done 7720
if Is_Equal then
   If_Result := True;
else
   --# expr-start 7723 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7721 '<GetSymbol>' Sym_2 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_2 := Get_Symbol (Self); 
--# expr-done 7721
--# expr-start 7722 '<Symbol Read>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Read); 
--# expr-done 7722
Is_Equal_1 := Sym_2 = Sym_3; 
--# expr-done 7723
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 7726 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7724 '<GetSymbol>' Sym_4 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_4 := Get_Symbol (Self); 
--# expr-done 7724
--# expr-start 7725 '<Symbol Input>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Input); 
--# expr-done 7725
Is_Equal_2 := Sym_4 = Sym_5; 
--# expr-done 7726
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 7729 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7727 '<GetSymbol>' Sym_6 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_6 := Get_Symbol (Self); 
--# expr-done 7727
--# expr-start 7728 '<Symbol Output>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Output); 
--# expr-done 7728
Is_Equal_3 := Sym_6 = Sym_7; 
--# expr-done 7729
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   If_Result_3 := True;
else
   --# expr-start 7732 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7730 '<GetSymbol>' Sym_8 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_8 := Get_Symbol (Self); 
--# expr-done 7730
--# expr-start 7731 '<Symbol Succ>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Succ); 
--# expr-done 7731
Is_Equal_4 := Sym_8 = Sym_9; 
--# expr-done 7732
   If_Result_3 := Is_Equal_4;
end if;
if If_Result_3 then
   If_Result_4 := True;
else
   --# expr-start 7735 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7733 '<GetSymbol>' Sym_10 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_10 := Get_Symbol (Self); 
--# expr-done 7733
--# expr-start 7734 '<Symbol Pred>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pred); 
--# expr-done 7734
Is_Equal_5 := Sym_10 = Sym_11; 
--# expr-done 7735
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 7738 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7736 '<GetSymbol>' Sym_12 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_12 := Get_Symbol (Self); 
--# expr-done 7736
--# expr-start 7737 '<Symbol Min>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Min); 
--# expr-done 7737
Is_Equal_6 := Sym_12 = Sym_13; 
--# expr-done 7738
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 7741 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7739 '<GetSymbol>' Sym_14 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_14 := Get_Symbol (Self); 
--# expr-done 7739
--# expr-start 7740 '<Symbol Max>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Max); 
--# expr-done 7740
Is_Equal_7 := Sym_14 = Sym_15; 
--# expr-done 7741
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 7744 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7742 '<GetSymbol>' Sym_16 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_16 := Get_Symbol (Self); 
--# expr-done 7742
--# expr-start 7743 '<Symbol Value>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Value); 
--# expr-done 7743
Is_Equal_8 := Sym_16 = Sym_17; 
--# expr-done 7744
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 7747 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7745 '<GetSymbol>' Sym_18 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_18 := Get_Symbol (Self); 
--# expr-done 7745
--# expr-start 7746 '<Symbol Wide_Value>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Value); 
--# expr-done 7746
Is_Equal_9 := Sym_18 = Sym_19; 
--# expr-done 7747
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 7750 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7748 '<GetSymbol>' Sym_20 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_20 := Get_Symbol (Self); 
--# expr-done 7748
--# expr-start 7749 '<Symbol Wide_Wide_Value>' Sym_21 None
Sym_21 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Value); 
--# expr-done 7749
Is_Equal_10 := Sym_20 = Sym_21; 
--# expr-done 7750
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 7753 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7751 '<GetSymbol>' Sym_22 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_22 := Get_Symbol (Self); 
--# expr-done 7751
--# expr-start 7752 '<Symbol Pos>' Sym_23 None
Sym_23 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pos); 
--# expr-done 7752
Is_Equal_11 := Sym_22 = Sym_23; 
--# expr-done 7753
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 7756 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7754 '<GetSymbol>' Sym_24 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_24 := Get_Symbol (Self); 
--# expr-done 7754
--# expr-start 7755 '<Symbol Val>' Sym_25 None
Sym_25 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Val); 
--# expr-done 7755
Is_Equal_12 := Sym_24 = Sym_25; 
--# expr-done 7756
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 7759 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7757 '<GetSymbol>' Sym_26 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_26 := Get_Symbol (Self); 
--# expr-done 7757
--# expr-start 7758 '<Symbol Enum_Val>' Sym_27 None
Sym_27 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Enum_Val); 
--# expr-done 7758
Is_Equal_13 := Sym_26 = Sym_27; 
--# expr-done 7759
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 7762 '<Eq>' Is_Equal_14 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7760 '<GetSymbol>' Sym_28 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_28 := Get_Symbol (Self); 
--# expr-done 7760
--# expr-start 7761 '<Symbol First>' Sym_29 None
Sym_29 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_First); 
--# expr-done 7761
Is_Equal_14 := Sym_28 = Sym_29; 
--# expr-done 7762
   If_Result_13 := Is_Equal_14;
end if;
if If_Result_13 then
   If_Result_14 := True;
else
   --# expr-start 7765 '<Eq>' Is_Equal_15 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7763 '<GetSymbol>' Sym_30 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_30 := Get_Symbol (Self); 
--# expr-done 7763
--# expr-start 7764 '<Symbol Last>' Sym_31 None
Sym_31 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Last); 
--# expr-done 7764
Is_Equal_15 := Sym_30 = Sym_31; 
--# expr-done 7765
   If_Result_14 := Is_Equal_15;
end if;
if If_Result_14 then
   If_Result_15 := True;
else
   --# expr-start 7768 '<Eq>' Is_Equal_16 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7766 '<GetSymbol>' Sym_32 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_32 := Get_Symbol (Self); 
--# expr-done 7766
--# expr-start 7767 '<Symbol Range>' Sym_33 None
Sym_33 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Range); 
--# expr-done 7767
Is_Equal_16 := Sym_32 = Sym_33; 
--# expr-done 7768
   If_Result_15 := Is_Equal_16;
end if;
if If_Result_15 then
   If_Result_16 := True;
else
   --# expr-start 7771 '<Eq>' Is_Equal_17 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7769 '<GetSymbol>' Sym_34 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_34 := Get_Symbol (Self); 
--# expr-done 7769
--# expr-start 7770 '<Symbol Length>' Sym_35 None
Sym_35 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Length); 
--# expr-done 7770
Is_Equal_17 := Sym_34 = Sym_35; 
--# expr-done 7771
   If_Result_16 := Is_Equal_17;
end if;
if If_Result_16 then
   If_Result_17 := True;
else
   --# expr-start 7774 '<Eq>' Is_Equal_18 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7772 '<GetSymbol>' Sym_36 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_36 := Get_Symbol (Self); 
--# expr-done 7772
--# expr-start 7773 '<Symbol Image>' Sym_37 None
Sym_37 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Image); 
--# expr-done 7773
Is_Equal_18 := Sym_36 = Sym_37; 
--# expr-done 7774
   If_Result_17 := Is_Equal_18;
end if;
if If_Result_17 then
   If_Result_18 := True;
else
   --# expr-start 7777 '<Eq>' Is_Equal_19 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7775 '<GetSymbol>' Sym_38 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_38 := Get_Symbol (Self); 
--# expr-done 7775
--# expr-start 7776 '<Symbol Wide_Image>' Sym_39 None
Sym_39 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Image); 
--# expr-done 7776
Is_Equal_19 := Sym_38 = Sym_39; 
--# expr-done 7777
   If_Result_18 := Is_Equal_19;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 7780 '<Eq>' Is_Equal_20 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7778 '<GetSymbol>' Sym_40 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_40 := Get_Symbol (Self); 
--# expr-done 7778
--# expr-start 7779 '<Symbol Wide_Wide_Image>' Sym_41 None
Sym_41 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Wide_Wide_Image); 
--# expr-done 7779
Is_Equal_20 := Sym_40 = Sym_41; 
--# expr-done 7780
   If_Result_19 := Is_Equal_20;
end if;
if If_Result_19 then
   If_Result_20 := True;
else
   --# expr-start 7783 '<Eq>' Is_Equal_21 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7781 '<GetSymbol>' Sym_42 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_42 := Get_Symbol (Self); 
--# expr-done 7781
--# expr-start 7782 '<Symbol Asm_Input>' Sym_43 None
Sym_43 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Input); 
--# expr-done 7782
Is_Equal_21 := Sym_42 = Sym_43; 
--# expr-done 7783
   If_Result_20 := Is_Equal_21;
end if;
if If_Result_20 then
   If_Result_21 := True;
else
   --# expr-start 7786 '<Eq>' Is_Equal_22 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7784 '<GetSymbol>' Sym_44 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_44 := Get_Symbol (Self); 
--# expr-done 7784
--# expr-start 7785 '<Symbol Asm_Output>' Sym_45 None
Sym_45 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Asm_Output); 
--# expr-done 7785
Is_Equal_22 := Sym_44 = Sym_45; 
--# expr-done 7786
   If_Result_21 := Is_Equal_22;
end if;
if If_Result_21 then
   If_Result_22 := True;
else
   --# expr-start 7789 '<Eq>' Is_Equal_23 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7787 '<GetSymbol>' Sym_46 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_46 := Get_Symbol (Self); 
--# expr-done 7787
--# expr-start 7788 '<Symbol Rounding>' Sym_47 None
Sym_47 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rounding); 
--# expr-done 7788
Is_Equal_23 := Sym_46 = Sym_47; 
--# expr-done 7789
   If_Result_22 := Is_Equal_23;
end if;
if If_Result_22 then
   If_Result_23 := True;
else
   --# expr-start 7792 '<Eq>' Is_Equal_24 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7790 '<GetSymbol>' Sym_48 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_48 := Get_Symbol (Self); 
--# expr-done 7790
--# expr-start 7791 '<Symbol Round>' Sym_49 None
Sym_49 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Round); 
--# expr-done 7791
Is_Equal_24 := Sym_48 = Sym_49; 
--# expr-done 7792
   If_Result_23 := Is_Equal_24;
end if;
if If_Result_23 then
   If_Result_24 := True;
else
   --# expr-start 7795 '<Eq>' Is_Equal_25 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7793 '<GetSymbol>' Sym_50 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_50 := Get_Symbol (Self); 
--# expr-done 7793
--# expr-start 7794 '<Symbol Ceiling>' Sym_51 None
Sym_51 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Ceiling); 
--# expr-done 7794
Is_Equal_25 := Sym_50 = Sym_51; 
--# expr-done 7795
   If_Result_24 := Is_Equal_25;
end if;
if If_Result_24 then
   If_Result_25 := True;
else
   --# expr-start 7798 '<Eq>' Is_Equal_26 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7796 '<GetSymbol>' Sym_52 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_52 := Get_Symbol (Self); 
--# expr-done 7796
--# expr-start 7797 '<Symbol Floor>' Sym_53 None
Sym_53 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Floor); 
--# expr-done 7797
Is_Equal_26 := Sym_52 = Sym_53; 
--# expr-done 7798
   If_Result_25 := Is_Equal_26;
end if;
if If_Result_25 then
   If_Result_26 := True;
else
   --# expr-start 7801 '<Eq>' Is_Equal_27 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7799 '<GetSymbol>' Sym_54 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_54 := Get_Symbol (Self); 
--# expr-done 7799
--# expr-start 7800 '<Symbol Truncation>' Sym_55 None
Sym_55 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Truncation); 
--# expr-done 7800
Is_Equal_27 := Sym_54 = Sym_55; 
--# expr-done 7801
   If_Result_26 := Is_Equal_27;
end if;
if If_Result_26 then
   If_Result_27 := True;
else
   --# expr-start 7804 '<Eq>' Is_Equal_28 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7802 '<GetSymbol>' Sym_56 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_56 := Get_Symbol (Self); 
--# expr-done 7802
--# expr-start 7803 '<Symbol Copy_Sign>' Sym_57 None
Sym_57 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Copy_Sign); 
--# expr-done 7803
Is_Equal_28 := Sym_56 = Sym_57; 
--# expr-done 7804
   If_Result_27 := Is_Equal_28;
end if;
if If_Result_27 then
   If_Result_28 := True;
else
   --# expr-start 7807 '<Eq>' Is_Equal_29 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7805 '<GetSymbol>' Sym_58 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_58 := Get_Symbol (Self); 
--# expr-done 7805
--# expr-start 7806 '<Symbol Remainder>' Sym_59 None
Sym_59 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Remainder); 
--# expr-done 7806
Is_Equal_29 := Sym_58 = Sym_59; 
--# expr-done 7807
   If_Result_28 := Is_Equal_29;
end if;
if If_Result_28 then
   If_Result_29 := True;
else
   --# expr-start 7810 '<Eq>' Is_Equal_30 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7808 '<GetSymbol>' Sym_60 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_60 := Get_Symbol (Self); 
--# expr-done 7808
--# expr-start 7809 '<Symbol Adjacent>' Sym_61 None
Sym_61 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Adjacent); 
--# expr-done 7809
Is_Equal_30 := Sym_60 = Sym_61; 
--# expr-done 7810
   If_Result_29 := Is_Equal_30;
end if;
if If_Result_29 then
   If_Result_30 := True;
else
   --# expr-start 7813 '<Eq>' Is_Equal_31 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7811 '<GetSymbol>' Sym_62 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_62 := Get_Symbol (Self); 
--# expr-done 7811
--# expr-start 7812 '<Symbol Mod>' Sym_63 None
Sym_63 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod_110); 
--# expr-done 7812
Is_Equal_31 := Sym_62 = Sym_63; 
--# expr-done 7813
   If_Result_30 := Is_Equal_31;
end if;
if If_Result_30 then
   If_Result_31 := True;
else
   --# expr-start 7816 '<Eq>' Is_Equal_32 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7814 '<GetSymbol>' Sym_64 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_64 := Get_Symbol (Self); 
--# expr-done 7814
--# expr-start 7815 '<Symbol Machine_Rounding>' Sym_65 None
Sym_65 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Rounding); 
--# expr-done 7815
Is_Equal_32 := Sym_64 = Sym_65; 
--# expr-done 7816
   If_Result_31 := Is_Equal_32;
end if;
if If_Result_31 then
   If_Result_32 := True;
else
   --# expr-start 7819 '<Eq>' Is_Equal_33 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7817 '<GetSymbol>' Sym_66 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_66 := Get_Symbol (Self); 
--# expr-done 7817
--# expr-start 7818 '<Symbol Exponent>' Sym_67 None
Sym_67 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Exponent); 
--# expr-done 7818
Is_Equal_33 := Sym_66 = Sym_67; 
--# expr-done 7819
   If_Result_32 := Is_Equal_33;
end if;
if If_Result_32 then
   If_Result_33 := True;
else
   --# expr-start 7822 '<Eq>' Is_Equal_34 /usr/src/packages/BUILD/ada/ast.py:12461
--# expr-start 7820 '<GetSymbol>' Sym_68 /usr/src/packages/BUILD/ada/ast.py:12461
Sym_68 := Get_Symbol (Self); 
--# expr-done 7820
--# expr-start 7821 '<Symbol Machine_Radix>' Sym_69 None
Sym_69 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Machine_Radix); 
--# expr-done 7821
Is_Equal_34 := Sym_68 = Sym_69; 
--# expr-done 7822
   If_Result_33 := Is_Equal_34;
end if;
      Property_Result := If_Result_33;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Identifier_P_Is_Attr_With_Args;
--# end
   --
   --  Primitives for Bare_Op
   --
      procedure Initialize_Fields_For_Op
        (Self : Bare_Op
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_Op;
--# property-start Op.subprogram_symbol /usr/src/packages/BUILD/ada/ast.py:12386
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprogram_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Match_0 : Bare_Op_And;
Match_1 : Bare_Op_Or;
Match_2 : Bare_Op_Xor;
Match_3 : Bare_Op_Abs;
Match_4 : Bare_Op_Not;
Match_5 : Bare_Op_Pow;
Match_6 : Bare_Op_Mult;
Match_7 : Bare_Op_Div;
Match_8 : Bare_Op_Mod;
Match_9 : Bare_Op_Rem;
Match_10 : Bare_Op_Plus;
Match_11 : Bare_Op_Minus;
Match_12 : Bare_Op_Concat;
Match_13 : Bare_Op_Eq;
Match_14 : Bare_Op_Neq;
Match_15 : Bare_Op_Lt;
Match_16 : Bare_Op_Lte;
Match_17 : Bare_Op_Gt;
Match_18 : Bare_Op_Gte;
Match_19 : Bare_Op;
Sym : Symbol_Type;
Sym_1 : Symbol_Type;
Sym_2 : Symbol_Type;
Sym_3 : Symbol_Type;
Sym_4 : Symbol_Type;
Sym_5 : Symbol_Type;
Sym_6 : Symbol_Type;
Sym_7 : Symbol_Type;
Sym_8 : Symbol_Type;
Sym_9 : Symbol_Type;
Sym_10 : Symbol_Type;
Sym_11 : Symbol_Type;
Sym_12 : Symbol_Type;
Sym_13 : Symbol_Type;
Sym_14 : Symbol_Type;
Sym_15 : Symbol_Type;
Sym_16 : Symbol_Type;
Sym_17 : Symbol_Type;
Sym_18 : Symbol_Type;
Sym_19 : Symbol_Type;
Match_Prefix : Bare_Op;
Cast_Result : Bare_Op_And;
Let_Result : Symbol_Type;
Scope_Result : Symbol_Type;
Cast_Result_1 : Bare_Op_Or;
Let_Result_1 : Symbol_Type;
Scope_Result_1 : Symbol_Type;
Cast_Result_2 : Bare_Op_Xor;
Let_Result_2 : Symbol_Type;
Scope_Result_2 : Symbol_Type;
Cast_Result_3 : Bare_Op_Abs;
Let_Result_3 : Symbol_Type;
Scope_Result_3 : Symbol_Type;
Cast_Result_4 : Bare_Op_Not;
Let_Result_4 : Symbol_Type;
Scope_Result_4 : Symbol_Type;
Cast_Result_5 : Bare_Op_Pow;
Let_Result_5 : Symbol_Type;
Scope_Result_5 : Symbol_Type;
Cast_Result_6 : Bare_Op_Mult;
Let_Result_6 : Symbol_Type;
Scope_Result_6 : Symbol_Type;
Cast_Result_7 : Bare_Op_Div;
Let_Result_7 : Symbol_Type;
Scope_Result_7 : Symbol_Type;
Cast_Result_8 : Bare_Op_Mod;
Let_Result_8 : Symbol_Type;
Scope_Result_8 : Symbol_Type;
Cast_Result_9 : Bare_Op_Rem;
Let_Result_9 : Symbol_Type;
Scope_Result_9 : Symbol_Type;
Cast_Result_10 : Bare_Op_Plus;
Let_Result_10 : Symbol_Type;
Scope_Result_10 : Symbol_Type;
Cast_Result_11 : Bare_Op_Minus;
Let_Result_11 : Symbol_Type;
Scope_Result_11 : Symbol_Type;
Cast_Result_12 : Bare_Op_Concat;
Let_Result_12 : Symbol_Type;
Scope_Result_12 : Symbol_Type;
Cast_Result_13 : Bare_Op_Eq;
Let_Result_13 : Symbol_Type;
Scope_Result_13 : Symbol_Type;
Cast_Result_14 : Bare_Op_Neq;
Let_Result_14 : Symbol_Type;
Scope_Result_14 : Symbol_Type;
Cast_Result_15 : Bare_Op_Lt;
Let_Result_15 : Symbol_Type;
Scope_Result_15 : Symbol_Type;
Cast_Result_16 : Bare_Op_Lte;
Let_Result_16 : Symbol_Type;
Scope_Result_16 : Symbol_Type;
Cast_Result_17 : Bare_Op_Gt;
Let_Result_17 : Symbol_Type;
Scope_Result_17 : Symbol_Type;
Cast_Result_18 : Bare_Op_Gte;
Let_Result_18 : Symbol_Type;
Scope_Result_18 : Symbol_Type;
Cast_Result_19 : Bare_Op;
Let_Result_19 : Symbol_Type;
Scope_Result_19 : Symbol_Type;
Match_Result : Symbol_Type;
Let_Result_20 : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7844 '<Block>' Let_Result_20 None
--# scope-start
--# expr-start 7843 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:12391
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Self; 
case Ada_Op (Match_Prefix.Kind) is
         when Ada_Op_And =>
   --# scope-start
--# scope-start
      Cast_Result := Match_Prefix;
Match_0 := Cast_Result; 
--# bind _ Match_0
--# expr-start 7823 '<Symbol "and">' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_And); 
--# expr-done 7823
Let_Result := Sym; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Op_Or =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Match_Prefix;
Match_1 := Cast_Result_1; 
--# bind _ Match_1
--# expr-start 7824 '<Symbol "or">' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Or); 
--# expr-done 7824
Let_Result_1 := Sym_1; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Op_Xor =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Match_Prefix;
Match_2 := Cast_Result_2; 
--# bind _ Match_2
--# expr-start 7825 '<Symbol "xor">' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Xor); 
--# expr-done 7825
Let_Result_2 := Sym_2; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
         when Ada_Op_Abs =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Match_Prefix;
Match_3 := Cast_Result_3; 
--# bind _ Match_3
--# expr-start 7826 '<Symbol "abs">' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Abs); 
--# expr-done 7826
Let_Result_3 := Sym_3; 
--# end
Scope_Result_3 := Let_Result_3;
   --# end
            Match_Result := Scope_Result_3; 
         when Ada_Op_Not =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Match_Prefix;
Match_4 := Cast_Result_4; 
--# bind _ Match_4
--# expr-start 7827 '<Symbol "not">' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Not); 
--# expr-done 7827
Let_Result_4 := Sym_4; 
--# end
Scope_Result_4 := Let_Result_4;
   --# end
            Match_Result := Scope_Result_4; 
         when Ada_Op_Pow =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Match_Prefix;
Match_5 := Cast_Result_5; 
--# bind _ Match_5
--# expr-start 7828 '<Symbol "**">' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_3); 
--# expr-done 7828
Let_Result_5 := Sym_5; 
--# end
Scope_Result_5 := Let_Result_5;
   --# end
            Match_Result := Scope_Result_5; 
         when Ada_Op_Mult =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Match_Prefix;
Match_6 := Cast_Result_6; 
--# bind _ Match_6
--# expr-start 7829 '<Symbol "*">' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_2); 
--# expr-done 7829
Let_Result_6 := Sym_6; 
--# end
Scope_Result_6 := Let_Result_6;
   --# end
            Match_Result := Scope_Result_6; 
         when Ada_Op_Div =>
   --# scope-start
--# scope-start
      Cast_Result_7 := Match_Prefix;
Match_7 := Cast_Result_7; 
--# bind _ Match_7
--# expr-start 7830 '<Symbol "/">' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_6); 
--# expr-done 7830
Let_Result_7 := Sym_7; 
--# end
Scope_Result_7 := Let_Result_7;
   --# end
            Match_Result := Scope_Result_7; 
         when Ada_Op_Mod =>
   --# scope-start
--# scope-start
      Cast_Result_8 := Match_Prefix;
Match_8 := Cast_Result_8; 
--# bind _ Match_8
--# expr-start 7831 '<Symbol "mod">' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Mod); 
--# expr-done 7831
Let_Result_8 := Sym_8; 
--# end
Scope_Result_8 := Let_Result_8;
   --# end
            Match_Result := Scope_Result_8; 
         when Ada_Op_Rem =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix;
Match_9 := Cast_Result_9; 
--# bind _ Match_9
--# expr-start 7832 '<Symbol "rem">' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Rem); 
--# expr-done 7832
Let_Result_9 := Sym_9; 
--# end
Scope_Result_9 := Let_Result_9;
   --# end
            Match_Result := Scope_Result_9; 
         when Ada_Op_Plus =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix;
Match_10 := Cast_Result_10; 
--# bind _ Match_10
--# expr-start 7833 '<Symbol "+">' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_4); 
--# expr-done 7833
Let_Result_10 := Sym_10; 
--# end
Scope_Result_10 := Let_Result_10;
   --# end
            Match_Result := Scope_Result_10; 
         when Ada_Op_Minus =>
   --# scope-start
--# scope-start
      Cast_Result_11 := Match_Prefix;
Match_11 := Cast_Result_11; 
--# bind _ Match_11
--# expr-start 7834 '<Symbol "-">' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_5); 
--# expr-done 7834
Let_Result_11 := Sym_11; 
--# end
Scope_Result_11 := Let_Result_11;
   --# end
            Match_Result := Scope_Result_11; 
         when Ada_Op_Concat =>
   --# scope-start
--# scope-start
      Cast_Result_12 := Match_Prefix;
Match_12 := Cast_Result_12; 
--# bind _ Match_12
--# expr-start 7835 '<Symbol "&">' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol); 
--# expr-done 7835
Let_Result_12 := Sym_12; 
--# end
Scope_Result_12 := Let_Result_12;
   --# end
            Match_Result := Scope_Result_12; 
         when Ada_Op_Eq =>
   --# scope-start
--# scope-start
      Cast_Result_13 := Match_Prefix;
Match_13 := Cast_Result_13; 
--# bind _ Match_13
--# expr-start 7836 '<Symbol "=">' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_10); 
--# expr-done 7836
Let_Result_13 := Sym_13; 
--# end
Scope_Result_13 := Let_Result_13;
   --# end
            Match_Result := Scope_Result_13; 
         when Ada_Op_Neq =>
   --# scope-start
--# scope-start
      Cast_Result_14 := Match_Prefix;
Match_14 := Cast_Result_14; 
--# bind _ Match_14
--# expr-start 7837 '<Symbol "/=">' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_7); 
--# expr-done 7837
Let_Result_14 := Sym_14; 
--# end
Scope_Result_14 := Let_Result_14;
   --# end
            Match_Result := Scope_Result_14; 
         when Ada_Op_Lt =>
   --# scope-start
--# scope-start
      Cast_Result_15 := Match_Prefix;
Match_15 := Cast_Result_15; 
--# bind _ Match_15
--# expr-start 7838 '<Symbol "<">' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_8); 
--# expr-done 7838
Let_Result_15 := Sym_15; 
--# end
Scope_Result_15 := Let_Result_15;
   --# end
            Match_Result := Scope_Result_15; 
         when Ada_Op_Lte =>
   --# scope-start
--# scope-start
      Cast_Result_16 := Match_Prefix;
Match_16 := Cast_Result_16; 
--# bind _ Match_16
--# expr-start 7839 '<Symbol "<=">' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_9); 
--# expr-done 7839
Let_Result_16 := Sym_16; 
--# end
Scope_Result_16 := Let_Result_16;
   --# end
            Match_Result := Scope_Result_16; 
         when Ada_Op_Gt =>
   --# scope-start
--# scope-start
      Cast_Result_17 := Match_Prefix;
Match_17 := Cast_Result_17; 
--# bind _ Match_17
--# expr-start 7840 '<Symbol ">">' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_11); 
--# expr-done 7840
Let_Result_17 := Sym_17; 
--# end
Scope_Result_17 := Let_Result_17;
   --# end
            Match_Result := Scope_Result_17; 
         when Ada_Op_Gte =>
   --# scope-start
--# scope-start
      Cast_Result_18 := Match_Prefix;
Match_18 := Cast_Result_18; 
--# bind _ Match_18
--# expr-start 7841 '<Symbol ">=">' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_12); 
--# expr-done 7841
Let_Result_18 := Sym_18; 
--# end
Scope_Result_18 := Let_Result_18;
   --# end
            Match_Result := Scope_Result_18; 
         when Ada_Op_And_Then | Ada_Op_Double_Dot | Ada_Op_In | Ada_Op_Not_In | Ada_Op_Or_Else =>
   --# scope-start
--# scope-start
      Cast_Result_19 := Match_Prefix;
Match_19 := Cast_Result_19; 
--# bind _ Match_19
--# expr-start 7842 '<Symbol <<>>>' Sym_19 None
Sym_19 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_20); 
--# expr-done 7842
Let_Result_19 := Sym_19; 
--# end
Scope_Result_19 := Let_Result_19;
   --# end
            Match_Result := Scope_Result_19; 
end case;
--# expr-done 7843
Let_Result_20 := Match_Result; 
--# end
--# expr-done 7844
      Property_Result := Let_Result_20;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprogram_Symbol;
--# end
--# property-start Op.subprograms_for_symbol /usr/src/packages/BUILD/ada/ast.py:12415
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms_For_Symbol
  (Node : Bare_Op
      ; Sym : Symbol_Type
      ; From_Node : Internal_Entity
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind sym Sym
   --# bind from_node From_Node
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_954;
      Fld : Lexical_Env := Empty_Env;
Fld_1 : Bare_Ada_Node;
Fld_2 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Cast_Expr_1 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Basic_Decl;
Fld_3 : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_954 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_954;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7852 '<Block>' Let_Result None
--# scope-start
--# expr-start 7851 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:12416
--# expr-start 7847 '<FieldAccess .env_get(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12416
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7845 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:12417
      if From_Node.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => From_Node.Node, E_Info => From_Node.Info);
--# expr-done 7845
--# expr-start 7846 '<FieldAccess .node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12417
Fld_1 := From_Node.Node;
--# expr-done 7846
--# property-call-start AdaNode.env_get
Fld_2 := Libadalang.Implementation.Ada_Node_P_Env_Get (Node => Self, Env => Fld, Symbol => Sym, From_Node => Fld_1);
--# end
--# expr-done 7847
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_2;
      begin
         for Item_238 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_238
               --# expr-start 7849 '<FieldAccess .is_subprogram>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12420
--# expr-start 7848 '<Cast to BasicDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:12420
Cast_Expr_1 := Item_238; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7848
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.is_subprogram
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Is_Subprogram (Node => Cast_Result_1.Node);
--# end
--# expr-done 7849
               if Fld_3 then
      --# expr-start 7850 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:12419
Cast_Expr := Item_238; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7850
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7851
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7852
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_954;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_954;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprograms_For_Symbol;
--# end
--# property-start Op.subprograms /usr/src/packages/BUILD/ada/ast.py:12423
pragma Warnings (Off, "is not referenced");
 function Op_P_Subprograms
  (Node : Bare_Op
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Op :=
     Internal_Entity_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_955;
      Fld : Symbol_Type;
Cast_Expr : Internal_Entity_Op;
Cast_Result : Internal_Entity;
Fld_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_955 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_955;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7854 '<FieldAccess .subprograms_for_symbol(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12424
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7853 '<FieldAccess .subprogram_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:12424
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 7853
Cast_Expr := Ent; 
      Cast_Result := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
--# property-call-start Op.subprograms_for_symbol
Fld_1 := Libadalang.Implementation.Op_P_Subprograms_For_Symbol (Node => Self, Sym => Fld, From_Node => Cast_Result);
--# end
--# expr-done 7854
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_955;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_955;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Subprograms;
--# end
--# property-start Op.name_symbol /usr/src/packages/BUILD/ada/ast.py:12431
pragma Warnings (Off, "is not referenced");
 function Op_P_Name_Symbol
  (Node : Bare_Op
  )
   return Symbol_Type
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7855 '<FieldAccess .subprogram_symbol>' Fld /usr/src/packages/BUILD/ada/ast.py:12431
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprogram_symbol
Fld := Libadalang.Implementation.Op_P_Subprogram_Symbol (Node => Self);
--# end
--# expr-done 7855
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Name_Symbol;
--# end
--# property-start Op.xref_equation /usr/src/packages/BUILD/ada/ast.py:12434
pragma Warnings (Off, "is not referenced");
 function Op_P_Xref_Equation
  (Node : Bare_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Op := Bare_Op (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_957;
      Logic_False : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_957 is
            begin
                     Dec_Ref (Logic_False);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_957;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7856 '<Block>' Let_Result None
--# scope-start
Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12437") else null)); 
Let_Result := Logic_False; Inc_Ref (Let_Result);
--# end
--# expr-done 7856
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_957;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_957;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Op_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Op_Abs
   --
      procedure Initialize_Fields_For_Op_Abs
        (Self : Bare_Op_Abs
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Abs;
   --
   --  Primitives for Bare_Op_And
   --
      procedure Initialize_Fields_For_Op_And
        (Self : Bare_Op_And
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And;
   --
   --  Primitives for Bare_Op_And_Then
   --
      procedure Initialize_Fields_For_Op_And_Then
        (Self : Bare_Op_And_Then
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_And_Then;
   --
   --  Primitives for Bare_Op_Concat
   --
      procedure Initialize_Fields_For_Op_Concat
        (Self : Bare_Op_Concat
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Concat;
   --
   --  Primitives for Bare_Op_Div
   --
      procedure Initialize_Fields_For_Op_Div
        (Self : Bare_Op_Div
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Div;
   --
   --  Primitives for Bare_Op_Double_Dot
   --
      procedure Initialize_Fields_For_Op_Double_Dot
        (Self : Bare_Op_Double_Dot
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Double_Dot;
   --
   --  Primitives for Bare_Op_Eq
   --
      procedure Initialize_Fields_For_Op_Eq
        (Self : Bare_Op_Eq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Eq;
   --
   --  Primitives for Bare_Op_Gt
   --
      procedure Initialize_Fields_For_Op_Gt
        (Self : Bare_Op_Gt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gt;
   --
   --  Primitives for Bare_Op_Gte
   --
      procedure Initialize_Fields_For_Op_Gte
        (Self : Bare_Op_Gte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Gte;
   --
   --  Primitives for Bare_Op_In
   --
      procedure Initialize_Fields_For_Op_In
        (Self : Bare_Op_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_In;
   --
   --  Primitives for Bare_Op_Lt
   --
      procedure Initialize_Fields_For_Op_Lt
        (Self : Bare_Op_Lt
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lt;
   --
   --  Primitives for Bare_Op_Lte
   --
      procedure Initialize_Fields_For_Op_Lte
        (Self : Bare_Op_Lte
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Lte;
   --
   --  Primitives for Bare_Op_Minus
   --
      procedure Initialize_Fields_For_Op_Minus
        (Self : Bare_Op_Minus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Minus;
   --
   --  Primitives for Bare_Op_Mod
   --
      procedure Initialize_Fields_For_Op_Mod
        (Self : Bare_Op_Mod
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mod;
   --
   --  Primitives for Bare_Op_Mult
   --
      procedure Initialize_Fields_For_Op_Mult
        (Self : Bare_Op_Mult
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Mult;
   --
   --  Primitives for Bare_Op_Neq
   --
      procedure Initialize_Fields_For_Op_Neq
        (Self : Bare_Op_Neq
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Neq;
   --
   --  Primitives for Bare_Op_Not
   --
      procedure Initialize_Fields_For_Op_Not
        (Self : Bare_Op_Not
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not;
   --
   --  Primitives for Bare_Op_Not_In
   --
      procedure Initialize_Fields_For_Op_Not_In
        (Self : Bare_Op_Not_In
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Not_In;
   --
   --  Primitives for Bare_Op_Or
   --
      procedure Initialize_Fields_For_Op_Or
        (Self : Bare_Op_Or
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or;
   --
   --  Primitives for Bare_Op_Or_Else
   --
      procedure Initialize_Fields_For_Op_Or_Else
        (Self : Bare_Op_Or_Else
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Or_Else;
   --
   --  Primitives for Bare_Op_Plus
   --
      procedure Initialize_Fields_For_Op_Plus
        (Self : Bare_Op_Plus
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Plus;
   --
   --  Primitives for Bare_Op_Pow
   --
      procedure Initialize_Fields_For_Op_Pow
        (Self : Bare_Op_Pow
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Pow;
   --
   --  Primitives for Bare_Op_Rem
   --
      procedure Initialize_Fields_For_Op_Rem
        (Self : Bare_Op_Rem
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Rem;
   --
   --  Primitives for Bare_Op_Xor
   --
      procedure Initialize_Fields_For_Op_Xor
        (Self : Bare_Op_Xor
        ) is
      begin
            Initialize_Fields_For_Op
              (Self);
      end Initialize_Fields_For_Op_Xor;
   --
   --  Primitives for Bare_String_Literal
   --
      procedure Initialize_Fields_For_String_Literal
        (Self : Bare_String_Literal
        ) is
      begin
            Initialize_Fields_For_Base_Id
              (Self);
      end Initialize_Fields_For_String_Literal;
--# property-start StringLiteral.xref_equation /usr/src/packages/BUILD/ada/ast.py:12497
pragma Warnings (Off, "is not referenced");
 function String_Literal_P_Xref_Equation
  (Node : Bare_String_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_String_Literal := Bare_String_Literal (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_String_Literal :=
     Internal_Entity_String_Literal'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_960;
      Fld : Logic_Equation;
Fld_1 : Logic_Var;
Sym : Symbol_Type;
Fld_2 : Internal_Entity;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_5 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_960 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_960;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7868 '<Block>' Let_Result None
--# scope-start
--# expr-start 7867 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:12498
--# expr-start 7858 '<IsA Name>' Is_A /usr/src/packages/BUILD/ada/ast.py:12501
--# expr-start 7857 '<parent(<Var Self>)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:12501
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Self.Parent; 
--# expr-done 7857
Is_A := Fld_5 /= null 
and then Fld_5.Kind in Ada_Name; 
--# expr-done 7858
if Is_A then
   --# expr-start 7859 '<FieldAccess .base_id_xref_equation>' Fld /usr/src/packages/BUILD/ada/ast.py:12502
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseId.base_id_xref_equation
Fld := Libadalang.Implementation.Base_Id_P_Base_Id_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 7859
   If_Result := Fld;
else
   --# expr-start 7866 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:12503
--# expr-start 7863 '<FieldAccess .type_bind_val(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:12504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7860 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7860
--# expr-start 7862 '<FieldAccess .std_entity(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:12504
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7861 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 7861
--# property-call-start AdaNode.std_entity
Fld_2 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7862
--# property-call-start AdaNode.type_bind_val
Fld_3 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_1, Right => Fld_2, Origin => Origin);
--# end
--# expr-done 7863
--# expr-start 7865 '<Predicate on BaseTypeDecl.is_str_type_or_null>' Pred /usr/src/packages/BUILD/ada/ast.py:12505
--# expr-start 7864 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:12505
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7864
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Pred.Create (Fld_4, Create_Base_Type_Decl_P_Is_Str_Type_Or_Null_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Str_Type_Or_Null")            else null))); 
--# expr-done 7865
Or_Pred := Logic_Or (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:12503") else null)); 
--# expr-done 7866
   If_Result := Or_Pred;
end if;
      Inc_Ref (If_Result);
--# expr-done 7867
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7868
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_960;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_960;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end String_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Literal
   --
      procedure Initialize_Fields_For_Null_Literal
        (Self : Bare_Null_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Null_Literal;
--# property-start NullLiteral.xref_equation /usr/src/packages/BUILD/ada/ast.py:12624
pragma Warnings (Off, "is not referenced");
 function Null_Literal_P_Xref_Equation
  (Node : Bare_Null_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Literal := Bare_Null_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_971;
      Fld : Logic_Var;
Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_971 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_971;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7871 '<Block>' Let_Result None
--# scope-start
--# expr-start 7870 '<Predicate on BaseTypeDecl.is_access_type_predicate>' Pred /usr/src/packages/BUILD/ada/ast.py:12625
--# expr-start 7869 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:12625
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7869
Fld.Value := No_Entity;
Eq_Node.Refs.Reset (Fld.all);
Eq_Node.Refs.Destroy (Fld.all);
Pred := Base_Type_Decl_P_Is_Access_Type_Predicate_0_Pred.Create (Fld, Create_Base_Type_Decl_P_Is_Access_Type_Predicate_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Access_Type_Predicate")            else null))); 
--# expr-done 7870
Let_Result := Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7871
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_971;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_971;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Null_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Num_Literal
   --
      procedure Initialize_Fields_For_Num_Literal
        (Self : Bare_Num_Literal
        ) is
      begin
            Initialize_Fields_For_Single_Tok_Node
              (Self);
      end Initialize_Fields_For_Num_Literal;
   --
   --  Primitives for Bare_Int_Literal
   --
      procedure Initialize_Fields_For_Int_Literal
        (Self : Bare_Int_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Int_Literal;
--# property-start IntLiteral.xref_equation /usr/src/packages/BUILD/ada/ast.py:12604
pragma Warnings (Off, "is not referenced");
 function Int_Literal_P_Xref_Equation
  (Node : Bare_Int_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Int_Literal := Bare_Int_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_969;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_969 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_969;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7874 '<Block>' Let_Result None
--# scope-start
--# expr-start 7873 '<FieldAccess .universal_int_bind(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7872 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:12605
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7872
--# property-call-start AdaNode.universal_int_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Int_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 7873
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7874
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_969;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_969;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Int_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Literal
   --
      procedure Initialize_Fields_For_Real_Literal
        (Self : Bare_Real_Literal
        ) is
      begin
            Initialize_Fields_For_Num_Literal
              (Self);
      end Initialize_Fields_For_Real_Literal;
--# property-start RealLiteral.xref_equation /usr/src/packages/BUILD/ada/ast.py:12592
pragma Warnings (Off, "is not referenced");
 function Real_Literal_P_Xref_Equation
  (Node : Bare_Real_Literal
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Literal := Bare_Real_Literal (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_968;
      Fld : Logic_Var;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_968 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_968;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7877 '<Block>' Let_Result None
--# scope-start
--# expr-start 7876 '<FieldAccess .universal_real_bind(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:12593
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7875 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:12593
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7875
--# property-call-start AdaNode.universal_real_bind
Fld_1 := Libadalang.Implementation.Ada_Node_P_Universal_Real_Bind (Node => Self, Type_Var => Fld, Origin => Origin);
--# end
--# expr-done 7876
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7877
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_968;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_968;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Literal_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Target_Name
   --
      procedure Initialize_Fields_For_Target_Name
        (Self : Bare_Target_Name
        ) is
      begin
            Initialize_Fields_For_Name
              (Self);
      Self.Target_Name_R_Ref_Var := Null_Var_Record;
      end Initialize_Fields_For_Target_Name;
--# property-start TargetName.ref_var /usr/src/packages/BUILD/ada/ast.py:10467
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Ref_Var
  (Node : Bare_Target_Name
  )
   return Logic_Var
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Logic_Var;
      Fld : Logic_Var;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7878 '<FieldAccess .r_ref_var>' Fld /usr/src/packages/BUILD/ada/ast.py:10467
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Target_Name_R_Ref_Var'Unrestricted_Access;
--# expr-done 7878
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Ref_Var;
--# end
--# property-start TargetName.assign_statement /usr/src/packages/BUILD/ada/ast.py:10469
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Assign_Statement
  (Node : Bare_Target_Name
  )
   return Bare_Assign_Stmt
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_846;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Assign_Stmt;
            procedure Finalizer_Scope_846 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_846;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7883 '<Cast to AssignStmt>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:10470
--# expr-start 7882 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:10470
--# expr-start 7881 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:10470
--# expr-start 7879 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:10470
Node_Parents := Parents (Self, True); 
--# expr-done 7879
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_239 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_239
               --# expr-start 7880 '<IsA AssignStmt>' Is_A /usr/src/packages/BUILD/ada/ast.py:10470
Is_A := Item_239 /= null 
and then Item_239.Kind in Ada_Assign_Stmt_Range; 
--# expr-done 7880
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_239;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7881
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 7882
   if Get_Result = null
      or else Get_Result.Kind in Ada_Assign_Stmt_Range
   then
      Cast_Result := Get_Result;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7883
      Property_Result := Cast_Result;
   --# end
      Finalizer_Scope_846;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_846;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Assign_Statement;
--# end
--# property-start TargetName.relative_name /usr/src/packages/BUILD/ada/ast.py:10475
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Relative_Name
  (Node : Bare_Target_Name
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Single_Tok_Node
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Single_Tok_Node;
      Fld : Bare_Assign_Stmt;
Fld_1 : Bare_Name;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Single_Tok_Node;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7887 '<FieldAccess .relative_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10476
--# expr-start 7886 '<as_entity(<FieldAccess .dest>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:10476
--# expr-start 7885 '<FieldAccess .dest>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10476
--# expr-start 7884 '<FieldAccess .assign_statement>' Fld /usr/src/packages/BUILD/ada/ast.py:10476
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7884
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Assign_Stmt_F_Dest;
--# expr-done 7885
Is_Null := Fld_1 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => E_Info, Node => Fld_1); 
   If_Result := As_Entity;
end if;
--# expr-done 7886
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.relative_name'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Relative_Name (Node => If_Result.Node, E_Info => If_Result.Info);
--# end
--# expr-done 7887
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Relative_Name;
--# end
--# property-start TargetName.xref_equation /usr/src/packages/BUILD/ada/ast.py:10480
pragma Warnings (Off, "is not referenced");
 function Target_Name_P_Xref_Equation
  (Node : Bare_Target_Name
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Target_Name := Bare_Target_Name (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_848;
      Fld : Logic_Var;
Fld_1 : Bare_Assign_Stmt;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
Fld_5 : Logic_Var;
Fld_6 : Bare_Assign_Stmt;
Fld_7 : Bare_Name;
Fld_8 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_848 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_848;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7899 '<Block>' Let_Result None
--# scope-start
--# expr-start 7898 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:10481
--# expr-start 7892 '<FieldAccess .type_bind_var(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:10482
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7888 '<FieldAccess .type_var>' Fld /usr/src/packages/BUILD/ada/ast.py:10483
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7888
--# expr-start 7891 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:10483
--# expr-start 7890 '<FieldAccess .dest>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:10483
--# expr-start 7889 '<FieldAccess .assign_statement>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:10483
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld_1 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7889
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Assign_Stmt_F_Dest;
--# expr-done 7890
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7891
--# property-call-start AdaNode.type_bind_var
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld, Right => Fld_3, Origin => Origin);
--# end
--# expr-done 7892
--# expr-start 7897 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:10486
--# expr-start 7893 '<FieldAccess .ref_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:10486
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Self);
--# end
--# expr-done 7893
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 7896 '<FieldAccess .ref_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:10486
--# expr-start 7895 '<FieldAccess .dest>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:10486
--# expr-start 7894 '<FieldAccess .assign_statement>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:10486
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TargetName.assign_statement
Fld_6 := Libadalang.Implementation.Target_Name_P_Assign_Statement (Node => Self);
--# end
--# expr-done 7894
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Fld_6.Assign_Stmt_F_Dest;
--# expr-done 7895
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_8 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_7);
--# end
--# expr-done 7896
Fld_8.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_8.all);
Eq_Node.Refs.Destroy (Fld_8.all);
Bind_Result := Bind_Default_Default.Create (Fld_5, Fld_8, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10486") else null)); 
--# expr-done 7897
And_Pred := Logic_And (Fld_4, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:10481") else null)); 
--# expr-done 7898
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7899
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_848;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_848;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Target_Name_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Paren_Expr
   --
      procedure Initialize_Fields_For_Paren_Expr
        (Self : Bare_Paren_Expr
         ; Paren_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Paren_Expr_F_Expr := Paren_Expr_F_Expr;
      end Initialize_Fields_For_Paren_Expr;
   function Paren_Expr_F_Expr
     (Node : Bare_Paren_Expr) return Bare_Expr
   is
   begin
         return Node.Paren_Expr_F_Expr;
   end;
--# property-start ParenExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:9023
pragma Warnings (Off, "is not referenced");
 function Paren_Expr_P_Xref_Equation
  (Node : Bare_Paren_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Paren_Expr := Bare_Paren_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Paren_Expr :=
     Internal_Entity_Paren_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_774;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Var;
Fld_5 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_774 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_774;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7907 '<Block>' Let_Result None
--# scope-start
--# expr-start 7906 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9025
--# expr-start 7901 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9025
--# expr-start 7900 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:9025
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Paren_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7900
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7901
--# expr-start 7905 '<FieldAccess .type_bind_var(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9026
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7903 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9026
--# expr-start 7902 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9026
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Paren_Expr_F_Expr;
--# expr-done 7902
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 7903
--# expr-start 7904 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9026
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7904
--# property-call-start AdaNode.type_bind_var
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7905
And_Pred := Logic_And (Fld_1, Fld_5, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9025") else null)); 
--# expr-done 7906
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 7907
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_774;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_774;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Paren_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Quantified_Expr
   --
      procedure Initialize_Fields_For_Quantified_Expr
        (Self : Bare_Quantified_Expr
         ; Quantified_Expr_F_Quantifier : Bare_Quantifier
         ; Quantified_Expr_F_Loop_Spec : Bare_For_Loop_Spec
         ; Quantified_Expr_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Quantified_Expr_F_Quantifier := Quantified_Expr_F_Quantifier;
            Self.Quantified_Expr_F_Loop_Spec := Quantified_Expr_F_Loop_Spec;
            Self.Quantified_Expr_F_Expr := Quantified_Expr_F_Expr;
      end Initialize_Fields_For_Quantified_Expr;
   function Quantified_Expr_F_Quantifier
     (Node : Bare_Quantified_Expr) return Bare_Quantifier
   is
   begin
         return Node.Quantified_Expr_F_Quantifier;
   end;
   function Quantified_Expr_F_Loop_Spec
     (Node : Bare_Quantified_Expr) return Bare_For_Loop_Spec
   is
   begin
         return Node.Quantified_Expr_F_Loop_Spec;
   end;
   function Quantified_Expr_F_Expr
     (Node : Bare_Quantified_Expr) return Bare_Expr
   is
   begin
         return Node.Quantified_Expr_F_Expr;
   end;
--# property-start QuantifiedExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:13232
pragma Warnings (Off, "is not referenced");
 function Quantified_Expr_P_Xref_Equation
  (Node : Bare_Quantified_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Quantified_Expr := Bare_Quantified_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Quantified_Expr :=
     Internal_Entity_Quantified_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1014;
      Block_Var : Boolean;
Fld : Internal_Entity_For_Loop_Spec;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Expr;
Fld_5 : Logic_Var;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_7 : Logic_Var;
Fld_8 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1014 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_8);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1014;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7920 '<Block>' Let_Result None
--# scope-start
--# expr-start 7909 '<FieldAccess .resolve_names>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13235
--# expr-start 7908 '<FieldAccess .loop_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:13235
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_For_Loop_Spec (Node => Ent.Node.Quantified_Expr_F_Loop_Spec, Info => Ent.Info);
--# expr-done 7908
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 7909
Block_Var := Fld_1; 
--# bind spec_success Block_Var
--# expr-start 7919 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:13237
if Block_Var then
   --# expr-start 7918 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13239
--# expr-start 7915 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13239
--# expr-start 7911 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13239
--# expr-start 7910 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13239
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7910
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7911
--# expr-start 7914 '<FieldAccess .bool_bind(...)>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13240
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7913 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13240
--# expr-start 7912 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13240
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Expr (Node => Ent.Node.Quantified_Expr_F_Expr, Info => Ent.Info);
--# expr-done 7912
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4.Node);
--# expr-done 7913
--# property-call-start AdaNode.bool_bind
Fld_6 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_5, Origin => Origin);
--# end
--# expr-done 7914
And_Pred := Logic_And (Fld_3, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13239") else null)); 
--# expr-done 7915
--# expr-start 7917 '<FieldAccess .bool_bind(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13241
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7916 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13241
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Ent.Node);
--# expr-done 7916
--# property-call-start AdaNode.bool_bind
Fld_8 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_7, Origin => Origin);
--# end
--# expr-done 7917
And_Pred_1 := Logic_And (And_Pred, Fld_8, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13239") else null)); 
--# expr-done 7918
   If_Result := And_Pred_1;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13242") else null)); 
   If_Result := Logic_False;
end if;
      Inc_Ref (If_Result);
--# expr-done 7919
Let_Result := If_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 7920
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1014;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1014;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Quantified_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Expr
   --
      procedure Initialize_Fields_For_Raise_Expr
        (Self : Bare_Raise_Expr
         ; Raise_Expr_F_Exception_Name : Bare_Name
         ; Raise_Expr_F_Error_Message : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Raise_Expr_F_Exception_Name := Raise_Expr_F_Exception_Name;
            Self.Raise_Expr_F_Error_Message := Raise_Expr_F_Error_Message;
      end Initialize_Fields_For_Raise_Expr;
   function Raise_Expr_F_Exception_Name
     (Node : Bare_Raise_Expr) return Bare_Name
   is
   begin
         return Node.Raise_Expr_F_Exception_Name;
   end;
   function Raise_Expr_F_Error_Message
     (Node : Bare_Raise_Expr) return Bare_Expr
   is
   begin
         return Node.Raise_Expr_F_Error_Message;
   end;
--# property-start RaiseExpr.xref_equation /usr/src/packages/BUILD/ada/ast.py:14021
pragma Warnings (Off, "is not referenced");
 function Raise_Expr_P_Xref_Equation
  (Node : Bare_Raise_Expr
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Expr := Bare_Raise_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Raise_Expr :=
     Internal_Entity_Raise_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1060;
            procedure Finalizer_Scope_2593;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1060 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1060;
            procedure Finalizer_Scope_2593 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2593;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7931 '<Block>' Let_Result None
--# scope-start
--# expr-start 7930 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14022
--# expr-start 7922 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14023
--# expr-start 7921 '<FieldAccess .exception_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14023
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Expr_F_Exception_Name, Info => Ent.Info);
--# expr-done 7921
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7922
--# expr-start 7923 '<FieldAccess .error_message>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14024
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Expr_F_Error_Message, Info => Ent.Info);
--# expr-done 7923
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr
--# expr-start 7929 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14025
--# expr-start 7927 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14028
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7924 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14028
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 7924
--# expr-start 7926 '<FieldAccess .std_entity(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14028
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7925 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 7925
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 7926
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 7927
--# expr-start 7928 '<FieldAccess .sub_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14029
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 7928
And_Pred := Logic_And (Fld_5, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14025") else null)); 
--# expr-done 7929
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2593;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14031") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred_1 := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14022") else null)); 
--# expr-done 7930
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7931
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1060;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1060;
               Finalizer_Scope_2593;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Raise_Expr_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Un_Op
   --
      procedure Initialize_Fields_For_Un_Op
        (Self : Bare_Un_Op
         ; Un_Op_F_Op : Bare_Op
         ; Un_Op_F_Expr : Bare_Expr
        ) is
      begin
            Initialize_Fields_For_Expr
              (Self);
            Self.Un_Op_F_Op := Un_Op_F_Op;
            Self.Un_Op_F_Expr := Un_Op_F_Expr;
      end Initialize_Fields_For_Un_Op;
   function Un_Op_F_Op
     (Node : Bare_Un_Op) return Bare_Op
   is
   begin
         return Node.Un_Op_F_Op;
   end;
   function Un_Op_F_Expr
     (Node : Bare_Un_Op) return Bare_Expr
   is
   begin
         return Node.Un_Op_F_Expr;
   end;
--# property-start UnOp.xref_equation /usr/src/packages/BUILD/ada/ast.py:9039
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Xref_Equation
  (Node : Bare_Un_Op
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Un_Op := Bare_Un_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Un_Op :=
     Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_775;
            procedure Finalizer_Scope_2595;
      Ps : Internal_Single_Formal_Array_Access;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
Fld_4 : Internal_Entity_Op;
Fld_5 : Internal_Entity_Basic_Decl_Array_Access;
Fld_6 : Internal_Entity_Base_Subp_Spec;
Fld_7 : Integer;
Is_Equal : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Fld_8 : Internal_Entity_Base_Subp_Spec;
Fld_9 : Internal_Single_Formal_Array_Access;
Fld_10 : Internal_Entity_Base_Subp_Spec;
Get_Result : Internal_Single_Formal;
Fld_11 : Internal_Entity_Base_Formal_Param_Decl;
Fld_12 : Internal_Entity_Expr;
Fld_13 : Logic_Equation;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Subp_Spec;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_17 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_18 : Bare_Op;
Fld_19 : Logic_Var;
Cast_Expr : Internal_Entity_Basic_Decl;
Cast_Result_1 : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_20 : Logic_Var;
Fld_21 : Bare_Expr;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
Or_Pred : Logic_Equation;
Sym : Symbol_Type;
Fld_24 : Boolean;
Sym_1 : Symbol_Type;
Fld_25 : Boolean;
If_Result : Boolean;
If_Result_1 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_775 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Fld_23);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (And_Pred_2);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_775;
            procedure Finalizer_Scope_2595 is
            begin
                     Dec_Ref (Fld_9);
                     Dec_Ref (Ps);
                     Dec_Ref (Fld_13);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2595;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7972 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 7971 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:9040
--# expr-start 7933 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9040
--# expr-start 7932 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:9040
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7932
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 7933
--# expr-start 7970 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:9040
--# expr-start 7935 '<FieldAccess .in_aspect(...)>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:9041
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7934 '<Symbol Depends>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Depends); 
--# expr-done 7934
--# property-call-start AdaNode.in_aspect
Fld_24 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym);
--# end
--# expr-done 7935
if Fld_24 then
   If_Result := True;
else
   --# expr-start 7937 '<FieldAccess .in_aspect(...)>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:9041
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7936 '<Symbol Refined_Depends>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Refined_Depends); 
--# expr-done 7936
--# property-call-start AdaNode.in_aspect
Fld_25 := Libadalang.Implementation.Ada_Node_P_In_Aspect (Node => Self, Name => Sym_1);
--# end
--# expr-done 7937
   If_Result := Fld_25;
end if;
if If_Result then
   --# expr-start 7939 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:9042
--# expr-start 7938 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9042
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7938
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7939
   If_Result_1 := Fld_3;
else
   --# expr-start 7969 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:9044
--# expr-start 7964 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:9044
--# expr-start 7963 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:9044
--# expr-start 7945 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:9044
--# expr-start 7941 '<FieldAccess .subprograms>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:9044
--# expr-start 7940 '<FieldAccess .op>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:9044
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 7940
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Op.subprograms
Fld_5 := Libadalang.Implementation.Op_P_Subprograms (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 7941
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Fld_5;
      begin
         for Item_241 of
               Collection.Items
         loop
   --# scope-start
               --# bind s Item_241
               --# expr-start 7944 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:9045
--# expr-start 7943 '<FieldAccess .nb_max_params>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:9045
--# expr-start 7942 '<FieldAccess .subp_spec_or_null>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:9045
      if Item_241.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_241.Node, E_Info => Item_241.Info);
--# end
--# expr-done 7942
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_7 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Fld_6.Node);
--# end
--# expr-done 7943
Is_Equal := Fld_7 = 1; 
--# expr-done 7944
               if Is_Equal then
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Item_241;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 7945
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Basic_Decl_Array_Access := Map_Result;
      begin
         for Item_240 of
               Collection.Items
         loop
   --# scope-start
               --# bind subp Item_240
      --# expr-start 7962 '<Let ps>' Let_Result /usr/src/packages/BUILD/ada/ast.py:9046
--# scope-start
--# expr-start 7947 '<FieldAccess .unpacked_formal_params>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:9047
--# expr-start 7946 '<FieldAccess .subp_spec_or_null>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:9047
      if Item_240.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_8 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_240.Node, E_Info => Item_240.Info);
--# end
--# expr-done 7946
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.unpacked_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Unpacked_Formal_Params (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 7947
Ps := Fld_9; Inc_Ref (Ps);
--# bind ps Ps
--# expr-start 7961 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:9051
--# expr-start 7957 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:9051
--# expr-start 7952 '<FieldAccess .call_argument_equation(...)>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:9051
--# expr-start 7948 '<FieldAccess .subp_spec_or_null>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:9051
      if Item_240.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_10 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_240.Node, E_Info => Item_240.Info);
--# end
--# expr-done 7948
      if Fld_10.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7950 '<FieldAccess .spec>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:9052
--# expr-start 7949 '<at(<Var ps>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9052
Get_Result := Get (Ps, 0, True); 
--# expr-done 7949
Fld_11 := Get_Result.Spec;
--# expr-done 7950
--# expr-start 7951 '<FieldAccess .expr>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:9053
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7951
--# property-call-start BaseFormalParamHolder.call_argument_equation
Fld_13 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Call_Argument_Equation (Node => Fld_10.Node, Param => Fld_11, Arg => Fld_12, Env => Env, Origin => Origin, E_Info => Fld_10.Info);
--# end
--# expr-done 7952
--# expr-start 7956 '<FieldAccess .type_bind_val(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:9057
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7953 '<FieldAccess .type_var>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:9057
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7953
--# expr-start 7955 '<FieldAccess .return_type>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:9058
--# expr-start 7954 '<FieldAccess .subp_spec_or_null>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:9058
      if Item_240.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_15 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Item_240.Node, E_Info => Item_240.Info);
--# end
--# expr-done 7954
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_16 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld_15.Node, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 7955
      Cast_Result := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
--# property-call-start AdaNode.type_bind_val
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_14, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 7956
And_Pred := Logic_And (Fld_13, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9051") else null)); 
--# expr-done 7957
--# expr-start 7960 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:9061
--# expr-start 7959 '<FieldAccess .ref_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:9061
--# expr-start 7958 '<FieldAccess .op>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:9061
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Self.Un_Op_F_Op;
--# expr-done 7958
      if Fld_18 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld_18);
--# end
--# expr-done 7959
Fld_19.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_19.all);
Eq_Node.Refs.Destroy (Fld_19.all);
Cast_Expr := Item_240; 
      Cast_Result_1 := Create_Internal_Entity
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
Bind_Result := Bind_Default_Default.Create (Fld_19, Cast_Result_1, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9061") else null)); 
--# expr-done 7960
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9051") else null)); 
--# expr-done 7961
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 7962
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2595;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
end;
--# expr-done 7963
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9044") else null)); 
--# expr-done 7964
--# expr-start 7968 '<FieldAccess .type_bind_var(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:9062
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7965 '<FieldAccess .type_var>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:9062
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Self);
--# expr-done 7965
--# expr-start 7967 '<FieldAccess .type_var>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:9062
--# expr-start 7966 '<FieldAccess .expr>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:9062
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Self.Un_Op_F_Expr;
--# expr-done 7966
      if Fld_21 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_21);
--# expr-done 7967
--# property-call-start AdaNode.type_bind_var
Fld_23 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_20, Right => Fld_22, Origin => Origin);
--# end
--# expr-done 7968
Or_Pred := Logic_Or (Logic_Boolean_Op, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9044") else null)); 
--# expr-done 7969
   If_Result_1 := Or_Pred;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 7970
And_Pred_2 := Logic_And (Fld_1, If_Result_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:9040") else null)); 
--# expr-done 7971
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
--# expr-done 7972
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_775;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_775;
               Finalizer_Scope_2595;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Un_Op_P_Xref_Equation;
--# end
--# property-start UnOp.potential_actuals_for_dispatch /usr/src/packages/BUILD/ada/ast.py:9066
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Potential_Actuals_For_Dispatch
  (Node : Bare_Un_Op
      ; Spec : Internal_Entity_Base_Subp_Spec
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Expected_Type_For_Expr_Array_Access
is
   Self : Bare_Un_Op := Bare_Un_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Un_Op :=
     Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind spec Spec
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_776;
      Fld : Internal_Entity_Param_Spec_Array_Access;
Get_Result : Internal_Entity_Param_Spec;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Expr;
New_Struct : Internal_Expected_Type_For_Expr;
Singleton : Internal_Expected_Type_For_Expr_Array_Access;
Let_Result : Internal_Expected_Type_For_Expr_Array_Access;
            procedure Finalizer_Scope_776 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Singleton);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_776;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 7978 '<Block>' Let_Result None
--# scope-start
--# expr-start 7977 '<New InternalExpectedTypeForExpr>' New_Struct /usr/src/packages/BUILD/ada/ast.py:9067
--# expr-start 7975 '<FieldAccess .type_expression>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9068
--# expr-start 7974 '<at(<FieldAccess .params>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:9068
--# expr-start 7973 '<FieldAccess .params>' Fld /usr/src/packages/BUILD/ada/ast.py:9068
      if Spec.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.params'
Fld := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Params (Node => Spec.Node, E_Info => Spec.Info);
--# end
--# expr-done 7973
Get_Result := Get (Fld, 0, True); 
--# expr-done 7974
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.type_expression'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Type_Expression (Node => Get_Result.Node, E_Info => Get_Result.Info);
--# end
--# expr-done 7975
--# expr-start 7976 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9069
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Un_Op_F_Expr, Info => Ent.Info);
--# expr-done 7976
New_Struct := (Expected_Type => Fld_1, Expr => Fld_2); 
--# expr-done 7977
                Singleton := Create_Internal_Expected_Type_For_Expr_Array (Items_Count => 1);
                Singleton.Items (1) := New_Struct;
Let_Result := Singleton; Inc_Ref (Let_Result);
--# end
--# expr-done 7978
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_776;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_776;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Un_Op_P_Potential_Actuals_For_Dispatch;
--# end
--# property-start UnOp.is_dispatching_call /usr/src/packages/BUILD/ada/ast.py:9073
pragma Warnings (Off, "is not referenced");
 function Un_Op_P_Is_Dispatching_Call
  (Node : Bare_Un_Op
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Un_Op := Bare_Un_Op (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Un_Op :=
     Internal_Entity_Un_Op'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Basic_Decl;
Fld : Internal_Entity_Op;
Fld_1 : Internal_Entity_Basic_Decl;
Fld_2 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 7982 '<Block>' Let_Result None
--# scope-start
--# expr-start 7980 '<FieldAccess .referenced_decl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:9074
--# expr-start 7979 '<FieldAccess .op>' Fld /usr/src/packages/BUILD/ada/ast.py:9074
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Op (Node => Ent.Node.Un_Op_F_Op, Info => Ent.Info);
--# expr-done 7979
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.referenced_decl
Fld_1 := Libadalang.Implementation.Name_P_Referenced_Decl (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 7980
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind decl Var_Expr
--# expr-start 7981 '<FieldAccess .is_dispatching_call_impl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:9075
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_dispatching_call_impl
Fld_2 := Libadalang.Implementation.Expr_P_Is_Dispatching_Call_Impl (Node => Ent.Node, Decl => Var_Expr, Imprecise_Fallback => Imprecise_Fallback, E_Info => Ent.Info);
--# end
--# expr-done 7981
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 7982
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Un_Op_P_Is_Dispatching_Call;
--# end
   --
   --  Primitives for Bare_Handled_Stmts
   --
      procedure Initialize_Fields_For_Handled_Stmts
        (Self : Bare_Handled_Stmts
         ; Handled_Stmts_F_Stmts : Bare_Stmt_List
         ; Handled_Stmts_F_Exceptions : Bare_Ada_Node_List
        ) is
      begin
            Self.Handled_Stmts_F_Stmts := Handled_Stmts_F_Stmts;
            Self.Handled_Stmts_F_Exceptions := Handled_Stmts_F_Exceptions;
      end Initialize_Fields_For_Handled_Stmts;
   function Handled_Stmts_F_Stmts
     (Node : Bare_Handled_Stmts) return Bare_Stmt_List
   is
   begin
         return Node.Handled_Stmts_F_Stmts;
   end;
   function Handled_Stmts_F_Exceptions
     (Node : Bare_Handled_Stmts) return Bare_Ada_Node_List
   is
   begin
         return Node.Handled_Stmts_F_Exceptions;
   end;
   --
   --  Primitives for Bare_Interface_Kind
   --
   --
   --  Primitives for Bare_Interface_Kind_Limited
   --
   --
   --  Primitives for Bare_Interface_Kind_Protected
   --
   --
   --  Primitives for Bare_Interface_Kind_Synchronized
   --
   --
   --  Primitives for Bare_Interface_Kind_Task
   --
   --
   --  Primitives for Bare_Iter_Type
   --
   --
   --  Primitives for Bare_Iter_Type_In
   --
   --
   --  Primitives for Bare_Iter_Type_Of
   --
   --
   --  Primitives for Bare_Library_Item
   --
      procedure Initialize_Fields_For_Library_Item
        (Self : Bare_Library_Item
         ; Library_Item_F_Has_Private : Bare_Private_Node
         ; Library_Item_F_Item : Bare_Basic_Decl
        ) is
      begin
            Self.Library_Item_F_Has_Private := Library_Item_F_Has_Private;
            Self.Library_Item_F_Item := Library_Item_F_Item;
      end Initialize_Fields_For_Library_Item;
   function Library_Item_F_Has_Private
     (Node : Bare_Library_Item) return Bare_Private_Node
   is
   begin
         return Node.Library_Item_F_Has_Private;
   end;
   function Library_Item_F_Item
     (Node : Bare_Library_Item) return Bare_Basic_Decl
   is
   begin
         return Node.Library_Item_F_Item;
   end;
   --
   --  Primitives for Bare_Limited_Node
   --
--# property-start '[dispatcher]Limited.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Limited_Node_P_As_Bool
  (Node : Bare_Limited_Node
  )
   return Boolean
is
   Self : Bare_Limited_Node := Bare_Limited_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Limited_Node (Self.Kind) is
               when Ada_Limited_Absent =>
                  --# property-call-start Limited.Absent.as_bool
                  Property_Result := Limited_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Limited_Present =>
                  --# property-call-start Limited.Present.as_bool
                  Property_Result := Limited_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Limited_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Absent
   --
--# property-start Limited.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3871
pragma Warnings (Off, "is not referenced");
 function Limited_Absent_P_As_Bool
  (Node : Bare_Limited_Absent
  )
   return Boolean
is
   Self : Bare_Limited_Absent := Bare_Limited_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Limited_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Limited_Present
   --
--# property-start Limited.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3871
pragma Warnings (Off, "is not referenced");
 function Limited_Present_P_As_Bool
  (Node : Bare_Limited_Present
  )
   return Boolean
is
   Self : Bare_Limited_Present := Bare_Limited_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Limited_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Loop_Spec
   --
   --
   --  Primitives for Bare_For_Loop_Spec
   --
      procedure Initialize_Fields_For_For_Loop_Spec
        (Self : Bare_For_Loop_Spec
         ; For_Loop_Spec_F_Var_Decl : Bare_For_Loop_Var_Decl
         ; For_Loop_Spec_F_Loop_Type : Bare_Iter_Type
         ; For_Loop_Spec_F_Has_Reverse : Bare_Reverse_Node
         ; For_Loop_Spec_F_Iter_Expr : Bare_Ada_Node
        ) is
      begin
            Self.For_Loop_Spec_F_Var_Decl := For_Loop_Spec_F_Var_Decl;
            Self.For_Loop_Spec_F_Loop_Type := For_Loop_Spec_F_Loop_Type;
            Self.For_Loop_Spec_F_Has_Reverse := For_Loop_Spec_F_Has_Reverse;
            Self.For_Loop_Spec_F_Iter_Expr := For_Loop_Spec_F_Iter_Expr;
      end Initialize_Fields_For_For_Loop_Spec;
   function For_Loop_Spec_F_Var_Decl
     (Node : Bare_For_Loop_Spec) return Bare_For_Loop_Var_Decl
   is
   begin
         return Node.For_Loop_Spec_F_Var_Decl;
   end;
   function For_Loop_Spec_F_Loop_Type
     (Node : Bare_For_Loop_Spec) return Bare_Iter_Type
   is
   begin
         return Node.For_Loop_Spec_F_Loop_Type;
   end;
   function For_Loop_Spec_F_Has_Reverse
     (Node : Bare_For_Loop_Spec) return Bare_Reverse_Node
   is
   begin
         return Node.For_Loop_Spec_F_Has_Reverse;
   end;
   function For_Loop_Spec_F_Iter_Expr
     (Node : Bare_For_Loop_Spec) return Bare_Ada_Node
   is
   begin
         return Node.For_Loop_Spec_F_Iter_Expr;
   end;
--# property-start ForLoopSpec.iter_type /usr/src/packages/BUILD/ada/ast.py:13110
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iter_Type
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1010;
      Block_Var : Logic_Var;
Block_Var_1 : Boolean;
Block_Var_2 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity;
Pred : Logic_Equation;
Fld_3 : Boolean;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
If_Result_1 : Internal_Entity_Base_Type_Decl;
Fld_4 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_5 : Internal_Entity_Base_Type_Decl;
Fld_6 : Boolean;
If_Result_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_1010 is
            begin
                     Dec_Ref (Pred);
            end Finalizer_Scope_1010;
      Mmz_Handle : Memoization_Handle;
      Mmz_Val    : Mmz_Value;
      Mmz_Stored : Boolean;
      --  Whether the memoization couple was actually stored. Used to determine
      --  whether to inc-ref the memoized value.
      function Create_Mmz_Key return Mmz_Key;
      --  Create a memoization key for this property call and return it
      --------------------
      -- Create_Mmz_Key --
      --------------------
      function Create_Mmz_Key return Mmz_Key is
      begin
         return Mmz_K : Mmz_Key :=
           (Property => Mmz_Bare_For_Loop_Spec_For_Loop_Spec_P_Iter_Type,
            Items    => new Mmz_Key_Array (1 ..  4))
         do
            Mmz_K.Items (1) := (Kind => Mmz_Bare_For_Loop_Spec,
                                As_Bare_For_Loop_Spec => Self);
               Mmz_K.Items (2) := (Kind => Mmz_Lexical_Env,
                                      As_Lexical_Env => Env);
                  Inc_Ref (Mmz_K.Items (2).As_Lexical_Env);
               Mmz_K.Items (3) := (Kind => Mmz_Bare_Ada_Node,
                                      As_Bare_Ada_Node => Origin);
               Mmz_K.Items (4) :=
                 (Kind => Mmz_Internal_Entity_Info,
                  As_Internal_Entity_Info => E_Info);
         end return;
      end Create_Mmz_Key;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "property called on null node";
      end if;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         if Find_Memoized_Value
           (Self.Unit, Mmz_Handle, Mmz_Val, Create_Mmz_Key'Access)
         then
            --# memoization-lookup
            if Mmz_Val.Kind = Mmz_Evaluating then
               --# memoization-return
               raise Property_Error with "Infinite recursion detected";
            elsif Mmz_Val.Kind = Mmz_Property_Error then
               --# memoization-return
               raise Property_Error with "Memoized error";
            else
               Property_Result := Mmz_Val.As_Internal_Entity_Base_Type_Decl;
               --# memoization-return
               Exit_Call (Self.Unit.Context, Call_Depth);
               return Property_Result;
            end if;
            --# end
         end if;
      end if;
   --# scope-start
      --# expr-start 7997 '<Block>' Let_Result None
--# scope-start
--# expr-start 7985 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13111
--# expr-start 7984 '<Cast to Expr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13111
--# expr-start 7983 '<FieldAccess .iter_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:13111
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7983
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 7984
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Cast_Result.Node);
--# expr-done 7985
Block_Var := Fld_1; 
--# bind type_var Block_Var
--# expr-start 7988 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13113
--# expr-start 7986 '<FieldAccess .iter_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13113
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7986
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 7987 '<Predicate on AdaNode.is_not_null>' Pred /usr/src/packages/BUILD/ada/ast.py:13115
Block_Var.Value := No_Entity;
Eq_Node.Refs.Reset (Block_Var.all);
Eq_Node.Refs.Destroy (Block_Var.all);
Pred := Ada_Node_P_Is_Not_Null_0_Pred.Create (Block_Var, Create_Ada_Node_P_Is_Not_Null_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Ada_Node.Ada_Node_P_Is_Not_Null")            else null))); 
--# expr-done 7987
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_3 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Fld_2.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 7988
Block_Var_1 := Fld_3; 
--# bind p Block_Var_1
--# expr-start 7991 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:13118
if Block_Var_1 then
   --# expr-start 7990 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:13120
--# expr-start 7989 '<get_value(<Var type_var>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13120
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Block_Var); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Block_Var); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 7989
   if If_Result.Node = null
      or else If_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => If_Result.Node,
         Info => If_Result.Info);
   else
         Cast_Result_1 := No_Entity_Base_Type_Decl;
   end if;
--# expr-done 7990
   If_Result_1 := Cast_Result_1;
else
   If_Result_1 := No_Entity_Base_Type_Decl;
end if;
--# expr-done 7991
Block_Var_2 := If_Result_1; 
--# bind typ Block_Var_2
--# expr-start 7996 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <If>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:13124
--# expr-start 7992 '<FieldAccess .origin_node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13124
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_4 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 7992
Bound_Origin := Fld_4; 
--# expr-start 7995 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:13124
--# expr-start 7993 '<FieldAccess .is_implicit_deref>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13125
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.is_implicit_deref
Fld_6 := Libadalang.Implementation.Base_Type_Decl_P_Is_Implicit_Deref (Node => Block_Var_2.Node, Origin => Bound_Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 7993
if Fld_6 then
   --# expr-start 7994 '<FieldAccess .accessed_type>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13126
      if Block_Var_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Block_Var_2.Node, Origin => Bound_Origin, E_Info => Block_Var_2.Info);
--# end
--# expr-done 7994
   If_Result_2 := Fld_5;
else
   If_Result_2 := Block_Var_2;
end if;
--# expr-done 7995
Dyn_Var_Bind_Result := If_Result_2; 
--# expr-done 7996
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 7997
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1010;
      if not Self.Unit.Context.In_Populate_Lexical_Env then
         Mmz_Val := (Kind => Mmz_Internal_Entity_Base_Type_Decl,
                     As_Internal_Entity_Base_Type_Decl => Property_Result);
         Add_Memoized_Value (Self.Unit, Mmz_Handle, Mmz_Val, Mmz_Stored);
      end if;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1010;
         if Self /= null then
            if not Self.Unit.Context.In_Populate_Lexical_Env then
               Add_Memoized_Value
                 (Self.Unit,
                  Mmz_Handle,
                  (Kind => Mmz_Property_Error),
                  Mmz_Stored);
            end if;
         end if;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Iter_Type;
--# end
--# property-start ForLoopSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:13131
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1011;
            procedure Finalizer_Scope_2597;
            procedure Finalizer_Scope_2598;
            procedure Finalizer_Scope_2599;
            procedure Finalizer_Scope_2600;
            procedure Finalizer_Scope_2601;
            procedure Finalizer_Scope_2602;
            procedure Finalizer_Scope_2603;
            procedure Finalizer_Scope_2604;
      Match_0 : Bare_Iter_Type_In;
Match_1 : Bare_Iter_Type_Of;
Match_0_1 : Internal_Entity_Bin_Op;
Match_1_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Attribute_Ref;
Match_3 : Internal_Entity_Name;
Match_4 : Internal_Entity;
Var_Expr : Internal_Entity_Base_Type_Decl;
It_Typ : Internal_Entity_Base_Type_Decl;
Fld : Bare_Iter_Type;
Fld_1 : Internal_Entity;
Fld_2 : Logic_Equation;
Fld_3 : Logic_Var;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Logic_True : Logic_Equation;
Or_Pred : Logic_Equation;
And_Pred : Logic_Equation;
Fld_6 : Bare_For_Loop_Var_Decl;
Fld_7 : Bare_Defining_Name;
Fld_8 : Logic_Var;
Fld_9 : Logic_Var;
Fld_10 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Fld_11 : Logic_Equation;
Fld_12 : Bare_For_Loop_Var_Decl;
Fld_13 : Bare_Defining_Name;
Fld_14 : Logic_Var;
Fld_15 : Internal_Entity_Base_Type_Decl;
Fld_16 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Fld_17 : Logic_Equation;
And_Pred_2 : Logic_Equation;
Fld_18 : Logic_Equation;
Fld_19 : Bare_For_Loop_Var_Decl;
Fld_20 : Bare_Defining_Name;
Fld_21 : Logic_Var;
Fld_22 : Logic_Var;
Fld_23 : Logic_Equation;
And_Pred_3 : Logic_Equation;
Fld_24 : Internal_Entity_Base_Type_Decl;
Fld_25 : Logic_Equation;
Fld_26 : Bare_For_Loop_Var_Decl;
Fld_27 : Bare_Defining_Name;
Fld_28 : Logic_Var;
Fld_29 : Internal_Entity_Base_Type_Decl;
Cast_Result_1 : Internal_Entity;
Fld_30 : Logic_Equation;
And_Pred_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Fld_31 : Logic_Equation;
Result_Var : Logic_Equation;
Logic_True_1 : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result_2 : Internal_Entity_Bin_Op;
Let_Result : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_4 : Internal_Entity_Attribute_Ref;
Let_Result_2 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Cast_Result_5 : Internal_Entity_Name;
Let_Result_3 : Logic_Equation;
Scope_Result_4 : Logic_Equation;
Cast_Result_6 : Internal_Entity;
Let_Result_4 : Logic_Equation;
Scope_Result_5 : Logic_Equation;
Match_Result : Logic_Equation;
Fld_32 : Internal_Entity_Base_Type_Decl;
Fld_33 : Internal_Entity;
Fld_34 : Logic_Equation;
Fld_35 : Bare_For_Loop_Var_Decl;
Fld_36 : Bare_Defining_Name;
Fld_37 : Logic_Var;
Fld_38 : Internal_Entity_Base_Type_Decl;
Cast_Result_7 : Internal_Entity;
Fld_39 : Logic_Equation;
And_Pred_5 : Logic_Equation;
Logic_True_2 : Logic_Equation;
Fld_40 : Bare_For_Loop_Var_Decl;
Fld_41 : Bare_Defining_Name;
Fld_42 : Logic_Var;
Fld_43 : Internal_Entity_For_Loop_Var_Decl;
Fld_44 : Internal_Entity_Subtype_Indication;
Fld_45 : Internal_Entity_Base_Type_Decl;
Cast_Result_8 : Internal_Entity;
Fld_46 : Logic_Equation;
Fld_47 : Bare_For_Loop_Var_Decl;
Fld_48 : Bare_Subtype_Indication;
Is_Null : Boolean;
If_Result : Logic_Equation;
And_Pred_6 : Logic_Equation;
Logic_False : Logic_Equation;
Fld_49 : Boolean;
If_Result_1 : Logic_Equation;
Let_Result_5 : Logic_Equation;
Match_Prefix_1 : Bare_Iter_Type;
Cast_Result_9 : Bare_Iter_Type_In;
Let_Result_6 : Logic_Equation;
Scope_Result_6 : Logic_Equation;
Cast_Result_10 : Bare_Iter_Type_Of;
Let_Result_7 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Match_Result_1 : Logic_Equation;
Let_Result_8 : Logic_Equation;
            procedure Finalizer_Scope_1011 is
            begin
                     Dec_Ref (Let_Result_6);
                     Dec_Ref (Scope_Result_6);
                     Dec_Ref (Let_Result_7);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Match_Result_1);
                     Dec_Ref (Let_Result_8);
            end Finalizer_Scope_1011;
            procedure Finalizer_Scope_2597 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Let_Result_4);
                     Dec_Ref (Scope_Result_5);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2597;
            procedure Finalizer_Scope_2598 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Fld_10);
                     Dec_Ref (And_Pred_1);
            end Finalizer_Scope_2598;
            procedure Finalizer_Scope_2599 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_17);
                     Dec_Ref (And_Pred_2);
            end Finalizer_Scope_2599;
            procedure Finalizer_Scope_2600 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Fld_23);
                     Dec_Ref (And_Pred_3);
            end Finalizer_Scope_2600;
            procedure Finalizer_Scope_2601 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Fld_31);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2601;
            procedure Finalizer_Scope_2602 is
            begin
                     Dec_Ref (Fld_25);
                     Dec_Ref (Fld_30);
                     Dec_Ref (And_Pred_4);
            end Finalizer_Scope_2602;
            procedure Finalizer_Scope_2603 is
            begin
                     Dec_Ref (Logic_True_1);
            end Finalizer_Scope_2603;
            procedure Finalizer_Scope_2604 is
            begin
                     Dec_Ref (Fld_34);
                     Dec_Ref (Fld_39);
                     Dec_Ref (And_Pred_5);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Fld_46);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred_6);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_5);
            end Finalizer_Scope_2604;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8062 '<Block>' Let_Result_8 None
--# scope-start
--# expr-start 8061 '<Match>' Match_Result_1 /usr/src/packages/BUILD/ada/ast.py:13132
--# expr-start 7998 '<FieldAccess .loop_type>' Fld /usr/src/packages/BUILD/ada/ast.py:13132
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.For_Loop_Spec_F_Loop_Type;
--# expr-done 7998
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix_1 := Fld; 
case Ada_Iter_Type (Match_Prefix_1.Kind) is
         when Ada_Iter_Type_In =>
   --# scope-start
--# scope-start
      Cast_Result_9 := Match_Prefix_1;
Match_0 := Cast_Result_9; 
--# bind _ Match_0
--# expr-start 8036 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:13138
--# expr-start 7999 '<FieldAccess .iter_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13138
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 7999
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Fld_1; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Bin_Op .. Ada_Relation_Op =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Bin_Op
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0_1 := Cast_Result_2; 
--# bind binop Match_0_1
--# expr-start 8011 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:13141
--# expr-start 8005 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:13141
--# expr-start 8000 '<FieldAccess .sub_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13141
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_2 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0_1.Node, Env => Env, Origin => Origin, E_Info => Match_0_1.Info);
--# end
--# expr-done 8000
--# expr-start 8004 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:13144
--# expr-start 8003 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8001 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13144
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 8001
--# expr-start 8002 '<FieldAccess .int_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13144
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.int_type
Fld_4 := Libadalang.Implementation.Ada_Node_P_Int_Type (Node => Self);
--# end
--# expr-done 8002
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 8003
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13145") else null)); 
Or_Pred := Logic_Or (Fld_5, Logic_True, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13144") else null)); 
--# expr-done 8004
And_Pred := Logic_And (Fld_2, Or_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13141") else null)); 
--# expr-done 8005
--# expr-start 8010 '<FieldAccess .type_bind_var(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:13146
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8008 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13146
--# expr-start 8007 '<FieldAccess .id>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13146
--# expr-start 8006 '<FieldAccess .var_decl>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13146
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8006
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Fld_6.For_Loop_Var_Decl_F_Id;
--# expr-done 8007
      if Fld_7 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_7);
--# expr-done 8008
--# expr-start 8009 '<FieldAccess .type_var>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:13147
      if Match_0_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0_1.Node);
--# expr-done 8009
--# property-call-start AdaNode.type_bind_var
Fld_10 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_8, Right => Fld_9, Origin => Origin);
--# end
--# expr-done 8010
And_Pred_1 := Logic_And (And_Pred, Fld_10, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13141") else null)); 
--# expr-done 8011
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
Scope_Result_1 := Let_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2598;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1_1 := Cast_Result_3; 
--# bind t Match_1_1
--# expr-start 8019 '<And>' And_Pred_2 /usr/src/packages/BUILD/ada/ast.py:13152
--# expr-start 8012 '<FieldAccess .sub_equation>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:13152
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_11 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_1_1.Node, Env => Env, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 8012
--# expr-start 8018 '<FieldAccess .type_bind_val(...)>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:13153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8015 '<FieldAccess .type_var>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:13153
--# expr-start 8014 '<FieldAccess .id>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:13153
--# expr-start 8013 '<FieldAccess .var_decl>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:13153
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8013
      if Fld_12 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Fld_12.For_Loop_Var_Decl_F_Id;
--# expr-done 8014
      if Fld_13 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_13);
--# expr-done 8015
--# expr-start 8017 '<FieldAccess .canonical_type>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:13154
--# expr-start 8016 '<FieldAccess .designated_type>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:13154
      if Match_1_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_15 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Match_1_1.Node, Origin => Origin, E_Info => Match_1_1.Info);
--# end
--# expr-done 8016
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_16 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Fld_15.Node, Origin => Origin, E_Info => Fld_15.Info);
--# end
--# expr-done 8017
      Cast_Result := Create_Internal_Entity
        (Node => Fld_16.Node,
         Info => Fld_16.Info);
--# property-call-start AdaNode.type_bind_val
Fld_17 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_14, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 8018
And_Pred_2 := Logic_And (Fld_11, Fld_17, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13152") else null)); 
--# expr-done 8019
Let_Result_1 := And_Pred_2; Inc_Ref (Let_Result_1);
--# end
Scope_Result_2 := Let_Result_1;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2599;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Attribute_Ref .. Ada_Update_Attribute_Ref =>
   --# scope-start
--# scope-start
      Cast_Result_4 := Create_Internal_Entity_Attribute_Ref
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_4; 
--# bind r Match_2
--# expr-start 8026 '<And>' And_Pred_3 /usr/src/packages/BUILD/ada/ast.py:13157
--# expr-start 8020 '<FieldAccess .sub_equation>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:13157
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_2.Node, Env => Env, Origin => Origin, E_Info => Match_2.Info);
--# end
--# expr-done 8020
--# expr-start 8025 '<FieldAccess .type_bind_var(...)>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:13158
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8023 '<FieldAccess .type_var>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:13158
--# expr-start 8022 '<FieldAccess .id>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:13158
--# expr-start 8021 '<FieldAccess .var_decl>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:13158
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8021
      if Fld_19 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Fld_19.For_Loop_Var_Decl_F_Id;
--# expr-done 8022
      if Fld_20 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_21 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_20);
--# expr-done 8023
--# expr-start 8024 '<FieldAccess .type_var>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:13158
      if Match_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_2.Node);
--# expr-done 8024
--# property-call-start AdaNode.type_bind_var
Fld_23 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Var (Node => Self, Left => Fld_21, Right => Fld_22, Origin => Origin);
--# end
--# expr-done 8025
And_Pred_3 := Logic_And (Fld_18, Fld_23, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13157") else null)); 
--# expr-done 8026
Let_Result_2 := And_Pred_3; Inc_Ref (Let_Result_2);
--# end
Scope_Result_3 := Let_Result_2;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2600;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Call_Expr .. Ada_Target_Name =>
   --# scope-start
--# scope-start
      Cast_Result_5 := Create_Internal_Entity_Name
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_5; 
--# bind t Match_3
--# expr-start 8027 '<FieldAccess .name_designated_type>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:13164
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_24 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Match_3.Node, E_Info => Match_3.Info);
--# end
--# expr-done 8027
Var_Expr := Fld_24;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind typ Var_Expr
--# expr-start 8034 '<And>' And_Pred_4 /usr/src/packages/BUILD/ada/ast.py:13166
--# expr-start 8028 '<FieldAccess .sub_equation>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:13166
      if Match_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_25 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_3.Node, Env => Env, Origin => Origin, E_Info => Match_3.Info);
--# end
--# expr-done 8028
--# expr-start 8033 '<FieldAccess .type_bind_val(...)>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:13167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8031 '<FieldAccess .type_var>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:13167
--# expr-start 8030 '<FieldAccess .id>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:13167
--# expr-start 8029 '<FieldAccess .var_decl>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:13167
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8029
      if Fld_26 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_27 := Fld_26.For_Loop_Var_Decl_F_Id;
--# expr-done 8030
      if Fld_27 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_27);
--# expr-done 8031
--# expr-start 8032 '<FieldAccess .canonical_type>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:13168
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_29 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8032
      Cast_Result_1 := Create_Internal_Entity
        (Node => Fld_29.Node,
         Info => Fld_29.Info);
--# property-call-start AdaNode.type_bind_val
Fld_30 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_28, Right => Cast_Result_1, Origin => Origin);
--# end
--# expr-done 8033
And_Pred_4 := Logic_And (Fld_25, Fld_30, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13166") else null)); 
--# expr-done 8034
Scope_Result := And_Pred_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2602;
   Result_Var := Scope_Result;
else
   --# expr-start 8035 '<FieldAccess .iterator_xref_equation>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:13170
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ForLoopSpec.iterator_xref_equation
Fld_31 := Libadalang.Implementation.For_Loop_Spec_P_Iterator_Xref_Equation (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8035
   Result_Var := Fld_31;
end if;
      Inc_Ref (Result_Var);
Let_Result_3 := Result_Var; Inc_Ref (Let_Result_3);
--# end
Scope_Result_4 := Let_Result_3;
   Inc_Ref (Scope_Result_4);
   --# end
      Finalizer_Scope_2601;
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Null_Record_Aggregate | Ada_Box_Expr .. Ada_Membership_Expr | Ada_Paren_Expr .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_6 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_4 := Cast_Result_6; 
--# bind _ Match_4
Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13173") else null)); 
Let_Result_4 := Logic_True_1; Inc_Ref (Let_Result_4);
--# end
Scope_Result_5 := Let_Result_4;
   Inc_Ref (Scope_Result_5);
   --# end
      Finalizer_Scope_2603;
            Match_Result := Scope_Result_5; Inc_Ref (Match_Result);
end case;
--# expr-done 8036
Let_Result_6 := Match_Result; Inc_Ref (Let_Result_6);
--# end
Scope_Result_6 := Let_Result_6;
   Inc_Ref (Scope_Result_6);
   --# end
      Finalizer_Scope_2597;
            Match_Result_1 := Scope_Result_6; Inc_Ref (Match_Result_1);
         when Ada_Iter_Type_Of =>
   --# scope-start
--# scope-start
      Cast_Result_10 := Match_Prefix_1;
Match_1 := Cast_Result_10; 
--# bind _ Match_1
--# expr-start 8060 '<Let it_typ>' Let_Result_5 /usr/src/packages/BUILD/ada/ast.py:13177
--# scope-start
--# expr-start 8037 '<FieldAccess .iter_type>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:13177
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ForLoopSpec.iter_type
Fld_32 := Libadalang.Implementation.For_Loop_Spec_P_Iter_Type (Node => Ent.Node, Env => Env, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8037
It_Typ := Fld_32; 
--# bind it_typ It_Typ
--# expr-start 8059 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:13177
--# expr-start 8038 '<FieldAccess .is_iterable_type>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:13179
      if It_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_iterable_type'
Fld_49 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Iterable_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 8038
if Fld_49 then
   --# expr-start 8058 '<And>' And_Pred_6 /usr/src/packages/BUILD/ada/ast.py:13182
--# expr-start 8046 '<And>' And_Pred_5 /usr/src/packages/BUILD/ada/ast.py:13182
--# expr-start 8040 '<FieldAccess .sub_equation>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:13182
--# expr-start 8039 '<FieldAccess .iter_expr>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:13182
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_33 := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 8039
      if Fld_33.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_34 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_33.Node, Env => Env, Origin => Origin, E_Info => Fld_33.Info);
--# end
--# expr-done 8040
--# expr-start 8045 '<FieldAccess .type_bind_val(...)>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:13186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8043 '<FieldAccess .type_var>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:13186
--# expr-start 8042 '<FieldAccess .id>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:13186
--# expr-start 8041 '<FieldAccess .var_decl>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:13186
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_35 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8041
      if Fld_35 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Fld_35.For_Loop_Var_Decl_F_Id;
--# expr-done 8042
      if Fld_36 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_37 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_36);
--# expr-done 8043
--# expr-start 8044 '<FieldAccess .iterable_comp_type>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:13187
      if It_Typ.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.iterable_comp_type'
Fld_38 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Iterable_Comp_Type (Node => It_Typ.Node, Origin => Origin, E_Info => It_Typ.Info);
--# end
--# expr-done 8044
      Cast_Result_7 := Create_Internal_Entity
        (Node => Fld_38.Node,
         Info => Fld_38.Info);
--# property-call-start AdaNode.type_bind_val
Fld_39 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_37, Right => Cast_Result_7, Origin => Origin);
--# end
--# expr-done 8045
And_Pred_5 := Logic_And (Fld_34, Fld_39, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13182") else null)); 
--# expr-done 8046
--# expr-start 8057 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:13191
--# expr-start 8049 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:13191
--# expr-start 8048 '<FieldAccess .id_type>' Fld_48 /usr/src/packages/BUILD/ada/ast.py:13191
--# expr-start 8047 '<FieldAccess .var_decl>' Fld_47 /usr/src/packages/BUILD/ada/ast.py:13191
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_47 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8047
      if Fld_47 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_48 := Fld_47.For_Loop_Var_Decl_F_Id_Type;
--# expr-done 8048
Is_Null := Fld_48 = null; 
--# expr-done 8049
if Is_Null then
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13192") else null)); 
   If_Result := Logic_True_2;
else
   --# expr-start 8056 '<FieldAccess .type_bind_val(...)>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:13193
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8052 '<FieldAccess .type_var>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:13194
--# expr-start 8051 '<FieldAccess .id>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:13194
--# expr-start 8050 '<FieldAccess .var_decl>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:13194
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8050
      if Fld_40 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_41 := Fld_40.For_Loop_Var_Decl_F_Id;
--# expr-done 8051
      if Fld_41 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_42 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_41);
--# expr-done 8052
--# expr-start 8055 '<FieldAccess .designated_type>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:13195
--# expr-start 8054 '<FieldAccess .id_type>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:13195
--# expr-start 8053 '<FieldAccess .var_decl>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:13195
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_43 := Create_Internal_Entity_For_Loop_Var_Decl (Node => Ent.Node.For_Loop_Spec_F_Var_Decl, Info => Ent.Info);
--# expr-done 8053
      if Fld_43.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Create_Internal_Entity_Subtype_Indication (Node => Fld_43.Node.For_Loop_Var_Decl_F_Id_Type, Info => Fld_43.Info);
--# expr-done 8054
      if Fld_44.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_45 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_44.Node, Origin => Origin, E_Info => Fld_44.Info);
--# end
--# expr-done 8055
      Cast_Result_8 := Create_Internal_Entity
        (Node => Fld_45.Node,
         Info => Fld_45.Info);
--# property-call-start AdaNode.type_bind_val
Fld_46 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_42, Right => Cast_Result_8, Origin => Origin);
--# end
--# expr-done 8056
   If_Result := Fld_46;
end if;
      Inc_Ref (If_Result);
--# expr-done 8057
And_Pred_6 := Logic_And (And_Pred_5, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13182") else null)); 
--# expr-done 8058
   If_Result_1 := And_Pred_6;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13197") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8059
Let_Result_5 := If_Result_1; Inc_Ref (Let_Result_5);
--# end
--# expr-done 8060
Let_Result_7 := Let_Result_5; Inc_Ref (Let_Result_7);
--# end
Scope_Result_7 := Let_Result_7;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2604;
            Match_Result_1 := Scope_Result_7; Inc_Ref (Match_Result_1);
end case;
--# expr-done 8061
Let_Result_8 := Match_Result_1; Inc_Ref (Let_Result_8);
--# end
--# expr-done 8062
      Property_Result := Let_Result_8;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1011;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1011;
               Finalizer_Scope_2597;
               Finalizer_Scope_2598;
               Finalizer_Scope_2599;
               Finalizer_Scope_2600;
               Finalizer_Scope_2601;
               Finalizer_Scope_2602;
               Finalizer_Scope_2603;
               Finalizer_Scope_2604;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Xref_Equation;
--# end
--# property-start ForLoopSpec.iterator_xref_equation /usr/src/packages/BUILD/ada/ast.py:13202
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Iterator_Xref_Equation
  (Node : Bare_For_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_For_Loop_Spec :=
     Internal_Entity_For_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1012;
      Block_Var : Internal_Entity_Expr;
Block_Var_1 : Boolean;
Fld : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Fld_1 : Logic_Var;
Pred : Logic_Equation;
Fld_2 : Boolean;
Fld_3 : Bare_For_Loop_Var_Decl;
Fld_4 : Bare_Defining_Name;
Fld_5 : Logic_Var;
Fld_6 : Logic_Var;
Is_Logic_Var_Defined : Boolean;
Eq_Solution : Internal_Entity;
If_Result : Internal_Entity;
Fld_7 : Lexical_Env := Empty_Env;
Sym : Symbol_Type;
Env_Get_Result : Internal_Entity;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
Cast_Result_2 : Internal_Entity;
Fld_8 : Logic_Equation;
Logic_False : Logic_Equation;
If_Result_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1012 is
            begin
                     Dec_Ref (Pred);
                     Dec_Ref (Fld_7);
                     Dec_Ref (Fld_8);
                     Dec_Ref (Logic_False);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1012;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8079 '<Block>' Let_Result None
--# scope-start
--# expr-start 8064 '<Cast to Expr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:13203
--# expr-start 8063 '<FieldAccess .iter_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:13203
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity (Node => Ent.Node.For_Loop_Spec_F_Iter_Expr, Info => Ent.Info);
--# expr-done 8063
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Expr
   then
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8064
Block_Var := Cast_Result; 
--# bind iter_expr Block_Var
--# expr-start 8067 '<FieldAccess .resolve_names_internal_with_eq(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:13205
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8066 '<Predicate on BaseTypeDecl.is_iterator_type>' Pred /usr/src/packages/BUILD/ada/ast.py:13206
--# expr-start 8065 '<FieldAccess .type_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:13207
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var.Node);
--# expr-done 8065
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
Pred := Base_Type_Decl_P_Is_Iterator_Type_0_Pred.Create (Fld_1, Create_Base_Type_Decl_P_Is_Iterator_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Iterator_Type")            else null))); 
--# expr-done 8066
--# property-call-start AdaNode.resolve_names_internal_with_eq
Fld_2 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal_With_Eq (Node => Block_Var.Node, Additional_Equation => Pred, Env => Env, Origin => Origin, E_Info => Block_Var.Info);
--# end
--# expr-done 8067
Block_Var_1 := Fld_2; 
--# bind p Block_Var_1
--# expr-start 8078 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:13210
if Block_Var_1 then
   --# expr-start 8077 '<FieldAccess .type_bind_val(...)>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:13212
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8070 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:13213
--# expr-start 8069 '<FieldAccess .id>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:13213
--# expr-start 8068 '<FieldAccess .var_decl>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:13213
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Self.For_Loop_Spec_F_Var_Decl;
--# expr-done 8068
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Fld_3.For_Loop_Var_Decl_F_Id;
--# expr-done 8069
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 8070
--# expr-start 8076 '<Cast to BaseTypeDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:13214
--# expr-start 8075 '<EnvGet(<FieldAccess .children_env>, Cursor)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:13214
--# expr-start 8073 '<FieldAccess .children_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:13214
--# expr-start 8072 '<get_value(<FieldAccess .type_var>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:13214
--# expr-start 8071 '<FieldAccess .type_var>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:13214
      if Block_Var.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Block_Var.Node);
--# expr-done 8071
Is_Logic_Var_Defined := Eq_Node.Refs.Is_Defined (Fld_6); 
if Is_Logic_Var_Defined then
Eq_Solution := Eq_Node.Refs.Get_Value (Fld_6); 
   If_Result := Eq_Solution;
else
   If_Result := No_Entity;
end if;
--# expr-done 8072
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Children_Env (Node => If_Result.Node, E_Info => If_Result.Info);
--# expr-done 8073
--# expr-start 8074 '<Symbol Cursor>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Cursor); 
--# expr-done 8074
Env_Get_Result := AST_Envs.Get_First (Self => Fld_7, Key => Sym, Lookup_Kind => To_Lookup_Kind_Type (Recursive), Categories => (Inherited_Primitives => True, Nocat => True)); 
--# expr-done 8075
   if Env_Get_Result.Node = null
      or else Env_Get_Result.Node.Kind in Ada_Base_Type_Decl
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Env_Get_Result.Node,
         Info => Env_Get_Result.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8076
      Cast_Result_2 := Create_Internal_Entity
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
--# property-call-start AdaNode.type_bind_val
Fld_8 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_5, Right => Cast_Result_2, Origin => Origin);
--# end
--# expr-done 8077
   If_Result_1 := Fld_8;
else
   Logic_False := False_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:13217") else null)); 
   If_Result_1 := Logic_False;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8078
Let_Result := If_Result_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8079
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1012;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1012;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Iterator_Xref_Equation;
--# end
--# property-start ForLoopSpec.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:13220
pragma Warnings (Off, "is not referenced");
 function For_Loop_Spec_P_Xref_Entry_Point
  (Node : Bare_For_Loop_Spec
  )
   return Boolean
is
   Self : Bare_For_Loop_Spec := Bare_For_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end For_Loop_Spec_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_While_Loop_Spec
   --
      procedure Initialize_Fields_For_While_Loop_Spec
        (Self : Bare_While_Loop_Spec
         ; While_Loop_Spec_F_Expr : Bare_Expr
        ) is
      begin
            Self.While_Loop_Spec_F_Expr := While_Loop_Spec_F_Expr;
      end Initialize_Fields_For_While_Loop_Spec;
   function While_Loop_Spec_F_Expr
     (Node : Bare_While_Loop_Spec) return Bare_Expr
   is
   begin
         return Node.While_Loop_Spec_F_Expr;
   end;
--# property-start WhileLoopSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:14936
pragma Warnings (Off, "is not referenced");
 function While_Loop_Spec_P_Xref_Equation
  (Node : Bare_While_Loop_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_While_Loop_Spec := Bare_While_Loop_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_While_Loop_Spec :=
     Internal_Entity_While_Loop_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1119;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1119 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1119;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8086 '<Block>' Let_Result None
--# scope-start
--# expr-start 8085 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14937
--# expr-start 8081 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14937
--# expr-start 8080 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14937
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.While_Loop_Spec_F_Expr, Info => Ent.Info);
--# expr-done 8080
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8081
--# expr-start 8084 '<FieldAccess .bool_bind(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14938
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8083 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14938
--# expr-start 8082 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14938
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.While_Loop_Spec_F_Expr;
--# expr-done 8082
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8083
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 8084
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14937") else null)); 
--# expr-done 8085
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8086
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1119;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1119;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end While_Loop_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mode
   --
--# property-start Mode.is_writable /usr/src/packages/BUILD/ada/ast.py:6701
pragma Warnings (Off, "is not referenced");
 function Mode_P_Is_Writable
  (Node : Bare_Mode
  )
   return Boolean
is
   Self : Bare_Mode := Bare_Mode (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Is_A : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8088 '<Block>' Let_Result None
--# scope-start
--# expr-start 8087 '<IsA Mode.Out, Mode.InOut>' Is_A /usr/src/packages/BUILD/ada/ast.py:6706
Is_A := Self /= null 
and then Self.Kind in Ada_Mode_Out_Range | Ada_Mode_In_Out_Range; 
--# expr-done 8087
Let_Result := Is_A; 
--# end
--# expr-done 8088
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mode_P_Is_Writable;
--# end
   --
   --  Primitives for Bare_Mode_Default
   --
   --
   --  Primitives for Bare_Mode_In
   --
   --
   --  Primitives for Bare_Mode_In_Out
   --
   --
   --  Primitives for Bare_Mode_Out
   --
   --
   --  Primitives for Bare_Multi_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Multi_Abstract_State_Decl
        (Self : Bare_Multi_Abstract_State_Decl
         ; Multi_Abstract_State_Decl_F_Decls : Bare_Abstract_State_Decl_List
        ) is
      begin
            Self.Multi_Abstract_State_Decl_F_Decls := Multi_Abstract_State_Decl_F_Decls;
      end Initialize_Fields_For_Multi_Abstract_State_Decl;
   function Multi_Abstract_State_Decl_F_Decls
     (Node : Bare_Multi_Abstract_State_Decl) return Bare_Abstract_State_Decl_List
   is
   begin
         return Node.Multi_Abstract_State_Decl_F_Decls;
   end;
   --
   --  Primitives for Bare_Not_Null
   --
--# property-start '[dispatcher]NotNull.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Not_Null_P_As_Bool
  (Node : Bare_Not_Null
  )
   return Boolean
is
   Self : Bare_Not_Null := Bare_Not_Null (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Not_Null (Self.Kind) is
               when Ada_Not_Null_Absent =>
                  --# property-call-start NotNull.Absent.as_bool
                  Property_Result := Not_Null_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Not_Null_Present =>
                  --# property-call-start NotNull.Present.as_bool
                  Property_Result := Not_Null_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Not_Null_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Absent
   --
--# property-start NotNull.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3895
pragma Warnings (Off, "is not referenced");
 function Not_Null_Absent_P_As_Bool
  (Node : Bare_Not_Null_Absent
  )
   return Boolean
is
   Self : Bare_Not_Null_Absent := Bare_Not_Null_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Not_Null_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Not_Null_Present
   --
--# property-start NotNull.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3895
pragma Warnings (Off, "is not referenced");
 function Not_Null_Present_P_As_Bool
  (Node : Bare_Not_Null_Present
  )
   return Boolean
is
   Self : Bare_Not_Null_Present := Bare_Not_Null_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Not_Null_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Null_Component_Decl
   --
   --
   --  Primitives for Bare_Others_Designator
   --
--# property-start OthersDesignator.xref_equation /usr/src/packages/BUILD/ada/ast.py:11505
pragma Warnings (Off, "is not referenced");
 function Others_Designator_P_Xref_Equation
  (Node : Bare_Others_Designator
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Others_Designator := Bare_Others_Designator (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_900;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_900 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_900;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:11505") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_900;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_900;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Others_Designator_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Overriding_Node
   --
   --
   --  Primitives for Bare_Overriding_Not_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Overriding
   --
   --
   --  Primitives for Bare_Overriding_Unspecified
   --
   --
   --  Primitives for Bare_Params
   --
      procedure Initialize_Fields_For_Params
        (Self : Bare_Params
         ; Params_F_Params : Bare_Param_Spec_List
        ) is
      begin
            Self.Params_F_Params := Params_F_Params;
      end Initialize_Fields_For_Params;
   function Params_F_Params
     (Node : Bare_Params) return Bare_Param_Spec_List
   is
   begin
         return Node.Params_F_Params;
   end;
   --
   --  Primitives for Bare_Paren_Abstract_State_Decl
   --
      procedure Initialize_Fields_For_Paren_Abstract_State_Decl
        (Self : Bare_Paren_Abstract_State_Decl
         ; Paren_Abstract_State_Decl_F_Decl : Bare_Ada_Node
        ) is
      begin
            Self.Paren_Abstract_State_Decl_F_Decl := Paren_Abstract_State_Decl_F_Decl;
      end Initialize_Fields_For_Paren_Abstract_State_Decl;
   function Paren_Abstract_State_Decl_F_Decl
     (Node : Bare_Paren_Abstract_State_Decl) return Bare_Ada_Node
   is
   begin
         return Node.Paren_Abstract_State_Decl_F_Decl;
   end;
   --
   --  Primitives for Bare_Pragma_Node
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Pragma_Node_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Pragma_Node := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_41 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:7099" & ")";
         end if;
      return Initial_Env;
   end Bare_Pragma_Node_Initial_Env_Getter_Fn;
      procedure Pragma_Pre_Env_Actions
        (Self            : Bare_Pragma_Node;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_40 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Pragma_Node_Initial_Env_Getter_Fn'Access);
      end;
      end;
      procedure Initialize_Fields_For_Pragma_Node
        (Self : Bare_Pragma_Node
         ; Pragma_Node_F_Id : Bare_Identifier
         ; Pragma_Node_F_Args : Bare_Base_Assoc_List
        ) is
      begin
            Self.Pragma_Node_F_Id := Pragma_Node_F_Id;
            Self.Pragma_Node_F_Args := Pragma_Node_F_Args;
      end Initialize_Fields_For_Pragma_Node;
   function Pragma_Node_F_Id
     (Node : Bare_Pragma_Node) return Bare_Identifier
   is
   begin
         return Node.Pragma_Node_F_Id;
   end;
   function Pragma_Node_F_Args
     (Node : Bare_Pragma_Node) return Bare_Base_Assoc_List
   is
   begin
         return Node.Pragma_Node_F_Args;
   end;
--# property-start Pragma.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6953
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Entry_Point
  (Node : Bare_Pragma_Node
  )
   return Boolean
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Xref_Entry_Point;
--# end
--# property-start Pragma.xref_equation /usr/src/packages/BUILD/ada/ast.py:6956
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Xref_Equation
  (Node : Bare_Pragma_Node
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_635;
            procedure Finalizer_Scope_2605;
            procedure Finalizer_Scope_2606;
            procedure Finalizer_Scope_2607;
      Expr : Internal_Entity_Expr;
Var_Expr : Internal_Entity_Name;
Fld : Internal_Entity_Identifier;
Sym : Symbol_Type;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Identifier;
Sym_1 : Symbol_Type;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Sym_2 : Symbol_Type;
Fld_5 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Sym_3 : Symbol_Type;
Fld_7 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Fld_12 : Logic_Equation;
Fld_13 : Logic_Var;
Fld_14 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
Fld_15 : Internal_Entity_Identifier;
Sym_4 : Symbol_Type;
Fld_16 : Boolean;
Fld_17 : Internal_Entity_Base_Assoc_List;
Fld_18 : Bare_Base_Assoc_List;
Fld_19 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_4 : Internal_Entity_Base_Assoc;
Fld_20 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_21 : Logic_Equation;
Item_242 : Internal_Entity_Base_Assoc;
Bare_Item_242 : Bare_Base_Assoc;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result_5 : Boolean;
Fld_26 : Internal_Entity_Identifier;
Fld_27 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_6 : Boolean;
Fld_28 : Internal_Entity_Identifier;
Fld_29 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_7 : Boolean;
Fld_30 : Internal_Entity_Identifier;
Fld_31 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_4 : Boolean;
If_Result_8 : Boolean;
Fld_32 : Internal_Entity_Identifier;
Fld_33 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_9 : Boolean;
Fld_34 : Internal_Entity_Identifier;
Fld_35 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_10 : Boolean;
Fld_36 : Internal_Entity_Identifier;
Fld_37 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_11 : Boolean;
Fld_38 : Internal_Entity_Identifier;
Fld_39 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_12 : Boolean;
Fld_40 : Internal_Entity_Identifier;
Fld_41 : Symbol_Type;
Sym_14 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_13 : Boolean;
Fld_42 : Internal_Entity_Name;
Fld_43 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_44 : Internal_Entity_Identifier;
Sym_15 : Symbol_Type;
Fld_45 : Boolean;
Fld_46 : Internal_Entity_Base_Assoc_List;
Fld_47 : Bare_Base_Assoc_List;
Fld_48 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_2 : Internal_Entity_Base_Assoc;
Is_Null_2 : Boolean;
If_Result_14 : Internal_Entity_Base_Assoc;
Fld_49 : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Fld_50 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Fld_51 : Internal_Entity_Base_Assoc_List;
Fld_52 : Bare_Base_Assoc_List;
Len : Integer;
Is_Equal_10 : Boolean;
Fld_53 : Internal_Entity_Base_Assoc_List;
Fld_54 : Bare_Base_Assoc_List;
Fld_55 : Internal_Entity_Info;
Get_Result_2 : Bare_Base_Assoc;
As_Entity_3 : Internal_Entity_Base_Assoc;
Is_Null_3 : Boolean;
If_Result_15 : Internal_Entity_Base_Assoc;
Fld_56 : Internal_Entity_Expr;
Is_A : Boolean;
If_Result_16 : Boolean;
If_Result_17 : Logic_Equation;
Fld_57 : Internal_Entity_Identifier;
Fld_58 : Symbol_Type;
Sym_16 : Symbol_Type;
Is_Equal_11 : Boolean;
Fld_59 : Internal_Entity_Identifier;
Fld_60 : Symbol_Type;
Sym_17 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_18 : Boolean;
Fld_61 : Internal_Entity_Identifier;
Fld_62 : Symbol_Type;
Sym_18 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_19 : Boolean;
Logic_True_2 : Logic_Equation;
Fld_63 : Internal_Entity_Base_Assoc_List;
Fld_64 : Bare_Base_Assoc_List;
Fld_65 : Internal_Entity_Info;
As_Entity_4 : Internal_Entity_Base_Assoc;
Is_Null_4 : Boolean;
If_Result_20 : Internal_Entity_Base_Assoc;
Fld_66 : Internal_Entity_Expr;
Fld_67 : Logic_Equation;
Logic_True_3 : Logic_Equation;
Or_Pred : Logic_Equation;
Item_243 : Internal_Entity_Base_Assoc;
Bare_Item_243 : Bare_Base_Assoc;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
If_Result_21 : Logic_Equation;
If_Result_22 : Logic_Equation;
If_Result_23 : Logic_Equation;
If_Result_24 : Logic_Equation;
If_Result_25 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_635 is
            begin
                     Dec_Ref (Fld_12);
                     Dec_Ref (Fld_14);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_50);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (If_Result_17);
                     Dec_Ref (Logic_True_2);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (If_Result_21);
                     Dec_Ref (If_Result_22);
                     Dec_Ref (If_Result_23);
                     Dec_Ref (If_Result_24);
                     Dec_Ref (If_Result_25);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_635;
            procedure Finalizer_Scope_2605 is
            begin
                     Dec_Ref (Fld_21);
            end Finalizer_Scope_2605;
            procedure Finalizer_Scope_2606 is
            begin
                     Dec_Ref (Fld_43);
            end Finalizer_Scope_2606;
            procedure Finalizer_Scope_2607 is
            begin
                     Dec_Ref (Fld_67);
                     Dec_Ref (Logic_True_3);
                     Dec_Ref (Or_Pred);
            end Finalizer_Scope_2607;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8192 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 8191 '<Cond>' If_Result_25 /usr/src/packages/BUILD/ada/ast.py:6957
--# expr-start 8091 '<FieldAccess .name_is(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6959
--# expr-start 8089 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:6959
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8089
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8090 '<Symbol Assert>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Assert); 
--# expr-done 8090
--# property-call-start Name.name_is
Fld_1 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld.Node, Sym => Sym);
--# end
--# expr-done 8091
if Fld_1 then
   If_Result := True;
else
   --# expr-start 8094 '<FieldAccess .name_is(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6960
--# expr-start 8092 '<FieldAccess .id>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6960
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8092
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8093 '<Symbol Loop_Invariant>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Loop_Invariant); 
--# expr-done 8093
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_2.Node, Sym => Sym_1);
--# end
--# expr-done 8094
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8097 '<FieldAccess .name_is(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6961
--# expr-start 8095 '<FieldAccess .id>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6961
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8095
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8096 '<Symbol Compile_Time_Warning>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Compile_Time_Warning); 
--# expr-done 8096
--# property-call-start Name.name_is
Fld_5 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_4.Node, Sym => Sym_2);
--# end
--# expr-done 8097
   If_Result_1 := Fld_5;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8100 '<FieldAccess .name_is(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:6962
--# expr-start 8098 '<FieldAccess .id>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:6962
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8098
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8099 '<Symbol Compile_Time_Error>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Compile_Time_Error); 
--# expr-done 8099
--# property-call-start Name.name_is
Fld_7 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_6.Node, Sym => Sym_3);
--# end
--# expr-done 8100
   If_Result_2 := Fld_7;
end if;
if If_Result_2 then
   --# expr-start 8107 '<Let expr>' Let_Result /usr/src/packages/BUILD/ada/ast.py:6964
--# scope-start
--# expr-start 8102 '<FieldAccess .assoc_expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:6964
--# expr-start 8101 '<FieldAccess .args>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:6964
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8101
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 0, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 8102
Expr := Fld_11; 
--# bind expr Expr
--# expr-start 8106 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:6965
--# expr-start 8103 '<FieldAccess .sub_equation>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:6965
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Expr.Node, Env => Env, Origin => Origin, E_Info => Expr.Info);
--# end
--# expr-done 8103
--# expr-start 8105 '<FieldAccess .bool_bind(...)>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:6965
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8104 '<FieldAccess .type_var>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:6965
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_13 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Expr.Node);
--# expr-done 8104
--# property-call-start AdaNode.bool_bind
Fld_14 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_13, Origin => Origin);
--# end
--# expr-done 8105
And_Pred := Logic_And (Fld_12, Fld_14, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6965") else null)); 
--# expr-done 8106
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8107
   If_Result_25 := Let_Result;
else
--# expr-start 8110 '<FieldAccess .name_is(...)>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:6967
--# expr-start 8108 '<FieldAccess .id>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:6967
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_15 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8108
      if Fld_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8109 '<Symbol Unreferenced>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unreferenced); 
--# expr-done 8109
--# property-call-start Name.name_is
Fld_16 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_15.Node, Sym => Sym_4);
--# end
--# expr-done 8110
if Fld_16 then
   --# expr-start 8116 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6968
--# expr-start 8115 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6968
--# expr-start 8111 '<FieldAccess .args>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:6968
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_17 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8111
Fld_18 := Fld_17.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_18 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_18;
      begin
         for Untyped_Item_242 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_242 := Untyped_Item_242; 
Is_Null_1 := Bare_Item_242 = null; 
if Is_Null_1 then
   If_Result_4 := No_Entity_Base_Assoc;
else
Fld_19 := Fld_17.Info;
As_Entity_1 := (Info => Fld_19, Node => Bare_Item_242); 
   If_Result_4 := As_Entity_1;
end if;
                  Item_242 := If_Result_4; 
   --# scope-start
               --# bind assoc Item_242
      --# expr-start 8114 '<FieldAccess .xref_no_overloading>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:6970
--# expr-start 8113 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6970
--# expr-start 8112 '<FieldAccess .assoc_expr>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:6970
      if Item_242.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_20 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_242.Node, E_Info => Item_242.Info);
--# end
--# expr-done 8112
   if Fld_20.Node = null
      or else Fld_20.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_20.Node,
         Info => Fld_20.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8113
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_21 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Cast_Result.Node, Env => Env, Origin => Origin, E_Info => Cast_Result.Info);
--# end
--# expr-done 8114
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_21;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2605;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8115
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6968") else null)); 
--# expr-done 8116
   If_Result_24 := Logic_Boolean_Op;
else
--# expr-start 8120 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8118 '<FieldAccess .name_symbol>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8117 '<FieldAccess .id>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8117
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_22.Node);
--# end
--# expr-done 8118
--# expr-start 8119 '<Symbol Import>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 8119
Is_Equal := Fld_23 = Sym_5; 
--# expr-done 8120
if Is_Equal then
   If_Result_5 := True;
else
   --# expr-start 8124 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8122 '<FieldAccess .name_symbol>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8121 '<FieldAccess .id>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8121
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 8122
--# expr-start 8123 '<Symbol Export>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Export); 
--# expr-done 8123
Is_Equal_1 := Fld_25 = Sym_6; 
--# expr-done 8124
   If_Result_5 := Is_Equal_1;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 8128 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8126 '<FieldAccess .name_symbol>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8125 '<FieldAccess .id>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8125
      if Fld_26.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_26.Node);
--# end
--# expr-done 8126
--# expr-start 8127 '<Symbol Interface>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 8127
Is_Equal_2 := Fld_27 = Sym_7; 
--# expr-done 8128
   If_Result_6 := Is_Equal_2;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 8132 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8130 '<FieldAccess .name_symbol>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8129 '<FieldAccess .id>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8129
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_28.Node);
--# end
--# expr-done 8130
--# expr-start 8131 '<Symbol Convention>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 8131
Is_Equal_3 := Fld_29 = Sym_8; 
--# expr-done 8132
   If_Result_7 := Is_Equal_3;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 8136 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8134 '<FieldAccess .name_symbol>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8133 '<FieldAccess .id>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8133
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 8134
--# expr-start 8135 '<Symbol Pack>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pack); 
--# expr-done 8135
Is_Equal_4 := Fld_31 = Sym_9; 
--# expr-done 8136
   If_Result_8 := Is_Equal_4;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 8140 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8138 '<FieldAccess .name_symbol>' Fld_33 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8137 '<FieldAccess .id>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8137
      if Fld_32.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_33 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_32.Node);
--# end
--# expr-done 8138
--# expr-start 8139 '<Symbol Pure>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 8139
Is_Equal_5 := Fld_33 = Sym_10; 
--# expr-done 8140
   If_Result_9 := Is_Equal_5;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 8144 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8142 '<FieldAccess .name_symbol>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8141 '<FieldAccess .id>' Fld_34 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_34 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8141
      if Fld_34.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_35 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_34.Node);
--# end
--# expr-done 8142
--# expr-start 8143 '<Symbol Preelaborate>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 8143
Is_Equal_6 := Fld_35 = Sym_11; 
--# expr-done 8144
   If_Result_10 := Is_Equal_6;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 8148 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8146 '<FieldAccess .name_symbol>' Fld_37 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8145 '<FieldAccess .id>' Fld_36 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_36 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8145
      if Fld_36.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_37 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_36.Node);
--# end
--# expr-done 8146
--# expr-start 8147 '<Symbol Elaborate_Body>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elaborate_Body); 
--# expr-done 8147
Is_Equal_7 := Fld_37 = Sym_12; 
--# expr-done 8148
   If_Result_11 := Is_Equal_7;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 8152 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8150 '<FieldAccess .name_symbol>' Fld_39 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8149 '<FieldAccess .id>' Fld_38 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_38 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8149
      if Fld_38.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_39 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_38.Node);
--# end
--# expr-done 8150
--# expr-start 8151 '<Symbol Inline>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Inline); 
--# expr-done 8151
Is_Equal_8 := Fld_39 = Sym_13; 
--# expr-done 8152
   If_Result_12 := Is_Equal_8;
end if;
if If_Result_12 then
   If_Result_13 := True;
else
   --# expr-start 8156 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8154 '<FieldAccess .name_symbol>' Fld_41 /usr/src/packages/BUILD/ada/ast.py:6973
--# expr-start 8153 '<FieldAccess .id>' Fld_40 /usr/src/packages/BUILD/ada/ast.py:6973
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_40 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8153
      if Fld_40.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_41 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_40.Node);
--# end
--# expr-done 8154
--# expr-start 8155 '<Symbol Volatile>' Sym_14 None
Sym_14 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Volatile); 
--# expr-done 8155
Is_Equal_9 := Fld_41 = Sym_14; 
--# expr-done 8156
   If_Result_13 := Is_Equal_9;
end if;
if If_Result_13 then
--# expr-start 8157 '<FieldAccess .associated_entity_name>' Fld_42 /usr/src/packages/BUILD/ada/ast.py:6978
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld_42 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8157
Var_Expr := Fld_42;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind n Var_Expr
--# expr-start 8158 '<FieldAccess .xref_no_overloading>' Fld_43 /usr/src/packages/BUILD/ada/ast.py:6979
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_43 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8158
Scope_Result := Fld_43;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2606;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6979") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
   If_Result_23 := Result_Var;
else
--# expr-start 8161 '<FieldAccess .name_is(...)>' Fld_45 /usr/src/packages/BUILD/ada/ast.py:6982
--# expr-start 8159 '<FieldAccess .id>' Fld_44 /usr/src/packages/BUILD/ada/ast.py:6982
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_44 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8159
      if Fld_44.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8160 '<Symbol Warnings>' Sym_15 None
Sym_15 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Warnings); 
--# expr-done 8160
--# property-call-start Name.name_is
Fld_45 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_44.Node, Sym => Sym_15);
--# end
--# expr-done 8161
if Fld_45 then
   --# expr-start 8172 '<If>' If_Result_17 /usr/src/packages/BUILD/ada/ast.py:6983
--# expr-start 8164 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:6983
--# expr-start 8163 '<length(<FieldAccess .args>)>' Len /usr/src/packages/BUILD/ada/ast.py:6983
--# expr-start 8162 '<FieldAccess .args>' Fld_51 /usr/src/packages/BUILD/ada/ast.py:6983
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_51 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8162
Fld_52 := Fld_51.Node; 
Len := Length (Fld_52); 
--# expr-done 8163
Is_Equal_10 := Len = 2; 
--# expr-done 8164
if Is_Equal_10 then
   --# expr-start 8167 '<IsA BaseId>' Is_A /usr/src/packages/BUILD/ada/ast.py:6984
--# expr-start 8166 '<FieldAccess .assoc_expr>' Fld_56 /usr/src/packages/BUILD/ada/ast.py:6984
--# expr-start 8165 '<FieldAccess .args>' Fld_53 /usr/src/packages/BUILD/ada/ast.py:6984
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_53 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8165
Fld_54 := Fld_53.Node;
Get_Result_2 := Get (Fld_54, 1, True); 
Is_Null_3 := Get_Result_2 = null; 
if Is_Null_3 then
   If_Result_15 := No_Entity_Base_Assoc;
else
Fld_55 := Fld_53.Info;
As_Entity_3 := (Info => Fld_55, Node => Get_Result_2); 
   If_Result_15 := As_Entity_3;
end if;
      if If_Result_15.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_56 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_15.Node, E_Info => If_Result_15.Info);
--# end
--# expr-done 8166
Is_A := Fld_56.Node /= null 
and then Fld_56.Node.Kind in Ada_Base_Id; 
--# expr-done 8167
   If_Result_16 := Is_A;
else
   If_Result_16 := False;
end if;
if If_Result_16 then
   --# expr-start 8171 '<FieldAccess .sub_equation>' Fld_50 /usr/src/packages/BUILD/ada/ast.py:6985
--# expr-start 8170 '<Cast to BaseId>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:6985
--# expr-start 8169 '<FieldAccess .assoc_expr>' Fld_49 /usr/src/packages/BUILD/ada/ast.py:6985
--# expr-start 8168 '<FieldAccess .args>' Fld_46 /usr/src/packages/BUILD/ada/ast.py:6985
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_46 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8168
Fld_47 := Fld_46.Node;
Get_Result_1 := Get (Fld_47, 1, True); 
Is_Null_2 := Get_Result_1 = null; 
if Is_Null_2 then
   If_Result_14 := No_Entity_Base_Assoc;
else
Fld_48 := Fld_46.Info;
As_Entity_2 := (Info => Fld_48, Node => Get_Result_1); 
   If_Result_14 := As_Entity_2;
end if;
      if If_Result_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_49 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_14.Node, E_Info => If_Result_14.Info);
--# end
--# expr-done 8169
   if Fld_49.Node = null
      or else Fld_49.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Fld_49.Node,
         Info => Fld_49.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 8170
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_50 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Cast_Result_1.Node, Env => Env, Origin => Origin, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8171
   If_Result_17 := Fld_50;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6986") else null)); 
   If_Result_17 := Logic_True_1;
end if;
      Inc_Ref (If_Result_17);
--# expr-done 8172
   If_Result_22 := If_Result_17;
else
--# expr-start 8176 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8174 '<FieldAccess .name_symbol>' Fld_58 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8173 '<FieldAccess .id>' Fld_57 /usr/src/packages/BUILD/ada/ast.py:6993
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_57 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8173
      if Fld_57.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_58 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_57.Node);
--# end
--# expr-done 8174
--# expr-start 8175 '<Symbol Style_Checks>' Sym_16 None
Sym_16 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Style_Checks); 
--# expr-done 8175
Is_Equal_11 := Fld_58 = Sym_16; 
--# expr-done 8176
if Is_Equal_11 then
   If_Result_18 := True;
else
   --# expr-start 8180 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8178 '<FieldAccess .name_symbol>' Fld_60 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8177 '<FieldAccess .id>' Fld_59 /usr/src/packages/BUILD/ada/ast.py:6993
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_59 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8177
      if Fld_59.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_60 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_59.Node);
--# end
--# expr-done 8178
--# expr-start 8179 '<Symbol Import_Function>' Sym_17 None
Sym_17 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import_Function); 
--# expr-done 8179
Is_Equal_12 := Fld_60 = Sym_17; 
--# expr-done 8180
   If_Result_18 := Is_Equal_12;
end if;
if If_Result_18 then
   If_Result_19 := True;
else
   --# expr-start 8184 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8182 '<FieldAccess .name_symbol>' Fld_62 /usr/src/packages/BUILD/ada/ast.py:6993
--# expr-start 8181 '<FieldAccess .id>' Fld_61 /usr/src/packages/BUILD/ada/ast.py:6993
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_61 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8181
      if Fld_61.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_62 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_61.Node);
--# end
--# expr-done 8182
--# expr-start 8183 '<Symbol Import_Procedure>' Sym_18 None
Sym_18 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import_Procedure); 
--# expr-done 8183
Is_Equal_13 := Fld_62 = Sym_18; 
--# expr-done 8184
   If_Result_19 := Is_Equal_13;
end if;
if If_Result_19 then
   Logic_True_2 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6996") else null)); 
   If_Result_21 := Logic_True_2;
else
   --# expr-start 8190 '<LogicAll>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:6998
--# expr-start 8189 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:6998
--# expr-start 8185 '<FieldAccess .args>' Fld_63 /usr/src/packages/BUILD/ada/ast.py:6998
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_63 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8185
Fld_64 := Fld_63.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_64 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Base_Assoc_List := Fld_64;
      begin
         for Untyped_Item_243 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_243 := Untyped_Item_243; 
Is_Null_4 := Bare_Item_243 = null; 
if Is_Null_4 then
   If_Result_20 := No_Entity_Base_Assoc;
else
Fld_65 := Fld_63.Info;
As_Entity_4 := (Info => Fld_65, Node => Bare_Item_243); 
   If_Result_20 := As_Entity_4;
end if;
                  Item_243 := If_Result_20; 
   --# scope-start
               --# bind a Item_243
      --# expr-start 8188 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:7002
--# expr-start 8187 '<FieldAccess .sub_equation>' Fld_67 /usr/src/packages/BUILD/ada/ast.py:7002
--# expr-start 8186 '<FieldAccess .assoc_expr>' Fld_66 /usr/src/packages/BUILD/ada/ast.py:7002
      if Item_243.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_66 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Item_243.Node, E_Info => Item_243.Info);
--# end
--# expr-done 8186
      if Fld_66.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_67 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_66.Node, Env => Env, Origin => Origin, E_Info => Fld_66.Info);
--# end
--# expr-done 8187
Logic_True_3 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7002") else null)); 
Or_Pred := Logic_Or (Fld_67, Logic_True_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:7002") else null)); 
--# expr-done 8188
         declare
            Item_To_Append : constant Logic_Equation :=
               Or_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2607;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8189
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6998") else null)); 
--# expr-done 8190
   If_Result_21 := Logic_Boolean_Op_1;
end if;
      Inc_Ref (If_Result_21);
   If_Result_22 := If_Result_21;
end if;
      Inc_Ref (If_Result_22);
   If_Result_23 := If_Result_22;
end if;
      Inc_Ref (If_Result_23);
   If_Result_24 := If_Result_23;
end if;
      Inc_Ref (If_Result_24);
   If_Result_25 := If_Result_24;
end if;
      Inc_Ref (If_Result_25);
--# expr-done 8191
Let_Result_1 := If_Result_25; Inc_Ref (Let_Result_1);
--# end
--# expr-done 8192
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_635;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_635;
               Finalizer_Scope_2605;
               Finalizer_Scope_2606;
               Finalizer_Scope_2607;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Xref_Equation;
--# end
--# property-start Pragma.associated_entity_name /usr/src/packages/BUILD/ada/ast.py:7007
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Entity_Name
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Base_Assoc;
Fld : Internal_Entity_Identifier;
Fld_1 : Symbol_Type;
Sym : Symbol_Type;
Is_Equal : Boolean;
Fld_2 : Internal_Entity_Identifier;
Fld_3 : Symbol_Type;
Sym_1 : Symbol_Type;
Is_Equal_1 : Boolean;
If_Result : Boolean;
Fld_4 : Internal_Entity_Identifier;
Fld_5 : Symbol_Type;
Sym_2 : Symbol_Type;
Is_Equal_2 : Boolean;
If_Result_1 : Boolean;
Fld_6 : Internal_Entity_Identifier;
Fld_7 : Symbol_Type;
Sym_3 : Symbol_Type;
Is_Equal_3 : Boolean;
If_Result_2 : Boolean;
Fld_8 : Internal_Entity_Base_Assoc_List;
Fld_9 : Bare_Base_Assoc_List;
Fld_10 : Internal_Entity_Info;
Get_Result : Bare_Base_Assoc;
As_Entity : Internal_Entity_Base_Assoc;
Is_Null : Boolean;
If_Result_3 : Internal_Entity_Base_Assoc;
Fld_11 : Internal_Entity_Expr;
Cast_Result : Internal_Entity_Name;
Fld_12 : Internal_Entity_Identifier;
Fld_13 : Symbol_Type;
Sym_4 : Symbol_Type;
Is_Equal_4 : Boolean;
Fld_14 : Internal_Entity_Identifier;
Fld_15 : Symbol_Type;
Sym_5 : Symbol_Type;
Is_Equal_5 : Boolean;
If_Result_4 : Boolean;
Fld_16 : Internal_Entity_Identifier;
Fld_17 : Symbol_Type;
Sym_6 : Symbol_Type;
Is_Equal_6 : Boolean;
If_Result_5 : Boolean;
Fld_18 : Internal_Entity_Identifier;
Fld_19 : Symbol_Type;
Sym_7 : Symbol_Type;
Is_Equal_7 : Boolean;
If_Result_6 : Boolean;
Fld_20 : Internal_Entity_Identifier;
Fld_21 : Symbol_Type;
Sym_8 : Symbol_Type;
Is_Equal_8 : Boolean;
If_Result_7 : Boolean;
Fld_22 : Internal_Entity_Identifier;
Fld_23 : Symbol_Type;
Sym_9 : Symbol_Type;
Is_Equal_9 : Boolean;
If_Result_8 : Boolean;
Fld_24 : Internal_Entity_Identifier;
Fld_25 : Symbol_Type;
Sym_10 : Symbol_Type;
Is_Equal_10 : Boolean;
If_Result_9 : Boolean;
Fld_26 : Internal_Entity_Identifier;
Fld_27 : Symbol_Type;
Sym_11 : Symbol_Type;
Is_Equal_11 : Boolean;
If_Result_10 : Boolean;
Fld_28 : Internal_Entity_Identifier;
Fld_29 : Symbol_Type;
Sym_12 : Symbol_Type;
Is_Equal_12 : Boolean;
If_Result_11 : Boolean;
Fld_30 : Internal_Entity_Identifier;
Fld_31 : Symbol_Type;
Sym_13 : Symbol_Type;
Is_Equal_13 : Boolean;
If_Result_12 : Boolean;
Fld_32 : Internal_Entity_Base_Assoc_List;
Fld_33 : Bare_Base_Assoc_List;
Fld_34 : Internal_Entity_Info;
Get_Result_1 : Bare_Base_Assoc;
As_Entity_1 : Internal_Entity_Base_Assoc;
Is_Null_1 : Boolean;
If_Result_13 : Internal_Entity_Base_Assoc;
Fld_35 : Internal_Entity_Expr;
Scope_Result : Internal_Entity_Expr;
Result_Var : Internal_Entity_Expr;
Cast_Result_1 : Internal_Entity_Base_Id;
Cast_Expr : Internal_Entity_Base_Id;
Cast_Result_2 : Internal_Entity_Name;
Cast_Result_3 : Internal_Entity_Name;
If_Result_14 : Internal_Entity_Name;
If_Result_15 : Internal_Entity_Name;
Let_Result : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8256 '<Block>' Let_Result None
--# scope-start
--# expr-start 8255 '<Cond>' If_Result_15 /usr/src/packages/BUILD/ada/ast.py:7008
--# expr-start 8196 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8194 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8193 '<FieldAccess .id>' Fld /usr/src/packages/BUILD/ada/ast.py:7009
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8193
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld.Node);
--# end
--# expr-done 8194
--# expr-start 8195 '<Symbol Import>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Import); 
--# expr-done 8195
Is_Equal := Fld_1 = Sym; 
--# expr-done 8196
if Is_Equal then
   If_Result := True;
else
   --# expr-start 8200 '<Eq>' Is_Equal_1 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8198 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8197 '<FieldAccess .id>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7009
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8197
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_2.Node);
--# end
--# expr-done 8198
--# expr-start 8199 '<Symbol Export>' Sym_1 None
Sym_1 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Export); 
--# expr-done 8199
Is_Equal_1 := Fld_3 = Sym_1; 
--# expr-done 8200
   If_Result := Is_Equal_1;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8204 '<Eq>' Is_Equal_2 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8202 '<FieldAccess .name_symbol>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8201 '<FieldAccess .id>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7009
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8201
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_5 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_4.Node);
--# end
--# expr-done 8202
--# expr-start 8203 '<Symbol Interface>' Sym_2 None
Sym_2 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Interface); 
--# expr-done 8203
Is_Equal_2 := Fld_5 = Sym_2; 
--# expr-done 8204
   If_Result_1 := Is_Equal_2;
end if;
if If_Result_1 then
   If_Result_2 := True;
else
   --# expr-start 8208 '<Eq>' Is_Equal_3 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8206 '<FieldAccess .name_symbol>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7009
--# expr-start 8205 '<FieldAccess .id>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7009
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8205
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_7 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_6.Node);
--# end
--# expr-done 8206
--# expr-start 8207 '<Symbol Convention>' Sym_3 None
Sym_3 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Convention); 
--# expr-done 8207
Is_Equal_3 := Fld_7 = Sym_3; 
--# expr-done 8208
   If_Result_2 := Is_Equal_3;
end if;
if If_Result_2 then
   --# expr-start 8211 '<Cast to Name>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7012
--# expr-start 8210 '<FieldAccess .assoc_expr>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:7012
--# expr-start 8209 '<FieldAccess .args>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:7012
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8209
Fld_9 := Fld_8.Node;
Get_Result := Get (Fld_9, 1, True); 
Is_Null := Get_Result = null; 
if Is_Null then
   If_Result_3 := No_Entity_Base_Assoc;
else
Fld_10 := Fld_8.Info;
As_Entity := (Info => Fld_10, Node => Get_Result); 
   If_Result_3 := As_Entity;
end if;
      if If_Result_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_11 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => If_Result_3.Node, E_Info => If_Result_3.Info);
--# end
--# expr-done 8210
   if Fld_11.Node = null
      or else Fld_11.Node.Kind in Ada_Name
   then
      Cast_Result := Create_Internal_Entity_Name
        (Node => Fld_11.Node,
         Info => Fld_11.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8211
   If_Result_15 := Cast_Result;
else
--# expr-start 8215 '<Eq>' Is_Equal_4 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8213 '<FieldAccess .name_symbol>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8212 '<FieldAccess .id>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8212
      if Fld_12.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_13 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_12.Node);
--# end
--# expr-done 8213
--# expr-start 8214 '<Symbol Pack>' Sym_4 None
Sym_4 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pack); 
--# expr-done 8214
Is_Equal_4 := Fld_13 = Sym_4; 
--# expr-done 8215
if Is_Equal_4 then
   If_Result_4 := True;
else
   --# expr-start 8219 '<Eq>' Is_Equal_5 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8217 '<FieldAccess .name_symbol>' Fld_15 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8216 '<FieldAccess .id>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_14 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8216
      if Fld_14.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_15 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_14.Node);
--# end
--# expr-done 8217
--# expr-start 8218 '<Symbol Pure>' Sym_5 None
Sym_5 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Pure); 
--# expr-done 8218
Is_Equal_5 := Fld_15 = Sym_5; 
--# expr-done 8219
   If_Result_4 := Is_Equal_5;
end if;
if If_Result_4 then
   If_Result_5 := True;
else
   --# expr-start 8223 '<Eq>' Is_Equal_6 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8221 '<FieldAccess .name_symbol>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8220 '<FieldAccess .id>' Fld_16 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_16 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8220
      if Fld_16.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_17 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_16.Node);
--# end
--# expr-done 8221
--# expr-start 8222 '<Symbol Preelaborate>' Sym_6 None
Sym_6 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Preelaborate); 
--# expr-done 8222
Is_Equal_6 := Fld_17 = Sym_6; 
--# expr-done 8223
   If_Result_5 := Is_Equal_6;
end if;
if If_Result_5 then
   If_Result_6 := True;
else
   --# expr-start 8227 '<Eq>' Is_Equal_7 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8225 '<FieldAccess .name_symbol>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8224 '<FieldAccess .id>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_18 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8224
      if Fld_18.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_19 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_18.Node);
--# end
--# expr-done 8225
--# expr-start 8226 '<Symbol Elaborate_Body>' Sym_7 None
Sym_7 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Elaborate_Body); 
--# expr-done 8226
Is_Equal_7 := Fld_19 = Sym_7; 
--# expr-done 8227
   If_Result_6 := Is_Equal_7;
end if;
if If_Result_6 then
   If_Result_7 := True;
else
   --# expr-start 8231 '<Eq>' Is_Equal_8 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8229 '<FieldAccess .name_symbol>' Fld_21 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8228 '<FieldAccess .id>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_20 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8228
      if Fld_20.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_21 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_20.Node);
--# end
--# expr-done 8229
--# expr-start 8230 '<Symbol Inline>' Sym_8 None
Sym_8 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Inline); 
--# expr-done 8230
Is_Equal_8 := Fld_21 = Sym_8; 
--# expr-done 8231
   If_Result_7 := Is_Equal_8;
end if;
if If_Result_7 then
   If_Result_8 := True;
else
   --# expr-start 8235 '<Eq>' Is_Equal_9 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8233 '<FieldAccess .name_symbol>' Fld_23 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8232 '<FieldAccess .id>' Fld_22 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_22 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8232
      if Fld_22.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_23 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_22.Node);
--# end
--# expr-done 8233
--# expr-start 8234 '<Symbol Volatile>' Sym_9 None
Sym_9 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Volatile); 
--# expr-done 8234
Is_Equal_9 := Fld_23 = Sym_9; 
--# expr-done 8235
   If_Result_8 := Is_Equal_9;
end if;
if If_Result_8 then
   If_Result_9 := True;
else
   --# expr-start 8239 '<Eq>' Is_Equal_10 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8237 '<FieldAccess .name_symbol>' Fld_25 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8236 '<FieldAccess .id>' Fld_24 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_24 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8236
      if Fld_24.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_25 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_24.Node);
--# end
--# expr-done 8237
--# expr-start 8238 '<Symbol Unchecked_Union>' Sym_10 None
Sym_10 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Unchecked_Union); 
--# expr-done 8238
Is_Equal_10 := Fld_25 = Sym_10; 
--# expr-done 8239
   If_Result_9 := Is_Equal_10;
end if;
if If_Result_9 then
   If_Result_10 := True;
else
   --# expr-start 8243 '<Eq>' Is_Equal_11 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8241 '<FieldAccess .name_symbol>' Fld_27 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8240 '<FieldAccess .id>' Fld_26 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_26 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8240
      if Fld_26.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_27 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_26.Node);
--# end
--# expr-done 8241
--# expr-start 8242 '<Symbol Atomic>' Sym_11 None
Sym_11 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Atomic); 
--# expr-done 8242
Is_Equal_11 := Fld_27 = Sym_11; 
--# expr-done 8243
   If_Result_10 := Is_Equal_11;
end if;
if If_Result_10 then
   If_Result_11 := True;
else
   --# expr-start 8247 '<Eq>' Is_Equal_12 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8245 '<FieldAccess .name_symbol>' Fld_29 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8244 '<FieldAccess .id>' Fld_28 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_28 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8244
      if Fld_28.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_29 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_28.Node);
--# end
--# expr-done 8245
--# expr-start 8246 '<Symbol Atomic_Components>' Sym_12 None
Sym_12 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Atomic_Components); 
--# expr-done 8246
Is_Equal_12 := Fld_29 = Sym_12; 
--# expr-done 8247
   If_Result_11 := Is_Equal_12;
end if;
if If_Result_11 then
   If_Result_12 := True;
else
   --# expr-start 8251 '<Eq>' Is_Equal_13 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8249 '<FieldAccess .name_symbol>' Fld_31 /usr/src/packages/BUILD/ada/ast.py:7013
--# expr-start 8248 '<FieldAccess .id>' Fld_30 /usr/src/packages/BUILD/ada/ast.py:7013
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_30 := Create_Internal_Entity_Identifier (Node => Ent.Node.Pragma_Node_F_Id, Info => Ent.Info);
--# expr-done 8248
      if Fld_30.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_31 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Fld_30.Node);
--# end
--# expr-done 8249
--# expr-start 8250 '<Symbol No_Return>' Sym_13 None
Sym_13 := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_No_Return); 
--# expr-done 8250
Is_Equal_13 := Fld_31 = Sym_13; 
--# expr-done 8251
   If_Result_12 := Is_Equal_13;
end if;
if If_Result_12 then
--# expr-start 8254 '<Cast to BaseId>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7018
--# expr-start 8252 '<FieldAccess .args>' Fld_32 /usr/src/packages/BUILD/ada/ast.py:7018
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_32 := Create_Internal_Entity_Base_Assoc_List (Node => Ent.Node.Pragma_Node_F_Args, Info => Ent.Info);
--# expr-done 8252
Fld_33 := Fld_32.Node;
Get_Result_1 := Get (Fld_33, 0, True); 
Is_Null_1 := Get_Result_1 = null; 
if Is_Null_1 then
   If_Result_13 := No_Entity_Base_Assoc;
else
Fld_34 := Fld_32.Info;
As_Entity_1 := (Info => Fld_34, Node => Get_Result_1); 
   If_Result_13 := As_Entity_1;
end if;
Var_Expr := If_Result_13;
if Var_Expr /= No_Entity_Base_Assoc then
   --# scope-start
--# expr-start 8253 '<FieldAccess .assoc_expr>' Fld_35 /usr/src/packages/BUILD/ada/ast.py:7018
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseAssoc.assoc_expr'
Fld_35 := Libadalang.Implementation.Dispatcher_Base_Assoc_P_Assoc_Expr (Node => Var_Expr.Node, E_Info => Var_Expr.Info);
--# end
--# expr-done 8253
Scope_Result := Fld_35;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Expr;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Base_Id
   then
      Cast_Result_1 := Create_Internal_Entity_Base_Id
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Base_Id;
   end if;
--# expr-done 8254
      Cast_Result_3 := Create_Internal_Entity_Name
        (Node => Cast_Result_1.Node,
         Info => Cast_Result_1.Info);
   If_Result_14 := Cast_Result_3;
else
Cast_Expr := No_Entity_Base_Id; 
      Cast_Result_2 := Create_Internal_Entity_Name
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   If_Result_14 := Cast_Result_2;
end if;
   If_Result_15 := If_Result_14;
end if;
--# expr-done 8255
Let_Result := If_Result_15; 
--# end
--# expr-done 8256
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Entity_Name;
--# end
--# property-start Pragma.associated_decls_helper /usr/src/packages/BUILD/ada/ast.py:7024
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Decls_Helper
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_637;
            procedure Finalizer_Scope_1786;
            procedure Finalizer_Scope_1787;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Bare_Declarative_Part;
Fld : Internal_Entity_Name;
Fld_1 : Bare_Declarative_Part;
As_Entity : Internal_Entity_Declarative_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Declarative_Part;
Fld_2 : Lexical_Env := Empty_Env;
Fld_3 : Symbol_Type;
Env_Get_Result : Internal_Entity_Array_Access;
Scope_Result : Internal_Entity_Array_Access;
Result_Var : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Basic_Decl;
Fld_4 : Bare_Ada_Node;
Node_Comp : Boolean;
Map_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_637 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_637;
            procedure Finalizer_Scope_1786 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1786;
            procedure Finalizer_Scope_1787 is
            begin
                     Dec_Ref (Fld_2);
                     Dec_Ref (Env_Get_Result);
            end Finalizer_Scope_1787;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8267 '<Block>' Let_Result None
--# scope-start
--# expr-start 8257 '<FieldAccess .associated_entity_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7025
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8257
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr
--# expr-start 8266 '<filter_map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7027
--# expr-start 8258 '<FieldAccess .declarative_scope>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7027
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.declarative_scope
Fld_1 := Libadalang.Implementation.Ada_Node_P_Declarative_Scope (Node => Ent.Node);
--# end
--# expr-done 8258
Var_Expr_1 := Fld_1;
if Var_Expr_1 /= No_Bare_Ada_Node then
   --# scope-start
   --# bind decl_scope Var_Expr_1
--# expr-start 8262 '<EnvGet(<FieldAccess .children_env>, <FieldAccess .name_symbol>)>' Env_Get_Result /usr/src/packages/BUILD/ada/ast.py:7029
--# expr-start 8260 '<FieldAccess .children_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7029
--# expr-start 8259 '<as_entity(<Var decl_scope>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:7029
Is_Null := Var_Expr_1 = null; 
if Is_Null then
   If_Result := No_Entity_Declarative_Part;
else
As_Entity := (Info => E_Info, Node => Var_Expr_1); 
   If_Result := As_Entity;
end if;
--# expr-done 8259
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Children_Env (Node => If_Result.Node, E_Info => If_Result.Info);
--# expr-done 8260
--# expr-start 8261 '<FieldAccess .name_symbol>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7030
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.name_symbol'
Fld_3 := Libadalang.Implementation.Dispatcher_Name_P_Name_Symbol (Node => Var_Expr.Node);
--# end
--# expr-done 8261
Env_Get_Result := Construct_Entity_Array (AST_Envs.Get (Self => Fld_2, Key => Fld_3, Lookup_Kind => To_Lookup_Kind_Type (Flat), Categories => (Inherited_Primitives => False, Nocat => True))); 
--# expr-done 8262
Scope_Result := Env_Get_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1787;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Array_Type;
end if;
      Inc_Ref (Result_Var);
declare
   Map_Result_Vec : Internal_Entity_Basic_Decl_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Result_Var;
      begin
         for Item_244 of
               Collection.Items
         loop
   --# scope-start
               --# bind ent Item_244
               --# expr-start 8264 '<OrderingTest '"'"'lt'"'"'>' Node_Comp /usr/src/packages/BUILD/ada/ast.py:7035
--# expr-start 8263 '<FieldAccess .node>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7035
Fld_4 := Item_244.Node;
--# expr-done 8263
Node_Comp := Compare (Fld_4, Self, Less_Than); 
--# expr-done 8264
               if Node_Comp then
      --# expr-start 8265 '<Cast to BasicDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7034
Cast_Expr := Item_244; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Basic_Decl
   then
      Cast_Result := Create_Internal_Entity_Basic_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Basic_Decl;
   end if;
--# expr-done 8265
         declare
            Item_To_Append : constant Internal_Entity_Basic_Decl :=
               Cast_Result;
         begin
            Internal_Entity_Basic_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Basic_Decl_Array
        (Items_Count => Natural (Internal_Entity_Basic_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Basic_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Basic_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Basic_Decl_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8266
Scope_Result_1 := Map_Result;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1786;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var_1);
Let_Result := Result_Var_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8267
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_637;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_637;
               Finalizer_Scope_1786;
               Finalizer_Scope_1787;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Decls_Helper;
--# end
--# property-start Pragma.associated_decls /usr/src/packages/BUILD/ada/ast.py:7039
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Associated_Decls
  (Node : Bare_Pragma_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Basic_Decl_Array_Access
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Pragma_Node :=
     Internal_Entity_Pragma_Node'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_638;
            procedure Finalizer_Scope_1782;
            procedure Finalizer_Scope_1785;
            procedure Finalizer_Scope_1789;
            procedure Finalizer_Scope_1790;
      Block_Var : Internal_Entity_Basic_Decl_Array_Access;
Block_Var_1 : Internal_Entity_Basic_Decl;
Var_Expr : Bare_Compilation_Unit;
Var_Expr_1 : Internal_Entity_Name;
P : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_2 : Internal_Entity_Basic_Decl_Array_Access;
Var_Expr_3 : Internal_Entity_Basic_Decl;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Cast_Result : Bare_Compilation_Unit;
Fld_2 : Bare_Ada_Node;
Cast_Result_1 : Bare_Library_Item;
Fld_3 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Singleton : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result : Internal_Entity_Basic_Decl_Array_Access;
Result_Var : Internal_Entity_Basic_Decl_Array_Access;
Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result_2 : Bare_Basic_Decl;
As_Entity_1 : Internal_Entity_Basic_Decl;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity_Basic_Decl;
Fld_4 : Internal_Entity_Name;
Fld_5 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_1 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_1 : Internal_Entity_Basic_Decl_Array_Access;
Fld_6 : Internal_Entity_Defining_Name;
Fld_7 : Boolean;
If_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_2 : Internal_Entity_Basic_Decl_Array_Access;
Is_Equal : Boolean;
Not_Val : Boolean;
If_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result : Internal_Entity_Basic_Decl_Array_Access;
Scope_Result_3 : Internal_Entity_Basic_Decl_Array_Access;
Singleton_2 : Internal_Entity_Basic_Decl_Array_Access;
Result_Var_3 : Internal_Entity_Basic_Decl_Array_Access;
Let_Result_1 : Internal_Entity_Basic_Decl_Array_Access;
            procedure Finalizer_Scope_638 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Block_Var);
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Singleton_2);
                     Dec_Ref (Result_Var_3);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_638;
            procedure Finalizer_Scope_1782 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_1782;
            procedure Finalizer_Scope_1785 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (P);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Result_Var_2);
                     Dec_Ref (If_Result_3);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1785;
            procedure Finalizer_Scope_1789 is
            begin
                     Dec_Ref (Var_Expr_2);
            end Finalizer_Scope_1789;
            procedure Finalizer_Scope_1790 is
            begin
                     Dec_Ref (Singleton_1);
                     Dec_Ref (If_Result_2);
            end Finalizer_Scope_1790;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8290 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 8270 '<Cast to CompilationUnit>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:7044
--# expr-start 8269 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7044
--# expr-start 8268 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7044
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8268
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8269
   if Fld_1 = null
      or else Fld_1.Kind in Ada_Compilation_Unit_Range
   then
      Cast_Result := Fld_1;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8270
Var_Expr := Cast_Result;
if Var_Expr /= No_Bare_Ada_Node then
   --# scope-start
   --# bind cu Var_Expr
--# expr-start 8274 '<as_entity(<FieldAccess .item>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:7045
--# expr-start 8273 '<FieldAccess .item>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7045
--# expr-start 8272 '<Cast to LibraryItem>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:7045
--# expr-start 8271 '<FieldAccess .body>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7045
      if Var_Expr = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Var_Expr.Compilation_Unit_F_Body;
--# expr-done 8271
   if Fld_2 = null
      or else Fld_2.Kind in Ada_Library_Item_Range
   then
      Cast_Result_1 := Fld_2;
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8272
      if Cast_Result_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Cast_Result_1.Library_Item_F_Item;
--# expr-done 8273
Is_Null := Fld_3 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => E_Info, Node => Fld_3); 
   If_Result := As_Entity;
end if;
--# expr-done 8274
                Singleton := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := If_Result;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_1782;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
Block_Var := Result_Var; Inc_Ref (Block_Var);
--# bind top_level_decl Block_Var
--# expr-start 8280 '<as_entity(<Cast to BasicDecl>)>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7050
--# expr-start 8279 '<Cast to BasicDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:7050
--# expr-start 8278 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:7050
--# expr-start 8277 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:7050
--# expr-start 8275 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:7050
Node_Parents := Parents (Self, True); 
--# expr-done 8275
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_245 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_245
               --# expr-start 8276 '<IsA BasicDecl>' Is_A /usr/src/packages/BUILD/ada/ast.py:7051
Is_A := Item_245 /= null 
and then Item_245.Kind in Ada_Basic_Decl; 
--# expr-done 8276
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_245;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8277
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8278
   if Get_Result = null
      or else Get_Result.Kind in Ada_Basic_Decl
   then
      Cast_Result_2 := Get_Result;
   else
         Cast_Result_2 := No_Bare_Ada_Node;
   end if;
--# expr-done 8279
Is_Null_1 := Cast_Result_2 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity_Basic_Decl;
else
As_Entity_1 := (Info => E_Info, Node => Cast_Result_2); 
   If_Result_1 := As_Entity_1;
end if;
--# expr-done 8280
Block_Var_1 := If_Result_1; 
--# bind enclosing_program_unit Block_Var_1
--# expr-start 8281 '<FieldAccess .associated_entity_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7065
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_entity_name
Fld_4 := Libadalang.Implementation.Pragma_Node_P_Associated_Entity_Name (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8281
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind name Var_Expr_1
--# expr-start 8289 '<Let p>' Let_Result /usr/src/packages/BUILD/ada/ast.py:7066
--# scope-start
--# expr-start 8282 '<FieldAccess .associated_decls_helper>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7067
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.associated_decls_helper
Fld_5 := Libadalang.Implementation.Pragma_Node_P_Associated_Decls_Helper (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8282
Var_Expr_2 := Fld_5;
      Inc_Ref (Var_Expr_2);
if Var_Expr_2 /= No_Internal_Entity_Basic_Decl_Array_Type then
   --# scope-start
   --# bind e Var_Expr_2
Scope_Result_1 := Var_Expr_2;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_1789;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Block_Var;
end if;
      Inc_Ref (Result_Var_1);
P := Result_Var_1; Inc_Ref (P);
--# bind p P
--# expr-start 8288 '<If>' If_Result_3 /usr/src/packages/BUILD/ada/ast.py:7068
--# expr-start 8284 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:7069
--# expr-start 8283 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:7069
Is_Equal := Equivalent (P, No_Internal_Entity_Basic_Decl_Array_Type); 
--# expr-done 8283
Not_Val := not (Is_Equal); 
--# expr-done 8284
if Not_Val then
   If_Result_3 := P;
else
Var_Expr_3 := Block_Var_1;
if Var_Expr_3 /= No_Entity_Basic_Decl then
   --# scope-start
   --# bind epu Var_Expr_3
--# expr-start 8287 '<If>' If_Result_2 /usr/src/packages/BUILD/ada/ast.py:7071
--# expr-start 8286 '<FieldAccess .name_matches(...)>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:7072
--# expr-start 8285 '<FieldAccess .defining_name>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:7072
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_6 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Var_Expr_3.Node, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 8285
      if Fld_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_matches
Fld_7 := Libadalang.Implementation.Name_P_Name_Matches (Node => Fld_6.Node, N => Var_Expr_1);
--# end
--# expr-done 8286
if Fld_7 then
                Singleton_1 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_1.Items (1) := Var_Expr_3;
   If_Result_2 := Singleton_1;
else
   If_Result_2 := No_Internal_Entity_Basic_Decl_Array_Type;
end if;
      Inc_Ref (If_Result_2);
--# expr-done 8287
Scope_Result_2 := If_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_1790;
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := Block_Var;
end if;
      Inc_Ref (Result_Var_2);
   If_Result_3 := Result_Var_2;
end if;
      Inc_Ref (If_Result_3);
--# expr-done 8288
Let_Result := If_Result_3; Inc_Ref (Let_Result);
--# end
--# expr-done 8289
Scope_Result_3 := Let_Result;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_1785;
   Result_Var_3 := Scope_Result_3;
else
                Singleton_2 := Create_Internal_Entity_Basic_Decl_Array (Items_Count => 1);
                Singleton_2.Items (1) := Block_Var_1;
   Result_Var_3 := Singleton_2;
end if;
      Inc_Ref (Result_Var_3);
Let_Result_1 := Result_Var_3; Inc_Ref (Let_Result_1);
--# end
--# expr-done 8290
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_638;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_638;
               Finalizer_Scope_1782;
               Finalizer_Scope_1785;
               Finalizer_Scope_1789;
               Finalizer_Scope_1790;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Associated_Decls;
--# end
--# property-start Pragma.initial_env_name /usr/src/packages/BUILD/ada/ast.py:7083
pragma Warnings (Off, "is not referenced");
 function Pragma_Node_P_Initial_Env_Name
  (Node : Bare_Pragma_Node
  )
   return Symbol_Type
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
            procedure Finalizer_Scope_639;
      Block_Var : Bare_Ada_Node;
Block_Var_1 : Bare_Ada_Node;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Fld_2 : Bare_Ada_Node;
Fld_3 : Bare_Compilation_Unit;
Fld_4 : Bare_Basic_Decl;
Fld_5 : Character_Type_Array_Access;
Sym : Symbol_Type;
Is_A : Boolean;
Is_A_1 : Boolean;
If_Result : Boolean;
If_Result_1 : Symbol_Type;
Let_Result : Symbol_Type;
            procedure Finalizer_Scope_639 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_639;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8301 '<Block>' Let_Result None
--# scope-start
--# expr-start 8291 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:7090
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8291
Block_Var := Fld; 
--# bind p Block_Var
--# expr-start 8293 '<parent(<parent(<Var Self>)>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:7091
--# expr-start 8292 '<parent(<Var Self>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:7091
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Self.Parent; 
--# expr-done 8292
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Parent; 
--# expr-done 8293
Block_Var_1 := Fld_2; 
--# bind gp Block_Var_1
--# expr-start 8300 '<If>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:7092
--# expr-start 8294 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:7093
Is_A := Block_Var /= null 
and then Block_Var.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8294
if Is_A then
   If_Result := True;
else
   --# expr-start 8295 '<IsA CompilationUnit>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:7093
Is_A_1 := Block_Var_1 /= null 
and then Block_Var_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8295
   If_Result := Is_A_1;
end if;
if If_Result then
   --# expr-start 8299 '<to_symbol(<FieldAccess .top_level_env_name>)>' Sym /usr/src/packages/BUILD/ada/ast.py:7094
--# expr-start 8298 '<FieldAccess .top_level_env_name>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:7094
--# expr-start 8297 '<FieldAccess .decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:7094
--# expr-start 8296 '<FieldAccess .enclosing_compilation_unit>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:7094
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.enclosing_compilation_unit
Fld_3 := Libadalang.Implementation.Ada_Node_P_Enclosing_Compilation_Unit (Node => Self);
--# end
--# expr-done 8296
      if Fld_3 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CompilationUnit.decl
Fld_4 := Libadalang.Implementation.Compilation_Unit_P_Decl (Node => Fld_3);
--# end
--# expr-done 8297
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.top_level_env_name
Fld_5 := Libadalang.Implementation.Basic_Decl_P_Top_Level_Env_Name (Node => Fld_4);
--# end
--# expr-done 8298
Sym := String_To_Symbol (Self.Unit.Context, Fld_5); 
--# expr-done 8299
   If_Result_1 := Sym;
else
   If_Result_1 := null;
end if;
--# expr-done 8300
Let_Result := If_Result_1; 
--# end
--# expr-done 8301
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_639;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_639;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Pragma_Node_P_Initial_Env_Name;
--# end
--# property-start Pragma._initial_env_name_40 /usr/src/packages/BUILD/ada/ast.py:7100
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_40
  (Node : Bare_Pragma_Node
  )
   return Symbol_Type
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8302 '<FieldAccess .initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:7100
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Pragma.initial_env_name
Fld := Libadalang.Implementation.Pragma_Node_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 8302
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_40;
--# end
--# property-start Pragma._initial_env_41 /usr/src/packages/BUILD/ada/ast.py:7101
pragma Warnings (Off, "is not referenced");
 function Initial_Env_41
  (Node : Bare_Pragma_Node
  )
   return Lexical_Env
is
   Self : Bare_Pragma_Node := Bare_Pragma_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1211;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1211 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1211;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8303 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:7101
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 8303
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1211;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1211;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_41;
--# end
   --
   --  Primitives for Bare_Private_Node
   --
--# property-start '[dispatcher]Private.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Private_Node_P_As_Bool
  (Node : Bare_Private_Node
  )
   return Boolean
is
   Self : Bare_Private_Node := Bare_Private_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Private_Node (Self.Kind) is
               when Ada_Private_Absent =>
                  --# property-call-start Private.Absent.as_bool
                  Property_Result := Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Private_Present =>
                  --# property-call-start Private.Present.as_bool
                  Property_Result := Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Private_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Absent
   --
--# property-start Private.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3879
pragma Warnings (Off, "is not referenced");
 function Private_Absent_P_As_Bool
  (Node : Bare_Private_Absent
  )
   return Boolean
is
   Self : Bare_Private_Absent := Bare_Private_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Private_Present
   --
--# property-start Private.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3879
pragma Warnings (Off, "is not referenced");
 function Private_Present_P_As_Bool
  (Node : Bare_Private_Present
  )
   return Boolean
is
   Self : Bare_Private_Present := Bare_Private_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Def
   --
      procedure Initialize_Fields_For_Protected_Def
        (Self : Bare_Protected_Def
         ; Protected_Def_F_Public_Part : Bare_Public_Part
         ; Protected_Def_F_Private_Part : Bare_Private_Part
         ; Protected_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Protected_Def_F_Public_Part := Protected_Def_F_Public_Part;
            Self.Protected_Def_F_Private_Part := Protected_Def_F_Private_Part;
            Self.Protected_Def_F_End_Name := Protected_Def_F_End_Name;
      end Initialize_Fields_For_Protected_Def;
   function Protected_Def_F_Public_Part
     (Node : Bare_Protected_Def) return Bare_Public_Part
   is
   begin
         return Node.Protected_Def_F_Public_Part;
   end;
   function Protected_Def_F_Private_Part
     (Node : Bare_Protected_Def) return Bare_Private_Part
   is
   begin
         return Node.Protected_Def_F_Private_Part;
   end;
   function Protected_Def_F_End_Name
     (Node : Bare_Protected_Def) return Bare_End_Name
   is
   begin
         return Node.Protected_Def_F_End_Name;
   end;
   --
   --  Primitives for Bare_Protected_Node
   --
--# property-start '[dispatcher]Protected.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Protected_Node_P_As_Bool
  (Node : Bare_Protected_Node
  )
   return Boolean
is
   Self : Bare_Protected_Node := Bare_Protected_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Protected_Node (Self.Kind) is
               when Ada_Protected_Absent =>
                  --# property-call-start Protected.Absent.as_bool
                  Property_Result := Protected_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Protected_Present =>
                  --# property-call-start Protected.Present.as_bool
                  Property_Result := Protected_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Protected_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Absent
   --
--# property-start Protected.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3959
pragma Warnings (Off, "is not referenced");
 function Protected_Absent_P_As_Bool
  (Node : Bare_Protected_Absent
  )
   return Boolean
is
   Self : Bare_Protected_Absent := Bare_Protected_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Protected_Present
   --
--# property-start Protected.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3959
pragma Warnings (Off, "is not referenced");
 function Protected_Present_P_As_Bool
  (Node : Bare_Protected_Present
  )
   return Boolean
is
   Self : Bare_Protected_Present := Bare_Protected_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Protected_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Quantifier
   --
   --
   --  Primitives for Bare_Quantifier_All
   --
   --
   --  Primitives for Bare_Quantifier_Some
   --
   --
   --  Primitives for Bare_Range_Spec
   --
      procedure Initialize_Fields_For_Range_Spec
        (Self : Bare_Range_Spec
         ; Range_Spec_F_Range : Bare_Expr
        ) is
      begin
            Self.Range_Spec_F_Range := Range_Spec_F_Range;
      end Initialize_Fields_For_Range_Spec;
   function Range_Spec_F_Range
     (Node : Bare_Range_Spec) return Bare_Expr
   is
   begin
         return Node.Range_Spec_F_Range;
   end;
--# property-start RangeSpec.xref_equation /usr/src/packages/BUILD/ada/ast.py:15687
pragma Warnings (Off, "is not referenced");
 function Range_Spec_P_Xref_Equation
  (Node : Bare_Range_Spec
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Range_Spec := Bare_Range_Spec (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Range_Spec :=
     Internal_Entity_Range_Spec'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1160;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_1160 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1160;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8305 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15687
--# expr-start 8304 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:15687
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Range_Spec_F_Range, Info => Ent.Info);
--# expr-done 8304
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8305
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1160;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1160;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Range_Spec_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Renaming_Clause
   --
      procedure Initialize_Fields_For_Renaming_Clause
        (Self : Bare_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Self.Renaming_Clause_F_Renamed_Object := Renaming_Clause_F_Renamed_Object;
      end Initialize_Fields_For_Renaming_Clause;
   function Renaming_Clause_F_Renamed_Object
     (Node : Bare_Renaming_Clause) return Bare_Name
   is
   begin
         return Node.Renaming_Clause_F_Renamed_Object;
   end;
   --
   --  Primitives for Bare_Synthetic_Renaming_Clause
   --
      procedure Initialize_Fields_For_Synthetic_Renaming_Clause
        (Self : Bare_Synthetic_Renaming_Clause
         ; Renaming_Clause_F_Renamed_Object : Bare_Name
        ) is
      begin
            Initialize_Fields_For_Renaming_Clause
              (Self, Renaming_Clause_F_Renamed_Object);
      end Initialize_Fields_For_Synthetic_Renaming_Clause;
   --
   --  Primitives for Bare_Reverse_Node
   --
--# property-start '[dispatcher]Reverse.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Reverse_Node_P_As_Bool
  (Node : Bare_Reverse_Node
  )
   return Boolean
is
   Self : Bare_Reverse_Node := Bare_Reverse_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Reverse_Node (Self.Kind) is
               when Ada_Reverse_Absent =>
                  --# property-call-start Reverse.Absent.as_bool
                  Property_Result := Reverse_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Reverse_Present =>
                  --# property-call-start Reverse.Present.as_bool
                  Property_Result := Reverse_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Reverse_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Absent
   --
--# property-start Reverse.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3927
pragma Warnings (Off, "is not referenced");
 function Reverse_Absent_P_As_Bool
  (Node : Bare_Reverse_Absent
  )
   return Boolean
is
   Self : Bare_Reverse_Absent := Bare_Reverse_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Reverse_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Reverse_Present
   --
--# property-start Reverse.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3927
pragma Warnings (Off, "is not referenced");
 function Reverse_Present_P_As_Bool
  (Node : Bare_Reverse_Present
  )
   return Boolean
is
   Self : Bare_Reverse_Present := Bare_Reverse_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Reverse_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Select_When_Part
   --
      procedure Initialize_Fields_For_Select_When_Part
        (Self : Bare_Select_When_Part
         ; Select_When_Part_F_Cond_Expr : Bare_Expr
         ; Select_When_Part_F_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_When_Part_F_Cond_Expr := Select_When_Part_F_Cond_Expr;
            Self.Select_When_Part_F_Stmts := Select_When_Part_F_Stmts;
      end Initialize_Fields_For_Select_When_Part;
   function Select_When_Part_F_Cond_Expr
     (Node : Bare_Select_When_Part) return Bare_Expr
   is
   begin
         return Node.Select_When_Part_F_Cond_Expr;
   end;
   function Select_When_Part_F_Stmts
     (Node : Bare_Select_When_Part) return Bare_Stmt_List
   is
   begin
         return Node.Select_When_Part_F_Stmts;
   end;
--# property-start SelectWhenPart.xref_equation /usr/src/packages/BUILD/ada/ast.py:15182
pragma Warnings (Off, "is not referenced");
 function Select_When_Part_P_Xref_Equation
  (Node : Bare_Select_When_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_When_Part := Bare_Select_When_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Select_When_Part :=
     Internal_Entity_Select_When_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1133;
            procedure Finalizer_Scope_2608;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1133 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1133;
            procedure Finalizer_Scope_2608 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2608;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8312 '<Block>' Let_Result None
--# scope-start
--# expr-start 8306 '<FieldAccess .cond_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:15183
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Select_When_Part_F_Cond_Expr, Info => Ent.Info);
--# expr-done 8306
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 8311 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:15185
--# expr-start 8307 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15185
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8307
--# expr-start 8310 '<FieldAccess .bool_bind(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15185
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8309 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15185
--# expr-start 8308 '<FieldAccess .cond_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15185
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Select_When_Part_F_Cond_Expr;
--# expr-done 8308
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8309
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 8310
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15185") else null)); 
--# expr-done 8311
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2608;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15186") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 8312
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1133;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1133;
               Finalizer_Scope_2608;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Select_When_Part_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Stmt
   --
--# property-start Stmt.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:14607
pragma Warnings (Off, "is not referenced");
 function Stmt_P_Xref_Entry_Point
  (Node : Bare_Stmt
  )
   return Boolean
is
   Self : Bare_Stmt := Bare_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Stmt_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Composite_Stmt
   --
   --
   --  Primitives for Bare_Accept_Stmt
   --
      procedure Accept_Stmt_Pre_Env_Actions
        (Self            : Bare_Accept_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_179 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Accept_Stmt
        (Self : Bare_Accept_Stmt
         ; Accept_Stmt_F_Name : Bare_Identifier
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
        ) is
      begin
            Self.Accept_Stmt_F_Name := Accept_Stmt_F_Name;
            Self.Accept_Stmt_F_Entry_Index_Expr := Accept_Stmt_F_Entry_Index_Expr;
            Self.Accept_Stmt_F_Params := Accept_Stmt_F_Params;
      end Initialize_Fields_For_Accept_Stmt;
   function Accept_Stmt_F_Name
     (Node : Bare_Accept_Stmt) return Bare_Identifier
   is
   begin
         return Node.Accept_Stmt_F_Name;
   end;
   function Accept_Stmt_F_Entry_Index_Expr
     (Node : Bare_Accept_Stmt) return Bare_Expr
   is
   begin
         return Node.Accept_Stmt_F_Entry_Index_Expr;
   end;
   function Accept_Stmt_F_Params
     (Node : Bare_Accept_Stmt) return Bare_Entry_Completion_Formal_Params
   is
   begin
         return Node.Accept_Stmt_F_Params;
   end;
--# property-start AcceptStmt.designated_entry /usr/src/packages/BUILD/ada/ast.py:15132
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Designated_Entry
  (Node : Bare_Accept_Stmt
      ; Origin : Bare_Ada_Node
      ; Env : Lexical_Env
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Entry_Decl
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Accept_Stmt :=
     Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   --# bind env Env
   Property_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1130;
      Var_Expr : Internal_Entity_Entry_Decl;
Fld : Internal_Entity_Identifier;
Fld_1 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result : Internal_Entity_Entry_Decl;
Fld_2 : Internal_Entity_Entry_Spec;
Fld_3 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Base_Formal_Param_Holder;
Fld_4 : Boolean;
Scope_Result : Boolean;
Result_Var : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Let_Result : Internal_Entity_Entry_Decl;
            procedure Finalizer_Scope_1130 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1130;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8322 '<Block>' Let_Result None
--# scope-start
--# expr-start 8321 '<Cast to EntryDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:15133
--# expr-start 8320 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:15133
--# expr-start 8319 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:15133
--# expr-start 8314 '<FieldAccess .all_env_els_impl>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15133
--# expr-start 8313 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15133
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Accept_Stmt_F_Name, Info => Ent.Info);
--# expr-done 8313
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.all_env_els_impl'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_All_Env_Els_Impl (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8314
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_1;
      begin
         for Item_246 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_246
--# expr-start 8315 '<Cast to EntryDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:15134
Cast_Expr := Item_246; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Entry_Decl;
   end if;
--# expr-done 8315
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind d Var_Expr
--# expr-start 8318 '<FieldAccess .match_formal_params(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15135
--# expr-start 8316 '<FieldAccess .spec>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15135
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr.Node.Entry_Decl_F_Spec, Info => Var_Expr.Info);
--# expr-done 8316
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8317 '<FieldAccess .params>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15135
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Ent.Node.Accept_Stmt_F_Params, Info => Ent.Info);
--# expr-done 8317
      Cast_Result_1 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_4 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_2.Node, Other => Cast_Result_1, E_Info => Fld_2.Info);
--# end
--# expr-done 8318
Scope_Result := Fld_4;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
               if Result_Var then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_246;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8319
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8320
   if Get_Result.Node = null
      or else Get_Result.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Get_Result.Node,
         Info => Get_Result.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 8321
Let_Result := Cast_Result_2; 
--# end
--# expr-done 8322
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1130;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1130;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Accept_Stmt_P_Designated_Entry;
--# end
--# property-start AcceptStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:15140
pragma Warnings (Off, "is not referenced");
 function Accept_Stmt_P_Xref_Equation
  (Node : Bare_Accept_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Accept_Stmt :=
     Internal_Entity_Accept_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1131;
            procedure Finalizer_Scope_2611;
      Var_Expr : Internal_Entity_Expr;
Fld : Bare_Identifier;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Entry_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
Fld_3 : Internal_Entity_Expr;
Fld_4 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1131 is
            begin
                     Dec_Ref (Bind_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1131;
            procedure Finalizer_Scope_2611 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2611;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8330 '<Block>' Let_Result None
--# scope-start
--# expr-start 8329 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:15141
--# expr-start 8326 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:15142
--# expr-start 8324 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15142
--# expr-start 8323 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15142
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Accept_Stmt_F_Name;
--# expr-done 8323
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 8324
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 8325 '<FieldAccess .designated_entry>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15142
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AcceptStmt.designated_entry
Fld_2 := Libadalang.Implementation.Accept_Stmt_P_Designated_Entry (Node => Ent.Node, Origin => Origin, Env => Env, E_Info => Ent.Info);
--# end
--# expr-done 8325
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15142") else null)); 
--# expr-done 8326
--# expr-start 8327 '<FieldAccess .entry_index_expr>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15143
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Expr (Node => Ent.Node.Accept_Stmt_F_Entry_Index_Expr, Info => Ent.Info);
--# expr-done 8327
Var_Expr := Fld_3;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind e Var_Expr
--# expr-start 8328 '<FieldAccess .sub_equation>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15144
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_4 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8328
Scope_Result := Fld_4;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2611;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15145") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Bind_Result, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15141") else null)); 
--# expr-done 8329
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8330
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1131;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1131;
               Finalizer_Scope_2611;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Accept_Stmt_P_Xref_Equation;
--# end
--# property-start AcceptStmt._env_trans_parent_179 /usr/src/packages/BUILD/ada/ast.py:15128
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_179
  (Node : Bare_Accept_Stmt
  )
   return Boolean
is
   Self : Bare_Accept_Stmt := Bare_Accept_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_179;
--# end
   --
   --  Primitives for Bare_Accept_Stmt_With_Stmts
   --
      procedure Initialize_Fields_For_Accept_Stmt_With_Stmts
        (Self : Bare_Accept_Stmt_With_Stmts
         ; Accept_Stmt_F_Name : Bare_Identifier
         ; Accept_Stmt_F_Entry_Index_Expr : Bare_Expr
         ; Accept_Stmt_F_Params : Bare_Entry_Completion_Formal_Params
         ; Accept_Stmt_With_Stmts_F_Stmts : Bare_Handled_Stmts
         ; Accept_Stmt_With_Stmts_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Accept_Stmt
              (Self, Accept_Stmt_F_Name, Accept_Stmt_F_Entry_Index_Expr, Accept_Stmt_F_Params);
            Self.Accept_Stmt_With_Stmts_F_Stmts := Accept_Stmt_With_Stmts_F_Stmts;
            Self.Accept_Stmt_With_Stmts_F_End_Name := Accept_Stmt_With_Stmts_F_End_Name;
      end Initialize_Fields_For_Accept_Stmt_With_Stmts;
   function Accept_Stmt_With_Stmts_F_Stmts
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_Handled_Stmts
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_Stmts;
   end;
   function Accept_Stmt_With_Stmts_F_End_Name
     (Node : Bare_Accept_Stmt_With_Stmts) return Bare_End_Name
   is
   begin
         return Node.Accept_Stmt_With_Stmts_F_End_Name;
   end;
   --
   --  Primitives for Bare_Base_Loop_Stmt
   --
      procedure Initialize_Fields_For_Base_Loop_Stmt
        (Self : Bare_Base_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Base_Loop_Stmt_F_Spec := Base_Loop_Stmt_F_Spec;
            Self.Base_Loop_Stmt_F_Stmts := Base_Loop_Stmt_F_Stmts;
            Self.Base_Loop_Stmt_F_End_Name := Base_Loop_Stmt_F_End_Name;
      end Initialize_Fields_For_Base_Loop_Stmt;
   function Base_Loop_Stmt_F_Spec
     (Node : Bare_Base_Loop_Stmt) return Bare_Loop_Spec
   is
   begin
         return Node.Base_Loop_Stmt_F_Spec;
   end;
   function Base_Loop_Stmt_F_Stmts
     (Node : Bare_Base_Loop_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Base_Loop_Stmt_F_Stmts;
   end;
   function Base_Loop_Stmt_F_End_Name
     (Node : Bare_Base_Loop_Stmt) return Bare_End_Name
   is
   begin
         return Node.Base_Loop_Stmt_F_End_Name;
   end;
--# property-start BaseLoopStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14981
pragma Warnings (Off, "is not referenced");
 function Base_Loop_Stmt_P_Xref_Equation
  (Node : Bare_Base_Loop_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Base_Loop_Stmt := Bare_Base_Loop_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Base_Loop_Stmt :=
     Internal_Entity_Base_Loop_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1123;
            procedure Finalizer_Scope_2612;
      Var_Expr : Internal_Entity_Loop_Spec;
Fld : Internal_Entity_Loop_Spec;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1123 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1123;
            procedure Finalizer_Scope_2612 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2612;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8333 '<Block>' Let_Result None
--# scope-start
--# expr-start 8331 '<FieldAccess .spec>' Fld /usr/src/packages/BUILD/ada/ast.py:14982
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Loop_Spec (Node => Ent.Node.Base_Loop_Stmt_F_Spec, Info => Ent.Info);
--# expr-done 8331
Var_Expr := Fld;
if Var_Expr /= No_Entity_Loop_Spec then
   --# scope-start
   --# bind s Var_Expr
--# expr-start 8332 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14982
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8332
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2612;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14983") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 8333
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1123;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1123;
               Finalizer_Scope_2612;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Base_Loop_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_For_Loop_Stmt
   --
      procedure For_Loop_Stmt_Pre_Env_Actions
        (Self            : Bare_For_Loop_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_176 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_For_Loop_Stmt
        (Self : Bare_For_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_For_Loop_Stmt;
--# property-start ForLoopStmt._env_trans_parent_176 /usr/src/packages/BUILD/ada/ast.py:14999
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_176
  (Node : Bare_For_Loop_Stmt
  )
   return Boolean
is
   Self : Bare_For_Loop_Stmt := Bare_For_Loop_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_176;
--# end
   --
   --  Primitives for Bare_Loop_Stmt
   --
      procedure Initialize_Fields_For_Loop_Stmt
        (Self : Bare_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_Loop_Stmt;
   --
   --  Primitives for Bare_While_Loop_Stmt
   --
      procedure Initialize_Fields_For_While_Loop_Stmt
        (Self : Bare_While_Loop_Stmt
         ; Base_Loop_Stmt_F_Spec : Bare_Loop_Spec
         ; Base_Loop_Stmt_F_Stmts : Bare_Stmt_List
         ; Base_Loop_Stmt_F_End_Name : Bare_End_Name
        ) is
      begin
            Initialize_Fields_For_Base_Loop_Stmt
              (Self, Base_Loop_Stmt_F_Spec, Base_Loop_Stmt_F_Stmts, Base_Loop_Stmt_F_End_Name);
      end Initialize_Fields_For_While_Loop_Stmt;
   --
   --  Primitives for Bare_Block_Stmt
   --
      procedure Block_Stmt_Pre_Env_Actions
        (Self            : Bare_Block_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_177 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
--# property-start BlockStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:15018
pragma Warnings (Off, "is not referenced");
 function Block_Stmt_P_Xref_Equation
  (Node : Bare_Block_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Block_Stmt := Bare_Block_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1124;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1124 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1124;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15018") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1124;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1124;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Block_Stmt_P_Xref_Equation;
--# end
--# property-start BlockStmt._env_trans_parent_177 /usr/src/packages/BUILD/ada/ast.py:15016
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_177
  (Node : Bare_Block_Stmt
  )
   return Boolean
is
   Self : Bare_Block_Stmt := Bare_Block_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_177;
--# end
   --
   --  Primitives for Bare_Begin_Block
   --
      procedure Initialize_Fields_For_Begin_Block
        (Self : Bare_Begin_Block
         ; Begin_Block_F_Stmts : Bare_Handled_Stmts
         ; Begin_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Begin_Block_F_Stmts := Begin_Block_F_Stmts;
            Self.Begin_Block_F_End_Name := Begin_Block_F_End_Name;
      end Initialize_Fields_For_Begin_Block;
   function Begin_Block_F_Stmts
     (Node : Bare_Begin_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Begin_Block_F_Stmts;
   end;
   function Begin_Block_F_End_Name
     (Node : Bare_Begin_Block) return Bare_End_Name
   is
   begin
         return Node.Begin_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Decl_Block
   --
      procedure Initialize_Fields_For_Decl_Block
        (Self : Bare_Decl_Block
         ; Decl_Block_F_Decls : Bare_Declarative_Part
         ; Decl_Block_F_Stmts : Bare_Handled_Stmts
         ; Decl_Block_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Decl_Block_F_Decls := Decl_Block_F_Decls;
            Self.Decl_Block_F_Stmts := Decl_Block_F_Stmts;
            Self.Decl_Block_F_End_Name := Decl_Block_F_End_Name;
      end Initialize_Fields_For_Decl_Block;
   function Decl_Block_F_Decls
     (Node : Bare_Decl_Block) return Bare_Declarative_Part
   is
   begin
         return Node.Decl_Block_F_Decls;
   end;
   function Decl_Block_F_Stmts
     (Node : Bare_Decl_Block) return Bare_Handled_Stmts
   is
   begin
         return Node.Decl_Block_F_Stmts;
   end;
   function Decl_Block_F_End_Name
     (Node : Bare_Decl_Block) return Bare_End_Name
   is
   begin
         return Node.Decl_Block_F_End_Name;
   end;
   --
   --  Primitives for Bare_Case_Stmt
   --
      procedure Initialize_Fields_For_Case_Stmt
        (Self : Bare_Case_Stmt
         ; Case_Stmt_F_Expr : Bare_Expr
         ; Case_Stmt_F_Alternatives : Bare_Case_Stmt_Alternative_List
        ) is
      begin
            Self.Case_Stmt_F_Expr := Case_Stmt_F_Expr;
            Self.Case_Stmt_F_Alternatives := Case_Stmt_F_Alternatives;
      end Initialize_Fields_For_Case_Stmt;
   function Case_Stmt_F_Expr
     (Node : Bare_Case_Stmt) return Bare_Expr
   is
   begin
         return Node.Case_Stmt_F_Expr;
   end;
   function Case_Stmt_F_Alternatives
     (Node : Bare_Case_Stmt) return Bare_Case_Stmt_Alternative_List
   is
   begin
         return Node.Case_Stmt_F_Alternatives;
   end;
--# property-start CaseStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:15064
pragma Warnings (Off, "is not referenced");
 function Case_Stmt_P_Xref_Equation
  (Node : Bare_Case_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Case_Stmt := Bare_Case_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Case_Stmt :=
     Internal_Entity_Case_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1126;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Pred_1 : Logic_Equation;
And_Pred : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1126 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Pred);
                     Dec_Ref (Pred_1);
                     Dec_Ref (And_Pred);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1126;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8344 '<Block>' Let_Result None
--# scope-start
--# expr-start 8343 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:15065
--# expr-start 8335 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15065
--# expr-start 8334 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:15065
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Case_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 8334
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8335
--# expr-start 8342 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:15069
--# expr-start 8338 '<Predicate on AdaNode.is_not_null>' Pred /usr/src/packages/BUILD/ada/ast.py:15069
--# expr-start 8337 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15069
--# expr-start 8336 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15069
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Case_Stmt_F_Expr;
--# expr-done 8336
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8337
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Pred := Ada_Node_P_Is_Not_Null_0_Pred.Create (Fld_3, Create_Ada_Node_P_Is_Not_Null_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Ada_Node.Ada_Node_P_Is_Not_Null")            else null))); 
--# expr-done 8338
--# expr-start 8341 '<Predicate on BaseTypeDecl.is_discrete_type>' Pred_1 /usr/src/packages/BUILD/ada/ast.py:15071
--# expr-start 8340 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:15071
--# expr-start 8339 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:15071
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Case_Stmt_F_Expr;
--# expr-done 8339
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 8340
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
Pred_1 := Base_Type_Decl_P_Is_Discrete_Type_0_Pred.Create (Fld_5, Create_Base_Type_Decl_P_Is_Discrete_Type_0_Predicate (Origin, Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Discrete_Type")            else null))); 
--# expr-done 8341
And_Pred := Logic_And (Pred, Pred_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15069") else null)); 
--# expr-done 8342
And_Pred_1 := Logic_And (Fld_1, And_Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15065") else null)); 
--# expr-done 8343
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8344
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1126;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1126;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Case_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Extended_Return_Stmt
   --
      procedure Extended_Return_Stmt_Pre_Env_Actions
        (Self            : Bare_Extended_Return_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_178 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Extended_Return_Stmt
        (Self : Bare_Extended_Return_Stmt
         ; Extended_Return_Stmt_F_Decl : Bare_Extended_Return_Stmt_Object_Decl
         ; Extended_Return_Stmt_F_Stmts : Bare_Handled_Stmts
        ) is
      begin
            Self.Extended_Return_Stmt_F_Decl := Extended_Return_Stmt_F_Decl;
            Self.Extended_Return_Stmt_F_Stmts := Extended_Return_Stmt_F_Stmts;
      end Initialize_Fields_For_Extended_Return_Stmt;
   function Extended_Return_Stmt_F_Decl
     (Node : Bare_Extended_Return_Stmt) return Bare_Extended_Return_Stmt_Object_Decl
   is
   begin
         return Node.Extended_Return_Stmt_F_Decl;
   end;
   function Extended_Return_Stmt_F_Stmts
     (Node : Bare_Extended_Return_Stmt) return Bare_Handled_Stmts
   is
   begin
         return Node.Extended_Return_Stmt_F_Stmts;
   end;
--# property-start ExtendedReturnStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:15049
pragma Warnings (Off, "is not referenced");
 function Extended_Return_Stmt_P_Xref_Equation
  (Node : Bare_Extended_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Extended_Return_Stmt := Bare_Extended_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1125;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1125 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1125;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8345 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15050") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 8345
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1125;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1125;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Extended_Return_Stmt_P_Xref_Equation;
--# end
--# property-start ExtendedReturnStmt._env_trans_parent_178 /usr/src/packages/BUILD/ada/ast.py:15052
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_178
  (Node : Bare_Extended_Return_Stmt
  )
   return Boolean
is
   Self : Bare_Extended_Return_Stmt := Bare_Extended_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_178;
--# end
   --
   --  Primitives for Bare_If_Stmt
   --
      procedure Initialize_Fields_For_If_Stmt
        (Self : Bare_If_Stmt
         ; If_Stmt_F_Cond_Expr : Bare_Expr
         ; If_Stmt_F_Then_Stmts : Bare_Stmt_List
         ; If_Stmt_F_Alternatives : Bare_Elsif_Stmt_Part_List
         ; If_Stmt_F_Else_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.If_Stmt_F_Cond_Expr := If_Stmt_F_Cond_Expr;
            Self.If_Stmt_F_Then_Stmts := If_Stmt_F_Then_Stmts;
            Self.If_Stmt_F_Alternatives := If_Stmt_F_Alternatives;
            Self.If_Stmt_F_Else_Stmts := If_Stmt_F_Else_Stmts;
      end Initialize_Fields_For_If_Stmt;
   function If_Stmt_F_Cond_Expr
     (Node : Bare_If_Stmt) return Bare_Expr
   is
   begin
         return Node.If_Stmt_F_Cond_Expr;
   end;
   function If_Stmt_F_Then_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Then_Stmts;
   end;
   function If_Stmt_F_Alternatives
     (Node : Bare_If_Stmt) return Bare_Elsif_Stmt_Part_List
   is
   begin
         return Node.If_Stmt_F_Alternatives;
   end;
   function If_Stmt_F_Else_Stmts
     (Node : Bare_If_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.If_Stmt_F_Else_Stmts;
   end;
--# property-start IfStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14878
pragma Warnings (Off, "is not referenced");
 function If_Stmt_P_Xref_Equation
  (Node : Bare_If_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_If_Stmt := Bare_If_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_If_Stmt :=
     Internal_Entity_If_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1114;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Fld_4 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1114 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_4);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1114;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8352 '<Block>' Let_Result None
--# scope-start
--# expr-start 8351 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14880
--# expr-start 8347 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14880
--# expr-start 8346 '<FieldAccess .cond_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14880
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.If_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 8346
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8347
--# expr-start 8350 '<FieldAccess .bool_bind(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14881
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8349 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14881
--# expr-start 8348 '<FieldAccess .cond_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14881
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.If_Stmt_F_Cond_Expr;
--# expr-done 8348
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8349
--# property-call-start AdaNode.bool_bind
Fld_4 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_3, Origin => Origin);
--# end
--# expr-done 8350
And_Pred := Logic_And (Fld_1, Fld_4, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14880") else null)); 
--# expr-done 8351
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8352
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1114;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1114;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end If_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Named_Stmt
   --
      procedure Named_Stmt_Pre_Env_Actions
        (Self            : Bare_Named_Stmt;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Dest_Env_Name : constant Symbol_Type :=
            null;
         Resolver      : constant Entity_Resolver :=
            null;
         Mapping : Internal_Env_Assoc :=
            Libadalang.Implementation.Env_Mappings_174 (Node => 
Self);
      begin
         Add_To_Env
           (Self,
            State,
            Mapping.Key,
            Mapping.Val,
            Mapping.Metadata,
            Resolver,
            Dest_Env_Name,
            Mapping.Dest_Env,
            DSL_Location => "ast.py:14963");
         Dec_Ref (Mapping.Dest_Env);
      end;
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_175 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Named_Stmt
        (Self : Bare_Named_Stmt
         ; Named_Stmt_F_Decl : Bare_Named_Stmt_Decl
         ; Named_Stmt_F_Stmt : Bare_Composite_Stmt
        ) is
      begin
            Self.Named_Stmt_F_Decl := Named_Stmt_F_Decl;
            Self.Named_Stmt_F_Stmt := Named_Stmt_F_Stmt;
      end Initialize_Fields_For_Named_Stmt;
   function Named_Stmt_F_Decl
     (Node : Bare_Named_Stmt) return Bare_Named_Stmt_Decl
   is
   begin
         return Node.Named_Stmt_F_Decl;
   end;
   function Named_Stmt_F_Stmt
     (Node : Bare_Named_Stmt) return Bare_Composite_Stmt
   is
   begin
         return Node.Named_Stmt_F_Stmt;
   end;
--# property-start NamedStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14967
pragma Warnings (Off, "is not referenced");
 function Named_Stmt_P_Xref_Equation
  (Node : Bare_Named_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1122;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1122 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1122;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14967") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1122;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1122;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Named_Stmt_P_Xref_Equation;
--# end
--# property-start NamedStmt._env_mappings_174 /usr/src/packages/BUILD/ada/ast.py:14963
pragma Warnings (Off, "is not referenced");
 function Env_Mappings_174
  (Node : Bare_Named_Stmt
  )
   return Internal_Env_Assoc
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Env_Assoc;
            procedure Finalizer_Scope_1344;
      Fld : Bare_Named_Stmt_Decl;
Fld_1 : Symbol_Type;
Fld_2 : Bare_Named_Stmt_Decl;
Cast_Result : Bare_Ada_Node;
New_Struct : Internal_Env_Assoc := No_Env_Assoc;
            procedure Finalizer_Scope_1344 is
            begin
                     Dec_Ref (New_Struct);
            end Finalizer_Scope_1344;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8356 '<New InternalEnvAssoc>' New_Struct /usr/src/packages/BUILD/ada/ast.py:14963
--# expr-start 8354 '<FieldAccess .name_symbol>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14963
--# expr-start 8353 '<FieldAccess .decl>' Fld /usr/src/packages/BUILD/ada/ast.py:14963
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Named_Stmt_F_Decl;
--# expr-done 8353
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.name_symbol
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Name_Symbol (Node => Fld);
--# end
--# expr-done 8354
--# expr-start 8355 '<FieldAccess .decl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14963
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Named_Stmt_F_Decl;
--# expr-done 8355
      Cast_Result := Fld_2;
Inc_Ref (Empty_Env);
New_Struct := (Dest_Env => Empty_Env, Key => Fld_1, Metadata => No_Metadata, Val => Cast_Result); 
--# expr-done 8356
      Property_Result := New_Struct;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1344;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1344;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Mappings_174;
--# end
--# property-start NamedStmt._env_trans_parent_175 /usr/src/packages/BUILD/ada/ast.py:14962
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_175
  (Node : Bare_Named_Stmt
  )
   return Boolean
is
   Self : Bare_Named_Stmt := Bare_Named_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_175;
--# end
   --
   --  Primitives for Bare_Select_Stmt
   --
      procedure Initialize_Fields_For_Select_Stmt
        (Self : Bare_Select_Stmt
         ; Select_Stmt_F_Guards : Bare_Select_When_Part_List
         ; Select_Stmt_F_Else_Stmts : Bare_Stmt_List
         ; Select_Stmt_F_Abort_Stmts : Bare_Stmt_List
        ) is
      begin
            Self.Select_Stmt_F_Guards := Select_Stmt_F_Guards;
            Self.Select_Stmt_F_Else_Stmts := Select_Stmt_F_Else_Stmts;
            Self.Select_Stmt_F_Abort_Stmts := Select_Stmt_F_Abort_Stmts;
      end Initialize_Fields_For_Select_Stmt;
   function Select_Stmt_F_Guards
     (Node : Bare_Select_Stmt) return Bare_Select_When_Part_List
   is
   begin
         return Node.Select_Stmt_F_Guards;
   end;
   function Select_Stmt_F_Else_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Else_Stmts;
   end;
   function Select_Stmt_F_Abort_Stmts
     (Node : Bare_Select_Stmt) return Bare_Stmt_List
   is
   begin
         return Node.Select_Stmt_F_Abort_Stmts;
   end;
--# property-start SelectStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:15169
pragma Warnings (Off, "is not referenced");
 function Select_Stmt_P_Xref_Equation
  (Node : Bare_Select_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Select_Stmt := Bare_Select_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Select_Stmt :=
     Internal_Entity_Select_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1132;
            procedure Finalizer_Scope_2613;
      Fld : Internal_Entity_Select_When_Part_List;
Fld_1 : Bare_Select_When_Part_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Select_When_Part;
Is_Null : Boolean;
If_Result : Internal_Entity_Select_When_Part;
Fld_3 : Logic_Equation;
Item_247 : Internal_Entity_Select_When_Part;
Bare_Item_247 : Bare_Select_When_Part;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1132 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1132;
            procedure Finalizer_Scope_2613 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2613;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8361 '<Block>' Let_Result None
--# scope-start
--# expr-start 8360 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:15170
--# expr-start 8359 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:15170
--# expr-start 8357 '<FieldAccess .guards>' Fld /usr/src/packages/BUILD/ada/ast.py:15170
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Select_When_Part_List (Node => Ent.Node.Select_Stmt_F_Guards, Info => Ent.Info);
--# expr-done 8357
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Select_When_Part_List := Fld_1;
      begin
         for Untyped_Item_247 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_247 := Untyped_Item_247; 
Is_Null := Bare_Item_247 = null; 
if Is_Null then
   If_Result := No_Entity_Select_When_Part;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_247); 
   If_Result := As_Entity;
end if;
                  Item_247 := If_Result; 
   --# scope-start
               --# bind wp Item_247
      --# expr-start 8358 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:15170
      if Item_247.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_247.Node, Env => Env, Origin => Origin, E_Info => Item_247.Info);
--# end
--# expr-done 8358
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2613;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8359
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15170") else null)); 
--# expr-done 8360
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 8361
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1132;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1132;
               Finalizer_Scope_2613;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Select_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Error_Stmt
   --
   --
   --  Primitives for Bare_Simple_Stmt
   --
   --
   --  Primitives for Bare_Abort_Stmt
   --
      procedure Initialize_Fields_For_Abort_Stmt
        (Self : Bare_Abort_Stmt
         ; Abort_Stmt_F_Names : Bare_Name_List
        ) is
      begin
            Self.Abort_Stmt_F_Names := Abort_Stmt_F_Names;
      end Initialize_Fields_For_Abort_Stmt;
   function Abort_Stmt_F_Names
     (Node : Bare_Abort_Stmt) return Bare_Name_List
   is
   begin
         return Node.Abort_Stmt_F_Names;
   end;
--# property-start AbortStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14816
pragma Warnings (Off, "is not referenced");
 function Abort_Stmt_P_Xref_Equation
  (Node : Bare_Abort_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Abort_Stmt := Bare_Abort_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Abort_Stmt :=
     Internal_Entity_Abort_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1111;
            procedure Finalizer_Scope_2614;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Fld_4 : Logic_Var;
Pred : Logic_Equation;
And_Pred : Logic_Equation;
Item_248 : Internal_Entity_Name;
Bare_Item_248 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1111 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1111;
            procedure Finalizer_Scope_2614 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Pred);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2614;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8369 '<Block>' Let_Result None
--# scope-start
--# expr-start 8368 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:14817
--# expr-start 8367 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14817
--# expr-start 8362 '<FieldAccess .names>' Fld /usr/src/packages/BUILD/ada/ast.py:14817
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Abort_Stmt_F_Names, Info => Ent.Info);
--# expr-done 8362
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_248 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_248 := Untyped_Item_248; 
Is_Null := Bare_Item_248 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_248); 
   If_Result := As_Entity;
end if;
                  Item_248 := If_Result; 
   --# scope-start
               --# bind name Item_248
      --# expr-start 8366 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14819
--# expr-start 8363 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14819
      if Item_248.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Item_248.Node, Env => Env, Origin => Origin, E_Info => Item_248.Info);
--# end
--# expr-done 8363
--# expr-start 8365 '<Predicate on BaseTypeDecl.is_task_type>' Pred /usr/src/packages/BUILD/ada/ast.py:14819
--# expr-start 8364 '<FieldAccess .type_var>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14820
      if Item_248.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Item_248.Node);
--# expr-done 8364
Fld_4.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_4.all);
Eq_Node.Refs.Destroy (Fld_4.all);
Pred := Base_Type_Decl_P_Is_Task_Type_0_Pred.Create (Fld_4, Create_Base_Type_Decl_P_Is_Task_Type_0_Predicate (Dbg_Img => (if Debug then new String'("Bare_Base_Type_Decl.Base_Type_Decl_P_Is_Task_Type")            else null))); 
--# expr-done 8365
And_Pred := Logic_And (Fld_3, Pred, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14819") else null)); 
--# expr-done 8366
         declare
            Item_To_Append : constant Logic_Equation :=
               And_Pred;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2614;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8367
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14817") else null)); 
--# expr-done 8368
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 8369
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1111;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1111;
               Finalizer_Scope_2614;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Abort_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Assign_Stmt
   --
      procedure Initialize_Fields_For_Assign_Stmt
        (Self : Bare_Assign_Stmt
         ; Assign_Stmt_F_Dest : Bare_Name
         ; Assign_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Assign_Stmt_F_Dest := Assign_Stmt_F_Dest;
            Self.Assign_Stmt_F_Expr := Assign_Stmt_F_Expr;
      end Initialize_Fields_For_Assign_Stmt;
   function Assign_Stmt_F_Dest
     (Node : Bare_Assign_Stmt) return Bare_Name
   is
   begin
         return Node.Assign_Stmt_F_Dest;
   end;
   function Assign_Stmt_F_Expr
     (Node : Bare_Assign_Stmt) return Bare_Expr
   is
   begin
         return Node.Assign_Stmt_F_Expr;
   end;
--# property-start AssignStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14671
pragma Warnings (Off, "is not referenced");
 function Assign_Stmt_P_Xref_Equation
  (Node : Bare_Assign_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Assign_Stmt := Bare_Assign_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Assign_Stmt :=
     Internal_Entity_Assign_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1104;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_4 : Bare_Expr;
Fld_5 : Logic_Var;
Fld_6 : Bare_Name;
Fld_7 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1104 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1104;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8381 '<Block>' Let_Result None
--# scope-start
--# expr-start 8380 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14673
--# expr-start 8374 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14673
--# expr-start 8371 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14673
--# expr-start 8370 '<FieldAccess .dest>' Fld /usr/src/packages/BUILD/ada/ast.py:14673
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Assign_Stmt_F_Dest, Info => Ent.Info);
--# expr-done 8370
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8371
--# expr-start 8373 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14674
--# expr-start 8372 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14674
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Assign_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 8372
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8373
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14673") else null)); 
--# expr-done 8374
--# expr-start 8379 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:14675
--# expr-start 8376 '<FieldAccess .type_var>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14675
--# expr-start 8375 '<FieldAccess .expr>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14675
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Self.Assign_Stmt_F_Expr;
--# expr-done 8375
      if Fld_4 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_4);
--# expr-done 8376
Fld_5.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_5.all);
Eq_Node.Refs.Destroy (Fld_5.all);
--# expr-start 8378 '<FieldAccess .type_var>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14675
--# expr-start 8377 '<FieldAccess .dest>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14675
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Assign_Stmt_F_Dest;
--# expr-done 8377
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_6);
--# expr-done 8378
Fld_7.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_7.all);
Eq_Node.Refs.Destroy (Fld_7.all);
Bind_Result := Bind_Default_566.Create (Fld_5, Fld_7, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14675") else null)); 
--# expr-done 8379
And_Pred_1 := Logic_And (And_Pred, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14673") else null)); 
--# expr-done 8380
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8381
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1104;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1104;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Assign_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Call_Stmt
   --
      procedure Initialize_Fields_For_Call_Stmt
        (Self : Bare_Call_Stmt
         ; Call_Stmt_F_Call : Bare_Name
        ) is
      begin
            Self.Call_Stmt_F_Call := Call_Stmt_F_Call;
      end Initialize_Fields_For_Call_Stmt;
   function Call_Stmt_F_Call
     (Node : Bare_Call_Stmt) return Bare_Name
   is
   begin
         return Node.Call_Stmt_F_Call;
   end;
--# property-start CallStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14643
pragma Warnings (Off, "is not referenced");
 function Call_Stmt_P_Xref_Equation
  (Node : Bare_Call_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Call_Stmt := Bare_Call_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Call_Stmt :=
     Internal_Entity_Call_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1102;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Bare_Name;
Fld_3 : Logic_Var;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1102 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1102;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8388 '<Block>' Let_Result None
--# scope-start
--# expr-start 8387 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14645
--# expr-start 8383 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14645
--# expr-start 8382 '<FieldAccess .call>' Fld /usr/src/packages/BUILD/ada/ast.py:14645
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Call_Stmt_F_Call, Info => Ent.Info);
--# expr-done 8382
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8383
--# expr-start 8386 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:14648
--# expr-start 8385 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14648
--# expr-start 8384 '<FieldAccess .call>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14648
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Call_Stmt_F_Call;
--# expr-done 8384
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8385
Fld_3.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_3.all);
Eq_Node.Refs.Destroy (Fld_3.all);
Bind_Result := Bind_Default_Default.Create (Fld_3, No_Entity, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14648") else null)); 
--# expr-done 8386
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14645") else null)); 
--# expr-done 8387
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8388
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1102;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1102;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Call_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Delay_Stmt
   --
      procedure Initialize_Fields_For_Delay_Stmt
        (Self : Bare_Delay_Stmt
         ; Delay_Stmt_F_Has_Until : Bare_Until_Node
         ; Delay_Stmt_F_Expr : Bare_Expr
        ) is
      begin
            Self.Delay_Stmt_F_Has_Until := Delay_Stmt_F_Has_Until;
            Self.Delay_Stmt_F_Expr := Delay_Stmt_F_Expr;
      end Initialize_Fields_For_Delay_Stmt;
   function Delay_Stmt_F_Has_Until
     (Node : Bare_Delay_Stmt) return Bare_Until_Node
   is
   begin
         return Node.Delay_Stmt_F_Has_Until;
   end;
   function Delay_Stmt_F_Expr
     (Node : Bare_Delay_Stmt) return Bare_Expr
   is
   begin
         return Node.Delay_Stmt_F_Expr;
   end;
--# property-start DelayStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14833
pragma Warnings (Off, "is not referenced");
 function Delay_Stmt_P_Xref_Equation
  (Node : Bare_Delay_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Delay_Stmt := Bare_Delay_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Delay_Stmt :=
     Internal_Entity_Delay_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1112;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Logic_True : Logic_Equation;
Fld_2 : Bare_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Bare_Until_Node;
Fld_7 : Boolean;
If_Result : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1112 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Fld_5);
                     Dec_Ref (If_Result);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1112;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8400 '<Block>' Let_Result None
--# scope-start
--# expr-start 8399 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14834
--# expr-start 8390 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14834
--# expr-start 8389 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Delay_Stmt_F_Expr, Info => Ent.Info);
--# expr-done 8389
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8390
--# expr-start 8398 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:14834
--# expr-start 8392 '<FieldAccess .as_bool>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14835
--# expr-start 8391 '<FieldAccess .has_until>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14835
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_6 := Self.Delay_Stmt_F_Has_Until;
--# expr-done 8391
      if Fld_6 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Until.as_bool'
Fld_7 := Libadalang.Implementation.Dispatcher_Until_Node_P_As_Bool (Node => Fld_6);
--# end
--# expr-done 8392
if Fld_7 then
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14835") else null)); 
   If_Result := Logic_True;
else
   --# expr-start 8397 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14836
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8394 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14836
--# expr-start 8393 '<FieldAccess .expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14836
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Delay_Stmt_F_Expr;
--# expr-done 8393
      if Fld_2 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_2);
--# expr-done 8394
--# expr-start 8396 '<FieldAccess .std_entity(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14836
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8395 '<Symbol Duration>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Duration); 
--# expr-done 8395
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 8396
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 8397
   If_Result := Fld_5;
end if;
      Inc_Ref (If_Result);
--# expr-done 8398
And_Pred := Logic_And (Fld_1, If_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14834") else null)); 
--# expr-done 8399
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8400
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1112;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1112;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Delay_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Exit_Stmt
   --
      procedure Initialize_Fields_For_Exit_Stmt
        (Self : Bare_Exit_Stmt
         ; Exit_Stmt_F_Loop_Name : Bare_Name
         ; Exit_Stmt_F_Cond_Expr : Bare_Expr
        ) is
      begin
            Self.Exit_Stmt_F_Loop_Name := Exit_Stmt_F_Loop_Name;
            Self.Exit_Stmt_F_Cond_Expr := Exit_Stmt_F_Cond_Expr;
      end Initialize_Fields_For_Exit_Stmt;
   function Exit_Stmt_F_Loop_Name
     (Node : Bare_Exit_Stmt) return Bare_Name
   is
   begin
         return Node.Exit_Stmt_F_Loop_Name;
   end;
   function Exit_Stmt_F_Cond_Expr
     (Node : Bare_Exit_Stmt) return Bare_Expr
   is
   begin
         return Node.Exit_Stmt_F_Cond_Expr;
   end;
--# property-start ExitStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14701
pragma Warnings (Off, "is not referenced");
 function Exit_Stmt_P_Xref_Equation
  (Node : Bare_Exit_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Exit_Stmt := Bare_Exit_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Exit_Stmt :=
     Internal_Entity_Exit_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1106;
            procedure Finalizer_Scope_2615;
            procedure Finalizer_Scope_2616;
      Var_Expr : Internal_Entity_Expr;
Var_Expr_1 : Internal_Entity_Name;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_4 : Internal_Entity_Name;
Fld_5 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1106 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1106;
            procedure Finalizer_Scope_2615 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2615;
            procedure Finalizer_Scope_2616 is
            begin
                     Dec_Ref (Fld_5);
            end Finalizer_Scope_2616;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8409 '<Block>' Let_Result None
--# scope-start
--# expr-start 8408 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14702
--# expr-start 8401 '<FieldAccess .cond_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14703
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Exit_Stmt_F_Cond_Expr, Info => Ent.Info);
--# expr-done 8401
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind cond Var_Expr
--# expr-start 8405 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14704
--# expr-start 8402 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14704
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8402
--# expr-start 8404 '<FieldAccess .bool_bind(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14705
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8403 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14705
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 8403
--# property-call-start AdaNode.bool_bind
Fld_3 := Libadalang.Implementation.Ada_Node_P_Bool_Bind (Node => Self, Type_Var => Fld_2, Origin => Origin);
--# end
--# expr-done 8404
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14704") else null)); 
--# expr-done 8405
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2615;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14706") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8406 '<FieldAccess .loop_name>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14708
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Name (Node => Ent.Node.Exit_Stmt_F_Loop_Name, Info => Ent.Info);
--# expr-done 8406
Var_Expr_1 := Fld_4;
if Var_Expr_1 /= No_Entity_Name then
   --# scope-start
   --# bind ln Var_Expr_1
--# expr-start 8407 '<FieldAccess .xref_no_overloading>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14709
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_5 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8407
Scope_Result_1 := Fld_5;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2616;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14710") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14702") else null)); 
--# expr-done 8408
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8409
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1106;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1106;
               Finalizer_Scope_2615;
               Finalizer_Scope_2616;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Exit_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Goto_Stmt
   --
      procedure Initialize_Fields_For_Goto_Stmt
        (Self : Bare_Goto_Stmt
         ; Goto_Stmt_F_Label_Name : Bare_Name
        ) is
      begin
            Self.Goto_Stmt_F_Label_Name := Goto_Stmt_F_Label_Name;
      end Initialize_Fields_For_Goto_Stmt;
   function Goto_Stmt_F_Label_Name
     (Node : Bare_Goto_Stmt) return Bare_Name
   is
   begin
         return Node.Goto_Stmt_F_Label_Name;
   end;
--# property-start GotoStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14688
pragma Warnings (Off, "is not referenced");
 function Goto_Stmt_P_Xref_Equation
  (Node : Bare_Goto_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Goto_Stmt := Bare_Goto_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Goto_Stmt :=
     Internal_Entity_Goto_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1105;
      Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1105 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1105;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8412 '<Block>' Let_Result None
--# scope-start
--# expr-start 8411 '<FieldAccess .xref_no_overloading(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14689
--# expr-start 8410 '<FieldAccess .label_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14689
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Goto_Stmt_F_Label_Name, Info => Ent.Info);
--# expr-done 8410
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_1 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Fld.Node, Sequential => False, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8411
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8412
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1105;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1105;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Goto_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Label
   --
      procedure Initialize_Fields_For_Label
        (Self : Bare_Label
         ; Label_F_Decl : Bare_Label_Decl
        ) is
      begin
            Self.Label_F_Decl := Label_F_Decl;
      end Initialize_Fields_For_Label;
   function Label_F_Decl
     (Node : Bare_Label) return Bare_Label_Decl
   is
   begin
         return Node.Label_F_Decl;
   end;
--# property-start Label.xref_equation /usr/src/packages/BUILD/ada/ast.py:14924
pragma Warnings (Off, "is not referenced");
 function Label_P_Xref_Equation
  (Node : Bare_Label
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Label := Bare_Label (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1118;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1118 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1118;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8413 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14925") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 8413
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1118;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1118;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Label_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Null_Stmt
   --
--# property-start NullStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14658
pragma Warnings (Off, "is not referenced");
 function Null_Stmt_P_Xref_Equation
  (Node : Bare_Null_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Null_Stmt := Bare_Null_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1103;
      Logic_True : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1103 is
            begin
                     Dec_Ref (Logic_True);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1103;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8414 '<Block>' Let_Result None
--# scope-start
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14659") else null)); 
Let_Result := Logic_True; Inc_Ref (Let_Result);
--# end
--# expr-done 8414
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1103;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1103;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Null_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Raise_Stmt
   --
      procedure Initialize_Fields_For_Raise_Stmt
        (Self : Bare_Raise_Stmt
         ; Raise_Stmt_F_Exception_Name : Bare_Name
         ; Raise_Stmt_F_Error_Message : Bare_Expr
        ) is
      begin
            Self.Raise_Stmt_F_Exception_Name := Raise_Stmt_F_Exception_Name;
            Self.Raise_Stmt_F_Error_Message := Raise_Stmt_F_Error_Message;
      end Initialize_Fields_For_Raise_Stmt;
   function Raise_Stmt_F_Exception_Name
     (Node : Bare_Raise_Stmt) return Bare_Name
   is
   begin
         return Node.Raise_Stmt_F_Exception_Name;
   end;
   function Raise_Stmt_F_Error_Message
     (Node : Bare_Raise_Stmt) return Bare_Expr
   is
   begin
         return Node.Raise_Stmt_F_Error_Message;
   end;
--# property-start RaiseStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14849
pragma Warnings (Off, "is not referenced");
 function Raise_Stmt_P_Xref_Equation
  (Node : Bare_Raise_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Raise_Stmt := Bare_Raise_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Raise_Stmt :=
     Internal_Entity_Raise_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1113;
            procedure Finalizer_Scope_2617;
            procedure Finalizer_Scope_2618;
      Var_Expr : Internal_Entity_Name;
Var_Expr_1 : Internal_Entity_Expr;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Logic_Var;
Sym : Symbol_Type;
Fld_4 : Internal_Entity;
Fld_5 : Logic_Equation;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_1 : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1113 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_1);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1113;
            procedure Finalizer_Scope_2617 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2617;
            procedure Finalizer_Scope_2618 is
            begin
                     Dec_Ref (Fld_5);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2618;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8425 '<Block>' Let_Result None
--# scope-start
--# expr-start 8424 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14850
--# expr-start 8415 '<FieldAccess .exception_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14851
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Raise_Stmt_F_Exception_Name, Info => Ent.Info);
--# expr-done 8415
Var_Expr := Fld;
if Var_Expr /= No_Entity_Name then
   --# scope-start
   --# bind en Var_Expr
--# expr-start 8416 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14852
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8416
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2617;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14853") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8417 '<FieldAccess .error_message>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14855
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Ent.Node.Raise_Stmt_F_Error_Message, Info => Ent.Info);
--# expr-done 8417
Var_Expr_1 := Fld_2;
if Var_Expr_1 /= No_Entity_Expr then
   --# scope-start
   --# bind er Var_Expr_1
--# expr-start 8423 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14856
--# expr-start 8421 '<FieldAccess .type_bind_val(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14859
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8418 '<FieldAccess .type_var>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14859
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_1.Node);
--# expr-done 8418
--# expr-start 8420 '<FieldAccess .std_entity(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14859
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8419 '<Symbol String>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_String); 
--# expr-done 8419
--# property-call-start AdaNode.std_entity
Fld_4 := Libadalang.Implementation.Ada_Node_P_Std_Entity (Node => Self, Sym => Sym);
--# end
--# expr-done 8420
--# property-call-start AdaNode.type_bind_val
Fld_5 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_3, Right => Fld_4, Origin => Origin);
--# end
--# expr-done 8421
--# expr-start 8422 '<FieldAccess .sub_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14860
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_6 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_1.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8422
And_Pred := Logic_And (Fld_5, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14856") else null)); 
--# expr-done 8423
Scope_Result_1 := And_Pred;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2618;
   Result_Var_1 := Scope_Result_1;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14862") else null)); 
   Result_Var_1 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_1);
And_Pred_1 := Logic_And (Result_Var, Result_Var_1, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14850") else null)); 
--# expr-done 8424
Let_Result := And_Pred_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8425
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1113;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1113;
               Finalizer_Scope_2617;
               Finalizer_Scope_2618;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Raise_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Requeue_Stmt
   --
      procedure Initialize_Fields_For_Requeue_Stmt
        (Self : Bare_Requeue_Stmt
         ; Requeue_Stmt_F_Call_Name : Bare_Name
         ; Requeue_Stmt_F_Has_Abort : Bare_Abort_Node
        ) is
      begin
            Self.Requeue_Stmt_F_Call_Name := Requeue_Stmt_F_Call_Name;
            Self.Requeue_Stmt_F_Has_Abort := Requeue_Stmt_F_Has_Abort;
      end Initialize_Fields_For_Requeue_Stmt;
   function Requeue_Stmt_F_Call_Name
     (Node : Bare_Requeue_Stmt) return Bare_Name
   is
   begin
         return Node.Requeue_Stmt_F_Call_Name;
   end;
   function Requeue_Stmt_F_Has_Abort
     (Node : Bare_Requeue_Stmt) return Bare_Abort_Node
   is
   begin
         return Node.Requeue_Stmt_F_Has_Abort;
   end;
--# property-start RequeueStmt.innermost_entry_or_accept_stmt_params /usr/src/packages/BUILD/ada/ast.py:14749
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params
  (Node : Bare_Requeue_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Holder
is
   Self : Bare_Requeue_Stmt := Bare_Requeue_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Requeue_Stmt :=
     Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1109;
      Match_0 : Internal_Entity_Accept_Stmt_With_Stmts;
Match_1 : Internal_Entity_Entry_Body;
Match_2 : Internal_Entity;
Fld : Internal_Entity_Array_Access;
Is_A : Boolean;
Map_Result : Internal_Entity_Array_Access;
Get_Result : Internal_Entity;
Fld_1 : Internal_Entity_Entry_Completion_Formal_Params;
Fld_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Accept_Stmt_With_Stmts;
Let_Result : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_1 : Internal_Entity_Entry_Body;
Let_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_1 : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_2 : Internal_Entity;
Let_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Scope_Result_2 : Internal_Entity_Entry_Completion_Formal_Params;
Match_Result : Internal_Entity_Entry_Completion_Formal_Params;
Cast_Result_3 : Internal_Entity_Base_Formal_Param_Holder;
            procedure Finalizer_Scope_1109 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1109;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8433 '<Cast to BaseFormalParamHolder.entity>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:14750
--# expr-start 8432 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:14750
--# expr-start 8429 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:14750
--# expr-start 8428 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14750
--# expr-start 8426 '<FieldAccess .parents>' Fld /usr/src/packages/BUILD/ada/ast.py:14750
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parents (Node => Ent.Node, With_Self => True, E_Info => Ent.Info);
--# expr-done 8426
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld;
      begin
         for Item_249 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_249
               --# expr-start 8427 '<IsA AcceptStmtWithStmts, EntryBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:14751
Is_A := Item_249.Node /= null 
and then Item_249.Node.Kind in Ada_Accept_Stmt_With_Stmts_Range | Ada_Entry_Body_Range; 
--# expr-done 8427
               if Is_A then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_249;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8428
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8429
      if Get_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Get_Result; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Accept_Stmt_With_Stmts =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Accept_Stmt_With_Stmts
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind a Match_0
--# expr-start 8430 '<FieldAccess .params>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14753
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_0.Node.Accept_Stmt_F_Params, Info => Match_0.Info);
--# expr-done 8430
Let_Result := Fld_1; 
--# end
Scope_Result := Let_Result;
   --# end
            Match_Result := Scope_Result; 
         when Ada_Entry_Body =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Entry_Body
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind b Match_1
--# expr-start 8431 '<FieldAccess .params>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14754
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Entry_Completion_Formal_Params (Node => Match_1.Node.Entry_Body_F_Params, Info => Match_1.Info);
--# expr-done 8431
Let_Result_1 := Fld_2; 
--# end
Scope_Result_1 := Let_Result_1;
   --# end
            Match_Result := Scope_Result_1; 
         when Ada_Abort_Absent .. Ada_Task_Body_Stub | Ada_Package_Body .. Ada_Accept_Stmt | Ada_For_Loop_Stmt .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := No_Entity_Entry_Completion_Formal_Params; 
--# end
Scope_Result_2 := Let_Result_2;
   --# end
            Match_Result := Scope_Result_2; 
end case;
--# expr-done 8432
      Cast_Result_3 := Create_Internal_Entity_Base_Formal_Param_Holder
        (Node => Match_Result.Node,
         Info => Match_Result.Info);
--# expr-done 8433
      Property_Result := Cast_Result_3;
   --# end
      Finalizer_Scope_1109;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1109;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params;
--# end
--# property-start RequeueStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14760
pragma Warnings (Off, "is not referenced");
 function Requeue_Stmt_P_Xref_Equation
  (Node : Bare_Requeue_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Requeue_Stmt := Bare_Requeue_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Requeue_Stmt :=
     Internal_Entity_Requeue_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1110;
            procedure Finalizer_Scope_2627;
            procedure Finalizer_Scope_2633;
            procedure Finalizer_Scope_2634;
      Block_Var : Internal_Entity_Call_Expr;
Block_Var_1 : Internal_Entity_Name;
Block_Var_2 : Internal_Entity_Array_Access;
Var_Expr : Internal_Entity_Call_Expr;
Var_Expr_1 : Internal_Entity_Entry_Decl;
Var_Expr_2 : Internal_Entity_Base_Subp_Spec;
Fam_Type : Internal_Entity_Base_Type_Decl;
First_Param : Internal_Entity_Expr;
Var_Expr_3 : Internal_Entity_Subtype_Indication;
Var_Expr_4 : Internal_Entity_Entry_Decl;
Var_Expr_5 : Internal_Entity_Basic_Assoc;
Var_Expr_6 : Internal_Entity_Call_Expr;
Var_Expr_7 : Internal_Entity_Expr;
Var_Expr_8 : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Name;
Cast_Result : Internal_Entity_Call_Expr;
Fld_1 : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Fld_2 : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
Fld_3 : Internal_Entity_Array_Access;
Cast_Expr : Internal_Entity;
Cast_Result_1 : Internal_Entity_Entry_Decl;
Fld_4 : Internal_Entity_Base_Subp_Spec;
Fld_5 : Integer;
Is_Equal : Boolean;
Fld_6 : Internal_Entity_Type_Expr;
Is_Null : Boolean;
If_Result : Boolean;
Scope_Result_1 : Boolean;
Result_Var_1 : Boolean;
Fld_7 : Internal_Entity_Entry_Spec;
Fld_8 : Internal_Entity_Base_Formal_Param_Holder;
Fld_9 : Boolean;
If_Result_1 : Boolean;
Scope_Result_2 : Boolean;
Result_Var_2 : Boolean;
Map_Result : Internal_Entity_Array_Access;
Fld_10 : Logic_Equation;
Cast_Expr_1 : Internal_Entity;
Cast_Result_2 : Internal_Entity_Entry_Decl;
Fld_11 : Internal_Entity_Entry_Spec;
Fld_12 : Internal_Entity;
Scope_Result_3 : Internal_Entity;
Result_Var_3 : Internal_Entity;
Cast_Result_3 : Internal_Entity_Subtype_Indication;
Fld_13 : Internal_Entity_Base_Type_Decl;
Scope_Result_4 : Internal_Entity_Base_Type_Decl;
Result_Var_4 : Internal_Entity_Base_Type_Decl;
Fld_14 : Internal_Entity_Assoc_List;
Scope_Result_5 : Internal_Entity_Assoc_List;
Result_Var_5 : Internal_Entity_Assoc_List;
Fld_15 : Bare_Assoc_List;
Fld_16 : Internal_Entity_Info;
As_Entity : Internal_Entity_Basic_Assoc;
Is_Null_1 : Boolean;
If_Result_2 : Internal_Entity_Basic_Assoc;
Item_278 : Internal_Entity_Basic_Assoc;
Bare_Item_278 : Bare_Basic_Assoc;
Map_Result_1 : Internal_Entity_Basic_Assoc_Array_Access;
Get_Result : Internal_Entity_Basic_Assoc;
Fld_17 : Internal_Entity_Expr;
Scope_Result_6 : Internal_Entity_Expr;
Result_Var_6 : Internal_Entity_Expr;
Fld_18 : Logic_Equation;
Fld_19 : Logic_Var;
Cast_Expr_2 : Internal_Entity_Base_Type_Decl;
Cast_Result_4 : Internal_Entity;
Fld_20 : Logic_Equation;
Scope_Result_7 : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var_7 : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result_8 : Logic_Equation;
Logic_True_1 : Logic_Equation;
Result_Var_8 : Logic_Equation;
Let_Result : Logic_Equation;
Map_Result_2 : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred_1 : Logic_Equation;
Let_Result_1 : Logic_Equation;
            procedure Finalizer_Scope_1110 is
            begin
                     Dec_Ref (Fld_3);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Block_Var_2);
                     Dec_Ref (Fld_10);
                     Dec_Ref (Map_Result_2);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred_1);
                     Dec_Ref (Let_Result_1);
            end Finalizer_Scope_1110;
            procedure Finalizer_Scope_2627 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Scope_Result_8);
                     Dec_Ref (Logic_True_1);
                     Dec_Ref (Result_Var_8);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_2627;
            procedure Finalizer_Scope_2633 is
            begin
                     Dec_Ref (Fld_18);
                     Dec_Ref (Scope_Result_7);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var_7);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2633;
            procedure Finalizer_Scope_2634 is
            begin
                     Dec_Ref (Fld_20);
            end Finalizer_Scope_2634;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8467 '<Block>' Let_Result_1 None
--# scope-start
--# expr-start 8435 '<Cast to CallExpr>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14761
--# expr-start 8434 '<FieldAccess .call_name>' Fld /usr/src/packages/BUILD/ada/ast.py:14761
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 8434
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Call_Expr_Range
   then
      Cast_Result := Create_Internal_Entity_Call_Expr
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Call_Expr;
   end if;
--# expr-done 8435
Block_Var := Cast_Result; 
--# bind ce Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Call_Expr then
   --# scope-start
   --# bind ce Var_Expr
--# expr-start 8436 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14762
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Var_Expr.Node.Call_Expr_F_Name, Info => Var_Expr.Info);
--# expr-done 8436
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8437 '<FieldAccess .call_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14763
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Name (Node => Ent.Node.Requeue_Stmt_F_Call_Name, Info => Ent.Info);
--# expr-done 8437
   Result_Var := Fld_2;
end if;
Block_Var_1 := Result_Var; 
--# bind name Block_Var_1
--# expr-start 8448 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14765
--# expr-start 8438 '<FieldAccess .all_env_elements_internal>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14765
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.all_env_elements_internal
Fld_3 := Libadalang.Implementation.Name_P_All_Env_Elements_Internal (Node => Block_Var_1.Node, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8438
declare
   Map_Result_Vec : Internal_Entity_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Fld_3;
      begin
         for Item_250 of
               Collection.Items
         loop
   --# scope-start
               --# bind n Item_250
--# expr-start 8439 '<Cast to EntryDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:14768
Cast_Expr := Item_250; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result_1 := No_Entity_Entry_Decl;
   end if;
--# expr-done 8439
Var_Expr_1 := Cast_Result_1;
if Var_Expr_1 /= No_Entity_Entry_Decl then
   --# scope-start
   --# bind e Var_Expr_1
--# expr-start 8440 '<FieldAccess .subp_spec_or_null>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14770
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.subp_spec_or_null
Fld_4 := Libadalang.Implementation.Basic_Decl_P_Subp_Spec_Or_Null (Node => Var_Expr_1.Node, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8440
Var_Expr_2 := Fld_4;
if Var_Expr_2 /= No_Entity_Base_Subp_Spec then
   --# scope-start
   --# bind ss Var_Expr_2
--# expr-start 8442 '<Eq>' Is_Equal /usr/src/packages/BUILD/ada/ast.py:14771
--# expr-start 8441 '<FieldAccess .nb_max_params>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14771
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseFormalParamHolder.nb_max_params
Fld_5 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Nb_Max_Params (Node => Var_Expr_2.Node);
--# end
--# expr-done 8441
Is_Equal := Fld_5 = 0; 
--# expr-done 8442
if Is_Equal then
   --# expr-start 8444 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:14772
--# expr-start 8443 '<FieldAccess .returns>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14772
      if Var_Expr_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_6 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Var_Expr_2.Node, E_Info => Var_Expr_2.Info);
--# end
--# expr-done 8443
Is_Null := Fld_6.Node = null; 
--# expr-done 8444
   If_Result := Is_Null;
else
   If_Result := False;
end if;
Scope_Result_1 := If_Result;
   --# end
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := False;
end if;
if Result_Var_1 then
   If_Result_1 := True;
else
   --# expr-start 8447 '<FieldAccess .match_formal_params(...)>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:14775
--# expr-start 8445 '<FieldAccess .spec>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:14775
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_7 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_1.Node.Entry_Decl_F_Spec, Info => Var_Expr_1.Info);
--# expr-done 8445
      if Fld_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8446 '<FieldAccess .innermost_entry_or_accept_stmt_params>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:14776
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start RequeueStmt.innermost_entry_or_accept_stmt_params
Fld_8 := Libadalang.Implementation.Requeue_Stmt_P_Innermost_Entry_Or_Accept_Stmt_Params (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8446
--# property-call-start BaseFormalParamHolder.match_formal_params
Fld_9 := Libadalang.Implementation.Base_Formal_Param_Holder_P_Match_Formal_Params (Node => Fld_7.Node, Other => Fld_8, E_Info => Fld_7.Info);
--# end
--# expr-done 8447
   If_Result_1 := Fld_9;
end if;
Scope_Result_2 := If_Result_1;
   --# end
   Result_Var_2 := Scope_Result_2;
else
   Result_Var_2 := False;
end if;
               if Result_Var_2 then
         declare
            Item_To_Append : constant Internal_Entity :=
               Item_250;
         begin
            Internal_Entity_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Array
        (Items_Count => Natural (Internal_Entity_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8448
Block_Var_2 := Map_Result; Inc_Ref (Block_Var_2);
--# bind entries Block_Var_2
--# expr-start 8466 '<And>' And_Pred_1 /usr/src/packages/BUILD/ada/ast.py:14781
--# expr-start 8449 '<FieldAccess .xref_no_overloading>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:14784
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_10 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Block_Var_1.Node, Env => Env, Origin => Origin, E_Info => Block_Var_1.Info);
--# end
--# expr-done 8449
--# expr-start 8465 '<LogicAny>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:14787
--# expr-start 8464 '<map>' Map_Result_2 /usr/src/packages/BUILD/ada/ast.py:14787
declare
   Map_Result_2_Vec : Logic_Equation_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Array_Access := Block_Var_2;
      begin
         for Item_251 of
               Collection.Items
         loop
   --# scope-start
               --# bind e Item_251
      --# expr-start 8463 '<Let fam_type, first_param>' Let_Result /usr/src/packages/BUILD/ada/ast.py:14787
--# scope-start
--# expr-start 8453 '<Cast to SubtypeIndication>' Cast_Result_3 /usr/src/packages/BUILD/ada/ast.py:14791
--# expr-start 8450 '<Cast to EntryDecl>' Cast_Result_2 /usr/src/packages/BUILD/ada/ast.py:14791
Cast_Expr_1 := Item_251; 
   if Cast_Expr_1.Node = null
      or else Cast_Expr_1.Node.Kind in Ada_Entry_Decl_Range
   then
      Cast_Result_2 := Create_Internal_Entity_Entry_Decl
        (Node => Cast_Expr_1.Node,
         Info => Cast_Expr_1.Info);
   else
         Cast_Result_2 := No_Entity_Entry_Decl;
   end if;
--# expr-done 8450
Var_Expr_4 := Cast_Result_2;
if Var_Expr_4 /= No_Entity_Entry_Decl then
   --# scope-start
--# expr-start 8452 '<FieldAccess .family_type>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:14791
--# expr-start 8451 '<FieldAccess .spec>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:14791
      if Var_Expr_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Entry_Spec (Node => Var_Expr_4.Node.Entry_Decl_F_Spec, Info => Var_Expr_4.Info);
--# expr-done 8451
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_12 := Create_Internal_Entity (Node => Fld_11.Node.Entry_Spec_F_Family_Type, Info => Fld_11.Info);
--# expr-done 8452
Scope_Result_3 := Fld_12;
   --# end
   Result_Var_3 := Scope_Result_3;
else
   Result_Var_3 := No_Entity;
end if;
   if Result_Var_3.Node = null
      or else Result_Var_3.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result_3 := Create_Internal_Entity_Subtype_Indication
        (Node => Result_Var_3.Node,
         Info => Result_Var_3.Info);
   else
         Cast_Result_3 := No_Entity_Subtype_Indication;
   end if;
--# expr-done 8453
Var_Expr_3 := Cast_Result_3;
if Var_Expr_3 /= No_Entity_Subtype_Indication then
   --# scope-start
--# expr-start 8454 '<FieldAccess .designated_type>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:14791
      if Var_Expr_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_13 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Var_Expr_3.Node, Origin => Origin, E_Info => Var_Expr_3.Info);
--# end
--# expr-done 8454
Scope_Result_4 := Fld_13;
   --# end
   Result_Var_4 := Scope_Result_4;
else
   Result_Var_4 := No_Entity_Base_Type_Decl;
end if;
Fam_Type := Result_Var_4; 
--# bind fam_type Fam_Type
--# expr-start 8457 '<at(<as_array(<Then <Var ce>: <Var Var_Expr_6> <FieldAccess .params>>)>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:14796
--# expr-start 8456 '<take_while>' Map_Result_1 None
Var_Expr_6 := Block_Var;
if Var_Expr_6 /= No_Entity_Call_Expr then
   --# scope-start
--# expr-start 8455 '<FieldAccess .params>' Fld_14 /usr/src/packages/BUILD/ada/ast.py:14796
      if Var_Expr_6.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start CallExpr.params
Fld_14 := Libadalang.Implementation.Call_Expr_P_Params (Node => Var_Expr_6.Node, E_Info => Var_Expr_6.Info);
--# end
--# expr-done 8455
Scope_Result_5 := Fld_14;
   --# end
   Result_Var_5 := Scope_Result_5;
else
   Result_Var_5 := No_Entity_Assoc_List;
end if;
Fld_15 := Result_Var_5.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Basic_Assoc_Vectors.Vector;
begin
      if Fld_15 = null then
         Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array (0);
      else
      declare
         Collection : constant Bare_Assoc_List := Fld_15;
      begin
         for Untyped_Item_278 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_278 := Untyped_Item_278; 
Is_Null_1 := Bare_Item_278 = null; 
if Is_Null_1 then
   If_Result_2 := No_Entity_Basic_Assoc;
else
Fld_16 := Result_Var_5.Info;
As_Entity := (Info => Fld_16, Node => Bare_Item_278); 
   If_Result_2 := As_Entity;
end if;
                  Item_278 := If_Result_2; 
   --# scope-start
         declare
            Item_To_Append : constant Internal_Entity_Basic_Assoc :=
               Item_278;
         begin
            Internal_Entity_Basic_Assoc_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Basic_Assoc_Array
        (Items_Count => Natural (Internal_Entity_Basic_Assoc_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Basic_Assoc_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Basic_Assoc_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Basic_Assoc_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8456
Get_Result := Get (Map_Result_1, 0, True); 
--# expr-done 8457
Var_Expr_5 := Get_Result;
if Var_Expr_5 /= No_Entity_Basic_Assoc then
   --# scope-start
--# expr-start 8458 '<FieldAccess .expr>' Fld_17 /usr/src/packages/BUILD/ada/ast.py:14796
      if Var_Expr_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_17 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Var_Expr_5.Node, E_Info => Var_Expr_5.Info);
--# end
--# expr-done 8458
Scope_Result_6 := Fld_17;
   --# end
   Result_Var_6 := Scope_Result_6;
else
   Result_Var_6 := No_Entity_Expr;
end if;
First_Param := Result_Var_6; 
--# bind first_param First_Param
Var_Expr_7 := First_Param;
if Var_Expr_7 /= No_Entity_Expr then
   --# scope-start
   --# bind p Var_Expr_7
--# expr-start 8462 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14799
--# expr-start 8459 '<FieldAccess .sub_equation>' Fld_18 /usr/src/packages/BUILD/ada/ast.py:14799
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_18 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr_7.Node, Env => Env, Origin => Origin, E_Info => Var_Expr_7.Info);
--# end
--# expr-done 8459
Var_Expr_8 := Fam_Type;
if Var_Expr_8 /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind eft Var_Expr_8
--# expr-start 8461 '<FieldAccess .type_bind_val(...)>' Fld_20 /usr/src/packages/BUILD/ada/ast.py:14800
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8460 '<FieldAccess .type_var>' Fld_19 /usr/src/packages/BUILD/ada/ast.py:14800
      if Var_Expr_7.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_19 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr_7.Node);
--# expr-done 8460
Cast_Expr_2 := Var_Expr_8; 
      Cast_Result_4 := Create_Internal_Entity
        (Node => Cast_Expr_2.Node,
         Info => Cast_Expr_2.Info);
--# property-call-start AdaNode.type_bind_val
Fld_20 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_19, Right => Cast_Result_4, Origin => Origin);
--# end
--# expr-done 8461
Scope_Result_7 := Fld_20;
   Inc_Ref (Scope_Result_7);
   --# end
      Finalizer_Scope_2634;
   Result_Var_7 := Scope_Result_7;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14801") else null)); 
   Result_Var_7 := Logic_True;
end if;
      Inc_Ref (Result_Var_7);
And_Pred := Logic_And (Fld_18, Result_Var_7, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14799") else null)); 
--# expr-done 8462
Scope_Result_8 := And_Pred;
   Inc_Ref (Scope_Result_8);
   --# end
      Finalizer_Scope_2633;
   Result_Var_8 := Scope_Result_8;
else
   Logic_True_1 := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14802") else null)); 
   Result_Var_8 := Logic_True_1;
end if;
      Inc_Ref (Result_Var_8);
Let_Result := Result_Var_8; Inc_Ref (Let_Result);
--# end
--# expr-done 8463
         declare
            Item_To_Append : constant Logic_Equation :=
               Let_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_2_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2627;
         end loop;
      end;
      Map_Result_2 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_2_Vec)));
      for I in Map_Result_2.Items'Range loop
         Map_Result_2.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_2_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_2.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_2_Vec);
end;
--# expr-done 8464
Logic_Boolean_Op := Logic_Any (Relation_Array (Map_Result_2.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14787") else null)); 
--# expr-done 8465
And_Pred_1 := Logic_And (Fld_10, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14781") else null)); 
--# expr-done 8466
Let_Result_1 := And_Pred_1; Inc_Ref (Let_Result_1);
--# end
--# expr-done 8467
      Property_Result := Let_Result_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1110;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1110;
               Finalizer_Scope_2627;
               Finalizer_Scope_2633;
               Finalizer_Scope_2634;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Requeue_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Return_Stmt
   --
      procedure Initialize_Fields_For_Return_Stmt
        (Self : Bare_Return_Stmt
         ; Return_Stmt_F_Return_Expr : Bare_Expr
        ) is
      begin
            Self.Return_Stmt_F_Return_Expr := Return_Stmt_F_Return_Expr;
      end Initialize_Fields_For_Return_Stmt;
   function Return_Stmt_F_Return_Expr
     (Node : Bare_Return_Stmt) return Bare_Expr
   is
   begin
         return Node.Return_Stmt_F_Return_Expr;
   end;
--# property-start ReturnStmt.subp /usr/src/packages/BUILD/ada/ast.py:14722
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Subp
  (Node : Bare_Return_Stmt
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Subp_Body
is
   Self : Bare_Return_Stmt := Bare_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1107;
      Node_Parents : Bare_Ada_Node_Array_Access;
Is_A : Boolean;
Map_Result : Bare_Ada_Node_Array_Access;
Get_Result : Bare_Ada_Node;
Cast_Result : Bare_Subp_Body;
As_Entity : Internal_Entity_Subp_Body;
Is_Null : Boolean;
If_Result : Internal_Entity_Subp_Body;
            procedure Finalizer_Scope_1107 is
            begin
                     Dec_Ref (Node_Parents);
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1107;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8473 '<as_entity(<Cast to SubpBody>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:14723
--# expr-start 8472 '<Cast to SubpBody>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:14723
--# expr-start 8471 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:14723
--# expr-start 8470 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:14723
--# expr-start 8468 '<FieldAccess .parents>' Node_Parents /usr/src/packages/BUILD/ada/ast.py:14723
Node_Parents := Parents (Self, True); 
--# expr-done 8468
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      declare
         Collection : constant Bare_Ada_Node_Array_Access := Node_Parents;
      begin
         for Item_252 of
               Collection.Items
         loop
   --# scope-start
               --# bind p Item_252
               --# expr-start 8469 '<IsA SubpBody>' Is_A /usr/src/packages/BUILD/ada/ast.py:14723
Is_A := Item_252 /= null 
and then Item_252.Kind in Ada_Subp_Body_Range; 
--# expr-done 8469
               if Is_A then
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Item_252;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8470
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8471
   if Get_Result = null
      or else Get_Result.Kind in Ada_Subp_Body_Range
   then
      Cast_Result := Get_Result;
   else
         Cast_Result := No_Bare_Ada_Node;
   end if;
--# expr-done 8472
Is_Null := Cast_Result = null; 
if Is_Null then
   If_Result := No_Entity_Subp_Body;
else
As_Entity := (Info => E_Info, Node => Cast_Result); 
   If_Result := As_Entity;
end if;
--# expr-done 8473
      Property_Result := If_Result;
   --# end
      Finalizer_Scope_1107;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1107;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Return_Stmt_P_Subp;
--# end
--# property-start ReturnStmt.xref_equation /usr/src/packages/BUILD/ada/ast.py:14728
pragma Warnings (Off, "is not referenced");
 function Return_Stmt_P_Xref_Equation
  (Node : Bare_Return_Stmt
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Return_Stmt := Bare_Return_Stmt (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Return_Stmt :=
     Internal_Entity_Return_Stmt'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1108;
            procedure Finalizer_Scope_2636;
      Var_Expr : Internal_Entity_Expr;
Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Subp_Body;
Fld_4 : Internal_Entity_Subp_Spec;
Fld_5 : Internal_Entity_Type_Expr;
Fld_6 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
And_Pred : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_1108 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_1108;
            procedure Finalizer_Scope_2636 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bind_Result);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2636;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8483 '<Block>' Let_Result None
--# scope-start
--# expr-start 8474 '<FieldAccess .return_expr>' Fld /usr/src/packages/BUILD/ada/ast.py:14729
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Return_Stmt_F_Return_Expr, Info => Ent.Info);
--# expr-done 8474
Var_Expr := Fld;
if Var_Expr /= No_Entity_Expr then
   --# scope-start
   --# bind rexpr Var_Expr
--# expr-start 8482 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:14731
--# expr-start 8475 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:14731
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8475
--# expr-start 8481 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:14732
--# expr-start 8476 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:14733
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Var_Expr.Node);
--# expr-done 8476
Fld_2.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_2.all);
Eq_Node.Refs.Destroy (Fld_2.all);
--# expr-start 8480 '<FieldAccess .designated_type>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:14734
--# expr-start 8479 '<FieldAccess .returns>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:14734
--# expr-start 8478 '<FieldAccess .subp_spec>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:14734
--# expr-start 8477 '<FieldAccess .subp>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:14734
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ReturnStmt.subp
Fld_3 := Libadalang.Implementation.Return_Stmt_P_Subp (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8477
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Subp_Spec (Node => Fld_3.Node.Base_Subp_Body_F_Subp_Spec, Info => Fld_3.Info);
--# expr-done 8478
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseSubpSpec.returns'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Subp_Spec_P_Returns (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 8479
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_6 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_5.Node, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8480
      Cast_Result := Create_Internal_Entity
        (Node => Fld_6.Node,
         Info => Fld_6.Info);
Bind_Result := Bind_Default_566.Create (Fld_2, Cast_Result, No_Logic_Converter_Default, Equals_Data_566'(Origin => Origin), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14732") else null)); 
--# expr-done 8481
And_Pred := Logic_And (Fld_1, Bind_Result, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14731") else null)); 
--# expr-done 8482
Scope_Result := And_Pred;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2636;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:14737") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
Let_Result := Result_Var; Inc_Ref (Let_Result);
--# end
--# expr-done 8483
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1108;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1108;
               Finalizer_Scope_2636;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Return_Stmt_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Terminate_Alternative
   --
--# property-start TerminateAlternative.xref_equation /usr/src/packages/BUILD/ada/ast.py:15195
pragma Warnings (Off, "is not referenced");
 function Terminate_Alternative_P_Xref_Equation
  (Node : Bare_Terminate_Alternative
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Terminate_Alternative := Bare_Terminate_Alternative (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1134;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_1134 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_1134;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15195") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1134;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1134;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Terminate_Alternative_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Subp_Kind
   --
   --
   --  Primitives for Bare_Subp_Kind_Function
   --
   --
   --  Primitives for Bare_Subp_Kind_Procedure
   --
   --
   --  Primitives for Bare_Subunit
   --
      procedure Initialize_Fields_For_Subunit
        (Self : Bare_Subunit
         ; Subunit_F_Name : Bare_Name
         ; Subunit_F_Body : Bare_Body_Node
        ) is
      begin
            Self.Subunit_F_Name := Subunit_F_Name;
            Self.Subunit_F_Body := Subunit_F_Body;
      end Initialize_Fields_For_Subunit;
   function Subunit_F_Name
     (Node : Bare_Subunit) return Bare_Name
   is
   begin
         return Node.Subunit_F_Name;
   end;
   function Subunit_F_Body
     (Node : Bare_Subunit) return Bare_Body_Node
   is
   begin
         return Node.Subunit_F_Body;
   end;
--# property-start Subunit.env_hook_subunit /usr/src/packages/BUILD/ada/ast.py:15555
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Env_Hook_Subunit
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
            procedure Finalizer_Scope_1151;
      Var_Expr : Internal_Unit;
Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Internal_Unit;
Scope_Result : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_1151 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1151;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8487 '<Block>' Let_Result None
--# scope-start
--# expr-start 8486 '<FieldAccess .get_unit(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15561
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8485 '<FieldAccess .as_symbol_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15561
--# expr-start 8484 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15561
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8484
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 8485
Fld_2 := Libadalang.Implementation.Extensions.Ada_Node_P_Get_Unit (Node => Self, Name => Fld_1, Kind => Unit_Body, Load_If_Needed => True);
--# expr-done 8486
Var_Expr := Fld_2;
if Var_Expr /= null then
   --# scope-start
   --# bind _ Var_Expr
Scope_Result := False;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := False;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8487
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1151;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1151;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Env_Hook_Subunit;
--# end
--# property-start Subunit.body_root /usr/src/packages/BUILD/ada/ast.py:15566
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Body_Root
  (Node : Bare_Subunit
  )
   return Internal_Entity_Basic_Decl
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1152;
      Fld : Bare_Name;
Fld_1 : Symbol_Type_Array_Access;
Fld_2 : Bare_Basic_Decl;
As_Entity : Internal_Entity_Basic_Decl;
Is_Null : Boolean;
If_Result : Internal_Entity_Basic_Decl;
Let_Result : Internal_Entity_Basic_Decl;
            procedure Finalizer_Scope_1152 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_1152;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8492 '<Block>' Let_Result None
--# scope-start
--# expr-start 8491 '<as_bare_entity(<FieldAccess .get_unit_root_decl(...)>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:15570
--# expr-start 8490 '<FieldAccess .get_unit_root_decl(...)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15570
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8489 '<FieldAccess .as_symbol_array>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15571
--# expr-start 8488 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15571
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8488
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_1 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld);
--# end
--# expr-done 8489
--# property-call-start AdaNode.get_unit_root_decl
Fld_2 := Libadalang.Implementation.Ada_Node_P_Get_Unit_Root_Decl (Node => Self, Name => Fld_1, Kind => Unit_Body);
--# end
--# expr-done 8490
Is_Null := Fld_2 = null; 
if Is_Null then
   If_Result := No_Entity_Basic_Decl;
else
As_Entity := (Info => No_Entity_Info, Node => Fld_2); 
   If_Result := As_Entity;
end if;
--# expr-done 8491
Let_Result := If_Result; 
--# end
--# expr-done 8492
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_1152;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1152;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Body_Root;
--# end
--# property-start Subunit.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:15574
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Entry_Point
  (Node : Bare_Subunit
  )
   return Boolean
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Xref_Entry_Point;
--# end
--# property-start Subunit.xref_equation /usr/src/packages/BUILD/ada/ast.py:15575
pragma Warnings (Off, "is not referenced");
 function Subunit_P_Xref_Equation
  (Node : Bare_Subunit
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subunit := Bare_Subunit (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subunit :=
     Internal_Entity_Subunit'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_1154;
      Fld : Bare_Name;
Fld_1 : Logic_Var;
Fld_2 : Internal_Entity_Basic_Decl;
Cast_Result : Internal_Entity;
Bind_Result : Logic_Equation;
            procedure Finalizer_Scope_1154 is
            begin
                     Dec_Ref (Bind_Result);
            end Finalizer_Scope_1154;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8496 '<Bind>' Bind_Result /usr/src/packages/BUILD/ada/ast.py:15575
--# expr-start 8494 '<FieldAccess .ref_var>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15575
--# expr-start 8493 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:15575
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Subunit_F_Name;
--# expr-done 8493
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.ref_var'
Fld_1 := Libadalang.Implementation.Dispatcher_Name_P_Ref_Var (Node => Fld);
--# end
--# expr-done 8494
Fld_1.Value := No_Entity;
Eq_Node.Refs.Reset (Fld_1.all);
Eq_Node.Refs.Destroy (Fld_1.all);
--# expr-start 8495 '<FieldAccess .body_root>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:15575
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Subunit.body_root
Fld_2 := Libadalang.Implementation.Subunit_P_Body_Root (Node => Ent.Node);
--# end
--# expr-done 8495
      Cast_Result := Create_Internal_Entity
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
Bind_Result := Bind_Default_Default.Create (Fld_1, Cast_Result, No_Logic_Converter_Default, No_Equals_Data_Default, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:15575") else null)); 
--# expr-done 8496
      Property_Result := Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1154;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1154;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subunit_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Synchronized_Node
   --
--# property-start '[dispatcher]Synchronized.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Synchronized_Node_P_As_Bool
  (Node : Bare_Synchronized_Node
  )
   return Boolean
is
   Self : Bare_Synchronized_Node := Bare_Synchronized_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Synchronized_Node (Self.Kind) is
               when Ada_Synchronized_Absent =>
                  --# property-call-start Synchronized.Absent.as_bool
                  Property_Result := Synchronized_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Synchronized_Present =>
                  --# property-call-start Synchronized.Present.as_bool
                  Property_Result := Synchronized_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Synchronized_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Absent
   --
--# property-start Synchronized.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3951
pragma Warnings (Off, "is not referenced");
 function Synchronized_Absent_P_As_Bool
  (Node : Bare_Synchronized_Absent
  )
   return Boolean
is
   Self : Bare_Synchronized_Absent := Bare_Synchronized_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Synchronized_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Synchronized_Present
   --
--# property-start Synchronized.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3951
pragma Warnings (Off, "is not referenced");
 function Synchronized_Present_P_As_Bool
  (Node : Bare_Synchronized_Present
  )
   return Boolean
is
   Self : Bare_Synchronized_Present := Bare_Synchronized_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Synchronized_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Node
   --
--# property-start '[dispatcher]Tagged.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Tagged_Node_P_As_Bool
  (Node : Bare_Tagged_Node
  )
   return Boolean
is
   Self : Bare_Tagged_Node := Bare_Tagged_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Tagged_Node (Self.Kind) is
               when Ada_Tagged_Absent =>
                  --# property-call-start Tagged.Absent.as_bool
                  Property_Result := Tagged_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Tagged_Present =>
                  --# property-call-start Tagged.Present.as_bool
                  Property_Result := Tagged_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Tagged_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Absent
   --
--# property-start Tagged.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3855
pragma Warnings (Off, "is not referenced");
 function Tagged_Absent_P_As_Bool
  (Node : Bare_Tagged_Absent
  )
   return Boolean
is
   Self : Bare_Tagged_Absent := Bare_Tagged_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Tagged_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Tagged_Present
   --
--# property-start Tagged.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3855
pragma Warnings (Off, "is not referenced");
 function Tagged_Present_P_As_Bool
  (Node : Bare_Tagged_Present
  )
   return Boolean
is
   Self : Bare_Tagged_Present := Bare_Tagged_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Tagged_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Task_Def
   --
      procedure Initialize_Fields_For_Task_Def
        (Self : Bare_Task_Def
         ; Task_Def_F_Interfaces : Bare_Parent_List
         ; Task_Def_F_Public_Part : Bare_Public_Part
         ; Task_Def_F_Private_Part : Bare_Private_Part
         ; Task_Def_F_End_Name : Bare_End_Name
        ) is
      begin
            Self.Task_Def_F_Interfaces := Task_Def_F_Interfaces;
            Self.Task_Def_F_Public_Part := Task_Def_F_Public_Part;
            Self.Task_Def_F_Private_Part := Task_Def_F_Private_Part;
            Self.Task_Def_F_End_Name := Task_Def_F_End_Name;
      end Initialize_Fields_For_Task_Def;
   function Task_Def_F_Interfaces
     (Node : Bare_Task_Def) return Bare_Parent_List
   is
   begin
         return Node.Task_Def_F_Interfaces;
   end;
   function Task_Def_F_Public_Part
     (Node : Bare_Task_Def) return Bare_Public_Part
   is
   begin
         return Node.Task_Def_F_Public_Part;
   end;
   function Task_Def_F_Private_Part
     (Node : Bare_Task_Def) return Bare_Private_Part
   is
   begin
         return Node.Task_Def_F_Private_Part;
   end;
   function Task_Def_F_End_Name
     (Node : Bare_Task_Def) return Bare_End_Name
   is
   begin
         return Node.Task_Def_F_End_Name;
   end;
--# property-start TaskDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6227
pragma Warnings (Off, "is not referenced");
 function Task_Def_P_Xref_Equation
  (Node : Bare_Task_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Task_Def := Bare_Task_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Task_Def :=
     Internal_Entity_Task_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_568;
            procedure Finalizer_Scope_2637;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_253 : Internal_Entity_Name;
Bare_Item_253 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_568 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_568;
            procedure Finalizer_Scope_2637 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2637;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8501 '<Block>' Let_Result None
--# scope-start
--# expr-start 8500 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6228
--# expr-start 8499 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6228
--# expr-start 8497 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:6228
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Task_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8497
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_253 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_253 := Untyped_Item_253; 
Is_Null := Bare_Item_253 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_253); 
   If_Result := As_Entity;
end if;
                  Item_253 := If_Result; 
   --# scope-start
               --# bind ifc Item_253
      --# expr-start 8498 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6228
      if Item_253.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_253.Node, Env => Env, Origin => Origin, E_Info => Item_253.Info);
--# end
--# expr-done 8498
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2637;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8499
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6228") else null)); 
--# expr-done 8500
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 8501
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_568;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_568;
               Finalizer_Scope_2637;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Task_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Type_Def
   --
--# property-start '[dispatcher]TypeDef.discrete_range' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.discrete_range
                  Property_Result := Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.discrete_range
                  Property_Result := Derived_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.discrete_range
                  Property_Result := Mod_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.discrete_range
                  Property_Result := Signed_Int_Type_Def_P_Discrete_Range
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Discrete_Range;
--# end
--# property-start '[dispatcher]TypeDef.array_ndims' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.array_ndims
                  Property_Result := Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.array_ndims
                  Property_Result := Array_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.array_ndims
                  Property_Result := Derived_Type_Def_P_Array_Ndims
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_real_type /usr/src/packages/BUILD/ada/ast.py:3428
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Real_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8502 '<FieldAccess .is_float_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3429
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_float_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Float_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8502
if Fld then
   If_Result := True;
else
   --# expr-start 8503 '<FieldAccess .is_fixed_point>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3429
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Fixed_Point (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8503
   If_Result := Fld_1;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Real_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_float_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Decimal_Fixed_Point_Def | Ada_Ordinary_Fixed_Point_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_float_type
                  Property_Result := Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_float_type
                  Property_Result := Derived_Type_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Floating_Point_Def =>
                  --# property-call-start FloatingPointDef.is_float_type
                  Property_Result := Floating_Point_Def_P_Is_Float_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Float_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_fixed_point' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Private_Type_Def | Ada_Floating_Point_Def | Ada_Record_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_fixed_point
                  Property_Result := Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_fixed_point
                  Property_Result := Derived_Type_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def =>
                  --# property-call-start DecimalFixedPointDef.is_fixed_point
                  Property_Result := Decimal_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start OrdinaryFixedPointDef.is_fixed_point
                  Property_Result := Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start '[dispatcher]TypeDef.is_discrete_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Enum_Type_Def | Ada_Interface_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_discrete_type
                  Property_Result := Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Formal_Discrete_Type_Def =>
                  --# property-call-start FormalDiscreteTypeDef.is_discrete_type
                  Property_Result := Formal_Discrete_Type_Def_P_Is_Discrete_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Discrete_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_int_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def .. Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_int_type
                  Property_Result := Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_int_type
                  Property_Result := Derived_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_int_type
                  Property_Result := Mod_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_int_type
                  Property_Result := Signed_Int_Type_Def_P_Is_Int_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Int_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_access_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_access_type
                  Property_Result := Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
                  --# property-call-start AccessDef.is_access_type
                  Property_Result := Access_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_access_type
                  Property_Result := Derived_Type_Def_P_Is_Access_Type
                    (Self
                        , Origin
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Access_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_char_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_char_type
                  Property_Result := Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_char_type
                  Property_Result := Derived_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_char_type
                  Property_Result := Enum_Type_Def_P_Is_Char_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Char_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_enum_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Formal_Discrete_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_enum_type
                  Property_Result := Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_enum_type
                  Property_Result := Derived_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_enum_type
                  Property_Result := Enum_Type_Def_P_Is_Enum_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Enum_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_record_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Record_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_record_type
                  Property_Result := Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_record_type
                  Property_Result := Derived_Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_record_type
                  Property_Result := Record_Type_Def_P_Is_Record_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Record_Type;
--# end
--# property-start '[dispatcher]TypeDef.accessed_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.accessed_type
                  Property_Result := Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Access_To_Subp_Def =>
                  --# property-call-start AccessToSubpDef.accessed_type
                  Property_Result := Access_To_Subp_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Anonymous_Type_Access_Def =>
                  --# property-call-start AnonymousTypeAccessDef.accessed_type
                  Property_Result := Anonymous_Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Type_Access_Def =>
                  --# property-call-start TypeAccessDef.accessed_type
                  Property_Result := Type_Access_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.accessed_type
                  Property_Result := Derived_Type_Def_P_Accessed_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Accessed_Type;
--# end
--# property-start '[dispatcher]TypeDef.is_tagged_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def | Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def | Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.is_tagged_type
                  Property_Result := Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_tagged_type
                  Property_Result := Derived_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.is_tagged_type
                  Property_Result := Interface_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Private_Type_Def =>
                  --# property-call-start PrivateTypeDef.is_tagged_type
                  Property_Result := Private_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Record_Type_Def =>
                  --# property-call-start RecordTypeDef.is_tagged_type
                  Property_Result := Record_Type_Def_P_Is_Tagged_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start '[dispatcher]TypeDef.base_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_type
                  Property_Result := Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_type
                  Property_Result := Derived_Type_Def_P_Base_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_types /usr/src/packages/BUILD/ada/ast.py:3488
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Types
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_237;
            procedure Finalizer_Scope_2031;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Singleton : Internal_Entity_Base_Type_Decl_Array_Access;
Scope_Result : Internal_Entity_Base_Type_Decl_Array_Access;
Result_Var : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_1 : Internal_Entity_Base_Type_Decl_Array_Access;
Concat_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_237 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Concat_Result);
            end Finalizer_Scope_237;
            procedure Finalizer_Scope_2031 is
            begin
                     Dec_Ref (Singleton);
            end Finalizer_Scope_2031;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8506 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:3489
--# expr-start 8504 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3489
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8504
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
                Singleton := Create_Internal_Entity_Base_Type_Decl_Array (Items_Count => 1);
                Singleton.Items (1) := Var_Expr;
Scope_Result := Singleton;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2031;
   Result_Var := Scope_Result;
else
   Result_Var := No_Internal_Entity_Base_Type_Decl_Array_Type;
end if;
      Inc_Ref (Result_Var);
--# expr-start 8505 '<FieldAccess .base_interfaces>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3490
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_interfaces'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Interfaces (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8505
Concat_Result := Concat (Result_Var, Fld_1); 
--# expr-done 8506
      Property_Result := Concat_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_237;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_237;
               Finalizer_Scope_2031;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Types;
--# end
--# property-start '[dispatcher]TypeDef.base_interfaces' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Array_Type_Def | Ada_Enum_Type_Def .. Ada_Formal_Discrete_Type_Def | Ada_Mod_Int_Type_Def .. Ada_Signed_Int_Type_Def =>
                  --# property-call-start TypeDef.base_interfaces
                  Property_Result := Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.base_interfaces
                  Property_Result := Derived_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
               when Ada_Interface_Type_Def =>
                  --# property-call-start InterfaceTypeDef.base_interfaces
                  Property_Result := Interface_Type_Def_P_Base_Interfaces
                    (Self
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.defining_env /usr/src/packages/BUILD/ada/ast.py:3504
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Defining_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_239;
            procedure Finalizer_Scope_2032;
            procedure Finalizer_Scope_2033;
            procedure Finalizer_Scope_2034;
            procedure Finalizer_Scope_2035;
            procedure Finalizer_Scope_2036;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Match_0 : Internal_Entity_Array_Type_Def;
Match_1 : Internal_Entity_Access_Def;
Match_2 : Internal_Entity_Type_Def;
Var_Expr_1 : Internal_Entity_Base_Type_Decl;
Is_A : Boolean;
Fld : Lexical_Env := Empty_Env;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Lexical_Env := Empty_Env;
Array_Lit : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Is_A_1 : Boolean;
Fld_3 : Lexical_Env := Empty_Env;
Fld_4 : Lexical_Env := Empty_Env;
Fld_5 : Lexical_Env := Empty_Env;
Array_Lit_1 : Lexical_Env_Array_Access;
Fld_6 : Internal_Entity_Base_Type_Decl_Array_Access;
Fld_7 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
Map_Result : Lexical_Env_Array_Access;
Concat_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Fld_8 : Internal_Entity_Base_Type_Decl;
Fld_9 : Lexical_Env := Empty_Env;
Fld_10 : Internal_Entity_Base_Type_Decl;
Fld_11 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Result_Var_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Type_Def;
Cast_Result : Internal_Entity_Array_Type_Def;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result_2 : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Access_Def;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_3 : Lexical_Env := Empty_Env;
Cast_Result_2 : Internal_Entity_Type_Def;
Let_Result_2 : Lexical_Env := Empty_Env;
Scope_Result_4 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
If_Result : Lexical_Env := Empty_Env;
If_Result_1 : Lexical_Env := Empty_Env;
Let_Result_3 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_239 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Group_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_6);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Group_Env_1);
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_4);
                     Dec_Ref (Match_Result);
                     Dec_Ref (If_Result);
                     Dec_Ref (If_Result_1);
                     Dec_Ref (Let_Result_3);
            end Finalizer_Scope_239;
            procedure Finalizer_Scope_2032 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_2032;
            procedure Finalizer_Scope_2033 is
            begin
                     Dec_Ref (Fld_7);
            end Finalizer_Scope_2033;
            procedure Finalizer_Scope_2034 is
            begin
                     Dec_Ref (Fld_9);
            end Finalizer_Scope_2034;
            procedure Finalizer_Scope_2035 is
            begin
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Result_Var_1);
            end Finalizer_Scope_2035;
            procedure Finalizer_Scope_2036 is
            begin
                     Dec_Ref (Fld_11);
            end Finalizer_Scope_2036;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8529 '<Block>' Let_Result_3 None
--# scope-start
--# expr-start 8528 '<Cond>' If_Result_1 /usr/src/packages/BUILD/ada/ast.py:3507
--# expr-start 8507 '<IsA RecordTypeDef, PrivateTypeDef>' Is_A /usr/src/packages/BUILD/ada/ast.py:3511
Is_A := Self /= null 
and then Self.Kind in Ada_Record_Type_Def_Range | Ada_Private_Type_Def_Range; 
--# expr-done 8507
if Is_A then
   --# expr-start 8512 '<env_group(<ArrayLiteral>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:3512
--# expr-start 8511 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:3512
--# expr-start 8508 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:3513
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8508
--# expr-start 8509 '<FieldAccess .dottable_subps_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3514
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_1 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8509
--# expr-start 8510 '<FieldAccess .previous_part_env>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3515
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_2 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8510
Array_Lit := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld, 2 => Fld_1, 3 => Fld_2)); 
--# expr-done 8511
Group_Env := Group (Array_Lit, No_Metadata); 
--# expr-done 8512
   If_Result_1 := Group_Env;
else
--# expr-start 8513 '<IsA DerivedTypeDef, InterfaceTypeDef>' Is_A_1 /usr/src/packages/BUILD/ada/ast.py:3520
Is_A_1 := Self /= null 
and then Self.Kind in Ada_Derived_Type_Def_Range | Ada_Interface_Type_Def_Range; 
--# expr-done 8513
if Is_A_1 then
   --# expr-start 8522 '<env_group(<Concat>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:3524
--# expr-start 8521 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:3524
--# expr-start 8517 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:3524
--# expr-start 8514 '<FieldAccess .children_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3525
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8514
--# expr-start 8515 '<FieldAccess .dottable_subps_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3526
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.dottable_subps_env
Fld_4 := Libadalang.Implementation.Type_Def_P_Dottable_Subps_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8515
--# expr-start 8516 '<FieldAccess .previous_part_env>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3527
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part_env
Fld_5 := Libadalang.Implementation.Type_Def_P_Previous_Part_Env (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8516
Array_Lit_1 := Create_Lexical_Env_Array (Internal_Lexical_Env_Array'(1 => Fld_3, 2 => Fld_4, 3 => Fld_5)); 
--# expr-done 8517
--# expr-start 8520 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3529
--# expr-start 8518 '<FieldAccess .base_types>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:3529
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.base_types
Fld_6 := Libadalang.Implementation.Type_Def_P_Base_Types (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8518
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Entity_Base_Type_Decl_Array_Access := Fld_6;
      begin
         for Item_254 of
               Collection.Items
         loop
   --# scope-start
               --# bind bt Item_254
Var_Expr := Item_254;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8519 '<FieldAccess .defining_env>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3529
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_7 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8519
Scope_Result := Fld_7;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2033;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
         declare
            Item_To_Append : constant Lexical_Env :=
               Result_Var;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2032;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8520
Concat_Result := Concat (Array_Lit_1, Map_Result); 
--# expr-done 8521
Group_Env_1 := Group (Concat_Result, No_Metadata); 
--# expr-done 8522
   If_Result := Group_Env_1;
else
   --# expr-start 8527 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:3532
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Type_Def (Match_Prefix.Node.Kind) is
         when Ada_Array_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Array_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind ar Match_0
--# expr-start 8524 '<FieldAccess .defining_env>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3533
--# expr-start 8523 '<FieldAccess .comp_type>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3533
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ArrayTypeDef.comp_type
Fld_8 := Libadalang.Implementation.Array_Type_Def_P_Comp_Type (Node => Match_0.Node, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 8523
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld_8.Node, Origin => Origin, E_Info => Fld_8.Info);
--# end
--# expr-done 8524
Let_Result := Fld_9; Inc_Ref (Let_Result);
--# end
Scope_Result_2 := Let_Result;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2034;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind ac Match_1
--# expr-start 8525 '<FieldAccess .accessed_type>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3537
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.accessed_type'
Fld_10 := Libadalang.Implementation.Dispatcher_Type_Def_P_Accessed_Type (Node => Match_1.Node, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 8525
Var_Expr_1 := Fld_10;
if Var_Expr_1 /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8526 '<FieldAccess .defining_env>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3537
      if Var_Expr_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_11 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr_1.Node, Origin => Origin, E_Info => Var_Expr_1.Info);
--# end
--# expr-done 8526
Scope_Result_1 := Fld_11;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2036;
   Result_Var_1 := Scope_Result_1;
else
   Result_Var_1 := Empty_Env;
end if;
      Inc_Ref (Result_Var_1);
Let_Result_1 := Result_Var_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_3 := Let_Result_1;
   Inc_Ref (Scope_Result_3);
   --# end
      Finalizer_Scope_2035;
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
         when Ada_Derived_Type_Def .. Ada_Signed_Int_Type_Def =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Type_Def
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Let_Result_2 := AST_Envs.Empty_Env; Inc_Ref (Let_Result_2);
--# end
Scope_Result_4 := Let_Result_2;
   Inc_Ref (Scope_Result_4);
   --# end
            Match_Result := Scope_Result_4; Inc_Ref (Match_Result);
end case;
--# expr-done 8527
   If_Result := Match_Result;
end if;
      Inc_Ref (If_Result);
   If_Result_1 := If_Result;
end if;
      Inc_Ref (If_Result_1);
--# expr-done 8528
Let_Result_3 := If_Result_1; Inc_Ref (Let_Result_3);
--# end
--# expr-done 8529
      Property_Result := Let_Result_3;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_239;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_239;
               Finalizer_Scope_2032;
               Finalizer_Scope_2033;
               Finalizer_Scope_2034;
               Finalizer_Scope_2035;
               Finalizer_Scope_2036;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Defining_Env;
--# end
--# property-start TypeDef.containing_type /usr/src/packages/BUILD/ada/ast.py:3543
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Containing_Type
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8531 '<Cast to TypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:3544
--# expr-start 8530 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:3544
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8530
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8531
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Containing_Type;
--# end
--# property-start TypeDef.previous_part /usr/src/packages/BUILD/ada/ast.py:3549
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8533 '<FieldAccess .previous_part(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3549
--# expr-start 8532 '<FieldAccess .containing_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3549
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8532
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.previous_part'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Previous_Part (Node => Fld.Node, Go_To_Incomplete => True, E_Info => Fld.Info);
--# end
--# expr-done 8533
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Previous_Part;
--# end
--# property-start TypeDef.previous_part_env /usr/src/packages/BUILD/ada/ast.py:3551
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Previous_Part_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_242;
            procedure Finalizer_Scope_2030;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Result_Var : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_242 is
            begin
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Result_Var);
            end Finalizer_Scope_242;
            procedure Finalizer_Scope_2030 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_2030;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8534 '<FieldAccess .previous_part>' Fld /usr/src/packages/BUILD/ada/ast.py:3552
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.previous_part
Fld := Libadalang.Implementation.Type_Def_P_Previous_Part (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8534
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8535 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3552
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8535
Scope_Result := Fld_1;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2030;
   Result_Var := Scope_Result;
else
   Result_Var := Empty_Env;
end if;
      Inc_Ref (Result_Var);
      Property_Result := Result_Var;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_242;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_242;
               Finalizer_Scope_2030;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Previous_Part_Env;
--# end
--# property-start TypeDef.dottable_subps_env /usr/src/packages/BUILD/ada/ast.py:3556
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Dottable_Subps_Env
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_243;
      Fld : Internal_Entity_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Fld_2 : Internal_Entity_Info;
Fld_3 : Env_Rebindings;
Rebound_Env : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_243 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Rebound_Env);
            end Finalizer_Scope_243;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8540 '<rebind_env(<FieldAccess .dottable_subps_env>, <FieldAccess .rebindings>)>' Rebound_Env /usr/src/packages/BUILD/ada/ast.py:3562
--# expr-start 8537 '<FieldAccess .dottable_subps_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3562
--# expr-start 8536 '<FieldAccess .containing_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3562
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.containing_type
Fld := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8536
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseTypeDecl.dottable_subps_env
Fld_1 := Libadalang.Implementation.Base_Type_Decl_F_Dottable_Subps_Env (Node => Fld.Node);
--# end
--# expr-done 8537
--# expr-start 8539 '<FieldAccess .rebindings>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3563
--# expr-start 8538 '<FieldAccess .info>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3563
Fld_2 := Ent.Info;
--# expr-done 8538
Fld_3 := Fld_2.Rebindings;
--# expr-done 8539
Rebound_Env := Rebind_Env (Fld_1, Fld_3); 
--# expr-done 8540
      Property_Result := Rebound_Env;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_243;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_243;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Dottable_Subps_Env;
--# end
--# property-start '[dispatcher]TypeDef.is_static' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Def (Self.Kind) is
               when Ada_Access_To_Subp_Def .. Ada_Type_Access_Def | Ada_Formal_Discrete_Type_Def .. Ada_Interface_Type_Def | Ada_Private_Type_Def | Ada_Record_Type_Def =>
                  --# property-call-start TypeDef.is_static
                  Property_Result := Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Array_Type_Def =>
                  --# property-call-start ArrayTypeDef.is_static
                  Property_Result := Array_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Derived_Type_Def =>
                  --# property-call-start DerivedTypeDef.is_static
                  Property_Result := Derived_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Type_Def =>
                  --# property-call-start EnumTypeDef.is_static
                  Property_Result := Enum_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Mod_Int_Type_Def =>
                  --# property-call-start ModIntTypeDef.is_static
                  Property_Result := Mod_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Decimal_Fixed_Point_Def .. Ada_Ordinary_Fixed_Point_Def =>
                  --# property-call-start RealTypeDef.is_static
                  Property_Result := Real_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
               when Ada_Signed_Int_Type_Def =>
                  --# property-call-start SignedIntTypeDef.is_static
                  Property_Result := Signed_Int_Type_Def_P_Is_Static
                    (Self
                        , Imprecise_Fallback
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Def_P_Is_Static;
--# end
--# property-start TypeDef.discrete_range /usr/src/packages/BUILD/ada/ast.py:3413
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Discrete_Range
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Discrete_Range;
      Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8541 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Discrete_Range; 
--# end
--# expr-done 8541
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Discrete_Range;
--# end
--# property-start TypeDef.array_ndims /usr/src/packages/BUILD/ada/ast.py:3419
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Array_Ndims
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := 0;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Array_Ndims;
--# end
--# property-start TypeDef.is_float_type /usr/src/packages/BUILD/ada/ast.py:3434
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Float_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Float_Type;
--# end
--# property-start TypeDef.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:3438
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Fixed_Point
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Fixed_Point;
--# end
--# property-start TypeDef.is_discrete_type /usr/src/packages/BUILD/ada/ast.py:3444
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Discrete_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Def :=
     Internal_Entity_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
Scope_Result : Boolean;
Fld_2 : Boolean;
Fld_3 : Boolean;
If_Result : Boolean;
Fld_4 : Boolean;
If_Result_1 : Boolean;
Result_Var : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8547 '<Block>' Let_Result None
--# scope-start
--# expr-start 8542 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:3445
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8542
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
   --# bind bt Var_Expr
--# expr-start 8543 '<FieldAccess .is_discrete_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3446
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_discrete_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Discrete_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8543
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
--# expr-start 8544 '<FieldAccess .is_int_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_int_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Int_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8544
if Fld_2 then
   If_Result := True;
else
   --# expr-start 8545 '<FieldAccess .is_enum_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_enum_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Enum_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8545
   If_Result := Fld_3;
end if;
if If_Result then
   If_Result_1 := True;
else
   --# expr-start 8546 '<FieldAccess .is_char_type>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3448
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_char_type'
Fld_4 := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Char_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8546
   If_Result_1 := Fld_4;
end if;
   Result_Var := If_Result_1;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8547
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Discrete_Type;
--# end
--# property-start TypeDef.is_int_type /usr/src/packages/BUILD/ada/ast.py:3453
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Int_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8548 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8548
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Int_Type;
--# end
--# property-start TypeDef.is_access_type /usr/src/packages/BUILD/ada/ast.py:3457
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Access_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Access_Type;
--# end
--# property-start TypeDef.is_char_type /usr/src/packages/BUILD/ada/ast.py:3463
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Char_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Char_Type;
--# end
--# property-start TypeDef.is_enum_type /usr/src/packages/BUILD/ada/ast.py:3464
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Enum_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Enum_Type;
--# end
--# property-start TypeDef.is_record_type /usr/src/packages/BUILD/ada/ast.py:3467
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Record_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8549 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8549
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Record_Type;
--# end
--# property-start TypeDef.accessed_type /usr/src/packages/BUILD/ada/ast.py:3471
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Accessed_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8550 '<Block>' Let_Result None
--# scope-start
Let_Result := No_Entity_Base_Type_Decl; 
--# end
--# expr-done 8550
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Accessed_Type;
--# end
--# property-start TypeDef.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:3475
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Tagged_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8551 '<Block>' Let_Result None
--# scope-start
Let_Result := False; 
--# end
--# expr-done 8551
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Tagged_Type;
--# end
--# property-start TypeDef.base_type /usr/src/packages/BUILD/ada/ast.py:3481
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Type
  (Node : Bare_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Entity_Base_Type_Decl;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Type;
--# end
--# property-start TypeDef.base_interfaces /usr/src/packages/BUILD/ada/ast.py:3497
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Base_Interfaces
  (Node : Bare_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := No_Internal_Entity_Base_Type_Decl_Array_Type;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Base_Interfaces;
--# end
--# property-start TypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:3568
pragma Warnings (Off, "is not referenced");
 function Type_Def_P_Is_Static
  (Node : Bare_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Type_Def := Bare_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Access_Def
   --
      procedure Initialize_Fields_For_Access_Def
        (Self : Bare_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Self.Access_Def_F_Has_Not_Null := Access_Def_F_Has_Not_Null;
      end Initialize_Fields_For_Access_Def;
   function Access_Def_F_Has_Not_Null
     (Node : Bare_Access_Def) return Bare_Not_Null
   is
   begin
         return Node.Access_Def_F_Has_Not_Null;
   end;
--# property-start AccessDef.is_access_type /usr/src/packages/BUILD/ada/ast.py:6326
pragma Warnings (Off, "is not referenced");
 function Access_Def_P_Is_Access_Type
  (Node : Bare_Access_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Access_Def := Bare_Access_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_Def_P_Is_Access_Type;
--# end
   --
   --  Primitives for Bare_Access_To_Subp_Def
   --
      procedure Access_To_Subp_Def_Pre_Env_Actions
        (Self            : Bare_Access_To_Subp_Def;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      if Add_To_Env_Only then
         return;
      end if;
      declare
         No_Parent         : constant Boolean :=
            False;
         Transitive_Parent : constant Boolean :=
            Libadalang.Implementation.Env_Trans_Parent_15 (Node => 
Self);
         Resolver          : constant Lexical_Env_Resolver :=
            null;
         Names             : Symbol_Type_Array_Access :=
            null;
      begin
         Add_Env (Self, State, No_Parent, Transitive_Parent, Resolver, Names);
      end;
      end;
      procedure Initialize_Fields_For_Access_To_Subp_Def
        (Self : Bare_Access_To_Subp_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Access_To_Subp_Def_F_Has_Protected : Bare_Protected_Node
         ; Access_To_Subp_Def_F_Subp_Spec : Bare_Subp_Spec
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Access_To_Subp_Def_F_Has_Protected := Access_To_Subp_Def_F_Has_Protected;
            Self.Access_To_Subp_Def_F_Subp_Spec := Access_To_Subp_Def_F_Subp_Spec;
      end Initialize_Fields_For_Access_To_Subp_Def;
   function Access_To_Subp_Def_F_Has_Protected
     (Node : Bare_Access_To_Subp_Def) return Bare_Protected_Node
   is
   begin
         return Node.Access_To_Subp_Def_F_Has_Protected;
   end;
   function Access_To_Subp_Def_F_Subp_Spec
     (Node : Bare_Access_To_Subp_Def) return Bare_Subp_Spec
   is
   begin
         return Node.Access_To_Subp_Def_F_Subp_Spec;
   end;
--# property-start AccessToSubpDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6336
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Xref_Equation
  (Node : Bare_Access_To_Subp_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_582;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_582 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_582;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6336") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_582;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_582;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_To_Subp_Def_P_Xref_Equation;
--# end
--# property-start AccessToSubpDef.accessed_type /usr/src/packages/BUILD/ada/ast.py:6338
pragma Warnings (Off, "is not referenced");
 function Access_To_Subp_Def_P_Accessed_Type
  (Node : Bare_Access_To_Subp_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Access_To_Subp_Def :=
     Internal_Entity_Access_To_Subp_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subp_Spec;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8553 '<FieldAccess .return_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6338
--# expr-start 8552 '<FieldAccess .subp_spec>' Fld /usr/src/packages/BUILD/ada/ast.py:6338
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subp_Spec (Node => Ent.Node.Access_To_Subp_Def_F_Subp_Spec, Info => Ent.Info);
--# expr-done 8552
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BaseSubpSpec.return_type
Fld_1 := Libadalang.Implementation.Base_Subp_Spec_P_Return_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8553
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Access_To_Subp_Def_P_Accessed_Type;
--# end
--# property-start AccessToSubpDef._env_trans_parent_15 /usr/src/packages/BUILD/ada/ast.py:6342
pragma Warnings (Off, "is not referenced");
 function Env_Trans_Parent_15
  (Node : Bare_Access_To_Subp_Def
  )
   return Boolean
is
   Self : Bare_Access_To_Subp_Def := Bare_Access_To_Subp_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Env_Trans_Parent_15;
--# end
   --
   --  Primitives for Bare_Base_Type_Access_Def
   --
      procedure Initialize_Fields_For_Base_Type_Access_Def
        (Self : Bare_Base_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
        ) is
      begin
            Initialize_Fields_For_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
      end Initialize_Fields_For_Base_Type_Access_Def;
   --
   --  Primitives for Bare_Anonymous_Type_Access_Def
   --
      procedure Initialize_Fields_For_Anonymous_Type_Access_Def
        (Self : Bare_Anonymous_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Anonymous_Type_Access_Def_F_Type_Decl : Bare_Base_Type_Decl
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Anonymous_Type_Access_Def_F_Type_Decl := Anonymous_Type_Access_Def_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type_Access_Def;
   function Anonymous_Type_Access_Def_F_Type_Decl
     (Node : Bare_Anonymous_Type_Access_Def) return Bare_Base_Type_Decl
   is
   begin
         return Node.Anonymous_Type_Access_Def_F_Type_Decl;
   end;
--# property-start AnonymousTypeAccessDef.accessed_type /usr/src/packages/BUILD/ada/ast.py:6373
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_Access_Def_P_Accessed_Type
  (Node : Bare_Anonymous_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type_Access_Def := Bare_Anonymous_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type_Access_Def :=
     Internal_Entity_Anonymous_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8554 '<FieldAccess .type_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6373
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Type_Decl (Node => Ent.Node.Anonymous_Type_Access_Def_F_Type_Decl, Info => Ent.Info);
--# expr-done 8554
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_Access_Def_P_Accessed_Type;
--# end
   --
   --  Primitives for Bare_Type_Access_Def
   --
      procedure Initialize_Fields_For_Type_Access_Def
        (Self : Bare_Type_Access_Def
         ; Access_Def_F_Has_Not_Null : Bare_Not_Null
         ; Type_Access_Def_F_Has_All : Bare_All_Node
         ; Type_Access_Def_F_Has_Constant : Bare_Constant_Node
         ; Type_Access_Def_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Initialize_Fields_For_Base_Type_Access_Def
              (Self, Access_Def_F_Has_Not_Null);
            Self.Type_Access_Def_F_Has_All := Type_Access_Def_F_Has_All;
            Self.Type_Access_Def_F_Has_Constant := Type_Access_Def_F_Has_Constant;
            Self.Type_Access_Def_F_Subtype_Indication := Type_Access_Def_F_Subtype_Indication;
      end Initialize_Fields_For_Type_Access_Def;
   function Type_Access_Def_F_Has_All
     (Node : Bare_Type_Access_Def) return Bare_All_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_All;
   end;
   function Type_Access_Def_F_Has_Constant
     (Node : Bare_Type_Access_Def) return Bare_Constant_Node
   is
   begin
         return Node.Type_Access_Def_F_Has_Constant;
   end;
   function Type_Access_Def_F_Subtype_Indication
     (Node : Bare_Type_Access_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Type_Access_Def_F_Subtype_Indication;
   end;
--# property-start TypeAccessDef.accessed_type /usr/src/packages/BUILD/ada/ast.py:6361
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Accessed_Type
  (Node : Bare_Type_Access_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Access_Def := Bare_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Access_Def :=
     Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8556 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6361
--# expr-start 8555 '<FieldAccess .subtype_indication>' Fld /usr/src/packages/BUILD/ada/ast.py:6361
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8555
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8556
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Access_Def_P_Accessed_Type;
--# end
--# property-start TypeAccessDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6362
pragma Warnings (Off, "is not referenced");
 function Type_Access_Def_P_Xref_Equation
  (Node : Bare_Type_Access_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Type_Access_Def := Bare_Type_Access_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Access_Def :=
     Internal_Entity_Type_Access_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_585;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_585 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_585;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8558 '<FieldAccess .xref_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6362
--# expr-start 8557 '<FieldAccess .subtype_indication>' Fld /usr/src/packages/BUILD/ada/ast.py:6362
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Type_Access_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8557
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_1 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8558
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_585;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_585;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Access_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Array_Type_Def
   --
      procedure Initialize_Fields_For_Array_Type_Def
        (Self : Bare_Array_Type_Def
         ; Array_Type_Def_F_Indices : Bare_Array_Indices
         ; Array_Type_Def_F_Component_Type : Bare_Component_Def
        ) is
      begin
            Self.Array_Type_Def_F_Indices := Array_Type_Def_F_Indices;
            Self.Array_Type_Def_F_Component_Type := Array_Type_Def_F_Component_Type;
      end Initialize_Fields_For_Array_Type_Def;
   function Array_Type_Def_F_Indices
     (Node : Bare_Array_Type_Def) return Bare_Array_Indices
   is
   begin
         return Node.Array_Type_Def_F_Indices;
   end;
   function Array_Type_Def_F_Component_Type
     (Node : Bare_Array_Type_Def) return Bare_Component_Def
   is
   begin
         return Node.Array_Type_Def_F_Component_Type;
   end;
--# property-start ArrayTypeDef.comp_type /usr/src/packages/BUILD/ada/ast.py:6061
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Comp_Type
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Component_Def;
Fld_1 : Internal_Entity_Type_Expr;
Fld_2 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8562 '<Block>' Let_Result None
--# scope-start
--# expr-start 8561 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6063
--# expr-start 8560 '<FieldAccess .type_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6063
--# expr-start 8559 '<FieldAccess .component_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6063
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 8559
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Type_Expr (Node => Fld.Node.Component_Def_F_Type_Expr, Info => Fld.Info);
--# expr-done 8560
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8561
Let_Result := Fld_2; 
--# end
--# expr-done 8562
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Comp_Type;
--# end
--# property-start ArrayTypeDef.index_type /usr/src/packages/BUILD/ada/ast.py:6066
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Index_Type
  (Node : Bare_Array_Type_Def
      ; Dim : Integer
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind dim Dim
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8565 '<Block>' Let_Result None
--# scope-start
--# expr-start 8564 '<FieldAccess .index_type(...)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6067
--# expr-start 8563 '<FieldAccess .indices>' Fld /usr/src/packages/BUILD/ada/ast.py:6067
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8563
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.index_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Index_Type (Node => Fld.Node, Dim => Dim, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8564
Let_Result := Fld_1; 
--# end
--# expr-done 8565
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Index_Type;
--# end
--# property-start ArrayTypeDef.array_ndims /usr/src/packages/BUILD/ada/ast.py:6069
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Array_Ndims
  (Node : Bare_Array_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Bare_Array_Indices;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8567 '<FieldAccess .ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6069
--# expr-start 8566 '<FieldAccess .indices>' Fld /usr/src/packages/BUILD/ada/ast.py:6069
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Array_Type_Def_F_Indices;
--# expr-done 8566
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Ndims (Node => Fld);
--# end
--# expr-done 8567
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Array_Ndims;
--# end
--# property-start ArrayTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6072
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Equation
  (Node : Bare_Array_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_524;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Component_Def;
Fld_3 : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_524 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Fld_3);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_524;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8573 '<Block>' Let_Result None
--# scope-start
--# expr-start 8572 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:6073
--# expr-start 8569 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6074
--# expr-start 8568 '<FieldAccess .indices>' Fld /usr/src/packages/BUILD/ada/ast.py:6074
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8568
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8569
--# expr-start 8571 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6075
--# expr-start 8570 '<FieldAccess .component_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6075
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Component_Def (Node => Ent.Node.Array_Type_Def_F_Component_Type, Info => Ent.Info);
--# expr-done 8570
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld_2.Node, Env => Env, Origin => Origin, E_Info => Fld_2.Info);
--# end
--# expr-done 8571
And_Pred := Logic_And (Fld_1, Fld_3, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6073") else null)); 
--# expr-done 8572
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8573
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_524;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_524;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Xref_Equation;
--# end
--# property-start ArrayTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:6078
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Is_Static
  (Node : Bare_Array_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Array_Type_Def :=
     Internal_Entity_Array_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Array_Indices;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8575 '<FieldAccess .is_static>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6078
--# expr-start 8574 '<FieldAccess .indices>' Fld /usr/src/packages/BUILD/ada/ast.py:6078
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Array_Indices (Node => Ent.Node.Array_Type_Def_F_Indices, Info => Ent.Info);
--# expr-done 8574
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]ArrayIndices.is_static'
Fld_1 := Libadalang.Implementation.Dispatcher_Array_Indices_P_Is_Static (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8575
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Is_Static;
--# end
--# property-start ArrayTypeDef.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6080
pragma Warnings (Off, "is not referenced");
 function Array_Type_Def_P_Xref_Entry_Point
  (Node : Bare_Array_Type_Def
  )
   return Boolean
is
   Self : Bare_Array_Type_Def := Bare_Array_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Array_Type_Def_P_Xref_Entry_Point;
--# end
   --
   --  Primitives for Bare_Derived_Type_Def
   --
      procedure Initialize_Fields_For_Derived_Type_Def
        (Self : Bare_Derived_Type_Def
         ; Derived_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Derived_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Derived_Type_Def_F_Has_Synchronized : Bare_Synchronized_Node
         ; Derived_Type_Def_F_Subtype_Indication : Bare_Subtype_Indication
         ; Derived_Type_Def_F_Interfaces : Bare_Parent_List
         ; Derived_Type_Def_F_Record_Extension : Bare_Base_Record_Def
         ; Derived_Type_Def_F_Has_With_Private : Bare_With_Private
        ) is
      begin
            Self.Derived_Type_Def_F_Has_Abstract := Derived_Type_Def_F_Has_Abstract;
            Self.Derived_Type_Def_F_Has_Limited := Derived_Type_Def_F_Has_Limited;
            Self.Derived_Type_Def_F_Has_Synchronized := Derived_Type_Def_F_Has_Synchronized;
            Self.Derived_Type_Def_F_Subtype_Indication := Derived_Type_Def_F_Subtype_Indication;
            Self.Derived_Type_Def_F_Interfaces := Derived_Type_Def_F_Interfaces;
            Self.Derived_Type_Def_F_Record_Extension := Derived_Type_Def_F_Record_Extension;
            Self.Derived_Type_Def_F_Has_With_Private := Derived_Type_Def_F_Has_With_Private;
      end Initialize_Fields_For_Derived_Type_Def;
   function Derived_Type_Def_F_Has_Abstract
     (Node : Bare_Derived_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Abstract;
   end;
   function Derived_Type_Def_F_Has_Limited
     (Node : Bare_Derived_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Limited;
   end;
   function Derived_Type_Def_F_Has_Synchronized
     (Node : Bare_Derived_Type_Def) return Bare_Synchronized_Node
   is
   begin
         return Node.Derived_Type_Def_F_Has_Synchronized;
   end;
   function Derived_Type_Def_F_Subtype_Indication
     (Node : Bare_Derived_Type_Def) return Bare_Subtype_Indication
   is
   begin
         return Node.Derived_Type_Def_F_Subtype_Indication;
   end;
   function Derived_Type_Def_F_Interfaces
     (Node : Bare_Derived_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Derived_Type_Def_F_Interfaces;
   end;
   function Derived_Type_Def_F_Record_Extension
     (Node : Bare_Derived_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Derived_Type_Def_F_Record_Extension;
   end;
   function Derived_Type_Def_F_Has_With_Private
     (Node : Bare_Derived_Type_Def) return Bare_With_Private
   is
   begin
         return Node.Derived_Type_Def_F_Has_With_Private;
   end;
--# property-start DerivedTypeDef.array_ndims /usr/src/packages/BUILD/ada/ast.py:5829
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Array_Ndims
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Integer;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8577 '<FieldAccess .array_ndims>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5829
--# expr-start 8576 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5829
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8576
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8577
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Array_Ndims;
--# end
--# property-start DerivedTypeDef.base_type /usr/src/packages/BUILD/ada/ast.py:5832
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8581 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5833
--# expr-start 8578 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:5833
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8578
Bound_Origin := Fld; 
--# expr-start 8580 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5834
--# expr-start 8579 '<FieldAccess .subtype_indication>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5834
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8579
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8580
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8581
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Base_Type;
--# end
--# property-start DerivedTypeDef.base_interfaces /usr/src/packages/BUILD/ada/ast.py:5837
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Base_Interfaces
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_483;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_255 : Internal_Entity_Name;
Bare_Item_255 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_483 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_483;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8584 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5838
--# expr-start 8582 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:5838
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8582
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_255 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_255 := Untyped_Item_255; 
Is_Null := Bare_Item_255 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_255); 
   If_Result := As_Entity;
end if;
                  Item_255 := If_Result; 
   --# scope-start
               --# bind i Item_255
      --# expr-start 8583 '<FieldAccess .name_designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5838
      if Item_255.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_255.Node, E_Info => Item_255.Info);
--# end
--# expr-done 8583
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8584
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_483;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_483;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Base_Interfaces;
--# end
--# property-start DerivedTypeDef.is_int_type /usr/src/packages/BUILD/ada/ast.py:5841
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Int_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8586 '<FieldAccess .is_int_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5841
--# expr-start 8585 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5841
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8585
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_int_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Int_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8586
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Int_Type;
--# end
--# property-start DerivedTypeDef.is_access_type /usr/src/packages/BUILD/ada/ast.py:5842
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Access_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      As_Entity : Internal_Entity_Derived_Type_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Derived_Type_Def;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8589 '<FieldAccess .is_access_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5842
--# expr-start 8588 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5842
--# expr-start 8587 '<as_bare_entity(<Var Self>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:5842
Is_Null := Self = null; 
if Is_Null then
   If_Result := No_Entity_Derived_Type_Def;
else
As_Entity := (Info => No_Entity_Info, Node => Self); 
   If_Result := As_Entity;
end if;
--# expr-done 8587
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => If_Result.Node, Origin => Origin, E_Info => If_Result.Info);
--# end
--# expr-done 8588
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_access_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Access_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8589
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Access_Type;
--# end
--# property-start DerivedTypeDef.is_char_type /usr/src/packages/BUILD/ada/ast.py:5843
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Char_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8591 '<FieldAccess .is_char_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5843
--# expr-start 8590 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5843
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8590
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_char_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Char_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8591
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Char_Type;
--# end
--# property-start DerivedTypeDef.is_float_type /usr/src/packages/BUILD/ada/ast.py:5844
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Float_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8593 '<FieldAccess .is_float_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5844
--# expr-start 8592 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5844
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8592
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_float_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Float_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8593
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Float_Type;
--# end
--# property-start DerivedTypeDef.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:5845
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Fixed_Point
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8595 '<FieldAccess .is_fixed_point>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5845
--# expr-start 8594 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5845
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8594
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_fixed_point'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Fixed_Point (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8595
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Fixed_Point;
--# end
--# property-start DerivedTypeDef.accessed_type /usr/src/packages/BUILD/ada/ast.py:5846
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Accessed_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8597 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5846
--# expr-start 8596 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5846
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8596
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8597
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Accessed_Type;
--# end
--# property-start DerivedTypeDef.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:5847
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Record_Def;
Is_Null : Boolean;
Not_Val : Boolean;
Fld_1 : Internal_Entity_With_Private;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8600 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:5848
--# expr-start 8599 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:5848
--# expr-start 8598 '<FieldAccess .record_extension>' Fld /usr/src/packages/BUILD/ada/ast.py:5848
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Base_Record_Def (Node => Ent.Node.Derived_Type_Def_F_Record_Extension, Info => Ent.Info);
--# expr-done 8598
Is_Null := Fld.Node = null; 
--# expr-done 8599
Not_Val := not (Is_Null); 
--# expr-done 8600
if Not_Val then
   If_Result := True;
else
   --# expr-start 8602 '<FieldAccess .as_bool>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5848
--# expr-start 8601 '<FieldAccess .has_with_private>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5848
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_With_Private (Node => Ent.Node.Derived_Type_Def_F_Has_With_Private, Info => Ent.Info);
--# expr-done 8601
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]WithPrivate.as_bool'
Fld_2 := Libadalang.Implementation.Dispatcher_With_Private_P_As_Bool (Node => Fld_1.Node);
--# end
--# expr-done 8602
   If_Result := Fld_2;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start DerivedTypeDef.is_enum_type /usr/src/packages/BUILD/ada/ast.py:5851
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Enum_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8604 '<FieldAccess .is_enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5851
--# expr-start 8603 '<FieldAccess .base_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5851
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8603
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_enum_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Enum_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8604
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Enum_Type;
--# end
--# property-start DerivedTypeDef.is_record_type /usr/src/packages/BUILD/ada/ast.py:5852
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Record_Type
  (Node : Bare_Derived_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Boolean;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Boolean;
If_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
--# expr-start 8605 '<FieldAccess .is_tagged_type>' Fld /usr/src/packages/BUILD/ada/ast.py:5853
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.is_tagged_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Def_P_Is_Tagged_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8605
if Fld then
   If_Result := True;
else
   --# expr-start 8607 '<FieldAccess .is_record_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5853
--# expr-start 8606 '<FieldAccess .base_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5853
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeDef.base_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Def_P_Base_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8606
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.is_record_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Is_Record_Type (Node => Fld_1.Node, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8607
   If_Result := Fld_2;
end if;
      Property_Result := If_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Record_Type;
--# end
--# property-start DerivedTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:5855
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Is_Static
  (Node : Bare_Derived_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8609 '<FieldAccess .is_static_subtype>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5855
--# expr-start 8608 '<FieldAccess .subtype_indication>' Fld /usr/src/packages/BUILD/ada/ast.py:5855
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8608
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.is_static_subtype
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Is_Static_Subtype (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8609
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Is_Static;
--# end
--# property-start DerivedTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:5858
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Xref_Equation
  (Node : Bare_Derived_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_494;
            procedure Finalizer_Scope_2639;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Subtype_Indication;
Fld_2 : Logic_Equation;
Fld_3 : Internal_Entity_Parent_List;
Fld_4 : Bare_Parent_List;
Fld_5 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_6 : Logic_Equation;
Item_256 : Internal_Entity_Name;
Bare_Item_256 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
And_Pred : Logic_Equation;
Dyn_Var_Bind_Result : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_494 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_2);
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_494;
            procedure Finalizer_Scope_2639 is
            begin
                     Dec_Ref (Fld_6);
            end Finalizer_Scope_2639;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8619 '<Block>' Let_Result None
--# scope-start
--# expr-start 8618 '<bind(<DynamicVariable env>, <FieldAccess .children_env>, <And>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:5861
--# expr-start 8610 '<FieldAccess .children_env>' Fld /usr/src/packages/BUILD/ada/ast.py:5861
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Children_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8610
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8617 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5862
--# expr-start 8612 '<FieldAccess .xref_equation>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5862
--# expr-start 8611 '<FieldAccess .subtype_indication>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5862
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8611
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_2 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8612
--# expr-start 8616 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:5863
--# expr-start 8615 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:5863
--# expr-start 8613 '<FieldAccess .interfaces>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5863
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Parent_List (Node => Ent.Node.Derived_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8613
Fld_4 := Fld_3.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_4 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_4;
      begin
         for Untyped_Item_256 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_256 := Untyped_Item_256; 
Is_Null := Bare_Item_256 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_5 := Fld_3.Info;
As_Entity := (Info => Fld_5, Node => Bare_Item_256); 
   If_Result := As_Entity;
end if;
                  Item_256 := If_Result; 
   --# scope-start
               --# bind ifc Item_256
      --# expr-start 8614 '<FieldAccess .xref_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5863
      if Item_256.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_256.Node, Env => Bound_Env, Origin => Origin, E_Info => Item_256.Info);
--# end
--# expr-done 8614
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_6;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2639;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8615
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5863") else null)); 
--# expr-done 8616
And_Pred := Logic_And (Fld_2, Logic_Boolean_Op, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5862") else null)); 
--# expr-done 8617
Dyn_Var_Bind_Result := And_Pred; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8618
Let_Result := Dyn_Var_Bind_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8619
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_494;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_494;
               Finalizer_Scope_2639;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Xref_Equation;
--# end
--# property-start DerivedTypeDef.discrete_range /usr/src/packages/BUILD/ada/ast.py:5867
pragma Warnings (Off, "is not referenced");
 function Derived_Type_Def_P_Discrete_Range
  (Node : Bare_Derived_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Derived_Type_Def := Bare_Derived_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Derived_Type_Def :=
     Internal_Entity_Derived_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8622 '<Block>' Let_Result None
--# scope-start
--# expr-start 8621 '<FieldAccess .discrete_range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5868
--# expr-start 8620 '<FieldAccess .subtype_indication>' Fld /usr/src/packages/BUILD/ada/ast.py:5868
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Derived_Type_Def_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8620
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start SubtypeIndication.discrete_range
Fld_1 := Libadalang.Implementation.Subtype_Indication_P_Discrete_Range (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8621
Let_Result := Fld_1; 
--# end
--# expr-done 8622
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Derived_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Enum_Type_Def
   --
      procedure Initialize_Fields_For_Enum_Type_Def
        (Self : Bare_Enum_Type_Def
         ; Enum_Type_Def_F_Enum_Literals : Bare_Enum_Literal_Decl_List
        ) is
      begin
            Self.Enum_Type_Def_F_Enum_Literals := Enum_Type_Def_F_Enum_Literals;
      end Initialize_Fields_For_Enum_Type_Def;
   function Enum_Type_Def_F_Enum_Literals
     (Node : Bare_Enum_Type_Def) return Bare_Enum_Literal_Decl_List
   is
   begin
         return Node.Enum_Type_Def_F_Enum_Literals;
   end;
--# property-start EnumTypeDef.is_char_type /usr/src/packages/BUILD/ada/ast.py:5593
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Char_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Enum_Literal_Decl_List;
Fld_1 : Bare_Defining_Name;
Fld_2 : Bare_Name;
Is_A : Boolean;
Item_257 : Bare_Enum_Literal_Decl;
Quantifier_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8623 '<FieldAccess .enum_literals>' Fld /usr/src/packages/BUILD/ada/ast.py:5593
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Enum_Type_Def_F_Enum_Literals;
--# expr-done 8623
Quantifier_Result := False;
   if Fld /= null then
   declare
      Collection : constant Bare_Enum_Literal_Decl_List := Fld;
   begin
      for Untyped_Item_257 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Item_257 := Untyped_Item_257; 
   --# scope-start
            --# bind lit Item_257
         --# expr-start 8626 '<IsA CharLiteral>' Is_A /usr/src/packages/BUILD/ada/ast.py:5594
--# expr-start 8625 '<FieldAccess .name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5594
--# expr-start 8624 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5594
      if Item_257 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Item_257.Enum_Literal_Decl_F_Name;
--# expr-done 8624
      if Fld_1 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Fld_1.Defining_Name_F_Name;
--# expr-done 8625
Is_A := Fld_2 /= null 
and then Fld_2.Kind in Ada_Char_Literal_Range; 
--# expr-done 8626
            if Is_A then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
      Property_Result := Quantifier_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Char_Type;
--# end
--# property-start EnumTypeDef.is_enum_type /usr/src/packages/BUILD/ada/ast.py:5597
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Enum_Type
  (Node : Bare_Enum_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Enum_Type;
--# end
--# property-start EnumTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:5599
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Xref_Equation
  (Node : Bare_Enum_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_463;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_463 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_463;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5599") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_463;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_463;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Xref_Equation;
--# end
--# property-start EnumTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:5601
pragma Warnings (Off, "is not referenced");
 function Enum_Type_Def_P_Is_Static
  (Node : Bare_Enum_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Enum_Type_Def := Bare_Enum_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Formal_Discrete_Type_Def
   --
--# property-start FormalDiscreteTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6380
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Xref_Equation
  (Node : Bare_Formal_Discrete_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Formal_Discrete_Type_Def := Bare_Formal_Discrete_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_587;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_587 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_587;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6380") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_587;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_587;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Discrete_Type_Def_P_Xref_Equation;
--# end
--# property-start FormalDiscreteTypeDef.is_discrete_type /usr/src/packages/BUILD/ada/ast.py:6382
pragma Warnings (Off, "is not referenced");
 function Formal_Discrete_Type_Def_P_Is_Discrete_Type
  (Node : Bare_Formal_Discrete_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Formal_Discrete_Type_Def := Bare_Formal_Discrete_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Formal_Discrete_Type_Def_P_Is_Discrete_Type;
--# end
   --
   --  Primitives for Bare_Interface_Type_Def
   --
      procedure Initialize_Fields_For_Interface_Type_Def
        (Self : Bare_Interface_Type_Def
         ; Interface_Type_Def_F_Interface_Kind : Bare_Interface_Kind
         ; Interface_Type_Def_F_Interfaces : Bare_Parent_List
        ) is
      begin
            Self.Interface_Type_Def_F_Interface_Kind := Interface_Type_Def_F_Interface_Kind;
            Self.Interface_Type_Def_F_Interfaces := Interface_Type_Def_F_Interfaces;
      end Initialize_Fields_For_Interface_Type_Def;
   function Interface_Type_Def_F_Interface_Kind
     (Node : Bare_Interface_Type_Def) return Bare_Interface_Kind
   is
   begin
         return Node.Interface_Type_Def_F_Interface_Kind;
   end;
   function Interface_Type_Def_F_Interfaces
     (Node : Bare_Interface_Type_Def) return Bare_Parent_List
   is
   begin
         return Node.Interface_Type_Def_F_Interfaces;
   end;
--# property-start InterfaceTypeDef.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:6098
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Interface_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start InterfaceTypeDef.base_interfaces /usr/src/packages/BUILD/ada/ast.py:6100
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Base_Interfaces
  (Node : Bare_Interface_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl_Array_Access
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Interface_Type_Def :=
     Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_528;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Internal_Entity_Base_Type_Decl;
Item_258 : Internal_Entity_Name;
Bare_Item_258 : Bare_Name;
Map_Result : Internal_Entity_Base_Type_Decl_Array_Access;
            procedure Finalizer_Scope_528 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_528;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8629 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6101
--# expr-start 8627 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:6101
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8627
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Internal_Entity_Base_Type_Decl_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Internal_Entity_Base_Type_Decl_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_258 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_258 := Untyped_Item_258; 
Is_Null := Bare_Item_258 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_258); 
   If_Result := As_Entity;
end if;
                  Item_258 := If_Result; 
   --# scope-start
               --# bind i Item_258
      --# expr-start 8628 '<FieldAccess .name_designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6101
      if Item_258.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type
Fld_3 := Libadalang.Implementation.Name_P_Name_Designated_Type (Node => Item_258.Node, E_Info => Item_258.Info);
--# end
--# expr-done 8628
         declare
            Item_To_Append : constant Internal_Entity_Base_Type_Decl :=
               Fld_3;
         begin
            Internal_Entity_Base_Type_Decl_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Entity_Base_Type_Decl_Array
        (Items_Count => Natural (Internal_Entity_Base_Type_Decl_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Entity_Base_Type_Decl_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Entity_Base_Type_Decl_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Entity_Base_Type_Decl_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8629
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_528;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_528;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Base_Interfaces;
--# end
--# property-start InterfaceTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:6105
pragma Warnings (Off, "is not referenced");
 function Interface_Type_Def_P_Xref_Equation
  (Node : Bare_Interface_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Interface_Type_Def := Bare_Interface_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Interface_Type_Def :=
     Internal_Entity_Interface_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_529;
            procedure Finalizer_Scope_2642;
      Fld : Internal_Entity_Parent_List;
Fld_1 : Bare_Parent_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_259 : Internal_Entity_Name;
Bare_Item_259 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_529 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_529;
            procedure Finalizer_Scope_2642 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2642;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8634 '<Block>' Let_Result None
--# scope-start
--# expr-start 8633 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6106
--# expr-start 8632 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6106
--# expr-start 8630 '<FieldAccess .interfaces>' Fld /usr/src/packages/BUILD/ada/ast.py:6106
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Parent_List (Node => Ent.Node.Interface_Type_Def_F_Interfaces, Info => Ent.Info);
--# expr-done 8630
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Parent_List := Fld_1;
      begin
         for Untyped_Item_259 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_259 := Untyped_Item_259; 
Is_Null := Bare_Item_259 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_259); 
   If_Result := As_Entity;
end if;
                  Item_259 := If_Result; 
   --# scope-start
               --# bind ifc Item_259
      --# expr-start 8631 '<FieldAccess .xref_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6106
      if Item_259.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_3 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Item_259.Node, Env => Env, Origin => Origin, E_Info => Item_259.Info);
--# end
--# expr-done 8631
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2642;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8632
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6106") else null)); 
--# expr-done 8633
Let_Result := Logic_Boolean_Op; Inc_Ref (Let_Result);
--# end
--# expr-done 8634
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_529;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_529;
               Finalizer_Scope_2642;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Interface_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Mod_Int_Type_Def
   --
      procedure Initialize_Fields_For_Mod_Int_Type_Def
        (Self : Bare_Mod_Int_Type_Def
         ; Mod_Int_Type_Def_F_Expr : Bare_Expr
        ) is
      begin
            Self.Mod_Int_Type_Def_F_Expr := Mod_Int_Type_Def_F_Expr;
      end Initialize_Fields_For_Mod_Int_Type_Def;
   function Mod_Int_Type_Def_F_Expr
     (Node : Bare_Mod_Int_Type_Def) return Bare_Expr
   is
   begin
         return Node.Mod_Int_Type_Def_F_Expr;
   end;
--# property-start ModIntTypeDef.is_int_type /usr/src/packages/BUILD/ada/ast.py:5915
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Mod_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start ModIntTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:5917
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Mod_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_503;
      Fld : Internal_Entity_Expr;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_503 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_503;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8636 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5917
--# expr-start 8635 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:5917
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8635
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8636
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_503;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_503;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start ModIntTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:5919
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Is_Static
  (Node : Bare_Mod_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Expr;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8638 '<FieldAccess .is_static_expr>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5919
--# expr-start 8637 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:5919
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8637
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_1 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld.Info);
--# end
--# expr-done 8638
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Is_Static;
--# end
--# property-start ModIntTypeDef.discrete_range /usr/src/packages/BUILD/ada/ast.py:5922
pragma Warnings (Off, "is not referenced");
 function Mod_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Mod_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Mod_Int_Type_Def := Bare_Mod_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Mod_Int_Type_Def :=
     Internal_Entity_Mod_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Expr;
New_Struct : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8641 '<Block>' Let_Result None
--# scope-start
--# expr-start 8640 '<New InternalDiscreteRange>' New_Struct /usr/src/packages/BUILD/ada/ast.py:5923
--# expr-start 8639 '<FieldAccess .expr>' Fld /usr/src/packages/BUILD/ada/ast.py:5924
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Expr (Node => Ent.Node.Mod_Int_Type_Def_F_Expr, Info => Ent.Info);
--# expr-done 8639
New_Struct := (High_Bound => Fld, Low_Bound => No_Entity_Expr); 
--# expr-done 8640
Let_Result := New_Struct; 
--# end
--# expr-done 8641
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Mod_Int_Type_Def_P_Discrete_Range;
--# end
   --
   --  Primitives for Bare_Private_Type_Def
   --
      procedure Initialize_Fields_For_Private_Type_Def
        (Self : Bare_Private_Type_Def
         ; Private_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Private_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Private_Type_Def_F_Has_Limited : Bare_Limited_Node
        ) is
      begin
            Self.Private_Type_Def_F_Has_Abstract := Private_Type_Def_F_Has_Abstract;
            Self.Private_Type_Def_F_Has_Tagged := Private_Type_Def_F_Has_Tagged;
            Self.Private_Type_Def_F_Has_Limited := Private_Type_Def_F_Has_Limited;
      end Initialize_Fields_For_Private_Type_Def;
   function Private_Type_Def_F_Has_Abstract
     (Node : Bare_Private_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Abstract;
   end;
   function Private_Type_Def_F_Has_Tagged
     (Node : Bare_Private_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Tagged;
   end;
   function Private_Type_Def_F_Has_Limited
     (Node : Bare_Private_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Private_Type_Def_F_Has_Limited;
   end;
--# property-start PrivateTypeDef.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:5879
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Private_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Private_Type_Def := Bare_Private_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8643 '<FieldAccess .as_bool>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5879
--# expr-start 8642 '<FieldAccess .has_tagged>' Fld /usr/src/packages/BUILD/ada/ast.py:5879
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Private_Type_Def_F_Has_Tagged;
--# expr-done 8642
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 8643
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start PrivateTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:5881
pragma Warnings (Off, "is not referenced");
 function Private_Type_Def_P_Xref_Equation
  (Node : Bare_Private_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Private_Type_Def := Bare_Private_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_497;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_497 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_497;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5881") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_497;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_497;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Private_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Real_Type_Def
   --
--# property-start RealTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:3987
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Xref_Equation
  (Node : Bare_Real_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Real_Type_Def := Bare_Real_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_311;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_311 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_311;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3987") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_311;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_311;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Type_Def_P_Xref_Equation;
--# end
--# property-start RealTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:3989
pragma Warnings (Off, "is not referenced");
 function Real_Type_Def_P_Is_Static
  (Node : Bare_Real_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Real_Type_Def := Bare_Real_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Real_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Decimal_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Decimal_Fixed_Point_Def
        (Self : Bare_Decimal_Fixed_Point_Def
         ; Decimal_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Digits : Bare_Expr
         ; Decimal_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Decimal_Fixed_Point_Def_F_Delta := Decimal_Fixed_Point_Def_F_Delta;
            Self.Decimal_Fixed_Point_Def_F_Digits := Decimal_Fixed_Point_Def_F_Digits;
            Self.Decimal_Fixed_Point_Def_F_Range := Decimal_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Decimal_Fixed_Point_Def;
   function Decimal_Fixed_Point_Def_F_Delta
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Delta;
   end;
   function Decimal_Fixed_Point_Def_F_Digits
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Digits;
   end;
   function Decimal_Fixed_Point_Def_F_Range
     (Node : Bare_Decimal_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Decimal_Fixed_Point_Def_F_Range;
   end;
--# property-start DecimalFixedPointDef.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:5631
pragma Warnings (Off, "is not referenced");
 function Decimal_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Decimal_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Decimal_Fixed_Point_Def := Bare_Decimal_Fixed_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Decimal_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
   --
   --  Primitives for Bare_Floating_Point_Def
   --
      procedure Initialize_Fields_For_Floating_Point_Def
        (Self : Bare_Floating_Point_Def
         ; Floating_Point_Def_F_Num_Digits : Bare_Expr
         ; Floating_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Floating_Point_Def_F_Num_Digits := Floating_Point_Def_F_Num_Digits;
            Self.Floating_Point_Def_F_Range := Floating_Point_Def_F_Range;
      end Initialize_Fields_For_Floating_Point_Def;
   function Floating_Point_Def_F_Num_Digits
     (Node : Bare_Floating_Point_Def) return Bare_Expr
   is
   begin
         return Node.Floating_Point_Def_F_Num_Digits;
   end;
   function Floating_Point_Def_F_Range
     (Node : Bare_Floating_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Floating_Point_Def_F_Range;
   end;
--# property-start FloatingPointDef.is_float_type /usr/src/packages/BUILD/ada/ast.py:5610
pragma Warnings (Off, "is not referenced");
 function Floating_Point_Def_P_Is_Float_Type
  (Node : Bare_Floating_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Floating_Point_Def := Bare_Floating_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Floating_Point_Def_P_Is_Float_Type;
--# end
   --
   --  Primitives for Bare_Ordinary_Fixed_Point_Def
   --
      procedure Initialize_Fields_For_Ordinary_Fixed_Point_Def
        (Self : Bare_Ordinary_Fixed_Point_Def
         ; Ordinary_Fixed_Point_Def_F_Delta : Bare_Expr
         ; Ordinary_Fixed_Point_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Ordinary_Fixed_Point_Def_F_Delta := Ordinary_Fixed_Point_Def_F_Delta;
            Self.Ordinary_Fixed_Point_Def_F_Range := Ordinary_Fixed_Point_Def_F_Range;
      end Initialize_Fields_For_Ordinary_Fixed_Point_Def;
   function Ordinary_Fixed_Point_Def_F_Delta
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Expr
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Delta;
   end;
   function Ordinary_Fixed_Point_Def_F_Range
     (Node : Bare_Ordinary_Fixed_Point_Def) return Bare_Range_Spec
   is
   begin
         return Node.Ordinary_Fixed_Point_Def_F_Range;
   end;
--# property-start OrdinaryFixedPointDef.is_fixed_point /usr/src/packages/BUILD/ada/ast.py:5620
pragma Warnings (Off, "is not referenced");
 function Ordinary_Fixed_Point_Def_P_Is_Fixed_Point
  (Node : Bare_Ordinary_Fixed_Point_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Ordinary_Fixed_Point_Def := Bare_Ordinary_Fixed_Point_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ordinary_Fixed_Point_Def_P_Is_Fixed_Point;
--# end
   --
   --  Primitives for Bare_Record_Type_Def
   --
      procedure Initialize_Fields_For_Record_Type_Def
        (Self : Bare_Record_Type_Def
         ; Record_Type_Def_F_Has_Abstract : Bare_Abstract_Node
         ; Record_Type_Def_F_Has_Tagged : Bare_Tagged_Node
         ; Record_Type_Def_F_Has_Limited : Bare_Limited_Node
         ; Record_Type_Def_F_Record_Def : Bare_Base_Record_Def
        ) is
      begin
            Self.Record_Type_Def_F_Has_Abstract := Record_Type_Def_F_Has_Abstract;
            Self.Record_Type_Def_F_Has_Tagged := Record_Type_Def_F_Has_Tagged;
            Self.Record_Type_Def_F_Has_Limited := Record_Type_Def_F_Has_Limited;
            Self.Record_Type_Def_F_Record_Def := Record_Type_Def_F_Record_Def;
      end Initialize_Fields_For_Record_Type_Def;
   function Record_Type_Def_F_Has_Abstract
     (Node : Bare_Record_Type_Def) return Bare_Abstract_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Abstract;
   end;
   function Record_Type_Def_F_Has_Tagged
     (Node : Bare_Record_Type_Def) return Bare_Tagged_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Tagged;
   end;
   function Record_Type_Def_F_Has_Limited
     (Node : Bare_Record_Type_Def) return Bare_Limited_Node
   is
   begin
         return Node.Record_Type_Def_F_Has_Limited;
   end;
   function Record_Type_Def_F_Record_Def
     (Node : Bare_Record_Type_Def) return Bare_Base_Record_Def
   is
   begin
         return Node.Record_Type_Def_F_Record_Def;
   end;
--# property-start RecordTypeDef.is_tagged_type /usr/src/packages/BUILD/ada/ast.py:3976
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Tagged_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
         := No_Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
      Fld : Bare_Tagged_Node;
Fld_1 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8645 '<FieldAccess .as_bool>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3976
--# expr-start 8644 '<FieldAccess .has_tagged>' Fld /usr/src/packages/BUILD/ada/ast.py:3976
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Record_Type_Def_F_Has_Tagged;
--# expr-done 8644
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Tagged.as_bool'
Fld_1 := Libadalang.Implementation.Dispatcher_Tagged_Node_P_As_Bool (Node => Fld);
--# end
--# expr-done 8645
      Property_Result := Fld_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Is_Tagged_Type;
--# end
--# property-start RecordTypeDef.is_record_type /usr/src/packages/BUILD/ada/ast.py:3977
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Is_Record_Type
  (Node : Bare_Record_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Is_Record_Type;
--# end
--# property-start RecordTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:3979
pragma Warnings (Off, "is not referenced");
 function Record_Type_Def_P_Xref_Equation
  (Node : Bare_Record_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Record_Type_Def := Bare_Record_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_310;
      Logic_True : Logic_Equation;
            procedure Finalizer_Scope_310 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_310;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3979") else null)); 
      Property_Result := Logic_True;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_310;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_310;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Record_Type_Def_P_Xref_Equation;
--# end
   --
   --  Primitives for Bare_Signed_Int_Type_Def
   --
      procedure Initialize_Fields_For_Signed_Int_Type_Def
        (Self : Bare_Signed_Int_Type_Def
         ; Signed_Int_Type_Def_F_Range : Bare_Range_Spec
        ) is
      begin
            Self.Signed_Int_Type_Def_F_Range := Signed_Int_Type_Def_F_Range;
      end Initialize_Fields_For_Signed_Int_Type_Def;
   function Signed_Int_Type_Def_F_Range
     (Node : Bare_Signed_Int_Type_Def) return Bare_Range_Spec
   is
   begin
         return Node.Signed_Int_Type_Def_F_Range;
   end;
--# property-start SignedIntTypeDef.is_int_type /usr/src/packages/BUILD/ada/ast.py:5889
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Int_Type
  (Node : Bare_Signed_Int_Type_Def
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Is_Int_Type;
--# end
--# property-start SignedIntTypeDef.xref_equation /usr/src/packages/BUILD/ada/ast.py:5891
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Xref_Equation
  (Node : Bare_Signed_Int_Type_Def
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_499;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Logic_Var;
Fld_3 : Internal_Entity_Type_Decl;
Cast_Result : Internal_Entity;
Fld_4 : Logic_Equation;
Logic_True : Logic_Equation;
Or_Pred : Logic_Equation;
Fld_5 : Internal_Entity_Range_Spec;
Fld_6 : Logic_Equation;
And_Pred : Logic_Equation;
            procedure Finalizer_Scope_499 is
            begin
                     Dec_Ref (Fld_4);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Or_Pred);
                     Dec_Ref (Fld_6);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_499;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8654 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:5895
--# expr-start 8651 '<Or>' Or_Pred /usr/src/packages/BUILD/ada/ast.py:5895
--# expr-start 8650 '<FieldAccess .type_bind_val(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:5896
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8648 '<FieldAccess .type_var>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5896
--# expr-start 8647 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5896
--# expr-start 8646 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:5896
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8646
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8647
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Fld_1.Node);
--# expr-done 8648
--# expr-start 8649 '<FieldAccess .containing_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:5897
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeDef.containing_type
Fld_3 := Libadalang.Implementation.Type_Def_P_Containing_Type (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8649
      Cast_Result := Create_Internal_Entity
        (Node => Fld_3.Node,
         Info => Fld_3.Info);
--# property-call-start AdaNode.type_bind_val
Fld_4 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_2, Right => Cast_Result, Origin => Origin);
--# end
--# expr-done 8650
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5898") else null)); 
Or_Pred := Logic_Or (Fld_4, Logic_True, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5895") else null)); 
--# expr-done 8651
--# expr-start 8653 '<FieldAccess .xref_equation>' Fld_6 /usr/src/packages/BUILD/ada/ast.py:5900
--# expr-start 8652 '<FieldAccess .range>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:5900
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8652
      if Fld_5.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_6 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Fld_5.Node, Env => Env, Origin => Origin, E_Info => Fld_5.Info);
--# end
--# expr-done 8653
And_Pred := Logic_And (Or_Pred, Fld_6, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:5895") else null)); 
--# expr-done 8654
      Property_Result := And_Pred;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_499;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_499;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Xref_Equation;
--# end
--# property-start SignedIntTypeDef.discrete_range /usr/src/packages/BUILD/ada/ast.py:5904
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Discrete_Range
  (Node : Bare_Signed_Int_Type_Def
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8658 '<Block>' Let_Result None
--# scope-start
--# expr-start 8657 '<FieldAccess .discrete_range>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5905
--# expr-start 8656 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5905
--# expr-start 8655 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:5905
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8655
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8656
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_2 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_1.Node, E_Info => Fld_1.Info);
--# end
--# expr-done 8657
Let_Result := Fld_2; 
--# end
--# expr-done 8658
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Discrete_Range;
--# end
--# property-start SignedIntTypeDef.is_static /usr/src/packages/BUILD/ada/ast.py:5907
pragma Warnings (Off, "is not referenced");
 function Signed_Int_Type_Def_P_Is_Static
  (Node : Bare_Signed_Int_Type_Def
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Signed_Int_Type_Def := Bare_Signed_Int_Type_Def (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Signed_Int_Type_Def :=
     Internal_Entity_Signed_Int_Type_Def'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Fld : Internal_Entity_Range_Spec;
Fld_1 : Internal_Entity_Expr;
Fld_2 : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8661 '<FieldAccess .is_static_expr>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:5907
--# expr-start 8660 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:5907
--# expr-start 8659 '<FieldAccess .range>' Fld /usr/src/packages/BUILD/ada/ast.py:5907
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Range_Spec (Node => Ent.Node.Signed_Int_Type_Def_F_Range, Info => Ent.Info);
--# expr-done 8659
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Expr (Node => Fld.Node.Range_Spec_F_Range, Info => Fld.Info);
--# expr-done 8660
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.is_static_expr
Fld_2 := Libadalang.Implementation.Expr_P_Is_Static_Expr (Node => Fld_1.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_1.Info);
--# end
--# expr-done 8661
      Property_Result := Fld_2;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Signed_Int_Type_Def_P_Is_Static;
--# end
   --
   --  Primitives for Bare_Type_Expr
   --
--# property-start TypeExpr.array_ndims /usr/src/packages/BUILD/ada/ast.py:6517
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Array_Ndims
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Integer
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Integer;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Integer;
Dyn_Var_Bind_Result : Integer;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8665 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .array_ndims>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6518
--# expr-start 8662 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:6518
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8662
Bound_Origin := Fld; 
--# expr-start 8664 '<FieldAccess .array_ndims>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6518
--# expr-start 8663 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6518
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8663
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.array_ndims'
Fld_2 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Array_Ndims (Node => Fld_1.Node, Origin => Bound_Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8664
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8665
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Array_Ndims;
--# end
--# property-start TypeExpr.type_name /usr/src/packages/BUILD/ada/ast.py:6521
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Type_Name
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Name
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Name;
      Var_Expr : Internal_Entity_Subtype_Indication;
Cast_Expr : Internal_Entity_Type_Expr;
Cast_Result : Internal_Entity_Subtype_Indication;
Fld : Internal_Entity_Name;
Scope_Result : Internal_Entity_Name;
Result_Var : Internal_Entity_Name;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8666 '<Cast to SubtypeIndication>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6522
Cast_Expr := Ent; 
   if Cast_Expr.Node = null
      or else Cast_Expr.Node.Kind in Ada_Subtype_Indication_Range
   then
      Cast_Result := Create_Internal_Entity_Subtype_Indication
        (Node => Cast_Expr.Node,
         Info => Cast_Expr.Info);
   else
         Cast_Result := No_Entity_Subtype_Indication;
   end if;
--# expr-done 8666
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Subtype_Indication then
   --# scope-start
   --# bind sti Var_Expr
--# expr-start 8667 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:6522
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Var_Expr.Node.Subtype_Indication_F_Name, Info => Var_Expr.Info);
--# expr-done 8667
Scope_Result := Fld;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Name;
end if;
      Property_Result := Result_Var;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Type_Name;
--# end
--# property-start TypeExpr.accessed_type /usr/src/packages/BUILD/ada/ast.py:6529
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Accessed_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8670 '<Block>' Let_Result None
--# scope-start
--# expr-start 8668 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6530
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8668
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8669 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6530
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.accessed_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Accessed_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8669
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8670
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Accessed_Type;
--# end
--# property-start TypeExpr.defining_env /usr/src/packages/BUILD/ada/ast.py:6533
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Defining_Env
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_600;
      Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Lexical_Env := Empty_Env;
Let_Result : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_600 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_600;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8673 '<Block>' Let_Result None
--# scope-start
--# expr-start 8672 '<FieldAccess .defining_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6534
--# expr-start 8671 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6534
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8671
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.defining_env'
Fld_1 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Defining_Env (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8672
Let_Result := Fld_1; Inc_Ref (Let_Result);
--# end
--# expr-done 8673
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_600;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_600;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Defining_Env;
--# end
--# property-start '[dispatcher]TypeExpr.designated_type' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Type_Expr_P_Designated_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Type_Expr (Self.Kind) is
               when Ada_Anonymous_Type =>
                  --# property-call-start AnonymousType.designated_type
                  Property_Result := Anonymous_Type_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Enum_Lit_Synth_Type_Expr =>
                  --# property-call-start EnumLitSynthTypeExpr.designated_type
                  Property_Result := Enum_Lit_Synth_Type_Expr_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
               when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
                  --# property-call-start SubtypeIndication.designated_type
                  Property_Result := Subtype_Indication_P_Designated_Type
                    (Self
                        , Origin
                        , E_Info
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Type_Expr_P_Designated_Type;
--# end
--# property-start TypeExpr.designated_type_decl /usr/src/packages/BUILD/ada/ast.py:6544
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl
  (Node : Bare_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8676 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6545
--# expr-start 8674 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:6545
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8674
Bound_Origin := Fld; 
--# expr-start 8675 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6545
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8675
Dyn_Var_Bind_Result := Fld_1; 
--# expr-done 8676
      Property_Result := Dyn_Var_Bind_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Designated_Type_Decl;
--# end
--# property-start TypeExpr.designated_type_decl_from /usr/src/packages/BUILD/ada/ast.py:6553
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Designated_Type_Decl_From
  (Node : Bare_Type_Expr
      ; Origin_Node : Internal_Entity
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin_node Origin_Node
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8681 '<Block>' Let_Result None
--# scope-start
--# expr-start 8680 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_type>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6558
--# expr-start 8678 '<FieldAccess .origin_node>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6558
--# expr-start 8677 '<FieldAccess .node>' Fld /usr/src/packages/BUILD/ada/ast.py:6558
Fld := Origin_Node.Node;
--# expr-done 8677
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_1 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Fld);
--# end
--# expr-done 8678
Bound_Origin := Fld_1; 
--# expr-start 8679 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6559
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8679
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8680
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 8681
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Designated_Type_Decl_From;
--# end
--# property-start TypeExpr.element_type /usr/src/packages/BUILD/ada/ast.py:6562
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Element_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Anonymous_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Fld_2 : Internal_Entity_Base_Type_Decl;
Cast_Result : Internal_Entity_Anonymous_Type_Decl;
Fld_3 : Internal_Entity_Type_Def;
Scope_Result : Internal_Entity_Type_Def;
Result_Var : Internal_Entity_Type_Def;
Cast_Result_1 : Internal_Entity_Access_Def;
Is_Null : Boolean;
If_Result : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8690 '<Block>' Let_Result None
--# scope-start
--# expr-start 8689 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:6568
--# expr-start 8686 '<IsNull>' Is_Null /usr/src/packages/BUILD/ada/ast.py:6569
--# expr-start 8685 '<Cast to AccessDef>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:6569
--# expr-start 8683 '<Cast to AnonymousTypeDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6569
--# expr-start 8682 '<FieldAccess .designated_type>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6567
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_2 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8682
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Anonymous_Type_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Anonymous_Type_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result := No_Entity_Anonymous_Type_Decl;
   end if;
--# expr-done 8683
Var_Expr := Cast_Result;
if Var_Expr /= No_Entity_Anonymous_Type_Decl then
   --# scope-start
--# expr-start 8684 '<FieldAccess .type_def>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6569
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_3 := Create_Internal_Entity_Type_Def (Node => Var_Expr.Node.Type_Decl_F_Type_Def, Info => Var_Expr.Info);
--# expr-done 8684
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Type_Def;
end if;
   if Result_Var.Node = null
      or else Result_Var.Node.Kind in Ada_Access_Def
   then
      Cast_Result_1 := Create_Internal_Entity_Access_Def
        (Node => Result_Var.Node,
         Info => Result_Var.Info);
   else
         Cast_Result_1 := No_Entity_Access_Def;
   end if;
--# expr-done 8685
Is_Null := Cast_Result_1.Node = null; 
--# expr-done 8686
if Is_Null then
   --# expr-start 8687 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6567
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8687
   If_Result := Fld;
else
   --# expr-start 8688 '<FieldAccess .accessed_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6571
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.accessed_type
Fld_1 := Libadalang.Implementation.Type_Expr_P_Accessed_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8688
   If_Result := Fld_1;
end if;
--# expr-done 8689
Let_Result := If_Result; 
--# end
--# expr-done 8690
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Element_Type;
--# end
--# property-start TypeExpr.canonical_type /usr/src/packages/BUILD/ada/ast.py:6576
pragma Warnings (Off, "is not referenced");
 function Type_Expr_P_Canonical_Type
  (Node : Bare_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Type_Expr := Bare_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Type_Expr :=
     Internal_Entity_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Var_Expr : Internal_Entity_Base_Type_Decl;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Internal_Entity_Base_Type_Decl;
Scope_Result : Internal_Entity_Base_Type_Decl;
Result_Var : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8693 '<Block>' Let_Result None
--# scope-start
--# expr-start 8691 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6577
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Origin, E_Info => Ent.Info);
--# end
--# expr-done 8691
Var_Expr := Fld;
if Var_Expr /= No_Entity_Base_Type_Decl then
   --# scope-start
--# expr-start 8692 '<FieldAccess .canonical_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6577
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.canonical_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Canonical_Type (Node => Var_Expr.Node, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8692
Scope_Result := Fld_1;
   --# end
   Result_Var := Scope_Result;
else
   Result_Var := No_Entity_Base_Type_Decl;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8693
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Type_Expr_P_Canonical_Type;
--# end
   --
   --  Primitives for Bare_Anonymous_Type
   --
      procedure Initialize_Fields_For_Anonymous_Type
        (Self : Bare_Anonymous_Type
         ; Anonymous_Type_F_Type_Decl : Bare_Anonymous_Type_Decl
        ) is
      begin
            Self.Anonymous_Type_F_Type_Decl := Anonymous_Type_F_Type_Decl;
      end Initialize_Fields_For_Anonymous_Type;
   function Anonymous_Type_F_Type_Decl
     (Node : Bare_Anonymous_Type) return Bare_Anonymous_Type_Decl
   is
   begin
         return Node.Anonymous_Type_F_Type_Decl;
   end;
--# property-start AnonymousType.designated_type /usr/src/packages/BUILD/ada/ast.py:6613
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Designated_Type
  (Node : Bare_Anonymous_Type
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Cast_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8694 '<FieldAccess .type_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6613
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8694
      Cast_Result := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
      Property_Result := Cast_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Designated_Type;
--# end
--# property-start AnonymousType.xref_equation /usr/src/packages/BUILD/ada/ast.py:6614
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Xref_Equation
  (Node : Bare_Anonymous_Type
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_609;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Logic_Equation;
            procedure Finalizer_Scope_609 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_609;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8696 '<FieldAccess .sub_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6614
--# expr-start 8695 '<FieldAccess .type_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6614
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8695
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_1 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8696
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_609;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_609;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Xref_Equation;
--# end
--# property-start AnonymousType.custom_id_text /usr/src/packages/BUILD/ada/ast.py:6621
pragma Warnings (Off, "is not referenced");
 function Anonymous_Type_P_Custom_Id_Text
  (Node : Bare_Anonymous_Type
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Anonymous_Type := Bare_Anonymous_Type (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Anonymous_Type :=
     Internal_Entity_Anonymous_Type'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_610;
      Fld : Internal_Entity_Anonymous_Type_Decl;
Fld_1 : Character_Type_Array_Access;
            procedure Finalizer_Scope_610 is
            begin
                     Dec_Ref (Fld_1);
            end Finalizer_Scope_610;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8698 '<FieldAccess .text>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6621
--# expr-start 8697 '<FieldAccess .type_decl>' Fld /usr/src/packages/BUILD/ada/ast.py:6621
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Anonymous_Type_Decl (Node => Ent.Node.Anonymous_Type_F_Type_Decl, Info => Ent.Info);
--# expr-done 8697
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Text (Node => Fld.Node);
--# expr-done 8698
      Property_Result := Fld_1;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_610;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_610;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Anonymous_Type_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Enum_Lit_Synth_Type_Expr
   --
--# property-start EnumLitSynthTypeExpr.designated_type /usr/src/packages/BUILD/ada/ast.py:6585
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Designated_Type
  (Node : Bare_Enum_Lit_Synth_Type_Expr
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Enum_Lit_Synth_Type_Expr := Bare_Enum_Lit_Synth_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
     Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity;
Cast_Result : Internal_Entity_Enum_Literal_Decl;
Fld_1 : Internal_Entity_Type_Decl;
Cast_Result_1 : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
--# expr-start 8701 '<FieldAccess .enum_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6586
--# expr-start 8700 '<Cast to EnumLiteralDecl>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6586
--# expr-start 8699 '<parent(<Var Ent>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6586
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8699
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         Cast_Result := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 8700
      if Cast_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start EnumLiteralDecl.enum_type
Fld_1 := Libadalang.Implementation.Enum_Literal_Decl_P_Enum_Type (Node => Cast_Result.Node, E_Info => Cast_Result.Info);
--# end
--# expr-done 8701
      Cast_Result_1 := Create_Internal_Entity_Base_Type_Decl
        (Node => Fld_1.Node,
         Info => Fld_1.Info);
      Property_Result := Cast_Result_1;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Lit_Synth_Type_Expr_P_Designated_Type;
--# end
--# property-start EnumLitSynthTypeExpr.custom_id_text /usr/src/packages/BUILD/ada/ast.py:6589
pragma Warnings (Off, "is not referenced");
 function Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text
  (Node : Bare_Enum_Lit_Synth_Type_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Enum_Lit_Synth_Type_Expr := Bare_Enum_Lit_Synth_Type_Expr (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Enum_Lit_Synth_Type_Expr :=
     Internal_Entity_Enum_Lit_Synth_Type_Expr'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_607;
      Cast_Expr : Bare_Enum_Lit_Synth_Type_Expr;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Character_Type_Array_Access;
Array_Lit : Character_Type_Array_Access;
Concat_Result : Character_Type_Array_Access;
Fld_2 : Internal_Entity;
Cast_Result_1 : Internal_Entity_Enum_Literal_Decl;
Fld_3 : Internal_Entity_Defining_Name;
Fld_4 : Symbol_Type_Array_Access;
Array_Lit_1 : Character_Type_Array_Access;
Fld_5 : Character_Type_Array_Access;
Concat_Result_1 : Character_Type_Array_Access;
Dyn_Var_Bind_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_607 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Array_Lit);
                     Dec_Ref (Concat_Result);
                     Dec_Ref (Fld_4);
                     Dec_Ref (Array_Lit_1);
                     Dec_Ref (Fld_5);
                     Dec_Ref (Concat_Result_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_607;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8713 '<bind(<DynamicVariable origin>, <Var Self>, <Concat>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6592
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 8712 '<Concat>' Concat_Result_1 /usr/src/packages/BUILD/ada/ast.py:6594
--# expr-start 8705 '<Concat>' Concat_Result /usr/src/packages/BUILD/ada/ast.py:6594
--# expr-start 8703 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6594
--# expr-start 8702 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6594
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8702
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8703
--# expr-start 8704 '<ArrayLiteral>' Array_Lit /usr/src/packages/BUILD/ada/ast.py:6595
Array_Lit := Create_Character_Type_Array (Text_Type'(1 => Character_Type'Val (46))); 
--# expr-done 8704
Concat_Result := Concat (Fld_1, Array_Lit); 
--# expr-done 8705
--# expr-start 8711 '<FieldAccess .sym_join(...)>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6597
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8709 '<FieldAccess .as_symbol_array>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6598
--# expr-start 8708 '<FieldAccess .defining_name>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6598
--# expr-start 8707 '<Cast to EnumLiteralDecl>' Cast_Result_1 /usr/src/packages/BUILD/ada/ast.py:6598
--# expr-start 8706 '<parent(<Var Ent>)>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6598
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Libadalang.Implementation.Parent (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8706
   if Fld_2.Node = null
      or else Fld_2.Node.Kind in Ada_Enum_Literal_Decl_Range
   then
      Cast_Result_1 := Create_Internal_Entity_Enum_Literal_Decl
        (Node => Fld_2.Node,
         Info => Fld_2.Info);
   else
         Cast_Result_1 := No_Entity_Enum_Literal_Decl;
   end if;
--# expr-done 8707
      if Cast_Result_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.defining_name
Fld_3 := Libadalang.Implementation.Basic_Decl_P_Defining_Name (Node => Cast_Result_1.Node, E_Info => Cast_Result_1.Info);
--# end
--# expr-done 8708
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.as_symbol_array
Fld_4 := Libadalang.Implementation.Name_P_As_Symbol_Array (Node => Fld_3.Node);
--# end
--# expr-done 8709
--# expr-start 8710 '<ArrayLiteral>' Array_Lit_1 /usr/src/packages/BUILD/ada/ast.py:6600
Array_Lit_1 := Create_Character_Type_Array (Items_Count => 0); 
--# expr-done 8710
--# property-call-start AdaNode.sym_join
Fld_5 := Libadalang.Implementation.Ada_Node_P_Sym_Join (Node => Ent.Node, Syms => Fld_4, Sep => Array_Lit_1);
--# end
--# expr-done 8711
Concat_Result_1 := Concat (Concat_Result, Fld_5); 
--# expr-done 8712
Dyn_Var_Bind_Result := Concat_Result_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8713
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_607;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_607;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Enum_Lit_Synth_Type_Expr_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Subtype_Indication
   --
      procedure Initialize_Fields_For_Subtype_Indication
        (Self : Bare_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Self.Subtype_Indication_F_Has_Not_Null := Subtype_Indication_F_Has_Not_Null;
            Self.Subtype_Indication_F_Name := Subtype_Indication_F_Name;
            Self.Subtype_Indication_F_Constraint := Subtype_Indication_F_Constraint;
      end Initialize_Fields_For_Subtype_Indication;
   function Subtype_Indication_F_Has_Not_Null
     (Node : Bare_Subtype_Indication) return Bare_Not_Null
   is
   begin
         return Node.Subtype_Indication_F_Has_Not_Null;
   end;
   function Subtype_Indication_F_Name
     (Node : Bare_Subtype_Indication) return Bare_Name
   is
   begin
         return Node.Subtype_Indication_F_Name;
   end;
   function Subtype_Indication_F_Constraint
     (Node : Bare_Subtype_Indication) return Bare_Constraint
   is
   begin
         return Node.Subtype_Indication_F_Constraint;
   end;
--# property-start SubtypeIndication.designated_type /usr/src/packages/BUILD/ada/ast.py:6636
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Designated_Type
  (Node : Bare_Subtype_Indication
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_611;
      Fld : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name;
Fld_2 : Internal_Entity_Base_Type_Decl;
Dyn_Var_Bind_Result : Internal_Entity_Base_Type_Decl;
            procedure Finalizer_Scope_611 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Bound_Env);
            end Finalizer_Scope_611;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8717 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <FieldAccess .designated_type_impl>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6637
--# expr-start 8714 '<FieldAccess .node_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6637
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Libadalang.Implementation.Node_Env (Node => Ent.Node, E_Info => Ent.Info);
--# expr-done 8714
Bound_Env := Fld; Inc_Ref (Bound_Env);
--# expr-start 8716 '<FieldAccess .designated_type_impl>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6637
--# expr-start 8715 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6637
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 8715
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Name.designated_type_impl'
Fld_2 := Libadalang.Implementation.Dispatcher_Name_P_Designated_Type_Impl (Node => Fld_1.Node, Env => Bound_Env, Origin => Origin, E_Info => Fld_1.Info);
--# end
--# expr-done 8716
Dyn_Var_Bind_Result := Fld_2; 
--# expr-done 8717
      Property_Result := Dyn_Var_Bind_Result;
   --# end
      Finalizer_Scope_611;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_611;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Designated_Type;
--# end
--# property-start SubtypeIndication.xref_equation /usr/src/packages/BUILD/ada/ast.py:6641
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Xref_Equation
  (Node : Bare_Subtype_Indication
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_612;
            procedure Finalizer_Scope_2647;
      Var_Expr : Internal_Entity_Constraint;
Fld : Internal_Entity_Name;
Fld_1 : Logic_Equation;
Fld_2 : Internal_Entity_Constraint;
Fld_3 : Logic_Equation;
Scope_Result : Logic_Equation;
Logic_True : Logic_Equation;
Result_Var : Logic_Equation;
And_Pred : Logic_Equation;
Let_Result : Logic_Equation;
            procedure Finalizer_Scope_612 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Logic_True);
                     Dec_Ref (Result_Var);
                     Dec_Ref (And_Pred);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_612;
            procedure Finalizer_Scope_2647 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2647;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8723 '<Block>' Let_Result None
--# scope-start
--# expr-start 8722 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:6644
--# expr-start 8719 '<FieldAccess .subtype_indication_equation>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6645
--# expr-start 8718 '<FieldAccess .name>' Fld /usr/src/packages/BUILD/ada/ast.py:6645
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name (Node => Ent.Node.Subtype_Indication_F_Name, Info => Ent.Info);
--# expr-done 8718
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.subtype_indication_equation
Fld_1 := Libadalang.Implementation.Name_P_Subtype_Indication_Equation (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8719
--# expr-start 8720 '<FieldAccess .constraint>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6646
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8720
Var_Expr := Fld_2;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 8721 '<FieldAccess .sub_equation>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6647
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_3 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Var_Expr.Node, Env => Env, Origin => Origin, E_Info => Var_Expr.Info);
--# end
--# expr-done 8721
Scope_Result := Fld_3;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2647;
   Result_Var := Scope_Result;
else
   Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6647") else null)); 
   Result_Var := Logic_True;
end if;
      Inc_Ref (Result_Var);
And_Pred := Logic_And (Fld_1, Result_Var, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6644") else null)); 
--# expr-done 8722
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
--# expr-done 8723
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_612;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_612;
               Finalizer_Scope_2647;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Xref_Equation;
--# end
--# property-start SubtypeIndication.discrete_range /usr/src/packages/BUILD/ada/ast.py:6652
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Discrete_Range
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Discrete_Range
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Internal_Discrete_Range;
      Block_Var : Internal_Entity_Range_Constraint;
Var_Expr : Internal_Entity_Range_Constraint;
Fld : Internal_Entity_Constraint;
Cast_Result : Internal_Entity_Range_Constraint;
Fld_1 : Internal_Entity_Range_Spec;
Fld_2 : Internal_Entity_Expr;
Fld_3 : Internal_Discrete_Range;
Scope_Result : Internal_Discrete_Range;
Fld_4 : Internal_Entity_Base_Type_Decl;
Fld_5 : Internal_Discrete_Range;
Result_Var : Internal_Discrete_Range;
Let_Result : Internal_Discrete_Range;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8731 '<Block>' Let_Result None
--# scope-start
--# expr-start 8725 '<Cast to RangeConstraint>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6653
--# expr-start 8724 '<FieldAccess .constraint>' Fld /usr/src/packages/BUILD/ada/ast.py:6653
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8724
   if Fld.Node = null
      or else Fld.Node.Kind in Ada_Range_Constraint_Range
   then
      Cast_Result := Create_Internal_Entity_Range_Constraint
        (Node => Fld.Node,
         Info => Fld.Info);
   else
         raise Property_Error with "invalid object cast";
   end if;
--# expr-done 8725
Block_Var := Cast_Result; 
--# bind rc Block_Var
Var_Expr := Block_Var;
if Var_Expr /= No_Entity_Range_Constraint then
   --# scope-start
   --# bind r Var_Expr
--# expr-start 8728 '<FieldAccess .discrete_range>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6655
--# expr-start 8727 '<FieldAccess .range>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6655
--# expr-start 8726 '<FieldAccess .range>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6655
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Range_Spec (Node => Var_Expr.Node.Range_Constraint_F_Range, Info => Var_Expr.Info);
--# expr-done 8726
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Expr (Node => Fld_1.Node.Range_Spec_F_Range, Info => Fld_1.Info);
--# expr-done 8727
      if Fld_2.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.discrete_range
Fld_3 := Libadalang.Implementation.Expr_P_Discrete_Range (Node => Fld_2.Node, E_Info => Fld_2.Info);
--# end
--# expr-done 8728
Scope_Result := Fld_3;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8730 '<FieldAccess .discrete_range>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:6658
--# expr-start 8729 '<FieldAccess .designated_type_decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6658
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start TypeExpr.designated_type_decl
Fld_4 := Libadalang.Implementation.Type_Expr_P_Designated_Type_Decl (Node => Ent.Node, E_Info => Ent.Info);
--# end
--# expr-done 8729
      if Fld_4.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BaseTypeDecl.discrete_range'
Fld_5 := Libadalang.Implementation.Dispatcher_Base_Type_Decl_P_Discrete_Range (Node => Fld_4.Node, E_Info => Fld_4.Info);
--# end
--# expr-done 8730
   Result_Var := Fld_5;
end if;
Let_Result := Result_Var; 
--# end
--# expr-done 8731
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Discrete_Range;
--# end
--# property-start SubtypeIndication.is_static_subtype /usr/src/packages/BUILD/ada/ast.py:6664
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Is_Static_Subtype
  (Node : Bare_Subtype_Indication
      ; Imprecise_Fallback : Boolean
         := False
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind imprecise_fallback Imprecise_Fallback
   Property_Result : Boolean;
      Var_Expr : Internal_Entity_Constraint;
Fld : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld_1 : Internal_Entity_Constraint;
Fld_2 : Boolean;
Scope_Result : Boolean;
Fld_3 : Internal_Entity_Base_Type_Decl;
Fld_4 : Boolean;
Result_Var : Boolean;
Dyn_Var_Bind_Result : Boolean;
Let_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
            Populate_Lexical_Env (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8738 '<Block>' Let_Result None
--# scope-start
--# expr-start 8737 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <Then <FieldAccess .constraint>: <Var c> <FieldAccess .is_static>>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6668
--# expr-start 8732 '<FieldAccess .origin_node>' Fld /usr/src/packages/BUILD/ada/ast.py:6668
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Self);
--# end
--# expr-done 8732
Bound_Origin := Fld; 
--# expr-start 8733 '<FieldAccess .constraint>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6668
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Constraint (Node => Ent.Node.Subtype_Indication_F_Constraint, Info => Ent.Info);
--# expr-done 8733
Var_Expr := Fld_1;
if Var_Expr /= No_Entity_Constraint then
   --# scope-start
   --# bind c Var_Expr
--# expr-start 8734 '<FieldAccess .is_static>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6669
      if Var_Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Constraint.is_static
Fld_2 := Libadalang.Implementation.Constraint_P_Is_Static (Node => Var_Expr.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Var_Expr.Info);
--# end
--# expr-done 8734
Scope_Result := Fld_2;
   --# end
   Result_Var := Scope_Result;
else
   --# expr-start 8736 '<FieldAccess .is_static_decl>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6670
--# expr-start 8735 '<FieldAccess .designated_type>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6670
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_3 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8735
      if Fld_3.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicDecl.is_static_decl'
Fld_4 := Libadalang.Implementation.Dispatcher_Basic_Decl_P_Is_Static_Decl (Node => Fld_3.Node, Imprecise_Fallback => Imprecise_Fallback, E_Info => Fld_3.Info);
--# end
--# expr-done 8736
   Result_Var := Fld_4;
end if;
Dyn_Var_Bind_Result := Result_Var; 
--# expr-done 8737
Let_Result := Dyn_Var_Bind_Result; 
--# end
--# expr-done 8738
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Is_Static_Subtype;
--# end
--# property-start SubtypeIndication.custom_id_text /usr/src/packages/BUILD/ada/ast.py:6673
pragma Warnings (Off, "is not referenced");
 function Subtype_Indication_P_Custom_Id_Text
  (Node : Bare_Subtype_Indication
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Character_Type_Array_Access
is
   Self : Bare_Subtype_Indication := Bare_Subtype_Indication (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Subtype_Indication :=
     Internal_Entity_Subtype_Indication'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_615;
      Cast_Expr : Bare_Subtype_Indication;
Cast_Result : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
Fld : Internal_Entity_Base_Type_Decl;
Fld_1 : Character_Type_Array_Access;
Dyn_Var_Bind_Result : Character_Type_Array_Access;
            procedure Finalizer_Scope_615 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Dyn_Var_Bind_Result);
            end Finalizer_Scope_615;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8741 '<bind(<DynamicVariable origin>, <Var Self>, <FieldAccess .canonical_fully_qualified_name>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6673
Cast_Expr := Self; 
      Cast_Result := Cast_Expr;
Bound_Origin := Cast_Result; 
--# expr-start 8740 '<FieldAccess .canonical_fully_qualified_name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6675
--# expr-start 8739 '<FieldAccess .designated_type>' Fld /usr/src/packages/BUILD/ada/ast.py:6675
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Ent.Node, Origin => Bound_Origin, E_Info => Ent.Info);
--# end
--# expr-done 8739
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start BasicDecl.canonical_fully_qualified_name
Fld_1 := Libadalang.Implementation.Basic_Decl_P_Canonical_Fully_Qualified_Name (Node => Fld.Node, E_Info => Fld.Info);
--# end
--# expr-done 8740
Dyn_Var_Bind_Result := Fld_1; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8741
      Property_Result := Dyn_Var_Bind_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_615;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_615;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Subtype_Indication_P_Custom_Id_Text;
--# end
   --
   --  Primitives for Bare_Constrained_Subtype_Indication
   --
      procedure Initialize_Fields_For_Constrained_Subtype_Indication
        (Self : Bare_Constrained_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Constrained_Subtype_Indication;
   --
   --  Primitives for Bare_Discrete_Subtype_Indication
   --
      procedure Initialize_Fields_For_Discrete_Subtype_Indication
        (Self : Bare_Discrete_Subtype_Indication
         ; Subtype_Indication_F_Has_Not_Null : Bare_Not_Null
         ; Subtype_Indication_F_Name : Bare_Name
         ; Subtype_Indication_F_Constraint : Bare_Constraint
        ) is
      begin
            Initialize_Fields_For_Subtype_Indication
              (Self, Subtype_Indication_F_Has_Not_Null, Subtype_Indication_F_Name, Subtype_Indication_F_Constraint);
      end Initialize_Fields_For_Discrete_Subtype_Indication;
   --
   --  Primitives for Bare_Unconstrained_Array_Index
   --
      procedure Initialize_Fields_For_Unconstrained_Array_Index
        (Self : Bare_Unconstrained_Array_Index
         ; Unconstrained_Array_Index_F_Subtype_Indication : Bare_Subtype_Indication
        ) is
      begin
            Self.Unconstrained_Array_Index_F_Subtype_Indication := Unconstrained_Array_Index_F_Subtype_Indication;
      end Initialize_Fields_For_Unconstrained_Array_Index;
   function Unconstrained_Array_Index_F_Subtype_Indication
     (Node : Bare_Unconstrained_Array_Index) return Bare_Subtype_Indication
   is
   begin
         return Node.Unconstrained_Array_Index_F_Subtype_Indication;
   end;
--# property-start UnconstrainedArrayIndex.designated_type /usr/src/packages/BUILD/ada/ast.py:15791
pragma Warnings (Off, "is not referenced");
 function Unconstrained_Array_Index_P_Designated_Type
  (Node : Bare_Unconstrained_Array_Index
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Type_Decl
is
   Self : Bare_Unconstrained_Array_Index := Bare_Unconstrained_Array_Index (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Unconstrained_Array_Index :=
     Internal_Entity_Unconstrained_Array_Index'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind origin Origin
   Property_Result : Internal_Entity_Base_Type_Decl;
      Fld : Internal_Entity_Subtype_Indication;
Fld_1 : Internal_Entity_Base_Type_Decl;
Let_Result : Internal_Entity_Base_Type_Decl;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8744 '<Block>' Let_Result None
--# scope-start
--# expr-start 8743 '<FieldAccess .designated_type>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:15792
--# expr-start 8742 '<FieldAccess .subtype_indication>' Fld /usr/src/packages/BUILD/ada/ast.py:15792
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Subtype_Indication (Node => Ent.Node.Unconstrained_Array_Index_F_Subtype_Indication, Info => Ent.Info);
--# expr-done 8742
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]TypeExpr.designated_type'
Fld_1 := Libadalang.Implementation.Dispatcher_Type_Expr_P_Designated_Type (Node => Fld.Node, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8743
Let_Result := Fld_1; 
--# end
--# expr-done 8744
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Unconstrained_Array_Index_P_Designated_Type;
--# end
   --
   --  Primitives for Bare_Until_Node
   --
--# property-start '[dispatcher]Until.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_Until_Node_P_As_Bool
  (Node : Bare_Until_Node
  )
   return Boolean
is
   Self : Bare_Until_Node := Bare_Until_Node (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_Until_Node (Self.Kind) is
               when Ada_Until_Absent =>
                  --# property-call-start Until.Absent.as_bool
                  Property_Result := Until_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_Until_Present =>
                  --# property-call-start Until.Present.as_bool
                  Property_Result := Until_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_Until_Node_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Absent
   --
--# property-start Until.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3943
pragma Warnings (Off, "is not referenced");
 function Until_Absent_P_As_Bool
  (Node : Bare_Until_Absent
  )
   return Boolean
is
   Self : Bare_Until_Absent := Bare_Until_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Until_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Until_Present
   --
--# property-start Until.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3943
pragma Warnings (Off, "is not referenced");
 function Until_Present_P_As_Bool
  (Node : Bare_Until_Present
  )
   return Boolean
is
   Self : Bare_Until_Present := Bare_Until_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Until_Present_P_As_Bool;
--# end
   --
   --  Primitives for Bare_Use_Clause
   --
--# property-start UseClause.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6418
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Xref_Entry_Point
  (Node : Bare_Use_Clause
  )
   return Boolean
is
   Self : Bare_Use_Clause := Bare_Use_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Clause_P_Xref_Entry_Point;
--# end
--# property-start UseClause.used_envs /usr/src/packages/BUILD/ada/ast.py:6421
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Used_Envs
  (Node : Bare_Use_Clause
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Lexical_Env
is
   Self : Bare_Use_Clause := Bare_Use_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Use_Clause :=
     Internal_Entity_Use_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_592;
            procedure Finalizer_Scope_2649;
            procedure Finalizer_Scope_2650;
            procedure Finalizer_Scope_2651;
      Match_0 : Internal_Entity_Use_Package_Clause;
Match_1 : Internal_Entity_Use_Type_Clause;
Fld : Lexical_Env_Array_Access;
Group_Env : Lexical_Env := Empty_Env;
Fld_1 : Internal_Entity_Name_List;
Fld_2 : Bare_Name_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_4 : Lexical_Env := Empty_Env;
Item_260 : Internal_Entity_Name;
Bare_Item_260 : Bare_Name;
Map_Result : Lexical_Env_Array_Access;
Group_Env_1 : Lexical_Env := Empty_Env;
Match_Prefix : Internal_Entity_Use_Clause;
Cast_Result : Internal_Entity_Use_Package_Clause;
Let_Result : Lexical_Env := Empty_Env;
Scope_Result : Lexical_Env := Empty_Env;
Cast_Result_1 : Internal_Entity_Use_Type_Clause;
Let_Result_1 : Lexical_Env := Empty_Env;
Scope_Result_1 : Lexical_Env := Empty_Env;
Match_Result : Lexical_Env := Empty_Env;
Let_Result_2 : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_592 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Match_Result);
                     Dec_Ref (Let_Result_2);
            end Finalizer_Scope_592;
            procedure Finalizer_Scope_2649 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Group_Env);
            end Finalizer_Scope_2649;
            procedure Finalizer_Scope_2650 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Group_Env_1);
            end Finalizer_Scope_2650;
            procedure Finalizer_Scope_2651 is
            begin
                     Dec_Ref (Fld_4);
            end Finalizer_Scope_2651;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8752 '<Block>' Let_Result_2 None
--# scope-start
--# expr-start 8751 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:6426
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Ent; 
case Ada_Use_Clause (Match_Prefix.Node.Kind) is
         when Ada_Use_Package_Clause =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Use_Package_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind upc Match_0
--# expr-start 8746 '<env_group(<FieldAccess .designated_envs>)>' Group_Env /usr/src/packages/BUILD/ada/ast.py:6427
--# expr-start 8745 '<FieldAccess .designated_envs>' Fld /usr/src/packages/BUILD/ada/ast.py:6427
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UsePackageClause.designated_envs
Fld := Libadalang.Implementation.Use_Package_Clause_P_Designated_Envs (Node => Match_0.Node);
--# end
--# expr-done 8745
Group_Env := Group (Fld, No_Metadata); 
--# expr-done 8746
Let_Result := Group_Env; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2649;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Use_Type_Clause =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Use_Type_Clause
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind utc Match_1
--# expr-start 8750 '<env_group(<map>)>' Group_Env_1 /usr/src/packages/BUILD/ada/ast.py:6429
--# expr-start 8749 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6429
--# expr-start 8747 '<FieldAccess .types>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6429
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Name_List (Node => Match_1.Node.Use_Type_Clause_F_Types, Info => Match_1.Info);
--# expr-done 8747
Fld_2 := Fld_1.Node;
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      if Fld_2 = null then
         Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_2;
      begin
         for Untyped_Item_260 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_260 := Untyped_Item_260; 
Is_Null := Bare_Item_260 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_260); 
   If_Result := As_Entity;
end if;
                  Item_260 := If_Result; 
   --# scope-start
               --# bind n Item_260
      --# expr-start 8748 '<FieldAccess .name_designated_type_env>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:6429
      if Item_260.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.name_designated_type_env
Fld_4 := Libadalang.Implementation.Name_P_Name_Designated_Type_Env (Node => Item_260.Node, E_Info => Item_260.Info);
--# end
--# expr-done 8748
         declare
            Item_To_Append : constant Lexical_Env :=
               Fld_4;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2651;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8749
Group_Env_1 := Group (Map_Result, No_Metadata); 
--# expr-done 8750
Let_Result_1 := Group_Env_1; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2650;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
end case;
--# expr-done 8751
Let_Result_2 := Match_Result; Inc_Ref (Let_Result_2);
--# end
--# expr-done 8752
      Property_Result := Let_Result_2;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_592;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_592;
               Finalizer_Scope_2649;
               Finalizer_Scope_2650;
               Finalizer_Scope_2651;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Clause_P_Used_Envs;
--# end
--# property-start UseClause.initial_env_name /usr/src/packages/BUILD/ada/ast.py:6433
pragma Warnings (Off, "is not referenced");
 function Use_Clause_P_Initial_Env_Name
  (Node : Bare_Use_Clause
  )
   return Symbol_Type
is
   Self : Bare_Use_Clause := Bare_Use_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
If_Result : Symbol_Type;
Let_Result : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8758 '<Block>' Let_Result None
--# scope-start
--# expr-start 8757 '<If>' If_Result /usr/src/packages/BUILD/ada/ast.py:6438
--# expr-start 8755 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:6439
--# expr-start 8754 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6439
--# expr-start 8753 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6439
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8753
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8754
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8755
if Is_A then
   --# expr-start 8756 '<Symbol Standard>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Standard); 
--# expr-done 8756
   If_Result := Sym;
else
   If_Result := null;
end if;
--# expr-done 8757
Let_Result := If_Result; 
--# end
--# expr-done 8758
      Property_Result := Let_Result;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Clause_P_Initial_Env_Name;
--# end
   --
   --  Primitives for Bare_Use_Package_Clause
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Use_Package_Clause_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Use_Package_Clause := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_19 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:6452" & ")";
         end if;
      return Initial_Env;
   end Bare_Use_Package_Clause_Initial_Env_Getter_Fn;
      procedure Use_Package_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Package_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_18 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Use_Package_Clause_Initial_Env_Getter_Fn'Access);
      end;
      if Libadalang.Implementation.Ref_Cond_21 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_20 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Use_Package_Name_Designated_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Package_Clause
        (Self : Bare_Use_Package_Clause
         ; Use_Package_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.Use_Package_Clause_F_Packages := Use_Package_Clause_F_Packages;
      end Initialize_Fields_For_Use_Package_Clause;
   function Use_Package_Clause_F_Packages
     (Node : Bare_Use_Package_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Package_Clause_F_Packages;
   end;
--# property-start UsePackageClause.designated_envs /usr/src/packages/BUILD/ada/ast.py:6464
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Designated_Envs
  (Node : Bare_Use_Package_Clause
  )
   return Lexical_Env_Array_Access
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_594;
            procedure Finalizer_Scope_2652;
      Fld : Bare_Name_List;
Fld_1 : Lexical_Env := Empty_Env;
Bound_Env : Lexical_Env := Empty_Env;
Fld_2 : Bare_Ada_Node;
Bound_Origin : Bare_Ada_Node;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result : Lexical_Env := Empty_Env;
Dyn_Var_Bind_Result_1 : Lexical_Env := Empty_Env;
Item_261 : Bare_Name;
Map_Result : Lexical_Env_Array_Access;
Let_Result : Lexical_Env_Array_Access;
            procedure Finalizer_Scope_594 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_594;
            procedure Finalizer_Scope_2652 is
            begin
                     Dec_Ref (Fld_1);
                     Dec_Ref (Bound_Env);
                     Dec_Ref (Fld_3);
                     Dec_Ref (Dyn_Var_Bind_Result);
                     Dec_Ref (Dyn_Var_Bind_Result_1);
            end Finalizer_Scope_2652;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8767 '<Block>' Let_Result None
--# scope-start
--# expr-start 8766 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6471
--# expr-start 8759 '<FieldAccess .packages>' Fld /usr/src/packages/BUILD/ada/ast.py:6471
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 8759
declare
   Map_Result_Vec : Lexical_Env_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Lexical_Env_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_261 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_261 := Untyped_Item_261; 
   --# scope-start
               --# bind n Item_261
      --# expr-start 8765 '<bind(<DynamicVariable env>, <FieldAccess .node_env>, <bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>)>' Dyn_Var_Bind_Result_1 /usr/src/packages/BUILD/ada/ast.py:6473
--# expr-start 8760 '<FieldAccess .node_env>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6473
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Libadalang.Implementation.Node_Env (Node => Self);
--# expr-done 8760
Bound_Env := Fld_1; Inc_Ref (Bound_Env);
--# expr-start 8764 '<bind(<DynamicVariable origin>, <FieldAccess .origin_node>, <FieldAccess .designated_env>)>' Dyn_Var_Bind_Result /usr/src/packages/BUILD/ada/ast.py:6474
--# expr-start 8761 '<FieldAccess .origin_node>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:6474
      if Item_261 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.origin_node
Fld_2 := Libadalang.Implementation.Ada_Node_P_Origin_Node (Node => Item_261);
--# end
--# expr-done 8761
Bound_Origin := Fld_2; 
--# expr-start 8763 '<FieldAccess .designated_env>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6475
--# expr-start 8762 '<as_bare_entity(<Var n>)>' If_Result /usr/src/packages/BUILD/ada/ast.py:6475
Is_Null := Item_261 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
As_Entity := (Info => No_Entity_Info, Node => Item_261); 
   If_Result := As_Entity;
end if;
--# expr-done 8762
      if If_Result.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]Expr.designated_env'
Fld_3 := Libadalang.Implementation.Dispatcher_Expr_P_Designated_Env (Node => If_Result.Node, Env => Bound_Env, Origin => Bound_Origin, E_Info => If_Result.Info);
--# end
--# expr-done 8763
Dyn_Var_Bind_Result := Fld_3; Inc_Ref (Dyn_Var_Bind_Result);
--# expr-done 8764
Dyn_Var_Bind_Result_1 := Dyn_Var_Bind_Result; Inc_Ref (Dyn_Var_Bind_Result_1);
--# expr-done 8765
         declare
            Item_To_Append : constant Lexical_Env :=
               Dyn_Var_Bind_Result_1;
         begin
               Inc_Ref (Item_To_Append);
            Lexical_Env_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2652;
         end loop;
      end;
      Map_Result := Create_Lexical_Env_Array
        (Items_Count => Natural (Lexical_Env_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Lexical_Env_Vectors.Get
           (Map_Result_Vec,
            I + Lexical_Env_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Lexical_Env_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8766
Let_Result := Map_Result; Inc_Ref (Let_Result);
--# end
--# expr-done 8767
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_594;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_594;
               Finalizer_Scope_2652;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Package_Clause_P_Designated_Envs;
--# end
--# property-start UsePackageClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:6478
pragma Warnings (Off, "is not referenced");
 function Use_Package_Clause_P_Xref_Equation
  (Node : Bare_Use_Package_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Use_Package_Clause :=
     Internal_Entity_Use_Package_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_595;
            procedure Finalizer_Scope_2653;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_262 : Internal_Entity_Name;
Bare_Item_262 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_595 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_595;
            procedure Finalizer_Scope_2653 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2653;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8771 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6479
--# expr-start 8770 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6479
--# expr-start 8768 '<FieldAccess .packages>' Fld /usr/src/packages/BUILD/ada/ast.py:6479
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Package_Clause_F_Packages, Info => Ent.Info);
--# expr-done 8768
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_262 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_262 := Untyped_Item_262; 
Is_Null := Bare_Item_262 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_262); 
   If_Result := As_Entity;
end if;
                  Item_262 := If_Result; 
   --# scope-start
               --# bind p Item_262
      --# expr-start 8769 '<FieldAccess .xref_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6479
      if Item_262.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_262.Node, Env => Env, Origin => Origin, E_Info => Item_262.Info);
--# end
--# expr-done 8769
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2653;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8770
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6479") else null)); 
--# expr-done 8771
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_595;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_595;
               Finalizer_Scope_2653;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Package_Clause_P_Xref_Equation;
--# end
--# property-start UsePackageClause._initial_env_name_18 /usr/src/packages/BUILD/ada/ast.py:6453
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_18
  (Node : Bare_Use_Package_Clause
  )
   return Symbol_Type
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8772 '<FieldAccess .initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:6453
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UseClause.initial_env_name
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 8772
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_18;
--# end
--# property-start UsePackageClause._initial_env_19 /usr/src/packages/BUILD/ada/ast.py:6454
pragma Warnings (Off, "is not referenced");
 function Initial_Env_19
  (Node : Bare_Use_Package_Clause
  )
   return Lexical_Env
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1189;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1189 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1189;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8773 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6454
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 8773
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1189;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1189;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_19;
--# end
--# property-start UsePackageClause._ref_env_nodes_20 /usr/src/packages/BUILD/ada/ast.py:6457
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_20
  (Node : Bare_Use_Package_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1190;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_263 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1190 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1190;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8776 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6457
--# expr-start 8774 '<FieldAccess .packages>' Fld /usr/src/packages/BUILD/ada/ast.py:6457
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Package_Clause_F_Packages;
--# expr-done 8774
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_263 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_263 := Untyped_Item_263; 
   --# scope-start
               --# bind n Item_263
      --# expr-start 8775 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6457
Cast_Expr := Item_263; 
      Cast_Result := Cast_Expr;
--# expr-done 8775
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8776
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1190;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1190;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_20;
--# end
--# property-start UsePackageClause._ref_cond_21 /usr/src/packages/BUILD/ada/ast.py:6459
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_21
  (Node : Bare_Use_Package_Clause
  )
   return Boolean
is
   Self : Bare_Use_Package_Clause := Bare_Use_Package_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8780 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:6459
--# expr-start 8779 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:6459
--# expr-start 8778 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6459
--# expr-start 8777 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6459
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8777
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8778
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8779
Not_Val := not (Is_A); 
--# expr-done 8780
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_21;
--# end
   --
   --  Primitives for Bare_Use_Type_Clause
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_Use_Type_Clause_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_Use_Type_Clause := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_23 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:6491" & ")";
         end if;
      return Initial_Env;
   end Bare_Use_Type_Clause_Initial_Env_Getter_Fn;
      procedure Use_Type_Clause_Pre_Env_Actions
        (Self            : Bare_Use_Type_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_22 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_Use_Type_Clause_Initial_Env_Getter_Fn'Access);
      end;
      if Libadalang.Implementation.Ref_Cond_25 (Node => 
Self) then
         declare
            Ref_Env_Nodes : Bare_Ada_Node_Array_Access :=
               Libadalang.Implementation.Ref_Env_Nodes_24 (Node => 
Self);
            Env : Lexical_Env :=
              Self.Self_Env;
         begin
            Ref_Env
              (Self,
               Env,
               Ref_Env_Nodes,
               Name_P_Name_Designated_Type_Env'Access,
               Normal,
               All_Cats,
               False);
            Dec_Ref (Ref_Env_Nodes);
         end;
      end if;
      end;
      procedure Initialize_Fields_For_Use_Type_Clause
        (Self : Bare_Use_Type_Clause
         ; Use_Type_Clause_F_Has_All : Bare_All_Node
         ; Use_Type_Clause_F_Types : Bare_Name_List
        ) is
      begin
            Self.Use_Type_Clause_F_Has_All := Use_Type_Clause_F_Has_All;
            Self.Use_Type_Clause_F_Types := Use_Type_Clause_F_Types;
      end Initialize_Fields_For_Use_Type_Clause;
   function Use_Type_Clause_F_Has_All
     (Node : Bare_Use_Type_Clause) return Bare_All_Node
   is
   begin
         return Node.Use_Type_Clause_F_Has_All;
   end;
   function Use_Type_Clause_F_Types
     (Node : Bare_Use_Type_Clause) return Bare_Name_List
   is
   begin
         return Node.Use_Type_Clause_F_Types;
   end;
--# property-start UseTypeClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:6502
pragma Warnings (Off, "is not referenced");
 function Use_Type_Clause_P_Xref_Equation
  (Node : Bare_Use_Type_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Use_Type_Clause :=
     Internal_Entity_Use_Type_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_596;
            procedure Finalizer_Scope_2655;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_264 : Internal_Entity_Name;
Bare_Item_264 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_596 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_596;
            procedure Finalizer_Scope_2655 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2655;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8784 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6503
--# expr-start 8783 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6503
--# expr-start 8781 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:6503
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.Use_Type_Clause_F_Types, Info => Ent.Info);
--# expr-done 8781
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_264 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_264 := Untyped_Item_264; 
Is_Null := Bare_Item_264 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_264); 
   If_Result := As_Entity;
end if;
                  Item_264 := If_Result; 
   --# scope-start
               --# bind p Item_264
      --# expr-start 8782 '<FieldAccess .xref_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6503
      if Item_264.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_264.Node, Env => Env, Origin => Origin, E_Info => Item_264.Info);
--# end
--# expr-done 8782
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2655;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8783
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6503") else null)); 
--# expr-done 8784
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_596;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_596;
               Finalizer_Scope_2655;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Use_Type_Clause_P_Xref_Equation;
--# end
--# property-start UseTypeClause._initial_env_name_22 /usr/src/packages/BUILD/ada/ast.py:6492
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_22
  (Node : Bare_Use_Type_Clause
  )
   return Symbol_Type
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Fld : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8785 '<FieldAccess .initial_env_name>' Fld /usr/src/packages/BUILD/ada/ast.py:6492
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start UseClause.initial_env_name
Fld := Libadalang.Implementation.Use_Clause_P_Initial_Env_Name (Node => Self);
--# end
--# expr-done 8785
      Property_Result := Fld;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_22;
--# end
--# property-start UseTypeClause._initial_env_23 /usr/src/packages/BUILD/ada/ast.py:6493
pragma Warnings (Off, "is not referenced");
 function Initial_Env_23
  (Node : Bare_Use_Type_Clause
  )
   return Lexical_Env
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
            procedure Finalizer_Scope_1193;
      Fld : Lexical_Env := Empty_Env;
            procedure Finalizer_Scope_1193 is
            begin
                     Dec_Ref (Fld);
            end Finalizer_Scope_1193;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8786 '<FieldAccess .default_initial_env>' Fld /usr/src/packages/BUILD/ada/ast.py:6493
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.default_initial_env
Fld := Libadalang.Implementation.Ada_Node_P_Default_Initial_Env (Node => Self);
--# end
--# expr-done 8786
      Property_Result := Fld;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1193;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1193;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_23;
--# end
--# property-start UseTypeClause._ref_env_nodes_24 /usr/src/packages/BUILD/ada/ast.py:6496
pragma Warnings (Off, "is not referenced");
 function Ref_Env_Nodes_24
  (Node : Bare_Use_Type_Clause
  )
   return Bare_Ada_Node_Array_Access
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1194;
      Fld : Bare_Name_List;
Cast_Expr : Bare_Name;
Cast_Result : Bare_Ada_Node;
Item_265 : Bare_Name;
Map_Result : Bare_Ada_Node_Array_Access;
            procedure Finalizer_Scope_1194 is
            begin
                     Dec_Ref (Map_Result);
            end Finalizer_Scope_1194;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8789 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6496
--# expr-start 8787 '<FieldAccess .types>' Fld /usr/src/packages/BUILD/ada/ast.py:6496
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Use_Type_Clause_F_Types;
--# expr-done 8787
declare
   Map_Result_Vec : Bare_Ada_Node_Vectors.Vector;
begin
      if Fld = null then
         Map_Result := Create_Bare_Ada_Node_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld;
      begin
         for Untyped_Item_265 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Item_265 := Untyped_Item_265; 
   --# scope-start
               --# bind n Item_265
      --# expr-start 8788 '<Cast to AdaNode>' Cast_Result /usr/src/packages/BUILD/ada/ast.py:6496
Cast_Expr := Item_265; 
      Cast_Result := Cast_Expr;
--# expr-done 8788
         declare
            Item_To_Append : constant Bare_Ada_Node :=
               Cast_Result;
         begin
            Bare_Ada_Node_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
         end loop;
      end;
      Map_Result := Create_Bare_Ada_Node_Array
        (Items_Count => Natural (Bare_Ada_Node_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Bare_Ada_Node_Vectors.Get
           (Map_Result_Vec,
            I + Bare_Ada_Node_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Bare_Ada_Node_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8789
      Property_Result := Map_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_1194;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_1194;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Env_Nodes_24;
--# end
--# property-start UseTypeClause._ref_cond_25 /usr/src/packages/BUILD/ada/ast.py:6498
pragma Warnings (Off, "is not referenced");
 function Ref_Cond_25
  (Node : Bare_Use_Type_Clause
  )
   return Boolean
is
   Self : Bare_Use_Type_Clause := Bare_Use_Type_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
      Fld : Bare_Ada_Node;
Fld_1 : Bare_Ada_Node;
Is_A : Boolean;
Not_Val : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8793 '<Not>' Not_Val /usr/src/packages/BUILD/ada/ast.py:6498
--# expr-start 8792 '<IsA CompilationUnit>' Is_A /usr/src/packages/BUILD/ada/ast.py:6498
--# expr-start 8791 '<parent(<parent(<Var Self>)>)>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:6498
--# expr-start 8790 '<parent(<Var Self>)>' Fld /usr/src/packages/BUILD/ada/ast.py:6498
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Self.Parent; 
--# expr-done 8790
      if Fld = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Fld.Parent; 
--# expr-done 8791
Is_A := Fld_1 /= null 
and then Fld_1.Kind in Ada_Compilation_Unit_Range; 
--# expr-done 8792
Not_Val := not (Is_A); 
--# expr-done 8793
      Property_Result := Not_Val;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Ref_Cond_25;
--# end
   --
   --  Primitives for Bare_Variant
   --
      procedure Initialize_Fields_For_Variant
        (Self : Bare_Variant
         ; Variant_F_Choices : Bare_Alternatives_List
         ; Variant_F_Components : Bare_Component_List
        ) is
      begin
            Self.Variant_F_Choices := Variant_F_Choices;
            Self.Variant_F_Components := Variant_F_Components;
      end Initialize_Fields_For_Variant;
   function Variant_F_Choices
     (Node : Bare_Variant) return Bare_Alternatives_List
   is
   begin
         return Node.Variant_F_Choices;
   end;
   function Variant_F_Components
     (Node : Bare_Variant) return Bare_Component_List
   is
   begin
         return Node.Variant_F_Components;
   end;
--# property-start Variant.matches /usr/src/packages/BUILD/ada/ast.py:3581
pragma Warnings (Off, "is not referenced");
 function Variant_P_Matches
  (Node : Bare_Variant
      ; Expr : Internal_Entity_Expr
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Boolean
is
   Self : Bare_Variant := Bare_Variant (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant :=
     Internal_Entity_Variant'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind expr Expr
   Property_Result : Boolean;
            procedure Finalizer_Scope_245;
      Block_Var : Big_Integer_Type;
Fld : Big_Integer_Type;
Fld_1 : Internal_Entity_Alternatives_List;
Fld_2 : Bare_Alternatives_List;
Fld_3 : Internal_Entity_Info;
As_Entity : Internal_Entity;
Is_Null : Boolean;
If_Result : Internal_Entity;
Fld_4 : Boolean;
Item_266 : Internal_Entity;
Bare_Item_266 : Bare_Ada_Node;
Quantifier_Result : Boolean;
Let_Result : Boolean;
            procedure Finalizer_Scope_245 is
            begin
                     Dec_Ref (Fld);
                     Dec_Ref (Block_Var);
            end Finalizer_Scope_245;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8797 '<Block>' Let_Result None
--# scope-start
--# expr-start 8794 '<FieldAccess .eval_as_int>' Fld /usr/src/packages/BUILD/ada/ast.py:3586
      if Expr.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.eval_as_int
Fld := Libadalang.Implementation.Expr_P_Eval_As_Int (Node => Expr.Node, E_Info => Expr.Info);
--# end
--# expr-done 8794
Block_Var := Fld; Inc_Ref (Block_Var);
--# bind expr_val Block_Var
--# expr-start 8795 '<FieldAccess .choices>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3588
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_1 := Create_Internal_Entity_Alternatives_List (Node => Ent.Node.Variant_F_Choices, Info => Ent.Info);
--# expr-done 8795
Fld_2 := Fld_1.Node;
Quantifier_Result := False;
   if Fld_2 /= null then
   declare
      Collection : constant Bare_Alternatives_List := Fld_2;
   begin
      for Untyped_Item_266 of
            Collection.Nodes (1 .. Children_Count (Collection))
      loop
               Bare_Item_266 := Untyped_Item_266; 
Is_Null := Bare_Item_266 = null; 
if Is_Null then
   If_Result := No_Entity;
else
Fld_3 := Fld_1.Info;
As_Entity := (Info => Fld_3, Node => Bare_Item_266); 
   If_Result := As_Entity;
end if;
               Item_266 := If_Result; 
   --# scope-start
            --# bind c Item_266
         --# expr-start 8796 '<FieldAccess .choice_match(...)>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3589
      if Item_266.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.choice_match
Fld_4 := Libadalang.Implementation.Ada_Node_P_Choice_Match (Node => Item_266.Node, Value => Block_Var, E_Info => Item_266.Info);
--# end
--# expr-done 8796
            if Fld_4 then
               Quantifier_Result := True;
               exit;
            end if;
   --# end
      end loop;
   end;
   end if;
Let_Result := Quantifier_Result; 
--# end
--# expr-done 8797
      Property_Result := Let_Result;
   --# end
      Finalizer_Scope_245;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_245;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_P_Matches;
--# end
   --
   --  Primitives for Bare_Variant_Part
   --
      procedure Initialize_Fields_For_Variant_Part
        (Self : Bare_Variant_Part
         ; Variant_Part_F_Discr_Name : Bare_Identifier
         ; Variant_Part_F_Variant : Bare_Variant_List
        ) is
      begin
            Self.Variant_Part_F_Discr_Name := Variant_Part_F_Discr_Name;
            Self.Variant_Part_F_Variant := Variant_Part_F_Variant;
      end Initialize_Fields_For_Variant_Part;
   function Variant_Part_F_Discr_Name
     (Node : Bare_Variant_Part) return Bare_Identifier
   is
   begin
         return Node.Variant_Part_F_Discr_Name;
   end;
   function Variant_Part_F_Variant
     (Node : Bare_Variant_Part) return Bare_Variant_List
   is
   begin
         return Node.Variant_Part_F_Variant;
   end;
--# property-start VariantPart.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:3602
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Entry_Point
  (Node : Bare_Variant_Part
  )
   return Boolean
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Xref_Entry_Point;
--# end
--# property-start VariantPart.xref_equation /usr/src/packages/BUILD/ada/ast.py:3605
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Xref_Equation
  (Node : Bare_Variant_Part
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant_Part :=
     Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_247;
            procedure Finalizer_Scope_2658;
            procedure Finalizer_Scope_2659;
            procedure Finalizer_Scope_2660;
            procedure Finalizer_Scope_2661;
            procedure Finalizer_Scope_2662;
      Block_Var : Boolean;
Match_0 : Internal_Entity_Expr;
Match_1 : Internal_Entity_Subtype_Indication;
Match_2 : Internal_Entity_Others_Designator;
Match_3 : Internal_Entity;
Fld : Internal_Entity_Identifier;
Fld_1 : Boolean;
Fld_2 : Internal_Entity_Variant_List;
Fld_3 : Bare_Variant_List;
Fld_4 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Fld_5 : Internal_Entity_Alternatives_List;
Fld_6 : Bare_Alternatives_List;
Fld_7 : Internal_Entity_Info;
As_Entity_1 : Internal_Entity;
Is_Null_1 : Boolean;
If_Result_1 : Internal_Entity;
Fld_8 : Logic_Var;
Fld_9 : Bare_Identifier;
Fld_10 : Internal_Entity;
Fld_11 : Logic_Equation;
Fld_12 : Logic_Equation;
And_Pred : Logic_Equation;
Fld_13 : Logic_Equation;
Logic_True : Logic_Equation;
Match_Prefix : Internal_Entity;
Cast_Result : Internal_Entity_Expr;
Let_Result : Logic_Equation;
Scope_Result : Logic_Equation;
Cast_Result_1 : Internal_Entity_Subtype_Indication;
Let_Result_1 : Logic_Equation;
Scope_Result_1 : Logic_Equation;
Cast_Result_2 : Internal_Entity_Others_Designator;
Let_Result_2 : Logic_Equation;
Scope_Result_2 : Logic_Equation;
Cast_Result_3 : Internal_Entity;
Let_Result_3 : Logic_Equation;
Scope_Result_3 : Logic_Equation;
Match_Result : Logic_Equation;
Item_268 : Internal_Entity;
Bare_Item_268 : Bare_Ada_Node;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
Item_267 : Internal_Entity_Variant;
Bare_Item_267 : Bare_Variant;
Map_Result_1 : Logic_Equation_Array_Access;
Logic_Boolean_Op_1 : Logic_Equation;
Let_Result_4 : Logic_Equation;
            procedure Finalizer_Scope_247 is
            begin
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Logic_Boolean_Op_1);
                     Dec_Ref (Let_Result_4);
            end Finalizer_Scope_247;
            procedure Finalizer_Scope_2658 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_2658;
            procedure Finalizer_Scope_2659 is
            begin
                     Dec_Ref (Let_Result);
                     Dec_Ref (Scope_Result);
                     Dec_Ref (Let_Result_1);
                     Dec_Ref (Scope_Result_1);
                     Dec_Ref (Let_Result_2);
                     Dec_Ref (Scope_Result_2);
                     Dec_Ref (Let_Result_3);
                     Dec_Ref (Scope_Result_3);
                     Dec_Ref (Match_Result);
            end Finalizer_Scope_2659;
            procedure Finalizer_Scope_2660 is
            begin
                     Dec_Ref (Fld_11);
                     Dec_Ref (Fld_12);
                     Dec_Ref (And_Pred);
            end Finalizer_Scope_2660;
            procedure Finalizer_Scope_2661 is
            begin
                     Dec_Ref (Fld_13);
            end Finalizer_Scope_2661;
            procedure Finalizer_Scope_2662 is
            begin
                     Dec_Ref (Logic_True);
            end Finalizer_Scope_2662;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8814 '<Block>' Let_Result_4 None
--# scope-start
--# expr-start 8799 '<FieldAccess .resolve_names_internal>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3606
--# expr-start 8798 '<FieldAccess .discr_name>' Fld /usr/src/packages/BUILD/ada/ast.py:3606
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Identifier (Node => Ent.Node.Variant_Part_F_Discr_Name, Info => Ent.Info);
--# expr-done 8798
      if Fld.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.resolve_names_internal
Fld_1 := Libadalang.Implementation.Ada_Node_P_Resolve_Names_Internal (Node => Fld.Node, Env => Env, Origin => Origin, E_Info => Fld.Info);
--# end
--# expr-done 8799
Block_Var := Fld_1; 
--# expr-start 8813 '<LogicAll>' Logic_Boolean_Op_1 /usr/src/packages/BUILD/ada/ast.py:3608
--# expr-start 8812 '<map>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:3608
--# expr-start 8800 '<FieldAccess .variant>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3608
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 8800
Fld_3 := Fld_2.Node;
declare
   Map_Result_1_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_3 = null then
         Map_Result_1 := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_3;
      begin
         for Untyped_Item_267 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_267 := Untyped_Item_267; 
Is_Null := Bare_Item_267 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_4 := Fld_2.Info;
As_Entity := (Info => Fld_4, Node => Bare_Item_267); 
   If_Result := As_Entity;
end if;
                  Item_267 := If_Result; 
   --# scope-start
               --# bind var Item_267
      --# expr-start 8811 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:3609
--# expr-start 8810 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3609
--# expr-start 8801 '<FieldAccess .choices>' Fld_5 /usr/src/packages/BUILD/ada/ast.py:3609
      if Item_267.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_5 := Create_Internal_Entity_Alternatives_List (Node => Item_267.Node.Variant_F_Choices, Info => Item_267.Info);
--# expr-done 8801
Fld_6 := Fld_5.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_6 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Alternatives_List := Fld_6;
      begin
         for Untyped_Item_268 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_268 := Untyped_Item_268; 
Is_Null_1 := Bare_Item_268 = null; 
if Is_Null_1 then
   If_Result_1 := No_Entity;
else
Fld_7 := Fld_5.Info;
As_Entity_1 := (Info => Fld_7, Node => Bare_Item_268); 
   If_Result_1 := As_Entity_1;
end if;
                  Item_268 := If_Result_1; 
   --# scope-start
               --# bind c Item_268
      --# expr-start 8809 '<Match>' Match_Result /usr/src/packages/BUILD/ada/ast.py:3609
      if Item_268.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Match_Prefix := Item_268; 
case Ada_Ada_Node (Match_Prefix.Node.Kind) is
         when Ada_Abstract_State_Decl_Expr .. Ada_Un_Op =>
   --# scope-start
--# scope-start
      Cast_Result := Create_Internal_Entity_Expr
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_0 := Cast_Result; 
--# bind e Match_0
--# expr-start 8807 '<And>' And_Pred /usr/src/packages/BUILD/ada/ast.py:3612
--# expr-start 8805 '<FieldAccess .type_bind_val(...)>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3612
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8802 '<FieldAccess .type_var>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3612
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_8 := Libadalang.Implementation.Extensions.Expr_P_Type_Var (Node => Match_0.Node);
--# expr-done 8802
--# expr-start 8804 '<FieldAccess .type_val>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3612
--# expr-start 8803 '<FieldAccess .discr_name>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3612
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_9 := Self.Variant_Part_F_Discr_Name;
--# expr-done 8803
      if Fld_9 = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Expr.type_val
Fld_10 := Libadalang.Implementation.Expr_P_Type_Val (Node => Fld_9);
--# end
--# expr-done 8804
--# property-call-start AdaNode.type_bind_val
Fld_11 := Libadalang.Implementation.Ada_Node_P_Type_Bind_Val (Node => Self, Left => Fld_8, Right => Fld_10, Origin => Origin);
--# end
--# expr-done 8805
--# expr-start 8806 '<FieldAccess .sub_equation>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3613
      if Match_0.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start AdaNode.sub_equation
Fld_12 := Libadalang.Implementation.Ada_Node_P_Sub_Equation (Node => Match_0.Node, Env => Env, Origin => Origin, E_Info => Match_0.Info);
--# end
--# expr-done 8806
And_Pred := Logic_And (Fld_11, Fld_12, (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3612") else null)); 
--# expr-done 8807
Let_Result := And_Pred; Inc_Ref (Let_Result);
--# end
Scope_Result := Let_Result;
   Inc_Ref (Scope_Result);
   --# end
      Finalizer_Scope_2660;
            Match_Result := Scope_Result; Inc_Ref (Match_Result);
         when Ada_Subtype_Indication .. Ada_Discrete_Subtype_Indication =>
   --# scope-start
--# scope-start
      Cast_Result_1 := Create_Internal_Entity_Subtype_Indication
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_1 := Cast_Result_1; 
--# bind t Match_1
--# expr-start 8808 '<FieldAccess .xref_equation>' Fld_13 /usr/src/packages/BUILD/ada/ast.py:3616
      if Match_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]AdaNode.xref_equation'
Fld_13 := Libadalang.Implementation.Dispatcher_Ada_Node_P_Xref_Equation (Node => Match_1.Node, Env => Env, Origin => Origin, E_Info => Match_1.Info);
--# end
--# expr-done 8808
Let_Result_1 := Fld_13; Inc_Ref (Let_Result_1);
--# end
Scope_Result_1 := Let_Result_1;
   Inc_Ref (Scope_Result_1);
   --# end
      Finalizer_Scope_2661;
            Match_Result := Scope_Result_1; Inc_Ref (Match_Result);
         when Ada_Others_Designator =>
   --# scope-start
--# scope-start
      Cast_Result_2 := Create_Internal_Entity_Others_Designator
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_2 := Cast_Result_2; 
--# bind _ Match_2
Logic_True := True_Rel ((if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3618") else null)); 
Let_Result_2 := Logic_True; Inc_Ref (Let_Result_2);
--# end
Scope_Result_2 := Let_Result_2;
   Inc_Ref (Scope_Result_2);
   --# end
      Finalizer_Scope_2662;
            Match_Result := Scope_Result_2; Inc_Ref (Match_Result);
         when Ada_Abort_Absent .. Ada_Elsif_Stmt_Part | Ada_Handled_Stmts .. Ada_Null_Component_Decl | Ada_Overriding_Not_Overriding .. Ada_Enum_Lit_Synth_Type_Expr | Ada_Unconstrained_Array_Index .. Ada_With_Private_Present =>
   --# scope-start
--# scope-start
      Cast_Result_3 := Create_Internal_Entity
        (Node => Match_Prefix.Node,
         Info => Match_Prefix.Info);
Match_3 := Cast_Result_3; 
--# bind _ Match_3
Let_Result_3 := raise Property_Error with "Should not happen"; Inc_Ref (Let_Result_3);
--# end
Scope_Result_3 := Let_Result_3;
   Inc_Ref (Scope_Result_3);
   --# end
            Match_Result := Scope_Result_3; Inc_Ref (Match_Result);
end case;
--# expr-done 8809
         declare
            Item_To_Append : constant Logic_Equation :=
               Match_Result;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2659;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8810
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3609") else null)); 
--# expr-done 8811
         declare
            Item_To_Append : constant Logic_Equation :=
               Logic_Boolean_Op;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2658;
         end loop;
      end;
      Map_Result_1 := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_1_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8812
Logic_Boolean_Op_1 := Logic_All (Relation_Array (Map_Result_1.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:3608") else null)); 
--# expr-done 8813
Let_Result_4 := Logic_Boolean_Op_1; Inc_Ref (Let_Result_4);
--# end
--# expr-done 8814
      Property_Result := Let_Result_4;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_247;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_247;
               Finalizer_Scope_2658;
               Finalizer_Scope_2659;
               Finalizer_Scope_2660;
               Finalizer_Scope_2661;
               Finalizer_Scope_2662;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Xref_Equation;
--# end
--# property-start VariantPart.get_components /usr/src/packages/BUILD/ada/ast.py:3625
pragma Warnings (Off, "is not referenced");
 function Variant_Part_P_Get_Components
  (Node : Bare_Variant_Part
      ; Discriminants : Internal_Param_Match_Array_Access
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Internal_Entity_Base_Formal_Param_Decl_Array_Access
is
   Self : Bare_Variant_Part := Bare_Variant_Part (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_Variant_Part :=
     Internal_Entity_Variant_Part'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind discriminants Discriminants
   Property_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_248;
      Block_Var : Internal_Param_Match;
Block_Var_1 : Internal_Entity_Variant;
Fld : Internal_Single_Formal;
Fld_1 : Internal_Entity_Defining_Name;
Fld_2 : Bare_Identifier;
Sym : Symbol_Type;
Fld_3 : Boolean;
Map_Result : Internal_Param_Match_Array_Access;
Get_Result : Internal_Param_Match;
Fld_4 : Internal_Entity_Variant_List;
Fld_5 : Bare_Variant_List;
Fld_6 : Internal_Entity_Info;
As_Entity : Internal_Entity_Variant;
Is_Null : Boolean;
If_Result : Internal_Entity_Variant;
Item_270 : Internal_Entity_Variant;
Bare_Item_270 : Bare_Variant;
Fld_7 : Internal_Single_Actual;
Fld_8 : Internal_Entity_Basic_Assoc;
Fld_9 : Internal_Entity_Expr;
Fld_10 : Boolean;
Map_Result_1 : Internal_Entity_Variant_Array_Access;
Get_Result_1 : Internal_Entity_Variant;
Fld_11 : Internal_Entity_Component_List;
Fld_12 : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
Let_Result : Internal_Entity_Base_Formal_Param_Decl_Array_Access;
            procedure Finalizer_Scope_248 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Map_Result_1);
                     Dec_Ref (Fld_12);
                     Dec_Ref (Let_Result);
            end Finalizer_Scope_248;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8831 '<Block>' Let_Result None
--# scope-start
--# expr-start 8821 '<at(<filter>, 0)>' Get_Result /usr/src/packages/BUILD/ada/ast.py:3631
--# expr-start 8820 '<filter>' Map_Result /usr/src/packages/BUILD/ada/ast.py:3631
declare
   Map_Result_Vec : Internal_Param_Match_Vectors.Vector;
begin
      declare
         Collection : constant Internal_Param_Match_Array_Access := Discriminants;
      begin
         for Item_269 of
               Collection.Items
         loop
   --# scope-start
               --# bind d Item_269
               --# expr-start 8819 '<FieldAccess .name_is(...)>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:3632
--# expr-start 8816 '<FieldAccess .name>' Fld_1 /usr/src/packages/BUILD/ada/ast.py:3632
--# expr-start 8815 '<FieldAccess .formal>' Fld /usr/src/packages/BUILD/ada/ast.py:3632
Fld := Item_269.Formal;
--# expr-done 8815
Fld_1 := Fld.Name;
--# expr-done 8816
      if Fld_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8818 '<GetSymbol>' Sym /usr/src/packages/BUILD/ada/ast.py:3632
--# expr-start 8817 '<FieldAccess .discr_name>' Fld_2 /usr/src/packages/BUILD/ada/ast.py:3632
      if Self = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_2 := Self.Variant_Part_F_Discr_Name;
--# expr-done 8817
Sym := Get_Symbol (Fld_2); 
--# expr-done 8818
--# property-call-start Name.name_is
Fld_3 := Libadalang.Implementation.Name_P_Name_Is (Node => Fld_1.Node, Sym => Sym);
--# end
--# expr-done 8819
               if Fld_3 then
         declare
            Item_To_Append : constant Internal_Param_Match :=
               Item_269;
         begin
            Internal_Param_Match_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result := Create_Internal_Param_Match_Array
        (Items_Count => Natural (Internal_Param_Match_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Internal_Param_Match_Vectors.Get
           (Map_Result_Vec,
            I + Internal_Param_Match_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Internal_Param_Match_Vectors.Destroy (Map_Result_Vec);
end;
--# expr-done 8820
Get_Result := Get (Map_Result, 0, True); 
--# expr-done 8821
Block_Var := Get_Result; 
--# bind discr Block_Var
--# expr-start 8828 '<at(<filter>, 0)>' Get_Result_1 /usr/src/packages/BUILD/ada/ast.py:3637
--# expr-start 8827 '<filter>' Map_Result_1 /usr/src/packages/BUILD/ada/ast.py:3637
--# expr-start 8822 '<FieldAccess .variant>' Fld_4 /usr/src/packages/BUILD/ada/ast.py:3637
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_4 := Create_Internal_Entity_Variant_List (Node => Ent.Node.Variant_Part_F_Variant, Info => Ent.Info);
--# expr-done 8822
Fld_5 := Fld_4.Node;
declare
   Map_Result_1_Vec : Internal_Entity_Variant_Vectors.Vector;
begin
      if Fld_5 = null then
         Map_Result_1 := Create_Internal_Entity_Variant_Array (0);
      else
      declare
         Collection : constant Bare_Variant_List := Fld_5;
      begin
         for Untyped_Item_270 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_270 := Untyped_Item_270; 
Is_Null := Bare_Item_270 = null; 
if Is_Null then
   If_Result := No_Entity_Variant;
else
Fld_6 := Fld_4.Info;
As_Entity := (Info => Fld_6, Node => Bare_Item_270); 
   If_Result := As_Entity;
end if;
                  Item_270 := If_Result; 
   --# scope-start
               --# bind v Item_270
               --# expr-start 8826 '<FieldAccess .matches(...)>' Fld_10 /usr/src/packages/BUILD/ada/ast.py:3638
      if Item_270.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# expr-start 8825 '<FieldAccess .expr>' Fld_9 /usr/src/packages/BUILD/ada/ast.py:3638
--# expr-start 8824 '<FieldAccess .assoc>' Fld_8 /usr/src/packages/BUILD/ada/ast.py:3638
--# expr-start 8823 '<FieldAccess .actual>' Fld_7 /usr/src/packages/BUILD/ada/ast.py:3638
Fld_7 := Block_Var.Actual;
--# expr-done 8823
Fld_8 := Fld_7.Assoc;
--# expr-done 8824
      if Fld_8.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start '[dispatcher]BasicAssoc.expr'
Fld_9 := Libadalang.Implementation.Dispatcher_Basic_Assoc_P_Expr (Node => Fld_8.Node, E_Info => Fld_8.Info);
--# end
--# expr-done 8825
--# property-call-start Variant.matches
Fld_10 := Libadalang.Implementation.Variant_P_Matches (Node => Item_270.Node, Expr => Fld_9, E_Info => Item_270.Info);
--# end
--# expr-done 8826
               if Fld_10 then
         declare
            Item_To_Append : constant Internal_Entity_Variant :=
               Item_270;
         begin
            Internal_Entity_Variant_Vectors.Append (Map_Result_1_Vec, Item_To_Append);
         end;
               end if;
   --# end
         end loop;
      end;
      Map_Result_1 := Create_Internal_Entity_Variant_Array
        (Items_Count => Natural (Internal_Entity_Variant_Vectors.Length (Map_Result_1_Vec)));
      for I in Map_Result_1.Items'Range loop
         Map_Result_1.Items (I) := Internal_Entity_Variant_Vectors.Get
           (Map_Result_1_Vec,
            I + Internal_Entity_Variant_Vectors.Index_Type'First - Map_Result_1.Items'First);
      end loop;
      Internal_Entity_Variant_Vectors.Destroy (Map_Result_1_Vec);
      end if;
end;
--# expr-done 8827
Get_Result_1 := Get (Map_Result_1, 0, True); 
--# expr-done 8828
Block_Var_1 := Get_Result_1; 
--# bind variant Block_Var_1
--# expr-start 8830 '<FieldAccess .abstract_formal_params_impl(...)>' Fld_12 /usr/src/packages/BUILD/ada/ast.py:3644
--# expr-start 8829 '<FieldAccess .components>' Fld_11 /usr/src/packages/BUILD/ada/ast.py:3644
      if Block_Var_1.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld_11 := Create_Internal_Entity_Component_List (Node => Block_Var_1.Node.Variant_F_Components, Info => Block_Var_1.Info);
--# expr-done 8829
      if Fld_11.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start ComponentList.abstract_formal_params_impl
Fld_12 := Libadalang.Implementation.Component_List_P_Abstract_Formal_Params_Impl (Node => Fld_11.Node, Discriminants => Discriminants, Include_Discriminants => False, Recurse => False, E_Info => Fld_11.Info);
--# end
--# expr-done 8830
Let_Result := Fld_12; Inc_Ref (Let_Result);
--# end
--# expr-done 8831
      Property_Result := Let_Result;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_248;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_248;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Variant_Part_P_Get_Components;
--# end
   --
   --  Primitives for Bare_With_Clause
   --
   ---------------------------
   -- Initial_Env_Getter_Fn --
   ---------------------------
   function Bare_With_Clause_Initial_Env_Getter_Fn (E : Entity) return Lexical_Env is
      Self : constant Bare_With_Clause := E.Node;
      Bound_Env : constant Lexical_Env :=
        (if Self.Parent /= null
         then Self.Parent.Self_Env
         else Self.Self_Env);
      Initial_Env : Lexical_Env := Bound_Env;
   begin
      Initial_Env := Libadalang.Implementation.Initial_Env_17 (Node => 
Self);
      if Initial_Env.Kind /= Static_Primary then
         raise Property_Error with
            "Cannot set an env that is not static-primary as the initial env";
      end if;
         if Is_Foreign_Strict (Initial_Env, Self) then
            raise Property_Error with
               "unsound foreign environment in SetInitialEnv ("
               & "ast.py:6406" & ")";
         end if;
      return Initial_Env;
   end Bare_With_Clause_Initial_Env_Getter_Fn;
      procedure With_Clause_Pre_Env_Actions
        (Self            : Bare_With_Clause;
         State           : in out PLE_Node_State;
         Add_To_Env_Only : Boolean := False) is
      begin
      declare
         Name_Result : Symbol_Type :=
            Libadalang.Implementation.Initial_Env_Name_16 (Node => 
Self);
      begin
         Set_Initial_Env (Self, State, Name_Result, Bare_With_Clause_Initial_Env_Getter_Fn'Access);
      end;
      end;
      procedure Initialize_Fields_For_With_Clause
        (Self : Bare_With_Clause
         ; With_Clause_F_Has_Limited : Bare_Limited_Node
         ; With_Clause_F_Has_Private : Bare_Private_Node
         ; With_Clause_F_Packages : Bare_Name_List
        ) is
      begin
            Self.With_Clause_F_Has_Limited := With_Clause_F_Has_Limited;
            Self.With_Clause_F_Has_Private := With_Clause_F_Has_Private;
            Self.With_Clause_F_Packages := With_Clause_F_Packages;
      end Initialize_Fields_For_With_Clause;
   function With_Clause_F_Has_Limited
     (Node : Bare_With_Clause) return Bare_Limited_Node
   is
   begin
         return Node.With_Clause_F_Has_Limited;
   end;
   function With_Clause_F_Has_Private
     (Node : Bare_With_Clause) return Bare_Private_Node
   is
   begin
         return Node.With_Clause_F_Has_Private;
   end;
   function With_Clause_F_Packages
     (Node : Bare_With_Clause) return Bare_Name_List
   is
   begin
         return Node.With_Clause_F_Packages;
   end;
--# property-start WithClause.xref_entry_point /usr/src/packages/BUILD/ada/ast.py:6400
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Entry_Point
  (Node : Bare_With_Clause
  )
   return Boolean
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Clause_P_Xref_Entry_Point;
--# end
--# property-start WithClause.xref_equation /usr/src/packages/BUILD/ada/ast.py:6401
pragma Warnings (Off, "is not referenced");
 function With_Clause_P_Xref_Equation
  (Node : Bare_With_Clause
      ; Env : Lexical_Env
      ; Origin : Bare_Ada_Node
   ; E_Info : Internal_Entity_Info :=
      No_Entity_Info
  )
   return Logic_Equation
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   Ent : Internal_Entity_With_Clause :=
     Internal_Entity_With_Clause'(Node => Self, Info => E_Info);
   --# bind entity Ent
   --# bind env Env
   --# bind origin Origin
   Property_Result : Logic_Equation;
            procedure Finalizer_Scope_590;
            procedure Finalizer_Scope_2667;
      Fld : Internal_Entity_Name_List;
Fld_1 : Bare_Name_List;
Fld_2 : Internal_Entity_Info;
As_Entity : Internal_Entity_Name;
Is_Null : Boolean;
If_Result : Internal_Entity_Name;
Fld_3 : Logic_Equation;
Item_271 : Internal_Entity_Name;
Bare_Item_271 : Bare_Name;
Map_Result : Logic_Equation_Array_Access;
Logic_Boolean_Op : Logic_Equation;
            procedure Finalizer_Scope_590 is
            begin
                     Dec_Ref (Map_Result);
                     Dec_Ref (Logic_Boolean_Op);
            end Finalizer_Scope_590;
            procedure Finalizer_Scope_2667 is
            begin
                     Dec_Ref (Fld_3);
            end Finalizer_Scope_2667;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self /= null then
         Reset_Caches (Self.Unit);
      end if;
   --# scope-start
      --# expr-start 8835 '<LogicAll>' Logic_Boolean_Op /usr/src/packages/BUILD/ada/ast.py:6402
--# expr-start 8834 '<map>' Map_Result /usr/src/packages/BUILD/ada/ast.py:6402
--# expr-start 8832 '<FieldAccess .packages>' Fld /usr/src/packages/BUILD/ada/ast.py:6402
      if Ent.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
Fld := Create_Internal_Entity_Name_List (Node => Ent.Node.With_Clause_F_Packages, Info => Ent.Info);
--# expr-done 8832
Fld_1 := Fld.Node;
declare
   Map_Result_Vec : Logic_Equation_Vectors.Vector;
begin
      if Fld_1 = null then
         Map_Result := Create_Logic_Equation_Array (0);
      else
      declare
         Collection : constant Bare_Name_List := Fld_1;
      begin
         for Untyped_Item_271 of
               Collection.Nodes (1 .. Children_Count (Collection))
         loop
                  Bare_Item_271 := Untyped_Item_271; 
Is_Null := Bare_Item_271 = null; 
if Is_Null then
   If_Result := No_Entity_Name;
else
Fld_2 := Fld.Info;
As_Entity := (Info => Fld_2, Node => Bare_Item_271); 
   If_Result := As_Entity;
end if;
                  Item_271 := If_Result; 
   --# scope-start
               --# bind p Item_271
      --# expr-start 8833 '<FieldAccess .xref_no_overloading>' Fld_3 /usr/src/packages/BUILD/ada/ast.py:6402
      if Item_271.Node = null then
         raise Property_Error with "dereferencing a null access";
      end if;
--# property-call-start Name.xref_no_overloading
Fld_3 := Libadalang.Implementation.Name_P_Xref_No_Overloading (Node => Item_271.Node, Env => Env, Origin => Origin, E_Info => Item_271.Info);
--# end
--# expr-done 8833
         declare
            Item_To_Append : constant Logic_Equation :=
               Fld_3;
         begin
               Inc_Ref (Item_To_Append);
            Logic_Equation_Vectors.Append (Map_Result_Vec, Item_To_Append);
         end;
   --# end
      Finalizer_Scope_2667;
         end loop;
      end;
      Map_Result := Create_Logic_Equation_Array
        (Items_Count => Natural (Logic_Equation_Vectors.Length (Map_Result_Vec)));
      for I in Map_Result.Items'Range loop
         Map_Result.Items (I) := Logic_Equation_Vectors.Get
           (Map_Result_Vec,
            I + Logic_Equation_Vectors.Index_Type'First - Map_Result.Items'First);
      end loop;
      Logic_Equation_Vectors.Destroy (Map_Result_Vec);
      end if;
end;
--# expr-done 8834
Logic_Boolean_Op := Logic_All (Relation_Array (Map_Result.Items), (if Langkit_Support.Adalog.Debug.Debug then new String'("ast.py:6402") else null)); 
--# expr-done 8835
      Property_Result := Logic_Boolean_Op;
         Inc_Ref (Property_Result);
   --# end
      Finalizer_Scope_590;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when Property_Error =>
               Finalizer_Scope_590;
               Finalizer_Scope_2667;
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Clause_P_Xref_Equation;
--# end
--# property-start WithClause._initial_env_name_16 /usr/src/packages/BUILD/ada/ast.py:6405
pragma Warnings (Off, "is not referenced");
 function Initial_Env_Name_16
  (Node : Bare_With_Clause
  )
   return Symbol_Type
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Symbol_Type;
      Sym : Symbol_Type;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      --# expr-start 8836 '<Symbol Standard>' Sym None
Sym := Precomputed_Symbol (Precomputed_Symbol_Table (Self.Unit.Context.Symbols), Precomputed_Symbol_Standard); 
--# expr-done 8836
      Property_Result := Sym;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_Name_16;
--# end
--# property-start WithClause._initial_env_17 /usr/src/packages/BUILD/ada/ast.py:6408
pragma Warnings (Off, "is not referenced");
 function Initial_Env_17
  (Node : Bare_With_Clause
  )
   return Lexical_Env
is
   Self : Bare_With_Clause := Bare_With_Clause (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Lexical_Env;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := Empty_Env;
         Inc_Ref (Property_Result);
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Initial_Env_17;
--# end
   --
   --  Primitives for Bare_With_Private
   --
--# property-start '[dispatcher]WithPrivate.as_bool' dispatcher
pragma Warnings (Off, "is not referenced");
 function Dispatcher_With_Private_P_As_Bool
  (Node : Bare_With_Private
  )
   return Boolean
is
   Self : Bare_With_Private := Bare_With_Private (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
      if Self = null then
         raise Property_Error with "dispatching on null node";
      end if;
      case Ada_With_Private (Self.Kind) is
               when Ada_With_Private_Absent =>
                  --# property-call-start WithPrivate.Absent.as_bool
                  Property_Result := With_Private_Absent_P_As_Bool
                    (Self
                    );
                  --# end
               when Ada_With_Private_Present =>
                  --# property-call-start WithPrivate.Present.as_bool
                  Property_Result := With_Private_Present_P_As_Bool
                    (Self
                    );
                  --# end
      end case;
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end Dispatcher_With_Private_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Absent
   --
--# property-start WithPrivate.Absent.as_bool /usr/src/packages/BUILD/ada/ast.py:3935
pragma Warnings (Off, "is not referenced");
 function With_Private_Absent_P_As_Bool
  (Node : Bare_With_Private_Absent
  )
   return Boolean
is
   Self : Bare_With_Private_Absent := Bare_With_Private_Absent (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := False;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Private_Absent_P_As_Bool;
--# end
   --
   --  Primitives for Bare_With_Private_Present
   --
--# property-start WithPrivate.Present.as_bool /usr/src/packages/BUILD/ada/ast.py:3935
pragma Warnings (Off, "is not referenced");
 function With_Private_Present_P_As_Bool
  (Node : Bare_With_Private_Present
  )
   return Boolean
is
   Self : Bare_With_Private_Present := Bare_With_Private_Present (Node);
   Call_Depth : aliased Natural;
   --# bind self Self
   Property_Result : Boolean;
begin
   --# property-body-start
   if Self /= null then
      Enter_Call (Self.Unit.Context, Call_Depth'Access);
   end if;
   --# scope-start
      Property_Result := True;
   --# end
   if Self /= null then
      Exit_Call (Self.Unit.Context, Call_Depth);
   end if;
   return Property_Result;
exception
   when others =>
      if Self /= null then
         Exit_Call (Self.Unit.Context, Call_Depth);
      end if;
      raise;
end With_Private_Present_P_As_Bool;
--# end
   ----------------------------
   -- Destroy_Synthetic_Node --
   ----------------------------
   procedure Destroy_Synthetic_Node (Node : in out Bare_Ada_Node) is
      procedure Free is new Ada.Unchecked_Deallocation
        (Root_Node_Record, Bare_Ada_Node);
   begin
      --  Don't call Node.Destroy, as Node's children may be gone already: they
      --  have their own destructor and there is no specified order for the
      --  call of these destructors.
      Free_User_Fields (Node);
      Free (Node);
   end Destroy_Synthetic_Node;
   -----------
   -- Image --
   -----------
   function Image (Value : Boolean) return String
   is (if Value then "True" else "False");
      -----------------
      -- Trace_Image --
      -----------------
      function Trace_Image
        (Node       : Bare_Ada_Node;
         Decoration : Boolean := True) return String is
      begin
         if Node = null then
            return "None";
         else
            declare
               Result : constant String :=
                 (Kind_Name (Node) & " "
                  & Basename (Node.Unit) & ":"
                  & Image (Sloc_Range (Node)));
            begin
               return (if Decoration then "<" & Result & ">" else Result);
            end;
         end if;
      end Trace_Image;
   Kind_Names : array (Ada_Node_Kind_Type) of Unbounded_String :=
     (Ada_Abort_Absent => To_Unbounded_String ("AbortAbsent"), 
Ada_Abort_Present => To_Unbounded_String ("AbortPresent"), 
Ada_Abstract_Absent => To_Unbounded_String ("AbstractAbsent"), 
Ada_Abstract_Present => To_Unbounded_String ("AbstractPresent"), 
Ada_Ada_Node_List => To_Unbounded_String ("AdaNodeList"), 
Ada_Abstract_State_Decl_List => To_Unbounded_String ("AbstractStateDeclList"), 
Ada_Alternatives_List => To_Unbounded_String ("AlternativesList"), 
Ada_Constraint_List => To_Unbounded_String ("ConstraintList"), 
Ada_Decl_List => To_Unbounded_String ("DeclList"), 
Ada_Stmt_List => To_Unbounded_String ("StmtList"), 
Ada_Aspect_Assoc_List => To_Unbounded_String ("AspectAssocList"), 
Ada_Base_Assoc_List => To_Unbounded_String ("BaseAssocList"), 
Ada_Assoc_List => To_Unbounded_String ("AssocList"), 
Ada_Basic_Decl_List => To_Unbounded_String ("BasicDeclList"), 
Ada_Case_Expr_Alternative_List => To_Unbounded_String ("CaseExprAlternativeList"), 
Ada_Case_Stmt_Alternative_List => To_Unbounded_String ("CaseStmtAlternativeList"), 
Ada_Compilation_Unit_List => To_Unbounded_String ("CompilationUnitList"), 
Ada_Contract_Case_Assoc_List => To_Unbounded_String ("ContractCaseAssocList"), 
Ada_Defining_Name_List => To_Unbounded_String ("DefiningNameList"), 
Ada_Discriminant_Spec_List => To_Unbounded_String ("DiscriminantSpecList"), 
Ada_Elsif_Expr_Part_List => To_Unbounded_String ("ElsifExprPartList"), 
Ada_Elsif_Stmt_Part_List => To_Unbounded_String ("ElsifStmtPartList"), 
Ada_Enum_Literal_Decl_List => To_Unbounded_String ("EnumLiteralDeclList"), 
Ada_Expr_Alternatives_List => To_Unbounded_String ("ExprAlternativesList"), 
Ada_Discriminant_Choice_List => To_Unbounded_String ("DiscriminantChoiceList"), 
Ada_Name_List => To_Unbounded_String ("NameList"), 
Ada_Parent_List => To_Unbounded_String ("ParentList"), 
Ada_Param_Spec_List => To_Unbounded_String ("ParamSpecList"), 
Ada_Pragma_Node_List => To_Unbounded_String ("PragmaNodeList"), 
Ada_Select_When_Part_List => To_Unbounded_String ("SelectWhenPartList"), 
Ada_Unconstrained_Array_Index_List => To_Unbounded_String ("UnconstrainedArrayIndexList"), 
Ada_Variant_List => To_Unbounded_String ("VariantList"), 
Ada_Aliased_Absent => To_Unbounded_String ("AliasedAbsent"), 
Ada_Aliased_Present => To_Unbounded_String ("AliasedPresent"), 
Ada_All_Absent => To_Unbounded_String ("AllAbsent"), 
Ada_All_Present => To_Unbounded_String ("AllPresent"), 
Ada_Constrained_Array_Indices => To_Unbounded_String ("ConstrainedArrayIndices"), 
Ada_Unconstrained_Array_Indices => To_Unbounded_String ("UnconstrainedArrayIndices"), 
Ada_Aspect_Assoc => To_Unbounded_String ("AspectAssoc"), 
Ada_At_Clause => To_Unbounded_String ("AtClause"), 
Ada_Attribute_Def_Clause => To_Unbounded_String ("AttributeDefClause"), 
Ada_Enum_Rep_Clause => To_Unbounded_String ("EnumRepClause"), 
Ada_Record_Rep_Clause => To_Unbounded_String ("RecordRepClause"), 
Ada_Aspect_Spec => To_Unbounded_String ("AspectSpec"), 
Ada_Contract_Case_Assoc => To_Unbounded_String ("ContractCaseAssoc"), 
Ada_Pragma_Argument_Assoc => To_Unbounded_String ("PragmaArgumentAssoc"), 
Ada_Entry_Spec => To_Unbounded_String ("EntrySpec"), 
Ada_Enum_Subp_Spec => To_Unbounded_String ("EnumSubpSpec"), 
Ada_Subp_Spec => To_Unbounded_String ("SubpSpec"), 
Ada_Component_List => To_Unbounded_String ("ComponentList"), 
Ada_Known_Discriminant_Part => To_Unbounded_String ("KnownDiscriminantPart"), 
Ada_Unknown_Discriminant_Part => To_Unbounded_String ("UnknownDiscriminantPart"), 
Ada_Entry_Completion_Formal_Params => To_Unbounded_String ("EntryCompletionFormalParams"), 
Ada_Generic_Formal_Part => To_Unbounded_String ("GenericFormalPart"), 
Ada_Null_Record_Def => To_Unbounded_String ("NullRecordDef"), 
Ada_Record_Def => To_Unbounded_String ("RecordDef"), 
Ada_Aggregate_Assoc => To_Unbounded_String ("AggregateAssoc"), 
Ada_Multi_Dim_Array_Assoc => To_Unbounded_String ("MultiDimArrayAssoc"), 
Ada_Discriminant_Assoc => To_Unbounded_String ("DiscriminantAssoc"), 
Ada_Iterated_Assoc => To_Unbounded_String ("IteratedAssoc"), 
Ada_Param_Assoc => To_Unbounded_String ("ParamAssoc"), 
Ada_Abstract_State_Decl => To_Unbounded_String ("AbstractStateDecl"), 
Ada_Anonymous_Expr_Decl => To_Unbounded_String ("AnonymousExprDecl"), 
Ada_Component_Decl => To_Unbounded_String ("ComponentDecl"), 
Ada_Discriminant_Spec => To_Unbounded_String ("DiscriminantSpec"), 
Ada_Generic_Formal_Obj_Decl => To_Unbounded_String ("GenericFormalObjDecl"), 
Ada_Generic_Formal_Package => To_Unbounded_String ("GenericFormalPackage"), 
Ada_Generic_Formal_Subp_Decl => To_Unbounded_String ("GenericFormalSubpDecl"), 
Ada_Generic_Formal_Type_Decl => To_Unbounded_String ("GenericFormalTypeDecl"), 
Ada_Param_Spec => To_Unbounded_String ("ParamSpec"), 
Ada_Generic_Package_Internal => To_Unbounded_String ("GenericPackageInternal"), 
Ada_Package_Decl => To_Unbounded_String ("PackageDecl"), 
Ada_Discrete_Base_Subtype_Decl => To_Unbounded_String ("DiscreteBaseSubtypeDecl"), 
Ada_Subtype_Decl => To_Unbounded_String ("SubtypeDecl"), 
Ada_Classwide_Type_Decl => To_Unbounded_String ("ClasswideTypeDecl"), 
Ada_Incomplete_Type_Decl => To_Unbounded_String ("IncompleteTypeDecl"), 
Ada_Incomplete_Tagged_Type_Decl => To_Unbounded_String ("IncompleteTaggedTypeDecl"), 
Ada_Protected_Type_Decl => To_Unbounded_String ("ProtectedTypeDecl"), 
Ada_Task_Type_Decl => To_Unbounded_String ("TaskTypeDecl"), 
Ada_Single_Task_Type_Decl => To_Unbounded_String ("SingleTaskTypeDecl"), 
Ada_Type_Decl => To_Unbounded_String ("TypeDecl"), 
Ada_Anonymous_Type_Decl => To_Unbounded_String ("AnonymousTypeDecl"), 
Ada_Synth_Anonymous_Type_Decl => To_Unbounded_String ("SynthAnonymousTypeDecl"), 
Ada_Abstract_Subp_Decl => To_Unbounded_String ("AbstractSubpDecl"), 
Ada_Abstract_Formal_Subp_Decl => To_Unbounded_String ("AbstractFormalSubpDecl"), 
Ada_Concrete_Formal_Subp_Decl => To_Unbounded_String ("ConcreteFormalSubpDecl"), 
Ada_Subp_Decl => To_Unbounded_String ("SubpDecl"), 
Ada_Entry_Decl => To_Unbounded_String ("EntryDecl"), 
Ada_Enum_Literal_Decl => To_Unbounded_String ("EnumLiteralDecl"), 
Ada_Generic_Subp_Internal => To_Unbounded_String ("GenericSubpInternal"), 
Ada_Expr_Function => To_Unbounded_String ("ExprFunction"), 
Ada_Null_Subp_Decl => To_Unbounded_String ("NullSubpDecl"), 
Ada_Subp_Body => To_Unbounded_String ("SubpBody"), 
Ada_Subp_Renaming_Decl => To_Unbounded_String ("SubpRenamingDecl"), 
Ada_Package_Body_Stub => To_Unbounded_String ("PackageBodyStub"), 
Ada_Protected_Body_Stub => To_Unbounded_String ("ProtectedBodyStub"), 
Ada_Subp_Body_Stub => To_Unbounded_String ("SubpBodyStub"), 
Ada_Task_Body_Stub => To_Unbounded_String ("TaskBodyStub"), 
Ada_Entry_Body => To_Unbounded_String ("EntryBody"), 
Ada_Package_Body => To_Unbounded_String ("PackageBody"), 
Ada_Protected_Body => To_Unbounded_String ("ProtectedBody"), 
Ada_Task_Body => To_Unbounded_String ("TaskBody"), 
Ada_Entry_Index_Spec => To_Unbounded_String ("EntryIndexSpec"), 
Ada_Error_Decl => To_Unbounded_String ("ErrorDecl"), 
Ada_Exception_Decl => To_Unbounded_String ("ExceptionDecl"), 
Ada_Exception_Handler => To_Unbounded_String ("ExceptionHandler"), 
Ada_For_Loop_Var_Decl => To_Unbounded_String ("ForLoopVarDecl"), 
Ada_Generic_Package_Decl => To_Unbounded_String ("GenericPackageDecl"), 
Ada_Generic_Subp_Decl => To_Unbounded_String ("GenericSubpDecl"), 
Ada_Generic_Package_Instantiation => To_Unbounded_String ("GenericPackageInstantiation"), 
Ada_Generic_Subp_Instantiation => To_Unbounded_String ("GenericSubpInstantiation"), 
Ada_Generic_Package_Renaming_Decl => To_Unbounded_String ("GenericPackageRenamingDecl"), 
Ada_Generic_Subp_Renaming_Decl => To_Unbounded_String ("GenericSubpRenamingDecl"), 
Ada_Label_Decl => To_Unbounded_String ("LabelDecl"), 
Ada_Named_Stmt_Decl => To_Unbounded_String ("NamedStmtDecl"), 
Ada_Number_Decl => To_Unbounded_String ("NumberDecl"), 
Ada_Object_Decl => To_Unbounded_String ("ObjectDecl"), 
Ada_Extended_Return_Stmt_Object_Decl => To_Unbounded_String ("ExtendedReturnStmtObjectDecl"), 
Ada_Package_Renaming_Decl => To_Unbounded_String ("PackageRenamingDecl"), 
Ada_Single_Protected_Decl => To_Unbounded_String ("SingleProtectedDecl"), 
Ada_Single_Task_Decl => To_Unbounded_String ("SingleTaskDecl"), 
Ada_Case_Stmt_Alternative => To_Unbounded_String ("CaseStmtAlternative"), 
Ada_Compilation_Unit => To_Unbounded_String ("CompilationUnit"), 
Ada_Component_Clause => To_Unbounded_String ("ComponentClause"), 
Ada_Component_Def => To_Unbounded_String ("ComponentDef"), 
Ada_Constant_Absent => To_Unbounded_String ("ConstantAbsent"), 
Ada_Constant_Present => To_Unbounded_String ("ConstantPresent"), 
Ada_Delta_Constraint => To_Unbounded_String ("DeltaConstraint"), 
Ada_Digits_Constraint => To_Unbounded_String ("DigitsConstraint"), 
Ada_Discriminant_Constraint => To_Unbounded_String ("DiscriminantConstraint"), 
Ada_Index_Constraint => To_Unbounded_String ("IndexConstraint"), 
Ada_Range_Constraint => To_Unbounded_String ("RangeConstraint"), 
Ada_Declarative_Part => To_Unbounded_String ("DeclarativePart"), 
Ada_Private_Part => To_Unbounded_String ("PrivatePart"), 
Ada_Public_Part => To_Unbounded_String ("PublicPart"), 
Ada_Elsif_Expr_Part => To_Unbounded_String ("ElsifExprPart"), 
Ada_Elsif_Stmt_Part => To_Unbounded_String ("ElsifStmtPart"), 
Ada_Abstract_State_Decl_Expr => To_Unbounded_String ("AbstractStateDeclExpr"), 
Ada_Allocator => To_Unbounded_String ("Allocator"), 
Ada_Aggregate => To_Unbounded_String ("Aggregate"), 
Ada_Bracket_Aggregate => To_Unbounded_String ("BracketAggregate"), 
Ada_Delta_Aggregate => To_Unbounded_String ("DeltaAggregate"), 
Ada_Bracket_Delta_Aggregate => To_Unbounded_String ("BracketDeltaAggregate"), 
Ada_Null_Record_Aggregate => To_Unbounded_String ("NullRecordAggregate"), 
Ada_Bin_Op => To_Unbounded_String ("BinOp"), 
Ada_Relation_Op => To_Unbounded_String ("RelationOp"), 
Ada_Box_Expr => To_Unbounded_String ("BoxExpr"), 
Ada_Case_Expr_Alternative => To_Unbounded_String ("CaseExprAlternative"), 
Ada_Case_Expr => To_Unbounded_String ("CaseExpr"), 
Ada_If_Expr => To_Unbounded_String ("IfExpr"), 
Ada_Contract_Cases => To_Unbounded_String ("ContractCases"), 
Ada_Decl_Expr => To_Unbounded_String ("DeclExpr"), 
Ada_Membership_Expr => To_Unbounded_String ("MembershipExpr"), 
Ada_Attribute_Ref => To_Unbounded_String ("AttributeRef"), 
Ada_Update_Attribute_Ref => To_Unbounded_String ("UpdateAttributeRef"), 
Ada_Call_Expr => To_Unbounded_String ("CallExpr"), 
Ada_Defining_Name => To_Unbounded_String ("DefiningName"), 
Ada_Discrete_Subtype_Name => To_Unbounded_String ("DiscreteSubtypeName"), 
Ada_Dotted_Name => To_Unbounded_String ("DottedName"), 
Ada_End_Name => To_Unbounded_String ("EndName"), 
Ada_Explicit_Deref => To_Unbounded_String ("ExplicitDeref"), 
Ada_Qual_Expr => To_Unbounded_String ("QualExpr"), 
Ada_Char_Literal => To_Unbounded_String ("Chr"), 
Ada_Identifier => To_Unbounded_String ("Id"), 
Ada_Op_Abs => To_Unbounded_String ("OpAbs"), 
Ada_Op_And => To_Unbounded_String ("OpAnd"), 
Ada_Op_And_Then => To_Unbounded_String ("OpAndThen"), 
Ada_Op_Concat => To_Unbounded_String ("OpConcat"), 
Ada_Op_Div => To_Unbounded_String ("OpDiv"), 
Ada_Op_Double_Dot => To_Unbounded_String ("OpDoubleDot"), 
Ada_Op_Eq => To_Unbounded_String ("OpEq"), 
Ada_Op_Gt => To_Unbounded_String ("OpGt"), 
Ada_Op_Gte => To_Unbounded_String ("OpGte"), 
Ada_Op_In => To_Unbounded_String ("OpIn"), 
Ada_Op_Lt => To_Unbounded_String ("OpLt"), 
Ada_Op_Lte => To_Unbounded_String ("OpLte"), 
Ada_Op_Minus => To_Unbounded_String ("OpMinus"), 
Ada_Op_Mod => To_Unbounded_String ("OpMod"), 
Ada_Op_Mult => To_Unbounded_String ("OpMult"), 
Ada_Op_Neq => To_Unbounded_String ("OpNeq"), 
Ada_Op_Not => To_Unbounded_String ("OpNot"), 
Ada_Op_Not_In => To_Unbounded_String ("OpNotIn"), 
Ada_Op_Or => To_Unbounded_String ("OpOr"), 
Ada_Op_Or_Else => To_Unbounded_String ("OpOrElse"), 
Ada_Op_Plus => To_Unbounded_String ("OpPlus"), 
Ada_Op_Pow => To_Unbounded_String ("OpPow"), 
Ada_Op_Rem => To_Unbounded_String ("OpRem"), 
Ada_Op_Xor => To_Unbounded_String ("OpXor"), 
Ada_String_Literal => To_Unbounded_String ("Str"), 
Ada_Null_Literal => To_Unbounded_String ("Null"), 
Ada_Int_Literal => To_Unbounded_String ("Int"), 
Ada_Real_Literal => To_Unbounded_String ("Real"), 
Ada_Target_Name => To_Unbounded_String ("TargetName"), 
Ada_Paren_Expr => To_Unbounded_String ("ParenExpr"), 
Ada_Quantified_Expr => To_Unbounded_String ("QuantifiedExpr"), 
Ada_Raise_Expr => To_Unbounded_String ("RaiseExpr"), 
Ada_Un_Op => To_Unbounded_String ("UnOp"), 
Ada_Handled_Stmts => To_Unbounded_String ("HandledStmts"), 
Ada_Interface_Kind_Limited => To_Unbounded_String ("InterfaceKindLimited"), 
Ada_Interface_Kind_Protected => To_Unbounded_String ("InterfaceKindProtected"), 
Ada_Interface_Kind_Synchronized => To_Unbounded_String ("InterfaceKindSynchronized"), 
Ada_Interface_Kind_Task => To_Unbounded_String ("InterfaceKindTask"), 
Ada_Iter_Type_In => To_Unbounded_String ("IterTypeIn"), 
Ada_Iter_Type_Of => To_Unbounded_String ("IterTypeOf"), 
Ada_Library_Item => To_Unbounded_String ("LibraryItem"), 
Ada_Limited_Absent => To_Unbounded_String ("LimitedAbsent"), 
Ada_Limited_Present => To_Unbounded_String ("LimitedPresent"), 
Ada_For_Loop_Spec => To_Unbounded_String ("ForLoopSpec"), 
Ada_While_Loop_Spec => To_Unbounded_String ("WhileLoopSpec"), 
Ada_Mode_Default => To_Unbounded_String ("ModeDefault"), 
Ada_Mode_In => To_Unbounded_String ("ModeIn"), 
Ada_Mode_In_Out => To_Unbounded_String ("ModeInOut"), 
Ada_Mode_Out => To_Unbounded_String ("ModeOut"), 
Ada_Multi_Abstract_State_Decl => To_Unbounded_String ("MultiAbstractStateDecl"), 
Ada_Not_Null_Absent => To_Unbounded_String ("NotNullAbsent"), 
Ada_Not_Null_Present => To_Unbounded_String ("NotNullPresent"), 
Ada_Null_Component_Decl => To_Unbounded_String ("NullComponentDecl"), 
Ada_Others_Designator => To_Unbounded_String ("OthersDesignator"), 
Ada_Overriding_Not_Overriding => To_Unbounded_String ("OverridingNotOverriding"), 
Ada_Overriding_Overriding => To_Unbounded_String ("OverridingOverriding"), 
Ada_Overriding_Unspecified => To_Unbounded_String ("OverridingUnspecified"), 
Ada_Params => To_Unbounded_String ("Params"), 
Ada_Paren_Abstract_State_Decl => To_Unbounded_String ("ParenAbstractStateDecl"), 
Ada_Pragma_Node => To_Unbounded_String ("PragmaNode"), 
Ada_Private_Absent => To_Unbounded_String ("PrivateAbsent"), 
Ada_Private_Present => To_Unbounded_String ("PrivatePresent"), 
Ada_Protected_Def => To_Unbounded_String ("ProtectedDef"), 
Ada_Protected_Absent => To_Unbounded_String ("ProtectedAbsent"), 
Ada_Protected_Present => To_Unbounded_String ("ProtectedPresent"), 
Ada_Quantifier_All => To_Unbounded_String ("QuantifierAll"), 
Ada_Quantifier_Some => To_Unbounded_String ("QuantifierSome"), 
Ada_Range_Spec => To_Unbounded_String ("RangeSpec"), 
Ada_Renaming_Clause => To_Unbounded_String ("RenamingClause"), 
Ada_Synthetic_Renaming_Clause => To_Unbounded_String ("SyntheticRenamingClause"), 
Ada_Reverse_Absent => To_Unbounded_String ("ReverseAbsent"), 
Ada_Reverse_Present => To_Unbounded_String ("ReversePresent"), 
Ada_Select_When_Part => To_Unbounded_String ("SelectWhenPart"), 
Ada_Accept_Stmt => To_Unbounded_String ("AcceptStmt"), 
Ada_Accept_Stmt_With_Stmts => To_Unbounded_String ("AcceptStmtWithStmts"), 
Ada_For_Loop_Stmt => To_Unbounded_String ("ForLoopStmt"), 
Ada_Loop_Stmt => To_Unbounded_String ("LoopStmt"), 
Ada_While_Loop_Stmt => To_Unbounded_String ("WhileLoopStmt"), 
Ada_Begin_Block => To_Unbounded_String ("BeginBlock"), 
Ada_Decl_Block => To_Unbounded_String ("DeclBlock"), 
Ada_Case_Stmt => To_Unbounded_String ("CaseStmt"), 
Ada_Extended_Return_Stmt => To_Unbounded_String ("ExtendedReturnStmt"), 
Ada_If_Stmt => To_Unbounded_String ("IfStmt"), 
Ada_Named_Stmt => To_Unbounded_String ("NamedStmt"), 
Ada_Select_Stmt => To_Unbounded_String ("SelectStmt"), 
Ada_Error_Stmt => To_Unbounded_String ("ErrorStmt"), 
Ada_Abort_Stmt => To_Unbounded_String ("AbortStmt"), 
Ada_Assign_Stmt => To_Unbounded_String ("AssignStmt"), 
Ada_Call_Stmt => To_Unbounded_String ("CallStmt"), 
Ada_Delay_Stmt => To_Unbounded_String ("DelayStmt"), 
Ada_Exit_Stmt => To_Unbounded_String ("ExitStmt"), 
Ada_Goto_Stmt => To_Unbounded_String ("GotoStmt"), 
Ada_Label => To_Unbounded_String ("Label"), 
Ada_Null_Stmt => To_Unbounded_String ("NullStmt"), 
Ada_Raise_Stmt => To_Unbounded_String ("RaiseStmt"), 
Ada_Requeue_Stmt => To_Unbounded_String ("RequeueStmt"), 
Ada_Return_Stmt => To_Unbounded_String ("ReturnStmt"), 
Ada_Terminate_Alternative => To_Unbounded_String ("TerminateAlternative"), 
Ada_Subp_Kind_Function => To_Unbounded_String ("SubpKindFunction"), 
Ada_Subp_Kind_Procedure => To_Unbounded_String ("SubpKindProcedure"), 
Ada_Subunit => To_Unbounded_String ("Subunit"), 
Ada_Synchronized_Absent => To_Unbounded_String ("SynchronizedAbsent"), 
Ada_Synchronized_Present => To_Unbounded_String ("SynchronizedPresent"), 
Ada_Tagged_Absent => To_Unbounded_String ("TaggedAbsent"), 
Ada_Tagged_Present => To_Unbounded_String ("TaggedPresent"), 
Ada_Task_Def => To_Unbounded_String ("TaskDef"), 
Ada_Access_To_Subp_Def => To_Unbounded_String ("AccessToSubpDef"), 
Ada_Anonymous_Type_Access_Def => To_Unbounded_String ("AnonymousTypeAccessDef"), 
Ada_Type_Access_Def => To_Unbounded_String ("TypeAccessDef"), 
Ada_Array_Type_Def => To_Unbounded_String ("ArrayTypeDef"), 
Ada_Derived_Type_Def => To_Unbounded_String ("DerivedTypeDef"), 
Ada_Enum_Type_Def => To_Unbounded_String ("EnumTypeDef"), 
Ada_Formal_Discrete_Type_Def => To_Unbounded_String ("FormalDiscreteTypeDef"), 
Ada_Interface_Type_Def => To_Unbounded_String ("InterfaceTypeDef"), 
Ada_Mod_Int_Type_Def => To_Unbounded_String ("ModIntTypeDef"), 
Ada_Private_Type_Def => To_Unbounded_String ("PrivateTypeDef"), 
Ada_Decimal_Fixed_Point_Def => To_Unbounded_String ("DecimalFixedPointDef"), 
Ada_Floating_Point_Def => To_Unbounded_String ("FloatingPointDef"), 
Ada_Ordinary_Fixed_Point_Def => To_Unbounded_String ("OrdinaryFixedPointDef"), 
Ada_Record_Type_Def => To_Unbounded_String ("RecordTypeDef"), 
Ada_Signed_Int_Type_Def => To_Unbounded_String ("SignedIntTypeDef"), 
Ada_Anonymous_Type => To_Unbounded_String ("AnonymousType"), 
Ada_Enum_Lit_Synth_Type_Expr => To_Unbounded_String ("EnumLitSynthTypeExpr"), 
Ada_Subtype_Indication => To_Unbounded_String ("SubtypeIndication"), 
Ada_Constrained_Subtype_Indication => To_Unbounded_String ("ConstrainedSubtypeIndication"), 
Ada_Discrete_Subtype_Indication => To_Unbounded_String ("DiscreteSubtypeIndication"), 
Ada_Unconstrained_Array_Index => To_Unbounded_String ("UnconstrainedArrayIndex"), 
Ada_Until_Absent => To_Unbounded_String ("UntilAbsent"), 
Ada_Until_Present => To_Unbounded_String ("UntilPresent"), 
Ada_Use_Package_Clause => To_Unbounded_String ("UsePackageClause"), 
Ada_Use_Type_Clause => To_Unbounded_String ("UseTypeClause"), 
Ada_Variant => To_Unbounded_String ("Variant"), 
Ada_Variant_Part => To_Unbounded_String ("VariantPart"), 
Ada_With_Clause => To_Unbounded_String ("WithClause"), 
Ada_With_Private_Absent => To_Unbounded_String ("WithPrivateAbsent"), 
Ada_With_Private_Present => To_Unbounded_String ("WithPrivatePresent"));
   ---------------
   -- Kind_Name --
   ---------------
   function Kind_Name (Node : Bare_Ada_Node) return String is
   begin
      return To_String (Kind_Names (Node.Kind));
   end Kind_Name;
   --------------------
   -- Children_Count --
   --------------------
   function Children_Count (Node : Bare_Ada_Node) return Natural is
      C : Integer := Kind_To_Node_Children_Count (Node.Kind);
   begin
      if C = -1 then
         return Node.Count;
      else
         return C;
      end if;
   end Children_Count;
   ----------------------
   -- Free_User_Fields --
   ----------------------
   procedure Free_User_Fields (Node : Bare_Ada_Node) is
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record);
      --  Reset the LV logic variable, clearing the value it stores
      ---------------------
      -- Reset_Logic_Var --
      ---------------------
      procedure Reset_Logic_Var (LV : in out Logic_Var_Record) is
      begin
         --  TODO??? Fix Adalog so that Destroy resets the value it stores
         LV.Value := No_Entity;
         Eq_Node.Refs.Reset (LV);
         Eq_Node.Refs.Destroy (LV);
      end Reset_Logic_Var;
      K : constant Ada_Node_Kind_Type := Node.Kind;
   begin
      case Ada_Ada_Node (K) is
when Ada_Base_Type_Decl =>
declare
N_Bare_Base_Type_Decl : constant Bare_Base_Type_Decl := Node;
begin
Dec_Ref (N_Bare_Base_Type_Decl.LF_Stg_Dottable_Subps_Env);
case Ada_Base_Type_Decl (K) is
when Ada_Type_Decl_Range =>
declare
N_Bare_Type_Decl : constant Bare_Type_Decl := N_Bare_Base_Type_Decl;
begin
Dec_Ref (N_Bare_Type_Decl.LF_Stg_Direct_Primitives_Env);
end;
when others => null;
end case;
end;
when Ada_Generic_Instantiation =>
declare
N_Bare_Generic_Instantiation : constant Bare_Generic_Instantiation := Node;
begin
Dec_Ref (N_Bare_Generic_Instantiation.Generic_Instantiation_Inst_Env);
Dec_Ref (N_Bare_Generic_Instantiation.LF_Stg_Instantiation_Env);
Dec_Ref (N_Bare_Generic_Instantiation.LF_Stg_Actual_Expr_Decls);
end;
when Ada_Compilation_Unit_Range =>
declare
N_Bare_Compilation_Unit : constant Bare_Compilation_Unit := Node;
begin
Dec_Ref (N_Bare_Compilation_Unit.Compilation_Unit_No_Env);
end;
when Ada_Attribute_Ref_Range =>
declare
N_Bare_Attribute_Ref : constant Bare_Attribute_Ref := Node;
begin
Reset_Logic_Var (N_Bare_Attribute_Ref.Attribute_Ref_R_Ref_Var);
end;
when Ada_Call_Expr_Range =>
declare
N_Bare_Call_Expr : constant Bare_Call_Expr := Node;
begin
Reset_Logic_Var (N_Bare_Call_Expr.Call_Expr_R_Called_Spec);
end;
when Ada_Explicit_Deref_Range =>
declare
N_Bare_Explicit_Deref : constant Bare_Explicit_Deref := Node;
begin
Reset_Logic_Var (N_Bare_Explicit_Deref.Explicit_Deref_R_Called_Spec);
end;
when Ada_Target_Name_Range =>
declare
N_Bare_Target_Name : constant Bare_Target_Name := Node;
begin
Reset_Logic_Var (N_Bare_Target_Name.Target_Name_R_Ref_Var);
end;
when others => null;
end case;
   end Free_User_Fields;
   ----------------
   -- Token_Data --
   ----------------
   function Token_Data (Unit : Internal_Unit) return Token_Data_Handler_Access
   is (Unit.TDH'Access);
   -------------------
   -- Lookup_Symbol --
   -------------------
   function Lookup_Symbol
     (Context : Internal_Context; Symbol : Text_Type) return Symbol_Type
   is
      Canon_Symbol : constant Symbolization_Result :=
            Libadalang.Sources.Canonicalize (Symbol)
      ;
   begin
      if Canon_Symbol.Success then
         return Get_Symbol
           (Context.Symbols, Find (Context.Symbols, Canon_Symbol.Symbol));
      else
         raise Invalid_Symbol_Error with Image (Canon_Symbol.Error_Message);
      end if;
   end Lookup_Symbol;
   -------------------------
   -- Create_Special_Unit --
   -------------------------
   function Create_Special_Unit
     (Context             : Internal_Context;
      Normalized_Filename : Virtual_File;
      Charset             : String;
      Rule                : Grammar_Rule) return Internal_Unit
   is
      Unit : Internal_Unit := new Analysis_Unit_Type'
        (Context                      => Context,
         Is_Internal                  => False,
         AST_Root                     => null,
         Filename                     => Normalized_Filename,
         Charset                      => To_Unbounded_String (Charset),
         TDH                          => <>,
         Diagnostics                  => <>,
         Is_Env_Populated             => False,
         Rule                         => Rule,
         AST_Mem_Pool                 => No_Pool,
         Destroyables                 => Destroyable_Vectors.Empty_Vector,
         Referenced_Units             => <>,
         Exiled_Entries               => Exiled_Entry_Vectors.Empty_Vector,
         Foreign_Nodes                =>
            Foreign_Node_Entry_Vectors.Empty_Vector,
         Exiled_Entries_In_NED        =>
            Exiled_Entry_In_NED_Vectors.Empty_Vector,
         Exiled_Envs                  => Exiled_Env_Vectors.Empty_Vector,
         Named_Envs                   => Named_Env_Vectors.Empty_Vector,
         Nodes_With_Foreign_Env       => <>,
         Rebindings                   => Env_Rebindings_Vectors.Empty_Vector,
         Cache_Version                => <>,
         Unit_Version                 => <>,
         Memoization_Map            => <>,
         others => <>
      );
   begin
      Initialize (Unit.TDH, Context.Symbols,
                  Context.Tab_Stop);
      return Unit;
   end Create_Special_Unit;
   --------------------
   -- Templates_Unit --
   --------------------
   function Templates_Unit (Context : Internal_Context) return Internal_Unit is
   begin
      if Context.Templates_Unit = No_Analysis_Unit then
         Context.Templates_Unit := Create_Special_Unit
           (Context             => Context,
            Normalized_Filename => No_File,
            Charset             => Default_Charset,
            Rule                => Compilation_Rule);
      end if;
      return Context.Templates_Unit;
   end Templates_Unit;
   --------------
   -- Set_Rule --
   --------------
   procedure Set_Rule (Unit : Internal_Unit; Rule : Grammar_Rule) is
   begin
      Unit.Rule := Rule;
   end Set_Rule;
   ------------------------------
   -- Normalized_Unit_Filename --
   ------------------------------
   function Normalized_Unit_Filename
     (Context : Internal_Context; Filename : String) return Virtual_File
   is
      use Virtual_File_Maps;
      Key : constant Unbounded_String := To_Unbounded_String (Filename);
      Cur : Cursor := Context.Filenames.Find (Key);
   begin
      if Cur = No_Element then
         declare
            F : constant Virtual_File := Create
              (Create_From_Base (+Filename).Full_Name,
               Normalize => True);
         begin
            Context.Filenames.Insert (Key, F);
            return F;
         end;
      else
         return Element (Cur);
      end if;
   end Normalized_Unit_Filename;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable_Helper
     (Unit    : Internal_Unit;
      Object  : System.Address;
      Destroy : Destroy_Procedure)
   is
   begin
      Destroyable_Vectors.Append (Unit.Destroyables, (Object, Destroy));
   end Register_Destroyable_Helper;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Node : Bare_Ada_Node)
   is
      procedure Helper is new Register_Destroyable_Gen
        (Root_Node_Record,
         Bare_Ada_Node,
         Destroy_Synthetic_Node);
   begin
      Helper (Unit, Node);
   end Register_Destroyable;
   --------------------------
   -- Register_Destroyable --
   --------------------------
   procedure Register_Destroyable
     (Unit : Internal_Unit; Env : AST_Envs.Lexical_Env_Access)
   is
      procedure Helper is new Register_Destroyable_Gen
        (AST_Envs.Lexical_Env_Record, AST_Envs.Lexical_Env_Access, Destroy);
   begin
      Helper (Unit, Env);
   end Register_Destroyable;
   -----------------------
   -- Invalidate_Caches --
   -----------------------
   procedure Invalidate_Caches
     (Context : Internal_Context; Invalidate_Envs : Boolean) is
   begin
      --  Increase Context's version number. If we are about to overflow, reset
      --  all version numbers from analysis units.
      if Context.Cache_Version = Natural'Last then
         Context.Cache_Version := 1;
         for Unit of Context.Units loop
            Unit.Cache_Version := 0;
         end loop;
      else
         Context.Cache_Version := Context.Cache_Version + 1;
      end if;
      if Invalidate_Envs then
         Context.Reparse_Cache_Version := Context.Cache_Version;
      end if;
   end Invalidate_Caches;
   ------------------
   --  Reset_Envs  --
   ------------------
   procedure Reset_Envs (Unit : Internal_Unit) is
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node);
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node);
      --------------------------
      -- Deactivate_Refd_Envs --
      --------------------------
      procedure Deactivate_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Deactivate_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Deactivate_Refd_Envs (Child (Node, I));
         end loop;
      end Deactivate_Refd_Envs;
      -------------------------
      -- Recompute_Refd_Envs --
      -------------------------
      procedure Recompute_Refd_Envs (Node : Bare_Ada_Node) is
      begin
         if Node = null then
            return;
         end if;
         Recompute_Referenced_Envs (Node.Self_Env);
         for I in 1 .. Children_Count (Node) loop
            Recompute_Refd_Envs (Child (Node, I));
         end loop;
      end Recompute_Refd_Envs;
   begin
      --  First pass will deactivate every referenced envs that Unit possesses
      Deactivate_Refd_Envs (Unit.AST_Root);
      --  Second pass will recompute the env they are pointing to
      Recompute_Refd_Envs (Unit.AST_Root);
   end Reset_Envs;
   -------------
   -- Destroy --
   -------------
   procedure Destroy (Reparsed : in out Reparsed_Unit) is
   begin
      Free (Reparsed.TDH);
      Reparsed.Diagnostics := Diagnostics_Vectors.Empty_Vector;
      Free (Reparsed.AST_Mem_Pool);
      Reparsed.AST_Root := null;
   end Destroy;
   --------------
   -- Basename --
   --------------
   function Basename (Filename : String) return String is
   begin
      return +Create (+Filename).Base_Name;
   end Basename;
   --------------
   -- Basename --
   --------------
   function Basename (Unit : Internal_Unit) return String is
   begin
      return +Unit.Filename.Base_Name;
   end Basename;
   ------------------
   -- Reset_Caches --
   ------------------
   procedure Reset_Caches (Unit : Internal_Unit) is
      Cache_Version : constant Natural := Unit.Cache_Version;
   begin
      if Cache_Version < Unit.Context.Reparse_Cache_Version then
         Unit.Cache_Version := Unit.Context.Reparse_Cache_Version;
         Reset_Envs (Unit);
      end if;
      if Cache_Version < Unit.Context.Cache_Version then
         Unit.Cache_Version := Unit.Context.Cache_Version;
            Destroy (Unit.Memoization_Map);
      end if;
   end Reset_Caches;
   --------------------
   -- Reference_Unit --
   --------------------
   procedure Reference_Unit (From, Referenced : Internal_Unit) is
      Dummy : Boolean;
   begin
      Dummy := Analysis_Unit_Sets.Add (From.Referenced_Units, Referenced);
   end Reference_Unit;
   ------------------------
   -- Is_Referenced_From --
   ------------------------
   function Is_Referenced_From
     (Self, Unit : Internal_Unit) return Boolean is
   begin
      if Unit = null or else Self = null then
         return False;
      elsif Unit = Self then
         return True;
      else
         return Analysis_Unit_Sets.Has (Unit.Referenced_Units, Self);
      end if;
   end Is_Referenced_From;
   ----------------
   -- Do_Parsing --
   ----------------
   procedure Do_Parsing
     (Unit   : Internal_Unit;
      Input  : Internal_Lexer_Input;
      Result : out Reparsed_Unit)
   is
      Context  : constant Internal_Context := Unit.Context;
      Unit_TDH : constant Token_Data_Handler_Access := Token_Data (Unit);
      Saved_TDH : Token_Data_Handler;
      --  Holder to save tokens data in Unit.
      --
      --  By design, parsing is required to bind the nodes it creates to an
      --  analysis unit. However, this procedure is supposed to preserve the
      --  Unit itself and return its parsing result in Result.
      --
      --  In order to implement this, we first move "old" token data in this
      --  variable, then we do parsing. Only then, we can move "new" token data
      --  from the unit to Result, and restore the "old" token data to Unit.
      --  This last step is what Rotate_TDH (see below) is above.
      procedure Rotate_TDH;
      --  Move token data from Unit to Result and restore data in Saved_TDH to
      --  Unit.
      ----------------
      -- Rotate_TDH --
      ----------------
      procedure Rotate_TDH is
      begin
         Move (Result.TDH, Unit_TDH.all);
         Move (Unit_TDH.all, Saved_TDH);
      end Rotate_TDH;
   begin
      GNATCOLL.Traces.Trace (Main_Trace, "Parsing unit " & Basename (Unit));
      Result.AST_Root := null;
      Move (Saved_TDH, Unit_TDH.all);
      Initialize (Unit_TDH.all, Saved_TDH.Symbols,
                  Unit.Context.Tab_Stop);
      --  This is where lexing occurs, so this is where we get most "setup"
      --  issues: missing input file, bad charset, etc. If we have such an
      --  error, catch it, turn it into diagnostics and abort parsing.
      --
      --  As it is quite common, first check if the file is readable: if not,
      --  don't bother opening it and directly emit a diagnostic. This avoid
      --  pointless exceptions which harm debugging. Note that this
      --  optimization is valid only when there is no file reader, which can
      --  work even when there is no real source file.
      if Context.File_Reader = null
         and then Input.Kind = File
         and then (Input.Filename.Is_Directory
                   or else (not Input.Filename.Is_Readable))
      then
         declare
            Name : constant String := Basename (Unit);
         begin
            GNATCOLL.Traces.Trace
              (Main_Trace, "WARNING: File is not readable: " & Name);
            Append
              (Result.Diagnostics,
               No_Source_Location_Range,
               "Cannot read " & To_Text (Name));
            Rotate_TDH;
            return;
         end;
      end if;
      --  Initialize the parser, which fetches the source buffer and extract
      --  all tokens.
      Init_Parser
        (Input, Context.With_Trivia, Unit, Unit_TDH, Unit.Context.Parser);
      --  If we could run the lexer, run the parser and get the root node
      if Unit_TDH.Source_Buffer /= null then
         Result.AST_Mem_Pool := Create;
         Unit.Context.Parser.Mem_Pool := Result.AST_Mem_Pool;
         Result.AST_Root := Bare_Ada_Node
           (Parse (Unit.Context.Parser, Rule => Unit.Rule));
      end if;
      --  Forward token data and diagnostics to the returned unit
      Rotate_TDH;
      Result.Diagnostics.Append (Unit.Context.Parser.Diagnostics);
   end Do_Parsing;
   --------------------------
   -- Update_After_Reparse --
   --------------------------
   procedure Update_After_Reparse
     (Unit : Internal_Unit; Reparsed : in out Reparsed_Unit) is
   begin
      --  Remove the `symbol -> AST node` associations for Unit's nodes in
      --  foreign lexical environments.
      Remove_Exiled_Entries (Unit);
      --  Remove the named envs that Unit created
      declare
         Named_Envs_Needing_Update : NED_Maps.Map;
      begin
         Remove_Named_Envs (Unit, Named_Envs_Needing_Update);
         Update_Named_Envs (Named_Envs_Needing_Update);
      end;
      --  At this point, envs and nodes that don't belong to this unit no
      --  longer reference this unit's envs and nodes. It is thus now safe to
      --  deallocate this unit's obsolete data.
      --  Replace Unit's diagnostics by Reparsed's
      Unit.Diagnostics := Reparsed.Diagnostics;
      Reparsed.Diagnostics.Clear;
      --  As (re-)loading a unit can change how any AST node property in the
      --  whole analysis context behaves, we have to invalidate caches. This
      --  is likely overkill, but kill all caches here as it's easy to do.
      --
      --  As an optimization, invalidate referenced envs cache only if this is
      --  not the first time we parse Unit.
      Invalidate_Caches
        (Unit.Context, Invalidate_Envs => Unit.AST_Root /= null);
      --  Likewise for token data
      Free (Unit.TDH);
      Move (Unit.TDH, Reparsed.TDH);
      --  Reparsing will invalidate all lexical environments related to this
      --  unit, so destroy all related rebindings as well. This browses AST
      --  nodes, so we have to do this before destroying the old AST nodes
      --  pool.
      Destroy_Rebindings (Unit.Rebindings'Access);
      --  Destroy the old AST node and replace it by the new one
      if Unit.AST_Root /= null then
         Destroy (Unit.AST_Root);
      end if;
      Unit.AST_Root := Reparsed.AST_Root;
      --  Likewise for memory pools
      Free (Unit.AST_Mem_Pool);
      Unit.AST_Mem_Pool := Reparsed.AST_Mem_Pool;
      Reparsed.AST_Mem_Pool := No_Pool;
      --  Increment unit version number to invalidate caches and stale node
      --  reference.
      Unit.Unit_Version := Unit.Unit_Version + 1;
      --  If Unit had its lexical environments populated, re-populate them
      if not Unit.Is_Env_Populated then
         return;
      end if;
      declare
         Unit_Name     : constant String := +Unit.Filename.Base_Name;
         Context       : constant Internal_Context := Unit.Context;
         Foreign_Nodes : Bare_Ada_Node_Vectors.Vector :=
            Bare_Ada_Node_Vectors.Empty_Vector;
         Saved_In_Populate_Lexical_Env : constant Boolean :=
            Context.In_Populate_Lexical_Env;
      begin
         GNATCOLL.Traces.Trace
           (Main_Trace, "Updating lexical envs for " & Unit_Name
                        & " after reparse");
         GNATCOLL.Traces.Increase_Indent (Main_Trace);
         Context.In_Populate_Lexical_Env := True;
         --  Collect all nodes that are foreign in this Unit's lexical envs.
         --  Exclude them from the corresponding lists of exiled entries.
         Extract_Foreign_Nodes (Unit, Foreign_Nodes);
         --  Reset the flag so that the call to Populate_Lexical_Env below does
         --  its work.
         Unit.Is_Env_Populated := False;
         --  Now that Unit has been reparsed, we can destroy all its
         --  destroyables, which refer to the old tree (i.e. dangling
         --  pointers).
         Destroy_Unit_Destroyables (Unit);
         for FN of Foreign_Nodes loop
            declare
               Node_Image : constant String := Image (Short_Text_Image (FN));
               Unit_Name  : constant String := +FN.Unit.Filename.Base_Name;
            begin
               GNATCOLL.Traces.Trace
                 (Main_Trace, "Rerooting: " & Node_Image
                              & " (from " & Unit_Name & ")");
            end;
            Reroot_Foreign_Node (FN);
         end loop;
         Foreign_Nodes.Destroy;
         Populate_Lexical_Env (Unit);
         Context.In_Populate_Lexical_Env := Saved_In_Populate_Lexical_Env;
         GNATCOLL.Traces.Decrease_Indent (Main_Trace);
      end;
   end Update_After_Reparse;
   -------------------------------
   -- Destroy_Unit_Destroyables --
   -------------------------------
   procedure Destroy_Unit_Destroyables (Unit : Internal_Unit) is
   begin
      for D of Unit.Destroyables loop
         D.Destroy (D.Object);
      end loop;
      Destroyable_Vectors.Clear (Unit.Destroyables);
   end Destroy_Unit_Destroyables;
   ---------------------------
   -- Remove_Exiled_Entries --
   ---------------------------
   procedure Remove_Exiled_Entries (Unit : Internal_Unit) is
   begin
      for EE of Unit.Exiled_Entries loop
         AST_Envs.Remove (EE.Env, EE.Key, EE.Node);
         --  Also strip foreign nodes information from "outer" units so that it
         --  does not contain stale information (i.e. dangling pointers to
         --  nodes that belong to the units in the queue).
         if EE.Env.Owner /= No_Generic_Unit then
            declare
               Foreign_Nodes : Foreign_Node_Entry_Vectors.Vector renames
                  Convert_Unit (EE.Env.Owner).Foreign_Nodes;
               Current       : Positive := Foreign_Nodes.First_Index;
            begin
               while Current <= Foreign_Nodes.Last_Index loop
                  if Foreign_Nodes.Get (Current).Node = EE.Node then
                     Foreign_Nodes.Pop (Current);
                  else
                     Current := Current + 1;
                  end if;
               end loop;
            end;
         end if;
      end loop;
      Unit.Exiled_Entries.Clear;
   end Remove_Exiled_Entries;
   -----------------------
   -- Remove_Named_Envs --
   -----------------------
   procedure Remove_Named_Envs
     (Unit                      : Internal_Unit;
      Named_Envs_Needing_Update : in out NED_Maps.Map) is
   begin
      --  Remove nodes in this unit from the Named_Env_Descriptor.Foreign_Nodes
      --  components in which they are registered and from the foreign
      --  environments themselves.
      for EE of Unit.Exiled_Entries_In_NED loop
         Remove (EE.Named_Env.Foreign_Nodes, EE.Key, EE.Node);
         Remove (EE.Named_Env.Env_With_Precedence, EE.Key, EE.Node);
      end loop;
      Unit.Exiled_Entries_In_NED.Clear;
      --  Remove nodes in this unit from the
      --  Named_Env_Descriptor.Nodes_With_Foreign_Env components in which they
      --  are registered.
      for Cur in Unit.Nodes_With_Foreign_Env.Iterate loop
         declare
            use Node_To_Named_Env_Maps;
            Node : constant Bare_Ada_Node := Key (Cur);
            NE   : constant Named_Env_Descriptor_Access := Element (Cur);
         begin
            NE.Nodes_With_Foreign_Env.Delete (Node);
         end;
      end loop;
      Unit.Nodes_With_Foreign_Env.Clear;
      --  Remove ends in this unit from the Named_Env_Descriptor.Foreign_Envs
      --  components in which they are registered.
      for EE of Unit.Exiled_Envs loop
         EE.Named_Env.Foreign_Envs.Delete (Env_Node (EE.Env));
      end loop;
      Unit.Exiled_Envs.Clear;
      --  Remove named envs that this unit created
      for NE of Unit.Named_Envs loop
         declare
            NED_Access : constant Named_Env_Descriptor_Access :=
               Unit.Context.Named_Envs.Element (NE.Name);
            NED        : Named_Env_Descriptor renames NED_Access.all;
         begin
            NED.Envs.Delete (Env_Node (NE.Env));
            --  If this named environment had precedence, we must schedule an
            --  update for this name environment entry.
            if NE.Env = NED.Env_With_Precedence then
               Named_Envs_Needing_Update.Include (NE.Name, NED_Access);
               NED.Env_With_Precedence := Empty_Env;
            end if;
         end;
      end loop;
      Unit.Named_Envs.Clear;
   end Remove_Named_Envs;
   ---------------------------
   -- Extract_Foreign_Nodes --
   ---------------------------
   procedure Extract_Foreign_Nodes
     (Unit          : Internal_Unit;
      Foreign_Nodes : in out Bare_Ada_Node_Vectors.Vector) is
   begin
      --  Go through all foreign nodes registered in Unit's lexical
      --  environments.
      for FN of Unit.Foreign_Nodes loop
         --  Collect them
         Foreign_Nodes.Append (FN.Node);
         --  For each foreign node, remove the corresponding exiled entry in
         --  that foreign unit (each foreign node in unit A has a corresponding
         --  exiled entry in unit B).
         declare
            Exiled_Entries : Exiled_Entry_Vectors.Vector renames
               FN.Unit.Exiled_Entries;
            Current        : Positive := Exiled_Entries.First_Index;
         begin
            while Current <= Exiled_Entries.Last_Index loop
               if Exiled_Entries.Get (Current).Node = FN.Node then
                  Exiled_Entries.Pop (Current);
               else
                  Current := Current + 1;
               end if;
            end loop;
         end;
      end loop;
      Unit.Foreign_Nodes.Clear;
   end Extract_Foreign_Nodes;
   --------------------------
   -- Reroot_Foreign_Nodes --
   --------------------------
   procedure Reroot_Foreign_Node (Node : Bare_Ada_Node) is
      Unit : constant Internal_Unit := Node.Unit;
   begin
      --  First, filter the exiled entries in foreign units so that they don't
      --  contain references to this unit's lexical environments.  We need to
      --  do that before running the partial Populate_Lexical_Env pass so that
      --  we don't remove exiled entries that this pass will produce.
      declare
         Exiled_Entries : Exiled_Entry_Vectors.Vector renames
            Unit.Exiled_Entries;
         Current        : Positive := Exiled_Entries.First_Index;
      begin
         while Current <= Exiled_Entries.Last_Index loop
            if Exiled_Entries.Get (Current).Node = Node then
               Exiled_Entries.Pop (Current);
            else
               Current := Current + 1;
            end if;
         end loop;
      end;
      --  Re-do a partial Populate_Lexical_Env pass for each foreign node that
      --  this unit contains so that they are relocated in our new lexical
      --  environments.
      declare
         Unit_State : aliased PLE_Unit_State :=
           (Named_Envs_Needing_Update => <>);
         State      : PLE_Node_State :=
           (Unit_State  => Unit_State'Unchecked_Access,
            Current_Env => Node.Self_Env,
            Current_NED => null);
      begin
         Pre_Env_Actions (Node, State, Add_To_Env_Only => True);
         Post_Env_Actions (Node, State);
      end;
   end Reroot_Foreign_Node;
   ----------
   -- Text --
   ----------
   function Text (Node : Bare_Ada_Node) return Character_Type_Array_Access is
      T      : constant Text_Type := Text (Node);
      Result : constant Character_Type_Array_Access :=
         Create_Character_Type_Array (T'Length);
   begin
      Result.Items := T;
      return Result;
   end Text;
   ------------------------
   -- Destroy_Rebindings --
   ------------------------
   procedure Destroy_Rebindings
     (Rebindings : access Env_Rebindings_Vectors.Vector)
   is
      procedure Recurse (R : in out Env_Rebindings);
      --  Destroy R's children and then destroy R. It is up to the caller to
      --  remove R from its parent's Children vector.
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector);
      --  Remove R from Rebindings
      -------------
      -- Recurse --
      -------------
      procedure Recurse (R : in out Env_Rebindings) is
      begin
         for C of R.Children loop
            declare
               C_Var : Env_Rebindings := C;
            begin
               Recurse (C_Var);
            end;
         end loop;
         R.Children.Destroy;
         Unregister (R, Convert_Unit (R.Old_Env.Owner).Rebindings);
         Unregister (R, Convert_Unit (R.New_Env.Owner).Rebindings);
         Release_Rebinding (R);
      end Recurse;
      ----------------
      -- Unregister --
      ----------------
      procedure Unregister
        (R          : Env_Rebindings;
         Rebindings : in out Env_Rebindings_Vectors.Vector) is
      begin
         for I in 1 .. Rebindings.Length loop
            if Rebindings.Get (I) = R then
               Rebindings.Pop (I);
               return;
            end if;
         end loop;
         --  We are always supposed to find R in Rebindings, so this should be
         --  unreachable.
         raise Program_Error;
      end Unregister;
   begin
      while Rebindings.Length > 0 loop
         declare
            R : Env_Rebindings := Rebindings.Get (1);
         begin
            --  Here, we basically undo what has been done in AST_Envs.Append
            --  If this rebinding has no parent, then during its creation we
            --  registered it in its Old_Env. Otherwise, it is registered
            --  in its Parent's Children list.
            if R.Parent = null then
               Unwrap (R.Old_Env).Rebindings_Pool.Delete (R.New_Env);
            else
               Unregister (R, R.Parent.Children);
            end if;
            --  In all cases it's registered in Old_Env's and New_Env's units
            Recurse (R);
         end;
      end loop;
   end Destroy_Rebindings;
   --------------------------
   -- Get_Rewriting_Handle --
   --------------------------
   function Get_Rewriting_Handle
     (Context : Internal_Context) return Rewriting_Handle_Pointer is
   begin
      return Context.Rewriting_Handle;
   end Get_Rewriting_Handle;
   --------------------------
   -- Set_Rewriting_Handle --
   --------------------------
   procedure Set_Rewriting_Handle
     (Context : Internal_Context; Handle : Rewriting_Handle_Pointer) is
   begin
      Context.Rewriting_Handle := Handle;
   end Set_Rewriting_Handle;
   -----------------------
   -- Create_Safety_Net --
   -----------------------
   function Create_Safety_Net
     (Context : Internal_Context) return Iterator_Safety_Net
   is
   begin
      return (Context         => Context,
              Context_Serial  => Context.Serial_Number,
              Context_Version => Context.Cache_Version);
   end Create_Safety_Net;
   ----------------------
   -- Check_Safety_Net --
   ----------------------
   procedure Check_Safety_Net (Self : Iterator_Safety_Net) is
   begin
      if Self.Context = null then
         return;
      end if;
      --  Check that the context is still the same (not released nor reused)
      if Self.Context.Serial_Number /= Self.Context_Serial
         or else Self.Context.Cache_Version /= Self.Context_Version
      then
         raise Stale_Reference_Error;
      end if;
   end Check_Safety_Net;
   ----------------------
   -- String_To_Symbol --
   ----------------------
   function String_To_Symbol
     (Context : Internal_Context; S : Character_Type_Array_Access) return Symbol_Type is
   begin
      return (if S.N > 0
              then Lookup_Symbol (Context, S.Items)
              else null);
   exception
      when Exc : Invalid_Symbol_Error =>
         raise Property_Error with Ada.Exceptions.Exception_Message (Exc);
   end String_To_Symbol;
begin
   No_Big_Integer.Value.Set (0);
end Libadalang.Implementation;